/**
 * Creating a sidebar enables you to:
 - create an ordered group of docs
 - render a sidebar for each doc of that group
 - provide next/previous navigation

 The sidebars can be generated from the filesystem, or explicitly defined here.

 Create as many sidebars as you want.
 */

// @ts-check

/** @type {import('@docusaurus/plugin-content-docs').SidebarsConfig} */
const sidebars = {
  // By default, Docusaurus generates a sidebar from the docs folder structure
  ///tutorialSidebar: [{ type: 'autogenerated', dirName: '.' }],

  // But you can create a sidebar manually

  tutorialSidebar: [
    {
      type: 'doc',
      id: 'introduction-to-morphir',
    },
    {
      type: 'category',
      label: 'Overview',
      items: [
          'intro',
          'morphir-entail',
          'what-is-it-about'
      ],
    },
      {
          type: 'category',
          label: 'Modelling Guides',
          items: [
              'domain-model',
              'model-versioning',
              'modeling-entire-application',
              'modeling-database-developers',
              'modeling-financial-concepts',
              'regtech-modeling'
          ],
      },
    {
      type: 'category',
      label: 'Getting Started',
      items: [
          "installation",
          "editor-setup",
          'morphir-testing-setup',
          'installation-and-usage'
      ]
    },
    {
      type: 'category',
      label: 'Concepts',
      items: [
          'morphir-sdk',
          'morphir-ir'
      ]
    },
    {
      type: 'category',
      label: 'Integration Guides',
      items: [
          {
              type: 'category',
              label: 'Scala Integration Guide',
              items: [
                  'scala-api',
                  'scala-backend',
                  'scala-codecs',
                  'codec-docs',
                  'generate-scala-test-case',
              ]
          },
          {
              type: 'category',
              label: 'Spark Integration Guide',
              items: [
                  'morphir-spark',
                  'spark-backend-api',
                  'spark-testing-framework',
                  'spark-optional-values',
                  'spark-backend-joins',
                  'spark-backend-relational',
              ]
          },
          {
              type: 'category',
              label: 'TypeScript Integration Guide',
              items: [
                  'morphir-typescript',
                  'typescript-api',
                  ]
          },

          {
              type: 'category',
              label: 'Json Schema Integration',
              items: [
                  'json-schema-sample',
                  'json-schema-mappings',
                  'json-schema-enabled-decorator',
                  'json-schema-config',
                  'json-schema-backend-test',
                  'json-schema-backend-test2',
              ]
          },
          'relational-backend',
          'springboot-generator',
          'json-codecs-doc',
          'decorations-users-guide',
          'morphir-typespec-mapping',
          'insight-api-guide',
      ]
    },
    {
      type: 'category',
      label: 'HowTos and Tutorials',
      items: [
          'modeling-database-developers',
          'modeling-financial-concepts',
          'publishing-elm-package',
          'morphir-quick-start',
          'generating-json-schema',
        {
          type: 'link',
          label: 'Surfing Board Tutorial',
          href: 'https://github.com/stephengoldbaum/morphir-examples/tree/master/tutorial'
        }
      ]
    },
    {
      type: 'category',
      label: 'Ecosystem',
      items: [
        {type: 'category', label: 'Morphir Scala', items: [
          'scala/morphir-scala',
          'scala/morphir-scala-support',
          { type: 'category',
            label: 'Morphir Data Model',
            items:[
              'scala/datamodel/datamodel',
              'scala/datamodel/schema',
              'scala/datamodel/primitives',
              'scala/datamodel/lists',
              'scala/datamodel/maps',
              'scala/datamodel/records',
              'scala/datamodel/enums',
            ]
          }
        ]
      },
      ]
    },
    {
      type: 'category',
      label: 'Contribution Guide',
      items: [
          'code-of-conduct',
          'contribution-guide',
          'contribution-and-governance-policies',
          'contributing-to-morphir'
      ]
    },
    {
      type: 'category',
      label: 'Further Reading',
      items: [
          'functional-programming',
          'dev-bots',
          'cli-cli2-merging',
          'results-in-elm',
          'user-guide-readme',
          'testing-framework',
          'table-template',
          'morphir-elm-commands-processing',
          'morphir-community',
          'adr-for-morphir-cli',
          'media-about-morphir',
          'soul-application',
          'error-append-not-supported',
          'command-line-tools'
      ]
    },
      {
          type: 'doc',
          id: 'faqs',
      }
  ]
};

module.exports = sidebars;
