{-
   Copyright 2022 Morgan Stanley

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
-}


module AntiquesDataSource exposing (antiquesDataSource)

import Csv.Decode as Decode exposing (..)
import CsvUtils exposing (..)
import SparkTests.DataDefinition.Persistence.Income.AntiqueShop exposing (Antique)
import SparkTests.Types exposing (..)


csvData : String
csvData =
    """category,product,priceValue,ageOfItem,handMade,requiresExpert,expertFeedBack,report
PaintCollections,Paintings,0.0,-1.0,True,True,Genuine,Report #1
PaintCollections,Paintings,0.0,-1.0,True,True,Genuine,
PaintCollections,Paintings,0.0,-1.0,True,True,Fake,Report #2
PaintCollections,Paintings,0.0,-1.0,True,True,Fake,
PaintCollections,Paintings,0.0,-1.0,True,True,,Report #3
PaintCollections,Paintings,0.0,-1.0,True,True,,
PaintCollections,Paintings,0.0,-1.0,True,False,Genuine,Report #4
PaintCollections,Paintings,0.0,-1.0,True,False,Genuine,
PaintCollections,Paintings,0.0,-1.0,True,False,Fake,Report #5
PaintCollections,Paintings,0.0,-1.0,True,False,Fake,
PaintCollections,Paintings,0.0,-1.0,True,False,,Report #6
PaintCollections,Paintings,0.0,-1.0,True,False,,
PaintCollections,Paintings,0.0,-1.0,False,True,Genuine,Report #7
PaintCollections,Paintings,0.0,-1.0,False,True,Genuine,
PaintCollections,Paintings,0.0,-1.0,False,True,Fake,Report #8
PaintCollections,Paintings,0.0,-1.0,False,True,Fake,
PaintCollections,Paintings,0.0,-1.0,False,True,,Report #9
PaintCollections,Paintings,0.0,-1.0,False,True,,
PaintCollections,Paintings,0.0,-1.0,False,False,Genuine,Report #10
PaintCollections,Paintings,0.0,-1.0,False,False,Genuine,
PaintCollections,Paintings,0.0,-1.0,False,False,Fake,Report #11
PaintCollections,Paintings,0.0,-1.0,False,False,Fake,
PaintCollections,Paintings,0.0,-1.0,False,False,,Report #12
PaintCollections,Paintings,0.0,-1.0,False,False,,
PaintCollections,Paintings,0.0,0.0,True,True,Genuine,Report #13
PaintCollections,Paintings,0.0,0.0,True,True,Genuine,
PaintCollections,Paintings,0.0,0.0,True,True,Fake,Report #14
PaintCollections,Paintings,0.0,0.0,True,True,Fake,
PaintCollections,Paintings,0.0,0.0,True,True,,Report #15
PaintCollections,Paintings,0.0,0.0,True,True,,
PaintCollections,Paintings,0.0,0.0,True,False,Genuine,Report #16
PaintCollections,Paintings,0.0,0.0,True,False,Genuine,
PaintCollections,Paintings,0.0,0.0,True,False,Fake,Report #17
PaintCollections,Paintings,0.0,0.0,True,False,Fake,
PaintCollections,Paintings,0.0,0.0,True,False,,Report #18
PaintCollections,Paintings,0.0,0.0,True,False,,
PaintCollections,Paintings,0.0,0.0,False,True,Genuine,Report #19
PaintCollections,Paintings,0.0,0.0,False,True,Genuine,
PaintCollections,Paintings,0.0,0.0,False,True,Fake,Report #20
PaintCollections,Paintings,0.0,0.0,False,True,Fake,
PaintCollections,Paintings,0.0,0.0,False,True,,Report #21
PaintCollections,Paintings,0.0,0.0,False,True,,
PaintCollections,Paintings,0.0,0.0,False,False,Genuine,Report #22
PaintCollections,Paintings,0.0,0.0,False,False,Genuine,
PaintCollections,Paintings,0.0,0.0,False,False,Fake,Report #23
PaintCollections,Paintings,0.0,0.0,False,False,Fake,
PaintCollections,Paintings,0.0,0.0,False,False,,Report #24
PaintCollections,Paintings,0.0,0.0,False,False,,
PaintCollections,Paintings,0.0,19.0,True,True,Genuine,Report #25
PaintCollections,Paintings,0.0,19.0,True,True,Genuine,
PaintCollections,Paintings,0.0,19.0,True,True,Fake,Report #26
PaintCollections,Paintings,0.0,19.0,True,True,Fake,
PaintCollections,Paintings,0.0,19.0,True,True,,Report #27
PaintCollections,Paintings,0.0,19.0,True,True,,
PaintCollections,Paintings,0.0,19.0,True,False,Genuine,Report #28
PaintCollections,Paintings,0.0,19.0,True,False,Genuine,
PaintCollections,Paintings,0.0,19.0,True,False,Fake,Report #29
PaintCollections,Paintings,0.0,19.0,True,False,Fake,
PaintCollections,Paintings,0.0,19.0,True,False,,Report #30
PaintCollections,Paintings,0.0,19.0,True,False,,
PaintCollections,Paintings,0.0,19.0,False,True,Genuine,Report #31
PaintCollections,Paintings,0.0,19.0,False,True,Genuine,
PaintCollections,Paintings,0.0,19.0,False,True,Fake,Report #32
PaintCollections,Paintings,0.0,19.0,False,True,Fake,
PaintCollections,Paintings,0.0,19.0,False,True,,Report #33
PaintCollections,Paintings,0.0,19.0,False,True,,
PaintCollections,Paintings,0.0,19.0,False,False,Genuine,Report #34
PaintCollections,Paintings,0.0,19.0,False,False,Genuine,
PaintCollections,Paintings,0.0,19.0,False,False,Fake,Report #35
PaintCollections,Paintings,0.0,19.0,False,False,Fake,
PaintCollections,Paintings,0.0,19.0,False,False,,Report #36
PaintCollections,Paintings,0.0,19.0,False,False,,
PaintCollections,Paintings,0.0,20.0,True,True,Genuine,Report #37
PaintCollections,Paintings,0.0,20.0,True,True,Genuine,
PaintCollections,Paintings,0.0,20.0,True,True,Fake,Report #38
PaintCollections,Paintings,0.0,20.0,True,True,Fake,
PaintCollections,Paintings,0.0,20.0,True,True,,Report #39
PaintCollections,Paintings,0.0,20.0,True,True,,
PaintCollections,Paintings,0.0,20.0,True,False,Genuine,Report #40
PaintCollections,Paintings,0.0,20.0,True,False,Genuine,
PaintCollections,Paintings,0.0,20.0,True,False,Fake,Report #41
PaintCollections,Paintings,0.0,20.0,True,False,Fake,
PaintCollections,Paintings,0.0,20.0,True,False,,Report #42
PaintCollections,Paintings,0.0,20.0,True,False,,
PaintCollections,Paintings,0.0,20.0,False,True,Genuine,Report #43
PaintCollections,Paintings,0.0,20.0,False,True,Genuine,
PaintCollections,Paintings,0.0,20.0,False,True,Fake,Report #44
PaintCollections,Paintings,0.0,20.0,False,True,Fake,
PaintCollections,Paintings,0.0,20.0,False,True,,Report #45
PaintCollections,Paintings,0.0,20.0,False,True,,
PaintCollections,Paintings,0.0,20.0,False,False,Genuine,Report #46
PaintCollections,Paintings,0.0,20.0,False,False,Genuine,
PaintCollections,Paintings,0.0,20.0,False,False,Fake,Report #47
PaintCollections,Paintings,0.0,20.0,False,False,Fake,
PaintCollections,Paintings,0.0,20.0,False,False,,Report #48
PaintCollections,Paintings,0.0,20.0,False,False,,
PaintCollections,Paintings,0.0,21.0,True,True,Genuine,Report #49
PaintCollections,Paintings,0.0,21.0,True,True,Genuine,
PaintCollections,Paintings,0.0,21.0,True,True,Fake,Report #50
PaintCollections,Paintings,0.0,21.0,True,True,Fake,
PaintCollections,Paintings,0.0,21.0,True,True,,Report #51
PaintCollections,Paintings,0.0,21.0,True,True,,
PaintCollections,Paintings,0.0,21.0,True,False,Genuine,Report #52
PaintCollections,Paintings,0.0,21.0,True,False,Genuine,
PaintCollections,Paintings,0.0,21.0,True,False,Fake,Report #53
PaintCollections,Paintings,0.0,21.0,True,False,Fake,
PaintCollections,Paintings,0.0,21.0,True,False,,Report #54
PaintCollections,Paintings,0.0,21.0,True,False,,
PaintCollections,Paintings,0.0,21.0,False,True,Genuine,Report #55
PaintCollections,Paintings,0.0,21.0,False,True,Genuine,
PaintCollections,Paintings,0.0,21.0,False,True,Fake,Report #56
PaintCollections,Paintings,0.0,21.0,False,True,Fake,
PaintCollections,Paintings,0.0,21.0,False,True,,Report #57
PaintCollections,Paintings,0.0,21.0,False,True,,
PaintCollections,Paintings,0.0,21.0,False,False,Genuine,Report #58
PaintCollections,Paintings,0.0,21.0,False,False,Genuine,
PaintCollections,Paintings,0.0,21.0,False,False,Fake,Report #59
PaintCollections,Paintings,0.0,21.0,False,False,Fake,
PaintCollections,Paintings,0.0,21.0,False,False,,Report #60
PaintCollections,Paintings,0.0,21.0,False,False,,
PaintCollections,Paintings,0.0,99.0,True,True,Genuine,Report #61
PaintCollections,Paintings,0.0,99.0,True,True,Genuine,
PaintCollections,Paintings,0.0,99.0,True,True,Fake,Report #62
PaintCollections,Paintings,0.0,99.0,True,True,Fake,
PaintCollections,Paintings,0.0,99.0,True,True,,Report #63
PaintCollections,Paintings,0.0,99.0,True,True,,
PaintCollections,Paintings,0.0,99.0,True,False,Genuine,Report #64
PaintCollections,Paintings,0.0,99.0,True,False,Genuine,
PaintCollections,Paintings,0.0,99.0,True,False,Fake,Report #65
PaintCollections,Paintings,0.0,99.0,True,False,Fake,
PaintCollections,Paintings,0.0,99.0,True,False,,Report #66
PaintCollections,Paintings,0.0,99.0,True,False,,
PaintCollections,Paintings,0.0,99.0,False,True,Genuine,Report #67
PaintCollections,Paintings,0.0,99.0,False,True,Genuine,
PaintCollections,Paintings,0.0,99.0,False,True,Fake,Report #68
PaintCollections,Paintings,0.0,99.0,False,True,Fake,
PaintCollections,Paintings,0.0,99.0,False,True,,Report #69
PaintCollections,Paintings,0.0,99.0,False,True,,
PaintCollections,Paintings,0.0,99.0,False,False,Genuine,Report #70
PaintCollections,Paintings,0.0,99.0,False,False,Genuine,
PaintCollections,Paintings,0.0,99.0,False,False,Fake,Report #71
PaintCollections,Paintings,0.0,99.0,False,False,Fake,
PaintCollections,Paintings,0.0,99.0,False,False,,Report #72
PaintCollections,Paintings,0.0,99.0,False,False,,
PaintCollections,Paintings,0.0,100.0,True,True,Genuine,Report #73
PaintCollections,Paintings,0.0,100.0,True,True,Genuine,
PaintCollections,Paintings,0.0,100.0,True,True,Fake,Report #74
PaintCollections,Paintings,0.0,100.0,True,True,Fake,
PaintCollections,Paintings,0.0,100.0,True,True,,Report #75
PaintCollections,Paintings,0.0,100.0,True,True,,
PaintCollections,Paintings,0.0,100.0,True,False,Genuine,Report #76
PaintCollections,Paintings,0.0,100.0,True,False,Genuine,
PaintCollections,Paintings,0.0,100.0,True,False,Fake,Report #77
PaintCollections,Paintings,0.0,100.0,True,False,Fake,
PaintCollections,Paintings,0.0,100.0,True,False,,Report #78
PaintCollections,Paintings,0.0,100.0,True,False,,
PaintCollections,Paintings,0.0,100.0,False,True,Genuine,Report #79
PaintCollections,Paintings,0.0,100.0,False,True,Genuine,
PaintCollections,Paintings,0.0,100.0,False,True,Fake,Report #80
PaintCollections,Paintings,0.0,100.0,False,True,Fake,
PaintCollections,Paintings,0.0,100.0,False,True,,Report #81
PaintCollections,Paintings,0.0,100.0,False,True,,
PaintCollections,Paintings,0.0,100.0,False,False,Genuine,Report #82
PaintCollections,Paintings,0.0,100.0,False,False,Genuine,
PaintCollections,Paintings,0.0,100.0,False,False,Fake,Report #83
PaintCollections,Paintings,0.0,100.0,False,False,Fake,
PaintCollections,Paintings,0.0,100.0,False,False,,Report #84
PaintCollections,Paintings,0.0,100.0,False,False,,
PaintCollections,Paintings,0.0,101.0,True,True,Genuine,Report #85
PaintCollections,Paintings,0.0,101.0,True,True,Genuine,
PaintCollections,Paintings,0.0,101.0,True,True,Fake,Report #86
PaintCollections,Paintings,0.0,101.0,True,True,Fake,
PaintCollections,Paintings,0.0,101.0,True,True,,Report #87
PaintCollections,Paintings,0.0,101.0,True,True,,
PaintCollections,Paintings,0.0,101.0,True,False,Genuine,Report #88
PaintCollections,Paintings,0.0,101.0,True,False,Genuine,
PaintCollections,Paintings,0.0,101.0,True,False,Fake,Report #89
PaintCollections,Paintings,0.0,101.0,True,False,Fake,
PaintCollections,Paintings,0.0,101.0,True,False,,Report #90
PaintCollections,Paintings,0.0,101.0,True,False,,
PaintCollections,Paintings,0.0,101.0,False,True,Genuine,Report #91
PaintCollections,Paintings,0.0,101.0,False,True,Genuine,
PaintCollections,Paintings,0.0,101.0,False,True,Fake,Report #92
PaintCollections,Paintings,0.0,101.0,False,True,Fake,
PaintCollections,Paintings,0.0,101.0,False,True,,Report #93
PaintCollections,Paintings,0.0,101.0,False,True,,
PaintCollections,Paintings,0.0,101.0,False,False,Genuine,Report #94
PaintCollections,Paintings,0.0,101.0,False,False,Genuine,
PaintCollections,Paintings,0.0,101.0,False,False,Fake,Report #95
PaintCollections,Paintings,0.0,101.0,False,False,Fake,
PaintCollections,Paintings,0.0,101.0,False,False,,Report #96
PaintCollections,Paintings,0.0,101.0,False,False,,
PaintCollections,Paintings,1.0,-1.0,True,True,Genuine,Report #97
PaintCollections,Paintings,1.0,-1.0,True,True,Genuine,
PaintCollections,Paintings,1.0,-1.0,True,True,Fake,Report #98
PaintCollections,Paintings,1.0,-1.0,True,True,Fake,
PaintCollections,Paintings,1.0,-1.0,True,True,,Report #99
PaintCollections,Paintings,1.0,-1.0,True,True,,
PaintCollections,Paintings,1.0,-1.0,True,False,Genuine,Report #100
PaintCollections,Paintings,1.0,-1.0,True,False,Genuine,
PaintCollections,Paintings,1.0,-1.0,True,False,Fake,Report #101
PaintCollections,Paintings,1.0,-1.0,True,False,Fake,
PaintCollections,Paintings,1.0,-1.0,True,False,,Report #102
PaintCollections,Paintings,1.0,-1.0,True,False,,
PaintCollections,Paintings,1.0,-1.0,False,True,Genuine,Report #103
PaintCollections,Paintings,1.0,-1.0,False,True,Genuine,
PaintCollections,Paintings,1.0,-1.0,False,True,Fake,Report #104
PaintCollections,Paintings,1.0,-1.0,False,True,Fake,
PaintCollections,Paintings,1.0,-1.0,False,True,,Report #105
PaintCollections,Paintings,1.0,-1.0,False,True,,
PaintCollections,Paintings,1.0,-1.0,False,False,Genuine,Report #106
PaintCollections,Paintings,1.0,-1.0,False,False,Genuine,
PaintCollections,Paintings,1.0,-1.0,False,False,Fake,Report #107
PaintCollections,Paintings,1.0,-1.0,False,False,Fake,
PaintCollections,Paintings,1.0,-1.0,False,False,,Report #108
PaintCollections,Paintings,1.0,-1.0,False,False,,
PaintCollections,Paintings,1.0,0.0,True,True,Genuine,Report #109
PaintCollections,Paintings,1.0,0.0,True,True,Genuine,
PaintCollections,Paintings,1.0,0.0,True,True,Fake,Report #110
PaintCollections,Paintings,1.0,0.0,True,True,Fake,
PaintCollections,Paintings,1.0,0.0,True,True,,Report #111
PaintCollections,Paintings,1.0,0.0,True,True,,
PaintCollections,Paintings,1.0,0.0,True,False,Genuine,Report #112
PaintCollections,Paintings,1.0,0.0,True,False,Genuine,
PaintCollections,Paintings,1.0,0.0,True,False,Fake,Report #113
PaintCollections,Paintings,1.0,0.0,True,False,Fake,
PaintCollections,Paintings,1.0,0.0,True,False,,Report #114
PaintCollections,Paintings,1.0,0.0,True,False,,
PaintCollections,Paintings,1.0,0.0,False,True,Genuine,Report #115
PaintCollections,Paintings,1.0,0.0,False,True,Genuine,
PaintCollections,Paintings,1.0,0.0,False,True,Fake,Report #116
PaintCollections,Paintings,1.0,0.0,False,True,Fake,
PaintCollections,Paintings,1.0,0.0,False,True,,Report #117
PaintCollections,Paintings,1.0,0.0,False,True,,
PaintCollections,Paintings,1.0,0.0,False,False,Genuine,Report #118
PaintCollections,Paintings,1.0,0.0,False,False,Genuine,
PaintCollections,Paintings,1.0,0.0,False,False,Fake,Report #119
PaintCollections,Paintings,1.0,0.0,False,False,Fake,
PaintCollections,Paintings,1.0,0.0,False,False,,Report #120
PaintCollections,Paintings,1.0,0.0,False,False,,
PaintCollections,Paintings,1.0,19.0,True,True,Genuine,Report #121
PaintCollections,Paintings,1.0,19.0,True,True,Genuine,
PaintCollections,Paintings,1.0,19.0,True,True,Fake,Report #122
PaintCollections,Paintings,1.0,19.0,True,True,Fake,
PaintCollections,Paintings,1.0,19.0,True,True,,Report #123
PaintCollections,Paintings,1.0,19.0,True,True,,
PaintCollections,Paintings,1.0,19.0,True,False,Genuine,Report #124
PaintCollections,Paintings,1.0,19.0,True,False,Genuine,
PaintCollections,Paintings,1.0,19.0,True,False,Fake,Report #125
PaintCollections,Paintings,1.0,19.0,True,False,Fake,
PaintCollections,Paintings,1.0,19.0,True,False,,Report #126
PaintCollections,Paintings,1.0,19.0,True,False,,
PaintCollections,Paintings,1.0,19.0,False,True,Genuine,Report #127
PaintCollections,Paintings,1.0,19.0,False,True,Genuine,
PaintCollections,Paintings,1.0,19.0,False,True,Fake,Report #128
PaintCollections,Paintings,1.0,19.0,False,True,Fake,
PaintCollections,Paintings,1.0,19.0,False,True,,Report #129
PaintCollections,Paintings,1.0,19.0,False,True,,
PaintCollections,Paintings,1.0,19.0,False,False,Genuine,Report #130
PaintCollections,Paintings,1.0,19.0,False,False,Genuine,
PaintCollections,Paintings,1.0,19.0,False,False,Fake,Report #131
PaintCollections,Paintings,1.0,19.0,False,False,Fake,
PaintCollections,Paintings,1.0,19.0,False,False,,Report #132
PaintCollections,Paintings,1.0,19.0,False,False,,
PaintCollections,Paintings,1.0,20.0,True,True,Genuine,Report #133
PaintCollections,Paintings,1.0,20.0,True,True,Genuine,
PaintCollections,Paintings,1.0,20.0,True,True,Fake,Report #134
PaintCollections,Paintings,1.0,20.0,True,True,Fake,
PaintCollections,Paintings,1.0,20.0,True,True,,Report #135
PaintCollections,Paintings,1.0,20.0,True,True,,
PaintCollections,Paintings,1.0,20.0,True,False,Genuine,Report #136
PaintCollections,Paintings,1.0,20.0,True,False,Genuine,
PaintCollections,Paintings,1.0,20.0,True,False,Fake,Report #137
PaintCollections,Paintings,1.0,20.0,True,False,Fake,
PaintCollections,Paintings,1.0,20.0,True,False,,Report #138
PaintCollections,Paintings,1.0,20.0,True,False,,
PaintCollections,Paintings,1.0,20.0,False,True,Genuine,Report #139
PaintCollections,Paintings,1.0,20.0,False,True,Genuine,
PaintCollections,Paintings,1.0,20.0,False,True,Fake,Report #140
PaintCollections,Paintings,1.0,20.0,False,True,Fake,
PaintCollections,Paintings,1.0,20.0,False,True,,Report #141
PaintCollections,Paintings,1.0,20.0,False,True,,
PaintCollections,Paintings,1.0,20.0,False,False,Genuine,Report #142
PaintCollections,Paintings,1.0,20.0,False,False,Genuine,
PaintCollections,Paintings,1.0,20.0,False,False,Fake,Report #143
PaintCollections,Paintings,1.0,20.0,False,False,Fake,
PaintCollections,Paintings,1.0,20.0,False,False,,Report #144
PaintCollections,Paintings,1.0,20.0,False,False,,
PaintCollections,Paintings,1.0,21.0,True,True,Genuine,Report #145
PaintCollections,Paintings,1.0,21.0,True,True,Genuine,
PaintCollections,Paintings,1.0,21.0,True,True,Fake,Report #146
PaintCollections,Paintings,1.0,21.0,True,True,Fake,
PaintCollections,Paintings,1.0,21.0,True,True,,Report #147
PaintCollections,Paintings,1.0,21.0,True,True,,
PaintCollections,Paintings,1.0,21.0,True,False,Genuine,Report #148
PaintCollections,Paintings,1.0,21.0,True,False,Genuine,
PaintCollections,Paintings,1.0,21.0,True,False,Fake,Report #149
PaintCollections,Paintings,1.0,21.0,True,False,Fake,
PaintCollections,Paintings,1.0,21.0,True,False,,Report #150
PaintCollections,Paintings,1.0,21.0,True,False,,
PaintCollections,Paintings,1.0,21.0,False,True,Genuine,Report #151
PaintCollections,Paintings,1.0,21.0,False,True,Genuine,
PaintCollections,Paintings,1.0,21.0,False,True,Fake,Report #152
PaintCollections,Paintings,1.0,21.0,False,True,Fake,
PaintCollections,Paintings,1.0,21.0,False,True,,Report #153
PaintCollections,Paintings,1.0,21.0,False,True,,
PaintCollections,Paintings,1.0,21.0,False,False,Genuine,Report #154
PaintCollections,Paintings,1.0,21.0,False,False,Genuine,
PaintCollections,Paintings,1.0,21.0,False,False,Fake,Report #155
PaintCollections,Paintings,1.0,21.0,False,False,Fake,
PaintCollections,Paintings,1.0,21.0,False,False,,Report #156
PaintCollections,Paintings,1.0,21.0,False,False,,
PaintCollections,Paintings,1.0,99.0,True,True,Genuine,Report #157
PaintCollections,Paintings,1.0,99.0,True,True,Genuine,
PaintCollections,Paintings,1.0,99.0,True,True,Fake,Report #158
PaintCollections,Paintings,1.0,99.0,True,True,Fake,
PaintCollections,Paintings,1.0,99.0,True,True,,Report #159
PaintCollections,Paintings,1.0,99.0,True,True,,
PaintCollections,Paintings,1.0,99.0,True,False,Genuine,Report #160
PaintCollections,Paintings,1.0,99.0,True,False,Genuine,
PaintCollections,Paintings,1.0,99.0,True,False,Fake,Report #161
PaintCollections,Paintings,1.0,99.0,True,False,Fake,
PaintCollections,Paintings,1.0,99.0,True,False,,Report #162
PaintCollections,Paintings,1.0,99.0,True,False,,
PaintCollections,Paintings,1.0,99.0,False,True,Genuine,Report #163
PaintCollections,Paintings,1.0,99.0,False,True,Genuine,
PaintCollections,Paintings,1.0,99.0,False,True,Fake,Report #164
PaintCollections,Paintings,1.0,99.0,False,True,Fake,
PaintCollections,Paintings,1.0,99.0,False,True,,Report #165
PaintCollections,Paintings,1.0,99.0,False,True,,
PaintCollections,Paintings,1.0,99.0,False,False,Genuine,Report #166
PaintCollections,Paintings,1.0,99.0,False,False,Genuine,
PaintCollections,Paintings,1.0,99.0,False,False,Fake,Report #167
PaintCollections,Paintings,1.0,99.0,False,False,Fake,
PaintCollections,Paintings,1.0,99.0,False,False,,Report #168
PaintCollections,Paintings,1.0,99.0,False,False,,
PaintCollections,Paintings,1.0,100.0,True,True,Genuine,Report #169
PaintCollections,Paintings,1.0,100.0,True,True,Genuine,
PaintCollections,Paintings,1.0,100.0,True,True,Fake,Report #170
PaintCollections,Paintings,1.0,100.0,True,True,Fake,
PaintCollections,Paintings,1.0,100.0,True,True,,Report #171
PaintCollections,Paintings,1.0,100.0,True,True,,
PaintCollections,Paintings,1.0,100.0,True,False,Genuine,Report #172
PaintCollections,Paintings,1.0,100.0,True,False,Genuine,
PaintCollections,Paintings,1.0,100.0,True,False,Fake,Report #173
PaintCollections,Paintings,1.0,100.0,True,False,Fake,
PaintCollections,Paintings,1.0,100.0,True,False,,Report #174
PaintCollections,Paintings,1.0,100.0,True,False,,
PaintCollections,Paintings,1.0,100.0,False,True,Genuine,Report #175
PaintCollections,Paintings,1.0,100.0,False,True,Genuine,
PaintCollections,Paintings,1.0,100.0,False,True,Fake,Report #176
PaintCollections,Paintings,1.0,100.0,False,True,Fake,
PaintCollections,Paintings,1.0,100.0,False,True,,Report #177
PaintCollections,Paintings,1.0,100.0,False,True,,
PaintCollections,Paintings,1.0,100.0,False,False,Genuine,Report #178
PaintCollections,Paintings,1.0,100.0,False,False,Genuine,
PaintCollections,Paintings,1.0,100.0,False,False,Fake,Report #179
PaintCollections,Paintings,1.0,100.0,False,False,Fake,
PaintCollections,Paintings,1.0,100.0,False,False,,Report #180
PaintCollections,Paintings,1.0,100.0,False,False,,
PaintCollections,Paintings,1.0,101.0,True,True,Genuine,Report #181
PaintCollections,Paintings,1.0,101.0,True,True,Genuine,
PaintCollections,Paintings,1.0,101.0,True,True,Fake,Report #182
PaintCollections,Paintings,1.0,101.0,True,True,Fake,
PaintCollections,Paintings,1.0,101.0,True,True,,Report #183
PaintCollections,Paintings,1.0,101.0,True,True,,
PaintCollections,Paintings,1.0,101.0,True,False,Genuine,Report #184
PaintCollections,Paintings,1.0,101.0,True,False,Genuine,
PaintCollections,Paintings,1.0,101.0,True,False,Fake,Report #185
PaintCollections,Paintings,1.0,101.0,True,False,Fake,
PaintCollections,Paintings,1.0,101.0,True,False,,Report #186
PaintCollections,Paintings,1.0,101.0,True,False,,
PaintCollections,Paintings,1.0,101.0,False,True,Genuine,Report #187
PaintCollections,Paintings,1.0,101.0,False,True,Genuine,
PaintCollections,Paintings,1.0,101.0,False,True,Fake,Report #188
PaintCollections,Paintings,1.0,101.0,False,True,Fake,
PaintCollections,Paintings,1.0,101.0,False,True,,Report #189
PaintCollections,Paintings,1.0,101.0,False,True,,
PaintCollections,Paintings,1.0,101.0,False,False,Genuine,Report #190
PaintCollections,Paintings,1.0,101.0,False,False,Genuine,
PaintCollections,Paintings,1.0,101.0,False,False,Fake,Report #191
PaintCollections,Paintings,1.0,101.0,False,False,Fake,
PaintCollections,Paintings,1.0,101.0,False,False,,Report #192
PaintCollections,Paintings,1.0,101.0,False,False,,
PaintCollections,Paintings,100.0,-1.0,True,True,Genuine,Report #193
PaintCollections,Paintings,100.0,-1.0,True,True,Genuine,
PaintCollections,Paintings,100.0,-1.0,True,True,Fake,Report #194
PaintCollections,Paintings,100.0,-1.0,True,True,Fake,
PaintCollections,Paintings,100.0,-1.0,True,True,,Report #195
PaintCollections,Paintings,100.0,-1.0,True,True,,
PaintCollections,Paintings,100.0,-1.0,True,False,Genuine,Report #196
PaintCollections,Paintings,100.0,-1.0,True,False,Genuine,
PaintCollections,Paintings,100.0,-1.0,True,False,Fake,Report #197
PaintCollections,Paintings,100.0,-1.0,True,False,Fake,
PaintCollections,Paintings,100.0,-1.0,True,False,,Report #198
PaintCollections,Paintings,100.0,-1.0,True,False,,
PaintCollections,Paintings,100.0,-1.0,False,True,Genuine,Report #199
PaintCollections,Paintings,100.0,-1.0,False,True,Genuine,
PaintCollections,Paintings,100.0,-1.0,False,True,Fake,Report #200
PaintCollections,Paintings,100.0,-1.0,False,True,Fake,
PaintCollections,Paintings,100.0,-1.0,False,True,,Report #201
PaintCollections,Paintings,100.0,-1.0,False,True,,
PaintCollections,Paintings,100.0,-1.0,False,False,Genuine,Report #202
PaintCollections,Paintings,100.0,-1.0,False,False,Genuine,
PaintCollections,Paintings,100.0,-1.0,False,False,Fake,Report #203
PaintCollections,Paintings,100.0,-1.0,False,False,Fake,
PaintCollections,Paintings,100.0,-1.0,False,False,,Report #204
PaintCollections,Paintings,100.0,-1.0,False,False,,
PaintCollections,Paintings,100.0,0.0,True,True,Genuine,Report #205
PaintCollections,Paintings,100.0,0.0,True,True,Genuine,
PaintCollections,Paintings,100.0,0.0,True,True,Fake,Report #206
PaintCollections,Paintings,100.0,0.0,True,True,Fake,
PaintCollections,Paintings,100.0,0.0,True,True,,Report #207
PaintCollections,Paintings,100.0,0.0,True,True,,
PaintCollections,Paintings,100.0,0.0,True,False,Genuine,Report #208
PaintCollections,Paintings,100.0,0.0,True,False,Genuine,
PaintCollections,Paintings,100.0,0.0,True,False,Fake,Report #209
PaintCollections,Paintings,100.0,0.0,True,False,Fake,
PaintCollections,Paintings,100.0,0.0,True,False,,Report #210
PaintCollections,Paintings,100.0,0.0,True,False,,
PaintCollections,Paintings,100.0,0.0,False,True,Genuine,Report #211
PaintCollections,Paintings,100.0,0.0,False,True,Genuine,
PaintCollections,Paintings,100.0,0.0,False,True,Fake,Report #212
PaintCollections,Paintings,100.0,0.0,False,True,Fake,
PaintCollections,Paintings,100.0,0.0,False,True,,Report #213
PaintCollections,Paintings,100.0,0.0,False,True,,
PaintCollections,Paintings,100.0,0.0,False,False,Genuine,Report #214
PaintCollections,Paintings,100.0,0.0,False,False,Genuine,
PaintCollections,Paintings,100.0,0.0,False,False,Fake,Report #215
PaintCollections,Paintings,100.0,0.0,False,False,Fake,
PaintCollections,Paintings,100.0,0.0,False,False,,Report #216
PaintCollections,Paintings,100.0,0.0,False,False,,
PaintCollections,Paintings,100.0,19.0,True,True,Genuine,Report #217
PaintCollections,Paintings,100.0,19.0,True,True,Genuine,
PaintCollections,Paintings,100.0,19.0,True,True,Fake,Report #218
PaintCollections,Paintings,100.0,19.0,True,True,Fake,
PaintCollections,Paintings,100.0,19.0,True,True,,Report #219
PaintCollections,Paintings,100.0,19.0,True,True,,
PaintCollections,Paintings,100.0,19.0,True,False,Genuine,Report #220
PaintCollections,Paintings,100.0,19.0,True,False,Genuine,
PaintCollections,Paintings,100.0,19.0,True,False,Fake,Report #221
PaintCollections,Paintings,100.0,19.0,True,False,Fake,
PaintCollections,Paintings,100.0,19.0,True,False,,Report #222
PaintCollections,Paintings,100.0,19.0,True,False,,
PaintCollections,Paintings,100.0,19.0,False,True,Genuine,Report #223
PaintCollections,Paintings,100.0,19.0,False,True,Genuine,
PaintCollections,Paintings,100.0,19.0,False,True,Fake,Report #224
PaintCollections,Paintings,100.0,19.0,False,True,Fake,
PaintCollections,Paintings,100.0,19.0,False,True,,Report #225
PaintCollections,Paintings,100.0,19.0,False,True,,
PaintCollections,Paintings,100.0,19.0,False,False,Genuine,Report #226
PaintCollections,Paintings,100.0,19.0,False,False,Genuine,
PaintCollections,Paintings,100.0,19.0,False,False,Fake,Report #227
PaintCollections,Paintings,100.0,19.0,False,False,Fake,
PaintCollections,Paintings,100.0,19.0,False,False,,Report #228
PaintCollections,Paintings,100.0,19.0,False,False,,
PaintCollections,Paintings,100.0,20.0,True,True,Genuine,Report #229
PaintCollections,Paintings,100.0,20.0,True,True,Genuine,
PaintCollections,Paintings,100.0,20.0,True,True,Fake,Report #230
PaintCollections,Paintings,100.0,20.0,True,True,Fake,
PaintCollections,Paintings,100.0,20.0,True,True,,Report #231
PaintCollections,Paintings,100.0,20.0,True,True,,
PaintCollections,Paintings,100.0,20.0,True,False,Genuine,Report #232
PaintCollections,Paintings,100.0,20.0,True,False,Genuine,
PaintCollections,Paintings,100.0,20.0,True,False,Fake,Report #233
PaintCollections,Paintings,100.0,20.0,True,False,Fake,
PaintCollections,Paintings,100.0,20.0,True,False,,Report #234
PaintCollections,Paintings,100.0,20.0,True,False,,
PaintCollections,Paintings,100.0,20.0,False,True,Genuine,Report #235
PaintCollections,Paintings,100.0,20.0,False,True,Genuine,
PaintCollections,Paintings,100.0,20.0,False,True,Fake,Report #236
PaintCollections,Paintings,100.0,20.0,False,True,Fake,
PaintCollections,Paintings,100.0,20.0,False,True,,Report #237
PaintCollections,Paintings,100.0,20.0,False,True,,
PaintCollections,Paintings,100.0,20.0,False,False,Genuine,Report #238
PaintCollections,Paintings,100.0,20.0,False,False,Genuine,
PaintCollections,Paintings,100.0,20.0,False,False,Fake,Report #239
PaintCollections,Paintings,100.0,20.0,False,False,Fake,
PaintCollections,Paintings,100.0,20.0,False,False,,Report #240
PaintCollections,Paintings,100.0,20.0,False,False,,
PaintCollections,Paintings,100.0,21.0,True,True,Genuine,Report #241
PaintCollections,Paintings,100.0,21.0,True,True,Genuine,
PaintCollections,Paintings,100.0,21.0,True,True,Fake,Report #242
PaintCollections,Paintings,100.0,21.0,True,True,Fake,
PaintCollections,Paintings,100.0,21.0,True,True,,Report #243
PaintCollections,Paintings,100.0,21.0,True,True,,
PaintCollections,Paintings,100.0,21.0,True,False,Genuine,Report #244
PaintCollections,Paintings,100.0,21.0,True,False,Genuine,
PaintCollections,Paintings,100.0,21.0,True,False,Fake,Report #245
PaintCollections,Paintings,100.0,21.0,True,False,Fake,
PaintCollections,Paintings,100.0,21.0,True,False,,Report #246
PaintCollections,Paintings,100.0,21.0,True,False,,
PaintCollections,Paintings,100.0,21.0,False,True,Genuine,Report #247
PaintCollections,Paintings,100.0,21.0,False,True,Genuine,
PaintCollections,Paintings,100.0,21.0,False,True,Fake,Report #248
PaintCollections,Paintings,100.0,21.0,False,True,Fake,
PaintCollections,Paintings,100.0,21.0,False,True,,Report #249
PaintCollections,Paintings,100.0,21.0,False,True,,
PaintCollections,Paintings,100.0,21.0,False,False,Genuine,Report #250
PaintCollections,Paintings,100.0,21.0,False,False,Genuine,
PaintCollections,Paintings,100.0,21.0,False,False,Fake,Report #251
PaintCollections,Paintings,100.0,21.0,False,False,Fake,
PaintCollections,Paintings,100.0,21.0,False,False,,Report #252
PaintCollections,Paintings,100.0,21.0,False,False,,
PaintCollections,Paintings,100.0,99.0,True,True,Genuine,Report #253
PaintCollections,Paintings,100.0,99.0,True,True,Genuine,
PaintCollections,Paintings,100.0,99.0,True,True,Fake,Report #254
PaintCollections,Paintings,100.0,99.0,True,True,Fake,
PaintCollections,Paintings,100.0,99.0,True,True,,Report #255
PaintCollections,Paintings,100.0,99.0,True,True,,
PaintCollections,Paintings,100.0,99.0,True,False,Genuine,Report #256
PaintCollections,Paintings,100.0,99.0,True,False,Genuine,
PaintCollections,Paintings,100.0,99.0,True,False,Fake,Report #257
PaintCollections,Paintings,100.0,99.0,True,False,Fake,
PaintCollections,Paintings,100.0,99.0,True,False,,Report #258
PaintCollections,Paintings,100.0,99.0,True,False,,
PaintCollections,Paintings,100.0,99.0,False,True,Genuine,Report #259
PaintCollections,Paintings,100.0,99.0,False,True,Genuine,
PaintCollections,Paintings,100.0,99.0,False,True,Fake,Report #260
PaintCollections,Paintings,100.0,99.0,False,True,Fake,
PaintCollections,Paintings,100.0,99.0,False,True,,Report #261
PaintCollections,Paintings,100.0,99.0,False,True,,
PaintCollections,Paintings,100.0,99.0,False,False,Genuine,Report #262
PaintCollections,Paintings,100.0,99.0,False,False,Genuine,
PaintCollections,Paintings,100.0,99.0,False,False,Fake,Report #263
PaintCollections,Paintings,100.0,99.0,False,False,Fake,
PaintCollections,Paintings,100.0,99.0,False,False,,Report #264
PaintCollections,Paintings,100.0,99.0,False,False,,
PaintCollections,Paintings,100.0,100.0,True,True,Genuine,Report #265
PaintCollections,Paintings,100.0,100.0,True,True,Genuine,
PaintCollections,Paintings,100.0,100.0,True,True,Fake,Report #266
PaintCollections,Paintings,100.0,100.0,True,True,Fake,
PaintCollections,Paintings,100.0,100.0,True,True,,Report #267
PaintCollections,Paintings,100.0,100.0,True,True,,
PaintCollections,Paintings,100.0,100.0,True,False,Genuine,Report #268
PaintCollections,Paintings,100.0,100.0,True,False,Genuine,
PaintCollections,Paintings,100.0,100.0,True,False,Fake,Report #269
PaintCollections,Paintings,100.0,100.0,True,False,Fake,
PaintCollections,Paintings,100.0,100.0,True,False,,Report #270
PaintCollections,Paintings,100.0,100.0,True,False,,
PaintCollections,Paintings,100.0,100.0,False,True,Genuine,Report #271
PaintCollections,Paintings,100.0,100.0,False,True,Genuine,
PaintCollections,Paintings,100.0,100.0,False,True,Fake,Report #272
PaintCollections,Paintings,100.0,100.0,False,True,Fake,
PaintCollections,Paintings,100.0,100.0,False,True,,Report #273
PaintCollections,Paintings,100.0,100.0,False,True,,
PaintCollections,Paintings,100.0,100.0,False,False,Genuine,Report #274
PaintCollections,Paintings,100.0,100.0,False,False,Genuine,
PaintCollections,Paintings,100.0,100.0,False,False,Fake,Report #275
PaintCollections,Paintings,100.0,100.0,False,False,Fake,
PaintCollections,Paintings,100.0,100.0,False,False,,Report #276
PaintCollections,Paintings,100.0,100.0,False,False,,
PaintCollections,Paintings,100.0,101.0,True,True,Genuine,Report #277
PaintCollections,Paintings,100.0,101.0,True,True,Genuine,
PaintCollections,Paintings,100.0,101.0,True,True,Fake,Report #278
PaintCollections,Paintings,100.0,101.0,True,True,Fake,
PaintCollections,Paintings,100.0,101.0,True,True,,Report #279
PaintCollections,Paintings,100.0,101.0,True,True,,
PaintCollections,Paintings,100.0,101.0,True,False,Genuine,Report #280
PaintCollections,Paintings,100.0,101.0,True,False,Genuine,
PaintCollections,Paintings,100.0,101.0,True,False,Fake,Report #281
PaintCollections,Paintings,100.0,101.0,True,False,Fake,
PaintCollections,Paintings,100.0,101.0,True,False,,Report #282
PaintCollections,Paintings,100.0,101.0,True,False,,
PaintCollections,Paintings,100.0,101.0,False,True,Genuine,Report #283
PaintCollections,Paintings,100.0,101.0,False,True,Genuine,
PaintCollections,Paintings,100.0,101.0,False,True,Fake,Report #284
PaintCollections,Paintings,100.0,101.0,False,True,Fake,
PaintCollections,Paintings,100.0,101.0,False,True,,Report #285
PaintCollections,Paintings,100.0,101.0,False,True,,
PaintCollections,Paintings,100.0,101.0,False,False,Genuine,Report #286
PaintCollections,Paintings,100.0,101.0,False,False,Genuine,
PaintCollections,Paintings,100.0,101.0,False,False,Fake,Report #287
PaintCollections,Paintings,100.0,101.0,False,False,Fake,
PaintCollections,Paintings,100.0,101.0,False,False,,Report #288
PaintCollections,Paintings,100.0,101.0,False,False,,
PaintCollections,Paintings,1000.0,-1.0,True,True,Genuine,Report #289
PaintCollections,Paintings,1000.0,-1.0,True,True,Genuine,
PaintCollections,Paintings,1000.0,-1.0,True,True,Fake,Report #290
PaintCollections,Paintings,1000.0,-1.0,True,True,Fake,
PaintCollections,Paintings,1000.0,-1.0,True,True,,Report #291
PaintCollections,Paintings,1000.0,-1.0,True,True,,
PaintCollections,Paintings,1000.0,-1.0,True,False,Genuine,Report #292
PaintCollections,Paintings,1000.0,-1.0,True,False,Genuine,
PaintCollections,Paintings,1000.0,-1.0,True,False,Fake,Report #293
PaintCollections,Paintings,1000.0,-1.0,True,False,Fake,
PaintCollections,Paintings,1000.0,-1.0,True,False,,Report #294
PaintCollections,Paintings,1000.0,-1.0,True,False,,
PaintCollections,Paintings,1000.0,-1.0,False,True,Genuine,Report #295
PaintCollections,Paintings,1000.0,-1.0,False,True,Genuine,
PaintCollections,Paintings,1000.0,-1.0,False,True,Fake,Report #296
PaintCollections,Paintings,1000.0,-1.0,False,True,Fake,
PaintCollections,Paintings,1000.0,-1.0,False,True,,Report #297
PaintCollections,Paintings,1000.0,-1.0,False,True,,
PaintCollections,Paintings,1000.0,-1.0,False,False,Genuine,Report #298
PaintCollections,Paintings,1000.0,-1.0,False,False,Genuine,
PaintCollections,Paintings,1000.0,-1.0,False,False,Fake,Report #299
PaintCollections,Paintings,1000.0,-1.0,False,False,Fake,
PaintCollections,Paintings,1000.0,-1.0,False,False,,Report #300
PaintCollections,Paintings,1000.0,-1.0,False,False,,
PaintCollections,Paintings,1000.0,0.0,True,True,Genuine,Report #301
PaintCollections,Paintings,1000.0,0.0,True,True,Genuine,
PaintCollections,Paintings,1000.0,0.0,True,True,Fake,Report #302
PaintCollections,Paintings,1000.0,0.0,True,True,Fake,
PaintCollections,Paintings,1000.0,0.0,True,True,,Report #303
PaintCollections,Paintings,1000.0,0.0,True,True,,
PaintCollections,Paintings,1000.0,0.0,True,False,Genuine,Report #304
PaintCollections,Paintings,1000.0,0.0,True,False,Genuine,
PaintCollections,Paintings,1000.0,0.0,True,False,Fake,Report #305
PaintCollections,Paintings,1000.0,0.0,True,False,Fake,
PaintCollections,Paintings,1000.0,0.0,True,False,,Report #306
PaintCollections,Paintings,1000.0,0.0,True,False,,
PaintCollections,Paintings,1000.0,0.0,False,True,Genuine,Report #307
PaintCollections,Paintings,1000.0,0.0,False,True,Genuine,
PaintCollections,Paintings,1000.0,0.0,False,True,Fake,Report #308
PaintCollections,Paintings,1000.0,0.0,False,True,Fake,
PaintCollections,Paintings,1000.0,0.0,False,True,,Report #309
PaintCollections,Paintings,1000.0,0.0,False,True,,
PaintCollections,Paintings,1000.0,0.0,False,False,Genuine,Report #310
PaintCollections,Paintings,1000.0,0.0,False,False,Genuine,
PaintCollections,Paintings,1000.0,0.0,False,False,Fake,Report #311
PaintCollections,Paintings,1000.0,0.0,False,False,Fake,
PaintCollections,Paintings,1000.0,0.0,False,False,,Report #312
PaintCollections,Paintings,1000.0,0.0,False,False,,
PaintCollections,Paintings,1000.0,19.0,True,True,Genuine,Report #313
PaintCollections,Paintings,1000.0,19.0,True,True,Genuine,
PaintCollections,Paintings,1000.0,19.0,True,True,Fake,Report #314
PaintCollections,Paintings,1000.0,19.0,True,True,Fake,
PaintCollections,Paintings,1000.0,19.0,True,True,,Report #315
PaintCollections,Paintings,1000.0,19.0,True,True,,
PaintCollections,Paintings,1000.0,19.0,True,False,Genuine,Report #316
PaintCollections,Paintings,1000.0,19.0,True,False,Genuine,
PaintCollections,Paintings,1000.0,19.0,True,False,Fake,Report #317
PaintCollections,Paintings,1000.0,19.0,True,False,Fake,
PaintCollections,Paintings,1000.0,19.0,True,False,,Report #318
PaintCollections,Paintings,1000.0,19.0,True,False,,
PaintCollections,Paintings,1000.0,19.0,False,True,Genuine,Report #319
PaintCollections,Paintings,1000.0,19.0,False,True,Genuine,
PaintCollections,Paintings,1000.0,19.0,False,True,Fake,Report #320
PaintCollections,Paintings,1000.0,19.0,False,True,Fake,
PaintCollections,Paintings,1000.0,19.0,False,True,,Report #321
PaintCollections,Paintings,1000.0,19.0,False,True,,
PaintCollections,Paintings,1000.0,19.0,False,False,Genuine,Report #322
PaintCollections,Paintings,1000.0,19.0,False,False,Genuine,
PaintCollections,Paintings,1000.0,19.0,False,False,Fake,Report #323
PaintCollections,Paintings,1000.0,19.0,False,False,Fake,
PaintCollections,Paintings,1000.0,19.0,False,False,,Report #324
PaintCollections,Paintings,1000.0,19.0,False,False,,
PaintCollections,Paintings,1000.0,20.0,True,True,Genuine,Report #325
PaintCollections,Paintings,1000.0,20.0,True,True,Genuine,
PaintCollections,Paintings,1000.0,20.0,True,True,Fake,Report #326
PaintCollections,Paintings,1000.0,20.0,True,True,Fake,
PaintCollections,Paintings,1000.0,20.0,True,True,,Report #327
PaintCollections,Paintings,1000.0,20.0,True,True,,
PaintCollections,Paintings,1000.0,20.0,True,False,Genuine,Report #328
PaintCollections,Paintings,1000.0,20.0,True,False,Genuine,
PaintCollections,Paintings,1000.0,20.0,True,False,Fake,Report #329
PaintCollections,Paintings,1000.0,20.0,True,False,Fake,
PaintCollections,Paintings,1000.0,20.0,True,False,,Report #330
PaintCollections,Paintings,1000.0,20.0,True,False,,
PaintCollections,Paintings,1000.0,20.0,False,True,Genuine,Report #331
PaintCollections,Paintings,1000.0,20.0,False,True,Genuine,
PaintCollections,Paintings,1000.0,20.0,False,True,Fake,Report #332
PaintCollections,Paintings,1000.0,20.0,False,True,Fake,
PaintCollections,Paintings,1000.0,20.0,False,True,,Report #333
PaintCollections,Paintings,1000.0,20.0,False,True,,
PaintCollections,Paintings,1000.0,20.0,False,False,Genuine,Report #334
PaintCollections,Paintings,1000.0,20.0,False,False,Genuine,
PaintCollections,Paintings,1000.0,20.0,False,False,Fake,Report #335
PaintCollections,Paintings,1000.0,20.0,False,False,Fake,
PaintCollections,Paintings,1000.0,20.0,False,False,,Report #336
PaintCollections,Paintings,1000.0,20.0,False,False,,
PaintCollections,Paintings,1000.0,21.0,True,True,Genuine,Report #337
PaintCollections,Paintings,1000.0,21.0,True,True,Genuine,
PaintCollections,Paintings,1000.0,21.0,True,True,Fake,Report #338
PaintCollections,Paintings,1000.0,21.0,True,True,Fake,
PaintCollections,Paintings,1000.0,21.0,True,True,,Report #339
PaintCollections,Paintings,1000.0,21.0,True,True,,
PaintCollections,Paintings,1000.0,21.0,True,False,Genuine,Report #340
PaintCollections,Paintings,1000.0,21.0,True,False,Genuine,
PaintCollections,Paintings,1000.0,21.0,True,False,Fake,Report #341
PaintCollections,Paintings,1000.0,21.0,True,False,Fake,
PaintCollections,Paintings,1000.0,21.0,True,False,,Report #342
PaintCollections,Paintings,1000.0,21.0,True,False,,
PaintCollections,Paintings,1000.0,21.0,False,True,Genuine,Report #343
PaintCollections,Paintings,1000.0,21.0,False,True,Genuine,
PaintCollections,Paintings,1000.0,21.0,False,True,Fake,Report #344
PaintCollections,Paintings,1000.0,21.0,False,True,Fake,
PaintCollections,Paintings,1000.0,21.0,False,True,,Report #345
PaintCollections,Paintings,1000.0,21.0,False,True,,
PaintCollections,Paintings,1000.0,21.0,False,False,Genuine,Report #346
PaintCollections,Paintings,1000.0,21.0,False,False,Genuine,
PaintCollections,Paintings,1000.0,21.0,False,False,Fake,Report #347
PaintCollections,Paintings,1000.0,21.0,False,False,Fake,
PaintCollections,Paintings,1000.0,21.0,False,False,,Report #348
PaintCollections,Paintings,1000.0,21.0,False,False,,
PaintCollections,Paintings,1000.0,99.0,True,True,Genuine,Report #349
PaintCollections,Paintings,1000.0,99.0,True,True,Genuine,
PaintCollections,Paintings,1000.0,99.0,True,True,Fake,Report #350
PaintCollections,Paintings,1000.0,99.0,True,True,Fake,
PaintCollections,Paintings,1000.0,99.0,True,True,,Report #351
PaintCollections,Paintings,1000.0,99.0,True,True,,
PaintCollections,Paintings,1000.0,99.0,True,False,Genuine,Report #352
PaintCollections,Paintings,1000.0,99.0,True,False,Genuine,
PaintCollections,Paintings,1000.0,99.0,True,False,Fake,Report #353
PaintCollections,Paintings,1000.0,99.0,True,False,Fake,
PaintCollections,Paintings,1000.0,99.0,True,False,,Report #354
PaintCollections,Paintings,1000.0,99.0,True,False,,
PaintCollections,Paintings,1000.0,99.0,False,True,Genuine,Report #355
PaintCollections,Paintings,1000.0,99.0,False,True,Genuine,
PaintCollections,Paintings,1000.0,99.0,False,True,Fake,Report #356
PaintCollections,Paintings,1000.0,99.0,False,True,Fake,
PaintCollections,Paintings,1000.0,99.0,False,True,,Report #357
PaintCollections,Paintings,1000.0,99.0,False,True,,
PaintCollections,Paintings,1000.0,99.0,False,False,Genuine,Report #358
PaintCollections,Paintings,1000.0,99.0,False,False,Genuine,
PaintCollections,Paintings,1000.0,99.0,False,False,Fake,Report #359
PaintCollections,Paintings,1000.0,99.0,False,False,Fake,
PaintCollections,Paintings,1000.0,99.0,False,False,,Report #360
PaintCollections,Paintings,1000.0,99.0,False,False,,
PaintCollections,Paintings,1000.0,100.0,True,True,Genuine,Report #361
PaintCollections,Paintings,1000.0,100.0,True,True,Genuine,
PaintCollections,Paintings,1000.0,100.0,True,True,Fake,Report #362
PaintCollections,Paintings,1000.0,100.0,True,True,Fake,
PaintCollections,Paintings,1000.0,100.0,True,True,,Report #363
PaintCollections,Paintings,1000.0,100.0,True,True,,
PaintCollections,Paintings,1000.0,100.0,True,False,Genuine,Report #364
PaintCollections,Paintings,1000.0,100.0,True,False,Genuine,
PaintCollections,Paintings,1000.0,100.0,True,False,Fake,Report #365
PaintCollections,Paintings,1000.0,100.0,True,False,Fake,
PaintCollections,Paintings,1000.0,100.0,True,False,,Report #366
PaintCollections,Paintings,1000.0,100.0,True,False,,
PaintCollections,Paintings,1000.0,100.0,False,True,Genuine,Report #367
PaintCollections,Paintings,1000.0,100.0,False,True,Genuine,
PaintCollections,Paintings,1000.0,100.0,False,True,Fake,Report #368
PaintCollections,Paintings,1000.0,100.0,False,True,Fake,
PaintCollections,Paintings,1000.0,100.0,False,True,,Report #369
PaintCollections,Paintings,1000.0,100.0,False,True,,
PaintCollections,Paintings,1000.0,100.0,False,False,Genuine,Report #370
PaintCollections,Paintings,1000.0,100.0,False,False,Genuine,
PaintCollections,Paintings,1000.0,100.0,False,False,Fake,Report #371
PaintCollections,Paintings,1000.0,100.0,False,False,Fake,
PaintCollections,Paintings,1000.0,100.0,False,False,,Report #372
PaintCollections,Paintings,1000.0,100.0,False,False,,
PaintCollections,Paintings,1000.0,101.0,True,True,Genuine,Report #373
PaintCollections,Paintings,1000.0,101.0,True,True,Genuine,
PaintCollections,Paintings,1000.0,101.0,True,True,Fake,Report #374
PaintCollections,Paintings,1000.0,101.0,True,True,Fake,
PaintCollections,Paintings,1000.0,101.0,True,True,,Report #375
PaintCollections,Paintings,1000.0,101.0,True,True,,
PaintCollections,Paintings,1000.0,101.0,True,False,Genuine,Report #376
PaintCollections,Paintings,1000.0,101.0,True,False,Genuine,
PaintCollections,Paintings,1000.0,101.0,True,False,Fake,Report #377
PaintCollections,Paintings,1000.0,101.0,True,False,Fake,
PaintCollections,Paintings,1000.0,101.0,True,False,,Report #378
PaintCollections,Paintings,1000.0,101.0,True,False,,
PaintCollections,Paintings,1000.0,101.0,False,True,Genuine,Report #379
PaintCollections,Paintings,1000.0,101.0,False,True,Genuine,
PaintCollections,Paintings,1000.0,101.0,False,True,Fake,Report #380
PaintCollections,Paintings,1000.0,101.0,False,True,Fake,
PaintCollections,Paintings,1000.0,101.0,False,True,,Report #381
PaintCollections,Paintings,1000.0,101.0,False,True,,
PaintCollections,Paintings,1000.0,101.0,False,False,Genuine,Report #382
PaintCollections,Paintings,1000.0,101.0,False,False,Genuine,
PaintCollections,Paintings,1000.0,101.0,False,False,Fake,Report #383
PaintCollections,Paintings,1000.0,101.0,False,False,Fake,
PaintCollections,Paintings,1000.0,101.0,False,False,,Report #384
PaintCollections,Paintings,1000.0,101.0,False,False,,
PaintCollections,Paintings,1000000.0,-1.0,True,True,Genuine,Report #385
PaintCollections,Paintings,1000000.0,-1.0,True,True,Genuine,
PaintCollections,Paintings,1000000.0,-1.0,True,True,Fake,Report #386
PaintCollections,Paintings,1000000.0,-1.0,True,True,Fake,
PaintCollections,Paintings,1000000.0,-1.0,True,True,,Report #387
PaintCollections,Paintings,1000000.0,-1.0,True,True,,
PaintCollections,Paintings,1000000.0,-1.0,True,False,Genuine,Report #388
PaintCollections,Paintings,1000000.0,-1.0,True,False,Genuine,
PaintCollections,Paintings,1000000.0,-1.0,True,False,Fake,Report #389
PaintCollections,Paintings,1000000.0,-1.0,True,False,Fake,
PaintCollections,Paintings,1000000.0,-1.0,True,False,,Report #390
PaintCollections,Paintings,1000000.0,-1.0,True,False,,
PaintCollections,Paintings,1000000.0,-1.0,False,True,Genuine,Report #391
PaintCollections,Paintings,1000000.0,-1.0,False,True,Genuine,
PaintCollections,Paintings,1000000.0,-1.0,False,True,Fake,Report #392
PaintCollections,Paintings,1000000.0,-1.0,False,True,Fake,
PaintCollections,Paintings,1000000.0,-1.0,False,True,,Report #393
PaintCollections,Paintings,1000000.0,-1.0,False,True,,
PaintCollections,Paintings,1000000.0,-1.0,False,False,Genuine,Report #394
PaintCollections,Paintings,1000000.0,-1.0,False,False,Genuine,
PaintCollections,Paintings,1000000.0,-1.0,False,False,Fake,Report #395
PaintCollections,Paintings,1000000.0,-1.0,False,False,Fake,
PaintCollections,Paintings,1000000.0,-1.0,False,False,,Report #396
PaintCollections,Paintings,1000000.0,-1.0,False,False,,
PaintCollections,Paintings,1000000.0,0.0,True,True,Genuine,Report #397
PaintCollections,Paintings,1000000.0,0.0,True,True,Genuine,
PaintCollections,Paintings,1000000.0,0.0,True,True,Fake,Report #398
PaintCollections,Paintings,1000000.0,0.0,True,True,Fake,
PaintCollections,Paintings,1000000.0,0.0,True,True,,Report #399
PaintCollections,Paintings,1000000.0,0.0,True,True,,
PaintCollections,Paintings,1000000.0,0.0,True,False,Genuine,Report #400
PaintCollections,Paintings,1000000.0,0.0,True,False,Genuine,
PaintCollections,Paintings,1000000.0,0.0,True,False,Fake,Report #401
PaintCollections,Paintings,1000000.0,0.0,True,False,Fake,
PaintCollections,Paintings,1000000.0,0.0,True,False,,Report #402
PaintCollections,Paintings,1000000.0,0.0,True,False,,
PaintCollections,Paintings,1000000.0,0.0,False,True,Genuine,Report #403
PaintCollections,Paintings,1000000.0,0.0,False,True,Genuine,
PaintCollections,Paintings,1000000.0,0.0,False,True,Fake,Report #404
PaintCollections,Paintings,1000000.0,0.0,False,True,Fake,
PaintCollections,Paintings,1000000.0,0.0,False,True,,Report #405
PaintCollections,Paintings,1000000.0,0.0,False,True,,
PaintCollections,Paintings,1000000.0,0.0,False,False,Genuine,Report #406
PaintCollections,Paintings,1000000.0,0.0,False,False,Genuine,
PaintCollections,Paintings,1000000.0,0.0,False,False,Fake,Report #407
PaintCollections,Paintings,1000000.0,0.0,False,False,Fake,
PaintCollections,Paintings,1000000.0,0.0,False,False,,Report #408
PaintCollections,Paintings,1000000.0,0.0,False,False,,
PaintCollections,Paintings,1000000.0,19.0,True,True,Genuine,Report #409
PaintCollections,Paintings,1000000.0,19.0,True,True,Genuine,
PaintCollections,Paintings,1000000.0,19.0,True,True,Fake,Report #410
PaintCollections,Paintings,1000000.0,19.0,True,True,Fake,
PaintCollections,Paintings,1000000.0,19.0,True,True,,Report #411
PaintCollections,Paintings,1000000.0,19.0,True,True,,
PaintCollections,Paintings,1000000.0,19.0,True,False,Genuine,Report #412
PaintCollections,Paintings,1000000.0,19.0,True,False,Genuine,
PaintCollections,Paintings,1000000.0,19.0,True,False,Fake,Report #413
PaintCollections,Paintings,1000000.0,19.0,True,False,Fake,
PaintCollections,Paintings,1000000.0,19.0,True,False,,Report #414
PaintCollections,Paintings,1000000.0,19.0,True,False,,
PaintCollections,Paintings,1000000.0,19.0,False,True,Genuine,Report #415
PaintCollections,Paintings,1000000.0,19.0,False,True,Genuine,
PaintCollections,Paintings,1000000.0,19.0,False,True,Fake,Report #416
PaintCollections,Paintings,1000000.0,19.0,False,True,Fake,
PaintCollections,Paintings,1000000.0,19.0,False,True,,Report #417
PaintCollections,Paintings,1000000.0,19.0,False,True,,
PaintCollections,Paintings,1000000.0,19.0,False,False,Genuine,Report #418
PaintCollections,Paintings,1000000.0,19.0,False,False,Genuine,
PaintCollections,Paintings,1000000.0,19.0,False,False,Fake,Report #419
PaintCollections,Paintings,1000000.0,19.0,False,False,Fake,
PaintCollections,Paintings,1000000.0,19.0,False,False,,Report #420
PaintCollections,Paintings,1000000.0,19.0,False,False,,
PaintCollections,Paintings,1000000.0,20.0,True,True,Genuine,Report #421
PaintCollections,Paintings,1000000.0,20.0,True,True,Genuine,
PaintCollections,Paintings,1000000.0,20.0,True,True,Fake,Report #422
PaintCollections,Paintings,1000000.0,20.0,True,True,Fake,
PaintCollections,Paintings,1000000.0,20.0,True,True,,Report #423
PaintCollections,Paintings,1000000.0,20.0,True,True,,
PaintCollections,Paintings,1000000.0,20.0,True,False,Genuine,Report #424
PaintCollections,Paintings,1000000.0,20.0,True,False,Genuine,
PaintCollections,Paintings,1000000.0,20.0,True,False,Fake,Report #425
PaintCollections,Paintings,1000000.0,20.0,True,False,Fake,
PaintCollections,Paintings,1000000.0,20.0,True,False,,Report #426
PaintCollections,Paintings,1000000.0,20.0,True,False,,
PaintCollections,Paintings,1000000.0,20.0,False,True,Genuine,Report #427
PaintCollections,Paintings,1000000.0,20.0,False,True,Genuine,
PaintCollections,Paintings,1000000.0,20.0,False,True,Fake,Report #428
PaintCollections,Paintings,1000000.0,20.0,False,True,Fake,
PaintCollections,Paintings,1000000.0,20.0,False,True,,Report #429
PaintCollections,Paintings,1000000.0,20.0,False,True,,
PaintCollections,Paintings,1000000.0,20.0,False,False,Genuine,Report #430
PaintCollections,Paintings,1000000.0,20.0,False,False,Genuine,
PaintCollections,Paintings,1000000.0,20.0,False,False,Fake,Report #431
PaintCollections,Paintings,1000000.0,20.0,False,False,Fake,
PaintCollections,Paintings,1000000.0,20.0,False,False,,Report #432
PaintCollections,Paintings,1000000.0,20.0,False,False,,
PaintCollections,Paintings,1000000.0,21.0,True,True,Genuine,Report #433
PaintCollections,Paintings,1000000.0,21.0,True,True,Genuine,
PaintCollections,Paintings,1000000.0,21.0,True,True,Fake,Report #434
PaintCollections,Paintings,1000000.0,21.0,True,True,Fake,
PaintCollections,Paintings,1000000.0,21.0,True,True,,Report #435
PaintCollections,Paintings,1000000.0,21.0,True,True,,
PaintCollections,Paintings,1000000.0,21.0,True,False,Genuine,Report #436
PaintCollections,Paintings,1000000.0,21.0,True,False,Genuine,
PaintCollections,Paintings,1000000.0,21.0,True,False,Fake,Report #437
PaintCollections,Paintings,1000000.0,21.0,True,False,Fake,
PaintCollections,Paintings,1000000.0,21.0,True,False,,Report #438
PaintCollections,Paintings,1000000.0,21.0,True,False,,
PaintCollections,Paintings,1000000.0,21.0,False,True,Genuine,Report #439
PaintCollections,Paintings,1000000.0,21.0,False,True,Genuine,
PaintCollections,Paintings,1000000.0,21.0,False,True,Fake,Report #440
PaintCollections,Paintings,1000000.0,21.0,False,True,Fake,
PaintCollections,Paintings,1000000.0,21.0,False,True,,Report #441
PaintCollections,Paintings,1000000.0,21.0,False,True,,
PaintCollections,Paintings,1000000.0,21.0,False,False,Genuine,Report #442
PaintCollections,Paintings,1000000.0,21.0,False,False,Genuine,
PaintCollections,Paintings,1000000.0,21.0,False,False,Fake,Report #443
PaintCollections,Paintings,1000000.0,21.0,False,False,Fake,
PaintCollections,Paintings,1000000.0,21.0,False,False,,Report #444
PaintCollections,Paintings,1000000.0,21.0,False,False,,
PaintCollections,Paintings,1000000.0,99.0,True,True,Genuine,Report #445
PaintCollections,Paintings,1000000.0,99.0,True,True,Genuine,
PaintCollections,Paintings,1000000.0,99.0,True,True,Fake,Report #446
PaintCollections,Paintings,1000000.0,99.0,True,True,Fake,
PaintCollections,Paintings,1000000.0,99.0,True,True,,Report #447
PaintCollections,Paintings,1000000.0,99.0,True,True,,
PaintCollections,Paintings,1000000.0,99.0,True,False,Genuine,Report #448
PaintCollections,Paintings,1000000.0,99.0,True,False,Genuine,
PaintCollections,Paintings,1000000.0,99.0,True,False,Fake,Report #449
PaintCollections,Paintings,1000000.0,99.0,True,False,Fake,
PaintCollections,Paintings,1000000.0,99.0,True,False,,Report #450
PaintCollections,Paintings,1000000.0,99.0,True,False,,
PaintCollections,Paintings,1000000.0,99.0,False,True,Genuine,Report #451
PaintCollections,Paintings,1000000.0,99.0,False,True,Genuine,
PaintCollections,Paintings,1000000.0,99.0,False,True,Fake,Report #452
PaintCollections,Paintings,1000000.0,99.0,False,True,Fake,
PaintCollections,Paintings,1000000.0,99.0,False,True,,Report #453
PaintCollections,Paintings,1000000.0,99.0,False,True,,
PaintCollections,Paintings,1000000.0,99.0,False,False,Genuine,Report #454
PaintCollections,Paintings,1000000.0,99.0,False,False,Genuine,
PaintCollections,Paintings,1000000.0,99.0,False,False,Fake,Report #455
PaintCollections,Paintings,1000000.0,99.0,False,False,Fake,
PaintCollections,Paintings,1000000.0,99.0,False,False,,Report #456
PaintCollections,Paintings,1000000.0,99.0,False,False,,
PaintCollections,Paintings,1000000.0,100.0,True,True,Genuine,Report #457
PaintCollections,Paintings,1000000.0,100.0,True,True,Genuine,
PaintCollections,Paintings,1000000.0,100.0,True,True,Fake,Report #458
PaintCollections,Paintings,1000000.0,100.0,True,True,Fake,
PaintCollections,Paintings,1000000.0,100.0,True,True,,Report #459
PaintCollections,Paintings,1000000.0,100.0,True,True,,
PaintCollections,Paintings,1000000.0,100.0,True,False,Genuine,Report #460
PaintCollections,Paintings,1000000.0,100.0,True,False,Genuine,
PaintCollections,Paintings,1000000.0,100.0,True,False,Fake,Report #461
PaintCollections,Paintings,1000000.0,100.0,True,False,Fake,
PaintCollections,Paintings,1000000.0,100.0,True,False,,Report #462
PaintCollections,Paintings,1000000.0,100.0,True,False,,
PaintCollections,Paintings,1000000.0,100.0,False,True,Genuine,Report #463
PaintCollections,Paintings,1000000.0,100.0,False,True,Genuine,
PaintCollections,Paintings,1000000.0,100.0,False,True,Fake,Report #464
PaintCollections,Paintings,1000000.0,100.0,False,True,Fake,
PaintCollections,Paintings,1000000.0,100.0,False,True,,Report #465
PaintCollections,Paintings,1000000.0,100.0,False,True,,
PaintCollections,Paintings,1000000.0,100.0,False,False,Genuine,Report #466
PaintCollections,Paintings,1000000.0,100.0,False,False,Genuine,
PaintCollections,Paintings,1000000.0,100.0,False,False,Fake,Report #467
PaintCollections,Paintings,1000000.0,100.0,False,False,Fake,
PaintCollections,Paintings,1000000.0,100.0,False,False,,Report #468
PaintCollections,Paintings,1000000.0,100.0,False,False,,
PaintCollections,Paintings,1000000.0,101.0,True,True,Genuine,Report #469
PaintCollections,Paintings,1000000.0,101.0,True,True,Genuine,
PaintCollections,Paintings,1000000.0,101.0,True,True,Fake,Report #470
PaintCollections,Paintings,1000000.0,101.0,True,True,Fake,
PaintCollections,Paintings,1000000.0,101.0,True,True,,Report #471
PaintCollections,Paintings,1000000.0,101.0,True,True,,
PaintCollections,Paintings,1000000.0,101.0,True,False,Genuine,Report #472
PaintCollections,Paintings,1000000.0,101.0,True,False,Genuine,
PaintCollections,Paintings,1000000.0,101.0,True,False,Fake,Report #473
PaintCollections,Paintings,1000000.0,101.0,True,False,Fake,
PaintCollections,Paintings,1000000.0,101.0,True,False,,Report #474
PaintCollections,Paintings,1000000.0,101.0,True,False,,
PaintCollections,Paintings,1000000.0,101.0,False,True,Genuine,Report #475
PaintCollections,Paintings,1000000.0,101.0,False,True,Genuine,
PaintCollections,Paintings,1000000.0,101.0,False,True,Fake,Report #476
PaintCollections,Paintings,1000000.0,101.0,False,True,Fake,
PaintCollections,Paintings,1000000.0,101.0,False,True,,Report #477
PaintCollections,Paintings,1000000.0,101.0,False,True,,
PaintCollections,Paintings,1000000.0,101.0,False,False,Genuine,Report #478
PaintCollections,Paintings,1000000.0,101.0,False,False,Genuine,
PaintCollections,Paintings,1000000.0,101.0,False,False,Fake,Report #479
PaintCollections,Paintings,1000000.0,101.0,False,False,Fake,
PaintCollections,Paintings,1000000.0,101.0,False,False,,Report #480
PaintCollections,Paintings,1000000.0,101.0,False,False,,
PaintCollections,Knife,0.0,-1.0,True,True,Genuine,Report #481
PaintCollections,Knife,0.0,-1.0,True,True,Genuine,
PaintCollections,Knife,0.0,-1.0,True,True,Fake,Report #482
PaintCollections,Knife,0.0,-1.0,True,True,Fake,
PaintCollections,Knife,0.0,-1.0,True,True,,Report #483
PaintCollections,Knife,0.0,-1.0,True,True,,
PaintCollections,Knife,0.0,-1.0,True,False,Genuine,Report #484
PaintCollections,Knife,0.0,-1.0,True,False,Genuine,
PaintCollections,Knife,0.0,-1.0,True,False,Fake,Report #485
PaintCollections,Knife,0.0,-1.0,True,False,Fake,
PaintCollections,Knife,0.0,-1.0,True,False,,Report #486
PaintCollections,Knife,0.0,-1.0,True,False,,
PaintCollections,Knife,0.0,-1.0,False,True,Genuine,Report #487
PaintCollections,Knife,0.0,-1.0,False,True,Genuine,
PaintCollections,Knife,0.0,-1.0,False,True,Fake,Report #488
PaintCollections,Knife,0.0,-1.0,False,True,Fake,
PaintCollections,Knife,0.0,-1.0,False,True,,Report #489
PaintCollections,Knife,0.0,-1.0,False,True,,
PaintCollections,Knife,0.0,-1.0,False,False,Genuine,Report #490
PaintCollections,Knife,0.0,-1.0,False,False,Genuine,
PaintCollections,Knife,0.0,-1.0,False,False,Fake,Report #491
PaintCollections,Knife,0.0,-1.0,False,False,Fake,
PaintCollections,Knife,0.0,-1.0,False,False,,Report #492
PaintCollections,Knife,0.0,-1.0,False,False,,
PaintCollections,Knife,0.0,0.0,True,True,Genuine,Report #493
PaintCollections,Knife,0.0,0.0,True,True,Genuine,
PaintCollections,Knife,0.0,0.0,True,True,Fake,Report #494
PaintCollections,Knife,0.0,0.0,True,True,Fake,
PaintCollections,Knife,0.0,0.0,True,True,,Report #495
PaintCollections,Knife,0.0,0.0,True,True,,
PaintCollections,Knife,0.0,0.0,True,False,Genuine,Report #496
PaintCollections,Knife,0.0,0.0,True,False,Genuine,
PaintCollections,Knife,0.0,0.0,True,False,Fake,Report #497
PaintCollections,Knife,0.0,0.0,True,False,Fake,
PaintCollections,Knife,0.0,0.0,True,False,,Report #498
PaintCollections,Knife,0.0,0.0,True,False,,
PaintCollections,Knife,0.0,0.0,False,True,Genuine,Report #499
PaintCollections,Knife,0.0,0.0,False,True,Genuine,
PaintCollections,Knife,0.0,0.0,False,True,Fake,Report #500
PaintCollections,Knife,0.0,0.0,False,True,Fake,
PaintCollections,Knife,0.0,0.0,False,True,,Report #501
PaintCollections,Knife,0.0,0.0,False,True,,
PaintCollections,Knife,0.0,0.0,False,False,Genuine,Report #502
PaintCollections,Knife,0.0,0.0,False,False,Genuine,
PaintCollections,Knife,0.0,0.0,False,False,Fake,Report #503
PaintCollections,Knife,0.0,0.0,False,False,Fake,
PaintCollections,Knife,0.0,0.0,False,False,,Report #504
PaintCollections,Knife,0.0,0.0,False,False,,
PaintCollections,Knife,0.0,19.0,True,True,Genuine,Report #505
PaintCollections,Knife,0.0,19.0,True,True,Genuine,
PaintCollections,Knife,0.0,19.0,True,True,Fake,Report #506
PaintCollections,Knife,0.0,19.0,True,True,Fake,
PaintCollections,Knife,0.0,19.0,True,True,,Report #507
PaintCollections,Knife,0.0,19.0,True,True,,
PaintCollections,Knife,0.0,19.0,True,False,Genuine,Report #508
PaintCollections,Knife,0.0,19.0,True,False,Genuine,
PaintCollections,Knife,0.0,19.0,True,False,Fake,Report #509
PaintCollections,Knife,0.0,19.0,True,False,Fake,
PaintCollections,Knife,0.0,19.0,True,False,,Report #510
PaintCollections,Knife,0.0,19.0,True,False,,
PaintCollections,Knife,0.0,19.0,False,True,Genuine,Report #511
PaintCollections,Knife,0.0,19.0,False,True,Genuine,
PaintCollections,Knife,0.0,19.0,False,True,Fake,Report #512
PaintCollections,Knife,0.0,19.0,False,True,Fake,
PaintCollections,Knife,0.0,19.0,False,True,,Report #513
PaintCollections,Knife,0.0,19.0,False,True,,
PaintCollections,Knife,0.0,19.0,False,False,Genuine,Report #514
PaintCollections,Knife,0.0,19.0,False,False,Genuine,
PaintCollections,Knife,0.0,19.0,False,False,Fake,Report #515
PaintCollections,Knife,0.0,19.0,False,False,Fake,
PaintCollections,Knife,0.0,19.0,False,False,,Report #516
PaintCollections,Knife,0.0,19.0,False,False,,
PaintCollections,Knife,0.0,20.0,True,True,Genuine,Report #517
PaintCollections,Knife,0.0,20.0,True,True,Genuine,
PaintCollections,Knife,0.0,20.0,True,True,Fake,Report #518
PaintCollections,Knife,0.0,20.0,True,True,Fake,
PaintCollections,Knife,0.0,20.0,True,True,,Report #519
PaintCollections,Knife,0.0,20.0,True,True,,
PaintCollections,Knife,0.0,20.0,True,False,Genuine,Report #520
PaintCollections,Knife,0.0,20.0,True,False,Genuine,
PaintCollections,Knife,0.0,20.0,True,False,Fake,Report #521
PaintCollections,Knife,0.0,20.0,True,False,Fake,
PaintCollections,Knife,0.0,20.0,True,False,,Report #522
PaintCollections,Knife,0.0,20.0,True,False,,
PaintCollections,Knife,0.0,20.0,False,True,Genuine,Report #523
PaintCollections,Knife,0.0,20.0,False,True,Genuine,
PaintCollections,Knife,0.0,20.0,False,True,Fake,Report #524
PaintCollections,Knife,0.0,20.0,False,True,Fake,
PaintCollections,Knife,0.0,20.0,False,True,,Report #525
PaintCollections,Knife,0.0,20.0,False,True,,
PaintCollections,Knife,0.0,20.0,False,False,Genuine,Report #526
PaintCollections,Knife,0.0,20.0,False,False,Genuine,
PaintCollections,Knife,0.0,20.0,False,False,Fake,Report #527
PaintCollections,Knife,0.0,20.0,False,False,Fake,
PaintCollections,Knife,0.0,20.0,False,False,,Report #528
PaintCollections,Knife,0.0,20.0,False,False,,
PaintCollections,Knife,0.0,21.0,True,True,Genuine,Report #529
PaintCollections,Knife,0.0,21.0,True,True,Genuine,
PaintCollections,Knife,0.0,21.0,True,True,Fake,Report #530
PaintCollections,Knife,0.0,21.0,True,True,Fake,
PaintCollections,Knife,0.0,21.0,True,True,,Report #531
PaintCollections,Knife,0.0,21.0,True,True,,
PaintCollections,Knife,0.0,21.0,True,False,Genuine,Report #532
PaintCollections,Knife,0.0,21.0,True,False,Genuine,
PaintCollections,Knife,0.0,21.0,True,False,Fake,Report #533
PaintCollections,Knife,0.0,21.0,True,False,Fake,
PaintCollections,Knife,0.0,21.0,True,False,,Report #534
PaintCollections,Knife,0.0,21.0,True,False,,
PaintCollections,Knife,0.0,21.0,False,True,Genuine,Report #535
PaintCollections,Knife,0.0,21.0,False,True,Genuine,
PaintCollections,Knife,0.0,21.0,False,True,Fake,Report #536
PaintCollections,Knife,0.0,21.0,False,True,Fake,
PaintCollections,Knife,0.0,21.0,False,True,,Report #537
PaintCollections,Knife,0.0,21.0,False,True,,
PaintCollections,Knife,0.0,21.0,False,False,Genuine,Report #538
PaintCollections,Knife,0.0,21.0,False,False,Genuine,
PaintCollections,Knife,0.0,21.0,False,False,Fake,Report #539
PaintCollections,Knife,0.0,21.0,False,False,Fake,
PaintCollections,Knife,0.0,21.0,False,False,,Report #540
PaintCollections,Knife,0.0,21.0,False,False,,
PaintCollections,Knife,0.0,99.0,True,True,Genuine,Report #541
PaintCollections,Knife,0.0,99.0,True,True,Genuine,
PaintCollections,Knife,0.0,99.0,True,True,Fake,Report #542
PaintCollections,Knife,0.0,99.0,True,True,Fake,
PaintCollections,Knife,0.0,99.0,True,True,,Report #543
PaintCollections,Knife,0.0,99.0,True,True,,
PaintCollections,Knife,0.0,99.0,True,False,Genuine,Report #544
PaintCollections,Knife,0.0,99.0,True,False,Genuine,
PaintCollections,Knife,0.0,99.0,True,False,Fake,Report #545
PaintCollections,Knife,0.0,99.0,True,False,Fake,
PaintCollections,Knife,0.0,99.0,True,False,,Report #546
PaintCollections,Knife,0.0,99.0,True,False,,
PaintCollections,Knife,0.0,99.0,False,True,Genuine,Report #547
PaintCollections,Knife,0.0,99.0,False,True,Genuine,
PaintCollections,Knife,0.0,99.0,False,True,Fake,Report #548
PaintCollections,Knife,0.0,99.0,False,True,Fake,
PaintCollections,Knife,0.0,99.0,False,True,,Report #549
PaintCollections,Knife,0.0,99.0,False,True,,
PaintCollections,Knife,0.0,99.0,False,False,Genuine,Report #550
PaintCollections,Knife,0.0,99.0,False,False,Genuine,
PaintCollections,Knife,0.0,99.0,False,False,Fake,Report #551
PaintCollections,Knife,0.0,99.0,False,False,Fake,
PaintCollections,Knife,0.0,99.0,False,False,,Report #552
PaintCollections,Knife,0.0,99.0,False,False,,
PaintCollections,Knife,0.0,100.0,True,True,Genuine,Report #553
PaintCollections,Knife,0.0,100.0,True,True,Genuine,
PaintCollections,Knife,0.0,100.0,True,True,Fake,Report #554
PaintCollections,Knife,0.0,100.0,True,True,Fake,
PaintCollections,Knife,0.0,100.0,True,True,,Report #555
PaintCollections,Knife,0.0,100.0,True,True,,
PaintCollections,Knife,0.0,100.0,True,False,Genuine,Report #556
PaintCollections,Knife,0.0,100.0,True,False,Genuine,
PaintCollections,Knife,0.0,100.0,True,False,Fake,Report #557
PaintCollections,Knife,0.0,100.0,True,False,Fake,
PaintCollections,Knife,0.0,100.0,True,False,,Report #558
PaintCollections,Knife,0.0,100.0,True,False,,
PaintCollections,Knife,0.0,100.0,False,True,Genuine,Report #559
PaintCollections,Knife,0.0,100.0,False,True,Genuine,
PaintCollections,Knife,0.0,100.0,False,True,Fake,Report #560
PaintCollections,Knife,0.0,100.0,False,True,Fake,
PaintCollections,Knife,0.0,100.0,False,True,,Report #561
PaintCollections,Knife,0.0,100.0,False,True,,
PaintCollections,Knife,0.0,100.0,False,False,Genuine,Report #562
PaintCollections,Knife,0.0,100.0,False,False,Genuine,
PaintCollections,Knife,0.0,100.0,False,False,Fake,Report #563
PaintCollections,Knife,0.0,100.0,False,False,Fake,
PaintCollections,Knife,0.0,100.0,False,False,,Report #564
PaintCollections,Knife,0.0,100.0,False,False,,
PaintCollections,Knife,0.0,101.0,True,True,Genuine,Report #565
PaintCollections,Knife,0.0,101.0,True,True,Genuine,
PaintCollections,Knife,0.0,101.0,True,True,Fake,Report #566
PaintCollections,Knife,0.0,101.0,True,True,Fake,
PaintCollections,Knife,0.0,101.0,True,True,,Report #567
PaintCollections,Knife,0.0,101.0,True,True,,
PaintCollections,Knife,0.0,101.0,True,False,Genuine,Report #568
PaintCollections,Knife,0.0,101.0,True,False,Genuine,
PaintCollections,Knife,0.0,101.0,True,False,Fake,Report #569
PaintCollections,Knife,0.0,101.0,True,False,Fake,
PaintCollections,Knife,0.0,101.0,True,False,,Report #570
PaintCollections,Knife,0.0,101.0,True,False,,
PaintCollections,Knife,0.0,101.0,False,True,Genuine,Report #571
PaintCollections,Knife,0.0,101.0,False,True,Genuine,
PaintCollections,Knife,0.0,101.0,False,True,Fake,Report #572
PaintCollections,Knife,0.0,101.0,False,True,Fake,
PaintCollections,Knife,0.0,101.0,False,True,,Report #573
PaintCollections,Knife,0.0,101.0,False,True,,
PaintCollections,Knife,0.0,101.0,False,False,Genuine,Report #574
PaintCollections,Knife,0.0,101.0,False,False,Genuine,
PaintCollections,Knife,0.0,101.0,False,False,Fake,Report #575
PaintCollections,Knife,0.0,101.0,False,False,Fake,
PaintCollections,Knife,0.0,101.0,False,False,,Report #576
PaintCollections,Knife,0.0,101.0,False,False,,
PaintCollections,Knife,1.0,-1.0,True,True,Genuine,Report #577
PaintCollections,Knife,1.0,-1.0,True,True,Genuine,
PaintCollections,Knife,1.0,-1.0,True,True,Fake,Report #578
PaintCollections,Knife,1.0,-1.0,True,True,Fake,
PaintCollections,Knife,1.0,-1.0,True,True,,Report #579
PaintCollections,Knife,1.0,-1.0,True,True,,
PaintCollections,Knife,1.0,-1.0,True,False,Genuine,Report #580
PaintCollections,Knife,1.0,-1.0,True,False,Genuine,
PaintCollections,Knife,1.0,-1.0,True,False,Fake,Report #581
PaintCollections,Knife,1.0,-1.0,True,False,Fake,
PaintCollections,Knife,1.0,-1.0,True,False,,Report #582
PaintCollections,Knife,1.0,-1.0,True,False,,
PaintCollections,Knife,1.0,-1.0,False,True,Genuine,Report #583
PaintCollections,Knife,1.0,-1.0,False,True,Genuine,
PaintCollections,Knife,1.0,-1.0,False,True,Fake,Report #584
PaintCollections,Knife,1.0,-1.0,False,True,Fake,
PaintCollections,Knife,1.0,-1.0,False,True,,Report #585
PaintCollections,Knife,1.0,-1.0,False,True,,
PaintCollections,Knife,1.0,-1.0,False,False,Genuine,Report #586
PaintCollections,Knife,1.0,-1.0,False,False,Genuine,
PaintCollections,Knife,1.0,-1.0,False,False,Fake,Report #587
PaintCollections,Knife,1.0,-1.0,False,False,Fake,
PaintCollections,Knife,1.0,-1.0,False,False,,Report #588
PaintCollections,Knife,1.0,-1.0,False,False,,
PaintCollections,Knife,1.0,0.0,True,True,Genuine,Report #589
PaintCollections,Knife,1.0,0.0,True,True,Genuine,
PaintCollections,Knife,1.0,0.0,True,True,Fake,Report #590
PaintCollections,Knife,1.0,0.0,True,True,Fake,
PaintCollections,Knife,1.0,0.0,True,True,,Report #591
PaintCollections,Knife,1.0,0.0,True,True,,
PaintCollections,Knife,1.0,0.0,True,False,Genuine,Report #592
PaintCollections,Knife,1.0,0.0,True,False,Genuine,
PaintCollections,Knife,1.0,0.0,True,False,Fake,Report #593
PaintCollections,Knife,1.0,0.0,True,False,Fake,
PaintCollections,Knife,1.0,0.0,True,False,,Report #594
PaintCollections,Knife,1.0,0.0,True,False,,
PaintCollections,Knife,1.0,0.0,False,True,Genuine,Report #595
PaintCollections,Knife,1.0,0.0,False,True,Genuine,
PaintCollections,Knife,1.0,0.0,False,True,Fake,Report #596
PaintCollections,Knife,1.0,0.0,False,True,Fake,
PaintCollections,Knife,1.0,0.0,False,True,,Report #597
PaintCollections,Knife,1.0,0.0,False,True,,
PaintCollections,Knife,1.0,0.0,False,False,Genuine,Report #598
PaintCollections,Knife,1.0,0.0,False,False,Genuine,
PaintCollections,Knife,1.0,0.0,False,False,Fake,Report #599
PaintCollections,Knife,1.0,0.0,False,False,Fake,
PaintCollections,Knife,1.0,0.0,False,False,,Report #600
PaintCollections,Knife,1.0,0.0,False,False,,
PaintCollections,Knife,1.0,19.0,True,True,Genuine,Report #601
PaintCollections,Knife,1.0,19.0,True,True,Genuine,
PaintCollections,Knife,1.0,19.0,True,True,Fake,Report #602
PaintCollections,Knife,1.0,19.0,True,True,Fake,
PaintCollections,Knife,1.0,19.0,True,True,,Report #603
PaintCollections,Knife,1.0,19.0,True,True,,
PaintCollections,Knife,1.0,19.0,True,False,Genuine,Report #604
PaintCollections,Knife,1.0,19.0,True,False,Genuine,
PaintCollections,Knife,1.0,19.0,True,False,Fake,Report #605
PaintCollections,Knife,1.0,19.0,True,False,Fake,
PaintCollections,Knife,1.0,19.0,True,False,,Report #606
PaintCollections,Knife,1.0,19.0,True,False,,
PaintCollections,Knife,1.0,19.0,False,True,Genuine,Report #607
PaintCollections,Knife,1.0,19.0,False,True,Genuine,
PaintCollections,Knife,1.0,19.0,False,True,Fake,Report #608
PaintCollections,Knife,1.0,19.0,False,True,Fake,
PaintCollections,Knife,1.0,19.0,False,True,,Report #609
PaintCollections,Knife,1.0,19.0,False,True,,
PaintCollections,Knife,1.0,19.0,False,False,Genuine,Report #610
PaintCollections,Knife,1.0,19.0,False,False,Genuine,
PaintCollections,Knife,1.0,19.0,False,False,Fake,Report #611
PaintCollections,Knife,1.0,19.0,False,False,Fake,
PaintCollections,Knife,1.0,19.0,False,False,,Report #612
PaintCollections,Knife,1.0,19.0,False,False,,
PaintCollections,Knife,1.0,20.0,True,True,Genuine,Report #613
PaintCollections,Knife,1.0,20.0,True,True,Genuine,
PaintCollections,Knife,1.0,20.0,True,True,Fake,Report #614
PaintCollections,Knife,1.0,20.0,True,True,Fake,
PaintCollections,Knife,1.0,20.0,True,True,,Report #615
PaintCollections,Knife,1.0,20.0,True,True,,
PaintCollections,Knife,1.0,20.0,True,False,Genuine,Report #616
PaintCollections,Knife,1.0,20.0,True,False,Genuine,
PaintCollections,Knife,1.0,20.0,True,False,Fake,Report #617
PaintCollections,Knife,1.0,20.0,True,False,Fake,
PaintCollections,Knife,1.0,20.0,True,False,,Report #618
PaintCollections,Knife,1.0,20.0,True,False,,
PaintCollections,Knife,1.0,20.0,False,True,Genuine,Report #619
PaintCollections,Knife,1.0,20.0,False,True,Genuine,
PaintCollections,Knife,1.0,20.0,False,True,Fake,Report #620
PaintCollections,Knife,1.0,20.0,False,True,Fake,
PaintCollections,Knife,1.0,20.0,False,True,,Report #621
PaintCollections,Knife,1.0,20.0,False,True,,
PaintCollections,Knife,1.0,20.0,False,False,Genuine,Report #622
PaintCollections,Knife,1.0,20.0,False,False,Genuine,
PaintCollections,Knife,1.0,20.0,False,False,Fake,Report #623
PaintCollections,Knife,1.0,20.0,False,False,Fake,
PaintCollections,Knife,1.0,20.0,False,False,,Report #624
PaintCollections,Knife,1.0,20.0,False,False,,
PaintCollections,Knife,1.0,21.0,True,True,Genuine,Report #625
PaintCollections,Knife,1.0,21.0,True,True,Genuine,
PaintCollections,Knife,1.0,21.0,True,True,Fake,Report #626
PaintCollections,Knife,1.0,21.0,True,True,Fake,
PaintCollections,Knife,1.0,21.0,True,True,,Report #627
PaintCollections,Knife,1.0,21.0,True,True,,
PaintCollections,Knife,1.0,21.0,True,False,Genuine,Report #628
PaintCollections,Knife,1.0,21.0,True,False,Genuine,
PaintCollections,Knife,1.0,21.0,True,False,Fake,Report #629
PaintCollections,Knife,1.0,21.0,True,False,Fake,
PaintCollections,Knife,1.0,21.0,True,False,,Report #630
PaintCollections,Knife,1.0,21.0,True,False,,
PaintCollections,Knife,1.0,21.0,False,True,Genuine,Report #631
PaintCollections,Knife,1.0,21.0,False,True,Genuine,
PaintCollections,Knife,1.0,21.0,False,True,Fake,Report #632
PaintCollections,Knife,1.0,21.0,False,True,Fake,
PaintCollections,Knife,1.0,21.0,False,True,,Report #633
PaintCollections,Knife,1.0,21.0,False,True,,
PaintCollections,Knife,1.0,21.0,False,False,Genuine,Report #634
PaintCollections,Knife,1.0,21.0,False,False,Genuine,
PaintCollections,Knife,1.0,21.0,False,False,Fake,Report #635
PaintCollections,Knife,1.0,21.0,False,False,Fake,
PaintCollections,Knife,1.0,21.0,False,False,,Report #636
PaintCollections,Knife,1.0,21.0,False,False,,
PaintCollections,Knife,1.0,99.0,True,True,Genuine,Report #637
PaintCollections,Knife,1.0,99.0,True,True,Genuine,
PaintCollections,Knife,1.0,99.0,True,True,Fake,Report #638
PaintCollections,Knife,1.0,99.0,True,True,Fake,
PaintCollections,Knife,1.0,99.0,True,True,,Report #639
PaintCollections,Knife,1.0,99.0,True,True,,
PaintCollections,Knife,1.0,99.0,True,False,Genuine,Report #640
PaintCollections,Knife,1.0,99.0,True,False,Genuine,
PaintCollections,Knife,1.0,99.0,True,False,Fake,Report #641
PaintCollections,Knife,1.0,99.0,True,False,Fake,
PaintCollections,Knife,1.0,99.0,True,False,,Report #642
PaintCollections,Knife,1.0,99.0,True,False,,
PaintCollections,Knife,1.0,99.0,False,True,Genuine,Report #643
PaintCollections,Knife,1.0,99.0,False,True,Genuine,
PaintCollections,Knife,1.0,99.0,False,True,Fake,Report #644
PaintCollections,Knife,1.0,99.0,False,True,Fake,
PaintCollections,Knife,1.0,99.0,False,True,,Report #645
PaintCollections,Knife,1.0,99.0,False,True,,
PaintCollections,Knife,1.0,99.0,False,False,Genuine,Report #646
PaintCollections,Knife,1.0,99.0,False,False,Genuine,
PaintCollections,Knife,1.0,99.0,False,False,Fake,Report #647
PaintCollections,Knife,1.0,99.0,False,False,Fake,
PaintCollections,Knife,1.0,99.0,False,False,,Report #648
PaintCollections,Knife,1.0,99.0,False,False,,
PaintCollections,Knife,1.0,100.0,True,True,Genuine,Report #649
PaintCollections,Knife,1.0,100.0,True,True,Genuine,
PaintCollections,Knife,1.0,100.0,True,True,Fake,Report #650
PaintCollections,Knife,1.0,100.0,True,True,Fake,
PaintCollections,Knife,1.0,100.0,True,True,,Report #651
PaintCollections,Knife,1.0,100.0,True,True,,
PaintCollections,Knife,1.0,100.0,True,False,Genuine,Report #652
PaintCollections,Knife,1.0,100.0,True,False,Genuine,
PaintCollections,Knife,1.0,100.0,True,False,Fake,Report #653
PaintCollections,Knife,1.0,100.0,True,False,Fake,
PaintCollections,Knife,1.0,100.0,True,False,,Report #654
PaintCollections,Knife,1.0,100.0,True,False,,
PaintCollections,Knife,1.0,100.0,False,True,Genuine,Report #655
PaintCollections,Knife,1.0,100.0,False,True,Genuine,
PaintCollections,Knife,1.0,100.0,False,True,Fake,Report #656
PaintCollections,Knife,1.0,100.0,False,True,Fake,
PaintCollections,Knife,1.0,100.0,False,True,,Report #657
PaintCollections,Knife,1.0,100.0,False,True,,
PaintCollections,Knife,1.0,100.0,False,False,Genuine,Report #658
PaintCollections,Knife,1.0,100.0,False,False,Genuine,
PaintCollections,Knife,1.0,100.0,False,False,Fake,Report #659
PaintCollections,Knife,1.0,100.0,False,False,Fake,
PaintCollections,Knife,1.0,100.0,False,False,,Report #660
PaintCollections,Knife,1.0,100.0,False,False,,
PaintCollections,Knife,1.0,101.0,True,True,Genuine,Report #661
PaintCollections,Knife,1.0,101.0,True,True,Genuine,
PaintCollections,Knife,1.0,101.0,True,True,Fake,Report #662
PaintCollections,Knife,1.0,101.0,True,True,Fake,
PaintCollections,Knife,1.0,101.0,True,True,,Report #663
PaintCollections,Knife,1.0,101.0,True,True,,
PaintCollections,Knife,1.0,101.0,True,False,Genuine,Report #664
PaintCollections,Knife,1.0,101.0,True,False,Genuine,
PaintCollections,Knife,1.0,101.0,True,False,Fake,Report #665
PaintCollections,Knife,1.0,101.0,True,False,Fake,
PaintCollections,Knife,1.0,101.0,True,False,,Report #666
PaintCollections,Knife,1.0,101.0,True,False,,
PaintCollections,Knife,1.0,101.0,False,True,Genuine,Report #667
PaintCollections,Knife,1.0,101.0,False,True,Genuine,
PaintCollections,Knife,1.0,101.0,False,True,Fake,Report #668
PaintCollections,Knife,1.0,101.0,False,True,Fake,
PaintCollections,Knife,1.0,101.0,False,True,,Report #669
PaintCollections,Knife,1.0,101.0,False,True,,
PaintCollections,Knife,1.0,101.0,False,False,Genuine,Report #670
PaintCollections,Knife,1.0,101.0,False,False,Genuine,
PaintCollections,Knife,1.0,101.0,False,False,Fake,Report #671
PaintCollections,Knife,1.0,101.0,False,False,Fake,
PaintCollections,Knife,1.0,101.0,False,False,,Report #672
PaintCollections,Knife,1.0,101.0,False,False,,
PaintCollections,Knife,100.0,-1.0,True,True,Genuine,Report #673
PaintCollections,Knife,100.0,-1.0,True,True,Genuine,
PaintCollections,Knife,100.0,-1.0,True,True,Fake,Report #674
PaintCollections,Knife,100.0,-1.0,True,True,Fake,
PaintCollections,Knife,100.0,-1.0,True,True,,Report #675
PaintCollections,Knife,100.0,-1.0,True,True,,
PaintCollections,Knife,100.0,-1.0,True,False,Genuine,Report #676
PaintCollections,Knife,100.0,-1.0,True,False,Genuine,
PaintCollections,Knife,100.0,-1.0,True,False,Fake,Report #677
PaintCollections,Knife,100.0,-1.0,True,False,Fake,
PaintCollections,Knife,100.0,-1.0,True,False,,Report #678
PaintCollections,Knife,100.0,-1.0,True,False,,
PaintCollections,Knife,100.0,-1.0,False,True,Genuine,Report #679
PaintCollections,Knife,100.0,-1.0,False,True,Genuine,
PaintCollections,Knife,100.0,-1.0,False,True,Fake,Report #680
PaintCollections,Knife,100.0,-1.0,False,True,Fake,
PaintCollections,Knife,100.0,-1.0,False,True,,Report #681
PaintCollections,Knife,100.0,-1.0,False,True,,
PaintCollections,Knife,100.0,-1.0,False,False,Genuine,Report #682
PaintCollections,Knife,100.0,-1.0,False,False,Genuine,
PaintCollections,Knife,100.0,-1.0,False,False,Fake,Report #683
PaintCollections,Knife,100.0,-1.0,False,False,Fake,
PaintCollections,Knife,100.0,-1.0,False,False,,Report #684
PaintCollections,Knife,100.0,-1.0,False,False,,
PaintCollections,Knife,100.0,0.0,True,True,Genuine,Report #685
PaintCollections,Knife,100.0,0.0,True,True,Genuine,
PaintCollections,Knife,100.0,0.0,True,True,Fake,Report #686
PaintCollections,Knife,100.0,0.0,True,True,Fake,
PaintCollections,Knife,100.0,0.0,True,True,,Report #687
PaintCollections,Knife,100.0,0.0,True,True,,
PaintCollections,Knife,100.0,0.0,True,False,Genuine,Report #688
PaintCollections,Knife,100.0,0.0,True,False,Genuine,
PaintCollections,Knife,100.0,0.0,True,False,Fake,Report #689
PaintCollections,Knife,100.0,0.0,True,False,Fake,
PaintCollections,Knife,100.0,0.0,True,False,,Report #690
PaintCollections,Knife,100.0,0.0,True,False,,
PaintCollections,Knife,100.0,0.0,False,True,Genuine,Report #691
PaintCollections,Knife,100.0,0.0,False,True,Genuine,
PaintCollections,Knife,100.0,0.0,False,True,Fake,Report #692
PaintCollections,Knife,100.0,0.0,False,True,Fake,
PaintCollections,Knife,100.0,0.0,False,True,,Report #693
PaintCollections,Knife,100.0,0.0,False,True,,
PaintCollections,Knife,100.0,0.0,False,False,Genuine,Report #694
PaintCollections,Knife,100.0,0.0,False,False,Genuine,
PaintCollections,Knife,100.0,0.0,False,False,Fake,Report #695
PaintCollections,Knife,100.0,0.0,False,False,Fake,
PaintCollections,Knife,100.0,0.0,False,False,,Report #696
PaintCollections,Knife,100.0,0.0,False,False,,
PaintCollections,Knife,100.0,19.0,True,True,Genuine,Report #697
PaintCollections,Knife,100.0,19.0,True,True,Genuine,
PaintCollections,Knife,100.0,19.0,True,True,Fake,Report #698
PaintCollections,Knife,100.0,19.0,True,True,Fake,
PaintCollections,Knife,100.0,19.0,True,True,,Report #699
PaintCollections,Knife,100.0,19.0,True,True,,
PaintCollections,Knife,100.0,19.0,True,False,Genuine,Report #700
PaintCollections,Knife,100.0,19.0,True,False,Genuine,
PaintCollections,Knife,100.0,19.0,True,False,Fake,Report #701
PaintCollections,Knife,100.0,19.0,True,False,Fake,
PaintCollections,Knife,100.0,19.0,True,False,,Report #702
PaintCollections,Knife,100.0,19.0,True,False,,
PaintCollections,Knife,100.0,19.0,False,True,Genuine,Report #703
PaintCollections,Knife,100.0,19.0,False,True,Genuine,
PaintCollections,Knife,100.0,19.0,False,True,Fake,Report #704
PaintCollections,Knife,100.0,19.0,False,True,Fake,
PaintCollections,Knife,100.0,19.0,False,True,,Report #705
PaintCollections,Knife,100.0,19.0,False,True,,
PaintCollections,Knife,100.0,19.0,False,False,Genuine,Report #706
PaintCollections,Knife,100.0,19.0,False,False,Genuine,
PaintCollections,Knife,100.0,19.0,False,False,Fake,Report #707
PaintCollections,Knife,100.0,19.0,False,False,Fake,
PaintCollections,Knife,100.0,19.0,False,False,,Report #708
PaintCollections,Knife,100.0,19.0,False,False,,
PaintCollections,Knife,100.0,20.0,True,True,Genuine,Report #709
PaintCollections,Knife,100.0,20.0,True,True,Genuine,
PaintCollections,Knife,100.0,20.0,True,True,Fake,Report #710
PaintCollections,Knife,100.0,20.0,True,True,Fake,
PaintCollections,Knife,100.0,20.0,True,True,,Report #711
PaintCollections,Knife,100.0,20.0,True,True,,
PaintCollections,Knife,100.0,20.0,True,False,Genuine,Report #712
PaintCollections,Knife,100.0,20.0,True,False,Genuine,
PaintCollections,Knife,100.0,20.0,True,False,Fake,Report #713
PaintCollections,Knife,100.0,20.0,True,False,Fake,
PaintCollections,Knife,100.0,20.0,True,False,,Report #714
PaintCollections,Knife,100.0,20.0,True,False,,
PaintCollections,Knife,100.0,20.0,False,True,Genuine,Report #715
PaintCollections,Knife,100.0,20.0,False,True,Genuine,
PaintCollections,Knife,100.0,20.0,False,True,Fake,Report #716
PaintCollections,Knife,100.0,20.0,False,True,Fake,
PaintCollections,Knife,100.0,20.0,False,True,,Report #717
PaintCollections,Knife,100.0,20.0,False,True,,
PaintCollections,Knife,100.0,20.0,False,False,Genuine,Report #718
PaintCollections,Knife,100.0,20.0,False,False,Genuine,
PaintCollections,Knife,100.0,20.0,False,False,Fake,Report #719
PaintCollections,Knife,100.0,20.0,False,False,Fake,
PaintCollections,Knife,100.0,20.0,False,False,,Report #720
PaintCollections,Knife,100.0,20.0,False,False,,
PaintCollections,Knife,100.0,21.0,True,True,Genuine,Report #721
PaintCollections,Knife,100.0,21.0,True,True,Genuine,
PaintCollections,Knife,100.0,21.0,True,True,Fake,Report #722
PaintCollections,Knife,100.0,21.0,True,True,Fake,
PaintCollections,Knife,100.0,21.0,True,True,,Report #723
PaintCollections,Knife,100.0,21.0,True,True,,
PaintCollections,Knife,100.0,21.0,True,False,Genuine,Report #724
PaintCollections,Knife,100.0,21.0,True,False,Genuine,
PaintCollections,Knife,100.0,21.0,True,False,Fake,Report #725
PaintCollections,Knife,100.0,21.0,True,False,Fake,
PaintCollections,Knife,100.0,21.0,True,False,,Report #726
PaintCollections,Knife,100.0,21.0,True,False,,
PaintCollections,Knife,100.0,21.0,False,True,Genuine,Report #727
PaintCollections,Knife,100.0,21.0,False,True,Genuine,
PaintCollections,Knife,100.0,21.0,False,True,Fake,Report #728
PaintCollections,Knife,100.0,21.0,False,True,Fake,
PaintCollections,Knife,100.0,21.0,False,True,,Report #729
PaintCollections,Knife,100.0,21.0,False,True,,
PaintCollections,Knife,100.0,21.0,False,False,Genuine,Report #730
PaintCollections,Knife,100.0,21.0,False,False,Genuine,
PaintCollections,Knife,100.0,21.0,False,False,Fake,Report #731
PaintCollections,Knife,100.0,21.0,False,False,Fake,
PaintCollections,Knife,100.0,21.0,False,False,,Report #732
PaintCollections,Knife,100.0,21.0,False,False,,
PaintCollections,Knife,100.0,99.0,True,True,Genuine,Report #733
PaintCollections,Knife,100.0,99.0,True,True,Genuine,
PaintCollections,Knife,100.0,99.0,True,True,Fake,Report #734
PaintCollections,Knife,100.0,99.0,True,True,Fake,
PaintCollections,Knife,100.0,99.0,True,True,,Report #735
PaintCollections,Knife,100.0,99.0,True,True,,
PaintCollections,Knife,100.0,99.0,True,False,Genuine,Report #736
PaintCollections,Knife,100.0,99.0,True,False,Genuine,
PaintCollections,Knife,100.0,99.0,True,False,Fake,Report #737
PaintCollections,Knife,100.0,99.0,True,False,Fake,
PaintCollections,Knife,100.0,99.0,True,False,,Report #738
PaintCollections,Knife,100.0,99.0,True,False,,
PaintCollections,Knife,100.0,99.0,False,True,Genuine,Report #739
PaintCollections,Knife,100.0,99.0,False,True,Genuine,
PaintCollections,Knife,100.0,99.0,False,True,Fake,Report #740
PaintCollections,Knife,100.0,99.0,False,True,Fake,
PaintCollections,Knife,100.0,99.0,False,True,,Report #741
PaintCollections,Knife,100.0,99.0,False,True,,
PaintCollections,Knife,100.0,99.0,False,False,Genuine,Report #742
PaintCollections,Knife,100.0,99.0,False,False,Genuine,
PaintCollections,Knife,100.0,99.0,False,False,Fake,Report #743
PaintCollections,Knife,100.0,99.0,False,False,Fake,
PaintCollections,Knife,100.0,99.0,False,False,,Report #744
PaintCollections,Knife,100.0,99.0,False,False,,
PaintCollections,Knife,100.0,100.0,True,True,Genuine,Report #745
PaintCollections,Knife,100.0,100.0,True,True,Genuine,
PaintCollections,Knife,100.0,100.0,True,True,Fake,Report #746
PaintCollections,Knife,100.0,100.0,True,True,Fake,
PaintCollections,Knife,100.0,100.0,True,True,,Report #747
PaintCollections,Knife,100.0,100.0,True,True,,
PaintCollections,Knife,100.0,100.0,True,False,Genuine,Report #748
PaintCollections,Knife,100.0,100.0,True,False,Genuine,
PaintCollections,Knife,100.0,100.0,True,False,Fake,Report #749
PaintCollections,Knife,100.0,100.0,True,False,Fake,
PaintCollections,Knife,100.0,100.0,True,False,,Report #750
PaintCollections,Knife,100.0,100.0,True,False,,
PaintCollections,Knife,100.0,100.0,False,True,Genuine,Report #751
PaintCollections,Knife,100.0,100.0,False,True,Genuine,
PaintCollections,Knife,100.0,100.0,False,True,Fake,Report #752
PaintCollections,Knife,100.0,100.0,False,True,Fake,
PaintCollections,Knife,100.0,100.0,False,True,,Report #753
PaintCollections,Knife,100.0,100.0,False,True,,
PaintCollections,Knife,100.0,100.0,False,False,Genuine,Report #754
PaintCollections,Knife,100.0,100.0,False,False,Genuine,
PaintCollections,Knife,100.0,100.0,False,False,Fake,Report #755
PaintCollections,Knife,100.0,100.0,False,False,Fake,
PaintCollections,Knife,100.0,100.0,False,False,,Report #756
PaintCollections,Knife,100.0,100.0,False,False,,
PaintCollections,Knife,100.0,101.0,True,True,Genuine,Report #757
PaintCollections,Knife,100.0,101.0,True,True,Genuine,
PaintCollections,Knife,100.0,101.0,True,True,Fake,Report #758
PaintCollections,Knife,100.0,101.0,True,True,Fake,
PaintCollections,Knife,100.0,101.0,True,True,,Report #759
PaintCollections,Knife,100.0,101.0,True,True,,
PaintCollections,Knife,100.0,101.0,True,False,Genuine,Report #760
PaintCollections,Knife,100.0,101.0,True,False,Genuine,
PaintCollections,Knife,100.0,101.0,True,False,Fake,Report #761
PaintCollections,Knife,100.0,101.0,True,False,Fake,
PaintCollections,Knife,100.0,101.0,True,False,,Report #762
PaintCollections,Knife,100.0,101.0,True,False,,
PaintCollections,Knife,100.0,101.0,False,True,Genuine,Report #763
PaintCollections,Knife,100.0,101.0,False,True,Genuine,
PaintCollections,Knife,100.0,101.0,False,True,Fake,Report #764
PaintCollections,Knife,100.0,101.0,False,True,Fake,
PaintCollections,Knife,100.0,101.0,False,True,,Report #765
PaintCollections,Knife,100.0,101.0,False,True,,
PaintCollections,Knife,100.0,101.0,False,False,Genuine,Report #766
PaintCollections,Knife,100.0,101.0,False,False,Genuine,
PaintCollections,Knife,100.0,101.0,False,False,Fake,Report #767
PaintCollections,Knife,100.0,101.0,False,False,Fake,
PaintCollections,Knife,100.0,101.0,False,False,,Report #768
PaintCollections,Knife,100.0,101.0,False,False,,
PaintCollections,Knife,1000.0,-1.0,True,True,Genuine,Report #769
PaintCollections,Knife,1000.0,-1.0,True,True,Genuine,
PaintCollections,Knife,1000.0,-1.0,True,True,Fake,Report #770
PaintCollections,Knife,1000.0,-1.0,True,True,Fake,
PaintCollections,Knife,1000.0,-1.0,True,True,,Report #771
PaintCollections,Knife,1000.0,-1.0,True,True,,
PaintCollections,Knife,1000.0,-1.0,True,False,Genuine,Report #772
PaintCollections,Knife,1000.0,-1.0,True,False,Genuine,
PaintCollections,Knife,1000.0,-1.0,True,False,Fake,Report #773
PaintCollections,Knife,1000.0,-1.0,True,False,Fake,
PaintCollections,Knife,1000.0,-1.0,True,False,,Report #774
PaintCollections,Knife,1000.0,-1.0,True,False,,
PaintCollections,Knife,1000.0,-1.0,False,True,Genuine,Report #775
PaintCollections,Knife,1000.0,-1.0,False,True,Genuine,
PaintCollections,Knife,1000.0,-1.0,False,True,Fake,Report #776
PaintCollections,Knife,1000.0,-1.0,False,True,Fake,
PaintCollections,Knife,1000.0,-1.0,False,True,,Report #777
PaintCollections,Knife,1000.0,-1.0,False,True,,
PaintCollections,Knife,1000.0,-1.0,False,False,Genuine,Report #778
PaintCollections,Knife,1000.0,-1.0,False,False,Genuine,
PaintCollections,Knife,1000.0,-1.0,False,False,Fake,Report #779
PaintCollections,Knife,1000.0,-1.0,False,False,Fake,
PaintCollections,Knife,1000.0,-1.0,False,False,,Report #780
PaintCollections,Knife,1000.0,-1.0,False,False,,
PaintCollections,Knife,1000.0,0.0,True,True,Genuine,Report #781
PaintCollections,Knife,1000.0,0.0,True,True,Genuine,
PaintCollections,Knife,1000.0,0.0,True,True,Fake,Report #782
PaintCollections,Knife,1000.0,0.0,True,True,Fake,
PaintCollections,Knife,1000.0,0.0,True,True,,Report #783
PaintCollections,Knife,1000.0,0.0,True,True,,
PaintCollections,Knife,1000.0,0.0,True,False,Genuine,Report #784
PaintCollections,Knife,1000.0,0.0,True,False,Genuine,
PaintCollections,Knife,1000.0,0.0,True,False,Fake,Report #785
PaintCollections,Knife,1000.0,0.0,True,False,Fake,
PaintCollections,Knife,1000.0,0.0,True,False,,Report #786
PaintCollections,Knife,1000.0,0.0,True,False,,
PaintCollections,Knife,1000.0,0.0,False,True,Genuine,Report #787
PaintCollections,Knife,1000.0,0.0,False,True,Genuine,
PaintCollections,Knife,1000.0,0.0,False,True,Fake,Report #788
PaintCollections,Knife,1000.0,0.0,False,True,Fake,
PaintCollections,Knife,1000.0,0.0,False,True,,Report #789
PaintCollections,Knife,1000.0,0.0,False,True,,
PaintCollections,Knife,1000.0,0.0,False,False,Genuine,Report #790
PaintCollections,Knife,1000.0,0.0,False,False,Genuine,
PaintCollections,Knife,1000.0,0.0,False,False,Fake,Report #791
PaintCollections,Knife,1000.0,0.0,False,False,Fake,
PaintCollections,Knife,1000.0,0.0,False,False,,Report #792
PaintCollections,Knife,1000.0,0.0,False,False,,
PaintCollections,Knife,1000.0,19.0,True,True,Genuine,Report #793
PaintCollections,Knife,1000.0,19.0,True,True,Genuine,
PaintCollections,Knife,1000.0,19.0,True,True,Fake,Report #794
PaintCollections,Knife,1000.0,19.0,True,True,Fake,
PaintCollections,Knife,1000.0,19.0,True,True,,Report #795
PaintCollections,Knife,1000.0,19.0,True,True,,
PaintCollections,Knife,1000.0,19.0,True,False,Genuine,Report #796
PaintCollections,Knife,1000.0,19.0,True,False,Genuine,
PaintCollections,Knife,1000.0,19.0,True,False,Fake,Report #797
PaintCollections,Knife,1000.0,19.0,True,False,Fake,
PaintCollections,Knife,1000.0,19.0,True,False,,Report #798
PaintCollections,Knife,1000.0,19.0,True,False,,
PaintCollections,Knife,1000.0,19.0,False,True,Genuine,Report #799
PaintCollections,Knife,1000.0,19.0,False,True,Genuine,
PaintCollections,Knife,1000.0,19.0,False,True,Fake,Report #800
PaintCollections,Knife,1000.0,19.0,False,True,Fake,
PaintCollections,Knife,1000.0,19.0,False,True,,Report #801
PaintCollections,Knife,1000.0,19.0,False,True,,
PaintCollections,Knife,1000.0,19.0,False,False,Genuine,Report #802
PaintCollections,Knife,1000.0,19.0,False,False,Genuine,
PaintCollections,Knife,1000.0,19.0,False,False,Fake,Report #803
PaintCollections,Knife,1000.0,19.0,False,False,Fake,
PaintCollections,Knife,1000.0,19.0,False,False,,Report #804
PaintCollections,Knife,1000.0,19.0,False,False,,
PaintCollections,Knife,1000.0,20.0,True,True,Genuine,Report #805
PaintCollections,Knife,1000.0,20.0,True,True,Genuine,
PaintCollections,Knife,1000.0,20.0,True,True,Fake,Report #806
PaintCollections,Knife,1000.0,20.0,True,True,Fake,
PaintCollections,Knife,1000.0,20.0,True,True,,Report #807
PaintCollections,Knife,1000.0,20.0,True,True,,
PaintCollections,Knife,1000.0,20.0,True,False,Genuine,Report #808
PaintCollections,Knife,1000.0,20.0,True,False,Genuine,
PaintCollections,Knife,1000.0,20.0,True,False,Fake,Report #809
PaintCollections,Knife,1000.0,20.0,True,False,Fake,
PaintCollections,Knife,1000.0,20.0,True,False,,Report #810
PaintCollections,Knife,1000.0,20.0,True,False,,
PaintCollections,Knife,1000.0,20.0,False,True,Genuine,Report #811
PaintCollections,Knife,1000.0,20.0,False,True,Genuine,
PaintCollections,Knife,1000.0,20.0,False,True,Fake,Report #812
PaintCollections,Knife,1000.0,20.0,False,True,Fake,
PaintCollections,Knife,1000.0,20.0,False,True,,Report #813
PaintCollections,Knife,1000.0,20.0,False,True,,
PaintCollections,Knife,1000.0,20.0,False,False,Genuine,Report #814
PaintCollections,Knife,1000.0,20.0,False,False,Genuine,
PaintCollections,Knife,1000.0,20.0,False,False,Fake,Report #815
PaintCollections,Knife,1000.0,20.0,False,False,Fake,
PaintCollections,Knife,1000.0,20.0,False,False,,Report #816
PaintCollections,Knife,1000.0,20.0,False,False,,
PaintCollections,Knife,1000.0,21.0,True,True,Genuine,Report #817
PaintCollections,Knife,1000.0,21.0,True,True,Genuine,
PaintCollections,Knife,1000.0,21.0,True,True,Fake,Report #818
PaintCollections,Knife,1000.0,21.0,True,True,Fake,
PaintCollections,Knife,1000.0,21.0,True,True,,Report #819
PaintCollections,Knife,1000.0,21.0,True,True,,
PaintCollections,Knife,1000.0,21.0,True,False,Genuine,Report #820
PaintCollections,Knife,1000.0,21.0,True,False,Genuine,
PaintCollections,Knife,1000.0,21.0,True,False,Fake,Report #821
PaintCollections,Knife,1000.0,21.0,True,False,Fake,
PaintCollections,Knife,1000.0,21.0,True,False,,Report #822
PaintCollections,Knife,1000.0,21.0,True,False,,
PaintCollections,Knife,1000.0,21.0,False,True,Genuine,Report #823
PaintCollections,Knife,1000.0,21.0,False,True,Genuine,
PaintCollections,Knife,1000.0,21.0,False,True,Fake,Report #824
PaintCollections,Knife,1000.0,21.0,False,True,Fake,
PaintCollections,Knife,1000.0,21.0,False,True,,Report #825
PaintCollections,Knife,1000.0,21.0,False,True,,
PaintCollections,Knife,1000.0,21.0,False,False,Genuine,Report #826
PaintCollections,Knife,1000.0,21.0,False,False,Genuine,
PaintCollections,Knife,1000.0,21.0,False,False,Fake,Report #827
PaintCollections,Knife,1000.0,21.0,False,False,Fake,
PaintCollections,Knife,1000.0,21.0,False,False,,Report #828
PaintCollections,Knife,1000.0,21.0,False,False,,
PaintCollections,Knife,1000.0,99.0,True,True,Genuine,Report #829
PaintCollections,Knife,1000.0,99.0,True,True,Genuine,
PaintCollections,Knife,1000.0,99.0,True,True,Fake,Report #830
PaintCollections,Knife,1000.0,99.0,True,True,Fake,
PaintCollections,Knife,1000.0,99.0,True,True,,Report #831
PaintCollections,Knife,1000.0,99.0,True,True,,
PaintCollections,Knife,1000.0,99.0,True,False,Genuine,Report #832
PaintCollections,Knife,1000.0,99.0,True,False,Genuine,
PaintCollections,Knife,1000.0,99.0,True,False,Fake,Report #833
PaintCollections,Knife,1000.0,99.0,True,False,Fake,
PaintCollections,Knife,1000.0,99.0,True,False,,Report #834
PaintCollections,Knife,1000.0,99.0,True,False,,
PaintCollections,Knife,1000.0,99.0,False,True,Genuine,Report #835
PaintCollections,Knife,1000.0,99.0,False,True,Genuine,
PaintCollections,Knife,1000.0,99.0,False,True,Fake,Report #836
PaintCollections,Knife,1000.0,99.0,False,True,Fake,
PaintCollections,Knife,1000.0,99.0,False,True,,Report #837
PaintCollections,Knife,1000.0,99.0,False,True,,
PaintCollections,Knife,1000.0,99.0,False,False,Genuine,Report #838
PaintCollections,Knife,1000.0,99.0,False,False,Genuine,
PaintCollections,Knife,1000.0,99.0,False,False,Fake,Report #839
PaintCollections,Knife,1000.0,99.0,False,False,Fake,
PaintCollections,Knife,1000.0,99.0,False,False,,Report #840
PaintCollections,Knife,1000.0,99.0,False,False,,
PaintCollections,Knife,1000.0,100.0,True,True,Genuine,Report #841
PaintCollections,Knife,1000.0,100.0,True,True,Genuine,
PaintCollections,Knife,1000.0,100.0,True,True,Fake,Report #842
PaintCollections,Knife,1000.0,100.0,True,True,Fake,
PaintCollections,Knife,1000.0,100.0,True,True,,Report #843
PaintCollections,Knife,1000.0,100.0,True,True,,
PaintCollections,Knife,1000.0,100.0,True,False,Genuine,Report #844
PaintCollections,Knife,1000.0,100.0,True,False,Genuine,
PaintCollections,Knife,1000.0,100.0,True,False,Fake,Report #845
PaintCollections,Knife,1000.0,100.0,True,False,Fake,
PaintCollections,Knife,1000.0,100.0,True,False,,Report #846
PaintCollections,Knife,1000.0,100.0,True,False,,
PaintCollections,Knife,1000.0,100.0,False,True,Genuine,Report #847
PaintCollections,Knife,1000.0,100.0,False,True,Genuine,
PaintCollections,Knife,1000.0,100.0,False,True,Fake,Report #848
PaintCollections,Knife,1000.0,100.0,False,True,Fake,
PaintCollections,Knife,1000.0,100.0,False,True,,Report #849
PaintCollections,Knife,1000.0,100.0,False,True,,
PaintCollections,Knife,1000.0,100.0,False,False,Genuine,Report #850
PaintCollections,Knife,1000.0,100.0,False,False,Genuine,
PaintCollections,Knife,1000.0,100.0,False,False,Fake,Report #851
PaintCollections,Knife,1000.0,100.0,False,False,Fake,
PaintCollections,Knife,1000.0,100.0,False,False,,Report #852
PaintCollections,Knife,1000.0,100.0,False,False,,
PaintCollections,Knife,1000.0,101.0,True,True,Genuine,Report #853
PaintCollections,Knife,1000.0,101.0,True,True,Genuine,
PaintCollections,Knife,1000.0,101.0,True,True,Fake,Report #854
PaintCollections,Knife,1000.0,101.0,True,True,Fake,
PaintCollections,Knife,1000.0,101.0,True,True,,Report #855
PaintCollections,Knife,1000.0,101.0,True,True,,
PaintCollections,Knife,1000.0,101.0,True,False,Genuine,Report #856
PaintCollections,Knife,1000.0,101.0,True,False,Genuine,
PaintCollections,Knife,1000.0,101.0,True,False,Fake,Report #857
PaintCollections,Knife,1000.0,101.0,True,False,Fake,
PaintCollections,Knife,1000.0,101.0,True,False,,Report #858
PaintCollections,Knife,1000.0,101.0,True,False,,
PaintCollections,Knife,1000.0,101.0,False,True,Genuine,Report #859
PaintCollections,Knife,1000.0,101.0,False,True,Genuine,
PaintCollections,Knife,1000.0,101.0,False,True,Fake,Report #860
PaintCollections,Knife,1000.0,101.0,False,True,Fake,
PaintCollections,Knife,1000.0,101.0,False,True,,Report #861
PaintCollections,Knife,1000.0,101.0,False,True,,
PaintCollections,Knife,1000.0,101.0,False,False,Genuine,Report #862
PaintCollections,Knife,1000.0,101.0,False,False,Genuine,
PaintCollections,Knife,1000.0,101.0,False,False,Fake,Report #863
PaintCollections,Knife,1000.0,101.0,False,False,Fake,
PaintCollections,Knife,1000.0,101.0,False,False,,Report #864
PaintCollections,Knife,1000.0,101.0,False,False,,
PaintCollections,Knife,1000000.0,-1.0,True,True,Genuine,Report #865
PaintCollections,Knife,1000000.0,-1.0,True,True,Genuine,
PaintCollections,Knife,1000000.0,-1.0,True,True,Fake,Report #866
PaintCollections,Knife,1000000.0,-1.0,True,True,Fake,
PaintCollections,Knife,1000000.0,-1.0,True,True,,Report #867
PaintCollections,Knife,1000000.0,-1.0,True,True,,
PaintCollections,Knife,1000000.0,-1.0,True,False,Genuine,Report #868
PaintCollections,Knife,1000000.0,-1.0,True,False,Genuine,
PaintCollections,Knife,1000000.0,-1.0,True,False,Fake,Report #869
PaintCollections,Knife,1000000.0,-1.0,True,False,Fake,
PaintCollections,Knife,1000000.0,-1.0,True,False,,Report #870
PaintCollections,Knife,1000000.0,-1.0,True,False,,
PaintCollections,Knife,1000000.0,-1.0,False,True,Genuine,Report #871
PaintCollections,Knife,1000000.0,-1.0,False,True,Genuine,
PaintCollections,Knife,1000000.0,-1.0,False,True,Fake,Report #872
PaintCollections,Knife,1000000.0,-1.0,False,True,Fake,
PaintCollections,Knife,1000000.0,-1.0,False,True,,Report #873
PaintCollections,Knife,1000000.0,-1.0,False,True,,
PaintCollections,Knife,1000000.0,-1.0,False,False,Genuine,Report #874
PaintCollections,Knife,1000000.0,-1.0,False,False,Genuine,
PaintCollections,Knife,1000000.0,-1.0,False,False,Fake,Report #875
PaintCollections,Knife,1000000.0,-1.0,False,False,Fake,
PaintCollections,Knife,1000000.0,-1.0,False,False,,Report #876
PaintCollections,Knife,1000000.0,-1.0,False,False,,
PaintCollections,Knife,1000000.0,0.0,True,True,Genuine,Report #877
PaintCollections,Knife,1000000.0,0.0,True,True,Genuine,
PaintCollections,Knife,1000000.0,0.0,True,True,Fake,Report #878
PaintCollections,Knife,1000000.0,0.0,True,True,Fake,
PaintCollections,Knife,1000000.0,0.0,True,True,,Report #879
PaintCollections,Knife,1000000.0,0.0,True,True,,
PaintCollections,Knife,1000000.0,0.0,True,False,Genuine,Report #880
PaintCollections,Knife,1000000.0,0.0,True,False,Genuine,
PaintCollections,Knife,1000000.0,0.0,True,False,Fake,Report #881
PaintCollections,Knife,1000000.0,0.0,True,False,Fake,
PaintCollections,Knife,1000000.0,0.0,True,False,,Report #882
PaintCollections,Knife,1000000.0,0.0,True,False,,
PaintCollections,Knife,1000000.0,0.0,False,True,Genuine,Report #883
PaintCollections,Knife,1000000.0,0.0,False,True,Genuine,
PaintCollections,Knife,1000000.0,0.0,False,True,Fake,Report #884
PaintCollections,Knife,1000000.0,0.0,False,True,Fake,
PaintCollections,Knife,1000000.0,0.0,False,True,,Report #885
PaintCollections,Knife,1000000.0,0.0,False,True,,
PaintCollections,Knife,1000000.0,0.0,False,False,Genuine,Report #886
PaintCollections,Knife,1000000.0,0.0,False,False,Genuine,
PaintCollections,Knife,1000000.0,0.0,False,False,Fake,Report #887
PaintCollections,Knife,1000000.0,0.0,False,False,Fake,
PaintCollections,Knife,1000000.0,0.0,False,False,,Report #888
PaintCollections,Knife,1000000.0,0.0,False,False,,
PaintCollections,Knife,1000000.0,19.0,True,True,Genuine,Report #889
PaintCollections,Knife,1000000.0,19.0,True,True,Genuine,
PaintCollections,Knife,1000000.0,19.0,True,True,Fake,Report #890
PaintCollections,Knife,1000000.0,19.0,True,True,Fake,
PaintCollections,Knife,1000000.0,19.0,True,True,,Report #891
PaintCollections,Knife,1000000.0,19.0,True,True,,
PaintCollections,Knife,1000000.0,19.0,True,False,Genuine,Report #892
PaintCollections,Knife,1000000.0,19.0,True,False,Genuine,
PaintCollections,Knife,1000000.0,19.0,True,False,Fake,Report #893
PaintCollections,Knife,1000000.0,19.0,True,False,Fake,
PaintCollections,Knife,1000000.0,19.0,True,False,,Report #894
PaintCollections,Knife,1000000.0,19.0,True,False,,
PaintCollections,Knife,1000000.0,19.0,False,True,Genuine,Report #895
PaintCollections,Knife,1000000.0,19.0,False,True,Genuine,
PaintCollections,Knife,1000000.0,19.0,False,True,Fake,Report #896
PaintCollections,Knife,1000000.0,19.0,False,True,Fake,
PaintCollections,Knife,1000000.0,19.0,False,True,,Report #897
PaintCollections,Knife,1000000.0,19.0,False,True,,
PaintCollections,Knife,1000000.0,19.0,False,False,Genuine,Report #898
PaintCollections,Knife,1000000.0,19.0,False,False,Genuine,
PaintCollections,Knife,1000000.0,19.0,False,False,Fake,Report #899
PaintCollections,Knife,1000000.0,19.0,False,False,Fake,
PaintCollections,Knife,1000000.0,19.0,False,False,,Report #900
PaintCollections,Knife,1000000.0,19.0,False,False,,
PaintCollections,Knife,1000000.0,20.0,True,True,Genuine,Report #901
PaintCollections,Knife,1000000.0,20.0,True,True,Genuine,
PaintCollections,Knife,1000000.0,20.0,True,True,Fake,Report #902
PaintCollections,Knife,1000000.0,20.0,True,True,Fake,
PaintCollections,Knife,1000000.0,20.0,True,True,,Report #903
PaintCollections,Knife,1000000.0,20.0,True,True,,
PaintCollections,Knife,1000000.0,20.0,True,False,Genuine,Report #904
PaintCollections,Knife,1000000.0,20.0,True,False,Genuine,
PaintCollections,Knife,1000000.0,20.0,True,False,Fake,Report #905
PaintCollections,Knife,1000000.0,20.0,True,False,Fake,
PaintCollections,Knife,1000000.0,20.0,True,False,,Report #906
PaintCollections,Knife,1000000.0,20.0,True,False,,
PaintCollections,Knife,1000000.0,20.0,False,True,Genuine,Report #907
PaintCollections,Knife,1000000.0,20.0,False,True,Genuine,
PaintCollections,Knife,1000000.0,20.0,False,True,Fake,Report #908
PaintCollections,Knife,1000000.0,20.0,False,True,Fake,
PaintCollections,Knife,1000000.0,20.0,False,True,,Report #909
PaintCollections,Knife,1000000.0,20.0,False,True,,
PaintCollections,Knife,1000000.0,20.0,False,False,Genuine,Report #910
PaintCollections,Knife,1000000.0,20.0,False,False,Genuine,
PaintCollections,Knife,1000000.0,20.0,False,False,Fake,Report #911
PaintCollections,Knife,1000000.0,20.0,False,False,Fake,
PaintCollections,Knife,1000000.0,20.0,False,False,,Report #912
PaintCollections,Knife,1000000.0,20.0,False,False,,
PaintCollections,Knife,1000000.0,21.0,True,True,Genuine,Report #913
PaintCollections,Knife,1000000.0,21.0,True,True,Genuine,
PaintCollections,Knife,1000000.0,21.0,True,True,Fake,Report #914
PaintCollections,Knife,1000000.0,21.0,True,True,Fake,
PaintCollections,Knife,1000000.0,21.0,True,True,,Report #915
PaintCollections,Knife,1000000.0,21.0,True,True,,
PaintCollections,Knife,1000000.0,21.0,True,False,Genuine,Report #916
PaintCollections,Knife,1000000.0,21.0,True,False,Genuine,
PaintCollections,Knife,1000000.0,21.0,True,False,Fake,Report #917
PaintCollections,Knife,1000000.0,21.0,True,False,Fake,
PaintCollections,Knife,1000000.0,21.0,True,False,,Report #918
PaintCollections,Knife,1000000.0,21.0,True,False,,
PaintCollections,Knife,1000000.0,21.0,False,True,Genuine,Report #919
PaintCollections,Knife,1000000.0,21.0,False,True,Genuine,
PaintCollections,Knife,1000000.0,21.0,False,True,Fake,Report #920
PaintCollections,Knife,1000000.0,21.0,False,True,Fake,
PaintCollections,Knife,1000000.0,21.0,False,True,,Report #921
PaintCollections,Knife,1000000.0,21.0,False,True,,
PaintCollections,Knife,1000000.0,21.0,False,False,Genuine,Report #922
PaintCollections,Knife,1000000.0,21.0,False,False,Genuine,
PaintCollections,Knife,1000000.0,21.0,False,False,Fake,Report #923
PaintCollections,Knife,1000000.0,21.0,False,False,Fake,
PaintCollections,Knife,1000000.0,21.0,False,False,,Report #924
PaintCollections,Knife,1000000.0,21.0,False,False,,
PaintCollections,Knife,1000000.0,99.0,True,True,Genuine,Report #925
PaintCollections,Knife,1000000.0,99.0,True,True,Genuine,
PaintCollections,Knife,1000000.0,99.0,True,True,Fake,Report #926
PaintCollections,Knife,1000000.0,99.0,True,True,Fake,
PaintCollections,Knife,1000000.0,99.0,True,True,,Report #927
PaintCollections,Knife,1000000.0,99.0,True,True,,
PaintCollections,Knife,1000000.0,99.0,True,False,Genuine,Report #928
PaintCollections,Knife,1000000.0,99.0,True,False,Genuine,
PaintCollections,Knife,1000000.0,99.0,True,False,Fake,Report #929
PaintCollections,Knife,1000000.0,99.0,True,False,Fake,
PaintCollections,Knife,1000000.0,99.0,True,False,,Report #930
PaintCollections,Knife,1000000.0,99.0,True,False,,
PaintCollections,Knife,1000000.0,99.0,False,True,Genuine,Report #931
PaintCollections,Knife,1000000.0,99.0,False,True,Genuine,
PaintCollections,Knife,1000000.0,99.0,False,True,Fake,Report #932
PaintCollections,Knife,1000000.0,99.0,False,True,Fake,
PaintCollections,Knife,1000000.0,99.0,False,True,,Report #933
PaintCollections,Knife,1000000.0,99.0,False,True,,
PaintCollections,Knife,1000000.0,99.0,False,False,Genuine,Report #934
PaintCollections,Knife,1000000.0,99.0,False,False,Genuine,
PaintCollections,Knife,1000000.0,99.0,False,False,Fake,Report #935
PaintCollections,Knife,1000000.0,99.0,False,False,Fake,
PaintCollections,Knife,1000000.0,99.0,False,False,,Report #936
PaintCollections,Knife,1000000.0,99.0,False,False,,
PaintCollections,Knife,1000000.0,100.0,True,True,Genuine,Report #937
PaintCollections,Knife,1000000.0,100.0,True,True,Genuine,
PaintCollections,Knife,1000000.0,100.0,True,True,Fake,Report #938
PaintCollections,Knife,1000000.0,100.0,True,True,Fake,
PaintCollections,Knife,1000000.0,100.0,True,True,,Report #939
PaintCollections,Knife,1000000.0,100.0,True,True,,
PaintCollections,Knife,1000000.0,100.0,True,False,Genuine,Report #940
PaintCollections,Knife,1000000.0,100.0,True,False,Genuine,
PaintCollections,Knife,1000000.0,100.0,True,False,Fake,Report #941
PaintCollections,Knife,1000000.0,100.0,True,False,Fake,
PaintCollections,Knife,1000000.0,100.0,True,False,,Report #942
PaintCollections,Knife,1000000.0,100.0,True,False,,
PaintCollections,Knife,1000000.0,100.0,False,True,Genuine,Report #943
PaintCollections,Knife,1000000.0,100.0,False,True,Genuine,
PaintCollections,Knife,1000000.0,100.0,False,True,Fake,Report #944
PaintCollections,Knife,1000000.0,100.0,False,True,Fake,
PaintCollections,Knife,1000000.0,100.0,False,True,,Report #945
PaintCollections,Knife,1000000.0,100.0,False,True,,
PaintCollections,Knife,1000000.0,100.0,False,False,Genuine,Report #946
PaintCollections,Knife,1000000.0,100.0,False,False,Genuine,
PaintCollections,Knife,1000000.0,100.0,False,False,Fake,Report #947
PaintCollections,Knife,1000000.0,100.0,False,False,Fake,
PaintCollections,Knife,1000000.0,100.0,False,False,,Report #948
PaintCollections,Knife,1000000.0,100.0,False,False,,
PaintCollections,Knife,1000000.0,101.0,True,True,Genuine,Report #949
PaintCollections,Knife,1000000.0,101.0,True,True,Genuine,
PaintCollections,Knife,1000000.0,101.0,True,True,Fake,Report #950
PaintCollections,Knife,1000000.0,101.0,True,True,Fake,
PaintCollections,Knife,1000000.0,101.0,True,True,,Report #951
PaintCollections,Knife,1000000.0,101.0,True,True,,
PaintCollections,Knife,1000000.0,101.0,True,False,Genuine,Report #952
PaintCollections,Knife,1000000.0,101.0,True,False,Genuine,
PaintCollections,Knife,1000000.0,101.0,True,False,Fake,Report #953
PaintCollections,Knife,1000000.0,101.0,True,False,Fake,
PaintCollections,Knife,1000000.0,101.0,True,False,,Report #954
PaintCollections,Knife,1000000.0,101.0,True,False,,
PaintCollections,Knife,1000000.0,101.0,False,True,Genuine,Report #955
PaintCollections,Knife,1000000.0,101.0,False,True,Genuine,
PaintCollections,Knife,1000000.0,101.0,False,True,Fake,Report #956
PaintCollections,Knife,1000000.0,101.0,False,True,Fake,
PaintCollections,Knife,1000000.0,101.0,False,True,,Report #957
PaintCollections,Knife,1000000.0,101.0,False,True,,
PaintCollections,Knife,1000000.0,101.0,False,False,Genuine,Report #958
PaintCollections,Knife,1000000.0,101.0,False,False,Genuine,
PaintCollections,Knife,1000000.0,101.0,False,False,Fake,Report #959
PaintCollections,Knife,1000000.0,101.0,False,False,Fake,
PaintCollections,Knife,1000000.0,101.0,False,False,,Report #960
PaintCollections,Knife,1000000.0,101.0,False,False,,
PaintCollections,Plates,0.0,-1.0,True,True,Genuine,Report #961
PaintCollections,Plates,0.0,-1.0,True,True,Genuine,
PaintCollections,Plates,0.0,-1.0,True,True,Fake,Report #962
PaintCollections,Plates,0.0,-1.0,True,True,Fake,
PaintCollections,Plates,0.0,-1.0,True,True,,Report #963
PaintCollections,Plates,0.0,-1.0,True,True,,
PaintCollections,Plates,0.0,-1.0,True,False,Genuine,Report #964
PaintCollections,Plates,0.0,-1.0,True,False,Genuine,
PaintCollections,Plates,0.0,-1.0,True,False,Fake,Report #965
PaintCollections,Plates,0.0,-1.0,True,False,Fake,
PaintCollections,Plates,0.0,-1.0,True,False,,Report #966
PaintCollections,Plates,0.0,-1.0,True,False,,
PaintCollections,Plates,0.0,-1.0,False,True,Genuine,Report #967
PaintCollections,Plates,0.0,-1.0,False,True,Genuine,
PaintCollections,Plates,0.0,-1.0,False,True,Fake,Report #968
PaintCollections,Plates,0.0,-1.0,False,True,Fake,
PaintCollections,Plates,0.0,-1.0,False,True,,Report #969
PaintCollections,Plates,0.0,-1.0,False,True,,
PaintCollections,Plates,0.0,-1.0,False,False,Genuine,Report #970
PaintCollections,Plates,0.0,-1.0,False,False,Genuine,
PaintCollections,Plates,0.0,-1.0,False,False,Fake,Report #971
PaintCollections,Plates,0.0,-1.0,False,False,Fake,
PaintCollections,Plates,0.0,-1.0,False,False,,Report #972
PaintCollections,Plates,0.0,-1.0,False,False,,
PaintCollections,Plates,0.0,0.0,True,True,Genuine,Report #973
PaintCollections,Plates,0.0,0.0,True,True,Genuine,
PaintCollections,Plates,0.0,0.0,True,True,Fake,Report #974
PaintCollections,Plates,0.0,0.0,True,True,Fake,
PaintCollections,Plates,0.0,0.0,True,True,,Report #975
PaintCollections,Plates,0.0,0.0,True,True,,
PaintCollections,Plates,0.0,0.0,True,False,Genuine,Report #976
PaintCollections,Plates,0.0,0.0,True,False,Genuine,
PaintCollections,Plates,0.0,0.0,True,False,Fake,Report #977
PaintCollections,Plates,0.0,0.0,True,False,Fake,
PaintCollections,Plates,0.0,0.0,True,False,,Report #978
PaintCollections,Plates,0.0,0.0,True,False,,
PaintCollections,Plates,0.0,0.0,False,True,Genuine,Report #979
PaintCollections,Plates,0.0,0.0,False,True,Genuine,
PaintCollections,Plates,0.0,0.0,False,True,Fake,Report #980
PaintCollections,Plates,0.0,0.0,False,True,Fake,
PaintCollections,Plates,0.0,0.0,False,True,,Report #981
PaintCollections,Plates,0.0,0.0,False,True,,
PaintCollections,Plates,0.0,0.0,False,False,Genuine,Report #982
PaintCollections,Plates,0.0,0.0,False,False,Genuine,
PaintCollections,Plates,0.0,0.0,False,False,Fake,Report #983
PaintCollections,Plates,0.0,0.0,False,False,Fake,
PaintCollections,Plates,0.0,0.0,False,False,,Report #984
PaintCollections,Plates,0.0,0.0,False,False,,
PaintCollections,Plates,0.0,19.0,True,True,Genuine,Report #985
PaintCollections,Plates,0.0,19.0,True,True,Genuine,
PaintCollections,Plates,0.0,19.0,True,True,Fake,Report #986
PaintCollections,Plates,0.0,19.0,True,True,Fake,
PaintCollections,Plates,0.0,19.0,True,True,,Report #987
PaintCollections,Plates,0.0,19.0,True,True,,
PaintCollections,Plates,0.0,19.0,True,False,Genuine,Report #988
PaintCollections,Plates,0.0,19.0,True,False,Genuine,
PaintCollections,Plates,0.0,19.0,True,False,Fake,Report #989
PaintCollections,Plates,0.0,19.0,True,False,Fake,
PaintCollections,Plates,0.0,19.0,True,False,,Report #990
PaintCollections,Plates,0.0,19.0,True,False,,
PaintCollections,Plates,0.0,19.0,False,True,Genuine,Report #991
PaintCollections,Plates,0.0,19.0,False,True,Genuine,
PaintCollections,Plates,0.0,19.0,False,True,Fake,Report #992
PaintCollections,Plates,0.0,19.0,False,True,Fake,
PaintCollections,Plates,0.0,19.0,False,True,,Report #993
PaintCollections,Plates,0.0,19.0,False,True,,
PaintCollections,Plates,0.0,19.0,False,False,Genuine,Report #994
PaintCollections,Plates,0.0,19.0,False,False,Genuine,
PaintCollections,Plates,0.0,19.0,False,False,Fake,Report #995
PaintCollections,Plates,0.0,19.0,False,False,Fake,
PaintCollections,Plates,0.0,19.0,False,False,,Report #996
PaintCollections,Plates,0.0,19.0,False,False,,
PaintCollections,Plates,0.0,20.0,True,True,Genuine,Report #997
PaintCollections,Plates,0.0,20.0,True,True,Genuine,
PaintCollections,Plates,0.0,20.0,True,True,Fake,Report #998
PaintCollections,Plates,0.0,20.0,True,True,Fake,
PaintCollections,Plates,0.0,20.0,True,True,,Report #999
PaintCollections,Plates,0.0,20.0,True,True,,
PaintCollections,Plates,0.0,20.0,True,False,Genuine,Report #1000
PaintCollections,Plates,0.0,20.0,True,False,Genuine,
PaintCollections,Plates,0.0,20.0,True,False,Fake,Report #1001
PaintCollections,Plates,0.0,20.0,True,False,Fake,
PaintCollections,Plates,0.0,20.0,True,False,,Report #1002
PaintCollections,Plates,0.0,20.0,True,False,,
PaintCollections,Plates,0.0,20.0,False,True,Genuine,Report #1003
PaintCollections,Plates,0.0,20.0,False,True,Genuine,
PaintCollections,Plates,0.0,20.0,False,True,Fake,Report #1004
PaintCollections,Plates,0.0,20.0,False,True,Fake,
PaintCollections,Plates,0.0,20.0,False,True,,Report #1005
PaintCollections,Plates,0.0,20.0,False,True,,
PaintCollections,Plates,0.0,20.0,False,False,Genuine,Report #1006
PaintCollections,Plates,0.0,20.0,False,False,Genuine,
PaintCollections,Plates,0.0,20.0,False,False,Fake,Report #1007
PaintCollections,Plates,0.0,20.0,False,False,Fake,
PaintCollections,Plates,0.0,20.0,False,False,,Report #1008
PaintCollections,Plates,0.0,20.0,False,False,,
PaintCollections,Plates,0.0,21.0,True,True,Genuine,Report #1009
PaintCollections,Plates,0.0,21.0,True,True,Genuine,
PaintCollections,Plates,0.0,21.0,True,True,Fake,Report #1010
PaintCollections,Plates,0.0,21.0,True,True,Fake,
PaintCollections,Plates,0.0,21.0,True,True,,Report #1011
PaintCollections,Plates,0.0,21.0,True,True,,
PaintCollections,Plates,0.0,21.0,True,False,Genuine,Report #1012
PaintCollections,Plates,0.0,21.0,True,False,Genuine,
PaintCollections,Plates,0.0,21.0,True,False,Fake,Report #1013
PaintCollections,Plates,0.0,21.0,True,False,Fake,
PaintCollections,Plates,0.0,21.0,True,False,,Report #1014
PaintCollections,Plates,0.0,21.0,True,False,,
PaintCollections,Plates,0.0,21.0,False,True,Genuine,Report #1015
PaintCollections,Plates,0.0,21.0,False,True,Genuine,
PaintCollections,Plates,0.0,21.0,False,True,Fake,Report #1016
PaintCollections,Plates,0.0,21.0,False,True,Fake,
PaintCollections,Plates,0.0,21.0,False,True,,Report #1017
PaintCollections,Plates,0.0,21.0,False,True,,
PaintCollections,Plates,0.0,21.0,False,False,Genuine,Report #1018
PaintCollections,Plates,0.0,21.0,False,False,Genuine,
PaintCollections,Plates,0.0,21.0,False,False,Fake,Report #1019
PaintCollections,Plates,0.0,21.0,False,False,Fake,
PaintCollections,Plates,0.0,21.0,False,False,,Report #1020
PaintCollections,Plates,0.0,21.0,False,False,,
PaintCollections,Plates,0.0,99.0,True,True,Genuine,Report #1021
PaintCollections,Plates,0.0,99.0,True,True,Genuine,
PaintCollections,Plates,0.0,99.0,True,True,Fake,Report #1022
PaintCollections,Plates,0.0,99.0,True,True,Fake,
PaintCollections,Plates,0.0,99.0,True,True,,Report #1023
PaintCollections,Plates,0.0,99.0,True,True,,
PaintCollections,Plates,0.0,99.0,True,False,Genuine,Report #1024
PaintCollections,Plates,0.0,99.0,True,False,Genuine,
PaintCollections,Plates,0.0,99.0,True,False,Fake,Report #1025
PaintCollections,Plates,0.0,99.0,True,False,Fake,
PaintCollections,Plates,0.0,99.0,True,False,,Report #1026
PaintCollections,Plates,0.0,99.0,True,False,,
PaintCollections,Plates,0.0,99.0,False,True,Genuine,Report #1027
PaintCollections,Plates,0.0,99.0,False,True,Genuine,
PaintCollections,Plates,0.0,99.0,False,True,Fake,Report #1028
PaintCollections,Plates,0.0,99.0,False,True,Fake,
PaintCollections,Plates,0.0,99.0,False,True,,Report #1029
PaintCollections,Plates,0.0,99.0,False,True,,
PaintCollections,Plates,0.0,99.0,False,False,Genuine,Report #1030
PaintCollections,Plates,0.0,99.0,False,False,Genuine,
PaintCollections,Plates,0.0,99.0,False,False,Fake,Report #1031
PaintCollections,Plates,0.0,99.0,False,False,Fake,
PaintCollections,Plates,0.0,99.0,False,False,,Report #1032
PaintCollections,Plates,0.0,99.0,False,False,,
PaintCollections,Plates,0.0,100.0,True,True,Genuine,Report #1033
PaintCollections,Plates,0.0,100.0,True,True,Genuine,
PaintCollections,Plates,0.0,100.0,True,True,Fake,Report #1034
PaintCollections,Plates,0.0,100.0,True,True,Fake,
PaintCollections,Plates,0.0,100.0,True,True,,Report #1035
PaintCollections,Plates,0.0,100.0,True,True,,
PaintCollections,Plates,0.0,100.0,True,False,Genuine,Report #1036
PaintCollections,Plates,0.0,100.0,True,False,Genuine,
PaintCollections,Plates,0.0,100.0,True,False,Fake,Report #1037
PaintCollections,Plates,0.0,100.0,True,False,Fake,
PaintCollections,Plates,0.0,100.0,True,False,,Report #1038
PaintCollections,Plates,0.0,100.0,True,False,,
PaintCollections,Plates,0.0,100.0,False,True,Genuine,Report #1039
PaintCollections,Plates,0.0,100.0,False,True,Genuine,
PaintCollections,Plates,0.0,100.0,False,True,Fake,Report #1040
PaintCollections,Plates,0.0,100.0,False,True,Fake,
PaintCollections,Plates,0.0,100.0,False,True,,Report #1041
PaintCollections,Plates,0.0,100.0,False,True,,
PaintCollections,Plates,0.0,100.0,False,False,Genuine,Report #1042
PaintCollections,Plates,0.0,100.0,False,False,Genuine,
PaintCollections,Plates,0.0,100.0,False,False,Fake,Report #1043
PaintCollections,Plates,0.0,100.0,False,False,Fake,
PaintCollections,Plates,0.0,100.0,False,False,,Report #1044
PaintCollections,Plates,0.0,100.0,False,False,,
PaintCollections,Plates,0.0,101.0,True,True,Genuine,Report #1045
PaintCollections,Plates,0.0,101.0,True,True,Genuine,
PaintCollections,Plates,0.0,101.0,True,True,Fake,Report #1046
PaintCollections,Plates,0.0,101.0,True,True,Fake,
PaintCollections,Plates,0.0,101.0,True,True,,Report #1047
PaintCollections,Plates,0.0,101.0,True,True,,
PaintCollections,Plates,0.0,101.0,True,False,Genuine,Report #1048
PaintCollections,Plates,0.0,101.0,True,False,Genuine,
PaintCollections,Plates,0.0,101.0,True,False,Fake,Report #1049
PaintCollections,Plates,0.0,101.0,True,False,Fake,
PaintCollections,Plates,0.0,101.0,True,False,,Report #1050
PaintCollections,Plates,0.0,101.0,True,False,,
PaintCollections,Plates,0.0,101.0,False,True,Genuine,Report #1051
PaintCollections,Plates,0.0,101.0,False,True,Genuine,
PaintCollections,Plates,0.0,101.0,False,True,Fake,Report #1052
PaintCollections,Plates,0.0,101.0,False,True,Fake,
PaintCollections,Plates,0.0,101.0,False,True,,Report #1053
PaintCollections,Plates,0.0,101.0,False,True,,
PaintCollections,Plates,0.0,101.0,False,False,Genuine,Report #1054
PaintCollections,Plates,0.0,101.0,False,False,Genuine,
PaintCollections,Plates,0.0,101.0,False,False,Fake,Report #1055
PaintCollections,Plates,0.0,101.0,False,False,Fake,
PaintCollections,Plates,0.0,101.0,False,False,,Report #1056
PaintCollections,Plates,0.0,101.0,False,False,,
PaintCollections,Plates,1.0,-1.0,True,True,Genuine,Report #1057
PaintCollections,Plates,1.0,-1.0,True,True,Genuine,
PaintCollections,Plates,1.0,-1.0,True,True,Fake,Report #1058
PaintCollections,Plates,1.0,-1.0,True,True,Fake,
PaintCollections,Plates,1.0,-1.0,True,True,,Report #1059
PaintCollections,Plates,1.0,-1.0,True,True,,
PaintCollections,Plates,1.0,-1.0,True,False,Genuine,Report #1060
PaintCollections,Plates,1.0,-1.0,True,False,Genuine,
PaintCollections,Plates,1.0,-1.0,True,False,Fake,Report #1061
PaintCollections,Plates,1.0,-1.0,True,False,Fake,
PaintCollections,Plates,1.0,-1.0,True,False,,Report #1062
PaintCollections,Plates,1.0,-1.0,True,False,,
PaintCollections,Plates,1.0,-1.0,False,True,Genuine,Report #1063
PaintCollections,Plates,1.0,-1.0,False,True,Genuine,
PaintCollections,Plates,1.0,-1.0,False,True,Fake,Report #1064
PaintCollections,Plates,1.0,-1.0,False,True,Fake,
PaintCollections,Plates,1.0,-1.0,False,True,,Report #1065
PaintCollections,Plates,1.0,-1.0,False,True,,
PaintCollections,Plates,1.0,-1.0,False,False,Genuine,Report #1066
PaintCollections,Plates,1.0,-1.0,False,False,Genuine,
PaintCollections,Plates,1.0,-1.0,False,False,Fake,Report #1067
PaintCollections,Plates,1.0,-1.0,False,False,Fake,
PaintCollections,Plates,1.0,-1.0,False,False,,Report #1068
PaintCollections,Plates,1.0,-1.0,False,False,,
PaintCollections,Plates,1.0,0.0,True,True,Genuine,Report #1069
PaintCollections,Plates,1.0,0.0,True,True,Genuine,
PaintCollections,Plates,1.0,0.0,True,True,Fake,Report #1070
PaintCollections,Plates,1.0,0.0,True,True,Fake,
PaintCollections,Plates,1.0,0.0,True,True,,Report #1071
PaintCollections,Plates,1.0,0.0,True,True,,
PaintCollections,Plates,1.0,0.0,True,False,Genuine,Report #1072
PaintCollections,Plates,1.0,0.0,True,False,Genuine,
PaintCollections,Plates,1.0,0.0,True,False,Fake,Report #1073
PaintCollections,Plates,1.0,0.0,True,False,Fake,
PaintCollections,Plates,1.0,0.0,True,False,,Report #1074
PaintCollections,Plates,1.0,0.0,True,False,,
PaintCollections,Plates,1.0,0.0,False,True,Genuine,Report #1075
PaintCollections,Plates,1.0,0.0,False,True,Genuine,
PaintCollections,Plates,1.0,0.0,False,True,Fake,Report #1076
PaintCollections,Plates,1.0,0.0,False,True,Fake,
PaintCollections,Plates,1.0,0.0,False,True,,Report #1077
PaintCollections,Plates,1.0,0.0,False,True,,
PaintCollections,Plates,1.0,0.0,False,False,Genuine,Report #1078
PaintCollections,Plates,1.0,0.0,False,False,Genuine,
PaintCollections,Plates,1.0,0.0,False,False,Fake,Report #1079
PaintCollections,Plates,1.0,0.0,False,False,Fake,
PaintCollections,Plates,1.0,0.0,False,False,,Report #1080
PaintCollections,Plates,1.0,0.0,False,False,,
PaintCollections,Plates,1.0,19.0,True,True,Genuine,Report #1081
PaintCollections,Plates,1.0,19.0,True,True,Genuine,
PaintCollections,Plates,1.0,19.0,True,True,Fake,Report #1082
PaintCollections,Plates,1.0,19.0,True,True,Fake,
PaintCollections,Plates,1.0,19.0,True,True,,Report #1083
PaintCollections,Plates,1.0,19.0,True,True,,
PaintCollections,Plates,1.0,19.0,True,False,Genuine,Report #1084
PaintCollections,Plates,1.0,19.0,True,False,Genuine,
PaintCollections,Plates,1.0,19.0,True,False,Fake,Report #1085
PaintCollections,Plates,1.0,19.0,True,False,Fake,
PaintCollections,Plates,1.0,19.0,True,False,,Report #1086
PaintCollections,Plates,1.0,19.0,True,False,,
PaintCollections,Plates,1.0,19.0,False,True,Genuine,Report #1087
PaintCollections,Plates,1.0,19.0,False,True,Genuine,
PaintCollections,Plates,1.0,19.0,False,True,Fake,Report #1088
PaintCollections,Plates,1.0,19.0,False,True,Fake,
PaintCollections,Plates,1.0,19.0,False,True,,Report #1089
PaintCollections,Plates,1.0,19.0,False,True,,
PaintCollections,Plates,1.0,19.0,False,False,Genuine,Report #1090
PaintCollections,Plates,1.0,19.0,False,False,Genuine,
PaintCollections,Plates,1.0,19.0,False,False,Fake,Report #1091
PaintCollections,Plates,1.0,19.0,False,False,Fake,
PaintCollections,Plates,1.0,19.0,False,False,,Report #1092
PaintCollections,Plates,1.0,19.0,False,False,,
PaintCollections,Plates,1.0,20.0,True,True,Genuine,Report #1093
PaintCollections,Plates,1.0,20.0,True,True,Genuine,
PaintCollections,Plates,1.0,20.0,True,True,Fake,Report #1094
PaintCollections,Plates,1.0,20.0,True,True,Fake,
PaintCollections,Plates,1.0,20.0,True,True,,Report #1095
PaintCollections,Plates,1.0,20.0,True,True,,
PaintCollections,Plates,1.0,20.0,True,False,Genuine,Report #1096
PaintCollections,Plates,1.0,20.0,True,False,Genuine,
PaintCollections,Plates,1.0,20.0,True,False,Fake,Report #1097
PaintCollections,Plates,1.0,20.0,True,False,Fake,
PaintCollections,Plates,1.0,20.0,True,False,,Report #1098
PaintCollections,Plates,1.0,20.0,True,False,,
PaintCollections,Plates,1.0,20.0,False,True,Genuine,Report #1099
PaintCollections,Plates,1.0,20.0,False,True,Genuine,
PaintCollections,Plates,1.0,20.0,False,True,Fake,Report #1100
PaintCollections,Plates,1.0,20.0,False,True,Fake,
PaintCollections,Plates,1.0,20.0,False,True,,Report #1101
PaintCollections,Plates,1.0,20.0,False,True,,
PaintCollections,Plates,1.0,20.0,False,False,Genuine,Report #1102
PaintCollections,Plates,1.0,20.0,False,False,Genuine,
PaintCollections,Plates,1.0,20.0,False,False,Fake,Report #1103
PaintCollections,Plates,1.0,20.0,False,False,Fake,
PaintCollections,Plates,1.0,20.0,False,False,,Report #1104
PaintCollections,Plates,1.0,20.0,False,False,,
PaintCollections,Plates,1.0,21.0,True,True,Genuine,Report #1105
PaintCollections,Plates,1.0,21.0,True,True,Genuine,
PaintCollections,Plates,1.0,21.0,True,True,Fake,Report #1106
PaintCollections,Plates,1.0,21.0,True,True,Fake,
PaintCollections,Plates,1.0,21.0,True,True,,Report #1107
PaintCollections,Plates,1.0,21.0,True,True,,
PaintCollections,Plates,1.0,21.0,True,False,Genuine,Report #1108
PaintCollections,Plates,1.0,21.0,True,False,Genuine,
PaintCollections,Plates,1.0,21.0,True,False,Fake,Report #1109
PaintCollections,Plates,1.0,21.0,True,False,Fake,
PaintCollections,Plates,1.0,21.0,True,False,,Report #1110
PaintCollections,Plates,1.0,21.0,True,False,,
PaintCollections,Plates,1.0,21.0,False,True,Genuine,Report #1111
PaintCollections,Plates,1.0,21.0,False,True,Genuine,
PaintCollections,Plates,1.0,21.0,False,True,Fake,Report #1112
PaintCollections,Plates,1.0,21.0,False,True,Fake,
PaintCollections,Plates,1.0,21.0,False,True,,Report #1113
PaintCollections,Plates,1.0,21.0,False,True,,
PaintCollections,Plates,1.0,21.0,False,False,Genuine,Report #1114
PaintCollections,Plates,1.0,21.0,False,False,Genuine,
PaintCollections,Plates,1.0,21.0,False,False,Fake,Report #1115
PaintCollections,Plates,1.0,21.0,False,False,Fake,
PaintCollections,Plates,1.0,21.0,False,False,,Report #1116
PaintCollections,Plates,1.0,21.0,False,False,,
PaintCollections,Plates,1.0,99.0,True,True,Genuine,Report #1117
PaintCollections,Plates,1.0,99.0,True,True,Genuine,
PaintCollections,Plates,1.0,99.0,True,True,Fake,Report #1118
PaintCollections,Plates,1.0,99.0,True,True,Fake,
PaintCollections,Plates,1.0,99.0,True,True,,Report #1119
PaintCollections,Plates,1.0,99.0,True,True,,
PaintCollections,Plates,1.0,99.0,True,False,Genuine,Report #1120
PaintCollections,Plates,1.0,99.0,True,False,Genuine,
PaintCollections,Plates,1.0,99.0,True,False,Fake,Report #1121
PaintCollections,Plates,1.0,99.0,True,False,Fake,
PaintCollections,Plates,1.0,99.0,True,False,,Report #1122
PaintCollections,Plates,1.0,99.0,True,False,,
PaintCollections,Plates,1.0,99.0,False,True,Genuine,Report #1123
PaintCollections,Plates,1.0,99.0,False,True,Genuine,
PaintCollections,Plates,1.0,99.0,False,True,Fake,Report #1124
PaintCollections,Plates,1.0,99.0,False,True,Fake,
PaintCollections,Plates,1.0,99.0,False,True,,Report #1125
PaintCollections,Plates,1.0,99.0,False,True,,
PaintCollections,Plates,1.0,99.0,False,False,Genuine,Report #1126
PaintCollections,Plates,1.0,99.0,False,False,Genuine,
PaintCollections,Plates,1.0,99.0,False,False,Fake,Report #1127
PaintCollections,Plates,1.0,99.0,False,False,Fake,
PaintCollections,Plates,1.0,99.0,False,False,,Report #1128
PaintCollections,Plates,1.0,99.0,False,False,,
PaintCollections,Plates,1.0,100.0,True,True,Genuine,Report #1129
PaintCollections,Plates,1.0,100.0,True,True,Genuine,
PaintCollections,Plates,1.0,100.0,True,True,Fake,Report #1130
PaintCollections,Plates,1.0,100.0,True,True,Fake,
PaintCollections,Plates,1.0,100.0,True,True,,Report #1131
PaintCollections,Plates,1.0,100.0,True,True,,
PaintCollections,Plates,1.0,100.0,True,False,Genuine,Report #1132
PaintCollections,Plates,1.0,100.0,True,False,Genuine,
PaintCollections,Plates,1.0,100.0,True,False,Fake,Report #1133
PaintCollections,Plates,1.0,100.0,True,False,Fake,
PaintCollections,Plates,1.0,100.0,True,False,,Report #1134
PaintCollections,Plates,1.0,100.0,True,False,,
PaintCollections,Plates,1.0,100.0,False,True,Genuine,Report #1135
PaintCollections,Plates,1.0,100.0,False,True,Genuine,
PaintCollections,Plates,1.0,100.0,False,True,Fake,Report #1136
PaintCollections,Plates,1.0,100.0,False,True,Fake,
PaintCollections,Plates,1.0,100.0,False,True,,Report #1137
PaintCollections,Plates,1.0,100.0,False,True,,
PaintCollections,Plates,1.0,100.0,False,False,Genuine,Report #1138
PaintCollections,Plates,1.0,100.0,False,False,Genuine,
PaintCollections,Plates,1.0,100.0,False,False,Fake,Report #1139
PaintCollections,Plates,1.0,100.0,False,False,Fake,
PaintCollections,Plates,1.0,100.0,False,False,,Report #1140
PaintCollections,Plates,1.0,100.0,False,False,,
PaintCollections,Plates,1.0,101.0,True,True,Genuine,Report #1141
PaintCollections,Plates,1.0,101.0,True,True,Genuine,
PaintCollections,Plates,1.0,101.0,True,True,Fake,Report #1142
PaintCollections,Plates,1.0,101.0,True,True,Fake,
PaintCollections,Plates,1.0,101.0,True,True,,Report #1143
PaintCollections,Plates,1.0,101.0,True,True,,
PaintCollections,Plates,1.0,101.0,True,False,Genuine,Report #1144
PaintCollections,Plates,1.0,101.0,True,False,Genuine,
PaintCollections,Plates,1.0,101.0,True,False,Fake,Report #1145
PaintCollections,Plates,1.0,101.0,True,False,Fake,
PaintCollections,Plates,1.0,101.0,True,False,,Report #1146
PaintCollections,Plates,1.0,101.0,True,False,,
PaintCollections,Plates,1.0,101.0,False,True,Genuine,Report #1147
PaintCollections,Plates,1.0,101.0,False,True,Genuine,
PaintCollections,Plates,1.0,101.0,False,True,Fake,Report #1148
PaintCollections,Plates,1.0,101.0,False,True,Fake,
PaintCollections,Plates,1.0,101.0,False,True,,Report #1149
PaintCollections,Plates,1.0,101.0,False,True,,
PaintCollections,Plates,1.0,101.0,False,False,Genuine,Report #1150
PaintCollections,Plates,1.0,101.0,False,False,Genuine,
PaintCollections,Plates,1.0,101.0,False,False,Fake,Report #1151
PaintCollections,Plates,1.0,101.0,False,False,Fake,
PaintCollections,Plates,1.0,101.0,False,False,,Report #1152
PaintCollections,Plates,1.0,101.0,False,False,,
PaintCollections,Plates,100.0,-1.0,True,True,Genuine,Report #1153
PaintCollections,Plates,100.0,-1.0,True,True,Genuine,
PaintCollections,Plates,100.0,-1.0,True,True,Fake,Report #1154
PaintCollections,Plates,100.0,-1.0,True,True,Fake,
PaintCollections,Plates,100.0,-1.0,True,True,,Report #1155
PaintCollections,Plates,100.0,-1.0,True,True,,
PaintCollections,Plates,100.0,-1.0,True,False,Genuine,Report #1156
PaintCollections,Plates,100.0,-1.0,True,False,Genuine,
PaintCollections,Plates,100.0,-1.0,True,False,Fake,Report #1157
PaintCollections,Plates,100.0,-1.0,True,False,Fake,
PaintCollections,Plates,100.0,-1.0,True,False,,Report #1158
PaintCollections,Plates,100.0,-1.0,True,False,,
PaintCollections,Plates,100.0,-1.0,False,True,Genuine,Report #1159
PaintCollections,Plates,100.0,-1.0,False,True,Genuine,
PaintCollections,Plates,100.0,-1.0,False,True,Fake,Report #1160
PaintCollections,Plates,100.0,-1.0,False,True,Fake,
PaintCollections,Plates,100.0,-1.0,False,True,,Report #1161
PaintCollections,Plates,100.0,-1.0,False,True,,
PaintCollections,Plates,100.0,-1.0,False,False,Genuine,Report #1162
PaintCollections,Plates,100.0,-1.0,False,False,Genuine,
PaintCollections,Plates,100.0,-1.0,False,False,Fake,Report #1163
PaintCollections,Plates,100.0,-1.0,False,False,Fake,
PaintCollections,Plates,100.0,-1.0,False,False,,Report #1164
PaintCollections,Plates,100.0,-1.0,False,False,,
PaintCollections,Plates,100.0,0.0,True,True,Genuine,Report #1165
PaintCollections,Plates,100.0,0.0,True,True,Genuine,
PaintCollections,Plates,100.0,0.0,True,True,Fake,Report #1166
PaintCollections,Plates,100.0,0.0,True,True,Fake,
PaintCollections,Plates,100.0,0.0,True,True,,Report #1167
PaintCollections,Plates,100.0,0.0,True,True,,
PaintCollections,Plates,100.0,0.0,True,False,Genuine,Report #1168
PaintCollections,Plates,100.0,0.0,True,False,Genuine,
PaintCollections,Plates,100.0,0.0,True,False,Fake,Report #1169
PaintCollections,Plates,100.0,0.0,True,False,Fake,
PaintCollections,Plates,100.0,0.0,True,False,,Report #1170
PaintCollections,Plates,100.0,0.0,True,False,,
PaintCollections,Plates,100.0,0.0,False,True,Genuine,Report #1171
PaintCollections,Plates,100.0,0.0,False,True,Genuine,
PaintCollections,Plates,100.0,0.0,False,True,Fake,Report #1172
PaintCollections,Plates,100.0,0.0,False,True,Fake,
PaintCollections,Plates,100.0,0.0,False,True,,Report #1173
PaintCollections,Plates,100.0,0.0,False,True,,
PaintCollections,Plates,100.0,0.0,False,False,Genuine,Report #1174
PaintCollections,Plates,100.0,0.0,False,False,Genuine,
PaintCollections,Plates,100.0,0.0,False,False,Fake,Report #1175
PaintCollections,Plates,100.0,0.0,False,False,Fake,
PaintCollections,Plates,100.0,0.0,False,False,,Report #1176
PaintCollections,Plates,100.0,0.0,False,False,,
PaintCollections,Plates,100.0,19.0,True,True,Genuine,Report #1177
PaintCollections,Plates,100.0,19.0,True,True,Genuine,
PaintCollections,Plates,100.0,19.0,True,True,Fake,Report #1178
PaintCollections,Plates,100.0,19.0,True,True,Fake,
PaintCollections,Plates,100.0,19.0,True,True,,Report #1179
PaintCollections,Plates,100.0,19.0,True,True,,
PaintCollections,Plates,100.0,19.0,True,False,Genuine,Report #1180
PaintCollections,Plates,100.0,19.0,True,False,Genuine,
PaintCollections,Plates,100.0,19.0,True,False,Fake,Report #1181
PaintCollections,Plates,100.0,19.0,True,False,Fake,
PaintCollections,Plates,100.0,19.0,True,False,,Report #1182
PaintCollections,Plates,100.0,19.0,True,False,,
PaintCollections,Plates,100.0,19.0,False,True,Genuine,Report #1183
PaintCollections,Plates,100.0,19.0,False,True,Genuine,
PaintCollections,Plates,100.0,19.0,False,True,Fake,Report #1184
PaintCollections,Plates,100.0,19.0,False,True,Fake,
PaintCollections,Plates,100.0,19.0,False,True,,Report #1185
PaintCollections,Plates,100.0,19.0,False,True,,
PaintCollections,Plates,100.0,19.0,False,False,Genuine,Report #1186
PaintCollections,Plates,100.0,19.0,False,False,Genuine,
PaintCollections,Plates,100.0,19.0,False,False,Fake,Report #1187
PaintCollections,Plates,100.0,19.0,False,False,Fake,
PaintCollections,Plates,100.0,19.0,False,False,,Report #1188
PaintCollections,Plates,100.0,19.0,False,False,,
PaintCollections,Plates,100.0,20.0,True,True,Genuine,Report #1189
PaintCollections,Plates,100.0,20.0,True,True,Genuine,
PaintCollections,Plates,100.0,20.0,True,True,Fake,Report #1190
PaintCollections,Plates,100.0,20.0,True,True,Fake,
PaintCollections,Plates,100.0,20.0,True,True,,Report #1191
PaintCollections,Plates,100.0,20.0,True,True,,
PaintCollections,Plates,100.0,20.0,True,False,Genuine,Report #1192
PaintCollections,Plates,100.0,20.0,True,False,Genuine,
PaintCollections,Plates,100.0,20.0,True,False,Fake,Report #1193
PaintCollections,Plates,100.0,20.0,True,False,Fake,
PaintCollections,Plates,100.0,20.0,True,False,,Report #1194
PaintCollections,Plates,100.0,20.0,True,False,,
PaintCollections,Plates,100.0,20.0,False,True,Genuine,Report #1195
PaintCollections,Plates,100.0,20.0,False,True,Genuine,
PaintCollections,Plates,100.0,20.0,False,True,Fake,Report #1196
PaintCollections,Plates,100.0,20.0,False,True,Fake,
PaintCollections,Plates,100.0,20.0,False,True,,Report #1197
PaintCollections,Plates,100.0,20.0,False,True,,
PaintCollections,Plates,100.0,20.0,False,False,Genuine,Report #1198
PaintCollections,Plates,100.0,20.0,False,False,Genuine,
PaintCollections,Plates,100.0,20.0,False,False,Fake,Report #1199
PaintCollections,Plates,100.0,20.0,False,False,Fake,
PaintCollections,Plates,100.0,20.0,False,False,,Report #1200
PaintCollections,Plates,100.0,20.0,False,False,,
PaintCollections,Plates,100.0,21.0,True,True,Genuine,Report #1201
PaintCollections,Plates,100.0,21.0,True,True,Genuine,
PaintCollections,Plates,100.0,21.0,True,True,Fake,Report #1202
PaintCollections,Plates,100.0,21.0,True,True,Fake,
PaintCollections,Plates,100.0,21.0,True,True,,Report #1203
PaintCollections,Plates,100.0,21.0,True,True,,
PaintCollections,Plates,100.0,21.0,True,False,Genuine,Report #1204
PaintCollections,Plates,100.0,21.0,True,False,Genuine,
PaintCollections,Plates,100.0,21.0,True,False,Fake,Report #1205
PaintCollections,Plates,100.0,21.0,True,False,Fake,
PaintCollections,Plates,100.0,21.0,True,False,,Report #1206
PaintCollections,Plates,100.0,21.0,True,False,,
PaintCollections,Plates,100.0,21.0,False,True,Genuine,Report #1207
PaintCollections,Plates,100.0,21.0,False,True,Genuine,
PaintCollections,Plates,100.0,21.0,False,True,Fake,Report #1208
PaintCollections,Plates,100.0,21.0,False,True,Fake,
PaintCollections,Plates,100.0,21.0,False,True,,Report #1209
PaintCollections,Plates,100.0,21.0,False,True,,
PaintCollections,Plates,100.0,21.0,False,False,Genuine,Report #1210
PaintCollections,Plates,100.0,21.0,False,False,Genuine,
PaintCollections,Plates,100.0,21.0,False,False,Fake,Report #1211
PaintCollections,Plates,100.0,21.0,False,False,Fake,
PaintCollections,Plates,100.0,21.0,False,False,,Report #1212
PaintCollections,Plates,100.0,21.0,False,False,,
PaintCollections,Plates,100.0,99.0,True,True,Genuine,Report #1213
PaintCollections,Plates,100.0,99.0,True,True,Genuine,
PaintCollections,Plates,100.0,99.0,True,True,Fake,Report #1214
PaintCollections,Plates,100.0,99.0,True,True,Fake,
PaintCollections,Plates,100.0,99.0,True,True,,Report #1215
PaintCollections,Plates,100.0,99.0,True,True,,
PaintCollections,Plates,100.0,99.0,True,False,Genuine,Report #1216
PaintCollections,Plates,100.0,99.0,True,False,Genuine,
PaintCollections,Plates,100.0,99.0,True,False,Fake,Report #1217
PaintCollections,Plates,100.0,99.0,True,False,Fake,
PaintCollections,Plates,100.0,99.0,True,False,,Report #1218
PaintCollections,Plates,100.0,99.0,True,False,,
PaintCollections,Plates,100.0,99.0,False,True,Genuine,Report #1219
PaintCollections,Plates,100.0,99.0,False,True,Genuine,
PaintCollections,Plates,100.0,99.0,False,True,Fake,Report #1220
PaintCollections,Plates,100.0,99.0,False,True,Fake,
PaintCollections,Plates,100.0,99.0,False,True,,Report #1221
PaintCollections,Plates,100.0,99.0,False,True,,
PaintCollections,Plates,100.0,99.0,False,False,Genuine,Report #1222
PaintCollections,Plates,100.0,99.0,False,False,Genuine,
PaintCollections,Plates,100.0,99.0,False,False,Fake,Report #1223
PaintCollections,Plates,100.0,99.0,False,False,Fake,
PaintCollections,Plates,100.0,99.0,False,False,,Report #1224
PaintCollections,Plates,100.0,99.0,False,False,,
PaintCollections,Plates,100.0,100.0,True,True,Genuine,Report #1225
PaintCollections,Plates,100.0,100.0,True,True,Genuine,
PaintCollections,Plates,100.0,100.0,True,True,Fake,Report #1226
PaintCollections,Plates,100.0,100.0,True,True,Fake,
PaintCollections,Plates,100.0,100.0,True,True,,Report #1227
PaintCollections,Plates,100.0,100.0,True,True,,
PaintCollections,Plates,100.0,100.0,True,False,Genuine,Report #1228
PaintCollections,Plates,100.0,100.0,True,False,Genuine,
PaintCollections,Plates,100.0,100.0,True,False,Fake,Report #1229
PaintCollections,Plates,100.0,100.0,True,False,Fake,
PaintCollections,Plates,100.0,100.0,True,False,,Report #1230
PaintCollections,Plates,100.0,100.0,True,False,,
PaintCollections,Plates,100.0,100.0,False,True,Genuine,Report #1231
PaintCollections,Plates,100.0,100.0,False,True,Genuine,
PaintCollections,Plates,100.0,100.0,False,True,Fake,Report #1232
PaintCollections,Plates,100.0,100.0,False,True,Fake,
PaintCollections,Plates,100.0,100.0,False,True,,Report #1233
PaintCollections,Plates,100.0,100.0,False,True,,
PaintCollections,Plates,100.0,100.0,False,False,Genuine,Report #1234
PaintCollections,Plates,100.0,100.0,False,False,Genuine,
PaintCollections,Plates,100.0,100.0,False,False,Fake,Report #1235
PaintCollections,Plates,100.0,100.0,False,False,Fake,
PaintCollections,Plates,100.0,100.0,False,False,,Report #1236
PaintCollections,Plates,100.0,100.0,False,False,,
PaintCollections,Plates,100.0,101.0,True,True,Genuine,Report #1237
PaintCollections,Plates,100.0,101.0,True,True,Genuine,
PaintCollections,Plates,100.0,101.0,True,True,Fake,Report #1238
PaintCollections,Plates,100.0,101.0,True,True,Fake,
PaintCollections,Plates,100.0,101.0,True,True,,Report #1239
PaintCollections,Plates,100.0,101.0,True,True,,
PaintCollections,Plates,100.0,101.0,True,False,Genuine,Report #1240
PaintCollections,Plates,100.0,101.0,True,False,Genuine,
PaintCollections,Plates,100.0,101.0,True,False,Fake,Report #1241
PaintCollections,Plates,100.0,101.0,True,False,Fake,
PaintCollections,Plates,100.0,101.0,True,False,,Report #1242
PaintCollections,Plates,100.0,101.0,True,False,,
PaintCollections,Plates,100.0,101.0,False,True,Genuine,Report #1243
PaintCollections,Plates,100.0,101.0,False,True,Genuine,
PaintCollections,Plates,100.0,101.0,False,True,Fake,Report #1244
PaintCollections,Plates,100.0,101.0,False,True,Fake,
PaintCollections,Plates,100.0,101.0,False,True,,Report #1245
PaintCollections,Plates,100.0,101.0,False,True,,
PaintCollections,Plates,100.0,101.0,False,False,Genuine,Report #1246
PaintCollections,Plates,100.0,101.0,False,False,Genuine,
PaintCollections,Plates,100.0,101.0,False,False,Fake,Report #1247
PaintCollections,Plates,100.0,101.0,False,False,Fake,
PaintCollections,Plates,100.0,101.0,False,False,,Report #1248
PaintCollections,Plates,100.0,101.0,False,False,,
PaintCollections,Plates,1000.0,-1.0,True,True,Genuine,Report #1249
PaintCollections,Plates,1000.0,-1.0,True,True,Genuine,
PaintCollections,Plates,1000.0,-1.0,True,True,Fake,Report #1250
PaintCollections,Plates,1000.0,-1.0,True,True,Fake,
PaintCollections,Plates,1000.0,-1.0,True,True,,Report #1251
PaintCollections,Plates,1000.0,-1.0,True,True,,
PaintCollections,Plates,1000.0,-1.0,True,False,Genuine,Report #1252
PaintCollections,Plates,1000.0,-1.0,True,False,Genuine,
PaintCollections,Plates,1000.0,-1.0,True,False,Fake,Report #1253
PaintCollections,Plates,1000.0,-1.0,True,False,Fake,
PaintCollections,Plates,1000.0,-1.0,True,False,,Report #1254
PaintCollections,Plates,1000.0,-1.0,True,False,,
PaintCollections,Plates,1000.0,-1.0,False,True,Genuine,Report #1255
PaintCollections,Plates,1000.0,-1.0,False,True,Genuine,
PaintCollections,Plates,1000.0,-1.0,False,True,Fake,Report #1256
PaintCollections,Plates,1000.0,-1.0,False,True,Fake,
PaintCollections,Plates,1000.0,-1.0,False,True,,Report #1257
PaintCollections,Plates,1000.0,-1.0,False,True,,
PaintCollections,Plates,1000.0,-1.0,False,False,Genuine,Report #1258
PaintCollections,Plates,1000.0,-1.0,False,False,Genuine,
PaintCollections,Plates,1000.0,-1.0,False,False,Fake,Report #1259
PaintCollections,Plates,1000.0,-1.0,False,False,Fake,
PaintCollections,Plates,1000.0,-1.0,False,False,,Report #1260
PaintCollections,Plates,1000.0,-1.0,False,False,,
PaintCollections,Plates,1000.0,0.0,True,True,Genuine,Report #1261
PaintCollections,Plates,1000.0,0.0,True,True,Genuine,
PaintCollections,Plates,1000.0,0.0,True,True,Fake,Report #1262
PaintCollections,Plates,1000.0,0.0,True,True,Fake,
PaintCollections,Plates,1000.0,0.0,True,True,,Report #1263
PaintCollections,Plates,1000.0,0.0,True,True,,
PaintCollections,Plates,1000.0,0.0,True,False,Genuine,Report #1264
PaintCollections,Plates,1000.0,0.0,True,False,Genuine,
PaintCollections,Plates,1000.0,0.0,True,False,Fake,Report #1265
PaintCollections,Plates,1000.0,0.0,True,False,Fake,
PaintCollections,Plates,1000.0,0.0,True,False,,Report #1266
PaintCollections,Plates,1000.0,0.0,True,False,,
PaintCollections,Plates,1000.0,0.0,False,True,Genuine,Report #1267
PaintCollections,Plates,1000.0,0.0,False,True,Genuine,
PaintCollections,Plates,1000.0,0.0,False,True,Fake,Report #1268
PaintCollections,Plates,1000.0,0.0,False,True,Fake,
PaintCollections,Plates,1000.0,0.0,False,True,,Report #1269
PaintCollections,Plates,1000.0,0.0,False,True,,
PaintCollections,Plates,1000.0,0.0,False,False,Genuine,Report #1270
PaintCollections,Plates,1000.0,0.0,False,False,Genuine,
PaintCollections,Plates,1000.0,0.0,False,False,Fake,Report #1271
PaintCollections,Plates,1000.0,0.0,False,False,Fake,
PaintCollections,Plates,1000.0,0.0,False,False,,Report #1272
PaintCollections,Plates,1000.0,0.0,False,False,,
PaintCollections,Plates,1000.0,19.0,True,True,Genuine,Report #1273
PaintCollections,Plates,1000.0,19.0,True,True,Genuine,
PaintCollections,Plates,1000.0,19.0,True,True,Fake,Report #1274
PaintCollections,Plates,1000.0,19.0,True,True,Fake,
PaintCollections,Plates,1000.0,19.0,True,True,,Report #1275
PaintCollections,Plates,1000.0,19.0,True,True,,
PaintCollections,Plates,1000.0,19.0,True,False,Genuine,Report #1276
PaintCollections,Plates,1000.0,19.0,True,False,Genuine,
PaintCollections,Plates,1000.0,19.0,True,False,Fake,Report #1277
PaintCollections,Plates,1000.0,19.0,True,False,Fake,
PaintCollections,Plates,1000.0,19.0,True,False,,Report #1278
PaintCollections,Plates,1000.0,19.0,True,False,,
PaintCollections,Plates,1000.0,19.0,False,True,Genuine,Report #1279
PaintCollections,Plates,1000.0,19.0,False,True,Genuine,
PaintCollections,Plates,1000.0,19.0,False,True,Fake,Report #1280
PaintCollections,Plates,1000.0,19.0,False,True,Fake,
PaintCollections,Plates,1000.0,19.0,False,True,,Report #1281
PaintCollections,Plates,1000.0,19.0,False,True,,
PaintCollections,Plates,1000.0,19.0,False,False,Genuine,Report #1282
PaintCollections,Plates,1000.0,19.0,False,False,Genuine,
PaintCollections,Plates,1000.0,19.0,False,False,Fake,Report #1283
PaintCollections,Plates,1000.0,19.0,False,False,Fake,
PaintCollections,Plates,1000.0,19.0,False,False,,Report #1284
PaintCollections,Plates,1000.0,19.0,False,False,,
PaintCollections,Plates,1000.0,20.0,True,True,Genuine,Report #1285
PaintCollections,Plates,1000.0,20.0,True,True,Genuine,
PaintCollections,Plates,1000.0,20.0,True,True,Fake,Report #1286
PaintCollections,Plates,1000.0,20.0,True,True,Fake,
PaintCollections,Plates,1000.0,20.0,True,True,,Report #1287
PaintCollections,Plates,1000.0,20.0,True,True,,
PaintCollections,Plates,1000.0,20.0,True,False,Genuine,Report #1288
PaintCollections,Plates,1000.0,20.0,True,False,Genuine,
PaintCollections,Plates,1000.0,20.0,True,False,Fake,Report #1289
PaintCollections,Plates,1000.0,20.0,True,False,Fake,
PaintCollections,Plates,1000.0,20.0,True,False,,Report #1290
PaintCollections,Plates,1000.0,20.0,True,False,,
PaintCollections,Plates,1000.0,20.0,False,True,Genuine,Report #1291
PaintCollections,Plates,1000.0,20.0,False,True,Genuine,
PaintCollections,Plates,1000.0,20.0,False,True,Fake,Report #1292
PaintCollections,Plates,1000.0,20.0,False,True,Fake,
PaintCollections,Plates,1000.0,20.0,False,True,,Report #1293
PaintCollections,Plates,1000.0,20.0,False,True,,
PaintCollections,Plates,1000.0,20.0,False,False,Genuine,Report #1294
PaintCollections,Plates,1000.0,20.0,False,False,Genuine,
PaintCollections,Plates,1000.0,20.0,False,False,Fake,Report #1295
PaintCollections,Plates,1000.0,20.0,False,False,Fake,
PaintCollections,Plates,1000.0,20.0,False,False,,Report #1296
PaintCollections,Plates,1000.0,20.0,False,False,,
PaintCollections,Plates,1000.0,21.0,True,True,Genuine,Report #1297
PaintCollections,Plates,1000.0,21.0,True,True,Genuine,
PaintCollections,Plates,1000.0,21.0,True,True,Fake,Report #1298
PaintCollections,Plates,1000.0,21.0,True,True,Fake,
PaintCollections,Plates,1000.0,21.0,True,True,,Report #1299
PaintCollections,Plates,1000.0,21.0,True,True,,
PaintCollections,Plates,1000.0,21.0,True,False,Genuine,Report #1300
PaintCollections,Plates,1000.0,21.0,True,False,Genuine,
PaintCollections,Plates,1000.0,21.0,True,False,Fake,Report #1301
PaintCollections,Plates,1000.0,21.0,True,False,Fake,
PaintCollections,Plates,1000.0,21.0,True,False,,Report #1302
PaintCollections,Plates,1000.0,21.0,True,False,,
PaintCollections,Plates,1000.0,21.0,False,True,Genuine,Report #1303
PaintCollections,Plates,1000.0,21.0,False,True,Genuine,
PaintCollections,Plates,1000.0,21.0,False,True,Fake,Report #1304
PaintCollections,Plates,1000.0,21.0,False,True,Fake,
PaintCollections,Plates,1000.0,21.0,False,True,,Report #1305
PaintCollections,Plates,1000.0,21.0,False,True,,
PaintCollections,Plates,1000.0,21.0,False,False,Genuine,Report #1306
PaintCollections,Plates,1000.0,21.0,False,False,Genuine,
PaintCollections,Plates,1000.0,21.0,False,False,Fake,Report #1307
PaintCollections,Plates,1000.0,21.0,False,False,Fake,
PaintCollections,Plates,1000.0,21.0,False,False,,Report #1308
PaintCollections,Plates,1000.0,21.0,False,False,,
PaintCollections,Plates,1000.0,99.0,True,True,Genuine,Report #1309
PaintCollections,Plates,1000.0,99.0,True,True,Genuine,
PaintCollections,Plates,1000.0,99.0,True,True,Fake,Report #1310
PaintCollections,Plates,1000.0,99.0,True,True,Fake,
PaintCollections,Plates,1000.0,99.0,True,True,,Report #1311
PaintCollections,Plates,1000.0,99.0,True,True,,
PaintCollections,Plates,1000.0,99.0,True,False,Genuine,Report #1312
PaintCollections,Plates,1000.0,99.0,True,False,Genuine,
PaintCollections,Plates,1000.0,99.0,True,False,Fake,Report #1313
PaintCollections,Plates,1000.0,99.0,True,False,Fake,
PaintCollections,Plates,1000.0,99.0,True,False,,Report #1314
PaintCollections,Plates,1000.0,99.0,True,False,,
PaintCollections,Plates,1000.0,99.0,False,True,Genuine,Report #1315
PaintCollections,Plates,1000.0,99.0,False,True,Genuine,
PaintCollections,Plates,1000.0,99.0,False,True,Fake,Report #1316
PaintCollections,Plates,1000.0,99.0,False,True,Fake,
PaintCollections,Plates,1000.0,99.0,False,True,,Report #1317
PaintCollections,Plates,1000.0,99.0,False,True,,
PaintCollections,Plates,1000.0,99.0,False,False,Genuine,Report #1318
PaintCollections,Plates,1000.0,99.0,False,False,Genuine,
PaintCollections,Plates,1000.0,99.0,False,False,Fake,Report #1319
PaintCollections,Plates,1000.0,99.0,False,False,Fake,
PaintCollections,Plates,1000.0,99.0,False,False,,Report #1320
PaintCollections,Plates,1000.0,99.0,False,False,,
PaintCollections,Plates,1000.0,100.0,True,True,Genuine,Report #1321
PaintCollections,Plates,1000.0,100.0,True,True,Genuine,
PaintCollections,Plates,1000.0,100.0,True,True,Fake,Report #1322
PaintCollections,Plates,1000.0,100.0,True,True,Fake,
PaintCollections,Plates,1000.0,100.0,True,True,,Report #1323
PaintCollections,Plates,1000.0,100.0,True,True,,
PaintCollections,Plates,1000.0,100.0,True,False,Genuine,Report #1324
PaintCollections,Plates,1000.0,100.0,True,False,Genuine,
PaintCollections,Plates,1000.0,100.0,True,False,Fake,Report #1325
PaintCollections,Plates,1000.0,100.0,True,False,Fake,
PaintCollections,Plates,1000.0,100.0,True,False,,Report #1326
PaintCollections,Plates,1000.0,100.0,True,False,,
PaintCollections,Plates,1000.0,100.0,False,True,Genuine,Report #1327
PaintCollections,Plates,1000.0,100.0,False,True,Genuine,
PaintCollections,Plates,1000.0,100.0,False,True,Fake,Report #1328
PaintCollections,Plates,1000.0,100.0,False,True,Fake,
PaintCollections,Plates,1000.0,100.0,False,True,,Report #1329
PaintCollections,Plates,1000.0,100.0,False,True,,
PaintCollections,Plates,1000.0,100.0,False,False,Genuine,Report #1330
PaintCollections,Plates,1000.0,100.0,False,False,Genuine,
PaintCollections,Plates,1000.0,100.0,False,False,Fake,Report #1331
PaintCollections,Plates,1000.0,100.0,False,False,Fake,
PaintCollections,Plates,1000.0,100.0,False,False,,Report #1332
PaintCollections,Plates,1000.0,100.0,False,False,,
PaintCollections,Plates,1000.0,101.0,True,True,Genuine,Report #1333
PaintCollections,Plates,1000.0,101.0,True,True,Genuine,
PaintCollections,Plates,1000.0,101.0,True,True,Fake,Report #1334
PaintCollections,Plates,1000.0,101.0,True,True,Fake,
PaintCollections,Plates,1000.0,101.0,True,True,,Report #1335
PaintCollections,Plates,1000.0,101.0,True,True,,
PaintCollections,Plates,1000.0,101.0,True,False,Genuine,Report #1336
PaintCollections,Plates,1000.0,101.0,True,False,Genuine,
PaintCollections,Plates,1000.0,101.0,True,False,Fake,Report #1337
PaintCollections,Plates,1000.0,101.0,True,False,Fake,
PaintCollections,Plates,1000.0,101.0,True,False,,Report #1338
PaintCollections,Plates,1000.0,101.0,True,False,,
PaintCollections,Plates,1000.0,101.0,False,True,Genuine,Report #1339
PaintCollections,Plates,1000.0,101.0,False,True,Genuine,
PaintCollections,Plates,1000.0,101.0,False,True,Fake,Report #1340
PaintCollections,Plates,1000.0,101.0,False,True,Fake,
PaintCollections,Plates,1000.0,101.0,False,True,,Report #1341
PaintCollections,Plates,1000.0,101.0,False,True,,
PaintCollections,Plates,1000.0,101.0,False,False,Genuine,Report #1342
PaintCollections,Plates,1000.0,101.0,False,False,Genuine,
PaintCollections,Plates,1000.0,101.0,False,False,Fake,Report #1343
PaintCollections,Plates,1000.0,101.0,False,False,Fake,
PaintCollections,Plates,1000.0,101.0,False,False,,Report #1344
PaintCollections,Plates,1000.0,101.0,False,False,,
PaintCollections,Plates,1000000.0,-1.0,True,True,Genuine,Report #1345
PaintCollections,Plates,1000000.0,-1.0,True,True,Genuine,
PaintCollections,Plates,1000000.0,-1.0,True,True,Fake,Report #1346
PaintCollections,Plates,1000000.0,-1.0,True,True,Fake,
PaintCollections,Plates,1000000.0,-1.0,True,True,,Report #1347
PaintCollections,Plates,1000000.0,-1.0,True,True,,
PaintCollections,Plates,1000000.0,-1.0,True,False,Genuine,Report #1348
PaintCollections,Plates,1000000.0,-1.0,True,False,Genuine,
PaintCollections,Plates,1000000.0,-1.0,True,False,Fake,Report #1349
PaintCollections,Plates,1000000.0,-1.0,True,False,Fake,
PaintCollections,Plates,1000000.0,-1.0,True,False,,Report #1350
PaintCollections,Plates,1000000.0,-1.0,True,False,,
PaintCollections,Plates,1000000.0,-1.0,False,True,Genuine,Report #1351
PaintCollections,Plates,1000000.0,-1.0,False,True,Genuine,
PaintCollections,Plates,1000000.0,-1.0,False,True,Fake,Report #1352
PaintCollections,Plates,1000000.0,-1.0,False,True,Fake,
PaintCollections,Plates,1000000.0,-1.0,False,True,,Report #1353
PaintCollections,Plates,1000000.0,-1.0,False,True,,
PaintCollections,Plates,1000000.0,-1.0,False,False,Genuine,Report #1354
PaintCollections,Plates,1000000.0,-1.0,False,False,Genuine,
PaintCollections,Plates,1000000.0,-1.0,False,False,Fake,Report #1355
PaintCollections,Plates,1000000.0,-1.0,False,False,Fake,
PaintCollections,Plates,1000000.0,-1.0,False,False,,Report #1356
PaintCollections,Plates,1000000.0,-1.0,False,False,,
PaintCollections,Plates,1000000.0,0.0,True,True,Genuine,Report #1357
PaintCollections,Plates,1000000.0,0.0,True,True,Genuine,
PaintCollections,Plates,1000000.0,0.0,True,True,Fake,Report #1358
PaintCollections,Plates,1000000.0,0.0,True,True,Fake,
PaintCollections,Plates,1000000.0,0.0,True,True,,Report #1359
PaintCollections,Plates,1000000.0,0.0,True,True,,
PaintCollections,Plates,1000000.0,0.0,True,False,Genuine,Report #1360
PaintCollections,Plates,1000000.0,0.0,True,False,Genuine,
PaintCollections,Plates,1000000.0,0.0,True,False,Fake,Report #1361
PaintCollections,Plates,1000000.0,0.0,True,False,Fake,
PaintCollections,Plates,1000000.0,0.0,True,False,,Report #1362
PaintCollections,Plates,1000000.0,0.0,True,False,,
PaintCollections,Plates,1000000.0,0.0,False,True,Genuine,Report #1363
PaintCollections,Plates,1000000.0,0.0,False,True,Genuine,
PaintCollections,Plates,1000000.0,0.0,False,True,Fake,Report #1364
PaintCollections,Plates,1000000.0,0.0,False,True,Fake,
PaintCollections,Plates,1000000.0,0.0,False,True,,Report #1365
PaintCollections,Plates,1000000.0,0.0,False,True,,
PaintCollections,Plates,1000000.0,0.0,False,False,Genuine,Report #1366
PaintCollections,Plates,1000000.0,0.0,False,False,Genuine,
PaintCollections,Plates,1000000.0,0.0,False,False,Fake,Report #1367
PaintCollections,Plates,1000000.0,0.0,False,False,Fake,
PaintCollections,Plates,1000000.0,0.0,False,False,,Report #1368
PaintCollections,Plates,1000000.0,0.0,False,False,,
PaintCollections,Plates,1000000.0,19.0,True,True,Genuine,Report #1369
PaintCollections,Plates,1000000.0,19.0,True,True,Genuine,
PaintCollections,Plates,1000000.0,19.0,True,True,Fake,Report #1370
PaintCollections,Plates,1000000.0,19.0,True,True,Fake,
PaintCollections,Plates,1000000.0,19.0,True,True,,Report #1371
PaintCollections,Plates,1000000.0,19.0,True,True,,
PaintCollections,Plates,1000000.0,19.0,True,False,Genuine,Report #1372
PaintCollections,Plates,1000000.0,19.0,True,False,Genuine,
PaintCollections,Plates,1000000.0,19.0,True,False,Fake,Report #1373
PaintCollections,Plates,1000000.0,19.0,True,False,Fake,
PaintCollections,Plates,1000000.0,19.0,True,False,,Report #1374
PaintCollections,Plates,1000000.0,19.0,True,False,,
PaintCollections,Plates,1000000.0,19.0,False,True,Genuine,Report #1375
PaintCollections,Plates,1000000.0,19.0,False,True,Genuine,
PaintCollections,Plates,1000000.0,19.0,False,True,Fake,Report #1376
PaintCollections,Plates,1000000.0,19.0,False,True,Fake,
PaintCollections,Plates,1000000.0,19.0,False,True,,Report #1377
PaintCollections,Plates,1000000.0,19.0,False,True,,
PaintCollections,Plates,1000000.0,19.0,False,False,Genuine,Report #1378
PaintCollections,Plates,1000000.0,19.0,False,False,Genuine,
PaintCollections,Plates,1000000.0,19.0,False,False,Fake,Report #1379
PaintCollections,Plates,1000000.0,19.0,False,False,Fake,
PaintCollections,Plates,1000000.0,19.0,False,False,,Report #1380
PaintCollections,Plates,1000000.0,19.0,False,False,,
PaintCollections,Plates,1000000.0,20.0,True,True,Genuine,Report #1381
PaintCollections,Plates,1000000.0,20.0,True,True,Genuine,
PaintCollections,Plates,1000000.0,20.0,True,True,Fake,Report #1382
PaintCollections,Plates,1000000.0,20.0,True,True,Fake,
PaintCollections,Plates,1000000.0,20.0,True,True,,Report #1383
PaintCollections,Plates,1000000.0,20.0,True,True,,
PaintCollections,Plates,1000000.0,20.0,True,False,Genuine,Report #1384
PaintCollections,Plates,1000000.0,20.0,True,False,Genuine,
PaintCollections,Plates,1000000.0,20.0,True,False,Fake,Report #1385
PaintCollections,Plates,1000000.0,20.0,True,False,Fake,
PaintCollections,Plates,1000000.0,20.0,True,False,,Report #1386
PaintCollections,Plates,1000000.0,20.0,True,False,,
PaintCollections,Plates,1000000.0,20.0,False,True,Genuine,Report #1387
PaintCollections,Plates,1000000.0,20.0,False,True,Genuine,
PaintCollections,Plates,1000000.0,20.0,False,True,Fake,Report #1388
PaintCollections,Plates,1000000.0,20.0,False,True,Fake,
PaintCollections,Plates,1000000.0,20.0,False,True,,Report #1389
PaintCollections,Plates,1000000.0,20.0,False,True,,
PaintCollections,Plates,1000000.0,20.0,False,False,Genuine,Report #1390
PaintCollections,Plates,1000000.0,20.0,False,False,Genuine,
PaintCollections,Plates,1000000.0,20.0,False,False,Fake,Report #1391
PaintCollections,Plates,1000000.0,20.0,False,False,Fake,
PaintCollections,Plates,1000000.0,20.0,False,False,,Report #1392
PaintCollections,Plates,1000000.0,20.0,False,False,,
PaintCollections,Plates,1000000.0,21.0,True,True,Genuine,Report #1393
PaintCollections,Plates,1000000.0,21.0,True,True,Genuine,
PaintCollections,Plates,1000000.0,21.0,True,True,Fake,Report #1394
PaintCollections,Plates,1000000.0,21.0,True,True,Fake,
PaintCollections,Plates,1000000.0,21.0,True,True,,Report #1395
PaintCollections,Plates,1000000.0,21.0,True,True,,
PaintCollections,Plates,1000000.0,21.0,True,False,Genuine,Report #1396
PaintCollections,Plates,1000000.0,21.0,True,False,Genuine,
PaintCollections,Plates,1000000.0,21.0,True,False,Fake,Report #1397
PaintCollections,Plates,1000000.0,21.0,True,False,Fake,
PaintCollections,Plates,1000000.0,21.0,True,False,,Report #1398
PaintCollections,Plates,1000000.0,21.0,True,False,,
PaintCollections,Plates,1000000.0,21.0,False,True,Genuine,Report #1399
PaintCollections,Plates,1000000.0,21.0,False,True,Genuine,
PaintCollections,Plates,1000000.0,21.0,False,True,Fake,Report #1400
PaintCollections,Plates,1000000.0,21.0,False,True,Fake,
PaintCollections,Plates,1000000.0,21.0,False,True,,Report #1401
PaintCollections,Plates,1000000.0,21.0,False,True,,
PaintCollections,Plates,1000000.0,21.0,False,False,Genuine,Report #1402
PaintCollections,Plates,1000000.0,21.0,False,False,Genuine,
PaintCollections,Plates,1000000.0,21.0,False,False,Fake,Report #1403
PaintCollections,Plates,1000000.0,21.0,False,False,Fake,
PaintCollections,Plates,1000000.0,21.0,False,False,,Report #1404
PaintCollections,Plates,1000000.0,21.0,False,False,,
PaintCollections,Plates,1000000.0,99.0,True,True,Genuine,Report #1405
PaintCollections,Plates,1000000.0,99.0,True,True,Genuine,
PaintCollections,Plates,1000000.0,99.0,True,True,Fake,Report #1406
PaintCollections,Plates,1000000.0,99.0,True,True,Fake,
PaintCollections,Plates,1000000.0,99.0,True,True,,Report #1407
PaintCollections,Plates,1000000.0,99.0,True,True,,
PaintCollections,Plates,1000000.0,99.0,True,False,Genuine,Report #1408
PaintCollections,Plates,1000000.0,99.0,True,False,Genuine,
PaintCollections,Plates,1000000.0,99.0,True,False,Fake,Report #1409
PaintCollections,Plates,1000000.0,99.0,True,False,Fake,
PaintCollections,Plates,1000000.0,99.0,True,False,,Report #1410
PaintCollections,Plates,1000000.0,99.0,True,False,,
PaintCollections,Plates,1000000.0,99.0,False,True,Genuine,Report #1411
PaintCollections,Plates,1000000.0,99.0,False,True,Genuine,
PaintCollections,Plates,1000000.0,99.0,False,True,Fake,Report #1412
PaintCollections,Plates,1000000.0,99.0,False,True,Fake,
PaintCollections,Plates,1000000.0,99.0,False,True,,Report #1413
PaintCollections,Plates,1000000.0,99.0,False,True,,
PaintCollections,Plates,1000000.0,99.0,False,False,Genuine,Report #1414
PaintCollections,Plates,1000000.0,99.0,False,False,Genuine,
PaintCollections,Plates,1000000.0,99.0,False,False,Fake,Report #1415
PaintCollections,Plates,1000000.0,99.0,False,False,Fake,
PaintCollections,Plates,1000000.0,99.0,False,False,,Report #1416
PaintCollections,Plates,1000000.0,99.0,False,False,,
PaintCollections,Plates,1000000.0,100.0,True,True,Genuine,Report #1417
PaintCollections,Plates,1000000.0,100.0,True,True,Genuine,
PaintCollections,Plates,1000000.0,100.0,True,True,Fake,Report #1418
PaintCollections,Plates,1000000.0,100.0,True,True,Fake,
PaintCollections,Plates,1000000.0,100.0,True,True,,Report #1419
PaintCollections,Plates,1000000.0,100.0,True,True,,
PaintCollections,Plates,1000000.0,100.0,True,False,Genuine,Report #1420
PaintCollections,Plates,1000000.0,100.0,True,False,Genuine,
PaintCollections,Plates,1000000.0,100.0,True,False,Fake,Report #1421
PaintCollections,Plates,1000000.0,100.0,True,False,Fake,
PaintCollections,Plates,1000000.0,100.0,True,False,,Report #1422
PaintCollections,Plates,1000000.0,100.0,True,False,,
PaintCollections,Plates,1000000.0,100.0,False,True,Genuine,Report #1423
PaintCollections,Plates,1000000.0,100.0,False,True,Genuine,
PaintCollections,Plates,1000000.0,100.0,False,True,Fake,Report #1424
PaintCollections,Plates,1000000.0,100.0,False,True,Fake,
PaintCollections,Plates,1000000.0,100.0,False,True,,Report #1425
PaintCollections,Plates,1000000.0,100.0,False,True,,
PaintCollections,Plates,1000000.0,100.0,False,False,Genuine,Report #1426
PaintCollections,Plates,1000000.0,100.0,False,False,Genuine,
PaintCollections,Plates,1000000.0,100.0,False,False,Fake,Report #1427
PaintCollections,Plates,1000000.0,100.0,False,False,Fake,
PaintCollections,Plates,1000000.0,100.0,False,False,,Report #1428
PaintCollections,Plates,1000000.0,100.0,False,False,,
PaintCollections,Plates,1000000.0,101.0,True,True,Genuine,Report #1429
PaintCollections,Plates,1000000.0,101.0,True,True,Genuine,
PaintCollections,Plates,1000000.0,101.0,True,True,Fake,Report #1430
PaintCollections,Plates,1000000.0,101.0,True,True,Fake,
PaintCollections,Plates,1000000.0,101.0,True,True,,Report #1431
PaintCollections,Plates,1000000.0,101.0,True,True,,
PaintCollections,Plates,1000000.0,101.0,True,False,Genuine,Report #1432
PaintCollections,Plates,1000000.0,101.0,True,False,Genuine,
PaintCollections,Plates,1000000.0,101.0,True,False,Fake,Report #1433
PaintCollections,Plates,1000000.0,101.0,True,False,Fake,
PaintCollections,Plates,1000000.0,101.0,True,False,,Report #1434
PaintCollections,Plates,1000000.0,101.0,True,False,,
PaintCollections,Plates,1000000.0,101.0,False,True,Genuine,Report #1435
PaintCollections,Plates,1000000.0,101.0,False,True,Genuine,
PaintCollections,Plates,1000000.0,101.0,False,True,Fake,Report #1436
PaintCollections,Plates,1000000.0,101.0,False,True,Fake,
PaintCollections,Plates,1000000.0,101.0,False,True,,Report #1437
PaintCollections,Plates,1000000.0,101.0,False,True,,
PaintCollections,Plates,1000000.0,101.0,False,False,Genuine,Report #1438
PaintCollections,Plates,1000000.0,101.0,False,False,Genuine,
PaintCollections,Plates,1000000.0,101.0,False,False,Fake,Report #1439
PaintCollections,Plates,1000000.0,101.0,False,False,Fake,
PaintCollections,Plates,1000000.0,101.0,False,False,,Report #1440
PaintCollections,Plates,1000000.0,101.0,False,False,,
PaintCollections,Furniture,0.0,-1.0,True,True,Genuine,Report #1441
PaintCollections,Furniture,0.0,-1.0,True,True,Genuine,
PaintCollections,Furniture,0.0,-1.0,True,True,Fake,Report #1442
PaintCollections,Furniture,0.0,-1.0,True,True,Fake,
PaintCollections,Furniture,0.0,-1.0,True,True,,Report #1443
PaintCollections,Furniture,0.0,-1.0,True,True,,
PaintCollections,Furniture,0.0,-1.0,True,False,Genuine,Report #1444
PaintCollections,Furniture,0.0,-1.0,True,False,Genuine,
PaintCollections,Furniture,0.0,-1.0,True,False,Fake,Report #1445
PaintCollections,Furniture,0.0,-1.0,True,False,Fake,
PaintCollections,Furniture,0.0,-1.0,True,False,,Report #1446
PaintCollections,Furniture,0.0,-1.0,True,False,,
PaintCollections,Furniture,0.0,-1.0,False,True,Genuine,Report #1447
PaintCollections,Furniture,0.0,-1.0,False,True,Genuine,
PaintCollections,Furniture,0.0,-1.0,False,True,Fake,Report #1448
PaintCollections,Furniture,0.0,-1.0,False,True,Fake,
PaintCollections,Furniture,0.0,-1.0,False,True,,Report #1449
PaintCollections,Furniture,0.0,-1.0,False,True,,
PaintCollections,Furniture,0.0,-1.0,False,False,Genuine,Report #1450
PaintCollections,Furniture,0.0,-1.0,False,False,Genuine,
PaintCollections,Furniture,0.0,-1.0,False,False,Fake,Report #1451
PaintCollections,Furniture,0.0,-1.0,False,False,Fake,
PaintCollections,Furniture,0.0,-1.0,False,False,,Report #1452
PaintCollections,Furniture,0.0,-1.0,False,False,,
PaintCollections,Furniture,0.0,0.0,True,True,Genuine,Report #1453
PaintCollections,Furniture,0.0,0.0,True,True,Genuine,
PaintCollections,Furniture,0.0,0.0,True,True,Fake,Report #1454
PaintCollections,Furniture,0.0,0.0,True,True,Fake,
PaintCollections,Furniture,0.0,0.0,True,True,,Report #1455
PaintCollections,Furniture,0.0,0.0,True,True,,
PaintCollections,Furniture,0.0,0.0,True,False,Genuine,Report #1456
PaintCollections,Furniture,0.0,0.0,True,False,Genuine,
PaintCollections,Furniture,0.0,0.0,True,False,Fake,Report #1457
PaintCollections,Furniture,0.0,0.0,True,False,Fake,
PaintCollections,Furniture,0.0,0.0,True,False,,Report #1458
PaintCollections,Furniture,0.0,0.0,True,False,,
PaintCollections,Furniture,0.0,0.0,False,True,Genuine,Report #1459
PaintCollections,Furniture,0.0,0.0,False,True,Genuine,
PaintCollections,Furniture,0.0,0.0,False,True,Fake,Report #1460
PaintCollections,Furniture,0.0,0.0,False,True,Fake,
PaintCollections,Furniture,0.0,0.0,False,True,,Report #1461
PaintCollections,Furniture,0.0,0.0,False,True,,
PaintCollections,Furniture,0.0,0.0,False,False,Genuine,Report #1462
PaintCollections,Furniture,0.0,0.0,False,False,Genuine,
PaintCollections,Furniture,0.0,0.0,False,False,Fake,Report #1463
PaintCollections,Furniture,0.0,0.0,False,False,Fake,
PaintCollections,Furniture,0.0,0.0,False,False,,Report #1464
PaintCollections,Furniture,0.0,0.0,False,False,,
PaintCollections,Furniture,0.0,19.0,True,True,Genuine,Report #1465
PaintCollections,Furniture,0.0,19.0,True,True,Genuine,
PaintCollections,Furniture,0.0,19.0,True,True,Fake,Report #1466
PaintCollections,Furniture,0.0,19.0,True,True,Fake,
PaintCollections,Furniture,0.0,19.0,True,True,,Report #1467
PaintCollections,Furniture,0.0,19.0,True,True,,
PaintCollections,Furniture,0.0,19.0,True,False,Genuine,Report #1468
PaintCollections,Furniture,0.0,19.0,True,False,Genuine,
PaintCollections,Furniture,0.0,19.0,True,False,Fake,Report #1469
PaintCollections,Furniture,0.0,19.0,True,False,Fake,
PaintCollections,Furniture,0.0,19.0,True,False,,Report #1470
PaintCollections,Furniture,0.0,19.0,True,False,,
PaintCollections,Furniture,0.0,19.0,False,True,Genuine,Report #1471
PaintCollections,Furniture,0.0,19.0,False,True,Genuine,
PaintCollections,Furniture,0.0,19.0,False,True,Fake,Report #1472
PaintCollections,Furniture,0.0,19.0,False,True,Fake,
PaintCollections,Furniture,0.0,19.0,False,True,,Report #1473
PaintCollections,Furniture,0.0,19.0,False,True,,
PaintCollections,Furniture,0.0,19.0,False,False,Genuine,Report #1474
PaintCollections,Furniture,0.0,19.0,False,False,Genuine,
PaintCollections,Furniture,0.0,19.0,False,False,Fake,Report #1475
PaintCollections,Furniture,0.0,19.0,False,False,Fake,
PaintCollections,Furniture,0.0,19.0,False,False,,Report #1476
PaintCollections,Furniture,0.0,19.0,False,False,,
PaintCollections,Furniture,0.0,20.0,True,True,Genuine,Report #1477
PaintCollections,Furniture,0.0,20.0,True,True,Genuine,
PaintCollections,Furniture,0.0,20.0,True,True,Fake,Report #1478
PaintCollections,Furniture,0.0,20.0,True,True,Fake,
PaintCollections,Furniture,0.0,20.0,True,True,,Report #1479
PaintCollections,Furniture,0.0,20.0,True,True,,
PaintCollections,Furniture,0.0,20.0,True,False,Genuine,Report #1480
PaintCollections,Furniture,0.0,20.0,True,False,Genuine,
PaintCollections,Furniture,0.0,20.0,True,False,Fake,Report #1481
PaintCollections,Furniture,0.0,20.0,True,False,Fake,
PaintCollections,Furniture,0.0,20.0,True,False,,Report #1482
PaintCollections,Furniture,0.0,20.0,True,False,,
PaintCollections,Furniture,0.0,20.0,False,True,Genuine,Report #1483
PaintCollections,Furniture,0.0,20.0,False,True,Genuine,
PaintCollections,Furniture,0.0,20.0,False,True,Fake,Report #1484
PaintCollections,Furniture,0.0,20.0,False,True,Fake,
PaintCollections,Furniture,0.0,20.0,False,True,,Report #1485
PaintCollections,Furniture,0.0,20.0,False,True,,
PaintCollections,Furniture,0.0,20.0,False,False,Genuine,Report #1486
PaintCollections,Furniture,0.0,20.0,False,False,Genuine,
PaintCollections,Furniture,0.0,20.0,False,False,Fake,Report #1487
PaintCollections,Furniture,0.0,20.0,False,False,Fake,
PaintCollections,Furniture,0.0,20.0,False,False,,Report #1488
PaintCollections,Furniture,0.0,20.0,False,False,,
PaintCollections,Furniture,0.0,21.0,True,True,Genuine,Report #1489
PaintCollections,Furniture,0.0,21.0,True,True,Genuine,
PaintCollections,Furniture,0.0,21.0,True,True,Fake,Report #1490
PaintCollections,Furniture,0.0,21.0,True,True,Fake,
PaintCollections,Furniture,0.0,21.0,True,True,,Report #1491
PaintCollections,Furniture,0.0,21.0,True,True,,
PaintCollections,Furniture,0.0,21.0,True,False,Genuine,Report #1492
PaintCollections,Furniture,0.0,21.0,True,False,Genuine,
PaintCollections,Furniture,0.0,21.0,True,False,Fake,Report #1493
PaintCollections,Furniture,0.0,21.0,True,False,Fake,
PaintCollections,Furniture,0.0,21.0,True,False,,Report #1494
PaintCollections,Furniture,0.0,21.0,True,False,,
PaintCollections,Furniture,0.0,21.0,False,True,Genuine,Report #1495
PaintCollections,Furniture,0.0,21.0,False,True,Genuine,
PaintCollections,Furniture,0.0,21.0,False,True,Fake,Report #1496
PaintCollections,Furniture,0.0,21.0,False,True,Fake,
PaintCollections,Furniture,0.0,21.0,False,True,,Report #1497
PaintCollections,Furniture,0.0,21.0,False,True,,
PaintCollections,Furniture,0.0,21.0,False,False,Genuine,Report #1498
PaintCollections,Furniture,0.0,21.0,False,False,Genuine,
PaintCollections,Furniture,0.0,21.0,False,False,Fake,Report #1499
PaintCollections,Furniture,0.0,21.0,False,False,Fake,
PaintCollections,Furniture,0.0,21.0,False,False,,Report #1500
PaintCollections,Furniture,0.0,21.0,False,False,,
PaintCollections,Furniture,0.0,99.0,True,True,Genuine,Report #1501
PaintCollections,Furniture,0.0,99.0,True,True,Genuine,
PaintCollections,Furniture,0.0,99.0,True,True,Fake,Report #1502
PaintCollections,Furniture,0.0,99.0,True,True,Fake,
PaintCollections,Furniture,0.0,99.0,True,True,,Report #1503
PaintCollections,Furniture,0.0,99.0,True,True,,
PaintCollections,Furniture,0.0,99.0,True,False,Genuine,Report #1504
PaintCollections,Furniture,0.0,99.0,True,False,Genuine,
PaintCollections,Furniture,0.0,99.0,True,False,Fake,Report #1505
PaintCollections,Furniture,0.0,99.0,True,False,Fake,
PaintCollections,Furniture,0.0,99.0,True,False,,Report #1506
PaintCollections,Furniture,0.0,99.0,True,False,,
PaintCollections,Furniture,0.0,99.0,False,True,Genuine,Report #1507
PaintCollections,Furniture,0.0,99.0,False,True,Genuine,
PaintCollections,Furniture,0.0,99.0,False,True,Fake,Report #1508
PaintCollections,Furniture,0.0,99.0,False,True,Fake,
PaintCollections,Furniture,0.0,99.0,False,True,,Report #1509
PaintCollections,Furniture,0.0,99.0,False,True,,
PaintCollections,Furniture,0.0,99.0,False,False,Genuine,Report #1510
PaintCollections,Furniture,0.0,99.0,False,False,Genuine,
PaintCollections,Furniture,0.0,99.0,False,False,Fake,Report #1511
PaintCollections,Furniture,0.0,99.0,False,False,Fake,
PaintCollections,Furniture,0.0,99.0,False,False,,Report #1512
PaintCollections,Furniture,0.0,99.0,False,False,,
PaintCollections,Furniture,0.0,100.0,True,True,Genuine,Report #1513
PaintCollections,Furniture,0.0,100.0,True,True,Genuine,
PaintCollections,Furniture,0.0,100.0,True,True,Fake,Report #1514
PaintCollections,Furniture,0.0,100.0,True,True,Fake,
PaintCollections,Furniture,0.0,100.0,True,True,,Report #1515
PaintCollections,Furniture,0.0,100.0,True,True,,
PaintCollections,Furniture,0.0,100.0,True,False,Genuine,Report #1516
PaintCollections,Furniture,0.0,100.0,True,False,Genuine,
PaintCollections,Furniture,0.0,100.0,True,False,Fake,Report #1517
PaintCollections,Furniture,0.0,100.0,True,False,Fake,
PaintCollections,Furniture,0.0,100.0,True,False,,Report #1518
PaintCollections,Furniture,0.0,100.0,True,False,,
PaintCollections,Furniture,0.0,100.0,False,True,Genuine,Report #1519
PaintCollections,Furniture,0.0,100.0,False,True,Genuine,
PaintCollections,Furniture,0.0,100.0,False,True,Fake,Report #1520
PaintCollections,Furniture,0.0,100.0,False,True,Fake,
PaintCollections,Furniture,0.0,100.0,False,True,,Report #1521
PaintCollections,Furniture,0.0,100.0,False,True,,
PaintCollections,Furniture,0.0,100.0,False,False,Genuine,Report #1522
PaintCollections,Furniture,0.0,100.0,False,False,Genuine,
PaintCollections,Furniture,0.0,100.0,False,False,Fake,Report #1523
PaintCollections,Furniture,0.0,100.0,False,False,Fake,
PaintCollections,Furniture,0.0,100.0,False,False,,Report #1524
PaintCollections,Furniture,0.0,100.0,False,False,,
PaintCollections,Furniture,0.0,101.0,True,True,Genuine,Report #1525
PaintCollections,Furniture,0.0,101.0,True,True,Genuine,
PaintCollections,Furniture,0.0,101.0,True,True,Fake,Report #1526
PaintCollections,Furniture,0.0,101.0,True,True,Fake,
PaintCollections,Furniture,0.0,101.0,True,True,,Report #1527
PaintCollections,Furniture,0.0,101.0,True,True,,
PaintCollections,Furniture,0.0,101.0,True,False,Genuine,Report #1528
PaintCollections,Furniture,0.0,101.0,True,False,Genuine,
PaintCollections,Furniture,0.0,101.0,True,False,Fake,Report #1529
PaintCollections,Furniture,0.0,101.0,True,False,Fake,
PaintCollections,Furniture,0.0,101.0,True,False,,Report #1530
PaintCollections,Furniture,0.0,101.0,True,False,,
PaintCollections,Furniture,0.0,101.0,False,True,Genuine,Report #1531
PaintCollections,Furniture,0.0,101.0,False,True,Genuine,
PaintCollections,Furniture,0.0,101.0,False,True,Fake,Report #1532
PaintCollections,Furniture,0.0,101.0,False,True,Fake,
PaintCollections,Furniture,0.0,101.0,False,True,,Report #1533
PaintCollections,Furniture,0.0,101.0,False,True,,
PaintCollections,Furniture,0.0,101.0,False,False,Genuine,Report #1534
PaintCollections,Furniture,0.0,101.0,False,False,Genuine,
PaintCollections,Furniture,0.0,101.0,False,False,Fake,Report #1535
PaintCollections,Furniture,0.0,101.0,False,False,Fake,
PaintCollections,Furniture,0.0,101.0,False,False,,Report #1536
PaintCollections,Furniture,0.0,101.0,False,False,,
PaintCollections,Furniture,1.0,-1.0,True,True,Genuine,Report #1537
PaintCollections,Furniture,1.0,-1.0,True,True,Genuine,
PaintCollections,Furniture,1.0,-1.0,True,True,Fake,Report #1538
PaintCollections,Furniture,1.0,-1.0,True,True,Fake,
PaintCollections,Furniture,1.0,-1.0,True,True,,Report #1539
PaintCollections,Furniture,1.0,-1.0,True,True,,
PaintCollections,Furniture,1.0,-1.0,True,False,Genuine,Report #1540
PaintCollections,Furniture,1.0,-1.0,True,False,Genuine,
PaintCollections,Furniture,1.0,-1.0,True,False,Fake,Report #1541
PaintCollections,Furniture,1.0,-1.0,True,False,Fake,
PaintCollections,Furniture,1.0,-1.0,True,False,,Report #1542
PaintCollections,Furniture,1.0,-1.0,True,False,,
PaintCollections,Furniture,1.0,-1.0,False,True,Genuine,Report #1543
PaintCollections,Furniture,1.0,-1.0,False,True,Genuine,
PaintCollections,Furniture,1.0,-1.0,False,True,Fake,Report #1544
PaintCollections,Furniture,1.0,-1.0,False,True,Fake,
PaintCollections,Furniture,1.0,-1.0,False,True,,Report #1545
PaintCollections,Furniture,1.0,-1.0,False,True,,
PaintCollections,Furniture,1.0,-1.0,False,False,Genuine,Report #1546
PaintCollections,Furniture,1.0,-1.0,False,False,Genuine,
PaintCollections,Furniture,1.0,-1.0,False,False,Fake,Report #1547
PaintCollections,Furniture,1.0,-1.0,False,False,Fake,
PaintCollections,Furniture,1.0,-1.0,False,False,,Report #1548
PaintCollections,Furniture,1.0,-1.0,False,False,,
PaintCollections,Furniture,1.0,0.0,True,True,Genuine,Report #1549
PaintCollections,Furniture,1.0,0.0,True,True,Genuine,
PaintCollections,Furniture,1.0,0.0,True,True,Fake,Report #1550
PaintCollections,Furniture,1.0,0.0,True,True,Fake,
PaintCollections,Furniture,1.0,0.0,True,True,,Report #1551
PaintCollections,Furniture,1.0,0.0,True,True,,
PaintCollections,Furniture,1.0,0.0,True,False,Genuine,Report #1552
PaintCollections,Furniture,1.0,0.0,True,False,Genuine,
PaintCollections,Furniture,1.0,0.0,True,False,Fake,Report #1553
PaintCollections,Furniture,1.0,0.0,True,False,Fake,
PaintCollections,Furniture,1.0,0.0,True,False,,Report #1554
PaintCollections,Furniture,1.0,0.0,True,False,,
PaintCollections,Furniture,1.0,0.0,False,True,Genuine,Report #1555
PaintCollections,Furniture,1.0,0.0,False,True,Genuine,
PaintCollections,Furniture,1.0,0.0,False,True,Fake,Report #1556
PaintCollections,Furniture,1.0,0.0,False,True,Fake,
PaintCollections,Furniture,1.0,0.0,False,True,,Report #1557
PaintCollections,Furniture,1.0,0.0,False,True,,
PaintCollections,Furniture,1.0,0.0,False,False,Genuine,Report #1558
PaintCollections,Furniture,1.0,0.0,False,False,Genuine,
PaintCollections,Furniture,1.0,0.0,False,False,Fake,Report #1559
PaintCollections,Furniture,1.0,0.0,False,False,Fake,
PaintCollections,Furniture,1.0,0.0,False,False,,Report #1560
PaintCollections,Furniture,1.0,0.0,False,False,,
PaintCollections,Furniture,1.0,19.0,True,True,Genuine,Report #1561
PaintCollections,Furniture,1.0,19.0,True,True,Genuine,
PaintCollections,Furniture,1.0,19.0,True,True,Fake,Report #1562
PaintCollections,Furniture,1.0,19.0,True,True,Fake,
PaintCollections,Furniture,1.0,19.0,True,True,,Report #1563
PaintCollections,Furniture,1.0,19.0,True,True,,
PaintCollections,Furniture,1.0,19.0,True,False,Genuine,Report #1564
PaintCollections,Furniture,1.0,19.0,True,False,Genuine,
PaintCollections,Furniture,1.0,19.0,True,False,Fake,Report #1565
PaintCollections,Furniture,1.0,19.0,True,False,Fake,
PaintCollections,Furniture,1.0,19.0,True,False,,Report #1566
PaintCollections,Furniture,1.0,19.0,True,False,,
PaintCollections,Furniture,1.0,19.0,False,True,Genuine,Report #1567
PaintCollections,Furniture,1.0,19.0,False,True,Genuine,
PaintCollections,Furniture,1.0,19.0,False,True,Fake,Report #1568
PaintCollections,Furniture,1.0,19.0,False,True,Fake,
PaintCollections,Furniture,1.0,19.0,False,True,,Report #1569
PaintCollections,Furniture,1.0,19.0,False,True,,
PaintCollections,Furniture,1.0,19.0,False,False,Genuine,Report #1570
PaintCollections,Furniture,1.0,19.0,False,False,Genuine,
PaintCollections,Furniture,1.0,19.0,False,False,Fake,Report #1571
PaintCollections,Furniture,1.0,19.0,False,False,Fake,
PaintCollections,Furniture,1.0,19.0,False,False,,Report #1572
PaintCollections,Furniture,1.0,19.0,False,False,,
PaintCollections,Furniture,1.0,20.0,True,True,Genuine,Report #1573
PaintCollections,Furniture,1.0,20.0,True,True,Genuine,
PaintCollections,Furniture,1.0,20.0,True,True,Fake,Report #1574
PaintCollections,Furniture,1.0,20.0,True,True,Fake,
PaintCollections,Furniture,1.0,20.0,True,True,,Report #1575
PaintCollections,Furniture,1.0,20.0,True,True,,
PaintCollections,Furniture,1.0,20.0,True,False,Genuine,Report #1576
PaintCollections,Furniture,1.0,20.0,True,False,Genuine,
PaintCollections,Furniture,1.0,20.0,True,False,Fake,Report #1577
PaintCollections,Furniture,1.0,20.0,True,False,Fake,
PaintCollections,Furniture,1.0,20.0,True,False,,Report #1578
PaintCollections,Furniture,1.0,20.0,True,False,,
PaintCollections,Furniture,1.0,20.0,False,True,Genuine,Report #1579
PaintCollections,Furniture,1.0,20.0,False,True,Genuine,
PaintCollections,Furniture,1.0,20.0,False,True,Fake,Report #1580
PaintCollections,Furniture,1.0,20.0,False,True,Fake,
PaintCollections,Furniture,1.0,20.0,False,True,,Report #1581
PaintCollections,Furniture,1.0,20.0,False,True,,
PaintCollections,Furniture,1.0,20.0,False,False,Genuine,Report #1582
PaintCollections,Furniture,1.0,20.0,False,False,Genuine,
PaintCollections,Furniture,1.0,20.0,False,False,Fake,Report #1583
PaintCollections,Furniture,1.0,20.0,False,False,Fake,
PaintCollections,Furniture,1.0,20.0,False,False,,Report #1584
PaintCollections,Furniture,1.0,20.0,False,False,,
PaintCollections,Furniture,1.0,21.0,True,True,Genuine,Report #1585
PaintCollections,Furniture,1.0,21.0,True,True,Genuine,
PaintCollections,Furniture,1.0,21.0,True,True,Fake,Report #1586
PaintCollections,Furniture,1.0,21.0,True,True,Fake,
PaintCollections,Furniture,1.0,21.0,True,True,,Report #1587
PaintCollections,Furniture,1.0,21.0,True,True,,
PaintCollections,Furniture,1.0,21.0,True,False,Genuine,Report #1588
PaintCollections,Furniture,1.0,21.0,True,False,Genuine,
PaintCollections,Furniture,1.0,21.0,True,False,Fake,Report #1589
PaintCollections,Furniture,1.0,21.0,True,False,Fake,
PaintCollections,Furniture,1.0,21.0,True,False,,Report #1590
PaintCollections,Furniture,1.0,21.0,True,False,,
PaintCollections,Furniture,1.0,21.0,False,True,Genuine,Report #1591
PaintCollections,Furniture,1.0,21.0,False,True,Genuine,
PaintCollections,Furniture,1.0,21.0,False,True,Fake,Report #1592
PaintCollections,Furniture,1.0,21.0,False,True,Fake,
PaintCollections,Furniture,1.0,21.0,False,True,,Report #1593
PaintCollections,Furniture,1.0,21.0,False,True,,
PaintCollections,Furniture,1.0,21.0,False,False,Genuine,Report #1594
PaintCollections,Furniture,1.0,21.0,False,False,Genuine,
PaintCollections,Furniture,1.0,21.0,False,False,Fake,Report #1595
PaintCollections,Furniture,1.0,21.0,False,False,Fake,
PaintCollections,Furniture,1.0,21.0,False,False,,Report #1596
PaintCollections,Furniture,1.0,21.0,False,False,,
PaintCollections,Furniture,1.0,99.0,True,True,Genuine,Report #1597
PaintCollections,Furniture,1.0,99.0,True,True,Genuine,
PaintCollections,Furniture,1.0,99.0,True,True,Fake,Report #1598
PaintCollections,Furniture,1.0,99.0,True,True,Fake,
PaintCollections,Furniture,1.0,99.0,True,True,,Report #1599
PaintCollections,Furniture,1.0,99.0,True,True,,
PaintCollections,Furniture,1.0,99.0,True,False,Genuine,Report #1600
PaintCollections,Furniture,1.0,99.0,True,False,Genuine,
PaintCollections,Furniture,1.0,99.0,True,False,Fake,Report #1601
PaintCollections,Furniture,1.0,99.0,True,False,Fake,
PaintCollections,Furniture,1.0,99.0,True,False,,Report #1602
PaintCollections,Furniture,1.0,99.0,True,False,,
PaintCollections,Furniture,1.0,99.0,False,True,Genuine,Report #1603
PaintCollections,Furniture,1.0,99.0,False,True,Genuine,
PaintCollections,Furniture,1.0,99.0,False,True,Fake,Report #1604
PaintCollections,Furniture,1.0,99.0,False,True,Fake,
PaintCollections,Furniture,1.0,99.0,False,True,,Report #1605
PaintCollections,Furniture,1.0,99.0,False,True,,
PaintCollections,Furniture,1.0,99.0,False,False,Genuine,Report #1606
PaintCollections,Furniture,1.0,99.0,False,False,Genuine,
PaintCollections,Furniture,1.0,99.0,False,False,Fake,Report #1607
PaintCollections,Furniture,1.0,99.0,False,False,Fake,
PaintCollections,Furniture,1.0,99.0,False,False,,Report #1608
PaintCollections,Furniture,1.0,99.0,False,False,,
PaintCollections,Furniture,1.0,100.0,True,True,Genuine,Report #1609
PaintCollections,Furniture,1.0,100.0,True,True,Genuine,
PaintCollections,Furniture,1.0,100.0,True,True,Fake,Report #1610
PaintCollections,Furniture,1.0,100.0,True,True,Fake,
PaintCollections,Furniture,1.0,100.0,True,True,,Report #1611
PaintCollections,Furniture,1.0,100.0,True,True,,
PaintCollections,Furniture,1.0,100.0,True,False,Genuine,Report #1612
PaintCollections,Furniture,1.0,100.0,True,False,Genuine,
PaintCollections,Furniture,1.0,100.0,True,False,Fake,Report #1613
PaintCollections,Furniture,1.0,100.0,True,False,Fake,
PaintCollections,Furniture,1.0,100.0,True,False,,Report #1614
PaintCollections,Furniture,1.0,100.0,True,False,,
PaintCollections,Furniture,1.0,100.0,False,True,Genuine,Report #1615
PaintCollections,Furniture,1.0,100.0,False,True,Genuine,
PaintCollections,Furniture,1.0,100.0,False,True,Fake,Report #1616
PaintCollections,Furniture,1.0,100.0,False,True,Fake,
PaintCollections,Furniture,1.0,100.0,False,True,,Report #1617
PaintCollections,Furniture,1.0,100.0,False,True,,
PaintCollections,Furniture,1.0,100.0,False,False,Genuine,Report #1618
PaintCollections,Furniture,1.0,100.0,False,False,Genuine,
PaintCollections,Furniture,1.0,100.0,False,False,Fake,Report #1619
PaintCollections,Furniture,1.0,100.0,False,False,Fake,
PaintCollections,Furniture,1.0,100.0,False,False,,Report #1620
PaintCollections,Furniture,1.0,100.0,False,False,,
PaintCollections,Furniture,1.0,101.0,True,True,Genuine,Report #1621
PaintCollections,Furniture,1.0,101.0,True,True,Genuine,
PaintCollections,Furniture,1.0,101.0,True,True,Fake,Report #1622
PaintCollections,Furniture,1.0,101.0,True,True,Fake,
PaintCollections,Furniture,1.0,101.0,True,True,,Report #1623
PaintCollections,Furniture,1.0,101.0,True,True,,
PaintCollections,Furniture,1.0,101.0,True,False,Genuine,Report #1624
PaintCollections,Furniture,1.0,101.0,True,False,Genuine,
PaintCollections,Furniture,1.0,101.0,True,False,Fake,Report #1625
PaintCollections,Furniture,1.0,101.0,True,False,Fake,
PaintCollections,Furniture,1.0,101.0,True,False,,Report #1626
PaintCollections,Furniture,1.0,101.0,True,False,,
PaintCollections,Furniture,1.0,101.0,False,True,Genuine,Report #1627
PaintCollections,Furniture,1.0,101.0,False,True,Genuine,
PaintCollections,Furniture,1.0,101.0,False,True,Fake,Report #1628
PaintCollections,Furniture,1.0,101.0,False,True,Fake,
PaintCollections,Furniture,1.0,101.0,False,True,,Report #1629
PaintCollections,Furniture,1.0,101.0,False,True,,
PaintCollections,Furniture,1.0,101.0,False,False,Genuine,Report #1630
PaintCollections,Furniture,1.0,101.0,False,False,Genuine,
PaintCollections,Furniture,1.0,101.0,False,False,Fake,Report #1631
PaintCollections,Furniture,1.0,101.0,False,False,Fake,
PaintCollections,Furniture,1.0,101.0,False,False,,Report #1632
PaintCollections,Furniture,1.0,101.0,False,False,,
PaintCollections,Furniture,100.0,-1.0,True,True,Genuine,Report #1633
PaintCollections,Furniture,100.0,-1.0,True,True,Genuine,
PaintCollections,Furniture,100.0,-1.0,True,True,Fake,Report #1634
PaintCollections,Furniture,100.0,-1.0,True,True,Fake,
PaintCollections,Furniture,100.0,-1.0,True,True,,Report #1635
PaintCollections,Furniture,100.0,-1.0,True,True,,
PaintCollections,Furniture,100.0,-1.0,True,False,Genuine,Report #1636
PaintCollections,Furniture,100.0,-1.0,True,False,Genuine,
PaintCollections,Furniture,100.0,-1.0,True,False,Fake,Report #1637
PaintCollections,Furniture,100.0,-1.0,True,False,Fake,
PaintCollections,Furniture,100.0,-1.0,True,False,,Report #1638
PaintCollections,Furniture,100.0,-1.0,True,False,,
PaintCollections,Furniture,100.0,-1.0,False,True,Genuine,Report #1639
PaintCollections,Furniture,100.0,-1.0,False,True,Genuine,
PaintCollections,Furniture,100.0,-1.0,False,True,Fake,Report #1640
PaintCollections,Furniture,100.0,-1.0,False,True,Fake,
PaintCollections,Furniture,100.0,-1.0,False,True,,Report #1641
PaintCollections,Furniture,100.0,-1.0,False,True,,
PaintCollections,Furniture,100.0,-1.0,False,False,Genuine,Report #1642
PaintCollections,Furniture,100.0,-1.0,False,False,Genuine,
PaintCollections,Furniture,100.0,-1.0,False,False,Fake,Report #1643
PaintCollections,Furniture,100.0,-1.0,False,False,Fake,
PaintCollections,Furniture,100.0,-1.0,False,False,,Report #1644
PaintCollections,Furniture,100.0,-1.0,False,False,,
PaintCollections,Furniture,100.0,0.0,True,True,Genuine,Report #1645
PaintCollections,Furniture,100.0,0.0,True,True,Genuine,
PaintCollections,Furniture,100.0,0.0,True,True,Fake,Report #1646
PaintCollections,Furniture,100.0,0.0,True,True,Fake,
PaintCollections,Furniture,100.0,0.0,True,True,,Report #1647
PaintCollections,Furniture,100.0,0.0,True,True,,
PaintCollections,Furniture,100.0,0.0,True,False,Genuine,Report #1648
PaintCollections,Furniture,100.0,0.0,True,False,Genuine,
PaintCollections,Furniture,100.0,0.0,True,False,Fake,Report #1649
PaintCollections,Furniture,100.0,0.0,True,False,Fake,
PaintCollections,Furniture,100.0,0.0,True,False,,Report #1650
PaintCollections,Furniture,100.0,0.0,True,False,,
PaintCollections,Furniture,100.0,0.0,False,True,Genuine,Report #1651
PaintCollections,Furniture,100.0,0.0,False,True,Genuine,
PaintCollections,Furniture,100.0,0.0,False,True,Fake,Report #1652
PaintCollections,Furniture,100.0,0.0,False,True,Fake,
PaintCollections,Furniture,100.0,0.0,False,True,,Report #1653
PaintCollections,Furniture,100.0,0.0,False,True,,
PaintCollections,Furniture,100.0,0.0,False,False,Genuine,Report #1654
PaintCollections,Furniture,100.0,0.0,False,False,Genuine,
PaintCollections,Furniture,100.0,0.0,False,False,Fake,Report #1655
PaintCollections,Furniture,100.0,0.0,False,False,Fake,
PaintCollections,Furniture,100.0,0.0,False,False,,Report #1656
PaintCollections,Furniture,100.0,0.0,False,False,,
PaintCollections,Furniture,100.0,19.0,True,True,Genuine,Report #1657
PaintCollections,Furniture,100.0,19.0,True,True,Genuine,
PaintCollections,Furniture,100.0,19.0,True,True,Fake,Report #1658
PaintCollections,Furniture,100.0,19.0,True,True,Fake,
PaintCollections,Furniture,100.0,19.0,True,True,,Report #1659
PaintCollections,Furniture,100.0,19.0,True,True,,
PaintCollections,Furniture,100.0,19.0,True,False,Genuine,Report #1660
PaintCollections,Furniture,100.0,19.0,True,False,Genuine,
PaintCollections,Furniture,100.0,19.0,True,False,Fake,Report #1661
PaintCollections,Furniture,100.0,19.0,True,False,Fake,
PaintCollections,Furniture,100.0,19.0,True,False,,Report #1662
PaintCollections,Furniture,100.0,19.0,True,False,,
PaintCollections,Furniture,100.0,19.0,False,True,Genuine,Report #1663
PaintCollections,Furniture,100.0,19.0,False,True,Genuine,
PaintCollections,Furniture,100.0,19.0,False,True,Fake,Report #1664
PaintCollections,Furniture,100.0,19.0,False,True,Fake,
PaintCollections,Furniture,100.0,19.0,False,True,,Report #1665
PaintCollections,Furniture,100.0,19.0,False,True,,
PaintCollections,Furniture,100.0,19.0,False,False,Genuine,Report #1666
PaintCollections,Furniture,100.0,19.0,False,False,Genuine,
PaintCollections,Furniture,100.0,19.0,False,False,Fake,Report #1667
PaintCollections,Furniture,100.0,19.0,False,False,Fake,
PaintCollections,Furniture,100.0,19.0,False,False,,Report #1668
PaintCollections,Furniture,100.0,19.0,False,False,,
PaintCollections,Furniture,100.0,20.0,True,True,Genuine,Report #1669
PaintCollections,Furniture,100.0,20.0,True,True,Genuine,
PaintCollections,Furniture,100.0,20.0,True,True,Fake,Report #1670
PaintCollections,Furniture,100.0,20.0,True,True,Fake,
PaintCollections,Furniture,100.0,20.0,True,True,,Report #1671
PaintCollections,Furniture,100.0,20.0,True,True,,
PaintCollections,Furniture,100.0,20.0,True,False,Genuine,Report #1672
PaintCollections,Furniture,100.0,20.0,True,False,Genuine,
PaintCollections,Furniture,100.0,20.0,True,False,Fake,Report #1673
PaintCollections,Furniture,100.0,20.0,True,False,Fake,
PaintCollections,Furniture,100.0,20.0,True,False,,Report #1674
PaintCollections,Furniture,100.0,20.0,True,False,,
PaintCollections,Furniture,100.0,20.0,False,True,Genuine,Report #1675
PaintCollections,Furniture,100.0,20.0,False,True,Genuine,
PaintCollections,Furniture,100.0,20.0,False,True,Fake,Report #1676
PaintCollections,Furniture,100.0,20.0,False,True,Fake,
PaintCollections,Furniture,100.0,20.0,False,True,,Report #1677
PaintCollections,Furniture,100.0,20.0,False,True,,
PaintCollections,Furniture,100.0,20.0,False,False,Genuine,Report #1678
PaintCollections,Furniture,100.0,20.0,False,False,Genuine,
PaintCollections,Furniture,100.0,20.0,False,False,Fake,Report #1679
PaintCollections,Furniture,100.0,20.0,False,False,Fake,
PaintCollections,Furniture,100.0,20.0,False,False,,Report #1680
PaintCollections,Furniture,100.0,20.0,False,False,,
PaintCollections,Furniture,100.0,21.0,True,True,Genuine,Report #1681
PaintCollections,Furniture,100.0,21.0,True,True,Genuine,
PaintCollections,Furniture,100.0,21.0,True,True,Fake,Report #1682
PaintCollections,Furniture,100.0,21.0,True,True,Fake,
PaintCollections,Furniture,100.0,21.0,True,True,,Report #1683
PaintCollections,Furniture,100.0,21.0,True,True,,
PaintCollections,Furniture,100.0,21.0,True,False,Genuine,Report #1684
PaintCollections,Furniture,100.0,21.0,True,False,Genuine,
PaintCollections,Furniture,100.0,21.0,True,False,Fake,Report #1685
PaintCollections,Furniture,100.0,21.0,True,False,Fake,
PaintCollections,Furniture,100.0,21.0,True,False,,Report #1686
PaintCollections,Furniture,100.0,21.0,True,False,,
PaintCollections,Furniture,100.0,21.0,False,True,Genuine,Report #1687
PaintCollections,Furniture,100.0,21.0,False,True,Genuine,
PaintCollections,Furniture,100.0,21.0,False,True,Fake,Report #1688
PaintCollections,Furniture,100.0,21.0,False,True,Fake,
PaintCollections,Furniture,100.0,21.0,False,True,,Report #1689
PaintCollections,Furniture,100.0,21.0,False,True,,
PaintCollections,Furniture,100.0,21.0,False,False,Genuine,Report #1690
PaintCollections,Furniture,100.0,21.0,False,False,Genuine,
PaintCollections,Furniture,100.0,21.0,False,False,Fake,Report #1691
PaintCollections,Furniture,100.0,21.0,False,False,Fake,
PaintCollections,Furniture,100.0,21.0,False,False,,Report #1692
PaintCollections,Furniture,100.0,21.0,False,False,,
PaintCollections,Furniture,100.0,99.0,True,True,Genuine,Report #1693
PaintCollections,Furniture,100.0,99.0,True,True,Genuine,
PaintCollections,Furniture,100.0,99.0,True,True,Fake,Report #1694
PaintCollections,Furniture,100.0,99.0,True,True,Fake,
PaintCollections,Furniture,100.0,99.0,True,True,,Report #1695
PaintCollections,Furniture,100.0,99.0,True,True,,
PaintCollections,Furniture,100.0,99.0,True,False,Genuine,Report #1696
PaintCollections,Furniture,100.0,99.0,True,False,Genuine,
PaintCollections,Furniture,100.0,99.0,True,False,Fake,Report #1697
PaintCollections,Furniture,100.0,99.0,True,False,Fake,
PaintCollections,Furniture,100.0,99.0,True,False,,Report #1698
PaintCollections,Furniture,100.0,99.0,True,False,,
PaintCollections,Furniture,100.0,99.0,False,True,Genuine,Report #1699
PaintCollections,Furniture,100.0,99.0,False,True,Genuine,
PaintCollections,Furniture,100.0,99.0,False,True,Fake,Report #1700
PaintCollections,Furniture,100.0,99.0,False,True,Fake,
PaintCollections,Furniture,100.0,99.0,False,True,,Report #1701
PaintCollections,Furniture,100.0,99.0,False,True,,
PaintCollections,Furniture,100.0,99.0,False,False,Genuine,Report #1702
PaintCollections,Furniture,100.0,99.0,False,False,Genuine,
PaintCollections,Furniture,100.0,99.0,False,False,Fake,Report #1703
PaintCollections,Furniture,100.0,99.0,False,False,Fake,
PaintCollections,Furniture,100.0,99.0,False,False,,Report #1704
PaintCollections,Furniture,100.0,99.0,False,False,,
PaintCollections,Furniture,100.0,100.0,True,True,Genuine,Report #1705
PaintCollections,Furniture,100.0,100.0,True,True,Genuine,
PaintCollections,Furniture,100.0,100.0,True,True,Fake,Report #1706
PaintCollections,Furniture,100.0,100.0,True,True,Fake,
PaintCollections,Furniture,100.0,100.0,True,True,,Report #1707
PaintCollections,Furniture,100.0,100.0,True,True,,
PaintCollections,Furniture,100.0,100.0,True,False,Genuine,Report #1708
PaintCollections,Furniture,100.0,100.0,True,False,Genuine,
PaintCollections,Furniture,100.0,100.0,True,False,Fake,Report #1709
PaintCollections,Furniture,100.0,100.0,True,False,Fake,
PaintCollections,Furniture,100.0,100.0,True,False,,Report #1710
PaintCollections,Furniture,100.0,100.0,True,False,,
PaintCollections,Furniture,100.0,100.0,False,True,Genuine,Report #1711
PaintCollections,Furniture,100.0,100.0,False,True,Genuine,
PaintCollections,Furniture,100.0,100.0,False,True,Fake,Report #1712
PaintCollections,Furniture,100.0,100.0,False,True,Fake,
PaintCollections,Furniture,100.0,100.0,False,True,,Report #1713
PaintCollections,Furniture,100.0,100.0,False,True,,
PaintCollections,Furniture,100.0,100.0,False,False,Genuine,Report #1714
PaintCollections,Furniture,100.0,100.0,False,False,Genuine,
PaintCollections,Furniture,100.0,100.0,False,False,Fake,Report #1715
PaintCollections,Furniture,100.0,100.0,False,False,Fake,
PaintCollections,Furniture,100.0,100.0,False,False,,Report #1716
PaintCollections,Furniture,100.0,100.0,False,False,,
PaintCollections,Furniture,100.0,101.0,True,True,Genuine,Report #1717
PaintCollections,Furniture,100.0,101.0,True,True,Genuine,
PaintCollections,Furniture,100.0,101.0,True,True,Fake,Report #1718
PaintCollections,Furniture,100.0,101.0,True,True,Fake,
PaintCollections,Furniture,100.0,101.0,True,True,,Report #1719
PaintCollections,Furniture,100.0,101.0,True,True,,
PaintCollections,Furniture,100.0,101.0,True,False,Genuine,Report #1720
PaintCollections,Furniture,100.0,101.0,True,False,Genuine,
PaintCollections,Furniture,100.0,101.0,True,False,Fake,Report #1721
PaintCollections,Furniture,100.0,101.0,True,False,Fake,
PaintCollections,Furniture,100.0,101.0,True,False,,Report #1722
PaintCollections,Furniture,100.0,101.0,True,False,,
PaintCollections,Furniture,100.0,101.0,False,True,Genuine,Report #1723
PaintCollections,Furniture,100.0,101.0,False,True,Genuine,
PaintCollections,Furniture,100.0,101.0,False,True,Fake,Report #1724
PaintCollections,Furniture,100.0,101.0,False,True,Fake,
PaintCollections,Furniture,100.0,101.0,False,True,,Report #1725
PaintCollections,Furniture,100.0,101.0,False,True,,
PaintCollections,Furniture,100.0,101.0,False,False,Genuine,Report #1726
PaintCollections,Furniture,100.0,101.0,False,False,Genuine,
PaintCollections,Furniture,100.0,101.0,False,False,Fake,Report #1727
PaintCollections,Furniture,100.0,101.0,False,False,Fake,
PaintCollections,Furniture,100.0,101.0,False,False,,Report #1728
PaintCollections,Furniture,100.0,101.0,False,False,,
PaintCollections,Furniture,1000.0,-1.0,True,True,Genuine,Report #1729
PaintCollections,Furniture,1000.0,-1.0,True,True,Genuine,
PaintCollections,Furniture,1000.0,-1.0,True,True,Fake,Report #1730
PaintCollections,Furniture,1000.0,-1.0,True,True,Fake,
PaintCollections,Furniture,1000.0,-1.0,True,True,,Report #1731
PaintCollections,Furniture,1000.0,-1.0,True,True,,
PaintCollections,Furniture,1000.0,-1.0,True,False,Genuine,Report #1732
PaintCollections,Furniture,1000.0,-1.0,True,False,Genuine,
PaintCollections,Furniture,1000.0,-1.0,True,False,Fake,Report #1733
PaintCollections,Furniture,1000.0,-1.0,True,False,Fake,
PaintCollections,Furniture,1000.0,-1.0,True,False,,Report #1734
PaintCollections,Furniture,1000.0,-1.0,True,False,,
PaintCollections,Furniture,1000.0,-1.0,False,True,Genuine,Report #1735
PaintCollections,Furniture,1000.0,-1.0,False,True,Genuine,
PaintCollections,Furniture,1000.0,-1.0,False,True,Fake,Report #1736
PaintCollections,Furniture,1000.0,-1.0,False,True,Fake,
PaintCollections,Furniture,1000.0,-1.0,False,True,,Report #1737
PaintCollections,Furniture,1000.0,-1.0,False,True,,
PaintCollections,Furniture,1000.0,-1.0,False,False,Genuine,Report #1738
PaintCollections,Furniture,1000.0,-1.0,False,False,Genuine,
PaintCollections,Furniture,1000.0,-1.0,False,False,Fake,Report #1739
PaintCollections,Furniture,1000.0,-1.0,False,False,Fake,
PaintCollections,Furniture,1000.0,-1.0,False,False,,Report #1740
PaintCollections,Furniture,1000.0,-1.0,False,False,,
PaintCollections,Furniture,1000.0,0.0,True,True,Genuine,Report #1741
PaintCollections,Furniture,1000.0,0.0,True,True,Genuine,
PaintCollections,Furniture,1000.0,0.0,True,True,Fake,Report #1742
PaintCollections,Furniture,1000.0,0.0,True,True,Fake,
PaintCollections,Furniture,1000.0,0.0,True,True,,Report #1743
PaintCollections,Furniture,1000.0,0.0,True,True,,
PaintCollections,Furniture,1000.0,0.0,True,False,Genuine,Report #1744
PaintCollections,Furniture,1000.0,0.0,True,False,Genuine,
PaintCollections,Furniture,1000.0,0.0,True,False,Fake,Report #1745
PaintCollections,Furniture,1000.0,0.0,True,False,Fake,
PaintCollections,Furniture,1000.0,0.0,True,False,,Report #1746
PaintCollections,Furniture,1000.0,0.0,True,False,,
PaintCollections,Furniture,1000.0,0.0,False,True,Genuine,Report #1747
PaintCollections,Furniture,1000.0,0.0,False,True,Genuine,
PaintCollections,Furniture,1000.0,0.0,False,True,Fake,Report #1748
PaintCollections,Furniture,1000.0,0.0,False,True,Fake,
PaintCollections,Furniture,1000.0,0.0,False,True,,Report #1749
PaintCollections,Furniture,1000.0,0.0,False,True,,
PaintCollections,Furniture,1000.0,0.0,False,False,Genuine,Report #1750
PaintCollections,Furniture,1000.0,0.0,False,False,Genuine,
PaintCollections,Furniture,1000.0,0.0,False,False,Fake,Report #1751
PaintCollections,Furniture,1000.0,0.0,False,False,Fake,
PaintCollections,Furniture,1000.0,0.0,False,False,,Report #1752
PaintCollections,Furniture,1000.0,0.0,False,False,,
PaintCollections,Furniture,1000.0,19.0,True,True,Genuine,Report #1753
PaintCollections,Furniture,1000.0,19.0,True,True,Genuine,
PaintCollections,Furniture,1000.0,19.0,True,True,Fake,Report #1754
PaintCollections,Furniture,1000.0,19.0,True,True,Fake,
PaintCollections,Furniture,1000.0,19.0,True,True,,Report #1755
PaintCollections,Furniture,1000.0,19.0,True,True,,
PaintCollections,Furniture,1000.0,19.0,True,False,Genuine,Report #1756
PaintCollections,Furniture,1000.0,19.0,True,False,Genuine,
PaintCollections,Furniture,1000.0,19.0,True,False,Fake,Report #1757
PaintCollections,Furniture,1000.0,19.0,True,False,Fake,
PaintCollections,Furniture,1000.0,19.0,True,False,,Report #1758
PaintCollections,Furniture,1000.0,19.0,True,False,,
PaintCollections,Furniture,1000.0,19.0,False,True,Genuine,Report #1759
PaintCollections,Furniture,1000.0,19.0,False,True,Genuine,
PaintCollections,Furniture,1000.0,19.0,False,True,Fake,Report #1760
PaintCollections,Furniture,1000.0,19.0,False,True,Fake,
PaintCollections,Furniture,1000.0,19.0,False,True,,Report #1761
PaintCollections,Furniture,1000.0,19.0,False,True,,
PaintCollections,Furniture,1000.0,19.0,False,False,Genuine,Report #1762
PaintCollections,Furniture,1000.0,19.0,False,False,Genuine,
PaintCollections,Furniture,1000.0,19.0,False,False,Fake,Report #1763
PaintCollections,Furniture,1000.0,19.0,False,False,Fake,
PaintCollections,Furniture,1000.0,19.0,False,False,,Report #1764
PaintCollections,Furniture,1000.0,19.0,False,False,,
PaintCollections,Furniture,1000.0,20.0,True,True,Genuine,Report #1765
PaintCollections,Furniture,1000.0,20.0,True,True,Genuine,
PaintCollections,Furniture,1000.0,20.0,True,True,Fake,Report #1766
PaintCollections,Furniture,1000.0,20.0,True,True,Fake,
PaintCollections,Furniture,1000.0,20.0,True,True,,Report #1767
PaintCollections,Furniture,1000.0,20.0,True,True,,
PaintCollections,Furniture,1000.0,20.0,True,False,Genuine,Report #1768
PaintCollections,Furniture,1000.0,20.0,True,False,Genuine,
PaintCollections,Furniture,1000.0,20.0,True,False,Fake,Report #1769
PaintCollections,Furniture,1000.0,20.0,True,False,Fake,
PaintCollections,Furniture,1000.0,20.0,True,False,,Report #1770
PaintCollections,Furniture,1000.0,20.0,True,False,,
PaintCollections,Furniture,1000.0,20.0,False,True,Genuine,Report #1771
PaintCollections,Furniture,1000.0,20.0,False,True,Genuine,
PaintCollections,Furniture,1000.0,20.0,False,True,Fake,Report #1772
PaintCollections,Furniture,1000.0,20.0,False,True,Fake,
PaintCollections,Furniture,1000.0,20.0,False,True,,Report #1773
PaintCollections,Furniture,1000.0,20.0,False,True,,
PaintCollections,Furniture,1000.0,20.0,False,False,Genuine,Report #1774
PaintCollections,Furniture,1000.0,20.0,False,False,Genuine,
PaintCollections,Furniture,1000.0,20.0,False,False,Fake,Report #1775
PaintCollections,Furniture,1000.0,20.0,False,False,Fake,
PaintCollections,Furniture,1000.0,20.0,False,False,,Report #1776
PaintCollections,Furniture,1000.0,20.0,False,False,,
PaintCollections,Furniture,1000.0,21.0,True,True,Genuine,Report #1777
PaintCollections,Furniture,1000.0,21.0,True,True,Genuine,
PaintCollections,Furniture,1000.0,21.0,True,True,Fake,Report #1778
PaintCollections,Furniture,1000.0,21.0,True,True,Fake,
PaintCollections,Furniture,1000.0,21.0,True,True,,Report #1779
PaintCollections,Furniture,1000.0,21.0,True,True,,
PaintCollections,Furniture,1000.0,21.0,True,False,Genuine,Report #1780
PaintCollections,Furniture,1000.0,21.0,True,False,Genuine,
PaintCollections,Furniture,1000.0,21.0,True,False,Fake,Report #1781
PaintCollections,Furniture,1000.0,21.0,True,False,Fake,
PaintCollections,Furniture,1000.0,21.0,True,False,,Report #1782
PaintCollections,Furniture,1000.0,21.0,True,False,,
PaintCollections,Furniture,1000.0,21.0,False,True,Genuine,Report #1783
PaintCollections,Furniture,1000.0,21.0,False,True,Genuine,
PaintCollections,Furniture,1000.0,21.0,False,True,Fake,Report #1784
PaintCollections,Furniture,1000.0,21.0,False,True,Fake,
PaintCollections,Furniture,1000.0,21.0,False,True,,Report #1785
PaintCollections,Furniture,1000.0,21.0,False,True,,
PaintCollections,Furniture,1000.0,21.0,False,False,Genuine,Report #1786
PaintCollections,Furniture,1000.0,21.0,False,False,Genuine,
PaintCollections,Furniture,1000.0,21.0,False,False,Fake,Report #1787
PaintCollections,Furniture,1000.0,21.0,False,False,Fake,
PaintCollections,Furniture,1000.0,21.0,False,False,,Report #1788
PaintCollections,Furniture,1000.0,21.0,False,False,,
PaintCollections,Furniture,1000.0,99.0,True,True,Genuine,Report #1789
PaintCollections,Furniture,1000.0,99.0,True,True,Genuine,
PaintCollections,Furniture,1000.0,99.0,True,True,Fake,Report #1790
PaintCollections,Furniture,1000.0,99.0,True,True,Fake,
PaintCollections,Furniture,1000.0,99.0,True,True,,Report #1791
PaintCollections,Furniture,1000.0,99.0,True,True,,
PaintCollections,Furniture,1000.0,99.0,True,False,Genuine,Report #1792
PaintCollections,Furniture,1000.0,99.0,True,False,Genuine,
PaintCollections,Furniture,1000.0,99.0,True,False,Fake,Report #1793
PaintCollections,Furniture,1000.0,99.0,True,False,Fake,
PaintCollections,Furniture,1000.0,99.0,True,False,,Report #1794
PaintCollections,Furniture,1000.0,99.0,True,False,,
PaintCollections,Furniture,1000.0,99.0,False,True,Genuine,Report #1795
PaintCollections,Furniture,1000.0,99.0,False,True,Genuine,
PaintCollections,Furniture,1000.0,99.0,False,True,Fake,Report #1796
PaintCollections,Furniture,1000.0,99.0,False,True,Fake,
PaintCollections,Furniture,1000.0,99.0,False,True,,Report #1797
PaintCollections,Furniture,1000.0,99.0,False,True,,
PaintCollections,Furniture,1000.0,99.0,False,False,Genuine,Report #1798
PaintCollections,Furniture,1000.0,99.0,False,False,Genuine,
PaintCollections,Furniture,1000.0,99.0,False,False,Fake,Report #1799
PaintCollections,Furniture,1000.0,99.0,False,False,Fake,
PaintCollections,Furniture,1000.0,99.0,False,False,,Report #1800
PaintCollections,Furniture,1000.0,99.0,False,False,,
PaintCollections,Furniture,1000.0,100.0,True,True,Genuine,Report #1801
PaintCollections,Furniture,1000.0,100.0,True,True,Genuine,
PaintCollections,Furniture,1000.0,100.0,True,True,Fake,Report #1802
PaintCollections,Furniture,1000.0,100.0,True,True,Fake,
PaintCollections,Furniture,1000.0,100.0,True,True,,Report #1803
PaintCollections,Furniture,1000.0,100.0,True,True,,
PaintCollections,Furniture,1000.0,100.0,True,False,Genuine,Report #1804
PaintCollections,Furniture,1000.0,100.0,True,False,Genuine,
PaintCollections,Furniture,1000.0,100.0,True,False,Fake,Report #1805
PaintCollections,Furniture,1000.0,100.0,True,False,Fake,
PaintCollections,Furniture,1000.0,100.0,True,False,,Report #1806
PaintCollections,Furniture,1000.0,100.0,True,False,,
PaintCollections,Furniture,1000.0,100.0,False,True,Genuine,Report #1807
PaintCollections,Furniture,1000.0,100.0,False,True,Genuine,
PaintCollections,Furniture,1000.0,100.0,False,True,Fake,Report #1808
PaintCollections,Furniture,1000.0,100.0,False,True,Fake,
PaintCollections,Furniture,1000.0,100.0,False,True,,Report #1809
PaintCollections,Furniture,1000.0,100.0,False,True,,
PaintCollections,Furniture,1000.0,100.0,False,False,Genuine,Report #1810
PaintCollections,Furniture,1000.0,100.0,False,False,Genuine,
PaintCollections,Furniture,1000.0,100.0,False,False,Fake,Report #1811
PaintCollections,Furniture,1000.0,100.0,False,False,Fake,
PaintCollections,Furniture,1000.0,100.0,False,False,,Report #1812
PaintCollections,Furniture,1000.0,100.0,False,False,,
PaintCollections,Furniture,1000.0,101.0,True,True,Genuine,Report #1813
PaintCollections,Furniture,1000.0,101.0,True,True,Genuine,
PaintCollections,Furniture,1000.0,101.0,True,True,Fake,Report #1814
PaintCollections,Furniture,1000.0,101.0,True,True,Fake,
PaintCollections,Furniture,1000.0,101.0,True,True,,Report #1815
PaintCollections,Furniture,1000.0,101.0,True,True,,
PaintCollections,Furniture,1000.0,101.0,True,False,Genuine,Report #1816
PaintCollections,Furniture,1000.0,101.0,True,False,Genuine,
PaintCollections,Furniture,1000.0,101.0,True,False,Fake,Report #1817
PaintCollections,Furniture,1000.0,101.0,True,False,Fake,
PaintCollections,Furniture,1000.0,101.0,True,False,,Report #1818
PaintCollections,Furniture,1000.0,101.0,True,False,,
PaintCollections,Furniture,1000.0,101.0,False,True,Genuine,Report #1819
PaintCollections,Furniture,1000.0,101.0,False,True,Genuine,
PaintCollections,Furniture,1000.0,101.0,False,True,Fake,Report #1820
PaintCollections,Furniture,1000.0,101.0,False,True,Fake,
PaintCollections,Furniture,1000.0,101.0,False,True,,Report #1821
PaintCollections,Furniture,1000.0,101.0,False,True,,
PaintCollections,Furniture,1000.0,101.0,False,False,Genuine,Report #1822
PaintCollections,Furniture,1000.0,101.0,False,False,Genuine,
PaintCollections,Furniture,1000.0,101.0,False,False,Fake,Report #1823
PaintCollections,Furniture,1000.0,101.0,False,False,Fake,
PaintCollections,Furniture,1000.0,101.0,False,False,,Report #1824
PaintCollections,Furniture,1000.0,101.0,False,False,,
PaintCollections,Furniture,1000000.0,-1.0,True,True,Genuine,Report #1825
PaintCollections,Furniture,1000000.0,-1.0,True,True,Genuine,
PaintCollections,Furniture,1000000.0,-1.0,True,True,Fake,Report #1826
PaintCollections,Furniture,1000000.0,-1.0,True,True,Fake,
PaintCollections,Furniture,1000000.0,-1.0,True,True,,Report #1827
PaintCollections,Furniture,1000000.0,-1.0,True,True,,
PaintCollections,Furniture,1000000.0,-1.0,True,False,Genuine,Report #1828
PaintCollections,Furniture,1000000.0,-1.0,True,False,Genuine,
PaintCollections,Furniture,1000000.0,-1.0,True,False,Fake,Report #1829
PaintCollections,Furniture,1000000.0,-1.0,True,False,Fake,
PaintCollections,Furniture,1000000.0,-1.0,True,False,,Report #1830
PaintCollections,Furniture,1000000.0,-1.0,True,False,,
PaintCollections,Furniture,1000000.0,-1.0,False,True,Genuine,Report #1831
PaintCollections,Furniture,1000000.0,-1.0,False,True,Genuine,
PaintCollections,Furniture,1000000.0,-1.0,False,True,Fake,Report #1832
PaintCollections,Furniture,1000000.0,-1.0,False,True,Fake,
PaintCollections,Furniture,1000000.0,-1.0,False,True,,Report #1833
PaintCollections,Furniture,1000000.0,-1.0,False,True,,
PaintCollections,Furniture,1000000.0,-1.0,False,False,Genuine,Report #1834
PaintCollections,Furniture,1000000.0,-1.0,False,False,Genuine,
PaintCollections,Furniture,1000000.0,-1.0,False,False,Fake,Report #1835
PaintCollections,Furniture,1000000.0,-1.0,False,False,Fake,
PaintCollections,Furniture,1000000.0,-1.0,False,False,,Report #1836
PaintCollections,Furniture,1000000.0,-1.0,False,False,,
PaintCollections,Furniture,1000000.0,0.0,True,True,Genuine,Report #1837
PaintCollections,Furniture,1000000.0,0.0,True,True,Genuine,
PaintCollections,Furniture,1000000.0,0.0,True,True,Fake,Report #1838
PaintCollections,Furniture,1000000.0,0.0,True,True,Fake,
PaintCollections,Furniture,1000000.0,0.0,True,True,,Report #1839
PaintCollections,Furniture,1000000.0,0.0,True,True,,
PaintCollections,Furniture,1000000.0,0.0,True,False,Genuine,Report #1840
PaintCollections,Furniture,1000000.0,0.0,True,False,Genuine,
PaintCollections,Furniture,1000000.0,0.0,True,False,Fake,Report #1841
PaintCollections,Furniture,1000000.0,0.0,True,False,Fake,
PaintCollections,Furniture,1000000.0,0.0,True,False,,Report #1842
PaintCollections,Furniture,1000000.0,0.0,True,False,,
PaintCollections,Furniture,1000000.0,0.0,False,True,Genuine,Report #1843
PaintCollections,Furniture,1000000.0,0.0,False,True,Genuine,
PaintCollections,Furniture,1000000.0,0.0,False,True,Fake,Report #1844
PaintCollections,Furniture,1000000.0,0.0,False,True,Fake,
PaintCollections,Furniture,1000000.0,0.0,False,True,,Report #1845
PaintCollections,Furniture,1000000.0,0.0,False,True,,
PaintCollections,Furniture,1000000.0,0.0,False,False,Genuine,Report #1846
PaintCollections,Furniture,1000000.0,0.0,False,False,Genuine,
PaintCollections,Furniture,1000000.0,0.0,False,False,Fake,Report #1847
PaintCollections,Furniture,1000000.0,0.0,False,False,Fake,
PaintCollections,Furniture,1000000.0,0.0,False,False,,Report #1848
PaintCollections,Furniture,1000000.0,0.0,False,False,,
PaintCollections,Furniture,1000000.0,19.0,True,True,Genuine,Report #1849
PaintCollections,Furniture,1000000.0,19.0,True,True,Genuine,
PaintCollections,Furniture,1000000.0,19.0,True,True,Fake,Report #1850
PaintCollections,Furniture,1000000.0,19.0,True,True,Fake,
PaintCollections,Furniture,1000000.0,19.0,True,True,,Report #1851
PaintCollections,Furniture,1000000.0,19.0,True,True,,
PaintCollections,Furniture,1000000.0,19.0,True,False,Genuine,Report #1852
PaintCollections,Furniture,1000000.0,19.0,True,False,Genuine,
PaintCollections,Furniture,1000000.0,19.0,True,False,Fake,Report #1853
PaintCollections,Furniture,1000000.0,19.0,True,False,Fake,
PaintCollections,Furniture,1000000.0,19.0,True,False,,Report #1854
PaintCollections,Furniture,1000000.0,19.0,True,False,,
PaintCollections,Furniture,1000000.0,19.0,False,True,Genuine,Report #1855
PaintCollections,Furniture,1000000.0,19.0,False,True,Genuine,
PaintCollections,Furniture,1000000.0,19.0,False,True,Fake,Report #1856
PaintCollections,Furniture,1000000.0,19.0,False,True,Fake,
PaintCollections,Furniture,1000000.0,19.0,False,True,,Report #1857
PaintCollections,Furniture,1000000.0,19.0,False,True,,
PaintCollections,Furniture,1000000.0,19.0,False,False,Genuine,Report #1858
PaintCollections,Furniture,1000000.0,19.0,False,False,Genuine,
PaintCollections,Furniture,1000000.0,19.0,False,False,Fake,Report #1859
PaintCollections,Furniture,1000000.0,19.0,False,False,Fake,
PaintCollections,Furniture,1000000.0,19.0,False,False,,Report #1860
PaintCollections,Furniture,1000000.0,19.0,False,False,,
PaintCollections,Furniture,1000000.0,20.0,True,True,Genuine,Report #1861
PaintCollections,Furniture,1000000.0,20.0,True,True,Genuine,
PaintCollections,Furniture,1000000.0,20.0,True,True,Fake,Report #1862
PaintCollections,Furniture,1000000.0,20.0,True,True,Fake,
PaintCollections,Furniture,1000000.0,20.0,True,True,,Report #1863
PaintCollections,Furniture,1000000.0,20.0,True,True,,
PaintCollections,Furniture,1000000.0,20.0,True,False,Genuine,Report #1864
PaintCollections,Furniture,1000000.0,20.0,True,False,Genuine,
PaintCollections,Furniture,1000000.0,20.0,True,False,Fake,Report #1865
PaintCollections,Furniture,1000000.0,20.0,True,False,Fake,
PaintCollections,Furniture,1000000.0,20.0,True,False,,Report #1866
PaintCollections,Furniture,1000000.0,20.0,True,False,,
PaintCollections,Furniture,1000000.0,20.0,False,True,Genuine,Report #1867
PaintCollections,Furniture,1000000.0,20.0,False,True,Genuine,
PaintCollections,Furniture,1000000.0,20.0,False,True,Fake,Report #1868
PaintCollections,Furniture,1000000.0,20.0,False,True,Fake,
PaintCollections,Furniture,1000000.0,20.0,False,True,,Report #1869
PaintCollections,Furniture,1000000.0,20.0,False,True,,
PaintCollections,Furniture,1000000.0,20.0,False,False,Genuine,Report #1870
PaintCollections,Furniture,1000000.0,20.0,False,False,Genuine,
PaintCollections,Furniture,1000000.0,20.0,False,False,Fake,Report #1871
PaintCollections,Furniture,1000000.0,20.0,False,False,Fake,
PaintCollections,Furniture,1000000.0,20.0,False,False,,Report #1872
PaintCollections,Furniture,1000000.0,20.0,False,False,,
PaintCollections,Furniture,1000000.0,21.0,True,True,Genuine,Report #1873
PaintCollections,Furniture,1000000.0,21.0,True,True,Genuine,
PaintCollections,Furniture,1000000.0,21.0,True,True,Fake,Report #1874
PaintCollections,Furniture,1000000.0,21.0,True,True,Fake,
PaintCollections,Furniture,1000000.0,21.0,True,True,,Report #1875
PaintCollections,Furniture,1000000.0,21.0,True,True,,
PaintCollections,Furniture,1000000.0,21.0,True,False,Genuine,Report #1876
PaintCollections,Furniture,1000000.0,21.0,True,False,Genuine,
PaintCollections,Furniture,1000000.0,21.0,True,False,Fake,Report #1877
PaintCollections,Furniture,1000000.0,21.0,True,False,Fake,
PaintCollections,Furniture,1000000.0,21.0,True,False,,Report #1878
PaintCollections,Furniture,1000000.0,21.0,True,False,,
PaintCollections,Furniture,1000000.0,21.0,False,True,Genuine,Report #1879
PaintCollections,Furniture,1000000.0,21.0,False,True,Genuine,
PaintCollections,Furniture,1000000.0,21.0,False,True,Fake,Report #1880
PaintCollections,Furniture,1000000.0,21.0,False,True,Fake,
PaintCollections,Furniture,1000000.0,21.0,False,True,,Report #1881
PaintCollections,Furniture,1000000.0,21.0,False,True,,
PaintCollections,Furniture,1000000.0,21.0,False,False,Genuine,Report #1882
PaintCollections,Furniture,1000000.0,21.0,False,False,Genuine,
PaintCollections,Furniture,1000000.0,21.0,False,False,Fake,Report #1883
PaintCollections,Furniture,1000000.0,21.0,False,False,Fake,
PaintCollections,Furniture,1000000.0,21.0,False,False,,Report #1884
PaintCollections,Furniture,1000000.0,21.0,False,False,,
PaintCollections,Furniture,1000000.0,99.0,True,True,Genuine,Report #1885
PaintCollections,Furniture,1000000.0,99.0,True,True,Genuine,
PaintCollections,Furniture,1000000.0,99.0,True,True,Fake,Report #1886
PaintCollections,Furniture,1000000.0,99.0,True,True,Fake,
PaintCollections,Furniture,1000000.0,99.0,True,True,,Report #1887
PaintCollections,Furniture,1000000.0,99.0,True,True,,
PaintCollections,Furniture,1000000.0,99.0,True,False,Genuine,Report #1888
PaintCollections,Furniture,1000000.0,99.0,True,False,Genuine,
PaintCollections,Furniture,1000000.0,99.0,True,False,Fake,Report #1889
PaintCollections,Furniture,1000000.0,99.0,True,False,Fake,
PaintCollections,Furniture,1000000.0,99.0,True,False,,Report #1890
PaintCollections,Furniture,1000000.0,99.0,True,False,,
PaintCollections,Furniture,1000000.0,99.0,False,True,Genuine,Report #1891
PaintCollections,Furniture,1000000.0,99.0,False,True,Genuine,
PaintCollections,Furniture,1000000.0,99.0,False,True,Fake,Report #1892
PaintCollections,Furniture,1000000.0,99.0,False,True,Fake,
PaintCollections,Furniture,1000000.0,99.0,False,True,,Report #1893
PaintCollections,Furniture,1000000.0,99.0,False,True,,
PaintCollections,Furniture,1000000.0,99.0,False,False,Genuine,Report #1894
PaintCollections,Furniture,1000000.0,99.0,False,False,Genuine,
PaintCollections,Furniture,1000000.0,99.0,False,False,Fake,Report #1895
PaintCollections,Furniture,1000000.0,99.0,False,False,Fake,
PaintCollections,Furniture,1000000.0,99.0,False,False,,Report #1896
PaintCollections,Furniture,1000000.0,99.0,False,False,,
PaintCollections,Furniture,1000000.0,100.0,True,True,Genuine,Report #1897
PaintCollections,Furniture,1000000.0,100.0,True,True,Genuine,
PaintCollections,Furniture,1000000.0,100.0,True,True,Fake,Report #1898
PaintCollections,Furniture,1000000.0,100.0,True,True,Fake,
PaintCollections,Furniture,1000000.0,100.0,True,True,,Report #1899
PaintCollections,Furniture,1000000.0,100.0,True,True,,
PaintCollections,Furniture,1000000.0,100.0,True,False,Genuine,Report #1900
PaintCollections,Furniture,1000000.0,100.0,True,False,Genuine,
PaintCollections,Furniture,1000000.0,100.0,True,False,Fake,Report #1901
PaintCollections,Furniture,1000000.0,100.0,True,False,Fake,
PaintCollections,Furniture,1000000.0,100.0,True,False,,Report #1902
PaintCollections,Furniture,1000000.0,100.0,True,False,,
PaintCollections,Furniture,1000000.0,100.0,False,True,Genuine,Report #1903
PaintCollections,Furniture,1000000.0,100.0,False,True,Genuine,
PaintCollections,Furniture,1000000.0,100.0,False,True,Fake,Report #1904
PaintCollections,Furniture,1000000.0,100.0,False,True,Fake,
PaintCollections,Furniture,1000000.0,100.0,False,True,,Report #1905
PaintCollections,Furniture,1000000.0,100.0,False,True,,
PaintCollections,Furniture,1000000.0,100.0,False,False,Genuine,Report #1906
PaintCollections,Furniture,1000000.0,100.0,False,False,Genuine,
PaintCollections,Furniture,1000000.0,100.0,False,False,Fake,Report #1907
PaintCollections,Furniture,1000000.0,100.0,False,False,Fake,
PaintCollections,Furniture,1000000.0,100.0,False,False,,Report #1908
PaintCollections,Furniture,1000000.0,100.0,False,False,,
PaintCollections,Furniture,1000000.0,101.0,True,True,Genuine,Report #1909
PaintCollections,Furniture,1000000.0,101.0,True,True,Genuine,
PaintCollections,Furniture,1000000.0,101.0,True,True,Fake,Report #1910
PaintCollections,Furniture,1000000.0,101.0,True,True,Fake,
PaintCollections,Furniture,1000000.0,101.0,True,True,,Report #1911
PaintCollections,Furniture,1000000.0,101.0,True,True,,
PaintCollections,Furniture,1000000.0,101.0,True,False,Genuine,Report #1912
PaintCollections,Furniture,1000000.0,101.0,True,False,Genuine,
PaintCollections,Furniture,1000000.0,101.0,True,False,Fake,Report #1913
PaintCollections,Furniture,1000000.0,101.0,True,False,Fake,
PaintCollections,Furniture,1000000.0,101.0,True,False,,Report #1914
PaintCollections,Furniture,1000000.0,101.0,True,False,,
PaintCollections,Furniture,1000000.0,101.0,False,True,Genuine,Report #1915
PaintCollections,Furniture,1000000.0,101.0,False,True,Genuine,
PaintCollections,Furniture,1000000.0,101.0,False,True,Fake,Report #1916
PaintCollections,Furniture,1000000.0,101.0,False,True,Fake,
PaintCollections,Furniture,1000000.0,101.0,False,True,,Report #1917
PaintCollections,Furniture,1000000.0,101.0,False,True,,
PaintCollections,Furniture,1000000.0,101.0,False,False,Genuine,Report #1918
PaintCollections,Furniture,1000000.0,101.0,False,False,Genuine,
PaintCollections,Furniture,1000000.0,101.0,False,False,Fake,Report #1919
PaintCollections,Furniture,1000000.0,101.0,False,False,Fake,
PaintCollections,Furniture,1000000.0,101.0,False,False,,Report #1920
PaintCollections,Furniture,1000000.0,101.0,False,False,,
PaintCollections,HistoryWritings,0.0,-1.0,True,True,Genuine,Report #1921
PaintCollections,HistoryWritings,0.0,-1.0,True,True,Genuine,
PaintCollections,HistoryWritings,0.0,-1.0,True,True,Fake,Report #1922
PaintCollections,HistoryWritings,0.0,-1.0,True,True,Fake,
PaintCollections,HistoryWritings,0.0,-1.0,True,True,,Report #1923
PaintCollections,HistoryWritings,0.0,-1.0,True,True,,
PaintCollections,HistoryWritings,0.0,-1.0,True,False,Genuine,Report #1924
PaintCollections,HistoryWritings,0.0,-1.0,True,False,Genuine,
PaintCollections,HistoryWritings,0.0,-1.0,True,False,Fake,Report #1925
PaintCollections,HistoryWritings,0.0,-1.0,True,False,Fake,
PaintCollections,HistoryWritings,0.0,-1.0,True,False,,Report #1926
PaintCollections,HistoryWritings,0.0,-1.0,True,False,,
PaintCollections,HistoryWritings,0.0,-1.0,False,True,Genuine,Report #1927
PaintCollections,HistoryWritings,0.0,-1.0,False,True,Genuine,
PaintCollections,HistoryWritings,0.0,-1.0,False,True,Fake,Report #1928
PaintCollections,HistoryWritings,0.0,-1.0,False,True,Fake,
PaintCollections,HistoryWritings,0.0,-1.0,False,True,,Report #1929
PaintCollections,HistoryWritings,0.0,-1.0,False,True,,
PaintCollections,HistoryWritings,0.0,-1.0,False,False,Genuine,Report #1930
PaintCollections,HistoryWritings,0.0,-1.0,False,False,Genuine,
PaintCollections,HistoryWritings,0.0,-1.0,False,False,Fake,Report #1931
PaintCollections,HistoryWritings,0.0,-1.0,False,False,Fake,
PaintCollections,HistoryWritings,0.0,-1.0,False,False,,Report #1932
PaintCollections,HistoryWritings,0.0,-1.0,False,False,,
PaintCollections,HistoryWritings,0.0,0.0,True,True,Genuine,Report #1933
PaintCollections,HistoryWritings,0.0,0.0,True,True,Genuine,
PaintCollections,HistoryWritings,0.0,0.0,True,True,Fake,Report #1934
PaintCollections,HistoryWritings,0.0,0.0,True,True,Fake,
PaintCollections,HistoryWritings,0.0,0.0,True,True,,Report #1935
PaintCollections,HistoryWritings,0.0,0.0,True,True,,
PaintCollections,HistoryWritings,0.0,0.0,True,False,Genuine,Report #1936
PaintCollections,HistoryWritings,0.0,0.0,True,False,Genuine,
PaintCollections,HistoryWritings,0.0,0.0,True,False,Fake,Report #1937
PaintCollections,HistoryWritings,0.0,0.0,True,False,Fake,
PaintCollections,HistoryWritings,0.0,0.0,True,False,,Report #1938
PaintCollections,HistoryWritings,0.0,0.0,True,False,,
PaintCollections,HistoryWritings,0.0,0.0,False,True,Genuine,Report #1939
PaintCollections,HistoryWritings,0.0,0.0,False,True,Genuine,
PaintCollections,HistoryWritings,0.0,0.0,False,True,Fake,Report #1940
PaintCollections,HistoryWritings,0.0,0.0,False,True,Fake,
PaintCollections,HistoryWritings,0.0,0.0,False,True,,Report #1941
PaintCollections,HistoryWritings,0.0,0.0,False,True,,
PaintCollections,HistoryWritings,0.0,0.0,False,False,Genuine,Report #1942
PaintCollections,HistoryWritings,0.0,0.0,False,False,Genuine,
PaintCollections,HistoryWritings,0.0,0.0,False,False,Fake,Report #1943
PaintCollections,HistoryWritings,0.0,0.0,False,False,Fake,
PaintCollections,HistoryWritings,0.0,0.0,False,False,,Report #1944
PaintCollections,HistoryWritings,0.0,0.0,False,False,,
PaintCollections,HistoryWritings,0.0,19.0,True,True,Genuine,Report #1945
PaintCollections,HistoryWritings,0.0,19.0,True,True,Genuine,
PaintCollections,HistoryWritings,0.0,19.0,True,True,Fake,Report #1946
PaintCollections,HistoryWritings,0.0,19.0,True,True,Fake,
PaintCollections,HistoryWritings,0.0,19.0,True,True,,Report #1947
PaintCollections,HistoryWritings,0.0,19.0,True,True,,
PaintCollections,HistoryWritings,0.0,19.0,True,False,Genuine,Report #1948
PaintCollections,HistoryWritings,0.0,19.0,True,False,Genuine,
PaintCollections,HistoryWritings,0.0,19.0,True,False,Fake,Report #1949
PaintCollections,HistoryWritings,0.0,19.0,True,False,Fake,
PaintCollections,HistoryWritings,0.0,19.0,True,False,,Report #1950
PaintCollections,HistoryWritings,0.0,19.0,True,False,,
PaintCollections,HistoryWritings,0.0,19.0,False,True,Genuine,Report #1951
PaintCollections,HistoryWritings,0.0,19.0,False,True,Genuine,
PaintCollections,HistoryWritings,0.0,19.0,False,True,Fake,Report #1952
PaintCollections,HistoryWritings,0.0,19.0,False,True,Fake,
PaintCollections,HistoryWritings,0.0,19.0,False,True,,Report #1953
PaintCollections,HistoryWritings,0.0,19.0,False,True,,
PaintCollections,HistoryWritings,0.0,19.0,False,False,Genuine,Report #1954
PaintCollections,HistoryWritings,0.0,19.0,False,False,Genuine,
PaintCollections,HistoryWritings,0.0,19.0,False,False,Fake,Report #1955
PaintCollections,HistoryWritings,0.0,19.0,False,False,Fake,
PaintCollections,HistoryWritings,0.0,19.0,False,False,,Report #1956
PaintCollections,HistoryWritings,0.0,19.0,False,False,,
PaintCollections,HistoryWritings,0.0,20.0,True,True,Genuine,Report #1957
PaintCollections,HistoryWritings,0.0,20.0,True,True,Genuine,
PaintCollections,HistoryWritings,0.0,20.0,True,True,Fake,Report #1958
PaintCollections,HistoryWritings,0.0,20.0,True,True,Fake,
PaintCollections,HistoryWritings,0.0,20.0,True,True,,Report #1959
PaintCollections,HistoryWritings,0.0,20.0,True,True,,
PaintCollections,HistoryWritings,0.0,20.0,True,False,Genuine,Report #1960
PaintCollections,HistoryWritings,0.0,20.0,True,False,Genuine,
PaintCollections,HistoryWritings,0.0,20.0,True,False,Fake,Report #1961
PaintCollections,HistoryWritings,0.0,20.0,True,False,Fake,
PaintCollections,HistoryWritings,0.0,20.0,True,False,,Report #1962
PaintCollections,HistoryWritings,0.0,20.0,True,False,,
PaintCollections,HistoryWritings,0.0,20.0,False,True,Genuine,Report #1963
PaintCollections,HistoryWritings,0.0,20.0,False,True,Genuine,
PaintCollections,HistoryWritings,0.0,20.0,False,True,Fake,Report #1964
PaintCollections,HistoryWritings,0.0,20.0,False,True,Fake,
PaintCollections,HistoryWritings,0.0,20.0,False,True,,Report #1965
PaintCollections,HistoryWritings,0.0,20.0,False,True,,
PaintCollections,HistoryWritings,0.0,20.0,False,False,Genuine,Report #1966
PaintCollections,HistoryWritings,0.0,20.0,False,False,Genuine,
PaintCollections,HistoryWritings,0.0,20.0,False,False,Fake,Report #1967
PaintCollections,HistoryWritings,0.0,20.0,False,False,Fake,
PaintCollections,HistoryWritings,0.0,20.0,False,False,,Report #1968
PaintCollections,HistoryWritings,0.0,20.0,False,False,,
PaintCollections,HistoryWritings,0.0,21.0,True,True,Genuine,Report #1969
PaintCollections,HistoryWritings,0.0,21.0,True,True,Genuine,
PaintCollections,HistoryWritings,0.0,21.0,True,True,Fake,Report #1970
PaintCollections,HistoryWritings,0.0,21.0,True,True,Fake,
PaintCollections,HistoryWritings,0.0,21.0,True,True,,Report #1971
PaintCollections,HistoryWritings,0.0,21.0,True,True,,
PaintCollections,HistoryWritings,0.0,21.0,True,False,Genuine,Report #1972
PaintCollections,HistoryWritings,0.0,21.0,True,False,Genuine,
PaintCollections,HistoryWritings,0.0,21.0,True,False,Fake,Report #1973
PaintCollections,HistoryWritings,0.0,21.0,True,False,Fake,
PaintCollections,HistoryWritings,0.0,21.0,True,False,,Report #1974
PaintCollections,HistoryWritings,0.0,21.0,True,False,,
PaintCollections,HistoryWritings,0.0,21.0,False,True,Genuine,Report #1975
PaintCollections,HistoryWritings,0.0,21.0,False,True,Genuine,
PaintCollections,HistoryWritings,0.0,21.0,False,True,Fake,Report #1976
PaintCollections,HistoryWritings,0.0,21.0,False,True,Fake,
PaintCollections,HistoryWritings,0.0,21.0,False,True,,Report #1977
PaintCollections,HistoryWritings,0.0,21.0,False,True,,
PaintCollections,HistoryWritings,0.0,21.0,False,False,Genuine,Report #1978
PaintCollections,HistoryWritings,0.0,21.0,False,False,Genuine,
PaintCollections,HistoryWritings,0.0,21.0,False,False,Fake,Report #1979
PaintCollections,HistoryWritings,0.0,21.0,False,False,Fake,
PaintCollections,HistoryWritings,0.0,21.0,False,False,,Report #1980
PaintCollections,HistoryWritings,0.0,21.0,False,False,,
PaintCollections,HistoryWritings,0.0,99.0,True,True,Genuine,Report #1981
PaintCollections,HistoryWritings,0.0,99.0,True,True,Genuine,
PaintCollections,HistoryWritings,0.0,99.0,True,True,Fake,Report #1982
PaintCollections,HistoryWritings,0.0,99.0,True,True,Fake,
PaintCollections,HistoryWritings,0.0,99.0,True,True,,Report #1983
PaintCollections,HistoryWritings,0.0,99.0,True,True,,
PaintCollections,HistoryWritings,0.0,99.0,True,False,Genuine,Report #1984
PaintCollections,HistoryWritings,0.0,99.0,True,False,Genuine,
PaintCollections,HistoryWritings,0.0,99.0,True,False,Fake,Report #1985
PaintCollections,HistoryWritings,0.0,99.0,True,False,Fake,
PaintCollections,HistoryWritings,0.0,99.0,True,False,,Report #1986
PaintCollections,HistoryWritings,0.0,99.0,True,False,,
PaintCollections,HistoryWritings,0.0,99.0,False,True,Genuine,Report #1987
PaintCollections,HistoryWritings,0.0,99.0,False,True,Genuine,
PaintCollections,HistoryWritings,0.0,99.0,False,True,Fake,Report #1988
PaintCollections,HistoryWritings,0.0,99.0,False,True,Fake,
PaintCollections,HistoryWritings,0.0,99.0,False,True,,Report #1989
PaintCollections,HistoryWritings,0.0,99.0,False,True,,
PaintCollections,HistoryWritings,0.0,99.0,False,False,Genuine,Report #1990
PaintCollections,HistoryWritings,0.0,99.0,False,False,Genuine,
PaintCollections,HistoryWritings,0.0,99.0,False,False,Fake,Report #1991
PaintCollections,HistoryWritings,0.0,99.0,False,False,Fake,
PaintCollections,HistoryWritings,0.0,99.0,False,False,,Report #1992
PaintCollections,HistoryWritings,0.0,99.0,False,False,,
PaintCollections,HistoryWritings,0.0,100.0,True,True,Genuine,Report #1993
PaintCollections,HistoryWritings,0.0,100.0,True,True,Genuine,
PaintCollections,HistoryWritings,0.0,100.0,True,True,Fake,Report #1994
PaintCollections,HistoryWritings,0.0,100.0,True,True,Fake,
PaintCollections,HistoryWritings,0.0,100.0,True,True,,Report #1995
PaintCollections,HistoryWritings,0.0,100.0,True,True,,
PaintCollections,HistoryWritings,0.0,100.0,True,False,Genuine,Report #1996
PaintCollections,HistoryWritings,0.0,100.0,True,False,Genuine,
PaintCollections,HistoryWritings,0.0,100.0,True,False,Fake,Report #1997
PaintCollections,HistoryWritings,0.0,100.0,True,False,Fake,
PaintCollections,HistoryWritings,0.0,100.0,True,False,,Report #1998
PaintCollections,HistoryWritings,0.0,100.0,True,False,,
PaintCollections,HistoryWritings,0.0,100.0,False,True,Genuine,Report #1999
PaintCollections,HistoryWritings,0.0,100.0,False,True,Genuine,
PaintCollections,HistoryWritings,0.0,100.0,False,True,Fake,Report #2000
PaintCollections,HistoryWritings,0.0,100.0,False,True,Fake,
PaintCollections,HistoryWritings,0.0,100.0,False,True,,Report #2001
PaintCollections,HistoryWritings,0.0,100.0,False,True,,
PaintCollections,HistoryWritings,0.0,100.0,False,False,Genuine,Report #2002
PaintCollections,HistoryWritings,0.0,100.0,False,False,Genuine,
PaintCollections,HistoryWritings,0.0,100.0,False,False,Fake,Report #2003
PaintCollections,HistoryWritings,0.0,100.0,False,False,Fake,
PaintCollections,HistoryWritings,0.0,100.0,False,False,,Report #2004
PaintCollections,HistoryWritings,0.0,100.0,False,False,,
PaintCollections,HistoryWritings,0.0,101.0,True,True,Genuine,Report #2005
PaintCollections,HistoryWritings,0.0,101.0,True,True,Genuine,
PaintCollections,HistoryWritings,0.0,101.0,True,True,Fake,Report #2006
PaintCollections,HistoryWritings,0.0,101.0,True,True,Fake,
PaintCollections,HistoryWritings,0.0,101.0,True,True,,Report #2007
PaintCollections,HistoryWritings,0.0,101.0,True,True,,
PaintCollections,HistoryWritings,0.0,101.0,True,False,Genuine,Report #2008
PaintCollections,HistoryWritings,0.0,101.0,True,False,Genuine,
PaintCollections,HistoryWritings,0.0,101.0,True,False,Fake,Report #2009
PaintCollections,HistoryWritings,0.0,101.0,True,False,Fake,
PaintCollections,HistoryWritings,0.0,101.0,True,False,,Report #2010
PaintCollections,HistoryWritings,0.0,101.0,True,False,,
PaintCollections,HistoryWritings,0.0,101.0,False,True,Genuine,Report #2011
PaintCollections,HistoryWritings,0.0,101.0,False,True,Genuine,
PaintCollections,HistoryWritings,0.0,101.0,False,True,Fake,Report #2012
PaintCollections,HistoryWritings,0.0,101.0,False,True,Fake,
PaintCollections,HistoryWritings,0.0,101.0,False,True,,Report #2013
PaintCollections,HistoryWritings,0.0,101.0,False,True,,
PaintCollections,HistoryWritings,0.0,101.0,False,False,Genuine,Report #2014
PaintCollections,HistoryWritings,0.0,101.0,False,False,Genuine,
PaintCollections,HistoryWritings,0.0,101.0,False,False,Fake,Report #2015
PaintCollections,HistoryWritings,0.0,101.0,False,False,Fake,
PaintCollections,HistoryWritings,0.0,101.0,False,False,,Report #2016
PaintCollections,HistoryWritings,0.0,101.0,False,False,,
PaintCollections,HistoryWritings,1.0,-1.0,True,True,Genuine,Report #2017
PaintCollections,HistoryWritings,1.0,-1.0,True,True,Genuine,
PaintCollections,HistoryWritings,1.0,-1.0,True,True,Fake,Report #2018
PaintCollections,HistoryWritings,1.0,-1.0,True,True,Fake,
PaintCollections,HistoryWritings,1.0,-1.0,True,True,,Report #2019
PaintCollections,HistoryWritings,1.0,-1.0,True,True,,
PaintCollections,HistoryWritings,1.0,-1.0,True,False,Genuine,Report #2020
PaintCollections,HistoryWritings,1.0,-1.0,True,False,Genuine,
PaintCollections,HistoryWritings,1.0,-1.0,True,False,Fake,Report #2021
PaintCollections,HistoryWritings,1.0,-1.0,True,False,Fake,
PaintCollections,HistoryWritings,1.0,-1.0,True,False,,Report #2022
PaintCollections,HistoryWritings,1.0,-1.0,True,False,,
PaintCollections,HistoryWritings,1.0,-1.0,False,True,Genuine,Report #2023
PaintCollections,HistoryWritings,1.0,-1.0,False,True,Genuine,
PaintCollections,HistoryWritings,1.0,-1.0,False,True,Fake,Report #2024
PaintCollections,HistoryWritings,1.0,-1.0,False,True,Fake,
PaintCollections,HistoryWritings,1.0,-1.0,False,True,,Report #2025
PaintCollections,HistoryWritings,1.0,-1.0,False,True,,
PaintCollections,HistoryWritings,1.0,-1.0,False,False,Genuine,Report #2026
PaintCollections,HistoryWritings,1.0,-1.0,False,False,Genuine,
PaintCollections,HistoryWritings,1.0,-1.0,False,False,Fake,Report #2027
PaintCollections,HistoryWritings,1.0,-1.0,False,False,Fake,
PaintCollections,HistoryWritings,1.0,-1.0,False,False,,Report #2028
PaintCollections,HistoryWritings,1.0,-1.0,False,False,,
PaintCollections,HistoryWritings,1.0,0.0,True,True,Genuine,Report #2029
PaintCollections,HistoryWritings,1.0,0.0,True,True,Genuine,
PaintCollections,HistoryWritings,1.0,0.0,True,True,Fake,Report #2030
PaintCollections,HistoryWritings,1.0,0.0,True,True,Fake,
PaintCollections,HistoryWritings,1.0,0.0,True,True,,Report #2031
PaintCollections,HistoryWritings,1.0,0.0,True,True,,
PaintCollections,HistoryWritings,1.0,0.0,True,False,Genuine,Report #2032
PaintCollections,HistoryWritings,1.0,0.0,True,False,Genuine,
PaintCollections,HistoryWritings,1.0,0.0,True,False,Fake,Report #2033
PaintCollections,HistoryWritings,1.0,0.0,True,False,Fake,
PaintCollections,HistoryWritings,1.0,0.0,True,False,,Report #2034
PaintCollections,HistoryWritings,1.0,0.0,True,False,,
PaintCollections,HistoryWritings,1.0,0.0,False,True,Genuine,Report #2035
PaintCollections,HistoryWritings,1.0,0.0,False,True,Genuine,
PaintCollections,HistoryWritings,1.0,0.0,False,True,Fake,Report #2036
PaintCollections,HistoryWritings,1.0,0.0,False,True,Fake,
PaintCollections,HistoryWritings,1.0,0.0,False,True,,Report #2037
PaintCollections,HistoryWritings,1.0,0.0,False,True,,
PaintCollections,HistoryWritings,1.0,0.0,False,False,Genuine,Report #2038
PaintCollections,HistoryWritings,1.0,0.0,False,False,Genuine,
PaintCollections,HistoryWritings,1.0,0.0,False,False,Fake,Report #2039
PaintCollections,HistoryWritings,1.0,0.0,False,False,Fake,
PaintCollections,HistoryWritings,1.0,0.0,False,False,,Report #2040
PaintCollections,HistoryWritings,1.0,0.0,False,False,,
PaintCollections,HistoryWritings,1.0,19.0,True,True,Genuine,Report #2041
PaintCollections,HistoryWritings,1.0,19.0,True,True,Genuine,
PaintCollections,HistoryWritings,1.0,19.0,True,True,Fake,Report #2042
PaintCollections,HistoryWritings,1.0,19.0,True,True,Fake,
PaintCollections,HistoryWritings,1.0,19.0,True,True,,Report #2043
PaintCollections,HistoryWritings,1.0,19.0,True,True,,
PaintCollections,HistoryWritings,1.0,19.0,True,False,Genuine,Report #2044
PaintCollections,HistoryWritings,1.0,19.0,True,False,Genuine,
PaintCollections,HistoryWritings,1.0,19.0,True,False,Fake,Report #2045
PaintCollections,HistoryWritings,1.0,19.0,True,False,Fake,
PaintCollections,HistoryWritings,1.0,19.0,True,False,,Report #2046
PaintCollections,HistoryWritings,1.0,19.0,True,False,,
PaintCollections,HistoryWritings,1.0,19.0,False,True,Genuine,Report #2047
PaintCollections,HistoryWritings,1.0,19.0,False,True,Genuine,
PaintCollections,HistoryWritings,1.0,19.0,False,True,Fake,Report #2048
PaintCollections,HistoryWritings,1.0,19.0,False,True,Fake,
PaintCollections,HistoryWritings,1.0,19.0,False,True,,Report #2049
PaintCollections,HistoryWritings,1.0,19.0,False,True,,
PaintCollections,HistoryWritings,1.0,19.0,False,False,Genuine,Report #2050
PaintCollections,HistoryWritings,1.0,19.0,False,False,Genuine,
PaintCollections,HistoryWritings,1.0,19.0,False,False,Fake,Report #2051
PaintCollections,HistoryWritings,1.0,19.0,False,False,Fake,
PaintCollections,HistoryWritings,1.0,19.0,False,False,,Report #2052
PaintCollections,HistoryWritings,1.0,19.0,False,False,,
PaintCollections,HistoryWritings,1.0,20.0,True,True,Genuine,Report #2053
PaintCollections,HistoryWritings,1.0,20.0,True,True,Genuine,
PaintCollections,HistoryWritings,1.0,20.0,True,True,Fake,Report #2054
PaintCollections,HistoryWritings,1.0,20.0,True,True,Fake,
PaintCollections,HistoryWritings,1.0,20.0,True,True,,Report #2055
PaintCollections,HistoryWritings,1.0,20.0,True,True,,
PaintCollections,HistoryWritings,1.0,20.0,True,False,Genuine,Report #2056
PaintCollections,HistoryWritings,1.0,20.0,True,False,Genuine,
PaintCollections,HistoryWritings,1.0,20.0,True,False,Fake,Report #2057
PaintCollections,HistoryWritings,1.0,20.0,True,False,Fake,
PaintCollections,HistoryWritings,1.0,20.0,True,False,,Report #2058
PaintCollections,HistoryWritings,1.0,20.0,True,False,,
PaintCollections,HistoryWritings,1.0,20.0,False,True,Genuine,Report #2059
PaintCollections,HistoryWritings,1.0,20.0,False,True,Genuine,
PaintCollections,HistoryWritings,1.0,20.0,False,True,Fake,Report #2060
PaintCollections,HistoryWritings,1.0,20.0,False,True,Fake,
PaintCollections,HistoryWritings,1.0,20.0,False,True,,Report #2061
PaintCollections,HistoryWritings,1.0,20.0,False,True,,
PaintCollections,HistoryWritings,1.0,20.0,False,False,Genuine,Report #2062
PaintCollections,HistoryWritings,1.0,20.0,False,False,Genuine,
PaintCollections,HistoryWritings,1.0,20.0,False,False,Fake,Report #2063
PaintCollections,HistoryWritings,1.0,20.0,False,False,Fake,
PaintCollections,HistoryWritings,1.0,20.0,False,False,,Report #2064
PaintCollections,HistoryWritings,1.0,20.0,False,False,,
PaintCollections,HistoryWritings,1.0,21.0,True,True,Genuine,Report #2065
PaintCollections,HistoryWritings,1.0,21.0,True,True,Genuine,
PaintCollections,HistoryWritings,1.0,21.0,True,True,Fake,Report #2066
PaintCollections,HistoryWritings,1.0,21.0,True,True,Fake,
PaintCollections,HistoryWritings,1.0,21.0,True,True,,Report #2067
PaintCollections,HistoryWritings,1.0,21.0,True,True,,
PaintCollections,HistoryWritings,1.0,21.0,True,False,Genuine,Report #2068
PaintCollections,HistoryWritings,1.0,21.0,True,False,Genuine,
PaintCollections,HistoryWritings,1.0,21.0,True,False,Fake,Report #2069
PaintCollections,HistoryWritings,1.0,21.0,True,False,Fake,
PaintCollections,HistoryWritings,1.0,21.0,True,False,,Report #2070
PaintCollections,HistoryWritings,1.0,21.0,True,False,,
PaintCollections,HistoryWritings,1.0,21.0,False,True,Genuine,Report #2071
PaintCollections,HistoryWritings,1.0,21.0,False,True,Genuine,
PaintCollections,HistoryWritings,1.0,21.0,False,True,Fake,Report #2072
PaintCollections,HistoryWritings,1.0,21.0,False,True,Fake,
PaintCollections,HistoryWritings,1.0,21.0,False,True,,Report #2073
PaintCollections,HistoryWritings,1.0,21.0,False,True,,
PaintCollections,HistoryWritings,1.0,21.0,False,False,Genuine,Report #2074
PaintCollections,HistoryWritings,1.0,21.0,False,False,Genuine,
PaintCollections,HistoryWritings,1.0,21.0,False,False,Fake,Report #2075
PaintCollections,HistoryWritings,1.0,21.0,False,False,Fake,
PaintCollections,HistoryWritings,1.0,21.0,False,False,,Report #2076
PaintCollections,HistoryWritings,1.0,21.0,False,False,,
PaintCollections,HistoryWritings,1.0,99.0,True,True,Genuine,Report #2077
PaintCollections,HistoryWritings,1.0,99.0,True,True,Genuine,
PaintCollections,HistoryWritings,1.0,99.0,True,True,Fake,Report #2078
PaintCollections,HistoryWritings,1.0,99.0,True,True,Fake,
PaintCollections,HistoryWritings,1.0,99.0,True,True,,Report #2079
PaintCollections,HistoryWritings,1.0,99.0,True,True,,
PaintCollections,HistoryWritings,1.0,99.0,True,False,Genuine,Report #2080
PaintCollections,HistoryWritings,1.0,99.0,True,False,Genuine,
PaintCollections,HistoryWritings,1.0,99.0,True,False,Fake,Report #2081
PaintCollections,HistoryWritings,1.0,99.0,True,False,Fake,
PaintCollections,HistoryWritings,1.0,99.0,True,False,,Report #2082
PaintCollections,HistoryWritings,1.0,99.0,True,False,,
PaintCollections,HistoryWritings,1.0,99.0,False,True,Genuine,Report #2083
PaintCollections,HistoryWritings,1.0,99.0,False,True,Genuine,
PaintCollections,HistoryWritings,1.0,99.0,False,True,Fake,Report #2084
PaintCollections,HistoryWritings,1.0,99.0,False,True,Fake,
PaintCollections,HistoryWritings,1.0,99.0,False,True,,Report #2085
PaintCollections,HistoryWritings,1.0,99.0,False,True,,
PaintCollections,HistoryWritings,1.0,99.0,False,False,Genuine,Report #2086
PaintCollections,HistoryWritings,1.0,99.0,False,False,Genuine,
PaintCollections,HistoryWritings,1.0,99.0,False,False,Fake,Report #2087
PaintCollections,HistoryWritings,1.0,99.0,False,False,Fake,
PaintCollections,HistoryWritings,1.0,99.0,False,False,,Report #2088
PaintCollections,HistoryWritings,1.0,99.0,False,False,,
PaintCollections,HistoryWritings,1.0,100.0,True,True,Genuine,Report #2089
PaintCollections,HistoryWritings,1.0,100.0,True,True,Genuine,
PaintCollections,HistoryWritings,1.0,100.0,True,True,Fake,Report #2090
PaintCollections,HistoryWritings,1.0,100.0,True,True,Fake,
PaintCollections,HistoryWritings,1.0,100.0,True,True,,Report #2091
PaintCollections,HistoryWritings,1.0,100.0,True,True,,
PaintCollections,HistoryWritings,1.0,100.0,True,False,Genuine,Report #2092
PaintCollections,HistoryWritings,1.0,100.0,True,False,Genuine,
PaintCollections,HistoryWritings,1.0,100.0,True,False,Fake,Report #2093
PaintCollections,HistoryWritings,1.0,100.0,True,False,Fake,
PaintCollections,HistoryWritings,1.0,100.0,True,False,,Report #2094
PaintCollections,HistoryWritings,1.0,100.0,True,False,,
PaintCollections,HistoryWritings,1.0,100.0,False,True,Genuine,Report #2095
PaintCollections,HistoryWritings,1.0,100.0,False,True,Genuine,
PaintCollections,HistoryWritings,1.0,100.0,False,True,Fake,Report #2096
PaintCollections,HistoryWritings,1.0,100.0,False,True,Fake,
PaintCollections,HistoryWritings,1.0,100.0,False,True,,Report #2097
PaintCollections,HistoryWritings,1.0,100.0,False,True,,
PaintCollections,HistoryWritings,1.0,100.0,False,False,Genuine,Report #2098
PaintCollections,HistoryWritings,1.0,100.0,False,False,Genuine,
PaintCollections,HistoryWritings,1.0,100.0,False,False,Fake,Report #2099
PaintCollections,HistoryWritings,1.0,100.0,False,False,Fake,
PaintCollections,HistoryWritings,1.0,100.0,False,False,,Report #2100
PaintCollections,HistoryWritings,1.0,100.0,False,False,,
PaintCollections,HistoryWritings,1.0,101.0,True,True,Genuine,Report #2101
PaintCollections,HistoryWritings,1.0,101.0,True,True,Genuine,
PaintCollections,HistoryWritings,1.0,101.0,True,True,Fake,Report #2102
PaintCollections,HistoryWritings,1.0,101.0,True,True,Fake,
PaintCollections,HistoryWritings,1.0,101.0,True,True,,Report #2103
PaintCollections,HistoryWritings,1.0,101.0,True,True,,
PaintCollections,HistoryWritings,1.0,101.0,True,False,Genuine,Report #2104
PaintCollections,HistoryWritings,1.0,101.0,True,False,Genuine,
PaintCollections,HistoryWritings,1.0,101.0,True,False,Fake,Report #2105
PaintCollections,HistoryWritings,1.0,101.0,True,False,Fake,
PaintCollections,HistoryWritings,1.0,101.0,True,False,,Report #2106
PaintCollections,HistoryWritings,1.0,101.0,True,False,,
PaintCollections,HistoryWritings,1.0,101.0,False,True,Genuine,Report #2107
PaintCollections,HistoryWritings,1.0,101.0,False,True,Genuine,
PaintCollections,HistoryWritings,1.0,101.0,False,True,Fake,Report #2108
PaintCollections,HistoryWritings,1.0,101.0,False,True,Fake,
PaintCollections,HistoryWritings,1.0,101.0,False,True,,Report #2109
PaintCollections,HistoryWritings,1.0,101.0,False,True,,
PaintCollections,HistoryWritings,1.0,101.0,False,False,Genuine,Report #2110
PaintCollections,HistoryWritings,1.0,101.0,False,False,Genuine,
PaintCollections,HistoryWritings,1.0,101.0,False,False,Fake,Report #2111
PaintCollections,HistoryWritings,1.0,101.0,False,False,Fake,
PaintCollections,HistoryWritings,1.0,101.0,False,False,,Report #2112
PaintCollections,HistoryWritings,1.0,101.0,False,False,,
PaintCollections,HistoryWritings,100.0,-1.0,True,True,Genuine,Report #2113
PaintCollections,HistoryWritings,100.0,-1.0,True,True,Genuine,
PaintCollections,HistoryWritings,100.0,-1.0,True,True,Fake,Report #2114
PaintCollections,HistoryWritings,100.0,-1.0,True,True,Fake,
PaintCollections,HistoryWritings,100.0,-1.0,True,True,,Report #2115
PaintCollections,HistoryWritings,100.0,-1.0,True,True,,
PaintCollections,HistoryWritings,100.0,-1.0,True,False,Genuine,Report #2116
PaintCollections,HistoryWritings,100.0,-1.0,True,False,Genuine,
PaintCollections,HistoryWritings,100.0,-1.0,True,False,Fake,Report #2117
PaintCollections,HistoryWritings,100.0,-1.0,True,False,Fake,
PaintCollections,HistoryWritings,100.0,-1.0,True,False,,Report #2118
PaintCollections,HistoryWritings,100.0,-1.0,True,False,,
PaintCollections,HistoryWritings,100.0,-1.0,False,True,Genuine,Report #2119
PaintCollections,HistoryWritings,100.0,-1.0,False,True,Genuine,
PaintCollections,HistoryWritings,100.0,-1.0,False,True,Fake,Report #2120
PaintCollections,HistoryWritings,100.0,-1.0,False,True,Fake,
PaintCollections,HistoryWritings,100.0,-1.0,False,True,,Report #2121
PaintCollections,HistoryWritings,100.0,-1.0,False,True,,
PaintCollections,HistoryWritings,100.0,-1.0,False,False,Genuine,Report #2122
PaintCollections,HistoryWritings,100.0,-1.0,False,False,Genuine,
PaintCollections,HistoryWritings,100.0,-1.0,False,False,Fake,Report #2123
PaintCollections,HistoryWritings,100.0,-1.0,False,False,Fake,
PaintCollections,HistoryWritings,100.0,-1.0,False,False,,Report #2124
PaintCollections,HistoryWritings,100.0,-1.0,False,False,,
PaintCollections,HistoryWritings,100.0,0.0,True,True,Genuine,Report #2125
PaintCollections,HistoryWritings,100.0,0.0,True,True,Genuine,
PaintCollections,HistoryWritings,100.0,0.0,True,True,Fake,Report #2126
PaintCollections,HistoryWritings,100.0,0.0,True,True,Fake,
PaintCollections,HistoryWritings,100.0,0.0,True,True,,Report #2127
PaintCollections,HistoryWritings,100.0,0.0,True,True,,
PaintCollections,HistoryWritings,100.0,0.0,True,False,Genuine,Report #2128
PaintCollections,HistoryWritings,100.0,0.0,True,False,Genuine,
PaintCollections,HistoryWritings,100.0,0.0,True,False,Fake,Report #2129
PaintCollections,HistoryWritings,100.0,0.0,True,False,Fake,
PaintCollections,HistoryWritings,100.0,0.0,True,False,,Report #2130
PaintCollections,HistoryWritings,100.0,0.0,True,False,,
PaintCollections,HistoryWritings,100.0,0.0,False,True,Genuine,Report #2131
PaintCollections,HistoryWritings,100.0,0.0,False,True,Genuine,
PaintCollections,HistoryWritings,100.0,0.0,False,True,Fake,Report #2132
PaintCollections,HistoryWritings,100.0,0.0,False,True,Fake,
PaintCollections,HistoryWritings,100.0,0.0,False,True,,Report #2133
PaintCollections,HistoryWritings,100.0,0.0,False,True,,
PaintCollections,HistoryWritings,100.0,0.0,False,False,Genuine,Report #2134
PaintCollections,HistoryWritings,100.0,0.0,False,False,Genuine,
PaintCollections,HistoryWritings,100.0,0.0,False,False,Fake,Report #2135
PaintCollections,HistoryWritings,100.0,0.0,False,False,Fake,
PaintCollections,HistoryWritings,100.0,0.0,False,False,,Report #2136
PaintCollections,HistoryWritings,100.0,0.0,False,False,,
PaintCollections,HistoryWritings,100.0,19.0,True,True,Genuine,Report #2137
PaintCollections,HistoryWritings,100.0,19.0,True,True,Genuine,
PaintCollections,HistoryWritings,100.0,19.0,True,True,Fake,Report #2138
PaintCollections,HistoryWritings,100.0,19.0,True,True,Fake,
PaintCollections,HistoryWritings,100.0,19.0,True,True,,Report #2139
PaintCollections,HistoryWritings,100.0,19.0,True,True,,
PaintCollections,HistoryWritings,100.0,19.0,True,False,Genuine,Report #2140
PaintCollections,HistoryWritings,100.0,19.0,True,False,Genuine,
PaintCollections,HistoryWritings,100.0,19.0,True,False,Fake,Report #2141
PaintCollections,HistoryWritings,100.0,19.0,True,False,Fake,
PaintCollections,HistoryWritings,100.0,19.0,True,False,,Report #2142
PaintCollections,HistoryWritings,100.0,19.0,True,False,,
PaintCollections,HistoryWritings,100.0,19.0,False,True,Genuine,Report #2143
PaintCollections,HistoryWritings,100.0,19.0,False,True,Genuine,
PaintCollections,HistoryWritings,100.0,19.0,False,True,Fake,Report #2144
PaintCollections,HistoryWritings,100.0,19.0,False,True,Fake,
PaintCollections,HistoryWritings,100.0,19.0,False,True,,Report #2145
PaintCollections,HistoryWritings,100.0,19.0,False,True,,
PaintCollections,HistoryWritings,100.0,19.0,False,False,Genuine,Report #2146
PaintCollections,HistoryWritings,100.0,19.0,False,False,Genuine,
PaintCollections,HistoryWritings,100.0,19.0,False,False,Fake,Report #2147
PaintCollections,HistoryWritings,100.0,19.0,False,False,Fake,
PaintCollections,HistoryWritings,100.0,19.0,False,False,,Report #2148
PaintCollections,HistoryWritings,100.0,19.0,False,False,,
PaintCollections,HistoryWritings,100.0,20.0,True,True,Genuine,Report #2149
PaintCollections,HistoryWritings,100.0,20.0,True,True,Genuine,
PaintCollections,HistoryWritings,100.0,20.0,True,True,Fake,Report #2150
PaintCollections,HistoryWritings,100.0,20.0,True,True,Fake,
PaintCollections,HistoryWritings,100.0,20.0,True,True,,Report #2151
PaintCollections,HistoryWritings,100.0,20.0,True,True,,
PaintCollections,HistoryWritings,100.0,20.0,True,False,Genuine,Report #2152
PaintCollections,HistoryWritings,100.0,20.0,True,False,Genuine,
PaintCollections,HistoryWritings,100.0,20.0,True,False,Fake,Report #2153
PaintCollections,HistoryWritings,100.0,20.0,True,False,Fake,
PaintCollections,HistoryWritings,100.0,20.0,True,False,,Report #2154
PaintCollections,HistoryWritings,100.0,20.0,True,False,,
PaintCollections,HistoryWritings,100.0,20.0,False,True,Genuine,Report #2155
PaintCollections,HistoryWritings,100.0,20.0,False,True,Genuine,
PaintCollections,HistoryWritings,100.0,20.0,False,True,Fake,Report #2156
PaintCollections,HistoryWritings,100.0,20.0,False,True,Fake,
PaintCollections,HistoryWritings,100.0,20.0,False,True,,Report #2157
PaintCollections,HistoryWritings,100.0,20.0,False,True,,
PaintCollections,HistoryWritings,100.0,20.0,False,False,Genuine,Report #2158
PaintCollections,HistoryWritings,100.0,20.0,False,False,Genuine,
PaintCollections,HistoryWritings,100.0,20.0,False,False,Fake,Report #2159
PaintCollections,HistoryWritings,100.0,20.0,False,False,Fake,
PaintCollections,HistoryWritings,100.0,20.0,False,False,,Report #2160
PaintCollections,HistoryWritings,100.0,20.0,False,False,,
PaintCollections,HistoryWritings,100.0,21.0,True,True,Genuine,Report #2161
PaintCollections,HistoryWritings,100.0,21.0,True,True,Genuine,
PaintCollections,HistoryWritings,100.0,21.0,True,True,Fake,Report #2162
PaintCollections,HistoryWritings,100.0,21.0,True,True,Fake,
PaintCollections,HistoryWritings,100.0,21.0,True,True,,Report #2163
PaintCollections,HistoryWritings,100.0,21.0,True,True,,
PaintCollections,HistoryWritings,100.0,21.0,True,False,Genuine,Report #2164
PaintCollections,HistoryWritings,100.0,21.0,True,False,Genuine,
PaintCollections,HistoryWritings,100.0,21.0,True,False,Fake,Report #2165
PaintCollections,HistoryWritings,100.0,21.0,True,False,Fake,
PaintCollections,HistoryWritings,100.0,21.0,True,False,,Report #2166
PaintCollections,HistoryWritings,100.0,21.0,True,False,,
PaintCollections,HistoryWritings,100.0,21.0,False,True,Genuine,Report #2167
PaintCollections,HistoryWritings,100.0,21.0,False,True,Genuine,
PaintCollections,HistoryWritings,100.0,21.0,False,True,Fake,Report #2168
PaintCollections,HistoryWritings,100.0,21.0,False,True,Fake,
PaintCollections,HistoryWritings,100.0,21.0,False,True,,Report #2169
PaintCollections,HistoryWritings,100.0,21.0,False,True,,
PaintCollections,HistoryWritings,100.0,21.0,False,False,Genuine,Report #2170
PaintCollections,HistoryWritings,100.0,21.0,False,False,Genuine,
PaintCollections,HistoryWritings,100.0,21.0,False,False,Fake,Report #2171
PaintCollections,HistoryWritings,100.0,21.0,False,False,Fake,
PaintCollections,HistoryWritings,100.0,21.0,False,False,,Report #2172
PaintCollections,HistoryWritings,100.0,21.0,False,False,,
PaintCollections,HistoryWritings,100.0,99.0,True,True,Genuine,Report #2173
PaintCollections,HistoryWritings,100.0,99.0,True,True,Genuine,
PaintCollections,HistoryWritings,100.0,99.0,True,True,Fake,Report #2174
PaintCollections,HistoryWritings,100.0,99.0,True,True,Fake,
PaintCollections,HistoryWritings,100.0,99.0,True,True,,Report #2175
PaintCollections,HistoryWritings,100.0,99.0,True,True,,
PaintCollections,HistoryWritings,100.0,99.0,True,False,Genuine,Report #2176
PaintCollections,HistoryWritings,100.0,99.0,True,False,Genuine,
PaintCollections,HistoryWritings,100.0,99.0,True,False,Fake,Report #2177
PaintCollections,HistoryWritings,100.0,99.0,True,False,Fake,
PaintCollections,HistoryWritings,100.0,99.0,True,False,,Report #2178
PaintCollections,HistoryWritings,100.0,99.0,True,False,,
PaintCollections,HistoryWritings,100.0,99.0,False,True,Genuine,Report #2179
PaintCollections,HistoryWritings,100.0,99.0,False,True,Genuine,
PaintCollections,HistoryWritings,100.0,99.0,False,True,Fake,Report #2180
PaintCollections,HistoryWritings,100.0,99.0,False,True,Fake,
PaintCollections,HistoryWritings,100.0,99.0,False,True,,Report #2181
PaintCollections,HistoryWritings,100.0,99.0,False,True,,
PaintCollections,HistoryWritings,100.0,99.0,False,False,Genuine,Report #2182
PaintCollections,HistoryWritings,100.0,99.0,False,False,Genuine,
PaintCollections,HistoryWritings,100.0,99.0,False,False,Fake,Report #2183
PaintCollections,HistoryWritings,100.0,99.0,False,False,Fake,
PaintCollections,HistoryWritings,100.0,99.0,False,False,,Report #2184
PaintCollections,HistoryWritings,100.0,99.0,False,False,,
PaintCollections,HistoryWritings,100.0,100.0,True,True,Genuine,Report #2185
PaintCollections,HistoryWritings,100.0,100.0,True,True,Genuine,
PaintCollections,HistoryWritings,100.0,100.0,True,True,Fake,Report #2186
PaintCollections,HistoryWritings,100.0,100.0,True,True,Fake,
PaintCollections,HistoryWritings,100.0,100.0,True,True,,Report #2187
PaintCollections,HistoryWritings,100.0,100.0,True,True,,
PaintCollections,HistoryWritings,100.0,100.0,True,False,Genuine,Report #2188
PaintCollections,HistoryWritings,100.0,100.0,True,False,Genuine,
PaintCollections,HistoryWritings,100.0,100.0,True,False,Fake,Report #2189
PaintCollections,HistoryWritings,100.0,100.0,True,False,Fake,
PaintCollections,HistoryWritings,100.0,100.0,True,False,,Report #2190
PaintCollections,HistoryWritings,100.0,100.0,True,False,,
PaintCollections,HistoryWritings,100.0,100.0,False,True,Genuine,Report #2191
PaintCollections,HistoryWritings,100.0,100.0,False,True,Genuine,
PaintCollections,HistoryWritings,100.0,100.0,False,True,Fake,Report #2192
PaintCollections,HistoryWritings,100.0,100.0,False,True,Fake,
PaintCollections,HistoryWritings,100.0,100.0,False,True,,Report #2193
PaintCollections,HistoryWritings,100.0,100.0,False,True,,
PaintCollections,HistoryWritings,100.0,100.0,False,False,Genuine,Report #2194
PaintCollections,HistoryWritings,100.0,100.0,False,False,Genuine,
PaintCollections,HistoryWritings,100.0,100.0,False,False,Fake,Report #2195
PaintCollections,HistoryWritings,100.0,100.0,False,False,Fake,
PaintCollections,HistoryWritings,100.0,100.0,False,False,,Report #2196
PaintCollections,HistoryWritings,100.0,100.0,False,False,,
PaintCollections,HistoryWritings,100.0,101.0,True,True,Genuine,Report #2197
PaintCollections,HistoryWritings,100.0,101.0,True,True,Genuine,
PaintCollections,HistoryWritings,100.0,101.0,True,True,Fake,Report #2198
PaintCollections,HistoryWritings,100.0,101.0,True,True,Fake,
PaintCollections,HistoryWritings,100.0,101.0,True,True,,Report #2199
PaintCollections,HistoryWritings,100.0,101.0,True,True,,
PaintCollections,HistoryWritings,100.0,101.0,True,False,Genuine,Report #2200
PaintCollections,HistoryWritings,100.0,101.0,True,False,Genuine,
PaintCollections,HistoryWritings,100.0,101.0,True,False,Fake,Report #2201
PaintCollections,HistoryWritings,100.0,101.0,True,False,Fake,
PaintCollections,HistoryWritings,100.0,101.0,True,False,,Report #2202
PaintCollections,HistoryWritings,100.0,101.0,True,False,,
PaintCollections,HistoryWritings,100.0,101.0,False,True,Genuine,Report #2203
PaintCollections,HistoryWritings,100.0,101.0,False,True,Genuine,
PaintCollections,HistoryWritings,100.0,101.0,False,True,Fake,Report #2204
PaintCollections,HistoryWritings,100.0,101.0,False,True,Fake,
PaintCollections,HistoryWritings,100.0,101.0,False,True,,Report #2205
PaintCollections,HistoryWritings,100.0,101.0,False,True,,
PaintCollections,HistoryWritings,100.0,101.0,False,False,Genuine,Report #2206
PaintCollections,HistoryWritings,100.0,101.0,False,False,Genuine,
PaintCollections,HistoryWritings,100.0,101.0,False,False,Fake,Report #2207
PaintCollections,HistoryWritings,100.0,101.0,False,False,Fake,
PaintCollections,HistoryWritings,100.0,101.0,False,False,,Report #2208
PaintCollections,HistoryWritings,100.0,101.0,False,False,,
PaintCollections,HistoryWritings,1000.0,-1.0,True,True,Genuine,Report #2209
PaintCollections,HistoryWritings,1000.0,-1.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000.0,-1.0,True,True,Fake,Report #2210
PaintCollections,HistoryWritings,1000.0,-1.0,True,True,Fake,
PaintCollections,HistoryWritings,1000.0,-1.0,True,True,,Report #2211
PaintCollections,HistoryWritings,1000.0,-1.0,True,True,,
PaintCollections,HistoryWritings,1000.0,-1.0,True,False,Genuine,Report #2212
PaintCollections,HistoryWritings,1000.0,-1.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000.0,-1.0,True,False,Fake,Report #2213
PaintCollections,HistoryWritings,1000.0,-1.0,True,False,Fake,
PaintCollections,HistoryWritings,1000.0,-1.0,True,False,,Report #2214
PaintCollections,HistoryWritings,1000.0,-1.0,True,False,,
PaintCollections,HistoryWritings,1000.0,-1.0,False,True,Genuine,Report #2215
PaintCollections,HistoryWritings,1000.0,-1.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000.0,-1.0,False,True,Fake,Report #2216
PaintCollections,HistoryWritings,1000.0,-1.0,False,True,Fake,
PaintCollections,HistoryWritings,1000.0,-1.0,False,True,,Report #2217
PaintCollections,HistoryWritings,1000.0,-1.0,False,True,,
PaintCollections,HistoryWritings,1000.0,-1.0,False,False,Genuine,Report #2218
PaintCollections,HistoryWritings,1000.0,-1.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000.0,-1.0,False,False,Fake,Report #2219
PaintCollections,HistoryWritings,1000.0,-1.0,False,False,Fake,
PaintCollections,HistoryWritings,1000.0,-1.0,False,False,,Report #2220
PaintCollections,HistoryWritings,1000.0,-1.0,False,False,,
PaintCollections,HistoryWritings,1000.0,0.0,True,True,Genuine,Report #2221
PaintCollections,HistoryWritings,1000.0,0.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000.0,0.0,True,True,Fake,Report #2222
PaintCollections,HistoryWritings,1000.0,0.0,True,True,Fake,
PaintCollections,HistoryWritings,1000.0,0.0,True,True,,Report #2223
PaintCollections,HistoryWritings,1000.0,0.0,True,True,,
PaintCollections,HistoryWritings,1000.0,0.0,True,False,Genuine,Report #2224
PaintCollections,HistoryWritings,1000.0,0.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000.0,0.0,True,False,Fake,Report #2225
PaintCollections,HistoryWritings,1000.0,0.0,True,False,Fake,
PaintCollections,HistoryWritings,1000.0,0.0,True,False,,Report #2226
PaintCollections,HistoryWritings,1000.0,0.0,True,False,,
PaintCollections,HistoryWritings,1000.0,0.0,False,True,Genuine,Report #2227
PaintCollections,HistoryWritings,1000.0,0.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000.0,0.0,False,True,Fake,Report #2228
PaintCollections,HistoryWritings,1000.0,0.0,False,True,Fake,
PaintCollections,HistoryWritings,1000.0,0.0,False,True,,Report #2229
PaintCollections,HistoryWritings,1000.0,0.0,False,True,,
PaintCollections,HistoryWritings,1000.0,0.0,False,False,Genuine,Report #2230
PaintCollections,HistoryWritings,1000.0,0.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000.0,0.0,False,False,Fake,Report #2231
PaintCollections,HistoryWritings,1000.0,0.0,False,False,Fake,
PaintCollections,HistoryWritings,1000.0,0.0,False,False,,Report #2232
PaintCollections,HistoryWritings,1000.0,0.0,False,False,,
PaintCollections,HistoryWritings,1000.0,19.0,True,True,Genuine,Report #2233
PaintCollections,HistoryWritings,1000.0,19.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000.0,19.0,True,True,Fake,Report #2234
PaintCollections,HistoryWritings,1000.0,19.0,True,True,Fake,
PaintCollections,HistoryWritings,1000.0,19.0,True,True,,Report #2235
PaintCollections,HistoryWritings,1000.0,19.0,True,True,,
PaintCollections,HistoryWritings,1000.0,19.0,True,False,Genuine,Report #2236
PaintCollections,HistoryWritings,1000.0,19.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000.0,19.0,True,False,Fake,Report #2237
PaintCollections,HistoryWritings,1000.0,19.0,True,False,Fake,
PaintCollections,HistoryWritings,1000.0,19.0,True,False,,Report #2238
PaintCollections,HistoryWritings,1000.0,19.0,True,False,,
PaintCollections,HistoryWritings,1000.0,19.0,False,True,Genuine,Report #2239
PaintCollections,HistoryWritings,1000.0,19.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000.0,19.0,False,True,Fake,Report #2240
PaintCollections,HistoryWritings,1000.0,19.0,False,True,Fake,
PaintCollections,HistoryWritings,1000.0,19.0,False,True,,Report #2241
PaintCollections,HistoryWritings,1000.0,19.0,False,True,,
PaintCollections,HistoryWritings,1000.0,19.0,False,False,Genuine,Report #2242
PaintCollections,HistoryWritings,1000.0,19.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000.0,19.0,False,False,Fake,Report #2243
PaintCollections,HistoryWritings,1000.0,19.0,False,False,Fake,
PaintCollections,HistoryWritings,1000.0,19.0,False,False,,Report #2244
PaintCollections,HistoryWritings,1000.0,19.0,False,False,,
PaintCollections,HistoryWritings,1000.0,20.0,True,True,Genuine,Report #2245
PaintCollections,HistoryWritings,1000.0,20.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000.0,20.0,True,True,Fake,Report #2246
PaintCollections,HistoryWritings,1000.0,20.0,True,True,Fake,
PaintCollections,HistoryWritings,1000.0,20.0,True,True,,Report #2247
PaintCollections,HistoryWritings,1000.0,20.0,True,True,,
PaintCollections,HistoryWritings,1000.0,20.0,True,False,Genuine,Report #2248
PaintCollections,HistoryWritings,1000.0,20.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000.0,20.0,True,False,Fake,Report #2249
PaintCollections,HistoryWritings,1000.0,20.0,True,False,Fake,
PaintCollections,HistoryWritings,1000.0,20.0,True,False,,Report #2250
PaintCollections,HistoryWritings,1000.0,20.0,True,False,,
PaintCollections,HistoryWritings,1000.0,20.0,False,True,Genuine,Report #2251
PaintCollections,HistoryWritings,1000.0,20.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000.0,20.0,False,True,Fake,Report #2252
PaintCollections,HistoryWritings,1000.0,20.0,False,True,Fake,
PaintCollections,HistoryWritings,1000.0,20.0,False,True,,Report #2253
PaintCollections,HistoryWritings,1000.0,20.0,False,True,,
PaintCollections,HistoryWritings,1000.0,20.0,False,False,Genuine,Report #2254
PaintCollections,HistoryWritings,1000.0,20.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000.0,20.0,False,False,Fake,Report #2255
PaintCollections,HistoryWritings,1000.0,20.0,False,False,Fake,
PaintCollections,HistoryWritings,1000.0,20.0,False,False,,Report #2256
PaintCollections,HistoryWritings,1000.0,20.0,False,False,,
PaintCollections,HistoryWritings,1000.0,21.0,True,True,Genuine,Report #2257
PaintCollections,HistoryWritings,1000.0,21.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000.0,21.0,True,True,Fake,Report #2258
PaintCollections,HistoryWritings,1000.0,21.0,True,True,Fake,
PaintCollections,HistoryWritings,1000.0,21.0,True,True,,Report #2259
PaintCollections,HistoryWritings,1000.0,21.0,True,True,,
PaintCollections,HistoryWritings,1000.0,21.0,True,False,Genuine,Report #2260
PaintCollections,HistoryWritings,1000.0,21.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000.0,21.0,True,False,Fake,Report #2261
PaintCollections,HistoryWritings,1000.0,21.0,True,False,Fake,
PaintCollections,HistoryWritings,1000.0,21.0,True,False,,Report #2262
PaintCollections,HistoryWritings,1000.0,21.0,True,False,,
PaintCollections,HistoryWritings,1000.0,21.0,False,True,Genuine,Report #2263
PaintCollections,HistoryWritings,1000.0,21.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000.0,21.0,False,True,Fake,Report #2264
PaintCollections,HistoryWritings,1000.0,21.0,False,True,Fake,
PaintCollections,HistoryWritings,1000.0,21.0,False,True,,Report #2265
PaintCollections,HistoryWritings,1000.0,21.0,False,True,,
PaintCollections,HistoryWritings,1000.0,21.0,False,False,Genuine,Report #2266
PaintCollections,HistoryWritings,1000.0,21.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000.0,21.0,False,False,Fake,Report #2267
PaintCollections,HistoryWritings,1000.0,21.0,False,False,Fake,
PaintCollections,HistoryWritings,1000.0,21.0,False,False,,Report #2268
PaintCollections,HistoryWritings,1000.0,21.0,False,False,,
PaintCollections,HistoryWritings,1000.0,99.0,True,True,Genuine,Report #2269
PaintCollections,HistoryWritings,1000.0,99.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000.0,99.0,True,True,Fake,Report #2270
PaintCollections,HistoryWritings,1000.0,99.0,True,True,Fake,
PaintCollections,HistoryWritings,1000.0,99.0,True,True,,Report #2271
PaintCollections,HistoryWritings,1000.0,99.0,True,True,,
PaintCollections,HistoryWritings,1000.0,99.0,True,False,Genuine,Report #2272
PaintCollections,HistoryWritings,1000.0,99.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000.0,99.0,True,False,Fake,Report #2273
PaintCollections,HistoryWritings,1000.0,99.0,True,False,Fake,
PaintCollections,HistoryWritings,1000.0,99.0,True,False,,Report #2274
PaintCollections,HistoryWritings,1000.0,99.0,True,False,,
PaintCollections,HistoryWritings,1000.0,99.0,False,True,Genuine,Report #2275
PaintCollections,HistoryWritings,1000.0,99.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000.0,99.0,False,True,Fake,Report #2276
PaintCollections,HistoryWritings,1000.0,99.0,False,True,Fake,
PaintCollections,HistoryWritings,1000.0,99.0,False,True,,Report #2277
PaintCollections,HistoryWritings,1000.0,99.0,False,True,,
PaintCollections,HistoryWritings,1000.0,99.0,False,False,Genuine,Report #2278
PaintCollections,HistoryWritings,1000.0,99.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000.0,99.0,False,False,Fake,Report #2279
PaintCollections,HistoryWritings,1000.0,99.0,False,False,Fake,
PaintCollections,HistoryWritings,1000.0,99.0,False,False,,Report #2280
PaintCollections,HistoryWritings,1000.0,99.0,False,False,,
PaintCollections,HistoryWritings,1000.0,100.0,True,True,Genuine,Report #2281
PaintCollections,HistoryWritings,1000.0,100.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000.0,100.0,True,True,Fake,Report #2282
PaintCollections,HistoryWritings,1000.0,100.0,True,True,Fake,
PaintCollections,HistoryWritings,1000.0,100.0,True,True,,Report #2283
PaintCollections,HistoryWritings,1000.0,100.0,True,True,,
PaintCollections,HistoryWritings,1000.0,100.0,True,False,Genuine,Report #2284
PaintCollections,HistoryWritings,1000.0,100.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000.0,100.0,True,False,Fake,Report #2285
PaintCollections,HistoryWritings,1000.0,100.0,True,False,Fake,
PaintCollections,HistoryWritings,1000.0,100.0,True,False,,Report #2286
PaintCollections,HistoryWritings,1000.0,100.0,True,False,,
PaintCollections,HistoryWritings,1000.0,100.0,False,True,Genuine,Report #2287
PaintCollections,HistoryWritings,1000.0,100.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000.0,100.0,False,True,Fake,Report #2288
PaintCollections,HistoryWritings,1000.0,100.0,False,True,Fake,
PaintCollections,HistoryWritings,1000.0,100.0,False,True,,Report #2289
PaintCollections,HistoryWritings,1000.0,100.0,False,True,,
PaintCollections,HistoryWritings,1000.0,100.0,False,False,Genuine,Report #2290
PaintCollections,HistoryWritings,1000.0,100.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000.0,100.0,False,False,Fake,Report #2291
PaintCollections,HistoryWritings,1000.0,100.0,False,False,Fake,
PaintCollections,HistoryWritings,1000.0,100.0,False,False,,Report #2292
PaintCollections,HistoryWritings,1000.0,100.0,False,False,,
PaintCollections,HistoryWritings,1000.0,101.0,True,True,Genuine,Report #2293
PaintCollections,HistoryWritings,1000.0,101.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000.0,101.0,True,True,Fake,Report #2294
PaintCollections,HistoryWritings,1000.0,101.0,True,True,Fake,
PaintCollections,HistoryWritings,1000.0,101.0,True,True,,Report #2295
PaintCollections,HistoryWritings,1000.0,101.0,True,True,,
PaintCollections,HistoryWritings,1000.0,101.0,True,False,Genuine,Report #2296
PaintCollections,HistoryWritings,1000.0,101.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000.0,101.0,True,False,Fake,Report #2297
PaintCollections,HistoryWritings,1000.0,101.0,True,False,Fake,
PaintCollections,HistoryWritings,1000.0,101.0,True,False,,Report #2298
PaintCollections,HistoryWritings,1000.0,101.0,True,False,,
PaintCollections,HistoryWritings,1000.0,101.0,False,True,Genuine,Report #2299
PaintCollections,HistoryWritings,1000.0,101.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000.0,101.0,False,True,Fake,Report #2300
PaintCollections,HistoryWritings,1000.0,101.0,False,True,Fake,
PaintCollections,HistoryWritings,1000.0,101.0,False,True,,Report #2301
PaintCollections,HistoryWritings,1000.0,101.0,False,True,,
PaintCollections,HistoryWritings,1000.0,101.0,False,False,Genuine,Report #2302
PaintCollections,HistoryWritings,1000.0,101.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000.0,101.0,False,False,Fake,Report #2303
PaintCollections,HistoryWritings,1000.0,101.0,False,False,Fake,
PaintCollections,HistoryWritings,1000.0,101.0,False,False,,Report #2304
PaintCollections,HistoryWritings,1000.0,101.0,False,False,,
PaintCollections,HistoryWritings,1000000.0,-1.0,True,True,Genuine,Report #2305
PaintCollections,HistoryWritings,1000000.0,-1.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,-1.0,True,True,Fake,Report #2306
PaintCollections,HistoryWritings,1000000.0,-1.0,True,True,Fake,
PaintCollections,HistoryWritings,1000000.0,-1.0,True,True,,Report #2307
PaintCollections,HistoryWritings,1000000.0,-1.0,True,True,,
PaintCollections,HistoryWritings,1000000.0,-1.0,True,False,Genuine,Report #2308
PaintCollections,HistoryWritings,1000000.0,-1.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,-1.0,True,False,Fake,Report #2309
PaintCollections,HistoryWritings,1000000.0,-1.0,True,False,Fake,
PaintCollections,HistoryWritings,1000000.0,-1.0,True,False,,Report #2310
PaintCollections,HistoryWritings,1000000.0,-1.0,True,False,,
PaintCollections,HistoryWritings,1000000.0,-1.0,False,True,Genuine,Report #2311
PaintCollections,HistoryWritings,1000000.0,-1.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,-1.0,False,True,Fake,Report #2312
PaintCollections,HistoryWritings,1000000.0,-1.0,False,True,Fake,
PaintCollections,HistoryWritings,1000000.0,-1.0,False,True,,Report #2313
PaintCollections,HistoryWritings,1000000.0,-1.0,False,True,,
PaintCollections,HistoryWritings,1000000.0,-1.0,False,False,Genuine,Report #2314
PaintCollections,HistoryWritings,1000000.0,-1.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,-1.0,False,False,Fake,Report #2315
PaintCollections,HistoryWritings,1000000.0,-1.0,False,False,Fake,
PaintCollections,HistoryWritings,1000000.0,-1.0,False,False,,Report #2316
PaintCollections,HistoryWritings,1000000.0,-1.0,False,False,,
PaintCollections,HistoryWritings,1000000.0,0.0,True,True,Genuine,Report #2317
PaintCollections,HistoryWritings,1000000.0,0.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,0.0,True,True,Fake,Report #2318
PaintCollections,HistoryWritings,1000000.0,0.0,True,True,Fake,
PaintCollections,HistoryWritings,1000000.0,0.0,True,True,,Report #2319
PaintCollections,HistoryWritings,1000000.0,0.0,True,True,,
PaintCollections,HistoryWritings,1000000.0,0.0,True,False,Genuine,Report #2320
PaintCollections,HistoryWritings,1000000.0,0.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,0.0,True,False,Fake,Report #2321
PaintCollections,HistoryWritings,1000000.0,0.0,True,False,Fake,
PaintCollections,HistoryWritings,1000000.0,0.0,True,False,,Report #2322
PaintCollections,HistoryWritings,1000000.0,0.0,True,False,,
PaintCollections,HistoryWritings,1000000.0,0.0,False,True,Genuine,Report #2323
PaintCollections,HistoryWritings,1000000.0,0.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,0.0,False,True,Fake,Report #2324
PaintCollections,HistoryWritings,1000000.0,0.0,False,True,Fake,
PaintCollections,HistoryWritings,1000000.0,0.0,False,True,,Report #2325
PaintCollections,HistoryWritings,1000000.0,0.0,False,True,,
PaintCollections,HistoryWritings,1000000.0,0.0,False,False,Genuine,Report #2326
PaintCollections,HistoryWritings,1000000.0,0.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,0.0,False,False,Fake,Report #2327
PaintCollections,HistoryWritings,1000000.0,0.0,False,False,Fake,
PaintCollections,HistoryWritings,1000000.0,0.0,False,False,,Report #2328
PaintCollections,HistoryWritings,1000000.0,0.0,False,False,,
PaintCollections,HistoryWritings,1000000.0,19.0,True,True,Genuine,Report #2329
PaintCollections,HistoryWritings,1000000.0,19.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,19.0,True,True,Fake,Report #2330
PaintCollections,HistoryWritings,1000000.0,19.0,True,True,Fake,
PaintCollections,HistoryWritings,1000000.0,19.0,True,True,,Report #2331
PaintCollections,HistoryWritings,1000000.0,19.0,True,True,,
PaintCollections,HistoryWritings,1000000.0,19.0,True,False,Genuine,Report #2332
PaintCollections,HistoryWritings,1000000.0,19.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,19.0,True,False,Fake,Report #2333
PaintCollections,HistoryWritings,1000000.0,19.0,True,False,Fake,
PaintCollections,HistoryWritings,1000000.0,19.0,True,False,,Report #2334
PaintCollections,HistoryWritings,1000000.0,19.0,True,False,,
PaintCollections,HistoryWritings,1000000.0,19.0,False,True,Genuine,Report #2335
PaintCollections,HistoryWritings,1000000.0,19.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,19.0,False,True,Fake,Report #2336
PaintCollections,HistoryWritings,1000000.0,19.0,False,True,Fake,
PaintCollections,HistoryWritings,1000000.0,19.0,False,True,,Report #2337
PaintCollections,HistoryWritings,1000000.0,19.0,False,True,,
PaintCollections,HistoryWritings,1000000.0,19.0,False,False,Genuine,Report #2338
PaintCollections,HistoryWritings,1000000.0,19.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,19.0,False,False,Fake,Report #2339
PaintCollections,HistoryWritings,1000000.0,19.0,False,False,Fake,
PaintCollections,HistoryWritings,1000000.0,19.0,False,False,,Report #2340
PaintCollections,HistoryWritings,1000000.0,19.0,False,False,,
PaintCollections,HistoryWritings,1000000.0,20.0,True,True,Genuine,Report #2341
PaintCollections,HistoryWritings,1000000.0,20.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,20.0,True,True,Fake,Report #2342
PaintCollections,HistoryWritings,1000000.0,20.0,True,True,Fake,
PaintCollections,HistoryWritings,1000000.0,20.0,True,True,,Report #2343
PaintCollections,HistoryWritings,1000000.0,20.0,True,True,,
PaintCollections,HistoryWritings,1000000.0,20.0,True,False,Genuine,Report #2344
PaintCollections,HistoryWritings,1000000.0,20.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,20.0,True,False,Fake,Report #2345
PaintCollections,HistoryWritings,1000000.0,20.0,True,False,Fake,
PaintCollections,HistoryWritings,1000000.0,20.0,True,False,,Report #2346
PaintCollections,HistoryWritings,1000000.0,20.0,True,False,,
PaintCollections,HistoryWritings,1000000.0,20.0,False,True,Genuine,Report #2347
PaintCollections,HistoryWritings,1000000.0,20.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,20.0,False,True,Fake,Report #2348
PaintCollections,HistoryWritings,1000000.0,20.0,False,True,Fake,
PaintCollections,HistoryWritings,1000000.0,20.0,False,True,,Report #2349
PaintCollections,HistoryWritings,1000000.0,20.0,False,True,,
PaintCollections,HistoryWritings,1000000.0,20.0,False,False,Genuine,Report #2350
PaintCollections,HistoryWritings,1000000.0,20.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,20.0,False,False,Fake,Report #2351
PaintCollections,HistoryWritings,1000000.0,20.0,False,False,Fake,
PaintCollections,HistoryWritings,1000000.0,20.0,False,False,,Report #2352
PaintCollections,HistoryWritings,1000000.0,20.0,False,False,,
PaintCollections,HistoryWritings,1000000.0,21.0,True,True,Genuine,Report #2353
PaintCollections,HistoryWritings,1000000.0,21.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,21.0,True,True,Fake,Report #2354
PaintCollections,HistoryWritings,1000000.0,21.0,True,True,Fake,
PaintCollections,HistoryWritings,1000000.0,21.0,True,True,,Report #2355
PaintCollections,HistoryWritings,1000000.0,21.0,True,True,,
PaintCollections,HistoryWritings,1000000.0,21.0,True,False,Genuine,Report #2356
PaintCollections,HistoryWritings,1000000.0,21.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,21.0,True,False,Fake,Report #2357
PaintCollections,HistoryWritings,1000000.0,21.0,True,False,Fake,
PaintCollections,HistoryWritings,1000000.0,21.0,True,False,,Report #2358
PaintCollections,HistoryWritings,1000000.0,21.0,True,False,,
PaintCollections,HistoryWritings,1000000.0,21.0,False,True,Genuine,Report #2359
PaintCollections,HistoryWritings,1000000.0,21.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,21.0,False,True,Fake,Report #2360
PaintCollections,HistoryWritings,1000000.0,21.0,False,True,Fake,
PaintCollections,HistoryWritings,1000000.0,21.0,False,True,,Report #2361
PaintCollections,HistoryWritings,1000000.0,21.0,False,True,,
PaintCollections,HistoryWritings,1000000.0,21.0,False,False,Genuine,Report #2362
PaintCollections,HistoryWritings,1000000.0,21.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,21.0,False,False,Fake,Report #2363
PaintCollections,HistoryWritings,1000000.0,21.0,False,False,Fake,
PaintCollections,HistoryWritings,1000000.0,21.0,False,False,,Report #2364
PaintCollections,HistoryWritings,1000000.0,21.0,False,False,,
PaintCollections,HistoryWritings,1000000.0,99.0,True,True,Genuine,Report #2365
PaintCollections,HistoryWritings,1000000.0,99.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,99.0,True,True,Fake,Report #2366
PaintCollections,HistoryWritings,1000000.0,99.0,True,True,Fake,
PaintCollections,HistoryWritings,1000000.0,99.0,True,True,,Report #2367
PaintCollections,HistoryWritings,1000000.0,99.0,True,True,,
PaintCollections,HistoryWritings,1000000.0,99.0,True,False,Genuine,Report #2368
PaintCollections,HistoryWritings,1000000.0,99.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,99.0,True,False,Fake,Report #2369
PaintCollections,HistoryWritings,1000000.0,99.0,True,False,Fake,
PaintCollections,HistoryWritings,1000000.0,99.0,True,False,,Report #2370
PaintCollections,HistoryWritings,1000000.0,99.0,True,False,,
PaintCollections,HistoryWritings,1000000.0,99.0,False,True,Genuine,Report #2371
PaintCollections,HistoryWritings,1000000.0,99.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,99.0,False,True,Fake,Report #2372
PaintCollections,HistoryWritings,1000000.0,99.0,False,True,Fake,
PaintCollections,HistoryWritings,1000000.0,99.0,False,True,,Report #2373
PaintCollections,HistoryWritings,1000000.0,99.0,False,True,,
PaintCollections,HistoryWritings,1000000.0,99.0,False,False,Genuine,Report #2374
PaintCollections,HistoryWritings,1000000.0,99.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,99.0,False,False,Fake,Report #2375
PaintCollections,HistoryWritings,1000000.0,99.0,False,False,Fake,
PaintCollections,HistoryWritings,1000000.0,99.0,False,False,,Report #2376
PaintCollections,HistoryWritings,1000000.0,99.0,False,False,,
PaintCollections,HistoryWritings,1000000.0,100.0,True,True,Genuine,Report #2377
PaintCollections,HistoryWritings,1000000.0,100.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,100.0,True,True,Fake,Report #2378
PaintCollections,HistoryWritings,1000000.0,100.0,True,True,Fake,
PaintCollections,HistoryWritings,1000000.0,100.0,True,True,,Report #2379
PaintCollections,HistoryWritings,1000000.0,100.0,True,True,,
PaintCollections,HistoryWritings,1000000.0,100.0,True,False,Genuine,Report #2380
PaintCollections,HistoryWritings,1000000.0,100.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,100.0,True,False,Fake,Report #2381
PaintCollections,HistoryWritings,1000000.0,100.0,True,False,Fake,
PaintCollections,HistoryWritings,1000000.0,100.0,True,False,,Report #2382
PaintCollections,HistoryWritings,1000000.0,100.0,True,False,,
PaintCollections,HistoryWritings,1000000.0,100.0,False,True,Genuine,Report #2383
PaintCollections,HistoryWritings,1000000.0,100.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,100.0,False,True,Fake,Report #2384
PaintCollections,HistoryWritings,1000000.0,100.0,False,True,Fake,
PaintCollections,HistoryWritings,1000000.0,100.0,False,True,,Report #2385
PaintCollections,HistoryWritings,1000000.0,100.0,False,True,,
PaintCollections,HistoryWritings,1000000.0,100.0,False,False,Genuine,Report #2386
PaintCollections,HistoryWritings,1000000.0,100.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,100.0,False,False,Fake,Report #2387
PaintCollections,HistoryWritings,1000000.0,100.0,False,False,Fake,
PaintCollections,HistoryWritings,1000000.0,100.0,False,False,,Report #2388
PaintCollections,HistoryWritings,1000000.0,100.0,False,False,,
PaintCollections,HistoryWritings,1000000.0,101.0,True,True,Genuine,Report #2389
PaintCollections,HistoryWritings,1000000.0,101.0,True,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,101.0,True,True,Fake,Report #2390
PaintCollections,HistoryWritings,1000000.0,101.0,True,True,Fake,
PaintCollections,HistoryWritings,1000000.0,101.0,True,True,,Report #2391
PaintCollections,HistoryWritings,1000000.0,101.0,True,True,,
PaintCollections,HistoryWritings,1000000.0,101.0,True,False,Genuine,Report #2392
PaintCollections,HistoryWritings,1000000.0,101.0,True,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,101.0,True,False,Fake,Report #2393
PaintCollections,HistoryWritings,1000000.0,101.0,True,False,Fake,
PaintCollections,HistoryWritings,1000000.0,101.0,True,False,,Report #2394
PaintCollections,HistoryWritings,1000000.0,101.0,True,False,,
PaintCollections,HistoryWritings,1000000.0,101.0,False,True,Genuine,Report #2395
PaintCollections,HistoryWritings,1000000.0,101.0,False,True,Genuine,
PaintCollections,HistoryWritings,1000000.0,101.0,False,True,Fake,Report #2396
PaintCollections,HistoryWritings,1000000.0,101.0,False,True,Fake,
PaintCollections,HistoryWritings,1000000.0,101.0,False,True,,Report #2397
PaintCollections,HistoryWritings,1000000.0,101.0,False,True,,
PaintCollections,HistoryWritings,1000000.0,101.0,False,False,Genuine,Report #2398
PaintCollections,HistoryWritings,1000000.0,101.0,False,False,Genuine,
PaintCollections,HistoryWritings,1000000.0,101.0,False,False,Fake,Report #2399
PaintCollections,HistoryWritings,1000000.0,101.0,False,False,Fake,
PaintCollections,HistoryWritings,1000000.0,101.0,False,False,,Report #2400
PaintCollections,HistoryWritings,1000000.0,101.0,False,False,,
HouseHoldCollection,Paintings,0.0,-1.0,True,True,Genuine,Report #2401
HouseHoldCollection,Paintings,0.0,-1.0,True,True,Genuine,
HouseHoldCollection,Paintings,0.0,-1.0,True,True,Fake,Report #2402
HouseHoldCollection,Paintings,0.0,-1.0,True,True,Fake,
HouseHoldCollection,Paintings,0.0,-1.0,True,True,,Report #2403
HouseHoldCollection,Paintings,0.0,-1.0,True,True,,
HouseHoldCollection,Paintings,0.0,-1.0,True,False,Genuine,Report #2404
HouseHoldCollection,Paintings,0.0,-1.0,True,False,Genuine,
HouseHoldCollection,Paintings,0.0,-1.0,True,False,Fake,Report #2405
HouseHoldCollection,Paintings,0.0,-1.0,True,False,Fake,
HouseHoldCollection,Paintings,0.0,-1.0,True,False,,Report #2406
HouseHoldCollection,Paintings,0.0,-1.0,True,False,,
HouseHoldCollection,Paintings,0.0,-1.0,False,True,Genuine,Report #2407
HouseHoldCollection,Paintings,0.0,-1.0,False,True,Genuine,
HouseHoldCollection,Paintings,0.0,-1.0,False,True,Fake,Report #2408
HouseHoldCollection,Paintings,0.0,-1.0,False,True,Fake,
HouseHoldCollection,Paintings,0.0,-1.0,False,True,,Report #2409
HouseHoldCollection,Paintings,0.0,-1.0,False,True,,
HouseHoldCollection,Paintings,0.0,-1.0,False,False,Genuine,Report #2410
HouseHoldCollection,Paintings,0.0,-1.0,False,False,Genuine,
HouseHoldCollection,Paintings,0.0,-1.0,False,False,Fake,Report #2411
HouseHoldCollection,Paintings,0.0,-1.0,False,False,Fake,
HouseHoldCollection,Paintings,0.0,-1.0,False,False,,Report #2412
HouseHoldCollection,Paintings,0.0,-1.0,False,False,,
HouseHoldCollection,Paintings,0.0,0.0,True,True,Genuine,Report #2413
HouseHoldCollection,Paintings,0.0,0.0,True,True,Genuine,
HouseHoldCollection,Paintings,0.0,0.0,True,True,Fake,Report #2414
HouseHoldCollection,Paintings,0.0,0.0,True,True,Fake,
HouseHoldCollection,Paintings,0.0,0.0,True,True,,Report #2415
HouseHoldCollection,Paintings,0.0,0.0,True,True,,
HouseHoldCollection,Paintings,0.0,0.0,True,False,Genuine,Report #2416
HouseHoldCollection,Paintings,0.0,0.0,True,False,Genuine,
HouseHoldCollection,Paintings,0.0,0.0,True,False,Fake,Report #2417
HouseHoldCollection,Paintings,0.0,0.0,True,False,Fake,
HouseHoldCollection,Paintings,0.0,0.0,True,False,,Report #2418
HouseHoldCollection,Paintings,0.0,0.0,True,False,,
HouseHoldCollection,Paintings,0.0,0.0,False,True,Genuine,Report #2419
HouseHoldCollection,Paintings,0.0,0.0,False,True,Genuine,
HouseHoldCollection,Paintings,0.0,0.0,False,True,Fake,Report #2420
HouseHoldCollection,Paintings,0.0,0.0,False,True,Fake,
HouseHoldCollection,Paintings,0.0,0.0,False,True,,Report #2421
HouseHoldCollection,Paintings,0.0,0.0,False,True,,
HouseHoldCollection,Paintings,0.0,0.0,False,False,Genuine,Report #2422
HouseHoldCollection,Paintings,0.0,0.0,False,False,Genuine,
HouseHoldCollection,Paintings,0.0,0.0,False,False,Fake,Report #2423
HouseHoldCollection,Paintings,0.0,0.0,False,False,Fake,
HouseHoldCollection,Paintings,0.0,0.0,False,False,,Report #2424
HouseHoldCollection,Paintings,0.0,0.0,False,False,,
HouseHoldCollection,Paintings,0.0,19.0,True,True,Genuine,Report #2425
HouseHoldCollection,Paintings,0.0,19.0,True,True,Genuine,
HouseHoldCollection,Paintings,0.0,19.0,True,True,Fake,Report #2426
HouseHoldCollection,Paintings,0.0,19.0,True,True,Fake,
HouseHoldCollection,Paintings,0.0,19.0,True,True,,Report #2427
HouseHoldCollection,Paintings,0.0,19.0,True,True,,
HouseHoldCollection,Paintings,0.0,19.0,True,False,Genuine,Report #2428
HouseHoldCollection,Paintings,0.0,19.0,True,False,Genuine,
HouseHoldCollection,Paintings,0.0,19.0,True,False,Fake,Report #2429
HouseHoldCollection,Paintings,0.0,19.0,True,False,Fake,
HouseHoldCollection,Paintings,0.0,19.0,True,False,,Report #2430
HouseHoldCollection,Paintings,0.0,19.0,True,False,,
HouseHoldCollection,Paintings,0.0,19.0,False,True,Genuine,Report #2431
HouseHoldCollection,Paintings,0.0,19.0,False,True,Genuine,
HouseHoldCollection,Paintings,0.0,19.0,False,True,Fake,Report #2432
HouseHoldCollection,Paintings,0.0,19.0,False,True,Fake,
HouseHoldCollection,Paintings,0.0,19.0,False,True,,Report #2433
HouseHoldCollection,Paintings,0.0,19.0,False,True,,
HouseHoldCollection,Paintings,0.0,19.0,False,False,Genuine,Report #2434
HouseHoldCollection,Paintings,0.0,19.0,False,False,Genuine,
HouseHoldCollection,Paintings,0.0,19.0,False,False,Fake,Report #2435
HouseHoldCollection,Paintings,0.0,19.0,False,False,Fake,
HouseHoldCollection,Paintings,0.0,19.0,False,False,,Report #2436
HouseHoldCollection,Paintings,0.0,19.0,False,False,,
HouseHoldCollection,Paintings,0.0,20.0,True,True,Genuine,Report #2437
HouseHoldCollection,Paintings,0.0,20.0,True,True,Genuine,
HouseHoldCollection,Paintings,0.0,20.0,True,True,Fake,Report #2438
HouseHoldCollection,Paintings,0.0,20.0,True,True,Fake,
HouseHoldCollection,Paintings,0.0,20.0,True,True,,Report #2439
HouseHoldCollection,Paintings,0.0,20.0,True,True,,
HouseHoldCollection,Paintings,0.0,20.0,True,False,Genuine,Report #2440
HouseHoldCollection,Paintings,0.0,20.0,True,False,Genuine,
HouseHoldCollection,Paintings,0.0,20.0,True,False,Fake,Report #2441
HouseHoldCollection,Paintings,0.0,20.0,True,False,Fake,
HouseHoldCollection,Paintings,0.0,20.0,True,False,,Report #2442
HouseHoldCollection,Paintings,0.0,20.0,True,False,,
HouseHoldCollection,Paintings,0.0,20.0,False,True,Genuine,Report #2443
HouseHoldCollection,Paintings,0.0,20.0,False,True,Genuine,
HouseHoldCollection,Paintings,0.0,20.0,False,True,Fake,Report #2444
HouseHoldCollection,Paintings,0.0,20.0,False,True,Fake,
HouseHoldCollection,Paintings,0.0,20.0,False,True,,Report #2445
HouseHoldCollection,Paintings,0.0,20.0,False,True,,
HouseHoldCollection,Paintings,0.0,20.0,False,False,Genuine,Report #2446
HouseHoldCollection,Paintings,0.0,20.0,False,False,Genuine,
HouseHoldCollection,Paintings,0.0,20.0,False,False,Fake,Report #2447
HouseHoldCollection,Paintings,0.0,20.0,False,False,Fake,
HouseHoldCollection,Paintings,0.0,20.0,False,False,,Report #2448
HouseHoldCollection,Paintings,0.0,20.0,False,False,,
HouseHoldCollection,Paintings,0.0,21.0,True,True,Genuine,Report #2449
HouseHoldCollection,Paintings,0.0,21.0,True,True,Genuine,
HouseHoldCollection,Paintings,0.0,21.0,True,True,Fake,Report #2450
HouseHoldCollection,Paintings,0.0,21.0,True,True,Fake,
HouseHoldCollection,Paintings,0.0,21.0,True,True,,Report #2451
HouseHoldCollection,Paintings,0.0,21.0,True,True,,
HouseHoldCollection,Paintings,0.0,21.0,True,False,Genuine,Report #2452
HouseHoldCollection,Paintings,0.0,21.0,True,False,Genuine,
HouseHoldCollection,Paintings,0.0,21.0,True,False,Fake,Report #2453
HouseHoldCollection,Paintings,0.0,21.0,True,False,Fake,
HouseHoldCollection,Paintings,0.0,21.0,True,False,,Report #2454
HouseHoldCollection,Paintings,0.0,21.0,True,False,,
HouseHoldCollection,Paintings,0.0,21.0,False,True,Genuine,Report #2455
HouseHoldCollection,Paintings,0.0,21.0,False,True,Genuine,
HouseHoldCollection,Paintings,0.0,21.0,False,True,Fake,Report #2456
HouseHoldCollection,Paintings,0.0,21.0,False,True,Fake,
HouseHoldCollection,Paintings,0.0,21.0,False,True,,Report #2457
HouseHoldCollection,Paintings,0.0,21.0,False,True,,
HouseHoldCollection,Paintings,0.0,21.0,False,False,Genuine,Report #2458
HouseHoldCollection,Paintings,0.0,21.0,False,False,Genuine,
HouseHoldCollection,Paintings,0.0,21.0,False,False,Fake,Report #2459
HouseHoldCollection,Paintings,0.0,21.0,False,False,Fake,
HouseHoldCollection,Paintings,0.0,21.0,False,False,,Report #2460
HouseHoldCollection,Paintings,0.0,21.0,False,False,,
HouseHoldCollection,Paintings,0.0,99.0,True,True,Genuine,Report #2461
HouseHoldCollection,Paintings,0.0,99.0,True,True,Genuine,
HouseHoldCollection,Paintings,0.0,99.0,True,True,Fake,Report #2462
HouseHoldCollection,Paintings,0.0,99.0,True,True,Fake,
HouseHoldCollection,Paintings,0.0,99.0,True,True,,Report #2463
HouseHoldCollection,Paintings,0.0,99.0,True,True,,
HouseHoldCollection,Paintings,0.0,99.0,True,False,Genuine,Report #2464
HouseHoldCollection,Paintings,0.0,99.0,True,False,Genuine,
HouseHoldCollection,Paintings,0.0,99.0,True,False,Fake,Report #2465
HouseHoldCollection,Paintings,0.0,99.0,True,False,Fake,
HouseHoldCollection,Paintings,0.0,99.0,True,False,,Report #2466
HouseHoldCollection,Paintings,0.0,99.0,True,False,,
HouseHoldCollection,Paintings,0.0,99.0,False,True,Genuine,Report #2467
HouseHoldCollection,Paintings,0.0,99.0,False,True,Genuine,
HouseHoldCollection,Paintings,0.0,99.0,False,True,Fake,Report #2468
HouseHoldCollection,Paintings,0.0,99.0,False,True,Fake,
HouseHoldCollection,Paintings,0.0,99.0,False,True,,Report #2469
HouseHoldCollection,Paintings,0.0,99.0,False,True,,
HouseHoldCollection,Paintings,0.0,99.0,False,False,Genuine,Report #2470
HouseHoldCollection,Paintings,0.0,99.0,False,False,Genuine,
HouseHoldCollection,Paintings,0.0,99.0,False,False,Fake,Report #2471
HouseHoldCollection,Paintings,0.0,99.0,False,False,Fake,
HouseHoldCollection,Paintings,0.0,99.0,False,False,,Report #2472
HouseHoldCollection,Paintings,0.0,99.0,False,False,,
HouseHoldCollection,Paintings,0.0,100.0,True,True,Genuine,Report #2473
HouseHoldCollection,Paintings,0.0,100.0,True,True,Genuine,
HouseHoldCollection,Paintings,0.0,100.0,True,True,Fake,Report #2474
HouseHoldCollection,Paintings,0.0,100.0,True,True,Fake,
HouseHoldCollection,Paintings,0.0,100.0,True,True,,Report #2475
HouseHoldCollection,Paintings,0.0,100.0,True,True,,
HouseHoldCollection,Paintings,0.0,100.0,True,False,Genuine,Report #2476
HouseHoldCollection,Paintings,0.0,100.0,True,False,Genuine,
HouseHoldCollection,Paintings,0.0,100.0,True,False,Fake,Report #2477
HouseHoldCollection,Paintings,0.0,100.0,True,False,Fake,
HouseHoldCollection,Paintings,0.0,100.0,True,False,,Report #2478
HouseHoldCollection,Paintings,0.0,100.0,True,False,,
HouseHoldCollection,Paintings,0.0,100.0,False,True,Genuine,Report #2479
HouseHoldCollection,Paintings,0.0,100.0,False,True,Genuine,
HouseHoldCollection,Paintings,0.0,100.0,False,True,Fake,Report #2480
HouseHoldCollection,Paintings,0.0,100.0,False,True,Fake,
HouseHoldCollection,Paintings,0.0,100.0,False,True,,Report #2481
HouseHoldCollection,Paintings,0.0,100.0,False,True,,
HouseHoldCollection,Paintings,0.0,100.0,False,False,Genuine,Report #2482
HouseHoldCollection,Paintings,0.0,100.0,False,False,Genuine,
HouseHoldCollection,Paintings,0.0,100.0,False,False,Fake,Report #2483
HouseHoldCollection,Paintings,0.0,100.0,False,False,Fake,
HouseHoldCollection,Paintings,0.0,100.0,False,False,,Report #2484
HouseHoldCollection,Paintings,0.0,100.0,False,False,,
HouseHoldCollection,Paintings,0.0,101.0,True,True,Genuine,Report #2485
HouseHoldCollection,Paintings,0.0,101.0,True,True,Genuine,
HouseHoldCollection,Paintings,0.0,101.0,True,True,Fake,Report #2486
HouseHoldCollection,Paintings,0.0,101.0,True,True,Fake,
HouseHoldCollection,Paintings,0.0,101.0,True,True,,Report #2487
HouseHoldCollection,Paintings,0.0,101.0,True,True,,
HouseHoldCollection,Paintings,0.0,101.0,True,False,Genuine,Report #2488
HouseHoldCollection,Paintings,0.0,101.0,True,False,Genuine,
HouseHoldCollection,Paintings,0.0,101.0,True,False,Fake,Report #2489
HouseHoldCollection,Paintings,0.0,101.0,True,False,Fake,
HouseHoldCollection,Paintings,0.0,101.0,True,False,,Report #2490
HouseHoldCollection,Paintings,0.0,101.0,True,False,,
HouseHoldCollection,Paintings,0.0,101.0,False,True,Genuine,Report #2491
HouseHoldCollection,Paintings,0.0,101.0,False,True,Genuine,
HouseHoldCollection,Paintings,0.0,101.0,False,True,Fake,Report #2492
HouseHoldCollection,Paintings,0.0,101.0,False,True,Fake,
HouseHoldCollection,Paintings,0.0,101.0,False,True,,Report #2493
HouseHoldCollection,Paintings,0.0,101.0,False,True,,
HouseHoldCollection,Paintings,0.0,101.0,False,False,Genuine,Report #2494
HouseHoldCollection,Paintings,0.0,101.0,False,False,Genuine,
HouseHoldCollection,Paintings,0.0,101.0,False,False,Fake,Report #2495
HouseHoldCollection,Paintings,0.0,101.0,False,False,Fake,
HouseHoldCollection,Paintings,0.0,101.0,False,False,,Report #2496
HouseHoldCollection,Paintings,0.0,101.0,False,False,,
HouseHoldCollection,Paintings,1.0,-1.0,True,True,Genuine,Report #2497
HouseHoldCollection,Paintings,1.0,-1.0,True,True,Genuine,
HouseHoldCollection,Paintings,1.0,-1.0,True,True,Fake,Report #2498
HouseHoldCollection,Paintings,1.0,-1.0,True,True,Fake,
HouseHoldCollection,Paintings,1.0,-1.0,True,True,,Report #2499
HouseHoldCollection,Paintings,1.0,-1.0,True,True,,
HouseHoldCollection,Paintings,1.0,-1.0,True,False,Genuine,Report #2500
HouseHoldCollection,Paintings,1.0,-1.0,True,False,Genuine,
HouseHoldCollection,Paintings,1.0,-1.0,True,False,Fake,Report #2501
HouseHoldCollection,Paintings,1.0,-1.0,True,False,Fake,
HouseHoldCollection,Paintings,1.0,-1.0,True,False,,Report #2502
HouseHoldCollection,Paintings,1.0,-1.0,True,False,,
HouseHoldCollection,Paintings,1.0,-1.0,False,True,Genuine,Report #2503
HouseHoldCollection,Paintings,1.0,-1.0,False,True,Genuine,
HouseHoldCollection,Paintings,1.0,-1.0,False,True,Fake,Report #2504
HouseHoldCollection,Paintings,1.0,-1.0,False,True,Fake,
HouseHoldCollection,Paintings,1.0,-1.0,False,True,,Report #2505
HouseHoldCollection,Paintings,1.0,-1.0,False,True,,
HouseHoldCollection,Paintings,1.0,-1.0,False,False,Genuine,Report #2506
HouseHoldCollection,Paintings,1.0,-1.0,False,False,Genuine,
HouseHoldCollection,Paintings,1.0,-1.0,False,False,Fake,Report #2507
HouseHoldCollection,Paintings,1.0,-1.0,False,False,Fake,
HouseHoldCollection,Paintings,1.0,-1.0,False,False,,Report #2508
HouseHoldCollection,Paintings,1.0,-1.0,False,False,,
HouseHoldCollection,Paintings,1.0,0.0,True,True,Genuine,Report #2509
HouseHoldCollection,Paintings,1.0,0.0,True,True,Genuine,
HouseHoldCollection,Paintings,1.0,0.0,True,True,Fake,Report #2510
HouseHoldCollection,Paintings,1.0,0.0,True,True,Fake,
HouseHoldCollection,Paintings,1.0,0.0,True,True,,Report #2511
HouseHoldCollection,Paintings,1.0,0.0,True,True,,
HouseHoldCollection,Paintings,1.0,0.0,True,False,Genuine,Report #2512
HouseHoldCollection,Paintings,1.0,0.0,True,False,Genuine,
HouseHoldCollection,Paintings,1.0,0.0,True,False,Fake,Report #2513
HouseHoldCollection,Paintings,1.0,0.0,True,False,Fake,
HouseHoldCollection,Paintings,1.0,0.0,True,False,,Report #2514
HouseHoldCollection,Paintings,1.0,0.0,True,False,,
HouseHoldCollection,Paintings,1.0,0.0,False,True,Genuine,Report #2515
HouseHoldCollection,Paintings,1.0,0.0,False,True,Genuine,
HouseHoldCollection,Paintings,1.0,0.0,False,True,Fake,Report #2516
HouseHoldCollection,Paintings,1.0,0.0,False,True,Fake,
HouseHoldCollection,Paintings,1.0,0.0,False,True,,Report #2517
HouseHoldCollection,Paintings,1.0,0.0,False,True,,
HouseHoldCollection,Paintings,1.0,0.0,False,False,Genuine,Report #2518
HouseHoldCollection,Paintings,1.0,0.0,False,False,Genuine,
HouseHoldCollection,Paintings,1.0,0.0,False,False,Fake,Report #2519
HouseHoldCollection,Paintings,1.0,0.0,False,False,Fake,
HouseHoldCollection,Paintings,1.0,0.0,False,False,,Report #2520
HouseHoldCollection,Paintings,1.0,0.0,False,False,,
HouseHoldCollection,Paintings,1.0,19.0,True,True,Genuine,Report #2521
HouseHoldCollection,Paintings,1.0,19.0,True,True,Genuine,
HouseHoldCollection,Paintings,1.0,19.0,True,True,Fake,Report #2522
HouseHoldCollection,Paintings,1.0,19.0,True,True,Fake,
HouseHoldCollection,Paintings,1.0,19.0,True,True,,Report #2523
HouseHoldCollection,Paintings,1.0,19.0,True,True,,
HouseHoldCollection,Paintings,1.0,19.0,True,False,Genuine,Report #2524
HouseHoldCollection,Paintings,1.0,19.0,True,False,Genuine,
HouseHoldCollection,Paintings,1.0,19.0,True,False,Fake,Report #2525
HouseHoldCollection,Paintings,1.0,19.0,True,False,Fake,
HouseHoldCollection,Paintings,1.0,19.0,True,False,,Report #2526
HouseHoldCollection,Paintings,1.0,19.0,True,False,,
HouseHoldCollection,Paintings,1.0,19.0,False,True,Genuine,Report #2527
HouseHoldCollection,Paintings,1.0,19.0,False,True,Genuine,
HouseHoldCollection,Paintings,1.0,19.0,False,True,Fake,Report #2528
HouseHoldCollection,Paintings,1.0,19.0,False,True,Fake,
HouseHoldCollection,Paintings,1.0,19.0,False,True,,Report #2529
HouseHoldCollection,Paintings,1.0,19.0,False,True,,
HouseHoldCollection,Paintings,1.0,19.0,False,False,Genuine,Report #2530
HouseHoldCollection,Paintings,1.0,19.0,False,False,Genuine,
HouseHoldCollection,Paintings,1.0,19.0,False,False,Fake,Report #2531
HouseHoldCollection,Paintings,1.0,19.0,False,False,Fake,
HouseHoldCollection,Paintings,1.0,19.0,False,False,,Report #2532
HouseHoldCollection,Paintings,1.0,19.0,False,False,,
HouseHoldCollection,Paintings,1.0,20.0,True,True,Genuine,Report #2533
HouseHoldCollection,Paintings,1.0,20.0,True,True,Genuine,
HouseHoldCollection,Paintings,1.0,20.0,True,True,Fake,Report #2534
HouseHoldCollection,Paintings,1.0,20.0,True,True,Fake,
HouseHoldCollection,Paintings,1.0,20.0,True,True,,Report #2535
HouseHoldCollection,Paintings,1.0,20.0,True,True,,
HouseHoldCollection,Paintings,1.0,20.0,True,False,Genuine,Report #2536
HouseHoldCollection,Paintings,1.0,20.0,True,False,Genuine,
HouseHoldCollection,Paintings,1.0,20.0,True,False,Fake,Report #2537
HouseHoldCollection,Paintings,1.0,20.0,True,False,Fake,
HouseHoldCollection,Paintings,1.0,20.0,True,False,,Report #2538
HouseHoldCollection,Paintings,1.0,20.0,True,False,,
HouseHoldCollection,Paintings,1.0,20.0,False,True,Genuine,Report #2539
HouseHoldCollection,Paintings,1.0,20.0,False,True,Genuine,
HouseHoldCollection,Paintings,1.0,20.0,False,True,Fake,Report #2540
HouseHoldCollection,Paintings,1.0,20.0,False,True,Fake,
HouseHoldCollection,Paintings,1.0,20.0,False,True,,Report #2541
HouseHoldCollection,Paintings,1.0,20.0,False,True,,
HouseHoldCollection,Paintings,1.0,20.0,False,False,Genuine,Report #2542
HouseHoldCollection,Paintings,1.0,20.0,False,False,Genuine,
HouseHoldCollection,Paintings,1.0,20.0,False,False,Fake,Report #2543
HouseHoldCollection,Paintings,1.0,20.0,False,False,Fake,
HouseHoldCollection,Paintings,1.0,20.0,False,False,,Report #2544
HouseHoldCollection,Paintings,1.0,20.0,False,False,,
HouseHoldCollection,Paintings,1.0,21.0,True,True,Genuine,Report #2545
HouseHoldCollection,Paintings,1.0,21.0,True,True,Genuine,
HouseHoldCollection,Paintings,1.0,21.0,True,True,Fake,Report #2546
HouseHoldCollection,Paintings,1.0,21.0,True,True,Fake,
HouseHoldCollection,Paintings,1.0,21.0,True,True,,Report #2547
HouseHoldCollection,Paintings,1.0,21.0,True,True,,
HouseHoldCollection,Paintings,1.0,21.0,True,False,Genuine,Report #2548
HouseHoldCollection,Paintings,1.0,21.0,True,False,Genuine,
HouseHoldCollection,Paintings,1.0,21.0,True,False,Fake,Report #2549
HouseHoldCollection,Paintings,1.0,21.0,True,False,Fake,
HouseHoldCollection,Paintings,1.0,21.0,True,False,,Report #2550
HouseHoldCollection,Paintings,1.0,21.0,True,False,,
HouseHoldCollection,Paintings,1.0,21.0,False,True,Genuine,Report #2551
HouseHoldCollection,Paintings,1.0,21.0,False,True,Genuine,
HouseHoldCollection,Paintings,1.0,21.0,False,True,Fake,Report #2552
HouseHoldCollection,Paintings,1.0,21.0,False,True,Fake,
HouseHoldCollection,Paintings,1.0,21.0,False,True,,Report #2553
HouseHoldCollection,Paintings,1.0,21.0,False,True,,
HouseHoldCollection,Paintings,1.0,21.0,False,False,Genuine,Report #2554
HouseHoldCollection,Paintings,1.0,21.0,False,False,Genuine,
HouseHoldCollection,Paintings,1.0,21.0,False,False,Fake,Report #2555
HouseHoldCollection,Paintings,1.0,21.0,False,False,Fake,
HouseHoldCollection,Paintings,1.0,21.0,False,False,,Report #2556
HouseHoldCollection,Paintings,1.0,21.0,False,False,,
HouseHoldCollection,Paintings,1.0,99.0,True,True,Genuine,Report #2557
HouseHoldCollection,Paintings,1.0,99.0,True,True,Genuine,
HouseHoldCollection,Paintings,1.0,99.0,True,True,Fake,Report #2558
HouseHoldCollection,Paintings,1.0,99.0,True,True,Fake,
HouseHoldCollection,Paintings,1.0,99.0,True,True,,Report #2559
HouseHoldCollection,Paintings,1.0,99.0,True,True,,
HouseHoldCollection,Paintings,1.0,99.0,True,False,Genuine,Report #2560
HouseHoldCollection,Paintings,1.0,99.0,True,False,Genuine,
HouseHoldCollection,Paintings,1.0,99.0,True,False,Fake,Report #2561
HouseHoldCollection,Paintings,1.0,99.0,True,False,Fake,
HouseHoldCollection,Paintings,1.0,99.0,True,False,,Report #2562
HouseHoldCollection,Paintings,1.0,99.0,True,False,,
HouseHoldCollection,Paintings,1.0,99.0,False,True,Genuine,Report #2563
HouseHoldCollection,Paintings,1.0,99.0,False,True,Genuine,
HouseHoldCollection,Paintings,1.0,99.0,False,True,Fake,Report #2564
HouseHoldCollection,Paintings,1.0,99.0,False,True,Fake,
HouseHoldCollection,Paintings,1.0,99.0,False,True,,Report #2565
HouseHoldCollection,Paintings,1.0,99.0,False,True,,
HouseHoldCollection,Paintings,1.0,99.0,False,False,Genuine,Report #2566
HouseHoldCollection,Paintings,1.0,99.0,False,False,Genuine,
HouseHoldCollection,Paintings,1.0,99.0,False,False,Fake,Report #2567
HouseHoldCollection,Paintings,1.0,99.0,False,False,Fake,
HouseHoldCollection,Paintings,1.0,99.0,False,False,,Report #2568
HouseHoldCollection,Paintings,1.0,99.0,False,False,,
HouseHoldCollection,Paintings,1.0,100.0,True,True,Genuine,Report #2569
HouseHoldCollection,Paintings,1.0,100.0,True,True,Genuine,
HouseHoldCollection,Paintings,1.0,100.0,True,True,Fake,Report #2570
HouseHoldCollection,Paintings,1.0,100.0,True,True,Fake,
HouseHoldCollection,Paintings,1.0,100.0,True,True,,Report #2571
HouseHoldCollection,Paintings,1.0,100.0,True,True,,
HouseHoldCollection,Paintings,1.0,100.0,True,False,Genuine,Report #2572
HouseHoldCollection,Paintings,1.0,100.0,True,False,Genuine,
HouseHoldCollection,Paintings,1.0,100.0,True,False,Fake,Report #2573
HouseHoldCollection,Paintings,1.0,100.0,True,False,Fake,
HouseHoldCollection,Paintings,1.0,100.0,True,False,,Report #2574
HouseHoldCollection,Paintings,1.0,100.0,True,False,,
HouseHoldCollection,Paintings,1.0,100.0,False,True,Genuine,Report #2575
HouseHoldCollection,Paintings,1.0,100.0,False,True,Genuine,
HouseHoldCollection,Paintings,1.0,100.0,False,True,Fake,Report #2576
HouseHoldCollection,Paintings,1.0,100.0,False,True,Fake,
HouseHoldCollection,Paintings,1.0,100.0,False,True,,Report #2577
HouseHoldCollection,Paintings,1.0,100.0,False,True,,
HouseHoldCollection,Paintings,1.0,100.0,False,False,Genuine,Report #2578
HouseHoldCollection,Paintings,1.0,100.0,False,False,Genuine,
HouseHoldCollection,Paintings,1.0,100.0,False,False,Fake,Report #2579
HouseHoldCollection,Paintings,1.0,100.0,False,False,Fake,
HouseHoldCollection,Paintings,1.0,100.0,False,False,,Report #2580
HouseHoldCollection,Paintings,1.0,100.0,False,False,,
HouseHoldCollection,Paintings,1.0,101.0,True,True,Genuine,Report #2581
HouseHoldCollection,Paintings,1.0,101.0,True,True,Genuine,
HouseHoldCollection,Paintings,1.0,101.0,True,True,Fake,Report #2582
HouseHoldCollection,Paintings,1.0,101.0,True,True,Fake,
HouseHoldCollection,Paintings,1.0,101.0,True,True,,Report #2583
HouseHoldCollection,Paintings,1.0,101.0,True,True,,
HouseHoldCollection,Paintings,1.0,101.0,True,False,Genuine,Report #2584
HouseHoldCollection,Paintings,1.0,101.0,True,False,Genuine,
HouseHoldCollection,Paintings,1.0,101.0,True,False,Fake,Report #2585
HouseHoldCollection,Paintings,1.0,101.0,True,False,Fake,
HouseHoldCollection,Paintings,1.0,101.0,True,False,,Report #2586
HouseHoldCollection,Paintings,1.0,101.0,True,False,,
HouseHoldCollection,Paintings,1.0,101.0,False,True,Genuine,Report #2587
HouseHoldCollection,Paintings,1.0,101.0,False,True,Genuine,
HouseHoldCollection,Paintings,1.0,101.0,False,True,Fake,Report #2588
HouseHoldCollection,Paintings,1.0,101.0,False,True,Fake,
HouseHoldCollection,Paintings,1.0,101.0,False,True,,Report #2589
HouseHoldCollection,Paintings,1.0,101.0,False,True,,
HouseHoldCollection,Paintings,1.0,101.0,False,False,Genuine,Report #2590
HouseHoldCollection,Paintings,1.0,101.0,False,False,Genuine,
HouseHoldCollection,Paintings,1.0,101.0,False,False,Fake,Report #2591
HouseHoldCollection,Paintings,1.0,101.0,False,False,Fake,
HouseHoldCollection,Paintings,1.0,101.0,False,False,,Report #2592
HouseHoldCollection,Paintings,1.0,101.0,False,False,,
HouseHoldCollection,Paintings,100.0,-1.0,True,True,Genuine,Report #2593
HouseHoldCollection,Paintings,100.0,-1.0,True,True,Genuine,
HouseHoldCollection,Paintings,100.0,-1.0,True,True,Fake,Report #2594
HouseHoldCollection,Paintings,100.0,-1.0,True,True,Fake,
HouseHoldCollection,Paintings,100.0,-1.0,True,True,,Report #2595
HouseHoldCollection,Paintings,100.0,-1.0,True,True,,
HouseHoldCollection,Paintings,100.0,-1.0,True,False,Genuine,Report #2596
HouseHoldCollection,Paintings,100.0,-1.0,True,False,Genuine,
HouseHoldCollection,Paintings,100.0,-1.0,True,False,Fake,Report #2597
HouseHoldCollection,Paintings,100.0,-1.0,True,False,Fake,
HouseHoldCollection,Paintings,100.0,-1.0,True,False,,Report #2598
HouseHoldCollection,Paintings,100.0,-1.0,True,False,,
HouseHoldCollection,Paintings,100.0,-1.0,False,True,Genuine,Report #2599
HouseHoldCollection,Paintings,100.0,-1.0,False,True,Genuine,
HouseHoldCollection,Paintings,100.0,-1.0,False,True,Fake,Report #2600
HouseHoldCollection,Paintings,100.0,-1.0,False,True,Fake,
HouseHoldCollection,Paintings,100.0,-1.0,False,True,,Report #2601
HouseHoldCollection,Paintings,100.0,-1.0,False,True,,
HouseHoldCollection,Paintings,100.0,-1.0,False,False,Genuine,Report #2602
HouseHoldCollection,Paintings,100.0,-1.0,False,False,Genuine,
HouseHoldCollection,Paintings,100.0,-1.0,False,False,Fake,Report #2603
HouseHoldCollection,Paintings,100.0,-1.0,False,False,Fake,
HouseHoldCollection,Paintings,100.0,-1.0,False,False,,Report #2604
HouseHoldCollection,Paintings,100.0,-1.0,False,False,,
HouseHoldCollection,Paintings,100.0,0.0,True,True,Genuine,Report #2605
HouseHoldCollection,Paintings,100.0,0.0,True,True,Genuine,
HouseHoldCollection,Paintings,100.0,0.0,True,True,Fake,Report #2606
HouseHoldCollection,Paintings,100.0,0.0,True,True,Fake,
HouseHoldCollection,Paintings,100.0,0.0,True,True,,Report #2607
HouseHoldCollection,Paintings,100.0,0.0,True,True,,
HouseHoldCollection,Paintings,100.0,0.0,True,False,Genuine,Report #2608
HouseHoldCollection,Paintings,100.0,0.0,True,False,Genuine,
HouseHoldCollection,Paintings,100.0,0.0,True,False,Fake,Report #2609
HouseHoldCollection,Paintings,100.0,0.0,True,False,Fake,
HouseHoldCollection,Paintings,100.0,0.0,True,False,,Report #2610
HouseHoldCollection,Paintings,100.0,0.0,True,False,,
HouseHoldCollection,Paintings,100.0,0.0,False,True,Genuine,Report #2611
HouseHoldCollection,Paintings,100.0,0.0,False,True,Genuine,
HouseHoldCollection,Paintings,100.0,0.0,False,True,Fake,Report #2612
HouseHoldCollection,Paintings,100.0,0.0,False,True,Fake,
HouseHoldCollection,Paintings,100.0,0.0,False,True,,Report #2613
HouseHoldCollection,Paintings,100.0,0.0,False,True,,
HouseHoldCollection,Paintings,100.0,0.0,False,False,Genuine,Report #2614
HouseHoldCollection,Paintings,100.0,0.0,False,False,Genuine,
HouseHoldCollection,Paintings,100.0,0.0,False,False,Fake,Report #2615
HouseHoldCollection,Paintings,100.0,0.0,False,False,Fake,
HouseHoldCollection,Paintings,100.0,0.0,False,False,,Report #2616
HouseHoldCollection,Paintings,100.0,0.0,False,False,,
HouseHoldCollection,Paintings,100.0,19.0,True,True,Genuine,Report #2617
HouseHoldCollection,Paintings,100.0,19.0,True,True,Genuine,
HouseHoldCollection,Paintings,100.0,19.0,True,True,Fake,Report #2618
HouseHoldCollection,Paintings,100.0,19.0,True,True,Fake,
HouseHoldCollection,Paintings,100.0,19.0,True,True,,Report #2619
HouseHoldCollection,Paintings,100.0,19.0,True,True,,
HouseHoldCollection,Paintings,100.0,19.0,True,False,Genuine,Report #2620
HouseHoldCollection,Paintings,100.0,19.0,True,False,Genuine,
HouseHoldCollection,Paintings,100.0,19.0,True,False,Fake,Report #2621
HouseHoldCollection,Paintings,100.0,19.0,True,False,Fake,
HouseHoldCollection,Paintings,100.0,19.0,True,False,,Report #2622
HouseHoldCollection,Paintings,100.0,19.0,True,False,,
HouseHoldCollection,Paintings,100.0,19.0,False,True,Genuine,Report #2623
HouseHoldCollection,Paintings,100.0,19.0,False,True,Genuine,
HouseHoldCollection,Paintings,100.0,19.0,False,True,Fake,Report #2624
HouseHoldCollection,Paintings,100.0,19.0,False,True,Fake,
HouseHoldCollection,Paintings,100.0,19.0,False,True,,Report #2625
HouseHoldCollection,Paintings,100.0,19.0,False,True,,
HouseHoldCollection,Paintings,100.0,19.0,False,False,Genuine,Report #2626
HouseHoldCollection,Paintings,100.0,19.0,False,False,Genuine,
HouseHoldCollection,Paintings,100.0,19.0,False,False,Fake,Report #2627
HouseHoldCollection,Paintings,100.0,19.0,False,False,Fake,
HouseHoldCollection,Paintings,100.0,19.0,False,False,,Report #2628
HouseHoldCollection,Paintings,100.0,19.0,False,False,,
HouseHoldCollection,Paintings,100.0,20.0,True,True,Genuine,Report #2629
HouseHoldCollection,Paintings,100.0,20.0,True,True,Genuine,
HouseHoldCollection,Paintings,100.0,20.0,True,True,Fake,Report #2630
HouseHoldCollection,Paintings,100.0,20.0,True,True,Fake,
HouseHoldCollection,Paintings,100.0,20.0,True,True,,Report #2631
HouseHoldCollection,Paintings,100.0,20.0,True,True,,
HouseHoldCollection,Paintings,100.0,20.0,True,False,Genuine,Report #2632
HouseHoldCollection,Paintings,100.0,20.0,True,False,Genuine,
HouseHoldCollection,Paintings,100.0,20.0,True,False,Fake,Report #2633
HouseHoldCollection,Paintings,100.0,20.0,True,False,Fake,
HouseHoldCollection,Paintings,100.0,20.0,True,False,,Report #2634
HouseHoldCollection,Paintings,100.0,20.0,True,False,,
HouseHoldCollection,Paintings,100.0,20.0,False,True,Genuine,Report #2635
HouseHoldCollection,Paintings,100.0,20.0,False,True,Genuine,
HouseHoldCollection,Paintings,100.0,20.0,False,True,Fake,Report #2636
HouseHoldCollection,Paintings,100.0,20.0,False,True,Fake,
HouseHoldCollection,Paintings,100.0,20.0,False,True,,Report #2637
HouseHoldCollection,Paintings,100.0,20.0,False,True,,
HouseHoldCollection,Paintings,100.0,20.0,False,False,Genuine,Report #2638
HouseHoldCollection,Paintings,100.0,20.0,False,False,Genuine,
HouseHoldCollection,Paintings,100.0,20.0,False,False,Fake,Report #2639
HouseHoldCollection,Paintings,100.0,20.0,False,False,Fake,
HouseHoldCollection,Paintings,100.0,20.0,False,False,,Report #2640
HouseHoldCollection,Paintings,100.0,20.0,False,False,,
HouseHoldCollection,Paintings,100.0,21.0,True,True,Genuine,Report #2641
HouseHoldCollection,Paintings,100.0,21.0,True,True,Genuine,
HouseHoldCollection,Paintings,100.0,21.0,True,True,Fake,Report #2642
HouseHoldCollection,Paintings,100.0,21.0,True,True,Fake,
HouseHoldCollection,Paintings,100.0,21.0,True,True,,Report #2643
HouseHoldCollection,Paintings,100.0,21.0,True,True,,
HouseHoldCollection,Paintings,100.0,21.0,True,False,Genuine,Report #2644
HouseHoldCollection,Paintings,100.0,21.0,True,False,Genuine,
HouseHoldCollection,Paintings,100.0,21.0,True,False,Fake,Report #2645
HouseHoldCollection,Paintings,100.0,21.0,True,False,Fake,
HouseHoldCollection,Paintings,100.0,21.0,True,False,,Report #2646
HouseHoldCollection,Paintings,100.0,21.0,True,False,,
HouseHoldCollection,Paintings,100.0,21.0,False,True,Genuine,Report #2647
HouseHoldCollection,Paintings,100.0,21.0,False,True,Genuine,
HouseHoldCollection,Paintings,100.0,21.0,False,True,Fake,Report #2648
HouseHoldCollection,Paintings,100.0,21.0,False,True,Fake,
HouseHoldCollection,Paintings,100.0,21.0,False,True,,Report #2649
HouseHoldCollection,Paintings,100.0,21.0,False,True,,
HouseHoldCollection,Paintings,100.0,21.0,False,False,Genuine,Report #2650
HouseHoldCollection,Paintings,100.0,21.0,False,False,Genuine,
HouseHoldCollection,Paintings,100.0,21.0,False,False,Fake,Report #2651
HouseHoldCollection,Paintings,100.0,21.0,False,False,Fake,
HouseHoldCollection,Paintings,100.0,21.0,False,False,,Report #2652
HouseHoldCollection,Paintings,100.0,21.0,False,False,,
HouseHoldCollection,Paintings,100.0,99.0,True,True,Genuine,Report #2653
HouseHoldCollection,Paintings,100.0,99.0,True,True,Genuine,
HouseHoldCollection,Paintings,100.0,99.0,True,True,Fake,Report #2654
HouseHoldCollection,Paintings,100.0,99.0,True,True,Fake,
HouseHoldCollection,Paintings,100.0,99.0,True,True,,Report #2655
HouseHoldCollection,Paintings,100.0,99.0,True,True,,
HouseHoldCollection,Paintings,100.0,99.0,True,False,Genuine,Report #2656
HouseHoldCollection,Paintings,100.0,99.0,True,False,Genuine,
HouseHoldCollection,Paintings,100.0,99.0,True,False,Fake,Report #2657
HouseHoldCollection,Paintings,100.0,99.0,True,False,Fake,
HouseHoldCollection,Paintings,100.0,99.0,True,False,,Report #2658
HouseHoldCollection,Paintings,100.0,99.0,True,False,,
HouseHoldCollection,Paintings,100.0,99.0,False,True,Genuine,Report #2659
HouseHoldCollection,Paintings,100.0,99.0,False,True,Genuine,
HouseHoldCollection,Paintings,100.0,99.0,False,True,Fake,Report #2660
HouseHoldCollection,Paintings,100.0,99.0,False,True,Fake,
HouseHoldCollection,Paintings,100.0,99.0,False,True,,Report #2661
HouseHoldCollection,Paintings,100.0,99.0,False,True,,
HouseHoldCollection,Paintings,100.0,99.0,False,False,Genuine,Report #2662
HouseHoldCollection,Paintings,100.0,99.0,False,False,Genuine,
HouseHoldCollection,Paintings,100.0,99.0,False,False,Fake,Report #2663
HouseHoldCollection,Paintings,100.0,99.0,False,False,Fake,
HouseHoldCollection,Paintings,100.0,99.0,False,False,,Report #2664
HouseHoldCollection,Paintings,100.0,99.0,False,False,,
HouseHoldCollection,Paintings,100.0,100.0,True,True,Genuine,Report #2665
HouseHoldCollection,Paintings,100.0,100.0,True,True,Genuine,
HouseHoldCollection,Paintings,100.0,100.0,True,True,Fake,Report #2666
HouseHoldCollection,Paintings,100.0,100.0,True,True,Fake,
HouseHoldCollection,Paintings,100.0,100.0,True,True,,Report #2667
HouseHoldCollection,Paintings,100.0,100.0,True,True,,
HouseHoldCollection,Paintings,100.0,100.0,True,False,Genuine,Report #2668
HouseHoldCollection,Paintings,100.0,100.0,True,False,Genuine,
HouseHoldCollection,Paintings,100.0,100.0,True,False,Fake,Report #2669
HouseHoldCollection,Paintings,100.0,100.0,True,False,Fake,
HouseHoldCollection,Paintings,100.0,100.0,True,False,,Report #2670
HouseHoldCollection,Paintings,100.0,100.0,True,False,,
HouseHoldCollection,Paintings,100.0,100.0,False,True,Genuine,Report #2671
HouseHoldCollection,Paintings,100.0,100.0,False,True,Genuine,
HouseHoldCollection,Paintings,100.0,100.0,False,True,Fake,Report #2672
HouseHoldCollection,Paintings,100.0,100.0,False,True,Fake,
HouseHoldCollection,Paintings,100.0,100.0,False,True,,Report #2673
HouseHoldCollection,Paintings,100.0,100.0,False,True,,
HouseHoldCollection,Paintings,100.0,100.0,False,False,Genuine,Report #2674
HouseHoldCollection,Paintings,100.0,100.0,False,False,Genuine,
HouseHoldCollection,Paintings,100.0,100.0,False,False,Fake,Report #2675
HouseHoldCollection,Paintings,100.0,100.0,False,False,Fake,
HouseHoldCollection,Paintings,100.0,100.0,False,False,,Report #2676
HouseHoldCollection,Paintings,100.0,100.0,False,False,,
HouseHoldCollection,Paintings,100.0,101.0,True,True,Genuine,Report #2677
HouseHoldCollection,Paintings,100.0,101.0,True,True,Genuine,
HouseHoldCollection,Paintings,100.0,101.0,True,True,Fake,Report #2678
HouseHoldCollection,Paintings,100.0,101.0,True,True,Fake,
HouseHoldCollection,Paintings,100.0,101.0,True,True,,Report #2679
HouseHoldCollection,Paintings,100.0,101.0,True,True,,
HouseHoldCollection,Paintings,100.0,101.0,True,False,Genuine,Report #2680
HouseHoldCollection,Paintings,100.0,101.0,True,False,Genuine,
HouseHoldCollection,Paintings,100.0,101.0,True,False,Fake,Report #2681
HouseHoldCollection,Paintings,100.0,101.0,True,False,Fake,
HouseHoldCollection,Paintings,100.0,101.0,True,False,,Report #2682
HouseHoldCollection,Paintings,100.0,101.0,True,False,,
HouseHoldCollection,Paintings,100.0,101.0,False,True,Genuine,Report #2683
HouseHoldCollection,Paintings,100.0,101.0,False,True,Genuine,
HouseHoldCollection,Paintings,100.0,101.0,False,True,Fake,Report #2684
HouseHoldCollection,Paintings,100.0,101.0,False,True,Fake,
HouseHoldCollection,Paintings,100.0,101.0,False,True,,Report #2685
HouseHoldCollection,Paintings,100.0,101.0,False,True,,
HouseHoldCollection,Paintings,100.0,101.0,False,False,Genuine,Report #2686
HouseHoldCollection,Paintings,100.0,101.0,False,False,Genuine,
HouseHoldCollection,Paintings,100.0,101.0,False,False,Fake,Report #2687
HouseHoldCollection,Paintings,100.0,101.0,False,False,Fake,
HouseHoldCollection,Paintings,100.0,101.0,False,False,,Report #2688
HouseHoldCollection,Paintings,100.0,101.0,False,False,,
HouseHoldCollection,Paintings,1000.0,-1.0,True,True,Genuine,Report #2689
HouseHoldCollection,Paintings,1000.0,-1.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000.0,-1.0,True,True,Fake,Report #2690
HouseHoldCollection,Paintings,1000.0,-1.0,True,True,Fake,
HouseHoldCollection,Paintings,1000.0,-1.0,True,True,,Report #2691
HouseHoldCollection,Paintings,1000.0,-1.0,True,True,,
HouseHoldCollection,Paintings,1000.0,-1.0,True,False,Genuine,Report #2692
HouseHoldCollection,Paintings,1000.0,-1.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000.0,-1.0,True,False,Fake,Report #2693
HouseHoldCollection,Paintings,1000.0,-1.0,True,False,Fake,
HouseHoldCollection,Paintings,1000.0,-1.0,True,False,,Report #2694
HouseHoldCollection,Paintings,1000.0,-1.0,True,False,,
HouseHoldCollection,Paintings,1000.0,-1.0,False,True,Genuine,Report #2695
HouseHoldCollection,Paintings,1000.0,-1.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000.0,-1.0,False,True,Fake,Report #2696
HouseHoldCollection,Paintings,1000.0,-1.0,False,True,Fake,
HouseHoldCollection,Paintings,1000.0,-1.0,False,True,,Report #2697
HouseHoldCollection,Paintings,1000.0,-1.0,False,True,,
HouseHoldCollection,Paintings,1000.0,-1.0,False,False,Genuine,Report #2698
HouseHoldCollection,Paintings,1000.0,-1.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000.0,-1.0,False,False,Fake,Report #2699
HouseHoldCollection,Paintings,1000.0,-1.0,False,False,Fake,
HouseHoldCollection,Paintings,1000.0,-1.0,False,False,,Report #2700
HouseHoldCollection,Paintings,1000.0,-1.0,False,False,,
HouseHoldCollection,Paintings,1000.0,0.0,True,True,Genuine,Report #2701
HouseHoldCollection,Paintings,1000.0,0.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000.0,0.0,True,True,Fake,Report #2702
HouseHoldCollection,Paintings,1000.0,0.0,True,True,Fake,
HouseHoldCollection,Paintings,1000.0,0.0,True,True,,Report #2703
HouseHoldCollection,Paintings,1000.0,0.0,True,True,,
HouseHoldCollection,Paintings,1000.0,0.0,True,False,Genuine,Report #2704
HouseHoldCollection,Paintings,1000.0,0.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000.0,0.0,True,False,Fake,Report #2705
HouseHoldCollection,Paintings,1000.0,0.0,True,False,Fake,
HouseHoldCollection,Paintings,1000.0,0.0,True,False,,Report #2706
HouseHoldCollection,Paintings,1000.0,0.0,True,False,,
HouseHoldCollection,Paintings,1000.0,0.0,False,True,Genuine,Report #2707
HouseHoldCollection,Paintings,1000.0,0.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000.0,0.0,False,True,Fake,Report #2708
HouseHoldCollection,Paintings,1000.0,0.0,False,True,Fake,
HouseHoldCollection,Paintings,1000.0,0.0,False,True,,Report #2709
HouseHoldCollection,Paintings,1000.0,0.0,False,True,,
HouseHoldCollection,Paintings,1000.0,0.0,False,False,Genuine,Report #2710
HouseHoldCollection,Paintings,1000.0,0.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000.0,0.0,False,False,Fake,Report #2711
HouseHoldCollection,Paintings,1000.0,0.0,False,False,Fake,
HouseHoldCollection,Paintings,1000.0,0.0,False,False,,Report #2712
HouseHoldCollection,Paintings,1000.0,0.0,False,False,,
HouseHoldCollection,Paintings,1000.0,19.0,True,True,Genuine,Report #2713
HouseHoldCollection,Paintings,1000.0,19.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000.0,19.0,True,True,Fake,Report #2714
HouseHoldCollection,Paintings,1000.0,19.0,True,True,Fake,
HouseHoldCollection,Paintings,1000.0,19.0,True,True,,Report #2715
HouseHoldCollection,Paintings,1000.0,19.0,True,True,,
HouseHoldCollection,Paintings,1000.0,19.0,True,False,Genuine,Report #2716
HouseHoldCollection,Paintings,1000.0,19.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000.0,19.0,True,False,Fake,Report #2717
HouseHoldCollection,Paintings,1000.0,19.0,True,False,Fake,
HouseHoldCollection,Paintings,1000.0,19.0,True,False,,Report #2718
HouseHoldCollection,Paintings,1000.0,19.0,True,False,,
HouseHoldCollection,Paintings,1000.0,19.0,False,True,Genuine,Report #2719
HouseHoldCollection,Paintings,1000.0,19.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000.0,19.0,False,True,Fake,Report #2720
HouseHoldCollection,Paintings,1000.0,19.0,False,True,Fake,
HouseHoldCollection,Paintings,1000.0,19.0,False,True,,Report #2721
HouseHoldCollection,Paintings,1000.0,19.0,False,True,,
HouseHoldCollection,Paintings,1000.0,19.0,False,False,Genuine,Report #2722
HouseHoldCollection,Paintings,1000.0,19.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000.0,19.0,False,False,Fake,Report #2723
HouseHoldCollection,Paintings,1000.0,19.0,False,False,Fake,
HouseHoldCollection,Paintings,1000.0,19.0,False,False,,Report #2724
HouseHoldCollection,Paintings,1000.0,19.0,False,False,,
HouseHoldCollection,Paintings,1000.0,20.0,True,True,Genuine,Report #2725
HouseHoldCollection,Paintings,1000.0,20.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000.0,20.0,True,True,Fake,Report #2726
HouseHoldCollection,Paintings,1000.0,20.0,True,True,Fake,
HouseHoldCollection,Paintings,1000.0,20.0,True,True,,Report #2727
HouseHoldCollection,Paintings,1000.0,20.0,True,True,,
HouseHoldCollection,Paintings,1000.0,20.0,True,False,Genuine,Report #2728
HouseHoldCollection,Paintings,1000.0,20.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000.0,20.0,True,False,Fake,Report #2729
HouseHoldCollection,Paintings,1000.0,20.0,True,False,Fake,
HouseHoldCollection,Paintings,1000.0,20.0,True,False,,Report #2730
HouseHoldCollection,Paintings,1000.0,20.0,True,False,,
HouseHoldCollection,Paintings,1000.0,20.0,False,True,Genuine,Report #2731
HouseHoldCollection,Paintings,1000.0,20.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000.0,20.0,False,True,Fake,Report #2732
HouseHoldCollection,Paintings,1000.0,20.0,False,True,Fake,
HouseHoldCollection,Paintings,1000.0,20.0,False,True,,Report #2733
HouseHoldCollection,Paintings,1000.0,20.0,False,True,,
HouseHoldCollection,Paintings,1000.0,20.0,False,False,Genuine,Report #2734
HouseHoldCollection,Paintings,1000.0,20.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000.0,20.0,False,False,Fake,Report #2735
HouseHoldCollection,Paintings,1000.0,20.0,False,False,Fake,
HouseHoldCollection,Paintings,1000.0,20.0,False,False,,Report #2736
HouseHoldCollection,Paintings,1000.0,20.0,False,False,,
HouseHoldCollection,Paintings,1000.0,21.0,True,True,Genuine,Report #2737
HouseHoldCollection,Paintings,1000.0,21.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000.0,21.0,True,True,Fake,Report #2738
HouseHoldCollection,Paintings,1000.0,21.0,True,True,Fake,
HouseHoldCollection,Paintings,1000.0,21.0,True,True,,Report #2739
HouseHoldCollection,Paintings,1000.0,21.0,True,True,,
HouseHoldCollection,Paintings,1000.0,21.0,True,False,Genuine,Report #2740
HouseHoldCollection,Paintings,1000.0,21.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000.0,21.0,True,False,Fake,Report #2741
HouseHoldCollection,Paintings,1000.0,21.0,True,False,Fake,
HouseHoldCollection,Paintings,1000.0,21.0,True,False,,Report #2742
HouseHoldCollection,Paintings,1000.0,21.0,True,False,,
HouseHoldCollection,Paintings,1000.0,21.0,False,True,Genuine,Report #2743
HouseHoldCollection,Paintings,1000.0,21.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000.0,21.0,False,True,Fake,Report #2744
HouseHoldCollection,Paintings,1000.0,21.0,False,True,Fake,
HouseHoldCollection,Paintings,1000.0,21.0,False,True,,Report #2745
HouseHoldCollection,Paintings,1000.0,21.0,False,True,,
HouseHoldCollection,Paintings,1000.0,21.0,False,False,Genuine,Report #2746
HouseHoldCollection,Paintings,1000.0,21.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000.0,21.0,False,False,Fake,Report #2747
HouseHoldCollection,Paintings,1000.0,21.0,False,False,Fake,
HouseHoldCollection,Paintings,1000.0,21.0,False,False,,Report #2748
HouseHoldCollection,Paintings,1000.0,21.0,False,False,,
HouseHoldCollection,Paintings,1000.0,99.0,True,True,Genuine,Report #2749
HouseHoldCollection,Paintings,1000.0,99.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000.0,99.0,True,True,Fake,Report #2750
HouseHoldCollection,Paintings,1000.0,99.0,True,True,Fake,
HouseHoldCollection,Paintings,1000.0,99.0,True,True,,Report #2751
HouseHoldCollection,Paintings,1000.0,99.0,True,True,,
HouseHoldCollection,Paintings,1000.0,99.0,True,False,Genuine,Report #2752
HouseHoldCollection,Paintings,1000.0,99.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000.0,99.0,True,False,Fake,Report #2753
HouseHoldCollection,Paintings,1000.0,99.0,True,False,Fake,
HouseHoldCollection,Paintings,1000.0,99.0,True,False,,Report #2754
HouseHoldCollection,Paintings,1000.0,99.0,True,False,,
HouseHoldCollection,Paintings,1000.0,99.0,False,True,Genuine,Report #2755
HouseHoldCollection,Paintings,1000.0,99.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000.0,99.0,False,True,Fake,Report #2756
HouseHoldCollection,Paintings,1000.0,99.0,False,True,Fake,
HouseHoldCollection,Paintings,1000.0,99.0,False,True,,Report #2757
HouseHoldCollection,Paintings,1000.0,99.0,False,True,,
HouseHoldCollection,Paintings,1000.0,99.0,False,False,Genuine,Report #2758
HouseHoldCollection,Paintings,1000.0,99.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000.0,99.0,False,False,Fake,Report #2759
HouseHoldCollection,Paintings,1000.0,99.0,False,False,Fake,
HouseHoldCollection,Paintings,1000.0,99.0,False,False,,Report #2760
HouseHoldCollection,Paintings,1000.0,99.0,False,False,,
HouseHoldCollection,Paintings,1000.0,100.0,True,True,Genuine,Report #2761
HouseHoldCollection,Paintings,1000.0,100.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000.0,100.0,True,True,Fake,Report #2762
HouseHoldCollection,Paintings,1000.0,100.0,True,True,Fake,
HouseHoldCollection,Paintings,1000.0,100.0,True,True,,Report #2763
HouseHoldCollection,Paintings,1000.0,100.0,True,True,,
HouseHoldCollection,Paintings,1000.0,100.0,True,False,Genuine,Report #2764
HouseHoldCollection,Paintings,1000.0,100.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000.0,100.0,True,False,Fake,Report #2765
HouseHoldCollection,Paintings,1000.0,100.0,True,False,Fake,
HouseHoldCollection,Paintings,1000.0,100.0,True,False,,Report #2766
HouseHoldCollection,Paintings,1000.0,100.0,True,False,,
HouseHoldCollection,Paintings,1000.0,100.0,False,True,Genuine,Report #2767
HouseHoldCollection,Paintings,1000.0,100.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000.0,100.0,False,True,Fake,Report #2768
HouseHoldCollection,Paintings,1000.0,100.0,False,True,Fake,
HouseHoldCollection,Paintings,1000.0,100.0,False,True,,Report #2769
HouseHoldCollection,Paintings,1000.0,100.0,False,True,,
HouseHoldCollection,Paintings,1000.0,100.0,False,False,Genuine,Report #2770
HouseHoldCollection,Paintings,1000.0,100.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000.0,100.0,False,False,Fake,Report #2771
HouseHoldCollection,Paintings,1000.0,100.0,False,False,Fake,
HouseHoldCollection,Paintings,1000.0,100.0,False,False,,Report #2772
HouseHoldCollection,Paintings,1000.0,100.0,False,False,,
HouseHoldCollection,Paintings,1000.0,101.0,True,True,Genuine,Report #2773
HouseHoldCollection,Paintings,1000.0,101.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000.0,101.0,True,True,Fake,Report #2774
HouseHoldCollection,Paintings,1000.0,101.0,True,True,Fake,
HouseHoldCollection,Paintings,1000.0,101.0,True,True,,Report #2775
HouseHoldCollection,Paintings,1000.0,101.0,True,True,,
HouseHoldCollection,Paintings,1000.0,101.0,True,False,Genuine,Report #2776
HouseHoldCollection,Paintings,1000.0,101.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000.0,101.0,True,False,Fake,Report #2777
HouseHoldCollection,Paintings,1000.0,101.0,True,False,Fake,
HouseHoldCollection,Paintings,1000.0,101.0,True,False,,Report #2778
HouseHoldCollection,Paintings,1000.0,101.0,True,False,,
HouseHoldCollection,Paintings,1000.0,101.0,False,True,Genuine,Report #2779
HouseHoldCollection,Paintings,1000.0,101.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000.0,101.0,False,True,Fake,Report #2780
HouseHoldCollection,Paintings,1000.0,101.0,False,True,Fake,
HouseHoldCollection,Paintings,1000.0,101.0,False,True,,Report #2781
HouseHoldCollection,Paintings,1000.0,101.0,False,True,,
HouseHoldCollection,Paintings,1000.0,101.0,False,False,Genuine,Report #2782
HouseHoldCollection,Paintings,1000.0,101.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000.0,101.0,False,False,Fake,Report #2783
HouseHoldCollection,Paintings,1000.0,101.0,False,False,Fake,
HouseHoldCollection,Paintings,1000.0,101.0,False,False,,Report #2784
HouseHoldCollection,Paintings,1000.0,101.0,False,False,,
HouseHoldCollection,Paintings,1000000.0,-1.0,True,True,Genuine,Report #2785
HouseHoldCollection,Paintings,1000000.0,-1.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,-1.0,True,True,Fake,Report #2786
HouseHoldCollection,Paintings,1000000.0,-1.0,True,True,Fake,
HouseHoldCollection,Paintings,1000000.0,-1.0,True,True,,Report #2787
HouseHoldCollection,Paintings,1000000.0,-1.0,True,True,,
HouseHoldCollection,Paintings,1000000.0,-1.0,True,False,Genuine,Report #2788
HouseHoldCollection,Paintings,1000000.0,-1.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,-1.0,True,False,Fake,Report #2789
HouseHoldCollection,Paintings,1000000.0,-1.0,True,False,Fake,
HouseHoldCollection,Paintings,1000000.0,-1.0,True,False,,Report #2790
HouseHoldCollection,Paintings,1000000.0,-1.0,True,False,,
HouseHoldCollection,Paintings,1000000.0,-1.0,False,True,Genuine,Report #2791
HouseHoldCollection,Paintings,1000000.0,-1.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,-1.0,False,True,Fake,Report #2792
HouseHoldCollection,Paintings,1000000.0,-1.0,False,True,Fake,
HouseHoldCollection,Paintings,1000000.0,-1.0,False,True,,Report #2793
HouseHoldCollection,Paintings,1000000.0,-1.0,False,True,,
HouseHoldCollection,Paintings,1000000.0,-1.0,False,False,Genuine,Report #2794
HouseHoldCollection,Paintings,1000000.0,-1.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,-1.0,False,False,Fake,Report #2795
HouseHoldCollection,Paintings,1000000.0,-1.0,False,False,Fake,
HouseHoldCollection,Paintings,1000000.0,-1.0,False,False,,Report #2796
HouseHoldCollection,Paintings,1000000.0,-1.0,False,False,,
HouseHoldCollection,Paintings,1000000.0,0.0,True,True,Genuine,Report #2797
HouseHoldCollection,Paintings,1000000.0,0.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,0.0,True,True,Fake,Report #2798
HouseHoldCollection,Paintings,1000000.0,0.0,True,True,Fake,
HouseHoldCollection,Paintings,1000000.0,0.0,True,True,,Report #2799
HouseHoldCollection,Paintings,1000000.0,0.0,True,True,,
HouseHoldCollection,Paintings,1000000.0,0.0,True,False,Genuine,Report #2800
HouseHoldCollection,Paintings,1000000.0,0.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,0.0,True,False,Fake,Report #2801
HouseHoldCollection,Paintings,1000000.0,0.0,True,False,Fake,
HouseHoldCollection,Paintings,1000000.0,0.0,True,False,,Report #2802
HouseHoldCollection,Paintings,1000000.0,0.0,True,False,,
HouseHoldCollection,Paintings,1000000.0,0.0,False,True,Genuine,Report #2803
HouseHoldCollection,Paintings,1000000.0,0.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,0.0,False,True,Fake,Report #2804
HouseHoldCollection,Paintings,1000000.0,0.0,False,True,Fake,
HouseHoldCollection,Paintings,1000000.0,0.0,False,True,,Report #2805
HouseHoldCollection,Paintings,1000000.0,0.0,False,True,,
HouseHoldCollection,Paintings,1000000.0,0.0,False,False,Genuine,Report #2806
HouseHoldCollection,Paintings,1000000.0,0.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,0.0,False,False,Fake,Report #2807
HouseHoldCollection,Paintings,1000000.0,0.0,False,False,Fake,
HouseHoldCollection,Paintings,1000000.0,0.0,False,False,,Report #2808
HouseHoldCollection,Paintings,1000000.0,0.0,False,False,,
HouseHoldCollection,Paintings,1000000.0,19.0,True,True,Genuine,Report #2809
HouseHoldCollection,Paintings,1000000.0,19.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,19.0,True,True,Fake,Report #2810
HouseHoldCollection,Paintings,1000000.0,19.0,True,True,Fake,
HouseHoldCollection,Paintings,1000000.0,19.0,True,True,,Report #2811
HouseHoldCollection,Paintings,1000000.0,19.0,True,True,,
HouseHoldCollection,Paintings,1000000.0,19.0,True,False,Genuine,Report #2812
HouseHoldCollection,Paintings,1000000.0,19.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,19.0,True,False,Fake,Report #2813
HouseHoldCollection,Paintings,1000000.0,19.0,True,False,Fake,
HouseHoldCollection,Paintings,1000000.0,19.0,True,False,,Report #2814
HouseHoldCollection,Paintings,1000000.0,19.0,True,False,,
HouseHoldCollection,Paintings,1000000.0,19.0,False,True,Genuine,Report #2815
HouseHoldCollection,Paintings,1000000.0,19.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,19.0,False,True,Fake,Report #2816
HouseHoldCollection,Paintings,1000000.0,19.0,False,True,Fake,
HouseHoldCollection,Paintings,1000000.0,19.0,False,True,,Report #2817
HouseHoldCollection,Paintings,1000000.0,19.0,False,True,,
HouseHoldCollection,Paintings,1000000.0,19.0,False,False,Genuine,Report #2818
HouseHoldCollection,Paintings,1000000.0,19.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,19.0,False,False,Fake,Report #2819
HouseHoldCollection,Paintings,1000000.0,19.0,False,False,Fake,
HouseHoldCollection,Paintings,1000000.0,19.0,False,False,,Report #2820
HouseHoldCollection,Paintings,1000000.0,19.0,False,False,,
HouseHoldCollection,Paintings,1000000.0,20.0,True,True,Genuine,Report #2821
HouseHoldCollection,Paintings,1000000.0,20.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,20.0,True,True,Fake,Report #2822
HouseHoldCollection,Paintings,1000000.0,20.0,True,True,Fake,
HouseHoldCollection,Paintings,1000000.0,20.0,True,True,,Report #2823
HouseHoldCollection,Paintings,1000000.0,20.0,True,True,,
HouseHoldCollection,Paintings,1000000.0,20.0,True,False,Genuine,Report #2824
HouseHoldCollection,Paintings,1000000.0,20.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,20.0,True,False,Fake,Report #2825
HouseHoldCollection,Paintings,1000000.0,20.0,True,False,Fake,
HouseHoldCollection,Paintings,1000000.0,20.0,True,False,,Report #2826
HouseHoldCollection,Paintings,1000000.0,20.0,True,False,,
HouseHoldCollection,Paintings,1000000.0,20.0,False,True,Genuine,Report #2827
HouseHoldCollection,Paintings,1000000.0,20.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,20.0,False,True,Fake,Report #2828
HouseHoldCollection,Paintings,1000000.0,20.0,False,True,Fake,
HouseHoldCollection,Paintings,1000000.0,20.0,False,True,,Report #2829
HouseHoldCollection,Paintings,1000000.0,20.0,False,True,,
HouseHoldCollection,Paintings,1000000.0,20.0,False,False,Genuine,Report #2830
HouseHoldCollection,Paintings,1000000.0,20.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,20.0,False,False,Fake,Report #2831
HouseHoldCollection,Paintings,1000000.0,20.0,False,False,Fake,
HouseHoldCollection,Paintings,1000000.0,20.0,False,False,,Report #2832
HouseHoldCollection,Paintings,1000000.0,20.0,False,False,,
HouseHoldCollection,Paintings,1000000.0,21.0,True,True,Genuine,Report #2833
HouseHoldCollection,Paintings,1000000.0,21.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,21.0,True,True,Fake,Report #2834
HouseHoldCollection,Paintings,1000000.0,21.0,True,True,Fake,
HouseHoldCollection,Paintings,1000000.0,21.0,True,True,,Report #2835
HouseHoldCollection,Paintings,1000000.0,21.0,True,True,,
HouseHoldCollection,Paintings,1000000.0,21.0,True,False,Genuine,Report #2836
HouseHoldCollection,Paintings,1000000.0,21.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,21.0,True,False,Fake,Report #2837
HouseHoldCollection,Paintings,1000000.0,21.0,True,False,Fake,
HouseHoldCollection,Paintings,1000000.0,21.0,True,False,,Report #2838
HouseHoldCollection,Paintings,1000000.0,21.0,True,False,,
HouseHoldCollection,Paintings,1000000.0,21.0,False,True,Genuine,Report #2839
HouseHoldCollection,Paintings,1000000.0,21.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,21.0,False,True,Fake,Report #2840
HouseHoldCollection,Paintings,1000000.0,21.0,False,True,Fake,
HouseHoldCollection,Paintings,1000000.0,21.0,False,True,,Report #2841
HouseHoldCollection,Paintings,1000000.0,21.0,False,True,,
HouseHoldCollection,Paintings,1000000.0,21.0,False,False,Genuine,Report #2842
HouseHoldCollection,Paintings,1000000.0,21.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,21.0,False,False,Fake,Report #2843
HouseHoldCollection,Paintings,1000000.0,21.0,False,False,Fake,
HouseHoldCollection,Paintings,1000000.0,21.0,False,False,,Report #2844
HouseHoldCollection,Paintings,1000000.0,21.0,False,False,,
HouseHoldCollection,Paintings,1000000.0,99.0,True,True,Genuine,Report #2845
HouseHoldCollection,Paintings,1000000.0,99.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,99.0,True,True,Fake,Report #2846
HouseHoldCollection,Paintings,1000000.0,99.0,True,True,Fake,
HouseHoldCollection,Paintings,1000000.0,99.0,True,True,,Report #2847
HouseHoldCollection,Paintings,1000000.0,99.0,True,True,,
HouseHoldCollection,Paintings,1000000.0,99.0,True,False,Genuine,Report #2848
HouseHoldCollection,Paintings,1000000.0,99.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,99.0,True,False,Fake,Report #2849
HouseHoldCollection,Paintings,1000000.0,99.0,True,False,Fake,
HouseHoldCollection,Paintings,1000000.0,99.0,True,False,,Report #2850
HouseHoldCollection,Paintings,1000000.0,99.0,True,False,,
HouseHoldCollection,Paintings,1000000.0,99.0,False,True,Genuine,Report #2851
HouseHoldCollection,Paintings,1000000.0,99.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,99.0,False,True,Fake,Report #2852
HouseHoldCollection,Paintings,1000000.0,99.0,False,True,Fake,
HouseHoldCollection,Paintings,1000000.0,99.0,False,True,,Report #2853
HouseHoldCollection,Paintings,1000000.0,99.0,False,True,,
HouseHoldCollection,Paintings,1000000.0,99.0,False,False,Genuine,Report #2854
HouseHoldCollection,Paintings,1000000.0,99.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,99.0,False,False,Fake,Report #2855
HouseHoldCollection,Paintings,1000000.0,99.0,False,False,Fake,
HouseHoldCollection,Paintings,1000000.0,99.0,False,False,,Report #2856
HouseHoldCollection,Paintings,1000000.0,99.0,False,False,,
HouseHoldCollection,Paintings,1000000.0,100.0,True,True,Genuine,Report #2857
HouseHoldCollection,Paintings,1000000.0,100.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,100.0,True,True,Fake,Report #2858
HouseHoldCollection,Paintings,1000000.0,100.0,True,True,Fake,
HouseHoldCollection,Paintings,1000000.0,100.0,True,True,,Report #2859
HouseHoldCollection,Paintings,1000000.0,100.0,True,True,,
HouseHoldCollection,Paintings,1000000.0,100.0,True,False,Genuine,Report #2860
HouseHoldCollection,Paintings,1000000.0,100.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,100.0,True,False,Fake,Report #2861
HouseHoldCollection,Paintings,1000000.0,100.0,True,False,Fake,
HouseHoldCollection,Paintings,1000000.0,100.0,True,False,,Report #2862
HouseHoldCollection,Paintings,1000000.0,100.0,True,False,,
HouseHoldCollection,Paintings,1000000.0,100.0,False,True,Genuine,Report #2863
HouseHoldCollection,Paintings,1000000.0,100.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,100.0,False,True,Fake,Report #2864
HouseHoldCollection,Paintings,1000000.0,100.0,False,True,Fake,
HouseHoldCollection,Paintings,1000000.0,100.0,False,True,,Report #2865
HouseHoldCollection,Paintings,1000000.0,100.0,False,True,,
HouseHoldCollection,Paintings,1000000.0,100.0,False,False,Genuine,Report #2866
HouseHoldCollection,Paintings,1000000.0,100.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,100.0,False,False,Fake,Report #2867
HouseHoldCollection,Paintings,1000000.0,100.0,False,False,Fake,
HouseHoldCollection,Paintings,1000000.0,100.0,False,False,,Report #2868
HouseHoldCollection,Paintings,1000000.0,100.0,False,False,,
HouseHoldCollection,Paintings,1000000.0,101.0,True,True,Genuine,Report #2869
HouseHoldCollection,Paintings,1000000.0,101.0,True,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,101.0,True,True,Fake,Report #2870
HouseHoldCollection,Paintings,1000000.0,101.0,True,True,Fake,
HouseHoldCollection,Paintings,1000000.0,101.0,True,True,,Report #2871
HouseHoldCollection,Paintings,1000000.0,101.0,True,True,,
HouseHoldCollection,Paintings,1000000.0,101.0,True,False,Genuine,Report #2872
HouseHoldCollection,Paintings,1000000.0,101.0,True,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,101.0,True,False,Fake,Report #2873
HouseHoldCollection,Paintings,1000000.0,101.0,True,False,Fake,
HouseHoldCollection,Paintings,1000000.0,101.0,True,False,,Report #2874
HouseHoldCollection,Paintings,1000000.0,101.0,True,False,,
HouseHoldCollection,Paintings,1000000.0,101.0,False,True,Genuine,Report #2875
HouseHoldCollection,Paintings,1000000.0,101.0,False,True,Genuine,
HouseHoldCollection,Paintings,1000000.0,101.0,False,True,Fake,Report #2876
HouseHoldCollection,Paintings,1000000.0,101.0,False,True,Fake,
HouseHoldCollection,Paintings,1000000.0,101.0,False,True,,Report #2877
HouseHoldCollection,Paintings,1000000.0,101.0,False,True,,
HouseHoldCollection,Paintings,1000000.0,101.0,False,False,Genuine,Report #2878
HouseHoldCollection,Paintings,1000000.0,101.0,False,False,Genuine,
HouseHoldCollection,Paintings,1000000.0,101.0,False,False,Fake,Report #2879
HouseHoldCollection,Paintings,1000000.0,101.0,False,False,Fake,
HouseHoldCollection,Paintings,1000000.0,101.0,False,False,,Report #2880
HouseHoldCollection,Paintings,1000000.0,101.0,False,False,,
HouseHoldCollection,Knife,0.0,-1.0,True,True,Genuine,Report #2881
HouseHoldCollection,Knife,0.0,-1.0,True,True,Genuine,
HouseHoldCollection,Knife,0.0,-1.0,True,True,Fake,Report #2882
HouseHoldCollection,Knife,0.0,-1.0,True,True,Fake,
HouseHoldCollection,Knife,0.0,-1.0,True,True,,Report #2883
HouseHoldCollection,Knife,0.0,-1.0,True,True,,
HouseHoldCollection,Knife,0.0,-1.0,True,False,Genuine,Report #2884
HouseHoldCollection,Knife,0.0,-1.0,True,False,Genuine,
HouseHoldCollection,Knife,0.0,-1.0,True,False,Fake,Report #2885
HouseHoldCollection,Knife,0.0,-1.0,True,False,Fake,
HouseHoldCollection,Knife,0.0,-1.0,True,False,,Report #2886
HouseHoldCollection,Knife,0.0,-1.0,True,False,,
HouseHoldCollection,Knife,0.0,-1.0,False,True,Genuine,Report #2887
HouseHoldCollection,Knife,0.0,-1.0,False,True,Genuine,
HouseHoldCollection,Knife,0.0,-1.0,False,True,Fake,Report #2888
HouseHoldCollection,Knife,0.0,-1.0,False,True,Fake,
HouseHoldCollection,Knife,0.0,-1.0,False,True,,Report #2889
HouseHoldCollection,Knife,0.0,-1.0,False,True,,
HouseHoldCollection,Knife,0.0,-1.0,False,False,Genuine,Report #2890
HouseHoldCollection,Knife,0.0,-1.0,False,False,Genuine,
HouseHoldCollection,Knife,0.0,-1.0,False,False,Fake,Report #2891
HouseHoldCollection,Knife,0.0,-1.0,False,False,Fake,
HouseHoldCollection,Knife,0.0,-1.0,False,False,,Report #2892
HouseHoldCollection,Knife,0.0,-1.0,False,False,,
HouseHoldCollection,Knife,0.0,0.0,True,True,Genuine,Report #2893
HouseHoldCollection,Knife,0.0,0.0,True,True,Genuine,
HouseHoldCollection,Knife,0.0,0.0,True,True,Fake,Report #2894
HouseHoldCollection,Knife,0.0,0.0,True,True,Fake,
HouseHoldCollection,Knife,0.0,0.0,True,True,,Report #2895
HouseHoldCollection,Knife,0.0,0.0,True,True,,
HouseHoldCollection,Knife,0.0,0.0,True,False,Genuine,Report #2896
HouseHoldCollection,Knife,0.0,0.0,True,False,Genuine,
HouseHoldCollection,Knife,0.0,0.0,True,False,Fake,Report #2897
HouseHoldCollection,Knife,0.0,0.0,True,False,Fake,
HouseHoldCollection,Knife,0.0,0.0,True,False,,Report #2898
HouseHoldCollection,Knife,0.0,0.0,True,False,,
HouseHoldCollection,Knife,0.0,0.0,False,True,Genuine,Report #2899
HouseHoldCollection,Knife,0.0,0.0,False,True,Genuine,
HouseHoldCollection,Knife,0.0,0.0,False,True,Fake,Report #2900
HouseHoldCollection,Knife,0.0,0.0,False,True,Fake,
HouseHoldCollection,Knife,0.0,0.0,False,True,,Report #2901
HouseHoldCollection,Knife,0.0,0.0,False,True,,
HouseHoldCollection,Knife,0.0,0.0,False,False,Genuine,Report #2902
HouseHoldCollection,Knife,0.0,0.0,False,False,Genuine,
HouseHoldCollection,Knife,0.0,0.0,False,False,Fake,Report #2903
HouseHoldCollection,Knife,0.0,0.0,False,False,Fake,
HouseHoldCollection,Knife,0.0,0.0,False,False,,Report #2904
HouseHoldCollection,Knife,0.0,0.0,False,False,,
HouseHoldCollection,Knife,0.0,19.0,True,True,Genuine,Report #2905
HouseHoldCollection,Knife,0.0,19.0,True,True,Genuine,
HouseHoldCollection,Knife,0.0,19.0,True,True,Fake,Report #2906
HouseHoldCollection,Knife,0.0,19.0,True,True,Fake,
HouseHoldCollection,Knife,0.0,19.0,True,True,,Report #2907
HouseHoldCollection,Knife,0.0,19.0,True,True,,
HouseHoldCollection,Knife,0.0,19.0,True,False,Genuine,Report #2908
HouseHoldCollection,Knife,0.0,19.0,True,False,Genuine,
HouseHoldCollection,Knife,0.0,19.0,True,False,Fake,Report #2909
HouseHoldCollection,Knife,0.0,19.0,True,False,Fake,
HouseHoldCollection,Knife,0.0,19.0,True,False,,Report #2910
HouseHoldCollection,Knife,0.0,19.0,True,False,,
HouseHoldCollection,Knife,0.0,19.0,False,True,Genuine,Report #2911
HouseHoldCollection,Knife,0.0,19.0,False,True,Genuine,
HouseHoldCollection,Knife,0.0,19.0,False,True,Fake,Report #2912
HouseHoldCollection,Knife,0.0,19.0,False,True,Fake,
HouseHoldCollection,Knife,0.0,19.0,False,True,,Report #2913
HouseHoldCollection,Knife,0.0,19.0,False,True,,
HouseHoldCollection,Knife,0.0,19.0,False,False,Genuine,Report #2914
HouseHoldCollection,Knife,0.0,19.0,False,False,Genuine,
HouseHoldCollection,Knife,0.0,19.0,False,False,Fake,Report #2915
HouseHoldCollection,Knife,0.0,19.0,False,False,Fake,
HouseHoldCollection,Knife,0.0,19.0,False,False,,Report #2916
HouseHoldCollection,Knife,0.0,19.0,False,False,,
HouseHoldCollection,Knife,0.0,20.0,True,True,Genuine,Report #2917
HouseHoldCollection,Knife,0.0,20.0,True,True,Genuine,
HouseHoldCollection,Knife,0.0,20.0,True,True,Fake,Report #2918
HouseHoldCollection,Knife,0.0,20.0,True,True,Fake,
HouseHoldCollection,Knife,0.0,20.0,True,True,,Report #2919
HouseHoldCollection,Knife,0.0,20.0,True,True,,
HouseHoldCollection,Knife,0.0,20.0,True,False,Genuine,Report #2920
HouseHoldCollection,Knife,0.0,20.0,True,False,Genuine,
HouseHoldCollection,Knife,0.0,20.0,True,False,Fake,Report #2921
HouseHoldCollection,Knife,0.0,20.0,True,False,Fake,
HouseHoldCollection,Knife,0.0,20.0,True,False,,Report #2922
HouseHoldCollection,Knife,0.0,20.0,True,False,,
HouseHoldCollection,Knife,0.0,20.0,False,True,Genuine,Report #2923
HouseHoldCollection,Knife,0.0,20.0,False,True,Genuine,
HouseHoldCollection,Knife,0.0,20.0,False,True,Fake,Report #2924
HouseHoldCollection,Knife,0.0,20.0,False,True,Fake,
HouseHoldCollection,Knife,0.0,20.0,False,True,,Report #2925
HouseHoldCollection,Knife,0.0,20.0,False,True,,
HouseHoldCollection,Knife,0.0,20.0,False,False,Genuine,Report #2926
HouseHoldCollection,Knife,0.0,20.0,False,False,Genuine,
HouseHoldCollection,Knife,0.0,20.0,False,False,Fake,Report #2927
HouseHoldCollection,Knife,0.0,20.0,False,False,Fake,
HouseHoldCollection,Knife,0.0,20.0,False,False,,Report #2928
HouseHoldCollection,Knife,0.0,20.0,False,False,,
HouseHoldCollection,Knife,0.0,21.0,True,True,Genuine,Report #2929
HouseHoldCollection,Knife,0.0,21.0,True,True,Genuine,
HouseHoldCollection,Knife,0.0,21.0,True,True,Fake,Report #2930
HouseHoldCollection,Knife,0.0,21.0,True,True,Fake,
HouseHoldCollection,Knife,0.0,21.0,True,True,,Report #2931
HouseHoldCollection,Knife,0.0,21.0,True,True,,
HouseHoldCollection,Knife,0.0,21.0,True,False,Genuine,Report #2932
HouseHoldCollection,Knife,0.0,21.0,True,False,Genuine,
HouseHoldCollection,Knife,0.0,21.0,True,False,Fake,Report #2933
HouseHoldCollection,Knife,0.0,21.0,True,False,Fake,
HouseHoldCollection,Knife,0.0,21.0,True,False,,Report #2934
HouseHoldCollection,Knife,0.0,21.0,True,False,,
HouseHoldCollection,Knife,0.0,21.0,False,True,Genuine,Report #2935
HouseHoldCollection,Knife,0.0,21.0,False,True,Genuine,
HouseHoldCollection,Knife,0.0,21.0,False,True,Fake,Report #2936
HouseHoldCollection,Knife,0.0,21.0,False,True,Fake,
HouseHoldCollection,Knife,0.0,21.0,False,True,,Report #2937
HouseHoldCollection,Knife,0.0,21.0,False,True,,
HouseHoldCollection,Knife,0.0,21.0,False,False,Genuine,Report #2938
HouseHoldCollection,Knife,0.0,21.0,False,False,Genuine,
HouseHoldCollection,Knife,0.0,21.0,False,False,Fake,Report #2939
HouseHoldCollection,Knife,0.0,21.0,False,False,Fake,
HouseHoldCollection,Knife,0.0,21.0,False,False,,Report #2940
HouseHoldCollection,Knife,0.0,21.0,False,False,,
HouseHoldCollection,Knife,0.0,99.0,True,True,Genuine,Report #2941
HouseHoldCollection,Knife,0.0,99.0,True,True,Genuine,
HouseHoldCollection,Knife,0.0,99.0,True,True,Fake,Report #2942
HouseHoldCollection,Knife,0.0,99.0,True,True,Fake,
HouseHoldCollection,Knife,0.0,99.0,True,True,,Report #2943
HouseHoldCollection,Knife,0.0,99.0,True,True,,
HouseHoldCollection,Knife,0.0,99.0,True,False,Genuine,Report #2944
HouseHoldCollection,Knife,0.0,99.0,True,False,Genuine,
HouseHoldCollection,Knife,0.0,99.0,True,False,Fake,Report #2945
HouseHoldCollection,Knife,0.0,99.0,True,False,Fake,
HouseHoldCollection,Knife,0.0,99.0,True,False,,Report #2946
HouseHoldCollection,Knife,0.0,99.0,True,False,,
HouseHoldCollection,Knife,0.0,99.0,False,True,Genuine,Report #2947
HouseHoldCollection,Knife,0.0,99.0,False,True,Genuine,
HouseHoldCollection,Knife,0.0,99.0,False,True,Fake,Report #2948
HouseHoldCollection,Knife,0.0,99.0,False,True,Fake,
HouseHoldCollection,Knife,0.0,99.0,False,True,,Report #2949
HouseHoldCollection,Knife,0.0,99.0,False,True,,
HouseHoldCollection,Knife,0.0,99.0,False,False,Genuine,Report #2950
HouseHoldCollection,Knife,0.0,99.0,False,False,Genuine,
HouseHoldCollection,Knife,0.0,99.0,False,False,Fake,Report #2951
HouseHoldCollection,Knife,0.0,99.0,False,False,Fake,
HouseHoldCollection,Knife,0.0,99.0,False,False,,Report #2952
HouseHoldCollection,Knife,0.0,99.0,False,False,,
HouseHoldCollection,Knife,0.0,100.0,True,True,Genuine,Report #2953
HouseHoldCollection,Knife,0.0,100.0,True,True,Genuine,
HouseHoldCollection,Knife,0.0,100.0,True,True,Fake,Report #2954
HouseHoldCollection,Knife,0.0,100.0,True,True,Fake,
HouseHoldCollection,Knife,0.0,100.0,True,True,,Report #2955
HouseHoldCollection,Knife,0.0,100.0,True,True,,
HouseHoldCollection,Knife,0.0,100.0,True,False,Genuine,Report #2956
HouseHoldCollection,Knife,0.0,100.0,True,False,Genuine,
HouseHoldCollection,Knife,0.0,100.0,True,False,Fake,Report #2957
HouseHoldCollection,Knife,0.0,100.0,True,False,Fake,
HouseHoldCollection,Knife,0.0,100.0,True,False,,Report #2958
HouseHoldCollection,Knife,0.0,100.0,True,False,,
HouseHoldCollection,Knife,0.0,100.0,False,True,Genuine,Report #2959
HouseHoldCollection,Knife,0.0,100.0,False,True,Genuine,
HouseHoldCollection,Knife,0.0,100.0,False,True,Fake,Report #2960
HouseHoldCollection,Knife,0.0,100.0,False,True,Fake,
HouseHoldCollection,Knife,0.0,100.0,False,True,,Report #2961
HouseHoldCollection,Knife,0.0,100.0,False,True,,
HouseHoldCollection,Knife,0.0,100.0,False,False,Genuine,Report #2962
HouseHoldCollection,Knife,0.0,100.0,False,False,Genuine,
HouseHoldCollection,Knife,0.0,100.0,False,False,Fake,Report #2963
HouseHoldCollection,Knife,0.0,100.0,False,False,Fake,
HouseHoldCollection,Knife,0.0,100.0,False,False,,Report #2964
HouseHoldCollection,Knife,0.0,100.0,False,False,,
HouseHoldCollection,Knife,0.0,101.0,True,True,Genuine,Report #2965
HouseHoldCollection,Knife,0.0,101.0,True,True,Genuine,
HouseHoldCollection,Knife,0.0,101.0,True,True,Fake,Report #2966
HouseHoldCollection,Knife,0.0,101.0,True,True,Fake,
HouseHoldCollection,Knife,0.0,101.0,True,True,,Report #2967
HouseHoldCollection,Knife,0.0,101.0,True,True,,
HouseHoldCollection,Knife,0.0,101.0,True,False,Genuine,Report #2968
HouseHoldCollection,Knife,0.0,101.0,True,False,Genuine,
HouseHoldCollection,Knife,0.0,101.0,True,False,Fake,Report #2969
HouseHoldCollection,Knife,0.0,101.0,True,False,Fake,
HouseHoldCollection,Knife,0.0,101.0,True,False,,Report #2970
HouseHoldCollection,Knife,0.0,101.0,True,False,,
HouseHoldCollection,Knife,0.0,101.0,False,True,Genuine,Report #2971
HouseHoldCollection,Knife,0.0,101.0,False,True,Genuine,
HouseHoldCollection,Knife,0.0,101.0,False,True,Fake,Report #2972
HouseHoldCollection,Knife,0.0,101.0,False,True,Fake,
HouseHoldCollection,Knife,0.0,101.0,False,True,,Report #2973
HouseHoldCollection,Knife,0.0,101.0,False,True,,
HouseHoldCollection,Knife,0.0,101.0,False,False,Genuine,Report #2974
HouseHoldCollection,Knife,0.0,101.0,False,False,Genuine,
HouseHoldCollection,Knife,0.0,101.0,False,False,Fake,Report #2975
HouseHoldCollection,Knife,0.0,101.0,False,False,Fake,
HouseHoldCollection,Knife,0.0,101.0,False,False,,Report #2976
HouseHoldCollection,Knife,0.0,101.0,False,False,,
HouseHoldCollection,Knife,1.0,-1.0,True,True,Genuine,Report #2977
HouseHoldCollection,Knife,1.0,-1.0,True,True,Genuine,
HouseHoldCollection,Knife,1.0,-1.0,True,True,Fake,Report #2978
HouseHoldCollection,Knife,1.0,-1.0,True,True,Fake,
HouseHoldCollection,Knife,1.0,-1.0,True,True,,Report #2979
HouseHoldCollection,Knife,1.0,-1.0,True,True,,
HouseHoldCollection,Knife,1.0,-1.0,True,False,Genuine,Report #2980
HouseHoldCollection,Knife,1.0,-1.0,True,False,Genuine,
HouseHoldCollection,Knife,1.0,-1.0,True,False,Fake,Report #2981
HouseHoldCollection,Knife,1.0,-1.0,True,False,Fake,
HouseHoldCollection,Knife,1.0,-1.0,True,False,,Report #2982
HouseHoldCollection,Knife,1.0,-1.0,True,False,,
HouseHoldCollection,Knife,1.0,-1.0,False,True,Genuine,Report #2983
HouseHoldCollection,Knife,1.0,-1.0,False,True,Genuine,
HouseHoldCollection,Knife,1.0,-1.0,False,True,Fake,Report #2984
HouseHoldCollection,Knife,1.0,-1.0,False,True,Fake,
HouseHoldCollection,Knife,1.0,-1.0,False,True,,Report #2985
HouseHoldCollection,Knife,1.0,-1.0,False,True,,
HouseHoldCollection,Knife,1.0,-1.0,False,False,Genuine,Report #2986
HouseHoldCollection,Knife,1.0,-1.0,False,False,Genuine,
HouseHoldCollection,Knife,1.0,-1.0,False,False,Fake,Report #2987
HouseHoldCollection,Knife,1.0,-1.0,False,False,Fake,
HouseHoldCollection,Knife,1.0,-1.0,False,False,,Report #2988
HouseHoldCollection,Knife,1.0,-1.0,False,False,,
HouseHoldCollection,Knife,1.0,0.0,True,True,Genuine,Report #2989
HouseHoldCollection,Knife,1.0,0.0,True,True,Genuine,
HouseHoldCollection,Knife,1.0,0.0,True,True,Fake,Report #2990
HouseHoldCollection,Knife,1.0,0.0,True,True,Fake,
HouseHoldCollection,Knife,1.0,0.0,True,True,,Report #2991
HouseHoldCollection,Knife,1.0,0.0,True,True,,
HouseHoldCollection,Knife,1.0,0.0,True,False,Genuine,Report #2992
HouseHoldCollection,Knife,1.0,0.0,True,False,Genuine,
HouseHoldCollection,Knife,1.0,0.0,True,False,Fake,Report #2993
HouseHoldCollection,Knife,1.0,0.0,True,False,Fake,
HouseHoldCollection,Knife,1.0,0.0,True,False,,Report #2994
HouseHoldCollection,Knife,1.0,0.0,True,False,,
HouseHoldCollection,Knife,1.0,0.0,False,True,Genuine,Report #2995
HouseHoldCollection,Knife,1.0,0.0,False,True,Genuine,
HouseHoldCollection,Knife,1.0,0.0,False,True,Fake,Report #2996
HouseHoldCollection,Knife,1.0,0.0,False,True,Fake,
HouseHoldCollection,Knife,1.0,0.0,False,True,,Report #2997
HouseHoldCollection,Knife,1.0,0.0,False,True,,
HouseHoldCollection,Knife,1.0,0.0,False,False,Genuine,Report #2998
HouseHoldCollection,Knife,1.0,0.0,False,False,Genuine,
HouseHoldCollection,Knife,1.0,0.0,False,False,Fake,Report #2999
HouseHoldCollection,Knife,1.0,0.0,False,False,Fake,
HouseHoldCollection,Knife,1.0,0.0,False,False,,Report #3000
HouseHoldCollection,Knife,1.0,0.0,False,False,,
HouseHoldCollection,Knife,1.0,19.0,True,True,Genuine,Report #3001
HouseHoldCollection,Knife,1.0,19.0,True,True,Genuine,
HouseHoldCollection,Knife,1.0,19.0,True,True,Fake,Report #3002
HouseHoldCollection,Knife,1.0,19.0,True,True,Fake,
HouseHoldCollection,Knife,1.0,19.0,True,True,,Report #3003
HouseHoldCollection,Knife,1.0,19.0,True,True,,
HouseHoldCollection,Knife,1.0,19.0,True,False,Genuine,Report #3004
HouseHoldCollection,Knife,1.0,19.0,True,False,Genuine,
HouseHoldCollection,Knife,1.0,19.0,True,False,Fake,Report #3005
HouseHoldCollection,Knife,1.0,19.0,True,False,Fake,
HouseHoldCollection,Knife,1.0,19.0,True,False,,Report #3006
HouseHoldCollection,Knife,1.0,19.0,True,False,,
HouseHoldCollection,Knife,1.0,19.0,False,True,Genuine,Report #3007
HouseHoldCollection,Knife,1.0,19.0,False,True,Genuine,
HouseHoldCollection,Knife,1.0,19.0,False,True,Fake,Report #3008
HouseHoldCollection,Knife,1.0,19.0,False,True,Fake,
HouseHoldCollection,Knife,1.0,19.0,False,True,,Report #3009
HouseHoldCollection,Knife,1.0,19.0,False,True,,
HouseHoldCollection,Knife,1.0,19.0,False,False,Genuine,Report #3010
HouseHoldCollection,Knife,1.0,19.0,False,False,Genuine,
HouseHoldCollection,Knife,1.0,19.0,False,False,Fake,Report #3011
HouseHoldCollection,Knife,1.0,19.0,False,False,Fake,
HouseHoldCollection,Knife,1.0,19.0,False,False,,Report #3012
HouseHoldCollection,Knife,1.0,19.0,False,False,,
HouseHoldCollection,Knife,1.0,20.0,True,True,Genuine,Report #3013
HouseHoldCollection,Knife,1.0,20.0,True,True,Genuine,
HouseHoldCollection,Knife,1.0,20.0,True,True,Fake,Report #3014
HouseHoldCollection,Knife,1.0,20.0,True,True,Fake,
HouseHoldCollection,Knife,1.0,20.0,True,True,,Report #3015
HouseHoldCollection,Knife,1.0,20.0,True,True,,
HouseHoldCollection,Knife,1.0,20.0,True,False,Genuine,Report #3016
HouseHoldCollection,Knife,1.0,20.0,True,False,Genuine,
HouseHoldCollection,Knife,1.0,20.0,True,False,Fake,Report #3017
HouseHoldCollection,Knife,1.0,20.0,True,False,Fake,
HouseHoldCollection,Knife,1.0,20.0,True,False,,Report #3018
HouseHoldCollection,Knife,1.0,20.0,True,False,,
HouseHoldCollection,Knife,1.0,20.0,False,True,Genuine,Report #3019
HouseHoldCollection,Knife,1.0,20.0,False,True,Genuine,
HouseHoldCollection,Knife,1.0,20.0,False,True,Fake,Report #3020
HouseHoldCollection,Knife,1.0,20.0,False,True,Fake,
HouseHoldCollection,Knife,1.0,20.0,False,True,,Report #3021
HouseHoldCollection,Knife,1.0,20.0,False,True,,
HouseHoldCollection,Knife,1.0,20.0,False,False,Genuine,Report #3022
HouseHoldCollection,Knife,1.0,20.0,False,False,Genuine,
HouseHoldCollection,Knife,1.0,20.0,False,False,Fake,Report #3023
HouseHoldCollection,Knife,1.0,20.0,False,False,Fake,
HouseHoldCollection,Knife,1.0,20.0,False,False,,Report #3024
HouseHoldCollection,Knife,1.0,20.0,False,False,,
HouseHoldCollection,Knife,1.0,21.0,True,True,Genuine,Report #3025
HouseHoldCollection,Knife,1.0,21.0,True,True,Genuine,
HouseHoldCollection,Knife,1.0,21.0,True,True,Fake,Report #3026
HouseHoldCollection,Knife,1.0,21.0,True,True,Fake,
HouseHoldCollection,Knife,1.0,21.0,True,True,,Report #3027
HouseHoldCollection,Knife,1.0,21.0,True,True,,
HouseHoldCollection,Knife,1.0,21.0,True,False,Genuine,Report #3028
HouseHoldCollection,Knife,1.0,21.0,True,False,Genuine,
HouseHoldCollection,Knife,1.0,21.0,True,False,Fake,Report #3029
HouseHoldCollection,Knife,1.0,21.0,True,False,Fake,
HouseHoldCollection,Knife,1.0,21.0,True,False,,Report #3030
HouseHoldCollection,Knife,1.0,21.0,True,False,,
HouseHoldCollection,Knife,1.0,21.0,False,True,Genuine,Report #3031
HouseHoldCollection,Knife,1.0,21.0,False,True,Genuine,
HouseHoldCollection,Knife,1.0,21.0,False,True,Fake,Report #3032
HouseHoldCollection,Knife,1.0,21.0,False,True,Fake,
HouseHoldCollection,Knife,1.0,21.0,False,True,,Report #3033
HouseHoldCollection,Knife,1.0,21.0,False,True,,
HouseHoldCollection,Knife,1.0,21.0,False,False,Genuine,Report #3034
HouseHoldCollection,Knife,1.0,21.0,False,False,Genuine,
HouseHoldCollection,Knife,1.0,21.0,False,False,Fake,Report #3035
HouseHoldCollection,Knife,1.0,21.0,False,False,Fake,
HouseHoldCollection,Knife,1.0,21.0,False,False,,Report #3036
HouseHoldCollection,Knife,1.0,21.0,False,False,,
HouseHoldCollection,Knife,1.0,99.0,True,True,Genuine,Report #3037
HouseHoldCollection,Knife,1.0,99.0,True,True,Genuine,
HouseHoldCollection,Knife,1.0,99.0,True,True,Fake,Report #3038
HouseHoldCollection,Knife,1.0,99.0,True,True,Fake,
HouseHoldCollection,Knife,1.0,99.0,True,True,,Report #3039
HouseHoldCollection,Knife,1.0,99.0,True,True,,
HouseHoldCollection,Knife,1.0,99.0,True,False,Genuine,Report #3040
HouseHoldCollection,Knife,1.0,99.0,True,False,Genuine,
HouseHoldCollection,Knife,1.0,99.0,True,False,Fake,Report #3041
HouseHoldCollection,Knife,1.0,99.0,True,False,Fake,
HouseHoldCollection,Knife,1.0,99.0,True,False,,Report #3042
HouseHoldCollection,Knife,1.0,99.0,True,False,,
HouseHoldCollection,Knife,1.0,99.0,False,True,Genuine,Report #3043
HouseHoldCollection,Knife,1.0,99.0,False,True,Genuine,
HouseHoldCollection,Knife,1.0,99.0,False,True,Fake,Report #3044
HouseHoldCollection,Knife,1.0,99.0,False,True,Fake,
HouseHoldCollection,Knife,1.0,99.0,False,True,,Report #3045
HouseHoldCollection,Knife,1.0,99.0,False,True,,
HouseHoldCollection,Knife,1.0,99.0,False,False,Genuine,Report #3046
HouseHoldCollection,Knife,1.0,99.0,False,False,Genuine,
HouseHoldCollection,Knife,1.0,99.0,False,False,Fake,Report #3047
HouseHoldCollection,Knife,1.0,99.0,False,False,Fake,
HouseHoldCollection,Knife,1.0,99.0,False,False,,Report #3048
HouseHoldCollection,Knife,1.0,99.0,False,False,,
HouseHoldCollection,Knife,1.0,100.0,True,True,Genuine,Report #3049
HouseHoldCollection,Knife,1.0,100.0,True,True,Genuine,
HouseHoldCollection,Knife,1.0,100.0,True,True,Fake,Report #3050
HouseHoldCollection,Knife,1.0,100.0,True,True,Fake,
HouseHoldCollection,Knife,1.0,100.0,True,True,,Report #3051
HouseHoldCollection,Knife,1.0,100.0,True,True,,
HouseHoldCollection,Knife,1.0,100.0,True,False,Genuine,Report #3052
HouseHoldCollection,Knife,1.0,100.0,True,False,Genuine,
HouseHoldCollection,Knife,1.0,100.0,True,False,Fake,Report #3053
HouseHoldCollection,Knife,1.0,100.0,True,False,Fake,
HouseHoldCollection,Knife,1.0,100.0,True,False,,Report #3054
HouseHoldCollection,Knife,1.0,100.0,True,False,,
HouseHoldCollection,Knife,1.0,100.0,False,True,Genuine,Report #3055
HouseHoldCollection,Knife,1.0,100.0,False,True,Genuine,
HouseHoldCollection,Knife,1.0,100.0,False,True,Fake,Report #3056
HouseHoldCollection,Knife,1.0,100.0,False,True,Fake,
HouseHoldCollection,Knife,1.0,100.0,False,True,,Report #3057
HouseHoldCollection,Knife,1.0,100.0,False,True,,
HouseHoldCollection,Knife,1.0,100.0,False,False,Genuine,Report #3058
HouseHoldCollection,Knife,1.0,100.0,False,False,Genuine,
HouseHoldCollection,Knife,1.0,100.0,False,False,Fake,Report #3059
HouseHoldCollection,Knife,1.0,100.0,False,False,Fake,
HouseHoldCollection,Knife,1.0,100.0,False,False,,Report #3060
HouseHoldCollection,Knife,1.0,100.0,False,False,,
HouseHoldCollection,Knife,1.0,101.0,True,True,Genuine,Report #3061
HouseHoldCollection,Knife,1.0,101.0,True,True,Genuine,
HouseHoldCollection,Knife,1.0,101.0,True,True,Fake,Report #3062
HouseHoldCollection,Knife,1.0,101.0,True,True,Fake,
HouseHoldCollection,Knife,1.0,101.0,True,True,,Report #3063
HouseHoldCollection,Knife,1.0,101.0,True,True,,
HouseHoldCollection,Knife,1.0,101.0,True,False,Genuine,Report #3064
HouseHoldCollection,Knife,1.0,101.0,True,False,Genuine,
HouseHoldCollection,Knife,1.0,101.0,True,False,Fake,Report #3065
HouseHoldCollection,Knife,1.0,101.0,True,False,Fake,
HouseHoldCollection,Knife,1.0,101.0,True,False,,Report #3066
HouseHoldCollection,Knife,1.0,101.0,True,False,,
HouseHoldCollection,Knife,1.0,101.0,False,True,Genuine,Report #3067
HouseHoldCollection,Knife,1.0,101.0,False,True,Genuine,
HouseHoldCollection,Knife,1.0,101.0,False,True,Fake,Report #3068
HouseHoldCollection,Knife,1.0,101.0,False,True,Fake,
HouseHoldCollection,Knife,1.0,101.0,False,True,,Report #3069
HouseHoldCollection,Knife,1.0,101.0,False,True,,
HouseHoldCollection,Knife,1.0,101.0,False,False,Genuine,Report #3070
HouseHoldCollection,Knife,1.0,101.0,False,False,Genuine,
HouseHoldCollection,Knife,1.0,101.0,False,False,Fake,Report #3071
HouseHoldCollection,Knife,1.0,101.0,False,False,Fake,
HouseHoldCollection,Knife,1.0,101.0,False,False,,Report #3072
HouseHoldCollection,Knife,1.0,101.0,False,False,,
HouseHoldCollection,Knife,100.0,-1.0,True,True,Genuine,Report #3073
HouseHoldCollection,Knife,100.0,-1.0,True,True,Genuine,
HouseHoldCollection,Knife,100.0,-1.0,True,True,Fake,Report #3074
HouseHoldCollection,Knife,100.0,-1.0,True,True,Fake,
HouseHoldCollection,Knife,100.0,-1.0,True,True,,Report #3075
HouseHoldCollection,Knife,100.0,-1.0,True,True,,
HouseHoldCollection,Knife,100.0,-1.0,True,False,Genuine,Report #3076
HouseHoldCollection,Knife,100.0,-1.0,True,False,Genuine,
HouseHoldCollection,Knife,100.0,-1.0,True,False,Fake,Report #3077
HouseHoldCollection,Knife,100.0,-1.0,True,False,Fake,
HouseHoldCollection,Knife,100.0,-1.0,True,False,,Report #3078
HouseHoldCollection,Knife,100.0,-1.0,True,False,,
HouseHoldCollection,Knife,100.0,-1.0,False,True,Genuine,Report #3079
HouseHoldCollection,Knife,100.0,-1.0,False,True,Genuine,
HouseHoldCollection,Knife,100.0,-1.0,False,True,Fake,Report #3080
HouseHoldCollection,Knife,100.0,-1.0,False,True,Fake,
HouseHoldCollection,Knife,100.0,-1.0,False,True,,Report #3081
HouseHoldCollection,Knife,100.0,-1.0,False,True,,
HouseHoldCollection,Knife,100.0,-1.0,False,False,Genuine,Report #3082
HouseHoldCollection,Knife,100.0,-1.0,False,False,Genuine,
HouseHoldCollection,Knife,100.0,-1.0,False,False,Fake,Report #3083
HouseHoldCollection,Knife,100.0,-1.0,False,False,Fake,
HouseHoldCollection,Knife,100.0,-1.0,False,False,,Report #3084
HouseHoldCollection,Knife,100.0,-1.0,False,False,,
HouseHoldCollection,Knife,100.0,0.0,True,True,Genuine,Report #3085
HouseHoldCollection,Knife,100.0,0.0,True,True,Genuine,
HouseHoldCollection,Knife,100.0,0.0,True,True,Fake,Report #3086
HouseHoldCollection,Knife,100.0,0.0,True,True,Fake,
HouseHoldCollection,Knife,100.0,0.0,True,True,,Report #3087
HouseHoldCollection,Knife,100.0,0.0,True,True,,
HouseHoldCollection,Knife,100.0,0.0,True,False,Genuine,Report #3088
HouseHoldCollection,Knife,100.0,0.0,True,False,Genuine,
HouseHoldCollection,Knife,100.0,0.0,True,False,Fake,Report #3089
HouseHoldCollection,Knife,100.0,0.0,True,False,Fake,
HouseHoldCollection,Knife,100.0,0.0,True,False,,Report #3090
HouseHoldCollection,Knife,100.0,0.0,True,False,,
HouseHoldCollection,Knife,100.0,0.0,False,True,Genuine,Report #3091
HouseHoldCollection,Knife,100.0,0.0,False,True,Genuine,
HouseHoldCollection,Knife,100.0,0.0,False,True,Fake,Report #3092
HouseHoldCollection,Knife,100.0,0.0,False,True,Fake,
HouseHoldCollection,Knife,100.0,0.0,False,True,,Report #3093
HouseHoldCollection,Knife,100.0,0.0,False,True,,
HouseHoldCollection,Knife,100.0,0.0,False,False,Genuine,Report #3094
HouseHoldCollection,Knife,100.0,0.0,False,False,Genuine,
HouseHoldCollection,Knife,100.0,0.0,False,False,Fake,Report #3095
HouseHoldCollection,Knife,100.0,0.0,False,False,Fake,
HouseHoldCollection,Knife,100.0,0.0,False,False,,Report #3096
HouseHoldCollection,Knife,100.0,0.0,False,False,,
HouseHoldCollection,Knife,100.0,19.0,True,True,Genuine,Report #3097
HouseHoldCollection,Knife,100.0,19.0,True,True,Genuine,
HouseHoldCollection,Knife,100.0,19.0,True,True,Fake,Report #3098
HouseHoldCollection,Knife,100.0,19.0,True,True,Fake,
HouseHoldCollection,Knife,100.0,19.0,True,True,,Report #3099
HouseHoldCollection,Knife,100.0,19.0,True,True,,
HouseHoldCollection,Knife,100.0,19.0,True,False,Genuine,Report #3100
HouseHoldCollection,Knife,100.0,19.0,True,False,Genuine,
HouseHoldCollection,Knife,100.0,19.0,True,False,Fake,Report #3101
HouseHoldCollection,Knife,100.0,19.0,True,False,Fake,
HouseHoldCollection,Knife,100.0,19.0,True,False,,Report #3102
HouseHoldCollection,Knife,100.0,19.0,True,False,,
HouseHoldCollection,Knife,100.0,19.0,False,True,Genuine,Report #3103
HouseHoldCollection,Knife,100.0,19.0,False,True,Genuine,
HouseHoldCollection,Knife,100.0,19.0,False,True,Fake,Report #3104
HouseHoldCollection,Knife,100.0,19.0,False,True,Fake,
HouseHoldCollection,Knife,100.0,19.0,False,True,,Report #3105
HouseHoldCollection,Knife,100.0,19.0,False,True,,
HouseHoldCollection,Knife,100.0,19.0,False,False,Genuine,Report #3106
HouseHoldCollection,Knife,100.0,19.0,False,False,Genuine,
HouseHoldCollection,Knife,100.0,19.0,False,False,Fake,Report #3107
HouseHoldCollection,Knife,100.0,19.0,False,False,Fake,
HouseHoldCollection,Knife,100.0,19.0,False,False,,Report #3108
HouseHoldCollection,Knife,100.0,19.0,False,False,,
HouseHoldCollection,Knife,100.0,20.0,True,True,Genuine,Report #3109
HouseHoldCollection,Knife,100.0,20.0,True,True,Genuine,
HouseHoldCollection,Knife,100.0,20.0,True,True,Fake,Report #3110
HouseHoldCollection,Knife,100.0,20.0,True,True,Fake,
HouseHoldCollection,Knife,100.0,20.0,True,True,,Report #3111
HouseHoldCollection,Knife,100.0,20.0,True,True,,
HouseHoldCollection,Knife,100.0,20.0,True,False,Genuine,Report #3112
HouseHoldCollection,Knife,100.0,20.0,True,False,Genuine,
HouseHoldCollection,Knife,100.0,20.0,True,False,Fake,Report #3113
HouseHoldCollection,Knife,100.0,20.0,True,False,Fake,
HouseHoldCollection,Knife,100.0,20.0,True,False,,Report #3114
HouseHoldCollection,Knife,100.0,20.0,True,False,,
HouseHoldCollection,Knife,100.0,20.0,False,True,Genuine,Report #3115
HouseHoldCollection,Knife,100.0,20.0,False,True,Genuine,
HouseHoldCollection,Knife,100.0,20.0,False,True,Fake,Report #3116
HouseHoldCollection,Knife,100.0,20.0,False,True,Fake,
HouseHoldCollection,Knife,100.0,20.0,False,True,,Report #3117
HouseHoldCollection,Knife,100.0,20.0,False,True,,
HouseHoldCollection,Knife,100.0,20.0,False,False,Genuine,Report #3118
HouseHoldCollection,Knife,100.0,20.0,False,False,Genuine,
HouseHoldCollection,Knife,100.0,20.0,False,False,Fake,Report #3119
HouseHoldCollection,Knife,100.0,20.0,False,False,Fake,
HouseHoldCollection,Knife,100.0,20.0,False,False,,Report #3120
HouseHoldCollection,Knife,100.0,20.0,False,False,,
HouseHoldCollection,Knife,100.0,21.0,True,True,Genuine,Report #3121
HouseHoldCollection,Knife,100.0,21.0,True,True,Genuine,
HouseHoldCollection,Knife,100.0,21.0,True,True,Fake,Report #3122
HouseHoldCollection,Knife,100.0,21.0,True,True,Fake,
HouseHoldCollection,Knife,100.0,21.0,True,True,,Report #3123
HouseHoldCollection,Knife,100.0,21.0,True,True,,
HouseHoldCollection,Knife,100.0,21.0,True,False,Genuine,Report #3124
HouseHoldCollection,Knife,100.0,21.0,True,False,Genuine,
HouseHoldCollection,Knife,100.0,21.0,True,False,Fake,Report #3125
HouseHoldCollection,Knife,100.0,21.0,True,False,Fake,
HouseHoldCollection,Knife,100.0,21.0,True,False,,Report #3126
HouseHoldCollection,Knife,100.0,21.0,True,False,,
HouseHoldCollection,Knife,100.0,21.0,False,True,Genuine,Report #3127
HouseHoldCollection,Knife,100.0,21.0,False,True,Genuine,
HouseHoldCollection,Knife,100.0,21.0,False,True,Fake,Report #3128
HouseHoldCollection,Knife,100.0,21.0,False,True,Fake,
HouseHoldCollection,Knife,100.0,21.0,False,True,,Report #3129
HouseHoldCollection,Knife,100.0,21.0,False,True,,
HouseHoldCollection,Knife,100.0,21.0,False,False,Genuine,Report #3130
HouseHoldCollection,Knife,100.0,21.0,False,False,Genuine,
HouseHoldCollection,Knife,100.0,21.0,False,False,Fake,Report #3131
HouseHoldCollection,Knife,100.0,21.0,False,False,Fake,
HouseHoldCollection,Knife,100.0,21.0,False,False,,Report #3132
HouseHoldCollection,Knife,100.0,21.0,False,False,,
HouseHoldCollection,Knife,100.0,99.0,True,True,Genuine,Report #3133
HouseHoldCollection,Knife,100.0,99.0,True,True,Genuine,
HouseHoldCollection,Knife,100.0,99.0,True,True,Fake,Report #3134
HouseHoldCollection,Knife,100.0,99.0,True,True,Fake,
HouseHoldCollection,Knife,100.0,99.0,True,True,,Report #3135
HouseHoldCollection,Knife,100.0,99.0,True,True,,
HouseHoldCollection,Knife,100.0,99.0,True,False,Genuine,Report #3136
HouseHoldCollection,Knife,100.0,99.0,True,False,Genuine,
HouseHoldCollection,Knife,100.0,99.0,True,False,Fake,Report #3137
HouseHoldCollection,Knife,100.0,99.0,True,False,Fake,
HouseHoldCollection,Knife,100.0,99.0,True,False,,Report #3138
HouseHoldCollection,Knife,100.0,99.0,True,False,,
HouseHoldCollection,Knife,100.0,99.0,False,True,Genuine,Report #3139
HouseHoldCollection,Knife,100.0,99.0,False,True,Genuine,
HouseHoldCollection,Knife,100.0,99.0,False,True,Fake,Report #3140
HouseHoldCollection,Knife,100.0,99.0,False,True,Fake,
HouseHoldCollection,Knife,100.0,99.0,False,True,,Report #3141
HouseHoldCollection,Knife,100.0,99.0,False,True,,
HouseHoldCollection,Knife,100.0,99.0,False,False,Genuine,Report #3142
HouseHoldCollection,Knife,100.0,99.0,False,False,Genuine,
HouseHoldCollection,Knife,100.0,99.0,False,False,Fake,Report #3143
HouseHoldCollection,Knife,100.0,99.0,False,False,Fake,
HouseHoldCollection,Knife,100.0,99.0,False,False,,Report #3144
HouseHoldCollection,Knife,100.0,99.0,False,False,,
HouseHoldCollection,Knife,100.0,100.0,True,True,Genuine,Report #3145
HouseHoldCollection,Knife,100.0,100.0,True,True,Genuine,
HouseHoldCollection,Knife,100.0,100.0,True,True,Fake,Report #3146
HouseHoldCollection,Knife,100.0,100.0,True,True,Fake,
HouseHoldCollection,Knife,100.0,100.0,True,True,,Report #3147
HouseHoldCollection,Knife,100.0,100.0,True,True,,
HouseHoldCollection,Knife,100.0,100.0,True,False,Genuine,Report #3148
HouseHoldCollection,Knife,100.0,100.0,True,False,Genuine,
HouseHoldCollection,Knife,100.0,100.0,True,False,Fake,Report #3149
HouseHoldCollection,Knife,100.0,100.0,True,False,Fake,
HouseHoldCollection,Knife,100.0,100.0,True,False,,Report #3150
HouseHoldCollection,Knife,100.0,100.0,True,False,,
HouseHoldCollection,Knife,100.0,100.0,False,True,Genuine,Report #3151
HouseHoldCollection,Knife,100.0,100.0,False,True,Genuine,
HouseHoldCollection,Knife,100.0,100.0,False,True,Fake,Report #3152
HouseHoldCollection,Knife,100.0,100.0,False,True,Fake,
HouseHoldCollection,Knife,100.0,100.0,False,True,,Report #3153
HouseHoldCollection,Knife,100.0,100.0,False,True,,
HouseHoldCollection,Knife,100.0,100.0,False,False,Genuine,Report #3154
HouseHoldCollection,Knife,100.0,100.0,False,False,Genuine,
HouseHoldCollection,Knife,100.0,100.0,False,False,Fake,Report #3155
HouseHoldCollection,Knife,100.0,100.0,False,False,Fake,
HouseHoldCollection,Knife,100.0,100.0,False,False,,Report #3156
HouseHoldCollection,Knife,100.0,100.0,False,False,,
HouseHoldCollection,Knife,100.0,101.0,True,True,Genuine,Report #3157
HouseHoldCollection,Knife,100.0,101.0,True,True,Genuine,
HouseHoldCollection,Knife,100.0,101.0,True,True,Fake,Report #3158
HouseHoldCollection,Knife,100.0,101.0,True,True,Fake,
HouseHoldCollection,Knife,100.0,101.0,True,True,,Report #3159
HouseHoldCollection,Knife,100.0,101.0,True,True,,
HouseHoldCollection,Knife,100.0,101.0,True,False,Genuine,Report #3160
HouseHoldCollection,Knife,100.0,101.0,True,False,Genuine,
HouseHoldCollection,Knife,100.0,101.0,True,False,Fake,Report #3161
HouseHoldCollection,Knife,100.0,101.0,True,False,Fake,
HouseHoldCollection,Knife,100.0,101.0,True,False,,Report #3162
HouseHoldCollection,Knife,100.0,101.0,True,False,,
HouseHoldCollection,Knife,100.0,101.0,False,True,Genuine,Report #3163
HouseHoldCollection,Knife,100.0,101.0,False,True,Genuine,
HouseHoldCollection,Knife,100.0,101.0,False,True,Fake,Report #3164
HouseHoldCollection,Knife,100.0,101.0,False,True,Fake,
HouseHoldCollection,Knife,100.0,101.0,False,True,,Report #3165
HouseHoldCollection,Knife,100.0,101.0,False,True,,
HouseHoldCollection,Knife,100.0,101.0,False,False,Genuine,Report #3166
HouseHoldCollection,Knife,100.0,101.0,False,False,Genuine,
HouseHoldCollection,Knife,100.0,101.0,False,False,Fake,Report #3167
HouseHoldCollection,Knife,100.0,101.0,False,False,Fake,
HouseHoldCollection,Knife,100.0,101.0,False,False,,Report #3168
HouseHoldCollection,Knife,100.0,101.0,False,False,,
HouseHoldCollection,Knife,1000.0,-1.0,True,True,Genuine,Report #3169
HouseHoldCollection,Knife,1000.0,-1.0,True,True,Genuine,
HouseHoldCollection,Knife,1000.0,-1.0,True,True,Fake,Report #3170
HouseHoldCollection,Knife,1000.0,-1.0,True,True,Fake,
HouseHoldCollection,Knife,1000.0,-1.0,True,True,,Report #3171
HouseHoldCollection,Knife,1000.0,-1.0,True,True,,
HouseHoldCollection,Knife,1000.0,-1.0,True,False,Genuine,Report #3172
HouseHoldCollection,Knife,1000.0,-1.0,True,False,Genuine,
HouseHoldCollection,Knife,1000.0,-1.0,True,False,Fake,Report #3173
HouseHoldCollection,Knife,1000.0,-1.0,True,False,Fake,
HouseHoldCollection,Knife,1000.0,-1.0,True,False,,Report #3174
HouseHoldCollection,Knife,1000.0,-1.0,True,False,,
HouseHoldCollection,Knife,1000.0,-1.0,False,True,Genuine,Report #3175
HouseHoldCollection,Knife,1000.0,-1.0,False,True,Genuine,
HouseHoldCollection,Knife,1000.0,-1.0,False,True,Fake,Report #3176
HouseHoldCollection,Knife,1000.0,-1.0,False,True,Fake,
HouseHoldCollection,Knife,1000.0,-1.0,False,True,,Report #3177
HouseHoldCollection,Knife,1000.0,-1.0,False,True,,
HouseHoldCollection,Knife,1000.0,-1.0,False,False,Genuine,Report #3178
HouseHoldCollection,Knife,1000.0,-1.0,False,False,Genuine,
HouseHoldCollection,Knife,1000.0,-1.0,False,False,Fake,Report #3179
HouseHoldCollection,Knife,1000.0,-1.0,False,False,Fake,
HouseHoldCollection,Knife,1000.0,-1.0,False,False,,Report #3180
HouseHoldCollection,Knife,1000.0,-1.0,False,False,,
HouseHoldCollection,Knife,1000.0,0.0,True,True,Genuine,Report #3181
HouseHoldCollection,Knife,1000.0,0.0,True,True,Genuine,
HouseHoldCollection,Knife,1000.0,0.0,True,True,Fake,Report #3182
HouseHoldCollection,Knife,1000.0,0.0,True,True,Fake,
HouseHoldCollection,Knife,1000.0,0.0,True,True,,Report #3183
HouseHoldCollection,Knife,1000.0,0.0,True,True,,
HouseHoldCollection,Knife,1000.0,0.0,True,False,Genuine,Report #3184
HouseHoldCollection,Knife,1000.0,0.0,True,False,Genuine,
HouseHoldCollection,Knife,1000.0,0.0,True,False,Fake,Report #3185
HouseHoldCollection,Knife,1000.0,0.0,True,False,Fake,
HouseHoldCollection,Knife,1000.0,0.0,True,False,,Report #3186
HouseHoldCollection,Knife,1000.0,0.0,True,False,,
HouseHoldCollection,Knife,1000.0,0.0,False,True,Genuine,Report #3187
HouseHoldCollection,Knife,1000.0,0.0,False,True,Genuine,
HouseHoldCollection,Knife,1000.0,0.0,False,True,Fake,Report #3188
HouseHoldCollection,Knife,1000.0,0.0,False,True,Fake,
HouseHoldCollection,Knife,1000.0,0.0,False,True,,Report #3189
HouseHoldCollection,Knife,1000.0,0.0,False,True,,
HouseHoldCollection,Knife,1000.0,0.0,False,False,Genuine,Report #3190
HouseHoldCollection,Knife,1000.0,0.0,False,False,Genuine,
HouseHoldCollection,Knife,1000.0,0.0,False,False,Fake,Report #3191
HouseHoldCollection,Knife,1000.0,0.0,False,False,Fake,
HouseHoldCollection,Knife,1000.0,0.0,False,False,,Report #3192
HouseHoldCollection,Knife,1000.0,0.0,False,False,,
HouseHoldCollection,Knife,1000.0,19.0,True,True,Genuine,Report #3193
HouseHoldCollection,Knife,1000.0,19.0,True,True,Genuine,
HouseHoldCollection,Knife,1000.0,19.0,True,True,Fake,Report #3194
HouseHoldCollection,Knife,1000.0,19.0,True,True,Fake,
HouseHoldCollection,Knife,1000.0,19.0,True,True,,Report #3195
HouseHoldCollection,Knife,1000.0,19.0,True,True,,
HouseHoldCollection,Knife,1000.0,19.0,True,False,Genuine,Report #3196
HouseHoldCollection,Knife,1000.0,19.0,True,False,Genuine,
HouseHoldCollection,Knife,1000.0,19.0,True,False,Fake,Report #3197
HouseHoldCollection,Knife,1000.0,19.0,True,False,Fake,
HouseHoldCollection,Knife,1000.0,19.0,True,False,,Report #3198
HouseHoldCollection,Knife,1000.0,19.0,True,False,,
HouseHoldCollection,Knife,1000.0,19.0,False,True,Genuine,Report #3199
HouseHoldCollection,Knife,1000.0,19.0,False,True,Genuine,
HouseHoldCollection,Knife,1000.0,19.0,False,True,Fake,Report #3200
HouseHoldCollection,Knife,1000.0,19.0,False,True,Fake,
HouseHoldCollection,Knife,1000.0,19.0,False,True,,Report #3201
HouseHoldCollection,Knife,1000.0,19.0,False,True,,
HouseHoldCollection,Knife,1000.0,19.0,False,False,Genuine,Report #3202
HouseHoldCollection,Knife,1000.0,19.0,False,False,Genuine,
HouseHoldCollection,Knife,1000.0,19.0,False,False,Fake,Report #3203
HouseHoldCollection,Knife,1000.0,19.0,False,False,Fake,
HouseHoldCollection,Knife,1000.0,19.0,False,False,,Report #3204
HouseHoldCollection,Knife,1000.0,19.0,False,False,,
HouseHoldCollection,Knife,1000.0,20.0,True,True,Genuine,Report #3205
HouseHoldCollection,Knife,1000.0,20.0,True,True,Genuine,
HouseHoldCollection,Knife,1000.0,20.0,True,True,Fake,Report #3206
HouseHoldCollection,Knife,1000.0,20.0,True,True,Fake,
HouseHoldCollection,Knife,1000.0,20.0,True,True,,Report #3207
HouseHoldCollection,Knife,1000.0,20.0,True,True,,
HouseHoldCollection,Knife,1000.0,20.0,True,False,Genuine,Report #3208
HouseHoldCollection,Knife,1000.0,20.0,True,False,Genuine,
HouseHoldCollection,Knife,1000.0,20.0,True,False,Fake,Report #3209
HouseHoldCollection,Knife,1000.0,20.0,True,False,Fake,
HouseHoldCollection,Knife,1000.0,20.0,True,False,,Report #3210
HouseHoldCollection,Knife,1000.0,20.0,True,False,,
HouseHoldCollection,Knife,1000.0,20.0,False,True,Genuine,Report #3211
HouseHoldCollection,Knife,1000.0,20.0,False,True,Genuine,
HouseHoldCollection,Knife,1000.0,20.0,False,True,Fake,Report #3212
HouseHoldCollection,Knife,1000.0,20.0,False,True,Fake,
HouseHoldCollection,Knife,1000.0,20.0,False,True,,Report #3213
HouseHoldCollection,Knife,1000.0,20.0,False,True,,
HouseHoldCollection,Knife,1000.0,20.0,False,False,Genuine,Report #3214
HouseHoldCollection,Knife,1000.0,20.0,False,False,Genuine,
HouseHoldCollection,Knife,1000.0,20.0,False,False,Fake,Report #3215
HouseHoldCollection,Knife,1000.0,20.0,False,False,Fake,
HouseHoldCollection,Knife,1000.0,20.0,False,False,,Report #3216
HouseHoldCollection,Knife,1000.0,20.0,False,False,,
HouseHoldCollection,Knife,1000.0,21.0,True,True,Genuine,Report #3217
HouseHoldCollection,Knife,1000.0,21.0,True,True,Genuine,
HouseHoldCollection,Knife,1000.0,21.0,True,True,Fake,Report #3218
HouseHoldCollection,Knife,1000.0,21.0,True,True,Fake,
HouseHoldCollection,Knife,1000.0,21.0,True,True,,Report #3219
HouseHoldCollection,Knife,1000.0,21.0,True,True,,
HouseHoldCollection,Knife,1000.0,21.0,True,False,Genuine,Report #3220
HouseHoldCollection,Knife,1000.0,21.0,True,False,Genuine,
HouseHoldCollection,Knife,1000.0,21.0,True,False,Fake,Report #3221
HouseHoldCollection,Knife,1000.0,21.0,True,False,Fake,
HouseHoldCollection,Knife,1000.0,21.0,True,False,,Report #3222
HouseHoldCollection,Knife,1000.0,21.0,True,False,,
HouseHoldCollection,Knife,1000.0,21.0,False,True,Genuine,Report #3223
HouseHoldCollection,Knife,1000.0,21.0,False,True,Genuine,
HouseHoldCollection,Knife,1000.0,21.0,False,True,Fake,Report #3224
HouseHoldCollection,Knife,1000.0,21.0,False,True,Fake,
HouseHoldCollection,Knife,1000.0,21.0,False,True,,Report #3225
HouseHoldCollection,Knife,1000.0,21.0,False,True,,
HouseHoldCollection,Knife,1000.0,21.0,False,False,Genuine,Report #3226
HouseHoldCollection,Knife,1000.0,21.0,False,False,Genuine,
HouseHoldCollection,Knife,1000.0,21.0,False,False,Fake,Report #3227
HouseHoldCollection,Knife,1000.0,21.0,False,False,Fake,
HouseHoldCollection,Knife,1000.0,21.0,False,False,,Report #3228
HouseHoldCollection,Knife,1000.0,21.0,False,False,,
HouseHoldCollection,Knife,1000.0,99.0,True,True,Genuine,Report #3229
HouseHoldCollection,Knife,1000.0,99.0,True,True,Genuine,
HouseHoldCollection,Knife,1000.0,99.0,True,True,Fake,Report #3230
HouseHoldCollection,Knife,1000.0,99.0,True,True,Fake,
HouseHoldCollection,Knife,1000.0,99.0,True,True,,Report #3231
HouseHoldCollection,Knife,1000.0,99.0,True,True,,
HouseHoldCollection,Knife,1000.0,99.0,True,False,Genuine,Report #3232
HouseHoldCollection,Knife,1000.0,99.0,True,False,Genuine,
HouseHoldCollection,Knife,1000.0,99.0,True,False,Fake,Report #3233
HouseHoldCollection,Knife,1000.0,99.0,True,False,Fake,
HouseHoldCollection,Knife,1000.0,99.0,True,False,,Report #3234
HouseHoldCollection,Knife,1000.0,99.0,True,False,,
HouseHoldCollection,Knife,1000.0,99.0,False,True,Genuine,Report #3235
HouseHoldCollection,Knife,1000.0,99.0,False,True,Genuine,
HouseHoldCollection,Knife,1000.0,99.0,False,True,Fake,Report #3236
HouseHoldCollection,Knife,1000.0,99.0,False,True,Fake,
HouseHoldCollection,Knife,1000.0,99.0,False,True,,Report #3237
HouseHoldCollection,Knife,1000.0,99.0,False,True,,
HouseHoldCollection,Knife,1000.0,99.0,False,False,Genuine,Report #3238
HouseHoldCollection,Knife,1000.0,99.0,False,False,Genuine,
HouseHoldCollection,Knife,1000.0,99.0,False,False,Fake,Report #3239
HouseHoldCollection,Knife,1000.0,99.0,False,False,Fake,
HouseHoldCollection,Knife,1000.0,99.0,False,False,,Report #3240
HouseHoldCollection,Knife,1000.0,99.0,False,False,,
HouseHoldCollection,Knife,1000.0,100.0,True,True,Genuine,Report #3241
HouseHoldCollection,Knife,1000.0,100.0,True,True,Genuine,
HouseHoldCollection,Knife,1000.0,100.0,True,True,Fake,Report #3242
HouseHoldCollection,Knife,1000.0,100.0,True,True,Fake,
HouseHoldCollection,Knife,1000.0,100.0,True,True,,Report #3243
HouseHoldCollection,Knife,1000.0,100.0,True,True,,
HouseHoldCollection,Knife,1000.0,100.0,True,False,Genuine,Report #3244
HouseHoldCollection,Knife,1000.0,100.0,True,False,Genuine,
HouseHoldCollection,Knife,1000.0,100.0,True,False,Fake,Report #3245
HouseHoldCollection,Knife,1000.0,100.0,True,False,Fake,
HouseHoldCollection,Knife,1000.0,100.0,True,False,,Report #3246
HouseHoldCollection,Knife,1000.0,100.0,True,False,,
HouseHoldCollection,Knife,1000.0,100.0,False,True,Genuine,Report #3247
HouseHoldCollection,Knife,1000.0,100.0,False,True,Genuine,
HouseHoldCollection,Knife,1000.0,100.0,False,True,Fake,Report #3248
HouseHoldCollection,Knife,1000.0,100.0,False,True,Fake,
HouseHoldCollection,Knife,1000.0,100.0,False,True,,Report #3249
HouseHoldCollection,Knife,1000.0,100.0,False,True,,
HouseHoldCollection,Knife,1000.0,100.0,False,False,Genuine,Report #3250
HouseHoldCollection,Knife,1000.0,100.0,False,False,Genuine,
HouseHoldCollection,Knife,1000.0,100.0,False,False,Fake,Report #3251
HouseHoldCollection,Knife,1000.0,100.0,False,False,Fake,
HouseHoldCollection,Knife,1000.0,100.0,False,False,,Report #3252
HouseHoldCollection,Knife,1000.0,100.0,False,False,,
HouseHoldCollection,Knife,1000.0,101.0,True,True,Genuine,Report #3253
HouseHoldCollection,Knife,1000.0,101.0,True,True,Genuine,
HouseHoldCollection,Knife,1000.0,101.0,True,True,Fake,Report #3254
HouseHoldCollection,Knife,1000.0,101.0,True,True,Fake,
HouseHoldCollection,Knife,1000.0,101.0,True,True,,Report #3255
HouseHoldCollection,Knife,1000.0,101.0,True,True,,
HouseHoldCollection,Knife,1000.0,101.0,True,False,Genuine,Report #3256
HouseHoldCollection,Knife,1000.0,101.0,True,False,Genuine,
HouseHoldCollection,Knife,1000.0,101.0,True,False,Fake,Report #3257
HouseHoldCollection,Knife,1000.0,101.0,True,False,Fake,
HouseHoldCollection,Knife,1000.0,101.0,True,False,,Report #3258
HouseHoldCollection,Knife,1000.0,101.0,True,False,,
HouseHoldCollection,Knife,1000.0,101.0,False,True,Genuine,Report #3259
HouseHoldCollection,Knife,1000.0,101.0,False,True,Genuine,
HouseHoldCollection,Knife,1000.0,101.0,False,True,Fake,Report #3260
HouseHoldCollection,Knife,1000.0,101.0,False,True,Fake,
HouseHoldCollection,Knife,1000.0,101.0,False,True,,Report #3261
HouseHoldCollection,Knife,1000.0,101.0,False,True,,
HouseHoldCollection,Knife,1000.0,101.0,False,False,Genuine,Report #3262
HouseHoldCollection,Knife,1000.0,101.0,False,False,Genuine,
HouseHoldCollection,Knife,1000.0,101.0,False,False,Fake,Report #3263
HouseHoldCollection,Knife,1000.0,101.0,False,False,Fake,
HouseHoldCollection,Knife,1000.0,101.0,False,False,,Report #3264
HouseHoldCollection,Knife,1000.0,101.0,False,False,,
HouseHoldCollection,Knife,1000000.0,-1.0,True,True,Genuine,Report #3265
HouseHoldCollection,Knife,1000000.0,-1.0,True,True,Genuine,
HouseHoldCollection,Knife,1000000.0,-1.0,True,True,Fake,Report #3266
HouseHoldCollection,Knife,1000000.0,-1.0,True,True,Fake,
HouseHoldCollection,Knife,1000000.0,-1.0,True,True,,Report #3267
HouseHoldCollection,Knife,1000000.0,-1.0,True,True,,
HouseHoldCollection,Knife,1000000.0,-1.0,True,False,Genuine,Report #3268
HouseHoldCollection,Knife,1000000.0,-1.0,True,False,Genuine,
HouseHoldCollection,Knife,1000000.0,-1.0,True,False,Fake,Report #3269
HouseHoldCollection,Knife,1000000.0,-1.0,True,False,Fake,
HouseHoldCollection,Knife,1000000.0,-1.0,True,False,,Report #3270
HouseHoldCollection,Knife,1000000.0,-1.0,True,False,,
HouseHoldCollection,Knife,1000000.0,-1.0,False,True,Genuine,Report #3271
HouseHoldCollection,Knife,1000000.0,-1.0,False,True,Genuine,
HouseHoldCollection,Knife,1000000.0,-1.0,False,True,Fake,Report #3272
HouseHoldCollection,Knife,1000000.0,-1.0,False,True,Fake,
HouseHoldCollection,Knife,1000000.0,-1.0,False,True,,Report #3273
HouseHoldCollection,Knife,1000000.0,-1.0,False,True,,
HouseHoldCollection,Knife,1000000.0,-1.0,False,False,Genuine,Report #3274
HouseHoldCollection,Knife,1000000.0,-1.0,False,False,Genuine,
HouseHoldCollection,Knife,1000000.0,-1.0,False,False,Fake,Report #3275
HouseHoldCollection,Knife,1000000.0,-1.0,False,False,Fake,
HouseHoldCollection,Knife,1000000.0,-1.0,False,False,,Report #3276
HouseHoldCollection,Knife,1000000.0,-1.0,False,False,,
HouseHoldCollection,Knife,1000000.0,0.0,True,True,Genuine,Report #3277
HouseHoldCollection,Knife,1000000.0,0.0,True,True,Genuine,
HouseHoldCollection,Knife,1000000.0,0.0,True,True,Fake,Report #3278
HouseHoldCollection,Knife,1000000.0,0.0,True,True,Fake,
HouseHoldCollection,Knife,1000000.0,0.0,True,True,,Report #3279
HouseHoldCollection,Knife,1000000.0,0.0,True,True,,
HouseHoldCollection,Knife,1000000.0,0.0,True,False,Genuine,Report #3280
HouseHoldCollection,Knife,1000000.0,0.0,True,False,Genuine,
HouseHoldCollection,Knife,1000000.0,0.0,True,False,Fake,Report #3281
HouseHoldCollection,Knife,1000000.0,0.0,True,False,Fake,
HouseHoldCollection,Knife,1000000.0,0.0,True,False,,Report #3282
HouseHoldCollection,Knife,1000000.0,0.0,True,False,,
HouseHoldCollection,Knife,1000000.0,0.0,False,True,Genuine,Report #3283
HouseHoldCollection,Knife,1000000.0,0.0,False,True,Genuine,
HouseHoldCollection,Knife,1000000.0,0.0,False,True,Fake,Report #3284
HouseHoldCollection,Knife,1000000.0,0.0,False,True,Fake,
HouseHoldCollection,Knife,1000000.0,0.0,False,True,,Report #3285
HouseHoldCollection,Knife,1000000.0,0.0,False,True,,
HouseHoldCollection,Knife,1000000.0,0.0,False,False,Genuine,Report #3286
HouseHoldCollection,Knife,1000000.0,0.0,False,False,Genuine,
HouseHoldCollection,Knife,1000000.0,0.0,False,False,Fake,Report #3287
HouseHoldCollection,Knife,1000000.0,0.0,False,False,Fake,
HouseHoldCollection,Knife,1000000.0,0.0,False,False,,Report #3288
HouseHoldCollection,Knife,1000000.0,0.0,False,False,,
HouseHoldCollection,Knife,1000000.0,19.0,True,True,Genuine,Report #3289
HouseHoldCollection,Knife,1000000.0,19.0,True,True,Genuine,
HouseHoldCollection,Knife,1000000.0,19.0,True,True,Fake,Report #3290
HouseHoldCollection,Knife,1000000.0,19.0,True,True,Fake,
HouseHoldCollection,Knife,1000000.0,19.0,True,True,,Report #3291
HouseHoldCollection,Knife,1000000.0,19.0,True,True,,
HouseHoldCollection,Knife,1000000.0,19.0,True,False,Genuine,Report #3292
HouseHoldCollection,Knife,1000000.0,19.0,True,False,Genuine,
HouseHoldCollection,Knife,1000000.0,19.0,True,False,Fake,Report #3293
HouseHoldCollection,Knife,1000000.0,19.0,True,False,Fake,
HouseHoldCollection,Knife,1000000.0,19.0,True,False,,Report #3294
HouseHoldCollection,Knife,1000000.0,19.0,True,False,,
HouseHoldCollection,Knife,1000000.0,19.0,False,True,Genuine,Report #3295
HouseHoldCollection,Knife,1000000.0,19.0,False,True,Genuine,
HouseHoldCollection,Knife,1000000.0,19.0,False,True,Fake,Report #3296
HouseHoldCollection,Knife,1000000.0,19.0,False,True,Fake,
HouseHoldCollection,Knife,1000000.0,19.0,False,True,,Report #3297
HouseHoldCollection,Knife,1000000.0,19.0,False,True,,
HouseHoldCollection,Knife,1000000.0,19.0,False,False,Genuine,Report #3298
HouseHoldCollection,Knife,1000000.0,19.0,False,False,Genuine,
HouseHoldCollection,Knife,1000000.0,19.0,False,False,Fake,Report #3299
HouseHoldCollection,Knife,1000000.0,19.0,False,False,Fake,
HouseHoldCollection,Knife,1000000.0,19.0,False,False,,Report #3300
HouseHoldCollection,Knife,1000000.0,19.0,False,False,,
HouseHoldCollection,Knife,1000000.0,20.0,True,True,Genuine,Report #3301
HouseHoldCollection,Knife,1000000.0,20.0,True,True,Genuine,
HouseHoldCollection,Knife,1000000.0,20.0,True,True,Fake,Report #3302
HouseHoldCollection,Knife,1000000.0,20.0,True,True,Fake,
HouseHoldCollection,Knife,1000000.0,20.0,True,True,,Report #3303
HouseHoldCollection,Knife,1000000.0,20.0,True,True,,
HouseHoldCollection,Knife,1000000.0,20.0,True,False,Genuine,Report #3304
HouseHoldCollection,Knife,1000000.0,20.0,True,False,Genuine,
HouseHoldCollection,Knife,1000000.0,20.0,True,False,Fake,Report #3305
HouseHoldCollection,Knife,1000000.0,20.0,True,False,Fake,
HouseHoldCollection,Knife,1000000.0,20.0,True,False,,Report #3306
HouseHoldCollection,Knife,1000000.0,20.0,True,False,,
HouseHoldCollection,Knife,1000000.0,20.0,False,True,Genuine,Report #3307
HouseHoldCollection,Knife,1000000.0,20.0,False,True,Genuine,
HouseHoldCollection,Knife,1000000.0,20.0,False,True,Fake,Report #3308
HouseHoldCollection,Knife,1000000.0,20.0,False,True,Fake,
HouseHoldCollection,Knife,1000000.0,20.0,False,True,,Report #3309
HouseHoldCollection,Knife,1000000.0,20.0,False,True,,
HouseHoldCollection,Knife,1000000.0,20.0,False,False,Genuine,Report #3310
HouseHoldCollection,Knife,1000000.0,20.0,False,False,Genuine,
HouseHoldCollection,Knife,1000000.0,20.0,False,False,Fake,Report #3311
HouseHoldCollection,Knife,1000000.0,20.0,False,False,Fake,
HouseHoldCollection,Knife,1000000.0,20.0,False,False,,Report #3312
HouseHoldCollection,Knife,1000000.0,20.0,False,False,,
HouseHoldCollection,Knife,1000000.0,21.0,True,True,Genuine,Report #3313
HouseHoldCollection,Knife,1000000.0,21.0,True,True,Genuine,
HouseHoldCollection,Knife,1000000.0,21.0,True,True,Fake,Report #3314
HouseHoldCollection,Knife,1000000.0,21.0,True,True,Fake,
HouseHoldCollection,Knife,1000000.0,21.0,True,True,,Report #3315
HouseHoldCollection,Knife,1000000.0,21.0,True,True,,
HouseHoldCollection,Knife,1000000.0,21.0,True,False,Genuine,Report #3316
HouseHoldCollection,Knife,1000000.0,21.0,True,False,Genuine,
HouseHoldCollection,Knife,1000000.0,21.0,True,False,Fake,Report #3317
HouseHoldCollection,Knife,1000000.0,21.0,True,False,Fake,
HouseHoldCollection,Knife,1000000.0,21.0,True,False,,Report #3318
HouseHoldCollection,Knife,1000000.0,21.0,True,False,,
HouseHoldCollection,Knife,1000000.0,21.0,False,True,Genuine,Report #3319
HouseHoldCollection,Knife,1000000.0,21.0,False,True,Genuine,
HouseHoldCollection,Knife,1000000.0,21.0,False,True,Fake,Report #3320
HouseHoldCollection,Knife,1000000.0,21.0,False,True,Fake,
HouseHoldCollection,Knife,1000000.0,21.0,False,True,,Report #3321
HouseHoldCollection,Knife,1000000.0,21.0,False,True,,
HouseHoldCollection,Knife,1000000.0,21.0,False,False,Genuine,Report #3322
HouseHoldCollection,Knife,1000000.0,21.0,False,False,Genuine,
HouseHoldCollection,Knife,1000000.0,21.0,False,False,Fake,Report #3323
HouseHoldCollection,Knife,1000000.0,21.0,False,False,Fake,
HouseHoldCollection,Knife,1000000.0,21.0,False,False,,Report #3324
HouseHoldCollection,Knife,1000000.0,21.0,False,False,,
HouseHoldCollection,Knife,1000000.0,99.0,True,True,Genuine,Report #3325
HouseHoldCollection,Knife,1000000.0,99.0,True,True,Genuine,
HouseHoldCollection,Knife,1000000.0,99.0,True,True,Fake,Report #3326
HouseHoldCollection,Knife,1000000.0,99.0,True,True,Fake,
HouseHoldCollection,Knife,1000000.0,99.0,True,True,,Report #3327
HouseHoldCollection,Knife,1000000.0,99.0,True,True,,
HouseHoldCollection,Knife,1000000.0,99.0,True,False,Genuine,Report #3328
HouseHoldCollection,Knife,1000000.0,99.0,True,False,Genuine,
HouseHoldCollection,Knife,1000000.0,99.0,True,False,Fake,Report #3329
HouseHoldCollection,Knife,1000000.0,99.0,True,False,Fake,
HouseHoldCollection,Knife,1000000.0,99.0,True,False,,Report #3330
HouseHoldCollection,Knife,1000000.0,99.0,True,False,,
HouseHoldCollection,Knife,1000000.0,99.0,False,True,Genuine,Report #3331
HouseHoldCollection,Knife,1000000.0,99.0,False,True,Genuine,
HouseHoldCollection,Knife,1000000.0,99.0,False,True,Fake,Report #3332
HouseHoldCollection,Knife,1000000.0,99.0,False,True,Fake,
HouseHoldCollection,Knife,1000000.0,99.0,False,True,,Report #3333
HouseHoldCollection,Knife,1000000.0,99.0,False,True,,
HouseHoldCollection,Knife,1000000.0,99.0,False,False,Genuine,Report #3334
HouseHoldCollection,Knife,1000000.0,99.0,False,False,Genuine,
HouseHoldCollection,Knife,1000000.0,99.0,False,False,Fake,Report #3335
HouseHoldCollection,Knife,1000000.0,99.0,False,False,Fake,
HouseHoldCollection,Knife,1000000.0,99.0,False,False,,Report #3336
HouseHoldCollection,Knife,1000000.0,99.0,False,False,,
HouseHoldCollection,Knife,1000000.0,100.0,True,True,Genuine,Report #3337
HouseHoldCollection,Knife,1000000.0,100.0,True,True,Genuine,
HouseHoldCollection,Knife,1000000.0,100.0,True,True,Fake,Report #3338
HouseHoldCollection,Knife,1000000.0,100.0,True,True,Fake,
HouseHoldCollection,Knife,1000000.0,100.0,True,True,,Report #3339
HouseHoldCollection,Knife,1000000.0,100.0,True,True,,
HouseHoldCollection,Knife,1000000.0,100.0,True,False,Genuine,Report #3340
HouseHoldCollection,Knife,1000000.0,100.0,True,False,Genuine,
HouseHoldCollection,Knife,1000000.0,100.0,True,False,Fake,Report #3341
HouseHoldCollection,Knife,1000000.0,100.0,True,False,Fake,
HouseHoldCollection,Knife,1000000.0,100.0,True,False,,Report #3342
HouseHoldCollection,Knife,1000000.0,100.0,True,False,,
HouseHoldCollection,Knife,1000000.0,100.0,False,True,Genuine,Report #3343
HouseHoldCollection,Knife,1000000.0,100.0,False,True,Genuine,
HouseHoldCollection,Knife,1000000.0,100.0,False,True,Fake,Report #3344
HouseHoldCollection,Knife,1000000.0,100.0,False,True,Fake,
HouseHoldCollection,Knife,1000000.0,100.0,False,True,,Report #3345
HouseHoldCollection,Knife,1000000.0,100.0,False,True,,
HouseHoldCollection,Knife,1000000.0,100.0,False,False,Genuine,Report #3346
HouseHoldCollection,Knife,1000000.0,100.0,False,False,Genuine,
HouseHoldCollection,Knife,1000000.0,100.0,False,False,Fake,Report #3347
HouseHoldCollection,Knife,1000000.0,100.0,False,False,Fake,
HouseHoldCollection,Knife,1000000.0,100.0,False,False,,Report #3348
HouseHoldCollection,Knife,1000000.0,100.0,False,False,,
HouseHoldCollection,Knife,1000000.0,101.0,True,True,Genuine,Report #3349
HouseHoldCollection,Knife,1000000.0,101.0,True,True,Genuine,
HouseHoldCollection,Knife,1000000.0,101.0,True,True,Fake,Report #3350
HouseHoldCollection,Knife,1000000.0,101.0,True,True,Fake,
HouseHoldCollection,Knife,1000000.0,101.0,True,True,,Report #3351
HouseHoldCollection,Knife,1000000.0,101.0,True,True,,
HouseHoldCollection,Knife,1000000.0,101.0,True,False,Genuine,Report #3352
HouseHoldCollection,Knife,1000000.0,101.0,True,False,Genuine,
HouseHoldCollection,Knife,1000000.0,101.0,True,False,Fake,Report #3353
HouseHoldCollection,Knife,1000000.0,101.0,True,False,Fake,
HouseHoldCollection,Knife,1000000.0,101.0,True,False,,Report #3354
HouseHoldCollection,Knife,1000000.0,101.0,True,False,,
HouseHoldCollection,Knife,1000000.0,101.0,False,True,Genuine,Report #3355
HouseHoldCollection,Knife,1000000.0,101.0,False,True,Genuine,
HouseHoldCollection,Knife,1000000.0,101.0,False,True,Fake,Report #3356
HouseHoldCollection,Knife,1000000.0,101.0,False,True,Fake,
HouseHoldCollection,Knife,1000000.0,101.0,False,True,,Report #3357
HouseHoldCollection,Knife,1000000.0,101.0,False,True,,
HouseHoldCollection,Knife,1000000.0,101.0,False,False,Genuine,Report #3358
HouseHoldCollection,Knife,1000000.0,101.0,False,False,Genuine,
HouseHoldCollection,Knife,1000000.0,101.0,False,False,Fake,Report #3359
HouseHoldCollection,Knife,1000000.0,101.0,False,False,Fake,
HouseHoldCollection,Knife,1000000.0,101.0,False,False,,Report #3360
HouseHoldCollection,Knife,1000000.0,101.0,False,False,,
HouseHoldCollection,Plates,0.0,-1.0,True,True,Genuine,Report #3361
HouseHoldCollection,Plates,0.0,-1.0,True,True,Genuine,
HouseHoldCollection,Plates,0.0,-1.0,True,True,Fake,Report #3362
HouseHoldCollection,Plates,0.0,-1.0,True,True,Fake,
HouseHoldCollection,Plates,0.0,-1.0,True,True,,Report #3363
HouseHoldCollection,Plates,0.0,-1.0,True,True,,
HouseHoldCollection,Plates,0.0,-1.0,True,False,Genuine,Report #3364
HouseHoldCollection,Plates,0.0,-1.0,True,False,Genuine,
HouseHoldCollection,Plates,0.0,-1.0,True,False,Fake,Report #3365
HouseHoldCollection,Plates,0.0,-1.0,True,False,Fake,
HouseHoldCollection,Plates,0.0,-1.0,True,False,,Report #3366
HouseHoldCollection,Plates,0.0,-1.0,True,False,,
HouseHoldCollection,Plates,0.0,-1.0,False,True,Genuine,Report #3367
HouseHoldCollection,Plates,0.0,-1.0,False,True,Genuine,
HouseHoldCollection,Plates,0.0,-1.0,False,True,Fake,Report #3368
HouseHoldCollection,Plates,0.0,-1.0,False,True,Fake,
HouseHoldCollection,Plates,0.0,-1.0,False,True,,Report #3369
HouseHoldCollection,Plates,0.0,-1.0,False,True,,
HouseHoldCollection,Plates,0.0,-1.0,False,False,Genuine,Report #3370
HouseHoldCollection,Plates,0.0,-1.0,False,False,Genuine,
HouseHoldCollection,Plates,0.0,-1.0,False,False,Fake,Report #3371
HouseHoldCollection,Plates,0.0,-1.0,False,False,Fake,
HouseHoldCollection,Plates,0.0,-1.0,False,False,,Report #3372
HouseHoldCollection,Plates,0.0,-1.0,False,False,,
HouseHoldCollection,Plates,0.0,0.0,True,True,Genuine,Report #3373
HouseHoldCollection,Plates,0.0,0.0,True,True,Genuine,
HouseHoldCollection,Plates,0.0,0.0,True,True,Fake,Report #3374
HouseHoldCollection,Plates,0.0,0.0,True,True,Fake,
HouseHoldCollection,Plates,0.0,0.0,True,True,,Report #3375
HouseHoldCollection,Plates,0.0,0.0,True,True,,
HouseHoldCollection,Plates,0.0,0.0,True,False,Genuine,Report #3376
HouseHoldCollection,Plates,0.0,0.0,True,False,Genuine,
HouseHoldCollection,Plates,0.0,0.0,True,False,Fake,Report #3377
HouseHoldCollection,Plates,0.0,0.0,True,False,Fake,
HouseHoldCollection,Plates,0.0,0.0,True,False,,Report #3378
HouseHoldCollection,Plates,0.0,0.0,True,False,,
HouseHoldCollection,Plates,0.0,0.0,False,True,Genuine,Report #3379
HouseHoldCollection,Plates,0.0,0.0,False,True,Genuine,
HouseHoldCollection,Plates,0.0,0.0,False,True,Fake,Report #3380
HouseHoldCollection,Plates,0.0,0.0,False,True,Fake,
HouseHoldCollection,Plates,0.0,0.0,False,True,,Report #3381
HouseHoldCollection,Plates,0.0,0.0,False,True,,
HouseHoldCollection,Plates,0.0,0.0,False,False,Genuine,Report #3382
HouseHoldCollection,Plates,0.0,0.0,False,False,Genuine,
HouseHoldCollection,Plates,0.0,0.0,False,False,Fake,Report #3383
HouseHoldCollection,Plates,0.0,0.0,False,False,Fake,
HouseHoldCollection,Plates,0.0,0.0,False,False,,Report #3384
HouseHoldCollection,Plates,0.0,0.0,False,False,,
HouseHoldCollection,Plates,0.0,19.0,True,True,Genuine,Report #3385
HouseHoldCollection,Plates,0.0,19.0,True,True,Genuine,
HouseHoldCollection,Plates,0.0,19.0,True,True,Fake,Report #3386
HouseHoldCollection,Plates,0.0,19.0,True,True,Fake,
HouseHoldCollection,Plates,0.0,19.0,True,True,,Report #3387
HouseHoldCollection,Plates,0.0,19.0,True,True,,
HouseHoldCollection,Plates,0.0,19.0,True,False,Genuine,Report #3388
HouseHoldCollection,Plates,0.0,19.0,True,False,Genuine,
HouseHoldCollection,Plates,0.0,19.0,True,False,Fake,Report #3389
HouseHoldCollection,Plates,0.0,19.0,True,False,Fake,
HouseHoldCollection,Plates,0.0,19.0,True,False,,Report #3390
HouseHoldCollection,Plates,0.0,19.0,True,False,,
HouseHoldCollection,Plates,0.0,19.0,False,True,Genuine,Report #3391
HouseHoldCollection,Plates,0.0,19.0,False,True,Genuine,
HouseHoldCollection,Plates,0.0,19.0,False,True,Fake,Report #3392
HouseHoldCollection,Plates,0.0,19.0,False,True,Fake,
HouseHoldCollection,Plates,0.0,19.0,False,True,,Report #3393
HouseHoldCollection,Plates,0.0,19.0,False,True,,
HouseHoldCollection,Plates,0.0,19.0,False,False,Genuine,Report #3394
HouseHoldCollection,Plates,0.0,19.0,False,False,Genuine,
HouseHoldCollection,Plates,0.0,19.0,False,False,Fake,Report #3395
HouseHoldCollection,Plates,0.0,19.0,False,False,Fake,
HouseHoldCollection,Plates,0.0,19.0,False,False,,Report #3396
HouseHoldCollection,Plates,0.0,19.0,False,False,,
HouseHoldCollection,Plates,0.0,20.0,True,True,Genuine,Report #3397
HouseHoldCollection,Plates,0.0,20.0,True,True,Genuine,
HouseHoldCollection,Plates,0.0,20.0,True,True,Fake,Report #3398
HouseHoldCollection,Plates,0.0,20.0,True,True,Fake,
HouseHoldCollection,Plates,0.0,20.0,True,True,,Report #3399
HouseHoldCollection,Plates,0.0,20.0,True,True,,
HouseHoldCollection,Plates,0.0,20.0,True,False,Genuine,Report #3400
HouseHoldCollection,Plates,0.0,20.0,True,False,Genuine,
HouseHoldCollection,Plates,0.0,20.0,True,False,Fake,Report #3401
HouseHoldCollection,Plates,0.0,20.0,True,False,Fake,
HouseHoldCollection,Plates,0.0,20.0,True,False,,Report #3402
HouseHoldCollection,Plates,0.0,20.0,True,False,,
HouseHoldCollection,Plates,0.0,20.0,False,True,Genuine,Report #3403
HouseHoldCollection,Plates,0.0,20.0,False,True,Genuine,
HouseHoldCollection,Plates,0.0,20.0,False,True,Fake,Report #3404
HouseHoldCollection,Plates,0.0,20.0,False,True,Fake,
HouseHoldCollection,Plates,0.0,20.0,False,True,,Report #3405
HouseHoldCollection,Plates,0.0,20.0,False,True,,
HouseHoldCollection,Plates,0.0,20.0,False,False,Genuine,Report #3406
HouseHoldCollection,Plates,0.0,20.0,False,False,Genuine,
HouseHoldCollection,Plates,0.0,20.0,False,False,Fake,Report #3407
HouseHoldCollection,Plates,0.0,20.0,False,False,Fake,
HouseHoldCollection,Plates,0.0,20.0,False,False,,Report #3408
HouseHoldCollection,Plates,0.0,20.0,False,False,,
HouseHoldCollection,Plates,0.0,21.0,True,True,Genuine,Report #3409
HouseHoldCollection,Plates,0.0,21.0,True,True,Genuine,
HouseHoldCollection,Plates,0.0,21.0,True,True,Fake,Report #3410
HouseHoldCollection,Plates,0.0,21.0,True,True,Fake,
HouseHoldCollection,Plates,0.0,21.0,True,True,,Report #3411
HouseHoldCollection,Plates,0.0,21.0,True,True,,
HouseHoldCollection,Plates,0.0,21.0,True,False,Genuine,Report #3412
HouseHoldCollection,Plates,0.0,21.0,True,False,Genuine,
HouseHoldCollection,Plates,0.0,21.0,True,False,Fake,Report #3413
HouseHoldCollection,Plates,0.0,21.0,True,False,Fake,
HouseHoldCollection,Plates,0.0,21.0,True,False,,Report #3414
HouseHoldCollection,Plates,0.0,21.0,True,False,,
HouseHoldCollection,Plates,0.0,21.0,False,True,Genuine,Report #3415
HouseHoldCollection,Plates,0.0,21.0,False,True,Genuine,
HouseHoldCollection,Plates,0.0,21.0,False,True,Fake,Report #3416
HouseHoldCollection,Plates,0.0,21.0,False,True,Fake,
HouseHoldCollection,Plates,0.0,21.0,False,True,,Report #3417
HouseHoldCollection,Plates,0.0,21.0,False,True,,
HouseHoldCollection,Plates,0.0,21.0,False,False,Genuine,Report #3418
HouseHoldCollection,Plates,0.0,21.0,False,False,Genuine,
HouseHoldCollection,Plates,0.0,21.0,False,False,Fake,Report #3419
HouseHoldCollection,Plates,0.0,21.0,False,False,Fake,
HouseHoldCollection,Plates,0.0,21.0,False,False,,Report #3420
HouseHoldCollection,Plates,0.0,21.0,False,False,,
HouseHoldCollection,Plates,0.0,99.0,True,True,Genuine,Report #3421
HouseHoldCollection,Plates,0.0,99.0,True,True,Genuine,
HouseHoldCollection,Plates,0.0,99.0,True,True,Fake,Report #3422
HouseHoldCollection,Plates,0.0,99.0,True,True,Fake,
HouseHoldCollection,Plates,0.0,99.0,True,True,,Report #3423
HouseHoldCollection,Plates,0.0,99.0,True,True,,
HouseHoldCollection,Plates,0.0,99.0,True,False,Genuine,Report #3424
HouseHoldCollection,Plates,0.0,99.0,True,False,Genuine,
HouseHoldCollection,Plates,0.0,99.0,True,False,Fake,Report #3425
HouseHoldCollection,Plates,0.0,99.0,True,False,Fake,
HouseHoldCollection,Plates,0.0,99.0,True,False,,Report #3426
HouseHoldCollection,Plates,0.0,99.0,True,False,,
HouseHoldCollection,Plates,0.0,99.0,False,True,Genuine,Report #3427
HouseHoldCollection,Plates,0.0,99.0,False,True,Genuine,
HouseHoldCollection,Plates,0.0,99.0,False,True,Fake,Report #3428
HouseHoldCollection,Plates,0.0,99.0,False,True,Fake,
HouseHoldCollection,Plates,0.0,99.0,False,True,,Report #3429
HouseHoldCollection,Plates,0.0,99.0,False,True,,
HouseHoldCollection,Plates,0.0,99.0,False,False,Genuine,Report #3430
HouseHoldCollection,Plates,0.0,99.0,False,False,Genuine,
HouseHoldCollection,Plates,0.0,99.0,False,False,Fake,Report #3431
HouseHoldCollection,Plates,0.0,99.0,False,False,Fake,
HouseHoldCollection,Plates,0.0,99.0,False,False,,Report #3432
HouseHoldCollection,Plates,0.0,99.0,False,False,,
HouseHoldCollection,Plates,0.0,100.0,True,True,Genuine,Report #3433
HouseHoldCollection,Plates,0.0,100.0,True,True,Genuine,
HouseHoldCollection,Plates,0.0,100.0,True,True,Fake,Report #3434
HouseHoldCollection,Plates,0.0,100.0,True,True,Fake,
HouseHoldCollection,Plates,0.0,100.0,True,True,,Report #3435
HouseHoldCollection,Plates,0.0,100.0,True,True,,
HouseHoldCollection,Plates,0.0,100.0,True,False,Genuine,Report #3436
HouseHoldCollection,Plates,0.0,100.0,True,False,Genuine,
HouseHoldCollection,Plates,0.0,100.0,True,False,Fake,Report #3437
HouseHoldCollection,Plates,0.0,100.0,True,False,Fake,
HouseHoldCollection,Plates,0.0,100.0,True,False,,Report #3438
HouseHoldCollection,Plates,0.0,100.0,True,False,,
HouseHoldCollection,Plates,0.0,100.0,False,True,Genuine,Report #3439
HouseHoldCollection,Plates,0.0,100.0,False,True,Genuine,
HouseHoldCollection,Plates,0.0,100.0,False,True,Fake,Report #3440
HouseHoldCollection,Plates,0.0,100.0,False,True,Fake,
HouseHoldCollection,Plates,0.0,100.0,False,True,,Report #3441
HouseHoldCollection,Plates,0.0,100.0,False,True,,
HouseHoldCollection,Plates,0.0,100.0,False,False,Genuine,Report #3442
HouseHoldCollection,Plates,0.0,100.0,False,False,Genuine,
HouseHoldCollection,Plates,0.0,100.0,False,False,Fake,Report #3443
HouseHoldCollection,Plates,0.0,100.0,False,False,Fake,
HouseHoldCollection,Plates,0.0,100.0,False,False,,Report #3444
HouseHoldCollection,Plates,0.0,100.0,False,False,,
HouseHoldCollection,Plates,0.0,101.0,True,True,Genuine,Report #3445
HouseHoldCollection,Plates,0.0,101.0,True,True,Genuine,
HouseHoldCollection,Plates,0.0,101.0,True,True,Fake,Report #3446
HouseHoldCollection,Plates,0.0,101.0,True,True,Fake,
HouseHoldCollection,Plates,0.0,101.0,True,True,,Report #3447
HouseHoldCollection,Plates,0.0,101.0,True,True,,
HouseHoldCollection,Plates,0.0,101.0,True,False,Genuine,Report #3448
HouseHoldCollection,Plates,0.0,101.0,True,False,Genuine,
HouseHoldCollection,Plates,0.0,101.0,True,False,Fake,Report #3449
HouseHoldCollection,Plates,0.0,101.0,True,False,Fake,
HouseHoldCollection,Plates,0.0,101.0,True,False,,Report #3450
HouseHoldCollection,Plates,0.0,101.0,True,False,,
HouseHoldCollection,Plates,0.0,101.0,False,True,Genuine,Report #3451
HouseHoldCollection,Plates,0.0,101.0,False,True,Genuine,
HouseHoldCollection,Plates,0.0,101.0,False,True,Fake,Report #3452
HouseHoldCollection,Plates,0.0,101.0,False,True,Fake,
HouseHoldCollection,Plates,0.0,101.0,False,True,,Report #3453
HouseHoldCollection,Plates,0.0,101.0,False,True,,
HouseHoldCollection,Plates,0.0,101.0,False,False,Genuine,Report #3454
HouseHoldCollection,Plates,0.0,101.0,False,False,Genuine,
HouseHoldCollection,Plates,0.0,101.0,False,False,Fake,Report #3455
HouseHoldCollection,Plates,0.0,101.0,False,False,Fake,
HouseHoldCollection,Plates,0.0,101.0,False,False,,Report #3456
HouseHoldCollection,Plates,0.0,101.0,False,False,,
HouseHoldCollection,Plates,1.0,-1.0,True,True,Genuine,Report #3457
HouseHoldCollection,Plates,1.0,-1.0,True,True,Genuine,
HouseHoldCollection,Plates,1.0,-1.0,True,True,Fake,Report #3458
HouseHoldCollection,Plates,1.0,-1.0,True,True,Fake,
HouseHoldCollection,Plates,1.0,-1.0,True,True,,Report #3459
HouseHoldCollection,Plates,1.0,-1.0,True,True,,
HouseHoldCollection,Plates,1.0,-1.0,True,False,Genuine,Report #3460
HouseHoldCollection,Plates,1.0,-1.0,True,False,Genuine,
HouseHoldCollection,Plates,1.0,-1.0,True,False,Fake,Report #3461
HouseHoldCollection,Plates,1.0,-1.0,True,False,Fake,
HouseHoldCollection,Plates,1.0,-1.0,True,False,,Report #3462
HouseHoldCollection,Plates,1.0,-1.0,True,False,,
HouseHoldCollection,Plates,1.0,-1.0,False,True,Genuine,Report #3463
HouseHoldCollection,Plates,1.0,-1.0,False,True,Genuine,
HouseHoldCollection,Plates,1.0,-1.0,False,True,Fake,Report #3464
HouseHoldCollection,Plates,1.0,-1.0,False,True,Fake,
HouseHoldCollection,Plates,1.0,-1.0,False,True,,Report #3465
HouseHoldCollection,Plates,1.0,-1.0,False,True,,
HouseHoldCollection,Plates,1.0,-1.0,False,False,Genuine,Report #3466
HouseHoldCollection,Plates,1.0,-1.0,False,False,Genuine,
HouseHoldCollection,Plates,1.0,-1.0,False,False,Fake,Report #3467
HouseHoldCollection,Plates,1.0,-1.0,False,False,Fake,
HouseHoldCollection,Plates,1.0,-1.0,False,False,,Report #3468
HouseHoldCollection,Plates,1.0,-1.0,False,False,,
HouseHoldCollection,Plates,1.0,0.0,True,True,Genuine,Report #3469
HouseHoldCollection,Plates,1.0,0.0,True,True,Genuine,
HouseHoldCollection,Plates,1.0,0.0,True,True,Fake,Report #3470
HouseHoldCollection,Plates,1.0,0.0,True,True,Fake,
HouseHoldCollection,Plates,1.0,0.0,True,True,,Report #3471
HouseHoldCollection,Plates,1.0,0.0,True,True,,
HouseHoldCollection,Plates,1.0,0.0,True,False,Genuine,Report #3472
HouseHoldCollection,Plates,1.0,0.0,True,False,Genuine,
HouseHoldCollection,Plates,1.0,0.0,True,False,Fake,Report #3473
HouseHoldCollection,Plates,1.0,0.0,True,False,Fake,
HouseHoldCollection,Plates,1.0,0.0,True,False,,Report #3474
HouseHoldCollection,Plates,1.0,0.0,True,False,,
HouseHoldCollection,Plates,1.0,0.0,False,True,Genuine,Report #3475
HouseHoldCollection,Plates,1.0,0.0,False,True,Genuine,
HouseHoldCollection,Plates,1.0,0.0,False,True,Fake,Report #3476
HouseHoldCollection,Plates,1.0,0.0,False,True,Fake,
HouseHoldCollection,Plates,1.0,0.0,False,True,,Report #3477
HouseHoldCollection,Plates,1.0,0.0,False,True,,
HouseHoldCollection,Plates,1.0,0.0,False,False,Genuine,Report #3478
HouseHoldCollection,Plates,1.0,0.0,False,False,Genuine,
HouseHoldCollection,Plates,1.0,0.0,False,False,Fake,Report #3479
HouseHoldCollection,Plates,1.0,0.0,False,False,Fake,
HouseHoldCollection,Plates,1.0,0.0,False,False,,Report #3480
HouseHoldCollection,Plates,1.0,0.0,False,False,,
HouseHoldCollection,Plates,1.0,19.0,True,True,Genuine,Report #3481
HouseHoldCollection,Plates,1.0,19.0,True,True,Genuine,
HouseHoldCollection,Plates,1.0,19.0,True,True,Fake,Report #3482
HouseHoldCollection,Plates,1.0,19.0,True,True,Fake,
HouseHoldCollection,Plates,1.0,19.0,True,True,,Report #3483
HouseHoldCollection,Plates,1.0,19.0,True,True,,
HouseHoldCollection,Plates,1.0,19.0,True,False,Genuine,Report #3484
HouseHoldCollection,Plates,1.0,19.0,True,False,Genuine,
HouseHoldCollection,Plates,1.0,19.0,True,False,Fake,Report #3485
HouseHoldCollection,Plates,1.0,19.0,True,False,Fake,
HouseHoldCollection,Plates,1.0,19.0,True,False,,Report #3486
HouseHoldCollection,Plates,1.0,19.0,True,False,,
HouseHoldCollection,Plates,1.0,19.0,False,True,Genuine,Report #3487
HouseHoldCollection,Plates,1.0,19.0,False,True,Genuine,
HouseHoldCollection,Plates,1.0,19.0,False,True,Fake,Report #3488
HouseHoldCollection,Plates,1.0,19.0,False,True,Fake,
HouseHoldCollection,Plates,1.0,19.0,False,True,,Report #3489
HouseHoldCollection,Plates,1.0,19.0,False,True,,
HouseHoldCollection,Plates,1.0,19.0,False,False,Genuine,Report #3490
HouseHoldCollection,Plates,1.0,19.0,False,False,Genuine,
HouseHoldCollection,Plates,1.0,19.0,False,False,Fake,Report #3491
HouseHoldCollection,Plates,1.0,19.0,False,False,Fake,
HouseHoldCollection,Plates,1.0,19.0,False,False,,Report #3492
HouseHoldCollection,Plates,1.0,19.0,False,False,,
HouseHoldCollection,Plates,1.0,20.0,True,True,Genuine,Report #3493
HouseHoldCollection,Plates,1.0,20.0,True,True,Genuine,
HouseHoldCollection,Plates,1.0,20.0,True,True,Fake,Report #3494
HouseHoldCollection,Plates,1.0,20.0,True,True,Fake,
HouseHoldCollection,Plates,1.0,20.0,True,True,,Report #3495
HouseHoldCollection,Plates,1.0,20.0,True,True,,
HouseHoldCollection,Plates,1.0,20.0,True,False,Genuine,Report #3496
HouseHoldCollection,Plates,1.0,20.0,True,False,Genuine,
HouseHoldCollection,Plates,1.0,20.0,True,False,Fake,Report #3497
HouseHoldCollection,Plates,1.0,20.0,True,False,Fake,
HouseHoldCollection,Plates,1.0,20.0,True,False,,Report #3498
HouseHoldCollection,Plates,1.0,20.0,True,False,,
HouseHoldCollection,Plates,1.0,20.0,False,True,Genuine,Report #3499
HouseHoldCollection,Plates,1.0,20.0,False,True,Genuine,
HouseHoldCollection,Plates,1.0,20.0,False,True,Fake,Report #3500
HouseHoldCollection,Plates,1.0,20.0,False,True,Fake,
HouseHoldCollection,Plates,1.0,20.0,False,True,,Report #3501
HouseHoldCollection,Plates,1.0,20.0,False,True,,
HouseHoldCollection,Plates,1.0,20.0,False,False,Genuine,Report #3502
HouseHoldCollection,Plates,1.0,20.0,False,False,Genuine,
HouseHoldCollection,Plates,1.0,20.0,False,False,Fake,Report #3503
HouseHoldCollection,Plates,1.0,20.0,False,False,Fake,
HouseHoldCollection,Plates,1.0,20.0,False,False,,Report #3504
HouseHoldCollection,Plates,1.0,20.0,False,False,,
HouseHoldCollection,Plates,1.0,21.0,True,True,Genuine,Report #3505
HouseHoldCollection,Plates,1.0,21.0,True,True,Genuine,
HouseHoldCollection,Plates,1.0,21.0,True,True,Fake,Report #3506
HouseHoldCollection,Plates,1.0,21.0,True,True,Fake,
HouseHoldCollection,Plates,1.0,21.0,True,True,,Report #3507
HouseHoldCollection,Plates,1.0,21.0,True,True,,
HouseHoldCollection,Plates,1.0,21.0,True,False,Genuine,Report #3508
HouseHoldCollection,Plates,1.0,21.0,True,False,Genuine,
HouseHoldCollection,Plates,1.0,21.0,True,False,Fake,Report #3509
HouseHoldCollection,Plates,1.0,21.0,True,False,Fake,
HouseHoldCollection,Plates,1.0,21.0,True,False,,Report #3510
HouseHoldCollection,Plates,1.0,21.0,True,False,,
HouseHoldCollection,Plates,1.0,21.0,False,True,Genuine,Report #3511
HouseHoldCollection,Plates,1.0,21.0,False,True,Genuine,
HouseHoldCollection,Plates,1.0,21.0,False,True,Fake,Report #3512
HouseHoldCollection,Plates,1.0,21.0,False,True,Fake,
HouseHoldCollection,Plates,1.0,21.0,False,True,,Report #3513
HouseHoldCollection,Plates,1.0,21.0,False,True,,
HouseHoldCollection,Plates,1.0,21.0,False,False,Genuine,Report #3514
HouseHoldCollection,Plates,1.0,21.0,False,False,Genuine,
HouseHoldCollection,Plates,1.0,21.0,False,False,Fake,Report #3515
HouseHoldCollection,Plates,1.0,21.0,False,False,Fake,
HouseHoldCollection,Plates,1.0,21.0,False,False,,Report #3516
HouseHoldCollection,Plates,1.0,21.0,False,False,,
HouseHoldCollection,Plates,1.0,99.0,True,True,Genuine,Report #3517
HouseHoldCollection,Plates,1.0,99.0,True,True,Genuine,
HouseHoldCollection,Plates,1.0,99.0,True,True,Fake,Report #3518
HouseHoldCollection,Plates,1.0,99.0,True,True,Fake,
HouseHoldCollection,Plates,1.0,99.0,True,True,,Report #3519
HouseHoldCollection,Plates,1.0,99.0,True,True,,
HouseHoldCollection,Plates,1.0,99.0,True,False,Genuine,Report #3520
HouseHoldCollection,Plates,1.0,99.0,True,False,Genuine,
HouseHoldCollection,Plates,1.0,99.0,True,False,Fake,Report #3521
HouseHoldCollection,Plates,1.0,99.0,True,False,Fake,
HouseHoldCollection,Plates,1.0,99.0,True,False,,Report #3522
HouseHoldCollection,Plates,1.0,99.0,True,False,,
HouseHoldCollection,Plates,1.0,99.0,False,True,Genuine,Report #3523
HouseHoldCollection,Plates,1.0,99.0,False,True,Genuine,
HouseHoldCollection,Plates,1.0,99.0,False,True,Fake,Report #3524
HouseHoldCollection,Plates,1.0,99.0,False,True,Fake,
HouseHoldCollection,Plates,1.0,99.0,False,True,,Report #3525
HouseHoldCollection,Plates,1.0,99.0,False,True,,
HouseHoldCollection,Plates,1.0,99.0,False,False,Genuine,Report #3526
HouseHoldCollection,Plates,1.0,99.0,False,False,Genuine,
HouseHoldCollection,Plates,1.0,99.0,False,False,Fake,Report #3527
HouseHoldCollection,Plates,1.0,99.0,False,False,Fake,
HouseHoldCollection,Plates,1.0,99.0,False,False,,Report #3528
HouseHoldCollection,Plates,1.0,99.0,False,False,,
HouseHoldCollection,Plates,1.0,100.0,True,True,Genuine,Report #3529
HouseHoldCollection,Plates,1.0,100.0,True,True,Genuine,
HouseHoldCollection,Plates,1.0,100.0,True,True,Fake,Report #3530
HouseHoldCollection,Plates,1.0,100.0,True,True,Fake,
HouseHoldCollection,Plates,1.0,100.0,True,True,,Report #3531
HouseHoldCollection,Plates,1.0,100.0,True,True,,
HouseHoldCollection,Plates,1.0,100.0,True,False,Genuine,Report #3532
HouseHoldCollection,Plates,1.0,100.0,True,False,Genuine,
HouseHoldCollection,Plates,1.0,100.0,True,False,Fake,Report #3533
HouseHoldCollection,Plates,1.0,100.0,True,False,Fake,
HouseHoldCollection,Plates,1.0,100.0,True,False,,Report #3534
HouseHoldCollection,Plates,1.0,100.0,True,False,,
HouseHoldCollection,Plates,1.0,100.0,False,True,Genuine,Report #3535
HouseHoldCollection,Plates,1.0,100.0,False,True,Genuine,
HouseHoldCollection,Plates,1.0,100.0,False,True,Fake,Report #3536
HouseHoldCollection,Plates,1.0,100.0,False,True,Fake,
HouseHoldCollection,Plates,1.0,100.0,False,True,,Report #3537
HouseHoldCollection,Plates,1.0,100.0,False,True,,
HouseHoldCollection,Plates,1.0,100.0,False,False,Genuine,Report #3538
HouseHoldCollection,Plates,1.0,100.0,False,False,Genuine,
HouseHoldCollection,Plates,1.0,100.0,False,False,Fake,Report #3539
HouseHoldCollection,Plates,1.0,100.0,False,False,Fake,
HouseHoldCollection,Plates,1.0,100.0,False,False,,Report #3540
HouseHoldCollection,Plates,1.0,100.0,False,False,,
HouseHoldCollection,Plates,1.0,101.0,True,True,Genuine,Report #3541
HouseHoldCollection,Plates,1.0,101.0,True,True,Genuine,
HouseHoldCollection,Plates,1.0,101.0,True,True,Fake,Report #3542
HouseHoldCollection,Plates,1.0,101.0,True,True,Fake,
HouseHoldCollection,Plates,1.0,101.0,True,True,,Report #3543
HouseHoldCollection,Plates,1.0,101.0,True,True,,
HouseHoldCollection,Plates,1.0,101.0,True,False,Genuine,Report #3544
HouseHoldCollection,Plates,1.0,101.0,True,False,Genuine,
HouseHoldCollection,Plates,1.0,101.0,True,False,Fake,Report #3545
HouseHoldCollection,Plates,1.0,101.0,True,False,Fake,
HouseHoldCollection,Plates,1.0,101.0,True,False,,Report #3546
HouseHoldCollection,Plates,1.0,101.0,True,False,,
HouseHoldCollection,Plates,1.0,101.0,False,True,Genuine,Report #3547
HouseHoldCollection,Plates,1.0,101.0,False,True,Genuine,
HouseHoldCollection,Plates,1.0,101.0,False,True,Fake,Report #3548
HouseHoldCollection,Plates,1.0,101.0,False,True,Fake,
HouseHoldCollection,Plates,1.0,101.0,False,True,,Report #3549
HouseHoldCollection,Plates,1.0,101.0,False,True,,
HouseHoldCollection,Plates,1.0,101.0,False,False,Genuine,Report #3550
HouseHoldCollection,Plates,1.0,101.0,False,False,Genuine,
HouseHoldCollection,Plates,1.0,101.0,False,False,Fake,Report #3551
HouseHoldCollection,Plates,1.0,101.0,False,False,Fake,
HouseHoldCollection,Plates,1.0,101.0,False,False,,Report #3552
HouseHoldCollection,Plates,1.0,101.0,False,False,,
HouseHoldCollection,Plates,100.0,-1.0,True,True,Genuine,Report #3553
HouseHoldCollection,Plates,100.0,-1.0,True,True,Genuine,
HouseHoldCollection,Plates,100.0,-1.0,True,True,Fake,Report #3554
HouseHoldCollection,Plates,100.0,-1.0,True,True,Fake,
HouseHoldCollection,Plates,100.0,-1.0,True,True,,Report #3555
HouseHoldCollection,Plates,100.0,-1.0,True,True,,
HouseHoldCollection,Plates,100.0,-1.0,True,False,Genuine,Report #3556
HouseHoldCollection,Plates,100.0,-1.0,True,False,Genuine,
HouseHoldCollection,Plates,100.0,-1.0,True,False,Fake,Report #3557
HouseHoldCollection,Plates,100.0,-1.0,True,False,Fake,
HouseHoldCollection,Plates,100.0,-1.0,True,False,,Report #3558
HouseHoldCollection,Plates,100.0,-1.0,True,False,,
HouseHoldCollection,Plates,100.0,-1.0,False,True,Genuine,Report #3559
HouseHoldCollection,Plates,100.0,-1.0,False,True,Genuine,
HouseHoldCollection,Plates,100.0,-1.0,False,True,Fake,Report #3560
HouseHoldCollection,Plates,100.0,-1.0,False,True,Fake,
HouseHoldCollection,Plates,100.0,-1.0,False,True,,Report #3561
HouseHoldCollection,Plates,100.0,-1.0,False,True,,
HouseHoldCollection,Plates,100.0,-1.0,False,False,Genuine,Report #3562
HouseHoldCollection,Plates,100.0,-1.0,False,False,Genuine,
HouseHoldCollection,Plates,100.0,-1.0,False,False,Fake,Report #3563
HouseHoldCollection,Plates,100.0,-1.0,False,False,Fake,
HouseHoldCollection,Plates,100.0,-1.0,False,False,,Report #3564
HouseHoldCollection,Plates,100.0,-1.0,False,False,,
HouseHoldCollection,Plates,100.0,0.0,True,True,Genuine,Report #3565
HouseHoldCollection,Plates,100.0,0.0,True,True,Genuine,
HouseHoldCollection,Plates,100.0,0.0,True,True,Fake,Report #3566
HouseHoldCollection,Plates,100.0,0.0,True,True,Fake,
HouseHoldCollection,Plates,100.0,0.0,True,True,,Report #3567
HouseHoldCollection,Plates,100.0,0.0,True,True,,
HouseHoldCollection,Plates,100.0,0.0,True,False,Genuine,Report #3568
HouseHoldCollection,Plates,100.0,0.0,True,False,Genuine,
HouseHoldCollection,Plates,100.0,0.0,True,False,Fake,Report #3569
HouseHoldCollection,Plates,100.0,0.0,True,False,Fake,
HouseHoldCollection,Plates,100.0,0.0,True,False,,Report #3570
HouseHoldCollection,Plates,100.0,0.0,True,False,,
HouseHoldCollection,Plates,100.0,0.0,False,True,Genuine,Report #3571
HouseHoldCollection,Plates,100.0,0.0,False,True,Genuine,
HouseHoldCollection,Plates,100.0,0.0,False,True,Fake,Report #3572
HouseHoldCollection,Plates,100.0,0.0,False,True,Fake,
HouseHoldCollection,Plates,100.0,0.0,False,True,,Report #3573
HouseHoldCollection,Plates,100.0,0.0,False,True,,
HouseHoldCollection,Plates,100.0,0.0,False,False,Genuine,Report #3574
HouseHoldCollection,Plates,100.0,0.0,False,False,Genuine,
HouseHoldCollection,Plates,100.0,0.0,False,False,Fake,Report #3575
HouseHoldCollection,Plates,100.0,0.0,False,False,Fake,
HouseHoldCollection,Plates,100.0,0.0,False,False,,Report #3576
HouseHoldCollection,Plates,100.0,0.0,False,False,,
HouseHoldCollection,Plates,100.0,19.0,True,True,Genuine,Report #3577
HouseHoldCollection,Plates,100.0,19.0,True,True,Genuine,
HouseHoldCollection,Plates,100.0,19.0,True,True,Fake,Report #3578
HouseHoldCollection,Plates,100.0,19.0,True,True,Fake,
HouseHoldCollection,Plates,100.0,19.0,True,True,,Report #3579
HouseHoldCollection,Plates,100.0,19.0,True,True,,
HouseHoldCollection,Plates,100.0,19.0,True,False,Genuine,Report #3580
HouseHoldCollection,Plates,100.0,19.0,True,False,Genuine,
HouseHoldCollection,Plates,100.0,19.0,True,False,Fake,Report #3581
HouseHoldCollection,Plates,100.0,19.0,True,False,Fake,
HouseHoldCollection,Plates,100.0,19.0,True,False,,Report #3582
HouseHoldCollection,Plates,100.0,19.0,True,False,,
HouseHoldCollection,Plates,100.0,19.0,False,True,Genuine,Report #3583
HouseHoldCollection,Plates,100.0,19.0,False,True,Genuine,
HouseHoldCollection,Plates,100.0,19.0,False,True,Fake,Report #3584
HouseHoldCollection,Plates,100.0,19.0,False,True,Fake,
HouseHoldCollection,Plates,100.0,19.0,False,True,,Report #3585
HouseHoldCollection,Plates,100.0,19.0,False,True,,
HouseHoldCollection,Plates,100.0,19.0,False,False,Genuine,Report #3586
HouseHoldCollection,Plates,100.0,19.0,False,False,Genuine,
HouseHoldCollection,Plates,100.0,19.0,False,False,Fake,Report #3587
HouseHoldCollection,Plates,100.0,19.0,False,False,Fake,
HouseHoldCollection,Plates,100.0,19.0,False,False,,Report #3588
HouseHoldCollection,Plates,100.0,19.0,False,False,,
HouseHoldCollection,Plates,100.0,20.0,True,True,Genuine,Report #3589
HouseHoldCollection,Plates,100.0,20.0,True,True,Genuine,
HouseHoldCollection,Plates,100.0,20.0,True,True,Fake,Report #3590
HouseHoldCollection,Plates,100.0,20.0,True,True,Fake,
HouseHoldCollection,Plates,100.0,20.0,True,True,,Report #3591
HouseHoldCollection,Plates,100.0,20.0,True,True,,
HouseHoldCollection,Plates,100.0,20.0,True,False,Genuine,Report #3592
HouseHoldCollection,Plates,100.0,20.0,True,False,Genuine,
HouseHoldCollection,Plates,100.0,20.0,True,False,Fake,Report #3593
HouseHoldCollection,Plates,100.0,20.0,True,False,Fake,
HouseHoldCollection,Plates,100.0,20.0,True,False,,Report #3594
HouseHoldCollection,Plates,100.0,20.0,True,False,,
HouseHoldCollection,Plates,100.0,20.0,False,True,Genuine,Report #3595
HouseHoldCollection,Plates,100.0,20.0,False,True,Genuine,
HouseHoldCollection,Plates,100.0,20.0,False,True,Fake,Report #3596
HouseHoldCollection,Plates,100.0,20.0,False,True,Fake,
HouseHoldCollection,Plates,100.0,20.0,False,True,,Report #3597
HouseHoldCollection,Plates,100.0,20.0,False,True,,
HouseHoldCollection,Plates,100.0,20.0,False,False,Genuine,Report #3598
HouseHoldCollection,Plates,100.0,20.0,False,False,Genuine,
HouseHoldCollection,Plates,100.0,20.0,False,False,Fake,Report #3599
HouseHoldCollection,Plates,100.0,20.0,False,False,Fake,
HouseHoldCollection,Plates,100.0,20.0,False,False,,Report #3600
HouseHoldCollection,Plates,100.0,20.0,False,False,,
HouseHoldCollection,Plates,100.0,21.0,True,True,Genuine,Report #3601
HouseHoldCollection,Plates,100.0,21.0,True,True,Genuine,
HouseHoldCollection,Plates,100.0,21.0,True,True,Fake,Report #3602
HouseHoldCollection,Plates,100.0,21.0,True,True,Fake,
HouseHoldCollection,Plates,100.0,21.0,True,True,,Report #3603
HouseHoldCollection,Plates,100.0,21.0,True,True,,
HouseHoldCollection,Plates,100.0,21.0,True,False,Genuine,Report #3604
HouseHoldCollection,Plates,100.0,21.0,True,False,Genuine,
HouseHoldCollection,Plates,100.0,21.0,True,False,Fake,Report #3605
HouseHoldCollection,Plates,100.0,21.0,True,False,Fake,
HouseHoldCollection,Plates,100.0,21.0,True,False,,Report #3606
HouseHoldCollection,Plates,100.0,21.0,True,False,,
HouseHoldCollection,Plates,100.0,21.0,False,True,Genuine,Report #3607
HouseHoldCollection,Plates,100.0,21.0,False,True,Genuine,
HouseHoldCollection,Plates,100.0,21.0,False,True,Fake,Report #3608
HouseHoldCollection,Plates,100.0,21.0,False,True,Fake,
HouseHoldCollection,Plates,100.0,21.0,False,True,,Report #3609
HouseHoldCollection,Plates,100.0,21.0,False,True,,
HouseHoldCollection,Plates,100.0,21.0,False,False,Genuine,Report #3610
HouseHoldCollection,Plates,100.0,21.0,False,False,Genuine,
HouseHoldCollection,Plates,100.0,21.0,False,False,Fake,Report #3611
HouseHoldCollection,Plates,100.0,21.0,False,False,Fake,
HouseHoldCollection,Plates,100.0,21.0,False,False,,Report #3612
HouseHoldCollection,Plates,100.0,21.0,False,False,,
HouseHoldCollection,Plates,100.0,99.0,True,True,Genuine,Report #3613
HouseHoldCollection,Plates,100.0,99.0,True,True,Genuine,
HouseHoldCollection,Plates,100.0,99.0,True,True,Fake,Report #3614
HouseHoldCollection,Plates,100.0,99.0,True,True,Fake,
HouseHoldCollection,Plates,100.0,99.0,True,True,,Report #3615
HouseHoldCollection,Plates,100.0,99.0,True,True,,
HouseHoldCollection,Plates,100.0,99.0,True,False,Genuine,Report #3616
HouseHoldCollection,Plates,100.0,99.0,True,False,Genuine,
HouseHoldCollection,Plates,100.0,99.0,True,False,Fake,Report #3617
HouseHoldCollection,Plates,100.0,99.0,True,False,Fake,
HouseHoldCollection,Plates,100.0,99.0,True,False,,Report #3618
HouseHoldCollection,Plates,100.0,99.0,True,False,,
HouseHoldCollection,Plates,100.0,99.0,False,True,Genuine,Report #3619
HouseHoldCollection,Plates,100.0,99.0,False,True,Genuine,
HouseHoldCollection,Plates,100.0,99.0,False,True,Fake,Report #3620
HouseHoldCollection,Plates,100.0,99.0,False,True,Fake,
HouseHoldCollection,Plates,100.0,99.0,False,True,,Report #3621
HouseHoldCollection,Plates,100.0,99.0,False,True,,
HouseHoldCollection,Plates,100.0,99.0,False,False,Genuine,Report #3622
HouseHoldCollection,Plates,100.0,99.0,False,False,Genuine,
HouseHoldCollection,Plates,100.0,99.0,False,False,Fake,Report #3623
HouseHoldCollection,Plates,100.0,99.0,False,False,Fake,
HouseHoldCollection,Plates,100.0,99.0,False,False,,Report #3624
HouseHoldCollection,Plates,100.0,99.0,False,False,,
HouseHoldCollection,Plates,100.0,100.0,True,True,Genuine,Report #3625
HouseHoldCollection,Plates,100.0,100.0,True,True,Genuine,
HouseHoldCollection,Plates,100.0,100.0,True,True,Fake,Report #3626
HouseHoldCollection,Plates,100.0,100.0,True,True,Fake,
HouseHoldCollection,Plates,100.0,100.0,True,True,,Report #3627
HouseHoldCollection,Plates,100.0,100.0,True,True,,
HouseHoldCollection,Plates,100.0,100.0,True,False,Genuine,Report #3628
HouseHoldCollection,Plates,100.0,100.0,True,False,Genuine,
HouseHoldCollection,Plates,100.0,100.0,True,False,Fake,Report #3629
HouseHoldCollection,Plates,100.0,100.0,True,False,Fake,
HouseHoldCollection,Plates,100.0,100.0,True,False,,Report #3630
HouseHoldCollection,Plates,100.0,100.0,True,False,,
HouseHoldCollection,Plates,100.0,100.0,False,True,Genuine,Report #3631
HouseHoldCollection,Plates,100.0,100.0,False,True,Genuine,
HouseHoldCollection,Plates,100.0,100.0,False,True,Fake,Report #3632
HouseHoldCollection,Plates,100.0,100.0,False,True,Fake,
HouseHoldCollection,Plates,100.0,100.0,False,True,,Report #3633
HouseHoldCollection,Plates,100.0,100.0,False,True,,
HouseHoldCollection,Plates,100.0,100.0,False,False,Genuine,Report #3634
HouseHoldCollection,Plates,100.0,100.0,False,False,Genuine,
HouseHoldCollection,Plates,100.0,100.0,False,False,Fake,Report #3635
HouseHoldCollection,Plates,100.0,100.0,False,False,Fake,
HouseHoldCollection,Plates,100.0,100.0,False,False,,Report #3636
HouseHoldCollection,Plates,100.0,100.0,False,False,,
HouseHoldCollection,Plates,100.0,101.0,True,True,Genuine,Report #3637
HouseHoldCollection,Plates,100.0,101.0,True,True,Genuine,
HouseHoldCollection,Plates,100.0,101.0,True,True,Fake,Report #3638
HouseHoldCollection,Plates,100.0,101.0,True,True,Fake,
HouseHoldCollection,Plates,100.0,101.0,True,True,,Report #3639
HouseHoldCollection,Plates,100.0,101.0,True,True,,
HouseHoldCollection,Plates,100.0,101.0,True,False,Genuine,Report #3640
HouseHoldCollection,Plates,100.0,101.0,True,False,Genuine,
HouseHoldCollection,Plates,100.0,101.0,True,False,Fake,Report #3641
HouseHoldCollection,Plates,100.0,101.0,True,False,Fake,
HouseHoldCollection,Plates,100.0,101.0,True,False,,Report #3642
HouseHoldCollection,Plates,100.0,101.0,True,False,,
HouseHoldCollection,Plates,100.0,101.0,False,True,Genuine,Report #3643
HouseHoldCollection,Plates,100.0,101.0,False,True,Genuine,
HouseHoldCollection,Plates,100.0,101.0,False,True,Fake,Report #3644
HouseHoldCollection,Plates,100.0,101.0,False,True,Fake,
HouseHoldCollection,Plates,100.0,101.0,False,True,,Report #3645
HouseHoldCollection,Plates,100.0,101.0,False,True,,
HouseHoldCollection,Plates,100.0,101.0,False,False,Genuine,Report #3646
HouseHoldCollection,Plates,100.0,101.0,False,False,Genuine,
HouseHoldCollection,Plates,100.0,101.0,False,False,Fake,Report #3647
HouseHoldCollection,Plates,100.0,101.0,False,False,Fake,
HouseHoldCollection,Plates,100.0,101.0,False,False,,Report #3648
HouseHoldCollection,Plates,100.0,101.0,False,False,,
HouseHoldCollection,Plates,1000.0,-1.0,True,True,Genuine,Report #3649
HouseHoldCollection,Plates,1000.0,-1.0,True,True,Genuine,
HouseHoldCollection,Plates,1000.0,-1.0,True,True,Fake,Report #3650
HouseHoldCollection,Plates,1000.0,-1.0,True,True,Fake,
HouseHoldCollection,Plates,1000.0,-1.0,True,True,,Report #3651
HouseHoldCollection,Plates,1000.0,-1.0,True,True,,
HouseHoldCollection,Plates,1000.0,-1.0,True,False,Genuine,Report #3652
HouseHoldCollection,Plates,1000.0,-1.0,True,False,Genuine,
HouseHoldCollection,Plates,1000.0,-1.0,True,False,Fake,Report #3653
HouseHoldCollection,Plates,1000.0,-1.0,True,False,Fake,
HouseHoldCollection,Plates,1000.0,-1.0,True,False,,Report #3654
HouseHoldCollection,Plates,1000.0,-1.0,True,False,,
HouseHoldCollection,Plates,1000.0,-1.0,False,True,Genuine,Report #3655
HouseHoldCollection,Plates,1000.0,-1.0,False,True,Genuine,
HouseHoldCollection,Plates,1000.0,-1.0,False,True,Fake,Report #3656
HouseHoldCollection,Plates,1000.0,-1.0,False,True,Fake,
HouseHoldCollection,Plates,1000.0,-1.0,False,True,,Report #3657
HouseHoldCollection,Plates,1000.0,-1.0,False,True,,
HouseHoldCollection,Plates,1000.0,-1.0,False,False,Genuine,Report #3658
HouseHoldCollection,Plates,1000.0,-1.0,False,False,Genuine,
HouseHoldCollection,Plates,1000.0,-1.0,False,False,Fake,Report #3659
HouseHoldCollection,Plates,1000.0,-1.0,False,False,Fake,
HouseHoldCollection,Plates,1000.0,-1.0,False,False,,Report #3660
HouseHoldCollection,Plates,1000.0,-1.0,False,False,,
HouseHoldCollection,Plates,1000.0,0.0,True,True,Genuine,Report #3661
HouseHoldCollection,Plates,1000.0,0.0,True,True,Genuine,
HouseHoldCollection,Plates,1000.0,0.0,True,True,Fake,Report #3662
HouseHoldCollection,Plates,1000.0,0.0,True,True,Fake,
HouseHoldCollection,Plates,1000.0,0.0,True,True,,Report #3663
HouseHoldCollection,Plates,1000.0,0.0,True,True,,
HouseHoldCollection,Plates,1000.0,0.0,True,False,Genuine,Report #3664
HouseHoldCollection,Plates,1000.0,0.0,True,False,Genuine,
HouseHoldCollection,Plates,1000.0,0.0,True,False,Fake,Report #3665
HouseHoldCollection,Plates,1000.0,0.0,True,False,Fake,
HouseHoldCollection,Plates,1000.0,0.0,True,False,,Report #3666
HouseHoldCollection,Plates,1000.0,0.0,True,False,,
HouseHoldCollection,Plates,1000.0,0.0,False,True,Genuine,Report #3667
HouseHoldCollection,Plates,1000.0,0.0,False,True,Genuine,
HouseHoldCollection,Plates,1000.0,0.0,False,True,Fake,Report #3668
HouseHoldCollection,Plates,1000.0,0.0,False,True,Fake,
HouseHoldCollection,Plates,1000.0,0.0,False,True,,Report #3669
HouseHoldCollection,Plates,1000.0,0.0,False,True,,
HouseHoldCollection,Plates,1000.0,0.0,False,False,Genuine,Report #3670
HouseHoldCollection,Plates,1000.0,0.0,False,False,Genuine,
HouseHoldCollection,Plates,1000.0,0.0,False,False,Fake,Report #3671
HouseHoldCollection,Plates,1000.0,0.0,False,False,Fake,
HouseHoldCollection,Plates,1000.0,0.0,False,False,,Report #3672
HouseHoldCollection,Plates,1000.0,0.0,False,False,,
HouseHoldCollection,Plates,1000.0,19.0,True,True,Genuine,Report #3673
HouseHoldCollection,Plates,1000.0,19.0,True,True,Genuine,
HouseHoldCollection,Plates,1000.0,19.0,True,True,Fake,Report #3674
HouseHoldCollection,Plates,1000.0,19.0,True,True,Fake,
HouseHoldCollection,Plates,1000.0,19.0,True,True,,Report #3675
HouseHoldCollection,Plates,1000.0,19.0,True,True,,
HouseHoldCollection,Plates,1000.0,19.0,True,False,Genuine,Report #3676
HouseHoldCollection,Plates,1000.0,19.0,True,False,Genuine,
HouseHoldCollection,Plates,1000.0,19.0,True,False,Fake,Report #3677
HouseHoldCollection,Plates,1000.0,19.0,True,False,Fake,
HouseHoldCollection,Plates,1000.0,19.0,True,False,,Report #3678
HouseHoldCollection,Plates,1000.0,19.0,True,False,,
HouseHoldCollection,Plates,1000.0,19.0,False,True,Genuine,Report #3679
HouseHoldCollection,Plates,1000.0,19.0,False,True,Genuine,
HouseHoldCollection,Plates,1000.0,19.0,False,True,Fake,Report #3680
HouseHoldCollection,Plates,1000.0,19.0,False,True,Fake,
HouseHoldCollection,Plates,1000.0,19.0,False,True,,Report #3681
HouseHoldCollection,Plates,1000.0,19.0,False,True,,
HouseHoldCollection,Plates,1000.0,19.0,False,False,Genuine,Report #3682
HouseHoldCollection,Plates,1000.0,19.0,False,False,Genuine,
HouseHoldCollection,Plates,1000.0,19.0,False,False,Fake,Report #3683
HouseHoldCollection,Plates,1000.0,19.0,False,False,Fake,
HouseHoldCollection,Plates,1000.0,19.0,False,False,,Report #3684
HouseHoldCollection,Plates,1000.0,19.0,False,False,,
HouseHoldCollection,Plates,1000.0,20.0,True,True,Genuine,Report #3685
HouseHoldCollection,Plates,1000.0,20.0,True,True,Genuine,
HouseHoldCollection,Plates,1000.0,20.0,True,True,Fake,Report #3686
HouseHoldCollection,Plates,1000.0,20.0,True,True,Fake,
HouseHoldCollection,Plates,1000.0,20.0,True,True,,Report #3687
HouseHoldCollection,Plates,1000.0,20.0,True,True,,
HouseHoldCollection,Plates,1000.0,20.0,True,False,Genuine,Report #3688
HouseHoldCollection,Plates,1000.0,20.0,True,False,Genuine,
HouseHoldCollection,Plates,1000.0,20.0,True,False,Fake,Report #3689
HouseHoldCollection,Plates,1000.0,20.0,True,False,Fake,
HouseHoldCollection,Plates,1000.0,20.0,True,False,,Report #3690
HouseHoldCollection,Plates,1000.0,20.0,True,False,,
HouseHoldCollection,Plates,1000.0,20.0,False,True,Genuine,Report #3691
HouseHoldCollection,Plates,1000.0,20.0,False,True,Genuine,
HouseHoldCollection,Plates,1000.0,20.0,False,True,Fake,Report #3692
HouseHoldCollection,Plates,1000.0,20.0,False,True,Fake,
HouseHoldCollection,Plates,1000.0,20.0,False,True,,Report #3693
HouseHoldCollection,Plates,1000.0,20.0,False,True,,
HouseHoldCollection,Plates,1000.0,20.0,False,False,Genuine,Report #3694
HouseHoldCollection,Plates,1000.0,20.0,False,False,Genuine,
HouseHoldCollection,Plates,1000.0,20.0,False,False,Fake,Report #3695
HouseHoldCollection,Plates,1000.0,20.0,False,False,Fake,
HouseHoldCollection,Plates,1000.0,20.0,False,False,,Report #3696
HouseHoldCollection,Plates,1000.0,20.0,False,False,,
HouseHoldCollection,Plates,1000.0,21.0,True,True,Genuine,Report #3697
HouseHoldCollection,Plates,1000.0,21.0,True,True,Genuine,
HouseHoldCollection,Plates,1000.0,21.0,True,True,Fake,Report #3698
HouseHoldCollection,Plates,1000.0,21.0,True,True,Fake,
HouseHoldCollection,Plates,1000.0,21.0,True,True,,Report #3699
HouseHoldCollection,Plates,1000.0,21.0,True,True,,
HouseHoldCollection,Plates,1000.0,21.0,True,False,Genuine,Report #3700
HouseHoldCollection,Plates,1000.0,21.0,True,False,Genuine,
HouseHoldCollection,Plates,1000.0,21.0,True,False,Fake,Report #3701
HouseHoldCollection,Plates,1000.0,21.0,True,False,Fake,
HouseHoldCollection,Plates,1000.0,21.0,True,False,,Report #3702
HouseHoldCollection,Plates,1000.0,21.0,True,False,,
HouseHoldCollection,Plates,1000.0,21.0,False,True,Genuine,Report #3703
HouseHoldCollection,Plates,1000.0,21.0,False,True,Genuine,
HouseHoldCollection,Plates,1000.0,21.0,False,True,Fake,Report #3704
HouseHoldCollection,Plates,1000.0,21.0,False,True,Fake,
HouseHoldCollection,Plates,1000.0,21.0,False,True,,Report #3705
HouseHoldCollection,Plates,1000.0,21.0,False,True,,
HouseHoldCollection,Plates,1000.0,21.0,False,False,Genuine,Report #3706
HouseHoldCollection,Plates,1000.0,21.0,False,False,Genuine,
HouseHoldCollection,Plates,1000.0,21.0,False,False,Fake,Report #3707
HouseHoldCollection,Plates,1000.0,21.0,False,False,Fake,
HouseHoldCollection,Plates,1000.0,21.0,False,False,,Report #3708
HouseHoldCollection,Plates,1000.0,21.0,False,False,,
HouseHoldCollection,Plates,1000.0,99.0,True,True,Genuine,Report #3709
HouseHoldCollection,Plates,1000.0,99.0,True,True,Genuine,
HouseHoldCollection,Plates,1000.0,99.0,True,True,Fake,Report #3710
HouseHoldCollection,Plates,1000.0,99.0,True,True,Fake,
HouseHoldCollection,Plates,1000.0,99.0,True,True,,Report #3711
HouseHoldCollection,Plates,1000.0,99.0,True,True,,
HouseHoldCollection,Plates,1000.0,99.0,True,False,Genuine,Report #3712
HouseHoldCollection,Plates,1000.0,99.0,True,False,Genuine,
HouseHoldCollection,Plates,1000.0,99.0,True,False,Fake,Report #3713
HouseHoldCollection,Plates,1000.0,99.0,True,False,Fake,
HouseHoldCollection,Plates,1000.0,99.0,True,False,,Report #3714
HouseHoldCollection,Plates,1000.0,99.0,True,False,,
HouseHoldCollection,Plates,1000.0,99.0,False,True,Genuine,Report #3715
HouseHoldCollection,Plates,1000.0,99.0,False,True,Genuine,
HouseHoldCollection,Plates,1000.0,99.0,False,True,Fake,Report #3716
HouseHoldCollection,Plates,1000.0,99.0,False,True,Fake,
HouseHoldCollection,Plates,1000.0,99.0,False,True,,Report #3717
HouseHoldCollection,Plates,1000.0,99.0,False,True,,
HouseHoldCollection,Plates,1000.0,99.0,False,False,Genuine,Report #3718
HouseHoldCollection,Plates,1000.0,99.0,False,False,Genuine,
HouseHoldCollection,Plates,1000.0,99.0,False,False,Fake,Report #3719
HouseHoldCollection,Plates,1000.0,99.0,False,False,Fake,
HouseHoldCollection,Plates,1000.0,99.0,False,False,,Report #3720
HouseHoldCollection,Plates,1000.0,99.0,False,False,,
HouseHoldCollection,Plates,1000.0,100.0,True,True,Genuine,Report #3721
HouseHoldCollection,Plates,1000.0,100.0,True,True,Genuine,
HouseHoldCollection,Plates,1000.0,100.0,True,True,Fake,Report #3722
HouseHoldCollection,Plates,1000.0,100.0,True,True,Fake,
HouseHoldCollection,Plates,1000.0,100.0,True,True,,Report #3723
HouseHoldCollection,Plates,1000.0,100.0,True,True,,
HouseHoldCollection,Plates,1000.0,100.0,True,False,Genuine,Report #3724
HouseHoldCollection,Plates,1000.0,100.0,True,False,Genuine,
HouseHoldCollection,Plates,1000.0,100.0,True,False,Fake,Report #3725
HouseHoldCollection,Plates,1000.0,100.0,True,False,Fake,
HouseHoldCollection,Plates,1000.0,100.0,True,False,,Report #3726
HouseHoldCollection,Plates,1000.0,100.0,True,False,,
HouseHoldCollection,Plates,1000.0,100.0,False,True,Genuine,Report #3727
HouseHoldCollection,Plates,1000.0,100.0,False,True,Genuine,
HouseHoldCollection,Plates,1000.0,100.0,False,True,Fake,Report #3728
HouseHoldCollection,Plates,1000.0,100.0,False,True,Fake,
HouseHoldCollection,Plates,1000.0,100.0,False,True,,Report #3729
HouseHoldCollection,Plates,1000.0,100.0,False,True,,
HouseHoldCollection,Plates,1000.0,100.0,False,False,Genuine,Report #3730
HouseHoldCollection,Plates,1000.0,100.0,False,False,Genuine,
HouseHoldCollection,Plates,1000.0,100.0,False,False,Fake,Report #3731
HouseHoldCollection,Plates,1000.0,100.0,False,False,Fake,
HouseHoldCollection,Plates,1000.0,100.0,False,False,,Report #3732
HouseHoldCollection,Plates,1000.0,100.0,False,False,,
HouseHoldCollection,Plates,1000.0,101.0,True,True,Genuine,Report #3733
HouseHoldCollection,Plates,1000.0,101.0,True,True,Genuine,
HouseHoldCollection,Plates,1000.0,101.0,True,True,Fake,Report #3734
HouseHoldCollection,Plates,1000.0,101.0,True,True,Fake,
HouseHoldCollection,Plates,1000.0,101.0,True,True,,Report #3735
HouseHoldCollection,Plates,1000.0,101.0,True,True,,
HouseHoldCollection,Plates,1000.0,101.0,True,False,Genuine,Report #3736
HouseHoldCollection,Plates,1000.0,101.0,True,False,Genuine,
HouseHoldCollection,Plates,1000.0,101.0,True,False,Fake,Report #3737
HouseHoldCollection,Plates,1000.0,101.0,True,False,Fake,
HouseHoldCollection,Plates,1000.0,101.0,True,False,,Report #3738
HouseHoldCollection,Plates,1000.0,101.0,True,False,,
HouseHoldCollection,Plates,1000.0,101.0,False,True,Genuine,Report #3739
HouseHoldCollection,Plates,1000.0,101.0,False,True,Genuine,
HouseHoldCollection,Plates,1000.0,101.0,False,True,Fake,Report #3740
HouseHoldCollection,Plates,1000.0,101.0,False,True,Fake,
HouseHoldCollection,Plates,1000.0,101.0,False,True,,Report #3741
HouseHoldCollection,Plates,1000.0,101.0,False,True,,
HouseHoldCollection,Plates,1000.0,101.0,False,False,Genuine,Report #3742
HouseHoldCollection,Plates,1000.0,101.0,False,False,Genuine,
HouseHoldCollection,Plates,1000.0,101.0,False,False,Fake,Report #3743
HouseHoldCollection,Plates,1000.0,101.0,False,False,Fake,
HouseHoldCollection,Plates,1000.0,101.0,False,False,,Report #3744
HouseHoldCollection,Plates,1000.0,101.0,False,False,,
HouseHoldCollection,Plates,1000000.0,-1.0,True,True,Genuine,Report #3745
HouseHoldCollection,Plates,1000000.0,-1.0,True,True,Genuine,
HouseHoldCollection,Plates,1000000.0,-1.0,True,True,Fake,Report #3746
HouseHoldCollection,Plates,1000000.0,-1.0,True,True,Fake,
HouseHoldCollection,Plates,1000000.0,-1.0,True,True,,Report #3747
HouseHoldCollection,Plates,1000000.0,-1.0,True,True,,
HouseHoldCollection,Plates,1000000.0,-1.0,True,False,Genuine,Report #3748
HouseHoldCollection,Plates,1000000.0,-1.0,True,False,Genuine,
HouseHoldCollection,Plates,1000000.0,-1.0,True,False,Fake,Report #3749
HouseHoldCollection,Plates,1000000.0,-1.0,True,False,Fake,
HouseHoldCollection,Plates,1000000.0,-1.0,True,False,,Report #3750
HouseHoldCollection,Plates,1000000.0,-1.0,True,False,,
HouseHoldCollection,Plates,1000000.0,-1.0,False,True,Genuine,Report #3751
HouseHoldCollection,Plates,1000000.0,-1.0,False,True,Genuine,
HouseHoldCollection,Plates,1000000.0,-1.0,False,True,Fake,Report #3752
HouseHoldCollection,Plates,1000000.0,-1.0,False,True,Fake,
HouseHoldCollection,Plates,1000000.0,-1.0,False,True,,Report #3753
HouseHoldCollection,Plates,1000000.0,-1.0,False,True,,
HouseHoldCollection,Plates,1000000.0,-1.0,False,False,Genuine,Report #3754
HouseHoldCollection,Plates,1000000.0,-1.0,False,False,Genuine,
HouseHoldCollection,Plates,1000000.0,-1.0,False,False,Fake,Report #3755
HouseHoldCollection,Plates,1000000.0,-1.0,False,False,Fake,
HouseHoldCollection,Plates,1000000.0,-1.0,False,False,,Report #3756
HouseHoldCollection,Plates,1000000.0,-1.0,False,False,,
HouseHoldCollection,Plates,1000000.0,0.0,True,True,Genuine,Report #3757
HouseHoldCollection,Plates,1000000.0,0.0,True,True,Genuine,
HouseHoldCollection,Plates,1000000.0,0.0,True,True,Fake,Report #3758
HouseHoldCollection,Plates,1000000.0,0.0,True,True,Fake,
HouseHoldCollection,Plates,1000000.0,0.0,True,True,,Report #3759
HouseHoldCollection,Plates,1000000.0,0.0,True,True,,
HouseHoldCollection,Plates,1000000.0,0.0,True,False,Genuine,Report #3760
HouseHoldCollection,Plates,1000000.0,0.0,True,False,Genuine,
HouseHoldCollection,Plates,1000000.0,0.0,True,False,Fake,Report #3761
HouseHoldCollection,Plates,1000000.0,0.0,True,False,Fake,
HouseHoldCollection,Plates,1000000.0,0.0,True,False,,Report #3762
HouseHoldCollection,Plates,1000000.0,0.0,True,False,,
HouseHoldCollection,Plates,1000000.0,0.0,False,True,Genuine,Report #3763
HouseHoldCollection,Plates,1000000.0,0.0,False,True,Genuine,
HouseHoldCollection,Plates,1000000.0,0.0,False,True,Fake,Report #3764
HouseHoldCollection,Plates,1000000.0,0.0,False,True,Fake,
HouseHoldCollection,Plates,1000000.0,0.0,False,True,,Report #3765
HouseHoldCollection,Plates,1000000.0,0.0,False,True,,
HouseHoldCollection,Plates,1000000.0,0.0,False,False,Genuine,Report #3766
HouseHoldCollection,Plates,1000000.0,0.0,False,False,Genuine,
HouseHoldCollection,Plates,1000000.0,0.0,False,False,Fake,Report #3767
HouseHoldCollection,Plates,1000000.0,0.0,False,False,Fake,
HouseHoldCollection,Plates,1000000.0,0.0,False,False,,Report #3768
HouseHoldCollection,Plates,1000000.0,0.0,False,False,,
HouseHoldCollection,Plates,1000000.0,19.0,True,True,Genuine,Report #3769
HouseHoldCollection,Plates,1000000.0,19.0,True,True,Genuine,
HouseHoldCollection,Plates,1000000.0,19.0,True,True,Fake,Report #3770
HouseHoldCollection,Plates,1000000.0,19.0,True,True,Fake,
HouseHoldCollection,Plates,1000000.0,19.0,True,True,,Report #3771
HouseHoldCollection,Plates,1000000.0,19.0,True,True,,
HouseHoldCollection,Plates,1000000.0,19.0,True,False,Genuine,Report #3772
HouseHoldCollection,Plates,1000000.0,19.0,True,False,Genuine,
HouseHoldCollection,Plates,1000000.0,19.0,True,False,Fake,Report #3773
HouseHoldCollection,Plates,1000000.0,19.0,True,False,Fake,
HouseHoldCollection,Plates,1000000.0,19.0,True,False,,Report #3774
HouseHoldCollection,Plates,1000000.0,19.0,True,False,,
HouseHoldCollection,Plates,1000000.0,19.0,False,True,Genuine,Report #3775
HouseHoldCollection,Plates,1000000.0,19.0,False,True,Genuine,
HouseHoldCollection,Plates,1000000.0,19.0,False,True,Fake,Report #3776
HouseHoldCollection,Plates,1000000.0,19.0,False,True,Fake,
HouseHoldCollection,Plates,1000000.0,19.0,False,True,,Report #3777
HouseHoldCollection,Plates,1000000.0,19.0,False,True,,
HouseHoldCollection,Plates,1000000.0,19.0,False,False,Genuine,Report #3778
HouseHoldCollection,Plates,1000000.0,19.0,False,False,Genuine,
HouseHoldCollection,Plates,1000000.0,19.0,False,False,Fake,Report #3779
HouseHoldCollection,Plates,1000000.0,19.0,False,False,Fake,
HouseHoldCollection,Plates,1000000.0,19.0,False,False,,Report #3780
HouseHoldCollection,Plates,1000000.0,19.0,False,False,,
HouseHoldCollection,Plates,1000000.0,20.0,True,True,Genuine,Report #3781
HouseHoldCollection,Plates,1000000.0,20.0,True,True,Genuine,
HouseHoldCollection,Plates,1000000.0,20.0,True,True,Fake,Report #3782
HouseHoldCollection,Plates,1000000.0,20.0,True,True,Fake,
HouseHoldCollection,Plates,1000000.0,20.0,True,True,,Report #3783
HouseHoldCollection,Plates,1000000.0,20.0,True,True,,
HouseHoldCollection,Plates,1000000.0,20.0,True,False,Genuine,Report #3784
HouseHoldCollection,Plates,1000000.0,20.0,True,False,Genuine,
HouseHoldCollection,Plates,1000000.0,20.0,True,False,Fake,Report #3785
HouseHoldCollection,Plates,1000000.0,20.0,True,False,Fake,
HouseHoldCollection,Plates,1000000.0,20.0,True,False,,Report #3786
HouseHoldCollection,Plates,1000000.0,20.0,True,False,,
HouseHoldCollection,Plates,1000000.0,20.0,False,True,Genuine,Report #3787
HouseHoldCollection,Plates,1000000.0,20.0,False,True,Genuine,
HouseHoldCollection,Plates,1000000.0,20.0,False,True,Fake,Report #3788
HouseHoldCollection,Plates,1000000.0,20.0,False,True,Fake,
HouseHoldCollection,Plates,1000000.0,20.0,False,True,,Report #3789
HouseHoldCollection,Plates,1000000.0,20.0,False,True,,
HouseHoldCollection,Plates,1000000.0,20.0,False,False,Genuine,Report #3790
HouseHoldCollection,Plates,1000000.0,20.0,False,False,Genuine,
HouseHoldCollection,Plates,1000000.0,20.0,False,False,Fake,Report #3791
HouseHoldCollection,Plates,1000000.0,20.0,False,False,Fake,
HouseHoldCollection,Plates,1000000.0,20.0,False,False,,Report #3792
HouseHoldCollection,Plates,1000000.0,20.0,False,False,,
HouseHoldCollection,Plates,1000000.0,21.0,True,True,Genuine,Report #3793
HouseHoldCollection,Plates,1000000.0,21.0,True,True,Genuine,
HouseHoldCollection,Plates,1000000.0,21.0,True,True,Fake,Report #3794
HouseHoldCollection,Plates,1000000.0,21.0,True,True,Fake,
HouseHoldCollection,Plates,1000000.0,21.0,True,True,,Report #3795
HouseHoldCollection,Plates,1000000.0,21.0,True,True,,
HouseHoldCollection,Plates,1000000.0,21.0,True,False,Genuine,Report #3796
HouseHoldCollection,Plates,1000000.0,21.0,True,False,Genuine,
HouseHoldCollection,Plates,1000000.0,21.0,True,False,Fake,Report #3797
HouseHoldCollection,Plates,1000000.0,21.0,True,False,Fake,
HouseHoldCollection,Plates,1000000.0,21.0,True,False,,Report #3798
HouseHoldCollection,Plates,1000000.0,21.0,True,False,,
HouseHoldCollection,Plates,1000000.0,21.0,False,True,Genuine,Report #3799
HouseHoldCollection,Plates,1000000.0,21.0,False,True,Genuine,
HouseHoldCollection,Plates,1000000.0,21.0,False,True,Fake,Report #3800
HouseHoldCollection,Plates,1000000.0,21.0,False,True,Fake,
HouseHoldCollection,Plates,1000000.0,21.0,False,True,,Report #3801
HouseHoldCollection,Plates,1000000.0,21.0,False,True,,
HouseHoldCollection,Plates,1000000.0,21.0,False,False,Genuine,Report #3802
HouseHoldCollection,Plates,1000000.0,21.0,False,False,Genuine,
HouseHoldCollection,Plates,1000000.0,21.0,False,False,Fake,Report #3803
HouseHoldCollection,Plates,1000000.0,21.0,False,False,Fake,
HouseHoldCollection,Plates,1000000.0,21.0,False,False,,Report #3804
HouseHoldCollection,Plates,1000000.0,21.0,False,False,,
HouseHoldCollection,Plates,1000000.0,99.0,True,True,Genuine,Report #3805
HouseHoldCollection,Plates,1000000.0,99.0,True,True,Genuine,
HouseHoldCollection,Plates,1000000.0,99.0,True,True,Fake,Report #3806
HouseHoldCollection,Plates,1000000.0,99.0,True,True,Fake,
HouseHoldCollection,Plates,1000000.0,99.0,True,True,,Report #3807
HouseHoldCollection,Plates,1000000.0,99.0,True,True,,
HouseHoldCollection,Plates,1000000.0,99.0,True,False,Genuine,Report #3808
HouseHoldCollection,Plates,1000000.0,99.0,True,False,Genuine,
HouseHoldCollection,Plates,1000000.0,99.0,True,False,Fake,Report #3809
HouseHoldCollection,Plates,1000000.0,99.0,True,False,Fake,
HouseHoldCollection,Plates,1000000.0,99.0,True,False,,Report #3810
HouseHoldCollection,Plates,1000000.0,99.0,True,False,,
HouseHoldCollection,Plates,1000000.0,99.0,False,True,Genuine,Report #3811
HouseHoldCollection,Plates,1000000.0,99.0,False,True,Genuine,
HouseHoldCollection,Plates,1000000.0,99.0,False,True,Fake,Report #3812
HouseHoldCollection,Plates,1000000.0,99.0,False,True,Fake,
HouseHoldCollection,Plates,1000000.0,99.0,False,True,,Report #3813
HouseHoldCollection,Plates,1000000.0,99.0,False,True,,
HouseHoldCollection,Plates,1000000.0,99.0,False,False,Genuine,Report #3814
HouseHoldCollection,Plates,1000000.0,99.0,False,False,Genuine,
HouseHoldCollection,Plates,1000000.0,99.0,False,False,Fake,Report #3815
HouseHoldCollection,Plates,1000000.0,99.0,False,False,Fake,
HouseHoldCollection,Plates,1000000.0,99.0,False,False,,Report #3816
HouseHoldCollection,Plates,1000000.0,99.0,False,False,,
HouseHoldCollection,Plates,1000000.0,100.0,True,True,Genuine,Report #3817
HouseHoldCollection,Plates,1000000.0,100.0,True,True,Genuine,
HouseHoldCollection,Plates,1000000.0,100.0,True,True,Fake,Report #3818
HouseHoldCollection,Plates,1000000.0,100.0,True,True,Fake,
HouseHoldCollection,Plates,1000000.0,100.0,True,True,,Report #3819
HouseHoldCollection,Plates,1000000.0,100.0,True,True,,
HouseHoldCollection,Plates,1000000.0,100.0,True,False,Genuine,Report #3820
HouseHoldCollection,Plates,1000000.0,100.0,True,False,Genuine,
HouseHoldCollection,Plates,1000000.0,100.0,True,False,Fake,Report #3821
HouseHoldCollection,Plates,1000000.0,100.0,True,False,Fake,
HouseHoldCollection,Plates,1000000.0,100.0,True,False,,Report #3822
HouseHoldCollection,Plates,1000000.0,100.0,True,False,,
HouseHoldCollection,Plates,1000000.0,100.0,False,True,Genuine,Report #3823
HouseHoldCollection,Plates,1000000.0,100.0,False,True,Genuine,
HouseHoldCollection,Plates,1000000.0,100.0,False,True,Fake,Report #3824
HouseHoldCollection,Plates,1000000.0,100.0,False,True,Fake,
HouseHoldCollection,Plates,1000000.0,100.0,False,True,,Report #3825
HouseHoldCollection,Plates,1000000.0,100.0,False,True,,
HouseHoldCollection,Plates,1000000.0,100.0,False,False,Genuine,Report #3826
HouseHoldCollection,Plates,1000000.0,100.0,False,False,Genuine,
HouseHoldCollection,Plates,1000000.0,100.0,False,False,Fake,Report #3827
HouseHoldCollection,Plates,1000000.0,100.0,False,False,Fake,
HouseHoldCollection,Plates,1000000.0,100.0,False,False,,Report #3828
HouseHoldCollection,Plates,1000000.0,100.0,False,False,,
HouseHoldCollection,Plates,1000000.0,101.0,True,True,Genuine,Report #3829
HouseHoldCollection,Plates,1000000.0,101.0,True,True,Genuine,
HouseHoldCollection,Plates,1000000.0,101.0,True,True,Fake,Report #3830
HouseHoldCollection,Plates,1000000.0,101.0,True,True,Fake,
HouseHoldCollection,Plates,1000000.0,101.0,True,True,,Report #3831
HouseHoldCollection,Plates,1000000.0,101.0,True,True,,
HouseHoldCollection,Plates,1000000.0,101.0,True,False,Genuine,Report #3832
HouseHoldCollection,Plates,1000000.0,101.0,True,False,Genuine,
HouseHoldCollection,Plates,1000000.0,101.0,True,False,Fake,Report #3833
HouseHoldCollection,Plates,1000000.0,101.0,True,False,Fake,
HouseHoldCollection,Plates,1000000.0,101.0,True,False,,Report #3834
HouseHoldCollection,Plates,1000000.0,101.0,True,False,,
HouseHoldCollection,Plates,1000000.0,101.0,False,True,Genuine,Report #3835
HouseHoldCollection,Plates,1000000.0,101.0,False,True,Genuine,
HouseHoldCollection,Plates,1000000.0,101.0,False,True,Fake,Report #3836
HouseHoldCollection,Plates,1000000.0,101.0,False,True,Fake,
HouseHoldCollection,Plates,1000000.0,101.0,False,True,,Report #3837
HouseHoldCollection,Plates,1000000.0,101.0,False,True,,
HouseHoldCollection,Plates,1000000.0,101.0,False,False,Genuine,Report #3838
HouseHoldCollection,Plates,1000000.0,101.0,False,False,Genuine,
HouseHoldCollection,Plates,1000000.0,101.0,False,False,Fake,Report #3839
HouseHoldCollection,Plates,1000000.0,101.0,False,False,Fake,
HouseHoldCollection,Plates,1000000.0,101.0,False,False,,Report #3840
HouseHoldCollection,Plates,1000000.0,101.0,False,False,,
HouseHoldCollection,Furniture,0.0,-1.0,True,True,Genuine,Report #3841
HouseHoldCollection,Furniture,0.0,-1.0,True,True,Genuine,
HouseHoldCollection,Furniture,0.0,-1.0,True,True,Fake,Report #3842
HouseHoldCollection,Furniture,0.0,-1.0,True,True,Fake,
HouseHoldCollection,Furniture,0.0,-1.0,True,True,,Report #3843
HouseHoldCollection,Furniture,0.0,-1.0,True,True,,
HouseHoldCollection,Furniture,0.0,-1.0,True,False,Genuine,Report #3844
HouseHoldCollection,Furniture,0.0,-1.0,True,False,Genuine,
HouseHoldCollection,Furniture,0.0,-1.0,True,False,Fake,Report #3845
HouseHoldCollection,Furniture,0.0,-1.0,True,False,Fake,
HouseHoldCollection,Furniture,0.0,-1.0,True,False,,Report #3846
HouseHoldCollection,Furniture,0.0,-1.0,True,False,,
HouseHoldCollection,Furniture,0.0,-1.0,False,True,Genuine,Report #3847
HouseHoldCollection,Furniture,0.0,-1.0,False,True,Genuine,
HouseHoldCollection,Furniture,0.0,-1.0,False,True,Fake,Report #3848
HouseHoldCollection,Furniture,0.0,-1.0,False,True,Fake,
HouseHoldCollection,Furniture,0.0,-1.0,False,True,,Report #3849
HouseHoldCollection,Furniture,0.0,-1.0,False,True,,
HouseHoldCollection,Furniture,0.0,-1.0,False,False,Genuine,Report #3850
HouseHoldCollection,Furniture,0.0,-1.0,False,False,Genuine,
HouseHoldCollection,Furniture,0.0,-1.0,False,False,Fake,Report #3851
HouseHoldCollection,Furniture,0.0,-1.0,False,False,Fake,
HouseHoldCollection,Furniture,0.0,-1.0,False,False,,Report #3852
HouseHoldCollection,Furniture,0.0,-1.0,False,False,,
HouseHoldCollection,Furniture,0.0,0.0,True,True,Genuine,Report #3853
HouseHoldCollection,Furniture,0.0,0.0,True,True,Genuine,
HouseHoldCollection,Furniture,0.0,0.0,True,True,Fake,Report #3854
HouseHoldCollection,Furniture,0.0,0.0,True,True,Fake,
HouseHoldCollection,Furniture,0.0,0.0,True,True,,Report #3855
HouseHoldCollection,Furniture,0.0,0.0,True,True,,
HouseHoldCollection,Furniture,0.0,0.0,True,False,Genuine,Report #3856
HouseHoldCollection,Furniture,0.0,0.0,True,False,Genuine,
HouseHoldCollection,Furniture,0.0,0.0,True,False,Fake,Report #3857
HouseHoldCollection,Furniture,0.0,0.0,True,False,Fake,
HouseHoldCollection,Furniture,0.0,0.0,True,False,,Report #3858
HouseHoldCollection,Furniture,0.0,0.0,True,False,,
HouseHoldCollection,Furniture,0.0,0.0,False,True,Genuine,Report #3859
HouseHoldCollection,Furniture,0.0,0.0,False,True,Genuine,
HouseHoldCollection,Furniture,0.0,0.0,False,True,Fake,Report #3860
HouseHoldCollection,Furniture,0.0,0.0,False,True,Fake,
HouseHoldCollection,Furniture,0.0,0.0,False,True,,Report #3861
HouseHoldCollection,Furniture,0.0,0.0,False,True,,
HouseHoldCollection,Furniture,0.0,0.0,False,False,Genuine,Report #3862
HouseHoldCollection,Furniture,0.0,0.0,False,False,Genuine,
HouseHoldCollection,Furniture,0.0,0.0,False,False,Fake,Report #3863
HouseHoldCollection,Furniture,0.0,0.0,False,False,Fake,
HouseHoldCollection,Furniture,0.0,0.0,False,False,,Report #3864
HouseHoldCollection,Furniture,0.0,0.0,False,False,,
HouseHoldCollection,Furniture,0.0,19.0,True,True,Genuine,Report #3865
HouseHoldCollection,Furniture,0.0,19.0,True,True,Genuine,
HouseHoldCollection,Furniture,0.0,19.0,True,True,Fake,Report #3866
HouseHoldCollection,Furniture,0.0,19.0,True,True,Fake,
HouseHoldCollection,Furniture,0.0,19.0,True,True,,Report #3867
HouseHoldCollection,Furniture,0.0,19.0,True,True,,
HouseHoldCollection,Furniture,0.0,19.0,True,False,Genuine,Report #3868
HouseHoldCollection,Furniture,0.0,19.0,True,False,Genuine,
HouseHoldCollection,Furniture,0.0,19.0,True,False,Fake,Report #3869
HouseHoldCollection,Furniture,0.0,19.0,True,False,Fake,
HouseHoldCollection,Furniture,0.0,19.0,True,False,,Report #3870
HouseHoldCollection,Furniture,0.0,19.0,True,False,,
HouseHoldCollection,Furniture,0.0,19.0,False,True,Genuine,Report #3871
HouseHoldCollection,Furniture,0.0,19.0,False,True,Genuine,
HouseHoldCollection,Furniture,0.0,19.0,False,True,Fake,Report #3872
HouseHoldCollection,Furniture,0.0,19.0,False,True,Fake,
HouseHoldCollection,Furniture,0.0,19.0,False,True,,Report #3873
HouseHoldCollection,Furniture,0.0,19.0,False,True,,
HouseHoldCollection,Furniture,0.0,19.0,False,False,Genuine,Report #3874
HouseHoldCollection,Furniture,0.0,19.0,False,False,Genuine,
HouseHoldCollection,Furniture,0.0,19.0,False,False,Fake,Report #3875
HouseHoldCollection,Furniture,0.0,19.0,False,False,Fake,
HouseHoldCollection,Furniture,0.0,19.0,False,False,,Report #3876
HouseHoldCollection,Furniture,0.0,19.0,False,False,,
HouseHoldCollection,Furniture,0.0,20.0,True,True,Genuine,Report #3877
HouseHoldCollection,Furniture,0.0,20.0,True,True,Genuine,
HouseHoldCollection,Furniture,0.0,20.0,True,True,Fake,Report #3878
HouseHoldCollection,Furniture,0.0,20.0,True,True,Fake,
HouseHoldCollection,Furniture,0.0,20.0,True,True,,Report #3879
HouseHoldCollection,Furniture,0.0,20.0,True,True,,
HouseHoldCollection,Furniture,0.0,20.0,True,False,Genuine,Report #3880
HouseHoldCollection,Furniture,0.0,20.0,True,False,Genuine,
HouseHoldCollection,Furniture,0.0,20.0,True,False,Fake,Report #3881
HouseHoldCollection,Furniture,0.0,20.0,True,False,Fake,
HouseHoldCollection,Furniture,0.0,20.0,True,False,,Report #3882
HouseHoldCollection,Furniture,0.0,20.0,True,False,,
HouseHoldCollection,Furniture,0.0,20.0,False,True,Genuine,Report #3883
HouseHoldCollection,Furniture,0.0,20.0,False,True,Genuine,
HouseHoldCollection,Furniture,0.0,20.0,False,True,Fake,Report #3884
HouseHoldCollection,Furniture,0.0,20.0,False,True,Fake,
HouseHoldCollection,Furniture,0.0,20.0,False,True,,Report #3885
HouseHoldCollection,Furniture,0.0,20.0,False,True,,
HouseHoldCollection,Furniture,0.0,20.0,False,False,Genuine,Report #3886
HouseHoldCollection,Furniture,0.0,20.0,False,False,Genuine,
HouseHoldCollection,Furniture,0.0,20.0,False,False,Fake,Report #3887
HouseHoldCollection,Furniture,0.0,20.0,False,False,Fake,
HouseHoldCollection,Furniture,0.0,20.0,False,False,,Report #3888
HouseHoldCollection,Furniture,0.0,20.0,False,False,,
HouseHoldCollection,Furniture,0.0,21.0,True,True,Genuine,Report #3889
HouseHoldCollection,Furniture,0.0,21.0,True,True,Genuine,
HouseHoldCollection,Furniture,0.0,21.0,True,True,Fake,Report #3890
HouseHoldCollection,Furniture,0.0,21.0,True,True,Fake,
HouseHoldCollection,Furniture,0.0,21.0,True,True,,Report #3891
HouseHoldCollection,Furniture,0.0,21.0,True,True,,
HouseHoldCollection,Furniture,0.0,21.0,True,False,Genuine,Report #3892
HouseHoldCollection,Furniture,0.0,21.0,True,False,Genuine,
HouseHoldCollection,Furniture,0.0,21.0,True,False,Fake,Report #3893
HouseHoldCollection,Furniture,0.0,21.0,True,False,Fake,
HouseHoldCollection,Furniture,0.0,21.0,True,False,,Report #3894
HouseHoldCollection,Furniture,0.0,21.0,True,False,,
HouseHoldCollection,Furniture,0.0,21.0,False,True,Genuine,Report #3895
HouseHoldCollection,Furniture,0.0,21.0,False,True,Genuine,
HouseHoldCollection,Furniture,0.0,21.0,False,True,Fake,Report #3896
HouseHoldCollection,Furniture,0.0,21.0,False,True,Fake,
HouseHoldCollection,Furniture,0.0,21.0,False,True,,Report #3897
HouseHoldCollection,Furniture,0.0,21.0,False,True,,
HouseHoldCollection,Furniture,0.0,21.0,False,False,Genuine,Report #3898
HouseHoldCollection,Furniture,0.0,21.0,False,False,Genuine,
HouseHoldCollection,Furniture,0.0,21.0,False,False,Fake,Report #3899
HouseHoldCollection,Furniture,0.0,21.0,False,False,Fake,
HouseHoldCollection,Furniture,0.0,21.0,False,False,,Report #3900
HouseHoldCollection,Furniture,0.0,21.0,False,False,,
HouseHoldCollection,Furniture,0.0,99.0,True,True,Genuine,Report #3901
HouseHoldCollection,Furniture,0.0,99.0,True,True,Genuine,
HouseHoldCollection,Furniture,0.0,99.0,True,True,Fake,Report #3902
HouseHoldCollection,Furniture,0.0,99.0,True,True,Fake,
HouseHoldCollection,Furniture,0.0,99.0,True,True,,Report #3903
HouseHoldCollection,Furniture,0.0,99.0,True,True,,
HouseHoldCollection,Furniture,0.0,99.0,True,False,Genuine,Report #3904
HouseHoldCollection,Furniture,0.0,99.0,True,False,Genuine,
HouseHoldCollection,Furniture,0.0,99.0,True,False,Fake,Report #3905
HouseHoldCollection,Furniture,0.0,99.0,True,False,Fake,
HouseHoldCollection,Furniture,0.0,99.0,True,False,,Report #3906
HouseHoldCollection,Furniture,0.0,99.0,True,False,,
HouseHoldCollection,Furniture,0.0,99.0,False,True,Genuine,Report #3907
HouseHoldCollection,Furniture,0.0,99.0,False,True,Genuine,
HouseHoldCollection,Furniture,0.0,99.0,False,True,Fake,Report #3908
HouseHoldCollection,Furniture,0.0,99.0,False,True,Fake,
HouseHoldCollection,Furniture,0.0,99.0,False,True,,Report #3909
HouseHoldCollection,Furniture,0.0,99.0,False,True,,
HouseHoldCollection,Furniture,0.0,99.0,False,False,Genuine,Report #3910
HouseHoldCollection,Furniture,0.0,99.0,False,False,Genuine,
HouseHoldCollection,Furniture,0.0,99.0,False,False,Fake,Report #3911
HouseHoldCollection,Furniture,0.0,99.0,False,False,Fake,
HouseHoldCollection,Furniture,0.0,99.0,False,False,,Report #3912
HouseHoldCollection,Furniture,0.0,99.0,False,False,,
HouseHoldCollection,Furniture,0.0,100.0,True,True,Genuine,Report #3913
HouseHoldCollection,Furniture,0.0,100.0,True,True,Genuine,
HouseHoldCollection,Furniture,0.0,100.0,True,True,Fake,Report #3914
HouseHoldCollection,Furniture,0.0,100.0,True,True,Fake,
HouseHoldCollection,Furniture,0.0,100.0,True,True,,Report #3915
HouseHoldCollection,Furniture,0.0,100.0,True,True,,
HouseHoldCollection,Furniture,0.0,100.0,True,False,Genuine,Report #3916
HouseHoldCollection,Furniture,0.0,100.0,True,False,Genuine,
HouseHoldCollection,Furniture,0.0,100.0,True,False,Fake,Report #3917
HouseHoldCollection,Furniture,0.0,100.0,True,False,Fake,
HouseHoldCollection,Furniture,0.0,100.0,True,False,,Report #3918
HouseHoldCollection,Furniture,0.0,100.0,True,False,,
HouseHoldCollection,Furniture,0.0,100.0,False,True,Genuine,Report #3919
HouseHoldCollection,Furniture,0.0,100.0,False,True,Genuine,
HouseHoldCollection,Furniture,0.0,100.0,False,True,Fake,Report #3920
HouseHoldCollection,Furniture,0.0,100.0,False,True,Fake,
HouseHoldCollection,Furniture,0.0,100.0,False,True,,Report #3921
HouseHoldCollection,Furniture,0.0,100.0,False,True,,
HouseHoldCollection,Furniture,0.0,100.0,False,False,Genuine,Report #3922
HouseHoldCollection,Furniture,0.0,100.0,False,False,Genuine,
HouseHoldCollection,Furniture,0.0,100.0,False,False,Fake,Report #3923
HouseHoldCollection,Furniture,0.0,100.0,False,False,Fake,
HouseHoldCollection,Furniture,0.0,100.0,False,False,,Report #3924
HouseHoldCollection,Furniture,0.0,100.0,False,False,,
HouseHoldCollection,Furniture,0.0,101.0,True,True,Genuine,Report #3925
HouseHoldCollection,Furniture,0.0,101.0,True,True,Genuine,
HouseHoldCollection,Furniture,0.0,101.0,True,True,Fake,Report #3926
HouseHoldCollection,Furniture,0.0,101.0,True,True,Fake,
HouseHoldCollection,Furniture,0.0,101.0,True,True,,Report #3927
HouseHoldCollection,Furniture,0.0,101.0,True,True,,
HouseHoldCollection,Furniture,0.0,101.0,True,False,Genuine,Report #3928
HouseHoldCollection,Furniture,0.0,101.0,True,False,Genuine,
HouseHoldCollection,Furniture,0.0,101.0,True,False,Fake,Report #3929
HouseHoldCollection,Furniture,0.0,101.0,True,False,Fake,
HouseHoldCollection,Furniture,0.0,101.0,True,False,,Report #3930
HouseHoldCollection,Furniture,0.0,101.0,True,False,,
HouseHoldCollection,Furniture,0.0,101.0,False,True,Genuine,Report #3931
HouseHoldCollection,Furniture,0.0,101.0,False,True,Genuine,
HouseHoldCollection,Furniture,0.0,101.0,False,True,Fake,Report #3932
HouseHoldCollection,Furniture,0.0,101.0,False,True,Fake,
HouseHoldCollection,Furniture,0.0,101.0,False,True,,Report #3933
HouseHoldCollection,Furniture,0.0,101.0,False,True,,
HouseHoldCollection,Furniture,0.0,101.0,False,False,Genuine,Report #3934
HouseHoldCollection,Furniture,0.0,101.0,False,False,Genuine,
HouseHoldCollection,Furniture,0.0,101.0,False,False,Fake,Report #3935
HouseHoldCollection,Furniture,0.0,101.0,False,False,Fake,
HouseHoldCollection,Furniture,0.0,101.0,False,False,,Report #3936
HouseHoldCollection,Furniture,0.0,101.0,False,False,,
HouseHoldCollection,Furniture,1.0,-1.0,True,True,Genuine,Report #3937
HouseHoldCollection,Furniture,1.0,-1.0,True,True,Genuine,
HouseHoldCollection,Furniture,1.0,-1.0,True,True,Fake,Report #3938
HouseHoldCollection,Furniture,1.0,-1.0,True,True,Fake,
HouseHoldCollection,Furniture,1.0,-1.0,True,True,,Report #3939
HouseHoldCollection,Furniture,1.0,-1.0,True,True,,
HouseHoldCollection,Furniture,1.0,-1.0,True,False,Genuine,Report #3940
HouseHoldCollection,Furniture,1.0,-1.0,True,False,Genuine,
HouseHoldCollection,Furniture,1.0,-1.0,True,False,Fake,Report #3941
HouseHoldCollection,Furniture,1.0,-1.0,True,False,Fake,
HouseHoldCollection,Furniture,1.0,-1.0,True,False,,Report #3942
HouseHoldCollection,Furniture,1.0,-1.0,True,False,,
HouseHoldCollection,Furniture,1.0,-1.0,False,True,Genuine,Report #3943
HouseHoldCollection,Furniture,1.0,-1.0,False,True,Genuine,
HouseHoldCollection,Furniture,1.0,-1.0,False,True,Fake,Report #3944
HouseHoldCollection,Furniture,1.0,-1.0,False,True,Fake,
HouseHoldCollection,Furniture,1.0,-1.0,False,True,,Report #3945
HouseHoldCollection,Furniture,1.0,-1.0,False,True,,
HouseHoldCollection,Furniture,1.0,-1.0,False,False,Genuine,Report #3946
HouseHoldCollection,Furniture,1.0,-1.0,False,False,Genuine,
HouseHoldCollection,Furniture,1.0,-1.0,False,False,Fake,Report #3947
HouseHoldCollection,Furniture,1.0,-1.0,False,False,Fake,
HouseHoldCollection,Furniture,1.0,-1.0,False,False,,Report #3948
HouseHoldCollection,Furniture,1.0,-1.0,False,False,,
HouseHoldCollection,Furniture,1.0,0.0,True,True,Genuine,Report #3949
HouseHoldCollection,Furniture,1.0,0.0,True,True,Genuine,
HouseHoldCollection,Furniture,1.0,0.0,True,True,Fake,Report #3950
HouseHoldCollection,Furniture,1.0,0.0,True,True,Fake,
HouseHoldCollection,Furniture,1.0,0.0,True,True,,Report #3951
HouseHoldCollection,Furniture,1.0,0.0,True,True,,
HouseHoldCollection,Furniture,1.0,0.0,True,False,Genuine,Report #3952
HouseHoldCollection,Furniture,1.0,0.0,True,False,Genuine,
HouseHoldCollection,Furniture,1.0,0.0,True,False,Fake,Report #3953
HouseHoldCollection,Furniture,1.0,0.0,True,False,Fake,
HouseHoldCollection,Furniture,1.0,0.0,True,False,,Report #3954
HouseHoldCollection,Furniture,1.0,0.0,True,False,,
HouseHoldCollection,Furniture,1.0,0.0,False,True,Genuine,Report #3955
HouseHoldCollection,Furniture,1.0,0.0,False,True,Genuine,
HouseHoldCollection,Furniture,1.0,0.0,False,True,Fake,Report #3956
HouseHoldCollection,Furniture,1.0,0.0,False,True,Fake,
HouseHoldCollection,Furniture,1.0,0.0,False,True,,Report #3957
HouseHoldCollection,Furniture,1.0,0.0,False,True,,
HouseHoldCollection,Furniture,1.0,0.0,False,False,Genuine,Report #3958
HouseHoldCollection,Furniture,1.0,0.0,False,False,Genuine,
HouseHoldCollection,Furniture,1.0,0.0,False,False,Fake,Report #3959
HouseHoldCollection,Furniture,1.0,0.0,False,False,Fake,
HouseHoldCollection,Furniture,1.0,0.0,False,False,,Report #3960
HouseHoldCollection,Furniture,1.0,0.0,False,False,,
HouseHoldCollection,Furniture,1.0,19.0,True,True,Genuine,Report #3961
HouseHoldCollection,Furniture,1.0,19.0,True,True,Genuine,
HouseHoldCollection,Furniture,1.0,19.0,True,True,Fake,Report #3962
HouseHoldCollection,Furniture,1.0,19.0,True,True,Fake,
HouseHoldCollection,Furniture,1.0,19.0,True,True,,Report #3963
HouseHoldCollection,Furniture,1.0,19.0,True,True,,
HouseHoldCollection,Furniture,1.0,19.0,True,False,Genuine,Report #3964
HouseHoldCollection,Furniture,1.0,19.0,True,False,Genuine,
HouseHoldCollection,Furniture,1.0,19.0,True,False,Fake,Report #3965
HouseHoldCollection,Furniture,1.0,19.0,True,False,Fake,
HouseHoldCollection,Furniture,1.0,19.0,True,False,,Report #3966
HouseHoldCollection,Furniture,1.0,19.0,True,False,,
HouseHoldCollection,Furniture,1.0,19.0,False,True,Genuine,Report #3967
HouseHoldCollection,Furniture,1.0,19.0,False,True,Genuine,
HouseHoldCollection,Furniture,1.0,19.0,False,True,Fake,Report #3968
HouseHoldCollection,Furniture,1.0,19.0,False,True,Fake,
HouseHoldCollection,Furniture,1.0,19.0,False,True,,Report #3969
HouseHoldCollection,Furniture,1.0,19.0,False,True,,
HouseHoldCollection,Furniture,1.0,19.0,False,False,Genuine,Report #3970
HouseHoldCollection,Furniture,1.0,19.0,False,False,Genuine,
HouseHoldCollection,Furniture,1.0,19.0,False,False,Fake,Report #3971
HouseHoldCollection,Furniture,1.0,19.0,False,False,Fake,
HouseHoldCollection,Furniture,1.0,19.0,False,False,,Report #3972
HouseHoldCollection,Furniture,1.0,19.0,False,False,,
HouseHoldCollection,Furniture,1.0,20.0,True,True,Genuine,Report #3973
HouseHoldCollection,Furniture,1.0,20.0,True,True,Genuine,
HouseHoldCollection,Furniture,1.0,20.0,True,True,Fake,Report #3974
HouseHoldCollection,Furniture,1.0,20.0,True,True,Fake,
HouseHoldCollection,Furniture,1.0,20.0,True,True,,Report #3975
HouseHoldCollection,Furniture,1.0,20.0,True,True,,
HouseHoldCollection,Furniture,1.0,20.0,True,False,Genuine,Report #3976
HouseHoldCollection,Furniture,1.0,20.0,True,False,Genuine,
HouseHoldCollection,Furniture,1.0,20.0,True,False,Fake,Report #3977
HouseHoldCollection,Furniture,1.0,20.0,True,False,Fake,
HouseHoldCollection,Furniture,1.0,20.0,True,False,,Report #3978
HouseHoldCollection,Furniture,1.0,20.0,True,False,,
HouseHoldCollection,Furniture,1.0,20.0,False,True,Genuine,Report #3979
HouseHoldCollection,Furniture,1.0,20.0,False,True,Genuine,
HouseHoldCollection,Furniture,1.0,20.0,False,True,Fake,Report #3980
HouseHoldCollection,Furniture,1.0,20.0,False,True,Fake,
HouseHoldCollection,Furniture,1.0,20.0,False,True,,Report #3981
HouseHoldCollection,Furniture,1.0,20.0,False,True,,
HouseHoldCollection,Furniture,1.0,20.0,False,False,Genuine,Report #3982
HouseHoldCollection,Furniture,1.0,20.0,False,False,Genuine,
HouseHoldCollection,Furniture,1.0,20.0,False,False,Fake,Report #3983
HouseHoldCollection,Furniture,1.0,20.0,False,False,Fake,
HouseHoldCollection,Furniture,1.0,20.0,False,False,,Report #3984
HouseHoldCollection,Furniture,1.0,20.0,False,False,,
HouseHoldCollection,Furniture,1.0,21.0,True,True,Genuine,Report #3985
HouseHoldCollection,Furniture,1.0,21.0,True,True,Genuine,
HouseHoldCollection,Furniture,1.0,21.0,True,True,Fake,Report #3986
HouseHoldCollection,Furniture,1.0,21.0,True,True,Fake,
HouseHoldCollection,Furniture,1.0,21.0,True,True,,Report #3987
HouseHoldCollection,Furniture,1.0,21.0,True,True,,
HouseHoldCollection,Furniture,1.0,21.0,True,False,Genuine,Report #3988
HouseHoldCollection,Furniture,1.0,21.0,True,False,Genuine,
HouseHoldCollection,Furniture,1.0,21.0,True,False,Fake,Report #3989
HouseHoldCollection,Furniture,1.0,21.0,True,False,Fake,
HouseHoldCollection,Furniture,1.0,21.0,True,False,,Report #3990
HouseHoldCollection,Furniture,1.0,21.0,True,False,,
HouseHoldCollection,Furniture,1.0,21.0,False,True,Genuine,Report #3991
HouseHoldCollection,Furniture,1.0,21.0,False,True,Genuine,
HouseHoldCollection,Furniture,1.0,21.0,False,True,Fake,Report #3992
HouseHoldCollection,Furniture,1.0,21.0,False,True,Fake,
HouseHoldCollection,Furniture,1.0,21.0,False,True,,Report #3993
HouseHoldCollection,Furniture,1.0,21.0,False,True,,
HouseHoldCollection,Furniture,1.0,21.0,False,False,Genuine,Report #3994
HouseHoldCollection,Furniture,1.0,21.0,False,False,Genuine,
HouseHoldCollection,Furniture,1.0,21.0,False,False,Fake,Report #3995
HouseHoldCollection,Furniture,1.0,21.0,False,False,Fake,
HouseHoldCollection,Furniture,1.0,21.0,False,False,,Report #3996
HouseHoldCollection,Furniture,1.0,21.0,False,False,,
HouseHoldCollection,Furniture,1.0,99.0,True,True,Genuine,Report #3997
HouseHoldCollection,Furniture,1.0,99.0,True,True,Genuine,
HouseHoldCollection,Furniture,1.0,99.0,True,True,Fake,Report #3998
HouseHoldCollection,Furniture,1.0,99.0,True,True,Fake,
HouseHoldCollection,Furniture,1.0,99.0,True,True,,Report #3999
HouseHoldCollection,Furniture,1.0,99.0,True,True,,
HouseHoldCollection,Furniture,1.0,99.0,True,False,Genuine,Report #4000
HouseHoldCollection,Furniture,1.0,99.0,True,False,Genuine,
HouseHoldCollection,Furniture,1.0,99.0,True,False,Fake,Report #4001
HouseHoldCollection,Furniture,1.0,99.0,True,False,Fake,
HouseHoldCollection,Furniture,1.0,99.0,True,False,,Report #4002
HouseHoldCollection,Furniture,1.0,99.0,True,False,,
HouseHoldCollection,Furniture,1.0,99.0,False,True,Genuine,Report #4003
HouseHoldCollection,Furniture,1.0,99.0,False,True,Genuine,
HouseHoldCollection,Furniture,1.0,99.0,False,True,Fake,Report #4004
HouseHoldCollection,Furniture,1.0,99.0,False,True,Fake,
HouseHoldCollection,Furniture,1.0,99.0,False,True,,Report #4005
HouseHoldCollection,Furniture,1.0,99.0,False,True,,
HouseHoldCollection,Furniture,1.0,99.0,False,False,Genuine,Report #4006
HouseHoldCollection,Furniture,1.0,99.0,False,False,Genuine,
HouseHoldCollection,Furniture,1.0,99.0,False,False,Fake,Report #4007
HouseHoldCollection,Furniture,1.0,99.0,False,False,Fake,
HouseHoldCollection,Furniture,1.0,99.0,False,False,,Report #4008
HouseHoldCollection,Furniture,1.0,99.0,False,False,,
HouseHoldCollection,Furniture,1.0,100.0,True,True,Genuine,Report #4009
HouseHoldCollection,Furniture,1.0,100.0,True,True,Genuine,
HouseHoldCollection,Furniture,1.0,100.0,True,True,Fake,Report #4010
HouseHoldCollection,Furniture,1.0,100.0,True,True,Fake,
HouseHoldCollection,Furniture,1.0,100.0,True,True,,Report #4011
HouseHoldCollection,Furniture,1.0,100.0,True,True,,
HouseHoldCollection,Furniture,1.0,100.0,True,False,Genuine,Report #4012
HouseHoldCollection,Furniture,1.0,100.0,True,False,Genuine,
HouseHoldCollection,Furniture,1.0,100.0,True,False,Fake,Report #4013
HouseHoldCollection,Furniture,1.0,100.0,True,False,Fake,
HouseHoldCollection,Furniture,1.0,100.0,True,False,,Report #4014
HouseHoldCollection,Furniture,1.0,100.0,True,False,,
HouseHoldCollection,Furniture,1.0,100.0,False,True,Genuine,Report #4015
HouseHoldCollection,Furniture,1.0,100.0,False,True,Genuine,
HouseHoldCollection,Furniture,1.0,100.0,False,True,Fake,Report #4016
HouseHoldCollection,Furniture,1.0,100.0,False,True,Fake,
HouseHoldCollection,Furniture,1.0,100.0,False,True,,Report #4017
HouseHoldCollection,Furniture,1.0,100.0,False,True,,
HouseHoldCollection,Furniture,1.0,100.0,False,False,Genuine,Report #4018
HouseHoldCollection,Furniture,1.0,100.0,False,False,Genuine,
HouseHoldCollection,Furniture,1.0,100.0,False,False,Fake,Report #4019
HouseHoldCollection,Furniture,1.0,100.0,False,False,Fake,
HouseHoldCollection,Furniture,1.0,100.0,False,False,,Report #4020
HouseHoldCollection,Furniture,1.0,100.0,False,False,,
HouseHoldCollection,Furniture,1.0,101.0,True,True,Genuine,Report #4021
HouseHoldCollection,Furniture,1.0,101.0,True,True,Genuine,
HouseHoldCollection,Furniture,1.0,101.0,True,True,Fake,Report #4022
HouseHoldCollection,Furniture,1.0,101.0,True,True,Fake,
HouseHoldCollection,Furniture,1.0,101.0,True,True,,Report #4023
HouseHoldCollection,Furniture,1.0,101.0,True,True,,
HouseHoldCollection,Furniture,1.0,101.0,True,False,Genuine,Report #4024
HouseHoldCollection,Furniture,1.0,101.0,True,False,Genuine,
HouseHoldCollection,Furniture,1.0,101.0,True,False,Fake,Report #4025
HouseHoldCollection,Furniture,1.0,101.0,True,False,Fake,
HouseHoldCollection,Furniture,1.0,101.0,True,False,,Report #4026
HouseHoldCollection,Furniture,1.0,101.0,True,False,,
HouseHoldCollection,Furniture,1.0,101.0,False,True,Genuine,Report #4027
HouseHoldCollection,Furniture,1.0,101.0,False,True,Genuine,
HouseHoldCollection,Furniture,1.0,101.0,False,True,Fake,Report #4028
HouseHoldCollection,Furniture,1.0,101.0,False,True,Fake,
HouseHoldCollection,Furniture,1.0,101.0,False,True,,Report #4029
HouseHoldCollection,Furniture,1.0,101.0,False,True,,
HouseHoldCollection,Furniture,1.0,101.0,False,False,Genuine,Report #4030
HouseHoldCollection,Furniture,1.0,101.0,False,False,Genuine,
HouseHoldCollection,Furniture,1.0,101.0,False,False,Fake,Report #4031
HouseHoldCollection,Furniture,1.0,101.0,False,False,Fake,
HouseHoldCollection,Furniture,1.0,101.0,False,False,,Report #4032
HouseHoldCollection,Furniture,1.0,101.0,False,False,,
HouseHoldCollection,Furniture,100.0,-1.0,True,True,Genuine,Report #4033
HouseHoldCollection,Furniture,100.0,-1.0,True,True,Genuine,
HouseHoldCollection,Furniture,100.0,-1.0,True,True,Fake,Report #4034
HouseHoldCollection,Furniture,100.0,-1.0,True,True,Fake,
HouseHoldCollection,Furniture,100.0,-1.0,True,True,,Report #4035
HouseHoldCollection,Furniture,100.0,-1.0,True,True,,
HouseHoldCollection,Furniture,100.0,-1.0,True,False,Genuine,Report #4036
HouseHoldCollection,Furniture,100.0,-1.0,True,False,Genuine,
HouseHoldCollection,Furniture,100.0,-1.0,True,False,Fake,Report #4037
HouseHoldCollection,Furniture,100.0,-1.0,True,False,Fake,
HouseHoldCollection,Furniture,100.0,-1.0,True,False,,Report #4038
HouseHoldCollection,Furniture,100.0,-1.0,True,False,,
HouseHoldCollection,Furniture,100.0,-1.0,False,True,Genuine,Report #4039
HouseHoldCollection,Furniture,100.0,-1.0,False,True,Genuine,
HouseHoldCollection,Furniture,100.0,-1.0,False,True,Fake,Report #4040
HouseHoldCollection,Furniture,100.0,-1.0,False,True,Fake,
HouseHoldCollection,Furniture,100.0,-1.0,False,True,,Report #4041
HouseHoldCollection,Furniture,100.0,-1.0,False,True,,
HouseHoldCollection,Furniture,100.0,-1.0,False,False,Genuine,Report #4042
HouseHoldCollection,Furniture,100.0,-1.0,False,False,Genuine,
HouseHoldCollection,Furniture,100.0,-1.0,False,False,Fake,Report #4043
HouseHoldCollection,Furniture,100.0,-1.0,False,False,Fake,
HouseHoldCollection,Furniture,100.0,-1.0,False,False,,Report #4044
HouseHoldCollection,Furniture,100.0,-1.0,False,False,,
HouseHoldCollection,Furniture,100.0,0.0,True,True,Genuine,Report #4045
HouseHoldCollection,Furniture,100.0,0.0,True,True,Genuine,
HouseHoldCollection,Furniture,100.0,0.0,True,True,Fake,Report #4046
HouseHoldCollection,Furniture,100.0,0.0,True,True,Fake,
HouseHoldCollection,Furniture,100.0,0.0,True,True,,Report #4047
HouseHoldCollection,Furniture,100.0,0.0,True,True,,
HouseHoldCollection,Furniture,100.0,0.0,True,False,Genuine,Report #4048
HouseHoldCollection,Furniture,100.0,0.0,True,False,Genuine,
HouseHoldCollection,Furniture,100.0,0.0,True,False,Fake,Report #4049
HouseHoldCollection,Furniture,100.0,0.0,True,False,Fake,
HouseHoldCollection,Furniture,100.0,0.0,True,False,,Report #4050
HouseHoldCollection,Furniture,100.0,0.0,True,False,,
HouseHoldCollection,Furniture,100.0,0.0,False,True,Genuine,Report #4051
HouseHoldCollection,Furniture,100.0,0.0,False,True,Genuine,
HouseHoldCollection,Furniture,100.0,0.0,False,True,Fake,Report #4052
HouseHoldCollection,Furniture,100.0,0.0,False,True,Fake,
HouseHoldCollection,Furniture,100.0,0.0,False,True,,Report #4053
HouseHoldCollection,Furniture,100.0,0.0,False,True,,
HouseHoldCollection,Furniture,100.0,0.0,False,False,Genuine,Report #4054
HouseHoldCollection,Furniture,100.0,0.0,False,False,Genuine,
HouseHoldCollection,Furniture,100.0,0.0,False,False,Fake,Report #4055
HouseHoldCollection,Furniture,100.0,0.0,False,False,Fake,
HouseHoldCollection,Furniture,100.0,0.0,False,False,,Report #4056
HouseHoldCollection,Furniture,100.0,0.0,False,False,,
HouseHoldCollection,Furniture,100.0,19.0,True,True,Genuine,Report #4057
HouseHoldCollection,Furniture,100.0,19.0,True,True,Genuine,
HouseHoldCollection,Furniture,100.0,19.0,True,True,Fake,Report #4058
HouseHoldCollection,Furniture,100.0,19.0,True,True,Fake,
HouseHoldCollection,Furniture,100.0,19.0,True,True,,Report #4059
HouseHoldCollection,Furniture,100.0,19.0,True,True,,
HouseHoldCollection,Furniture,100.0,19.0,True,False,Genuine,Report #4060
HouseHoldCollection,Furniture,100.0,19.0,True,False,Genuine,
HouseHoldCollection,Furniture,100.0,19.0,True,False,Fake,Report #4061
HouseHoldCollection,Furniture,100.0,19.0,True,False,Fake,
HouseHoldCollection,Furniture,100.0,19.0,True,False,,Report #4062
HouseHoldCollection,Furniture,100.0,19.0,True,False,,
HouseHoldCollection,Furniture,100.0,19.0,False,True,Genuine,Report #4063
HouseHoldCollection,Furniture,100.0,19.0,False,True,Genuine,
HouseHoldCollection,Furniture,100.0,19.0,False,True,Fake,Report #4064
HouseHoldCollection,Furniture,100.0,19.0,False,True,Fake,
HouseHoldCollection,Furniture,100.0,19.0,False,True,,Report #4065
HouseHoldCollection,Furniture,100.0,19.0,False,True,,
HouseHoldCollection,Furniture,100.0,19.0,False,False,Genuine,Report #4066
HouseHoldCollection,Furniture,100.0,19.0,False,False,Genuine,
HouseHoldCollection,Furniture,100.0,19.0,False,False,Fake,Report #4067
HouseHoldCollection,Furniture,100.0,19.0,False,False,Fake,
HouseHoldCollection,Furniture,100.0,19.0,False,False,,Report #4068
HouseHoldCollection,Furniture,100.0,19.0,False,False,,
HouseHoldCollection,Furniture,100.0,20.0,True,True,Genuine,Report #4069
HouseHoldCollection,Furniture,100.0,20.0,True,True,Genuine,
HouseHoldCollection,Furniture,100.0,20.0,True,True,Fake,Report #4070
HouseHoldCollection,Furniture,100.0,20.0,True,True,Fake,
HouseHoldCollection,Furniture,100.0,20.0,True,True,,Report #4071
HouseHoldCollection,Furniture,100.0,20.0,True,True,,
HouseHoldCollection,Furniture,100.0,20.0,True,False,Genuine,Report #4072
HouseHoldCollection,Furniture,100.0,20.0,True,False,Genuine,
HouseHoldCollection,Furniture,100.0,20.0,True,False,Fake,Report #4073
HouseHoldCollection,Furniture,100.0,20.0,True,False,Fake,
HouseHoldCollection,Furniture,100.0,20.0,True,False,,Report #4074
HouseHoldCollection,Furniture,100.0,20.0,True,False,,
HouseHoldCollection,Furniture,100.0,20.0,False,True,Genuine,Report #4075
HouseHoldCollection,Furniture,100.0,20.0,False,True,Genuine,
HouseHoldCollection,Furniture,100.0,20.0,False,True,Fake,Report #4076
HouseHoldCollection,Furniture,100.0,20.0,False,True,Fake,
HouseHoldCollection,Furniture,100.0,20.0,False,True,,Report #4077
HouseHoldCollection,Furniture,100.0,20.0,False,True,,
HouseHoldCollection,Furniture,100.0,20.0,False,False,Genuine,Report #4078
HouseHoldCollection,Furniture,100.0,20.0,False,False,Genuine,
HouseHoldCollection,Furniture,100.0,20.0,False,False,Fake,Report #4079
HouseHoldCollection,Furniture,100.0,20.0,False,False,Fake,
HouseHoldCollection,Furniture,100.0,20.0,False,False,,Report #4080
HouseHoldCollection,Furniture,100.0,20.0,False,False,,
HouseHoldCollection,Furniture,100.0,21.0,True,True,Genuine,Report #4081
HouseHoldCollection,Furniture,100.0,21.0,True,True,Genuine,
HouseHoldCollection,Furniture,100.0,21.0,True,True,Fake,Report #4082
HouseHoldCollection,Furniture,100.0,21.0,True,True,Fake,
HouseHoldCollection,Furniture,100.0,21.0,True,True,,Report #4083
HouseHoldCollection,Furniture,100.0,21.0,True,True,,
HouseHoldCollection,Furniture,100.0,21.0,True,False,Genuine,Report #4084
HouseHoldCollection,Furniture,100.0,21.0,True,False,Genuine,
HouseHoldCollection,Furniture,100.0,21.0,True,False,Fake,Report #4085
HouseHoldCollection,Furniture,100.0,21.0,True,False,Fake,
HouseHoldCollection,Furniture,100.0,21.0,True,False,,Report #4086
HouseHoldCollection,Furniture,100.0,21.0,True,False,,
HouseHoldCollection,Furniture,100.0,21.0,False,True,Genuine,Report #4087
HouseHoldCollection,Furniture,100.0,21.0,False,True,Genuine,
HouseHoldCollection,Furniture,100.0,21.0,False,True,Fake,Report #4088
HouseHoldCollection,Furniture,100.0,21.0,False,True,Fake,
HouseHoldCollection,Furniture,100.0,21.0,False,True,,Report #4089
HouseHoldCollection,Furniture,100.0,21.0,False,True,,
HouseHoldCollection,Furniture,100.0,21.0,False,False,Genuine,Report #4090
HouseHoldCollection,Furniture,100.0,21.0,False,False,Genuine,
HouseHoldCollection,Furniture,100.0,21.0,False,False,Fake,Report #4091
HouseHoldCollection,Furniture,100.0,21.0,False,False,Fake,
HouseHoldCollection,Furniture,100.0,21.0,False,False,,Report #4092
HouseHoldCollection,Furniture,100.0,21.0,False,False,,
HouseHoldCollection,Furniture,100.0,99.0,True,True,Genuine,Report #4093
HouseHoldCollection,Furniture,100.0,99.0,True,True,Genuine,
HouseHoldCollection,Furniture,100.0,99.0,True,True,Fake,Report #4094
HouseHoldCollection,Furniture,100.0,99.0,True,True,Fake,
HouseHoldCollection,Furniture,100.0,99.0,True,True,,Report #4095
HouseHoldCollection,Furniture,100.0,99.0,True,True,,
HouseHoldCollection,Furniture,100.0,99.0,True,False,Genuine,Report #4096
HouseHoldCollection,Furniture,100.0,99.0,True,False,Genuine,
HouseHoldCollection,Furniture,100.0,99.0,True,False,Fake,Report #4097
HouseHoldCollection,Furniture,100.0,99.0,True,False,Fake,
HouseHoldCollection,Furniture,100.0,99.0,True,False,,Report #4098
HouseHoldCollection,Furniture,100.0,99.0,True,False,,
HouseHoldCollection,Furniture,100.0,99.0,False,True,Genuine,Report #4099
HouseHoldCollection,Furniture,100.0,99.0,False,True,Genuine,
HouseHoldCollection,Furniture,100.0,99.0,False,True,Fake,Report #4100
HouseHoldCollection,Furniture,100.0,99.0,False,True,Fake,
HouseHoldCollection,Furniture,100.0,99.0,False,True,,Report #4101
HouseHoldCollection,Furniture,100.0,99.0,False,True,,
HouseHoldCollection,Furniture,100.0,99.0,False,False,Genuine,Report #4102
HouseHoldCollection,Furniture,100.0,99.0,False,False,Genuine,
HouseHoldCollection,Furniture,100.0,99.0,False,False,Fake,Report #4103
HouseHoldCollection,Furniture,100.0,99.0,False,False,Fake,
HouseHoldCollection,Furniture,100.0,99.0,False,False,,Report #4104
HouseHoldCollection,Furniture,100.0,99.0,False,False,,
HouseHoldCollection,Furniture,100.0,100.0,True,True,Genuine,Report #4105
HouseHoldCollection,Furniture,100.0,100.0,True,True,Genuine,
HouseHoldCollection,Furniture,100.0,100.0,True,True,Fake,Report #4106
HouseHoldCollection,Furniture,100.0,100.0,True,True,Fake,
HouseHoldCollection,Furniture,100.0,100.0,True,True,,Report #4107
HouseHoldCollection,Furniture,100.0,100.0,True,True,,
HouseHoldCollection,Furniture,100.0,100.0,True,False,Genuine,Report #4108
HouseHoldCollection,Furniture,100.0,100.0,True,False,Genuine,
HouseHoldCollection,Furniture,100.0,100.0,True,False,Fake,Report #4109
HouseHoldCollection,Furniture,100.0,100.0,True,False,Fake,
HouseHoldCollection,Furniture,100.0,100.0,True,False,,Report #4110
HouseHoldCollection,Furniture,100.0,100.0,True,False,,
HouseHoldCollection,Furniture,100.0,100.0,False,True,Genuine,Report #4111
HouseHoldCollection,Furniture,100.0,100.0,False,True,Genuine,
HouseHoldCollection,Furniture,100.0,100.0,False,True,Fake,Report #4112
HouseHoldCollection,Furniture,100.0,100.0,False,True,Fake,
HouseHoldCollection,Furniture,100.0,100.0,False,True,,Report #4113
HouseHoldCollection,Furniture,100.0,100.0,False,True,,
HouseHoldCollection,Furniture,100.0,100.0,False,False,Genuine,Report #4114
HouseHoldCollection,Furniture,100.0,100.0,False,False,Genuine,
HouseHoldCollection,Furniture,100.0,100.0,False,False,Fake,Report #4115
HouseHoldCollection,Furniture,100.0,100.0,False,False,Fake,
HouseHoldCollection,Furniture,100.0,100.0,False,False,,Report #4116
HouseHoldCollection,Furniture,100.0,100.0,False,False,,
HouseHoldCollection,Furniture,100.0,101.0,True,True,Genuine,Report #4117
HouseHoldCollection,Furniture,100.0,101.0,True,True,Genuine,
HouseHoldCollection,Furniture,100.0,101.0,True,True,Fake,Report #4118
HouseHoldCollection,Furniture,100.0,101.0,True,True,Fake,
HouseHoldCollection,Furniture,100.0,101.0,True,True,,Report #4119
HouseHoldCollection,Furniture,100.0,101.0,True,True,,
HouseHoldCollection,Furniture,100.0,101.0,True,False,Genuine,Report #4120
HouseHoldCollection,Furniture,100.0,101.0,True,False,Genuine,
HouseHoldCollection,Furniture,100.0,101.0,True,False,Fake,Report #4121
HouseHoldCollection,Furniture,100.0,101.0,True,False,Fake,
HouseHoldCollection,Furniture,100.0,101.0,True,False,,Report #4122
HouseHoldCollection,Furniture,100.0,101.0,True,False,,
HouseHoldCollection,Furniture,100.0,101.0,False,True,Genuine,Report #4123
HouseHoldCollection,Furniture,100.0,101.0,False,True,Genuine,
HouseHoldCollection,Furniture,100.0,101.0,False,True,Fake,Report #4124
HouseHoldCollection,Furniture,100.0,101.0,False,True,Fake,
HouseHoldCollection,Furniture,100.0,101.0,False,True,,Report #4125
HouseHoldCollection,Furniture,100.0,101.0,False,True,,
HouseHoldCollection,Furniture,100.0,101.0,False,False,Genuine,Report #4126
HouseHoldCollection,Furniture,100.0,101.0,False,False,Genuine,
HouseHoldCollection,Furniture,100.0,101.0,False,False,Fake,Report #4127
HouseHoldCollection,Furniture,100.0,101.0,False,False,Fake,
HouseHoldCollection,Furniture,100.0,101.0,False,False,,Report #4128
HouseHoldCollection,Furniture,100.0,101.0,False,False,,
HouseHoldCollection,Furniture,1000.0,-1.0,True,True,Genuine,Report #4129
HouseHoldCollection,Furniture,1000.0,-1.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000.0,-1.0,True,True,Fake,Report #4130
HouseHoldCollection,Furniture,1000.0,-1.0,True,True,Fake,
HouseHoldCollection,Furniture,1000.0,-1.0,True,True,,Report #4131
HouseHoldCollection,Furniture,1000.0,-1.0,True,True,,
HouseHoldCollection,Furniture,1000.0,-1.0,True,False,Genuine,Report #4132
HouseHoldCollection,Furniture,1000.0,-1.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000.0,-1.0,True,False,Fake,Report #4133
HouseHoldCollection,Furniture,1000.0,-1.0,True,False,Fake,
HouseHoldCollection,Furniture,1000.0,-1.0,True,False,,Report #4134
HouseHoldCollection,Furniture,1000.0,-1.0,True,False,,
HouseHoldCollection,Furniture,1000.0,-1.0,False,True,Genuine,Report #4135
HouseHoldCollection,Furniture,1000.0,-1.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000.0,-1.0,False,True,Fake,Report #4136
HouseHoldCollection,Furniture,1000.0,-1.0,False,True,Fake,
HouseHoldCollection,Furniture,1000.0,-1.0,False,True,,Report #4137
HouseHoldCollection,Furniture,1000.0,-1.0,False,True,,
HouseHoldCollection,Furniture,1000.0,-1.0,False,False,Genuine,Report #4138
HouseHoldCollection,Furniture,1000.0,-1.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000.0,-1.0,False,False,Fake,Report #4139
HouseHoldCollection,Furniture,1000.0,-1.0,False,False,Fake,
HouseHoldCollection,Furniture,1000.0,-1.0,False,False,,Report #4140
HouseHoldCollection,Furniture,1000.0,-1.0,False,False,,
HouseHoldCollection,Furniture,1000.0,0.0,True,True,Genuine,Report #4141
HouseHoldCollection,Furniture,1000.0,0.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000.0,0.0,True,True,Fake,Report #4142
HouseHoldCollection,Furniture,1000.0,0.0,True,True,Fake,
HouseHoldCollection,Furniture,1000.0,0.0,True,True,,Report #4143
HouseHoldCollection,Furniture,1000.0,0.0,True,True,,
HouseHoldCollection,Furniture,1000.0,0.0,True,False,Genuine,Report #4144
HouseHoldCollection,Furniture,1000.0,0.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000.0,0.0,True,False,Fake,Report #4145
HouseHoldCollection,Furniture,1000.0,0.0,True,False,Fake,
HouseHoldCollection,Furniture,1000.0,0.0,True,False,,Report #4146
HouseHoldCollection,Furniture,1000.0,0.0,True,False,,
HouseHoldCollection,Furniture,1000.0,0.0,False,True,Genuine,Report #4147
HouseHoldCollection,Furniture,1000.0,0.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000.0,0.0,False,True,Fake,Report #4148
HouseHoldCollection,Furniture,1000.0,0.0,False,True,Fake,
HouseHoldCollection,Furniture,1000.0,0.0,False,True,,Report #4149
HouseHoldCollection,Furniture,1000.0,0.0,False,True,,
HouseHoldCollection,Furniture,1000.0,0.0,False,False,Genuine,Report #4150
HouseHoldCollection,Furniture,1000.0,0.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000.0,0.0,False,False,Fake,Report #4151
HouseHoldCollection,Furniture,1000.0,0.0,False,False,Fake,
HouseHoldCollection,Furniture,1000.0,0.0,False,False,,Report #4152
HouseHoldCollection,Furniture,1000.0,0.0,False,False,,
HouseHoldCollection,Furniture,1000.0,19.0,True,True,Genuine,Report #4153
HouseHoldCollection,Furniture,1000.0,19.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000.0,19.0,True,True,Fake,Report #4154
HouseHoldCollection,Furniture,1000.0,19.0,True,True,Fake,
HouseHoldCollection,Furniture,1000.0,19.0,True,True,,Report #4155
HouseHoldCollection,Furniture,1000.0,19.0,True,True,,
HouseHoldCollection,Furniture,1000.0,19.0,True,False,Genuine,Report #4156
HouseHoldCollection,Furniture,1000.0,19.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000.0,19.0,True,False,Fake,Report #4157
HouseHoldCollection,Furniture,1000.0,19.0,True,False,Fake,
HouseHoldCollection,Furniture,1000.0,19.0,True,False,,Report #4158
HouseHoldCollection,Furniture,1000.0,19.0,True,False,,
HouseHoldCollection,Furniture,1000.0,19.0,False,True,Genuine,Report #4159
HouseHoldCollection,Furniture,1000.0,19.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000.0,19.0,False,True,Fake,Report #4160
HouseHoldCollection,Furniture,1000.0,19.0,False,True,Fake,
HouseHoldCollection,Furniture,1000.0,19.0,False,True,,Report #4161
HouseHoldCollection,Furniture,1000.0,19.0,False,True,,
HouseHoldCollection,Furniture,1000.0,19.0,False,False,Genuine,Report #4162
HouseHoldCollection,Furniture,1000.0,19.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000.0,19.0,False,False,Fake,Report #4163
HouseHoldCollection,Furniture,1000.0,19.0,False,False,Fake,
HouseHoldCollection,Furniture,1000.0,19.0,False,False,,Report #4164
HouseHoldCollection,Furniture,1000.0,19.0,False,False,,
HouseHoldCollection,Furniture,1000.0,20.0,True,True,Genuine,Report #4165
HouseHoldCollection,Furniture,1000.0,20.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000.0,20.0,True,True,Fake,Report #4166
HouseHoldCollection,Furniture,1000.0,20.0,True,True,Fake,
HouseHoldCollection,Furniture,1000.0,20.0,True,True,,Report #4167
HouseHoldCollection,Furniture,1000.0,20.0,True,True,,
HouseHoldCollection,Furniture,1000.0,20.0,True,False,Genuine,Report #4168
HouseHoldCollection,Furniture,1000.0,20.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000.0,20.0,True,False,Fake,Report #4169
HouseHoldCollection,Furniture,1000.0,20.0,True,False,Fake,
HouseHoldCollection,Furniture,1000.0,20.0,True,False,,Report #4170
HouseHoldCollection,Furniture,1000.0,20.0,True,False,,
HouseHoldCollection,Furniture,1000.0,20.0,False,True,Genuine,Report #4171
HouseHoldCollection,Furniture,1000.0,20.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000.0,20.0,False,True,Fake,Report #4172
HouseHoldCollection,Furniture,1000.0,20.0,False,True,Fake,
HouseHoldCollection,Furniture,1000.0,20.0,False,True,,Report #4173
HouseHoldCollection,Furniture,1000.0,20.0,False,True,,
HouseHoldCollection,Furniture,1000.0,20.0,False,False,Genuine,Report #4174
HouseHoldCollection,Furniture,1000.0,20.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000.0,20.0,False,False,Fake,Report #4175
HouseHoldCollection,Furniture,1000.0,20.0,False,False,Fake,
HouseHoldCollection,Furniture,1000.0,20.0,False,False,,Report #4176
HouseHoldCollection,Furniture,1000.0,20.0,False,False,,
HouseHoldCollection,Furniture,1000.0,21.0,True,True,Genuine,Report #4177
HouseHoldCollection,Furniture,1000.0,21.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000.0,21.0,True,True,Fake,Report #4178
HouseHoldCollection,Furniture,1000.0,21.0,True,True,Fake,
HouseHoldCollection,Furniture,1000.0,21.0,True,True,,Report #4179
HouseHoldCollection,Furniture,1000.0,21.0,True,True,,
HouseHoldCollection,Furniture,1000.0,21.0,True,False,Genuine,Report #4180
HouseHoldCollection,Furniture,1000.0,21.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000.0,21.0,True,False,Fake,Report #4181
HouseHoldCollection,Furniture,1000.0,21.0,True,False,Fake,
HouseHoldCollection,Furniture,1000.0,21.0,True,False,,Report #4182
HouseHoldCollection,Furniture,1000.0,21.0,True,False,,
HouseHoldCollection,Furniture,1000.0,21.0,False,True,Genuine,Report #4183
HouseHoldCollection,Furniture,1000.0,21.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000.0,21.0,False,True,Fake,Report #4184
HouseHoldCollection,Furniture,1000.0,21.0,False,True,Fake,
HouseHoldCollection,Furniture,1000.0,21.0,False,True,,Report #4185
HouseHoldCollection,Furniture,1000.0,21.0,False,True,,
HouseHoldCollection,Furniture,1000.0,21.0,False,False,Genuine,Report #4186
HouseHoldCollection,Furniture,1000.0,21.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000.0,21.0,False,False,Fake,Report #4187
HouseHoldCollection,Furniture,1000.0,21.0,False,False,Fake,
HouseHoldCollection,Furniture,1000.0,21.0,False,False,,Report #4188
HouseHoldCollection,Furniture,1000.0,21.0,False,False,,
HouseHoldCollection,Furniture,1000.0,99.0,True,True,Genuine,Report #4189
HouseHoldCollection,Furniture,1000.0,99.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000.0,99.0,True,True,Fake,Report #4190
HouseHoldCollection,Furniture,1000.0,99.0,True,True,Fake,
HouseHoldCollection,Furniture,1000.0,99.0,True,True,,Report #4191
HouseHoldCollection,Furniture,1000.0,99.0,True,True,,
HouseHoldCollection,Furniture,1000.0,99.0,True,False,Genuine,Report #4192
HouseHoldCollection,Furniture,1000.0,99.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000.0,99.0,True,False,Fake,Report #4193
HouseHoldCollection,Furniture,1000.0,99.0,True,False,Fake,
HouseHoldCollection,Furniture,1000.0,99.0,True,False,,Report #4194
HouseHoldCollection,Furniture,1000.0,99.0,True,False,,
HouseHoldCollection,Furniture,1000.0,99.0,False,True,Genuine,Report #4195
HouseHoldCollection,Furniture,1000.0,99.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000.0,99.0,False,True,Fake,Report #4196
HouseHoldCollection,Furniture,1000.0,99.0,False,True,Fake,
HouseHoldCollection,Furniture,1000.0,99.0,False,True,,Report #4197
HouseHoldCollection,Furniture,1000.0,99.0,False,True,,
HouseHoldCollection,Furniture,1000.0,99.0,False,False,Genuine,Report #4198
HouseHoldCollection,Furniture,1000.0,99.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000.0,99.0,False,False,Fake,Report #4199
HouseHoldCollection,Furniture,1000.0,99.0,False,False,Fake,
HouseHoldCollection,Furniture,1000.0,99.0,False,False,,Report #4200
HouseHoldCollection,Furniture,1000.0,99.0,False,False,,
HouseHoldCollection,Furniture,1000.0,100.0,True,True,Genuine,Report #4201
HouseHoldCollection,Furniture,1000.0,100.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000.0,100.0,True,True,Fake,Report #4202
HouseHoldCollection,Furniture,1000.0,100.0,True,True,Fake,
HouseHoldCollection,Furniture,1000.0,100.0,True,True,,Report #4203
HouseHoldCollection,Furniture,1000.0,100.0,True,True,,
HouseHoldCollection,Furniture,1000.0,100.0,True,False,Genuine,Report #4204
HouseHoldCollection,Furniture,1000.0,100.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000.0,100.0,True,False,Fake,Report #4205
HouseHoldCollection,Furniture,1000.0,100.0,True,False,Fake,
HouseHoldCollection,Furniture,1000.0,100.0,True,False,,Report #4206
HouseHoldCollection,Furniture,1000.0,100.0,True,False,,
HouseHoldCollection,Furniture,1000.0,100.0,False,True,Genuine,Report #4207
HouseHoldCollection,Furniture,1000.0,100.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000.0,100.0,False,True,Fake,Report #4208
HouseHoldCollection,Furniture,1000.0,100.0,False,True,Fake,
HouseHoldCollection,Furniture,1000.0,100.0,False,True,,Report #4209
HouseHoldCollection,Furniture,1000.0,100.0,False,True,,
HouseHoldCollection,Furniture,1000.0,100.0,False,False,Genuine,Report #4210
HouseHoldCollection,Furniture,1000.0,100.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000.0,100.0,False,False,Fake,Report #4211
HouseHoldCollection,Furniture,1000.0,100.0,False,False,Fake,
HouseHoldCollection,Furniture,1000.0,100.0,False,False,,Report #4212
HouseHoldCollection,Furniture,1000.0,100.0,False,False,,
HouseHoldCollection,Furniture,1000.0,101.0,True,True,Genuine,Report #4213
HouseHoldCollection,Furniture,1000.0,101.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000.0,101.0,True,True,Fake,Report #4214
HouseHoldCollection,Furniture,1000.0,101.0,True,True,Fake,
HouseHoldCollection,Furniture,1000.0,101.0,True,True,,Report #4215
HouseHoldCollection,Furniture,1000.0,101.0,True,True,,
HouseHoldCollection,Furniture,1000.0,101.0,True,False,Genuine,Report #4216
HouseHoldCollection,Furniture,1000.0,101.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000.0,101.0,True,False,Fake,Report #4217
HouseHoldCollection,Furniture,1000.0,101.0,True,False,Fake,
HouseHoldCollection,Furniture,1000.0,101.0,True,False,,Report #4218
HouseHoldCollection,Furniture,1000.0,101.0,True,False,,
HouseHoldCollection,Furniture,1000.0,101.0,False,True,Genuine,Report #4219
HouseHoldCollection,Furniture,1000.0,101.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000.0,101.0,False,True,Fake,Report #4220
HouseHoldCollection,Furniture,1000.0,101.0,False,True,Fake,
HouseHoldCollection,Furniture,1000.0,101.0,False,True,,Report #4221
HouseHoldCollection,Furniture,1000.0,101.0,False,True,,
HouseHoldCollection,Furniture,1000.0,101.0,False,False,Genuine,Report #4222
HouseHoldCollection,Furniture,1000.0,101.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000.0,101.0,False,False,Fake,Report #4223
HouseHoldCollection,Furniture,1000.0,101.0,False,False,Fake,
HouseHoldCollection,Furniture,1000.0,101.0,False,False,,Report #4224
HouseHoldCollection,Furniture,1000.0,101.0,False,False,,
HouseHoldCollection,Furniture,1000000.0,-1.0,True,True,Genuine,Report #4225
HouseHoldCollection,Furniture,1000000.0,-1.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,-1.0,True,True,Fake,Report #4226
HouseHoldCollection,Furniture,1000000.0,-1.0,True,True,Fake,
HouseHoldCollection,Furniture,1000000.0,-1.0,True,True,,Report #4227
HouseHoldCollection,Furniture,1000000.0,-1.0,True,True,,
HouseHoldCollection,Furniture,1000000.0,-1.0,True,False,Genuine,Report #4228
HouseHoldCollection,Furniture,1000000.0,-1.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,-1.0,True,False,Fake,Report #4229
HouseHoldCollection,Furniture,1000000.0,-1.0,True,False,Fake,
HouseHoldCollection,Furniture,1000000.0,-1.0,True,False,,Report #4230
HouseHoldCollection,Furniture,1000000.0,-1.0,True,False,,
HouseHoldCollection,Furniture,1000000.0,-1.0,False,True,Genuine,Report #4231
HouseHoldCollection,Furniture,1000000.0,-1.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,-1.0,False,True,Fake,Report #4232
HouseHoldCollection,Furniture,1000000.0,-1.0,False,True,Fake,
HouseHoldCollection,Furniture,1000000.0,-1.0,False,True,,Report #4233
HouseHoldCollection,Furniture,1000000.0,-1.0,False,True,,
HouseHoldCollection,Furniture,1000000.0,-1.0,False,False,Genuine,Report #4234
HouseHoldCollection,Furniture,1000000.0,-1.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,-1.0,False,False,Fake,Report #4235
HouseHoldCollection,Furniture,1000000.0,-1.0,False,False,Fake,
HouseHoldCollection,Furniture,1000000.0,-1.0,False,False,,Report #4236
HouseHoldCollection,Furniture,1000000.0,-1.0,False,False,,
HouseHoldCollection,Furniture,1000000.0,0.0,True,True,Genuine,Report #4237
HouseHoldCollection,Furniture,1000000.0,0.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,0.0,True,True,Fake,Report #4238
HouseHoldCollection,Furniture,1000000.0,0.0,True,True,Fake,
HouseHoldCollection,Furniture,1000000.0,0.0,True,True,,Report #4239
HouseHoldCollection,Furniture,1000000.0,0.0,True,True,,
HouseHoldCollection,Furniture,1000000.0,0.0,True,False,Genuine,Report #4240
HouseHoldCollection,Furniture,1000000.0,0.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,0.0,True,False,Fake,Report #4241
HouseHoldCollection,Furniture,1000000.0,0.0,True,False,Fake,
HouseHoldCollection,Furniture,1000000.0,0.0,True,False,,Report #4242
HouseHoldCollection,Furniture,1000000.0,0.0,True,False,,
HouseHoldCollection,Furniture,1000000.0,0.0,False,True,Genuine,Report #4243
HouseHoldCollection,Furniture,1000000.0,0.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,0.0,False,True,Fake,Report #4244
HouseHoldCollection,Furniture,1000000.0,0.0,False,True,Fake,
HouseHoldCollection,Furniture,1000000.0,0.0,False,True,,Report #4245
HouseHoldCollection,Furniture,1000000.0,0.0,False,True,,
HouseHoldCollection,Furniture,1000000.0,0.0,False,False,Genuine,Report #4246
HouseHoldCollection,Furniture,1000000.0,0.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,0.0,False,False,Fake,Report #4247
HouseHoldCollection,Furniture,1000000.0,0.0,False,False,Fake,
HouseHoldCollection,Furniture,1000000.0,0.0,False,False,,Report #4248
HouseHoldCollection,Furniture,1000000.0,0.0,False,False,,
HouseHoldCollection,Furniture,1000000.0,19.0,True,True,Genuine,Report #4249
HouseHoldCollection,Furniture,1000000.0,19.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,19.0,True,True,Fake,Report #4250
HouseHoldCollection,Furniture,1000000.0,19.0,True,True,Fake,
HouseHoldCollection,Furniture,1000000.0,19.0,True,True,,Report #4251
HouseHoldCollection,Furniture,1000000.0,19.0,True,True,,
HouseHoldCollection,Furniture,1000000.0,19.0,True,False,Genuine,Report #4252
HouseHoldCollection,Furniture,1000000.0,19.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,19.0,True,False,Fake,Report #4253
HouseHoldCollection,Furniture,1000000.0,19.0,True,False,Fake,
HouseHoldCollection,Furniture,1000000.0,19.0,True,False,,Report #4254
HouseHoldCollection,Furniture,1000000.0,19.0,True,False,,
HouseHoldCollection,Furniture,1000000.0,19.0,False,True,Genuine,Report #4255
HouseHoldCollection,Furniture,1000000.0,19.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,19.0,False,True,Fake,Report #4256
HouseHoldCollection,Furniture,1000000.0,19.0,False,True,Fake,
HouseHoldCollection,Furniture,1000000.0,19.0,False,True,,Report #4257
HouseHoldCollection,Furniture,1000000.0,19.0,False,True,,
HouseHoldCollection,Furniture,1000000.0,19.0,False,False,Genuine,Report #4258
HouseHoldCollection,Furniture,1000000.0,19.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,19.0,False,False,Fake,Report #4259
HouseHoldCollection,Furniture,1000000.0,19.0,False,False,Fake,
HouseHoldCollection,Furniture,1000000.0,19.0,False,False,,Report #4260
HouseHoldCollection,Furniture,1000000.0,19.0,False,False,,
HouseHoldCollection,Furniture,1000000.0,20.0,True,True,Genuine,Report #4261
HouseHoldCollection,Furniture,1000000.0,20.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,20.0,True,True,Fake,Report #4262
HouseHoldCollection,Furniture,1000000.0,20.0,True,True,Fake,
HouseHoldCollection,Furniture,1000000.0,20.0,True,True,,Report #4263
HouseHoldCollection,Furniture,1000000.0,20.0,True,True,,
HouseHoldCollection,Furniture,1000000.0,20.0,True,False,Genuine,Report #4264
HouseHoldCollection,Furniture,1000000.0,20.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,20.0,True,False,Fake,Report #4265
HouseHoldCollection,Furniture,1000000.0,20.0,True,False,Fake,
HouseHoldCollection,Furniture,1000000.0,20.0,True,False,,Report #4266
HouseHoldCollection,Furniture,1000000.0,20.0,True,False,,
HouseHoldCollection,Furniture,1000000.0,20.0,False,True,Genuine,Report #4267
HouseHoldCollection,Furniture,1000000.0,20.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,20.0,False,True,Fake,Report #4268
HouseHoldCollection,Furniture,1000000.0,20.0,False,True,Fake,
HouseHoldCollection,Furniture,1000000.0,20.0,False,True,,Report #4269
HouseHoldCollection,Furniture,1000000.0,20.0,False,True,,
HouseHoldCollection,Furniture,1000000.0,20.0,False,False,Genuine,Report #4270
HouseHoldCollection,Furniture,1000000.0,20.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,20.0,False,False,Fake,Report #4271
HouseHoldCollection,Furniture,1000000.0,20.0,False,False,Fake,
HouseHoldCollection,Furniture,1000000.0,20.0,False,False,,Report #4272
HouseHoldCollection,Furniture,1000000.0,20.0,False,False,,
HouseHoldCollection,Furniture,1000000.0,21.0,True,True,Genuine,Report #4273
HouseHoldCollection,Furniture,1000000.0,21.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,21.0,True,True,Fake,Report #4274
HouseHoldCollection,Furniture,1000000.0,21.0,True,True,Fake,
HouseHoldCollection,Furniture,1000000.0,21.0,True,True,,Report #4275
HouseHoldCollection,Furniture,1000000.0,21.0,True,True,,
HouseHoldCollection,Furniture,1000000.0,21.0,True,False,Genuine,Report #4276
HouseHoldCollection,Furniture,1000000.0,21.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,21.0,True,False,Fake,Report #4277
HouseHoldCollection,Furniture,1000000.0,21.0,True,False,Fake,
HouseHoldCollection,Furniture,1000000.0,21.0,True,False,,Report #4278
HouseHoldCollection,Furniture,1000000.0,21.0,True,False,,
HouseHoldCollection,Furniture,1000000.0,21.0,False,True,Genuine,Report #4279
HouseHoldCollection,Furniture,1000000.0,21.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,21.0,False,True,Fake,Report #4280
HouseHoldCollection,Furniture,1000000.0,21.0,False,True,Fake,
HouseHoldCollection,Furniture,1000000.0,21.0,False,True,,Report #4281
HouseHoldCollection,Furniture,1000000.0,21.0,False,True,,
HouseHoldCollection,Furniture,1000000.0,21.0,False,False,Genuine,Report #4282
HouseHoldCollection,Furniture,1000000.0,21.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,21.0,False,False,Fake,Report #4283
HouseHoldCollection,Furniture,1000000.0,21.0,False,False,Fake,
HouseHoldCollection,Furniture,1000000.0,21.0,False,False,,Report #4284
HouseHoldCollection,Furniture,1000000.0,21.0,False,False,,
HouseHoldCollection,Furniture,1000000.0,99.0,True,True,Genuine,Report #4285
HouseHoldCollection,Furniture,1000000.0,99.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,99.0,True,True,Fake,Report #4286
HouseHoldCollection,Furniture,1000000.0,99.0,True,True,Fake,
HouseHoldCollection,Furniture,1000000.0,99.0,True,True,,Report #4287
HouseHoldCollection,Furniture,1000000.0,99.0,True,True,,
HouseHoldCollection,Furniture,1000000.0,99.0,True,False,Genuine,Report #4288
HouseHoldCollection,Furniture,1000000.0,99.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,99.0,True,False,Fake,Report #4289
HouseHoldCollection,Furniture,1000000.0,99.0,True,False,Fake,
HouseHoldCollection,Furniture,1000000.0,99.0,True,False,,Report #4290
HouseHoldCollection,Furniture,1000000.0,99.0,True,False,,
HouseHoldCollection,Furniture,1000000.0,99.0,False,True,Genuine,Report #4291
HouseHoldCollection,Furniture,1000000.0,99.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,99.0,False,True,Fake,Report #4292
HouseHoldCollection,Furniture,1000000.0,99.0,False,True,Fake,
HouseHoldCollection,Furniture,1000000.0,99.0,False,True,,Report #4293
HouseHoldCollection,Furniture,1000000.0,99.0,False,True,,
HouseHoldCollection,Furniture,1000000.0,99.0,False,False,Genuine,Report #4294
HouseHoldCollection,Furniture,1000000.0,99.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,99.0,False,False,Fake,Report #4295
HouseHoldCollection,Furniture,1000000.0,99.0,False,False,Fake,
HouseHoldCollection,Furniture,1000000.0,99.0,False,False,,Report #4296
HouseHoldCollection,Furniture,1000000.0,99.0,False,False,,
HouseHoldCollection,Furniture,1000000.0,100.0,True,True,Genuine,Report #4297
HouseHoldCollection,Furniture,1000000.0,100.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,100.0,True,True,Fake,Report #4298
HouseHoldCollection,Furniture,1000000.0,100.0,True,True,Fake,
HouseHoldCollection,Furniture,1000000.0,100.0,True,True,,Report #4299
HouseHoldCollection,Furniture,1000000.0,100.0,True,True,,
HouseHoldCollection,Furniture,1000000.0,100.0,True,False,Genuine,Report #4300
HouseHoldCollection,Furniture,1000000.0,100.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,100.0,True,False,Fake,Report #4301
HouseHoldCollection,Furniture,1000000.0,100.0,True,False,Fake,
HouseHoldCollection,Furniture,1000000.0,100.0,True,False,,Report #4302
HouseHoldCollection,Furniture,1000000.0,100.0,True,False,,
HouseHoldCollection,Furniture,1000000.0,100.0,False,True,Genuine,Report #4303
HouseHoldCollection,Furniture,1000000.0,100.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,100.0,False,True,Fake,Report #4304
HouseHoldCollection,Furniture,1000000.0,100.0,False,True,Fake,
HouseHoldCollection,Furniture,1000000.0,100.0,False,True,,Report #4305
HouseHoldCollection,Furniture,1000000.0,100.0,False,True,,
HouseHoldCollection,Furniture,1000000.0,100.0,False,False,Genuine,Report #4306
HouseHoldCollection,Furniture,1000000.0,100.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,100.0,False,False,Fake,Report #4307
HouseHoldCollection,Furniture,1000000.0,100.0,False,False,Fake,
HouseHoldCollection,Furniture,1000000.0,100.0,False,False,,Report #4308
HouseHoldCollection,Furniture,1000000.0,100.0,False,False,,
HouseHoldCollection,Furniture,1000000.0,101.0,True,True,Genuine,Report #4309
HouseHoldCollection,Furniture,1000000.0,101.0,True,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,101.0,True,True,Fake,Report #4310
HouseHoldCollection,Furniture,1000000.0,101.0,True,True,Fake,
HouseHoldCollection,Furniture,1000000.0,101.0,True,True,,Report #4311
HouseHoldCollection,Furniture,1000000.0,101.0,True,True,,
HouseHoldCollection,Furniture,1000000.0,101.0,True,False,Genuine,Report #4312
HouseHoldCollection,Furniture,1000000.0,101.0,True,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,101.0,True,False,Fake,Report #4313
HouseHoldCollection,Furniture,1000000.0,101.0,True,False,Fake,
HouseHoldCollection,Furniture,1000000.0,101.0,True,False,,Report #4314
HouseHoldCollection,Furniture,1000000.0,101.0,True,False,,
HouseHoldCollection,Furniture,1000000.0,101.0,False,True,Genuine,Report #4315
HouseHoldCollection,Furniture,1000000.0,101.0,False,True,Genuine,
HouseHoldCollection,Furniture,1000000.0,101.0,False,True,Fake,Report #4316
HouseHoldCollection,Furniture,1000000.0,101.0,False,True,Fake,
HouseHoldCollection,Furniture,1000000.0,101.0,False,True,,Report #4317
HouseHoldCollection,Furniture,1000000.0,101.0,False,True,,
HouseHoldCollection,Furniture,1000000.0,101.0,False,False,Genuine,Report #4318
HouseHoldCollection,Furniture,1000000.0,101.0,False,False,Genuine,
HouseHoldCollection,Furniture,1000000.0,101.0,False,False,Fake,Report #4319
HouseHoldCollection,Furniture,1000000.0,101.0,False,False,Fake,
HouseHoldCollection,Furniture,1000000.0,101.0,False,False,,Report #4320
HouseHoldCollection,Furniture,1000000.0,101.0,False,False,,
HouseHoldCollection,HistoryWritings,0.0,-1.0,True,True,Genuine,Report #4321
HouseHoldCollection,HistoryWritings,0.0,-1.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,-1.0,True,True,Fake,Report #4322
HouseHoldCollection,HistoryWritings,0.0,-1.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,-1.0,True,True,,Report #4323
HouseHoldCollection,HistoryWritings,0.0,-1.0,True,True,,
HouseHoldCollection,HistoryWritings,0.0,-1.0,True,False,Genuine,Report #4324
HouseHoldCollection,HistoryWritings,0.0,-1.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,-1.0,True,False,Fake,Report #4325
HouseHoldCollection,HistoryWritings,0.0,-1.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,-1.0,True,False,,Report #4326
HouseHoldCollection,HistoryWritings,0.0,-1.0,True,False,,
HouseHoldCollection,HistoryWritings,0.0,-1.0,False,True,Genuine,Report #4327
HouseHoldCollection,HistoryWritings,0.0,-1.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,-1.0,False,True,Fake,Report #4328
HouseHoldCollection,HistoryWritings,0.0,-1.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,-1.0,False,True,,Report #4329
HouseHoldCollection,HistoryWritings,0.0,-1.0,False,True,,
HouseHoldCollection,HistoryWritings,0.0,-1.0,False,False,Genuine,Report #4330
HouseHoldCollection,HistoryWritings,0.0,-1.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,-1.0,False,False,Fake,Report #4331
HouseHoldCollection,HistoryWritings,0.0,-1.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,-1.0,False,False,,Report #4332
HouseHoldCollection,HistoryWritings,0.0,-1.0,False,False,,
HouseHoldCollection,HistoryWritings,0.0,0.0,True,True,Genuine,Report #4333
HouseHoldCollection,HistoryWritings,0.0,0.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,0.0,True,True,Fake,Report #4334
HouseHoldCollection,HistoryWritings,0.0,0.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,0.0,True,True,,Report #4335
HouseHoldCollection,HistoryWritings,0.0,0.0,True,True,,
HouseHoldCollection,HistoryWritings,0.0,0.0,True,False,Genuine,Report #4336
HouseHoldCollection,HistoryWritings,0.0,0.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,0.0,True,False,Fake,Report #4337
HouseHoldCollection,HistoryWritings,0.0,0.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,0.0,True,False,,Report #4338
HouseHoldCollection,HistoryWritings,0.0,0.0,True,False,,
HouseHoldCollection,HistoryWritings,0.0,0.0,False,True,Genuine,Report #4339
HouseHoldCollection,HistoryWritings,0.0,0.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,0.0,False,True,Fake,Report #4340
HouseHoldCollection,HistoryWritings,0.0,0.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,0.0,False,True,,Report #4341
HouseHoldCollection,HistoryWritings,0.0,0.0,False,True,,
HouseHoldCollection,HistoryWritings,0.0,0.0,False,False,Genuine,Report #4342
HouseHoldCollection,HistoryWritings,0.0,0.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,0.0,False,False,Fake,Report #4343
HouseHoldCollection,HistoryWritings,0.0,0.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,0.0,False,False,,Report #4344
HouseHoldCollection,HistoryWritings,0.0,0.0,False,False,,
HouseHoldCollection,HistoryWritings,0.0,19.0,True,True,Genuine,Report #4345
HouseHoldCollection,HistoryWritings,0.0,19.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,19.0,True,True,Fake,Report #4346
HouseHoldCollection,HistoryWritings,0.0,19.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,19.0,True,True,,Report #4347
HouseHoldCollection,HistoryWritings,0.0,19.0,True,True,,
HouseHoldCollection,HistoryWritings,0.0,19.0,True,False,Genuine,Report #4348
HouseHoldCollection,HistoryWritings,0.0,19.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,19.0,True,False,Fake,Report #4349
HouseHoldCollection,HistoryWritings,0.0,19.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,19.0,True,False,,Report #4350
HouseHoldCollection,HistoryWritings,0.0,19.0,True,False,,
HouseHoldCollection,HistoryWritings,0.0,19.0,False,True,Genuine,Report #4351
HouseHoldCollection,HistoryWritings,0.0,19.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,19.0,False,True,Fake,Report #4352
HouseHoldCollection,HistoryWritings,0.0,19.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,19.0,False,True,,Report #4353
HouseHoldCollection,HistoryWritings,0.0,19.0,False,True,,
HouseHoldCollection,HistoryWritings,0.0,19.0,False,False,Genuine,Report #4354
HouseHoldCollection,HistoryWritings,0.0,19.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,19.0,False,False,Fake,Report #4355
HouseHoldCollection,HistoryWritings,0.0,19.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,19.0,False,False,,Report #4356
HouseHoldCollection,HistoryWritings,0.0,19.0,False,False,,
HouseHoldCollection,HistoryWritings,0.0,20.0,True,True,Genuine,Report #4357
HouseHoldCollection,HistoryWritings,0.0,20.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,20.0,True,True,Fake,Report #4358
HouseHoldCollection,HistoryWritings,0.0,20.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,20.0,True,True,,Report #4359
HouseHoldCollection,HistoryWritings,0.0,20.0,True,True,,
HouseHoldCollection,HistoryWritings,0.0,20.0,True,False,Genuine,Report #4360
HouseHoldCollection,HistoryWritings,0.0,20.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,20.0,True,False,Fake,Report #4361
HouseHoldCollection,HistoryWritings,0.0,20.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,20.0,True,False,,Report #4362
HouseHoldCollection,HistoryWritings,0.0,20.0,True,False,,
HouseHoldCollection,HistoryWritings,0.0,20.0,False,True,Genuine,Report #4363
HouseHoldCollection,HistoryWritings,0.0,20.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,20.0,False,True,Fake,Report #4364
HouseHoldCollection,HistoryWritings,0.0,20.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,20.0,False,True,,Report #4365
HouseHoldCollection,HistoryWritings,0.0,20.0,False,True,,
HouseHoldCollection,HistoryWritings,0.0,20.0,False,False,Genuine,Report #4366
HouseHoldCollection,HistoryWritings,0.0,20.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,20.0,False,False,Fake,Report #4367
HouseHoldCollection,HistoryWritings,0.0,20.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,20.0,False,False,,Report #4368
HouseHoldCollection,HistoryWritings,0.0,20.0,False,False,,
HouseHoldCollection,HistoryWritings,0.0,21.0,True,True,Genuine,Report #4369
HouseHoldCollection,HistoryWritings,0.0,21.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,21.0,True,True,Fake,Report #4370
HouseHoldCollection,HistoryWritings,0.0,21.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,21.0,True,True,,Report #4371
HouseHoldCollection,HistoryWritings,0.0,21.0,True,True,,
HouseHoldCollection,HistoryWritings,0.0,21.0,True,False,Genuine,Report #4372
HouseHoldCollection,HistoryWritings,0.0,21.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,21.0,True,False,Fake,Report #4373
HouseHoldCollection,HistoryWritings,0.0,21.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,21.0,True,False,,Report #4374
HouseHoldCollection,HistoryWritings,0.0,21.0,True,False,,
HouseHoldCollection,HistoryWritings,0.0,21.0,False,True,Genuine,Report #4375
HouseHoldCollection,HistoryWritings,0.0,21.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,21.0,False,True,Fake,Report #4376
HouseHoldCollection,HistoryWritings,0.0,21.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,21.0,False,True,,Report #4377
HouseHoldCollection,HistoryWritings,0.0,21.0,False,True,,
HouseHoldCollection,HistoryWritings,0.0,21.0,False,False,Genuine,Report #4378
HouseHoldCollection,HistoryWritings,0.0,21.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,21.0,False,False,Fake,Report #4379
HouseHoldCollection,HistoryWritings,0.0,21.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,21.0,False,False,,Report #4380
HouseHoldCollection,HistoryWritings,0.0,21.0,False,False,,
HouseHoldCollection,HistoryWritings,0.0,99.0,True,True,Genuine,Report #4381
HouseHoldCollection,HistoryWritings,0.0,99.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,99.0,True,True,Fake,Report #4382
HouseHoldCollection,HistoryWritings,0.0,99.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,99.0,True,True,,Report #4383
HouseHoldCollection,HistoryWritings,0.0,99.0,True,True,,
HouseHoldCollection,HistoryWritings,0.0,99.0,True,False,Genuine,Report #4384
HouseHoldCollection,HistoryWritings,0.0,99.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,99.0,True,False,Fake,Report #4385
HouseHoldCollection,HistoryWritings,0.0,99.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,99.0,True,False,,Report #4386
HouseHoldCollection,HistoryWritings,0.0,99.0,True,False,,
HouseHoldCollection,HistoryWritings,0.0,99.0,False,True,Genuine,Report #4387
HouseHoldCollection,HistoryWritings,0.0,99.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,99.0,False,True,Fake,Report #4388
HouseHoldCollection,HistoryWritings,0.0,99.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,99.0,False,True,,Report #4389
HouseHoldCollection,HistoryWritings,0.0,99.0,False,True,,
HouseHoldCollection,HistoryWritings,0.0,99.0,False,False,Genuine,Report #4390
HouseHoldCollection,HistoryWritings,0.0,99.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,99.0,False,False,Fake,Report #4391
HouseHoldCollection,HistoryWritings,0.0,99.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,99.0,False,False,,Report #4392
HouseHoldCollection,HistoryWritings,0.0,99.0,False,False,,
HouseHoldCollection,HistoryWritings,0.0,100.0,True,True,Genuine,Report #4393
HouseHoldCollection,HistoryWritings,0.0,100.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,100.0,True,True,Fake,Report #4394
HouseHoldCollection,HistoryWritings,0.0,100.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,100.0,True,True,,Report #4395
HouseHoldCollection,HistoryWritings,0.0,100.0,True,True,,
HouseHoldCollection,HistoryWritings,0.0,100.0,True,False,Genuine,Report #4396
HouseHoldCollection,HistoryWritings,0.0,100.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,100.0,True,False,Fake,Report #4397
HouseHoldCollection,HistoryWritings,0.0,100.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,100.0,True,False,,Report #4398
HouseHoldCollection,HistoryWritings,0.0,100.0,True,False,,
HouseHoldCollection,HistoryWritings,0.0,100.0,False,True,Genuine,Report #4399
HouseHoldCollection,HistoryWritings,0.0,100.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,100.0,False,True,Fake,Report #4400
HouseHoldCollection,HistoryWritings,0.0,100.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,100.0,False,True,,Report #4401
HouseHoldCollection,HistoryWritings,0.0,100.0,False,True,,
HouseHoldCollection,HistoryWritings,0.0,100.0,False,False,Genuine,Report #4402
HouseHoldCollection,HistoryWritings,0.0,100.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,100.0,False,False,Fake,Report #4403
HouseHoldCollection,HistoryWritings,0.0,100.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,100.0,False,False,,Report #4404
HouseHoldCollection,HistoryWritings,0.0,100.0,False,False,,
HouseHoldCollection,HistoryWritings,0.0,101.0,True,True,Genuine,Report #4405
HouseHoldCollection,HistoryWritings,0.0,101.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,101.0,True,True,Fake,Report #4406
HouseHoldCollection,HistoryWritings,0.0,101.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,101.0,True,True,,Report #4407
HouseHoldCollection,HistoryWritings,0.0,101.0,True,True,,
HouseHoldCollection,HistoryWritings,0.0,101.0,True,False,Genuine,Report #4408
HouseHoldCollection,HistoryWritings,0.0,101.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,101.0,True,False,Fake,Report #4409
HouseHoldCollection,HistoryWritings,0.0,101.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,101.0,True,False,,Report #4410
HouseHoldCollection,HistoryWritings,0.0,101.0,True,False,,
HouseHoldCollection,HistoryWritings,0.0,101.0,False,True,Genuine,Report #4411
HouseHoldCollection,HistoryWritings,0.0,101.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,0.0,101.0,False,True,Fake,Report #4412
HouseHoldCollection,HistoryWritings,0.0,101.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,0.0,101.0,False,True,,Report #4413
HouseHoldCollection,HistoryWritings,0.0,101.0,False,True,,
HouseHoldCollection,HistoryWritings,0.0,101.0,False,False,Genuine,Report #4414
HouseHoldCollection,HistoryWritings,0.0,101.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,0.0,101.0,False,False,Fake,Report #4415
HouseHoldCollection,HistoryWritings,0.0,101.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,0.0,101.0,False,False,,Report #4416
HouseHoldCollection,HistoryWritings,0.0,101.0,False,False,,
HouseHoldCollection,HistoryWritings,1.0,-1.0,True,True,Genuine,Report #4417
HouseHoldCollection,HistoryWritings,1.0,-1.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,-1.0,True,True,Fake,Report #4418
HouseHoldCollection,HistoryWritings,1.0,-1.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,-1.0,True,True,,Report #4419
HouseHoldCollection,HistoryWritings,1.0,-1.0,True,True,,
HouseHoldCollection,HistoryWritings,1.0,-1.0,True,False,Genuine,Report #4420
HouseHoldCollection,HistoryWritings,1.0,-1.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,-1.0,True,False,Fake,Report #4421
HouseHoldCollection,HistoryWritings,1.0,-1.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,-1.0,True,False,,Report #4422
HouseHoldCollection,HistoryWritings,1.0,-1.0,True,False,,
HouseHoldCollection,HistoryWritings,1.0,-1.0,False,True,Genuine,Report #4423
HouseHoldCollection,HistoryWritings,1.0,-1.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,-1.0,False,True,Fake,Report #4424
HouseHoldCollection,HistoryWritings,1.0,-1.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,-1.0,False,True,,Report #4425
HouseHoldCollection,HistoryWritings,1.0,-1.0,False,True,,
HouseHoldCollection,HistoryWritings,1.0,-1.0,False,False,Genuine,Report #4426
HouseHoldCollection,HistoryWritings,1.0,-1.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,-1.0,False,False,Fake,Report #4427
HouseHoldCollection,HistoryWritings,1.0,-1.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,-1.0,False,False,,Report #4428
HouseHoldCollection,HistoryWritings,1.0,-1.0,False,False,,
HouseHoldCollection,HistoryWritings,1.0,0.0,True,True,Genuine,Report #4429
HouseHoldCollection,HistoryWritings,1.0,0.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,0.0,True,True,Fake,Report #4430
HouseHoldCollection,HistoryWritings,1.0,0.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,0.0,True,True,,Report #4431
HouseHoldCollection,HistoryWritings,1.0,0.0,True,True,,
HouseHoldCollection,HistoryWritings,1.0,0.0,True,False,Genuine,Report #4432
HouseHoldCollection,HistoryWritings,1.0,0.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,0.0,True,False,Fake,Report #4433
HouseHoldCollection,HistoryWritings,1.0,0.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,0.0,True,False,,Report #4434
HouseHoldCollection,HistoryWritings,1.0,0.0,True,False,,
HouseHoldCollection,HistoryWritings,1.0,0.0,False,True,Genuine,Report #4435
HouseHoldCollection,HistoryWritings,1.0,0.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,0.0,False,True,Fake,Report #4436
HouseHoldCollection,HistoryWritings,1.0,0.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,0.0,False,True,,Report #4437
HouseHoldCollection,HistoryWritings,1.0,0.0,False,True,,
HouseHoldCollection,HistoryWritings,1.0,0.0,False,False,Genuine,Report #4438
HouseHoldCollection,HistoryWritings,1.0,0.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,0.0,False,False,Fake,Report #4439
HouseHoldCollection,HistoryWritings,1.0,0.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,0.0,False,False,,Report #4440
HouseHoldCollection,HistoryWritings,1.0,0.0,False,False,,
HouseHoldCollection,HistoryWritings,1.0,19.0,True,True,Genuine,Report #4441
HouseHoldCollection,HistoryWritings,1.0,19.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,19.0,True,True,Fake,Report #4442
HouseHoldCollection,HistoryWritings,1.0,19.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,19.0,True,True,,Report #4443
HouseHoldCollection,HistoryWritings,1.0,19.0,True,True,,
HouseHoldCollection,HistoryWritings,1.0,19.0,True,False,Genuine,Report #4444
HouseHoldCollection,HistoryWritings,1.0,19.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,19.0,True,False,Fake,Report #4445
HouseHoldCollection,HistoryWritings,1.0,19.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,19.0,True,False,,Report #4446
HouseHoldCollection,HistoryWritings,1.0,19.0,True,False,,
HouseHoldCollection,HistoryWritings,1.0,19.0,False,True,Genuine,Report #4447
HouseHoldCollection,HistoryWritings,1.0,19.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,19.0,False,True,Fake,Report #4448
HouseHoldCollection,HistoryWritings,1.0,19.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,19.0,False,True,,Report #4449
HouseHoldCollection,HistoryWritings,1.0,19.0,False,True,,
HouseHoldCollection,HistoryWritings,1.0,19.0,False,False,Genuine,Report #4450
HouseHoldCollection,HistoryWritings,1.0,19.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,19.0,False,False,Fake,Report #4451
HouseHoldCollection,HistoryWritings,1.0,19.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,19.0,False,False,,Report #4452
HouseHoldCollection,HistoryWritings,1.0,19.0,False,False,,
HouseHoldCollection,HistoryWritings,1.0,20.0,True,True,Genuine,Report #4453
HouseHoldCollection,HistoryWritings,1.0,20.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,20.0,True,True,Fake,Report #4454
HouseHoldCollection,HistoryWritings,1.0,20.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,20.0,True,True,,Report #4455
HouseHoldCollection,HistoryWritings,1.0,20.0,True,True,,
HouseHoldCollection,HistoryWritings,1.0,20.0,True,False,Genuine,Report #4456
HouseHoldCollection,HistoryWritings,1.0,20.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,20.0,True,False,Fake,Report #4457
HouseHoldCollection,HistoryWritings,1.0,20.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,20.0,True,False,,Report #4458
HouseHoldCollection,HistoryWritings,1.0,20.0,True,False,,
HouseHoldCollection,HistoryWritings,1.0,20.0,False,True,Genuine,Report #4459
HouseHoldCollection,HistoryWritings,1.0,20.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,20.0,False,True,Fake,Report #4460
HouseHoldCollection,HistoryWritings,1.0,20.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,20.0,False,True,,Report #4461
HouseHoldCollection,HistoryWritings,1.0,20.0,False,True,,
HouseHoldCollection,HistoryWritings,1.0,20.0,False,False,Genuine,Report #4462
HouseHoldCollection,HistoryWritings,1.0,20.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,20.0,False,False,Fake,Report #4463
HouseHoldCollection,HistoryWritings,1.0,20.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,20.0,False,False,,Report #4464
HouseHoldCollection,HistoryWritings,1.0,20.0,False,False,,
HouseHoldCollection,HistoryWritings,1.0,21.0,True,True,Genuine,Report #4465
HouseHoldCollection,HistoryWritings,1.0,21.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,21.0,True,True,Fake,Report #4466
HouseHoldCollection,HistoryWritings,1.0,21.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,21.0,True,True,,Report #4467
HouseHoldCollection,HistoryWritings,1.0,21.0,True,True,,
HouseHoldCollection,HistoryWritings,1.0,21.0,True,False,Genuine,Report #4468
HouseHoldCollection,HistoryWritings,1.0,21.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,21.0,True,False,Fake,Report #4469
HouseHoldCollection,HistoryWritings,1.0,21.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,21.0,True,False,,Report #4470
HouseHoldCollection,HistoryWritings,1.0,21.0,True,False,,
HouseHoldCollection,HistoryWritings,1.0,21.0,False,True,Genuine,Report #4471
HouseHoldCollection,HistoryWritings,1.0,21.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,21.0,False,True,Fake,Report #4472
HouseHoldCollection,HistoryWritings,1.0,21.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,21.0,False,True,,Report #4473
HouseHoldCollection,HistoryWritings,1.0,21.0,False,True,,
HouseHoldCollection,HistoryWritings,1.0,21.0,False,False,Genuine,Report #4474
HouseHoldCollection,HistoryWritings,1.0,21.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,21.0,False,False,Fake,Report #4475
HouseHoldCollection,HistoryWritings,1.0,21.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,21.0,False,False,,Report #4476
HouseHoldCollection,HistoryWritings,1.0,21.0,False,False,,
HouseHoldCollection,HistoryWritings,1.0,99.0,True,True,Genuine,Report #4477
HouseHoldCollection,HistoryWritings,1.0,99.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,99.0,True,True,Fake,Report #4478
HouseHoldCollection,HistoryWritings,1.0,99.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,99.0,True,True,,Report #4479
HouseHoldCollection,HistoryWritings,1.0,99.0,True,True,,
HouseHoldCollection,HistoryWritings,1.0,99.0,True,False,Genuine,Report #4480
HouseHoldCollection,HistoryWritings,1.0,99.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,99.0,True,False,Fake,Report #4481
HouseHoldCollection,HistoryWritings,1.0,99.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,99.0,True,False,,Report #4482
HouseHoldCollection,HistoryWritings,1.0,99.0,True,False,,
HouseHoldCollection,HistoryWritings,1.0,99.0,False,True,Genuine,Report #4483
HouseHoldCollection,HistoryWritings,1.0,99.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,99.0,False,True,Fake,Report #4484
HouseHoldCollection,HistoryWritings,1.0,99.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,99.0,False,True,,Report #4485
HouseHoldCollection,HistoryWritings,1.0,99.0,False,True,,
HouseHoldCollection,HistoryWritings,1.0,99.0,False,False,Genuine,Report #4486
HouseHoldCollection,HistoryWritings,1.0,99.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,99.0,False,False,Fake,Report #4487
HouseHoldCollection,HistoryWritings,1.0,99.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,99.0,False,False,,Report #4488
HouseHoldCollection,HistoryWritings,1.0,99.0,False,False,,
HouseHoldCollection,HistoryWritings,1.0,100.0,True,True,Genuine,Report #4489
HouseHoldCollection,HistoryWritings,1.0,100.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,100.0,True,True,Fake,Report #4490
HouseHoldCollection,HistoryWritings,1.0,100.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,100.0,True,True,,Report #4491
HouseHoldCollection,HistoryWritings,1.0,100.0,True,True,,
HouseHoldCollection,HistoryWritings,1.0,100.0,True,False,Genuine,Report #4492
HouseHoldCollection,HistoryWritings,1.0,100.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,100.0,True,False,Fake,Report #4493
HouseHoldCollection,HistoryWritings,1.0,100.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,100.0,True,False,,Report #4494
HouseHoldCollection,HistoryWritings,1.0,100.0,True,False,,
HouseHoldCollection,HistoryWritings,1.0,100.0,False,True,Genuine,Report #4495
HouseHoldCollection,HistoryWritings,1.0,100.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,100.0,False,True,Fake,Report #4496
HouseHoldCollection,HistoryWritings,1.0,100.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,100.0,False,True,,Report #4497
HouseHoldCollection,HistoryWritings,1.0,100.0,False,True,,
HouseHoldCollection,HistoryWritings,1.0,100.0,False,False,Genuine,Report #4498
HouseHoldCollection,HistoryWritings,1.0,100.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,100.0,False,False,Fake,Report #4499
HouseHoldCollection,HistoryWritings,1.0,100.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,100.0,False,False,,Report #4500
HouseHoldCollection,HistoryWritings,1.0,100.0,False,False,,
HouseHoldCollection,HistoryWritings,1.0,101.0,True,True,Genuine,Report #4501
HouseHoldCollection,HistoryWritings,1.0,101.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,101.0,True,True,Fake,Report #4502
HouseHoldCollection,HistoryWritings,1.0,101.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,101.0,True,True,,Report #4503
HouseHoldCollection,HistoryWritings,1.0,101.0,True,True,,
HouseHoldCollection,HistoryWritings,1.0,101.0,True,False,Genuine,Report #4504
HouseHoldCollection,HistoryWritings,1.0,101.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,101.0,True,False,Fake,Report #4505
HouseHoldCollection,HistoryWritings,1.0,101.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,101.0,True,False,,Report #4506
HouseHoldCollection,HistoryWritings,1.0,101.0,True,False,,
HouseHoldCollection,HistoryWritings,1.0,101.0,False,True,Genuine,Report #4507
HouseHoldCollection,HistoryWritings,1.0,101.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1.0,101.0,False,True,Fake,Report #4508
HouseHoldCollection,HistoryWritings,1.0,101.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1.0,101.0,False,True,,Report #4509
HouseHoldCollection,HistoryWritings,1.0,101.0,False,True,,
HouseHoldCollection,HistoryWritings,1.0,101.0,False,False,Genuine,Report #4510
HouseHoldCollection,HistoryWritings,1.0,101.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1.0,101.0,False,False,Fake,Report #4511
HouseHoldCollection,HistoryWritings,1.0,101.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1.0,101.0,False,False,,Report #4512
HouseHoldCollection,HistoryWritings,1.0,101.0,False,False,,
HouseHoldCollection,HistoryWritings,100.0,-1.0,True,True,Genuine,Report #4513
HouseHoldCollection,HistoryWritings,100.0,-1.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,-1.0,True,True,Fake,Report #4514
HouseHoldCollection,HistoryWritings,100.0,-1.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,-1.0,True,True,,Report #4515
HouseHoldCollection,HistoryWritings,100.0,-1.0,True,True,,
HouseHoldCollection,HistoryWritings,100.0,-1.0,True,False,Genuine,Report #4516
HouseHoldCollection,HistoryWritings,100.0,-1.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,-1.0,True,False,Fake,Report #4517
HouseHoldCollection,HistoryWritings,100.0,-1.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,-1.0,True,False,,Report #4518
HouseHoldCollection,HistoryWritings,100.0,-1.0,True,False,,
HouseHoldCollection,HistoryWritings,100.0,-1.0,False,True,Genuine,Report #4519
HouseHoldCollection,HistoryWritings,100.0,-1.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,-1.0,False,True,Fake,Report #4520
HouseHoldCollection,HistoryWritings,100.0,-1.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,-1.0,False,True,,Report #4521
HouseHoldCollection,HistoryWritings,100.0,-1.0,False,True,,
HouseHoldCollection,HistoryWritings,100.0,-1.0,False,False,Genuine,Report #4522
HouseHoldCollection,HistoryWritings,100.0,-1.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,-1.0,False,False,Fake,Report #4523
HouseHoldCollection,HistoryWritings,100.0,-1.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,-1.0,False,False,,Report #4524
HouseHoldCollection,HistoryWritings,100.0,-1.0,False,False,,
HouseHoldCollection,HistoryWritings,100.0,0.0,True,True,Genuine,Report #4525
HouseHoldCollection,HistoryWritings,100.0,0.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,0.0,True,True,Fake,Report #4526
HouseHoldCollection,HistoryWritings,100.0,0.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,0.0,True,True,,Report #4527
HouseHoldCollection,HistoryWritings,100.0,0.0,True,True,,
HouseHoldCollection,HistoryWritings,100.0,0.0,True,False,Genuine,Report #4528
HouseHoldCollection,HistoryWritings,100.0,0.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,0.0,True,False,Fake,Report #4529
HouseHoldCollection,HistoryWritings,100.0,0.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,0.0,True,False,,Report #4530
HouseHoldCollection,HistoryWritings,100.0,0.0,True,False,,
HouseHoldCollection,HistoryWritings,100.0,0.0,False,True,Genuine,Report #4531
HouseHoldCollection,HistoryWritings,100.0,0.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,0.0,False,True,Fake,Report #4532
HouseHoldCollection,HistoryWritings,100.0,0.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,0.0,False,True,,Report #4533
HouseHoldCollection,HistoryWritings,100.0,0.0,False,True,,
HouseHoldCollection,HistoryWritings,100.0,0.0,False,False,Genuine,Report #4534
HouseHoldCollection,HistoryWritings,100.0,0.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,0.0,False,False,Fake,Report #4535
HouseHoldCollection,HistoryWritings,100.0,0.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,0.0,False,False,,Report #4536
HouseHoldCollection,HistoryWritings,100.0,0.0,False,False,,
HouseHoldCollection,HistoryWritings,100.0,19.0,True,True,Genuine,Report #4537
HouseHoldCollection,HistoryWritings,100.0,19.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,19.0,True,True,Fake,Report #4538
HouseHoldCollection,HistoryWritings,100.0,19.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,19.0,True,True,,Report #4539
HouseHoldCollection,HistoryWritings,100.0,19.0,True,True,,
HouseHoldCollection,HistoryWritings,100.0,19.0,True,False,Genuine,Report #4540
HouseHoldCollection,HistoryWritings,100.0,19.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,19.0,True,False,Fake,Report #4541
HouseHoldCollection,HistoryWritings,100.0,19.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,19.0,True,False,,Report #4542
HouseHoldCollection,HistoryWritings,100.0,19.0,True,False,,
HouseHoldCollection,HistoryWritings,100.0,19.0,False,True,Genuine,Report #4543
HouseHoldCollection,HistoryWritings,100.0,19.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,19.0,False,True,Fake,Report #4544
HouseHoldCollection,HistoryWritings,100.0,19.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,19.0,False,True,,Report #4545
HouseHoldCollection,HistoryWritings,100.0,19.0,False,True,,
HouseHoldCollection,HistoryWritings,100.0,19.0,False,False,Genuine,Report #4546
HouseHoldCollection,HistoryWritings,100.0,19.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,19.0,False,False,Fake,Report #4547
HouseHoldCollection,HistoryWritings,100.0,19.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,19.0,False,False,,Report #4548
HouseHoldCollection,HistoryWritings,100.0,19.0,False,False,,
HouseHoldCollection,HistoryWritings,100.0,20.0,True,True,Genuine,Report #4549
HouseHoldCollection,HistoryWritings,100.0,20.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,20.0,True,True,Fake,Report #4550
HouseHoldCollection,HistoryWritings,100.0,20.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,20.0,True,True,,Report #4551
HouseHoldCollection,HistoryWritings,100.0,20.0,True,True,,
HouseHoldCollection,HistoryWritings,100.0,20.0,True,False,Genuine,Report #4552
HouseHoldCollection,HistoryWritings,100.0,20.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,20.0,True,False,Fake,Report #4553
HouseHoldCollection,HistoryWritings,100.0,20.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,20.0,True,False,,Report #4554
HouseHoldCollection,HistoryWritings,100.0,20.0,True,False,,
HouseHoldCollection,HistoryWritings,100.0,20.0,False,True,Genuine,Report #4555
HouseHoldCollection,HistoryWritings,100.0,20.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,20.0,False,True,Fake,Report #4556
HouseHoldCollection,HistoryWritings,100.0,20.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,20.0,False,True,,Report #4557
HouseHoldCollection,HistoryWritings,100.0,20.0,False,True,,
HouseHoldCollection,HistoryWritings,100.0,20.0,False,False,Genuine,Report #4558
HouseHoldCollection,HistoryWritings,100.0,20.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,20.0,False,False,Fake,Report #4559
HouseHoldCollection,HistoryWritings,100.0,20.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,20.0,False,False,,Report #4560
HouseHoldCollection,HistoryWritings,100.0,20.0,False,False,,
HouseHoldCollection,HistoryWritings,100.0,21.0,True,True,Genuine,Report #4561
HouseHoldCollection,HistoryWritings,100.0,21.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,21.0,True,True,Fake,Report #4562
HouseHoldCollection,HistoryWritings,100.0,21.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,21.0,True,True,,Report #4563
HouseHoldCollection,HistoryWritings,100.0,21.0,True,True,,
HouseHoldCollection,HistoryWritings,100.0,21.0,True,False,Genuine,Report #4564
HouseHoldCollection,HistoryWritings,100.0,21.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,21.0,True,False,Fake,Report #4565
HouseHoldCollection,HistoryWritings,100.0,21.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,21.0,True,False,,Report #4566
HouseHoldCollection,HistoryWritings,100.0,21.0,True,False,,
HouseHoldCollection,HistoryWritings,100.0,21.0,False,True,Genuine,Report #4567
HouseHoldCollection,HistoryWritings,100.0,21.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,21.0,False,True,Fake,Report #4568
HouseHoldCollection,HistoryWritings,100.0,21.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,21.0,False,True,,Report #4569
HouseHoldCollection,HistoryWritings,100.0,21.0,False,True,,
HouseHoldCollection,HistoryWritings,100.0,21.0,False,False,Genuine,Report #4570
HouseHoldCollection,HistoryWritings,100.0,21.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,21.0,False,False,Fake,Report #4571
HouseHoldCollection,HistoryWritings,100.0,21.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,21.0,False,False,,Report #4572
HouseHoldCollection,HistoryWritings,100.0,21.0,False,False,,
HouseHoldCollection,HistoryWritings,100.0,99.0,True,True,Genuine,Report #4573
HouseHoldCollection,HistoryWritings,100.0,99.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,99.0,True,True,Fake,Report #4574
HouseHoldCollection,HistoryWritings,100.0,99.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,99.0,True,True,,Report #4575
HouseHoldCollection,HistoryWritings,100.0,99.0,True,True,,
HouseHoldCollection,HistoryWritings,100.0,99.0,True,False,Genuine,Report #4576
HouseHoldCollection,HistoryWritings,100.0,99.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,99.0,True,False,Fake,Report #4577
HouseHoldCollection,HistoryWritings,100.0,99.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,99.0,True,False,,Report #4578
HouseHoldCollection,HistoryWritings,100.0,99.0,True,False,,
HouseHoldCollection,HistoryWritings,100.0,99.0,False,True,Genuine,Report #4579
HouseHoldCollection,HistoryWritings,100.0,99.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,99.0,False,True,Fake,Report #4580
HouseHoldCollection,HistoryWritings,100.0,99.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,99.0,False,True,,Report #4581
HouseHoldCollection,HistoryWritings,100.0,99.0,False,True,,
HouseHoldCollection,HistoryWritings,100.0,99.0,False,False,Genuine,Report #4582
HouseHoldCollection,HistoryWritings,100.0,99.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,99.0,False,False,Fake,Report #4583
HouseHoldCollection,HistoryWritings,100.0,99.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,99.0,False,False,,Report #4584
HouseHoldCollection,HistoryWritings,100.0,99.0,False,False,,
HouseHoldCollection,HistoryWritings,100.0,100.0,True,True,Genuine,Report #4585
HouseHoldCollection,HistoryWritings,100.0,100.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,100.0,True,True,Fake,Report #4586
HouseHoldCollection,HistoryWritings,100.0,100.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,100.0,True,True,,Report #4587
HouseHoldCollection,HistoryWritings,100.0,100.0,True,True,,
HouseHoldCollection,HistoryWritings,100.0,100.0,True,False,Genuine,Report #4588
HouseHoldCollection,HistoryWritings,100.0,100.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,100.0,True,False,Fake,Report #4589
HouseHoldCollection,HistoryWritings,100.0,100.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,100.0,True,False,,Report #4590
HouseHoldCollection,HistoryWritings,100.0,100.0,True,False,,
HouseHoldCollection,HistoryWritings,100.0,100.0,False,True,Genuine,Report #4591
HouseHoldCollection,HistoryWritings,100.0,100.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,100.0,False,True,Fake,Report #4592
HouseHoldCollection,HistoryWritings,100.0,100.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,100.0,False,True,,Report #4593
HouseHoldCollection,HistoryWritings,100.0,100.0,False,True,,
HouseHoldCollection,HistoryWritings,100.0,100.0,False,False,Genuine,Report #4594
HouseHoldCollection,HistoryWritings,100.0,100.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,100.0,False,False,Fake,Report #4595
HouseHoldCollection,HistoryWritings,100.0,100.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,100.0,False,False,,Report #4596
HouseHoldCollection,HistoryWritings,100.0,100.0,False,False,,
HouseHoldCollection,HistoryWritings,100.0,101.0,True,True,Genuine,Report #4597
HouseHoldCollection,HistoryWritings,100.0,101.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,101.0,True,True,Fake,Report #4598
HouseHoldCollection,HistoryWritings,100.0,101.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,101.0,True,True,,Report #4599
HouseHoldCollection,HistoryWritings,100.0,101.0,True,True,,
HouseHoldCollection,HistoryWritings,100.0,101.0,True,False,Genuine,Report #4600
HouseHoldCollection,HistoryWritings,100.0,101.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,101.0,True,False,Fake,Report #4601
HouseHoldCollection,HistoryWritings,100.0,101.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,101.0,True,False,,Report #4602
HouseHoldCollection,HistoryWritings,100.0,101.0,True,False,,
HouseHoldCollection,HistoryWritings,100.0,101.0,False,True,Genuine,Report #4603
HouseHoldCollection,HistoryWritings,100.0,101.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,100.0,101.0,False,True,Fake,Report #4604
HouseHoldCollection,HistoryWritings,100.0,101.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,100.0,101.0,False,True,,Report #4605
HouseHoldCollection,HistoryWritings,100.0,101.0,False,True,,
HouseHoldCollection,HistoryWritings,100.0,101.0,False,False,Genuine,Report #4606
HouseHoldCollection,HistoryWritings,100.0,101.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,100.0,101.0,False,False,Fake,Report #4607
HouseHoldCollection,HistoryWritings,100.0,101.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,100.0,101.0,False,False,,Report #4608
HouseHoldCollection,HistoryWritings,100.0,101.0,False,False,,
HouseHoldCollection,HistoryWritings,1000.0,-1.0,True,True,Genuine,Report #4609
HouseHoldCollection,HistoryWritings,1000.0,-1.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,-1.0,True,True,Fake,Report #4610
HouseHoldCollection,HistoryWritings,1000.0,-1.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,-1.0,True,True,,Report #4611
HouseHoldCollection,HistoryWritings,1000.0,-1.0,True,True,,
HouseHoldCollection,HistoryWritings,1000.0,-1.0,True,False,Genuine,Report #4612
HouseHoldCollection,HistoryWritings,1000.0,-1.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,-1.0,True,False,Fake,Report #4613
HouseHoldCollection,HistoryWritings,1000.0,-1.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,-1.0,True,False,,Report #4614
HouseHoldCollection,HistoryWritings,1000.0,-1.0,True,False,,
HouseHoldCollection,HistoryWritings,1000.0,-1.0,False,True,Genuine,Report #4615
HouseHoldCollection,HistoryWritings,1000.0,-1.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,-1.0,False,True,Fake,Report #4616
HouseHoldCollection,HistoryWritings,1000.0,-1.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,-1.0,False,True,,Report #4617
HouseHoldCollection,HistoryWritings,1000.0,-1.0,False,True,,
HouseHoldCollection,HistoryWritings,1000.0,-1.0,False,False,Genuine,Report #4618
HouseHoldCollection,HistoryWritings,1000.0,-1.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,-1.0,False,False,Fake,Report #4619
HouseHoldCollection,HistoryWritings,1000.0,-1.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,-1.0,False,False,,Report #4620
HouseHoldCollection,HistoryWritings,1000.0,-1.0,False,False,,
HouseHoldCollection,HistoryWritings,1000.0,0.0,True,True,Genuine,Report #4621
HouseHoldCollection,HistoryWritings,1000.0,0.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,0.0,True,True,Fake,Report #4622
HouseHoldCollection,HistoryWritings,1000.0,0.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,0.0,True,True,,Report #4623
HouseHoldCollection,HistoryWritings,1000.0,0.0,True,True,,
HouseHoldCollection,HistoryWritings,1000.0,0.0,True,False,Genuine,Report #4624
HouseHoldCollection,HistoryWritings,1000.0,0.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,0.0,True,False,Fake,Report #4625
HouseHoldCollection,HistoryWritings,1000.0,0.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,0.0,True,False,,Report #4626
HouseHoldCollection,HistoryWritings,1000.0,0.0,True,False,,
HouseHoldCollection,HistoryWritings,1000.0,0.0,False,True,Genuine,Report #4627
HouseHoldCollection,HistoryWritings,1000.0,0.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,0.0,False,True,Fake,Report #4628
HouseHoldCollection,HistoryWritings,1000.0,0.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,0.0,False,True,,Report #4629
HouseHoldCollection,HistoryWritings,1000.0,0.0,False,True,,
HouseHoldCollection,HistoryWritings,1000.0,0.0,False,False,Genuine,Report #4630
HouseHoldCollection,HistoryWritings,1000.0,0.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,0.0,False,False,Fake,Report #4631
HouseHoldCollection,HistoryWritings,1000.0,0.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,0.0,False,False,,Report #4632
HouseHoldCollection,HistoryWritings,1000.0,0.0,False,False,,
HouseHoldCollection,HistoryWritings,1000.0,19.0,True,True,Genuine,Report #4633
HouseHoldCollection,HistoryWritings,1000.0,19.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,19.0,True,True,Fake,Report #4634
HouseHoldCollection,HistoryWritings,1000.0,19.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,19.0,True,True,,Report #4635
HouseHoldCollection,HistoryWritings,1000.0,19.0,True,True,,
HouseHoldCollection,HistoryWritings,1000.0,19.0,True,False,Genuine,Report #4636
HouseHoldCollection,HistoryWritings,1000.0,19.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,19.0,True,False,Fake,Report #4637
HouseHoldCollection,HistoryWritings,1000.0,19.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,19.0,True,False,,Report #4638
HouseHoldCollection,HistoryWritings,1000.0,19.0,True,False,,
HouseHoldCollection,HistoryWritings,1000.0,19.0,False,True,Genuine,Report #4639
HouseHoldCollection,HistoryWritings,1000.0,19.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,19.0,False,True,Fake,Report #4640
HouseHoldCollection,HistoryWritings,1000.0,19.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,19.0,False,True,,Report #4641
HouseHoldCollection,HistoryWritings,1000.0,19.0,False,True,,
HouseHoldCollection,HistoryWritings,1000.0,19.0,False,False,Genuine,Report #4642
HouseHoldCollection,HistoryWritings,1000.0,19.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,19.0,False,False,Fake,Report #4643
HouseHoldCollection,HistoryWritings,1000.0,19.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,19.0,False,False,,Report #4644
HouseHoldCollection,HistoryWritings,1000.0,19.0,False,False,,
HouseHoldCollection,HistoryWritings,1000.0,20.0,True,True,Genuine,Report #4645
HouseHoldCollection,HistoryWritings,1000.0,20.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,20.0,True,True,Fake,Report #4646
HouseHoldCollection,HistoryWritings,1000.0,20.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,20.0,True,True,,Report #4647
HouseHoldCollection,HistoryWritings,1000.0,20.0,True,True,,
HouseHoldCollection,HistoryWritings,1000.0,20.0,True,False,Genuine,Report #4648
HouseHoldCollection,HistoryWritings,1000.0,20.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,20.0,True,False,Fake,Report #4649
HouseHoldCollection,HistoryWritings,1000.0,20.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,20.0,True,False,,Report #4650
HouseHoldCollection,HistoryWritings,1000.0,20.0,True,False,,
HouseHoldCollection,HistoryWritings,1000.0,20.0,False,True,Genuine,Report #4651
HouseHoldCollection,HistoryWritings,1000.0,20.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,20.0,False,True,Fake,Report #4652
HouseHoldCollection,HistoryWritings,1000.0,20.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,20.0,False,True,,Report #4653
HouseHoldCollection,HistoryWritings,1000.0,20.0,False,True,,
HouseHoldCollection,HistoryWritings,1000.0,20.0,False,False,Genuine,Report #4654
HouseHoldCollection,HistoryWritings,1000.0,20.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,20.0,False,False,Fake,Report #4655
HouseHoldCollection,HistoryWritings,1000.0,20.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,20.0,False,False,,Report #4656
HouseHoldCollection,HistoryWritings,1000.0,20.0,False,False,,
HouseHoldCollection,HistoryWritings,1000.0,21.0,True,True,Genuine,Report #4657
HouseHoldCollection,HistoryWritings,1000.0,21.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,21.0,True,True,Fake,Report #4658
HouseHoldCollection,HistoryWritings,1000.0,21.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,21.0,True,True,,Report #4659
HouseHoldCollection,HistoryWritings,1000.0,21.0,True,True,,
HouseHoldCollection,HistoryWritings,1000.0,21.0,True,False,Genuine,Report #4660
HouseHoldCollection,HistoryWritings,1000.0,21.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,21.0,True,False,Fake,Report #4661
HouseHoldCollection,HistoryWritings,1000.0,21.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,21.0,True,False,,Report #4662
HouseHoldCollection,HistoryWritings,1000.0,21.0,True,False,,
HouseHoldCollection,HistoryWritings,1000.0,21.0,False,True,Genuine,Report #4663
HouseHoldCollection,HistoryWritings,1000.0,21.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,21.0,False,True,Fake,Report #4664
HouseHoldCollection,HistoryWritings,1000.0,21.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,21.0,False,True,,Report #4665
HouseHoldCollection,HistoryWritings,1000.0,21.0,False,True,,
HouseHoldCollection,HistoryWritings,1000.0,21.0,False,False,Genuine,Report #4666
HouseHoldCollection,HistoryWritings,1000.0,21.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,21.0,False,False,Fake,Report #4667
HouseHoldCollection,HistoryWritings,1000.0,21.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,21.0,False,False,,Report #4668
HouseHoldCollection,HistoryWritings,1000.0,21.0,False,False,,
HouseHoldCollection,HistoryWritings,1000.0,99.0,True,True,Genuine,Report #4669
HouseHoldCollection,HistoryWritings,1000.0,99.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,99.0,True,True,Fake,Report #4670
HouseHoldCollection,HistoryWritings,1000.0,99.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,99.0,True,True,,Report #4671
HouseHoldCollection,HistoryWritings,1000.0,99.0,True,True,,
HouseHoldCollection,HistoryWritings,1000.0,99.0,True,False,Genuine,Report #4672
HouseHoldCollection,HistoryWritings,1000.0,99.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,99.0,True,False,Fake,Report #4673
HouseHoldCollection,HistoryWritings,1000.0,99.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,99.0,True,False,,Report #4674
HouseHoldCollection,HistoryWritings,1000.0,99.0,True,False,,
HouseHoldCollection,HistoryWritings,1000.0,99.0,False,True,Genuine,Report #4675
HouseHoldCollection,HistoryWritings,1000.0,99.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,99.0,False,True,Fake,Report #4676
HouseHoldCollection,HistoryWritings,1000.0,99.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,99.0,False,True,,Report #4677
HouseHoldCollection,HistoryWritings,1000.0,99.0,False,True,,
HouseHoldCollection,HistoryWritings,1000.0,99.0,False,False,Genuine,Report #4678
HouseHoldCollection,HistoryWritings,1000.0,99.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,99.0,False,False,Fake,Report #4679
HouseHoldCollection,HistoryWritings,1000.0,99.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,99.0,False,False,,Report #4680
HouseHoldCollection,HistoryWritings,1000.0,99.0,False,False,,
HouseHoldCollection,HistoryWritings,1000.0,100.0,True,True,Genuine,Report #4681
HouseHoldCollection,HistoryWritings,1000.0,100.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,100.0,True,True,Fake,Report #4682
HouseHoldCollection,HistoryWritings,1000.0,100.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,100.0,True,True,,Report #4683
HouseHoldCollection,HistoryWritings,1000.0,100.0,True,True,,
HouseHoldCollection,HistoryWritings,1000.0,100.0,True,False,Genuine,Report #4684
HouseHoldCollection,HistoryWritings,1000.0,100.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,100.0,True,False,Fake,Report #4685
HouseHoldCollection,HistoryWritings,1000.0,100.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,100.0,True,False,,Report #4686
HouseHoldCollection,HistoryWritings,1000.0,100.0,True,False,,
HouseHoldCollection,HistoryWritings,1000.0,100.0,False,True,Genuine,Report #4687
HouseHoldCollection,HistoryWritings,1000.0,100.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,100.0,False,True,Fake,Report #4688
HouseHoldCollection,HistoryWritings,1000.0,100.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,100.0,False,True,,Report #4689
HouseHoldCollection,HistoryWritings,1000.0,100.0,False,True,,
HouseHoldCollection,HistoryWritings,1000.0,100.0,False,False,Genuine,Report #4690
HouseHoldCollection,HistoryWritings,1000.0,100.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,100.0,False,False,Fake,Report #4691
HouseHoldCollection,HistoryWritings,1000.0,100.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,100.0,False,False,,Report #4692
HouseHoldCollection,HistoryWritings,1000.0,100.0,False,False,,
HouseHoldCollection,HistoryWritings,1000.0,101.0,True,True,Genuine,Report #4693
HouseHoldCollection,HistoryWritings,1000.0,101.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,101.0,True,True,Fake,Report #4694
HouseHoldCollection,HistoryWritings,1000.0,101.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,101.0,True,True,,Report #4695
HouseHoldCollection,HistoryWritings,1000.0,101.0,True,True,,
HouseHoldCollection,HistoryWritings,1000.0,101.0,True,False,Genuine,Report #4696
HouseHoldCollection,HistoryWritings,1000.0,101.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,101.0,True,False,Fake,Report #4697
HouseHoldCollection,HistoryWritings,1000.0,101.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,101.0,True,False,,Report #4698
HouseHoldCollection,HistoryWritings,1000.0,101.0,True,False,,
HouseHoldCollection,HistoryWritings,1000.0,101.0,False,True,Genuine,Report #4699
HouseHoldCollection,HistoryWritings,1000.0,101.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,101.0,False,True,Fake,Report #4700
HouseHoldCollection,HistoryWritings,1000.0,101.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000.0,101.0,False,True,,Report #4701
HouseHoldCollection,HistoryWritings,1000.0,101.0,False,True,,
HouseHoldCollection,HistoryWritings,1000.0,101.0,False,False,Genuine,Report #4702
HouseHoldCollection,HistoryWritings,1000.0,101.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000.0,101.0,False,False,Fake,Report #4703
HouseHoldCollection,HistoryWritings,1000.0,101.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000.0,101.0,False,False,,Report #4704
HouseHoldCollection,HistoryWritings,1000.0,101.0,False,False,,
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,True,True,Genuine,Report #4705
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,True,True,Fake,Report #4706
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,True,True,,Report #4707
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,True,True,,
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,True,False,Genuine,Report #4708
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,True,False,Fake,Report #4709
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,True,False,,Report #4710
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,True,False,,
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,False,True,Genuine,Report #4711
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,False,True,Fake,Report #4712
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,False,True,,Report #4713
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,False,True,,
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,False,False,Genuine,Report #4714
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,False,False,Fake,Report #4715
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,False,False,,Report #4716
HouseHoldCollection,HistoryWritings,1000000.0,-1.0,False,False,,
HouseHoldCollection,HistoryWritings,1000000.0,0.0,True,True,Genuine,Report #4717
HouseHoldCollection,HistoryWritings,1000000.0,0.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,0.0,True,True,Fake,Report #4718
HouseHoldCollection,HistoryWritings,1000000.0,0.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,0.0,True,True,,Report #4719
HouseHoldCollection,HistoryWritings,1000000.0,0.0,True,True,,
HouseHoldCollection,HistoryWritings,1000000.0,0.0,True,False,Genuine,Report #4720
HouseHoldCollection,HistoryWritings,1000000.0,0.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,0.0,True,False,Fake,Report #4721
HouseHoldCollection,HistoryWritings,1000000.0,0.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,0.0,True,False,,Report #4722
HouseHoldCollection,HistoryWritings,1000000.0,0.0,True,False,,
HouseHoldCollection,HistoryWritings,1000000.0,0.0,False,True,Genuine,Report #4723
HouseHoldCollection,HistoryWritings,1000000.0,0.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,0.0,False,True,Fake,Report #4724
HouseHoldCollection,HistoryWritings,1000000.0,0.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,0.0,False,True,,Report #4725
HouseHoldCollection,HistoryWritings,1000000.0,0.0,False,True,,
HouseHoldCollection,HistoryWritings,1000000.0,0.0,False,False,Genuine,Report #4726
HouseHoldCollection,HistoryWritings,1000000.0,0.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,0.0,False,False,Fake,Report #4727
HouseHoldCollection,HistoryWritings,1000000.0,0.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,0.0,False,False,,Report #4728
HouseHoldCollection,HistoryWritings,1000000.0,0.0,False,False,,
HouseHoldCollection,HistoryWritings,1000000.0,19.0,True,True,Genuine,Report #4729
HouseHoldCollection,HistoryWritings,1000000.0,19.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,19.0,True,True,Fake,Report #4730
HouseHoldCollection,HistoryWritings,1000000.0,19.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,19.0,True,True,,Report #4731
HouseHoldCollection,HistoryWritings,1000000.0,19.0,True,True,,
HouseHoldCollection,HistoryWritings,1000000.0,19.0,True,False,Genuine,Report #4732
HouseHoldCollection,HistoryWritings,1000000.0,19.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,19.0,True,False,Fake,Report #4733
HouseHoldCollection,HistoryWritings,1000000.0,19.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,19.0,True,False,,Report #4734
HouseHoldCollection,HistoryWritings,1000000.0,19.0,True,False,,
HouseHoldCollection,HistoryWritings,1000000.0,19.0,False,True,Genuine,Report #4735
HouseHoldCollection,HistoryWritings,1000000.0,19.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,19.0,False,True,Fake,Report #4736
HouseHoldCollection,HistoryWritings,1000000.0,19.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,19.0,False,True,,Report #4737
HouseHoldCollection,HistoryWritings,1000000.0,19.0,False,True,,
HouseHoldCollection,HistoryWritings,1000000.0,19.0,False,False,Genuine,Report #4738
HouseHoldCollection,HistoryWritings,1000000.0,19.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,19.0,False,False,Fake,Report #4739
HouseHoldCollection,HistoryWritings,1000000.0,19.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,19.0,False,False,,Report #4740
HouseHoldCollection,HistoryWritings,1000000.0,19.0,False,False,,
HouseHoldCollection,HistoryWritings,1000000.0,20.0,True,True,Genuine,Report #4741
HouseHoldCollection,HistoryWritings,1000000.0,20.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,20.0,True,True,Fake,Report #4742
HouseHoldCollection,HistoryWritings,1000000.0,20.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,20.0,True,True,,Report #4743
HouseHoldCollection,HistoryWritings,1000000.0,20.0,True,True,,
HouseHoldCollection,HistoryWritings,1000000.0,20.0,True,False,Genuine,Report #4744
HouseHoldCollection,HistoryWritings,1000000.0,20.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,20.0,True,False,Fake,Report #4745
HouseHoldCollection,HistoryWritings,1000000.0,20.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,20.0,True,False,,Report #4746
HouseHoldCollection,HistoryWritings,1000000.0,20.0,True,False,,
HouseHoldCollection,HistoryWritings,1000000.0,20.0,False,True,Genuine,Report #4747
HouseHoldCollection,HistoryWritings,1000000.0,20.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,20.0,False,True,Fake,Report #4748
HouseHoldCollection,HistoryWritings,1000000.0,20.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,20.0,False,True,,Report #4749
HouseHoldCollection,HistoryWritings,1000000.0,20.0,False,True,,
HouseHoldCollection,HistoryWritings,1000000.0,20.0,False,False,Genuine,Report #4750
HouseHoldCollection,HistoryWritings,1000000.0,20.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,20.0,False,False,Fake,Report #4751
HouseHoldCollection,HistoryWritings,1000000.0,20.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,20.0,False,False,,Report #4752
HouseHoldCollection,HistoryWritings,1000000.0,20.0,False,False,,
HouseHoldCollection,HistoryWritings,1000000.0,21.0,True,True,Genuine,Report #4753
HouseHoldCollection,HistoryWritings,1000000.0,21.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,21.0,True,True,Fake,Report #4754
HouseHoldCollection,HistoryWritings,1000000.0,21.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,21.0,True,True,,Report #4755
HouseHoldCollection,HistoryWritings,1000000.0,21.0,True,True,,
HouseHoldCollection,HistoryWritings,1000000.0,21.0,True,False,Genuine,Report #4756
HouseHoldCollection,HistoryWritings,1000000.0,21.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,21.0,True,False,Fake,Report #4757
HouseHoldCollection,HistoryWritings,1000000.0,21.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,21.0,True,False,,Report #4758
HouseHoldCollection,HistoryWritings,1000000.0,21.0,True,False,,
HouseHoldCollection,HistoryWritings,1000000.0,21.0,False,True,Genuine,Report #4759
HouseHoldCollection,HistoryWritings,1000000.0,21.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,21.0,False,True,Fake,Report #4760
HouseHoldCollection,HistoryWritings,1000000.0,21.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,21.0,False,True,,Report #4761
HouseHoldCollection,HistoryWritings,1000000.0,21.0,False,True,,
HouseHoldCollection,HistoryWritings,1000000.0,21.0,False,False,Genuine,Report #4762
HouseHoldCollection,HistoryWritings,1000000.0,21.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,21.0,False,False,Fake,Report #4763
HouseHoldCollection,HistoryWritings,1000000.0,21.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,21.0,False,False,,Report #4764
HouseHoldCollection,HistoryWritings,1000000.0,21.0,False,False,,
HouseHoldCollection,HistoryWritings,1000000.0,99.0,True,True,Genuine,Report #4765
HouseHoldCollection,HistoryWritings,1000000.0,99.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,99.0,True,True,Fake,Report #4766
HouseHoldCollection,HistoryWritings,1000000.0,99.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,99.0,True,True,,Report #4767
HouseHoldCollection,HistoryWritings,1000000.0,99.0,True,True,,
HouseHoldCollection,HistoryWritings,1000000.0,99.0,True,False,Genuine,Report #4768
HouseHoldCollection,HistoryWritings,1000000.0,99.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,99.0,True,False,Fake,Report #4769
HouseHoldCollection,HistoryWritings,1000000.0,99.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,99.0,True,False,,Report #4770
HouseHoldCollection,HistoryWritings,1000000.0,99.0,True,False,,
HouseHoldCollection,HistoryWritings,1000000.0,99.0,False,True,Genuine,Report #4771
HouseHoldCollection,HistoryWritings,1000000.0,99.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,99.0,False,True,Fake,Report #4772
HouseHoldCollection,HistoryWritings,1000000.0,99.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,99.0,False,True,,Report #4773
HouseHoldCollection,HistoryWritings,1000000.0,99.0,False,True,,
HouseHoldCollection,HistoryWritings,1000000.0,99.0,False,False,Genuine,Report #4774
HouseHoldCollection,HistoryWritings,1000000.0,99.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,99.0,False,False,Fake,Report #4775
HouseHoldCollection,HistoryWritings,1000000.0,99.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,99.0,False,False,,Report #4776
HouseHoldCollection,HistoryWritings,1000000.0,99.0,False,False,,
HouseHoldCollection,HistoryWritings,1000000.0,100.0,True,True,Genuine,Report #4777
HouseHoldCollection,HistoryWritings,1000000.0,100.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,100.0,True,True,Fake,Report #4778
HouseHoldCollection,HistoryWritings,1000000.0,100.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,100.0,True,True,,Report #4779
HouseHoldCollection,HistoryWritings,1000000.0,100.0,True,True,,
HouseHoldCollection,HistoryWritings,1000000.0,100.0,True,False,Genuine,Report #4780
HouseHoldCollection,HistoryWritings,1000000.0,100.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,100.0,True,False,Fake,Report #4781
HouseHoldCollection,HistoryWritings,1000000.0,100.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,100.0,True,False,,Report #4782
HouseHoldCollection,HistoryWritings,1000000.0,100.0,True,False,,
HouseHoldCollection,HistoryWritings,1000000.0,100.0,False,True,Genuine,Report #4783
HouseHoldCollection,HistoryWritings,1000000.0,100.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,100.0,False,True,Fake,Report #4784
HouseHoldCollection,HistoryWritings,1000000.0,100.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,100.0,False,True,,Report #4785
HouseHoldCollection,HistoryWritings,1000000.0,100.0,False,True,,
HouseHoldCollection,HistoryWritings,1000000.0,100.0,False,False,Genuine,Report #4786
HouseHoldCollection,HistoryWritings,1000000.0,100.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,100.0,False,False,Fake,Report #4787
HouseHoldCollection,HistoryWritings,1000000.0,100.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,100.0,False,False,,Report #4788
HouseHoldCollection,HistoryWritings,1000000.0,100.0,False,False,,
HouseHoldCollection,HistoryWritings,1000000.0,101.0,True,True,Genuine,Report #4789
HouseHoldCollection,HistoryWritings,1000000.0,101.0,True,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,101.0,True,True,Fake,Report #4790
HouseHoldCollection,HistoryWritings,1000000.0,101.0,True,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,101.0,True,True,,Report #4791
HouseHoldCollection,HistoryWritings,1000000.0,101.0,True,True,,
HouseHoldCollection,HistoryWritings,1000000.0,101.0,True,False,Genuine,Report #4792
HouseHoldCollection,HistoryWritings,1000000.0,101.0,True,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,101.0,True,False,Fake,Report #4793
HouseHoldCollection,HistoryWritings,1000000.0,101.0,True,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,101.0,True,False,,Report #4794
HouseHoldCollection,HistoryWritings,1000000.0,101.0,True,False,,
HouseHoldCollection,HistoryWritings,1000000.0,101.0,False,True,Genuine,Report #4795
HouseHoldCollection,HistoryWritings,1000000.0,101.0,False,True,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,101.0,False,True,Fake,Report #4796
HouseHoldCollection,HistoryWritings,1000000.0,101.0,False,True,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,101.0,False,True,,Report #4797
HouseHoldCollection,HistoryWritings,1000000.0,101.0,False,True,,
HouseHoldCollection,HistoryWritings,1000000.0,101.0,False,False,Genuine,Report #4798
HouseHoldCollection,HistoryWritings,1000000.0,101.0,False,False,Genuine,
HouseHoldCollection,HistoryWritings,1000000.0,101.0,False,False,Fake,Report #4799
HouseHoldCollection,HistoryWritings,1000000.0,101.0,False,False,Fake,
HouseHoldCollection,HistoryWritings,1000000.0,101.0,False,False,,Report #4800
HouseHoldCollection,HistoryWritings,1000000.0,101.0,False,False,,
SimpleToolCollection,Paintings,0.0,-1.0,True,True,Genuine,Report #4801
SimpleToolCollection,Paintings,0.0,-1.0,True,True,Genuine,
SimpleToolCollection,Paintings,0.0,-1.0,True,True,Fake,Report #4802
SimpleToolCollection,Paintings,0.0,-1.0,True,True,Fake,
SimpleToolCollection,Paintings,0.0,-1.0,True,True,,Report #4803
SimpleToolCollection,Paintings,0.0,-1.0,True,True,,
SimpleToolCollection,Paintings,0.0,-1.0,True,False,Genuine,Report #4804
SimpleToolCollection,Paintings,0.0,-1.0,True,False,Genuine,
SimpleToolCollection,Paintings,0.0,-1.0,True,False,Fake,Report #4805
SimpleToolCollection,Paintings,0.0,-1.0,True,False,Fake,
SimpleToolCollection,Paintings,0.0,-1.0,True,False,,Report #4806
SimpleToolCollection,Paintings,0.0,-1.0,True,False,,
SimpleToolCollection,Paintings,0.0,-1.0,False,True,Genuine,Report #4807
SimpleToolCollection,Paintings,0.0,-1.0,False,True,Genuine,
SimpleToolCollection,Paintings,0.0,-1.0,False,True,Fake,Report #4808
SimpleToolCollection,Paintings,0.0,-1.0,False,True,Fake,
SimpleToolCollection,Paintings,0.0,-1.0,False,True,,Report #4809
SimpleToolCollection,Paintings,0.0,-1.0,False,True,,
SimpleToolCollection,Paintings,0.0,-1.0,False,False,Genuine,Report #4810
SimpleToolCollection,Paintings,0.0,-1.0,False,False,Genuine,
SimpleToolCollection,Paintings,0.0,-1.0,False,False,Fake,Report #4811
SimpleToolCollection,Paintings,0.0,-1.0,False,False,Fake,
SimpleToolCollection,Paintings,0.0,-1.0,False,False,,Report #4812
SimpleToolCollection,Paintings,0.0,-1.0,False,False,,
SimpleToolCollection,Paintings,0.0,0.0,True,True,Genuine,Report #4813
SimpleToolCollection,Paintings,0.0,0.0,True,True,Genuine,
SimpleToolCollection,Paintings,0.0,0.0,True,True,Fake,Report #4814
SimpleToolCollection,Paintings,0.0,0.0,True,True,Fake,
SimpleToolCollection,Paintings,0.0,0.0,True,True,,Report #4815
SimpleToolCollection,Paintings,0.0,0.0,True,True,,
SimpleToolCollection,Paintings,0.0,0.0,True,False,Genuine,Report #4816
SimpleToolCollection,Paintings,0.0,0.0,True,False,Genuine,
SimpleToolCollection,Paintings,0.0,0.0,True,False,Fake,Report #4817
SimpleToolCollection,Paintings,0.0,0.0,True,False,Fake,
SimpleToolCollection,Paintings,0.0,0.0,True,False,,Report #4818
SimpleToolCollection,Paintings,0.0,0.0,True,False,,
SimpleToolCollection,Paintings,0.0,0.0,False,True,Genuine,Report #4819
SimpleToolCollection,Paintings,0.0,0.0,False,True,Genuine,
SimpleToolCollection,Paintings,0.0,0.0,False,True,Fake,Report #4820
SimpleToolCollection,Paintings,0.0,0.0,False,True,Fake,
SimpleToolCollection,Paintings,0.0,0.0,False,True,,Report #4821
SimpleToolCollection,Paintings,0.0,0.0,False,True,,
SimpleToolCollection,Paintings,0.0,0.0,False,False,Genuine,Report #4822
SimpleToolCollection,Paintings,0.0,0.0,False,False,Genuine,
SimpleToolCollection,Paintings,0.0,0.0,False,False,Fake,Report #4823
SimpleToolCollection,Paintings,0.0,0.0,False,False,Fake,
SimpleToolCollection,Paintings,0.0,0.0,False,False,,Report #4824
SimpleToolCollection,Paintings,0.0,0.0,False,False,,
SimpleToolCollection,Paintings,0.0,19.0,True,True,Genuine,Report #4825
SimpleToolCollection,Paintings,0.0,19.0,True,True,Genuine,
SimpleToolCollection,Paintings,0.0,19.0,True,True,Fake,Report #4826
SimpleToolCollection,Paintings,0.0,19.0,True,True,Fake,
SimpleToolCollection,Paintings,0.0,19.0,True,True,,Report #4827
SimpleToolCollection,Paintings,0.0,19.0,True,True,,
SimpleToolCollection,Paintings,0.0,19.0,True,False,Genuine,Report #4828
SimpleToolCollection,Paintings,0.0,19.0,True,False,Genuine,
SimpleToolCollection,Paintings,0.0,19.0,True,False,Fake,Report #4829
SimpleToolCollection,Paintings,0.0,19.0,True,False,Fake,
SimpleToolCollection,Paintings,0.0,19.0,True,False,,Report #4830
SimpleToolCollection,Paintings,0.0,19.0,True,False,,
SimpleToolCollection,Paintings,0.0,19.0,False,True,Genuine,Report #4831
SimpleToolCollection,Paintings,0.0,19.0,False,True,Genuine,
SimpleToolCollection,Paintings,0.0,19.0,False,True,Fake,Report #4832
SimpleToolCollection,Paintings,0.0,19.0,False,True,Fake,
SimpleToolCollection,Paintings,0.0,19.0,False,True,,Report #4833
SimpleToolCollection,Paintings,0.0,19.0,False,True,,
SimpleToolCollection,Paintings,0.0,19.0,False,False,Genuine,Report #4834
SimpleToolCollection,Paintings,0.0,19.0,False,False,Genuine,
SimpleToolCollection,Paintings,0.0,19.0,False,False,Fake,Report #4835
SimpleToolCollection,Paintings,0.0,19.0,False,False,Fake,
SimpleToolCollection,Paintings,0.0,19.0,False,False,,Report #4836
SimpleToolCollection,Paintings,0.0,19.0,False,False,,
SimpleToolCollection,Paintings,0.0,20.0,True,True,Genuine,Report #4837
SimpleToolCollection,Paintings,0.0,20.0,True,True,Genuine,
SimpleToolCollection,Paintings,0.0,20.0,True,True,Fake,Report #4838
SimpleToolCollection,Paintings,0.0,20.0,True,True,Fake,
SimpleToolCollection,Paintings,0.0,20.0,True,True,,Report #4839
SimpleToolCollection,Paintings,0.0,20.0,True,True,,
SimpleToolCollection,Paintings,0.0,20.0,True,False,Genuine,Report #4840
SimpleToolCollection,Paintings,0.0,20.0,True,False,Genuine,
SimpleToolCollection,Paintings,0.0,20.0,True,False,Fake,Report #4841
SimpleToolCollection,Paintings,0.0,20.0,True,False,Fake,
SimpleToolCollection,Paintings,0.0,20.0,True,False,,Report #4842
SimpleToolCollection,Paintings,0.0,20.0,True,False,,
SimpleToolCollection,Paintings,0.0,20.0,False,True,Genuine,Report #4843
SimpleToolCollection,Paintings,0.0,20.0,False,True,Genuine,
SimpleToolCollection,Paintings,0.0,20.0,False,True,Fake,Report #4844
SimpleToolCollection,Paintings,0.0,20.0,False,True,Fake,
SimpleToolCollection,Paintings,0.0,20.0,False,True,,Report #4845
SimpleToolCollection,Paintings,0.0,20.0,False,True,,
SimpleToolCollection,Paintings,0.0,20.0,False,False,Genuine,Report #4846
SimpleToolCollection,Paintings,0.0,20.0,False,False,Genuine,
SimpleToolCollection,Paintings,0.0,20.0,False,False,Fake,Report #4847
SimpleToolCollection,Paintings,0.0,20.0,False,False,Fake,
SimpleToolCollection,Paintings,0.0,20.0,False,False,,Report #4848
SimpleToolCollection,Paintings,0.0,20.0,False,False,,
SimpleToolCollection,Paintings,0.0,21.0,True,True,Genuine,Report #4849
SimpleToolCollection,Paintings,0.0,21.0,True,True,Genuine,
SimpleToolCollection,Paintings,0.0,21.0,True,True,Fake,Report #4850
SimpleToolCollection,Paintings,0.0,21.0,True,True,Fake,
SimpleToolCollection,Paintings,0.0,21.0,True,True,,Report #4851
SimpleToolCollection,Paintings,0.0,21.0,True,True,,
SimpleToolCollection,Paintings,0.0,21.0,True,False,Genuine,Report #4852
SimpleToolCollection,Paintings,0.0,21.0,True,False,Genuine,
SimpleToolCollection,Paintings,0.0,21.0,True,False,Fake,Report #4853
SimpleToolCollection,Paintings,0.0,21.0,True,False,Fake,
SimpleToolCollection,Paintings,0.0,21.0,True,False,,Report #4854
SimpleToolCollection,Paintings,0.0,21.0,True,False,,
SimpleToolCollection,Paintings,0.0,21.0,False,True,Genuine,Report #4855
SimpleToolCollection,Paintings,0.0,21.0,False,True,Genuine,
SimpleToolCollection,Paintings,0.0,21.0,False,True,Fake,Report #4856
SimpleToolCollection,Paintings,0.0,21.0,False,True,Fake,
SimpleToolCollection,Paintings,0.0,21.0,False,True,,Report #4857
SimpleToolCollection,Paintings,0.0,21.0,False,True,,
SimpleToolCollection,Paintings,0.0,21.0,False,False,Genuine,Report #4858
SimpleToolCollection,Paintings,0.0,21.0,False,False,Genuine,
SimpleToolCollection,Paintings,0.0,21.0,False,False,Fake,Report #4859
SimpleToolCollection,Paintings,0.0,21.0,False,False,Fake,
SimpleToolCollection,Paintings,0.0,21.0,False,False,,Report #4860
SimpleToolCollection,Paintings,0.0,21.0,False,False,,
SimpleToolCollection,Paintings,0.0,99.0,True,True,Genuine,Report #4861
SimpleToolCollection,Paintings,0.0,99.0,True,True,Genuine,
SimpleToolCollection,Paintings,0.0,99.0,True,True,Fake,Report #4862
SimpleToolCollection,Paintings,0.0,99.0,True,True,Fake,
SimpleToolCollection,Paintings,0.0,99.0,True,True,,Report #4863
SimpleToolCollection,Paintings,0.0,99.0,True,True,,
SimpleToolCollection,Paintings,0.0,99.0,True,False,Genuine,Report #4864
SimpleToolCollection,Paintings,0.0,99.0,True,False,Genuine,
SimpleToolCollection,Paintings,0.0,99.0,True,False,Fake,Report #4865
SimpleToolCollection,Paintings,0.0,99.0,True,False,Fake,
SimpleToolCollection,Paintings,0.0,99.0,True,False,,Report #4866
SimpleToolCollection,Paintings,0.0,99.0,True,False,,
SimpleToolCollection,Paintings,0.0,99.0,False,True,Genuine,Report #4867
SimpleToolCollection,Paintings,0.0,99.0,False,True,Genuine,
SimpleToolCollection,Paintings,0.0,99.0,False,True,Fake,Report #4868
SimpleToolCollection,Paintings,0.0,99.0,False,True,Fake,
SimpleToolCollection,Paintings,0.0,99.0,False,True,,Report #4869
SimpleToolCollection,Paintings,0.0,99.0,False,True,,
SimpleToolCollection,Paintings,0.0,99.0,False,False,Genuine,Report #4870
SimpleToolCollection,Paintings,0.0,99.0,False,False,Genuine,
SimpleToolCollection,Paintings,0.0,99.0,False,False,Fake,Report #4871
SimpleToolCollection,Paintings,0.0,99.0,False,False,Fake,
SimpleToolCollection,Paintings,0.0,99.0,False,False,,Report #4872
SimpleToolCollection,Paintings,0.0,99.0,False,False,,
SimpleToolCollection,Paintings,0.0,100.0,True,True,Genuine,Report #4873
SimpleToolCollection,Paintings,0.0,100.0,True,True,Genuine,
SimpleToolCollection,Paintings,0.0,100.0,True,True,Fake,Report #4874
SimpleToolCollection,Paintings,0.0,100.0,True,True,Fake,
SimpleToolCollection,Paintings,0.0,100.0,True,True,,Report #4875
SimpleToolCollection,Paintings,0.0,100.0,True,True,,
SimpleToolCollection,Paintings,0.0,100.0,True,False,Genuine,Report #4876
SimpleToolCollection,Paintings,0.0,100.0,True,False,Genuine,
SimpleToolCollection,Paintings,0.0,100.0,True,False,Fake,Report #4877
SimpleToolCollection,Paintings,0.0,100.0,True,False,Fake,
SimpleToolCollection,Paintings,0.0,100.0,True,False,,Report #4878
SimpleToolCollection,Paintings,0.0,100.0,True,False,,
SimpleToolCollection,Paintings,0.0,100.0,False,True,Genuine,Report #4879
SimpleToolCollection,Paintings,0.0,100.0,False,True,Genuine,
SimpleToolCollection,Paintings,0.0,100.0,False,True,Fake,Report #4880
SimpleToolCollection,Paintings,0.0,100.0,False,True,Fake,
SimpleToolCollection,Paintings,0.0,100.0,False,True,,Report #4881
SimpleToolCollection,Paintings,0.0,100.0,False,True,,
SimpleToolCollection,Paintings,0.0,100.0,False,False,Genuine,Report #4882
SimpleToolCollection,Paintings,0.0,100.0,False,False,Genuine,
SimpleToolCollection,Paintings,0.0,100.0,False,False,Fake,Report #4883
SimpleToolCollection,Paintings,0.0,100.0,False,False,Fake,
SimpleToolCollection,Paintings,0.0,100.0,False,False,,Report #4884
SimpleToolCollection,Paintings,0.0,100.0,False,False,,
SimpleToolCollection,Paintings,0.0,101.0,True,True,Genuine,Report #4885
SimpleToolCollection,Paintings,0.0,101.0,True,True,Genuine,
SimpleToolCollection,Paintings,0.0,101.0,True,True,Fake,Report #4886
SimpleToolCollection,Paintings,0.0,101.0,True,True,Fake,
SimpleToolCollection,Paintings,0.0,101.0,True,True,,Report #4887
SimpleToolCollection,Paintings,0.0,101.0,True,True,,
SimpleToolCollection,Paintings,0.0,101.0,True,False,Genuine,Report #4888
SimpleToolCollection,Paintings,0.0,101.0,True,False,Genuine,
SimpleToolCollection,Paintings,0.0,101.0,True,False,Fake,Report #4889
SimpleToolCollection,Paintings,0.0,101.0,True,False,Fake,
SimpleToolCollection,Paintings,0.0,101.0,True,False,,Report #4890
SimpleToolCollection,Paintings,0.0,101.0,True,False,,
SimpleToolCollection,Paintings,0.0,101.0,False,True,Genuine,Report #4891
SimpleToolCollection,Paintings,0.0,101.0,False,True,Genuine,
SimpleToolCollection,Paintings,0.0,101.0,False,True,Fake,Report #4892
SimpleToolCollection,Paintings,0.0,101.0,False,True,Fake,
SimpleToolCollection,Paintings,0.0,101.0,False,True,,Report #4893
SimpleToolCollection,Paintings,0.0,101.0,False,True,,
SimpleToolCollection,Paintings,0.0,101.0,False,False,Genuine,Report #4894
SimpleToolCollection,Paintings,0.0,101.0,False,False,Genuine,
SimpleToolCollection,Paintings,0.0,101.0,False,False,Fake,Report #4895
SimpleToolCollection,Paintings,0.0,101.0,False,False,Fake,
SimpleToolCollection,Paintings,0.0,101.0,False,False,,Report #4896
SimpleToolCollection,Paintings,0.0,101.0,False,False,,
SimpleToolCollection,Paintings,1.0,-1.0,True,True,Genuine,Report #4897
SimpleToolCollection,Paintings,1.0,-1.0,True,True,Genuine,
SimpleToolCollection,Paintings,1.0,-1.0,True,True,Fake,Report #4898
SimpleToolCollection,Paintings,1.0,-1.0,True,True,Fake,
SimpleToolCollection,Paintings,1.0,-1.0,True,True,,Report #4899
SimpleToolCollection,Paintings,1.0,-1.0,True,True,,
SimpleToolCollection,Paintings,1.0,-1.0,True,False,Genuine,Report #4900
SimpleToolCollection,Paintings,1.0,-1.0,True,False,Genuine,
SimpleToolCollection,Paintings,1.0,-1.0,True,False,Fake,Report #4901
SimpleToolCollection,Paintings,1.0,-1.0,True,False,Fake,
SimpleToolCollection,Paintings,1.0,-1.0,True,False,,Report #4902
SimpleToolCollection,Paintings,1.0,-1.0,True,False,,
SimpleToolCollection,Paintings,1.0,-1.0,False,True,Genuine,Report #4903
SimpleToolCollection,Paintings,1.0,-1.0,False,True,Genuine,
SimpleToolCollection,Paintings,1.0,-1.0,False,True,Fake,Report #4904
SimpleToolCollection,Paintings,1.0,-1.0,False,True,Fake,
SimpleToolCollection,Paintings,1.0,-1.0,False,True,,Report #4905
SimpleToolCollection,Paintings,1.0,-1.0,False,True,,
SimpleToolCollection,Paintings,1.0,-1.0,False,False,Genuine,Report #4906
SimpleToolCollection,Paintings,1.0,-1.0,False,False,Genuine,
SimpleToolCollection,Paintings,1.0,-1.0,False,False,Fake,Report #4907
SimpleToolCollection,Paintings,1.0,-1.0,False,False,Fake,
SimpleToolCollection,Paintings,1.0,-1.0,False,False,,Report #4908
SimpleToolCollection,Paintings,1.0,-1.0,False,False,,
SimpleToolCollection,Paintings,1.0,0.0,True,True,Genuine,Report #4909
SimpleToolCollection,Paintings,1.0,0.0,True,True,Genuine,
SimpleToolCollection,Paintings,1.0,0.0,True,True,Fake,Report #4910
SimpleToolCollection,Paintings,1.0,0.0,True,True,Fake,
SimpleToolCollection,Paintings,1.0,0.0,True,True,,Report #4911
SimpleToolCollection,Paintings,1.0,0.0,True,True,,
SimpleToolCollection,Paintings,1.0,0.0,True,False,Genuine,Report #4912
SimpleToolCollection,Paintings,1.0,0.0,True,False,Genuine,
SimpleToolCollection,Paintings,1.0,0.0,True,False,Fake,Report #4913
SimpleToolCollection,Paintings,1.0,0.0,True,False,Fake,
SimpleToolCollection,Paintings,1.0,0.0,True,False,,Report #4914
SimpleToolCollection,Paintings,1.0,0.0,True,False,,
SimpleToolCollection,Paintings,1.0,0.0,False,True,Genuine,Report #4915
SimpleToolCollection,Paintings,1.0,0.0,False,True,Genuine,
SimpleToolCollection,Paintings,1.0,0.0,False,True,Fake,Report #4916
SimpleToolCollection,Paintings,1.0,0.0,False,True,Fake,
SimpleToolCollection,Paintings,1.0,0.0,False,True,,Report #4917
SimpleToolCollection,Paintings,1.0,0.0,False,True,,
SimpleToolCollection,Paintings,1.0,0.0,False,False,Genuine,Report #4918
SimpleToolCollection,Paintings,1.0,0.0,False,False,Genuine,
SimpleToolCollection,Paintings,1.0,0.0,False,False,Fake,Report #4919
SimpleToolCollection,Paintings,1.0,0.0,False,False,Fake,
SimpleToolCollection,Paintings,1.0,0.0,False,False,,Report #4920
SimpleToolCollection,Paintings,1.0,0.0,False,False,,
SimpleToolCollection,Paintings,1.0,19.0,True,True,Genuine,Report #4921
SimpleToolCollection,Paintings,1.0,19.0,True,True,Genuine,
SimpleToolCollection,Paintings,1.0,19.0,True,True,Fake,Report #4922
SimpleToolCollection,Paintings,1.0,19.0,True,True,Fake,
SimpleToolCollection,Paintings,1.0,19.0,True,True,,Report #4923
SimpleToolCollection,Paintings,1.0,19.0,True,True,,
SimpleToolCollection,Paintings,1.0,19.0,True,False,Genuine,Report #4924
SimpleToolCollection,Paintings,1.0,19.0,True,False,Genuine,
SimpleToolCollection,Paintings,1.0,19.0,True,False,Fake,Report #4925
SimpleToolCollection,Paintings,1.0,19.0,True,False,Fake,
SimpleToolCollection,Paintings,1.0,19.0,True,False,,Report #4926
SimpleToolCollection,Paintings,1.0,19.0,True,False,,
SimpleToolCollection,Paintings,1.0,19.0,False,True,Genuine,Report #4927
SimpleToolCollection,Paintings,1.0,19.0,False,True,Genuine,
SimpleToolCollection,Paintings,1.0,19.0,False,True,Fake,Report #4928
SimpleToolCollection,Paintings,1.0,19.0,False,True,Fake,
SimpleToolCollection,Paintings,1.0,19.0,False,True,,Report #4929
SimpleToolCollection,Paintings,1.0,19.0,False,True,,
SimpleToolCollection,Paintings,1.0,19.0,False,False,Genuine,Report #4930
SimpleToolCollection,Paintings,1.0,19.0,False,False,Genuine,
SimpleToolCollection,Paintings,1.0,19.0,False,False,Fake,Report #4931
SimpleToolCollection,Paintings,1.0,19.0,False,False,Fake,
SimpleToolCollection,Paintings,1.0,19.0,False,False,,Report #4932
SimpleToolCollection,Paintings,1.0,19.0,False,False,,
SimpleToolCollection,Paintings,1.0,20.0,True,True,Genuine,Report #4933
SimpleToolCollection,Paintings,1.0,20.0,True,True,Genuine,
SimpleToolCollection,Paintings,1.0,20.0,True,True,Fake,Report #4934
SimpleToolCollection,Paintings,1.0,20.0,True,True,Fake,
SimpleToolCollection,Paintings,1.0,20.0,True,True,,Report #4935
SimpleToolCollection,Paintings,1.0,20.0,True,True,,
SimpleToolCollection,Paintings,1.0,20.0,True,False,Genuine,Report #4936
SimpleToolCollection,Paintings,1.0,20.0,True,False,Genuine,
SimpleToolCollection,Paintings,1.0,20.0,True,False,Fake,Report #4937
SimpleToolCollection,Paintings,1.0,20.0,True,False,Fake,
SimpleToolCollection,Paintings,1.0,20.0,True,False,,Report #4938
SimpleToolCollection,Paintings,1.0,20.0,True,False,,
SimpleToolCollection,Paintings,1.0,20.0,False,True,Genuine,Report #4939
SimpleToolCollection,Paintings,1.0,20.0,False,True,Genuine,
SimpleToolCollection,Paintings,1.0,20.0,False,True,Fake,Report #4940
SimpleToolCollection,Paintings,1.0,20.0,False,True,Fake,
SimpleToolCollection,Paintings,1.0,20.0,False,True,,Report #4941
SimpleToolCollection,Paintings,1.0,20.0,False,True,,
SimpleToolCollection,Paintings,1.0,20.0,False,False,Genuine,Report #4942
SimpleToolCollection,Paintings,1.0,20.0,False,False,Genuine,
SimpleToolCollection,Paintings,1.0,20.0,False,False,Fake,Report #4943
SimpleToolCollection,Paintings,1.0,20.0,False,False,Fake,
SimpleToolCollection,Paintings,1.0,20.0,False,False,,Report #4944
SimpleToolCollection,Paintings,1.0,20.0,False,False,,
SimpleToolCollection,Paintings,1.0,21.0,True,True,Genuine,Report #4945
SimpleToolCollection,Paintings,1.0,21.0,True,True,Genuine,
SimpleToolCollection,Paintings,1.0,21.0,True,True,Fake,Report #4946
SimpleToolCollection,Paintings,1.0,21.0,True,True,Fake,
SimpleToolCollection,Paintings,1.0,21.0,True,True,,Report #4947
SimpleToolCollection,Paintings,1.0,21.0,True,True,,
SimpleToolCollection,Paintings,1.0,21.0,True,False,Genuine,Report #4948
SimpleToolCollection,Paintings,1.0,21.0,True,False,Genuine,
SimpleToolCollection,Paintings,1.0,21.0,True,False,Fake,Report #4949
SimpleToolCollection,Paintings,1.0,21.0,True,False,Fake,
SimpleToolCollection,Paintings,1.0,21.0,True,False,,Report #4950
SimpleToolCollection,Paintings,1.0,21.0,True,False,,
SimpleToolCollection,Paintings,1.0,21.0,False,True,Genuine,Report #4951
SimpleToolCollection,Paintings,1.0,21.0,False,True,Genuine,
SimpleToolCollection,Paintings,1.0,21.0,False,True,Fake,Report #4952
SimpleToolCollection,Paintings,1.0,21.0,False,True,Fake,
SimpleToolCollection,Paintings,1.0,21.0,False,True,,Report #4953
SimpleToolCollection,Paintings,1.0,21.0,False,True,,
SimpleToolCollection,Paintings,1.0,21.0,False,False,Genuine,Report #4954
SimpleToolCollection,Paintings,1.0,21.0,False,False,Genuine,
SimpleToolCollection,Paintings,1.0,21.0,False,False,Fake,Report #4955
SimpleToolCollection,Paintings,1.0,21.0,False,False,Fake,
SimpleToolCollection,Paintings,1.0,21.0,False,False,,Report #4956
SimpleToolCollection,Paintings,1.0,21.0,False,False,,
SimpleToolCollection,Paintings,1.0,99.0,True,True,Genuine,Report #4957
SimpleToolCollection,Paintings,1.0,99.0,True,True,Genuine,
SimpleToolCollection,Paintings,1.0,99.0,True,True,Fake,Report #4958
SimpleToolCollection,Paintings,1.0,99.0,True,True,Fake,
SimpleToolCollection,Paintings,1.0,99.0,True,True,,Report #4959
SimpleToolCollection,Paintings,1.0,99.0,True,True,,
SimpleToolCollection,Paintings,1.0,99.0,True,False,Genuine,Report #4960
SimpleToolCollection,Paintings,1.0,99.0,True,False,Genuine,
SimpleToolCollection,Paintings,1.0,99.0,True,False,Fake,Report #4961
SimpleToolCollection,Paintings,1.0,99.0,True,False,Fake,
SimpleToolCollection,Paintings,1.0,99.0,True,False,,Report #4962
SimpleToolCollection,Paintings,1.0,99.0,True,False,,
SimpleToolCollection,Paintings,1.0,99.0,False,True,Genuine,Report #4963
SimpleToolCollection,Paintings,1.0,99.0,False,True,Genuine,
SimpleToolCollection,Paintings,1.0,99.0,False,True,Fake,Report #4964
SimpleToolCollection,Paintings,1.0,99.0,False,True,Fake,
SimpleToolCollection,Paintings,1.0,99.0,False,True,,Report #4965
SimpleToolCollection,Paintings,1.0,99.0,False,True,,
SimpleToolCollection,Paintings,1.0,99.0,False,False,Genuine,Report #4966
SimpleToolCollection,Paintings,1.0,99.0,False,False,Genuine,
SimpleToolCollection,Paintings,1.0,99.0,False,False,Fake,Report #4967
SimpleToolCollection,Paintings,1.0,99.0,False,False,Fake,
SimpleToolCollection,Paintings,1.0,99.0,False,False,,Report #4968
SimpleToolCollection,Paintings,1.0,99.0,False,False,,
SimpleToolCollection,Paintings,1.0,100.0,True,True,Genuine,Report #4969
SimpleToolCollection,Paintings,1.0,100.0,True,True,Genuine,
SimpleToolCollection,Paintings,1.0,100.0,True,True,Fake,Report #4970
SimpleToolCollection,Paintings,1.0,100.0,True,True,Fake,
SimpleToolCollection,Paintings,1.0,100.0,True,True,,Report #4971
SimpleToolCollection,Paintings,1.0,100.0,True,True,,
SimpleToolCollection,Paintings,1.0,100.0,True,False,Genuine,Report #4972
SimpleToolCollection,Paintings,1.0,100.0,True,False,Genuine,
SimpleToolCollection,Paintings,1.0,100.0,True,False,Fake,Report #4973
SimpleToolCollection,Paintings,1.0,100.0,True,False,Fake,
SimpleToolCollection,Paintings,1.0,100.0,True,False,,Report #4974
SimpleToolCollection,Paintings,1.0,100.0,True,False,,
SimpleToolCollection,Paintings,1.0,100.0,False,True,Genuine,Report #4975
SimpleToolCollection,Paintings,1.0,100.0,False,True,Genuine,
SimpleToolCollection,Paintings,1.0,100.0,False,True,Fake,Report #4976
SimpleToolCollection,Paintings,1.0,100.0,False,True,Fake,
SimpleToolCollection,Paintings,1.0,100.0,False,True,,Report #4977
SimpleToolCollection,Paintings,1.0,100.0,False,True,,
SimpleToolCollection,Paintings,1.0,100.0,False,False,Genuine,Report #4978
SimpleToolCollection,Paintings,1.0,100.0,False,False,Genuine,
SimpleToolCollection,Paintings,1.0,100.0,False,False,Fake,Report #4979
SimpleToolCollection,Paintings,1.0,100.0,False,False,Fake,
SimpleToolCollection,Paintings,1.0,100.0,False,False,,Report #4980
SimpleToolCollection,Paintings,1.0,100.0,False,False,,
SimpleToolCollection,Paintings,1.0,101.0,True,True,Genuine,Report #4981
SimpleToolCollection,Paintings,1.0,101.0,True,True,Genuine,
SimpleToolCollection,Paintings,1.0,101.0,True,True,Fake,Report #4982
SimpleToolCollection,Paintings,1.0,101.0,True,True,Fake,
SimpleToolCollection,Paintings,1.0,101.0,True,True,,Report #4983
SimpleToolCollection,Paintings,1.0,101.0,True,True,,
SimpleToolCollection,Paintings,1.0,101.0,True,False,Genuine,Report #4984
SimpleToolCollection,Paintings,1.0,101.0,True,False,Genuine,
SimpleToolCollection,Paintings,1.0,101.0,True,False,Fake,Report #4985
SimpleToolCollection,Paintings,1.0,101.0,True,False,Fake,
SimpleToolCollection,Paintings,1.0,101.0,True,False,,Report #4986
SimpleToolCollection,Paintings,1.0,101.0,True,False,,
SimpleToolCollection,Paintings,1.0,101.0,False,True,Genuine,Report #4987
SimpleToolCollection,Paintings,1.0,101.0,False,True,Genuine,
SimpleToolCollection,Paintings,1.0,101.0,False,True,Fake,Report #4988
SimpleToolCollection,Paintings,1.0,101.0,False,True,Fake,
SimpleToolCollection,Paintings,1.0,101.0,False,True,,Report #4989
SimpleToolCollection,Paintings,1.0,101.0,False,True,,
SimpleToolCollection,Paintings,1.0,101.0,False,False,Genuine,Report #4990
SimpleToolCollection,Paintings,1.0,101.0,False,False,Genuine,
SimpleToolCollection,Paintings,1.0,101.0,False,False,Fake,Report #4991
SimpleToolCollection,Paintings,1.0,101.0,False,False,Fake,
SimpleToolCollection,Paintings,1.0,101.0,False,False,,Report #4992
SimpleToolCollection,Paintings,1.0,101.0,False,False,,
SimpleToolCollection,Paintings,100.0,-1.0,True,True,Genuine,Report #4993
SimpleToolCollection,Paintings,100.0,-1.0,True,True,Genuine,
SimpleToolCollection,Paintings,100.0,-1.0,True,True,Fake,Report #4994
SimpleToolCollection,Paintings,100.0,-1.0,True,True,Fake,
SimpleToolCollection,Paintings,100.0,-1.0,True,True,,Report #4995
SimpleToolCollection,Paintings,100.0,-1.0,True,True,,
SimpleToolCollection,Paintings,100.0,-1.0,True,False,Genuine,Report #4996
SimpleToolCollection,Paintings,100.0,-1.0,True,False,Genuine,
SimpleToolCollection,Paintings,100.0,-1.0,True,False,Fake,Report #4997
SimpleToolCollection,Paintings,100.0,-1.0,True,False,Fake,
SimpleToolCollection,Paintings,100.0,-1.0,True,False,,Report #4998
SimpleToolCollection,Paintings,100.0,-1.0,True,False,,
SimpleToolCollection,Paintings,100.0,-1.0,False,True,Genuine,Report #4999
SimpleToolCollection,Paintings,100.0,-1.0,False,True,Genuine,
SimpleToolCollection,Paintings,100.0,-1.0,False,True,Fake,Report #5000
SimpleToolCollection,Paintings,100.0,-1.0,False,True,Fake,
SimpleToolCollection,Paintings,100.0,-1.0,False,True,,Report #5001
SimpleToolCollection,Paintings,100.0,-1.0,False,True,,
SimpleToolCollection,Paintings,100.0,-1.0,False,False,Genuine,Report #5002
SimpleToolCollection,Paintings,100.0,-1.0,False,False,Genuine,
SimpleToolCollection,Paintings,100.0,-1.0,False,False,Fake,Report #5003
SimpleToolCollection,Paintings,100.0,-1.0,False,False,Fake,
SimpleToolCollection,Paintings,100.0,-1.0,False,False,,Report #5004
SimpleToolCollection,Paintings,100.0,-1.0,False,False,,
SimpleToolCollection,Paintings,100.0,0.0,True,True,Genuine,Report #5005
SimpleToolCollection,Paintings,100.0,0.0,True,True,Genuine,
SimpleToolCollection,Paintings,100.0,0.0,True,True,Fake,Report #5006
SimpleToolCollection,Paintings,100.0,0.0,True,True,Fake,
SimpleToolCollection,Paintings,100.0,0.0,True,True,,Report #5007
SimpleToolCollection,Paintings,100.0,0.0,True,True,,
SimpleToolCollection,Paintings,100.0,0.0,True,False,Genuine,Report #5008
SimpleToolCollection,Paintings,100.0,0.0,True,False,Genuine,
SimpleToolCollection,Paintings,100.0,0.0,True,False,Fake,Report #5009
SimpleToolCollection,Paintings,100.0,0.0,True,False,Fake,
SimpleToolCollection,Paintings,100.0,0.0,True,False,,Report #5010
SimpleToolCollection,Paintings,100.0,0.0,True,False,,
SimpleToolCollection,Paintings,100.0,0.0,False,True,Genuine,Report #5011
SimpleToolCollection,Paintings,100.0,0.0,False,True,Genuine,
SimpleToolCollection,Paintings,100.0,0.0,False,True,Fake,Report #5012
SimpleToolCollection,Paintings,100.0,0.0,False,True,Fake,
SimpleToolCollection,Paintings,100.0,0.0,False,True,,Report #5013
SimpleToolCollection,Paintings,100.0,0.0,False,True,,
SimpleToolCollection,Paintings,100.0,0.0,False,False,Genuine,Report #5014
SimpleToolCollection,Paintings,100.0,0.0,False,False,Genuine,
SimpleToolCollection,Paintings,100.0,0.0,False,False,Fake,Report #5015
SimpleToolCollection,Paintings,100.0,0.0,False,False,Fake,
SimpleToolCollection,Paintings,100.0,0.0,False,False,,Report #5016
SimpleToolCollection,Paintings,100.0,0.0,False,False,,
SimpleToolCollection,Paintings,100.0,19.0,True,True,Genuine,Report #5017
SimpleToolCollection,Paintings,100.0,19.0,True,True,Genuine,
SimpleToolCollection,Paintings,100.0,19.0,True,True,Fake,Report #5018
SimpleToolCollection,Paintings,100.0,19.0,True,True,Fake,
SimpleToolCollection,Paintings,100.0,19.0,True,True,,Report #5019
SimpleToolCollection,Paintings,100.0,19.0,True,True,,
SimpleToolCollection,Paintings,100.0,19.0,True,False,Genuine,Report #5020
SimpleToolCollection,Paintings,100.0,19.0,True,False,Genuine,
SimpleToolCollection,Paintings,100.0,19.0,True,False,Fake,Report #5021
SimpleToolCollection,Paintings,100.0,19.0,True,False,Fake,
SimpleToolCollection,Paintings,100.0,19.0,True,False,,Report #5022
SimpleToolCollection,Paintings,100.0,19.0,True,False,,
SimpleToolCollection,Paintings,100.0,19.0,False,True,Genuine,Report #5023
SimpleToolCollection,Paintings,100.0,19.0,False,True,Genuine,
SimpleToolCollection,Paintings,100.0,19.0,False,True,Fake,Report #5024
SimpleToolCollection,Paintings,100.0,19.0,False,True,Fake,
SimpleToolCollection,Paintings,100.0,19.0,False,True,,Report #5025
SimpleToolCollection,Paintings,100.0,19.0,False,True,,
SimpleToolCollection,Paintings,100.0,19.0,False,False,Genuine,Report #5026
SimpleToolCollection,Paintings,100.0,19.0,False,False,Genuine,
SimpleToolCollection,Paintings,100.0,19.0,False,False,Fake,Report #5027
SimpleToolCollection,Paintings,100.0,19.0,False,False,Fake,
SimpleToolCollection,Paintings,100.0,19.0,False,False,,Report #5028
SimpleToolCollection,Paintings,100.0,19.0,False,False,,
SimpleToolCollection,Paintings,100.0,20.0,True,True,Genuine,Report #5029
SimpleToolCollection,Paintings,100.0,20.0,True,True,Genuine,
SimpleToolCollection,Paintings,100.0,20.0,True,True,Fake,Report #5030
SimpleToolCollection,Paintings,100.0,20.0,True,True,Fake,
SimpleToolCollection,Paintings,100.0,20.0,True,True,,Report #5031
SimpleToolCollection,Paintings,100.0,20.0,True,True,,
SimpleToolCollection,Paintings,100.0,20.0,True,False,Genuine,Report #5032
SimpleToolCollection,Paintings,100.0,20.0,True,False,Genuine,
SimpleToolCollection,Paintings,100.0,20.0,True,False,Fake,Report #5033
SimpleToolCollection,Paintings,100.0,20.0,True,False,Fake,
SimpleToolCollection,Paintings,100.0,20.0,True,False,,Report #5034
SimpleToolCollection,Paintings,100.0,20.0,True,False,,
SimpleToolCollection,Paintings,100.0,20.0,False,True,Genuine,Report #5035
SimpleToolCollection,Paintings,100.0,20.0,False,True,Genuine,
SimpleToolCollection,Paintings,100.0,20.0,False,True,Fake,Report #5036
SimpleToolCollection,Paintings,100.0,20.0,False,True,Fake,
SimpleToolCollection,Paintings,100.0,20.0,False,True,,Report #5037
SimpleToolCollection,Paintings,100.0,20.0,False,True,,
SimpleToolCollection,Paintings,100.0,20.0,False,False,Genuine,Report #5038
SimpleToolCollection,Paintings,100.0,20.0,False,False,Genuine,
SimpleToolCollection,Paintings,100.0,20.0,False,False,Fake,Report #5039
SimpleToolCollection,Paintings,100.0,20.0,False,False,Fake,
SimpleToolCollection,Paintings,100.0,20.0,False,False,,Report #5040
SimpleToolCollection,Paintings,100.0,20.0,False,False,,
SimpleToolCollection,Paintings,100.0,21.0,True,True,Genuine,Report #5041
SimpleToolCollection,Paintings,100.0,21.0,True,True,Genuine,
SimpleToolCollection,Paintings,100.0,21.0,True,True,Fake,Report #5042
SimpleToolCollection,Paintings,100.0,21.0,True,True,Fake,
SimpleToolCollection,Paintings,100.0,21.0,True,True,,Report #5043
SimpleToolCollection,Paintings,100.0,21.0,True,True,,
SimpleToolCollection,Paintings,100.0,21.0,True,False,Genuine,Report #5044
SimpleToolCollection,Paintings,100.0,21.0,True,False,Genuine,
SimpleToolCollection,Paintings,100.0,21.0,True,False,Fake,Report #5045
SimpleToolCollection,Paintings,100.0,21.0,True,False,Fake,
SimpleToolCollection,Paintings,100.0,21.0,True,False,,Report #5046
SimpleToolCollection,Paintings,100.0,21.0,True,False,,
SimpleToolCollection,Paintings,100.0,21.0,False,True,Genuine,Report #5047
SimpleToolCollection,Paintings,100.0,21.0,False,True,Genuine,
SimpleToolCollection,Paintings,100.0,21.0,False,True,Fake,Report #5048
SimpleToolCollection,Paintings,100.0,21.0,False,True,Fake,
SimpleToolCollection,Paintings,100.0,21.0,False,True,,Report #5049
SimpleToolCollection,Paintings,100.0,21.0,False,True,,
SimpleToolCollection,Paintings,100.0,21.0,False,False,Genuine,Report #5050
SimpleToolCollection,Paintings,100.0,21.0,False,False,Genuine,
SimpleToolCollection,Paintings,100.0,21.0,False,False,Fake,Report #5051
SimpleToolCollection,Paintings,100.0,21.0,False,False,Fake,
SimpleToolCollection,Paintings,100.0,21.0,False,False,,Report #5052
SimpleToolCollection,Paintings,100.0,21.0,False,False,,
SimpleToolCollection,Paintings,100.0,99.0,True,True,Genuine,Report #5053
SimpleToolCollection,Paintings,100.0,99.0,True,True,Genuine,
SimpleToolCollection,Paintings,100.0,99.0,True,True,Fake,Report #5054
SimpleToolCollection,Paintings,100.0,99.0,True,True,Fake,
SimpleToolCollection,Paintings,100.0,99.0,True,True,,Report #5055
SimpleToolCollection,Paintings,100.0,99.0,True,True,,
SimpleToolCollection,Paintings,100.0,99.0,True,False,Genuine,Report #5056
SimpleToolCollection,Paintings,100.0,99.0,True,False,Genuine,
SimpleToolCollection,Paintings,100.0,99.0,True,False,Fake,Report #5057
SimpleToolCollection,Paintings,100.0,99.0,True,False,Fake,
SimpleToolCollection,Paintings,100.0,99.0,True,False,,Report #5058
SimpleToolCollection,Paintings,100.0,99.0,True,False,,
SimpleToolCollection,Paintings,100.0,99.0,False,True,Genuine,Report #5059
SimpleToolCollection,Paintings,100.0,99.0,False,True,Genuine,
SimpleToolCollection,Paintings,100.0,99.0,False,True,Fake,Report #5060
SimpleToolCollection,Paintings,100.0,99.0,False,True,Fake,
SimpleToolCollection,Paintings,100.0,99.0,False,True,,Report #5061
SimpleToolCollection,Paintings,100.0,99.0,False,True,,
SimpleToolCollection,Paintings,100.0,99.0,False,False,Genuine,Report #5062
SimpleToolCollection,Paintings,100.0,99.0,False,False,Genuine,
SimpleToolCollection,Paintings,100.0,99.0,False,False,Fake,Report #5063
SimpleToolCollection,Paintings,100.0,99.0,False,False,Fake,
SimpleToolCollection,Paintings,100.0,99.0,False,False,,Report #5064
SimpleToolCollection,Paintings,100.0,99.0,False,False,,
SimpleToolCollection,Paintings,100.0,100.0,True,True,Genuine,Report #5065
SimpleToolCollection,Paintings,100.0,100.0,True,True,Genuine,
SimpleToolCollection,Paintings,100.0,100.0,True,True,Fake,Report #5066
SimpleToolCollection,Paintings,100.0,100.0,True,True,Fake,
SimpleToolCollection,Paintings,100.0,100.0,True,True,,Report #5067
SimpleToolCollection,Paintings,100.0,100.0,True,True,,
SimpleToolCollection,Paintings,100.0,100.0,True,False,Genuine,Report #5068
SimpleToolCollection,Paintings,100.0,100.0,True,False,Genuine,
SimpleToolCollection,Paintings,100.0,100.0,True,False,Fake,Report #5069
SimpleToolCollection,Paintings,100.0,100.0,True,False,Fake,
SimpleToolCollection,Paintings,100.0,100.0,True,False,,Report #5070
SimpleToolCollection,Paintings,100.0,100.0,True,False,,
SimpleToolCollection,Paintings,100.0,100.0,False,True,Genuine,Report #5071
SimpleToolCollection,Paintings,100.0,100.0,False,True,Genuine,
SimpleToolCollection,Paintings,100.0,100.0,False,True,Fake,Report #5072
SimpleToolCollection,Paintings,100.0,100.0,False,True,Fake,
SimpleToolCollection,Paintings,100.0,100.0,False,True,,Report #5073
SimpleToolCollection,Paintings,100.0,100.0,False,True,,
SimpleToolCollection,Paintings,100.0,100.0,False,False,Genuine,Report #5074
SimpleToolCollection,Paintings,100.0,100.0,False,False,Genuine,
SimpleToolCollection,Paintings,100.0,100.0,False,False,Fake,Report #5075
SimpleToolCollection,Paintings,100.0,100.0,False,False,Fake,
SimpleToolCollection,Paintings,100.0,100.0,False,False,,Report #5076
SimpleToolCollection,Paintings,100.0,100.0,False,False,,
SimpleToolCollection,Paintings,100.0,101.0,True,True,Genuine,Report #5077
SimpleToolCollection,Paintings,100.0,101.0,True,True,Genuine,
SimpleToolCollection,Paintings,100.0,101.0,True,True,Fake,Report #5078
SimpleToolCollection,Paintings,100.0,101.0,True,True,Fake,
SimpleToolCollection,Paintings,100.0,101.0,True,True,,Report #5079
SimpleToolCollection,Paintings,100.0,101.0,True,True,,
SimpleToolCollection,Paintings,100.0,101.0,True,False,Genuine,Report #5080
SimpleToolCollection,Paintings,100.0,101.0,True,False,Genuine,
SimpleToolCollection,Paintings,100.0,101.0,True,False,Fake,Report #5081
SimpleToolCollection,Paintings,100.0,101.0,True,False,Fake,
SimpleToolCollection,Paintings,100.0,101.0,True,False,,Report #5082
SimpleToolCollection,Paintings,100.0,101.0,True,False,,
SimpleToolCollection,Paintings,100.0,101.0,False,True,Genuine,Report #5083
SimpleToolCollection,Paintings,100.0,101.0,False,True,Genuine,
SimpleToolCollection,Paintings,100.0,101.0,False,True,Fake,Report #5084
SimpleToolCollection,Paintings,100.0,101.0,False,True,Fake,
SimpleToolCollection,Paintings,100.0,101.0,False,True,,Report #5085
SimpleToolCollection,Paintings,100.0,101.0,False,True,,
SimpleToolCollection,Paintings,100.0,101.0,False,False,Genuine,Report #5086
SimpleToolCollection,Paintings,100.0,101.0,False,False,Genuine,
SimpleToolCollection,Paintings,100.0,101.0,False,False,Fake,Report #5087
SimpleToolCollection,Paintings,100.0,101.0,False,False,Fake,
SimpleToolCollection,Paintings,100.0,101.0,False,False,,Report #5088
SimpleToolCollection,Paintings,100.0,101.0,False,False,,
SimpleToolCollection,Paintings,1000.0,-1.0,True,True,Genuine,Report #5089
SimpleToolCollection,Paintings,1000.0,-1.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000.0,-1.0,True,True,Fake,Report #5090
SimpleToolCollection,Paintings,1000.0,-1.0,True,True,Fake,
SimpleToolCollection,Paintings,1000.0,-1.0,True,True,,Report #5091
SimpleToolCollection,Paintings,1000.0,-1.0,True,True,,
SimpleToolCollection,Paintings,1000.0,-1.0,True,False,Genuine,Report #5092
SimpleToolCollection,Paintings,1000.0,-1.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000.0,-1.0,True,False,Fake,Report #5093
SimpleToolCollection,Paintings,1000.0,-1.0,True,False,Fake,
SimpleToolCollection,Paintings,1000.0,-1.0,True,False,,Report #5094
SimpleToolCollection,Paintings,1000.0,-1.0,True,False,,
SimpleToolCollection,Paintings,1000.0,-1.0,False,True,Genuine,Report #5095
SimpleToolCollection,Paintings,1000.0,-1.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000.0,-1.0,False,True,Fake,Report #5096
SimpleToolCollection,Paintings,1000.0,-1.0,False,True,Fake,
SimpleToolCollection,Paintings,1000.0,-1.0,False,True,,Report #5097
SimpleToolCollection,Paintings,1000.0,-1.0,False,True,,
SimpleToolCollection,Paintings,1000.0,-1.0,False,False,Genuine,Report #5098
SimpleToolCollection,Paintings,1000.0,-1.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000.0,-1.0,False,False,Fake,Report #5099
SimpleToolCollection,Paintings,1000.0,-1.0,False,False,Fake,
SimpleToolCollection,Paintings,1000.0,-1.0,False,False,,Report #5100
SimpleToolCollection,Paintings,1000.0,-1.0,False,False,,
SimpleToolCollection,Paintings,1000.0,0.0,True,True,Genuine,Report #5101
SimpleToolCollection,Paintings,1000.0,0.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000.0,0.0,True,True,Fake,Report #5102
SimpleToolCollection,Paintings,1000.0,0.0,True,True,Fake,
SimpleToolCollection,Paintings,1000.0,0.0,True,True,,Report #5103
SimpleToolCollection,Paintings,1000.0,0.0,True,True,,
SimpleToolCollection,Paintings,1000.0,0.0,True,False,Genuine,Report #5104
SimpleToolCollection,Paintings,1000.0,0.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000.0,0.0,True,False,Fake,Report #5105
SimpleToolCollection,Paintings,1000.0,0.0,True,False,Fake,
SimpleToolCollection,Paintings,1000.0,0.0,True,False,,Report #5106
SimpleToolCollection,Paintings,1000.0,0.0,True,False,,
SimpleToolCollection,Paintings,1000.0,0.0,False,True,Genuine,Report #5107
SimpleToolCollection,Paintings,1000.0,0.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000.0,0.0,False,True,Fake,Report #5108
SimpleToolCollection,Paintings,1000.0,0.0,False,True,Fake,
SimpleToolCollection,Paintings,1000.0,0.0,False,True,,Report #5109
SimpleToolCollection,Paintings,1000.0,0.0,False,True,,
SimpleToolCollection,Paintings,1000.0,0.0,False,False,Genuine,Report #5110
SimpleToolCollection,Paintings,1000.0,0.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000.0,0.0,False,False,Fake,Report #5111
SimpleToolCollection,Paintings,1000.0,0.0,False,False,Fake,
SimpleToolCollection,Paintings,1000.0,0.0,False,False,,Report #5112
SimpleToolCollection,Paintings,1000.0,0.0,False,False,,
SimpleToolCollection,Paintings,1000.0,19.0,True,True,Genuine,Report #5113
SimpleToolCollection,Paintings,1000.0,19.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000.0,19.0,True,True,Fake,Report #5114
SimpleToolCollection,Paintings,1000.0,19.0,True,True,Fake,
SimpleToolCollection,Paintings,1000.0,19.0,True,True,,Report #5115
SimpleToolCollection,Paintings,1000.0,19.0,True,True,,
SimpleToolCollection,Paintings,1000.0,19.0,True,False,Genuine,Report #5116
SimpleToolCollection,Paintings,1000.0,19.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000.0,19.0,True,False,Fake,Report #5117
SimpleToolCollection,Paintings,1000.0,19.0,True,False,Fake,
SimpleToolCollection,Paintings,1000.0,19.0,True,False,,Report #5118
SimpleToolCollection,Paintings,1000.0,19.0,True,False,,
SimpleToolCollection,Paintings,1000.0,19.0,False,True,Genuine,Report #5119
SimpleToolCollection,Paintings,1000.0,19.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000.0,19.0,False,True,Fake,Report #5120
SimpleToolCollection,Paintings,1000.0,19.0,False,True,Fake,
SimpleToolCollection,Paintings,1000.0,19.0,False,True,,Report #5121
SimpleToolCollection,Paintings,1000.0,19.0,False,True,,
SimpleToolCollection,Paintings,1000.0,19.0,False,False,Genuine,Report #5122
SimpleToolCollection,Paintings,1000.0,19.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000.0,19.0,False,False,Fake,Report #5123
SimpleToolCollection,Paintings,1000.0,19.0,False,False,Fake,
SimpleToolCollection,Paintings,1000.0,19.0,False,False,,Report #5124
SimpleToolCollection,Paintings,1000.0,19.0,False,False,,
SimpleToolCollection,Paintings,1000.0,20.0,True,True,Genuine,Report #5125
SimpleToolCollection,Paintings,1000.0,20.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000.0,20.0,True,True,Fake,Report #5126
SimpleToolCollection,Paintings,1000.0,20.0,True,True,Fake,
SimpleToolCollection,Paintings,1000.0,20.0,True,True,,Report #5127
SimpleToolCollection,Paintings,1000.0,20.0,True,True,,
SimpleToolCollection,Paintings,1000.0,20.0,True,False,Genuine,Report #5128
SimpleToolCollection,Paintings,1000.0,20.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000.0,20.0,True,False,Fake,Report #5129
SimpleToolCollection,Paintings,1000.0,20.0,True,False,Fake,
SimpleToolCollection,Paintings,1000.0,20.0,True,False,,Report #5130
SimpleToolCollection,Paintings,1000.0,20.0,True,False,,
SimpleToolCollection,Paintings,1000.0,20.0,False,True,Genuine,Report #5131
SimpleToolCollection,Paintings,1000.0,20.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000.0,20.0,False,True,Fake,Report #5132
SimpleToolCollection,Paintings,1000.0,20.0,False,True,Fake,
SimpleToolCollection,Paintings,1000.0,20.0,False,True,,Report #5133
SimpleToolCollection,Paintings,1000.0,20.0,False,True,,
SimpleToolCollection,Paintings,1000.0,20.0,False,False,Genuine,Report #5134
SimpleToolCollection,Paintings,1000.0,20.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000.0,20.0,False,False,Fake,Report #5135
SimpleToolCollection,Paintings,1000.0,20.0,False,False,Fake,
SimpleToolCollection,Paintings,1000.0,20.0,False,False,,Report #5136
SimpleToolCollection,Paintings,1000.0,20.0,False,False,,
SimpleToolCollection,Paintings,1000.0,21.0,True,True,Genuine,Report #5137
SimpleToolCollection,Paintings,1000.0,21.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000.0,21.0,True,True,Fake,Report #5138
SimpleToolCollection,Paintings,1000.0,21.0,True,True,Fake,
SimpleToolCollection,Paintings,1000.0,21.0,True,True,,Report #5139
SimpleToolCollection,Paintings,1000.0,21.0,True,True,,
SimpleToolCollection,Paintings,1000.0,21.0,True,False,Genuine,Report #5140
SimpleToolCollection,Paintings,1000.0,21.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000.0,21.0,True,False,Fake,Report #5141
SimpleToolCollection,Paintings,1000.0,21.0,True,False,Fake,
SimpleToolCollection,Paintings,1000.0,21.0,True,False,,Report #5142
SimpleToolCollection,Paintings,1000.0,21.0,True,False,,
SimpleToolCollection,Paintings,1000.0,21.0,False,True,Genuine,Report #5143
SimpleToolCollection,Paintings,1000.0,21.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000.0,21.0,False,True,Fake,Report #5144
SimpleToolCollection,Paintings,1000.0,21.0,False,True,Fake,
SimpleToolCollection,Paintings,1000.0,21.0,False,True,,Report #5145
SimpleToolCollection,Paintings,1000.0,21.0,False,True,,
SimpleToolCollection,Paintings,1000.0,21.0,False,False,Genuine,Report #5146
SimpleToolCollection,Paintings,1000.0,21.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000.0,21.0,False,False,Fake,Report #5147
SimpleToolCollection,Paintings,1000.0,21.0,False,False,Fake,
SimpleToolCollection,Paintings,1000.0,21.0,False,False,,Report #5148
SimpleToolCollection,Paintings,1000.0,21.0,False,False,,
SimpleToolCollection,Paintings,1000.0,99.0,True,True,Genuine,Report #5149
SimpleToolCollection,Paintings,1000.0,99.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000.0,99.0,True,True,Fake,Report #5150
SimpleToolCollection,Paintings,1000.0,99.0,True,True,Fake,
SimpleToolCollection,Paintings,1000.0,99.0,True,True,,Report #5151
SimpleToolCollection,Paintings,1000.0,99.0,True,True,,
SimpleToolCollection,Paintings,1000.0,99.0,True,False,Genuine,Report #5152
SimpleToolCollection,Paintings,1000.0,99.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000.0,99.0,True,False,Fake,Report #5153
SimpleToolCollection,Paintings,1000.0,99.0,True,False,Fake,
SimpleToolCollection,Paintings,1000.0,99.0,True,False,,Report #5154
SimpleToolCollection,Paintings,1000.0,99.0,True,False,,
SimpleToolCollection,Paintings,1000.0,99.0,False,True,Genuine,Report #5155
SimpleToolCollection,Paintings,1000.0,99.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000.0,99.0,False,True,Fake,Report #5156
SimpleToolCollection,Paintings,1000.0,99.0,False,True,Fake,
SimpleToolCollection,Paintings,1000.0,99.0,False,True,,Report #5157
SimpleToolCollection,Paintings,1000.0,99.0,False,True,,
SimpleToolCollection,Paintings,1000.0,99.0,False,False,Genuine,Report #5158
SimpleToolCollection,Paintings,1000.0,99.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000.0,99.0,False,False,Fake,Report #5159
SimpleToolCollection,Paintings,1000.0,99.0,False,False,Fake,
SimpleToolCollection,Paintings,1000.0,99.0,False,False,,Report #5160
SimpleToolCollection,Paintings,1000.0,99.0,False,False,,
SimpleToolCollection,Paintings,1000.0,100.0,True,True,Genuine,Report #5161
SimpleToolCollection,Paintings,1000.0,100.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000.0,100.0,True,True,Fake,Report #5162
SimpleToolCollection,Paintings,1000.0,100.0,True,True,Fake,
SimpleToolCollection,Paintings,1000.0,100.0,True,True,,Report #5163
SimpleToolCollection,Paintings,1000.0,100.0,True,True,,
SimpleToolCollection,Paintings,1000.0,100.0,True,False,Genuine,Report #5164
SimpleToolCollection,Paintings,1000.0,100.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000.0,100.0,True,False,Fake,Report #5165
SimpleToolCollection,Paintings,1000.0,100.0,True,False,Fake,
SimpleToolCollection,Paintings,1000.0,100.0,True,False,,Report #5166
SimpleToolCollection,Paintings,1000.0,100.0,True,False,,
SimpleToolCollection,Paintings,1000.0,100.0,False,True,Genuine,Report #5167
SimpleToolCollection,Paintings,1000.0,100.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000.0,100.0,False,True,Fake,Report #5168
SimpleToolCollection,Paintings,1000.0,100.0,False,True,Fake,
SimpleToolCollection,Paintings,1000.0,100.0,False,True,,Report #5169
SimpleToolCollection,Paintings,1000.0,100.0,False,True,,
SimpleToolCollection,Paintings,1000.0,100.0,False,False,Genuine,Report #5170
SimpleToolCollection,Paintings,1000.0,100.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000.0,100.0,False,False,Fake,Report #5171
SimpleToolCollection,Paintings,1000.0,100.0,False,False,Fake,
SimpleToolCollection,Paintings,1000.0,100.0,False,False,,Report #5172
SimpleToolCollection,Paintings,1000.0,100.0,False,False,,
SimpleToolCollection,Paintings,1000.0,101.0,True,True,Genuine,Report #5173
SimpleToolCollection,Paintings,1000.0,101.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000.0,101.0,True,True,Fake,Report #5174
SimpleToolCollection,Paintings,1000.0,101.0,True,True,Fake,
SimpleToolCollection,Paintings,1000.0,101.0,True,True,,Report #5175
SimpleToolCollection,Paintings,1000.0,101.0,True,True,,
SimpleToolCollection,Paintings,1000.0,101.0,True,False,Genuine,Report #5176
SimpleToolCollection,Paintings,1000.0,101.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000.0,101.0,True,False,Fake,Report #5177
SimpleToolCollection,Paintings,1000.0,101.0,True,False,Fake,
SimpleToolCollection,Paintings,1000.0,101.0,True,False,,Report #5178
SimpleToolCollection,Paintings,1000.0,101.0,True,False,,
SimpleToolCollection,Paintings,1000.0,101.0,False,True,Genuine,Report #5179
SimpleToolCollection,Paintings,1000.0,101.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000.0,101.0,False,True,Fake,Report #5180
SimpleToolCollection,Paintings,1000.0,101.0,False,True,Fake,
SimpleToolCollection,Paintings,1000.0,101.0,False,True,,Report #5181
SimpleToolCollection,Paintings,1000.0,101.0,False,True,,
SimpleToolCollection,Paintings,1000.0,101.0,False,False,Genuine,Report #5182
SimpleToolCollection,Paintings,1000.0,101.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000.0,101.0,False,False,Fake,Report #5183
SimpleToolCollection,Paintings,1000.0,101.0,False,False,Fake,
SimpleToolCollection,Paintings,1000.0,101.0,False,False,,Report #5184
SimpleToolCollection,Paintings,1000.0,101.0,False,False,,
SimpleToolCollection,Paintings,1000000.0,-1.0,True,True,Genuine,Report #5185
SimpleToolCollection,Paintings,1000000.0,-1.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,-1.0,True,True,Fake,Report #5186
SimpleToolCollection,Paintings,1000000.0,-1.0,True,True,Fake,
SimpleToolCollection,Paintings,1000000.0,-1.0,True,True,,Report #5187
SimpleToolCollection,Paintings,1000000.0,-1.0,True,True,,
SimpleToolCollection,Paintings,1000000.0,-1.0,True,False,Genuine,Report #5188
SimpleToolCollection,Paintings,1000000.0,-1.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,-1.0,True,False,Fake,Report #5189
SimpleToolCollection,Paintings,1000000.0,-1.0,True,False,Fake,
SimpleToolCollection,Paintings,1000000.0,-1.0,True,False,,Report #5190
SimpleToolCollection,Paintings,1000000.0,-1.0,True,False,,
SimpleToolCollection,Paintings,1000000.0,-1.0,False,True,Genuine,Report #5191
SimpleToolCollection,Paintings,1000000.0,-1.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,-1.0,False,True,Fake,Report #5192
SimpleToolCollection,Paintings,1000000.0,-1.0,False,True,Fake,
SimpleToolCollection,Paintings,1000000.0,-1.0,False,True,,Report #5193
SimpleToolCollection,Paintings,1000000.0,-1.0,False,True,,
SimpleToolCollection,Paintings,1000000.0,-1.0,False,False,Genuine,Report #5194
SimpleToolCollection,Paintings,1000000.0,-1.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,-1.0,False,False,Fake,Report #5195
SimpleToolCollection,Paintings,1000000.0,-1.0,False,False,Fake,
SimpleToolCollection,Paintings,1000000.0,-1.0,False,False,,Report #5196
SimpleToolCollection,Paintings,1000000.0,-1.0,False,False,,
SimpleToolCollection,Paintings,1000000.0,0.0,True,True,Genuine,Report #5197
SimpleToolCollection,Paintings,1000000.0,0.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,0.0,True,True,Fake,Report #5198
SimpleToolCollection,Paintings,1000000.0,0.0,True,True,Fake,
SimpleToolCollection,Paintings,1000000.0,0.0,True,True,,Report #5199
SimpleToolCollection,Paintings,1000000.0,0.0,True,True,,
SimpleToolCollection,Paintings,1000000.0,0.0,True,False,Genuine,Report #5200
SimpleToolCollection,Paintings,1000000.0,0.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,0.0,True,False,Fake,Report #5201
SimpleToolCollection,Paintings,1000000.0,0.0,True,False,Fake,
SimpleToolCollection,Paintings,1000000.0,0.0,True,False,,Report #5202
SimpleToolCollection,Paintings,1000000.0,0.0,True,False,,
SimpleToolCollection,Paintings,1000000.0,0.0,False,True,Genuine,Report #5203
SimpleToolCollection,Paintings,1000000.0,0.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,0.0,False,True,Fake,Report #5204
SimpleToolCollection,Paintings,1000000.0,0.0,False,True,Fake,
SimpleToolCollection,Paintings,1000000.0,0.0,False,True,,Report #5205
SimpleToolCollection,Paintings,1000000.0,0.0,False,True,,
SimpleToolCollection,Paintings,1000000.0,0.0,False,False,Genuine,Report #5206
SimpleToolCollection,Paintings,1000000.0,0.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,0.0,False,False,Fake,Report #5207
SimpleToolCollection,Paintings,1000000.0,0.0,False,False,Fake,
SimpleToolCollection,Paintings,1000000.0,0.0,False,False,,Report #5208
SimpleToolCollection,Paintings,1000000.0,0.0,False,False,,
SimpleToolCollection,Paintings,1000000.0,19.0,True,True,Genuine,Report #5209
SimpleToolCollection,Paintings,1000000.0,19.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,19.0,True,True,Fake,Report #5210
SimpleToolCollection,Paintings,1000000.0,19.0,True,True,Fake,
SimpleToolCollection,Paintings,1000000.0,19.0,True,True,,Report #5211
SimpleToolCollection,Paintings,1000000.0,19.0,True,True,,
SimpleToolCollection,Paintings,1000000.0,19.0,True,False,Genuine,Report #5212
SimpleToolCollection,Paintings,1000000.0,19.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,19.0,True,False,Fake,Report #5213
SimpleToolCollection,Paintings,1000000.0,19.0,True,False,Fake,
SimpleToolCollection,Paintings,1000000.0,19.0,True,False,,Report #5214
SimpleToolCollection,Paintings,1000000.0,19.0,True,False,,
SimpleToolCollection,Paintings,1000000.0,19.0,False,True,Genuine,Report #5215
SimpleToolCollection,Paintings,1000000.0,19.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,19.0,False,True,Fake,Report #5216
SimpleToolCollection,Paintings,1000000.0,19.0,False,True,Fake,
SimpleToolCollection,Paintings,1000000.0,19.0,False,True,,Report #5217
SimpleToolCollection,Paintings,1000000.0,19.0,False,True,,
SimpleToolCollection,Paintings,1000000.0,19.0,False,False,Genuine,Report #5218
SimpleToolCollection,Paintings,1000000.0,19.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,19.0,False,False,Fake,Report #5219
SimpleToolCollection,Paintings,1000000.0,19.0,False,False,Fake,
SimpleToolCollection,Paintings,1000000.0,19.0,False,False,,Report #5220
SimpleToolCollection,Paintings,1000000.0,19.0,False,False,,
SimpleToolCollection,Paintings,1000000.0,20.0,True,True,Genuine,Report #5221
SimpleToolCollection,Paintings,1000000.0,20.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,20.0,True,True,Fake,Report #5222
SimpleToolCollection,Paintings,1000000.0,20.0,True,True,Fake,
SimpleToolCollection,Paintings,1000000.0,20.0,True,True,,Report #5223
SimpleToolCollection,Paintings,1000000.0,20.0,True,True,,
SimpleToolCollection,Paintings,1000000.0,20.0,True,False,Genuine,Report #5224
SimpleToolCollection,Paintings,1000000.0,20.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,20.0,True,False,Fake,Report #5225
SimpleToolCollection,Paintings,1000000.0,20.0,True,False,Fake,
SimpleToolCollection,Paintings,1000000.0,20.0,True,False,,Report #5226
SimpleToolCollection,Paintings,1000000.0,20.0,True,False,,
SimpleToolCollection,Paintings,1000000.0,20.0,False,True,Genuine,Report #5227
SimpleToolCollection,Paintings,1000000.0,20.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,20.0,False,True,Fake,Report #5228
SimpleToolCollection,Paintings,1000000.0,20.0,False,True,Fake,
SimpleToolCollection,Paintings,1000000.0,20.0,False,True,,Report #5229
SimpleToolCollection,Paintings,1000000.0,20.0,False,True,,
SimpleToolCollection,Paintings,1000000.0,20.0,False,False,Genuine,Report #5230
SimpleToolCollection,Paintings,1000000.0,20.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,20.0,False,False,Fake,Report #5231
SimpleToolCollection,Paintings,1000000.0,20.0,False,False,Fake,
SimpleToolCollection,Paintings,1000000.0,20.0,False,False,,Report #5232
SimpleToolCollection,Paintings,1000000.0,20.0,False,False,,
SimpleToolCollection,Paintings,1000000.0,21.0,True,True,Genuine,Report #5233
SimpleToolCollection,Paintings,1000000.0,21.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,21.0,True,True,Fake,Report #5234
SimpleToolCollection,Paintings,1000000.0,21.0,True,True,Fake,
SimpleToolCollection,Paintings,1000000.0,21.0,True,True,,Report #5235
SimpleToolCollection,Paintings,1000000.0,21.0,True,True,,
SimpleToolCollection,Paintings,1000000.0,21.0,True,False,Genuine,Report #5236
SimpleToolCollection,Paintings,1000000.0,21.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,21.0,True,False,Fake,Report #5237
SimpleToolCollection,Paintings,1000000.0,21.0,True,False,Fake,
SimpleToolCollection,Paintings,1000000.0,21.0,True,False,,Report #5238
SimpleToolCollection,Paintings,1000000.0,21.0,True,False,,
SimpleToolCollection,Paintings,1000000.0,21.0,False,True,Genuine,Report #5239
SimpleToolCollection,Paintings,1000000.0,21.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,21.0,False,True,Fake,Report #5240
SimpleToolCollection,Paintings,1000000.0,21.0,False,True,Fake,
SimpleToolCollection,Paintings,1000000.0,21.0,False,True,,Report #5241
SimpleToolCollection,Paintings,1000000.0,21.0,False,True,,
SimpleToolCollection,Paintings,1000000.0,21.0,False,False,Genuine,Report #5242
SimpleToolCollection,Paintings,1000000.0,21.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,21.0,False,False,Fake,Report #5243
SimpleToolCollection,Paintings,1000000.0,21.0,False,False,Fake,
SimpleToolCollection,Paintings,1000000.0,21.0,False,False,,Report #5244
SimpleToolCollection,Paintings,1000000.0,21.0,False,False,,
SimpleToolCollection,Paintings,1000000.0,99.0,True,True,Genuine,Report #5245
SimpleToolCollection,Paintings,1000000.0,99.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,99.0,True,True,Fake,Report #5246
SimpleToolCollection,Paintings,1000000.0,99.0,True,True,Fake,
SimpleToolCollection,Paintings,1000000.0,99.0,True,True,,Report #5247
SimpleToolCollection,Paintings,1000000.0,99.0,True,True,,
SimpleToolCollection,Paintings,1000000.0,99.0,True,False,Genuine,Report #5248
SimpleToolCollection,Paintings,1000000.0,99.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,99.0,True,False,Fake,Report #5249
SimpleToolCollection,Paintings,1000000.0,99.0,True,False,Fake,
SimpleToolCollection,Paintings,1000000.0,99.0,True,False,,Report #5250
SimpleToolCollection,Paintings,1000000.0,99.0,True,False,,
SimpleToolCollection,Paintings,1000000.0,99.0,False,True,Genuine,Report #5251
SimpleToolCollection,Paintings,1000000.0,99.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,99.0,False,True,Fake,Report #5252
SimpleToolCollection,Paintings,1000000.0,99.0,False,True,Fake,
SimpleToolCollection,Paintings,1000000.0,99.0,False,True,,Report #5253
SimpleToolCollection,Paintings,1000000.0,99.0,False,True,,
SimpleToolCollection,Paintings,1000000.0,99.0,False,False,Genuine,Report #5254
SimpleToolCollection,Paintings,1000000.0,99.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,99.0,False,False,Fake,Report #5255
SimpleToolCollection,Paintings,1000000.0,99.0,False,False,Fake,
SimpleToolCollection,Paintings,1000000.0,99.0,False,False,,Report #5256
SimpleToolCollection,Paintings,1000000.0,99.0,False,False,,
SimpleToolCollection,Paintings,1000000.0,100.0,True,True,Genuine,Report #5257
SimpleToolCollection,Paintings,1000000.0,100.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,100.0,True,True,Fake,Report #5258
SimpleToolCollection,Paintings,1000000.0,100.0,True,True,Fake,
SimpleToolCollection,Paintings,1000000.0,100.0,True,True,,Report #5259
SimpleToolCollection,Paintings,1000000.0,100.0,True,True,,
SimpleToolCollection,Paintings,1000000.0,100.0,True,False,Genuine,Report #5260
SimpleToolCollection,Paintings,1000000.0,100.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,100.0,True,False,Fake,Report #5261
SimpleToolCollection,Paintings,1000000.0,100.0,True,False,Fake,
SimpleToolCollection,Paintings,1000000.0,100.0,True,False,,Report #5262
SimpleToolCollection,Paintings,1000000.0,100.0,True,False,,
SimpleToolCollection,Paintings,1000000.0,100.0,False,True,Genuine,Report #5263
SimpleToolCollection,Paintings,1000000.0,100.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,100.0,False,True,Fake,Report #5264
SimpleToolCollection,Paintings,1000000.0,100.0,False,True,Fake,
SimpleToolCollection,Paintings,1000000.0,100.0,False,True,,Report #5265
SimpleToolCollection,Paintings,1000000.0,100.0,False,True,,
SimpleToolCollection,Paintings,1000000.0,100.0,False,False,Genuine,Report #5266
SimpleToolCollection,Paintings,1000000.0,100.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,100.0,False,False,Fake,Report #5267
SimpleToolCollection,Paintings,1000000.0,100.0,False,False,Fake,
SimpleToolCollection,Paintings,1000000.0,100.0,False,False,,Report #5268
SimpleToolCollection,Paintings,1000000.0,100.0,False,False,,
SimpleToolCollection,Paintings,1000000.0,101.0,True,True,Genuine,Report #5269
SimpleToolCollection,Paintings,1000000.0,101.0,True,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,101.0,True,True,Fake,Report #5270
SimpleToolCollection,Paintings,1000000.0,101.0,True,True,Fake,
SimpleToolCollection,Paintings,1000000.0,101.0,True,True,,Report #5271
SimpleToolCollection,Paintings,1000000.0,101.0,True,True,,
SimpleToolCollection,Paintings,1000000.0,101.0,True,False,Genuine,Report #5272
SimpleToolCollection,Paintings,1000000.0,101.0,True,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,101.0,True,False,Fake,Report #5273
SimpleToolCollection,Paintings,1000000.0,101.0,True,False,Fake,
SimpleToolCollection,Paintings,1000000.0,101.0,True,False,,Report #5274
SimpleToolCollection,Paintings,1000000.0,101.0,True,False,,
SimpleToolCollection,Paintings,1000000.0,101.0,False,True,Genuine,Report #5275
SimpleToolCollection,Paintings,1000000.0,101.0,False,True,Genuine,
SimpleToolCollection,Paintings,1000000.0,101.0,False,True,Fake,Report #5276
SimpleToolCollection,Paintings,1000000.0,101.0,False,True,Fake,
SimpleToolCollection,Paintings,1000000.0,101.0,False,True,,Report #5277
SimpleToolCollection,Paintings,1000000.0,101.0,False,True,,
SimpleToolCollection,Paintings,1000000.0,101.0,False,False,Genuine,Report #5278
SimpleToolCollection,Paintings,1000000.0,101.0,False,False,Genuine,
SimpleToolCollection,Paintings,1000000.0,101.0,False,False,Fake,Report #5279
SimpleToolCollection,Paintings,1000000.0,101.0,False,False,Fake,
SimpleToolCollection,Paintings,1000000.0,101.0,False,False,,Report #5280
SimpleToolCollection,Paintings,1000000.0,101.0,False,False,,
SimpleToolCollection,Knife,0.0,-1.0,True,True,Genuine,Report #5281
SimpleToolCollection,Knife,0.0,-1.0,True,True,Genuine,
SimpleToolCollection,Knife,0.0,-1.0,True,True,Fake,Report #5282
SimpleToolCollection,Knife,0.0,-1.0,True,True,Fake,
SimpleToolCollection,Knife,0.0,-1.0,True,True,,Report #5283
SimpleToolCollection,Knife,0.0,-1.0,True,True,,
SimpleToolCollection,Knife,0.0,-1.0,True,False,Genuine,Report #5284
SimpleToolCollection,Knife,0.0,-1.0,True,False,Genuine,
SimpleToolCollection,Knife,0.0,-1.0,True,False,Fake,Report #5285
SimpleToolCollection,Knife,0.0,-1.0,True,False,Fake,
SimpleToolCollection,Knife,0.0,-1.0,True,False,,Report #5286
SimpleToolCollection,Knife,0.0,-1.0,True,False,,
SimpleToolCollection,Knife,0.0,-1.0,False,True,Genuine,Report #5287
SimpleToolCollection,Knife,0.0,-1.0,False,True,Genuine,
SimpleToolCollection,Knife,0.0,-1.0,False,True,Fake,Report #5288
SimpleToolCollection,Knife,0.0,-1.0,False,True,Fake,
SimpleToolCollection,Knife,0.0,-1.0,False,True,,Report #5289
SimpleToolCollection,Knife,0.0,-1.0,False,True,,
SimpleToolCollection,Knife,0.0,-1.0,False,False,Genuine,Report #5290
SimpleToolCollection,Knife,0.0,-1.0,False,False,Genuine,
SimpleToolCollection,Knife,0.0,-1.0,False,False,Fake,Report #5291
SimpleToolCollection,Knife,0.0,-1.0,False,False,Fake,
SimpleToolCollection,Knife,0.0,-1.0,False,False,,Report #5292
SimpleToolCollection,Knife,0.0,-1.0,False,False,,
SimpleToolCollection,Knife,0.0,0.0,True,True,Genuine,Report #5293
SimpleToolCollection,Knife,0.0,0.0,True,True,Genuine,
SimpleToolCollection,Knife,0.0,0.0,True,True,Fake,Report #5294
SimpleToolCollection,Knife,0.0,0.0,True,True,Fake,
SimpleToolCollection,Knife,0.0,0.0,True,True,,Report #5295
SimpleToolCollection,Knife,0.0,0.0,True,True,,
SimpleToolCollection,Knife,0.0,0.0,True,False,Genuine,Report #5296
SimpleToolCollection,Knife,0.0,0.0,True,False,Genuine,
SimpleToolCollection,Knife,0.0,0.0,True,False,Fake,Report #5297
SimpleToolCollection,Knife,0.0,0.0,True,False,Fake,
SimpleToolCollection,Knife,0.0,0.0,True,False,,Report #5298
SimpleToolCollection,Knife,0.0,0.0,True,False,,
SimpleToolCollection,Knife,0.0,0.0,False,True,Genuine,Report #5299
SimpleToolCollection,Knife,0.0,0.0,False,True,Genuine,
SimpleToolCollection,Knife,0.0,0.0,False,True,Fake,Report #5300
SimpleToolCollection,Knife,0.0,0.0,False,True,Fake,
SimpleToolCollection,Knife,0.0,0.0,False,True,,Report #5301
SimpleToolCollection,Knife,0.0,0.0,False,True,,
SimpleToolCollection,Knife,0.0,0.0,False,False,Genuine,Report #5302
SimpleToolCollection,Knife,0.0,0.0,False,False,Genuine,
SimpleToolCollection,Knife,0.0,0.0,False,False,Fake,Report #5303
SimpleToolCollection,Knife,0.0,0.0,False,False,Fake,
SimpleToolCollection,Knife,0.0,0.0,False,False,,Report #5304
SimpleToolCollection,Knife,0.0,0.0,False,False,,
SimpleToolCollection,Knife,0.0,19.0,True,True,Genuine,Report #5305
SimpleToolCollection,Knife,0.0,19.0,True,True,Genuine,
SimpleToolCollection,Knife,0.0,19.0,True,True,Fake,Report #5306
SimpleToolCollection,Knife,0.0,19.0,True,True,Fake,
SimpleToolCollection,Knife,0.0,19.0,True,True,,Report #5307
SimpleToolCollection,Knife,0.0,19.0,True,True,,
SimpleToolCollection,Knife,0.0,19.0,True,False,Genuine,Report #5308
SimpleToolCollection,Knife,0.0,19.0,True,False,Genuine,
SimpleToolCollection,Knife,0.0,19.0,True,False,Fake,Report #5309
SimpleToolCollection,Knife,0.0,19.0,True,False,Fake,
SimpleToolCollection,Knife,0.0,19.0,True,False,,Report #5310
SimpleToolCollection,Knife,0.0,19.0,True,False,,
SimpleToolCollection,Knife,0.0,19.0,False,True,Genuine,Report #5311
SimpleToolCollection,Knife,0.0,19.0,False,True,Genuine,
SimpleToolCollection,Knife,0.0,19.0,False,True,Fake,Report #5312
SimpleToolCollection,Knife,0.0,19.0,False,True,Fake,
SimpleToolCollection,Knife,0.0,19.0,False,True,,Report #5313
SimpleToolCollection,Knife,0.0,19.0,False,True,,
SimpleToolCollection,Knife,0.0,19.0,False,False,Genuine,Report #5314
SimpleToolCollection,Knife,0.0,19.0,False,False,Genuine,
SimpleToolCollection,Knife,0.0,19.0,False,False,Fake,Report #5315
SimpleToolCollection,Knife,0.0,19.0,False,False,Fake,
SimpleToolCollection,Knife,0.0,19.0,False,False,,Report #5316
SimpleToolCollection,Knife,0.0,19.0,False,False,,
SimpleToolCollection,Knife,0.0,20.0,True,True,Genuine,Report #5317
SimpleToolCollection,Knife,0.0,20.0,True,True,Genuine,
SimpleToolCollection,Knife,0.0,20.0,True,True,Fake,Report #5318
SimpleToolCollection,Knife,0.0,20.0,True,True,Fake,
SimpleToolCollection,Knife,0.0,20.0,True,True,,Report #5319
SimpleToolCollection,Knife,0.0,20.0,True,True,,
SimpleToolCollection,Knife,0.0,20.0,True,False,Genuine,Report #5320
SimpleToolCollection,Knife,0.0,20.0,True,False,Genuine,
SimpleToolCollection,Knife,0.0,20.0,True,False,Fake,Report #5321
SimpleToolCollection,Knife,0.0,20.0,True,False,Fake,
SimpleToolCollection,Knife,0.0,20.0,True,False,,Report #5322
SimpleToolCollection,Knife,0.0,20.0,True,False,,
SimpleToolCollection,Knife,0.0,20.0,False,True,Genuine,Report #5323
SimpleToolCollection,Knife,0.0,20.0,False,True,Genuine,
SimpleToolCollection,Knife,0.0,20.0,False,True,Fake,Report #5324
SimpleToolCollection,Knife,0.0,20.0,False,True,Fake,
SimpleToolCollection,Knife,0.0,20.0,False,True,,Report #5325
SimpleToolCollection,Knife,0.0,20.0,False,True,,
SimpleToolCollection,Knife,0.0,20.0,False,False,Genuine,Report #5326
SimpleToolCollection,Knife,0.0,20.0,False,False,Genuine,
SimpleToolCollection,Knife,0.0,20.0,False,False,Fake,Report #5327
SimpleToolCollection,Knife,0.0,20.0,False,False,Fake,
SimpleToolCollection,Knife,0.0,20.0,False,False,,Report #5328
SimpleToolCollection,Knife,0.0,20.0,False,False,,
SimpleToolCollection,Knife,0.0,21.0,True,True,Genuine,Report #5329
SimpleToolCollection,Knife,0.0,21.0,True,True,Genuine,
SimpleToolCollection,Knife,0.0,21.0,True,True,Fake,Report #5330
SimpleToolCollection,Knife,0.0,21.0,True,True,Fake,
SimpleToolCollection,Knife,0.0,21.0,True,True,,Report #5331
SimpleToolCollection,Knife,0.0,21.0,True,True,,
SimpleToolCollection,Knife,0.0,21.0,True,False,Genuine,Report #5332
SimpleToolCollection,Knife,0.0,21.0,True,False,Genuine,
SimpleToolCollection,Knife,0.0,21.0,True,False,Fake,Report #5333
SimpleToolCollection,Knife,0.0,21.0,True,False,Fake,
SimpleToolCollection,Knife,0.0,21.0,True,False,,Report #5334
SimpleToolCollection,Knife,0.0,21.0,True,False,,
SimpleToolCollection,Knife,0.0,21.0,False,True,Genuine,Report #5335
SimpleToolCollection,Knife,0.0,21.0,False,True,Genuine,
SimpleToolCollection,Knife,0.0,21.0,False,True,Fake,Report #5336
SimpleToolCollection,Knife,0.0,21.0,False,True,Fake,
SimpleToolCollection,Knife,0.0,21.0,False,True,,Report #5337
SimpleToolCollection,Knife,0.0,21.0,False,True,,
SimpleToolCollection,Knife,0.0,21.0,False,False,Genuine,Report #5338
SimpleToolCollection,Knife,0.0,21.0,False,False,Genuine,
SimpleToolCollection,Knife,0.0,21.0,False,False,Fake,Report #5339
SimpleToolCollection,Knife,0.0,21.0,False,False,Fake,
SimpleToolCollection,Knife,0.0,21.0,False,False,,Report #5340
SimpleToolCollection,Knife,0.0,21.0,False,False,,
SimpleToolCollection,Knife,0.0,99.0,True,True,Genuine,Report #5341
SimpleToolCollection,Knife,0.0,99.0,True,True,Genuine,
SimpleToolCollection,Knife,0.0,99.0,True,True,Fake,Report #5342
SimpleToolCollection,Knife,0.0,99.0,True,True,Fake,
SimpleToolCollection,Knife,0.0,99.0,True,True,,Report #5343
SimpleToolCollection,Knife,0.0,99.0,True,True,,
SimpleToolCollection,Knife,0.0,99.0,True,False,Genuine,Report #5344
SimpleToolCollection,Knife,0.0,99.0,True,False,Genuine,
SimpleToolCollection,Knife,0.0,99.0,True,False,Fake,Report #5345
SimpleToolCollection,Knife,0.0,99.0,True,False,Fake,
SimpleToolCollection,Knife,0.0,99.0,True,False,,Report #5346
SimpleToolCollection,Knife,0.0,99.0,True,False,,
SimpleToolCollection,Knife,0.0,99.0,False,True,Genuine,Report #5347
SimpleToolCollection,Knife,0.0,99.0,False,True,Genuine,
SimpleToolCollection,Knife,0.0,99.0,False,True,Fake,Report #5348
SimpleToolCollection,Knife,0.0,99.0,False,True,Fake,
SimpleToolCollection,Knife,0.0,99.0,False,True,,Report #5349
SimpleToolCollection,Knife,0.0,99.0,False,True,,
SimpleToolCollection,Knife,0.0,99.0,False,False,Genuine,Report #5350
SimpleToolCollection,Knife,0.0,99.0,False,False,Genuine,
SimpleToolCollection,Knife,0.0,99.0,False,False,Fake,Report #5351
SimpleToolCollection,Knife,0.0,99.0,False,False,Fake,
SimpleToolCollection,Knife,0.0,99.0,False,False,,Report #5352
SimpleToolCollection,Knife,0.0,99.0,False,False,,
SimpleToolCollection,Knife,0.0,100.0,True,True,Genuine,Report #5353
SimpleToolCollection,Knife,0.0,100.0,True,True,Genuine,
SimpleToolCollection,Knife,0.0,100.0,True,True,Fake,Report #5354
SimpleToolCollection,Knife,0.0,100.0,True,True,Fake,
SimpleToolCollection,Knife,0.0,100.0,True,True,,Report #5355
SimpleToolCollection,Knife,0.0,100.0,True,True,,
SimpleToolCollection,Knife,0.0,100.0,True,False,Genuine,Report #5356
SimpleToolCollection,Knife,0.0,100.0,True,False,Genuine,
SimpleToolCollection,Knife,0.0,100.0,True,False,Fake,Report #5357
SimpleToolCollection,Knife,0.0,100.0,True,False,Fake,
SimpleToolCollection,Knife,0.0,100.0,True,False,,Report #5358
SimpleToolCollection,Knife,0.0,100.0,True,False,,
SimpleToolCollection,Knife,0.0,100.0,False,True,Genuine,Report #5359
SimpleToolCollection,Knife,0.0,100.0,False,True,Genuine,
SimpleToolCollection,Knife,0.0,100.0,False,True,Fake,Report #5360
SimpleToolCollection,Knife,0.0,100.0,False,True,Fake,
SimpleToolCollection,Knife,0.0,100.0,False,True,,Report #5361
SimpleToolCollection,Knife,0.0,100.0,False,True,,
SimpleToolCollection,Knife,0.0,100.0,False,False,Genuine,Report #5362
SimpleToolCollection,Knife,0.0,100.0,False,False,Genuine,
SimpleToolCollection,Knife,0.0,100.0,False,False,Fake,Report #5363
SimpleToolCollection,Knife,0.0,100.0,False,False,Fake,
SimpleToolCollection,Knife,0.0,100.0,False,False,,Report #5364
SimpleToolCollection,Knife,0.0,100.0,False,False,,
SimpleToolCollection,Knife,0.0,101.0,True,True,Genuine,Report #5365
SimpleToolCollection,Knife,0.0,101.0,True,True,Genuine,
SimpleToolCollection,Knife,0.0,101.0,True,True,Fake,Report #5366
SimpleToolCollection,Knife,0.0,101.0,True,True,Fake,
SimpleToolCollection,Knife,0.0,101.0,True,True,,Report #5367
SimpleToolCollection,Knife,0.0,101.0,True,True,,
SimpleToolCollection,Knife,0.0,101.0,True,False,Genuine,Report #5368
SimpleToolCollection,Knife,0.0,101.0,True,False,Genuine,
SimpleToolCollection,Knife,0.0,101.0,True,False,Fake,Report #5369
SimpleToolCollection,Knife,0.0,101.0,True,False,Fake,
SimpleToolCollection,Knife,0.0,101.0,True,False,,Report #5370
SimpleToolCollection,Knife,0.0,101.0,True,False,,
SimpleToolCollection,Knife,0.0,101.0,False,True,Genuine,Report #5371
SimpleToolCollection,Knife,0.0,101.0,False,True,Genuine,
SimpleToolCollection,Knife,0.0,101.0,False,True,Fake,Report #5372
SimpleToolCollection,Knife,0.0,101.0,False,True,Fake,
SimpleToolCollection,Knife,0.0,101.0,False,True,,Report #5373
SimpleToolCollection,Knife,0.0,101.0,False,True,,
SimpleToolCollection,Knife,0.0,101.0,False,False,Genuine,Report #5374
SimpleToolCollection,Knife,0.0,101.0,False,False,Genuine,
SimpleToolCollection,Knife,0.0,101.0,False,False,Fake,Report #5375
SimpleToolCollection,Knife,0.0,101.0,False,False,Fake,
SimpleToolCollection,Knife,0.0,101.0,False,False,,Report #5376
SimpleToolCollection,Knife,0.0,101.0,False,False,,
SimpleToolCollection,Knife,1.0,-1.0,True,True,Genuine,Report #5377
SimpleToolCollection,Knife,1.0,-1.0,True,True,Genuine,
SimpleToolCollection,Knife,1.0,-1.0,True,True,Fake,Report #5378
SimpleToolCollection,Knife,1.0,-1.0,True,True,Fake,
SimpleToolCollection,Knife,1.0,-1.0,True,True,,Report #5379
SimpleToolCollection,Knife,1.0,-1.0,True,True,,
SimpleToolCollection,Knife,1.0,-1.0,True,False,Genuine,Report #5380
SimpleToolCollection,Knife,1.0,-1.0,True,False,Genuine,
SimpleToolCollection,Knife,1.0,-1.0,True,False,Fake,Report #5381
SimpleToolCollection,Knife,1.0,-1.0,True,False,Fake,
SimpleToolCollection,Knife,1.0,-1.0,True,False,,Report #5382
SimpleToolCollection,Knife,1.0,-1.0,True,False,,
SimpleToolCollection,Knife,1.0,-1.0,False,True,Genuine,Report #5383
SimpleToolCollection,Knife,1.0,-1.0,False,True,Genuine,
SimpleToolCollection,Knife,1.0,-1.0,False,True,Fake,Report #5384
SimpleToolCollection,Knife,1.0,-1.0,False,True,Fake,
SimpleToolCollection,Knife,1.0,-1.0,False,True,,Report #5385
SimpleToolCollection,Knife,1.0,-1.0,False,True,,
SimpleToolCollection,Knife,1.0,-1.0,False,False,Genuine,Report #5386
SimpleToolCollection,Knife,1.0,-1.0,False,False,Genuine,
SimpleToolCollection,Knife,1.0,-1.0,False,False,Fake,Report #5387
SimpleToolCollection,Knife,1.0,-1.0,False,False,Fake,
SimpleToolCollection,Knife,1.0,-1.0,False,False,,Report #5388
SimpleToolCollection,Knife,1.0,-1.0,False,False,,
SimpleToolCollection,Knife,1.0,0.0,True,True,Genuine,Report #5389
SimpleToolCollection,Knife,1.0,0.0,True,True,Genuine,
SimpleToolCollection,Knife,1.0,0.0,True,True,Fake,Report #5390
SimpleToolCollection,Knife,1.0,0.0,True,True,Fake,
SimpleToolCollection,Knife,1.0,0.0,True,True,,Report #5391
SimpleToolCollection,Knife,1.0,0.0,True,True,,
SimpleToolCollection,Knife,1.0,0.0,True,False,Genuine,Report #5392
SimpleToolCollection,Knife,1.0,0.0,True,False,Genuine,
SimpleToolCollection,Knife,1.0,0.0,True,False,Fake,Report #5393
SimpleToolCollection,Knife,1.0,0.0,True,False,Fake,
SimpleToolCollection,Knife,1.0,0.0,True,False,,Report #5394
SimpleToolCollection,Knife,1.0,0.0,True,False,,
SimpleToolCollection,Knife,1.0,0.0,False,True,Genuine,Report #5395
SimpleToolCollection,Knife,1.0,0.0,False,True,Genuine,
SimpleToolCollection,Knife,1.0,0.0,False,True,Fake,Report #5396
SimpleToolCollection,Knife,1.0,0.0,False,True,Fake,
SimpleToolCollection,Knife,1.0,0.0,False,True,,Report #5397
SimpleToolCollection,Knife,1.0,0.0,False,True,,
SimpleToolCollection,Knife,1.0,0.0,False,False,Genuine,Report #5398
SimpleToolCollection,Knife,1.0,0.0,False,False,Genuine,
SimpleToolCollection,Knife,1.0,0.0,False,False,Fake,Report #5399
SimpleToolCollection,Knife,1.0,0.0,False,False,Fake,
SimpleToolCollection,Knife,1.0,0.0,False,False,,Report #5400
SimpleToolCollection,Knife,1.0,0.0,False,False,,
SimpleToolCollection,Knife,1.0,19.0,True,True,Genuine,Report #5401
SimpleToolCollection,Knife,1.0,19.0,True,True,Genuine,
SimpleToolCollection,Knife,1.0,19.0,True,True,Fake,Report #5402
SimpleToolCollection,Knife,1.0,19.0,True,True,Fake,
SimpleToolCollection,Knife,1.0,19.0,True,True,,Report #5403
SimpleToolCollection,Knife,1.0,19.0,True,True,,
SimpleToolCollection,Knife,1.0,19.0,True,False,Genuine,Report #5404
SimpleToolCollection,Knife,1.0,19.0,True,False,Genuine,
SimpleToolCollection,Knife,1.0,19.0,True,False,Fake,Report #5405
SimpleToolCollection,Knife,1.0,19.0,True,False,Fake,
SimpleToolCollection,Knife,1.0,19.0,True,False,,Report #5406
SimpleToolCollection,Knife,1.0,19.0,True,False,,
SimpleToolCollection,Knife,1.0,19.0,False,True,Genuine,Report #5407
SimpleToolCollection,Knife,1.0,19.0,False,True,Genuine,
SimpleToolCollection,Knife,1.0,19.0,False,True,Fake,Report #5408
SimpleToolCollection,Knife,1.0,19.0,False,True,Fake,
SimpleToolCollection,Knife,1.0,19.0,False,True,,Report #5409
SimpleToolCollection,Knife,1.0,19.0,False,True,,
SimpleToolCollection,Knife,1.0,19.0,False,False,Genuine,Report #5410
SimpleToolCollection,Knife,1.0,19.0,False,False,Genuine,
SimpleToolCollection,Knife,1.0,19.0,False,False,Fake,Report #5411
SimpleToolCollection,Knife,1.0,19.0,False,False,Fake,
SimpleToolCollection,Knife,1.0,19.0,False,False,,Report #5412
SimpleToolCollection,Knife,1.0,19.0,False,False,,
SimpleToolCollection,Knife,1.0,20.0,True,True,Genuine,Report #5413
SimpleToolCollection,Knife,1.0,20.0,True,True,Genuine,
SimpleToolCollection,Knife,1.0,20.0,True,True,Fake,Report #5414
SimpleToolCollection,Knife,1.0,20.0,True,True,Fake,
SimpleToolCollection,Knife,1.0,20.0,True,True,,Report #5415
SimpleToolCollection,Knife,1.0,20.0,True,True,,
SimpleToolCollection,Knife,1.0,20.0,True,False,Genuine,Report #5416
SimpleToolCollection,Knife,1.0,20.0,True,False,Genuine,
SimpleToolCollection,Knife,1.0,20.0,True,False,Fake,Report #5417
SimpleToolCollection,Knife,1.0,20.0,True,False,Fake,
SimpleToolCollection,Knife,1.0,20.0,True,False,,Report #5418
SimpleToolCollection,Knife,1.0,20.0,True,False,,
SimpleToolCollection,Knife,1.0,20.0,False,True,Genuine,Report #5419
SimpleToolCollection,Knife,1.0,20.0,False,True,Genuine,
SimpleToolCollection,Knife,1.0,20.0,False,True,Fake,Report #5420
SimpleToolCollection,Knife,1.0,20.0,False,True,Fake,
SimpleToolCollection,Knife,1.0,20.0,False,True,,Report #5421
SimpleToolCollection,Knife,1.0,20.0,False,True,,
SimpleToolCollection,Knife,1.0,20.0,False,False,Genuine,Report #5422
SimpleToolCollection,Knife,1.0,20.0,False,False,Genuine,
SimpleToolCollection,Knife,1.0,20.0,False,False,Fake,Report #5423
SimpleToolCollection,Knife,1.0,20.0,False,False,Fake,
SimpleToolCollection,Knife,1.0,20.0,False,False,,Report #5424
SimpleToolCollection,Knife,1.0,20.0,False,False,,
SimpleToolCollection,Knife,1.0,21.0,True,True,Genuine,Report #5425
SimpleToolCollection,Knife,1.0,21.0,True,True,Genuine,
SimpleToolCollection,Knife,1.0,21.0,True,True,Fake,Report #5426
SimpleToolCollection,Knife,1.0,21.0,True,True,Fake,
SimpleToolCollection,Knife,1.0,21.0,True,True,,Report #5427
SimpleToolCollection,Knife,1.0,21.0,True,True,,
SimpleToolCollection,Knife,1.0,21.0,True,False,Genuine,Report #5428
SimpleToolCollection,Knife,1.0,21.0,True,False,Genuine,
SimpleToolCollection,Knife,1.0,21.0,True,False,Fake,Report #5429
SimpleToolCollection,Knife,1.0,21.0,True,False,Fake,
SimpleToolCollection,Knife,1.0,21.0,True,False,,Report #5430
SimpleToolCollection,Knife,1.0,21.0,True,False,,
SimpleToolCollection,Knife,1.0,21.0,False,True,Genuine,Report #5431
SimpleToolCollection,Knife,1.0,21.0,False,True,Genuine,
SimpleToolCollection,Knife,1.0,21.0,False,True,Fake,Report #5432
SimpleToolCollection,Knife,1.0,21.0,False,True,Fake,
SimpleToolCollection,Knife,1.0,21.0,False,True,,Report #5433
SimpleToolCollection,Knife,1.0,21.0,False,True,,
SimpleToolCollection,Knife,1.0,21.0,False,False,Genuine,Report #5434
SimpleToolCollection,Knife,1.0,21.0,False,False,Genuine,
SimpleToolCollection,Knife,1.0,21.0,False,False,Fake,Report #5435
SimpleToolCollection,Knife,1.0,21.0,False,False,Fake,
SimpleToolCollection,Knife,1.0,21.0,False,False,,Report #5436
SimpleToolCollection,Knife,1.0,21.0,False,False,,
SimpleToolCollection,Knife,1.0,99.0,True,True,Genuine,Report #5437
SimpleToolCollection,Knife,1.0,99.0,True,True,Genuine,
SimpleToolCollection,Knife,1.0,99.0,True,True,Fake,Report #5438
SimpleToolCollection,Knife,1.0,99.0,True,True,Fake,
SimpleToolCollection,Knife,1.0,99.0,True,True,,Report #5439
SimpleToolCollection,Knife,1.0,99.0,True,True,,
SimpleToolCollection,Knife,1.0,99.0,True,False,Genuine,Report #5440
SimpleToolCollection,Knife,1.0,99.0,True,False,Genuine,
SimpleToolCollection,Knife,1.0,99.0,True,False,Fake,Report #5441
SimpleToolCollection,Knife,1.0,99.0,True,False,Fake,
SimpleToolCollection,Knife,1.0,99.0,True,False,,Report #5442
SimpleToolCollection,Knife,1.0,99.0,True,False,,
SimpleToolCollection,Knife,1.0,99.0,False,True,Genuine,Report #5443
SimpleToolCollection,Knife,1.0,99.0,False,True,Genuine,
SimpleToolCollection,Knife,1.0,99.0,False,True,Fake,Report #5444
SimpleToolCollection,Knife,1.0,99.0,False,True,Fake,
SimpleToolCollection,Knife,1.0,99.0,False,True,,Report #5445
SimpleToolCollection,Knife,1.0,99.0,False,True,,
SimpleToolCollection,Knife,1.0,99.0,False,False,Genuine,Report #5446
SimpleToolCollection,Knife,1.0,99.0,False,False,Genuine,
SimpleToolCollection,Knife,1.0,99.0,False,False,Fake,Report #5447
SimpleToolCollection,Knife,1.0,99.0,False,False,Fake,
SimpleToolCollection,Knife,1.0,99.0,False,False,,Report #5448
SimpleToolCollection,Knife,1.0,99.0,False,False,,
SimpleToolCollection,Knife,1.0,100.0,True,True,Genuine,Report #5449
SimpleToolCollection,Knife,1.0,100.0,True,True,Genuine,
SimpleToolCollection,Knife,1.0,100.0,True,True,Fake,Report #5450
SimpleToolCollection,Knife,1.0,100.0,True,True,Fake,
SimpleToolCollection,Knife,1.0,100.0,True,True,,Report #5451
SimpleToolCollection,Knife,1.0,100.0,True,True,,
SimpleToolCollection,Knife,1.0,100.0,True,False,Genuine,Report #5452
SimpleToolCollection,Knife,1.0,100.0,True,False,Genuine,
SimpleToolCollection,Knife,1.0,100.0,True,False,Fake,Report #5453
SimpleToolCollection,Knife,1.0,100.0,True,False,Fake,
SimpleToolCollection,Knife,1.0,100.0,True,False,,Report #5454
SimpleToolCollection,Knife,1.0,100.0,True,False,,
SimpleToolCollection,Knife,1.0,100.0,False,True,Genuine,Report #5455
SimpleToolCollection,Knife,1.0,100.0,False,True,Genuine,
SimpleToolCollection,Knife,1.0,100.0,False,True,Fake,Report #5456
SimpleToolCollection,Knife,1.0,100.0,False,True,Fake,
SimpleToolCollection,Knife,1.0,100.0,False,True,,Report #5457
SimpleToolCollection,Knife,1.0,100.0,False,True,,
SimpleToolCollection,Knife,1.0,100.0,False,False,Genuine,Report #5458
SimpleToolCollection,Knife,1.0,100.0,False,False,Genuine,
SimpleToolCollection,Knife,1.0,100.0,False,False,Fake,Report #5459
SimpleToolCollection,Knife,1.0,100.0,False,False,Fake,
SimpleToolCollection,Knife,1.0,100.0,False,False,,Report #5460
SimpleToolCollection,Knife,1.0,100.0,False,False,,
SimpleToolCollection,Knife,1.0,101.0,True,True,Genuine,Report #5461
SimpleToolCollection,Knife,1.0,101.0,True,True,Genuine,
SimpleToolCollection,Knife,1.0,101.0,True,True,Fake,Report #5462
SimpleToolCollection,Knife,1.0,101.0,True,True,Fake,
SimpleToolCollection,Knife,1.0,101.0,True,True,,Report #5463
SimpleToolCollection,Knife,1.0,101.0,True,True,,
SimpleToolCollection,Knife,1.0,101.0,True,False,Genuine,Report #5464
SimpleToolCollection,Knife,1.0,101.0,True,False,Genuine,
SimpleToolCollection,Knife,1.0,101.0,True,False,Fake,Report #5465
SimpleToolCollection,Knife,1.0,101.0,True,False,Fake,
SimpleToolCollection,Knife,1.0,101.0,True,False,,Report #5466
SimpleToolCollection,Knife,1.0,101.0,True,False,,
SimpleToolCollection,Knife,1.0,101.0,False,True,Genuine,Report #5467
SimpleToolCollection,Knife,1.0,101.0,False,True,Genuine,
SimpleToolCollection,Knife,1.0,101.0,False,True,Fake,Report #5468
SimpleToolCollection,Knife,1.0,101.0,False,True,Fake,
SimpleToolCollection,Knife,1.0,101.0,False,True,,Report #5469
SimpleToolCollection,Knife,1.0,101.0,False,True,,
SimpleToolCollection,Knife,1.0,101.0,False,False,Genuine,Report #5470
SimpleToolCollection,Knife,1.0,101.0,False,False,Genuine,
SimpleToolCollection,Knife,1.0,101.0,False,False,Fake,Report #5471
SimpleToolCollection,Knife,1.0,101.0,False,False,Fake,
SimpleToolCollection,Knife,1.0,101.0,False,False,,Report #5472
SimpleToolCollection,Knife,1.0,101.0,False,False,,
SimpleToolCollection,Knife,100.0,-1.0,True,True,Genuine,Report #5473
SimpleToolCollection,Knife,100.0,-1.0,True,True,Genuine,
SimpleToolCollection,Knife,100.0,-1.0,True,True,Fake,Report #5474
SimpleToolCollection,Knife,100.0,-1.0,True,True,Fake,
SimpleToolCollection,Knife,100.0,-1.0,True,True,,Report #5475
SimpleToolCollection,Knife,100.0,-1.0,True,True,,
SimpleToolCollection,Knife,100.0,-1.0,True,False,Genuine,Report #5476
SimpleToolCollection,Knife,100.0,-1.0,True,False,Genuine,
SimpleToolCollection,Knife,100.0,-1.0,True,False,Fake,Report #5477
SimpleToolCollection,Knife,100.0,-1.0,True,False,Fake,
SimpleToolCollection,Knife,100.0,-1.0,True,False,,Report #5478
SimpleToolCollection,Knife,100.0,-1.0,True,False,,
SimpleToolCollection,Knife,100.0,-1.0,False,True,Genuine,Report #5479
SimpleToolCollection,Knife,100.0,-1.0,False,True,Genuine,
SimpleToolCollection,Knife,100.0,-1.0,False,True,Fake,Report #5480
SimpleToolCollection,Knife,100.0,-1.0,False,True,Fake,
SimpleToolCollection,Knife,100.0,-1.0,False,True,,Report #5481
SimpleToolCollection,Knife,100.0,-1.0,False,True,,
SimpleToolCollection,Knife,100.0,-1.0,False,False,Genuine,Report #5482
SimpleToolCollection,Knife,100.0,-1.0,False,False,Genuine,
SimpleToolCollection,Knife,100.0,-1.0,False,False,Fake,Report #5483
SimpleToolCollection,Knife,100.0,-1.0,False,False,Fake,
SimpleToolCollection,Knife,100.0,-1.0,False,False,,Report #5484
SimpleToolCollection,Knife,100.0,-1.0,False,False,,
SimpleToolCollection,Knife,100.0,0.0,True,True,Genuine,Report #5485
SimpleToolCollection,Knife,100.0,0.0,True,True,Genuine,
SimpleToolCollection,Knife,100.0,0.0,True,True,Fake,Report #5486
SimpleToolCollection,Knife,100.0,0.0,True,True,Fake,
SimpleToolCollection,Knife,100.0,0.0,True,True,,Report #5487
SimpleToolCollection,Knife,100.0,0.0,True,True,,
SimpleToolCollection,Knife,100.0,0.0,True,False,Genuine,Report #5488
SimpleToolCollection,Knife,100.0,0.0,True,False,Genuine,
SimpleToolCollection,Knife,100.0,0.0,True,False,Fake,Report #5489
SimpleToolCollection,Knife,100.0,0.0,True,False,Fake,
SimpleToolCollection,Knife,100.0,0.0,True,False,,Report #5490
SimpleToolCollection,Knife,100.0,0.0,True,False,,
SimpleToolCollection,Knife,100.0,0.0,False,True,Genuine,Report #5491
SimpleToolCollection,Knife,100.0,0.0,False,True,Genuine,
SimpleToolCollection,Knife,100.0,0.0,False,True,Fake,Report #5492
SimpleToolCollection,Knife,100.0,0.0,False,True,Fake,
SimpleToolCollection,Knife,100.0,0.0,False,True,,Report #5493
SimpleToolCollection,Knife,100.0,0.0,False,True,,
SimpleToolCollection,Knife,100.0,0.0,False,False,Genuine,Report #5494
SimpleToolCollection,Knife,100.0,0.0,False,False,Genuine,
SimpleToolCollection,Knife,100.0,0.0,False,False,Fake,Report #5495
SimpleToolCollection,Knife,100.0,0.0,False,False,Fake,
SimpleToolCollection,Knife,100.0,0.0,False,False,,Report #5496
SimpleToolCollection,Knife,100.0,0.0,False,False,,
SimpleToolCollection,Knife,100.0,19.0,True,True,Genuine,Report #5497
SimpleToolCollection,Knife,100.0,19.0,True,True,Genuine,
SimpleToolCollection,Knife,100.0,19.0,True,True,Fake,Report #5498
SimpleToolCollection,Knife,100.0,19.0,True,True,Fake,
SimpleToolCollection,Knife,100.0,19.0,True,True,,Report #5499
SimpleToolCollection,Knife,100.0,19.0,True,True,,
SimpleToolCollection,Knife,100.0,19.0,True,False,Genuine,Report #5500
SimpleToolCollection,Knife,100.0,19.0,True,False,Genuine,
SimpleToolCollection,Knife,100.0,19.0,True,False,Fake,Report #5501
SimpleToolCollection,Knife,100.0,19.0,True,False,Fake,
SimpleToolCollection,Knife,100.0,19.0,True,False,,Report #5502
SimpleToolCollection,Knife,100.0,19.0,True,False,,
SimpleToolCollection,Knife,100.0,19.0,False,True,Genuine,Report #5503
SimpleToolCollection,Knife,100.0,19.0,False,True,Genuine,
SimpleToolCollection,Knife,100.0,19.0,False,True,Fake,Report #5504
SimpleToolCollection,Knife,100.0,19.0,False,True,Fake,
SimpleToolCollection,Knife,100.0,19.0,False,True,,Report #5505
SimpleToolCollection,Knife,100.0,19.0,False,True,,
SimpleToolCollection,Knife,100.0,19.0,False,False,Genuine,Report #5506
SimpleToolCollection,Knife,100.0,19.0,False,False,Genuine,
SimpleToolCollection,Knife,100.0,19.0,False,False,Fake,Report #5507
SimpleToolCollection,Knife,100.0,19.0,False,False,Fake,
SimpleToolCollection,Knife,100.0,19.0,False,False,,Report #5508
SimpleToolCollection,Knife,100.0,19.0,False,False,,
SimpleToolCollection,Knife,100.0,20.0,True,True,Genuine,Report #5509
SimpleToolCollection,Knife,100.0,20.0,True,True,Genuine,
SimpleToolCollection,Knife,100.0,20.0,True,True,Fake,Report #5510
SimpleToolCollection,Knife,100.0,20.0,True,True,Fake,
SimpleToolCollection,Knife,100.0,20.0,True,True,,Report #5511
SimpleToolCollection,Knife,100.0,20.0,True,True,,
SimpleToolCollection,Knife,100.0,20.0,True,False,Genuine,Report #5512
SimpleToolCollection,Knife,100.0,20.0,True,False,Genuine,
SimpleToolCollection,Knife,100.0,20.0,True,False,Fake,Report #5513
SimpleToolCollection,Knife,100.0,20.0,True,False,Fake,
SimpleToolCollection,Knife,100.0,20.0,True,False,,Report #5514
SimpleToolCollection,Knife,100.0,20.0,True,False,,
SimpleToolCollection,Knife,100.0,20.0,False,True,Genuine,Report #5515
SimpleToolCollection,Knife,100.0,20.0,False,True,Genuine,
SimpleToolCollection,Knife,100.0,20.0,False,True,Fake,Report #5516
SimpleToolCollection,Knife,100.0,20.0,False,True,Fake,
SimpleToolCollection,Knife,100.0,20.0,False,True,,Report #5517
SimpleToolCollection,Knife,100.0,20.0,False,True,,
SimpleToolCollection,Knife,100.0,20.0,False,False,Genuine,Report #5518
SimpleToolCollection,Knife,100.0,20.0,False,False,Genuine,
SimpleToolCollection,Knife,100.0,20.0,False,False,Fake,Report #5519
SimpleToolCollection,Knife,100.0,20.0,False,False,Fake,
SimpleToolCollection,Knife,100.0,20.0,False,False,,Report #5520
SimpleToolCollection,Knife,100.0,20.0,False,False,,
SimpleToolCollection,Knife,100.0,21.0,True,True,Genuine,Report #5521
SimpleToolCollection,Knife,100.0,21.0,True,True,Genuine,
SimpleToolCollection,Knife,100.0,21.0,True,True,Fake,Report #5522
SimpleToolCollection,Knife,100.0,21.0,True,True,Fake,
SimpleToolCollection,Knife,100.0,21.0,True,True,,Report #5523
SimpleToolCollection,Knife,100.0,21.0,True,True,,
SimpleToolCollection,Knife,100.0,21.0,True,False,Genuine,Report #5524
SimpleToolCollection,Knife,100.0,21.0,True,False,Genuine,
SimpleToolCollection,Knife,100.0,21.0,True,False,Fake,Report #5525
SimpleToolCollection,Knife,100.0,21.0,True,False,Fake,
SimpleToolCollection,Knife,100.0,21.0,True,False,,Report #5526
SimpleToolCollection,Knife,100.0,21.0,True,False,,
SimpleToolCollection,Knife,100.0,21.0,False,True,Genuine,Report #5527
SimpleToolCollection,Knife,100.0,21.0,False,True,Genuine,
SimpleToolCollection,Knife,100.0,21.0,False,True,Fake,Report #5528
SimpleToolCollection,Knife,100.0,21.0,False,True,Fake,
SimpleToolCollection,Knife,100.0,21.0,False,True,,Report #5529
SimpleToolCollection,Knife,100.0,21.0,False,True,,
SimpleToolCollection,Knife,100.0,21.0,False,False,Genuine,Report #5530
SimpleToolCollection,Knife,100.0,21.0,False,False,Genuine,
SimpleToolCollection,Knife,100.0,21.0,False,False,Fake,Report #5531
SimpleToolCollection,Knife,100.0,21.0,False,False,Fake,
SimpleToolCollection,Knife,100.0,21.0,False,False,,Report #5532
SimpleToolCollection,Knife,100.0,21.0,False,False,,
SimpleToolCollection,Knife,100.0,99.0,True,True,Genuine,Report #5533
SimpleToolCollection,Knife,100.0,99.0,True,True,Genuine,
SimpleToolCollection,Knife,100.0,99.0,True,True,Fake,Report #5534
SimpleToolCollection,Knife,100.0,99.0,True,True,Fake,
SimpleToolCollection,Knife,100.0,99.0,True,True,,Report #5535
SimpleToolCollection,Knife,100.0,99.0,True,True,,
SimpleToolCollection,Knife,100.0,99.0,True,False,Genuine,Report #5536
SimpleToolCollection,Knife,100.0,99.0,True,False,Genuine,
SimpleToolCollection,Knife,100.0,99.0,True,False,Fake,Report #5537
SimpleToolCollection,Knife,100.0,99.0,True,False,Fake,
SimpleToolCollection,Knife,100.0,99.0,True,False,,Report #5538
SimpleToolCollection,Knife,100.0,99.0,True,False,,
SimpleToolCollection,Knife,100.0,99.0,False,True,Genuine,Report #5539
SimpleToolCollection,Knife,100.0,99.0,False,True,Genuine,
SimpleToolCollection,Knife,100.0,99.0,False,True,Fake,Report #5540
SimpleToolCollection,Knife,100.0,99.0,False,True,Fake,
SimpleToolCollection,Knife,100.0,99.0,False,True,,Report #5541
SimpleToolCollection,Knife,100.0,99.0,False,True,,
SimpleToolCollection,Knife,100.0,99.0,False,False,Genuine,Report #5542
SimpleToolCollection,Knife,100.0,99.0,False,False,Genuine,
SimpleToolCollection,Knife,100.0,99.0,False,False,Fake,Report #5543
SimpleToolCollection,Knife,100.0,99.0,False,False,Fake,
SimpleToolCollection,Knife,100.0,99.0,False,False,,Report #5544
SimpleToolCollection,Knife,100.0,99.0,False,False,,
SimpleToolCollection,Knife,100.0,100.0,True,True,Genuine,Report #5545
SimpleToolCollection,Knife,100.0,100.0,True,True,Genuine,
SimpleToolCollection,Knife,100.0,100.0,True,True,Fake,Report #5546
SimpleToolCollection,Knife,100.0,100.0,True,True,Fake,
SimpleToolCollection,Knife,100.0,100.0,True,True,,Report #5547
SimpleToolCollection,Knife,100.0,100.0,True,True,,
SimpleToolCollection,Knife,100.0,100.0,True,False,Genuine,Report #5548
SimpleToolCollection,Knife,100.0,100.0,True,False,Genuine,
SimpleToolCollection,Knife,100.0,100.0,True,False,Fake,Report #5549
SimpleToolCollection,Knife,100.0,100.0,True,False,Fake,
SimpleToolCollection,Knife,100.0,100.0,True,False,,Report #5550
SimpleToolCollection,Knife,100.0,100.0,True,False,,
SimpleToolCollection,Knife,100.0,100.0,False,True,Genuine,Report #5551
SimpleToolCollection,Knife,100.0,100.0,False,True,Genuine,
SimpleToolCollection,Knife,100.0,100.0,False,True,Fake,Report #5552
SimpleToolCollection,Knife,100.0,100.0,False,True,Fake,
SimpleToolCollection,Knife,100.0,100.0,False,True,,Report #5553
SimpleToolCollection,Knife,100.0,100.0,False,True,,
SimpleToolCollection,Knife,100.0,100.0,False,False,Genuine,Report #5554
SimpleToolCollection,Knife,100.0,100.0,False,False,Genuine,
SimpleToolCollection,Knife,100.0,100.0,False,False,Fake,Report #5555
SimpleToolCollection,Knife,100.0,100.0,False,False,Fake,
SimpleToolCollection,Knife,100.0,100.0,False,False,,Report #5556
SimpleToolCollection,Knife,100.0,100.0,False,False,,
SimpleToolCollection,Knife,100.0,101.0,True,True,Genuine,Report #5557
SimpleToolCollection,Knife,100.0,101.0,True,True,Genuine,
SimpleToolCollection,Knife,100.0,101.0,True,True,Fake,Report #5558
SimpleToolCollection,Knife,100.0,101.0,True,True,Fake,
SimpleToolCollection,Knife,100.0,101.0,True,True,,Report #5559
SimpleToolCollection,Knife,100.0,101.0,True,True,,
SimpleToolCollection,Knife,100.0,101.0,True,False,Genuine,Report #5560
SimpleToolCollection,Knife,100.0,101.0,True,False,Genuine,
SimpleToolCollection,Knife,100.0,101.0,True,False,Fake,Report #5561
SimpleToolCollection,Knife,100.0,101.0,True,False,Fake,
SimpleToolCollection,Knife,100.0,101.0,True,False,,Report #5562
SimpleToolCollection,Knife,100.0,101.0,True,False,,
SimpleToolCollection,Knife,100.0,101.0,False,True,Genuine,Report #5563
SimpleToolCollection,Knife,100.0,101.0,False,True,Genuine,
SimpleToolCollection,Knife,100.0,101.0,False,True,Fake,Report #5564
SimpleToolCollection,Knife,100.0,101.0,False,True,Fake,
SimpleToolCollection,Knife,100.0,101.0,False,True,,Report #5565
SimpleToolCollection,Knife,100.0,101.0,False,True,,
SimpleToolCollection,Knife,100.0,101.0,False,False,Genuine,Report #5566
SimpleToolCollection,Knife,100.0,101.0,False,False,Genuine,
SimpleToolCollection,Knife,100.0,101.0,False,False,Fake,Report #5567
SimpleToolCollection,Knife,100.0,101.0,False,False,Fake,
SimpleToolCollection,Knife,100.0,101.0,False,False,,Report #5568
SimpleToolCollection,Knife,100.0,101.0,False,False,,
SimpleToolCollection,Knife,1000.0,-1.0,True,True,Genuine,Report #5569
SimpleToolCollection,Knife,1000.0,-1.0,True,True,Genuine,
SimpleToolCollection,Knife,1000.0,-1.0,True,True,Fake,Report #5570
SimpleToolCollection,Knife,1000.0,-1.0,True,True,Fake,
SimpleToolCollection,Knife,1000.0,-1.0,True,True,,Report #5571
SimpleToolCollection,Knife,1000.0,-1.0,True,True,,
SimpleToolCollection,Knife,1000.0,-1.0,True,False,Genuine,Report #5572
SimpleToolCollection,Knife,1000.0,-1.0,True,False,Genuine,
SimpleToolCollection,Knife,1000.0,-1.0,True,False,Fake,Report #5573
SimpleToolCollection,Knife,1000.0,-1.0,True,False,Fake,
SimpleToolCollection,Knife,1000.0,-1.0,True,False,,Report #5574
SimpleToolCollection,Knife,1000.0,-1.0,True,False,,
SimpleToolCollection,Knife,1000.0,-1.0,False,True,Genuine,Report #5575
SimpleToolCollection,Knife,1000.0,-1.0,False,True,Genuine,
SimpleToolCollection,Knife,1000.0,-1.0,False,True,Fake,Report #5576
SimpleToolCollection,Knife,1000.0,-1.0,False,True,Fake,
SimpleToolCollection,Knife,1000.0,-1.0,False,True,,Report #5577
SimpleToolCollection,Knife,1000.0,-1.0,False,True,,
SimpleToolCollection,Knife,1000.0,-1.0,False,False,Genuine,Report #5578
SimpleToolCollection,Knife,1000.0,-1.0,False,False,Genuine,
SimpleToolCollection,Knife,1000.0,-1.0,False,False,Fake,Report #5579
SimpleToolCollection,Knife,1000.0,-1.0,False,False,Fake,
SimpleToolCollection,Knife,1000.0,-1.0,False,False,,Report #5580
SimpleToolCollection,Knife,1000.0,-1.0,False,False,,
SimpleToolCollection,Knife,1000.0,0.0,True,True,Genuine,Report #5581
SimpleToolCollection,Knife,1000.0,0.0,True,True,Genuine,
SimpleToolCollection,Knife,1000.0,0.0,True,True,Fake,Report #5582
SimpleToolCollection,Knife,1000.0,0.0,True,True,Fake,
SimpleToolCollection,Knife,1000.0,0.0,True,True,,Report #5583
SimpleToolCollection,Knife,1000.0,0.0,True,True,,
SimpleToolCollection,Knife,1000.0,0.0,True,False,Genuine,Report #5584
SimpleToolCollection,Knife,1000.0,0.0,True,False,Genuine,
SimpleToolCollection,Knife,1000.0,0.0,True,False,Fake,Report #5585
SimpleToolCollection,Knife,1000.0,0.0,True,False,Fake,
SimpleToolCollection,Knife,1000.0,0.0,True,False,,Report #5586
SimpleToolCollection,Knife,1000.0,0.0,True,False,,
SimpleToolCollection,Knife,1000.0,0.0,False,True,Genuine,Report #5587
SimpleToolCollection,Knife,1000.0,0.0,False,True,Genuine,
SimpleToolCollection,Knife,1000.0,0.0,False,True,Fake,Report #5588
SimpleToolCollection,Knife,1000.0,0.0,False,True,Fake,
SimpleToolCollection,Knife,1000.0,0.0,False,True,,Report #5589
SimpleToolCollection,Knife,1000.0,0.0,False,True,,
SimpleToolCollection,Knife,1000.0,0.0,False,False,Genuine,Report #5590
SimpleToolCollection,Knife,1000.0,0.0,False,False,Genuine,
SimpleToolCollection,Knife,1000.0,0.0,False,False,Fake,Report #5591
SimpleToolCollection,Knife,1000.0,0.0,False,False,Fake,
SimpleToolCollection,Knife,1000.0,0.0,False,False,,Report #5592
SimpleToolCollection,Knife,1000.0,0.0,False,False,,
SimpleToolCollection,Knife,1000.0,19.0,True,True,Genuine,Report #5593
SimpleToolCollection,Knife,1000.0,19.0,True,True,Genuine,
SimpleToolCollection,Knife,1000.0,19.0,True,True,Fake,Report #5594
SimpleToolCollection,Knife,1000.0,19.0,True,True,Fake,
SimpleToolCollection,Knife,1000.0,19.0,True,True,,Report #5595
SimpleToolCollection,Knife,1000.0,19.0,True,True,,
SimpleToolCollection,Knife,1000.0,19.0,True,False,Genuine,Report #5596
SimpleToolCollection,Knife,1000.0,19.0,True,False,Genuine,
SimpleToolCollection,Knife,1000.0,19.0,True,False,Fake,Report #5597
SimpleToolCollection,Knife,1000.0,19.0,True,False,Fake,
SimpleToolCollection,Knife,1000.0,19.0,True,False,,Report #5598
SimpleToolCollection,Knife,1000.0,19.0,True,False,,
SimpleToolCollection,Knife,1000.0,19.0,False,True,Genuine,Report #5599
SimpleToolCollection,Knife,1000.0,19.0,False,True,Genuine,
SimpleToolCollection,Knife,1000.0,19.0,False,True,Fake,Report #5600
SimpleToolCollection,Knife,1000.0,19.0,False,True,Fake,
SimpleToolCollection,Knife,1000.0,19.0,False,True,,Report #5601
SimpleToolCollection,Knife,1000.0,19.0,False,True,,
SimpleToolCollection,Knife,1000.0,19.0,False,False,Genuine,Report #5602
SimpleToolCollection,Knife,1000.0,19.0,False,False,Genuine,
SimpleToolCollection,Knife,1000.0,19.0,False,False,Fake,Report #5603
SimpleToolCollection,Knife,1000.0,19.0,False,False,Fake,
SimpleToolCollection,Knife,1000.0,19.0,False,False,,Report #5604
SimpleToolCollection,Knife,1000.0,19.0,False,False,,
SimpleToolCollection,Knife,1000.0,20.0,True,True,Genuine,Report #5605
SimpleToolCollection,Knife,1000.0,20.0,True,True,Genuine,
SimpleToolCollection,Knife,1000.0,20.0,True,True,Fake,Report #5606
SimpleToolCollection,Knife,1000.0,20.0,True,True,Fake,
SimpleToolCollection,Knife,1000.0,20.0,True,True,,Report #5607
SimpleToolCollection,Knife,1000.0,20.0,True,True,,
SimpleToolCollection,Knife,1000.0,20.0,True,False,Genuine,Report #5608
SimpleToolCollection,Knife,1000.0,20.0,True,False,Genuine,
SimpleToolCollection,Knife,1000.0,20.0,True,False,Fake,Report #5609
SimpleToolCollection,Knife,1000.0,20.0,True,False,Fake,
SimpleToolCollection,Knife,1000.0,20.0,True,False,,Report #5610
SimpleToolCollection,Knife,1000.0,20.0,True,False,,
SimpleToolCollection,Knife,1000.0,20.0,False,True,Genuine,Report #5611
SimpleToolCollection,Knife,1000.0,20.0,False,True,Genuine,
SimpleToolCollection,Knife,1000.0,20.0,False,True,Fake,Report #5612
SimpleToolCollection,Knife,1000.0,20.0,False,True,Fake,
SimpleToolCollection,Knife,1000.0,20.0,False,True,,Report #5613
SimpleToolCollection,Knife,1000.0,20.0,False,True,,
SimpleToolCollection,Knife,1000.0,20.0,False,False,Genuine,Report #5614
SimpleToolCollection,Knife,1000.0,20.0,False,False,Genuine,
SimpleToolCollection,Knife,1000.0,20.0,False,False,Fake,Report #5615
SimpleToolCollection,Knife,1000.0,20.0,False,False,Fake,
SimpleToolCollection,Knife,1000.0,20.0,False,False,,Report #5616
SimpleToolCollection,Knife,1000.0,20.0,False,False,,
SimpleToolCollection,Knife,1000.0,21.0,True,True,Genuine,Report #5617
SimpleToolCollection,Knife,1000.0,21.0,True,True,Genuine,
SimpleToolCollection,Knife,1000.0,21.0,True,True,Fake,Report #5618
SimpleToolCollection,Knife,1000.0,21.0,True,True,Fake,
SimpleToolCollection,Knife,1000.0,21.0,True,True,,Report #5619
SimpleToolCollection,Knife,1000.0,21.0,True,True,,
SimpleToolCollection,Knife,1000.0,21.0,True,False,Genuine,Report #5620
SimpleToolCollection,Knife,1000.0,21.0,True,False,Genuine,
SimpleToolCollection,Knife,1000.0,21.0,True,False,Fake,Report #5621
SimpleToolCollection,Knife,1000.0,21.0,True,False,Fake,
SimpleToolCollection,Knife,1000.0,21.0,True,False,,Report #5622
SimpleToolCollection,Knife,1000.0,21.0,True,False,,
SimpleToolCollection,Knife,1000.0,21.0,False,True,Genuine,Report #5623
SimpleToolCollection,Knife,1000.0,21.0,False,True,Genuine,
SimpleToolCollection,Knife,1000.0,21.0,False,True,Fake,Report #5624
SimpleToolCollection,Knife,1000.0,21.0,False,True,Fake,
SimpleToolCollection,Knife,1000.0,21.0,False,True,,Report #5625
SimpleToolCollection,Knife,1000.0,21.0,False,True,,
SimpleToolCollection,Knife,1000.0,21.0,False,False,Genuine,Report #5626
SimpleToolCollection,Knife,1000.0,21.0,False,False,Genuine,
SimpleToolCollection,Knife,1000.0,21.0,False,False,Fake,Report #5627
SimpleToolCollection,Knife,1000.0,21.0,False,False,Fake,
SimpleToolCollection,Knife,1000.0,21.0,False,False,,Report #5628
SimpleToolCollection,Knife,1000.0,21.0,False,False,,
SimpleToolCollection,Knife,1000.0,99.0,True,True,Genuine,Report #5629
SimpleToolCollection,Knife,1000.0,99.0,True,True,Genuine,
SimpleToolCollection,Knife,1000.0,99.0,True,True,Fake,Report #5630
SimpleToolCollection,Knife,1000.0,99.0,True,True,Fake,
SimpleToolCollection,Knife,1000.0,99.0,True,True,,Report #5631
SimpleToolCollection,Knife,1000.0,99.0,True,True,,
SimpleToolCollection,Knife,1000.0,99.0,True,False,Genuine,Report #5632
SimpleToolCollection,Knife,1000.0,99.0,True,False,Genuine,
SimpleToolCollection,Knife,1000.0,99.0,True,False,Fake,Report #5633
SimpleToolCollection,Knife,1000.0,99.0,True,False,Fake,
SimpleToolCollection,Knife,1000.0,99.0,True,False,,Report #5634
SimpleToolCollection,Knife,1000.0,99.0,True,False,,
SimpleToolCollection,Knife,1000.0,99.0,False,True,Genuine,Report #5635
SimpleToolCollection,Knife,1000.0,99.0,False,True,Genuine,
SimpleToolCollection,Knife,1000.0,99.0,False,True,Fake,Report #5636
SimpleToolCollection,Knife,1000.0,99.0,False,True,Fake,
SimpleToolCollection,Knife,1000.0,99.0,False,True,,Report #5637
SimpleToolCollection,Knife,1000.0,99.0,False,True,,
SimpleToolCollection,Knife,1000.0,99.0,False,False,Genuine,Report #5638
SimpleToolCollection,Knife,1000.0,99.0,False,False,Genuine,
SimpleToolCollection,Knife,1000.0,99.0,False,False,Fake,Report #5639
SimpleToolCollection,Knife,1000.0,99.0,False,False,Fake,
SimpleToolCollection,Knife,1000.0,99.0,False,False,,Report #5640
SimpleToolCollection,Knife,1000.0,99.0,False,False,,
SimpleToolCollection,Knife,1000.0,100.0,True,True,Genuine,Report #5641
SimpleToolCollection,Knife,1000.0,100.0,True,True,Genuine,
SimpleToolCollection,Knife,1000.0,100.0,True,True,Fake,Report #5642
SimpleToolCollection,Knife,1000.0,100.0,True,True,Fake,
SimpleToolCollection,Knife,1000.0,100.0,True,True,,Report #5643
SimpleToolCollection,Knife,1000.0,100.0,True,True,,
SimpleToolCollection,Knife,1000.0,100.0,True,False,Genuine,Report #5644
SimpleToolCollection,Knife,1000.0,100.0,True,False,Genuine,
SimpleToolCollection,Knife,1000.0,100.0,True,False,Fake,Report #5645
SimpleToolCollection,Knife,1000.0,100.0,True,False,Fake,
SimpleToolCollection,Knife,1000.0,100.0,True,False,,Report #5646
SimpleToolCollection,Knife,1000.0,100.0,True,False,,
SimpleToolCollection,Knife,1000.0,100.0,False,True,Genuine,Report #5647
SimpleToolCollection,Knife,1000.0,100.0,False,True,Genuine,
SimpleToolCollection,Knife,1000.0,100.0,False,True,Fake,Report #5648
SimpleToolCollection,Knife,1000.0,100.0,False,True,Fake,
SimpleToolCollection,Knife,1000.0,100.0,False,True,,Report #5649
SimpleToolCollection,Knife,1000.0,100.0,False,True,,
SimpleToolCollection,Knife,1000.0,100.0,False,False,Genuine,Report #5650
SimpleToolCollection,Knife,1000.0,100.0,False,False,Genuine,
SimpleToolCollection,Knife,1000.0,100.0,False,False,Fake,Report #5651
SimpleToolCollection,Knife,1000.0,100.0,False,False,Fake,
SimpleToolCollection,Knife,1000.0,100.0,False,False,,Report #5652
SimpleToolCollection,Knife,1000.0,100.0,False,False,,
SimpleToolCollection,Knife,1000.0,101.0,True,True,Genuine,Report #5653
SimpleToolCollection,Knife,1000.0,101.0,True,True,Genuine,
SimpleToolCollection,Knife,1000.0,101.0,True,True,Fake,Report #5654
SimpleToolCollection,Knife,1000.0,101.0,True,True,Fake,
SimpleToolCollection,Knife,1000.0,101.0,True,True,,Report #5655
SimpleToolCollection,Knife,1000.0,101.0,True,True,,
SimpleToolCollection,Knife,1000.0,101.0,True,False,Genuine,Report #5656
SimpleToolCollection,Knife,1000.0,101.0,True,False,Genuine,
SimpleToolCollection,Knife,1000.0,101.0,True,False,Fake,Report #5657
SimpleToolCollection,Knife,1000.0,101.0,True,False,Fake,
SimpleToolCollection,Knife,1000.0,101.0,True,False,,Report #5658
SimpleToolCollection,Knife,1000.0,101.0,True,False,,
SimpleToolCollection,Knife,1000.0,101.0,False,True,Genuine,Report #5659
SimpleToolCollection,Knife,1000.0,101.0,False,True,Genuine,
SimpleToolCollection,Knife,1000.0,101.0,False,True,Fake,Report #5660
SimpleToolCollection,Knife,1000.0,101.0,False,True,Fake,
SimpleToolCollection,Knife,1000.0,101.0,False,True,,Report #5661
SimpleToolCollection,Knife,1000.0,101.0,False,True,,
SimpleToolCollection,Knife,1000.0,101.0,False,False,Genuine,Report #5662
SimpleToolCollection,Knife,1000.0,101.0,False,False,Genuine,
SimpleToolCollection,Knife,1000.0,101.0,False,False,Fake,Report #5663
SimpleToolCollection,Knife,1000.0,101.0,False,False,Fake,
SimpleToolCollection,Knife,1000.0,101.0,False,False,,Report #5664
SimpleToolCollection,Knife,1000.0,101.0,False,False,,
SimpleToolCollection,Knife,1000000.0,-1.0,True,True,Genuine,Report #5665
SimpleToolCollection,Knife,1000000.0,-1.0,True,True,Genuine,
SimpleToolCollection,Knife,1000000.0,-1.0,True,True,Fake,Report #5666
SimpleToolCollection,Knife,1000000.0,-1.0,True,True,Fake,
SimpleToolCollection,Knife,1000000.0,-1.0,True,True,,Report #5667
SimpleToolCollection,Knife,1000000.0,-1.0,True,True,,
SimpleToolCollection,Knife,1000000.0,-1.0,True,False,Genuine,Report #5668
SimpleToolCollection,Knife,1000000.0,-1.0,True,False,Genuine,
SimpleToolCollection,Knife,1000000.0,-1.0,True,False,Fake,Report #5669
SimpleToolCollection,Knife,1000000.0,-1.0,True,False,Fake,
SimpleToolCollection,Knife,1000000.0,-1.0,True,False,,Report #5670
SimpleToolCollection,Knife,1000000.0,-1.0,True,False,,
SimpleToolCollection,Knife,1000000.0,-1.0,False,True,Genuine,Report #5671
SimpleToolCollection,Knife,1000000.0,-1.0,False,True,Genuine,
SimpleToolCollection,Knife,1000000.0,-1.0,False,True,Fake,Report #5672
SimpleToolCollection,Knife,1000000.0,-1.0,False,True,Fake,
SimpleToolCollection,Knife,1000000.0,-1.0,False,True,,Report #5673
SimpleToolCollection,Knife,1000000.0,-1.0,False,True,,
SimpleToolCollection,Knife,1000000.0,-1.0,False,False,Genuine,Report #5674
SimpleToolCollection,Knife,1000000.0,-1.0,False,False,Genuine,
SimpleToolCollection,Knife,1000000.0,-1.0,False,False,Fake,Report #5675
SimpleToolCollection,Knife,1000000.0,-1.0,False,False,Fake,
SimpleToolCollection,Knife,1000000.0,-1.0,False,False,,Report #5676
SimpleToolCollection,Knife,1000000.0,-1.0,False,False,,
SimpleToolCollection,Knife,1000000.0,0.0,True,True,Genuine,Report #5677
SimpleToolCollection,Knife,1000000.0,0.0,True,True,Genuine,
SimpleToolCollection,Knife,1000000.0,0.0,True,True,Fake,Report #5678
SimpleToolCollection,Knife,1000000.0,0.0,True,True,Fake,
SimpleToolCollection,Knife,1000000.0,0.0,True,True,,Report #5679
SimpleToolCollection,Knife,1000000.0,0.0,True,True,,
SimpleToolCollection,Knife,1000000.0,0.0,True,False,Genuine,Report #5680
SimpleToolCollection,Knife,1000000.0,0.0,True,False,Genuine,
SimpleToolCollection,Knife,1000000.0,0.0,True,False,Fake,Report #5681
SimpleToolCollection,Knife,1000000.0,0.0,True,False,Fake,
SimpleToolCollection,Knife,1000000.0,0.0,True,False,,Report #5682
SimpleToolCollection,Knife,1000000.0,0.0,True,False,,
SimpleToolCollection,Knife,1000000.0,0.0,False,True,Genuine,Report #5683
SimpleToolCollection,Knife,1000000.0,0.0,False,True,Genuine,
SimpleToolCollection,Knife,1000000.0,0.0,False,True,Fake,Report #5684
SimpleToolCollection,Knife,1000000.0,0.0,False,True,Fake,
SimpleToolCollection,Knife,1000000.0,0.0,False,True,,Report #5685
SimpleToolCollection,Knife,1000000.0,0.0,False,True,,
SimpleToolCollection,Knife,1000000.0,0.0,False,False,Genuine,Report #5686
SimpleToolCollection,Knife,1000000.0,0.0,False,False,Genuine,
SimpleToolCollection,Knife,1000000.0,0.0,False,False,Fake,Report #5687
SimpleToolCollection,Knife,1000000.0,0.0,False,False,Fake,
SimpleToolCollection,Knife,1000000.0,0.0,False,False,,Report #5688
SimpleToolCollection,Knife,1000000.0,0.0,False,False,,
SimpleToolCollection,Knife,1000000.0,19.0,True,True,Genuine,Report #5689
SimpleToolCollection,Knife,1000000.0,19.0,True,True,Genuine,
SimpleToolCollection,Knife,1000000.0,19.0,True,True,Fake,Report #5690
SimpleToolCollection,Knife,1000000.0,19.0,True,True,Fake,
SimpleToolCollection,Knife,1000000.0,19.0,True,True,,Report #5691
SimpleToolCollection,Knife,1000000.0,19.0,True,True,,
SimpleToolCollection,Knife,1000000.0,19.0,True,False,Genuine,Report #5692
SimpleToolCollection,Knife,1000000.0,19.0,True,False,Genuine,
SimpleToolCollection,Knife,1000000.0,19.0,True,False,Fake,Report #5693
SimpleToolCollection,Knife,1000000.0,19.0,True,False,Fake,
SimpleToolCollection,Knife,1000000.0,19.0,True,False,,Report #5694
SimpleToolCollection,Knife,1000000.0,19.0,True,False,,
SimpleToolCollection,Knife,1000000.0,19.0,False,True,Genuine,Report #5695
SimpleToolCollection,Knife,1000000.0,19.0,False,True,Genuine,
SimpleToolCollection,Knife,1000000.0,19.0,False,True,Fake,Report #5696
SimpleToolCollection,Knife,1000000.0,19.0,False,True,Fake,
SimpleToolCollection,Knife,1000000.0,19.0,False,True,,Report #5697
SimpleToolCollection,Knife,1000000.0,19.0,False,True,,
SimpleToolCollection,Knife,1000000.0,19.0,False,False,Genuine,Report #5698
SimpleToolCollection,Knife,1000000.0,19.0,False,False,Genuine,
SimpleToolCollection,Knife,1000000.0,19.0,False,False,Fake,Report #5699
SimpleToolCollection,Knife,1000000.0,19.0,False,False,Fake,
SimpleToolCollection,Knife,1000000.0,19.0,False,False,,Report #5700
SimpleToolCollection,Knife,1000000.0,19.0,False,False,,
SimpleToolCollection,Knife,1000000.0,20.0,True,True,Genuine,Report #5701
SimpleToolCollection,Knife,1000000.0,20.0,True,True,Genuine,
SimpleToolCollection,Knife,1000000.0,20.0,True,True,Fake,Report #5702
SimpleToolCollection,Knife,1000000.0,20.0,True,True,Fake,
SimpleToolCollection,Knife,1000000.0,20.0,True,True,,Report #5703
SimpleToolCollection,Knife,1000000.0,20.0,True,True,,
SimpleToolCollection,Knife,1000000.0,20.0,True,False,Genuine,Report #5704
SimpleToolCollection,Knife,1000000.0,20.0,True,False,Genuine,
SimpleToolCollection,Knife,1000000.0,20.0,True,False,Fake,Report #5705
SimpleToolCollection,Knife,1000000.0,20.0,True,False,Fake,
SimpleToolCollection,Knife,1000000.0,20.0,True,False,,Report #5706
SimpleToolCollection,Knife,1000000.0,20.0,True,False,,
SimpleToolCollection,Knife,1000000.0,20.0,False,True,Genuine,Report #5707
SimpleToolCollection,Knife,1000000.0,20.0,False,True,Genuine,
SimpleToolCollection,Knife,1000000.0,20.0,False,True,Fake,Report #5708
SimpleToolCollection,Knife,1000000.0,20.0,False,True,Fake,
SimpleToolCollection,Knife,1000000.0,20.0,False,True,,Report #5709
SimpleToolCollection,Knife,1000000.0,20.0,False,True,,
SimpleToolCollection,Knife,1000000.0,20.0,False,False,Genuine,Report #5710
SimpleToolCollection,Knife,1000000.0,20.0,False,False,Genuine,
SimpleToolCollection,Knife,1000000.0,20.0,False,False,Fake,Report #5711
SimpleToolCollection,Knife,1000000.0,20.0,False,False,Fake,
SimpleToolCollection,Knife,1000000.0,20.0,False,False,,Report #5712
SimpleToolCollection,Knife,1000000.0,20.0,False,False,,
SimpleToolCollection,Knife,1000000.0,21.0,True,True,Genuine,Report #5713
SimpleToolCollection,Knife,1000000.0,21.0,True,True,Genuine,
SimpleToolCollection,Knife,1000000.0,21.0,True,True,Fake,Report #5714
SimpleToolCollection,Knife,1000000.0,21.0,True,True,Fake,
SimpleToolCollection,Knife,1000000.0,21.0,True,True,,Report #5715
SimpleToolCollection,Knife,1000000.0,21.0,True,True,,
SimpleToolCollection,Knife,1000000.0,21.0,True,False,Genuine,Report #5716
SimpleToolCollection,Knife,1000000.0,21.0,True,False,Genuine,
SimpleToolCollection,Knife,1000000.0,21.0,True,False,Fake,Report #5717
SimpleToolCollection,Knife,1000000.0,21.0,True,False,Fake,
SimpleToolCollection,Knife,1000000.0,21.0,True,False,,Report #5718
SimpleToolCollection,Knife,1000000.0,21.0,True,False,,
SimpleToolCollection,Knife,1000000.0,21.0,False,True,Genuine,Report #5719
SimpleToolCollection,Knife,1000000.0,21.0,False,True,Genuine,
SimpleToolCollection,Knife,1000000.0,21.0,False,True,Fake,Report #5720
SimpleToolCollection,Knife,1000000.0,21.0,False,True,Fake,
SimpleToolCollection,Knife,1000000.0,21.0,False,True,,Report #5721
SimpleToolCollection,Knife,1000000.0,21.0,False,True,,
SimpleToolCollection,Knife,1000000.0,21.0,False,False,Genuine,Report #5722
SimpleToolCollection,Knife,1000000.0,21.0,False,False,Genuine,
SimpleToolCollection,Knife,1000000.0,21.0,False,False,Fake,Report #5723
SimpleToolCollection,Knife,1000000.0,21.0,False,False,Fake,
SimpleToolCollection,Knife,1000000.0,21.0,False,False,,Report #5724
SimpleToolCollection,Knife,1000000.0,21.0,False,False,,
SimpleToolCollection,Knife,1000000.0,99.0,True,True,Genuine,Report #5725
SimpleToolCollection,Knife,1000000.0,99.0,True,True,Genuine,
SimpleToolCollection,Knife,1000000.0,99.0,True,True,Fake,Report #5726
SimpleToolCollection,Knife,1000000.0,99.0,True,True,Fake,
SimpleToolCollection,Knife,1000000.0,99.0,True,True,,Report #5727
SimpleToolCollection,Knife,1000000.0,99.0,True,True,,
SimpleToolCollection,Knife,1000000.0,99.0,True,False,Genuine,Report #5728
SimpleToolCollection,Knife,1000000.0,99.0,True,False,Genuine,
SimpleToolCollection,Knife,1000000.0,99.0,True,False,Fake,Report #5729
SimpleToolCollection,Knife,1000000.0,99.0,True,False,Fake,
SimpleToolCollection,Knife,1000000.0,99.0,True,False,,Report #5730
SimpleToolCollection,Knife,1000000.0,99.0,True,False,,
SimpleToolCollection,Knife,1000000.0,99.0,False,True,Genuine,Report #5731
SimpleToolCollection,Knife,1000000.0,99.0,False,True,Genuine,
SimpleToolCollection,Knife,1000000.0,99.0,False,True,Fake,Report #5732
SimpleToolCollection,Knife,1000000.0,99.0,False,True,Fake,
SimpleToolCollection,Knife,1000000.0,99.0,False,True,,Report #5733
SimpleToolCollection,Knife,1000000.0,99.0,False,True,,
SimpleToolCollection,Knife,1000000.0,99.0,False,False,Genuine,Report #5734
SimpleToolCollection,Knife,1000000.0,99.0,False,False,Genuine,
SimpleToolCollection,Knife,1000000.0,99.0,False,False,Fake,Report #5735
SimpleToolCollection,Knife,1000000.0,99.0,False,False,Fake,
SimpleToolCollection,Knife,1000000.0,99.0,False,False,,Report #5736
SimpleToolCollection,Knife,1000000.0,99.0,False,False,,
SimpleToolCollection,Knife,1000000.0,100.0,True,True,Genuine,Report #5737
SimpleToolCollection,Knife,1000000.0,100.0,True,True,Genuine,
SimpleToolCollection,Knife,1000000.0,100.0,True,True,Fake,Report #5738
SimpleToolCollection,Knife,1000000.0,100.0,True,True,Fake,
SimpleToolCollection,Knife,1000000.0,100.0,True,True,,Report #5739
SimpleToolCollection,Knife,1000000.0,100.0,True,True,,
SimpleToolCollection,Knife,1000000.0,100.0,True,False,Genuine,Report #5740
SimpleToolCollection,Knife,1000000.0,100.0,True,False,Genuine,
SimpleToolCollection,Knife,1000000.0,100.0,True,False,Fake,Report #5741
SimpleToolCollection,Knife,1000000.0,100.0,True,False,Fake,
SimpleToolCollection,Knife,1000000.0,100.0,True,False,,Report #5742
SimpleToolCollection,Knife,1000000.0,100.0,True,False,,
SimpleToolCollection,Knife,1000000.0,100.0,False,True,Genuine,Report #5743
SimpleToolCollection,Knife,1000000.0,100.0,False,True,Genuine,
SimpleToolCollection,Knife,1000000.0,100.0,False,True,Fake,Report #5744
SimpleToolCollection,Knife,1000000.0,100.0,False,True,Fake,
SimpleToolCollection,Knife,1000000.0,100.0,False,True,,Report #5745
SimpleToolCollection,Knife,1000000.0,100.0,False,True,,
SimpleToolCollection,Knife,1000000.0,100.0,False,False,Genuine,Report #5746
SimpleToolCollection,Knife,1000000.0,100.0,False,False,Genuine,
SimpleToolCollection,Knife,1000000.0,100.0,False,False,Fake,Report #5747
SimpleToolCollection,Knife,1000000.0,100.0,False,False,Fake,
SimpleToolCollection,Knife,1000000.0,100.0,False,False,,Report #5748
SimpleToolCollection,Knife,1000000.0,100.0,False,False,,
SimpleToolCollection,Knife,1000000.0,101.0,True,True,Genuine,Report #5749
SimpleToolCollection,Knife,1000000.0,101.0,True,True,Genuine,
SimpleToolCollection,Knife,1000000.0,101.0,True,True,Fake,Report #5750
SimpleToolCollection,Knife,1000000.0,101.0,True,True,Fake,
SimpleToolCollection,Knife,1000000.0,101.0,True,True,,Report #5751
SimpleToolCollection,Knife,1000000.0,101.0,True,True,,
SimpleToolCollection,Knife,1000000.0,101.0,True,False,Genuine,Report #5752
SimpleToolCollection,Knife,1000000.0,101.0,True,False,Genuine,
SimpleToolCollection,Knife,1000000.0,101.0,True,False,Fake,Report #5753
SimpleToolCollection,Knife,1000000.0,101.0,True,False,Fake,
SimpleToolCollection,Knife,1000000.0,101.0,True,False,,Report #5754
SimpleToolCollection,Knife,1000000.0,101.0,True,False,,
SimpleToolCollection,Knife,1000000.0,101.0,False,True,Genuine,Report #5755
SimpleToolCollection,Knife,1000000.0,101.0,False,True,Genuine,
SimpleToolCollection,Knife,1000000.0,101.0,False,True,Fake,Report #5756
SimpleToolCollection,Knife,1000000.0,101.0,False,True,Fake,
SimpleToolCollection,Knife,1000000.0,101.0,False,True,,Report #5757
SimpleToolCollection,Knife,1000000.0,101.0,False,True,,
SimpleToolCollection,Knife,1000000.0,101.0,False,False,Genuine,Report #5758
SimpleToolCollection,Knife,1000000.0,101.0,False,False,Genuine,
SimpleToolCollection,Knife,1000000.0,101.0,False,False,Fake,Report #5759
SimpleToolCollection,Knife,1000000.0,101.0,False,False,Fake,
SimpleToolCollection,Knife,1000000.0,101.0,False,False,,Report #5760
SimpleToolCollection,Knife,1000000.0,101.0,False,False,,
SimpleToolCollection,Plates,0.0,-1.0,True,True,Genuine,Report #5761
SimpleToolCollection,Plates,0.0,-1.0,True,True,Genuine,
SimpleToolCollection,Plates,0.0,-1.0,True,True,Fake,Report #5762
SimpleToolCollection,Plates,0.0,-1.0,True,True,Fake,
SimpleToolCollection,Plates,0.0,-1.0,True,True,,Report #5763
SimpleToolCollection,Plates,0.0,-1.0,True,True,,
SimpleToolCollection,Plates,0.0,-1.0,True,False,Genuine,Report #5764
SimpleToolCollection,Plates,0.0,-1.0,True,False,Genuine,
SimpleToolCollection,Plates,0.0,-1.0,True,False,Fake,Report #5765
SimpleToolCollection,Plates,0.0,-1.0,True,False,Fake,
SimpleToolCollection,Plates,0.0,-1.0,True,False,,Report #5766
SimpleToolCollection,Plates,0.0,-1.0,True,False,,
SimpleToolCollection,Plates,0.0,-1.0,False,True,Genuine,Report #5767
SimpleToolCollection,Plates,0.0,-1.0,False,True,Genuine,
SimpleToolCollection,Plates,0.0,-1.0,False,True,Fake,Report #5768
SimpleToolCollection,Plates,0.0,-1.0,False,True,Fake,
SimpleToolCollection,Plates,0.0,-1.0,False,True,,Report #5769
SimpleToolCollection,Plates,0.0,-1.0,False,True,,
SimpleToolCollection,Plates,0.0,-1.0,False,False,Genuine,Report #5770
SimpleToolCollection,Plates,0.0,-1.0,False,False,Genuine,
SimpleToolCollection,Plates,0.0,-1.0,False,False,Fake,Report #5771
SimpleToolCollection,Plates,0.0,-1.0,False,False,Fake,
SimpleToolCollection,Plates,0.0,-1.0,False,False,,Report #5772
SimpleToolCollection,Plates,0.0,-1.0,False,False,,
SimpleToolCollection,Plates,0.0,0.0,True,True,Genuine,Report #5773
SimpleToolCollection,Plates,0.0,0.0,True,True,Genuine,
SimpleToolCollection,Plates,0.0,0.0,True,True,Fake,Report #5774
SimpleToolCollection,Plates,0.0,0.0,True,True,Fake,
SimpleToolCollection,Plates,0.0,0.0,True,True,,Report #5775
SimpleToolCollection,Plates,0.0,0.0,True,True,,
SimpleToolCollection,Plates,0.0,0.0,True,False,Genuine,Report #5776
SimpleToolCollection,Plates,0.0,0.0,True,False,Genuine,
SimpleToolCollection,Plates,0.0,0.0,True,False,Fake,Report #5777
SimpleToolCollection,Plates,0.0,0.0,True,False,Fake,
SimpleToolCollection,Plates,0.0,0.0,True,False,,Report #5778
SimpleToolCollection,Plates,0.0,0.0,True,False,,
SimpleToolCollection,Plates,0.0,0.0,False,True,Genuine,Report #5779
SimpleToolCollection,Plates,0.0,0.0,False,True,Genuine,
SimpleToolCollection,Plates,0.0,0.0,False,True,Fake,Report #5780
SimpleToolCollection,Plates,0.0,0.0,False,True,Fake,
SimpleToolCollection,Plates,0.0,0.0,False,True,,Report #5781
SimpleToolCollection,Plates,0.0,0.0,False,True,,
SimpleToolCollection,Plates,0.0,0.0,False,False,Genuine,Report #5782
SimpleToolCollection,Plates,0.0,0.0,False,False,Genuine,
SimpleToolCollection,Plates,0.0,0.0,False,False,Fake,Report #5783
SimpleToolCollection,Plates,0.0,0.0,False,False,Fake,
SimpleToolCollection,Plates,0.0,0.0,False,False,,Report #5784
SimpleToolCollection,Plates,0.0,0.0,False,False,,
SimpleToolCollection,Plates,0.0,19.0,True,True,Genuine,Report #5785
SimpleToolCollection,Plates,0.0,19.0,True,True,Genuine,
SimpleToolCollection,Plates,0.0,19.0,True,True,Fake,Report #5786
SimpleToolCollection,Plates,0.0,19.0,True,True,Fake,
SimpleToolCollection,Plates,0.0,19.0,True,True,,Report #5787
SimpleToolCollection,Plates,0.0,19.0,True,True,,
SimpleToolCollection,Plates,0.0,19.0,True,False,Genuine,Report #5788
SimpleToolCollection,Plates,0.0,19.0,True,False,Genuine,
SimpleToolCollection,Plates,0.0,19.0,True,False,Fake,Report #5789
SimpleToolCollection,Plates,0.0,19.0,True,False,Fake,
SimpleToolCollection,Plates,0.0,19.0,True,False,,Report #5790
SimpleToolCollection,Plates,0.0,19.0,True,False,,
SimpleToolCollection,Plates,0.0,19.0,False,True,Genuine,Report #5791
SimpleToolCollection,Plates,0.0,19.0,False,True,Genuine,
SimpleToolCollection,Plates,0.0,19.0,False,True,Fake,Report #5792
SimpleToolCollection,Plates,0.0,19.0,False,True,Fake,
SimpleToolCollection,Plates,0.0,19.0,False,True,,Report #5793
SimpleToolCollection,Plates,0.0,19.0,False,True,,
SimpleToolCollection,Plates,0.0,19.0,False,False,Genuine,Report #5794
SimpleToolCollection,Plates,0.0,19.0,False,False,Genuine,
SimpleToolCollection,Plates,0.0,19.0,False,False,Fake,Report #5795
SimpleToolCollection,Plates,0.0,19.0,False,False,Fake,
SimpleToolCollection,Plates,0.0,19.0,False,False,,Report #5796
SimpleToolCollection,Plates,0.0,19.0,False,False,,
SimpleToolCollection,Plates,0.0,20.0,True,True,Genuine,Report #5797
SimpleToolCollection,Plates,0.0,20.0,True,True,Genuine,
SimpleToolCollection,Plates,0.0,20.0,True,True,Fake,Report #5798
SimpleToolCollection,Plates,0.0,20.0,True,True,Fake,
SimpleToolCollection,Plates,0.0,20.0,True,True,,Report #5799
SimpleToolCollection,Plates,0.0,20.0,True,True,,
SimpleToolCollection,Plates,0.0,20.0,True,False,Genuine,Report #5800
SimpleToolCollection,Plates,0.0,20.0,True,False,Genuine,
SimpleToolCollection,Plates,0.0,20.0,True,False,Fake,Report #5801
SimpleToolCollection,Plates,0.0,20.0,True,False,Fake,
SimpleToolCollection,Plates,0.0,20.0,True,False,,Report #5802
SimpleToolCollection,Plates,0.0,20.0,True,False,,
SimpleToolCollection,Plates,0.0,20.0,False,True,Genuine,Report #5803
SimpleToolCollection,Plates,0.0,20.0,False,True,Genuine,
SimpleToolCollection,Plates,0.0,20.0,False,True,Fake,Report #5804
SimpleToolCollection,Plates,0.0,20.0,False,True,Fake,
SimpleToolCollection,Plates,0.0,20.0,False,True,,Report #5805
SimpleToolCollection,Plates,0.0,20.0,False,True,,
SimpleToolCollection,Plates,0.0,20.0,False,False,Genuine,Report #5806
SimpleToolCollection,Plates,0.0,20.0,False,False,Genuine,
SimpleToolCollection,Plates,0.0,20.0,False,False,Fake,Report #5807
SimpleToolCollection,Plates,0.0,20.0,False,False,Fake,
SimpleToolCollection,Plates,0.0,20.0,False,False,,Report #5808
SimpleToolCollection,Plates,0.0,20.0,False,False,,
SimpleToolCollection,Plates,0.0,21.0,True,True,Genuine,Report #5809
SimpleToolCollection,Plates,0.0,21.0,True,True,Genuine,
SimpleToolCollection,Plates,0.0,21.0,True,True,Fake,Report #5810
SimpleToolCollection,Plates,0.0,21.0,True,True,Fake,
SimpleToolCollection,Plates,0.0,21.0,True,True,,Report #5811
SimpleToolCollection,Plates,0.0,21.0,True,True,,
SimpleToolCollection,Plates,0.0,21.0,True,False,Genuine,Report #5812
SimpleToolCollection,Plates,0.0,21.0,True,False,Genuine,
SimpleToolCollection,Plates,0.0,21.0,True,False,Fake,Report #5813
SimpleToolCollection,Plates,0.0,21.0,True,False,Fake,
SimpleToolCollection,Plates,0.0,21.0,True,False,,Report #5814
SimpleToolCollection,Plates,0.0,21.0,True,False,,
SimpleToolCollection,Plates,0.0,21.0,False,True,Genuine,Report #5815
SimpleToolCollection,Plates,0.0,21.0,False,True,Genuine,
SimpleToolCollection,Plates,0.0,21.0,False,True,Fake,Report #5816
SimpleToolCollection,Plates,0.0,21.0,False,True,Fake,
SimpleToolCollection,Plates,0.0,21.0,False,True,,Report #5817
SimpleToolCollection,Plates,0.0,21.0,False,True,,
SimpleToolCollection,Plates,0.0,21.0,False,False,Genuine,Report #5818
SimpleToolCollection,Plates,0.0,21.0,False,False,Genuine,
SimpleToolCollection,Plates,0.0,21.0,False,False,Fake,Report #5819
SimpleToolCollection,Plates,0.0,21.0,False,False,Fake,
SimpleToolCollection,Plates,0.0,21.0,False,False,,Report #5820
SimpleToolCollection,Plates,0.0,21.0,False,False,,
SimpleToolCollection,Plates,0.0,99.0,True,True,Genuine,Report #5821
SimpleToolCollection,Plates,0.0,99.0,True,True,Genuine,
SimpleToolCollection,Plates,0.0,99.0,True,True,Fake,Report #5822
SimpleToolCollection,Plates,0.0,99.0,True,True,Fake,
SimpleToolCollection,Plates,0.0,99.0,True,True,,Report #5823
SimpleToolCollection,Plates,0.0,99.0,True,True,,
SimpleToolCollection,Plates,0.0,99.0,True,False,Genuine,Report #5824
SimpleToolCollection,Plates,0.0,99.0,True,False,Genuine,
SimpleToolCollection,Plates,0.0,99.0,True,False,Fake,Report #5825
SimpleToolCollection,Plates,0.0,99.0,True,False,Fake,
SimpleToolCollection,Plates,0.0,99.0,True,False,,Report #5826
SimpleToolCollection,Plates,0.0,99.0,True,False,,
SimpleToolCollection,Plates,0.0,99.0,False,True,Genuine,Report #5827
SimpleToolCollection,Plates,0.0,99.0,False,True,Genuine,
SimpleToolCollection,Plates,0.0,99.0,False,True,Fake,Report #5828
SimpleToolCollection,Plates,0.0,99.0,False,True,Fake,
SimpleToolCollection,Plates,0.0,99.0,False,True,,Report #5829
SimpleToolCollection,Plates,0.0,99.0,False,True,,
SimpleToolCollection,Plates,0.0,99.0,False,False,Genuine,Report #5830
SimpleToolCollection,Plates,0.0,99.0,False,False,Genuine,
SimpleToolCollection,Plates,0.0,99.0,False,False,Fake,Report #5831
SimpleToolCollection,Plates,0.0,99.0,False,False,Fake,
SimpleToolCollection,Plates,0.0,99.0,False,False,,Report #5832
SimpleToolCollection,Plates,0.0,99.0,False,False,,
SimpleToolCollection,Plates,0.0,100.0,True,True,Genuine,Report #5833
SimpleToolCollection,Plates,0.0,100.0,True,True,Genuine,
SimpleToolCollection,Plates,0.0,100.0,True,True,Fake,Report #5834
SimpleToolCollection,Plates,0.0,100.0,True,True,Fake,
SimpleToolCollection,Plates,0.0,100.0,True,True,,Report #5835
SimpleToolCollection,Plates,0.0,100.0,True,True,,
SimpleToolCollection,Plates,0.0,100.0,True,False,Genuine,Report #5836
SimpleToolCollection,Plates,0.0,100.0,True,False,Genuine,
SimpleToolCollection,Plates,0.0,100.0,True,False,Fake,Report #5837
SimpleToolCollection,Plates,0.0,100.0,True,False,Fake,
SimpleToolCollection,Plates,0.0,100.0,True,False,,Report #5838
SimpleToolCollection,Plates,0.0,100.0,True,False,,
SimpleToolCollection,Plates,0.0,100.0,False,True,Genuine,Report #5839
SimpleToolCollection,Plates,0.0,100.0,False,True,Genuine,
SimpleToolCollection,Plates,0.0,100.0,False,True,Fake,Report #5840
SimpleToolCollection,Plates,0.0,100.0,False,True,Fake,
SimpleToolCollection,Plates,0.0,100.0,False,True,,Report #5841
SimpleToolCollection,Plates,0.0,100.0,False,True,,
SimpleToolCollection,Plates,0.0,100.0,False,False,Genuine,Report #5842
SimpleToolCollection,Plates,0.0,100.0,False,False,Genuine,
SimpleToolCollection,Plates,0.0,100.0,False,False,Fake,Report #5843
SimpleToolCollection,Plates,0.0,100.0,False,False,Fake,
SimpleToolCollection,Plates,0.0,100.0,False,False,,Report #5844
SimpleToolCollection,Plates,0.0,100.0,False,False,,
SimpleToolCollection,Plates,0.0,101.0,True,True,Genuine,Report #5845
SimpleToolCollection,Plates,0.0,101.0,True,True,Genuine,
SimpleToolCollection,Plates,0.0,101.0,True,True,Fake,Report #5846
SimpleToolCollection,Plates,0.0,101.0,True,True,Fake,
SimpleToolCollection,Plates,0.0,101.0,True,True,,Report #5847
SimpleToolCollection,Plates,0.0,101.0,True,True,,
SimpleToolCollection,Plates,0.0,101.0,True,False,Genuine,Report #5848
SimpleToolCollection,Plates,0.0,101.0,True,False,Genuine,
SimpleToolCollection,Plates,0.0,101.0,True,False,Fake,Report #5849
SimpleToolCollection,Plates,0.0,101.0,True,False,Fake,
SimpleToolCollection,Plates,0.0,101.0,True,False,,Report #5850
SimpleToolCollection,Plates,0.0,101.0,True,False,,
SimpleToolCollection,Plates,0.0,101.0,False,True,Genuine,Report #5851
SimpleToolCollection,Plates,0.0,101.0,False,True,Genuine,
SimpleToolCollection,Plates,0.0,101.0,False,True,Fake,Report #5852
SimpleToolCollection,Plates,0.0,101.0,False,True,Fake,
SimpleToolCollection,Plates,0.0,101.0,False,True,,Report #5853
SimpleToolCollection,Plates,0.0,101.0,False,True,,
SimpleToolCollection,Plates,0.0,101.0,False,False,Genuine,Report #5854
SimpleToolCollection,Plates,0.0,101.0,False,False,Genuine,
SimpleToolCollection,Plates,0.0,101.0,False,False,Fake,Report #5855
SimpleToolCollection,Plates,0.0,101.0,False,False,Fake,
SimpleToolCollection,Plates,0.0,101.0,False,False,,Report #5856
SimpleToolCollection,Plates,0.0,101.0,False,False,,
SimpleToolCollection,Plates,1.0,-1.0,True,True,Genuine,Report #5857
SimpleToolCollection,Plates,1.0,-1.0,True,True,Genuine,
SimpleToolCollection,Plates,1.0,-1.0,True,True,Fake,Report #5858
SimpleToolCollection,Plates,1.0,-1.0,True,True,Fake,
SimpleToolCollection,Plates,1.0,-1.0,True,True,,Report #5859
SimpleToolCollection,Plates,1.0,-1.0,True,True,,
SimpleToolCollection,Plates,1.0,-1.0,True,False,Genuine,Report #5860
SimpleToolCollection,Plates,1.0,-1.0,True,False,Genuine,
SimpleToolCollection,Plates,1.0,-1.0,True,False,Fake,Report #5861
SimpleToolCollection,Plates,1.0,-1.0,True,False,Fake,
SimpleToolCollection,Plates,1.0,-1.0,True,False,,Report #5862
SimpleToolCollection,Plates,1.0,-1.0,True,False,,
SimpleToolCollection,Plates,1.0,-1.0,False,True,Genuine,Report #5863
SimpleToolCollection,Plates,1.0,-1.0,False,True,Genuine,
SimpleToolCollection,Plates,1.0,-1.0,False,True,Fake,Report #5864
SimpleToolCollection,Plates,1.0,-1.0,False,True,Fake,
SimpleToolCollection,Plates,1.0,-1.0,False,True,,Report #5865
SimpleToolCollection,Plates,1.0,-1.0,False,True,,
SimpleToolCollection,Plates,1.0,-1.0,False,False,Genuine,Report #5866
SimpleToolCollection,Plates,1.0,-1.0,False,False,Genuine,
SimpleToolCollection,Plates,1.0,-1.0,False,False,Fake,Report #5867
SimpleToolCollection,Plates,1.0,-1.0,False,False,Fake,
SimpleToolCollection,Plates,1.0,-1.0,False,False,,Report #5868
SimpleToolCollection,Plates,1.0,-1.0,False,False,,
SimpleToolCollection,Plates,1.0,0.0,True,True,Genuine,Report #5869
SimpleToolCollection,Plates,1.0,0.0,True,True,Genuine,
SimpleToolCollection,Plates,1.0,0.0,True,True,Fake,Report #5870
SimpleToolCollection,Plates,1.0,0.0,True,True,Fake,
SimpleToolCollection,Plates,1.0,0.0,True,True,,Report #5871
SimpleToolCollection,Plates,1.0,0.0,True,True,,
SimpleToolCollection,Plates,1.0,0.0,True,False,Genuine,Report #5872
SimpleToolCollection,Plates,1.0,0.0,True,False,Genuine,
SimpleToolCollection,Plates,1.0,0.0,True,False,Fake,Report #5873
SimpleToolCollection,Plates,1.0,0.0,True,False,Fake,
SimpleToolCollection,Plates,1.0,0.0,True,False,,Report #5874
SimpleToolCollection,Plates,1.0,0.0,True,False,,
SimpleToolCollection,Plates,1.0,0.0,False,True,Genuine,Report #5875
SimpleToolCollection,Plates,1.0,0.0,False,True,Genuine,
SimpleToolCollection,Plates,1.0,0.0,False,True,Fake,Report #5876
SimpleToolCollection,Plates,1.0,0.0,False,True,Fake,
SimpleToolCollection,Plates,1.0,0.0,False,True,,Report #5877
SimpleToolCollection,Plates,1.0,0.0,False,True,,
SimpleToolCollection,Plates,1.0,0.0,False,False,Genuine,Report #5878
SimpleToolCollection,Plates,1.0,0.0,False,False,Genuine,
SimpleToolCollection,Plates,1.0,0.0,False,False,Fake,Report #5879
SimpleToolCollection,Plates,1.0,0.0,False,False,Fake,
SimpleToolCollection,Plates,1.0,0.0,False,False,,Report #5880
SimpleToolCollection,Plates,1.0,0.0,False,False,,
SimpleToolCollection,Plates,1.0,19.0,True,True,Genuine,Report #5881
SimpleToolCollection,Plates,1.0,19.0,True,True,Genuine,
SimpleToolCollection,Plates,1.0,19.0,True,True,Fake,Report #5882
SimpleToolCollection,Plates,1.0,19.0,True,True,Fake,
SimpleToolCollection,Plates,1.0,19.0,True,True,,Report #5883
SimpleToolCollection,Plates,1.0,19.0,True,True,,
SimpleToolCollection,Plates,1.0,19.0,True,False,Genuine,Report #5884
SimpleToolCollection,Plates,1.0,19.0,True,False,Genuine,
SimpleToolCollection,Plates,1.0,19.0,True,False,Fake,Report #5885
SimpleToolCollection,Plates,1.0,19.0,True,False,Fake,
SimpleToolCollection,Plates,1.0,19.0,True,False,,Report #5886
SimpleToolCollection,Plates,1.0,19.0,True,False,,
SimpleToolCollection,Plates,1.0,19.0,False,True,Genuine,Report #5887
SimpleToolCollection,Plates,1.0,19.0,False,True,Genuine,
SimpleToolCollection,Plates,1.0,19.0,False,True,Fake,Report #5888
SimpleToolCollection,Plates,1.0,19.0,False,True,Fake,
SimpleToolCollection,Plates,1.0,19.0,False,True,,Report #5889
SimpleToolCollection,Plates,1.0,19.0,False,True,,
SimpleToolCollection,Plates,1.0,19.0,False,False,Genuine,Report #5890
SimpleToolCollection,Plates,1.0,19.0,False,False,Genuine,
SimpleToolCollection,Plates,1.0,19.0,False,False,Fake,Report #5891
SimpleToolCollection,Plates,1.0,19.0,False,False,Fake,
SimpleToolCollection,Plates,1.0,19.0,False,False,,Report #5892
SimpleToolCollection,Plates,1.0,19.0,False,False,,
SimpleToolCollection,Plates,1.0,20.0,True,True,Genuine,Report #5893
SimpleToolCollection,Plates,1.0,20.0,True,True,Genuine,
SimpleToolCollection,Plates,1.0,20.0,True,True,Fake,Report #5894
SimpleToolCollection,Plates,1.0,20.0,True,True,Fake,
SimpleToolCollection,Plates,1.0,20.0,True,True,,Report #5895
SimpleToolCollection,Plates,1.0,20.0,True,True,,
SimpleToolCollection,Plates,1.0,20.0,True,False,Genuine,Report #5896
SimpleToolCollection,Plates,1.0,20.0,True,False,Genuine,
SimpleToolCollection,Plates,1.0,20.0,True,False,Fake,Report #5897
SimpleToolCollection,Plates,1.0,20.0,True,False,Fake,
SimpleToolCollection,Plates,1.0,20.0,True,False,,Report #5898
SimpleToolCollection,Plates,1.0,20.0,True,False,,
SimpleToolCollection,Plates,1.0,20.0,False,True,Genuine,Report #5899
SimpleToolCollection,Plates,1.0,20.0,False,True,Genuine,
SimpleToolCollection,Plates,1.0,20.0,False,True,Fake,Report #5900
SimpleToolCollection,Plates,1.0,20.0,False,True,Fake,
SimpleToolCollection,Plates,1.0,20.0,False,True,,Report #5901
SimpleToolCollection,Plates,1.0,20.0,False,True,,
SimpleToolCollection,Plates,1.0,20.0,False,False,Genuine,Report #5902
SimpleToolCollection,Plates,1.0,20.0,False,False,Genuine,
SimpleToolCollection,Plates,1.0,20.0,False,False,Fake,Report #5903
SimpleToolCollection,Plates,1.0,20.0,False,False,Fake,
SimpleToolCollection,Plates,1.0,20.0,False,False,,Report #5904
SimpleToolCollection,Plates,1.0,20.0,False,False,,
SimpleToolCollection,Plates,1.0,21.0,True,True,Genuine,Report #5905
SimpleToolCollection,Plates,1.0,21.0,True,True,Genuine,
SimpleToolCollection,Plates,1.0,21.0,True,True,Fake,Report #5906
SimpleToolCollection,Plates,1.0,21.0,True,True,Fake,
SimpleToolCollection,Plates,1.0,21.0,True,True,,Report #5907
SimpleToolCollection,Plates,1.0,21.0,True,True,,
SimpleToolCollection,Plates,1.0,21.0,True,False,Genuine,Report #5908
SimpleToolCollection,Plates,1.0,21.0,True,False,Genuine,
SimpleToolCollection,Plates,1.0,21.0,True,False,Fake,Report #5909
SimpleToolCollection,Plates,1.0,21.0,True,False,Fake,
SimpleToolCollection,Plates,1.0,21.0,True,False,,Report #5910
SimpleToolCollection,Plates,1.0,21.0,True,False,,
SimpleToolCollection,Plates,1.0,21.0,False,True,Genuine,Report #5911
SimpleToolCollection,Plates,1.0,21.0,False,True,Genuine,
SimpleToolCollection,Plates,1.0,21.0,False,True,Fake,Report #5912
SimpleToolCollection,Plates,1.0,21.0,False,True,Fake,
SimpleToolCollection,Plates,1.0,21.0,False,True,,Report #5913
SimpleToolCollection,Plates,1.0,21.0,False,True,,
SimpleToolCollection,Plates,1.0,21.0,False,False,Genuine,Report #5914
SimpleToolCollection,Plates,1.0,21.0,False,False,Genuine,
SimpleToolCollection,Plates,1.0,21.0,False,False,Fake,Report #5915
SimpleToolCollection,Plates,1.0,21.0,False,False,Fake,
SimpleToolCollection,Plates,1.0,21.0,False,False,,Report #5916
SimpleToolCollection,Plates,1.0,21.0,False,False,,
SimpleToolCollection,Plates,1.0,99.0,True,True,Genuine,Report #5917
SimpleToolCollection,Plates,1.0,99.0,True,True,Genuine,
SimpleToolCollection,Plates,1.0,99.0,True,True,Fake,Report #5918
SimpleToolCollection,Plates,1.0,99.0,True,True,Fake,
SimpleToolCollection,Plates,1.0,99.0,True,True,,Report #5919
SimpleToolCollection,Plates,1.0,99.0,True,True,,
SimpleToolCollection,Plates,1.0,99.0,True,False,Genuine,Report #5920
SimpleToolCollection,Plates,1.0,99.0,True,False,Genuine,
SimpleToolCollection,Plates,1.0,99.0,True,False,Fake,Report #5921
SimpleToolCollection,Plates,1.0,99.0,True,False,Fake,
SimpleToolCollection,Plates,1.0,99.0,True,False,,Report #5922
SimpleToolCollection,Plates,1.0,99.0,True,False,,
SimpleToolCollection,Plates,1.0,99.0,False,True,Genuine,Report #5923
SimpleToolCollection,Plates,1.0,99.0,False,True,Genuine,
SimpleToolCollection,Plates,1.0,99.0,False,True,Fake,Report #5924
SimpleToolCollection,Plates,1.0,99.0,False,True,Fake,
SimpleToolCollection,Plates,1.0,99.0,False,True,,Report #5925
SimpleToolCollection,Plates,1.0,99.0,False,True,,
SimpleToolCollection,Plates,1.0,99.0,False,False,Genuine,Report #5926
SimpleToolCollection,Plates,1.0,99.0,False,False,Genuine,
SimpleToolCollection,Plates,1.0,99.0,False,False,Fake,Report #5927
SimpleToolCollection,Plates,1.0,99.0,False,False,Fake,
SimpleToolCollection,Plates,1.0,99.0,False,False,,Report #5928
SimpleToolCollection,Plates,1.0,99.0,False,False,,
SimpleToolCollection,Plates,1.0,100.0,True,True,Genuine,Report #5929
SimpleToolCollection,Plates,1.0,100.0,True,True,Genuine,
SimpleToolCollection,Plates,1.0,100.0,True,True,Fake,Report #5930
SimpleToolCollection,Plates,1.0,100.0,True,True,Fake,
SimpleToolCollection,Plates,1.0,100.0,True,True,,Report #5931
SimpleToolCollection,Plates,1.0,100.0,True,True,,
SimpleToolCollection,Plates,1.0,100.0,True,False,Genuine,Report #5932
SimpleToolCollection,Plates,1.0,100.0,True,False,Genuine,
SimpleToolCollection,Plates,1.0,100.0,True,False,Fake,Report #5933
SimpleToolCollection,Plates,1.0,100.0,True,False,Fake,
SimpleToolCollection,Plates,1.0,100.0,True,False,,Report #5934
SimpleToolCollection,Plates,1.0,100.0,True,False,,
SimpleToolCollection,Plates,1.0,100.0,False,True,Genuine,Report #5935
SimpleToolCollection,Plates,1.0,100.0,False,True,Genuine,
SimpleToolCollection,Plates,1.0,100.0,False,True,Fake,Report #5936
SimpleToolCollection,Plates,1.0,100.0,False,True,Fake,
SimpleToolCollection,Plates,1.0,100.0,False,True,,Report #5937
SimpleToolCollection,Plates,1.0,100.0,False,True,,
SimpleToolCollection,Plates,1.0,100.0,False,False,Genuine,Report #5938
SimpleToolCollection,Plates,1.0,100.0,False,False,Genuine,
SimpleToolCollection,Plates,1.0,100.0,False,False,Fake,Report #5939
SimpleToolCollection,Plates,1.0,100.0,False,False,Fake,
SimpleToolCollection,Plates,1.0,100.0,False,False,,Report #5940
SimpleToolCollection,Plates,1.0,100.0,False,False,,
SimpleToolCollection,Plates,1.0,101.0,True,True,Genuine,Report #5941
SimpleToolCollection,Plates,1.0,101.0,True,True,Genuine,
SimpleToolCollection,Plates,1.0,101.0,True,True,Fake,Report #5942
SimpleToolCollection,Plates,1.0,101.0,True,True,Fake,
SimpleToolCollection,Plates,1.0,101.0,True,True,,Report #5943
SimpleToolCollection,Plates,1.0,101.0,True,True,,
SimpleToolCollection,Plates,1.0,101.0,True,False,Genuine,Report #5944
SimpleToolCollection,Plates,1.0,101.0,True,False,Genuine,
SimpleToolCollection,Plates,1.0,101.0,True,False,Fake,Report #5945
SimpleToolCollection,Plates,1.0,101.0,True,False,Fake,
SimpleToolCollection,Plates,1.0,101.0,True,False,,Report #5946
SimpleToolCollection,Plates,1.0,101.0,True,False,,
SimpleToolCollection,Plates,1.0,101.0,False,True,Genuine,Report #5947
SimpleToolCollection,Plates,1.0,101.0,False,True,Genuine,
SimpleToolCollection,Plates,1.0,101.0,False,True,Fake,Report #5948
SimpleToolCollection,Plates,1.0,101.0,False,True,Fake,
SimpleToolCollection,Plates,1.0,101.0,False,True,,Report #5949
SimpleToolCollection,Plates,1.0,101.0,False,True,,
SimpleToolCollection,Plates,1.0,101.0,False,False,Genuine,Report #5950
SimpleToolCollection,Plates,1.0,101.0,False,False,Genuine,
SimpleToolCollection,Plates,1.0,101.0,False,False,Fake,Report #5951
SimpleToolCollection,Plates,1.0,101.0,False,False,Fake,
SimpleToolCollection,Plates,1.0,101.0,False,False,,Report #5952
SimpleToolCollection,Plates,1.0,101.0,False,False,,
SimpleToolCollection,Plates,100.0,-1.0,True,True,Genuine,Report #5953
SimpleToolCollection,Plates,100.0,-1.0,True,True,Genuine,
SimpleToolCollection,Plates,100.0,-1.0,True,True,Fake,Report #5954
SimpleToolCollection,Plates,100.0,-1.0,True,True,Fake,
SimpleToolCollection,Plates,100.0,-1.0,True,True,,Report #5955
SimpleToolCollection,Plates,100.0,-1.0,True,True,,
SimpleToolCollection,Plates,100.0,-1.0,True,False,Genuine,Report #5956
SimpleToolCollection,Plates,100.0,-1.0,True,False,Genuine,
SimpleToolCollection,Plates,100.0,-1.0,True,False,Fake,Report #5957
SimpleToolCollection,Plates,100.0,-1.0,True,False,Fake,
SimpleToolCollection,Plates,100.0,-1.0,True,False,,Report #5958
SimpleToolCollection,Plates,100.0,-1.0,True,False,,
SimpleToolCollection,Plates,100.0,-1.0,False,True,Genuine,Report #5959
SimpleToolCollection,Plates,100.0,-1.0,False,True,Genuine,
SimpleToolCollection,Plates,100.0,-1.0,False,True,Fake,Report #5960
SimpleToolCollection,Plates,100.0,-1.0,False,True,Fake,
SimpleToolCollection,Plates,100.0,-1.0,False,True,,Report #5961
SimpleToolCollection,Plates,100.0,-1.0,False,True,,
SimpleToolCollection,Plates,100.0,-1.0,False,False,Genuine,Report #5962
SimpleToolCollection,Plates,100.0,-1.0,False,False,Genuine,
SimpleToolCollection,Plates,100.0,-1.0,False,False,Fake,Report #5963
SimpleToolCollection,Plates,100.0,-1.0,False,False,Fake,
SimpleToolCollection,Plates,100.0,-1.0,False,False,,Report #5964
SimpleToolCollection,Plates,100.0,-1.0,False,False,,
SimpleToolCollection,Plates,100.0,0.0,True,True,Genuine,Report #5965
SimpleToolCollection,Plates,100.0,0.0,True,True,Genuine,
SimpleToolCollection,Plates,100.0,0.0,True,True,Fake,Report #5966
SimpleToolCollection,Plates,100.0,0.0,True,True,Fake,
SimpleToolCollection,Plates,100.0,0.0,True,True,,Report #5967
SimpleToolCollection,Plates,100.0,0.0,True,True,,
SimpleToolCollection,Plates,100.0,0.0,True,False,Genuine,Report #5968
SimpleToolCollection,Plates,100.0,0.0,True,False,Genuine,
SimpleToolCollection,Plates,100.0,0.0,True,False,Fake,Report #5969
SimpleToolCollection,Plates,100.0,0.0,True,False,Fake,
SimpleToolCollection,Plates,100.0,0.0,True,False,,Report #5970
SimpleToolCollection,Plates,100.0,0.0,True,False,,
SimpleToolCollection,Plates,100.0,0.0,False,True,Genuine,Report #5971
SimpleToolCollection,Plates,100.0,0.0,False,True,Genuine,
SimpleToolCollection,Plates,100.0,0.0,False,True,Fake,Report #5972
SimpleToolCollection,Plates,100.0,0.0,False,True,Fake,
SimpleToolCollection,Plates,100.0,0.0,False,True,,Report #5973
SimpleToolCollection,Plates,100.0,0.0,False,True,,
SimpleToolCollection,Plates,100.0,0.0,False,False,Genuine,Report #5974
SimpleToolCollection,Plates,100.0,0.0,False,False,Genuine,
SimpleToolCollection,Plates,100.0,0.0,False,False,Fake,Report #5975
SimpleToolCollection,Plates,100.0,0.0,False,False,Fake,
SimpleToolCollection,Plates,100.0,0.0,False,False,,Report #5976
SimpleToolCollection,Plates,100.0,0.0,False,False,,
SimpleToolCollection,Plates,100.0,19.0,True,True,Genuine,Report #5977
SimpleToolCollection,Plates,100.0,19.0,True,True,Genuine,
SimpleToolCollection,Plates,100.0,19.0,True,True,Fake,Report #5978
SimpleToolCollection,Plates,100.0,19.0,True,True,Fake,
SimpleToolCollection,Plates,100.0,19.0,True,True,,Report #5979
SimpleToolCollection,Plates,100.0,19.0,True,True,,
SimpleToolCollection,Plates,100.0,19.0,True,False,Genuine,Report #5980
SimpleToolCollection,Plates,100.0,19.0,True,False,Genuine,
SimpleToolCollection,Plates,100.0,19.0,True,False,Fake,Report #5981
SimpleToolCollection,Plates,100.0,19.0,True,False,Fake,
SimpleToolCollection,Plates,100.0,19.0,True,False,,Report #5982
SimpleToolCollection,Plates,100.0,19.0,True,False,,
SimpleToolCollection,Plates,100.0,19.0,False,True,Genuine,Report #5983
SimpleToolCollection,Plates,100.0,19.0,False,True,Genuine,
SimpleToolCollection,Plates,100.0,19.0,False,True,Fake,Report #5984
SimpleToolCollection,Plates,100.0,19.0,False,True,Fake,
SimpleToolCollection,Plates,100.0,19.0,False,True,,Report #5985
SimpleToolCollection,Plates,100.0,19.0,False,True,,
SimpleToolCollection,Plates,100.0,19.0,False,False,Genuine,Report #5986
SimpleToolCollection,Plates,100.0,19.0,False,False,Genuine,
SimpleToolCollection,Plates,100.0,19.0,False,False,Fake,Report #5987
SimpleToolCollection,Plates,100.0,19.0,False,False,Fake,
SimpleToolCollection,Plates,100.0,19.0,False,False,,Report #5988
SimpleToolCollection,Plates,100.0,19.0,False,False,,
SimpleToolCollection,Plates,100.0,20.0,True,True,Genuine,Report #5989
SimpleToolCollection,Plates,100.0,20.0,True,True,Genuine,
SimpleToolCollection,Plates,100.0,20.0,True,True,Fake,Report #5990
SimpleToolCollection,Plates,100.0,20.0,True,True,Fake,
SimpleToolCollection,Plates,100.0,20.0,True,True,,Report #5991
SimpleToolCollection,Plates,100.0,20.0,True,True,,
SimpleToolCollection,Plates,100.0,20.0,True,False,Genuine,Report #5992
SimpleToolCollection,Plates,100.0,20.0,True,False,Genuine,
SimpleToolCollection,Plates,100.0,20.0,True,False,Fake,Report #5993
SimpleToolCollection,Plates,100.0,20.0,True,False,Fake,
SimpleToolCollection,Plates,100.0,20.0,True,False,,Report #5994
SimpleToolCollection,Plates,100.0,20.0,True,False,,
SimpleToolCollection,Plates,100.0,20.0,False,True,Genuine,Report #5995
SimpleToolCollection,Plates,100.0,20.0,False,True,Genuine,
SimpleToolCollection,Plates,100.0,20.0,False,True,Fake,Report #5996
SimpleToolCollection,Plates,100.0,20.0,False,True,Fake,
SimpleToolCollection,Plates,100.0,20.0,False,True,,Report #5997
SimpleToolCollection,Plates,100.0,20.0,False,True,,
SimpleToolCollection,Plates,100.0,20.0,False,False,Genuine,Report #5998
SimpleToolCollection,Plates,100.0,20.0,False,False,Genuine,
SimpleToolCollection,Plates,100.0,20.0,False,False,Fake,Report #5999
SimpleToolCollection,Plates,100.0,20.0,False,False,Fake,
SimpleToolCollection,Plates,100.0,20.0,False,False,,Report #6000
SimpleToolCollection,Plates,100.0,20.0,False,False,,
SimpleToolCollection,Plates,100.0,21.0,True,True,Genuine,Report #6001
SimpleToolCollection,Plates,100.0,21.0,True,True,Genuine,
SimpleToolCollection,Plates,100.0,21.0,True,True,Fake,Report #6002
SimpleToolCollection,Plates,100.0,21.0,True,True,Fake,
SimpleToolCollection,Plates,100.0,21.0,True,True,,Report #6003
SimpleToolCollection,Plates,100.0,21.0,True,True,,
SimpleToolCollection,Plates,100.0,21.0,True,False,Genuine,Report #6004
SimpleToolCollection,Plates,100.0,21.0,True,False,Genuine,
SimpleToolCollection,Plates,100.0,21.0,True,False,Fake,Report #6005
SimpleToolCollection,Plates,100.0,21.0,True,False,Fake,
SimpleToolCollection,Plates,100.0,21.0,True,False,,Report #6006
SimpleToolCollection,Plates,100.0,21.0,True,False,,
SimpleToolCollection,Plates,100.0,21.0,False,True,Genuine,Report #6007
SimpleToolCollection,Plates,100.0,21.0,False,True,Genuine,
SimpleToolCollection,Plates,100.0,21.0,False,True,Fake,Report #6008
SimpleToolCollection,Plates,100.0,21.0,False,True,Fake,
SimpleToolCollection,Plates,100.0,21.0,False,True,,Report #6009
SimpleToolCollection,Plates,100.0,21.0,False,True,,
SimpleToolCollection,Plates,100.0,21.0,False,False,Genuine,Report #6010
SimpleToolCollection,Plates,100.0,21.0,False,False,Genuine,
SimpleToolCollection,Plates,100.0,21.0,False,False,Fake,Report #6011
SimpleToolCollection,Plates,100.0,21.0,False,False,Fake,
SimpleToolCollection,Plates,100.0,21.0,False,False,,Report #6012
SimpleToolCollection,Plates,100.0,21.0,False,False,,
SimpleToolCollection,Plates,100.0,99.0,True,True,Genuine,Report #6013
SimpleToolCollection,Plates,100.0,99.0,True,True,Genuine,
SimpleToolCollection,Plates,100.0,99.0,True,True,Fake,Report #6014
SimpleToolCollection,Plates,100.0,99.0,True,True,Fake,
SimpleToolCollection,Plates,100.0,99.0,True,True,,Report #6015
SimpleToolCollection,Plates,100.0,99.0,True,True,,
SimpleToolCollection,Plates,100.0,99.0,True,False,Genuine,Report #6016
SimpleToolCollection,Plates,100.0,99.0,True,False,Genuine,
SimpleToolCollection,Plates,100.0,99.0,True,False,Fake,Report #6017
SimpleToolCollection,Plates,100.0,99.0,True,False,Fake,
SimpleToolCollection,Plates,100.0,99.0,True,False,,Report #6018
SimpleToolCollection,Plates,100.0,99.0,True,False,,
SimpleToolCollection,Plates,100.0,99.0,False,True,Genuine,Report #6019
SimpleToolCollection,Plates,100.0,99.0,False,True,Genuine,
SimpleToolCollection,Plates,100.0,99.0,False,True,Fake,Report #6020
SimpleToolCollection,Plates,100.0,99.0,False,True,Fake,
SimpleToolCollection,Plates,100.0,99.0,False,True,,Report #6021
SimpleToolCollection,Plates,100.0,99.0,False,True,,
SimpleToolCollection,Plates,100.0,99.0,False,False,Genuine,Report #6022
SimpleToolCollection,Plates,100.0,99.0,False,False,Genuine,
SimpleToolCollection,Plates,100.0,99.0,False,False,Fake,Report #6023
SimpleToolCollection,Plates,100.0,99.0,False,False,Fake,
SimpleToolCollection,Plates,100.0,99.0,False,False,,Report #6024
SimpleToolCollection,Plates,100.0,99.0,False,False,,
SimpleToolCollection,Plates,100.0,100.0,True,True,Genuine,Report #6025
SimpleToolCollection,Plates,100.0,100.0,True,True,Genuine,
SimpleToolCollection,Plates,100.0,100.0,True,True,Fake,Report #6026
SimpleToolCollection,Plates,100.0,100.0,True,True,Fake,
SimpleToolCollection,Plates,100.0,100.0,True,True,,Report #6027
SimpleToolCollection,Plates,100.0,100.0,True,True,,
SimpleToolCollection,Plates,100.0,100.0,True,False,Genuine,Report #6028
SimpleToolCollection,Plates,100.0,100.0,True,False,Genuine,
SimpleToolCollection,Plates,100.0,100.0,True,False,Fake,Report #6029
SimpleToolCollection,Plates,100.0,100.0,True,False,Fake,
SimpleToolCollection,Plates,100.0,100.0,True,False,,Report #6030
SimpleToolCollection,Plates,100.0,100.0,True,False,,
SimpleToolCollection,Plates,100.0,100.0,False,True,Genuine,Report #6031
SimpleToolCollection,Plates,100.0,100.0,False,True,Genuine,
SimpleToolCollection,Plates,100.0,100.0,False,True,Fake,Report #6032
SimpleToolCollection,Plates,100.0,100.0,False,True,Fake,
SimpleToolCollection,Plates,100.0,100.0,False,True,,Report #6033
SimpleToolCollection,Plates,100.0,100.0,False,True,,
SimpleToolCollection,Plates,100.0,100.0,False,False,Genuine,Report #6034
SimpleToolCollection,Plates,100.0,100.0,False,False,Genuine,
SimpleToolCollection,Plates,100.0,100.0,False,False,Fake,Report #6035
SimpleToolCollection,Plates,100.0,100.0,False,False,Fake,
SimpleToolCollection,Plates,100.0,100.0,False,False,,Report #6036
SimpleToolCollection,Plates,100.0,100.0,False,False,,
SimpleToolCollection,Plates,100.0,101.0,True,True,Genuine,Report #6037
SimpleToolCollection,Plates,100.0,101.0,True,True,Genuine,
SimpleToolCollection,Plates,100.0,101.0,True,True,Fake,Report #6038
SimpleToolCollection,Plates,100.0,101.0,True,True,Fake,
SimpleToolCollection,Plates,100.0,101.0,True,True,,Report #6039
SimpleToolCollection,Plates,100.0,101.0,True,True,,
SimpleToolCollection,Plates,100.0,101.0,True,False,Genuine,Report #6040
SimpleToolCollection,Plates,100.0,101.0,True,False,Genuine,
SimpleToolCollection,Plates,100.0,101.0,True,False,Fake,Report #6041
SimpleToolCollection,Plates,100.0,101.0,True,False,Fake,
SimpleToolCollection,Plates,100.0,101.0,True,False,,Report #6042
SimpleToolCollection,Plates,100.0,101.0,True,False,,
SimpleToolCollection,Plates,100.0,101.0,False,True,Genuine,Report #6043
SimpleToolCollection,Plates,100.0,101.0,False,True,Genuine,
SimpleToolCollection,Plates,100.0,101.0,False,True,Fake,Report #6044
SimpleToolCollection,Plates,100.0,101.0,False,True,Fake,
SimpleToolCollection,Plates,100.0,101.0,False,True,,Report #6045
SimpleToolCollection,Plates,100.0,101.0,False,True,,
SimpleToolCollection,Plates,100.0,101.0,False,False,Genuine,Report #6046
SimpleToolCollection,Plates,100.0,101.0,False,False,Genuine,
SimpleToolCollection,Plates,100.0,101.0,False,False,Fake,Report #6047
SimpleToolCollection,Plates,100.0,101.0,False,False,Fake,
SimpleToolCollection,Plates,100.0,101.0,False,False,,Report #6048
SimpleToolCollection,Plates,100.0,101.0,False,False,,
SimpleToolCollection,Plates,1000.0,-1.0,True,True,Genuine,Report #6049
SimpleToolCollection,Plates,1000.0,-1.0,True,True,Genuine,
SimpleToolCollection,Plates,1000.0,-1.0,True,True,Fake,Report #6050
SimpleToolCollection,Plates,1000.0,-1.0,True,True,Fake,
SimpleToolCollection,Plates,1000.0,-1.0,True,True,,Report #6051
SimpleToolCollection,Plates,1000.0,-1.0,True,True,,
SimpleToolCollection,Plates,1000.0,-1.0,True,False,Genuine,Report #6052
SimpleToolCollection,Plates,1000.0,-1.0,True,False,Genuine,
SimpleToolCollection,Plates,1000.0,-1.0,True,False,Fake,Report #6053
SimpleToolCollection,Plates,1000.0,-1.0,True,False,Fake,
SimpleToolCollection,Plates,1000.0,-1.0,True,False,,Report #6054
SimpleToolCollection,Plates,1000.0,-1.0,True,False,,
SimpleToolCollection,Plates,1000.0,-1.0,False,True,Genuine,Report #6055
SimpleToolCollection,Plates,1000.0,-1.0,False,True,Genuine,
SimpleToolCollection,Plates,1000.0,-1.0,False,True,Fake,Report #6056
SimpleToolCollection,Plates,1000.0,-1.0,False,True,Fake,
SimpleToolCollection,Plates,1000.0,-1.0,False,True,,Report #6057
SimpleToolCollection,Plates,1000.0,-1.0,False,True,,
SimpleToolCollection,Plates,1000.0,-1.0,False,False,Genuine,Report #6058
SimpleToolCollection,Plates,1000.0,-1.0,False,False,Genuine,
SimpleToolCollection,Plates,1000.0,-1.0,False,False,Fake,Report #6059
SimpleToolCollection,Plates,1000.0,-1.0,False,False,Fake,
SimpleToolCollection,Plates,1000.0,-1.0,False,False,,Report #6060
SimpleToolCollection,Plates,1000.0,-1.0,False,False,,
SimpleToolCollection,Plates,1000.0,0.0,True,True,Genuine,Report #6061
SimpleToolCollection,Plates,1000.0,0.0,True,True,Genuine,
SimpleToolCollection,Plates,1000.0,0.0,True,True,Fake,Report #6062
SimpleToolCollection,Plates,1000.0,0.0,True,True,Fake,
SimpleToolCollection,Plates,1000.0,0.0,True,True,,Report #6063
SimpleToolCollection,Plates,1000.0,0.0,True,True,,
SimpleToolCollection,Plates,1000.0,0.0,True,False,Genuine,Report #6064
SimpleToolCollection,Plates,1000.0,0.0,True,False,Genuine,
SimpleToolCollection,Plates,1000.0,0.0,True,False,Fake,Report #6065
SimpleToolCollection,Plates,1000.0,0.0,True,False,Fake,
SimpleToolCollection,Plates,1000.0,0.0,True,False,,Report #6066
SimpleToolCollection,Plates,1000.0,0.0,True,False,,
SimpleToolCollection,Plates,1000.0,0.0,False,True,Genuine,Report #6067
SimpleToolCollection,Plates,1000.0,0.0,False,True,Genuine,
SimpleToolCollection,Plates,1000.0,0.0,False,True,Fake,Report #6068
SimpleToolCollection,Plates,1000.0,0.0,False,True,Fake,
SimpleToolCollection,Plates,1000.0,0.0,False,True,,Report #6069
SimpleToolCollection,Plates,1000.0,0.0,False,True,,
SimpleToolCollection,Plates,1000.0,0.0,False,False,Genuine,Report #6070
SimpleToolCollection,Plates,1000.0,0.0,False,False,Genuine,
SimpleToolCollection,Plates,1000.0,0.0,False,False,Fake,Report #6071
SimpleToolCollection,Plates,1000.0,0.0,False,False,Fake,
SimpleToolCollection,Plates,1000.0,0.0,False,False,,Report #6072
SimpleToolCollection,Plates,1000.0,0.0,False,False,,
SimpleToolCollection,Plates,1000.0,19.0,True,True,Genuine,Report #6073
SimpleToolCollection,Plates,1000.0,19.0,True,True,Genuine,
SimpleToolCollection,Plates,1000.0,19.0,True,True,Fake,Report #6074
SimpleToolCollection,Plates,1000.0,19.0,True,True,Fake,
SimpleToolCollection,Plates,1000.0,19.0,True,True,,Report #6075
SimpleToolCollection,Plates,1000.0,19.0,True,True,,
SimpleToolCollection,Plates,1000.0,19.0,True,False,Genuine,Report #6076
SimpleToolCollection,Plates,1000.0,19.0,True,False,Genuine,
SimpleToolCollection,Plates,1000.0,19.0,True,False,Fake,Report #6077
SimpleToolCollection,Plates,1000.0,19.0,True,False,Fake,
SimpleToolCollection,Plates,1000.0,19.0,True,False,,Report #6078
SimpleToolCollection,Plates,1000.0,19.0,True,False,,
SimpleToolCollection,Plates,1000.0,19.0,False,True,Genuine,Report #6079
SimpleToolCollection,Plates,1000.0,19.0,False,True,Genuine,
SimpleToolCollection,Plates,1000.0,19.0,False,True,Fake,Report #6080
SimpleToolCollection,Plates,1000.0,19.0,False,True,Fake,
SimpleToolCollection,Plates,1000.0,19.0,False,True,,Report #6081
SimpleToolCollection,Plates,1000.0,19.0,False,True,,
SimpleToolCollection,Plates,1000.0,19.0,False,False,Genuine,Report #6082
SimpleToolCollection,Plates,1000.0,19.0,False,False,Genuine,
SimpleToolCollection,Plates,1000.0,19.0,False,False,Fake,Report #6083
SimpleToolCollection,Plates,1000.0,19.0,False,False,Fake,
SimpleToolCollection,Plates,1000.0,19.0,False,False,,Report #6084
SimpleToolCollection,Plates,1000.0,19.0,False,False,,
SimpleToolCollection,Plates,1000.0,20.0,True,True,Genuine,Report #6085
SimpleToolCollection,Plates,1000.0,20.0,True,True,Genuine,
SimpleToolCollection,Plates,1000.0,20.0,True,True,Fake,Report #6086
SimpleToolCollection,Plates,1000.0,20.0,True,True,Fake,
SimpleToolCollection,Plates,1000.0,20.0,True,True,,Report #6087
SimpleToolCollection,Plates,1000.0,20.0,True,True,,
SimpleToolCollection,Plates,1000.0,20.0,True,False,Genuine,Report #6088
SimpleToolCollection,Plates,1000.0,20.0,True,False,Genuine,
SimpleToolCollection,Plates,1000.0,20.0,True,False,Fake,Report #6089
SimpleToolCollection,Plates,1000.0,20.0,True,False,Fake,
SimpleToolCollection,Plates,1000.0,20.0,True,False,,Report #6090
SimpleToolCollection,Plates,1000.0,20.0,True,False,,
SimpleToolCollection,Plates,1000.0,20.0,False,True,Genuine,Report #6091
SimpleToolCollection,Plates,1000.0,20.0,False,True,Genuine,
SimpleToolCollection,Plates,1000.0,20.0,False,True,Fake,Report #6092
SimpleToolCollection,Plates,1000.0,20.0,False,True,Fake,
SimpleToolCollection,Plates,1000.0,20.0,False,True,,Report #6093
SimpleToolCollection,Plates,1000.0,20.0,False,True,,
SimpleToolCollection,Plates,1000.0,20.0,False,False,Genuine,Report #6094
SimpleToolCollection,Plates,1000.0,20.0,False,False,Genuine,
SimpleToolCollection,Plates,1000.0,20.0,False,False,Fake,Report #6095
SimpleToolCollection,Plates,1000.0,20.0,False,False,Fake,
SimpleToolCollection,Plates,1000.0,20.0,False,False,,Report #6096
SimpleToolCollection,Plates,1000.0,20.0,False,False,,
SimpleToolCollection,Plates,1000.0,21.0,True,True,Genuine,Report #6097
SimpleToolCollection,Plates,1000.0,21.0,True,True,Genuine,
SimpleToolCollection,Plates,1000.0,21.0,True,True,Fake,Report #6098
SimpleToolCollection,Plates,1000.0,21.0,True,True,Fake,
SimpleToolCollection,Plates,1000.0,21.0,True,True,,Report #6099
SimpleToolCollection,Plates,1000.0,21.0,True,True,,
SimpleToolCollection,Plates,1000.0,21.0,True,False,Genuine,Report #6100
SimpleToolCollection,Plates,1000.0,21.0,True,False,Genuine,
SimpleToolCollection,Plates,1000.0,21.0,True,False,Fake,Report #6101
SimpleToolCollection,Plates,1000.0,21.0,True,False,Fake,
SimpleToolCollection,Plates,1000.0,21.0,True,False,,Report #6102
SimpleToolCollection,Plates,1000.0,21.0,True,False,,
SimpleToolCollection,Plates,1000.0,21.0,False,True,Genuine,Report #6103
SimpleToolCollection,Plates,1000.0,21.0,False,True,Genuine,
SimpleToolCollection,Plates,1000.0,21.0,False,True,Fake,Report #6104
SimpleToolCollection,Plates,1000.0,21.0,False,True,Fake,
SimpleToolCollection,Plates,1000.0,21.0,False,True,,Report #6105
SimpleToolCollection,Plates,1000.0,21.0,False,True,,
SimpleToolCollection,Plates,1000.0,21.0,False,False,Genuine,Report #6106
SimpleToolCollection,Plates,1000.0,21.0,False,False,Genuine,
SimpleToolCollection,Plates,1000.0,21.0,False,False,Fake,Report #6107
SimpleToolCollection,Plates,1000.0,21.0,False,False,Fake,
SimpleToolCollection,Plates,1000.0,21.0,False,False,,Report #6108
SimpleToolCollection,Plates,1000.0,21.0,False,False,,
SimpleToolCollection,Plates,1000.0,99.0,True,True,Genuine,Report #6109
SimpleToolCollection,Plates,1000.0,99.0,True,True,Genuine,
SimpleToolCollection,Plates,1000.0,99.0,True,True,Fake,Report #6110
SimpleToolCollection,Plates,1000.0,99.0,True,True,Fake,
SimpleToolCollection,Plates,1000.0,99.0,True,True,,Report #6111
SimpleToolCollection,Plates,1000.0,99.0,True,True,,
SimpleToolCollection,Plates,1000.0,99.0,True,False,Genuine,Report #6112
SimpleToolCollection,Plates,1000.0,99.0,True,False,Genuine,
SimpleToolCollection,Plates,1000.0,99.0,True,False,Fake,Report #6113
SimpleToolCollection,Plates,1000.0,99.0,True,False,Fake,
SimpleToolCollection,Plates,1000.0,99.0,True,False,,Report #6114
SimpleToolCollection,Plates,1000.0,99.0,True,False,,
SimpleToolCollection,Plates,1000.0,99.0,False,True,Genuine,Report #6115
SimpleToolCollection,Plates,1000.0,99.0,False,True,Genuine,
SimpleToolCollection,Plates,1000.0,99.0,False,True,Fake,Report #6116
SimpleToolCollection,Plates,1000.0,99.0,False,True,Fake,
SimpleToolCollection,Plates,1000.0,99.0,False,True,,Report #6117
SimpleToolCollection,Plates,1000.0,99.0,False,True,,
SimpleToolCollection,Plates,1000.0,99.0,False,False,Genuine,Report #6118
SimpleToolCollection,Plates,1000.0,99.0,False,False,Genuine,
SimpleToolCollection,Plates,1000.0,99.0,False,False,Fake,Report #6119
SimpleToolCollection,Plates,1000.0,99.0,False,False,Fake,
SimpleToolCollection,Plates,1000.0,99.0,False,False,,Report #6120
SimpleToolCollection,Plates,1000.0,99.0,False,False,,
SimpleToolCollection,Plates,1000.0,100.0,True,True,Genuine,Report #6121
SimpleToolCollection,Plates,1000.0,100.0,True,True,Genuine,
SimpleToolCollection,Plates,1000.0,100.0,True,True,Fake,Report #6122
SimpleToolCollection,Plates,1000.0,100.0,True,True,Fake,
SimpleToolCollection,Plates,1000.0,100.0,True,True,,Report #6123
SimpleToolCollection,Plates,1000.0,100.0,True,True,,
SimpleToolCollection,Plates,1000.0,100.0,True,False,Genuine,Report #6124
SimpleToolCollection,Plates,1000.0,100.0,True,False,Genuine,
SimpleToolCollection,Plates,1000.0,100.0,True,False,Fake,Report #6125
SimpleToolCollection,Plates,1000.0,100.0,True,False,Fake,
SimpleToolCollection,Plates,1000.0,100.0,True,False,,Report #6126
SimpleToolCollection,Plates,1000.0,100.0,True,False,,
SimpleToolCollection,Plates,1000.0,100.0,False,True,Genuine,Report #6127
SimpleToolCollection,Plates,1000.0,100.0,False,True,Genuine,
SimpleToolCollection,Plates,1000.0,100.0,False,True,Fake,Report #6128
SimpleToolCollection,Plates,1000.0,100.0,False,True,Fake,
SimpleToolCollection,Plates,1000.0,100.0,False,True,,Report #6129
SimpleToolCollection,Plates,1000.0,100.0,False,True,,
SimpleToolCollection,Plates,1000.0,100.0,False,False,Genuine,Report #6130
SimpleToolCollection,Plates,1000.0,100.0,False,False,Genuine,
SimpleToolCollection,Plates,1000.0,100.0,False,False,Fake,Report #6131
SimpleToolCollection,Plates,1000.0,100.0,False,False,Fake,
SimpleToolCollection,Plates,1000.0,100.0,False,False,,Report #6132
SimpleToolCollection,Plates,1000.0,100.0,False,False,,
SimpleToolCollection,Plates,1000.0,101.0,True,True,Genuine,Report #6133
SimpleToolCollection,Plates,1000.0,101.0,True,True,Genuine,
SimpleToolCollection,Plates,1000.0,101.0,True,True,Fake,Report #6134
SimpleToolCollection,Plates,1000.0,101.0,True,True,Fake,
SimpleToolCollection,Plates,1000.0,101.0,True,True,,Report #6135
SimpleToolCollection,Plates,1000.0,101.0,True,True,,
SimpleToolCollection,Plates,1000.0,101.0,True,False,Genuine,Report #6136
SimpleToolCollection,Plates,1000.0,101.0,True,False,Genuine,
SimpleToolCollection,Plates,1000.0,101.0,True,False,Fake,Report #6137
SimpleToolCollection,Plates,1000.0,101.0,True,False,Fake,
SimpleToolCollection,Plates,1000.0,101.0,True,False,,Report #6138
SimpleToolCollection,Plates,1000.0,101.0,True,False,,
SimpleToolCollection,Plates,1000.0,101.0,False,True,Genuine,Report #6139
SimpleToolCollection,Plates,1000.0,101.0,False,True,Genuine,
SimpleToolCollection,Plates,1000.0,101.0,False,True,Fake,Report #6140
SimpleToolCollection,Plates,1000.0,101.0,False,True,Fake,
SimpleToolCollection,Plates,1000.0,101.0,False,True,,Report #6141
SimpleToolCollection,Plates,1000.0,101.0,False,True,,
SimpleToolCollection,Plates,1000.0,101.0,False,False,Genuine,Report #6142
SimpleToolCollection,Plates,1000.0,101.0,False,False,Genuine,
SimpleToolCollection,Plates,1000.0,101.0,False,False,Fake,Report #6143
SimpleToolCollection,Plates,1000.0,101.0,False,False,Fake,
SimpleToolCollection,Plates,1000.0,101.0,False,False,,Report #6144
SimpleToolCollection,Plates,1000.0,101.0,False,False,,
SimpleToolCollection,Plates,1000000.0,-1.0,True,True,Genuine,Report #6145
SimpleToolCollection,Plates,1000000.0,-1.0,True,True,Genuine,
SimpleToolCollection,Plates,1000000.0,-1.0,True,True,Fake,Report #6146
SimpleToolCollection,Plates,1000000.0,-1.0,True,True,Fake,
SimpleToolCollection,Plates,1000000.0,-1.0,True,True,,Report #6147
SimpleToolCollection,Plates,1000000.0,-1.0,True,True,,
SimpleToolCollection,Plates,1000000.0,-1.0,True,False,Genuine,Report #6148
SimpleToolCollection,Plates,1000000.0,-1.0,True,False,Genuine,
SimpleToolCollection,Plates,1000000.0,-1.0,True,False,Fake,Report #6149
SimpleToolCollection,Plates,1000000.0,-1.0,True,False,Fake,
SimpleToolCollection,Plates,1000000.0,-1.0,True,False,,Report #6150
SimpleToolCollection,Plates,1000000.0,-1.0,True,False,,
SimpleToolCollection,Plates,1000000.0,-1.0,False,True,Genuine,Report #6151
SimpleToolCollection,Plates,1000000.0,-1.0,False,True,Genuine,
SimpleToolCollection,Plates,1000000.0,-1.0,False,True,Fake,Report #6152
SimpleToolCollection,Plates,1000000.0,-1.0,False,True,Fake,
SimpleToolCollection,Plates,1000000.0,-1.0,False,True,,Report #6153
SimpleToolCollection,Plates,1000000.0,-1.0,False,True,,
SimpleToolCollection,Plates,1000000.0,-1.0,False,False,Genuine,Report #6154
SimpleToolCollection,Plates,1000000.0,-1.0,False,False,Genuine,
SimpleToolCollection,Plates,1000000.0,-1.0,False,False,Fake,Report #6155
SimpleToolCollection,Plates,1000000.0,-1.0,False,False,Fake,
SimpleToolCollection,Plates,1000000.0,-1.0,False,False,,Report #6156
SimpleToolCollection,Plates,1000000.0,-1.0,False,False,,
SimpleToolCollection,Plates,1000000.0,0.0,True,True,Genuine,Report #6157
SimpleToolCollection,Plates,1000000.0,0.0,True,True,Genuine,
SimpleToolCollection,Plates,1000000.0,0.0,True,True,Fake,Report #6158
SimpleToolCollection,Plates,1000000.0,0.0,True,True,Fake,
SimpleToolCollection,Plates,1000000.0,0.0,True,True,,Report #6159
SimpleToolCollection,Plates,1000000.0,0.0,True,True,,
SimpleToolCollection,Plates,1000000.0,0.0,True,False,Genuine,Report #6160
SimpleToolCollection,Plates,1000000.0,0.0,True,False,Genuine,
SimpleToolCollection,Plates,1000000.0,0.0,True,False,Fake,Report #6161
SimpleToolCollection,Plates,1000000.0,0.0,True,False,Fake,
SimpleToolCollection,Plates,1000000.0,0.0,True,False,,Report #6162
SimpleToolCollection,Plates,1000000.0,0.0,True,False,,
SimpleToolCollection,Plates,1000000.0,0.0,False,True,Genuine,Report #6163
SimpleToolCollection,Plates,1000000.0,0.0,False,True,Genuine,
SimpleToolCollection,Plates,1000000.0,0.0,False,True,Fake,Report #6164
SimpleToolCollection,Plates,1000000.0,0.0,False,True,Fake,
SimpleToolCollection,Plates,1000000.0,0.0,False,True,,Report #6165
SimpleToolCollection,Plates,1000000.0,0.0,False,True,,
SimpleToolCollection,Plates,1000000.0,0.0,False,False,Genuine,Report #6166
SimpleToolCollection,Plates,1000000.0,0.0,False,False,Genuine,
SimpleToolCollection,Plates,1000000.0,0.0,False,False,Fake,Report #6167
SimpleToolCollection,Plates,1000000.0,0.0,False,False,Fake,
SimpleToolCollection,Plates,1000000.0,0.0,False,False,,Report #6168
SimpleToolCollection,Plates,1000000.0,0.0,False,False,,
SimpleToolCollection,Plates,1000000.0,19.0,True,True,Genuine,Report #6169
SimpleToolCollection,Plates,1000000.0,19.0,True,True,Genuine,
SimpleToolCollection,Plates,1000000.0,19.0,True,True,Fake,Report #6170
SimpleToolCollection,Plates,1000000.0,19.0,True,True,Fake,
SimpleToolCollection,Plates,1000000.0,19.0,True,True,,Report #6171
SimpleToolCollection,Plates,1000000.0,19.0,True,True,,
SimpleToolCollection,Plates,1000000.0,19.0,True,False,Genuine,Report #6172
SimpleToolCollection,Plates,1000000.0,19.0,True,False,Genuine,
SimpleToolCollection,Plates,1000000.0,19.0,True,False,Fake,Report #6173
SimpleToolCollection,Plates,1000000.0,19.0,True,False,Fake,
SimpleToolCollection,Plates,1000000.0,19.0,True,False,,Report #6174
SimpleToolCollection,Plates,1000000.0,19.0,True,False,,
SimpleToolCollection,Plates,1000000.0,19.0,False,True,Genuine,Report #6175
SimpleToolCollection,Plates,1000000.0,19.0,False,True,Genuine,
SimpleToolCollection,Plates,1000000.0,19.0,False,True,Fake,Report #6176
SimpleToolCollection,Plates,1000000.0,19.0,False,True,Fake,
SimpleToolCollection,Plates,1000000.0,19.0,False,True,,Report #6177
SimpleToolCollection,Plates,1000000.0,19.0,False,True,,
SimpleToolCollection,Plates,1000000.0,19.0,False,False,Genuine,Report #6178
SimpleToolCollection,Plates,1000000.0,19.0,False,False,Genuine,
SimpleToolCollection,Plates,1000000.0,19.0,False,False,Fake,Report #6179
SimpleToolCollection,Plates,1000000.0,19.0,False,False,Fake,
SimpleToolCollection,Plates,1000000.0,19.0,False,False,,Report #6180
SimpleToolCollection,Plates,1000000.0,19.0,False,False,,
SimpleToolCollection,Plates,1000000.0,20.0,True,True,Genuine,Report #6181
SimpleToolCollection,Plates,1000000.0,20.0,True,True,Genuine,
SimpleToolCollection,Plates,1000000.0,20.0,True,True,Fake,Report #6182
SimpleToolCollection,Plates,1000000.0,20.0,True,True,Fake,
SimpleToolCollection,Plates,1000000.0,20.0,True,True,,Report #6183
SimpleToolCollection,Plates,1000000.0,20.0,True,True,,
SimpleToolCollection,Plates,1000000.0,20.0,True,False,Genuine,Report #6184
SimpleToolCollection,Plates,1000000.0,20.0,True,False,Genuine,
SimpleToolCollection,Plates,1000000.0,20.0,True,False,Fake,Report #6185
SimpleToolCollection,Plates,1000000.0,20.0,True,False,Fake,
SimpleToolCollection,Plates,1000000.0,20.0,True,False,,Report #6186
SimpleToolCollection,Plates,1000000.0,20.0,True,False,,
SimpleToolCollection,Plates,1000000.0,20.0,False,True,Genuine,Report #6187
SimpleToolCollection,Plates,1000000.0,20.0,False,True,Genuine,
SimpleToolCollection,Plates,1000000.0,20.0,False,True,Fake,Report #6188
SimpleToolCollection,Plates,1000000.0,20.0,False,True,Fake,
SimpleToolCollection,Plates,1000000.0,20.0,False,True,,Report #6189
SimpleToolCollection,Plates,1000000.0,20.0,False,True,,
SimpleToolCollection,Plates,1000000.0,20.0,False,False,Genuine,Report #6190
SimpleToolCollection,Plates,1000000.0,20.0,False,False,Genuine,
SimpleToolCollection,Plates,1000000.0,20.0,False,False,Fake,Report #6191
SimpleToolCollection,Plates,1000000.0,20.0,False,False,Fake,
SimpleToolCollection,Plates,1000000.0,20.0,False,False,,Report #6192
SimpleToolCollection,Plates,1000000.0,20.0,False,False,,
SimpleToolCollection,Plates,1000000.0,21.0,True,True,Genuine,Report #6193
SimpleToolCollection,Plates,1000000.0,21.0,True,True,Genuine,
SimpleToolCollection,Plates,1000000.0,21.0,True,True,Fake,Report #6194
SimpleToolCollection,Plates,1000000.0,21.0,True,True,Fake,
SimpleToolCollection,Plates,1000000.0,21.0,True,True,,Report #6195
SimpleToolCollection,Plates,1000000.0,21.0,True,True,,
SimpleToolCollection,Plates,1000000.0,21.0,True,False,Genuine,Report #6196
SimpleToolCollection,Plates,1000000.0,21.0,True,False,Genuine,
SimpleToolCollection,Plates,1000000.0,21.0,True,False,Fake,Report #6197
SimpleToolCollection,Plates,1000000.0,21.0,True,False,Fake,
SimpleToolCollection,Plates,1000000.0,21.0,True,False,,Report #6198
SimpleToolCollection,Plates,1000000.0,21.0,True,False,,
SimpleToolCollection,Plates,1000000.0,21.0,False,True,Genuine,Report #6199
SimpleToolCollection,Plates,1000000.0,21.0,False,True,Genuine,
SimpleToolCollection,Plates,1000000.0,21.0,False,True,Fake,Report #6200
SimpleToolCollection,Plates,1000000.0,21.0,False,True,Fake,
SimpleToolCollection,Plates,1000000.0,21.0,False,True,,Report #6201
SimpleToolCollection,Plates,1000000.0,21.0,False,True,,
SimpleToolCollection,Plates,1000000.0,21.0,False,False,Genuine,Report #6202
SimpleToolCollection,Plates,1000000.0,21.0,False,False,Genuine,
SimpleToolCollection,Plates,1000000.0,21.0,False,False,Fake,Report #6203
SimpleToolCollection,Plates,1000000.0,21.0,False,False,Fake,
SimpleToolCollection,Plates,1000000.0,21.0,False,False,,Report #6204
SimpleToolCollection,Plates,1000000.0,21.0,False,False,,
SimpleToolCollection,Plates,1000000.0,99.0,True,True,Genuine,Report #6205
SimpleToolCollection,Plates,1000000.0,99.0,True,True,Genuine,
SimpleToolCollection,Plates,1000000.0,99.0,True,True,Fake,Report #6206
SimpleToolCollection,Plates,1000000.0,99.0,True,True,Fake,
SimpleToolCollection,Plates,1000000.0,99.0,True,True,,Report #6207
SimpleToolCollection,Plates,1000000.0,99.0,True,True,,
SimpleToolCollection,Plates,1000000.0,99.0,True,False,Genuine,Report #6208
SimpleToolCollection,Plates,1000000.0,99.0,True,False,Genuine,
SimpleToolCollection,Plates,1000000.0,99.0,True,False,Fake,Report #6209
SimpleToolCollection,Plates,1000000.0,99.0,True,False,Fake,
SimpleToolCollection,Plates,1000000.0,99.0,True,False,,Report #6210
SimpleToolCollection,Plates,1000000.0,99.0,True,False,,
SimpleToolCollection,Plates,1000000.0,99.0,False,True,Genuine,Report #6211
SimpleToolCollection,Plates,1000000.0,99.0,False,True,Genuine,
SimpleToolCollection,Plates,1000000.0,99.0,False,True,Fake,Report #6212
SimpleToolCollection,Plates,1000000.0,99.0,False,True,Fake,
SimpleToolCollection,Plates,1000000.0,99.0,False,True,,Report #6213
SimpleToolCollection,Plates,1000000.0,99.0,False,True,,
SimpleToolCollection,Plates,1000000.0,99.0,False,False,Genuine,Report #6214
SimpleToolCollection,Plates,1000000.0,99.0,False,False,Genuine,
SimpleToolCollection,Plates,1000000.0,99.0,False,False,Fake,Report #6215
SimpleToolCollection,Plates,1000000.0,99.0,False,False,Fake,
SimpleToolCollection,Plates,1000000.0,99.0,False,False,,Report #6216
SimpleToolCollection,Plates,1000000.0,99.0,False,False,,
SimpleToolCollection,Plates,1000000.0,100.0,True,True,Genuine,Report #6217
SimpleToolCollection,Plates,1000000.0,100.0,True,True,Genuine,
SimpleToolCollection,Plates,1000000.0,100.0,True,True,Fake,Report #6218
SimpleToolCollection,Plates,1000000.0,100.0,True,True,Fake,
SimpleToolCollection,Plates,1000000.0,100.0,True,True,,Report #6219
SimpleToolCollection,Plates,1000000.0,100.0,True,True,,
SimpleToolCollection,Plates,1000000.0,100.0,True,False,Genuine,Report #6220
SimpleToolCollection,Plates,1000000.0,100.0,True,False,Genuine,
SimpleToolCollection,Plates,1000000.0,100.0,True,False,Fake,Report #6221
SimpleToolCollection,Plates,1000000.0,100.0,True,False,Fake,
SimpleToolCollection,Plates,1000000.0,100.0,True,False,,Report #6222
SimpleToolCollection,Plates,1000000.0,100.0,True,False,,
SimpleToolCollection,Plates,1000000.0,100.0,False,True,Genuine,Report #6223
SimpleToolCollection,Plates,1000000.0,100.0,False,True,Genuine,
SimpleToolCollection,Plates,1000000.0,100.0,False,True,Fake,Report #6224
SimpleToolCollection,Plates,1000000.0,100.0,False,True,Fake,
SimpleToolCollection,Plates,1000000.0,100.0,False,True,,Report #6225
SimpleToolCollection,Plates,1000000.0,100.0,False,True,,
SimpleToolCollection,Plates,1000000.0,100.0,False,False,Genuine,Report #6226
SimpleToolCollection,Plates,1000000.0,100.0,False,False,Genuine,
SimpleToolCollection,Plates,1000000.0,100.0,False,False,Fake,Report #6227
SimpleToolCollection,Plates,1000000.0,100.0,False,False,Fake,
SimpleToolCollection,Plates,1000000.0,100.0,False,False,,Report #6228
SimpleToolCollection,Plates,1000000.0,100.0,False,False,,
SimpleToolCollection,Plates,1000000.0,101.0,True,True,Genuine,Report #6229
SimpleToolCollection,Plates,1000000.0,101.0,True,True,Genuine,
SimpleToolCollection,Plates,1000000.0,101.0,True,True,Fake,Report #6230
SimpleToolCollection,Plates,1000000.0,101.0,True,True,Fake,
SimpleToolCollection,Plates,1000000.0,101.0,True,True,,Report #6231
SimpleToolCollection,Plates,1000000.0,101.0,True,True,,
SimpleToolCollection,Plates,1000000.0,101.0,True,False,Genuine,Report #6232
SimpleToolCollection,Plates,1000000.0,101.0,True,False,Genuine,
SimpleToolCollection,Plates,1000000.0,101.0,True,False,Fake,Report #6233
SimpleToolCollection,Plates,1000000.0,101.0,True,False,Fake,
SimpleToolCollection,Plates,1000000.0,101.0,True,False,,Report #6234
SimpleToolCollection,Plates,1000000.0,101.0,True,False,,
SimpleToolCollection,Plates,1000000.0,101.0,False,True,Genuine,Report #6235
SimpleToolCollection,Plates,1000000.0,101.0,False,True,Genuine,
SimpleToolCollection,Plates,1000000.0,101.0,False,True,Fake,Report #6236
SimpleToolCollection,Plates,1000000.0,101.0,False,True,Fake,
SimpleToolCollection,Plates,1000000.0,101.0,False,True,,Report #6237
SimpleToolCollection,Plates,1000000.0,101.0,False,True,,
SimpleToolCollection,Plates,1000000.0,101.0,False,False,Genuine,Report #6238
SimpleToolCollection,Plates,1000000.0,101.0,False,False,Genuine,
SimpleToolCollection,Plates,1000000.0,101.0,False,False,Fake,Report #6239
SimpleToolCollection,Plates,1000000.0,101.0,False,False,Fake,
SimpleToolCollection,Plates,1000000.0,101.0,False,False,,Report #6240
SimpleToolCollection,Plates,1000000.0,101.0,False,False,,
SimpleToolCollection,Furniture,0.0,-1.0,True,True,Genuine,Report #6241
SimpleToolCollection,Furniture,0.0,-1.0,True,True,Genuine,
SimpleToolCollection,Furniture,0.0,-1.0,True,True,Fake,Report #6242
SimpleToolCollection,Furniture,0.0,-1.0,True,True,Fake,
SimpleToolCollection,Furniture,0.0,-1.0,True,True,,Report #6243
SimpleToolCollection,Furniture,0.0,-1.0,True,True,,
SimpleToolCollection,Furniture,0.0,-1.0,True,False,Genuine,Report #6244
SimpleToolCollection,Furniture,0.0,-1.0,True,False,Genuine,
SimpleToolCollection,Furniture,0.0,-1.0,True,False,Fake,Report #6245
SimpleToolCollection,Furniture,0.0,-1.0,True,False,Fake,
SimpleToolCollection,Furniture,0.0,-1.0,True,False,,Report #6246
SimpleToolCollection,Furniture,0.0,-1.0,True,False,,
SimpleToolCollection,Furniture,0.0,-1.0,False,True,Genuine,Report #6247
SimpleToolCollection,Furniture,0.0,-1.0,False,True,Genuine,
SimpleToolCollection,Furniture,0.0,-1.0,False,True,Fake,Report #6248
SimpleToolCollection,Furniture,0.0,-1.0,False,True,Fake,
SimpleToolCollection,Furniture,0.0,-1.0,False,True,,Report #6249
SimpleToolCollection,Furniture,0.0,-1.0,False,True,,
SimpleToolCollection,Furniture,0.0,-1.0,False,False,Genuine,Report #6250
SimpleToolCollection,Furniture,0.0,-1.0,False,False,Genuine,
SimpleToolCollection,Furniture,0.0,-1.0,False,False,Fake,Report #6251
SimpleToolCollection,Furniture,0.0,-1.0,False,False,Fake,
SimpleToolCollection,Furniture,0.0,-1.0,False,False,,Report #6252
SimpleToolCollection,Furniture,0.0,-1.0,False,False,,
SimpleToolCollection,Furniture,0.0,0.0,True,True,Genuine,Report #6253
SimpleToolCollection,Furniture,0.0,0.0,True,True,Genuine,
SimpleToolCollection,Furniture,0.0,0.0,True,True,Fake,Report #6254
SimpleToolCollection,Furniture,0.0,0.0,True,True,Fake,
SimpleToolCollection,Furniture,0.0,0.0,True,True,,Report #6255
SimpleToolCollection,Furniture,0.0,0.0,True,True,,
SimpleToolCollection,Furniture,0.0,0.0,True,False,Genuine,Report #6256
SimpleToolCollection,Furniture,0.0,0.0,True,False,Genuine,
SimpleToolCollection,Furniture,0.0,0.0,True,False,Fake,Report #6257
SimpleToolCollection,Furniture,0.0,0.0,True,False,Fake,
SimpleToolCollection,Furniture,0.0,0.0,True,False,,Report #6258
SimpleToolCollection,Furniture,0.0,0.0,True,False,,
SimpleToolCollection,Furniture,0.0,0.0,False,True,Genuine,Report #6259
SimpleToolCollection,Furniture,0.0,0.0,False,True,Genuine,
SimpleToolCollection,Furniture,0.0,0.0,False,True,Fake,Report #6260
SimpleToolCollection,Furniture,0.0,0.0,False,True,Fake,
SimpleToolCollection,Furniture,0.0,0.0,False,True,,Report #6261
SimpleToolCollection,Furniture,0.0,0.0,False,True,,
SimpleToolCollection,Furniture,0.0,0.0,False,False,Genuine,Report #6262
SimpleToolCollection,Furniture,0.0,0.0,False,False,Genuine,
SimpleToolCollection,Furniture,0.0,0.0,False,False,Fake,Report #6263
SimpleToolCollection,Furniture,0.0,0.0,False,False,Fake,
SimpleToolCollection,Furniture,0.0,0.0,False,False,,Report #6264
SimpleToolCollection,Furniture,0.0,0.0,False,False,,
SimpleToolCollection,Furniture,0.0,19.0,True,True,Genuine,Report #6265
SimpleToolCollection,Furniture,0.0,19.0,True,True,Genuine,
SimpleToolCollection,Furniture,0.0,19.0,True,True,Fake,Report #6266
SimpleToolCollection,Furniture,0.0,19.0,True,True,Fake,
SimpleToolCollection,Furniture,0.0,19.0,True,True,,Report #6267
SimpleToolCollection,Furniture,0.0,19.0,True,True,,
SimpleToolCollection,Furniture,0.0,19.0,True,False,Genuine,Report #6268
SimpleToolCollection,Furniture,0.0,19.0,True,False,Genuine,
SimpleToolCollection,Furniture,0.0,19.0,True,False,Fake,Report #6269
SimpleToolCollection,Furniture,0.0,19.0,True,False,Fake,
SimpleToolCollection,Furniture,0.0,19.0,True,False,,Report #6270
SimpleToolCollection,Furniture,0.0,19.0,True,False,,
SimpleToolCollection,Furniture,0.0,19.0,False,True,Genuine,Report #6271
SimpleToolCollection,Furniture,0.0,19.0,False,True,Genuine,
SimpleToolCollection,Furniture,0.0,19.0,False,True,Fake,Report #6272
SimpleToolCollection,Furniture,0.0,19.0,False,True,Fake,
SimpleToolCollection,Furniture,0.0,19.0,False,True,,Report #6273
SimpleToolCollection,Furniture,0.0,19.0,False,True,,
SimpleToolCollection,Furniture,0.0,19.0,False,False,Genuine,Report #6274
SimpleToolCollection,Furniture,0.0,19.0,False,False,Genuine,
SimpleToolCollection,Furniture,0.0,19.0,False,False,Fake,Report #6275
SimpleToolCollection,Furniture,0.0,19.0,False,False,Fake,
SimpleToolCollection,Furniture,0.0,19.0,False,False,,Report #6276
SimpleToolCollection,Furniture,0.0,19.0,False,False,,
SimpleToolCollection,Furniture,0.0,20.0,True,True,Genuine,Report #6277
SimpleToolCollection,Furniture,0.0,20.0,True,True,Genuine,
SimpleToolCollection,Furniture,0.0,20.0,True,True,Fake,Report #6278
SimpleToolCollection,Furniture,0.0,20.0,True,True,Fake,
SimpleToolCollection,Furniture,0.0,20.0,True,True,,Report #6279
SimpleToolCollection,Furniture,0.0,20.0,True,True,,
SimpleToolCollection,Furniture,0.0,20.0,True,False,Genuine,Report #6280
SimpleToolCollection,Furniture,0.0,20.0,True,False,Genuine,
SimpleToolCollection,Furniture,0.0,20.0,True,False,Fake,Report #6281
SimpleToolCollection,Furniture,0.0,20.0,True,False,Fake,
SimpleToolCollection,Furniture,0.0,20.0,True,False,,Report #6282
SimpleToolCollection,Furniture,0.0,20.0,True,False,,
SimpleToolCollection,Furniture,0.0,20.0,False,True,Genuine,Report #6283
SimpleToolCollection,Furniture,0.0,20.0,False,True,Genuine,
SimpleToolCollection,Furniture,0.0,20.0,False,True,Fake,Report #6284
SimpleToolCollection,Furniture,0.0,20.0,False,True,Fake,
SimpleToolCollection,Furniture,0.0,20.0,False,True,,Report #6285
SimpleToolCollection,Furniture,0.0,20.0,False,True,,
SimpleToolCollection,Furniture,0.0,20.0,False,False,Genuine,Report #6286
SimpleToolCollection,Furniture,0.0,20.0,False,False,Genuine,
SimpleToolCollection,Furniture,0.0,20.0,False,False,Fake,Report #6287
SimpleToolCollection,Furniture,0.0,20.0,False,False,Fake,
SimpleToolCollection,Furniture,0.0,20.0,False,False,,Report #6288
SimpleToolCollection,Furniture,0.0,20.0,False,False,,
SimpleToolCollection,Furniture,0.0,21.0,True,True,Genuine,Report #6289
SimpleToolCollection,Furniture,0.0,21.0,True,True,Genuine,
SimpleToolCollection,Furniture,0.0,21.0,True,True,Fake,Report #6290
SimpleToolCollection,Furniture,0.0,21.0,True,True,Fake,
SimpleToolCollection,Furniture,0.0,21.0,True,True,,Report #6291
SimpleToolCollection,Furniture,0.0,21.0,True,True,,
SimpleToolCollection,Furniture,0.0,21.0,True,False,Genuine,Report #6292
SimpleToolCollection,Furniture,0.0,21.0,True,False,Genuine,
SimpleToolCollection,Furniture,0.0,21.0,True,False,Fake,Report #6293
SimpleToolCollection,Furniture,0.0,21.0,True,False,Fake,
SimpleToolCollection,Furniture,0.0,21.0,True,False,,Report #6294
SimpleToolCollection,Furniture,0.0,21.0,True,False,,
SimpleToolCollection,Furniture,0.0,21.0,False,True,Genuine,Report #6295
SimpleToolCollection,Furniture,0.0,21.0,False,True,Genuine,
SimpleToolCollection,Furniture,0.0,21.0,False,True,Fake,Report #6296
SimpleToolCollection,Furniture,0.0,21.0,False,True,Fake,
SimpleToolCollection,Furniture,0.0,21.0,False,True,,Report #6297
SimpleToolCollection,Furniture,0.0,21.0,False,True,,
SimpleToolCollection,Furniture,0.0,21.0,False,False,Genuine,Report #6298
SimpleToolCollection,Furniture,0.0,21.0,False,False,Genuine,
SimpleToolCollection,Furniture,0.0,21.0,False,False,Fake,Report #6299
SimpleToolCollection,Furniture,0.0,21.0,False,False,Fake,
SimpleToolCollection,Furniture,0.0,21.0,False,False,,Report #6300
SimpleToolCollection,Furniture,0.0,21.0,False,False,,
SimpleToolCollection,Furniture,0.0,99.0,True,True,Genuine,Report #6301
SimpleToolCollection,Furniture,0.0,99.0,True,True,Genuine,
SimpleToolCollection,Furniture,0.0,99.0,True,True,Fake,Report #6302
SimpleToolCollection,Furniture,0.0,99.0,True,True,Fake,
SimpleToolCollection,Furniture,0.0,99.0,True,True,,Report #6303
SimpleToolCollection,Furniture,0.0,99.0,True,True,,
SimpleToolCollection,Furniture,0.0,99.0,True,False,Genuine,Report #6304
SimpleToolCollection,Furniture,0.0,99.0,True,False,Genuine,
SimpleToolCollection,Furniture,0.0,99.0,True,False,Fake,Report #6305
SimpleToolCollection,Furniture,0.0,99.0,True,False,Fake,
SimpleToolCollection,Furniture,0.0,99.0,True,False,,Report #6306
SimpleToolCollection,Furniture,0.0,99.0,True,False,,
SimpleToolCollection,Furniture,0.0,99.0,False,True,Genuine,Report #6307
SimpleToolCollection,Furniture,0.0,99.0,False,True,Genuine,
SimpleToolCollection,Furniture,0.0,99.0,False,True,Fake,Report #6308
SimpleToolCollection,Furniture,0.0,99.0,False,True,Fake,
SimpleToolCollection,Furniture,0.0,99.0,False,True,,Report #6309
SimpleToolCollection,Furniture,0.0,99.0,False,True,,
SimpleToolCollection,Furniture,0.0,99.0,False,False,Genuine,Report #6310
SimpleToolCollection,Furniture,0.0,99.0,False,False,Genuine,
SimpleToolCollection,Furniture,0.0,99.0,False,False,Fake,Report #6311
SimpleToolCollection,Furniture,0.0,99.0,False,False,Fake,
SimpleToolCollection,Furniture,0.0,99.0,False,False,,Report #6312
SimpleToolCollection,Furniture,0.0,99.0,False,False,,
SimpleToolCollection,Furniture,0.0,100.0,True,True,Genuine,Report #6313
SimpleToolCollection,Furniture,0.0,100.0,True,True,Genuine,
SimpleToolCollection,Furniture,0.0,100.0,True,True,Fake,Report #6314
SimpleToolCollection,Furniture,0.0,100.0,True,True,Fake,
SimpleToolCollection,Furniture,0.0,100.0,True,True,,Report #6315
SimpleToolCollection,Furniture,0.0,100.0,True,True,,
SimpleToolCollection,Furniture,0.0,100.0,True,False,Genuine,Report #6316
SimpleToolCollection,Furniture,0.0,100.0,True,False,Genuine,
SimpleToolCollection,Furniture,0.0,100.0,True,False,Fake,Report #6317
SimpleToolCollection,Furniture,0.0,100.0,True,False,Fake,
SimpleToolCollection,Furniture,0.0,100.0,True,False,,Report #6318
SimpleToolCollection,Furniture,0.0,100.0,True,False,,
SimpleToolCollection,Furniture,0.0,100.0,False,True,Genuine,Report #6319
SimpleToolCollection,Furniture,0.0,100.0,False,True,Genuine,
SimpleToolCollection,Furniture,0.0,100.0,False,True,Fake,Report #6320
SimpleToolCollection,Furniture,0.0,100.0,False,True,Fake,
SimpleToolCollection,Furniture,0.0,100.0,False,True,,Report #6321
SimpleToolCollection,Furniture,0.0,100.0,False,True,,
SimpleToolCollection,Furniture,0.0,100.0,False,False,Genuine,Report #6322
SimpleToolCollection,Furniture,0.0,100.0,False,False,Genuine,
SimpleToolCollection,Furniture,0.0,100.0,False,False,Fake,Report #6323
SimpleToolCollection,Furniture,0.0,100.0,False,False,Fake,
SimpleToolCollection,Furniture,0.0,100.0,False,False,,Report #6324
SimpleToolCollection,Furniture,0.0,100.0,False,False,,
SimpleToolCollection,Furniture,0.0,101.0,True,True,Genuine,Report #6325
SimpleToolCollection,Furniture,0.0,101.0,True,True,Genuine,
SimpleToolCollection,Furniture,0.0,101.0,True,True,Fake,Report #6326
SimpleToolCollection,Furniture,0.0,101.0,True,True,Fake,
SimpleToolCollection,Furniture,0.0,101.0,True,True,,Report #6327
SimpleToolCollection,Furniture,0.0,101.0,True,True,,
SimpleToolCollection,Furniture,0.0,101.0,True,False,Genuine,Report #6328
SimpleToolCollection,Furniture,0.0,101.0,True,False,Genuine,
SimpleToolCollection,Furniture,0.0,101.0,True,False,Fake,Report #6329
SimpleToolCollection,Furniture,0.0,101.0,True,False,Fake,
SimpleToolCollection,Furniture,0.0,101.0,True,False,,Report #6330
SimpleToolCollection,Furniture,0.0,101.0,True,False,,
SimpleToolCollection,Furniture,0.0,101.0,False,True,Genuine,Report #6331
SimpleToolCollection,Furniture,0.0,101.0,False,True,Genuine,
SimpleToolCollection,Furniture,0.0,101.0,False,True,Fake,Report #6332
SimpleToolCollection,Furniture,0.0,101.0,False,True,Fake,
SimpleToolCollection,Furniture,0.0,101.0,False,True,,Report #6333
SimpleToolCollection,Furniture,0.0,101.0,False,True,,
SimpleToolCollection,Furniture,0.0,101.0,False,False,Genuine,Report #6334
SimpleToolCollection,Furniture,0.0,101.0,False,False,Genuine,
SimpleToolCollection,Furniture,0.0,101.0,False,False,Fake,Report #6335
SimpleToolCollection,Furniture,0.0,101.0,False,False,Fake,
SimpleToolCollection,Furniture,0.0,101.0,False,False,,Report #6336
SimpleToolCollection,Furniture,0.0,101.0,False,False,,
SimpleToolCollection,Furniture,1.0,-1.0,True,True,Genuine,Report #6337
SimpleToolCollection,Furniture,1.0,-1.0,True,True,Genuine,
SimpleToolCollection,Furniture,1.0,-1.0,True,True,Fake,Report #6338
SimpleToolCollection,Furniture,1.0,-1.0,True,True,Fake,
SimpleToolCollection,Furniture,1.0,-1.0,True,True,,Report #6339
SimpleToolCollection,Furniture,1.0,-1.0,True,True,,
SimpleToolCollection,Furniture,1.0,-1.0,True,False,Genuine,Report #6340
SimpleToolCollection,Furniture,1.0,-1.0,True,False,Genuine,
SimpleToolCollection,Furniture,1.0,-1.0,True,False,Fake,Report #6341
SimpleToolCollection,Furniture,1.0,-1.0,True,False,Fake,
SimpleToolCollection,Furniture,1.0,-1.0,True,False,,Report #6342
SimpleToolCollection,Furniture,1.0,-1.0,True,False,,
SimpleToolCollection,Furniture,1.0,-1.0,False,True,Genuine,Report #6343
SimpleToolCollection,Furniture,1.0,-1.0,False,True,Genuine,
SimpleToolCollection,Furniture,1.0,-1.0,False,True,Fake,Report #6344
SimpleToolCollection,Furniture,1.0,-1.0,False,True,Fake,
SimpleToolCollection,Furniture,1.0,-1.0,False,True,,Report #6345
SimpleToolCollection,Furniture,1.0,-1.0,False,True,,
SimpleToolCollection,Furniture,1.0,-1.0,False,False,Genuine,Report #6346
SimpleToolCollection,Furniture,1.0,-1.0,False,False,Genuine,
SimpleToolCollection,Furniture,1.0,-1.0,False,False,Fake,Report #6347
SimpleToolCollection,Furniture,1.0,-1.0,False,False,Fake,
SimpleToolCollection,Furniture,1.0,-1.0,False,False,,Report #6348
SimpleToolCollection,Furniture,1.0,-1.0,False,False,,
SimpleToolCollection,Furniture,1.0,0.0,True,True,Genuine,Report #6349
SimpleToolCollection,Furniture,1.0,0.0,True,True,Genuine,
SimpleToolCollection,Furniture,1.0,0.0,True,True,Fake,Report #6350
SimpleToolCollection,Furniture,1.0,0.0,True,True,Fake,
SimpleToolCollection,Furniture,1.0,0.0,True,True,,Report #6351
SimpleToolCollection,Furniture,1.0,0.0,True,True,,
SimpleToolCollection,Furniture,1.0,0.0,True,False,Genuine,Report #6352
SimpleToolCollection,Furniture,1.0,0.0,True,False,Genuine,
SimpleToolCollection,Furniture,1.0,0.0,True,False,Fake,Report #6353
SimpleToolCollection,Furniture,1.0,0.0,True,False,Fake,
SimpleToolCollection,Furniture,1.0,0.0,True,False,,Report #6354
SimpleToolCollection,Furniture,1.0,0.0,True,False,,
SimpleToolCollection,Furniture,1.0,0.0,False,True,Genuine,Report #6355
SimpleToolCollection,Furniture,1.0,0.0,False,True,Genuine,
SimpleToolCollection,Furniture,1.0,0.0,False,True,Fake,Report #6356
SimpleToolCollection,Furniture,1.0,0.0,False,True,Fake,
SimpleToolCollection,Furniture,1.0,0.0,False,True,,Report #6357
SimpleToolCollection,Furniture,1.0,0.0,False,True,,
SimpleToolCollection,Furniture,1.0,0.0,False,False,Genuine,Report #6358
SimpleToolCollection,Furniture,1.0,0.0,False,False,Genuine,
SimpleToolCollection,Furniture,1.0,0.0,False,False,Fake,Report #6359
SimpleToolCollection,Furniture,1.0,0.0,False,False,Fake,
SimpleToolCollection,Furniture,1.0,0.0,False,False,,Report #6360
SimpleToolCollection,Furniture,1.0,0.0,False,False,,
SimpleToolCollection,Furniture,1.0,19.0,True,True,Genuine,Report #6361
SimpleToolCollection,Furniture,1.0,19.0,True,True,Genuine,
SimpleToolCollection,Furniture,1.0,19.0,True,True,Fake,Report #6362
SimpleToolCollection,Furniture,1.0,19.0,True,True,Fake,
SimpleToolCollection,Furniture,1.0,19.0,True,True,,Report #6363
SimpleToolCollection,Furniture,1.0,19.0,True,True,,
SimpleToolCollection,Furniture,1.0,19.0,True,False,Genuine,Report #6364
SimpleToolCollection,Furniture,1.0,19.0,True,False,Genuine,
SimpleToolCollection,Furniture,1.0,19.0,True,False,Fake,Report #6365
SimpleToolCollection,Furniture,1.0,19.0,True,False,Fake,
SimpleToolCollection,Furniture,1.0,19.0,True,False,,Report #6366
SimpleToolCollection,Furniture,1.0,19.0,True,False,,
SimpleToolCollection,Furniture,1.0,19.0,False,True,Genuine,Report #6367
SimpleToolCollection,Furniture,1.0,19.0,False,True,Genuine,
SimpleToolCollection,Furniture,1.0,19.0,False,True,Fake,Report #6368
SimpleToolCollection,Furniture,1.0,19.0,False,True,Fake,
SimpleToolCollection,Furniture,1.0,19.0,False,True,,Report #6369
SimpleToolCollection,Furniture,1.0,19.0,False,True,,
SimpleToolCollection,Furniture,1.0,19.0,False,False,Genuine,Report #6370
SimpleToolCollection,Furniture,1.0,19.0,False,False,Genuine,
SimpleToolCollection,Furniture,1.0,19.0,False,False,Fake,Report #6371
SimpleToolCollection,Furniture,1.0,19.0,False,False,Fake,
SimpleToolCollection,Furniture,1.0,19.0,False,False,,Report #6372
SimpleToolCollection,Furniture,1.0,19.0,False,False,,
SimpleToolCollection,Furniture,1.0,20.0,True,True,Genuine,Report #6373
SimpleToolCollection,Furniture,1.0,20.0,True,True,Genuine,
SimpleToolCollection,Furniture,1.0,20.0,True,True,Fake,Report #6374
SimpleToolCollection,Furniture,1.0,20.0,True,True,Fake,
SimpleToolCollection,Furniture,1.0,20.0,True,True,,Report #6375
SimpleToolCollection,Furniture,1.0,20.0,True,True,,
SimpleToolCollection,Furniture,1.0,20.0,True,False,Genuine,Report #6376
SimpleToolCollection,Furniture,1.0,20.0,True,False,Genuine,
SimpleToolCollection,Furniture,1.0,20.0,True,False,Fake,Report #6377
SimpleToolCollection,Furniture,1.0,20.0,True,False,Fake,
SimpleToolCollection,Furniture,1.0,20.0,True,False,,Report #6378
SimpleToolCollection,Furniture,1.0,20.0,True,False,,
SimpleToolCollection,Furniture,1.0,20.0,False,True,Genuine,Report #6379
SimpleToolCollection,Furniture,1.0,20.0,False,True,Genuine,
SimpleToolCollection,Furniture,1.0,20.0,False,True,Fake,Report #6380
SimpleToolCollection,Furniture,1.0,20.0,False,True,Fake,
SimpleToolCollection,Furniture,1.0,20.0,False,True,,Report #6381
SimpleToolCollection,Furniture,1.0,20.0,False,True,,
SimpleToolCollection,Furniture,1.0,20.0,False,False,Genuine,Report #6382
SimpleToolCollection,Furniture,1.0,20.0,False,False,Genuine,
SimpleToolCollection,Furniture,1.0,20.0,False,False,Fake,Report #6383
SimpleToolCollection,Furniture,1.0,20.0,False,False,Fake,
SimpleToolCollection,Furniture,1.0,20.0,False,False,,Report #6384
SimpleToolCollection,Furniture,1.0,20.0,False,False,,
SimpleToolCollection,Furniture,1.0,21.0,True,True,Genuine,Report #6385
SimpleToolCollection,Furniture,1.0,21.0,True,True,Genuine,
SimpleToolCollection,Furniture,1.0,21.0,True,True,Fake,Report #6386
SimpleToolCollection,Furniture,1.0,21.0,True,True,Fake,
SimpleToolCollection,Furniture,1.0,21.0,True,True,,Report #6387
SimpleToolCollection,Furniture,1.0,21.0,True,True,,
SimpleToolCollection,Furniture,1.0,21.0,True,False,Genuine,Report #6388
SimpleToolCollection,Furniture,1.0,21.0,True,False,Genuine,
SimpleToolCollection,Furniture,1.0,21.0,True,False,Fake,Report #6389
SimpleToolCollection,Furniture,1.0,21.0,True,False,Fake,
SimpleToolCollection,Furniture,1.0,21.0,True,False,,Report #6390
SimpleToolCollection,Furniture,1.0,21.0,True,False,,
SimpleToolCollection,Furniture,1.0,21.0,False,True,Genuine,Report #6391
SimpleToolCollection,Furniture,1.0,21.0,False,True,Genuine,
SimpleToolCollection,Furniture,1.0,21.0,False,True,Fake,Report #6392
SimpleToolCollection,Furniture,1.0,21.0,False,True,Fake,
SimpleToolCollection,Furniture,1.0,21.0,False,True,,Report #6393
SimpleToolCollection,Furniture,1.0,21.0,False,True,,
SimpleToolCollection,Furniture,1.0,21.0,False,False,Genuine,Report #6394
SimpleToolCollection,Furniture,1.0,21.0,False,False,Genuine,
SimpleToolCollection,Furniture,1.0,21.0,False,False,Fake,Report #6395
SimpleToolCollection,Furniture,1.0,21.0,False,False,Fake,
SimpleToolCollection,Furniture,1.0,21.0,False,False,,Report #6396
SimpleToolCollection,Furniture,1.0,21.0,False,False,,
SimpleToolCollection,Furniture,1.0,99.0,True,True,Genuine,Report #6397
SimpleToolCollection,Furniture,1.0,99.0,True,True,Genuine,
SimpleToolCollection,Furniture,1.0,99.0,True,True,Fake,Report #6398
SimpleToolCollection,Furniture,1.0,99.0,True,True,Fake,
SimpleToolCollection,Furniture,1.0,99.0,True,True,,Report #6399
SimpleToolCollection,Furniture,1.0,99.0,True,True,,
SimpleToolCollection,Furniture,1.0,99.0,True,False,Genuine,Report #6400
SimpleToolCollection,Furniture,1.0,99.0,True,False,Genuine,
SimpleToolCollection,Furniture,1.0,99.0,True,False,Fake,Report #6401
SimpleToolCollection,Furniture,1.0,99.0,True,False,Fake,
SimpleToolCollection,Furniture,1.0,99.0,True,False,,Report #6402
SimpleToolCollection,Furniture,1.0,99.0,True,False,,
SimpleToolCollection,Furniture,1.0,99.0,False,True,Genuine,Report #6403
SimpleToolCollection,Furniture,1.0,99.0,False,True,Genuine,
SimpleToolCollection,Furniture,1.0,99.0,False,True,Fake,Report #6404
SimpleToolCollection,Furniture,1.0,99.0,False,True,Fake,
SimpleToolCollection,Furniture,1.0,99.0,False,True,,Report #6405
SimpleToolCollection,Furniture,1.0,99.0,False,True,,
SimpleToolCollection,Furniture,1.0,99.0,False,False,Genuine,Report #6406
SimpleToolCollection,Furniture,1.0,99.0,False,False,Genuine,
SimpleToolCollection,Furniture,1.0,99.0,False,False,Fake,Report #6407
SimpleToolCollection,Furniture,1.0,99.0,False,False,Fake,
SimpleToolCollection,Furniture,1.0,99.0,False,False,,Report #6408
SimpleToolCollection,Furniture,1.0,99.0,False,False,,
SimpleToolCollection,Furniture,1.0,100.0,True,True,Genuine,Report #6409
SimpleToolCollection,Furniture,1.0,100.0,True,True,Genuine,
SimpleToolCollection,Furniture,1.0,100.0,True,True,Fake,Report #6410
SimpleToolCollection,Furniture,1.0,100.0,True,True,Fake,
SimpleToolCollection,Furniture,1.0,100.0,True,True,,Report #6411
SimpleToolCollection,Furniture,1.0,100.0,True,True,,
SimpleToolCollection,Furniture,1.0,100.0,True,False,Genuine,Report #6412
SimpleToolCollection,Furniture,1.0,100.0,True,False,Genuine,
SimpleToolCollection,Furniture,1.0,100.0,True,False,Fake,Report #6413
SimpleToolCollection,Furniture,1.0,100.0,True,False,Fake,
SimpleToolCollection,Furniture,1.0,100.0,True,False,,Report #6414
SimpleToolCollection,Furniture,1.0,100.0,True,False,,
SimpleToolCollection,Furniture,1.0,100.0,False,True,Genuine,Report #6415
SimpleToolCollection,Furniture,1.0,100.0,False,True,Genuine,
SimpleToolCollection,Furniture,1.0,100.0,False,True,Fake,Report #6416
SimpleToolCollection,Furniture,1.0,100.0,False,True,Fake,
SimpleToolCollection,Furniture,1.0,100.0,False,True,,Report #6417
SimpleToolCollection,Furniture,1.0,100.0,False,True,,
SimpleToolCollection,Furniture,1.0,100.0,False,False,Genuine,Report #6418
SimpleToolCollection,Furniture,1.0,100.0,False,False,Genuine,
SimpleToolCollection,Furniture,1.0,100.0,False,False,Fake,Report #6419
SimpleToolCollection,Furniture,1.0,100.0,False,False,Fake,
SimpleToolCollection,Furniture,1.0,100.0,False,False,,Report #6420
SimpleToolCollection,Furniture,1.0,100.0,False,False,,
SimpleToolCollection,Furniture,1.0,101.0,True,True,Genuine,Report #6421
SimpleToolCollection,Furniture,1.0,101.0,True,True,Genuine,
SimpleToolCollection,Furniture,1.0,101.0,True,True,Fake,Report #6422
SimpleToolCollection,Furniture,1.0,101.0,True,True,Fake,
SimpleToolCollection,Furniture,1.0,101.0,True,True,,Report #6423
SimpleToolCollection,Furniture,1.0,101.0,True,True,,
SimpleToolCollection,Furniture,1.0,101.0,True,False,Genuine,Report #6424
SimpleToolCollection,Furniture,1.0,101.0,True,False,Genuine,
SimpleToolCollection,Furniture,1.0,101.0,True,False,Fake,Report #6425
SimpleToolCollection,Furniture,1.0,101.0,True,False,Fake,
SimpleToolCollection,Furniture,1.0,101.0,True,False,,Report #6426
SimpleToolCollection,Furniture,1.0,101.0,True,False,,
SimpleToolCollection,Furniture,1.0,101.0,False,True,Genuine,Report #6427
SimpleToolCollection,Furniture,1.0,101.0,False,True,Genuine,
SimpleToolCollection,Furniture,1.0,101.0,False,True,Fake,Report #6428
SimpleToolCollection,Furniture,1.0,101.0,False,True,Fake,
SimpleToolCollection,Furniture,1.0,101.0,False,True,,Report #6429
SimpleToolCollection,Furniture,1.0,101.0,False,True,,
SimpleToolCollection,Furniture,1.0,101.0,False,False,Genuine,Report #6430
SimpleToolCollection,Furniture,1.0,101.0,False,False,Genuine,
SimpleToolCollection,Furniture,1.0,101.0,False,False,Fake,Report #6431
SimpleToolCollection,Furniture,1.0,101.0,False,False,Fake,
SimpleToolCollection,Furniture,1.0,101.0,False,False,,Report #6432
SimpleToolCollection,Furniture,1.0,101.0,False,False,,
SimpleToolCollection,Furniture,100.0,-1.0,True,True,Genuine,Report #6433
SimpleToolCollection,Furniture,100.0,-1.0,True,True,Genuine,
SimpleToolCollection,Furniture,100.0,-1.0,True,True,Fake,Report #6434
SimpleToolCollection,Furniture,100.0,-1.0,True,True,Fake,
SimpleToolCollection,Furniture,100.0,-1.0,True,True,,Report #6435
SimpleToolCollection,Furniture,100.0,-1.0,True,True,,
SimpleToolCollection,Furniture,100.0,-1.0,True,False,Genuine,Report #6436
SimpleToolCollection,Furniture,100.0,-1.0,True,False,Genuine,
SimpleToolCollection,Furniture,100.0,-1.0,True,False,Fake,Report #6437
SimpleToolCollection,Furniture,100.0,-1.0,True,False,Fake,
SimpleToolCollection,Furniture,100.0,-1.0,True,False,,Report #6438
SimpleToolCollection,Furniture,100.0,-1.0,True,False,,
SimpleToolCollection,Furniture,100.0,-1.0,False,True,Genuine,Report #6439
SimpleToolCollection,Furniture,100.0,-1.0,False,True,Genuine,
SimpleToolCollection,Furniture,100.0,-1.0,False,True,Fake,Report #6440
SimpleToolCollection,Furniture,100.0,-1.0,False,True,Fake,
SimpleToolCollection,Furniture,100.0,-1.0,False,True,,Report #6441
SimpleToolCollection,Furniture,100.0,-1.0,False,True,,
SimpleToolCollection,Furniture,100.0,-1.0,False,False,Genuine,Report #6442
SimpleToolCollection,Furniture,100.0,-1.0,False,False,Genuine,
SimpleToolCollection,Furniture,100.0,-1.0,False,False,Fake,Report #6443
SimpleToolCollection,Furniture,100.0,-1.0,False,False,Fake,
SimpleToolCollection,Furniture,100.0,-1.0,False,False,,Report #6444
SimpleToolCollection,Furniture,100.0,-1.0,False,False,,
SimpleToolCollection,Furniture,100.0,0.0,True,True,Genuine,Report #6445
SimpleToolCollection,Furniture,100.0,0.0,True,True,Genuine,
SimpleToolCollection,Furniture,100.0,0.0,True,True,Fake,Report #6446
SimpleToolCollection,Furniture,100.0,0.0,True,True,Fake,
SimpleToolCollection,Furniture,100.0,0.0,True,True,,Report #6447
SimpleToolCollection,Furniture,100.0,0.0,True,True,,
SimpleToolCollection,Furniture,100.0,0.0,True,False,Genuine,Report #6448
SimpleToolCollection,Furniture,100.0,0.0,True,False,Genuine,
SimpleToolCollection,Furniture,100.0,0.0,True,False,Fake,Report #6449
SimpleToolCollection,Furniture,100.0,0.0,True,False,Fake,
SimpleToolCollection,Furniture,100.0,0.0,True,False,,Report #6450
SimpleToolCollection,Furniture,100.0,0.0,True,False,,
SimpleToolCollection,Furniture,100.0,0.0,False,True,Genuine,Report #6451
SimpleToolCollection,Furniture,100.0,0.0,False,True,Genuine,
SimpleToolCollection,Furniture,100.0,0.0,False,True,Fake,Report #6452
SimpleToolCollection,Furniture,100.0,0.0,False,True,Fake,
SimpleToolCollection,Furniture,100.0,0.0,False,True,,Report #6453
SimpleToolCollection,Furniture,100.0,0.0,False,True,,
SimpleToolCollection,Furniture,100.0,0.0,False,False,Genuine,Report #6454
SimpleToolCollection,Furniture,100.0,0.0,False,False,Genuine,
SimpleToolCollection,Furniture,100.0,0.0,False,False,Fake,Report #6455
SimpleToolCollection,Furniture,100.0,0.0,False,False,Fake,
SimpleToolCollection,Furniture,100.0,0.0,False,False,,Report #6456
SimpleToolCollection,Furniture,100.0,0.0,False,False,,
SimpleToolCollection,Furniture,100.0,19.0,True,True,Genuine,Report #6457
SimpleToolCollection,Furniture,100.0,19.0,True,True,Genuine,
SimpleToolCollection,Furniture,100.0,19.0,True,True,Fake,Report #6458
SimpleToolCollection,Furniture,100.0,19.0,True,True,Fake,
SimpleToolCollection,Furniture,100.0,19.0,True,True,,Report #6459
SimpleToolCollection,Furniture,100.0,19.0,True,True,,
SimpleToolCollection,Furniture,100.0,19.0,True,False,Genuine,Report #6460
SimpleToolCollection,Furniture,100.0,19.0,True,False,Genuine,
SimpleToolCollection,Furniture,100.0,19.0,True,False,Fake,Report #6461
SimpleToolCollection,Furniture,100.0,19.0,True,False,Fake,
SimpleToolCollection,Furniture,100.0,19.0,True,False,,Report #6462
SimpleToolCollection,Furniture,100.0,19.0,True,False,,
SimpleToolCollection,Furniture,100.0,19.0,False,True,Genuine,Report #6463
SimpleToolCollection,Furniture,100.0,19.0,False,True,Genuine,
SimpleToolCollection,Furniture,100.0,19.0,False,True,Fake,Report #6464
SimpleToolCollection,Furniture,100.0,19.0,False,True,Fake,
SimpleToolCollection,Furniture,100.0,19.0,False,True,,Report #6465
SimpleToolCollection,Furniture,100.0,19.0,False,True,,
SimpleToolCollection,Furniture,100.0,19.0,False,False,Genuine,Report #6466
SimpleToolCollection,Furniture,100.0,19.0,False,False,Genuine,
SimpleToolCollection,Furniture,100.0,19.0,False,False,Fake,Report #6467
SimpleToolCollection,Furniture,100.0,19.0,False,False,Fake,
SimpleToolCollection,Furniture,100.0,19.0,False,False,,Report #6468
SimpleToolCollection,Furniture,100.0,19.0,False,False,,
SimpleToolCollection,Furniture,100.0,20.0,True,True,Genuine,Report #6469
SimpleToolCollection,Furniture,100.0,20.0,True,True,Genuine,
SimpleToolCollection,Furniture,100.0,20.0,True,True,Fake,Report #6470
SimpleToolCollection,Furniture,100.0,20.0,True,True,Fake,
SimpleToolCollection,Furniture,100.0,20.0,True,True,,Report #6471
SimpleToolCollection,Furniture,100.0,20.0,True,True,,
SimpleToolCollection,Furniture,100.0,20.0,True,False,Genuine,Report #6472
SimpleToolCollection,Furniture,100.0,20.0,True,False,Genuine,
SimpleToolCollection,Furniture,100.0,20.0,True,False,Fake,Report #6473
SimpleToolCollection,Furniture,100.0,20.0,True,False,Fake,
SimpleToolCollection,Furniture,100.0,20.0,True,False,,Report #6474
SimpleToolCollection,Furniture,100.0,20.0,True,False,,
SimpleToolCollection,Furniture,100.0,20.0,False,True,Genuine,Report #6475
SimpleToolCollection,Furniture,100.0,20.0,False,True,Genuine,
SimpleToolCollection,Furniture,100.0,20.0,False,True,Fake,Report #6476
SimpleToolCollection,Furniture,100.0,20.0,False,True,Fake,
SimpleToolCollection,Furniture,100.0,20.0,False,True,,Report #6477
SimpleToolCollection,Furniture,100.0,20.0,False,True,,
SimpleToolCollection,Furniture,100.0,20.0,False,False,Genuine,Report #6478
SimpleToolCollection,Furniture,100.0,20.0,False,False,Genuine,
SimpleToolCollection,Furniture,100.0,20.0,False,False,Fake,Report #6479
SimpleToolCollection,Furniture,100.0,20.0,False,False,Fake,
SimpleToolCollection,Furniture,100.0,20.0,False,False,,Report #6480
SimpleToolCollection,Furniture,100.0,20.0,False,False,,
SimpleToolCollection,Furniture,100.0,21.0,True,True,Genuine,Report #6481
SimpleToolCollection,Furniture,100.0,21.0,True,True,Genuine,
SimpleToolCollection,Furniture,100.0,21.0,True,True,Fake,Report #6482
SimpleToolCollection,Furniture,100.0,21.0,True,True,Fake,
SimpleToolCollection,Furniture,100.0,21.0,True,True,,Report #6483
SimpleToolCollection,Furniture,100.0,21.0,True,True,,
SimpleToolCollection,Furniture,100.0,21.0,True,False,Genuine,Report #6484
SimpleToolCollection,Furniture,100.0,21.0,True,False,Genuine,
SimpleToolCollection,Furniture,100.0,21.0,True,False,Fake,Report #6485
SimpleToolCollection,Furniture,100.0,21.0,True,False,Fake,
SimpleToolCollection,Furniture,100.0,21.0,True,False,,Report #6486
SimpleToolCollection,Furniture,100.0,21.0,True,False,,
SimpleToolCollection,Furniture,100.0,21.0,False,True,Genuine,Report #6487
SimpleToolCollection,Furniture,100.0,21.0,False,True,Genuine,
SimpleToolCollection,Furniture,100.0,21.0,False,True,Fake,Report #6488
SimpleToolCollection,Furniture,100.0,21.0,False,True,Fake,
SimpleToolCollection,Furniture,100.0,21.0,False,True,,Report #6489
SimpleToolCollection,Furniture,100.0,21.0,False,True,,
SimpleToolCollection,Furniture,100.0,21.0,False,False,Genuine,Report #6490
SimpleToolCollection,Furniture,100.0,21.0,False,False,Genuine,
SimpleToolCollection,Furniture,100.0,21.0,False,False,Fake,Report #6491
SimpleToolCollection,Furniture,100.0,21.0,False,False,Fake,
SimpleToolCollection,Furniture,100.0,21.0,False,False,,Report #6492
SimpleToolCollection,Furniture,100.0,21.0,False,False,,
SimpleToolCollection,Furniture,100.0,99.0,True,True,Genuine,Report #6493
SimpleToolCollection,Furniture,100.0,99.0,True,True,Genuine,
SimpleToolCollection,Furniture,100.0,99.0,True,True,Fake,Report #6494
SimpleToolCollection,Furniture,100.0,99.0,True,True,Fake,
SimpleToolCollection,Furniture,100.0,99.0,True,True,,Report #6495
SimpleToolCollection,Furniture,100.0,99.0,True,True,,
SimpleToolCollection,Furniture,100.0,99.0,True,False,Genuine,Report #6496
SimpleToolCollection,Furniture,100.0,99.0,True,False,Genuine,
SimpleToolCollection,Furniture,100.0,99.0,True,False,Fake,Report #6497
SimpleToolCollection,Furniture,100.0,99.0,True,False,Fake,
SimpleToolCollection,Furniture,100.0,99.0,True,False,,Report #6498
SimpleToolCollection,Furniture,100.0,99.0,True,False,,
SimpleToolCollection,Furniture,100.0,99.0,False,True,Genuine,Report #6499
SimpleToolCollection,Furniture,100.0,99.0,False,True,Genuine,
SimpleToolCollection,Furniture,100.0,99.0,False,True,Fake,Report #6500
SimpleToolCollection,Furniture,100.0,99.0,False,True,Fake,
SimpleToolCollection,Furniture,100.0,99.0,False,True,,Report #6501
SimpleToolCollection,Furniture,100.0,99.0,False,True,,
SimpleToolCollection,Furniture,100.0,99.0,False,False,Genuine,Report #6502
SimpleToolCollection,Furniture,100.0,99.0,False,False,Genuine,
SimpleToolCollection,Furniture,100.0,99.0,False,False,Fake,Report #6503
SimpleToolCollection,Furniture,100.0,99.0,False,False,Fake,
SimpleToolCollection,Furniture,100.0,99.0,False,False,,Report #6504
SimpleToolCollection,Furniture,100.0,99.0,False,False,,
SimpleToolCollection,Furniture,100.0,100.0,True,True,Genuine,Report #6505
SimpleToolCollection,Furniture,100.0,100.0,True,True,Genuine,
SimpleToolCollection,Furniture,100.0,100.0,True,True,Fake,Report #6506
SimpleToolCollection,Furniture,100.0,100.0,True,True,Fake,
SimpleToolCollection,Furniture,100.0,100.0,True,True,,Report #6507
SimpleToolCollection,Furniture,100.0,100.0,True,True,,
SimpleToolCollection,Furniture,100.0,100.0,True,False,Genuine,Report #6508
SimpleToolCollection,Furniture,100.0,100.0,True,False,Genuine,
SimpleToolCollection,Furniture,100.0,100.0,True,False,Fake,Report #6509
SimpleToolCollection,Furniture,100.0,100.0,True,False,Fake,
SimpleToolCollection,Furniture,100.0,100.0,True,False,,Report #6510
SimpleToolCollection,Furniture,100.0,100.0,True,False,,
SimpleToolCollection,Furniture,100.0,100.0,False,True,Genuine,Report #6511
SimpleToolCollection,Furniture,100.0,100.0,False,True,Genuine,
SimpleToolCollection,Furniture,100.0,100.0,False,True,Fake,Report #6512
SimpleToolCollection,Furniture,100.0,100.0,False,True,Fake,
SimpleToolCollection,Furniture,100.0,100.0,False,True,,Report #6513
SimpleToolCollection,Furniture,100.0,100.0,False,True,,
SimpleToolCollection,Furniture,100.0,100.0,False,False,Genuine,Report #6514
SimpleToolCollection,Furniture,100.0,100.0,False,False,Genuine,
SimpleToolCollection,Furniture,100.0,100.0,False,False,Fake,Report #6515
SimpleToolCollection,Furniture,100.0,100.0,False,False,Fake,
SimpleToolCollection,Furniture,100.0,100.0,False,False,,Report #6516
SimpleToolCollection,Furniture,100.0,100.0,False,False,,
SimpleToolCollection,Furniture,100.0,101.0,True,True,Genuine,Report #6517
SimpleToolCollection,Furniture,100.0,101.0,True,True,Genuine,
SimpleToolCollection,Furniture,100.0,101.0,True,True,Fake,Report #6518
SimpleToolCollection,Furniture,100.0,101.0,True,True,Fake,
SimpleToolCollection,Furniture,100.0,101.0,True,True,,Report #6519
SimpleToolCollection,Furniture,100.0,101.0,True,True,,
SimpleToolCollection,Furniture,100.0,101.0,True,False,Genuine,Report #6520
SimpleToolCollection,Furniture,100.0,101.0,True,False,Genuine,
SimpleToolCollection,Furniture,100.0,101.0,True,False,Fake,Report #6521
SimpleToolCollection,Furniture,100.0,101.0,True,False,Fake,
SimpleToolCollection,Furniture,100.0,101.0,True,False,,Report #6522
SimpleToolCollection,Furniture,100.0,101.0,True,False,,
SimpleToolCollection,Furniture,100.0,101.0,False,True,Genuine,Report #6523
SimpleToolCollection,Furniture,100.0,101.0,False,True,Genuine,
SimpleToolCollection,Furniture,100.0,101.0,False,True,Fake,Report #6524
SimpleToolCollection,Furniture,100.0,101.0,False,True,Fake,
SimpleToolCollection,Furniture,100.0,101.0,False,True,,Report #6525
SimpleToolCollection,Furniture,100.0,101.0,False,True,,
SimpleToolCollection,Furniture,100.0,101.0,False,False,Genuine,Report #6526
SimpleToolCollection,Furniture,100.0,101.0,False,False,Genuine,
SimpleToolCollection,Furniture,100.0,101.0,False,False,Fake,Report #6527
SimpleToolCollection,Furniture,100.0,101.0,False,False,Fake,
SimpleToolCollection,Furniture,100.0,101.0,False,False,,Report #6528
SimpleToolCollection,Furniture,100.0,101.0,False,False,,
SimpleToolCollection,Furniture,1000.0,-1.0,True,True,Genuine,Report #6529
SimpleToolCollection,Furniture,1000.0,-1.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000.0,-1.0,True,True,Fake,Report #6530
SimpleToolCollection,Furniture,1000.0,-1.0,True,True,Fake,
SimpleToolCollection,Furniture,1000.0,-1.0,True,True,,Report #6531
SimpleToolCollection,Furniture,1000.0,-1.0,True,True,,
SimpleToolCollection,Furniture,1000.0,-1.0,True,False,Genuine,Report #6532
SimpleToolCollection,Furniture,1000.0,-1.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000.0,-1.0,True,False,Fake,Report #6533
SimpleToolCollection,Furniture,1000.0,-1.0,True,False,Fake,
SimpleToolCollection,Furniture,1000.0,-1.0,True,False,,Report #6534
SimpleToolCollection,Furniture,1000.0,-1.0,True,False,,
SimpleToolCollection,Furniture,1000.0,-1.0,False,True,Genuine,Report #6535
SimpleToolCollection,Furniture,1000.0,-1.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000.0,-1.0,False,True,Fake,Report #6536
SimpleToolCollection,Furniture,1000.0,-1.0,False,True,Fake,
SimpleToolCollection,Furniture,1000.0,-1.0,False,True,,Report #6537
SimpleToolCollection,Furniture,1000.0,-1.0,False,True,,
SimpleToolCollection,Furniture,1000.0,-1.0,False,False,Genuine,Report #6538
SimpleToolCollection,Furniture,1000.0,-1.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000.0,-1.0,False,False,Fake,Report #6539
SimpleToolCollection,Furniture,1000.0,-1.0,False,False,Fake,
SimpleToolCollection,Furniture,1000.0,-1.0,False,False,,Report #6540
SimpleToolCollection,Furniture,1000.0,-1.0,False,False,,
SimpleToolCollection,Furniture,1000.0,0.0,True,True,Genuine,Report #6541
SimpleToolCollection,Furniture,1000.0,0.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000.0,0.0,True,True,Fake,Report #6542
SimpleToolCollection,Furniture,1000.0,0.0,True,True,Fake,
SimpleToolCollection,Furniture,1000.0,0.0,True,True,,Report #6543
SimpleToolCollection,Furniture,1000.0,0.0,True,True,,
SimpleToolCollection,Furniture,1000.0,0.0,True,False,Genuine,Report #6544
SimpleToolCollection,Furniture,1000.0,0.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000.0,0.0,True,False,Fake,Report #6545
SimpleToolCollection,Furniture,1000.0,0.0,True,False,Fake,
SimpleToolCollection,Furniture,1000.0,0.0,True,False,,Report #6546
SimpleToolCollection,Furniture,1000.0,0.0,True,False,,
SimpleToolCollection,Furniture,1000.0,0.0,False,True,Genuine,Report #6547
SimpleToolCollection,Furniture,1000.0,0.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000.0,0.0,False,True,Fake,Report #6548
SimpleToolCollection,Furniture,1000.0,0.0,False,True,Fake,
SimpleToolCollection,Furniture,1000.0,0.0,False,True,,Report #6549
SimpleToolCollection,Furniture,1000.0,0.0,False,True,,
SimpleToolCollection,Furniture,1000.0,0.0,False,False,Genuine,Report #6550
SimpleToolCollection,Furniture,1000.0,0.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000.0,0.0,False,False,Fake,Report #6551
SimpleToolCollection,Furniture,1000.0,0.0,False,False,Fake,
SimpleToolCollection,Furniture,1000.0,0.0,False,False,,Report #6552
SimpleToolCollection,Furniture,1000.0,0.0,False,False,,
SimpleToolCollection,Furniture,1000.0,19.0,True,True,Genuine,Report #6553
SimpleToolCollection,Furniture,1000.0,19.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000.0,19.0,True,True,Fake,Report #6554
SimpleToolCollection,Furniture,1000.0,19.0,True,True,Fake,
SimpleToolCollection,Furniture,1000.0,19.0,True,True,,Report #6555
SimpleToolCollection,Furniture,1000.0,19.0,True,True,,
SimpleToolCollection,Furniture,1000.0,19.0,True,False,Genuine,Report #6556
SimpleToolCollection,Furniture,1000.0,19.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000.0,19.0,True,False,Fake,Report #6557
SimpleToolCollection,Furniture,1000.0,19.0,True,False,Fake,
SimpleToolCollection,Furniture,1000.0,19.0,True,False,,Report #6558
SimpleToolCollection,Furniture,1000.0,19.0,True,False,,
SimpleToolCollection,Furniture,1000.0,19.0,False,True,Genuine,Report #6559
SimpleToolCollection,Furniture,1000.0,19.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000.0,19.0,False,True,Fake,Report #6560
SimpleToolCollection,Furniture,1000.0,19.0,False,True,Fake,
SimpleToolCollection,Furniture,1000.0,19.0,False,True,,Report #6561
SimpleToolCollection,Furniture,1000.0,19.0,False,True,,
SimpleToolCollection,Furniture,1000.0,19.0,False,False,Genuine,Report #6562
SimpleToolCollection,Furniture,1000.0,19.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000.0,19.0,False,False,Fake,Report #6563
SimpleToolCollection,Furniture,1000.0,19.0,False,False,Fake,
SimpleToolCollection,Furniture,1000.0,19.0,False,False,,Report #6564
SimpleToolCollection,Furniture,1000.0,19.0,False,False,,
SimpleToolCollection,Furniture,1000.0,20.0,True,True,Genuine,Report #6565
SimpleToolCollection,Furniture,1000.0,20.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000.0,20.0,True,True,Fake,Report #6566
SimpleToolCollection,Furniture,1000.0,20.0,True,True,Fake,
SimpleToolCollection,Furniture,1000.0,20.0,True,True,,Report #6567
SimpleToolCollection,Furniture,1000.0,20.0,True,True,,
SimpleToolCollection,Furniture,1000.0,20.0,True,False,Genuine,Report #6568
SimpleToolCollection,Furniture,1000.0,20.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000.0,20.0,True,False,Fake,Report #6569
SimpleToolCollection,Furniture,1000.0,20.0,True,False,Fake,
SimpleToolCollection,Furniture,1000.0,20.0,True,False,,Report #6570
SimpleToolCollection,Furniture,1000.0,20.0,True,False,,
SimpleToolCollection,Furniture,1000.0,20.0,False,True,Genuine,Report #6571
SimpleToolCollection,Furniture,1000.0,20.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000.0,20.0,False,True,Fake,Report #6572
SimpleToolCollection,Furniture,1000.0,20.0,False,True,Fake,
SimpleToolCollection,Furniture,1000.0,20.0,False,True,,Report #6573
SimpleToolCollection,Furniture,1000.0,20.0,False,True,,
SimpleToolCollection,Furniture,1000.0,20.0,False,False,Genuine,Report #6574
SimpleToolCollection,Furniture,1000.0,20.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000.0,20.0,False,False,Fake,Report #6575
SimpleToolCollection,Furniture,1000.0,20.0,False,False,Fake,
SimpleToolCollection,Furniture,1000.0,20.0,False,False,,Report #6576
SimpleToolCollection,Furniture,1000.0,20.0,False,False,,
SimpleToolCollection,Furniture,1000.0,21.0,True,True,Genuine,Report #6577
SimpleToolCollection,Furniture,1000.0,21.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000.0,21.0,True,True,Fake,Report #6578
SimpleToolCollection,Furniture,1000.0,21.0,True,True,Fake,
SimpleToolCollection,Furniture,1000.0,21.0,True,True,,Report #6579
SimpleToolCollection,Furniture,1000.0,21.0,True,True,,
SimpleToolCollection,Furniture,1000.0,21.0,True,False,Genuine,Report #6580
SimpleToolCollection,Furniture,1000.0,21.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000.0,21.0,True,False,Fake,Report #6581
SimpleToolCollection,Furniture,1000.0,21.0,True,False,Fake,
SimpleToolCollection,Furniture,1000.0,21.0,True,False,,Report #6582
SimpleToolCollection,Furniture,1000.0,21.0,True,False,,
SimpleToolCollection,Furniture,1000.0,21.0,False,True,Genuine,Report #6583
SimpleToolCollection,Furniture,1000.0,21.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000.0,21.0,False,True,Fake,Report #6584
SimpleToolCollection,Furniture,1000.0,21.0,False,True,Fake,
SimpleToolCollection,Furniture,1000.0,21.0,False,True,,Report #6585
SimpleToolCollection,Furniture,1000.0,21.0,False,True,,
SimpleToolCollection,Furniture,1000.0,21.0,False,False,Genuine,Report #6586
SimpleToolCollection,Furniture,1000.0,21.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000.0,21.0,False,False,Fake,Report #6587
SimpleToolCollection,Furniture,1000.0,21.0,False,False,Fake,
SimpleToolCollection,Furniture,1000.0,21.0,False,False,,Report #6588
SimpleToolCollection,Furniture,1000.0,21.0,False,False,,
SimpleToolCollection,Furniture,1000.0,99.0,True,True,Genuine,Report #6589
SimpleToolCollection,Furniture,1000.0,99.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000.0,99.0,True,True,Fake,Report #6590
SimpleToolCollection,Furniture,1000.0,99.0,True,True,Fake,
SimpleToolCollection,Furniture,1000.0,99.0,True,True,,Report #6591
SimpleToolCollection,Furniture,1000.0,99.0,True,True,,
SimpleToolCollection,Furniture,1000.0,99.0,True,False,Genuine,Report #6592
SimpleToolCollection,Furniture,1000.0,99.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000.0,99.0,True,False,Fake,Report #6593
SimpleToolCollection,Furniture,1000.0,99.0,True,False,Fake,
SimpleToolCollection,Furniture,1000.0,99.0,True,False,,Report #6594
SimpleToolCollection,Furniture,1000.0,99.0,True,False,,
SimpleToolCollection,Furniture,1000.0,99.0,False,True,Genuine,Report #6595
SimpleToolCollection,Furniture,1000.0,99.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000.0,99.0,False,True,Fake,Report #6596
SimpleToolCollection,Furniture,1000.0,99.0,False,True,Fake,
SimpleToolCollection,Furniture,1000.0,99.0,False,True,,Report #6597
SimpleToolCollection,Furniture,1000.0,99.0,False,True,,
SimpleToolCollection,Furniture,1000.0,99.0,False,False,Genuine,Report #6598
SimpleToolCollection,Furniture,1000.0,99.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000.0,99.0,False,False,Fake,Report #6599
SimpleToolCollection,Furniture,1000.0,99.0,False,False,Fake,
SimpleToolCollection,Furniture,1000.0,99.0,False,False,,Report #6600
SimpleToolCollection,Furniture,1000.0,99.0,False,False,,
SimpleToolCollection,Furniture,1000.0,100.0,True,True,Genuine,Report #6601
SimpleToolCollection,Furniture,1000.0,100.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000.0,100.0,True,True,Fake,Report #6602
SimpleToolCollection,Furniture,1000.0,100.0,True,True,Fake,
SimpleToolCollection,Furniture,1000.0,100.0,True,True,,Report #6603
SimpleToolCollection,Furniture,1000.0,100.0,True,True,,
SimpleToolCollection,Furniture,1000.0,100.0,True,False,Genuine,Report #6604
SimpleToolCollection,Furniture,1000.0,100.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000.0,100.0,True,False,Fake,Report #6605
SimpleToolCollection,Furniture,1000.0,100.0,True,False,Fake,
SimpleToolCollection,Furniture,1000.0,100.0,True,False,,Report #6606
SimpleToolCollection,Furniture,1000.0,100.0,True,False,,
SimpleToolCollection,Furniture,1000.0,100.0,False,True,Genuine,Report #6607
SimpleToolCollection,Furniture,1000.0,100.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000.0,100.0,False,True,Fake,Report #6608
SimpleToolCollection,Furniture,1000.0,100.0,False,True,Fake,
SimpleToolCollection,Furniture,1000.0,100.0,False,True,,Report #6609
SimpleToolCollection,Furniture,1000.0,100.0,False,True,,
SimpleToolCollection,Furniture,1000.0,100.0,False,False,Genuine,Report #6610
SimpleToolCollection,Furniture,1000.0,100.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000.0,100.0,False,False,Fake,Report #6611
SimpleToolCollection,Furniture,1000.0,100.0,False,False,Fake,
SimpleToolCollection,Furniture,1000.0,100.0,False,False,,Report #6612
SimpleToolCollection,Furniture,1000.0,100.0,False,False,,
SimpleToolCollection,Furniture,1000.0,101.0,True,True,Genuine,Report #6613
SimpleToolCollection,Furniture,1000.0,101.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000.0,101.0,True,True,Fake,Report #6614
SimpleToolCollection,Furniture,1000.0,101.0,True,True,Fake,
SimpleToolCollection,Furniture,1000.0,101.0,True,True,,Report #6615
SimpleToolCollection,Furniture,1000.0,101.0,True,True,,
SimpleToolCollection,Furniture,1000.0,101.0,True,False,Genuine,Report #6616
SimpleToolCollection,Furniture,1000.0,101.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000.0,101.0,True,False,Fake,Report #6617
SimpleToolCollection,Furniture,1000.0,101.0,True,False,Fake,
SimpleToolCollection,Furniture,1000.0,101.0,True,False,,Report #6618
SimpleToolCollection,Furniture,1000.0,101.0,True,False,,
SimpleToolCollection,Furniture,1000.0,101.0,False,True,Genuine,Report #6619
SimpleToolCollection,Furniture,1000.0,101.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000.0,101.0,False,True,Fake,Report #6620
SimpleToolCollection,Furniture,1000.0,101.0,False,True,Fake,
SimpleToolCollection,Furniture,1000.0,101.0,False,True,,Report #6621
SimpleToolCollection,Furniture,1000.0,101.0,False,True,,
SimpleToolCollection,Furniture,1000.0,101.0,False,False,Genuine,Report #6622
SimpleToolCollection,Furniture,1000.0,101.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000.0,101.0,False,False,Fake,Report #6623
SimpleToolCollection,Furniture,1000.0,101.0,False,False,Fake,
SimpleToolCollection,Furniture,1000.0,101.0,False,False,,Report #6624
SimpleToolCollection,Furniture,1000.0,101.0,False,False,,
SimpleToolCollection,Furniture,1000000.0,-1.0,True,True,Genuine,Report #6625
SimpleToolCollection,Furniture,1000000.0,-1.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,-1.0,True,True,Fake,Report #6626
SimpleToolCollection,Furniture,1000000.0,-1.0,True,True,Fake,
SimpleToolCollection,Furniture,1000000.0,-1.0,True,True,,Report #6627
SimpleToolCollection,Furniture,1000000.0,-1.0,True,True,,
SimpleToolCollection,Furniture,1000000.0,-1.0,True,False,Genuine,Report #6628
SimpleToolCollection,Furniture,1000000.0,-1.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,-1.0,True,False,Fake,Report #6629
SimpleToolCollection,Furniture,1000000.0,-1.0,True,False,Fake,
SimpleToolCollection,Furniture,1000000.0,-1.0,True,False,,Report #6630
SimpleToolCollection,Furniture,1000000.0,-1.0,True,False,,
SimpleToolCollection,Furniture,1000000.0,-1.0,False,True,Genuine,Report #6631
SimpleToolCollection,Furniture,1000000.0,-1.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,-1.0,False,True,Fake,Report #6632
SimpleToolCollection,Furniture,1000000.0,-1.0,False,True,Fake,
SimpleToolCollection,Furniture,1000000.0,-1.0,False,True,,Report #6633
SimpleToolCollection,Furniture,1000000.0,-1.0,False,True,,
SimpleToolCollection,Furniture,1000000.0,-1.0,False,False,Genuine,Report #6634
SimpleToolCollection,Furniture,1000000.0,-1.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,-1.0,False,False,Fake,Report #6635
SimpleToolCollection,Furniture,1000000.0,-1.0,False,False,Fake,
SimpleToolCollection,Furniture,1000000.0,-1.0,False,False,,Report #6636
SimpleToolCollection,Furniture,1000000.0,-1.0,False,False,,
SimpleToolCollection,Furniture,1000000.0,0.0,True,True,Genuine,Report #6637
SimpleToolCollection,Furniture,1000000.0,0.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,0.0,True,True,Fake,Report #6638
SimpleToolCollection,Furniture,1000000.0,0.0,True,True,Fake,
SimpleToolCollection,Furniture,1000000.0,0.0,True,True,,Report #6639
SimpleToolCollection,Furniture,1000000.0,0.0,True,True,,
SimpleToolCollection,Furniture,1000000.0,0.0,True,False,Genuine,Report #6640
SimpleToolCollection,Furniture,1000000.0,0.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,0.0,True,False,Fake,Report #6641
SimpleToolCollection,Furniture,1000000.0,0.0,True,False,Fake,
SimpleToolCollection,Furniture,1000000.0,0.0,True,False,,Report #6642
SimpleToolCollection,Furniture,1000000.0,0.0,True,False,,
SimpleToolCollection,Furniture,1000000.0,0.0,False,True,Genuine,Report #6643
SimpleToolCollection,Furniture,1000000.0,0.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,0.0,False,True,Fake,Report #6644
SimpleToolCollection,Furniture,1000000.0,0.0,False,True,Fake,
SimpleToolCollection,Furniture,1000000.0,0.0,False,True,,Report #6645
SimpleToolCollection,Furniture,1000000.0,0.0,False,True,,
SimpleToolCollection,Furniture,1000000.0,0.0,False,False,Genuine,Report #6646
SimpleToolCollection,Furniture,1000000.0,0.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,0.0,False,False,Fake,Report #6647
SimpleToolCollection,Furniture,1000000.0,0.0,False,False,Fake,
SimpleToolCollection,Furniture,1000000.0,0.0,False,False,,Report #6648
SimpleToolCollection,Furniture,1000000.0,0.0,False,False,,
SimpleToolCollection,Furniture,1000000.0,19.0,True,True,Genuine,Report #6649
SimpleToolCollection,Furniture,1000000.0,19.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,19.0,True,True,Fake,Report #6650
SimpleToolCollection,Furniture,1000000.0,19.0,True,True,Fake,
SimpleToolCollection,Furniture,1000000.0,19.0,True,True,,Report #6651
SimpleToolCollection,Furniture,1000000.0,19.0,True,True,,
SimpleToolCollection,Furniture,1000000.0,19.0,True,False,Genuine,Report #6652
SimpleToolCollection,Furniture,1000000.0,19.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,19.0,True,False,Fake,Report #6653
SimpleToolCollection,Furniture,1000000.0,19.0,True,False,Fake,
SimpleToolCollection,Furniture,1000000.0,19.0,True,False,,Report #6654
SimpleToolCollection,Furniture,1000000.0,19.0,True,False,,
SimpleToolCollection,Furniture,1000000.0,19.0,False,True,Genuine,Report #6655
SimpleToolCollection,Furniture,1000000.0,19.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,19.0,False,True,Fake,Report #6656
SimpleToolCollection,Furniture,1000000.0,19.0,False,True,Fake,
SimpleToolCollection,Furniture,1000000.0,19.0,False,True,,Report #6657
SimpleToolCollection,Furniture,1000000.0,19.0,False,True,,
SimpleToolCollection,Furniture,1000000.0,19.0,False,False,Genuine,Report #6658
SimpleToolCollection,Furniture,1000000.0,19.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,19.0,False,False,Fake,Report #6659
SimpleToolCollection,Furniture,1000000.0,19.0,False,False,Fake,
SimpleToolCollection,Furniture,1000000.0,19.0,False,False,,Report #6660
SimpleToolCollection,Furniture,1000000.0,19.0,False,False,,
SimpleToolCollection,Furniture,1000000.0,20.0,True,True,Genuine,Report #6661
SimpleToolCollection,Furniture,1000000.0,20.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,20.0,True,True,Fake,Report #6662
SimpleToolCollection,Furniture,1000000.0,20.0,True,True,Fake,
SimpleToolCollection,Furniture,1000000.0,20.0,True,True,,Report #6663
SimpleToolCollection,Furniture,1000000.0,20.0,True,True,,
SimpleToolCollection,Furniture,1000000.0,20.0,True,False,Genuine,Report #6664
SimpleToolCollection,Furniture,1000000.0,20.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,20.0,True,False,Fake,Report #6665
SimpleToolCollection,Furniture,1000000.0,20.0,True,False,Fake,
SimpleToolCollection,Furniture,1000000.0,20.0,True,False,,Report #6666
SimpleToolCollection,Furniture,1000000.0,20.0,True,False,,
SimpleToolCollection,Furniture,1000000.0,20.0,False,True,Genuine,Report #6667
SimpleToolCollection,Furniture,1000000.0,20.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,20.0,False,True,Fake,Report #6668
SimpleToolCollection,Furniture,1000000.0,20.0,False,True,Fake,
SimpleToolCollection,Furniture,1000000.0,20.0,False,True,,Report #6669
SimpleToolCollection,Furniture,1000000.0,20.0,False,True,,
SimpleToolCollection,Furniture,1000000.0,20.0,False,False,Genuine,Report #6670
SimpleToolCollection,Furniture,1000000.0,20.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,20.0,False,False,Fake,Report #6671
SimpleToolCollection,Furniture,1000000.0,20.0,False,False,Fake,
SimpleToolCollection,Furniture,1000000.0,20.0,False,False,,Report #6672
SimpleToolCollection,Furniture,1000000.0,20.0,False,False,,
SimpleToolCollection,Furniture,1000000.0,21.0,True,True,Genuine,Report #6673
SimpleToolCollection,Furniture,1000000.0,21.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,21.0,True,True,Fake,Report #6674
SimpleToolCollection,Furniture,1000000.0,21.0,True,True,Fake,
SimpleToolCollection,Furniture,1000000.0,21.0,True,True,,Report #6675
SimpleToolCollection,Furniture,1000000.0,21.0,True,True,,
SimpleToolCollection,Furniture,1000000.0,21.0,True,False,Genuine,Report #6676
SimpleToolCollection,Furniture,1000000.0,21.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,21.0,True,False,Fake,Report #6677
SimpleToolCollection,Furniture,1000000.0,21.0,True,False,Fake,
SimpleToolCollection,Furniture,1000000.0,21.0,True,False,,Report #6678
SimpleToolCollection,Furniture,1000000.0,21.0,True,False,,
SimpleToolCollection,Furniture,1000000.0,21.0,False,True,Genuine,Report #6679
SimpleToolCollection,Furniture,1000000.0,21.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,21.0,False,True,Fake,Report #6680
SimpleToolCollection,Furniture,1000000.0,21.0,False,True,Fake,
SimpleToolCollection,Furniture,1000000.0,21.0,False,True,,Report #6681
SimpleToolCollection,Furniture,1000000.0,21.0,False,True,,
SimpleToolCollection,Furniture,1000000.0,21.0,False,False,Genuine,Report #6682
SimpleToolCollection,Furniture,1000000.0,21.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,21.0,False,False,Fake,Report #6683
SimpleToolCollection,Furniture,1000000.0,21.0,False,False,Fake,
SimpleToolCollection,Furniture,1000000.0,21.0,False,False,,Report #6684
SimpleToolCollection,Furniture,1000000.0,21.0,False,False,,
SimpleToolCollection,Furniture,1000000.0,99.0,True,True,Genuine,Report #6685
SimpleToolCollection,Furniture,1000000.0,99.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,99.0,True,True,Fake,Report #6686
SimpleToolCollection,Furniture,1000000.0,99.0,True,True,Fake,
SimpleToolCollection,Furniture,1000000.0,99.0,True,True,,Report #6687
SimpleToolCollection,Furniture,1000000.0,99.0,True,True,,
SimpleToolCollection,Furniture,1000000.0,99.0,True,False,Genuine,Report #6688
SimpleToolCollection,Furniture,1000000.0,99.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,99.0,True,False,Fake,Report #6689
SimpleToolCollection,Furniture,1000000.0,99.0,True,False,Fake,
SimpleToolCollection,Furniture,1000000.0,99.0,True,False,,Report #6690
SimpleToolCollection,Furniture,1000000.0,99.0,True,False,,
SimpleToolCollection,Furniture,1000000.0,99.0,False,True,Genuine,Report #6691
SimpleToolCollection,Furniture,1000000.0,99.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,99.0,False,True,Fake,Report #6692
SimpleToolCollection,Furniture,1000000.0,99.0,False,True,Fake,
SimpleToolCollection,Furniture,1000000.0,99.0,False,True,,Report #6693
SimpleToolCollection,Furniture,1000000.0,99.0,False,True,,
SimpleToolCollection,Furniture,1000000.0,99.0,False,False,Genuine,Report #6694
SimpleToolCollection,Furniture,1000000.0,99.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,99.0,False,False,Fake,Report #6695
SimpleToolCollection,Furniture,1000000.0,99.0,False,False,Fake,
SimpleToolCollection,Furniture,1000000.0,99.0,False,False,,Report #6696
SimpleToolCollection,Furniture,1000000.0,99.0,False,False,,
SimpleToolCollection,Furniture,1000000.0,100.0,True,True,Genuine,Report #6697
SimpleToolCollection,Furniture,1000000.0,100.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,100.0,True,True,Fake,Report #6698
SimpleToolCollection,Furniture,1000000.0,100.0,True,True,Fake,
SimpleToolCollection,Furniture,1000000.0,100.0,True,True,,Report #6699
SimpleToolCollection,Furniture,1000000.0,100.0,True,True,,
SimpleToolCollection,Furniture,1000000.0,100.0,True,False,Genuine,Report #6700
SimpleToolCollection,Furniture,1000000.0,100.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,100.0,True,False,Fake,Report #6701
SimpleToolCollection,Furniture,1000000.0,100.0,True,False,Fake,
SimpleToolCollection,Furniture,1000000.0,100.0,True,False,,Report #6702
SimpleToolCollection,Furniture,1000000.0,100.0,True,False,,
SimpleToolCollection,Furniture,1000000.0,100.0,False,True,Genuine,Report #6703
SimpleToolCollection,Furniture,1000000.0,100.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,100.0,False,True,Fake,Report #6704
SimpleToolCollection,Furniture,1000000.0,100.0,False,True,Fake,
SimpleToolCollection,Furniture,1000000.0,100.0,False,True,,Report #6705
SimpleToolCollection,Furniture,1000000.0,100.0,False,True,,
SimpleToolCollection,Furniture,1000000.0,100.0,False,False,Genuine,Report #6706
SimpleToolCollection,Furniture,1000000.0,100.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,100.0,False,False,Fake,Report #6707
SimpleToolCollection,Furniture,1000000.0,100.0,False,False,Fake,
SimpleToolCollection,Furniture,1000000.0,100.0,False,False,,Report #6708
SimpleToolCollection,Furniture,1000000.0,100.0,False,False,,
SimpleToolCollection,Furniture,1000000.0,101.0,True,True,Genuine,Report #6709
SimpleToolCollection,Furniture,1000000.0,101.0,True,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,101.0,True,True,Fake,Report #6710
SimpleToolCollection,Furniture,1000000.0,101.0,True,True,Fake,
SimpleToolCollection,Furniture,1000000.0,101.0,True,True,,Report #6711
SimpleToolCollection,Furniture,1000000.0,101.0,True,True,,
SimpleToolCollection,Furniture,1000000.0,101.0,True,False,Genuine,Report #6712
SimpleToolCollection,Furniture,1000000.0,101.0,True,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,101.0,True,False,Fake,Report #6713
SimpleToolCollection,Furniture,1000000.0,101.0,True,False,Fake,
SimpleToolCollection,Furniture,1000000.0,101.0,True,False,,Report #6714
SimpleToolCollection,Furniture,1000000.0,101.0,True,False,,
SimpleToolCollection,Furniture,1000000.0,101.0,False,True,Genuine,Report #6715
SimpleToolCollection,Furniture,1000000.0,101.0,False,True,Genuine,
SimpleToolCollection,Furniture,1000000.0,101.0,False,True,Fake,Report #6716
SimpleToolCollection,Furniture,1000000.0,101.0,False,True,Fake,
SimpleToolCollection,Furniture,1000000.0,101.0,False,True,,Report #6717
SimpleToolCollection,Furniture,1000000.0,101.0,False,True,,
SimpleToolCollection,Furniture,1000000.0,101.0,False,False,Genuine,Report #6718
SimpleToolCollection,Furniture,1000000.0,101.0,False,False,Genuine,
SimpleToolCollection,Furniture,1000000.0,101.0,False,False,Fake,Report #6719
SimpleToolCollection,Furniture,1000000.0,101.0,False,False,Fake,
SimpleToolCollection,Furniture,1000000.0,101.0,False,False,,Report #6720
SimpleToolCollection,Furniture,1000000.0,101.0,False,False,,
SimpleToolCollection,HistoryWritings,0.0,-1.0,True,True,Genuine,Report #6721
SimpleToolCollection,HistoryWritings,0.0,-1.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,-1.0,True,True,Fake,Report #6722
SimpleToolCollection,HistoryWritings,0.0,-1.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,-1.0,True,True,,Report #6723
SimpleToolCollection,HistoryWritings,0.0,-1.0,True,True,,
SimpleToolCollection,HistoryWritings,0.0,-1.0,True,False,Genuine,Report #6724
SimpleToolCollection,HistoryWritings,0.0,-1.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,-1.0,True,False,Fake,Report #6725
SimpleToolCollection,HistoryWritings,0.0,-1.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,-1.0,True,False,,Report #6726
SimpleToolCollection,HistoryWritings,0.0,-1.0,True,False,,
SimpleToolCollection,HistoryWritings,0.0,-1.0,False,True,Genuine,Report #6727
SimpleToolCollection,HistoryWritings,0.0,-1.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,-1.0,False,True,Fake,Report #6728
SimpleToolCollection,HistoryWritings,0.0,-1.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,-1.0,False,True,,Report #6729
SimpleToolCollection,HistoryWritings,0.0,-1.0,False,True,,
SimpleToolCollection,HistoryWritings,0.0,-1.0,False,False,Genuine,Report #6730
SimpleToolCollection,HistoryWritings,0.0,-1.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,-1.0,False,False,Fake,Report #6731
SimpleToolCollection,HistoryWritings,0.0,-1.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,-1.0,False,False,,Report #6732
SimpleToolCollection,HistoryWritings,0.0,-1.0,False,False,,
SimpleToolCollection,HistoryWritings,0.0,0.0,True,True,Genuine,Report #6733
SimpleToolCollection,HistoryWritings,0.0,0.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,0.0,True,True,Fake,Report #6734
SimpleToolCollection,HistoryWritings,0.0,0.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,0.0,True,True,,Report #6735
SimpleToolCollection,HistoryWritings,0.0,0.0,True,True,,
SimpleToolCollection,HistoryWritings,0.0,0.0,True,False,Genuine,Report #6736
SimpleToolCollection,HistoryWritings,0.0,0.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,0.0,True,False,Fake,Report #6737
SimpleToolCollection,HistoryWritings,0.0,0.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,0.0,True,False,,Report #6738
SimpleToolCollection,HistoryWritings,0.0,0.0,True,False,,
SimpleToolCollection,HistoryWritings,0.0,0.0,False,True,Genuine,Report #6739
SimpleToolCollection,HistoryWritings,0.0,0.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,0.0,False,True,Fake,Report #6740
SimpleToolCollection,HistoryWritings,0.0,0.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,0.0,False,True,,Report #6741
SimpleToolCollection,HistoryWritings,0.0,0.0,False,True,,
SimpleToolCollection,HistoryWritings,0.0,0.0,False,False,Genuine,Report #6742
SimpleToolCollection,HistoryWritings,0.0,0.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,0.0,False,False,Fake,Report #6743
SimpleToolCollection,HistoryWritings,0.0,0.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,0.0,False,False,,Report #6744
SimpleToolCollection,HistoryWritings,0.0,0.0,False,False,,
SimpleToolCollection,HistoryWritings,0.0,19.0,True,True,Genuine,Report #6745
SimpleToolCollection,HistoryWritings,0.0,19.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,19.0,True,True,Fake,Report #6746
SimpleToolCollection,HistoryWritings,0.0,19.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,19.0,True,True,,Report #6747
SimpleToolCollection,HistoryWritings,0.0,19.0,True,True,,
SimpleToolCollection,HistoryWritings,0.0,19.0,True,False,Genuine,Report #6748
SimpleToolCollection,HistoryWritings,0.0,19.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,19.0,True,False,Fake,Report #6749
SimpleToolCollection,HistoryWritings,0.0,19.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,19.0,True,False,,Report #6750
SimpleToolCollection,HistoryWritings,0.0,19.0,True,False,,
SimpleToolCollection,HistoryWritings,0.0,19.0,False,True,Genuine,Report #6751
SimpleToolCollection,HistoryWritings,0.0,19.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,19.0,False,True,Fake,Report #6752
SimpleToolCollection,HistoryWritings,0.0,19.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,19.0,False,True,,Report #6753
SimpleToolCollection,HistoryWritings,0.0,19.0,False,True,,
SimpleToolCollection,HistoryWritings,0.0,19.0,False,False,Genuine,Report #6754
SimpleToolCollection,HistoryWritings,0.0,19.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,19.0,False,False,Fake,Report #6755
SimpleToolCollection,HistoryWritings,0.0,19.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,19.0,False,False,,Report #6756
SimpleToolCollection,HistoryWritings,0.0,19.0,False,False,,
SimpleToolCollection,HistoryWritings,0.0,20.0,True,True,Genuine,Report #6757
SimpleToolCollection,HistoryWritings,0.0,20.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,20.0,True,True,Fake,Report #6758
SimpleToolCollection,HistoryWritings,0.0,20.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,20.0,True,True,,Report #6759
SimpleToolCollection,HistoryWritings,0.0,20.0,True,True,,
SimpleToolCollection,HistoryWritings,0.0,20.0,True,False,Genuine,Report #6760
SimpleToolCollection,HistoryWritings,0.0,20.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,20.0,True,False,Fake,Report #6761
SimpleToolCollection,HistoryWritings,0.0,20.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,20.0,True,False,,Report #6762
SimpleToolCollection,HistoryWritings,0.0,20.0,True,False,,
SimpleToolCollection,HistoryWritings,0.0,20.0,False,True,Genuine,Report #6763
SimpleToolCollection,HistoryWritings,0.0,20.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,20.0,False,True,Fake,Report #6764
SimpleToolCollection,HistoryWritings,0.0,20.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,20.0,False,True,,Report #6765
SimpleToolCollection,HistoryWritings,0.0,20.0,False,True,,
SimpleToolCollection,HistoryWritings,0.0,20.0,False,False,Genuine,Report #6766
SimpleToolCollection,HistoryWritings,0.0,20.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,20.0,False,False,Fake,Report #6767
SimpleToolCollection,HistoryWritings,0.0,20.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,20.0,False,False,,Report #6768
SimpleToolCollection,HistoryWritings,0.0,20.0,False,False,,
SimpleToolCollection,HistoryWritings,0.0,21.0,True,True,Genuine,Report #6769
SimpleToolCollection,HistoryWritings,0.0,21.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,21.0,True,True,Fake,Report #6770
SimpleToolCollection,HistoryWritings,0.0,21.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,21.0,True,True,,Report #6771
SimpleToolCollection,HistoryWritings,0.0,21.0,True,True,,
SimpleToolCollection,HistoryWritings,0.0,21.0,True,False,Genuine,Report #6772
SimpleToolCollection,HistoryWritings,0.0,21.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,21.0,True,False,Fake,Report #6773
SimpleToolCollection,HistoryWritings,0.0,21.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,21.0,True,False,,Report #6774
SimpleToolCollection,HistoryWritings,0.0,21.0,True,False,,
SimpleToolCollection,HistoryWritings,0.0,21.0,False,True,Genuine,Report #6775
SimpleToolCollection,HistoryWritings,0.0,21.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,21.0,False,True,Fake,Report #6776
SimpleToolCollection,HistoryWritings,0.0,21.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,21.0,False,True,,Report #6777
SimpleToolCollection,HistoryWritings,0.0,21.0,False,True,,
SimpleToolCollection,HistoryWritings,0.0,21.0,False,False,Genuine,Report #6778
SimpleToolCollection,HistoryWritings,0.0,21.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,21.0,False,False,Fake,Report #6779
SimpleToolCollection,HistoryWritings,0.0,21.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,21.0,False,False,,Report #6780
SimpleToolCollection,HistoryWritings,0.0,21.0,False,False,,
SimpleToolCollection,HistoryWritings,0.0,99.0,True,True,Genuine,Report #6781
SimpleToolCollection,HistoryWritings,0.0,99.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,99.0,True,True,Fake,Report #6782
SimpleToolCollection,HistoryWritings,0.0,99.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,99.0,True,True,,Report #6783
SimpleToolCollection,HistoryWritings,0.0,99.0,True,True,,
SimpleToolCollection,HistoryWritings,0.0,99.0,True,False,Genuine,Report #6784
SimpleToolCollection,HistoryWritings,0.0,99.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,99.0,True,False,Fake,Report #6785
SimpleToolCollection,HistoryWritings,0.0,99.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,99.0,True,False,,Report #6786
SimpleToolCollection,HistoryWritings,0.0,99.0,True,False,,
SimpleToolCollection,HistoryWritings,0.0,99.0,False,True,Genuine,Report #6787
SimpleToolCollection,HistoryWritings,0.0,99.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,99.0,False,True,Fake,Report #6788
SimpleToolCollection,HistoryWritings,0.0,99.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,99.0,False,True,,Report #6789
SimpleToolCollection,HistoryWritings,0.0,99.0,False,True,,
SimpleToolCollection,HistoryWritings,0.0,99.0,False,False,Genuine,Report #6790
SimpleToolCollection,HistoryWritings,0.0,99.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,99.0,False,False,Fake,Report #6791
SimpleToolCollection,HistoryWritings,0.0,99.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,99.0,False,False,,Report #6792
SimpleToolCollection,HistoryWritings,0.0,99.0,False,False,,
SimpleToolCollection,HistoryWritings,0.0,100.0,True,True,Genuine,Report #6793
SimpleToolCollection,HistoryWritings,0.0,100.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,100.0,True,True,Fake,Report #6794
SimpleToolCollection,HistoryWritings,0.0,100.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,100.0,True,True,,Report #6795
SimpleToolCollection,HistoryWritings,0.0,100.0,True,True,,
SimpleToolCollection,HistoryWritings,0.0,100.0,True,False,Genuine,Report #6796
SimpleToolCollection,HistoryWritings,0.0,100.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,100.0,True,False,Fake,Report #6797
SimpleToolCollection,HistoryWritings,0.0,100.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,100.0,True,False,,Report #6798
SimpleToolCollection,HistoryWritings,0.0,100.0,True,False,,
SimpleToolCollection,HistoryWritings,0.0,100.0,False,True,Genuine,Report #6799
SimpleToolCollection,HistoryWritings,0.0,100.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,100.0,False,True,Fake,Report #6800
SimpleToolCollection,HistoryWritings,0.0,100.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,100.0,False,True,,Report #6801
SimpleToolCollection,HistoryWritings,0.0,100.0,False,True,,
SimpleToolCollection,HistoryWritings,0.0,100.0,False,False,Genuine,Report #6802
SimpleToolCollection,HistoryWritings,0.0,100.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,100.0,False,False,Fake,Report #6803
SimpleToolCollection,HistoryWritings,0.0,100.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,100.0,False,False,,Report #6804
SimpleToolCollection,HistoryWritings,0.0,100.0,False,False,,
SimpleToolCollection,HistoryWritings,0.0,101.0,True,True,Genuine,Report #6805
SimpleToolCollection,HistoryWritings,0.0,101.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,101.0,True,True,Fake,Report #6806
SimpleToolCollection,HistoryWritings,0.0,101.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,101.0,True,True,,Report #6807
SimpleToolCollection,HistoryWritings,0.0,101.0,True,True,,
SimpleToolCollection,HistoryWritings,0.0,101.0,True,False,Genuine,Report #6808
SimpleToolCollection,HistoryWritings,0.0,101.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,101.0,True,False,Fake,Report #6809
SimpleToolCollection,HistoryWritings,0.0,101.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,101.0,True,False,,Report #6810
SimpleToolCollection,HistoryWritings,0.0,101.0,True,False,,
SimpleToolCollection,HistoryWritings,0.0,101.0,False,True,Genuine,Report #6811
SimpleToolCollection,HistoryWritings,0.0,101.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,0.0,101.0,False,True,Fake,Report #6812
SimpleToolCollection,HistoryWritings,0.0,101.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,0.0,101.0,False,True,,Report #6813
SimpleToolCollection,HistoryWritings,0.0,101.0,False,True,,
SimpleToolCollection,HistoryWritings,0.0,101.0,False,False,Genuine,Report #6814
SimpleToolCollection,HistoryWritings,0.0,101.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,0.0,101.0,False,False,Fake,Report #6815
SimpleToolCollection,HistoryWritings,0.0,101.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,0.0,101.0,False,False,,Report #6816
SimpleToolCollection,HistoryWritings,0.0,101.0,False,False,,
SimpleToolCollection,HistoryWritings,1.0,-1.0,True,True,Genuine,Report #6817
SimpleToolCollection,HistoryWritings,1.0,-1.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,-1.0,True,True,Fake,Report #6818
SimpleToolCollection,HistoryWritings,1.0,-1.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,-1.0,True,True,,Report #6819
SimpleToolCollection,HistoryWritings,1.0,-1.0,True,True,,
SimpleToolCollection,HistoryWritings,1.0,-1.0,True,False,Genuine,Report #6820
SimpleToolCollection,HistoryWritings,1.0,-1.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,-1.0,True,False,Fake,Report #6821
SimpleToolCollection,HistoryWritings,1.0,-1.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,-1.0,True,False,,Report #6822
SimpleToolCollection,HistoryWritings,1.0,-1.0,True,False,,
SimpleToolCollection,HistoryWritings,1.0,-1.0,False,True,Genuine,Report #6823
SimpleToolCollection,HistoryWritings,1.0,-1.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,-1.0,False,True,Fake,Report #6824
SimpleToolCollection,HistoryWritings,1.0,-1.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,-1.0,False,True,,Report #6825
SimpleToolCollection,HistoryWritings,1.0,-1.0,False,True,,
SimpleToolCollection,HistoryWritings,1.0,-1.0,False,False,Genuine,Report #6826
SimpleToolCollection,HistoryWritings,1.0,-1.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,-1.0,False,False,Fake,Report #6827
SimpleToolCollection,HistoryWritings,1.0,-1.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,-1.0,False,False,,Report #6828
SimpleToolCollection,HistoryWritings,1.0,-1.0,False,False,,
SimpleToolCollection,HistoryWritings,1.0,0.0,True,True,Genuine,Report #6829
SimpleToolCollection,HistoryWritings,1.0,0.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,0.0,True,True,Fake,Report #6830
SimpleToolCollection,HistoryWritings,1.0,0.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,0.0,True,True,,Report #6831
SimpleToolCollection,HistoryWritings,1.0,0.0,True,True,,
SimpleToolCollection,HistoryWritings,1.0,0.0,True,False,Genuine,Report #6832
SimpleToolCollection,HistoryWritings,1.0,0.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,0.0,True,False,Fake,Report #6833
SimpleToolCollection,HistoryWritings,1.0,0.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,0.0,True,False,,Report #6834
SimpleToolCollection,HistoryWritings,1.0,0.0,True,False,,
SimpleToolCollection,HistoryWritings,1.0,0.0,False,True,Genuine,Report #6835
SimpleToolCollection,HistoryWritings,1.0,0.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,0.0,False,True,Fake,Report #6836
SimpleToolCollection,HistoryWritings,1.0,0.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,0.0,False,True,,Report #6837
SimpleToolCollection,HistoryWritings,1.0,0.0,False,True,,
SimpleToolCollection,HistoryWritings,1.0,0.0,False,False,Genuine,Report #6838
SimpleToolCollection,HistoryWritings,1.0,0.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,0.0,False,False,Fake,Report #6839
SimpleToolCollection,HistoryWritings,1.0,0.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,0.0,False,False,,Report #6840
SimpleToolCollection,HistoryWritings,1.0,0.0,False,False,,
SimpleToolCollection,HistoryWritings,1.0,19.0,True,True,Genuine,Report #6841
SimpleToolCollection,HistoryWritings,1.0,19.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,19.0,True,True,Fake,Report #6842
SimpleToolCollection,HistoryWritings,1.0,19.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,19.0,True,True,,Report #6843
SimpleToolCollection,HistoryWritings,1.0,19.0,True,True,,
SimpleToolCollection,HistoryWritings,1.0,19.0,True,False,Genuine,Report #6844
SimpleToolCollection,HistoryWritings,1.0,19.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,19.0,True,False,Fake,Report #6845
SimpleToolCollection,HistoryWritings,1.0,19.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,19.0,True,False,,Report #6846
SimpleToolCollection,HistoryWritings,1.0,19.0,True,False,,
SimpleToolCollection,HistoryWritings,1.0,19.0,False,True,Genuine,Report #6847
SimpleToolCollection,HistoryWritings,1.0,19.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,19.0,False,True,Fake,Report #6848
SimpleToolCollection,HistoryWritings,1.0,19.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,19.0,False,True,,Report #6849
SimpleToolCollection,HistoryWritings,1.0,19.0,False,True,,
SimpleToolCollection,HistoryWritings,1.0,19.0,False,False,Genuine,Report #6850
SimpleToolCollection,HistoryWritings,1.0,19.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,19.0,False,False,Fake,Report #6851
SimpleToolCollection,HistoryWritings,1.0,19.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,19.0,False,False,,Report #6852
SimpleToolCollection,HistoryWritings,1.0,19.0,False,False,,
SimpleToolCollection,HistoryWritings,1.0,20.0,True,True,Genuine,Report #6853
SimpleToolCollection,HistoryWritings,1.0,20.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,20.0,True,True,Fake,Report #6854
SimpleToolCollection,HistoryWritings,1.0,20.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,20.0,True,True,,Report #6855
SimpleToolCollection,HistoryWritings,1.0,20.0,True,True,,
SimpleToolCollection,HistoryWritings,1.0,20.0,True,False,Genuine,Report #6856
SimpleToolCollection,HistoryWritings,1.0,20.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,20.0,True,False,Fake,Report #6857
SimpleToolCollection,HistoryWritings,1.0,20.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,20.0,True,False,,Report #6858
SimpleToolCollection,HistoryWritings,1.0,20.0,True,False,,
SimpleToolCollection,HistoryWritings,1.0,20.0,False,True,Genuine,Report #6859
SimpleToolCollection,HistoryWritings,1.0,20.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,20.0,False,True,Fake,Report #6860
SimpleToolCollection,HistoryWritings,1.0,20.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,20.0,False,True,,Report #6861
SimpleToolCollection,HistoryWritings,1.0,20.0,False,True,,
SimpleToolCollection,HistoryWritings,1.0,20.0,False,False,Genuine,Report #6862
SimpleToolCollection,HistoryWritings,1.0,20.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,20.0,False,False,Fake,Report #6863
SimpleToolCollection,HistoryWritings,1.0,20.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,20.0,False,False,,Report #6864
SimpleToolCollection,HistoryWritings,1.0,20.0,False,False,,
SimpleToolCollection,HistoryWritings,1.0,21.0,True,True,Genuine,Report #6865
SimpleToolCollection,HistoryWritings,1.0,21.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,21.0,True,True,Fake,Report #6866
SimpleToolCollection,HistoryWritings,1.0,21.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,21.0,True,True,,Report #6867
SimpleToolCollection,HistoryWritings,1.0,21.0,True,True,,
SimpleToolCollection,HistoryWritings,1.0,21.0,True,False,Genuine,Report #6868
SimpleToolCollection,HistoryWritings,1.0,21.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,21.0,True,False,Fake,Report #6869
SimpleToolCollection,HistoryWritings,1.0,21.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,21.0,True,False,,Report #6870
SimpleToolCollection,HistoryWritings,1.0,21.0,True,False,,
SimpleToolCollection,HistoryWritings,1.0,21.0,False,True,Genuine,Report #6871
SimpleToolCollection,HistoryWritings,1.0,21.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,21.0,False,True,Fake,Report #6872
SimpleToolCollection,HistoryWritings,1.0,21.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,21.0,False,True,,Report #6873
SimpleToolCollection,HistoryWritings,1.0,21.0,False,True,,
SimpleToolCollection,HistoryWritings,1.0,21.0,False,False,Genuine,Report #6874
SimpleToolCollection,HistoryWritings,1.0,21.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,21.0,False,False,Fake,Report #6875
SimpleToolCollection,HistoryWritings,1.0,21.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,21.0,False,False,,Report #6876
SimpleToolCollection,HistoryWritings,1.0,21.0,False,False,,
SimpleToolCollection,HistoryWritings,1.0,99.0,True,True,Genuine,Report #6877
SimpleToolCollection,HistoryWritings,1.0,99.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,99.0,True,True,Fake,Report #6878
SimpleToolCollection,HistoryWritings,1.0,99.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,99.0,True,True,,Report #6879
SimpleToolCollection,HistoryWritings,1.0,99.0,True,True,,
SimpleToolCollection,HistoryWritings,1.0,99.0,True,False,Genuine,Report #6880
SimpleToolCollection,HistoryWritings,1.0,99.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,99.0,True,False,Fake,Report #6881
SimpleToolCollection,HistoryWritings,1.0,99.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,99.0,True,False,,Report #6882
SimpleToolCollection,HistoryWritings,1.0,99.0,True,False,,
SimpleToolCollection,HistoryWritings,1.0,99.0,False,True,Genuine,Report #6883
SimpleToolCollection,HistoryWritings,1.0,99.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,99.0,False,True,Fake,Report #6884
SimpleToolCollection,HistoryWritings,1.0,99.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,99.0,False,True,,Report #6885
SimpleToolCollection,HistoryWritings,1.0,99.0,False,True,,
SimpleToolCollection,HistoryWritings,1.0,99.0,False,False,Genuine,Report #6886
SimpleToolCollection,HistoryWritings,1.0,99.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,99.0,False,False,Fake,Report #6887
SimpleToolCollection,HistoryWritings,1.0,99.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,99.0,False,False,,Report #6888
SimpleToolCollection,HistoryWritings,1.0,99.0,False,False,,
SimpleToolCollection,HistoryWritings,1.0,100.0,True,True,Genuine,Report #6889
SimpleToolCollection,HistoryWritings,1.0,100.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,100.0,True,True,Fake,Report #6890
SimpleToolCollection,HistoryWritings,1.0,100.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,100.0,True,True,,Report #6891
SimpleToolCollection,HistoryWritings,1.0,100.0,True,True,,
SimpleToolCollection,HistoryWritings,1.0,100.0,True,False,Genuine,Report #6892
SimpleToolCollection,HistoryWritings,1.0,100.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,100.0,True,False,Fake,Report #6893
SimpleToolCollection,HistoryWritings,1.0,100.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,100.0,True,False,,Report #6894
SimpleToolCollection,HistoryWritings,1.0,100.0,True,False,,
SimpleToolCollection,HistoryWritings,1.0,100.0,False,True,Genuine,Report #6895
SimpleToolCollection,HistoryWritings,1.0,100.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,100.0,False,True,Fake,Report #6896
SimpleToolCollection,HistoryWritings,1.0,100.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,100.0,False,True,,Report #6897
SimpleToolCollection,HistoryWritings,1.0,100.0,False,True,,
SimpleToolCollection,HistoryWritings,1.0,100.0,False,False,Genuine,Report #6898
SimpleToolCollection,HistoryWritings,1.0,100.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,100.0,False,False,Fake,Report #6899
SimpleToolCollection,HistoryWritings,1.0,100.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,100.0,False,False,,Report #6900
SimpleToolCollection,HistoryWritings,1.0,100.0,False,False,,
SimpleToolCollection,HistoryWritings,1.0,101.0,True,True,Genuine,Report #6901
SimpleToolCollection,HistoryWritings,1.0,101.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,101.0,True,True,Fake,Report #6902
SimpleToolCollection,HistoryWritings,1.0,101.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,101.0,True,True,,Report #6903
SimpleToolCollection,HistoryWritings,1.0,101.0,True,True,,
SimpleToolCollection,HistoryWritings,1.0,101.0,True,False,Genuine,Report #6904
SimpleToolCollection,HistoryWritings,1.0,101.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,101.0,True,False,Fake,Report #6905
SimpleToolCollection,HistoryWritings,1.0,101.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,101.0,True,False,,Report #6906
SimpleToolCollection,HistoryWritings,1.0,101.0,True,False,,
SimpleToolCollection,HistoryWritings,1.0,101.0,False,True,Genuine,Report #6907
SimpleToolCollection,HistoryWritings,1.0,101.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1.0,101.0,False,True,Fake,Report #6908
SimpleToolCollection,HistoryWritings,1.0,101.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1.0,101.0,False,True,,Report #6909
SimpleToolCollection,HistoryWritings,1.0,101.0,False,True,,
SimpleToolCollection,HistoryWritings,1.0,101.0,False,False,Genuine,Report #6910
SimpleToolCollection,HistoryWritings,1.0,101.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1.0,101.0,False,False,Fake,Report #6911
SimpleToolCollection,HistoryWritings,1.0,101.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1.0,101.0,False,False,,Report #6912
SimpleToolCollection,HistoryWritings,1.0,101.0,False,False,,
SimpleToolCollection,HistoryWritings,100.0,-1.0,True,True,Genuine,Report #6913
SimpleToolCollection,HistoryWritings,100.0,-1.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,-1.0,True,True,Fake,Report #6914
SimpleToolCollection,HistoryWritings,100.0,-1.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,-1.0,True,True,,Report #6915
SimpleToolCollection,HistoryWritings,100.0,-1.0,True,True,,
SimpleToolCollection,HistoryWritings,100.0,-1.0,True,False,Genuine,Report #6916
SimpleToolCollection,HistoryWritings,100.0,-1.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,-1.0,True,False,Fake,Report #6917
SimpleToolCollection,HistoryWritings,100.0,-1.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,-1.0,True,False,,Report #6918
SimpleToolCollection,HistoryWritings,100.0,-1.0,True,False,,
SimpleToolCollection,HistoryWritings,100.0,-1.0,False,True,Genuine,Report #6919
SimpleToolCollection,HistoryWritings,100.0,-1.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,-1.0,False,True,Fake,Report #6920
SimpleToolCollection,HistoryWritings,100.0,-1.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,-1.0,False,True,,Report #6921
SimpleToolCollection,HistoryWritings,100.0,-1.0,False,True,,
SimpleToolCollection,HistoryWritings,100.0,-1.0,False,False,Genuine,Report #6922
SimpleToolCollection,HistoryWritings,100.0,-1.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,-1.0,False,False,Fake,Report #6923
SimpleToolCollection,HistoryWritings,100.0,-1.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,-1.0,False,False,,Report #6924
SimpleToolCollection,HistoryWritings,100.0,-1.0,False,False,,
SimpleToolCollection,HistoryWritings,100.0,0.0,True,True,Genuine,Report #6925
SimpleToolCollection,HistoryWritings,100.0,0.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,0.0,True,True,Fake,Report #6926
SimpleToolCollection,HistoryWritings,100.0,0.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,0.0,True,True,,Report #6927
SimpleToolCollection,HistoryWritings,100.0,0.0,True,True,,
SimpleToolCollection,HistoryWritings,100.0,0.0,True,False,Genuine,Report #6928
SimpleToolCollection,HistoryWritings,100.0,0.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,0.0,True,False,Fake,Report #6929
SimpleToolCollection,HistoryWritings,100.0,0.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,0.0,True,False,,Report #6930
SimpleToolCollection,HistoryWritings,100.0,0.0,True,False,,
SimpleToolCollection,HistoryWritings,100.0,0.0,False,True,Genuine,Report #6931
SimpleToolCollection,HistoryWritings,100.0,0.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,0.0,False,True,Fake,Report #6932
SimpleToolCollection,HistoryWritings,100.0,0.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,0.0,False,True,,Report #6933
SimpleToolCollection,HistoryWritings,100.0,0.0,False,True,,
SimpleToolCollection,HistoryWritings,100.0,0.0,False,False,Genuine,Report #6934
SimpleToolCollection,HistoryWritings,100.0,0.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,0.0,False,False,Fake,Report #6935
SimpleToolCollection,HistoryWritings,100.0,0.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,0.0,False,False,,Report #6936
SimpleToolCollection,HistoryWritings,100.0,0.0,False,False,,
SimpleToolCollection,HistoryWritings,100.0,19.0,True,True,Genuine,Report #6937
SimpleToolCollection,HistoryWritings,100.0,19.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,19.0,True,True,Fake,Report #6938
SimpleToolCollection,HistoryWritings,100.0,19.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,19.0,True,True,,Report #6939
SimpleToolCollection,HistoryWritings,100.0,19.0,True,True,,
SimpleToolCollection,HistoryWritings,100.0,19.0,True,False,Genuine,Report #6940
SimpleToolCollection,HistoryWritings,100.0,19.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,19.0,True,False,Fake,Report #6941
SimpleToolCollection,HistoryWritings,100.0,19.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,19.0,True,False,,Report #6942
SimpleToolCollection,HistoryWritings,100.0,19.0,True,False,,
SimpleToolCollection,HistoryWritings,100.0,19.0,False,True,Genuine,Report #6943
SimpleToolCollection,HistoryWritings,100.0,19.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,19.0,False,True,Fake,Report #6944
SimpleToolCollection,HistoryWritings,100.0,19.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,19.0,False,True,,Report #6945
SimpleToolCollection,HistoryWritings,100.0,19.0,False,True,,
SimpleToolCollection,HistoryWritings,100.0,19.0,False,False,Genuine,Report #6946
SimpleToolCollection,HistoryWritings,100.0,19.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,19.0,False,False,Fake,Report #6947
SimpleToolCollection,HistoryWritings,100.0,19.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,19.0,False,False,,Report #6948
SimpleToolCollection,HistoryWritings,100.0,19.0,False,False,,
SimpleToolCollection,HistoryWritings,100.0,20.0,True,True,Genuine,Report #6949
SimpleToolCollection,HistoryWritings,100.0,20.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,20.0,True,True,Fake,Report #6950
SimpleToolCollection,HistoryWritings,100.0,20.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,20.0,True,True,,Report #6951
SimpleToolCollection,HistoryWritings,100.0,20.0,True,True,,
SimpleToolCollection,HistoryWritings,100.0,20.0,True,False,Genuine,Report #6952
SimpleToolCollection,HistoryWritings,100.0,20.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,20.0,True,False,Fake,Report #6953
SimpleToolCollection,HistoryWritings,100.0,20.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,20.0,True,False,,Report #6954
SimpleToolCollection,HistoryWritings,100.0,20.0,True,False,,
SimpleToolCollection,HistoryWritings,100.0,20.0,False,True,Genuine,Report #6955
SimpleToolCollection,HistoryWritings,100.0,20.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,20.0,False,True,Fake,Report #6956
SimpleToolCollection,HistoryWritings,100.0,20.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,20.0,False,True,,Report #6957
SimpleToolCollection,HistoryWritings,100.0,20.0,False,True,,
SimpleToolCollection,HistoryWritings,100.0,20.0,False,False,Genuine,Report #6958
SimpleToolCollection,HistoryWritings,100.0,20.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,20.0,False,False,Fake,Report #6959
SimpleToolCollection,HistoryWritings,100.0,20.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,20.0,False,False,,Report #6960
SimpleToolCollection,HistoryWritings,100.0,20.0,False,False,,
SimpleToolCollection,HistoryWritings,100.0,21.0,True,True,Genuine,Report #6961
SimpleToolCollection,HistoryWritings,100.0,21.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,21.0,True,True,Fake,Report #6962
SimpleToolCollection,HistoryWritings,100.0,21.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,21.0,True,True,,Report #6963
SimpleToolCollection,HistoryWritings,100.0,21.0,True,True,,
SimpleToolCollection,HistoryWritings,100.0,21.0,True,False,Genuine,Report #6964
SimpleToolCollection,HistoryWritings,100.0,21.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,21.0,True,False,Fake,Report #6965
SimpleToolCollection,HistoryWritings,100.0,21.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,21.0,True,False,,Report #6966
SimpleToolCollection,HistoryWritings,100.0,21.0,True,False,,
SimpleToolCollection,HistoryWritings,100.0,21.0,False,True,Genuine,Report #6967
SimpleToolCollection,HistoryWritings,100.0,21.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,21.0,False,True,Fake,Report #6968
SimpleToolCollection,HistoryWritings,100.0,21.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,21.0,False,True,,Report #6969
SimpleToolCollection,HistoryWritings,100.0,21.0,False,True,,
SimpleToolCollection,HistoryWritings,100.0,21.0,False,False,Genuine,Report #6970
SimpleToolCollection,HistoryWritings,100.0,21.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,21.0,False,False,Fake,Report #6971
SimpleToolCollection,HistoryWritings,100.0,21.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,21.0,False,False,,Report #6972
SimpleToolCollection,HistoryWritings,100.0,21.0,False,False,,
SimpleToolCollection,HistoryWritings,100.0,99.0,True,True,Genuine,Report #6973
SimpleToolCollection,HistoryWritings,100.0,99.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,99.0,True,True,Fake,Report #6974
SimpleToolCollection,HistoryWritings,100.0,99.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,99.0,True,True,,Report #6975
SimpleToolCollection,HistoryWritings,100.0,99.0,True,True,,
SimpleToolCollection,HistoryWritings,100.0,99.0,True,False,Genuine,Report #6976
SimpleToolCollection,HistoryWritings,100.0,99.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,99.0,True,False,Fake,Report #6977
SimpleToolCollection,HistoryWritings,100.0,99.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,99.0,True,False,,Report #6978
SimpleToolCollection,HistoryWritings,100.0,99.0,True,False,,
SimpleToolCollection,HistoryWritings,100.0,99.0,False,True,Genuine,Report #6979
SimpleToolCollection,HistoryWritings,100.0,99.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,99.0,False,True,Fake,Report #6980
SimpleToolCollection,HistoryWritings,100.0,99.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,99.0,False,True,,Report #6981
SimpleToolCollection,HistoryWritings,100.0,99.0,False,True,,
SimpleToolCollection,HistoryWritings,100.0,99.0,False,False,Genuine,Report #6982
SimpleToolCollection,HistoryWritings,100.0,99.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,99.0,False,False,Fake,Report #6983
SimpleToolCollection,HistoryWritings,100.0,99.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,99.0,False,False,,Report #6984
SimpleToolCollection,HistoryWritings,100.0,99.0,False,False,,
SimpleToolCollection,HistoryWritings,100.0,100.0,True,True,Genuine,Report #6985
SimpleToolCollection,HistoryWritings,100.0,100.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,100.0,True,True,Fake,Report #6986
SimpleToolCollection,HistoryWritings,100.0,100.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,100.0,True,True,,Report #6987
SimpleToolCollection,HistoryWritings,100.0,100.0,True,True,,
SimpleToolCollection,HistoryWritings,100.0,100.0,True,False,Genuine,Report #6988
SimpleToolCollection,HistoryWritings,100.0,100.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,100.0,True,False,Fake,Report #6989
SimpleToolCollection,HistoryWritings,100.0,100.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,100.0,True,False,,Report #6990
SimpleToolCollection,HistoryWritings,100.0,100.0,True,False,,
SimpleToolCollection,HistoryWritings,100.0,100.0,False,True,Genuine,Report #6991
SimpleToolCollection,HistoryWritings,100.0,100.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,100.0,False,True,Fake,Report #6992
SimpleToolCollection,HistoryWritings,100.0,100.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,100.0,False,True,,Report #6993
SimpleToolCollection,HistoryWritings,100.0,100.0,False,True,,
SimpleToolCollection,HistoryWritings,100.0,100.0,False,False,Genuine,Report #6994
SimpleToolCollection,HistoryWritings,100.0,100.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,100.0,False,False,Fake,Report #6995
SimpleToolCollection,HistoryWritings,100.0,100.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,100.0,False,False,,Report #6996
SimpleToolCollection,HistoryWritings,100.0,100.0,False,False,,
SimpleToolCollection,HistoryWritings,100.0,101.0,True,True,Genuine,Report #6997
SimpleToolCollection,HistoryWritings,100.0,101.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,101.0,True,True,Fake,Report #6998
SimpleToolCollection,HistoryWritings,100.0,101.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,101.0,True,True,,Report #6999
SimpleToolCollection,HistoryWritings,100.0,101.0,True,True,,
SimpleToolCollection,HistoryWritings,100.0,101.0,True,False,Genuine,Report #7000
SimpleToolCollection,HistoryWritings,100.0,101.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,101.0,True,False,Fake,Report #7001
SimpleToolCollection,HistoryWritings,100.0,101.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,101.0,True,False,,Report #7002
SimpleToolCollection,HistoryWritings,100.0,101.0,True,False,,
SimpleToolCollection,HistoryWritings,100.0,101.0,False,True,Genuine,Report #7003
SimpleToolCollection,HistoryWritings,100.0,101.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,100.0,101.0,False,True,Fake,Report #7004
SimpleToolCollection,HistoryWritings,100.0,101.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,100.0,101.0,False,True,,Report #7005
SimpleToolCollection,HistoryWritings,100.0,101.0,False,True,,
SimpleToolCollection,HistoryWritings,100.0,101.0,False,False,Genuine,Report #7006
SimpleToolCollection,HistoryWritings,100.0,101.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,100.0,101.0,False,False,Fake,Report #7007
SimpleToolCollection,HistoryWritings,100.0,101.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,100.0,101.0,False,False,,Report #7008
SimpleToolCollection,HistoryWritings,100.0,101.0,False,False,,
SimpleToolCollection,HistoryWritings,1000.0,-1.0,True,True,Genuine,Report #7009
SimpleToolCollection,HistoryWritings,1000.0,-1.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,-1.0,True,True,Fake,Report #7010
SimpleToolCollection,HistoryWritings,1000.0,-1.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,-1.0,True,True,,Report #7011
SimpleToolCollection,HistoryWritings,1000.0,-1.0,True,True,,
SimpleToolCollection,HistoryWritings,1000.0,-1.0,True,False,Genuine,Report #7012
SimpleToolCollection,HistoryWritings,1000.0,-1.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,-1.0,True,False,Fake,Report #7013
SimpleToolCollection,HistoryWritings,1000.0,-1.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,-1.0,True,False,,Report #7014
SimpleToolCollection,HistoryWritings,1000.0,-1.0,True,False,,
SimpleToolCollection,HistoryWritings,1000.0,-1.0,False,True,Genuine,Report #7015
SimpleToolCollection,HistoryWritings,1000.0,-1.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,-1.0,False,True,Fake,Report #7016
SimpleToolCollection,HistoryWritings,1000.0,-1.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,-1.0,False,True,,Report #7017
SimpleToolCollection,HistoryWritings,1000.0,-1.0,False,True,,
SimpleToolCollection,HistoryWritings,1000.0,-1.0,False,False,Genuine,Report #7018
SimpleToolCollection,HistoryWritings,1000.0,-1.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,-1.0,False,False,Fake,Report #7019
SimpleToolCollection,HistoryWritings,1000.0,-1.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,-1.0,False,False,,Report #7020
SimpleToolCollection,HistoryWritings,1000.0,-1.0,False,False,,
SimpleToolCollection,HistoryWritings,1000.0,0.0,True,True,Genuine,Report #7021
SimpleToolCollection,HistoryWritings,1000.0,0.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,0.0,True,True,Fake,Report #7022
SimpleToolCollection,HistoryWritings,1000.0,0.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,0.0,True,True,,Report #7023
SimpleToolCollection,HistoryWritings,1000.0,0.0,True,True,,
SimpleToolCollection,HistoryWritings,1000.0,0.0,True,False,Genuine,Report #7024
SimpleToolCollection,HistoryWritings,1000.0,0.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,0.0,True,False,Fake,Report #7025
SimpleToolCollection,HistoryWritings,1000.0,0.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,0.0,True,False,,Report #7026
SimpleToolCollection,HistoryWritings,1000.0,0.0,True,False,,
SimpleToolCollection,HistoryWritings,1000.0,0.0,False,True,Genuine,Report #7027
SimpleToolCollection,HistoryWritings,1000.0,0.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,0.0,False,True,Fake,Report #7028
SimpleToolCollection,HistoryWritings,1000.0,0.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,0.0,False,True,,Report #7029
SimpleToolCollection,HistoryWritings,1000.0,0.0,False,True,,
SimpleToolCollection,HistoryWritings,1000.0,0.0,False,False,Genuine,Report #7030
SimpleToolCollection,HistoryWritings,1000.0,0.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,0.0,False,False,Fake,Report #7031
SimpleToolCollection,HistoryWritings,1000.0,0.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,0.0,False,False,,Report #7032
SimpleToolCollection,HistoryWritings,1000.0,0.0,False,False,,
SimpleToolCollection,HistoryWritings,1000.0,19.0,True,True,Genuine,Report #7033
SimpleToolCollection,HistoryWritings,1000.0,19.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,19.0,True,True,Fake,Report #7034
SimpleToolCollection,HistoryWritings,1000.0,19.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,19.0,True,True,,Report #7035
SimpleToolCollection,HistoryWritings,1000.0,19.0,True,True,,
SimpleToolCollection,HistoryWritings,1000.0,19.0,True,False,Genuine,Report #7036
SimpleToolCollection,HistoryWritings,1000.0,19.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,19.0,True,False,Fake,Report #7037
SimpleToolCollection,HistoryWritings,1000.0,19.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,19.0,True,False,,Report #7038
SimpleToolCollection,HistoryWritings,1000.0,19.0,True,False,,
SimpleToolCollection,HistoryWritings,1000.0,19.0,False,True,Genuine,Report #7039
SimpleToolCollection,HistoryWritings,1000.0,19.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,19.0,False,True,Fake,Report #7040
SimpleToolCollection,HistoryWritings,1000.0,19.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,19.0,False,True,,Report #7041
SimpleToolCollection,HistoryWritings,1000.0,19.0,False,True,,
SimpleToolCollection,HistoryWritings,1000.0,19.0,False,False,Genuine,Report #7042
SimpleToolCollection,HistoryWritings,1000.0,19.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,19.0,False,False,Fake,Report #7043
SimpleToolCollection,HistoryWritings,1000.0,19.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,19.0,False,False,,Report #7044
SimpleToolCollection,HistoryWritings,1000.0,19.0,False,False,,
SimpleToolCollection,HistoryWritings,1000.0,20.0,True,True,Genuine,Report #7045
SimpleToolCollection,HistoryWritings,1000.0,20.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,20.0,True,True,Fake,Report #7046
SimpleToolCollection,HistoryWritings,1000.0,20.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,20.0,True,True,,Report #7047
SimpleToolCollection,HistoryWritings,1000.0,20.0,True,True,,
SimpleToolCollection,HistoryWritings,1000.0,20.0,True,False,Genuine,Report #7048
SimpleToolCollection,HistoryWritings,1000.0,20.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,20.0,True,False,Fake,Report #7049
SimpleToolCollection,HistoryWritings,1000.0,20.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,20.0,True,False,,Report #7050
SimpleToolCollection,HistoryWritings,1000.0,20.0,True,False,,
SimpleToolCollection,HistoryWritings,1000.0,20.0,False,True,Genuine,Report #7051
SimpleToolCollection,HistoryWritings,1000.0,20.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,20.0,False,True,Fake,Report #7052
SimpleToolCollection,HistoryWritings,1000.0,20.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,20.0,False,True,,Report #7053
SimpleToolCollection,HistoryWritings,1000.0,20.0,False,True,,
SimpleToolCollection,HistoryWritings,1000.0,20.0,False,False,Genuine,Report #7054
SimpleToolCollection,HistoryWritings,1000.0,20.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,20.0,False,False,Fake,Report #7055
SimpleToolCollection,HistoryWritings,1000.0,20.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,20.0,False,False,,Report #7056
SimpleToolCollection,HistoryWritings,1000.0,20.0,False,False,,
SimpleToolCollection,HistoryWritings,1000.0,21.0,True,True,Genuine,Report #7057
SimpleToolCollection,HistoryWritings,1000.0,21.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,21.0,True,True,Fake,Report #7058
SimpleToolCollection,HistoryWritings,1000.0,21.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,21.0,True,True,,Report #7059
SimpleToolCollection,HistoryWritings,1000.0,21.0,True,True,,
SimpleToolCollection,HistoryWritings,1000.0,21.0,True,False,Genuine,Report #7060
SimpleToolCollection,HistoryWritings,1000.0,21.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,21.0,True,False,Fake,Report #7061
SimpleToolCollection,HistoryWritings,1000.0,21.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,21.0,True,False,,Report #7062
SimpleToolCollection,HistoryWritings,1000.0,21.0,True,False,,
SimpleToolCollection,HistoryWritings,1000.0,21.0,False,True,Genuine,Report #7063
SimpleToolCollection,HistoryWritings,1000.0,21.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,21.0,False,True,Fake,Report #7064
SimpleToolCollection,HistoryWritings,1000.0,21.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,21.0,False,True,,Report #7065
SimpleToolCollection,HistoryWritings,1000.0,21.0,False,True,,
SimpleToolCollection,HistoryWritings,1000.0,21.0,False,False,Genuine,Report #7066
SimpleToolCollection,HistoryWritings,1000.0,21.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,21.0,False,False,Fake,Report #7067
SimpleToolCollection,HistoryWritings,1000.0,21.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,21.0,False,False,,Report #7068
SimpleToolCollection,HistoryWritings,1000.0,21.0,False,False,,
SimpleToolCollection,HistoryWritings,1000.0,99.0,True,True,Genuine,Report #7069
SimpleToolCollection,HistoryWritings,1000.0,99.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,99.0,True,True,Fake,Report #7070
SimpleToolCollection,HistoryWritings,1000.0,99.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,99.0,True,True,,Report #7071
SimpleToolCollection,HistoryWritings,1000.0,99.0,True,True,,
SimpleToolCollection,HistoryWritings,1000.0,99.0,True,False,Genuine,Report #7072
SimpleToolCollection,HistoryWritings,1000.0,99.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,99.0,True,False,Fake,Report #7073
SimpleToolCollection,HistoryWritings,1000.0,99.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,99.0,True,False,,Report #7074
SimpleToolCollection,HistoryWritings,1000.0,99.0,True,False,,
SimpleToolCollection,HistoryWritings,1000.0,99.0,False,True,Genuine,Report #7075
SimpleToolCollection,HistoryWritings,1000.0,99.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,99.0,False,True,Fake,Report #7076
SimpleToolCollection,HistoryWritings,1000.0,99.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,99.0,False,True,,Report #7077
SimpleToolCollection,HistoryWritings,1000.0,99.0,False,True,,
SimpleToolCollection,HistoryWritings,1000.0,99.0,False,False,Genuine,Report #7078
SimpleToolCollection,HistoryWritings,1000.0,99.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,99.0,False,False,Fake,Report #7079
SimpleToolCollection,HistoryWritings,1000.0,99.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,99.0,False,False,,Report #7080
SimpleToolCollection,HistoryWritings,1000.0,99.0,False,False,,
SimpleToolCollection,HistoryWritings,1000.0,100.0,True,True,Genuine,Report #7081
SimpleToolCollection,HistoryWritings,1000.0,100.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,100.0,True,True,Fake,Report #7082
SimpleToolCollection,HistoryWritings,1000.0,100.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,100.0,True,True,,Report #7083
SimpleToolCollection,HistoryWritings,1000.0,100.0,True,True,,
SimpleToolCollection,HistoryWritings,1000.0,100.0,True,False,Genuine,Report #7084
SimpleToolCollection,HistoryWritings,1000.0,100.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,100.0,True,False,Fake,Report #7085
SimpleToolCollection,HistoryWritings,1000.0,100.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,100.0,True,False,,Report #7086
SimpleToolCollection,HistoryWritings,1000.0,100.0,True,False,,
SimpleToolCollection,HistoryWritings,1000.0,100.0,False,True,Genuine,Report #7087
SimpleToolCollection,HistoryWritings,1000.0,100.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,100.0,False,True,Fake,Report #7088
SimpleToolCollection,HistoryWritings,1000.0,100.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,100.0,False,True,,Report #7089
SimpleToolCollection,HistoryWritings,1000.0,100.0,False,True,,
SimpleToolCollection,HistoryWritings,1000.0,100.0,False,False,Genuine,Report #7090
SimpleToolCollection,HistoryWritings,1000.0,100.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,100.0,False,False,Fake,Report #7091
SimpleToolCollection,HistoryWritings,1000.0,100.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,100.0,False,False,,Report #7092
SimpleToolCollection,HistoryWritings,1000.0,100.0,False,False,,
SimpleToolCollection,HistoryWritings,1000.0,101.0,True,True,Genuine,Report #7093
SimpleToolCollection,HistoryWritings,1000.0,101.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,101.0,True,True,Fake,Report #7094
SimpleToolCollection,HistoryWritings,1000.0,101.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,101.0,True,True,,Report #7095
SimpleToolCollection,HistoryWritings,1000.0,101.0,True,True,,
SimpleToolCollection,HistoryWritings,1000.0,101.0,True,False,Genuine,Report #7096
SimpleToolCollection,HistoryWritings,1000.0,101.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,101.0,True,False,Fake,Report #7097
SimpleToolCollection,HistoryWritings,1000.0,101.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,101.0,True,False,,Report #7098
SimpleToolCollection,HistoryWritings,1000.0,101.0,True,False,,
SimpleToolCollection,HistoryWritings,1000.0,101.0,False,True,Genuine,Report #7099
SimpleToolCollection,HistoryWritings,1000.0,101.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,101.0,False,True,Fake,Report #7100
SimpleToolCollection,HistoryWritings,1000.0,101.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000.0,101.0,False,True,,Report #7101
SimpleToolCollection,HistoryWritings,1000.0,101.0,False,True,,
SimpleToolCollection,HistoryWritings,1000.0,101.0,False,False,Genuine,Report #7102
SimpleToolCollection,HistoryWritings,1000.0,101.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000.0,101.0,False,False,Fake,Report #7103
SimpleToolCollection,HistoryWritings,1000.0,101.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000.0,101.0,False,False,,Report #7104
SimpleToolCollection,HistoryWritings,1000.0,101.0,False,False,,
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,True,True,Genuine,Report #7105
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,True,True,Fake,Report #7106
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,True,True,,Report #7107
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,True,True,,
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,True,False,Genuine,Report #7108
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,True,False,Fake,Report #7109
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,True,False,,Report #7110
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,True,False,,
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,False,True,Genuine,Report #7111
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,False,True,Fake,Report #7112
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,False,True,,Report #7113
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,False,True,,
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,False,False,Genuine,Report #7114
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,False,False,Fake,Report #7115
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,False,False,,Report #7116
SimpleToolCollection,HistoryWritings,1000000.0,-1.0,False,False,,
SimpleToolCollection,HistoryWritings,1000000.0,0.0,True,True,Genuine,Report #7117
SimpleToolCollection,HistoryWritings,1000000.0,0.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,0.0,True,True,Fake,Report #7118
SimpleToolCollection,HistoryWritings,1000000.0,0.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,0.0,True,True,,Report #7119
SimpleToolCollection,HistoryWritings,1000000.0,0.0,True,True,,
SimpleToolCollection,HistoryWritings,1000000.0,0.0,True,False,Genuine,Report #7120
SimpleToolCollection,HistoryWritings,1000000.0,0.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,0.0,True,False,Fake,Report #7121
SimpleToolCollection,HistoryWritings,1000000.0,0.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,0.0,True,False,,Report #7122
SimpleToolCollection,HistoryWritings,1000000.0,0.0,True,False,,
SimpleToolCollection,HistoryWritings,1000000.0,0.0,False,True,Genuine,Report #7123
SimpleToolCollection,HistoryWritings,1000000.0,0.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,0.0,False,True,Fake,Report #7124
SimpleToolCollection,HistoryWritings,1000000.0,0.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,0.0,False,True,,Report #7125
SimpleToolCollection,HistoryWritings,1000000.0,0.0,False,True,,
SimpleToolCollection,HistoryWritings,1000000.0,0.0,False,False,Genuine,Report #7126
SimpleToolCollection,HistoryWritings,1000000.0,0.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,0.0,False,False,Fake,Report #7127
SimpleToolCollection,HistoryWritings,1000000.0,0.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,0.0,False,False,,Report #7128
SimpleToolCollection,HistoryWritings,1000000.0,0.0,False,False,,
SimpleToolCollection,HistoryWritings,1000000.0,19.0,True,True,Genuine,Report #7129
SimpleToolCollection,HistoryWritings,1000000.0,19.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,19.0,True,True,Fake,Report #7130
SimpleToolCollection,HistoryWritings,1000000.0,19.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,19.0,True,True,,Report #7131
SimpleToolCollection,HistoryWritings,1000000.0,19.0,True,True,,
SimpleToolCollection,HistoryWritings,1000000.0,19.0,True,False,Genuine,Report #7132
SimpleToolCollection,HistoryWritings,1000000.0,19.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,19.0,True,False,Fake,Report #7133
SimpleToolCollection,HistoryWritings,1000000.0,19.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,19.0,True,False,,Report #7134
SimpleToolCollection,HistoryWritings,1000000.0,19.0,True,False,,
SimpleToolCollection,HistoryWritings,1000000.0,19.0,False,True,Genuine,Report #7135
SimpleToolCollection,HistoryWritings,1000000.0,19.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,19.0,False,True,Fake,Report #7136
SimpleToolCollection,HistoryWritings,1000000.0,19.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,19.0,False,True,,Report #7137
SimpleToolCollection,HistoryWritings,1000000.0,19.0,False,True,,
SimpleToolCollection,HistoryWritings,1000000.0,19.0,False,False,Genuine,Report #7138
SimpleToolCollection,HistoryWritings,1000000.0,19.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,19.0,False,False,Fake,Report #7139
SimpleToolCollection,HistoryWritings,1000000.0,19.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,19.0,False,False,,Report #7140
SimpleToolCollection,HistoryWritings,1000000.0,19.0,False,False,,
SimpleToolCollection,HistoryWritings,1000000.0,20.0,True,True,Genuine,Report #7141
SimpleToolCollection,HistoryWritings,1000000.0,20.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,20.0,True,True,Fake,Report #7142
SimpleToolCollection,HistoryWritings,1000000.0,20.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,20.0,True,True,,Report #7143
SimpleToolCollection,HistoryWritings,1000000.0,20.0,True,True,,
SimpleToolCollection,HistoryWritings,1000000.0,20.0,True,False,Genuine,Report #7144
SimpleToolCollection,HistoryWritings,1000000.0,20.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,20.0,True,False,Fake,Report #7145
SimpleToolCollection,HistoryWritings,1000000.0,20.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,20.0,True,False,,Report #7146
SimpleToolCollection,HistoryWritings,1000000.0,20.0,True,False,,
SimpleToolCollection,HistoryWritings,1000000.0,20.0,False,True,Genuine,Report #7147
SimpleToolCollection,HistoryWritings,1000000.0,20.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,20.0,False,True,Fake,Report #7148
SimpleToolCollection,HistoryWritings,1000000.0,20.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,20.0,False,True,,Report #7149
SimpleToolCollection,HistoryWritings,1000000.0,20.0,False,True,,
SimpleToolCollection,HistoryWritings,1000000.0,20.0,False,False,Genuine,Report #7150
SimpleToolCollection,HistoryWritings,1000000.0,20.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,20.0,False,False,Fake,Report #7151
SimpleToolCollection,HistoryWritings,1000000.0,20.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,20.0,False,False,,Report #7152
SimpleToolCollection,HistoryWritings,1000000.0,20.0,False,False,,
SimpleToolCollection,HistoryWritings,1000000.0,21.0,True,True,Genuine,Report #7153
SimpleToolCollection,HistoryWritings,1000000.0,21.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,21.0,True,True,Fake,Report #7154
SimpleToolCollection,HistoryWritings,1000000.0,21.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,21.0,True,True,,Report #7155
SimpleToolCollection,HistoryWritings,1000000.0,21.0,True,True,,
SimpleToolCollection,HistoryWritings,1000000.0,21.0,True,False,Genuine,Report #7156
SimpleToolCollection,HistoryWritings,1000000.0,21.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,21.0,True,False,Fake,Report #7157
SimpleToolCollection,HistoryWritings,1000000.0,21.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,21.0,True,False,,Report #7158
SimpleToolCollection,HistoryWritings,1000000.0,21.0,True,False,,
SimpleToolCollection,HistoryWritings,1000000.0,21.0,False,True,Genuine,Report #7159
SimpleToolCollection,HistoryWritings,1000000.0,21.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,21.0,False,True,Fake,Report #7160
SimpleToolCollection,HistoryWritings,1000000.0,21.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,21.0,False,True,,Report #7161
SimpleToolCollection,HistoryWritings,1000000.0,21.0,False,True,,
SimpleToolCollection,HistoryWritings,1000000.0,21.0,False,False,Genuine,Report #7162
SimpleToolCollection,HistoryWritings,1000000.0,21.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,21.0,False,False,Fake,Report #7163
SimpleToolCollection,HistoryWritings,1000000.0,21.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,21.0,False,False,,Report #7164
SimpleToolCollection,HistoryWritings,1000000.0,21.0,False,False,,
SimpleToolCollection,HistoryWritings,1000000.0,99.0,True,True,Genuine,Report #7165
SimpleToolCollection,HistoryWritings,1000000.0,99.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,99.0,True,True,Fake,Report #7166
SimpleToolCollection,HistoryWritings,1000000.0,99.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,99.0,True,True,,Report #7167
SimpleToolCollection,HistoryWritings,1000000.0,99.0,True,True,,
SimpleToolCollection,HistoryWritings,1000000.0,99.0,True,False,Genuine,Report #7168
SimpleToolCollection,HistoryWritings,1000000.0,99.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,99.0,True,False,Fake,Report #7169
SimpleToolCollection,HistoryWritings,1000000.0,99.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,99.0,True,False,,Report #7170
SimpleToolCollection,HistoryWritings,1000000.0,99.0,True,False,,
SimpleToolCollection,HistoryWritings,1000000.0,99.0,False,True,Genuine,Report #7171
SimpleToolCollection,HistoryWritings,1000000.0,99.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,99.0,False,True,Fake,Report #7172
SimpleToolCollection,HistoryWritings,1000000.0,99.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,99.0,False,True,,Report #7173
SimpleToolCollection,HistoryWritings,1000000.0,99.0,False,True,,
SimpleToolCollection,HistoryWritings,1000000.0,99.0,False,False,Genuine,Report #7174
SimpleToolCollection,HistoryWritings,1000000.0,99.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,99.0,False,False,Fake,Report #7175
SimpleToolCollection,HistoryWritings,1000000.0,99.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,99.0,False,False,,Report #7176
SimpleToolCollection,HistoryWritings,1000000.0,99.0,False,False,,
SimpleToolCollection,HistoryWritings,1000000.0,100.0,True,True,Genuine,Report #7177
SimpleToolCollection,HistoryWritings,1000000.0,100.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,100.0,True,True,Fake,Report #7178
SimpleToolCollection,HistoryWritings,1000000.0,100.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,100.0,True,True,,Report #7179
SimpleToolCollection,HistoryWritings,1000000.0,100.0,True,True,,
SimpleToolCollection,HistoryWritings,1000000.0,100.0,True,False,Genuine,Report #7180
SimpleToolCollection,HistoryWritings,1000000.0,100.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,100.0,True,False,Fake,Report #7181
SimpleToolCollection,HistoryWritings,1000000.0,100.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,100.0,True,False,,Report #7182
SimpleToolCollection,HistoryWritings,1000000.0,100.0,True,False,,
SimpleToolCollection,HistoryWritings,1000000.0,100.0,False,True,Genuine,Report #7183
SimpleToolCollection,HistoryWritings,1000000.0,100.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,100.0,False,True,Fake,Report #7184
SimpleToolCollection,HistoryWritings,1000000.0,100.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,100.0,False,True,,Report #7185
SimpleToolCollection,HistoryWritings,1000000.0,100.0,False,True,,
SimpleToolCollection,HistoryWritings,1000000.0,100.0,False,False,Genuine,Report #7186
SimpleToolCollection,HistoryWritings,1000000.0,100.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,100.0,False,False,Fake,Report #7187
SimpleToolCollection,HistoryWritings,1000000.0,100.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,100.0,False,False,,Report #7188
SimpleToolCollection,HistoryWritings,1000000.0,100.0,False,False,,
SimpleToolCollection,HistoryWritings,1000000.0,101.0,True,True,Genuine,Report #7189
SimpleToolCollection,HistoryWritings,1000000.0,101.0,True,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,101.0,True,True,Fake,Report #7190
SimpleToolCollection,HistoryWritings,1000000.0,101.0,True,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,101.0,True,True,,Report #7191
SimpleToolCollection,HistoryWritings,1000000.0,101.0,True,True,,
SimpleToolCollection,HistoryWritings,1000000.0,101.0,True,False,Genuine,Report #7192
SimpleToolCollection,HistoryWritings,1000000.0,101.0,True,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,101.0,True,False,Fake,Report #7193
SimpleToolCollection,HistoryWritings,1000000.0,101.0,True,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,101.0,True,False,,Report #7194
SimpleToolCollection,HistoryWritings,1000000.0,101.0,True,False,,
SimpleToolCollection,HistoryWritings,1000000.0,101.0,False,True,Genuine,Report #7195
SimpleToolCollection,HistoryWritings,1000000.0,101.0,False,True,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,101.0,False,True,Fake,Report #7196
SimpleToolCollection,HistoryWritings,1000000.0,101.0,False,True,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,101.0,False,True,,Report #7197
SimpleToolCollection,HistoryWritings,1000000.0,101.0,False,True,,
SimpleToolCollection,HistoryWritings,1000000.0,101.0,False,False,Genuine,Report #7198
SimpleToolCollection,HistoryWritings,1000000.0,101.0,False,False,Genuine,
SimpleToolCollection,HistoryWritings,1000000.0,101.0,False,False,Fake,Report #7199
SimpleToolCollection,HistoryWritings,1000000.0,101.0,False,False,Fake,
SimpleToolCollection,HistoryWritings,1000000.0,101.0,False,False,,Report #7200
SimpleToolCollection,HistoryWritings,1000000.0,101.0,False,False,,
Diary,Paintings,0.0,-1.0,True,True,Genuine,Report #7201
Diary,Paintings,0.0,-1.0,True,True,Genuine,
Diary,Paintings,0.0,-1.0,True,True,Fake,Report #7202
Diary,Paintings,0.0,-1.0,True,True,Fake,
Diary,Paintings,0.0,-1.0,True,True,,Report #7203
Diary,Paintings,0.0,-1.0,True,True,,
Diary,Paintings,0.0,-1.0,True,False,Genuine,Report #7204
Diary,Paintings,0.0,-1.0,True,False,Genuine,
Diary,Paintings,0.0,-1.0,True,False,Fake,Report #7205
Diary,Paintings,0.0,-1.0,True,False,Fake,
Diary,Paintings,0.0,-1.0,True,False,,Report #7206
Diary,Paintings,0.0,-1.0,True,False,,
Diary,Paintings,0.0,-1.0,False,True,Genuine,Report #7207
Diary,Paintings,0.0,-1.0,False,True,Genuine,
Diary,Paintings,0.0,-1.0,False,True,Fake,Report #7208
Diary,Paintings,0.0,-1.0,False,True,Fake,
Diary,Paintings,0.0,-1.0,False,True,,Report #7209
Diary,Paintings,0.0,-1.0,False,True,,
Diary,Paintings,0.0,-1.0,False,False,Genuine,Report #7210
Diary,Paintings,0.0,-1.0,False,False,Genuine,
Diary,Paintings,0.0,-1.0,False,False,Fake,Report #7211
Diary,Paintings,0.0,-1.0,False,False,Fake,
Diary,Paintings,0.0,-1.0,False,False,,Report #7212
Diary,Paintings,0.0,-1.0,False,False,,
Diary,Paintings,0.0,0.0,True,True,Genuine,Report #7213
Diary,Paintings,0.0,0.0,True,True,Genuine,
Diary,Paintings,0.0,0.0,True,True,Fake,Report #7214
Diary,Paintings,0.0,0.0,True,True,Fake,
Diary,Paintings,0.0,0.0,True,True,,Report #7215
Diary,Paintings,0.0,0.0,True,True,,
Diary,Paintings,0.0,0.0,True,False,Genuine,Report #7216
Diary,Paintings,0.0,0.0,True,False,Genuine,
Diary,Paintings,0.0,0.0,True,False,Fake,Report #7217
Diary,Paintings,0.0,0.0,True,False,Fake,
Diary,Paintings,0.0,0.0,True,False,,Report #7218
Diary,Paintings,0.0,0.0,True,False,,
Diary,Paintings,0.0,0.0,False,True,Genuine,Report #7219
Diary,Paintings,0.0,0.0,False,True,Genuine,
Diary,Paintings,0.0,0.0,False,True,Fake,Report #7220
Diary,Paintings,0.0,0.0,False,True,Fake,
Diary,Paintings,0.0,0.0,False,True,,Report #7221
Diary,Paintings,0.0,0.0,False,True,,
Diary,Paintings,0.0,0.0,False,False,Genuine,Report #7222
Diary,Paintings,0.0,0.0,False,False,Genuine,
Diary,Paintings,0.0,0.0,False,False,Fake,Report #7223
Diary,Paintings,0.0,0.0,False,False,Fake,
Diary,Paintings,0.0,0.0,False,False,,Report #7224
Diary,Paintings,0.0,0.0,False,False,,
Diary,Paintings,0.0,19.0,True,True,Genuine,Report #7225
Diary,Paintings,0.0,19.0,True,True,Genuine,
Diary,Paintings,0.0,19.0,True,True,Fake,Report #7226
Diary,Paintings,0.0,19.0,True,True,Fake,
Diary,Paintings,0.0,19.0,True,True,,Report #7227
Diary,Paintings,0.0,19.0,True,True,,
Diary,Paintings,0.0,19.0,True,False,Genuine,Report #7228
Diary,Paintings,0.0,19.0,True,False,Genuine,
Diary,Paintings,0.0,19.0,True,False,Fake,Report #7229
Diary,Paintings,0.0,19.0,True,False,Fake,
Diary,Paintings,0.0,19.0,True,False,,Report #7230
Diary,Paintings,0.0,19.0,True,False,,
Diary,Paintings,0.0,19.0,False,True,Genuine,Report #7231
Diary,Paintings,0.0,19.0,False,True,Genuine,
Diary,Paintings,0.0,19.0,False,True,Fake,Report #7232
Diary,Paintings,0.0,19.0,False,True,Fake,
Diary,Paintings,0.0,19.0,False,True,,Report #7233
Diary,Paintings,0.0,19.0,False,True,,
Diary,Paintings,0.0,19.0,False,False,Genuine,Report #7234
Diary,Paintings,0.0,19.0,False,False,Genuine,
Diary,Paintings,0.0,19.0,False,False,Fake,Report #7235
Diary,Paintings,0.0,19.0,False,False,Fake,
Diary,Paintings,0.0,19.0,False,False,,Report #7236
Diary,Paintings,0.0,19.0,False,False,,
Diary,Paintings,0.0,20.0,True,True,Genuine,Report #7237
Diary,Paintings,0.0,20.0,True,True,Genuine,
Diary,Paintings,0.0,20.0,True,True,Fake,Report #7238
Diary,Paintings,0.0,20.0,True,True,Fake,
Diary,Paintings,0.0,20.0,True,True,,Report #7239
Diary,Paintings,0.0,20.0,True,True,,
Diary,Paintings,0.0,20.0,True,False,Genuine,Report #7240
Diary,Paintings,0.0,20.0,True,False,Genuine,
Diary,Paintings,0.0,20.0,True,False,Fake,Report #7241
Diary,Paintings,0.0,20.0,True,False,Fake,
Diary,Paintings,0.0,20.0,True,False,,Report #7242
Diary,Paintings,0.0,20.0,True,False,,
Diary,Paintings,0.0,20.0,False,True,Genuine,Report #7243
Diary,Paintings,0.0,20.0,False,True,Genuine,
Diary,Paintings,0.0,20.0,False,True,Fake,Report #7244
Diary,Paintings,0.0,20.0,False,True,Fake,
Diary,Paintings,0.0,20.0,False,True,,Report #7245
Diary,Paintings,0.0,20.0,False,True,,
Diary,Paintings,0.0,20.0,False,False,Genuine,Report #7246
Diary,Paintings,0.0,20.0,False,False,Genuine,
Diary,Paintings,0.0,20.0,False,False,Fake,Report #7247
Diary,Paintings,0.0,20.0,False,False,Fake,
Diary,Paintings,0.0,20.0,False,False,,Report #7248
Diary,Paintings,0.0,20.0,False,False,,
Diary,Paintings,0.0,21.0,True,True,Genuine,Report #7249
Diary,Paintings,0.0,21.0,True,True,Genuine,
Diary,Paintings,0.0,21.0,True,True,Fake,Report #7250
Diary,Paintings,0.0,21.0,True,True,Fake,
Diary,Paintings,0.0,21.0,True,True,,Report #7251
Diary,Paintings,0.0,21.0,True,True,,
Diary,Paintings,0.0,21.0,True,False,Genuine,Report #7252
Diary,Paintings,0.0,21.0,True,False,Genuine,
Diary,Paintings,0.0,21.0,True,False,Fake,Report #7253
Diary,Paintings,0.0,21.0,True,False,Fake,
Diary,Paintings,0.0,21.0,True,False,,Report #7254
Diary,Paintings,0.0,21.0,True,False,,
Diary,Paintings,0.0,21.0,False,True,Genuine,Report #7255
Diary,Paintings,0.0,21.0,False,True,Genuine,
Diary,Paintings,0.0,21.0,False,True,Fake,Report #7256
Diary,Paintings,0.0,21.0,False,True,Fake,
Diary,Paintings,0.0,21.0,False,True,,Report #7257
Diary,Paintings,0.0,21.0,False,True,,
Diary,Paintings,0.0,21.0,False,False,Genuine,Report #7258
Diary,Paintings,0.0,21.0,False,False,Genuine,
Diary,Paintings,0.0,21.0,False,False,Fake,Report #7259
Diary,Paintings,0.0,21.0,False,False,Fake,
Diary,Paintings,0.0,21.0,False,False,,Report #7260
Diary,Paintings,0.0,21.0,False,False,,
Diary,Paintings,0.0,99.0,True,True,Genuine,Report #7261
Diary,Paintings,0.0,99.0,True,True,Genuine,
Diary,Paintings,0.0,99.0,True,True,Fake,Report #7262
Diary,Paintings,0.0,99.0,True,True,Fake,
Diary,Paintings,0.0,99.0,True,True,,Report #7263
Diary,Paintings,0.0,99.0,True,True,,
Diary,Paintings,0.0,99.0,True,False,Genuine,Report #7264
Diary,Paintings,0.0,99.0,True,False,Genuine,
Diary,Paintings,0.0,99.0,True,False,Fake,Report #7265
Diary,Paintings,0.0,99.0,True,False,Fake,
Diary,Paintings,0.0,99.0,True,False,,Report #7266
Diary,Paintings,0.0,99.0,True,False,,
Diary,Paintings,0.0,99.0,False,True,Genuine,Report #7267
Diary,Paintings,0.0,99.0,False,True,Genuine,
Diary,Paintings,0.0,99.0,False,True,Fake,Report #7268
Diary,Paintings,0.0,99.0,False,True,Fake,
Diary,Paintings,0.0,99.0,False,True,,Report #7269
Diary,Paintings,0.0,99.0,False,True,,
Diary,Paintings,0.0,99.0,False,False,Genuine,Report #7270
Diary,Paintings,0.0,99.0,False,False,Genuine,
Diary,Paintings,0.0,99.0,False,False,Fake,Report #7271
Diary,Paintings,0.0,99.0,False,False,Fake,
Diary,Paintings,0.0,99.0,False,False,,Report #7272
Diary,Paintings,0.0,99.0,False,False,,
Diary,Paintings,0.0,100.0,True,True,Genuine,Report #7273
Diary,Paintings,0.0,100.0,True,True,Genuine,
Diary,Paintings,0.0,100.0,True,True,Fake,Report #7274
Diary,Paintings,0.0,100.0,True,True,Fake,
Diary,Paintings,0.0,100.0,True,True,,Report #7275
Diary,Paintings,0.0,100.0,True,True,,
Diary,Paintings,0.0,100.0,True,False,Genuine,Report #7276
Diary,Paintings,0.0,100.0,True,False,Genuine,
Diary,Paintings,0.0,100.0,True,False,Fake,Report #7277
Diary,Paintings,0.0,100.0,True,False,Fake,
Diary,Paintings,0.0,100.0,True,False,,Report #7278
Diary,Paintings,0.0,100.0,True,False,,
Diary,Paintings,0.0,100.0,False,True,Genuine,Report #7279
Diary,Paintings,0.0,100.0,False,True,Genuine,
Diary,Paintings,0.0,100.0,False,True,Fake,Report #7280
Diary,Paintings,0.0,100.0,False,True,Fake,
Diary,Paintings,0.0,100.0,False,True,,Report #7281
Diary,Paintings,0.0,100.0,False,True,,
Diary,Paintings,0.0,100.0,False,False,Genuine,Report #7282
Diary,Paintings,0.0,100.0,False,False,Genuine,
Diary,Paintings,0.0,100.0,False,False,Fake,Report #7283
Diary,Paintings,0.0,100.0,False,False,Fake,
Diary,Paintings,0.0,100.0,False,False,,Report #7284
Diary,Paintings,0.0,100.0,False,False,,
Diary,Paintings,0.0,101.0,True,True,Genuine,Report #7285
Diary,Paintings,0.0,101.0,True,True,Genuine,
Diary,Paintings,0.0,101.0,True,True,Fake,Report #7286
Diary,Paintings,0.0,101.0,True,True,Fake,
Diary,Paintings,0.0,101.0,True,True,,Report #7287
Diary,Paintings,0.0,101.0,True,True,,
Diary,Paintings,0.0,101.0,True,False,Genuine,Report #7288
Diary,Paintings,0.0,101.0,True,False,Genuine,
Diary,Paintings,0.0,101.0,True,False,Fake,Report #7289
Diary,Paintings,0.0,101.0,True,False,Fake,
Diary,Paintings,0.0,101.0,True,False,,Report #7290
Diary,Paintings,0.0,101.0,True,False,,
Diary,Paintings,0.0,101.0,False,True,Genuine,Report #7291
Diary,Paintings,0.0,101.0,False,True,Genuine,
Diary,Paintings,0.0,101.0,False,True,Fake,Report #7292
Diary,Paintings,0.0,101.0,False,True,Fake,
Diary,Paintings,0.0,101.0,False,True,,Report #7293
Diary,Paintings,0.0,101.0,False,True,,
Diary,Paintings,0.0,101.0,False,False,Genuine,Report #7294
Diary,Paintings,0.0,101.0,False,False,Genuine,
Diary,Paintings,0.0,101.0,False,False,Fake,Report #7295
Diary,Paintings,0.0,101.0,False,False,Fake,
Diary,Paintings,0.0,101.0,False,False,,Report #7296
Diary,Paintings,0.0,101.0,False,False,,
Diary,Paintings,1.0,-1.0,True,True,Genuine,Report #7297
Diary,Paintings,1.0,-1.0,True,True,Genuine,
Diary,Paintings,1.0,-1.0,True,True,Fake,Report #7298
Diary,Paintings,1.0,-1.0,True,True,Fake,
Diary,Paintings,1.0,-1.0,True,True,,Report #7299
Diary,Paintings,1.0,-1.0,True,True,,
Diary,Paintings,1.0,-1.0,True,False,Genuine,Report #7300
Diary,Paintings,1.0,-1.0,True,False,Genuine,
Diary,Paintings,1.0,-1.0,True,False,Fake,Report #7301
Diary,Paintings,1.0,-1.0,True,False,Fake,
Diary,Paintings,1.0,-1.0,True,False,,Report #7302
Diary,Paintings,1.0,-1.0,True,False,,
Diary,Paintings,1.0,-1.0,False,True,Genuine,Report #7303
Diary,Paintings,1.0,-1.0,False,True,Genuine,
Diary,Paintings,1.0,-1.0,False,True,Fake,Report #7304
Diary,Paintings,1.0,-1.0,False,True,Fake,
Diary,Paintings,1.0,-1.0,False,True,,Report #7305
Diary,Paintings,1.0,-1.0,False,True,,
Diary,Paintings,1.0,-1.0,False,False,Genuine,Report #7306
Diary,Paintings,1.0,-1.0,False,False,Genuine,
Diary,Paintings,1.0,-1.0,False,False,Fake,Report #7307
Diary,Paintings,1.0,-1.0,False,False,Fake,
Diary,Paintings,1.0,-1.0,False,False,,Report #7308
Diary,Paintings,1.0,-1.0,False,False,,
Diary,Paintings,1.0,0.0,True,True,Genuine,Report #7309
Diary,Paintings,1.0,0.0,True,True,Genuine,
Diary,Paintings,1.0,0.0,True,True,Fake,Report #7310
Diary,Paintings,1.0,0.0,True,True,Fake,
Diary,Paintings,1.0,0.0,True,True,,Report #7311
Diary,Paintings,1.0,0.0,True,True,,
Diary,Paintings,1.0,0.0,True,False,Genuine,Report #7312
Diary,Paintings,1.0,0.0,True,False,Genuine,
Diary,Paintings,1.0,0.0,True,False,Fake,Report #7313
Diary,Paintings,1.0,0.0,True,False,Fake,
Diary,Paintings,1.0,0.0,True,False,,Report #7314
Diary,Paintings,1.0,0.0,True,False,,
Diary,Paintings,1.0,0.0,False,True,Genuine,Report #7315
Diary,Paintings,1.0,0.0,False,True,Genuine,
Diary,Paintings,1.0,0.0,False,True,Fake,Report #7316
Diary,Paintings,1.0,0.0,False,True,Fake,
Diary,Paintings,1.0,0.0,False,True,,Report #7317
Diary,Paintings,1.0,0.0,False,True,,
Diary,Paintings,1.0,0.0,False,False,Genuine,Report #7318
Diary,Paintings,1.0,0.0,False,False,Genuine,
Diary,Paintings,1.0,0.0,False,False,Fake,Report #7319
Diary,Paintings,1.0,0.0,False,False,Fake,
Diary,Paintings,1.0,0.0,False,False,,Report #7320
Diary,Paintings,1.0,0.0,False,False,,
Diary,Paintings,1.0,19.0,True,True,Genuine,Report #7321
Diary,Paintings,1.0,19.0,True,True,Genuine,
Diary,Paintings,1.0,19.0,True,True,Fake,Report #7322
Diary,Paintings,1.0,19.0,True,True,Fake,
Diary,Paintings,1.0,19.0,True,True,,Report #7323
Diary,Paintings,1.0,19.0,True,True,,
Diary,Paintings,1.0,19.0,True,False,Genuine,Report #7324
Diary,Paintings,1.0,19.0,True,False,Genuine,
Diary,Paintings,1.0,19.0,True,False,Fake,Report #7325
Diary,Paintings,1.0,19.0,True,False,Fake,
Diary,Paintings,1.0,19.0,True,False,,Report #7326
Diary,Paintings,1.0,19.0,True,False,,
Diary,Paintings,1.0,19.0,False,True,Genuine,Report #7327
Diary,Paintings,1.0,19.0,False,True,Genuine,
Diary,Paintings,1.0,19.0,False,True,Fake,Report #7328
Diary,Paintings,1.0,19.0,False,True,Fake,
Diary,Paintings,1.0,19.0,False,True,,Report #7329
Diary,Paintings,1.0,19.0,False,True,,
Diary,Paintings,1.0,19.0,False,False,Genuine,Report #7330
Diary,Paintings,1.0,19.0,False,False,Genuine,
Diary,Paintings,1.0,19.0,False,False,Fake,Report #7331
Diary,Paintings,1.0,19.0,False,False,Fake,
Diary,Paintings,1.0,19.0,False,False,,Report #7332
Diary,Paintings,1.0,19.0,False,False,,
Diary,Paintings,1.0,20.0,True,True,Genuine,Report #7333
Diary,Paintings,1.0,20.0,True,True,Genuine,
Diary,Paintings,1.0,20.0,True,True,Fake,Report #7334
Diary,Paintings,1.0,20.0,True,True,Fake,
Diary,Paintings,1.0,20.0,True,True,,Report #7335
Diary,Paintings,1.0,20.0,True,True,,
Diary,Paintings,1.0,20.0,True,False,Genuine,Report #7336
Diary,Paintings,1.0,20.0,True,False,Genuine,
Diary,Paintings,1.0,20.0,True,False,Fake,Report #7337
Diary,Paintings,1.0,20.0,True,False,Fake,
Diary,Paintings,1.0,20.0,True,False,,Report #7338
Diary,Paintings,1.0,20.0,True,False,,
Diary,Paintings,1.0,20.0,False,True,Genuine,Report #7339
Diary,Paintings,1.0,20.0,False,True,Genuine,
Diary,Paintings,1.0,20.0,False,True,Fake,Report #7340
Diary,Paintings,1.0,20.0,False,True,Fake,
Diary,Paintings,1.0,20.0,False,True,,Report #7341
Diary,Paintings,1.0,20.0,False,True,,
Diary,Paintings,1.0,20.0,False,False,Genuine,Report #7342
Diary,Paintings,1.0,20.0,False,False,Genuine,
Diary,Paintings,1.0,20.0,False,False,Fake,Report #7343
Diary,Paintings,1.0,20.0,False,False,Fake,
Diary,Paintings,1.0,20.0,False,False,,Report #7344
Diary,Paintings,1.0,20.0,False,False,,
Diary,Paintings,1.0,21.0,True,True,Genuine,Report #7345
Diary,Paintings,1.0,21.0,True,True,Genuine,
Diary,Paintings,1.0,21.0,True,True,Fake,Report #7346
Diary,Paintings,1.0,21.0,True,True,Fake,
Diary,Paintings,1.0,21.0,True,True,,Report #7347
Diary,Paintings,1.0,21.0,True,True,,
Diary,Paintings,1.0,21.0,True,False,Genuine,Report #7348
Diary,Paintings,1.0,21.0,True,False,Genuine,
Diary,Paintings,1.0,21.0,True,False,Fake,Report #7349
Diary,Paintings,1.0,21.0,True,False,Fake,
Diary,Paintings,1.0,21.0,True,False,,Report #7350
Diary,Paintings,1.0,21.0,True,False,,
Diary,Paintings,1.0,21.0,False,True,Genuine,Report #7351
Diary,Paintings,1.0,21.0,False,True,Genuine,
Diary,Paintings,1.0,21.0,False,True,Fake,Report #7352
Diary,Paintings,1.0,21.0,False,True,Fake,
Diary,Paintings,1.0,21.0,False,True,,Report #7353
Diary,Paintings,1.0,21.0,False,True,,
Diary,Paintings,1.0,21.0,False,False,Genuine,Report #7354
Diary,Paintings,1.0,21.0,False,False,Genuine,
Diary,Paintings,1.0,21.0,False,False,Fake,Report #7355
Diary,Paintings,1.0,21.0,False,False,Fake,
Diary,Paintings,1.0,21.0,False,False,,Report #7356
Diary,Paintings,1.0,21.0,False,False,,
Diary,Paintings,1.0,99.0,True,True,Genuine,Report #7357
Diary,Paintings,1.0,99.0,True,True,Genuine,
Diary,Paintings,1.0,99.0,True,True,Fake,Report #7358
Diary,Paintings,1.0,99.0,True,True,Fake,
Diary,Paintings,1.0,99.0,True,True,,Report #7359
Diary,Paintings,1.0,99.0,True,True,,
Diary,Paintings,1.0,99.0,True,False,Genuine,Report #7360
Diary,Paintings,1.0,99.0,True,False,Genuine,
Diary,Paintings,1.0,99.0,True,False,Fake,Report #7361
Diary,Paintings,1.0,99.0,True,False,Fake,
Diary,Paintings,1.0,99.0,True,False,,Report #7362
Diary,Paintings,1.0,99.0,True,False,,
Diary,Paintings,1.0,99.0,False,True,Genuine,Report #7363
Diary,Paintings,1.0,99.0,False,True,Genuine,
Diary,Paintings,1.0,99.0,False,True,Fake,Report #7364
Diary,Paintings,1.0,99.0,False,True,Fake,
Diary,Paintings,1.0,99.0,False,True,,Report #7365
Diary,Paintings,1.0,99.0,False,True,,
Diary,Paintings,1.0,99.0,False,False,Genuine,Report #7366
Diary,Paintings,1.0,99.0,False,False,Genuine,
Diary,Paintings,1.0,99.0,False,False,Fake,Report #7367
Diary,Paintings,1.0,99.0,False,False,Fake,
Diary,Paintings,1.0,99.0,False,False,,Report #7368
Diary,Paintings,1.0,99.0,False,False,,
Diary,Paintings,1.0,100.0,True,True,Genuine,Report #7369
Diary,Paintings,1.0,100.0,True,True,Genuine,
Diary,Paintings,1.0,100.0,True,True,Fake,Report #7370
Diary,Paintings,1.0,100.0,True,True,Fake,
Diary,Paintings,1.0,100.0,True,True,,Report #7371
Diary,Paintings,1.0,100.0,True,True,,
Diary,Paintings,1.0,100.0,True,False,Genuine,Report #7372
Diary,Paintings,1.0,100.0,True,False,Genuine,
Diary,Paintings,1.0,100.0,True,False,Fake,Report #7373
Diary,Paintings,1.0,100.0,True,False,Fake,
Diary,Paintings,1.0,100.0,True,False,,Report #7374
Diary,Paintings,1.0,100.0,True,False,,
Diary,Paintings,1.0,100.0,False,True,Genuine,Report #7375
Diary,Paintings,1.0,100.0,False,True,Genuine,
Diary,Paintings,1.0,100.0,False,True,Fake,Report #7376
Diary,Paintings,1.0,100.0,False,True,Fake,
Diary,Paintings,1.0,100.0,False,True,,Report #7377
Diary,Paintings,1.0,100.0,False,True,,
Diary,Paintings,1.0,100.0,False,False,Genuine,Report #7378
Diary,Paintings,1.0,100.0,False,False,Genuine,
Diary,Paintings,1.0,100.0,False,False,Fake,Report #7379
Diary,Paintings,1.0,100.0,False,False,Fake,
Diary,Paintings,1.0,100.0,False,False,,Report #7380
Diary,Paintings,1.0,100.0,False,False,,
Diary,Paintings,1.0,101.0,True,True,Genuine,Report #7381
Diary,Paintings,1.0,101.0,True,True,Genuine,
Diary,Paintings,1.0,101.0,True,True,Fake,Report #7382
Diary,Paintings,1.0,101.0,True,True,Fake,
Diary,Paintings,1.0,101.0,True,True,,Report #7383
Diary,Paintings,1.0,101.0,True,True,,
Diary,Paintings,1.0,101.0,True,False,Genuine,Report #7384
Diary,Paintings,1.0,101.0,True,False,Genuine,
Diary,Paintings,1.0,101.0,True,False,Fake,Report #7385
Diary,Paintings,1.0,101.0,True,False,Fake,
Diary,Paintings,1.0,101.0,True,False,,Report #7386
Diary,Paintings,1.0,101.0,True,False,,
Diary,Paintings,1.0,101.0,False,True,Genuine,Report #7387
Diary,Paintings,1.0,101.0,False,True,Genuine,
Diary,Paintings,1.0,101.0,False,True,Fake,Report #7388
Diary,Paintings,1.0,101.0,False,True,Fake,
Diary,Paintings,1.0,101.0,False,True,,Report #7389
Diary,Paintings,1.0,101.0,False,True,,
Diary,Paintings,1.0,101.0,False,False,Genuine,Report #7390
Diary,Paintings,1.0,101.0,False,False,Genuine,
Diary,Paintings,1.0,101.0,False,False,Fake,Report #7391
Diary,Paintings,1.0,101.0,False,False,Fake,
Diary,Paintings,1.0,101.0,False,False,,Report #7392
Diary,Paintings,1.0,101.0,False,False,,
Diary,Paintings,100.0,-1.0,True,True,Genuine,Report #7393
Diary,Paintings,100.0,-1.0,True,True,Genuine,
Diary,Paintings,100.0,-1.0,True,True,Fake,Report #7394
Diary,Paintings,100.0,-1.0,True,True,Fake,
Diary,Paintings,100.0,-1.0,True,True,,Report #7395
Diary,Paintings,100.0,-1.0,True,True,,
Diary,Paintings,100.0,-1.0,True,False,Genuine,Report #7396
Diary,Paintings,100.0,-1.0,True,False,Genuine,
Diary,Paintings,100.0,-1.0,True,False,Fake,Report #7397
Diary,Paintings,100.0,-1.0,True,False,Fake,
Diary,Paintings,100.0,-1.0,True,False,,Report #7398
Diary,Paintings,100.0,-1.0,True,False,,
Diary,Paintings,100.0,-1.0,False,True,Genuine,Report #7399
Diary,Paintings,100.0,-1.0,False,True,Genuine,
Diary,Paintings,100.0,-1.0,False,True,Fake,Report #7400
Diary,Paintings,100.0,-1.0,False,True,Fake,
Diary,Paintings,100.0,-1.0,False,True,,Report #7401
Diary,Paintings,100.0,-1.0,False,True,,
Diary,Paintings,100.0,-1.0,False,False,Genuine,Report #7402
Diary,Paintings,100.0,-1.0,False,False,Genuine,
Diary,Paintings,100.0,-1.0,False,False,Fake,Report #7403
Diary,Paintings,100.0,-1.0,False,False,Fake,
Diary,Paintings,100.0,-1.0,False,False,,Report #7404
Diary,Paintings,100.0,-1.0,False,False,,
Diary,Paintings,100.0,0.0,True,True,Genuine,Report #7405
Diary,Paintings,100.0,0.0,True,True,Genuine,
Diary,Paintings,100.0,0.0,True,True,Fake,Report #7406
Diary,Paintings,100.0,0.0,True,True,Fake,
Diary,Paintings,100.0,0.0,True,True,,Report #7407
Diary,Paintings,100.0,0.0,True,True,,
Diary,Paintings,100.0,0.0,True,False,Genuine,Report #7408
Diary,Paintings,100.0,0.0,True,False,Genuine,
Diary,Paintings,100.0,0.0,True,False,Fake,Report #7409
Diary,Paintings,100.0,0.0,True,False,Fake,
Diary,Paintings,100.0,0.0,True,False,,Report #7410
Diary,Paintings,100.0,0.0,True,False,,
Diary,Paintings,100.0,0.0,False,True,Genuine,Report #7411
Diary,Paintings,100.0,0.0,False,True,Genuine,
Diary,Paintings,100.0,0.0,False,True,Fake,Report #7412
Diary,Paintings,100.0,0.0,False,True,Fake,
Diary,Paintings,100.0,0.0,False,True,,Report #7413
Diary,Paintings,100.0,0.0,False,True,,
Diary,Paintings,100.0,0.0,False,False,Genuine,Report #7414
Diary,Paintings,100.0,0.0,False,False,Genuine,
Diary,Paintings,100.0,0.0,False,False,Fake,Report #7415
Diary,Paintings,100.0,0.0,False,False,Fake,
Diary,Paintings,100.0,0.0,False,False,,Report #7416
Diary,Paintings,100.0,0.0,False,False,,
Diary,Paintings,100.0,19.0,True,True,Genuine,Report #7417
Diary,Paintings,100.0,19.0,True,True,Genuine,
Diary,Paintings,100.0,19.0,True,True,Fake,Report #7418
Diary,Paintings,100.0,19.0,True,True,Fake,
Diary,Paintings,100.0,19.0,True,True,,Report #7419
Diary,Paintings,100.0,19.0,True,True,,
Diary,Paintings,100.0,19.0,True,False,Genuine,Report #7420
Diary,Paintings,100.0,19.0,True,False,Genuine,
Diary,Paintings,100.0,19.0,True,False,Fake,Report #7421
Diary,Paintings,100.0,19.0,True,False,Fake,
Diary,Paintings,100.0,19.0,True,False,,Report #7422
Diary,Paintings,100.0,19.0,True,False,,
Diary,Paintings,100.0,19.0,False,True,Genuine,Report #7423
Diary,Paintings,100.0,19.0,False,True,Genuine,
Diary,Paintings,100.0,19.0,False,True,Fake,Report #7424
Diary,Paintings,100.0,19.0,False,True,Fake,
Diary,Paintings,100.0,19.0,False,True,,Report #7425
Diary,Paintings,100.0,19.0,False,True,,
Diary,Paintings,100.0,19.0,False,False,Genuine,Report #7426
Diary,Paintings,100.0,19.0,False,False,Genuine,
Diary,Paintings,100.0,19.0,False,False,Fake,Report #7427
Diary,Paintings,100.0,19.0,False,False,Fake,
Diary,Paintings,100.0,19.0,False,False,,Report #7428
Diary,Paintings,100.0,19.0,False,False,,
Diary,Paintings,100.0,20.0,True,True,Genuine,Report #7429
Diary,Paintings,100.0,20.0,True,True,Genuine,
Diary,Paintings,100.0,20.0,True,True,Fake,Report #7430
Diary,Paintings,100.0,20.0,True,True,Fake,
Diary,Paintings,100.0,20.0,True,True,,Report #7431
Diary,Paintings,100.0,20.0,True,True,,
Diary,Paintings,100.0,20.0,True,False,Genuine,Report #7432
Diary,Paintings,100.0,20.0,True,False,Genuine,
Diary,Paintings,100.0,20.0,True,False,Fake,Report #7433
Diary,Paintings,100.0,20.0,True,False,Fake,
Diary,Paintings,100.0,20.0,True,False,,Report #7434
Diary,Paintings,100.0,20.0,True,False,,
Diary,Paintings,100.0,20.0,False,True,Genuine,Report #7435
Diary,Paintings,100.0,20.0,False,True,Genuine,
Diary,Paintings,100.0,20.0,False,True,Fake,Report #7436
Diary,Paintings,100.0,20.0,False,True,Fake,
Diary,Paintings,100.0,20.0,False,True,,Report #7437
Diary,Paintings,100.0,20.0,False,True,,
Diary,Paintings,100.0,20.0,False,False,Genuine,Report #7438
Diary,Paintings,100.0,20.0,False,False,Genuine,
Diary,Paintings,100.0,20.0,False,False,Fake,Report #7439
Diary,Paintings,100.0,20.0,False,False,Fake,
Diary,Paintings,100.0,20.0,False,False,,Report #7440
Diary,Paintings,100.0,20.0,False,False,,
Diary,Paintings,100.0,21.0,True,True,Genuine,Report #7441
Diary,Paintings,100.0,21.0,True,True,Genuine,
Diary,Paintings,100.0,21.0,True,True,Fake,Report #7442
Diary,Paintings,100.0,21.0,True,True,Fake,
Diary,Paintings,100.0,21.0,True,True,,Report #7443
Diary,Paintings,100.0,21.0,True,True,,
Diary,Paintings,100.0,21.0,True,False,Genuine,Report #7444
Diary,Paintings,100.0,21.0,True,False,Genuine,
Diary,Paintings,100.0,21.0,True,False,Fake,Report #7445
Diary,Paintings,100.0,21.0,True,False,Fake,
Diary,Paintings,100.0,21.0,True,False,,Report #7446
Diary,Paintings,100.0,21.0,True,False,,
Diary,Paintings,100.0,21.0,False,True,Genuine,Report #7447
Diary,Paintings,100.0,21.0,False,True,Genuine,
Diary,Paintings,100.0,21.0,False,True,Fake,Report #7448
Diary,Paintings,100.0,21.0,False,True,Fake,
Diary,Paintings,100.0,21.0,False,True,,Report #7449
Diary,Paintings,100.0,21.0,False,True,,
Diary,Paintings,100.0,21.0,False,False,Genuine,Report #7450
Diary,Paintings,100.0,21.0,False,False,Genuine,
Diary,Paintings,100.0,21.0,False,False,Fake,Report #7451
Diary,Paintings,100.0,21.0,False,False,Fake,
Diary,Paintings,100.0,21.0,False,False,,Report #7452
Diary,Paintings,100.0,21.0,False,False,,
Diary,Paintings,100.0,99.0,True,True,Genuine,Report #7453
Diary,Paintings,100.0,99.0,True,True,Genuine,
Diary,Paintings,100.0,99.0,True,True,Fake,Report #7454
Diary,Paintings,100.0,99.0,True,True,Fake,
Diary,Paintings,100.0,99.0,True,True,,Report #7455
Diary,Paintings,100.0,99.0,True,True,,
Diary,Paintings,100.0,99.0,True,False,Genuine,Report #7456
Diary,Paintings,100.0,99.0,True,False,Genuine,
Diary,Paintings,100.0,99.0,True,False,Fake,Report #7457
Diary,Paintings,100.0,99.0,True,False,Fake,
Diary,Paintings,100.0,99.0,True,False,,Report #7458
Diary,Paintings,100.0,99.0,True,False,,
Diary,Paintings,100.0,99.0,False,True,Genuine,Report #7459
Diary,Paintings,100.0,99.0,False,True,Genuine,
Diary,Paintings,100.0,99.0,False,True,Fake,Report #7460
Diary,Paintings,100.0,99.0,False,True,Fake,
Diary,Paintings,100.0,99.0,False,True,,Report #7461
Diary,Paintings,100.0,99.0,False,True,,
Diary,Paintings,100.0,99.0,False,False,Genuine,Report #7462
Diary,Paintings,100.0,99.0,False,False,Genuine,
Diary,Paintings,100.0,99.0,False,False,Fake,Report #7463
Diary,Paintings,100.0,99.0,False,False,Fake,
Diary,Paintings,100.0,99.0,False,False,,Report #7464
Diary,Paintings,100.0,99.0,False,False,,
Diary,Paintings,100.0,100.0,True,True,Genuine,Report #7465
Diary,Paintings,100.0,100.0,True,True,Genuine,
Diary,Paintings,100.0,100.0,True,True,Fake,Report #7466
Diary,Paintings,100.0,100.0,True,True,Fake,
Diary,Paintings,100.0,100.0,True,True,,Report #7467
Diary,Paintings,100.0,100.0,True,True,,
Diary,Paintings,100.0,100.0,True,False,Genuine,Report #7468
Diary,Paintings,100.0,100.0,True,False,Genuine,
Diary,Paintings,100.0,100.0,True,False,Fake,Report #7469
Diary,Paintings,100.0,100.0,True,False,Fake,
Diary,Paintings,100.0,100.0,True,False,,Report #7470
Diary,Paintings,100.0,100.0,True,False,,
Diary,Paintings,100.0,100.0,False,True,Genuine,Report #7471
Diary,Paintings,100.0,100.0,False,True,Genuine,
Diary,Paintings,100.0,100.0,False,True,Fake,Report #7472
Diary,Paintings,100.0,100.0,False,True,Fake,
Diary,Paintings,100.0,100.0,False,True,,Report #7473
Diary,Paintings,100.0,100.0,False,True,,
Diary,Paintings,100.0,100.0,False,False,Genuine,Report #7474
Diary,Paintings,100.0,100.0,False,False,Genuine,
Diary,Paintings,100.0,100.0,False,False,Fake,Report #7475
Diary,Paintings,100.0,100.0,False,False,Fake,
Diary,Paintings,100.0,100.0,False,False,,Report #7476
Diary,Paintings,100.0,100.0,False,False,,
Diary,Paintings,100.0,101.0,True,True,Genuine,Report #7477
Diary,Paintings,100.0,101.0,True,True,Genuine,
Diary,Paintings,100.0,101.0,True,True,Fake,Report #7478
Diary,Paintings,100.0,101.0,True,True,Fake,
Diary,Paintings,100.0,101.0,True,True,,Report #7479
Diary,Paintings,100.0,101.0,True,True,,
Diary,Paintings,100.0,101.0,True,False,Genuine,Report #7480
Diary,Paintings,100.0,101.0,True,False,Genuine,
Diary,Paintings,100.0,101.0,True,False,Fake,Report #7481
Diary,Paintings,100.0,101.0,True,False,Fake,
Diary,Paintings,100.0,101.0,True,False,,Report #7482
Diary,Paintings,100.0,101.0,True,False,,
Diary,Paintings,100.0,101.0,False,True,Genuine,Report #7483
Diary,Paintings,100.0,101.0,False,True,Genuine,
Diary,Paintings,100.0,101.0,False,True,Fake,Report #7484
Diary,Paintings,100.0,101.0,False,True,Fake,
Diary,Paintings,100.0,101.0,False,True,,Report #7485
Diary,Paintings,100.0,101.0,False,True,,
Diary,Paintings,100.0,101.0,False,False,Genuine,Report #7486
Diary,Paintings,100.0,101.0,False,False,Genuine,
Diary,Paintings,100.0,101.0,False,False,Fake,Report #7487
Diary,Paintings,100.0,101.0,False,False,Fake,
Diary,Paintings,100.0,101.0,False,False,,Report #7488
Diary,Paintings,100.0,101.0,False,False,,
Diary,Paintings,1000.0,-1.0,True,True,Genuine,Report #7489
Diary,Paintings,1000.0,-1.0,True,True,Genuine,
Diary,Paintings,1000.0,-1.0,True,True,Fake,Report #7490
Diary,Paintings,1000.0,-1.0,True,True,Fake,
Diary,Paintings,1000.0,-1.0,True,True,,Report #7491
Diary,Paintings,1000.0,-1.0,True,True,,
Diary,Paintings,1000.0,-1.0,True,False,Genuine,Report #7492
Diary,Paintings,1000.0,-1.0,True,False,Genuine,
Diary,Paintings,1000.0,-1.0,True,False,Fake,Report #7493
Diary,Paintings,1000.0,-1.0,True,False,Fake,
Diary,Paintings,1000.0,-1.0,True,False,,Report #7494
Diary,Paintings,1000.0,-1.0,True,False,,
Diary,Paintings,1000.0,-1.0,False,True,Genuine,Report #7495
Diary,Paintings,1000.0,-1.0,False,True,Genuine,
Diary,Paintings,1000.0,-1.0,False,True,Fake,Report #7496
Diary,Paintings,1000.0,-1.0,False,True,Fake,
Diary,Paintings,1000.0,-1.0,False,True,,Report #7497
Diary,Paintings,1000.0,-1.0,False,True,,
Diary,Paintings,1000.0,-1.0,False,False,Genuine,Report #7498
Diary,Paintings,1000.0,-1.0,False,False,Genuine,
Diary,Paintings,1000.0,-1.0,False,False,Fake,Report #7499
Diary,Paintings,1000.0,-1.0,False,False,Fake,
Diary,Paintings,1000.0,-1.0,False,False,,Report #7500
Diary,Paintings,1000.0,-1.0,False,False,,
Diary,Paintings,1000.0,0.0,True,True,Genuine,Report #7501
Diary,Paintings,1000.0,0.0,True,True,Genuine,
Diary,Paintings,1000.0,0.0,True,True,Fake,Report #7502
Diary,Paintings,1000.0,0.0,True,True,Fake,
Diary,Paintings,1000.0,0.0,True,True,,Report #7503
Diary,Paintings,1000.0,0.0,True,True,,
Diary,Paintings,1000.0,0.0,True,False,Genuine,Report #7504
Diary,Paintings,1000.0,0.0,True,False,Genuine,
Diary,Paintings,1000.0,0.0,True,False,Fake,Report #7505
Diary,Paintings,1000.0,0.0,True,False,Fake,
Diary,Paintings,1000.0,0.0,True,False,,Report #7506
Diary,Paintings,1000.0,0.0,True,False,,
Diary,Paintings,1000.0,0.0,False,True,Genuine,Report #7507
Diary,Paintings,1000.0,0.0,False,True,Genuine,
Diary,Paintings,1000.0,0.0,False,True,Fake,Report #7508
Diary,Paintings,1000.0,0.0,False,True,Fake,
Diary,Paintings,1000.0,0.0,False,True,,Report #7509
Diary,Paintings,1000.0,0.0,False,True,,
Diary,Paintings,1000.0,0.0,False,False,Genuine,Report #7510
Diary,Paintings,1000.0,0.0,False,False,Genuine,
Diary,Paintings,1000.0,0.0,False,False,Fake,Report #7511
Diary,Paintings,1000.0,0.0,False,False,Fake,
Diary,Paintings,1000.0,0.0,False,False,,Report #7512
Diary,Paintings,1000.0,0.0,False,False,,
Diary,Paintings,1000.0,19.0,True,True,Genuine,Report #7513
Diary,Paintings,1000.0,19.0,True,True,Genuine,
Diary,Paintings,1000.0,19.0,True,True,Fake,Report #7514
Diary,Paintings,1000.0,19.0,True,True,Fake,
Diary,Paintings,1000.0,19.0,True,True,,Report #7515
Diary,Paintings,1000.0,19.0,True,True,,
Diary,Paintings,1000.0,19.0,True,False,Genuine,Report #7516
Diary,Paintings,1000.0,19.0,True,False,Genuine,
Diary,Paintings,1000.0,19.0,True,False,Fake,Report #7517
Diary,Paintings,1000.0,19.0,True,False,Fake,
Diary,Paintings,1000.0,19.0,True,False,,Report #7518
Diary,Paintings,1000.0,19.0,True,False,,
Diary,Paintings,1000.0,19.0,False,True,Genuine,Report #7519
Diary,Paintings,1000.0,19.0,False,True,Genuine,
Diary,Paintings,1000.0,19.0,False,True,Fake,Report #7520
Diary,Paintings,1000.0,19.0,False,True,Fake,
Diary,Paintings,1000.0,19.0,False,True,,Report #7521
Diary,Paintings,1000.0,19.0,False,True,,
Diary,Paintings,1000.0,19.0,False,False,Genuine,Report #7522
Diary,Paintings,1000.0,19.0,False,False,Genuine,
Diary,Paintings,1000.0,19.0,False,False,Fake,Report #7523
Diary,Paintings,1000.0,19.0,False,False,Fake,
Diary,Paintings,1000.0,19.0,False,False,,Report #7524
Diary,Paintings,1000.0,19.0,False,False,,
Diary,Paintings,1000.0,20.0,True,True,Genuine,Report #7525
Diary,Paintings,1000.0,20.0,True,True,Genuine,
Diary,Paintings,1000.0,20.0,True,True,Fake,Report #7526
Diary,Paintings,1000.0,20.0,True,True,Fake,
Diary,Paintings,1000.0,20.0,True,True,,Report #7527
Diary,Paintings,1000.0,20.0,True,True,,
Diary,Paintings,1000.0,20.0,True,False,Genuine,Report #7528
Diary,Paintings,1000.0,20.0,True,False,Genuine,
Diary,Paintings,1000.0,20.0,True,False,Fake,Report #7529
Diary,Paintings,1000.0,20.0,True,False,Fake,
Diary,Paintings,1000.0,20.0,True,False,,Report #7530
Diary,Paintings,1000.0,20.0,True,False,,
Diary,Paintings,1000.0,20.0,False,True,Genuine,Report #7531
Diary,Paintings,1000.0,20.0,False,True,Genuine,
Diary,Paintings,1000.0,20.0,False,True,Fake,Report #7532
Diary,Paintings,1000.0,20.0,False,True,Fake,
Diary,Paintings,1000.0,20.0,False,True,,Report #7533
Diary,Paintings,1000.0,20.0,False,True,,
Diary,Paintings,1000.0,20.0,False,False,Genuine,Report #7534
Diary,Paintings,1000.0,20.0,False,False,Genuine,
Diary,Paintings,1000.0,20.0,False,False,Fake,Report #7535
Diary,Paintings,1000.0,20.0,False,False,Fake,
Diary,Paintings,1000.0,20.0,False,False,,Report #7536
Diary,Paintings,1000.0,20.0,False,False,,
Diary,Paintings,1000.0,21.0,True,True,Genuine,Report #7537
Diary,Paintings,1000.0,21.0,True,True,Genuine,
Diary,Paintings,1000.0,21.0,True,True,Fake,Report #7538
Diary,Paintings,1000.0,21.0,True,True,Fake,
Diary,Paintings,1000.0,21.0,True,True,,Report #7539
Diary,Paintings,1000.0,21.0,True,True,,
Diary,Paintings,1000.0,21.0,True,False,Genuine,Report #7540
Diary,Paintings,1000.0,21.0,True,False,Genuine,
Diary,Paintings,1000.0,21.0,True,False,Fake,Report #7541
Diary,Paintings,1000.0,21.0,True,False,Fake,
Diary,Paintings,1000.0,21.0,True,False,,Report #7542
Diary,Paintings,1000.0,21.0,True,False,,
Diary,Paintings,1000.0,21.0,False,True,Genuine,Report #7543
Diary,Paintings,1000.0,21.0,False,True,Genuine,
Diary,Paintings,1000.0,21.0,False,True,Fake,Report #7544
Diary,Paintings,1000.0,21.0,False,True,Fake,
Diary,Paintings,1000.0,21.0,False,True,,Report #7545
Diary,Paintings,1000.0,21.0,False,True,,
Diary,Paintings,1000.0,21.0,False,False,Genuine,Report #7546
Diary,Paintings,1000.0,21.0,False,False,Genuine,
Diary,Paintings,1000.0,21.0,False,False,Fake,Report #7547
Diary,Paintings,1000.0,21.0,False,False,Fake,
Diary,Paintings,1000.0,21.0,False,False,,Report #7548
Diary,Paintings,1000.0,21.0,False,False,,
Diary,Paintings,1000.0,99.0,True,True,Genuine,Report #7549
Diary,Paintings,1000.0,99.0,True,True,Genuine,
Diary,Paintings,1000.0,99.0,True,True,Fake,Report #7550
Diary,Paintings,1000.0,99.0,True,True,Fake,
Diary,Paintings,1000.0,99.0,True,True,,Report #7551
Diary,Paintings,1000.0,99.0,True,True,,
Diary,Paintings,1000.0,99.0,True,False,Genuine,Report #7552
Diary,Paintings,1000.0,99.0,True,False,Genuine,
Diary,Paintings,1000.0,99.0,True,False,Fake,Report #7553
Diary,Paintings,1000.0,99.0,True,False,Fake,
Diary,Paintings,1000.0,99.0,True,False,,Report #7554
Diary,Paintings,1000.0,99.0,True,False,,
Diary,Paintings,1000.0,99.0,False,True,Genuine,Report #7555
Diary,Paintings,1000.0,99.0,False,True,Genuine,
Diary,Paintings,1000.0,99.0,False,True,Fake,Report #7556
Diary,Paintings,1000.0,99.0,False,True,Fake,
Diary,Paintings,1000.0,99.0,False,True,,Report #7557
Diary,Paintings,1000.0,99.0,False,True,,
Diary,Paintings,1000.0,99.0,False,False,Genuine,Report #7558
Diary,Paintings,1000.0,99.0,False,False,Genuine,
Diary,Paintings,1000.0,99.0,False,False,Fake,Report #7559
Diary,Paintings,1000.0,99.0,False,False,Fake,
Diary,Paintings,1000.0,99.0,False,False,,Report #7560
Diary,Paintings,1000.0,99.0,False,False,,
Diary,Paintings,1000.0,100.0,True,True,Genuine,Report #7561
Diary,Paintings,1000.0,100.0,True,True,Genuine,
Diary,Paintings,1000.0,100.0,True,True,Fake,Report #7562
Diary,Paintings,1000.0,100.0,True,True,Fake,
Diary,Paintings,1000.0,100.0,True,True,,Report #7563
Diary,Paintings,1000.0,100.0,True,True,,
Diary,Paintings,1000.0,100.0,True,False,Genuine,Report #7564
Diary,Paintings,1000.0,100.0,True,False,Genuine,
Diary,Paintings,1000.0,100.0,True,False,Fake,Report #7565
Diary,Paintings,1000.0,100.0,True,False,Fake,
Diary,Paintings,1000.0,100.0,True,False,,Report #7566
Diary,Paintings,1000.0,100.0,True,False,,
Diary,Paintings,1000.0,100.0,False,True,Genuine,Report #7567
Diary,Paintings,1000.0,100.0,False,True,Genuine,
Diary,Paintings,1000.0,100.0,False,True,Fake,Report #7568
Diary,Paintings,1000.0,100.0,False,True,Fake,
Diary,Paintings,1000.0,100.0,False,True,,Report #7569
Diary,Paintings,1000.0,100.0,False,True,,
Diary,Paintings,1000.0,100.0,False,False,Genuine,Report #7570
Diary,Paintings,1000.0,100.0,False,False,Genuine,
Diary,Paintings,1000.0,100.0,False,False,Fake,Report #7571
Diary,Paintings,1000.0,100.0,False,False,Fake,
Diary,Paintings,1000.0,100.0,False,False,,Report #7572
Diary,Paintings,1000.0,100.0,False,False,,
Diary,Paintings,1000.0,101.0,True,True,Genuine,Report #7573
Diary,Paintings,1000.0,101.0,True,True,Genuine,
Diary,Paintings,1000.0,101.0,True,True,Fake,Report #7574
Diary,Paintings,1000.0,101.0,True,True,Fake,
Diary,Paintings,1000.0,101.0,True,True,,Report #7575
Diary,Paintings,1000.0,101.0,True,True,,
Diary,Paintings,1000.0,101.0,True,False,Genuine,Report #7576
Diary,Paintings,1000.0,101.0,True,False,Genuine,
Diary,Paintings,1000.0,101.0,True,False,Fake,Report #7577
Diary,Paintings,1000.0,101.0,True,False,Fake,
Diary,Paintings,1000.0,101.0,True,False,,Report #7578
Diary,Paintings,1000.0,101.0,True,False,,
Diary,Paintings,1000.0,101.0,False,True,Genuine,Report #7579
Diary,Paintings,1000.0,101.0,False,True,Genuine,
Diary,Paintings,1000.0,101.0,False,True,Fake,Report #7580
Diary,Paintings,1000.0,101.0,False,True,Fake,
Diary,Paintings,1000.0,101.0,False,True,,Report #7581
Diary,Paintings,1000.0,101.0,False,True,,
Diary,Paintings,1000.0,101.0,False,False,Genuine,Report #7582
Diary,Paintings,1000.0,101.0,False,False,Genuine,
Diary,Paintings,1000.0,101.0,False,False,Fake,Report #7583
Diary,Paintings,1000.0,101.0,False,False,Fake,
Diary,Paintings,1000.0,101.0,False,False,,Report #7584
Diary,Paintings,1000.0,101.0,False,False,,
Diary,Paintings,1000000.0,-1.0,True,True,Genuine,Report #7585
Diary,Paintings,1000000.0,-1.0,True,True,Genuine,
Diary,Paintings,1000000.0,-1.0,True,True,Fake,Report #7586
Diary,Paintings,1000000.0,-1.0,True,True,Fake,
Diary,Paintings,1000000.0,-1.0,True,True,,Report #7587
Diary,Paintings,1000000.0,-1.0,True,True,,
Diary,Paintings,1000000.0,-1.0,True,False,Genuine,Report #7588
Diary,Paintings,1000000.0,-1.0,True,False,Genuine,
Diary,Paintings,1000000.0,-1.0,True,False,Fake,Report #7589
Diary,Paintings,1000000.0,-1.0,True,False,Fake,
Diary,Paintings,1000000.0,-1.0,True,False,,Report #7590
Diary,Paintings,1000000.0,-1.0,True,False,,
Diary,Paintings,1000000.0,-1.0,False,True,Genuine,Report #7591
Diary,Paintings,1000000.0,-1.0,False,True,Genuine,
Diary,Paintings,1000000.0,-1.0,False,True,Fake,Report #7592
Diary,Paintings,1000000.0,-1.0,False,True,Fake,
Diary,Paintings,1000000.0,-1.0,False,True,,Report #7593
Diary,Paintings,1000000.0,-1.0,False,True,,
Diary,Paintings,1000000.0,-1.0,False,False,Genuine,Report #7594
Diary,Paintings,1000000.0,-1.0,False,False,Genuine,
Diary,Paintings,1000000.0,-1.0,False,False,Fake,Report #7595
Diary,Paintings,1000000.0,-1.0,False,False,Fake,
Diary,Paintings,1000000.0,-1.0,False,False,,Report #7596
Diary,Paintings,1000000.0,-1.0,False,False,,
Diary,Paintings,1000000.0,0.0,True,True,Genuine,Report #7597
Diary,Paintings,1000000.0,0.0,True,True,Genuine,
Diary,Paintings,1000000.0,0.0,True,True,Fake,Report #7598
Diary,Paintings,1000000.0,0.0,True,True,Fake,
Diary,Paintings,1000000.0,0.0,True,True,,Report #7599
Diary,Paintings,1000000.0,0.0,True,True,,
Diary,Paintings,1000000.0,0.0,True,False,Genuine,Report #7600
Diary,Paintings,1000000.0,0.0,True,False,Genuine,
Diary,Paintings,1000000.0,0.0,True,False,Fake,Report #7601
Diary,Paintings,1000000.0,0.0,True,False,Fake,
Diary,Paintings,1000000.0,0.0,True,False,,Report #7602
Diary,Paintings,1000000.0,0.0,True,False,,
Diary,Paintings,1000000.0,0.0,False,True,Genuine,Report #7603
Diary,Paintings,1000000.0,0.0,False,True,Genuine,
Diary,Paintings,1000000.0,0.0,False,True,Fake,Report #7604
Diary,Paintings,1000000.0,0.0,False,True,Fake,
Diary,Paintings,1000000.0,0.0,False,True,,Report #7605
Diary,Paintings,1000000.0,0.0,False,True,,
Diary,Paintings,1000000.0,0.0,False,False,Genuine,Report #7606
Diary,Paintings,1000000.0,0.0,False,False,Genuine,
Diary,Paintings,1000000.0,0.0,False,False,Fake,Report #7607
Diary,Paintings,1000000.0,0.0,False,False,Fake,
Diary,Paintings,1000000.0,0.0,False,False,,Report #7608
Diary,Paintings,1000000.0,0.0,False,False,,
Diary,Paintings,1000000.0,19.0,True,True,Genuine,Report #7609
Diary,Paintings,1000000.0,19.0,True,True,Genuine,
Diary,Paintings,1000000.0,19.0,True,True,Fake,Report #7610
Diary,Paintings,1000000.0,19.0,True,True,Fake,
Diary,Paintings,1000000.0,19.0,True,True,,Report #7611
Diary,Paintings,1000000.0,19.0,True,True,,
Diary,Paintings,1000000.0,19.0,True,False,Genuine,Report #7612
Diary,Paintings,1000000.0,19.0,True,False,Genuine,
Diary,Paintings,1000000.0,19.0,True,False,Fake,Report #7613
Diary,Paintings,1000000.0,19.0,True,False,Fake,
Diary,Paintings,1000000.0,19.0,True,False,,Report #7614
Diary,Paintings,1000000.0,19.0,True,False,,
Diary,Paintings,1000000.0,19.0,False,True,Genuine,Report #7615
Diary,Paintings,1000000.0,19.0,False,True,Genuine,
Diary,Paintings,1000000.0,19.0,False,True,Fake,Report #7616
Diary,Paintings,1000000.0,19.0,False,True,Fake,
Diary,Paintings,1000000.0,19.0,False,True,,Report #7617
Diary,Paintings,1000000.0,19.0,False,True,,
Diary,Paintings,1000000.0,19.0,False,False,Genuine,Report #7618
Diary,Paintings,1000000.0,19.0,False,False,Genuine,
Diary,Paintings,1000000.0,19.0,False,False,Fake,Report #7619
Diary,Paintings,1000000.0,19.0,False,False,Fake,
Diary,Paintings,1000000.0,19.0,False,False,,Report #7620
Diary,Paintings,1000000.0,19.0,False,False,,
Diary,Paintings,1000000.0,20.0,True,True,Genuine,Report #7621
Diary,Paintings,1000000.0,20.0,True,True,Genuine,
Diary,Paintings,1000000.0,20.0,True,True,Fake,Report #7622
Diary,Paintings,1000000.0,20.0,True,True,Fake,
Diary,Paintings,1000000.0,20.0,True,True,,Report #7623
Diary,Paintings,1000000.0,20.0,True,True,,
Diary,Paintings,1000000.0,20.0,True,False,Genuine,Report #7624
Diary,Paintings,1000000.0,20.0,True,False,Genuine,
Diary,Paintings,1000000.0,20.0,True,False,Fake,Report #7625
Diary,Paintings,1000000.0,20.0,True,False,Fake,
Diary,Paintings,1000000.0,20.0,True,False,,Report #7626
Diary,Paintings,1000000.0,20.0,True,False,,
Diary,Paintings,1000000.0,20.0,False,True,Genuine,Report #7627
Diary,Paintings,1000000.0,20.0,False,True,Genuine,
Diary,Paintings,1000000.0,20.0,False,True,Fake,Report #7628
Diary,Paintings,1000000.0,20.0,False,True,Fake,
Diary,Paintings,1000000.0,20.0,False,True,,Report #7629
Diary,Paintings,1000000.0,20.0,False,True,,
Diary,Paintings,1000000.0,20.0,False,False,Genuine,Report #7630
Diary,Paintings,1000000.0,20.0,False,False,Genuine,
Diary,Paintings,1000000.0,20.0,False,False,Fake,Report #7631
Diary,Paintings,1000000.0,20.0,False,False,Fake,
Diary,Paintings,1000000.0,20.0,False,False,,Report #7632
Diary,Paintings,1000000.0,20.0,False,False,,
Diary,Paintings,1000000.0,21.0,True,True,Genuine,Report #7633
Diary,Paintings,1000000.0,21.0,True,True,Genuine,
Diary,Paintings,1000000.0,21.0,True,True,Fake,Report #7634
Diary,Paintings,1000000.0,21.0,True,True,Fake,
Diary,Paintings,1000000.0,21.0,True,True,,Report #7635
Diary,Paintings,1000000.0,21.0,True,True,,
Diary,Paintings,1000000.0,21.0,True,False,Genuine,Report #7636
Diary,Paintings,1000000.0,21.0,True,False,Genuine,
Diary,Paintings,1000000.0,21.0,True,False,Fake,Report #7637
Diary,Paintings,1000000.0,21.0,True,False,Fake,
Diary,Paintings,1000000.0,21.0,True,False,,Report #7638
Diary,Paintings,1000000.0,21.0,True,False,,
Diary,Paintings,1000000.0,21.0,False,True,Genuine,Report #7639
Diary,Paintings,1000000.0,21.0,False,True,Genuine,
Diary,Paintings,1000000.0,21.0,False,True,Fake,Report #7640
Diary,Paintings,1000000.0,21.0,False,True,Fake,
Diary,Paintings,1000000.0,21.0,False,True,,Report #7641
Diary,Paintings,1000000.0,21.0,False,True,,
Diary,Paintings,1000000.0,21.0,False,False,Genuine,Report #7642
Diary,Paintings,1000000.0,21.0,False,False,Genuine,
Diary,Paintings,1000000.0,21.0,False,False,Fake,Report #7643
Diary,Paintings,1000000.0,21.0,False,False,Fake,
Diary,Paintings,1000000.0,21.0,False,False,,Report #7644
Diary,Paintings,1000000.0,21.0,False,False,,
Diary,Paintings,1000000.0,99.0,True,True,Genuine,Report #7645
Diary,Paintings,1000000.0,99.0,True,True,Genuine,
Diary,Paintings,1000000.0,99.0,True,True,Fake,Report #7646
Diary,Paintings,1000000.0,99.0,True,True,Fake,
Diary,Paintings,1000000.0,99.0,True,True,,Report #7647
Diary,Paintings,1000000.0,99.0,True,True,,
Diary,Paintings,1000000.0,99.0,True,False,Genuine,Report #7648
Diary,Paintings,1000000.0,99.0,True,False,Genuine,
Diary,Paintings,1000000.0,99.0,True,False,Fake,Report #7649
Diary,Paintings,1000000.0,99.0,True,False,Fake,
Diary,Paintings,1000000.0,99.0,True,False,,Report #7650
Diary,Paintings,1000000.0,99.0,True,False,,
Diary,Paintings,1000000.0,99.0,False,True,Genuine,Report #7651
Diary,Paintings,1000000.0,99.0,False,True,Genuine,
Diary,Paintings,1000000.0,99.0,False,True,Fake,Report #7652
Diary,Paintings,1000000.0,99.0,False,True,Fake,
Diary,Paintings,1000000.0,99.0,False,True,,Report #7653
Diary,Paintings,1000000.0,99.0,False,True,,
Diary,Paintings,1000000.0,99.0,False,False,Genuine,Report #7654
Diary,Paintings,1000000.0,99.0,False,False,Genuine,
Diary,Paintings,1000000.0,99.0,False,False,Fake,Report #7655
Diary,Paintings,1000000.0,99.0,False,False,Fake,
Diary,Paintings,1000000.0,99.0,False,False,,Report #7656
Diary,Paintings,1000000.0,99.0,False,False,,
Diary,Paintings,1000000.0,100.0,True,True,Genuine,Report #7657
Diary,Paintings,1000000.0,100.0,True,True,Genuine,
Diary,Paintings,1000000.0,100.0,True,True,Fake,Report #7658
Diary,Paintings,1000000.0,100.0,True,True,Fake,
Diary,Paintings,1000000.0,100.0,True,True,,Report #7659
Diary,Paintings,1000000.0,100.0,True,True,,
Diary,Paintings,1000000.0,100.0,True,False,Genuine,Report #7660
Diary,Paintings,1000000.0,100.0,True,False,Genuine,
Diary,Paintings,1000000.0,100.0,True,False,Fake,Report #7661
Diary,Paintings,1000000.0,100.0,True,False,Fake,
Diary,Paintings,1000000.0,100.0,True,False,,Report #7662
Diary,Paintings,1000000.0,100.0,True,False,,
Diary,Paintings,1000000.0,100.0,False,True,Genuine,Report #7663
Diary,Paintings,1000000.0,100.0,False,True,Genuine,
Diary,Paintings,1000000.0,100.0,False,True,Fake,Report #7664
Diary,Paintings,1000000.0,100.0,False,True,Fake,
Diary,Paintings,1000000.0,100.0,False,True,,Report #7665
Diary,Paintings,1000000.0,100.0,False,True,,
Diary,Paintings,1000000.0,100.0,False,False,Genuine,Report #7666
Diary,Paintings,1000000.0,100.0,False,False,Genuine,
Diary,Paintings,1000000.0,100.0,False,False,Fake,Report #7667
Diary,Paintings,1000000.0,100.0,False,False,Fake,
Diary,Paintings,1000000.0,100.0,False,False,,Report #7668
Diary,Paintings,1000000.0,100.0,False,False,,
Diary,Paintings,1000000.0,101.0,True,True,Genuine,Report #7669
Diary,Paintings,1000000.0,101.0,True,True,Genuine,
Diary,Paintings,1000000.0,101.0,True,True,Fake,Report #7670
Diary,Paintings,1000000.0,101.0,True,True,Fake,
Diary,Paintings,1000000.0,101.0,True,True,,Report #7671
Diary,Paintings,1000000.0,101.0,True,True,,
Diary,Paintings,1000000.0,101.0,True,False,Genuine,Report #7672
Diary,Paintings,1000000.0,101.0,True,False,Genuine,
Diary,Paintings,1000000.0,101.0,True,False,Fake,Report #7673
Diary,Paintings,1000000.0,101.0,True,False,Fake,
Diary,Paintings,1000000.0,101.0,True,False,,Report #7674
Diary,Paintings,1000000.0,101.0,True,False,,
Diary,Paintings,1000000.0,101.0,False,True,Genuine,Report #7675
Diary,Paintings,1000000.0,101.0,False,True,Genuine,
Diary,Paintings,1000000.0,101.0,False,True,Fake,Report #7676
Diary,Paintings,1000000.0,101.0,False,True,Fake,
Diary,Paintings,1000000.0,101.0,False,True,,Report #7677
Diary,Paintings,1000000.0,101.0,False,True,,
Diary,Paintings,1000000.0,101.0,False,False,Genuine,Report #7678
Diary,Paintings,1000000.0,101.0,False,False,Genuine,
Diary,Paintings,1000000.0,101.0,False,False,Fake,Report #7679
Diary,Paintings,1000000.0,101.0,False,False,Fake,
Diary,Paintings,1000000.0,101.0,False,False,,Report #7680
Diary,Paintings,1000000.0,101.0,False,False,,
Diary,Knife,0.0,-1.0,True,True,Genuine,Report #7681
Diary,Knife,0.0,-1.0,True,True,Genuine,
Diary,Knife,0.0,-1.0,True,True,Fake,Report #7682
Diary,Knife,0.0,-1.0,True,True,Fake,
Diary,Knife,0.0,-1.0,True,True,,Report #7683
Diary,Knife,0.0,-1.0,True,True,,
Diary,Knife,0.0,-1.0,True,False,Genuine,Report #7684
Diary,Knife,0.0,-1.0,True,False,Genuine,
Diary,Knife,0.0,-1.0,True,False,Fake,Report #7685
Diary,Knife,0.0,-1.0,True,False,Fake,
Diary,Knife,0.0,-1.0,True,False,,Report #7686
Diary,Knife,0.0,-1.0,True,False,,
Diary,Knife,0.0,-1.0,False,True,Genuine,Report #7687
Diary,Knife,0.0,-1.0,False,True,Genuine,
Diary,Knife,0.0,-1.0,False,True,Fake,Report #7688
Diary,Knife,0.0,-1.0,False,True,Fake,
Diary,Knife,0.0,-1.0,False,True,,Report #7689
Diary,Knife,0.0,-1.0,False,True,,
Diary,Knife,0.0,-1.0,False,False,Genuine,Report #7690
Diary,Knife,0.0,-1.0,False,False,Genuine,
Diary,Knife,0.0,-1.0,False,False,Fake,Report #7691
Diary,Knife,0.0,-1.0,False,False,Fake,
Diary,Knife,0.0,-1.0,False,False,,Report #7692
Diary,Knife,0.0,-1.0,False,False,,
Diary,Knife,0.0,0.0,True,True,Genuine,Report #7693
Diary,Knife,0.0,0.0,True,True,Genuine,
Diary,Knife,0.0,0.0,True,True,Fake,Report #7694
Diary,Knife,0.0,0.0,True,True,Fake,
Diary,Knife,0.0,0.0,True,True,,Report #7695
Diary,Knife,0.0,0.0,True,True,,
Diary,Knife,0.0,0.0,True,False,Genuine,Report #7696
Diary,Knife,0.0,0.0,True,False,Genuine,
Diary,Knife,0.0,0.0,True,False,Fake,Report #7697
Diary,Knife,0.0,0.0,True,False,Fake,
Diary,Knife,0.0,0.0,True,False,,Report #7698
Diary,Knife,0.0,0.0,True,False,,
Diary,Knife,0.0,0.0,False,True,Genuine,Report #7699
Diary,Knife,0.0,0.0,False,True,Genuine,
Diary,Knife,0.0,0.0,False,True,Fake,Report #7700
Diary,Knife,0.0,0.0,False,True,Fake,
Diary,Knife,0.0,0.0,False,True,,Report #7701
Diary,Knife,0.0,0.0,False,True,,
Diary,Knife,0.0,0.0,False,False,Genuine,Report #7702
Diary,Knife,0.0,0.0,False,False,Genuine,
Diary,Knife,0.0,0.0,False,False,Fake,Report #7703
Diary,Knife,0.0,0.0,False,False,Fake,
Diary,Knife,0.0,0.0,False,False,,Report #7704
Diary,Knife,0.0,0.0,False,False,,
Diary,Knife,0.0,19.0,True,True,Genuine,Report #7705
Diary,Knife,0.0,19.0,True,True,Genuine,
Diary,Knife,0.0,19.0,True,True,Fake,Report #7706
Diary,Knife,0.0,19.0,True,True,Fake,
Diary,Knife,0.0,19.0,True,True,,Report #7707
Diary,Knife,0.0,19.0,True,True,,
Diary,Knife,0.0,19.0,True,False,Genuine,Report #7708
Diary,Knife,0.0,19.0,True,False,Genuine,
Diary,Knife,0.0,19.0,True,False,Fake,Report #7709
Diary,Knife,0.0,19.0,True,False,Fake,
Diary,Knife,0.0,19.0,True,False,,Report #7710
Diary,Knife,0.0,19.0,True,False,,
Diary,Knife,0.0,19.0,False,True,Genuine,Report #7711
Diary,Knife,0.0,19.0,False,True,Genuine,
Diary,Knife,0.0,19.0,False,True,Fake,Report #7712
Diary,Knife,0.0,19.0,False,True,Fake,
Diary,Knife,0.0,19.0,False,True,,Report #7713
Diary,Knife,0.0,19.0,False,True,,
Diary,Knife,0.0,19.0,False,False,Genuine,Report #7714
Diary,Knife,0.0,19.0,False,False,Genuine,
Diary,Knife,0.0,19.0,False,False,Fake,Report #7715
Diary,Knife,0.0,19.0,False,False,Fake,
Diary,Knife,0.0,19.0,False,False,,Report #7716
Diary,Knife,0.0,19.0,False,False,,
Diary,Knife,0.0,20.0,True,True,Genuine,Report #7717
Diary,Knife,0.0,20.0,True,True,Genuine,
Diary,Knife,0.0,20.0,True,True,Fake,Report #7718
Diary,Knife,0.0,20.0,True,True,Fake,
Diary,Knife,0.0,20.0,True,True,,Report #7719
Diary,Knife,0.0,20.0,True,True,,
Diary,Knife,0.0,20.0,True,False,Genuine,Report #7720
Diary,Knife,0.0,20.0,True,False,Genuine,
Diary,Knife,0.0,20.0,True,False,Fake,Report #7721
Diary,Knife,0.0,20.0,True,False,Fake,
Diary,Knife,0.0,20.0,True,False,,Report #7722
Diary,Knife,0.0,20.0,True,False,,
Diary,Knife,0.0,20.0,False,True,Genuine,Report #7723
Diary,Knife,0.0,20.0,False,True,Genuine,
Diary,Knife,0.0,20.0,False,True,Fake,Report #7724
Diary,Knife,0.0,20.0,False,True,Fake,
Diary,Knife,0.0,20.0,False,True,,Report #7725
Diary,Knife,0.0,20.0,False,True,,
Diary,Knife,0.0,20.0,False,False,Genuine,Report #7726
Diary,Knife,0.0,20.0,False,False,Genuine,
Diary,Knife,0.0,20.0,False,False,Fake,Report #7727
Diary,Knife,0.0,20.0,False,False,Fake,
Diary,Knife,0.0,20.0,False,False,,Report #7728
Diary,Knife,0.0,20.0,False,False,,
Diary,Knife,0.0,21.0,True,True,Genuine,Report #7729
Diary,Knife,0.0,21.0,True,True,Genuine,
Diary,Knife,0.0,21.0,True,True,Fake,Report #7730
Diary,Knife,0.0,21.0,True,True,Fake,
Diary,Knife,0.0,21.0,True,True,,Report #7731
Diary,Knife,0.0,21.0,True,True,,
Diary,Knife,0.0,21.0,True,False,Genuine,Report #7732
Diary,Knife,0.0,21.0,True,False,Genuine,
Diary,Knife,0.0,21.0,True,False,Fake,Report #7733
Diary,Knife,0.0,21.0,True,False,Fake,
Diary,Knife,0.0,21.0,True,False,,Report #7734
Diary,Knife,0.0,21.0,True,False,,
Diary,Knife,0.0,21.0,False,True,Genuine,Report #7735
Diary,Knife,0.0,21.0,False,True,Genuine,
Diary,Knife,0.0,21.0,False,True,Fake,Report #7736
Diary,Knife,0.0,21.0,False,True,Fake,
Diary,Knife,0.0,21.0,False,True,,Report #7737
Diary,Knife,0.0,21.0,False,True,,
Diary,Knife,0.0,21.0,False,False,Genuine,Report #7738
Diary,Knife,0.0,21.0,False,False,Genuine,
Diary,Knife,0.0,21.0,False,False,Fake,Report #7739
Diary,Knife,0.0,21.0,False,False,Fake,
Diary,Knife,0.0,21.0,False,False,,Report #7740
Diary,Knife,0.0,21.0,False,False,,
Diary,Knife,0.0,99.0,True,True,Genuine,Report #7741
Diary,Knife,0.0,99.0,True,True,Genuine,
Diary,Knife,0.0,99.0,True,True,Fake,Report #7742
Diary,Knife,0.0,99.0,True,True,Fake,
Diary,Knife,0.0,99.0,True,True,,Report #7743
Diary,Knife,0.0,99.0,True,True,,
Diary,Knife,0.0,99.0,True,False,Genuine,Report #7744
Diary,Knife,0.0,99.0,True,False,Genuine,
Diary,Knife,0.0,99.0,True,False,Fake,Report #7745
Diary,Knife,0.0,99.0,True,False,Fake,
Diary,Knife,0.0,99.0,True,False,,Report #7746
Diary,Knife,0.0,99.0,True,False,,
Diary,Knife,0.0,99.0,False,True,Genuine,Report #7747
Diary,Knife,0.0,99.0,False,True,Genuine,
Diary,Knife,0.0,99.0,False,True,Fake,Report #7748
Diary,Knife,0.0,99.0,False,True,Fake,
Diary,Knife,0.0,99.0,False,True,,Report #7749
Diary,Knife,0.0,99.0,False,True,,
Diary,Knife,0.0,99.0,False,False,Genuine,Report #7750
Diary,Knife,0.0,99.0,False,False,Genuine,
Diary,Knife,0.0,99.0,False,False,Fake,Report #7751
Diary,Knife,0.0,99.0,False,False,Fake,
Diary,Knife,0.0,99.0,False,False,,Report #7752
Diary,Knife,0.0,99.0,False,False,,
Diary,Knife,0.0,100.0,True,True,Genuine,Report #7753
Diary,Knife,0.0,100.0,True,True,Genuine,
Diary,Knife,0.0,100.0,True,True,Fake,Report #7754
Diary,Knife,0.0,100.0,True,True,Fake,
Diary,Knife,0.0,100.0,True,True,,Report #7755
Diary,Knife,0.0,100.0,True,True,,
Diary,Knife,0.0,100.0,True,False,Genuine,Report #7756
Diary,Knife,0.0,100.0,True,False,Genuine,
Diary,Knife,0.0,100.0,True,False,Fake,Report #7757
Diary,Knife,0.0,100.0,True,False,Fake,
Diary,Knife,0.0,100.0,True,False,,Report #7758
Diary,Knife,0.0,100.0,True,False,,
Diary,Knife,0.0,100.0,False,True,Genuine,Report #7759
Diary,Knife,0.0,100.0,False,True,Genuine,
Diary,Knife,0.0,100.0,False,True,Fake,Report #7760
Diary,Knife,0.0,100.0,False,True,Fake,
Diary,Knife,0.0,100.0,False,True,,Report #7761
Diary,Knife,0.0,100.0,False,True,,
Diary,Knife,0.0,100.0,False,False,Genuine,Report #7762
Diary,Knife,0.0,100.0,False,False,Genuine,
Diary,Knife,0.0,100.0,False,False,Fake,Report #7763
Diary,Knife,0.0,100.0,False,False,Fake,
Diary,Knife,0.0,100.0,False,False,,Report #7764
Diary,Knife,0.0,100.0,False,False,,
Diary,Knife,0.0,101.0,True,True,Genuine,Report #7765
Diary,Knife,0.0,101.0,True,True,Genuine,
Diary,Knife,0.0,101.0,True,True,Fake,Report #7766
Diary,Knife,0.0,101.0,True,True,Fake,
Diary,Knife,0.0,101.0,True,True,,Report #7767
Diary,Knife,0.0,101.0,True,True,,
Diary,Knife,0.0,101.0,True,False,Genuine,Report #7768
Diary,Knife,0.0,101.0,True,False,Genuine,
Diary,Knife,0.0,101.0,True,False,Fake,Report #7769
Diary,Knife,0.0,101.0,True,False,Fake,
Diary,Knife,0.0,101.0,True,False,,Report #7770
Diary,Knife,0.0,101.0,True,False,,
Diary,Knife,0.0,101.0,False,True,Genuine,Report #7771
Diary,Knife,0.0,101.0,False,True,Genuine,
Diary,Knife,0.0,101.0,False,True,Fake,Report #7772
Diary,Knife,0.0,101.0,False,True,Fake,
Diary,Knife,0.0,101.0,False,True,,Report #7773
Diary,Knife,0.0,101.0,False,True,,
Diary,Knife,0.0,101.0,False,False,Genuine,Report #7774
Diary,Knife,0.0,101.0,False,False,Genuine,
Diary,Knife,0.0,101.0,False,False,Fake,Report #7775
Diary,Knife,0.0,101.0,False,False,Fake,
Diary,Knife,0.0,101.0,False,False,,Report #7776
Diary,Knife,0.0,101.0,False,False,,
Diary,Knife,1.0,-1.0,True,True,Genuine,Report #7777
Diary,Knife,1.0,-1.0,True,True,Genuine,
Diary,Knife,1.0,-1.0,True,True,Fake,Report #7778
Diary,Knife,1.0,-1.0,True,True,Fake,
Diary,Knife,1.0,-1.0,True,True,,Report #7779
Diary,Knife,1.0,-1.0,True,True,,
Diary,Knife,1.0,-1.0,True,False,Genuine,Report #7780
Diary,Knife,1.0,-1.0,True,False,Genuine,
Diary,Knife,1.0,-1.0,True,False,Fake,Report #7781
Diary,Knife,1.0,-1.0,True,False,Fake,
Diary,Knife,1.0,-1.0,True,False,,Report #7782
Diary,Knife,1.0,-1.0,True,False,,
Diary,Knife,1.0,-1.0,False,True,Genuine,Report #7783
Diary,Knife,1.0,-1.0,False,True,Genuine,
Diary,Knife,1.0,-1.0,False,True,Fake,Report #7784
Diary,Knife,1.0,-1.0,False,True,Fake,
Diary,Knife,1.0,-1.0,False,True,,Report #7785
Diary,Knife,1.0,-1.0,False,True,,
Diary,Knife,1.0,-1.0,False,False,Genuine,Report #7786
Diary,Knife,1.0,-1.0,False,False,Genuine,
Diary,Knife,1.0,-1.0,False,False,Fake,Report #7787
Diary,Knife,1.0,-1.0,False,False,Fake,
Diary,Knife,1.0,-1.0,False,False,,Report #7788
Diary,Knife,1.0,-1.0,False,False,,
Diary,Knife,1.0,0.0,True,True,Genuine,Report #7789
Diary,Knife,1.0,0.0,True,True,Genuine,
Diary,Knife,1.0,0.0,True,True,Fake,Report #7790
Diary,Knife,1.0,0.0,True,True,Fake,
Diary,Knife,1.0,0.0,True,True,,Report #7791
Diary,Knife,1.0,0.0,True,True,,
Diary,Knife,1.0,0.0,True,False,Genuine,Report #7792
Diary,Knife,1.0,0.0,True,False,Genuine,
Diary,Knife,1.0,0.0,True,False,Fake,Report #7793
Diary,Knife,1.0,0.0,True,False,Fake,
Diary,Knife,1.0,0.0,True,False,,Report #7794
Diary,Knife,1.0,0.0,True,False,,
Diary,Knife,1.0,0.0,False,True,Genuine,Report #7795
Diary,Knife,1.0,0.0,False,True,Genuine,
Diary,Knife,1.0,0.0,False,True,Fake,Report #7796
Diary,Knife,1.0,0.0,False,True,Fake,
Diary,Knife,1.0,0.0,False,True,,Report #7797
Diary,Knife,1.0,0.0,False,True,,
Diary,Knife,1.0,0.0,False,False,Genuine,Report #7798
Diary,Knife,1.0,0.0,False,False,Genuine,
Diary,Knife,1.0,0.0,False,False,Fake,Report #7799
Diary,Knife,1.0,0.0,False,False,Fake,
Diary,Knife,1.0,0.0,False,False,,Report #7800
Diary,Knife,1.0,0.0,False,False,,
Diary,Knife,1.0,19.0,True,True,Genuine,Report #7801
Diary,Knife,1.0,19.0,True,True,Genuine,
Diary,Knife,1.0,19.0,True,True,Fake,Report #7802
Diary,Knife,1.0,19.0,True,True,Fake,
Diary,Knife,1.0,19.0,True,True,,Report #7803
Diary,Knife,1.0,19.0,True,True,,
Diary,Knife,1.0,19.0,True,False,Genuine,Report #7804
Diary,Knife,1.0,19.0,True,False,Genuine,
Diary,Knife,1.0,19.0,True,False,Fake,Report #7805
Diary,Knife,1.0,19.0,True,False,Fake,
Diary,Knife,1.0,19.0,True,False,,Report #7806
Diary,Knife,1.0,19.0,True,False,,
Diary,Knife,1.0,19.0,False,True,Genuine,Report #7807
Diary,Knife,1.0,19.0,False,True,Genuine,
Diary,Knife,1.0,19.0,False,True,Fake,Report #7808
Diary,Knife,1.0,19.0,False,True,Fake,
Diary,Knife,1.0,19.0,False,True,,Report #7809
Diary,Knife,1.0,19.0,False,True,,
Diary,Knife,1.0,19.0,False,False,Genuine,Report #7810
Diary,Knife,1.0,19.0,False,False,Genuine,
Diary,Knife,1.0,19.0,False,False,Fake,Report #7811
Diary,Knife,1.0,19.0,False,False,Fake,
Diary,Knife,1.0,19.0,False,False,,Report #7812
Diary,Knife,1.0,19.0,False,False,,
Diary,Knife,1.0,20.0,True,True,Genuine,Report #7813
Diary,Knife,1.0,20.0,True,True,Genuine,
Diary,Knife,1.0,20.0,True,True,Fake,Report #7814
Diary,Knife,1.0,20.0,True,True,Fake,
Diary,Knife,1.0,20.0,True,True,,Report #7815
Diary,Knife,1.0,20.0,True,True,,
Diary,Knife,1.0,20.0,True,False,Genuine,Report #7816
Diary,Knife,1.0,20.0,True,False,Genuine,
Diary,Knife,1.0,20.0,True,False,Fake,Report #7817
Diary,Knife,1.0,20.0,True,False,Fake,
Diary,Knife,1.0,20.0,True,False,,Report #7818
Diary,Knife,1.0,20.0,True,False,,
Diary,Knife,1.0,20.0,False,True,Genuine,Report #7819
Diary,Knife,1.0,20.0,False,True,Genuine,
Diary,Knife,1.0,20.0,False,True,Fake,Report #7820
Diary,Knife,1.0,20.0,False,True,Fake,
Diary,Knife,1.0,20.0,False,True,,Report #7821
Diary,Knife,1.0,20.0,False,True,,
Diary,Knife,1.0,20.0,False,False,Genuine,Report #7822
Diary,Knife,1.0,20.0,False,False,Genuine,
Diary,Knife,1.0,20.0,False,False,Fake,Report #7823
Diary,Knife,1.0,20.0,False,False,Fake,
Diary,Knife,1.0,20.0,False,False,,Report #7824
Diary,Knife,1.0,20.0,False,False,,
Diary,Knife,1.0,21.0,True,True,Genuine,Report #7825
Diary,Knife,1.0,21.0,True,True,Genuine,
Diary,Knife,1.0,21.0,True,True,Fake,Report #7826
Diary,Knife,1.0,21.0,True,True,Fake,
Diary,Knife,1.0,21.0,True,True,,Report #7827
Diary,Knife,1.0,21.0,True,True,,
Diary,Knife,1.0,21.0,True,False,Genuine,Report #7828
Diary,Knife,1.0,21.0,True,False,Genuine,
Diary,Knife,1.0,21.0,True,False,Fake,Report #7829
Diary,Knife,1.0,21.0,True,False,Fake,
Diary,Knife,1.0,21.0,True,False,,Report #7830
Diary,Knife,1.0,21.0,True,False,,
Diary,Knife,1.0,21.0,False,True,Genuine,Report #7831
Diary,Knife,1.0,21.0,False,True,Genuine,
Diary,Knife,1.0,21.0,False,True,Fake,Report #7832
Diary,Knife,1.0,21.0,False,True,Fake,
Diary,Knife,1.0,21.0,False,True,,Report #7833
Diary,Knife,1.0,21.0,False,True,,
Diary,Knife,1.0,21.0,False,False,Genuine,Report #7834
Diary,Knife,1.0,21.0,False,False,Genuine,
Diary,Knife,1.0,21.0,False,False,Fake,Report #7835
Diary,Knife,1.0,21.0,False,False,Fake,
Diary,Knife,1.0,21.0,False,False,,Report #7836
Diary,Knife,1.0,21.0,False,False,,
Diary,Knife,1.0,99.0,True,True,Genuine,Report #7837
Diary,Knife,1.0,99.0,True,True,Genuine,
Diary,Knife,1.0,99.0,True,True,Fake,Report #7838
Diary,Knife,1.0,99.0,True,True,Fake,
Diary,Knife,1.0,99.0,True,True,,Report #7839
Diary,Knife,1.0,99.0,True,True,,
Diary,Knife,1.0,99.0,True,False,Genuine,Report #7840
Diary,Knife,1.0,99.0,True,False,Genuine,
Diary,Knife,1.0,99.0,True,False,Fake,Report #7841
Diary,Knife,1.0,99.0,True,False,Fake,
Diary,Knife,1.0,99.0,True,False,,Report #7842
Diary,Knife,1.0,99.0,True,False,,
Diary,Knife,1.0,99.0,False,True,Genuine,Report #7843
Diary,Knife,1.0,99.0,False,True,Genuine,
Diary,Knife,1.0,99.0,False,True,Fake,Report #7844
Diary,Knife,1.0,99.0,False,True,Fake,
Diary,Knife,1.0,99.0,False,True,,Report #7845
Diary,Knife,1.0,99.0,False,True,,
Diary,Knife,1.0,99.0,False,False,Genuine,Report #7846
Diary,Knife,1.0,99.0,False,False,Genuine,
Diary,Knife,1.0,99.0,False,False,Fake,Report #7847
Diary,Knife,1.0,99.0,False,False,Fake,
Diary,Knife,1.0,99.0,False,False,,Report #7848
Diary,Knife,1.0,99.0,False,False,,
Diary,Knife,1.0,100.0,True,True,Genuine,Report #7849
Diary,Knife,1.0,100.0,True,True,Genuine,
Diary,Knife,1.0,100.0,True,True,Fake,Report #7850
Diary,Knife,1.0,100.0,True,True,Fake,
Diary,Knife,1.0,100.0,True,True,,Report #7851
Diary,Knife,1.0,100.0,True,True,,
Diary,Knife,1.0,100.0,True,False,Genuine,Report #7852
Diary,Knife,1.0,100.0,True,False,Genuine,
Diary,Knife,1.0,100.0,True,False,Fake,Report #7853
Diary,Knife,1.0,100.0,True,False,Fake,
Diary,Knife,1.0,100.0,True,False,,Report #7854
Diary,Knife,1.0,100.0,True,False,,
Diary,Knife,1.0,100.0,False,True,Genuine,Report #7855
Diary,Knife,1.0,100.0,False,True,Genuine,
Diary,Knife,1.0,100.0,False,True,Fake,Report #7856
Diary,Knife,1.0,100.0,False,True,Fake,
Diary,Knife,1.0,100.0,False,True,,Report #7857
Diary,Knife,1.0,100.0,False,True,,
Diary,Knife,1.0,100.0,False,False,Genuine,Report #7858
Diary,Knife,1.0,100.0,False,False,Genuine,
Diary,Knife,1.0,100.0,False,False,Fake,Report #7859
Diary,Knife,1.0,100.0,False,False,Fake,
Diary,Knife,1.0,100.0,False,False,,Report #7860
Diary,Knife,1.0,100.0,False,False,,
Diary,Knife,1.0,101.0,True,True,Genuine,Report #7861
Diary,Knife,1.0,101.0,True,True,Genuine,
Diary,Knife,1.0,101.0,True,True,Fake,Report #7862
Diary,Knife,1.0,101.0,True,True,Fake,
Diary,Knife,1.0,101.0,True,True,,Report #7863
Diary,Knife,1.0,101.0,True,True,,
Diary,Knife,1.0,101.0,True,False,Genuine,Report #7864
Diary,Knife,1.0,101.0,True,False,Genuine,
Diary,Knife,1.0,101.0,True,False,Fake,Report #7865
Diary,Knife,1.0,101.0,True,False,Fake,
Diary,Knife,1.0,101.0,True,False,,Report #7866
Diary,Knife,1.0,101.0,True,False,,
Diary,Knife,1.0,101.0,False,True,Genuine,Report #7867
Diary,Knife,1.0,101.0,False,True,Genuine,
Diary,Knife,1.0,101.0,False,True,Fake,Report #7868
Diary,Knife,1.0,101.0,False,True,Fake,
Diary,Knife,1.0,101.0,False,True,,Report #7869
Diary,Knife,1.0,101.0,False,True,,
Diary,Knife,1.0,101.0,False,False,Genuine,Report #7870
Diary,Knife,1.0,101.0,False,False,Genuine,
Diary,Knife,1.0,101.0,False,False,Fake,Report #7871
Diary,Knife,1.0,101.0,False,False,Fake,
Diary,Knife,1.0,101.0,False,False,,Report #7872
Diary,Knife,1.0,101.0,False,False,,
Diary,Knife,100.0,-1.0,True,True,Genuine,Report #7873
Diary,Knife,100.0,-1.0,True,True,Genuine,
Diary,Knife,100.0,-1.0,True,True,Fake,Report #7874
Diary,Knife,100.0,-1.0,True,True,Fake,
Diary,Knife,100.0,-1.0,True,True,,Report #7875
Diary,Knife,100.0,-1.0,True,True,,
Diary,Knife,100.0,-1.0,True,False,Genuine,Report #7876
Diary,Knife,100.0,-1.0,True,False,Genuine,
Diary,Knife,100.0,-1.0,True,False,Fake,Report #7877
Diary,Knife,100.0,-1.0,True,False,Fake,
Diary,Knife,100.0,-1.0,True,False,,Report #7878
Diary,Knife,100.0,-1.0,True,False,,
Diary,Knife,100.0,-1.0,False,True,Genuine,Report #7879
Diary,Knife,100.0,-1.0,False,True,Genuine,
Diary,Knife,100.0,-1.0,False,True,Fake,Report #7880
Diary,Knife,100.0,-1.0,False,True,Fake,
Diary,Knife,100.0,-1.0,False,True,,Report #7881
Diary,Knife,100.0,-1.0,False,True,,
Diary,Knife,100.0,-1.0,False,False,Genuine,Report #7882
Diary,Knife,100.0,-1.0,False,False,Genuine,
Diary,Knife,100.0,-1.0,False,False,Fake,Report #7883
Diary,Knife,100.0,-1.0,False,False,Fake,
Diary,Knife,100.0,-1.0,False,False,,Report #7884
Diary,Knife,100.0,-1.0,False,False,,
Diary,Knife,100.0,0.0,True,True,Genuine,Report #7885
Diary,Knife,100.0,0.0,True,True,Genuine,
Diary,Knife,100.0,0.0,True,True,Fake,Report #7886
Diary,Knife,100.0,0.0,True,True,Fake,
Diary,Knife,100.0,0.0,True,True,,Report #7887
Diary,Knife,100.0,0.0,True,True,,
Diary,Knife,100.0,0.0,True,False,Genuine,Report #7888
Diary,Knife,100.0,0.0,True,False,Genuine,
Diary,Knife,100.0,0.0,True,False,Fake,Report #7889
Diary,Knife,100.0,0.0,True,False,Fake,
Diary,Knife,100.0,0.0,True,False,,Report #7890
Diary,Knife,100.0,0.0,True,False,,
Diary,Knife,100.0,0.0,False,True,Genuine,Report #7891
Diary,Knife,100.0,0.0,False,True,Genuine,
Diary,Knife,100.0,0.0,False,True,Fake,Report #7892
Diary,Knife,100.0,0.0,False,True,Fake,
Diary,Knife,100.0,0.0,False,True,,Report #7893
Diary,Knife,100.0,0.0,False,True,,
Diary,Knife,100.0,0.0,False,False,Genuine,Report #7894
Diary,Knife,100.0,0.0,False,False,Genuine,
Diary,Knife,100.0,0.0,False,False,Fake,Report #7895
Diary,Knife,100.0,0.0,False,False,Fake,
Diary,Knife,100.0,0.0,False,False,,Report #7896
Diary,Knife,100.0,0.0,False,False,,
Diary,Knife,100.0,19.0,True,True,Genuine,Report #7897
Diary,Knife,100.0,19.0,True,True,Genuine,
Diary,Knife,100.0,19.0,True,True,Fake,Report #7898
Diary,Knife,100.0,19.0,True,True,Fake,
Diary,Knife,100.0,19.0,True,True,,Report #7899
Diary,Knife,100.0,19.0,True,True,,
Diary,Knife,100.0,19.0,True,False,Genuine,Report #7900
Diary,Knife,100.0,19.0,True,False,Genuine,
Diary,Knife,100.0,19.0,True,False,Fake,Report #7901
Diary,Knife,100.0,19.0,True,False,Fake,
Diary,Knife,100.0,19.0,True,False,,Report #7902
Diary,Knife,100.0,19.0,True,False,,
Diary,Knife,100.0,19.0,False,True,Genuine,Report #7903
Diary,Knife,100.0,19.0,False,True,Genuine,
Diary,Knife,100.0,19.0,False,True,Fake,Report #7904
Diary,Knife,100.0,19.0,False,True,Fake,
Diary,Knife,100.0,19.0,False,True,,Report #7905
Diary,Knife,100.0,19.0,False,True,,
Diary,Knife,100.0,19.0,False,False,Genuine,Report #7906
Diary,Knife,100.0,19.0,False,False,Genuine,
Diary,Knife,100.0,19.0,False,False,Fake,Report #7907
Diary,Knife,100.0,19.0,False,False,Fake,
Diary,Knife,100.0,19.0,False,False,,Report #7908
Diary,Knife,100.0,19.0,False,False,,
Diary,Knife,100.0,20.0,True,True,Genuine,Report #7909
Diary,Knife,100.0,20.0,True,True,Genuine,
Diary,Knife,100.0,20.0,True,True,Fake,Report #7910
Diary,Knife,100.0,20.0,True,True,Fake,
Diary,Knife,100.0,20.0,True,True,,Report #7911
Diary,Knife,100.0,20.0,True,True,,
Diary,Knife,100.0,20.0,True,False,Genuine,Report #7912
Diary,Knife,100.0,20.0,True,False,Genuine,
Diary,Knife,100.0,20.0,True,False,Fake,Report #7913
Diary,Knife,100.0,20.0,True,False,Fake,
Diary,Knife,100.0,20.0,True,False,,Report #7914
Diary,Knife,100.0,20.0,True,False,,
Diary,Knife,100.0,20.0,False,True,Genuine,Report #7915
Diary,Knife,100.0,20.0,False,True,Genuine,
Diary,Knife,100.0,20.0,False,True,Fake,Report #7916
Diary,Knife,100.0,20.0,False,True,Fake,
Diary,Knife,100.0,20.0,False,True,,Report #7917
Diary,Knife,100.0,20.0,False,True,,
Diary,Knife,100.0,20.0,False,False,Genuine,Report #7918
Diary,Knife,100.0,20.0,False,False,Genuine,
Diary,Knife,100.0,20.0,False,False,Fake,Report #7919
Diary,Knife,100.0,20.0,False,False,Fake,
Diary,Knife,100.0,20.0,False,False,,Report #7920
Diary,Knife,100.0,20.0,False,False,,
Diary,Knife,100.0,21.0,True,True,Genuine,Report #7921
Diary,Knife,100.0,21.0,True,True,Genuine,
Diary,Knife,100.0,21.0,True,True,Fake,Report #7922
Diary,Knife,100.0,21.0,True,True,Fake,
Diary,Knife,100.0,21.0,True,True,,Report #7923
Diary,Knife,100.0,21.0,True,True,,
Diary,Knife,100.0,21.0,True,False,Genuine,Report #7924
Diary,Knife,100.0,21.0,True,False,Genuine,
Diary,Knife,100.0,21.0,True,False,Fake,Report #7925
Diary,Knife,100.0,21.0,True,False,Fake,
Diary,Knife,100.0,21.0,True,False,,Report #7926
Diary,Knife,100.0,21.0,True,False,,
Diary,Knife,100.0,21.0,False,True,Genuine,Report #7927
Diary,Knife,100.0,21.0,False,True,Genuine,
Diary,Knife,100.0,21.0,False,True,Fake,Report #7928
Diary,Knife,100.0,21.0,False,True,Fake,
Diary,Knife,100.0,21.0,False,True,,Report #7929
Diary,Knife,100.0,21.0,False,True,,
Diary,Knife,100.0,21.0,False,False,Genuine,Report #7930
Diary,Knife,100.0,21.0,False,False,Genuine,
Diary,Knife,100.0,21.0,False,False,Fake,Report #7931
Diary,Knife,100.0,21.0,False,False,Fake,
Diary,Knife,100.0,21.0,False,False,,Report #7932
Diary,Knife,100.0,21.0,False,False,,
Diary,Knife,100.0,99.0,True,True,Genuine,Report #7933
Diary,Knife,100.0,99.0,True,True,Genuine,
Diary,Knife,100.0,99.0,True,True,Fake,Report #7934
Diary,Knife,100.0,99.0,True,True,Fake,
Diary,Knife,100.0,99.0,True,True,,Report #7935
Diary,Knife,100.0,99.0,True,True,,
Diary,Knife,100.0,99.0,True,False,Genuine,Report #7936
Diary,Knife,100.0,99.0,True,False,Genuine,
Diary,Knife,100.0,99.0,True,False,Fake,Report #7937
Diary,Knife,100.0,99.0,True,False,Fake,
Diary,Knife,100.0,99.0,True,False,,Report #7938
Diary,Knife,100.0,99.0,True,False,,
Diary,Knife,100.0,99.0,False,True,Genuine,Report #7939
Diary,Knife,100.0,99.0,False,True,Genuine,
Diary,Knife,100.0,99.0,False,True,Fake,Report #7940
Diary,Knife,100.0,99.0,False,True,Fake,
Diary,Knife,100.0,99.0,False,True,,Report #7941
Diary,Knife,100.0,99.0,False,True,,
Diary,Knife,100.0,99.0,False,False,Genuine,Report #7942
Diary,Knife,100.0,99.0,False,False,Genuine,
Diary,Knife,100.0,99.0,False,False,Fake,Report #7943
Diary,Knife,100.0,99.0,False,False,Fake,
Diary,Knife,100.0,99.0,False,False,,Report #7944
Diary,Knife,100.0,99.0,False,False,,
Diary,Knife,100.0,100.0,True,True,Genuine,Report #7945
Diary,Knife,100.0,100.0,True,True,Genuine,
Diary,Knife,100.0,100.0,True,True,Fake,Report #7946
Diary,Knife,100.0,100.0,True,True,Fake,
Diary,Knife,100.0,100.0,True,True,,Report #7947
Diary,Knife,100.0,100.0,True,True,,
Diary,Knife,100.0,100.0,True,False,Genuine,Report #7948
Diary,Knife,100.0,100.0,True,False,Genuine,
Diary,Knife,100.0,100.0,True,False,Fake,Report #7949
Diary,Knife,100.0,100.0,True,False,Fake,
Diary,Knife,100.0,100.0,True,False,,Report #7950
Diary,Knife,100.0,100.0,True,False,,
Diary,Knife,100.0,100.0,False,True,Genuine,Report #7951
Diary,Knife,100.0,100.0,False,True,Genuine,
Diary,Knife,100.0,100.0,False,True,Fake,Report #7952
Diary,Knife,100.0,100.0,False,True,Fake,
Diary,Knife,100.0,100.0,False,True,,Report #7953
Diary,Knife,100.0,100.0,False,True,,
Diary,Knife,100.0,100.0,False,False,Genuine,Report #7954
Diary,Knife,100.0,100.0,False,False,Genuine,
Diary,Knife,100.0,100.0,False,False,Fake,Report #7955
Diary,Knife,100.0,100.0,False,False,Fake,
Diary,Knife,100.0,100.0,False,False,,Report #7956
Diary,Knife,100.0,100.0,False,False,,
Diary,Knife,100.0,101.0,True,True,Genuine,Report #7957
Diary,Knife,100.0,101.0,True,True,Genuine,
Diary,Knife,100.0,101.0,True,True,Fake,Report #7958
Diary,Knife,100.0,101.0,True,True,Fake,
Diary,Knife,100.0,101.0,True,True,,Report #7959
Diary,Knife,100.0,101.0,True,True,,
Diary,Knife,100.0,101.0,True,False,Genuine,Report #7960
Diary,Knife,100.0,101.0,True,False,Genuine,
Diary,Knife,100.0,101.0,True,False,Fake,Report #7961
Diary,Knife,100.0,101.0,True,False,Fake,
Diary,Knife,100.0,101.0,True,False,,Report #7962
Diary,Knife,100.0,101.0,True,False,,
Diary,Knife,100.0,101.0,False,True,Genuine,Report #7963
Diary,Knife,100.0,101.0,False,True,Genuine,
Diary,Knife,100.0,101.0,False,True,Fake,Report #7964
Diary,Knife,100.0,101.0,False,True,Fake,
Diary,Knife,100.0,101.0,False,True,,Report #7965
Diary,Knife,100.0,101.0,False,True,,
Diary,Knife,100.0,101.0,False,False,Genuine,Report #7966
Diary,Knife,100.0,101.0,False,False,Genuine,
Diary,Knife,100.0,101.0,False,False,Fake,Report #7967
Diary,Knife,100.0,101.0,False,False,Fake,
Diary,Knife,100.0,101.0,False,False,,Report #7968
Diary,Knife,100.0,101.0,False,False,,
Diary,Knife,1000.0,-1.0,True,True,Genuine,Report #7969
Diary,Knife,1000.0,-1.0,True,True,Genuine,
Diary,Knife,1000.0,-1.0,True,True,Fake,Report #7970
Diary,Knife,1000.0,-1.0,True,True,Fake,
Diary,Knife,1000.0,-1.0,True,True,,Report #7971
Diary,Knife,1000.0,-1.0,True,True,,
Diary,Knife,1000.0,-1.0,True,False,Genuine,Report #7972
Diary,Knife,1000.0,-1.0,True,False,Genuine,
Diary,Knife,1000.0,-1.0,True,False,Fake,Report #7973
Diary,Knife,1000.0,-1.0,True,False,Fake,
Diary,Knife,1000.0,-1.0,True,False,,Report #7974
Diary,Knife,1000.0,-1.0,True,False,,
Diary,Knife,1000.0,-1.0,False,True,Genuine,Report #7975
Diary,Knife,1000.0,-1.0,False,True,Genuine,
Diary,Knife,1000.0,-1.0,False,True,Fake,Report #7976
Diary,Knife,1000.0,-1.0,False,True,Fake,
Diary,Knife,1000.0,-1.0,False,True,,Report #7977
Diary,Knife,1000.0,-1.0,False,True,,
Diary,Knife,1000.0,-1.0,False,False,Genuine,Report #7978
Diary,Knife,1000.0,-1.0,False,False,Genuine,
Diary,Knife,1000.0,-1.0,False,False,Fake,Report #7979
Diary,Knife,1000.0,-1.0,False,False,Fake,
Diary,Knife,1000.0,-1.0,False,False,,Report #7980
Diary,Knife,1000.0,-1.0,False,False,,
Diary,Knife,1000.0,0.0,True,True,Genuine,Report #7981
Diary,Knife,1000.0,0.0,True,True,Genuine,
Diary,Knife,1000.0,0.0,True,True,Fake,Report #7982
Diary,Knife,1000.0,0.0,True,True,Fake,
Diary,Knife,1000.0,0.0,True,True,,Report #7983
Diary,Knife,1000.0,0.0,True,True,,
Diary,Knife,1000.0,0.0,True,False,Genuine,Report #7984
Diary,Knife,1000.0,0.0,True,False,Genuine,
Diary,Knife,1000.0,0.0,True,False,Fake,Report #7985
Diary,Knife,1000.0,0.0,True,False,Fake,
Diary,Knife,1000.0,0.0,True,False,,Report #7986
Diary,Knife,1000.0,0.0,True,False,,
Diary,Knife,1000.0,0.0,False,True,Genuine,Report #7987
Diary,Knife,1000.0,0.0,False,True,Genuine,
Diary,Knife,1000.0,0.0,False,True,Fake,Report #7988
Diary,Knife,1000.0,0.0,False,True,Fake,
Diary,Knife,1000.0,0.0,False,True,,Report #7989
Diary,Knife,1000.0,0.0,False,True,,
Diary,Knife,1000.0,0.0,False,False,Genuine,Report #7990
Diary,Knife,1000.0,0.0,False,False,Genuine,
Diary,Knife,1000.0,0.0,False,False,Fake,Report #7991
Diary,Knife,1000.0,0.0,False,False,Fake,
Diary,Knife,1000.0,0.0,False,False,,Report #7992
Diary,Knife,1000.0,0.0,False,False,,
Diary,Knife,1000.0,19.0,True,True,Genuine,Report #7993
Diary,Knife,1000.0,19.0,True,True,Genuine,
Diary,Knife,1000.0,19.0,True,True,Fake,Report #7994
Diary,Knife,1000.0,19.0,True,True,Fake,
Diary,Knife,1000.0,19.0,True,True,,Report #7995
Diary,Knife,1000.0,19.0,True,True,,
Diary,Knife,1000.0,19.0,True,False,Genuine,Report #7996
Diary,Knife,1000.0,19.0,True,False,Genuine,
Diary,Knife,1000.0,19.0,True,False,Fake,Report #7997
Diary,Knife,1000.0,19.0,True,False,Fake,
Diary,Knife,1000.0,19.0,True,False,,Report #7998
Diary,Knife,1000.0,19.0,True,False,,
Diary,Knife,1000.0,19.0,False,True,Genuine,Report #7999
Diary,Knife,1000.0,19.0,False,True,Genuine,
Diary,Knife,1000.0,19.0,False,True,Fake,Report #8000
Diary,Knife,1000.0,19.0,False,True,Fake,
Diary,Knife,1000.0,19.0,False,True,,Report #8001
Diary,Knife,1000.0,19.0,False,True,,
Diary,Knife,1000.0,19.0,False,False,Genuine,Report #8002
Diary,Knife,1000.0,19.0,False,False,Genuine,
Diary,Knife,1000.0,19.0,False,False,Fake,Report #8003
Diary,Knife,1000.0,19.0,False,False,Fake,
Diary,Knife,1000.0,19.0,False,False,,Report #8004
Diary,Knife,1000.0,19.0,False,False,,
Diary,Knife,1000.0,20.0,True,True,Genuine,Report #8005
Diary,Knife,1000.0,20.0,True,True,Genuine,
Diary,Knife,1000.0,20.0,True,True,Fake,Report #8006
Diary,Knife,1000.0,20.0,True,True,Fake,
Diary,Knife,1000.0,20.0,True,True,,Report #8007
Diary,Knife,1000.0,20.0,True,True,,
Diary,Knife,1000.0,20.0,True,False,Genuine,Report #8008
Diary,Knife,1000.0,20.0,True,False,Genuine,
Diary,Knife,1000.0,20.0,True,False,Fake,Report #8009
Diary,Knife,1000.0,20.0,True,False,Fake,
Diary,Knife,1000.0,20.0,True,False,,Report #8010
Diary,Knife,1000.0,20.0,True,False,,
Diary,Knife,1000.0,20.0,False,True,Genuine,Report #8011
Diary,Knife,1000.0,20.0,False,True,Genuine,
Diary,Knife,1000.0,20.0,False,True,Fake,Report #8012
Diary,Knife,1000.0,20.0,False,True,Fake,
Diary,Knife,1000.0,20.0,False,True,,Report #8013
Diary,Knife,1000.0,20.0,False,True,,
Diary,Knife,1000.0,20.0,False,False,Genuine,Report #8014
Diary,Knife,1000.0,20.0,False,False,Genuine,
Diary,Knife,1000.0,20.0,False,False,Fake,Report #8015
Diary,Knife,1000.0,20.0,False,False,Fake,
Diary,Knife,1000.0,20.0,False,False,,Report #8016
Diary,Knife,1000.0,20.0,False,False,,
Diary,Knife,1000.0,21.0,True,True,Genuine,Report #8017
Diary,Knife,1000.0,21.0,True,True,Genuine,
Diary,Knife,1000.0,21.0,True,True,Fake,Report #8018
Diary,Knife,1000.0,21.0,True,True,Fake,
Diary,Knife,1000.0,21.0,True,True,,Report #8019
Diary,Knife,1000.0,21.0,True,True,,
Diary,Knife,1000.0,21.0,True,False,Genuine,Report #8020
Diary,Knife,1000.0,21.0,True,False,Genuine,
Diary,Knife,1000.0,21.0,True,False,Fake,Report #8021
Diary,Knife,1000.0,21.0,True,False,Fake,
Diary,Knife,1000.0,21.0,True,False,,Report #8022
Diary,Knife,1000.0,21.0,True,False,,
Diary,Knife,1000.0,21.0,False,True,Genuine,Report #8023
Diary,Knife,1000.0,21.0,False,True,Genuine,
Diary,Knife,1000.0,21.0,False,True,Fake,Report #8024
Diary,Knife,1000.0,21.0,False,True,Fake,
Diary,Knife,1000.0,21.0,False,True,,Report #8025
Diary,Knife,1000.0,21.0,False,True,,
Diary,Knife,1000.0,21.0,False,False,Genuine,Report #8026
Diary,Knife,1000.0,21.0,False,False,Genuine,
Diary,Knife,1000.0,21.0,False,False,Fake,Report #8027
Diary,Knife,1000.0,21.0,False,False,Fake,
Diary,Knife,1000.0,21.0,False,False,,Report #8028
Diary,Knife,1000.0,21.0,False,False,,
Diary,Knife,1000.0,99.0,True,True,Genuine,Report #8029
Diary,Knife,1000.0,99.0,True,True,Genuine,
Diary,Knife,1000.0,99.0,True,True,Fake,Report #8030
Diary,Knife,1000.0,99.0,True,True,Fake,
Diary,Knife,1000.0,99.0,True,True,,Report #8031
Diary,Knife,1000.0,99.0,True,True,,
Diary,Knife,1000.0,99.0,True,False,Genuine,Report #8032
Diary,Knife,1000.0,99.0,True,False,Genuine,
Diary,Knife,1000.0,99.0,True,False,Fake,Report #8033
Diary,Knife,1000.0,99.0,True,False,Fake,
Diary,Knife,1000.0,99.0,True,False,,Report #8034
Diary,Knife,1000.0,99.0,True,False,,
Diary,Knife,1000.0,99.0,False,True,Genuine,Report #8035
Diary,Knife,1000.0,99.0,False,True,Genuine,
Diary,Knife,1000.0,99.0,False,True,Fake,Report #8036
Diary,Knife,1000.0,99.0,False,True,Fake,
Diary,Knife,1000.0,99.0,False,True,,Report #8037
Diary,Knife,1000.0,99.0,False,True,,
Diary,Knife,1000.0,99.0,False,False,Genuine,Report #8038
Diary,Knife,1000.0,99.0,False,False,Genuine,
Diary,Knife,1000.0,99.0,False,False,Fake,Report #8039
Diary,Knife,1000.0,99.0,False,False,Fake,
Diary,Knife,1000.0,99.0,False,False,,Report #8040
Diary,Knife,1000.0,99.0,False,False,,
Diary,Knife,1000.0,100.0,True,True,Genuine,Report #8041
Diary,Knife,1000.0,100.0,True,True,Genuine,
Diary,Knife,1000.0,100.0,True,True,Fake,Report #8042
Diary,Knife,1000.0,100.0,True,True,Fake,
Diary,Knife,1000.0,100.0,True,True,,Report #8043
Diary,Knife,1000.0,100.0,True,True,,
Diary,Knife,1000.0,100.0,True,False,Genuine,Report #8044
Diary,Knife,1000.0,100.0,True,False,Genuine,
Diary,Knife,1000.0,100.0,True,False,Fake,Report #8045
Diary,Knife,1000.0,100.0,True,False,Fake,
Diary,Knife,1000.0,100.0,True,False,,Report #8046
Diary,Knife,1000.0,100.0,True,False,,
Diary,Knife,1000.0,100.0,False,True,Genuine,Report #8047
Diary,Knife,1000.0,100.0,False,True,Genuine,
Diary,Knife,1000.0,100.0,False,True,Fake,Report #8048
Diary,Knife,1000.0,100.0,False,True,Fake,
Diary,Knife,1000.0,100.0,False,True,,Report #8049
Diary,Knife,1000.0,100.0,False,True,,
Diary,Knife,1000.0,100.0,False,False,Genuine,Report #8050
Diary,Knife,1000.0,100.0,False,False,Genuine,
Diary,Knife,1000.0,100.0,False,False,Fake,Report #8051
Diary,Knife,1000.0,100.0,False,False,Fake,
Diary,Knife,1000.0,100.0,False,False,,Report #8052
Diary,Knife,1000.0,100.0,False,False,,
Diary,Knife,1000.0,101.0,True,True,Genuine,Report #8053
Diary,Knife,1000.0,101.0,True,True,Genuine,
Diary,Knife,1000.0,101.0,True,True,Fake,Report #8054
Diary,Knife,1000.0,101.0,True,True,Fake,
Diary,Knife,1000.0,101.0,True,True,,Report #8055
Diary,Knife,1000.0,101.0,True,True,,
Diary,Knife,1000.0,101.0,True,False,Genuine,Report #8056
Diary,Knife,1000.0,101.0,True,False,Genuine,
Diary,Knife,1000.0,101.0,True,False,Fake,Report #8057
Diary,Knife,1000.0,101.0,True,False,Fake,
Diary,Knife,1000.0,101.0,True,False,,Report #8058
Diary,Knife,1000.0,101.0,True,False,,
Diary,Knife,1000.0,101.0,False,True,Genuine,Report #8059
Diary,Knife,1000.0,101.0,False,True,Genuine,
Diary,Knife,1000.0,101.0,False,True,Fake,Report #8060
Diary,Knife,1000.0,101.0,False,True,Fake,
Diary,Knife,1000.0,101.0,False,True,,Report #8061
Diary,Knife,1000.0,101.0,False,True,,
Diary,Knife,1000.0,101.0,False,False,Genuine,Report #8062
Diary,Knife,1000.0,101.0,False,False,Genuine,
Diary,Knife,1000.0,101.0,False,False,Fake,Report #8063
Diary,Knife,1000.0,101.0,False,False,Fake,
Diary,Knife,1000.0,101.0,False,False,,Report #8064
Diary,Knife,1000.0,101.0,False,False,,
Diary,Knife,1000000.0,-1.0,True,True,Genuine,Report #8065
Diary,Knife,1000000.0,-1.0,True,True,Genuine,
Diary,Knife,1000000.0,-1.0,True,True,Fake,Report #8066
Diary,Knife,1000000.0,-1.0,True,True,Fake,
Diary,Knife,1000000.0,-1.0,True,True,,Report #8067
Diary,Knife,1000000.0,-1.0,True,True,,
Diary,Knife,1000000.0,-1.0,True,False,Genuine,Report #8068
Diary,Knife,1000000.0,-1.0,True,False,Genuine,
Diary,Knife,1000000.0,-1.0,True,False,Fake,Report #8069
Diary,Knife,1000000.0,-1.0,True,False,Fake,
Diary,Knife,1000000.0,-1.0,True,False,,Report #8070
Diary,Knife,1000000.0,-1.0,True,False,,
Diary,Knife,1000000.0,-1.0,False,True,Genuine,Report #8071
Diary,Knife,1000000.0,-1.0,False,True,Genuine,
Diary,Knife,1000000.0,-1.0,False,True,Fake,Report #8072
Diary,Knife,1000000.0,-1.0,False,True,Fake,
Diary,Knife,1000000.0,-1.0,False,True,,Report #8073
Diary,Knife,1000000.0,-1.0,False,True,,
Diary,Knife,1000000.0,-1.0,False,False,Genuine,Report #8074
Diary,Knife,1000000.0,-1.0,False,False,Genuine,
Diary,Knife,1000000.0,-1.0,False,False,Fake,Report #8075
Diary,Knife,1000000.0,-1.0,False,False,Fake,
Diary,Knife,1000000.0,-1.0,False,False,,Report #8076
Diary,Knife,1000000.0,-1.0,False,False,,
Diary,Knife,1000000.0,0.0,True,True,Genuine,Report #8077
Diary,Knife,1000000.0,0.0,True,True,Genuine,
Diary,Knife,1000000.0,0.0,True,True,Fake,Report #8078
Diary,Knife,1000000.0,0.0,True,True,Fake,
Diary,Knife,1000000.0,0.0,True,True,,Report #8079
Diary,Knife,1000000.0,0.0,True,True,,
Diary,Knife,1000000.0,0.0,True,False,Genuine,Report #8080
Diary,Knife,1000000.0,0.0,True,False,Genuine,
Diary,Knife,1000000.0,0.0,True,False,Fake,Report #8081
Diary,Knife,1000000.0,0.0,True,False,Fake,
Diary,Knife,1000000.0,0.0,True,False,,Report #8082
Diary,Knife,1000000.0,0.0,True,False,,
Diary,Knife,1000000.0,0.0,False,True,Genuine,Report #8083
Diary,Knife,1000000.0,0.0,False,True,Genuine,
Diary,Knife,1000000.0,0.0,False,True,Fake,Report #8084
Diary,Knife,1000000.0,0.0,False,True,Fake,
Diary,Knife,1000000.0,0.0,False,True,,Report #8085
Diary,Knife,1000000.0,0.0,False,True,,
Diary,Knife,1000000.0,0.0,False,False,Genuine,Report #8086
Diary,Knife,1000000.0,0.0,False,False,Genuine,
Diary,Knife,1000000.0,0.0,False,False,Fake,Report #8087
Diary,Knife,1000000.0,0.0,False,False,Fake,
Diary,Knife,1000000.0,0.0,False,False,,Report #8088
Diary,Knife,1000000.0,0.0,False,False,,
Diary,Knife,1000000.0,19.0,True,True,Genuine,Report #8089
Diary,Knife,1000000.0,19.0,True,True,Genuine,
Diary,Knife,1000000.0,19.0,True,True,Fake,Report #8090
Diary,Knife,1000000.0,19.0,True,True,Fake,
Diary,Knife,1000000.0,19.0,True,True,,Report #8091
Diary,Knife,1000000.0,19.0,True,True,,
Diary,Knife,1000000.0,19.0,True,False,Genuine,Report #8092
Diary,Knife,1000000.0,19.0,True,False,Genuine,
Diary,Knife,1000000.0,19.0,True,False,Fake,Report #8093
Diary,Knife,1000000.0,19.0,True,False,Fake,
Diary,Knife,1000000.0,19.0,True,False,,Report #8094
Diary,Knife,1000000.0,19.0,True,False,,
Diary,Knife,1000000.0,19.0,False,True,Genuine,Report #8095
Diary,Knife,1000000.0,19.0,False,True,Genuine,
Diary,Knife,1000000.0,19.0,False,True,Fake,Report #8096
Diary,Knife,1000000.0,19.0,False,True,Fake,
Diary,Knife,1000000.0,19.0,False,True,,Report #8097
Diary,Knife,1000000.0,19.0,False,True,,
Diary,Knife,1000000.0,19.0,False,False,Genuine,Report #8098
Diary,Knife,1000000.0,19.0,False,False,Genuine,
Diary,Knife,1000000.0,19.0,False,False,Fake,Report #8099
Diary,Knife,1000000.0,19.0,False,False,Fake,
Diary,Knife,1000000.0,19.0,False,False,,Report #8100
Diary,Knife,1000000.0,19.0,False,False,,
Diary,Knife,1000000.0,20.0,True,True,Genuine,Report #8101
Diary,Knife,1000000.0,20.0,True,True,Genuine,
Diary,Knife,1000000.0,20.0,True,True,Fake,Report #8102
Diary,Knife,1000000.0,20.0,True,True,Fake,
Diary,Knife,1000000.0,20.0,True,True,,Report #8103
Diary,Knife,1000000.0,20.0,True,True,,
Diary,Knife,1000000.0,20.0,True,False,Genuine,Report #8104
Diary,Knife,1000000.0,20.0,True,False,Genuine,
Diary,Knife,1000000.0,20.0,True,False,Fake,Report #8105
Diary,Knife,1000000.0,20.0,True,False,Fake,
Diary,Knife,1000000.0,20.0,True,False,,Report #8106
Diary,Knife,1000000.0,20.0,True,False,,
Diary,Knife,1000000.0,20.0,False,True,Genuine,Report #8107
Diary,Knife,1000000.0,20.0,False,True,Genuine,
Diary,Knife,1000000.0,20.0,False,True,Fake,Report #8108
Diary,Knife,1000000.0,20.0,False,True,Fake,
Diary,Knife,1000000.0,20.0,False,True,,Report #8109
Diary,Knife,1000000.0,20.0,False,True,,
Diary,Knife,1000000.0,20.0,False,False,Genuine,Report #8110
Diary,Knife,1000000.0,20.0,False,False,Genuine,
Diary,Knife,1000000.0,20.0,False,False,Fake,Report #8111
Diary,Knife,1000000.0,20.0,False,False,Fake,
Diary,Knife,1000000.0,20.0,False,False,,Report #8112
Diary,Knife,1000000.0,20.0,False,False,,
Diary,Knife,1000000.0,21.0,True,True,Genuine,Report #8113
Diary,Knife,1000000.0,21.0,True,True,Genuine,
Diary,Knife,1000000.0,21.0,True,True,Fake,Report #8114
Diary,Knife,1000000.0,21.0,True,True,Fake,
Diary,Knife,1000000.0,21.0,True,True,,Report #8115
Diary,Knife,1000000.0,21.0,True,True,,
Diary,Knife,1000000.0,21.0,True,False,Genuine,Report #8116
Diary,Knife,1000000.0,21.0,True,False,Genuine,
Diary,Knife,1000000.0,21.0,True,False,Fake,Report #8117
Diary,Knife,1000000.0,21.0,True,False,Fake,
Diary,Knife,1000000.0,21.0,True,False,,Report #8118
Diary,Knife,1000000.0,21.0,True,False,,
Diary,Knife,1000000.0,21.0,False,True,Genuine,Report #8119
Diary,Knife,1000000.0,21.0,False,True,Genuine,
Diary,Knife,1000000.0,21.0,False,True,Fake,Report #8120
Diary,Knife,1000000.0,21.0,False,True,Fake,
Diary,Knife,1000000.0,21.0,False,True,,Report #8121
Diary,Knife,1000000.0,21.0,False,True,,
Diary,Knife,1000000.0,21.0,False,False,Genuine,Report #8122
Diary,Knife,1000000.0,21.0,False,False,Genuine,
Diary,Knife,1000000.0,21.0,False,False,Fake,Report #8123
Diary,Knife,1000000.0,21.0,False,False,Fake,
Diary,Knife,1000000.0,21.0,False,False,,Report #8124
Diary,Knife,1000000.0,21.0,False,False,,
Diary,Knife,1000000.0,99.0,True,True,Genuine,Report #8125
Diary,Knife,1000000.0,99.0,True,True,Genuine,
Diary,Knife,1000000.0,99.0,True,True,Fake,Report #8126
Diary,Knife,1000000.0,99.0,True,True,Fake,
Diary,Knife,1000000.0,99.0,True,True,,Report #8127
Diary,Knife,1000000.0,99.0,True,True,,
Diary,Knife,1000000.0,99.0,True,False,Genuine,Report #8128
Diary,Knife,1000000.0,99.0,True,False,Genuine,
Diary,Knife,1000000.0,99.0,True,False,Fake,Report #8129
Diary,Knife,1000000.0,99.0,True,False,Fake,
Diary,Knife,1000000.0,99.0,True,False,,Report #8130
Diary,Knife,1000000.0,99.0,True,False,,
Diary,Knife,1000000.0,99.0,False,True,Genuine,Report #8131
Diary,Knife,1000000.0,99.0,False,True,Genuine,
Diary,Knife,1000000.0,99.0,False,True,Fake,Report #8132
Diary,Knife,1000000.0,99.0,False,True,Fake,
Diary,Knife,1000000.0,99.0,False,True,,Report #8133
Diary,Knife,1000000.0,99.0,False,True,,
Diary,Knife,1000000.0,99.0,False,False,Genuine,Report #8134
Diary,Knife,1000000.0,99.0,False,False,Genuine,
Diary,Knife,1000000.0,99.0,False,False,Fake,Report #8135
Diary,Knife,1000000.0,99.0,False,False,Fake,
Diary,Knife,1000000.0,99.0,False,False,,Report #8136
Diary,Knife,1000000.0,99.0,False,False,,
Diary,Knife,1000000.0,100.0,True,True,Genuine,Report #8137
Diary,Knife,1000000.0,100.0,True,True,Genuine,
Diary,Knife,1000000.0,100.0,True,True,Fake,Report #8138
Diary,Knife,1000000.0,100.0,True,True,Fake,
Diary,Knife,1000000.0,100.0,True,True,,Report #8139
Diary,Knife,1000000.0,100.0,True,True,,
Diary,Knife,1000000.0,100.0,True,False,Genuine,Report #8140
Diary,Knife,1000000.0,100.0,True,False,Genuine,
Diary,Knife,1000000.0,100.0,True,False,Fake,Report #8141
Diary,Knife,1000000.0,100.0,True,False,Fake,
Diary,Knife,1000000.0,100.0,True,False,,Report #8142
Diary,Knife,1000000.0,100.0,True,False,,
Diary,Knife,1000000.0,100.0,False,True,Genuine,Report #8143
Diary,Knife,1000000.0,100.0,False,True,Genuine,
Diary,Knife,1000000.0,100.0,False,True,Fake,Report #8144
Diary,Knife,1000000.0,100.0,False,True,Fake,
Diary,Knife,1000000.0,100.0,False,True,,Report #8145
Diary,Knife,1000000.0,100.0,False,True,,
Diary,Knife,1000000.0,100.0,False,False,Genuine,Report #8146
Diary,Knife,1000000.0,100.0,False,False,Genuine,
Diary,Knife,1000000.0,100.0,False,False,Fake,Report #8147
Diary,Knife,1000000.0,100.0,False,False,Fake,
Diary,Knife,1000000.0,100.0,False,False,,Report #8148
Diary,Knife,1000000.0,100.0,False,False,,
Diary,Knife,1000000.0,101.0,True,True,Genuine,Report #8149
Diary,Knife,1000000.0,101.0,True,True,Genuine,
Diary,Knife,1000000.0,101.0,True,True,Fake,Report #8150
Diary,Knife,1000000.0,101.0,True,True,Fake,
Diary,Knife,1000000.0,101.0,True,True,,Report #8151
Diary,Knife,1000000.0,101.0,True,True,,
Diary,Knife,1000000.0,101.0,True,False,Genuine,Report #8152
Diary,Knife,1000000.0,101.0,True,False,Genuine,
Diary,Knife,1000000.0,101.0,True,False,Fake,Report #8153
Diary,Knife,1000000.0,101.0,True,False,Fake,
Diary,Knife,1000000.0,101.0,True,False,,Report #8154
Diary,Knife,1000000.0,101.0,True,False,,
Diary,Knife,1000000.0,101.0,False,True,Genuine,Report #8155
Diary,Knife,1000000.0,101.0,False,True,Genuine,
Diary,Knife,1000000.0,101.0,False,True,Fake,Report #8156
Diary,Knife,1000000.0,101.0,False,True,Fake,
Diary,Knife,1000000.0,101.0,False,True,,Report #8157
Diary,Knife,1000000.0,101.0,False,True,,
Diary,Knife,1000000.0,101.0,False,False,Genuine,Report #8158
Diary,Knife,1000000.0,101.0,False,False,Genuine,
Diary,Knife,1000000.0,101.0,False,False,Fake,Report #8159
Diary,Knife,1000000.0,101.0,False,False,Fake,
Diary,Knife,1000000.0,101.0,False,False,,Report #8160
Diary,Knife,1000000.0,101.0,False,False,,
Diary,Plates,0.0,-1.0,True,True,Genuine,Report #8161
Diary,Plates,0.0,-1.0,True,True,Genuine,
Diary,Plates,0.0,-1.0,True,True,Fake,Report #8162
Diary,Plates,0.0,-1.0,True,True,Fake,
Diary,Plates,0.0,-1.0,True,True,,Report #8163
Diary,Plates,0.0,-1.0,True,True,,
Diary,Plates,0.0,-1.0,True,False,Genuine,Report #8164
Diary,Plates,0.0,-1.0,True,False,Genuine,
Diary,Plates,0.0,-1.0,True,False,Fake,Report #8165
Diary,Plates,0.0,-1.0,True,False,Fake,
Diary,Plates,0.0,-1.0,True,False,,Report #8166
Diary,Plates,0.0,-1.0,True,False,,
Diary,Plates,0.0,-1.0,False,True,Genuine,Report #8167
Diary,Plates,0.0,-1.0,False,True,Genuine,
Diary,Plates,0.0,-1.0,False,True,Fake,Report #8168
Diary,Plates,0.0,-1.0,False,True,Fake,
Diary,Plates,0.0,-1.0,False,True,,Report #8169
Diary,Plates,0.0,-1.0,False,True,,
Diary,Plates,0.0,-1.0,False,False,Genuine,Report #8170
Diary,Plates,0.0,-1.0,False,False,Genuine,
Diary,Plates,0.0,-1.0,False,False,Fake,Report #8171
Diary,Plates,0.0,-1.0,False,False,Fake,
Diary,Plates,0.0,-1.0,False,False,,Report #8172
Diary,Plates,0.0,-1.0,False,False,,
Diary,Plates,0.0,0.0,True,True,Genuine,Report #8173
Diary,Plates,0.0,0.0,True,True,Genuine,
Diary,Plates,0.0,0.0,True,True,Fake,Report #8174
Diary,Plates,0.0,0.0,True,True,Fake,
Diary,Plates,0.0,0.0,True,True,,Report #8175
Diary,Plates,0.0,0.0,True,True,,
Diary,Plates,0.0,0.0,True,False,Genuine,Report #8176
Diary,Plates,0.0,0.0,True,False,Genuine,
Diary,Plates,0.0,0.0,True,False,Fake,Report #8177
Diary,Plates,0.0,0.0,True,False,Fake,
Diary,Plates,0.0,0.0,True,False,,Report #8178
Diary,Plates,0.0,0.0,True,False,,
Diary,Plates,0.0,0.0,False,True,Genuine,Report #8179
Diary,Plates,0.0,0.0,False,True,Genuine,
Diary,Plates,0.0,0.0,False,True,Fake,Report #8180
Diary,Plates,0.0,0.0,False,True,Fake,
Diary,Plates,0.0,0.0,False,True,,Report #8181
Diary,Plates,0.0,0.0,False,True,,
Diary,Plates,0.0,0.0,False,False,Genuine,Report #8182
Diary,Plates,0.0,0.0,False,False,Genuine,
Diary,Plates,0.0,0.0,False,False,Fake,Report #8183
Diary,Plates,0.0,0.0,False,False,Fake,
Diary,Plates,0.0,0.0,False,False,,Report #8184
Diary,Plates,0.0,0.0,False,False,,
Diary,Plates,0.0,19.0,True,True,Genuine,Report #8185
Diary,Plates,0.0,19.0,True,True,Genuine,
Diary,Plates,0.0,19.0,True,True,Fake,Report #8186
Diary,Plates,0.0,19.0,True,True,Fake,
Diary,Plates,0.0,19.0,True,True,,Report #8187
Diary,Plates,0.0,19.0,True,True,,
Diary,Plates,0.0,19.0,True,False,Genuine,Report #8188
Diary,Plates,0.0,19.0,True,False,Genuine,
Diary,Plates,0.0,19.0,True,False,Fake,Report #8189
Diary,Plates,0.0,19.0,True,False,Fake,
Diary,Plates,0.0,19.0,True,False,,Report #8190
Diary,Plates,0.0,19.0,True,False,,
Diary,Plates,0.0,19.0,False,True,Genuine,Report #8191
Diary,Plates,0.0,19.0,False,True,Genuine,
Diary,Plates,0.0,19.0,False,True,Fake,Report #8192
Diary,Plates,0.0,19.0,False,True,Fake,
Diary,Plates,0.0,19.0,False,True,,Report #8193
Diary,Plates,0.0,19.0,False,True,,
Diary,Plates,0.0,19.0,False,False,Genuine,Report #8194
Diary,Plates,0.0,19.0,False,False,Genuine,
Diary,Plates,0.0,19.0,False,False,Fake,Report #8195
Diary,Plates,0.0,19.0,False,False,Fake,
Diary,Plates,0.0,19.0,False,False,,Report #8196
Diary,Plates,0.0,19.0,False,False,,
Diary,Plates,0.0,20.0,True,True,Genuine,Report #8197
Diary,Plates,0.0,20.0,True,True,Genuine,
Diary,Plates,0.0,20.0,True,True,Fake,Report #8198
Diary,Plates,0.0,20.0,True,True,Fake,
Diary,Plates,0.0,20.0,True,True,,Report #8199
Diary,Plates,0.0,20.0,True,True,,
Diary,Plates,0.0,20.0,True,False,Genuine,Report #8200
Diary,Plates,0.0,20.0,True,False,Genuine,
Diary,Plates,0.0,20.0,True,False,Fake,Report #8201
Diary,Plates,0.0,20.0,True,False,Fake,
Diary,Plates,0.0,20.0,True,False,,Report #8202
Diary,Plates,0.0,20.0,True,False,,
Diary,Plates,0.0,20.0,False,True,Genuine,Report #8203
Diary,Plates,0.0,20.0,False,True,Genuine,
Diary,Plates,0.0,20.0,False,True,Fake,Report #8204
Diary,Plates,0.0,20.0,False,True,Fake,
Diary,Plates,0.0,20.0,False,True,,Report #8205
Diary,Plates,0.0,20.0,False,True,,
Diary,Plates,0.0,20.0,False,False,Genuine,Report #8206
Diary,Plates,0.0,20.0,False,False,Genuine,
Diary,Plates,0.0,20.0,False,False,Fake,Report #8207
Diary,Plates,0.0,20.0,False,False,Fake,
Diary,Plates,0.0,20.0,False,False,,Report #8208
Diary,Plates,0.0,20.0,False,False,,
Diary,Plates,0.0,21.0,True,True,Genuine,Report #8209
Diary,Plates,0.0,21.0,True,True,Genuine,
Diary,Plates,0.0,21.0,True,True,Fake,Report #8210
Diary,Plates,0.0,21.0,True,True,Fake,
Diary,Plates,0.0,21.0,True,True,,Report #8211
Diary,Plates,0.0,21.0,True,True,,
Diary,Plates,0.0,21.0,True,False,Genuine,Report #8212
Diary,Plates,0.0,21.0,True,False,Genuine,
Diary,Plates,0.0,21.0,True,False,Fake,Report #8213
Diary,Plates,0.0,21.0,True,False,Fake,
Diary,Plates,0.0,21.0,True,False,,Report #8214
Diary,Plates,0.0,21.0,True,False,,
Diary,Plates,0.0,21.0,False,True,Genuine,Report #8215
Diary,Plates,0.0,21.0,False,True,Genuine,
Diary,Plates,0.0,21.0,False,True,Fake,Report #8216
Diary,Plates,0.0,21.0,False,True,Fake,
Diary,Plates,0.0,21.0,False,True,,Report #8217
Diary,Plates,0.0,21.0,False,True,,
Diary,Plates,0.0,21.0,False,False,Genuine,Report #8218
Diary,Plates,0.0,21.0,False,False,Genuine,
Diary,Plates,0.0,21.0,False,False,Fake,Report #8219
Diary,Plates,0.0,21.0,False,False,Fake,
Diary,Plates,0.0,21.0,False,False,,Report #8220
Diary,Plates,0.0,21.0,False,False,,
Diary,Plates,0.0,99.0,True,True,Genuine,Report #8221
Diary,Plates,0.0,99.0,True,True,Genuine,
Diary,Plates,0.0,99.0,True,True,Fake,Report #8222
Diary,Plates,0.0,99.0,True,True,Fake,
Diary,Plates,0.0,99.0,True,True,,Report #8223
Diary,Plates,0.0,99.0,True,True,,
Diary,Plates,0.0,99.0,True,False,Genuine,Report #8224
Diary,Plates,0.0,99.0,True,False,Genuine,
Diary,Plates,0.0,99.0,True,False,Fake,Report #8225
Diary,Plates,0.0,99.0,True,False,Fake,
Diary,Plates,0.0,99.0,True,False,,Report #8226
Diary,Plates,0.0,99.0,True,False,,
Diary,Plates,0.0,99.0,False,True,Genuine,Report #8227
Diary,Plates,0.0,99.0,False,True,Genuine,
Diary,Plates,0.0,99.0,False,True,Fake,Report #8228
Diary,Plates,0.0,99.0,False,True,Fake,
Diary,Plates,0.0,99.0,False,True,,Report #8229
Diary,Plates,0.0,99.0,False,True,,
Diary,Plates,0.0,99.0,False,False,Genuine,Report #8230
Diary,Plates,0.0,99.0,False,False,Genuine,
Diary,Plates,0.0,99.0,False,False,Fake,Report #8231
Diary,Plates,0.0,99.0,False,False,Fake,
Diary,Plates,0.0,99.0,False,False,,Report #8232
Diary,Plates,0.0,99.0,False,False,,
Diary,Plates,0.0,100.0,True,True,Genuine,Report #8233
Diary,Plates,0.0,100.0,True,True,Genuine,
Diary,Plates,0.0,100.0,True,True,Fake,Report #8234
Diary,Plates,0.0,100.0,True,True,Fake,
Diary,Plates,0.0,100.0,True,True,,Report #8235
Diary,Plates,0.0,100.0,True,True,,
Diary,Plates,0.0,100.0,True,False,Genuine,Report #8236
Diary,Plates,0.0,100.0,True,False,Genuine,
Diary,Plates,0.0,100.0,True,False,Fake,Report #8237
Diary,Plates,0.0,100.0,True,False,Fake,
Diary,Plates,0.0,100.0,True,False,,Report #8238
Diary,Plates,0.0,100.0,True,False,,
Diary,Plates,0.0,100.0,False,True,Genuine,Report #8239
Diary,Plates,0.0,100.0,False,True,Genuine,
Diary,Plates,0.0,100.0,False,True,Fake,Report #8240
Diary,Plates,0.0,100.0,False,True,Fake,
Diary,Plates,0.0,100.0,False,True,,Report #8241
Diary,Plates,0.0,100.0,False,True,,
Diary,Plates,0.0,100.0,False,False,Genuine,Report #8242
Diary,Plates,0.0,100.0,False,False,Genuine,
Diary,Plates,0.0,100.0,False,False,Fake,Report #8243
Diary,Plates,0.0,100.0,False,False,Fake,
Diary,Plates,0.0,100.0,False,False,,Report #8244
Diary,Plates,0.0,100.0,False,False,,
Diary,Plates,0.0,101.0,True,True,Genuine,Report #8245
Diary,Plates,0.0,101.0,True,True,Genuine,
Diary,Plates,0.0,101.0,True,True,Fake,Report #8246
Diary,Plates,0.0,101.0,True,True,Fake,
Diary,Plates,0.0,101.0,True,True,,Report #8247
Diary,Plates,0.0,101.0,True,True,,
Diary,Plates,0.0,101.0,True,False,Genuine,Report #8248
Diary,Plates,0.0,101.0,True,False,Genuine,
Diary,Plates,0.0,101.0,True,False,Fake,Report #8249
Diary,Plates,0.0,101.0,True,False,Fake,
Diary,Plates,0.0,101.0,True,False,,Report #8250
Diary,Plates,0.0,101.0,True,False,,
Diary,Plates,0.0,101.0,False,True,Genuine,Report #8251
Diary,Plates,0.0,101.0,False,True,Genuine,
Diary,Plates,0.0,101.0,False,True,Fake,Report #8252
Diary,Plates,0.0,101.0,False,True,Fake,
Diary,Plates,0.0,101.0,False,True,,Report #8253
Diary,Plates,0.0,101.0,False,True,,
Diary,Plates,0.0,101.0,False,False,Genuine,Report #8254
Diary,Plates,0.0,101.0,False,False,Genuine,
Diary,Plates,0.0,101.0,False,False,Fake,Report #8255
Diary,Plates,0.0,101.0,False,False,Fake,
Diary,Plates,0.0,101.0,False,False,,Report #8256
Diary,Plates,0.0,101.0,False,False,,
Diary,Plates,1.0,-1.0,True,True,Genuine,Report #8257
Diary,Plates,1.0,-1.0,True,True,Genuine,
Diary,Plates,1.0,-1.0,True,True,Fake,Report #8258
Diary,Plates,1.0,-1.0,True,True,Fake,
Diary,Plates,1.0,-1.0,True,True,,Report #8259
Diary,Plates,1.0,-1.0,True,True,,
Diary,Plates,1.0,-1.0,True,False,Genuine,Report #8260
Diary,Plates,1.0,-1.0,True,False,Genuine,
Diary,Plates,1.0,-1.0,True,False,Fake,Report #8261
Diary,Plates,1.0,-1.0,True,False,Fake,
Diary,Plates,1.0,-1.0,True,False,,Report #8262
Diary,Plates,1.0,-1.0,True,False,,
Diary,Plates,1.0,-1.0,False,True,Genuine,Report #8263
Diary,Plates,1.0,-1.0,False,True,Genuine,
Diary,Plates,1.0,-1.0,False,True,Fake,Report #8264
Diary,Plates,1.0,-1.0,False,True,Fake,
Diary,Plates,1.0,-1.0,False,True,,Report #8265
Diary,Plates,1.0,-1.0,False,True,,
Diary,Plates,1.0,-1.0,False,False,Genuine,Report #8266
Diary,Plates,1.0,-1.0,False,False,Genuine,
Diary,Plates,1.0,-1.0,False,False,Fake,Report #8267
Diary,Plates,1.0,-1.0,False,False,Fake,
Diary,Plates,1.0,-1.0,False,False,,Report #8268
Diary,Plates,1.0,-1.0,False,False,,
Diary,Plates,1.0,0.0,True,True,Genuine,Report #8269
Diary,Plates,1.0,0.0,True,True,Genuine,
Diary,Plates,1.0,0.0,True,True,Fake,Report #8270
Diary,Plates,1.0,0.0,True,True,Fake,
Diary,Plates,1.0,0.0,True,True,,Report #8271
Diary,Plates,1.0,0.0,True,True,,
Diary,Plates,1.0,0.0,True,False,Genuine,Report #8272
Diary,Plates,1.0,0.0,True,False,Genuine,
Diary,Plates,1.0,0.0,True,False,Fake,Report #8273
Diary,Plates,1.0,0.0,True,False,Fake,
Diary,Plates,1.0,0.0,True,False,,Report #8274
Diary,Plates,1.0,0.0,True,False,,
Diary,Plates,1.0,0.0,False,True,Genuine,Report #8275
Diary,Plates,1.0,0.0,False,True,Genuine,
Diary,Plates,1.0,0.0,False,True,Fake,Report #8276
Diary,Plates,1.0,0.0,False,True,Fake,
Diary,Plates,1.0,0.0,False,True,,Report #8277
Diary,Plates,1.0,0.0,False,True,,
Diary,Plates,1.0,0.0,False,False,Genuine,Report #8278
Diary,Plates,1.0,0.0,False,False,Genuine,
Diary,Plates,1.0,0.0,False,False,Fake,Report #8279
Diary,Plates,1.0,0.0,False,False,Fake,
Diary,Plates,1.0,0.0,False,False,,Report #8280
Diary,Plates,1.0,0.0,False,False,,
Diary,Plates,1.0,19.0,True,True,Genuine,Report #8281
Diary,Plates,1.0,19.0,True,True,Genuine,
Diary,Plates,1.0,19.0,True,True,Fake,Report #8282
Diary,Plates,1.0,19.0,True,True,Fake,
Diary,Plates,1.0,19.0,True,True,,Report #8283
Diary,Plates,1.0,19.0,True,True,,
Diary,Plates,1.0,19.0,True,False,Genuine,Report #8284
Diary,Plates,1.0,19.0,True,False,Genuine,
Diary,Plates,1.0,19.0,True,False,Fake,Report #8285
Diary,Plates,1.0,19.0,True,False,Fake,
Diary,Plates,1.0,19.0,True,False,,Report #8286
Diary,Plates,1.0,19.0,True,False,,
Diary,Plates,1.0,19.0,False,True,Genuine,Report #8287
Diary,Plates,1.0,19.0,False,True,Genuine,
Diary,Plates,1.0,19.0,False,True,Fake,Report #8288
Diary,Plates,1.0,19.0,False,True,Fake,
Diary,Plates,1.0,19.0,False,True,,Report #8289
Diary,Plates,1.0,19.0,False,True,,
Diary,Plates,1.0,19.0,False,False,Genuine,Report #8290
Diary,Plates,1.0,19.0,False,False,Genuine,
Diary,Plates,1.0,19.0,False,False,Fake,Report #8291
Diary,Plates,1.0,19.0,False,False,Fake,
Diary,Plates,1.0,19.0,False,False,,Report #8292
Diary,Plates,1.0,19.0,False,False,,
Diary,Plates,1.0,20.0,True,True,Genuine,Report #8293
Diary,Plates,1.0,20.0,True,True,Genuine,
Diary,Plates,1.0,20.0,True,True,Fake,Report #8294
Diary,Plates,1.0,20.0,True,True,Fake,
Diary,Plates,1.0,20.0,True,True,,Report #8295
Diary,Plates,1.0,20.0,True,True,,
Diary,Plates,1.0,20.0,True,False,Genuine,Report #8296
Diary,Plates,1.0,20.0,True,False,Genuine,
Diary,Plates,1.0,20.0,True,False,Fake,Report #8297
Diary,Plates,1.0,20.0,True,False,Fake,
Diary,Plates,1.0,20.0,True,False,,Report #8298
Diary,Plates,1.0,20.0,True,False,,
Diary,Plates,1.0,20.0,False,True,Genuine,Report #8299
Diary,Plates,1.0,20.0,False,True,Genuine,
Diary,Plates,1.0,20.0,False,True,Fake,Report #8300
Diary,Plates,1.0,20.0,False,True,Fake,
Diary,Plates,1.0,20.0,False,True,,Report #8301
Diary,Plates,1.0,20.0,False,True,,
Diary,Plates,1.0,20.0,False,False,Genuine,Report #8302
Diary,Plates,1.0,20.0,False,False,Genuine,
Diary,Plates,1.0,20.0,False,False,Fake,Report #8303
Diary,Plates,1.0,20.0,False,False,Fake,
Diary,Plates,1.0,20.0,False,False,,Report #8304
Diary,Plates,1.0,20.0,False,False,,
Diary,Plates,1.0,21.0,True,True,Genuine,Report #8305
Diary,Plates,1.0,21.0,True,True,Genuine,
Diary,Plates,1.0,21.0,True,True,Fake,Report #8306
Diary,Plates,1.0,21.0,True,True,Fake,
Diary,Plates,1.0,21.0,True,True,,Report #8307
Diary,Plates,1.0,21.0,True,True,,
Diary,Plates,1.0,21.0,True,False,Genuine,Report #8308
Diary,Plates,1.0,21.0,True,False,Genuine,
Diary,Plates,1.0,21.0,True,False,Fake,Report #8309
Diary,Plates,1.0,21.0,True,False,Fake,
Diary,Plates,1.0,21.0,True,False,,Report #8310
Diary,Plates,1.0,21.0,True,False,,
Diary,Plates,1.0,21.0,False,True,Genuine,Report #8311
Diary,Plates,1.0,21.0,False,True,Genuine,
Diary,Plates,1.0,21.0,False,True,Fake,Report #8312
Diary,Plates,1.0,21.0,False,True,Fake,
Diary,Plates,1.0,21.0,False,True,,Report #8313
Diary,Plates,1.0,21.0,False,True,,
Diary,Plates,1.0,21.0,False,False,Genuine,Report #8314
Diary,Plates,1.0,21.0,False,False,Genuine,
Diary,Plates,1.0,21.0,False,False,Fake,Report #8315
Diary,Plates,1.0,21.0,False,False,Fake,
Diary,Plates,1.0,21.0,False,False,,Report #8316
Diary,Plates,1.0,21.0,False,False,,
Diary,Plates,1.0,99.0,True,True,Genuine,Report #8317
Diary,Plates,1.0,99.0,True,True,Genuine,
Diary,Plates,1.0,99.0,True,True,Fake,Report #8318
Diary,Plates,1.0,99.0,True,True,Fake,
Diary,Plates,1.0,99.0,True,True,,Report #8319
Diary,Plates,1.0,99.0,True,True,,
Diary,Plates,1.0,99.0,True,False,Genuine,Report #8320
Diary,Plates,1.0,99.0,True,False,Genuine,
Diary,Plates,1.0,99.0,True,False,Fake,Report #8321
Diary,Plates,1.0,99.0,True,False,Fake,
Diary,Plates,1.0,99.0,True,False,,Report #8322
Diary,Plates,1.0,99.0,True,False,,
Diary,Plates,1.0,99.0,False,True,Genuine,Report #8323
Diary,Plates,1.0,99.0,False,True,Genuine,
Diary,Plates,1.0,99.0,False,True,Fake,Report #8324
Diary,Plates,1.0,99.0,False,True,Fake,
Diary,Plates,1.0,99.0,False,True,,Report #8325
Diary,Plates,1.0,99.0,False,True,,
Diary,Plates,1.0,99.0,False,False,Genuine,Report #8326
Diary,Plates,1.0,99.0,False,False,Genuine,
Diary,Plates,1.0,99.0,False,False,Fake,Report #8327
Diary,Plates,1.0,99.0,False,False,Fake,
Diary,Plates,1.0,99.0,False,False,,Report #8328
Diary,Plates,1.0,99.0,False,False,,
Diary,Plates,1.0,100.0,True,True,Genuine,Report #8329
Diary,Plates,1.0,100.0,True,True,Genuine,
Diary,Plates,1.0,100.0,True,True,Fake,Report #8330
Diary,Plates,1.0,100.0,True,True,Fake,
Diary,Plates,1.0,100.0,True,True,,Report #8331
Diary,Plates,1.0,100.0,True,True,,
Diary,Plates,1.0,100.0,True,False,Genuine,Report #8332
Diary,Plates,1.0,100.0,True,False,Genuine,
Diary,Plates,1.0,100.0,True,False,Fake,Report #8333
Diary,Plates,1.0,100.0,True,False,Fake,
Diary,Plates,1.0,100.0,True,False,,Report #8334
Diary,Plates,1.0,100.0,True,False,,
Diary,Plates,1.0,100.0,False,True,Genuine,Report #8335
Diary,Plates,1.0,100.0,False,True,Genuine,
Diary,Plates,1.0,100.0,False,True,Fake,Report #8336
Diary,Plates,1.0,100.0,False,True,Fake,
Diary,Plates,1.0,100.0,False,True,,Report #8337
Diary,Plates,1.0,100.0,False,True,,
Diary,Plates,1.0,100.0,False,False,Genuine,Report #8338
Diary,Plates,1.0,100.0,False,False,Genuine,
Diary,Plates,1.0,100.0,False,False,Fake,Report #8339
Diary,Plates,1.0,100.0,False,False,Fake,
Diary,Plates,1.0,100.0,False,False,,Report #8340
Diary,Plates,1.0,100.0,False,False,,
Diary,Plates,1.0,101.0,True,True,Genuine,Report #8341
Diary,Plates,1.0,101.0,True,True,Genuine,
Diary,Plates,1.0,101.0,True,True,Fake,Report #8342
Diary,Plates,1.0,101.0,True,True,Fake,
Diary,Plates,1.0,101.0,True,True,,Report #8343
Diary,Plates,1.0,101.0,True,True,,
Diary,Plates,1.0,101.0,True,False,Genuine,Report #8344
Diary,Plates,1.0,101.0,True,False,Genuine,
Diary,Plates,1.0,101.0,True,False,Fake,Report #8345
Diary,Plates,1.0,101.0,True,False,Fake,
Diary,Plates,1.0,101.0,True,False,,Report #8346
Diary,Plates,1.0,101.0,True,False,,
Diary,Plates,1.0,101.0,False,True,Genuine,Report #8347
Diary,Plates,1.0,101.0,False,True,Genuine,
Diary,Plates,1.0,101.0,False,True,Fake,Report #8348
Diary,Plates,1.0,101.0,False,True,Fake,
Diary,Plates,1.0,101.0,False,True,,Report #8349
Diary,Plates,1.0,101.0,False,True,,
Diary,Plates,1.0,101.0,False,False,Genuine,Report #8350
Diary,Plates,1.0,101.0,False,False,Genuine,
Diary,Plates,1.0,101.0,False,False,Fake,Report #8351
Diary,Plates,1.0,101.0,False,False,Fake,
Diary,Plates,1.0,101.0,False,False,,Report #8352
Diary,Plates,1.0,101.0,False,False,,
Diary,Plates,100.0,-1.0,True,True,Genuine,Report #8353
Diary,Plates,100.0,-1.0,True,True,Genuine,
Diary,Plates,100.0,-1.0,True,True,Fake,Report #8354
Diary,Plates,100.0,-1.0,True,True,Fake,
Diary,Plates,100.0,-1.0,True,True,,Report #8355
Diary,Plates,100.0,-1.0,True,True,,
Diary,Plates,100.0,-1.0,True,False,Genuine,Report #8356
Diary,Plates,100.0,-1.0,True,False,Genuine,
Diary,Plates,100.0,-1.0,True,False,Fake,Report #8357
Diary,Plates,100.0,-1.0,True,False,Fake,
Diary,Plates,100.0,-1.0,True,False,,Report #8358
Diary,Plates,100.0,-1.0,True,False,,
Diary,Plates,100.0,-1.0,False,True,Genuine,Report #8359
Diary,Plates,100.0,-1.0,False,True,Genuine,
Diary,Plates,100.0,-1.0,False,True,Fake,Report #8360
Diary,Plates,100.0,-1.0,False,True,Fake,
Diary,Plates,100.0,-1.0,False,True,,Report #8361
Diary,Plates,100.0,-1.0,False,True,,
Diary,Plates,100.0,-1.0,False,False,Genuine,Report #8362
Diary,Plates,100.0,-1.0,False,False,Genuine,
Diary,Plates,100.0,-1.0,False,False,Fake,Report #8363
Diary,Plates,100.0,-1.0,False,False,Fake,
Diary,Plates,100.0,-1.0,False,False,,Report #8364
Diary,Plates,100.0,-1.0,False,False,,
Diary,Plates,100.0,0.0,True,True,Genuine,Report #8365
Diary,Plates,100.0,0.0,True,True,Genuine,
Diary,Plates,100.0,0.0,True,True,Fake,Report #8366
Diary,Plates,100.0,0.0,True,True,Fake,
Diary,Plates,100.0,0.0,True,True,,Report #8367
Diary,Plates,100.0,0.0,True,True,,
Diary,Plates,100.0,0.0,True,False,Genuine,Report #8368
Diary,Plates,100.0,0.0,True,False,Genuine,
Diary,Plates,100.0,0.0,True,False,Fake,Report #8369
Diary,Plates,100.0,0.0,True,False,Fake,
Diary,Plates,100.0,0.0,True,False,,Report #8370
Diary,Plates,100.0,0.0,True,False,,
Diary,Plates,100.0,0.0,False,True,Genuine,Report #8371
Diary,Plates,100.0,0.0,False,True,Genuine,
Diary,Plates,100.0,0.0,False,True,Fake,Report #8372
Diary,Plates,100.0,0.0,False,True,Fake,
Diary,Plates,100.0,0.0,False,True,,Report #8373
Diary,Plates,100.0,0.0,False,True,,
Diary,Plates,100.0,0.0,False,False,Genuine,Report #8374
Diary,Plates,100.0,0.0,False,False,Genuine,
Diary,Plates,100.0,0.0,False,False,Fake,Report #8375
Diary,Plates,100.0,0.0,False,False,Fake,
Diary,Plates,100.0,0.0,False,False,,Report #8376
Diary,Plates,100.0,0.0,False,False,,
Diary,Plates,100.0,19.0,True,True,Genuine,Report #8377
Diary,Plates,100.0,19.0,True,True,Genuine,
Diary,Plates,100.0,19.0,True,True,Fake,Report #8378
Diary,Plates,100.0,19.0,True,True,Fake,
Diary,Plates,100.0,19.0,True,True,,Report #8379
Diary,Plates,100.0,19.0,True,True,,
Diary,Plates,100.0,19.0,True,False,Genuine,Report #8380
Diary,Plates,100.0,19.0,True,False,Genuine,
Diary,Plates,100.0,19.0,True,False,Fake,Report #8381
Diary,Plates,100.0,19.0,True,False,Fake,
Diary,Plates,100.0,19.0,True,False,,Report #8382
Diary,Plates,100.0,19.0,True,False,,
Diary,Plates,100.0,19.0,False,True,Genuine,Report #8383
Diary,Plates,100.0,19.0,False,True,Genuine,
Diary,Plates,100.0,19.0,False,True,Fake,Report #8384
Diary,Plates,100.0,19.0,False,True,Fake,
Diary,Plates,100.0,19.0,False,True,,Report #8385
Diary,Plates,100.0,19.0,False,True,,
Diary,Plates,100.0,19.0,False,False,Genuine,Report #8386
Diary,Plates,100.0,19.0,False,False,Genuine,
Diary,Plates,100.0,19.0,False,False,Fake,Report #8387
Diary,Plates,100.0,19.0,False,False,Fake,
Diary,Plates,100.0,19.0,False,False,,Report #8388
Diary,Plates,100.0,19.0,False,False,,
Diary,Plates,100.0,20.0,True,True,Genuine,Report #8389
Diary,Plates,100.0,20.0,True,True,Genuine,
Diary,Plates,100.0,20.0,True,True,Fake,Report #8390
Diary,Plates,100.0,20.0,True,True,Fake,
Diary,Plates,100.0,20.0,True,True,,Report #8391
Diary,Plates,100.0,20.0,True,True,,
Diary,Plates,100.0,20.0,True,False,Genuine,Report #8392
Diary,Plates,100.0,20.0,True,False,Genuine,
Diary,Plates,100.0,20.0,True,False,Fake,Report #8393
Diary,Plates,100.0,20.0,True,False,Fake,
Diary,Plates,100.0,20.0,True,False,,Report #8394
Diary,Plates,100.0,20.0,True,False,,
Diary,Plates,100.0,20.0,False,True,Genuine,Report #8395
Diary,Plates,100.0,20.0,False,True,Genuine,
Diary,Plates,100.0,20.0,False,True,Fake,Report #8396
Diary,Plates,100.0,20.0,False,True,Fake,
Diary,Plates,100.0,20.0,False,True,,Report #8397
Diary,Plates,100.0,20.0,False,True,,
Diary,Plates,100.0,20.0,False,False,Genuine,Report #8398
Diary,Plates,100.0,20.0,False,False,Genuine,
Diary,Plates,100.0,20.0,False,False,Fake,Report #8399
Diary,Plates,100.0,20.0,False,False,Fake,
Diary,Plates,100.0,20.0,False,False,,Report #8400
Diary,Plates,100.0,20.0,False,False,,
Diary,Plates,100.0,21.0,True,True,Genuine,Report #8401
Diary,Plates,100.0,21.0,True,True,Genuine,
Diary,Plates,100.0,21.0,True,True,Fake,Report #8402
Diary,Plates,100.0,21.0,True,True,Fake,
Diary,Plates,100.0,21.0,True,True,,Report #8403
Diary,Plates,100.0,21.0,True,True,,
Diary,Plates,100.0,21.0,True,False,Genuine,Report #8404
Diary,Plates,100.0,21.0,True,False,Genuine,
Diary,Plates,100.0,21.0,True,False,Fake,Report #8405
Diary,Plates,100.0,21.0,True,False,Fake,
Diary,Plates,100.0,21.0,True,False,,Report #8406
Diary,Plates,100.0,21.0,True,False,,
Diary,Plates,100.0,21.0,False,True,Genuine,Report #8407
Diary,Plates,100.0,21.0,False,True,Genuine,
Diary,Plates,100.0,21.0,False,True,Fake,Report #8408
Diary,Plates,100.0,21.0,False,True,Fake,
Diary,Plates,100.0,21.0,False,True,,Report #8409
Diary,Plates,100.0,21.0,False,True,,
Diary,Plates,100.0,21.0,False,False,Genuine,Report #8410
Diary,Plates,100.0,21.0,False,False,Genuine,
Diary,Plates,100.0,21.0,False,False,Fake,Report #8411
Diary,Plates,100.0,21.0,False,False,Fake,
Diary,Plates,100.0,21.0,False,False,,Report #8412
Diary,Plates,100.0,21.0,False,False,,
Diary,Plates,100.0,99.0,True,True,Genuine,Report #8413
Diary,Plates,100.0,99.0,True,True,Genuine,
Diary,Plates,100.0,99.0,True,True,Fake,Report #8414
Diary,Plates,100.0,99.0,True,True,Fake,
Diary,Plates,100.0,99.0,True,True,,Report #8415
Diary,Plates,100.0,99.0,True,True,,
Diary,Plates,100.0,99.0,True,False,Genuine,Report #8416
Diary,Plates,100.0,99.0,True,False,Genuine,
Diary,Plates,100.0,99.0,True,False,Fake,Report #8417
Diary,Plates,100.0,99.0,True,False,Fake,
Diary,Plates,100.0,99.0,True,False,,Report #8418
Diary,Plates,100.0,99.0,True,False,,
Diary,Plates,100.0,99.0,False,True,Genuine,Report #8419
Diary,Plates,100.0,99.0,False,True,Genuine,
Diary,Plates,100.0,99.0,False,True,Fake,Report #8420
Diary,Plates,100.0,99.0,False,True,Fake,
Diary,Plates,100.0,99.0,False,True,,Report #8421
Diary,Plates,100.0,99.0,False,True,,
Diary,Plates,100.0,99.0,False,False,Genuine,Report #8422
Diary,Plates,100.0,99.0,False,False,Genuine,
Diary,Plates,100.0,99.0,False,False,Fake,Report #8423
Diary,Plates,100.0,99.0,False,False,Fake,
Diary,Plates,100.0,99.0,False,False,,Report #8424
Diary,Plates,100.0,99.0,False,False,,
Diary,Plates,100.0,100.0,True,True,Genuine,Report #8425
Diary,Plates,100.0,100.0,True,True,Genuine,
Diary,Plates,100.0,100.0,True,True,Fake,Report #8426
Diary,Plates,100.0,100.0,True,True,Fake,
Diary,Plates,100.0,100.0,True,True,,Report #8427
Diary,Plates,100.0,100.0,True,True,,
Diary,Plates,100.0,100.0,True,False,Genuine,Report #8428
Diary,Plates,100.0,100.0,True,False,Genuine,
Diary,Plates,100.0,100.0,True,False,Fake,Report #8429
Diary,Plates,100.0,100.0,True,False,Fake,
Diary,Plates,100.0,100.0,True,False,,Report #8430
Diary,Plates,100.0,100.0,True,False,,
Diary,Plates,100.0,100.0,False,True,Genuine,Report #8431
Diary,Plates,100.0,100.0,False,True,Genuine,
Diary,Plates,100.0,100.0,False,True,Fake,Report #8432
Diary,Plates,100.0,100.0,False,True,Fake,
Diary,Plates,100.0,100.0,False,True,,Report #8433
Diary,Plates,100.0,100.0,False,True,,
Diary,Plates,100.0,100.0,False,False,Genuine,Report #8434
Diary,Plates,100.0,100.0,False,False,Genuine,
Diary,Plates,100.0,100.0,False,False,Fake,Report #8435
Diary,Plates,100.0,100.0,False,False,Fake,
Diary,Plates,100.0,100.0,False,False,,Report #8436
Diary,Plates,100.0,100.0,False,False,,
Diary,Plates,100.0,101.0,True,True,Genuine,Report #8437
Diary,Plates,100.0,101.0,True,True,Genuine,
Diary,Plates,100.0,101.0,True,True,Fake,Report #8438
Diary,Plates,100.0,101.0,True,True,Fake,
Diary,Plates,100.0,101.0,True,True,,Report #8439
Diary,Plates,100.0,101.0,True,True,,
Diary,Plates,100.0,101.0,True,False,Genuine,Report #8440
Diary,Plates,100.0,101.0,True,False,Genuine,
Diary,Plates,100.0,101.0,True,False,Fake,Report #8441
Diary,Plates,100.0,101.0,True,False,Fake,
Diary,Plates,100.0,101.0,True,False,,Report #8442
Diary,Plates,100.0,101.0,True,False,,
Diary,Plates,100.0,101.0,False,True,Genuine,Report #8443
Diary,Plates,100.0,101.0,False,True,Genuine,
Diary,Plates,100.0,101.0,False,True,Fake,Report #8444
Diary,Plates,100.0,101.0,False,True,Fake,
Diary,Plates,100.0,101.0,False,True,,Report #8445
Diary,Plates,100.0,101.0,False,True,,
Diary,Plates,100.0,101.0,False,False,Genuine,Report #8446
Diary,Plates,100.0,101.0,False,False,Genuine,
Diary,Plates,100.0,101.0,False,False,Fake,Report #8447
Diary,Plates,100.0,101.0,False,False,Fake,
Diary,Plates,100.0,101.0,False,False,,Report #8448
Diary,Plates,100.0,101.0,False,False,,
Diary,Plates,1000.0,-1.0,True,True,Genuine,Report #8449
Diary,Plates,1000.0,-1.0,True,True,Genuine,
Diary,Plates,1000.0,-1.0,True,True,Fake,Report #8450
Diary,Plates,1000.0,-1.0,True,True,Fake,
Diary,Plates,1000.0,-1.0,True,True,,Report #8451
Diary,Plates,1000.0,-1.0,True,True,,
Diary,Plates,1000.0,-1.0,True,False,Genuine,Report #8452
Diary,Plates,1000.0,-1.0,True,False,Genuine,
Diary,Plates,1000.0,-1.0,True,False,Fake,Report #8453
Diary,Plates,1000.0,-1.0,True,False,Fake,
Diary,Plates,1000.0,-1.0,True,False,,Report #8454
Diary,Plates,1000.0,-1.0,True,False,,
Diary,Plates,1000.0,-1.0,False,True,Genuine,Report #8455
Diary,Plates,1000.0,-1.0,False,True,Genuine,
Diary,Plates,1000.0,-1.0,False,True,Fake,Report #8456
Diary,Plates,1000.0,-1.0,False,True,Fake,
Diary,Plates,1000.0,-1.0,False,True,,Report #8457
Diary,Plates,1000.0,-1.0,False,True,,
Diary,Plates,1000.0,-1.0,False,False,Genuine,Report #8458
Diary,Plates,1000.0,-1.0,False,False,Genuine,
Diary,Plates,1000.0,-1.0,False,False,Fake,Report #8459
Diary,Plates,1000.0,-1.0,False,False,Fake,
Diary,Plates,1000.0,-1.0,False,False,,Report #8460
Diary,Plates,1000.0,-1.0,False,False,,
Diary,Plates,1000.0,0.0,True,True,Genuine,Report #8461
Diary,Plates,1000.0,0.0,True,True,Genuine,
Diary,Plates,1000.0,0.0,True,True,Fake,Report #8462
Diary,Plates,1000.0,0.0,True,True,Fake,
Diary,Plates,1000.0,0.0,True,True,,Report #8463
Diary,Plates,1000.0,0.0,True,True,,
Diary,Plates,1000.0,0.0,True,False,Genuine,Report #8464
Diary,Plates,1000.0,0.0,True,False,Genuine,
Diary,Plates,1000.0,0.0,True,False,Fake,Report #8465
Diary,Plates,1000.0,0.0,True,False,Fake,
Diary,Plates,1000.0,0.0,True,False,,Report #8466
Diary,Plates,1000.0,0.0,True,False,,
Diary,Plates,1000.0,0.0,False,True,Genuine,Report #8467
Diary,Plates,1000.0,0.0,False,True,Genuine,
Diary,Plates,1000.0,0.0,False,True,Fake,Report #8468
Diary,Plates,1000.0,0.0,False,True,Fake,
Diary,Plates,1000.0,0.0,False,True,,Report #8469
Diary,Plates,1000.0,0.0,False,True,,
Diary,Plates,1000.0,0.0,False,False,Genuine,Report #8470
Diary,Plates,1000.0,0.0,False,False,Genuine,
Diary,Plates,1000.0,0.0,False,False,Fake,Report #8471
Diary,Plates,1000.0,0.0,False,False,Fake,
Diary,Plates,1000.0,0.0,False,False,,Report #8472
Diary,Plates,1000.0,0.0,False,False,,
Diary,Plates,1000.0,19.0,True,True,Genuine,Report #8473
Diary,Plates,1000.0,19.0,True,True,Genuine,
Diary,Plates,1000.0,19.0,True,True,Fake,Report #8474
Diary,Plates,1000.0,19.0,True,True,Fake,
Diary,Plates,1000.0,19.0,True,True,,Report #8475
Diary,Plates,1000.0,19.0,True,True,,
Diary,Plates,1000.0,19.0,True,False,Genuine,Report #8476
Diary,Plates,1000.0,19.0,True,False,Genuine,
Diary,Plates,1000.0,19.0,True,False,Fake,Report #8477
Diary,Plates,1000.0,19.0,True,False,Fake,
Diary,Plates,1000.0,19.0,True,False,,Report #8478
Diary,Plates,1000.0,19.0,True,False,,
Diary,Plates,1000.0,19.0,False,True,Genuine,Report #8479
Diary,Plates,1000.0,19.0,False,True,Genuine,
Diary,Plates,1000.0,19.0,False,True,Fake,Report #8480
Diary,Plates,1000.0,19.0,False,True,Fake,
Diary,Plates,1000.0,19.0,False,True,,Report #8481
Diary,Plates,1000.0,19.0,False,True,,
Diary,Plates,1000.0,19.0,False,False,Genuine,Report #8482
Diary,Plates,1000.0,19.0,False,False,Genuine,
Diary,Plates,1000.0,19.0,False,False,Fake,Report #8483
Diary,Plates,1000.0,19.0,False,False,Fake,
Diary,Plates,1000.0,19.0,False,False,,Report #8484
Diary,Plates,1000.0,19.0,False,False,,
Diary,Plates,1000.0,20.0,True,True,Genuine,Report #8485
Diary,Plates,1000.0,20.0,True,True,Genuine,
Diary,Plates,1000.0,20.0,True,True,Fake,Report #8486
Diary,Plates,1000.0,20.0,True,True,Fake,
Diary,Plates,1000.0,20.0,True,True,,Report #8487
Diary,Plates,1000.0,20.0,True,True,,
Diary,Plates,1000.0,20.0,True,False,Genuine,Report #8488
Diary,Plates,1000.0,20.0,True,False,Genuine,
Diary,Plates,1000.0,20.0,True,False,Fake,Report #8489
Diary,Plates,1000.0,20.0,True,False,Fake,
Diary,Plates,1000.0,20.0,True,False,,Report #8490
Diary,Plates,1000.0,20.0,True,False,,
Diary,Plates,1000.0,20.0,False,True,Genuine,Report #8491
Diary,Plates,1000.0,20.0,False,True,Genuine,
Diary,Plates,1000.0,20.0,False,True,Fake,Report #8492
Diary,Plates,1000.0,20.0,False,True,Fake,
Diary,Plates,1000.0,20.0,False,True,,Report #8493
Diary,Plates,1000.0,20.0,False,True,,
Diary,Plates,1000.0,20.0,False,False,Genuine,Report #8494
Diary,Plates,1000.0,20.0,False,False,Genuine,
Diary,Plates,1000.0,20.0,False,False,Fake,Report #8495
Diary,Plates,1000.0,20.0,False,False,Fake,
Diary,Plates,1000.0,20.0,False,False,,Report #8496
Diary,Plates,1000.0,20.0,False,False,,
Diary,Plates,1000.0,21.0,True,True,Genuine,Report #8497
Diary,Plates,1000.0,21.0,True,True,Genuine,
Diary,Plates,1000.0,21.0,True,True,Fake,Report #8498
Diary,Plates,1000.0,21.0,True,True,Fake,
Diary,Plates,1000.0,21.0,True,True,,Report #8499
Diary,Plates,1000.0,21.0,True,True,,
Diary,Plates,1000.0,21.0,True,False,Genuine,Report #8500
Diary,Plates,1000.0,21.0,True,False,Genuine,
Diary,Plates,1000.0,21.0,True,False,Fake,Report #8501
Diary,Plates,1000.0,21.0,True,False,Fake,
Diary,Plates,1000.0,21.0,True,False,,Report #8502
Diary,Plates,1000.0,21.0,True,False,,
Diary,Plates,1000.0,21.0,False,True,Genuine,Report #8503
Diary,Plates,1000.0,21.0,False,True,Genuine,
Diary,Plates,1000.0,21.0,False,True,Fake,Report #8504
Diary,Plates,1000.0,21.0,False,True,Fake,
Diary,Plates,1000.0,21.0,False,True,,Report #8505
Diary,Plates,1000.0,21.0,False,True,,
Diary,Plates,1000.0,21.0,False,False,Genuine,Report #8506
Diary,Plates,1000.0,21.0,False,False,Genuine,
Diary,Plates,1000.0,21.0,False,False,Fake,Report #8507
Diary,Plates,1000.0,21.0,False,False,Fake,
Diary,Plates,1000.0,21.0,False,False,,Report #8508
Diary,Plates,1000.0,21.0,False,False,,
Diary,Plates,1000.0,99.0,True,True,Genuine,Report #8509
Diary,Plates,1000.0,99.0,True,True,Genuine,
Diary,Plates,1000.0,99.0,True,True,Fake,Report #8510
Diary,Plates,1000.0,99.0,True,True,Fake,
Diary,Plates,1000.0,99.0,True,True,,Report #8511
Diary,Plates,1000.0,99.0,True,True,,
Diary,Plates,1000.0,99.0,True,False,Genuine,Report #8512
Diary,Plates,1000.0,99.0,True,False,Genuine,
Diary,Plates,1000.0,99.0,True,False,Fake,Report #8513
Diary,Plates,1000.0,99.0,True,False,Fake,
Diary,Plates,1000.0,99.0,True,False,,Report #8514
Diary,Plates,1000.0,99.0,True,False,,
Diary,Plates,1000.0,99.0,False,True,Genuine,Report #8515
Diary,Plates,1000.0,99.0,False,True,Genuine,
Diary,Plates,1000.0,99.0,False,True,Fake,Report #8516
Diary,Plates,1000.0,99.0,False,True,Fake,
Diary,Plates,1000.0,99.0,False,True,,Report #8517
Diary,Plates,1000.0,99.0,False,True,,
Diary,Plates,1000.0,99.0,False,False,Genuine,Report #8518
Diary,Plates,1000.0,99.0,False,False,Genuine,
Diary,Plates,1000.0,99.0,False,False,Fake,Report #8519
Diary,Plates,1000.0,99.0,False,False,Fake,
Diary,Plates,1000.0,99.0,False,False,,Report #8520
Diary,Plates,1000.0,99.0,False,False,,
Diary,Plates,1000.0,100.0,True,True,Genuine,Report #8521
Diary,Plates,1000.0,100.0,True,True,Genuine,
Diary,Plates,1000.0,100.0,True,True,Fake,Report #8522
Diary,Plates,1000.0,100.0,True,True,Fake,
Diary,Plates,1000.0,100.0,True,True,,Report #8523
Diary,Plates,1000.0,100.0,True,True,,
Diary,Plates,1000.0,100.0,True,False,Genuine,Report #8524
Diary,Plates,1000.0,100.0,True,False,Genuine,
Diary,Plates,1000.0,100.0,True,False,Fake,Report #8525
Diary,Plates,1000.0,100.0,True,False,Fake,
Diary,Plates,1000.0,100.0,True,False,,Report #8526
Diary,Plates,1000.0,100.0,True,False,,
Diary,Plates,1000.0,100.0,False,True,Genuine,Report #8527
Diary,Plates,1000.0,100.0,False,True,Genuine,
Diary,Plates,1000.0,100.0,False,True,Fake,Report #8528
Diary,Plates,1000.0,100.0,False,True,Fake,
Diary,Plates,1000.0,100.0,False,True,,Report #8529
Diary,Plates,1000.0,100.0,False,True,,
Diary,Plates,1000.0,100.0,False,False,Genuine,Report #8530
Diary,Plates,1000.0,100.0,False,False,Genuine,
Diary,Plates,1000.0,100.0,False,False,Fake,Report #8531
Diary,Plates,1000.0,100.0,False,False,Fake,
Diary,Plates,1000.0,100.0,False,False,,Report #8532
Diary,Plates,1000.0,100.0,False,False,,
Diary,Plates,1000.0,101.0,True,True,Genuine,Report #8533
Diary,Plates,1000.0,101.0,True,True,Genuine,
Diary,Plates,1000.0,101.0,True,True,Fake,Report #8534
Diary,Plates,1000.0,101.0,True,True,Fake,
Diary,Plates,1000.0,101.0,True,True,,Report #8535
Diary,Plates,1000.0,101.0,True,True,,
Diary,Plates,1000.0,101.0,True,False,Genuine,Report #8536
Diary,Plates,1000.0,101.0,True,False,Genuine,
Diary,Plates,1000.0,101.0,True,False,Fake,Report #8537
Diary,Plates,1000.0,101.0,True,False,Fake,
Diary,Plates,1000.0,101.0,True,False,,Report #8538
Diary,Plates,1000.0,101.0,True,False,,
Diary,Plates,1000.0,101.0,False,True,Genuine,Report #8539
Diary,Plates,1000.0,101.0,False,True,Genuine,
Diary,Plates,1000.0,101.0,False,True,Fake,Report #8540
Diary,Plates,1000.0,101.0,False,True,Fake,
Diary,Plates,1000.0,101.0,False,True,,Report #8541
Diary,Plates,1000.0,101.0,False,True,,
Diary,Plates,1000.0,101.0,False,False,Genuine,Report #8542
Diary,Plates,1000.0,101.0,False,False,Genuine,
Diary,Plates,1000.0,101.0,False,False,Fake,Report #8543
Diary,Plates,1000.0,101.0,False,False,Fake,
Diary,Plates,1000.0,101.0,False,False,,Report #8544
Diary,Plates,1000.0,101.0,False,False,,
Diary,Plates,1000000.0,-1.0,True,True,Genuine,Report #8545
Diary,Plates,1000000.0,-1.0,True,True,Genuine,
Diary,Plates,1000000.0,-1.0,True,True,Fake,Report #8546
Diary,Plates,1000000.0,-1.0,True,True,Fake,
Diary,Plates,1000000.0,-1.0,True,True,,Report #8547
Diary,Plates,1000000.0,-1.0,True,True,,
Diary,Plates,1000000.0,-1.0,True,False,Genuine,Report #8548
Diary,Plates,1000000.0,-1.0,True,False,Genuine,
Diary,Plates,1000000.0,-1.0,True,False,Fake,Report #8549
Diary,Plates,1000000.0,-1.0,True,False,Fake,
Diary,Plates,1000000.0,-1.0,True,False,,Report #8550
Diary,Plates,1000000.0,-1.0,True,False,,
Diary,Plates,1000000.0,-1.0,False,True,Genuine,Report #8551
Diary,Plates,1000000.0,-1.0,False,True,Genuine,
Diary,Plates,1000000.0,-1.0,False,True,Fake,Report #8552
Diary,Plates,1000000.0,-1.0,False,True,Fake,
Diary,Plates,1000000.0,-1.0,False,True,,Report #8553
Diary,Plates,1000000.0,-1.0,False,True,,
Diary,Plates,1000000.0,-1.0,False,False,Genuine,Report #8554
Diary,Plates,1000000.0,-1.0,False,False,Genuine,
Diary,Plates,1000000.0,-1.0,False,False,Fake,Report #8555
Diary,Plates,1000000.0,-1.0,False,False,Fake,
Diary,Plates,1000000.0,-1.0,False,False,,Report #8556
Diary,Plates,1000000.0,-1.0,False,False,,
Diary,Plates,1000000.0,0.0,True,True,Genuine,Report #8557
Diary,Plates,1000000.0,0.0,True,True,Genuine,
Diary,Plates,1000000.0,0.0,True,True,Fake,Report #8558
Diary,Plates,1000000.0,0.0,True,True,Fake,
Diary,Plates,1000000.0,0.0,True,True,,Report #8559
Diary,Plates,1000000.0,0.0,True,True,,
Diary,Plates,1000000.0,0.0,True,False,Genuine,Report #8560
Diary,Plates,1000000.0,0.0,True,False,Genuine,
Diary,Plates,1000000.0,0.0,True,False,Fake,Report #8561
Diary,Plates,1000000.0,0.0,True,False,Fake,
Diary,Plates,1000000.0,0.0,True,False,,Report #8562
Diary,Plates,1000000.0,0.0,True,False,,
Diary,Plates,1000000.0,0.0,False,True,Genuine,Report #8563
Diary,Plates,1000000.0,0.0,False,True,Genuine,
Diary,Plates,1000000.0,0.0,False,True,Fake,Report #8564
Diary,Plates,1000000.0,0.0,False,True,Fake,
Diary,Plates,1000000.0,0.0,False,True,,Report #8565
Diary,Plates,1000000.0,0.0,False,True,,
Diary,Plates,1000000.0,0.0,False,False,Genuine,Report #8566
Diary,Plates,1000000.0,0.0,False,False,Genuine,
Diary,Plates,1000000.0,0.0,False,False,Fake,Report #8567
Diary,Plates,1000000.0,0.0,False,False,Fake,
Diary,Plates,1000000.0,0.0,False,False,,Report #8568
Diary,Plates,1000000.0,0.0,False,False,,
Diary,Plates,1000000.0,19.0,True,True,Genuine,Report #8569
Diary,Plates,1000000.0,19.0,True,True,Genuine,
Diary,Plates,1000000.0,19.0,True,True,Fake,Report #8570
Diary,Plates,1000000.0,19.0,True,True,Fake,
Diary,Plates,1000000.0,19.0,True,True,,Report #8571
Diary,Plates,1000000.0,19.0,True,True,,
Diary,Plates,1000000.0,19.0,True,False,Genuine,Report #8572
Diary,Plates,1000000.0,19.0,True,False,Genuine,
Diary,Plates,1000000.0,19.0,True,False,Fake,Report #8573
Diary,Plates,1000000.0,19.0,True,False,Fake,
Diary,Plates,1000000.0,19.0,True,False,,Report #8574
Diary,Plates,1000000.0,19.0,True,False,,
Diary,Plates,1000000.0,19.0,False,True,Genuine,Report #8575
Diary,Plates,1000000.0,19.0,False,True,Genuine,
Diary,Plates,1000000.0,19.0,False,True,Fake,Report #8576
Diary,Plates,1000000.0,19.0,False,True,Fake,
Diary,Plates,1000000.0,19.0,False,True,,Report #8577
Diary,Plates,1000000.0,19.0,False,True,,
Diary,Plates,1000000.0,19.0,False,False,Genuine,Report #8578
Diary,Plates,1000000.0,19.0,False,False,Genuine,
Diary,Plates,1000000.0,19.0,False,False,Fake,Report #8579
Diary,Plates,1000000.0,19.0,False,False,Fake,
Diary,Plates,1000000.0,19.0,False,False,,Report #8580
Diary,Plates,1000000.0,19.0,False,False,,
Diary,Plates,1000000.0,20.0,True,True,Genuine,Report #8581
Diary,Plates,1000000.0,20.0,True,True,Genuine,
Diary,Plates,1000000.0,20.0,True,True,Fake,Report #8582
Diary,Plates,1000000.0,20.0,True,True,Fake,
Diary,Plates,1000000.0,20.0,True,True,,Report #8583
Diary,Plates,1000000.0,20.0,True,True,,
Diary,Plates,1000000.0,20.0,True,False,Genuine,Report #8584
Diary,Plates,1000000.0,20.0,True,False,Genuine,
Diary,Plates,1000000.0,20.0,True,False,Fake,Report #8585
Diary,Plates,1000000.0,20.0,True,False,Fake,
Diary,Plates,1000000.0,20.0,True,False,,Report #8586
Diary,Plates,1000000.0,20.0,True,False,,
Diary,Plates,1000000.0,20.0,False,True,Genuine,Report #8587
Diary,Plates,1000000.0,20.0,False,True,Genuine,
Diary,Plates,1000000.0,20.0,False,True,Fake,Report #8588
Diary,Plates,1000000.0,20.0,False,True,Fake,
Diary,Plates,1000000.0,20.0,False,True,,Report #8589
Diary,Plates,1000000.0,20.0,False,True,,
Diary,Plates,1000000.0,20.0,False,False,Genuine,Report #8590
Diary,Plates,1000000.0,20.0,False,False,Genuine,
Diary,Plates,1000000.0,20.0,False,False,Fake,Report #8591
Diary,Plates,1000000.0,20.0,False,False,Fake,
Diary,Plates,1000000.0,20.0,False,False,,Report #8592
Diary,Plates,1000000.0,20.0,False,False,,
Diary,Plates,1000000.0,21.0,True,True,Genuine,Report #8593
Diary,Plates,1000000.0,21.0,True,True,Genuine,
Diary,Plates,1000000.0,21.0,True,True,Fake,Report #8594
Diary,Plates,1000000.0,21.0,True,True,Fake,
Diary,Plates,1000000.0,21.0,True,True,,Report #8595
Diary,Plates,1000000.0,21.0,True,True,,
Diary,Plates,1000000.0,21.0,True,False,Genuine,Report #8596
Diary,Plates,1000000.0,21.0,True,False,Genuine,
Diary,Plates,1000000.0,21.0,True,False,Fake,Report #8597
Diary,Plates,1000000.0,21.0,True,False,Fake,
Diary,Plates,1000000.0,21.0,True,False,,Report #8598
Diary,Plates,1000000.0,21.0,True,False,,
Diary,Plates,1000000.0,21.0,False,True,Genuine,Report #8599
Diary,Plates,1000000.0,21.0,False,True,Genuine,
Diary,Plates,1000000.0,21.0,False,True,Fake,Report #8600
Diary,Plates,1000000.0,21.0,False,True,Fake,
Diary,Plates,1000000.0,21.0,False,True,,Report #8601
Diary,Plates,1000000.0,21.0,False,True,,
Diary,Plates,1000000.0,21.0,False,False,Genuine,Report #8602
Diary,Plates,1000000.0,21.0,False,False,Genuine,
Diary,Plates,1000000.0,21.0,False,False,Fake,Report #8603
Diary,Plates,1000000.0,21.0,False,False,Fake,
Diary,Plates,1000000.0,21.0,False,False,,Report #8604
Diary,Plates,1000000.0,21.0,False,False,,
Diary,Plates,1000000.0,99.0,True,True,Genuine,Report #8605
Diary,Plates,1000000.0,99.0,True,True,Genuine,
Diary,Plates,1000000.0,99.0,True,True,Fake,Report #8606
Diary,Plates,1000000.0,99.0,True,True,Fake,
Diary,Plates,1000000.0,99.0,True,True,,Report #8607
Diary,Plates,1000000.0,99.0,True,True,,
Diary,Plates,1000000.0,99.0,True,False,Genuine,Report #8608
Diary,Plates,1000000.0,99.0,True,False,Genuine,
Diary,Plates,1000000.0,99.0,True,False,Fake,Report #8609
Diary,Plates,1000000.0,99.0,True,False,Fake,
Diary,Plates,1000000.0,99.0,True,False,,Report #8610
Diary,Plates,1000000.0,99.0,True,False,,
Diary,Plates,1000000.0,99.0,False,True,Genuine,Report #8611
Diary,Plates,1000000.0,99.0,False,True,Genuine,
Diary,Plates,1000000.0,99.0,False,True,Fake,Report #8612
Diary,Plates,1000000.0,99.0,False,True,Fake,
Diary,Plates,1000000.0,99.0,False,True,,Report #8613
Diary,Plates,1000000.0,99.0,False,True,,
Diary,Plates,1000000.0,99.0,False,False,Genuine,Report #8614
Diary,Plates,1000000.0,99.0,False,False,Genuine,
Diary,Plates,1000000.0,99.0,False,False,Fake,Report #8615
Diary,Plates,1000000.0,99.0,False,False,Fake,
Diary,Plates,1000000.0,99.0,False,False,,Report #8616
Diary,Plates,1000000.0,99.0,False,False,,
Diary,Plates,1000000.0,100.0,True,True,Genuine,Report #8617
Diary,Plates,1000000.0,100.0,True,True,Genuine,
Diary,Plates,1000000.0,100.0,True,True,Fake,Report #8618
Diary,Plates,1000000.0,100.0,True,True,Fake,
Diary,Plates,1000000.0,100.0,True,True,,Report #8619
Diary,Plates,1000000.0,100.0,True,True,,
Diary,Plates,1000000.0,100.0,True,False,Genuine,Report #8620
Diary,Plates,1000000.0,100.0,True,False,Genuine,
Diary,Plates,1000000.0,100.0,True,False,Fake,Report #8621
Diary,Plates,1000000.0,100.0,True,False,Fake,
Diary,Plates,1000000.0,100.0,True,False,,Report #8622
Diary,Plates,1000000.0,100.0,True,False,,
Diary,Plates,1000000.0,100.0,False,True,Genuine,Report #8623
Diary,Plates,1000000.0,100.0,False,True,Genuine,
Diary,Plates,1000000.0,100.0,False,True,Fake,Report #8624
Diary,Plates,1000000.0,100.0,False,True,Fake,
Diary,Plates,1000000.0,100.0,False,True,,Report #8625
Diary,Plates,1000000.0,100.0,False,True,,
Diary,Plates,1000000.0,100.0,False,False,Genuine,Report #8626
Diary,Plates,1000000.0,100.0,False,False,Genuine,
Diary,Plates,1000000.0,100.0,False,False,Fake,Report #8627
Diary,Plates,1000000.0,100.0,False,False,Fake,
Diary,Plates,1000000.0,100.0,False,False,,Report #8628
Diary,Plates,1000000.0,100.0,False,False,,
Diary,Plates,1000000.0,101.0,True,True,Genuine,Report #8629
Diary,Plates,1000000.0,101.0,True,True,Genuine,
Diary,Plates,1000000.0,101.0,True,True,Fake,Report #8630
Diary,Plates,1000000.0,101.0,True,True,Fake,
Diary,Plates,1000000.0,101.0,True,True,,Report #8631
Diary,Plates,1000000.0,101.0,True,True,,
Diary,Plates,1000000.0,101.0,True,False,Genuine,Report #8632
Diary,Plates,1000000.0,101.0,True,False,Genuine,
Diary,Plates,1000000.0,101.0,True,False,Fake,Report #8633
Diary,Plates,1000000.0,101.0,True,False,Fake,
Diary,Plates,1000000.0,101.0,True,False,,Report #8634
Diary,Plates,1000000.0,101.0,True,False,,
Diary,Plates,1000000.0,101.0,False,True,Genuine,Report #8635
Diary,Plates,1000000.0,101.0,False,True,Genuine,
Diary,Plates,1000000.0,101.0,False,True,Fake,Report #8636
Diary,Plates,1000000.0,101.0,False,True,Fake,
Diary,Plates,1000000.0,101.0,False,True,,Report #8637
Diary,Plates,1000000.0,101.0,False,True,,
Diary,Plates,1000000.0,101.0,False,False,Genuine,Report #8638
Diary,Plates,1000000.0,101.0,False,False,Genuine,
Diary,Plates,1000000.0,101.0,False,False,Fake,Report #8639
Diary,Plates,1000000.0,101.0,False,False,Fake,
Diary,Plates,1000000.0,101.0,False,False,,Report #8640
Diary,Plates,1000000.0,101.0,False,False,,
Diary,Furniture,0.0,-1.0,True,True,Genuine,Report #8641
Diary,Furniture,0.0,-1.0,True,True,Genuine,
Diary,Furniture,0.0,-1.0,True,True,Fake,Report #8642
Diary,Furniture,0.0,-1.0,True,True,Fake,
Diary,Furniture,0.0,-1.0,True,True,,Report #8643
Diary,Furniture,0.0,-1.0,True,True,,
Diary,Furniture,0.0,-1.0,True,False,Genuine,Report #8644
Diary,Furniture,0.0,-1.0,True,False,Genuine,
Diary,Furniture,0.0,-1.0,True,False,Fake,Report #8645
Diary,Furniture,0.0,-1.0,True,False,Fake,
Diary,Furniture,0.0,-1.0,True,False,,Report #8646
Diary,Furniture,0.0,-1.0,True,False,,
Diary,Furniture,0.0,-1.0,False,True,Genuine,Report #8647
Diary,Furniture,0.0,-1.0,False,True,Genuine,
Diary,Furniture,0.0,-1.0,False,True,Fake,Report #8648
Diary,Furniture,0.0,-1.0,False,True,Fake,
Diary,Furniture,0.0,-1.0,False,True,,Report #8649
Diary,Furniture,0.0,-1.0,False,True,,
Diary,Furniture,0.0,-1.0,False,False,Genuine,Report #8650
Diary,Furniture,0.0,-1.0,False,False,Genuine,
Diary,Furniture,0.0,-1.0,False,False,Fake,Report #8651
Diary,Furniture,0.0,-1.0,False,False,Fake,
Diary,Furniture,0.0,-1.0,False,False,,Report #8652
Diary,Furniture,0.0,-1.0,False,False,,
Diary,Furniture,0.0,0.0,True,True,Genuine,Report #8653
Diary,Furniture,0.0,0.0,True,True,Genuine,
Diary,Furniture,0.0,0.0,True,True,Fake,Report #8654
Diary,Furniture,0.0,0.0,True,True,Fake,
Diary,Furniture,0.0,0.0,True,True,,Report #8655
Diary,Furniture,0.0,0.0,True,True,,
Diary,Furniture,0.0,0.0,True,False,Genuine,Report #8656
Diary,Furniture,0.0,0.0,True,False,Genuine,
Diary,Furniture,0.0,0.0,True,False,Fake,Report #8657
Diary,Furniture,0.0,0.0,True,False,Fake,
Diary,Furniture,0.0,0.0,True,False,,Report #8658
Diary,Furniture,0.0,0.0,True,False,,
Diary,Furniture,0.0,0.0,False,True,Genuine,Report #8659
Diary,Furniture,0.0,0.0,False,True,Genuine,
Diary,Furniture,0.0,0.0,False,True,Fake,Report #8660
Diary,Furniture,0.0,0.0,False,True,Fake,
Diary,Furniture,0.0,0.0,False,True,,Report #8661
Diary,Furniture,0.0,0.0,False,True,,
Diary,Furniture,0.0,0.0,False,False,Genuine,Report #8662
Diary,Furniture,0.0,0.0,False,False,Genuine,
Diary,Furniture,0.0,0.0,False,False,Fake,Report #8663
Diary,Furniture,0.0,0.0,False,False,Fake,
Diary,Furniture,0.0,0.0,False,False,,Report #8664
Diary,Furniture,0.0,0.0,False,False,,
Diary,Furniture,0.0,19.0,True,True,Genuine,Report #8665
Diary,Furniture,0.0,19.0,True,True,Genuine,
Diary,Furniture,0.0,19.0,True,True,Fake,Report #8666
Diary,Furniture,0.0,19.0,True,True,Fake,
Diary,Furniture,0.0,19.0,True,True,,Report #8667
Diary,Furniture,0.0,19.0,True,True,,
Diary,Furniture,0.0,19.0,True,False,Genuine,Report #8668
Diary,Furniture,0.0,19.0,True,False,Genuine,
Diary,Furniture,0.0,19.0,True,False,Fake,Report #8669
Diary,Furniture,0.0,19.0,True,False,Fake,
Diary,Furniture,0.0,19.0,True,False,,Report #8670
Diary,Furniture,0.0,19.0,True,False,,
Diary,Furniture,0.0,19.0,False,True,Genuine,Report #8671
Diary,Furniture,0.0,19.0,False,True,Genuine,
Diary,Furniture,0.0,19.0,False,True,Fake,Report #8672
Diary,Furniture,0.0,19.0,False,True,Fake,
Diary,Furniture,0.0,19.0,False,True,,Report #8673
Diary,Furniture,0.0,19.0,False,True,,
Diary,Furniture,0.0,19.0,False,False,Genuine,Report #8674
Diary,Furniture,0.0,19.0,False,False,Genuine,
Diary,Furniture,0.0,19.0,False,False,Fake,Report #8675
Diary,Furniture,0.0,19.0,False,False,Fake,
Diary,Furniture,0.0,19.0,False,False,,Report #8676
Diary,Furniture,0.0,19.0,False,False,,
Diary,Furniture,0.0,20.0,True,True,Genuine,Report #8677
Diary,Furniture,0.0,20.0,True,True,Genuine,
Diary,Furniture,0.0,20.0,True,True,Fake,Report #8678
Diary,Furniture,0.0,20.0,True,True,Fake,
Diary,Furniture,0.0,20.0,True,True,,Report #8679
Diary,Furniture,0.0,20.0,True,True,,
Diary,Furniture,0.0,20.0,True,False,Genuine,Report #8680
Diary,Furniture,0.0,20.0,True,False,Genuine,
Diary,Furniture,0.0,20.0,True,False,Fake,Report #8681
Diary,Furniture,0.0,20.0,True,False,Fake,
Diary,Furniture,0.0,20.0,True,False,,Report #8682
Diary,Furniture,0.0,20.0,True,False,,
Diary,Furniture,0.0,20.0,False,True,Genuine,Report #8683
Diary,Furniture,0.0,20.0,False,True,Genuine,
Diary,Furniture,0.0,20.0,False,True,Fake,Report #8684
Diary,Furniture,0.0,20.0,False,True,Fake,
Diary,Furniture,0.0,20.0,False,True,,Report #8685
Diary,Furniture,0.0,20.0,False,True,,
Diary,Furniture,0.0,20.0,False,False,Genuine,Report #8686
Diary,Furniture,0.0,20.0,False,False,Genuine,
Diary,Furniture,0.0,20.0,False,False,Fake,Report #8687
Diary,Furniture,0.0,20.0,False,False,Fake,
Diary,Furniture,0.0,20.0,False,False,,Report #8688
Diary,Furniture,0.0,20.0,False,False,,
Diary,Furniture,0.0,21.0,True,True,Genuine,Report #8689
Diary,Furniture,0.0,21.0,True,True,Genuine,
Diary,Furniture,0.0,21.0,True,True,Fake,Report #8690
Diary,Furniture,0.0,21.0,True,True,Fake,
Diary,Furniture,0.0,21.0,True,True,,Report #8691
Diary,Furniture,0.0,21.0,True,True,,
Diary,Furniture,0.0,21.0,True,False,Genuine,Report #8692
Diary,Furniture,0.0,21.0,True,False,Genuine,
Diary,Furniture,0.0,21.0,True,False,Fake,Report #8693
Diary,Furniture,0.0,21.0,True,False,Fake,
Diary,Furniture,0.0,21.0,True,False,,Report #8694
Diary,Furniture,0.0,21.0,True,False,,
Diary,Furniture,0.0,21.0,False,True,Genuine,Report #8695
Diary,Furniture,0.0,21.0,False,True,Genuine,
Diary,Furniture,0.0,21.0,False,True,Fake,Report #8696
Diary,Furniture,0.0,21.0,False,True,Fake,
Diary,Furniture,0.0,21.0,False,True,,Report #8697
Diary,Furniture,0.0,21.0,False,True,,
Diary,Furniture,0.0,21.0,False,False,Genuine,Report #8698
Diary,Furniture,0.0,21.0,False,False,Genuine,
Diary,Furniture,0.0,21.0,False,False,Fake,Report #8699
Diary,Furniture,0.0,21.0,False,False,Fake,
Diary,Furniture,0.0,21.0,False,False,,Report #8700
Diary,Furniture,0.0,21.0,False,False,,
Diary,Furniture,0.0,99.0,True,True,Genuine,Report #8701
Diary,Furniture,0.0,99.0,True,True,Genuine,
Diary,Furniture,0.0,99.0,True,True,Fake,Report #8702
Diary,Furniture,0.0,99.0,True,True,Fake,
Diary,Furniture,0.0,99.0,True,True,,Report #8703
Diary,Furniture,0.0,99.0,True,True,,
Diary,Furniture,0.0,99.0,True,False,Genuine,Report #8704
Diary,Furniture,0.0,99.0,True,False,Genuine,
Diary,Furniture,0.0,99.0,True,False,Fake,Report #8705
Diary,Furniture,0.0,99.0,True,False,Fake,
Diary,Furniture,0.0,99.0,True,False,,Report #8706
Diary,Furniture,0.0,99.0,True,False,,
Diary,Furniture,0.0,99.0,False,True,Genuine,Report #8707
Diary,Furniture,0.0,99.0,False,True,Genuine,
Diary,Furniture,0.0,99.0,False,True,Fake,Report #8708
Diary,Furniture,0.0,99.0,False,True,Fake,
Diary,Furniture,0.0,99.0,False,True,,Report #8709
Diary,Furniture,0.0,99.0,False,True,,
Diary,Furniture,0.0,99.0,False,False,Genuine,Report #8710
Diary,Furniture,0.0,99.0,False,False,Genuine,
Diary,Furniture,0.0,99.0,False,False,Fake,Report #8711
Diary,Furniture,0.0,99.0,False,False,Fake,
Diary,Furniture,0.0,99.0,False,False,,Report #8712
Diary,Furniture,0.0,99.0,False,False,,
Diary,Furniture,0.0,100.0,True,True,Genuine,Report #8713
Diary,Furniture,0.0,100.0,True,True,Genuine,
Diary,Furniture,0.0,100.0,True,True,Fake,Report #8714
Diary,Furniture,0.0,100.0,True,True,Fake,
Diary,Furniture,0.0,100.0,True,True,,Report #8715
Diary,Furniture,0.0,100.0,True,True,,
Diary,Furniture,0.0,100.0,True,False,Genuine,Report #8716
Diary,Furniture,0.0,100.0,True,False,Genuine,
Diary,Furniture,0.0,100.0,True,False,Fake,Report #8717
Diary,Furniture,0.0,100.0,True,False,Fake,
Diary,Furniture,0.0,100.0,True,False,,Report #8718
Diary,Furniture,0.0,100.0,True,False,,
Diary,Furniture,0.0,100.0,False,True,Genuine,Report #8719
Diary,Furniture,0.0,100.0,False,True,Genuine,
Diary,Furniture,0.0,100.0,False,True,Fake,Report #8720
Diary,Furniture,0.0,100.0,False,True,Fake,
Diary,Furniture,0.0,100.0,False,True,,Report #8721
Diary,Furniture,0.0,100.0,False,True,,
Diary,Furniture,0.0,100.0,False,False,Genuine,Report #8722
Diary,Furniture,0.0,100.0,False,False,Genuine,
Diary,Furniture,0.0,100.0,False,False,Fake,Report #8723
Diary,Furniture,0.0,100.0,False,False,Fake,
Diary,Furniture,0.0,100.0,False,False,,Report #8724
Diary,Furniture,0.0,100.0,False,False,,
Diary,Furniture,0.0,101.0,True,True,Genuine,Report #8725
Diary,Furniture,0.0,101.0,True,True,Genuine,
Diary,Furniture,0.0,101.0,True,True,Fake,Report #8726
Diary,Furniture,0.0,101.0,True,True,Fake,
Diary,Furniture,0.0,101.0,True,True,,Report #8727
Diary,Furniture,0.0,101.0,True,True,,
Diary,Furniture,0.0,101.0,True,False,Genuine,Report #8728
Diary,Furniture,0.0,101.0,True,False,Genuine,
Diary,Furniture,0.0,101.0,True,False,Fake,Report #8729
Diary,Furniture,0.0,101.0,True,False,Fake,
Diary,Furniture,0.0,101.0,True,False,,Report #8730
Diary,Furniture,0.0,101.0,True,False,,
Diary,Furniture,0.0,101.0,False,True,Genuine,Report #8731
Diary,Furniture,0.0,101.0,False,True,Genuine,
Diary,Furniture,0.0,101.0,False,True,Fake,Report #8732
Diary,Furniture,0.0,101.0,False,True,Fake,
Diary,Furniture,0.0,101.0,False,True,,Report #8733
Diary,Furniture,0.0,101.0,False,True,,
Diary,Furniture,0.0,101.0,False,False,Genuine,Report #8734
Diary,Furniture,0.0,101.0,False,False,Genuine,
Diary,Furniture,0.0,101.0,False,False,Fake,Report #8735
Diary,Furniture,0.0,101.0,False,False,Fake,
Diary,Furniture,0.0,101.0,False,False,,Report #8736
Diary,Furniture,0.0,101.0,False,False,,
Diary,Furniture,1.0,-1.0,True,True,Genuine,Report #8737
Diary,Furniture,1.0,-1.0,True,True,Genuine,
Diary,Furniture,1.0,-1.0,True,True,Fake,Report #8738
Diary,Furniture,1.0,-1.0,True,True,Fake,
Diary,Furniture,1.0,-1.0,True,True,,Report #8739
Diary,Furniture,1.0,-1.0,True,True,,
Diary,Furniture,1.0,-1.0,True,False,Genuine,Report #8740
Diary,Furniture,1.0,-1.0,True,False,Genuine,
Diary,Furniture,1.0,-1.0,True,False,Fake,Report #8741
Diary,Furniture,1.0,-1.0,True,False,Fake,
Diary,Furniture,1.0,-1.0,True,False,,Report #8742
Diary,Furniture,1.0,-1.0,True,False,,
Diary,Furniture,1.0,-1.0,False,True,Genuine,Report #8743
Diary,Furniture,1.0,-1.0,False,True,Genuine,
Diary,Furniture,1.0,-1.0,False,True,Fake,Report #8744
Diary,Furniture,1.0,-1.0,False,True,Fake,
Diary,Furniture,1.0,-1.0,False,True,,Report #8745
Diary,Furniture,1.0,-1.0,False,True,,
Diary,Furniture,1.0,-1.0,False,False,Genuine,Report #8746
Diary,Furniture,1.0,-1.0,False,False,Genuine,
Diary,Furniture,1.0,-1.0,False,False,Fake,Report #8747
Diary,Furniture,1.0,-1.0,False,False,Fake,
Diary,Furniture,1.0,-1.0,False,False,,Report #8748
Diary,Furniture,1.0,-1.0,False,False,,
Diary,Furniture,1.0,0.0,True,True,Genuine,Report #8749
Diary,Furniture,1.0,0.0,True,True,Genuine,
Diary,Furniture,1.0,0.0,True,True,Fake,Report #8750
Diary,Furniture,1.0,0.0,True,True,Fake,
Diary,Furniture,1.0,0.0,True,True,,Report #8751
Diary,Furniture,1.0,0.0,True,True,,
Diary,Furniture,1.0,0.0,True,False,Genuine,Report #8752
Diary,Furniture,1.0,0.0,True,False,Genuine,
Diary,Furniture,1.0,0.0,True,False,Fake,Report #8753
Diary,Furniture,1.0,0.0,True,False,Fake,
Diary,Furniture,1.0,0.0,True,False,,Report #8754
Diary,Furniture,1.0,0.0,True,False,,
Diary,Furniture,1.0,0.0,False,True,Genuine,Report #8755
Diary,Furniture,1.0,0.0,False,True,Genuine,
Diary,Furniture,1.0,0.0,False,True,Fake,Report #8756
Diary,Furniture,1.0,0.0,False,True,Fake,
Diary,Furniture,1.0,0.0,False,True,,Report #8757
Diary,Furniture,1.0,0.0,False,True,,
Diary,Furniture,1.0,0.0,False,False,Genuine,Report #8758
Diary,Furniture,1.0,0.0,False,False,Genuine,
Diary,Furniture,1.0,0.0,False,False,Fake,Report #8759
Diary,Furniture,1.0,0.0,False,False,Fake,
Diary,Furniture,1.0,0.0,False,False,,Report #8760
Diary,Furniture,1.0,0.0,False,False,,
Diary,Furniture,1.0,19.0,True,True,Genuine,Report #8761
Diary,Furniture,1.0,19.0,True,True,Genuine,
Diary,Furniture,1.0,19.0,True,True,Fake,Report #8762
Diary,Furniture,1.0,19.0,True,True,Fake,
Diary,Furniture,1.0,19.0,True,True,,Report #8763
Diary,Furniture,1.0,19.0,True,True,,
Diary,Furniture,1.0,19.0,True,False,Genuine,Report #8764
Diary,Furniture,1.0,19.0,True,False,Genuine,
Diary,Furniture,1.0,19.0,True,False,Fake,Report #8765
Diary,Furniture,1.0,19.0,True,False,Fake,
Diary,Furniture,1.0,19.0,True,False,,Report #8766
Diary,Furniture,1.0,19.0,True,False,,
Diary,Furniture,1.0,19.0,False,True,Genuine,Report #8767
Diary,Furniture,1.0,19.0,False,True,Genuine,
Diary,Furniture,1.0,19.0,False,True,Fake,Report #8768
Diary,Furniture,1.0,19.0,False,True,Fake,
Diary,Furniture,1.0,19.0,False,True,,Report #8769
Diary,Furniture,1.0,19.0,False,True,,
Diary,Furniture,1.0,19.0,False,False,Genuine,Report #8770
Diary,Furniture,1.0,19.0,False,False,Genuine,
Diary,Furniture,1.0,19.0,False,False,Fake,Report #8771
Diary,Furniture,1.0,19.0,False,False,Fake,
Diary,Furniture,1.0,19.0,False,False,,Report #8772
Diary,Furniture,1.0,19.0,False,False,,
Diary,Furniture,1.0,20.0,True,True,Genuine,Report #8773
Diary,Furniture,1.0,20.0,True,True,Genuine,
Diary,Furniture,1.0,20.0,True,True,Fake,Report #8774
Diary,Furniture,1.0,20.0,True,True,Fake,
Diary,Furniture,1.0,20.0,True,True,,Report #8775
Diary,Furniture,1.0,20.0,True,True,,
Diary,Furniture,1.0,20.0,True,False,Genuine,Report #8776
Diary,Furniture,1.0,20.0,True,False,Genuine,
Diary,Furniture,1.0,20.0,True,False,Fake,Report #8777
Diary,Furniture,1.0,20.0,True,False,Fake,
Diary,Furniture,1.0,20.0,True,False,,Report #8778
Diary,Furniture,1.0,20.0,True,False,,
Diary,Furniture,1.0,20.0,False,True,Genuine,Report #8779
Diary,Furniture,1.0,20.0,False,True,Genuine,
Diary,Furniture,1.0,20.0,False,True,Fake,Report #8780
Diary,Furniture,1.0,20.0,False,True,Fake,
Diary,Furniture,1.0,20.0,False,True,,Report #8781
Diary,Furniture,1.0,20.0,False,True,,
Diary,Furniture,1.0,20.0,False,False,Genuine,Report #8782
Diary,Furniture,1.0,20.0,False,False,Genuine,
Diary,Furniture,1.0,20.0,False,False,Fake,Report #8783
Diary,Furniture,1.0,20.0,False,False,Fake,
Diary,Furniture,1.0,20.0,False,False,,Report #8784
Diary,Furniture,1.0,20.0,False,False,,
Diary,Furniture,1.0,21.0,True,True,Genuine,Report #8785
Diary,Furniture,1.0,21.0,True,True,Genuine,
Diary,Furniture,1.0,21.0,True,True,Fake,Report #8786
Diary,Furniture,1.0,21.0,True,True,Fake,
Diary,Furniture,1.0,21.0,True,True,,Report #8787
Diary,Furniture,1.0,21.0,True,True,,
Diary,Furniture,1.0,21.0,True,False,Genuine,Report #8788
Diary,Furniture,1.0,21.0,True,False,Genuine,
Diary,Furniture,1.0,21.0,True,False,Fake,Report #8789
Diary,Furniture,1.0,21.0,True,False,Fake,
Diary,Furniture,1.0,21.0,True,False,,Report #8790
Diary,Furniture,1.0,21.0,True,False,,
Diary,Furniture,1.0,21.0,False,True,Genuine,Report #8791
Diary,Furniture,1.0,21.0,False,True,Genuine,
Diary,Furniture,1.0,21.0,False,True,Fake,Report #8792
Diary,Furniture,1.0,21.0,False,True,Fake,
Diary,Furniture,1.0,21.0,False,True,,Report #8793
Diary,Furniture,1.0,21.0,False,True,,
Diary,Furniture,1.0,21.0,False,False,Genuine,Report #8794
Diary,Furniture,1.0,21.0,False,False,Genuine,
Diary,Furniture,1.0,21.0,False,False,Fake,Report #8795
Diary,Furniture,1.0,21.0,False,False,Fake,
Diary,Furniture,1.0,21.0,False,False,,Report #8796
Diary,Furniture,1.0,21.0,False,False,,
Diary,Furniture,1.0,99.0,True,True,Genuine,Report #8797
Diary,Furniture,1.0,99.0,True,True,Genuine,
Diary,Furniture,1.0,99.0,True,True,Fake,Report #8798
Diary,Furniture,1.0,99.0,True,True,Fake,
Diary,Furniture,1.0,99.0,True,True,,Report #8799
Diary,Furniture,1.0,99.0,True,True,,
Diary,Furniture,1.0,99.0,True,False,Genuine,Report #8800
Diary,Furniture,1.0,99.0,True,False,Genuine,
Diary,Furniture,1.0,99.0,True,False,Fake,Report #8801
Diary,Furniture,1.0,99.0,True,False,Fake,
Diary,Furniture,1.0,99.0,True,False,,Report #8802
Diary,Furniture,1.0,99.0,True,False,,
Diary,Furniture,1.0,99.0,False,True,Genuine,Report #8803
Diary,Furniture,1.0,99.0,False,True,Genuine,
Diary,Furniture,1.0,99.0,False,True,Fake,Report #8804
Diary,Furniture,1.0,99.0,False,True,Fake,
Diary,Furniture,1.0,99.0,False,True,,Report #8805
Diary,Furniture,1.0,99.0,False,True,,
Diary,Furniture,1.0,99.0,False,False,Genuine,Report #8806
Diary,Furniture,1.0,99.0,False,False,Genuine,
Diary,Furniture,1.0,99.0,False,False,Fake,Report #8807
Diary,Furniture,1.0,99.0,False,False,Fake,
Diary,Furniture,1.0,99.0,False,False,,Report #8808
Diary,Furniture,1.0,99.0,False,False,,
Diary,Furniture,1.0,100.0,True,True,Genuine,Report #8809
Diary,Furniture,1.0,100.0,True,True,Genuine,
Diary,Furniture,1.0,100.0,True,True,Fake,Report #8810
Diary,Furniture,1.0,100.0,True,True,Fake,
Diary,Furniture,1.0,100.0,True,True,,Report #8811
Diary,Furniture,1.0,100.0,True,True,,
Diary,Furniture,1.0,100.0,True,False,Genuine,Report #8812
Diary,Furniture,1.0,100.0,True,False,Genuine,
Diary,Furniture,1.0,100.0,True,False,Fake,Report #8813
Diary,Furniture,1.0,100.0,True,False,Fake,
Diary,Furniture,1.0,100.0,True,False,,Report #8814
Diary,Furniture,1.0,100.0,True,False,,
Diary,Furniture,1.0,100.0,False,True,Genuine,Report #8815
Diary,Furniture,1.0,100.0,False,True,Genuine,
Diary,Furniture,1.0,100.0,False,True,Fake,Report #8816
Diary,Furniture,1.0,100.0,False,True,Fake,
Diary,Furniture,1.0,100.0,False,True,,Report #8817
Diary,Furniture,1.0,100.0,False,True,,
Diary,Furniture,1.0,100.0,False,False,Genuine,Report #8818
Diary,Furniture,1.0,100.0,False,False,Genuine,
Diary,Furniture,1.0,100.0,False,False,Fake,Report #8819
Diary,Furniture,1.0,100.0,False,False,Fake,
Diary,Furniture,1.0,100.0,False,False,,Report #8820
Diary,Furniture,1.0,100.0,False,False,,
Diary,Furniture,1.0,101.0,True,True,Genuine,Report #8821
Diary,Furniture,1.0,101.0,True,True,Genuine,
Diary,Furniture,1.0,101.0,True,True,Fake,Report #8822
Diary,Furniture,1.0,101.0,True,True,Fake,
Diary,Furniture,1.0,101.0,True,True,,Report #8823
Diary,Furniture,1.0,101.0,True,True,,
Diary,Furniture,1.0,101.0,True,False,Genuine,Report #8824
Diary,Furniture,1.0,101.0,True,False,Genuine,
Diary,Furniture,1.0,101.0,True,False,Fake,Report #8825
Diary,Furniture,1.0,101.0,True,False,Fake,
Diary,Furniture,1.0,101.0,True,False,,Report #8826
Diary,Furniture,1.0,101.0,True,False,,
Diary,Furniture,1.0,101.0,False,True,Genuine,Report #8827
Diary,Furniture,1.0,101.0,False,True,Genuine,
Diary,Furniture,1.0,101.0,False,True,Fake,Report #8828
Diary,Furniture,1.0,101.0,False,True,Fake,
Diary,Furniture,1.0,101.0,False,True,,Report #8829
Diary,Furniture,1.0,101.0,False,True,,
Diary,Furniture,1.0,101.0,False,False,Genuine,Report #8830
Diary,Furniture,1.0,101.0,False,False,Genuine,
Diary,Furniture,1.0,101.0,False,False,Fake,Report #8831
Diary,Furniture,1.0,101.0,False,False,Fake,
Diary,Furniture,1.0,101.0,False,False,,Report #8832
Diary,Furniture,1.0,101.0,False,False,,
Diary,Furniture,100.0,-1.0,True,True,Genuine,Report #8833
Diary,Furniture,100.0,-1.0,True,True,Genuine,
Diary,Furniture,100.0,-1.0,True,True,Fake,Report #8834
Diary,Furniture,100.0,-1.0,True,True,Fake,
Diary,Furniture,100.0,-1.0,True,True,,Report #8835
Diary,Furniture,100.0,-1.0,True,True,,
Diary,Furniture,100.0,-1.0,True,False,Genuine,Report #8836
Diary,Furniture,100.0,-1.0,True,False,Genuine,
Diary,Furniture,100.0,-1.0,True,False,Fake,Report #8837
Diary,Furniture,100.0,-1.0,True,False,Fake,
Diary,Furniture,100.0,-1.0,True,False,,Report #8838
Diary,Furniture,100.0,-1.0,True,False,,
Diary,Furniture,100.0,-1.0,False,True,Genuine,Report #8839
Diary,Furniture,100.0,-1.0,False,True,Genuine,
Diary,Furniture,100.0,-1.0,False,True,Fake,Report #8840
Diary,Furniture,100.0,-1.0,False,True,Fake,
Diary,Furniture,100.0,-1.0,False,True,,Report #8841
Diary,Furniture,100.0,-1.0,False,True,,
Diary,Furniture,100.0,-1.0,False,False,Genuine,Report #8842
Diary,Furniture,100.0,-1.0,False,False,Genuine,
Diary,Furniture,100.0,-1.0,False,False,Fake,Report #8843
Diary,Furniture,100.0,-1.0,False,False,Fake,
Diary,Furniture,100.0,-1.0,False,False,,Report #8844
Diary,Furniture,100.0,-1.0,False,False,,
Diary,Furniture,100.0,0.0,True,True,Genuine,Report #8845
Diary,Furniture,100.0,0.0,True,True,Genuine,
Diary,Furniture,100.0,0.0,True,True,Fake,Report #8846
Diary,Furniture,100.0,0.0,True,True,Fake,
Diary,Furniture,100.0,0.0,True,True,,Report #8847
Diary,Furniture,100.0,0.0,True,True,,
Diary,Furniture,100.0,0.0,True,False,Genuine,Report #8848
Diary,Furniture,100.0,0.0,True,False,Genuine,
Diary,Furniture,100.0,0.0,True,False,Fake,Report #8849
Diary,Furniture,100.0,0.0,True,False,Fake,
Diary,Furniture,100.0,0.0,True,False,,Report #8850
Diary,Furniture,100.0,0.0,True,False,,
Diary,Furniture,100.0,0.0,False,True,Genuine,Report #8851
Diary,Furniture,100.0,0.0,False,True,Genuine,
Diary,Furniture,100.0,0.0,False,True,Fake,Report #8852
Diary,Furniture,100.0,0.0,False,True,Fake,
Diary,Furniture,100.0,0.0,False,True,,Report #8853
Diary,Furniture,100.0,0.0,False,True,,
Diary,Furniture,100.0,0.0,False,False,Genuine,Report #8854
Diary,Furniture,100.0,0.0,False,False,Genuine,
Diary,Furniture,100.0,0.0,False,False,Fake,Report #8855
Diary,Furniture,100.0,0.0,False,False,Fake,
Diary,Furniture,100.0,0.0,False,False,,Report #8856
Diary,Furniture,100.0,0.0,False,False,,
Diary,Furniture,100.0,19.0,True,True,Genuine,Report #8857
Diary,Furniture,100.0,19.0,True,True,Genuine,
Diary,Furniture,100.0,19.0,True,True,Fake,Report #8858
Diary,Furniture,100.0,19.0,True,True,Fake,
Diary,Furniture,100.0,19.0,True,True,,Report #8859
Diary,Furniture,100.0,19.0,True,True,,
Diary,Furniture,100.0,19.0,True,False,Genuine,Report #8860
Diary,Furniture,100.0,19.0,True,False,Genuine,
Diary,Furniture,100.0,19.0,True,False,Fake,Report #8861
Diary,Furniture,100.0,19.0,True,False,Fake,
Diary,Furniture,100.0,19.0,True,False,,Report #8862
Diary,Furniture,100.0,19.0,True,False,,
Diary,Furniture,100.0,19.0,False,True,Genuine,Report #8863
Diary,Furniture,100.0,19.0,False,True,Genuine,
Diary,Furniture,100.0,19.0,False,True,Fake,Report #8864
Diary,Furniture,100.0,19.0,False,True,Fake,
Diary,Furniture,100.0,19.0,False,True,,Report #8865
Diary,Furniture,100.0,19.0,False,True,,
Diary,Furniture,100.0,19.0,False,False,Genuine,Report #8866
Diary,Furniture,100.0,19.0,False,False,Genuine,
Diary,Furniture,100.0,19.0,False,False,Fake,Report #8867
Diary,Furniture,100.0,19.0,False,False,Fake,
Diary,Furniture,100.0,19.0,False,False,,Report #8868
Diary,Furniture,100.0,19.0,False,False,,
Diary,Furniture,100.0,20.0,True,True,Genuine,Report #8869
Diary,Furniture,100.0,20.0,True,True,Genuine,
Diary,Furniture,100.0,20.0,True,True,Fake,Report #8870
Diary,Furniture,100.0,20.0,True,True,Fake,
Diary,Furniture,100.0,20.0,True,True,,Report #8871
Diary,Furniture,100.0,20.0,True,True,,
Diary,Furniture,100.0,20.0,True,False,Genuine,Report #8872
Diary,Furniture,100.0,20.0,True,False,Genuine,
Diary,Furniture,100.0,20.0,True,False,Fake,Report #8873
Diary,Furniture,100.0,20.0,True,False,Fake,
Diary,Furniture,100.0,20.0,True,False,,Report #8874
Diary,Furniture,100.0,20.0,True,False,,
Diary,Furniture,100.0,20.0,False,True,Genuine,Report #8875
Diary,Furniture,100.0,20.0,False,True,Genuine,
Diary,Furniture,100.0,20.0,False,True,Fake,Report #8876
Diary,Furniture,100.0,20.0,False,True,Fake,
Diary,Furniture,100.0,20.0,False,True,,Report #8877
Diary,Furniture,100.0,20.0,False,True,,
Diary,Furniture,100.0,20.0,False,False,Genuine,Report #8878
Diary,Furniture,100.0,20.0,False,False,Genuine,
Diary,Furniture,100.0,20.0,False,False,Fake,Report #8879
Diary,Furniture,100.0,20.0,False,False,Fake,
Diary,Furniture,100.0,20.0,False,False,,Report #8880
Diary,Furniture,100.0,20.0,False,False,,
Diary,Furniture,100.0,21.0,True,True,Genuine,Report #8881
Diary,Furniture,100.0,21.0,True,True,Genuine,
Diary,Furniture,100.0,21.0,True,True,Fake,Report #8882
Diary,Furniture,100.0,21.0,True,True,Fake,
Diary,Furniture,100.0,21.0,True,True,,Report #8883
Diary,Furniture,100.0,21.0,True,True,,
Diary,Furniture,100.0,21.0,True,False,Genuine,Report #8884
Diary,Furniture,100.0,21.0,True,False,Genuine,
Diary,Furniture,100.0,21.0,True,False,Fake,Report #8885
Diary,Furniture,100.0,21.0,True,False,Fake,
Diary,Furniture,100.0,21.0,True,False,,Report #8886
Diary,Furniture,100.0,21.0,True,False,,
Diary,Furniture,100.0,21.0,False,True,Genuine,Report #8887
Diary,Furniture,100.0,21.0,False,True,Genuine,
Diary,Furniture,100.0,21.0,False,True,Fake,Report #8888
Diary,Furniture,100.0,21.0,False,True,Fake,
Diary,Furniture,100.0,21.0,False,True,,Report #8889
Diary,Furniture,100.0,21.0,False,True,,
Diary,Furniture,100.0,21.0,False,False,Genuine,Report #8890
Diary,Furniture,100.0,21.0,False,False,Genuine,
Diary,Furniture,100.0,21.0,False,False,Fake,Report #8891
Diary,Furniture,100.0,21.0,False,False,Fake,
Diary,Furniture,100.0,21.0,False,False,,Report #8892
Diary,Furniture,100.0,21.0,False,False,,
Diary,Furniture,100.0,99.0,True,True,Genuine,Report #8893
Diary,Furniture,100.0,99.0,True,True,Genuine,
Diary,Furniture,100.0,99.0,True,True,Fake,Report #8894
Diary,Furniture,100.0,99.0,True,True,Fake,
Diary,Furniture,100.0,99.0,True,True,,Report #8895
Diary,Furniture,100.0,99.0,True,True,,
Diary,Furniture,100.0,99.0,True,False,Genuine,Report #8896
Diary,Furniture,100.0,99.0,True,False,Genuine,
Diary,Furniture,100.0,99.0,True,False,Fake,Report #8897
Diary,Furniture,100.0,99.0,True,False,Fake,
Diary,Furniture,100.0,99.0,True,False,,Report #8898
Diary,Furniture,100.0,99.0,True,False,,
Diary,Furniture,100.0,99.0,False,True,Genuine,Report #8899
Diary,Furniture,100.0,99.0,False,True,Genuine,
Diary,Furniture,100.0,99.0,False,True,Fake,Report #8900
Diary,Furniture,100.0,99.0,False,True,Fake,
Diary,Furniture,100.0,99.0,False,True,,Report #8901
Diary,Furniture,100.0,99.0,False,True,,
Diary,Furniture,100.0,99.0,False,False,Genuine,Report #8902
Diary,Furniture,100.0,99.0,False,False,Genuine,
Diary,Furniture,100.0,99.0,False,False,Fake,Report #8903
Diary,Furniture,100.0,99.0,False,False,Fake,
Diary,Furniture,100.0,99.0,False,False,,Report #8904
Diary,Furniture,100.0,99.0,False,False,,
Diary,Furniture,100.0,100.0,True,True,Genuine,Report #8905
Diary,Furniture,100.0,100.0,True,True,Genuine,
Diary,Furniture,100.0,100.0,True,True,Fake,Report #8906
Diary,Furniture,100.0,100.0,True,True,Fake,
Diary,Furniture,100.0,100.0,True,True,,Report #8907
Diary,Furniture,100.0,100.0,True,True,,
Diary,Furniture,100.0,100.0,True,False,Genuine,Report #8908
Diary,Furniture,100.0,100.0,True,False,Genuine,
Diary,Furniture,100.0,100.0,True,False,Fake,Report #8909
Diary,Furniture,100.0,100.0,True,False,Fake,
Diary,Furniture,100.0,100.0,True,False,,Report #8910
Diary,Furniture,100.0,100.0,True,False,,
Diary,Furniture,100.0,100.0,False,True,Genuine,Report #8911
Diary,Furniture,100.0,100.0,False,True,Genuine,
Diary,Furniture,100.0,100.0,False,True,Fake,Report #8912
Diary,Furniture,100.0,100.0,False,True,Fake,
Diary,Furniture,100.0,100.0,False,True,,Report #8913
Diary,Furniture,100.0,100.0,False,True,,
Diary,Furniture,100.0,100.0,False,False,Genuine,Report #8914
Diary,Furniture,100.0,100.0,False,False,Genuine,
Diary,Furniture,100.0,100.0,False,False,Fake,Report #8915
Diary,Furniture,100.0,100.0,False,False,Fake,
Diary,Furniture,100.0,100.0,False,False,,Report #8916
Diary,Furniture,100.0,100.0,False,False,,
Diary,Furniture,100.0,101.0,True,True,Genuine,Report #8917
Diary,Furniture,100.0,101.0,True,True,Genuine,
Diary,Furniture,100.0,101.0,True,True,Fake,Report #8918
Diary,Furniture,100.0,101.0,True,True,Fake,
Diary,Furniture,100.0,101.0,True,True,,Report #8919
Diary,Furniture,100.0,101.0,True,True,,
Diary,Furniture,100.0,101.0,True,False,Genuine,Report #8920
Diary,Furniture,100.0,101.0,True,False,Genuine,
Diary,Furniture,100.0,101.0,True,False,Fake,Report #8921
Diary,Furniture,100.0,101.0,True,False,Fake,
Diary,Furniture,100.0,101.0,True,False,,Report #8922
Diary,Furniture,100.0,101.0,True,False,,
Diary,Furniture,100.0,101.0,False,True,Genuine,Report #8923
Diary,Furniture,100.0,101.0,False,True,Genuine,
Diary,Furniture,100.0,101.0,False,True,Fake,Report #8924
Diary,Furniture,100.0,101.0,False,True,Fake,
Diary,Furniture,100.0,101.0,False,True,,Report #8925
Diary,Furniture,100.0,101.0,False,True,,
Diary,Furniture,100.0,101.0,False,False,Genuine,Report #8926
Diary,Furniture,100.0,101.0,False,False,Genuine,
Diary,Furniture,100.0,101.0,False,False,Fake,Report #8927
Diary,Furniture,100.0,101.0,False,False,Fake,
Diary,Furniture,100.0,101.0,False,False,,Report #8928
Diary,Furniture,100.0,101.0,False,False,,
Diary,Furniture,1000.0,-1.0,True,True,Genuine,Report #8929
Diary,Furniture,1000.0,-1.0,True,True,Genuine,
Diary,Furniture,1000.0,-1.0,True,True,Fake,Report #8930
Diary,Furniture,1000.0,-1.0,True,True,Fake,
Diary,Furniture,1000.0,-1.0,True,True,,Report #8931
Diary,Furniture,1000.0,-1.0,True,True,,
Diary,Furniture,1000.0,-1.0,True,False,Genuine,Report #8932
Diary,Furniture,1000.0,-1.0,True,False,Genuine,
Diary,Furniture,1000.0,-1.0,True,False,Fake,Report #8933
Diary,Furniture,1000.0,-1.0,True,False,Fake,
Diary,Furniture,1000.0,-1.0,True,False,,Report #8934
Diary,Furniture,1000.0,-1.0,True,False,,
Diary,Furniture,1000.0,-1.0,False,True,Genuine,Report #8935
Diary,Furniture,1000.0,-1.0,False,True,Genuine,
Diary,Furniture,1000.0,-1.0,False,True,Fake,Report #8936
Diary,Furniture,1000.0,-1.0,False,True,Fake,
Diary,Furniture,1000.0,-1.0,False,True,,Report #8937
Diary,Furniture,1000.0,-1.0,False,True,,
Diary,Furniture,1000.0,-1.0,False,False,Genuine,Report #8938
Diary,Furniture,1000.0,-1.0,False,False,Genuine,
Diary,Furniture,1000.0,-1.0,False,False,Fake,Report #8939
Diary,Furniture,1000.0,-1.0,False,False,Fake,
Diary,Furniture,1000.0,-1.0,False,False,,Report #8940
Diary,Furniture,1000.0,-1.0,False,False,,
Diary,Furniture,1000.0,0.0,True,True,Genuine,Report #8941
Diary,Furniture,1000.0,0.0,True,True,Genuine,
Diary,Furniture,1000.0,0.0,True,True,Fake,Report #8942
Diary,Furniture,1000.0,0.0,True,True,Fake,
Diary,Furniture,1000.0,0.0,True,True,,Report #8943
Diary,Furniture,1000.0,0.0,True,True,,
Diary,Furniture,1000.0,0.0,True,False,Genuine,Report #8944
Diary,Furniture,1000.0,0.0,True,False,Genuine,
Diary,Furniture,1000.0,0.0,True,False,Fake,Report #8945
Diary,Furniture,1000.0,0.0,True,False,Fake,
Diary,Furniture,1000.0,0.0,True,False,,Report #8946
Diary,Furniture,1000.0,0.0,True,False,,
Diary,Furniture,1000.0,0.0,False,True,Genuine,Report #8947
Diary,Furniture,1000.0,0.0,False,True,Genuine,
Diary,Furniture,1000.0,0.0,False,True,Fake,Report #8948
Diary,Furniture,1000.0,0.0,False,True,Fake,
Diary,Furniture,1000.0,0.0,False,True,,Report #8949
Diary,Furniture,1000.0,0.0,False,True,,
Diary,Furniture,1000.0,0.0,False,False,Genuine,Report #8950
Diary,Furniture,1000.0,0.0,False,False,Genuine,
Diary,Furniture,1000.0,0.0,False,False,Fake,Report #8951
Diary,Furniture,1000.0,0.0,False,False,Fake,
Diary,Furniture,1000.0,0.0,False,False,,Report #8952
Diary,Furniture,1000.0,0.0,False,False,,
Diary,Furniture,1000.0,19.0,True,True,Genuine,Report #8953
Diary,Furniture,1000.0,19.0,True,True,Genuine,
Diary,Furniture,1000.0,19.0,True,True,Fake,Report #8954
Diary,Furniture,1000.0,19.0,True,True,Fake,
Diary,Furniture,1000.0,19.0,True,True,,Report #8955
Diary,Furniture,1000.0,19.0,True,True,,
Diary,Furniture,1000.0,19.0,True,False,Genuine,Report #8956
Diary,Furniture,1000.0,19.0,True,False,Genuine,
Diary,Furniture,1000.0,19.0,True,False,Fake,Report #8957
Diary,Furniture,1000.0,19.0,True,False,Fake,
Diary,Furniture,1000.0,19.0,True,False,,Report #8958
Diary,Furniture,1000.0,19.0,True,False,,
Diary,Furniture,1000.0,19.0,False,True,Genuine,Report #8959
Diary,Furniture,1000.0,19.0,False,True,Genuine,
Diary,Furniture,1000.0,19.0,False,True,Fake,Report #8960
Diary,Furniture,1000.0,19.0,False,True,Fake,
Diary,Furniture,1000.0,19.0,False,True,,Report #8961
Diary,Furniture,1000.0,19.0,False,True,,
Diary,Furniture,1000.0,19.0,False,False,Genuine,Report #8962
Diary,Furniture,1000.0,19.0,False,False,Genuine,
Diary,Furniture,1000.0,19.0,False,False,Fake,Report #8963
Diary,Furniture,1000.0,19.0,False,False,Fake,
Diary,Furniture,1000.0,19.0,False,False,,Report #8964
Diary,Furniture,1000.0,19.0,False,False,,
Diary,Furniture,1000.0,20.0,True,True,Genuine,Report #8965
Diary,Furniture,1000.0,20.0,True,True,Genuine,
Diary,Furniture,1000.0,20.0,True,True,Fake,Report #8966
Diary,Furniture,1000.0,20.0,True,True,Fake,
Diary,Furniture,1000.0,20.0,True,True,,Report #8967
Diary,Furniture,1000.0,20.0,True,True,,
Diary,Furniture,1000.0,20.0,True,False,Genuine,Report #8968
Diary,Furniture,1000.0,20.0,True,False,Genuine,
Diary,Furniture,1000.0,20.0,True,False,Fake,Report #8969
Diary,Furniture,1000.0,20.0,True,False,Fake,
Diary,Furniture,1000.0,20.0,True,False,,Report #8970
Diary,Furniture,1000.0,20.0,True,False,,
Diary,Furniture,1000.0,20.0,False,True,Genuine,Report #8971
Diary,Furniture,1000.0,20.0,False,True,Genuine,
Diary,Furniture,1000.0,20.0,False,True,Fake,Report #8972
Diary,Furniture,1000.0,20.0,False,True,Fake,
Diary,Furniture,1000.0,20.0,False,True,,Report #8973
Diary,Furniture,1000.0,20.0,False,True,,
Diary,Furniture,1000.0,20.0,False,False,Genuine,Report #8974
Diary,Furniture,1000.0,20.0,False,False,Genuine,
Diary,Furniture,1000.0,20.0,False,False,Fake,Report #8975
Diary,Furniture,1000.0,20.0,False,False,Fake,
Diary,Furniture,1000.0,20.0,False,False,,Report #8976
Diary,Furniture,1000.0,20.0,False,False,,
Diary,Furniture,1000.0,21.0,True,True,Genuine,Report #8977
Diary,Furniture,1000.0,21.0,True,True,Genuine,
Diary,Furniture,1000.0,21.0,True,True,Fake,Report #8978
Diary,Furniture,1000.0,21.0,True,True,Fake,
Diary,Furniture,1000.0,21.0,True,True,,Report #8979
Diary,Furniture,1000.0,21.0,True,True,,
Diary,Furniture,1000.0,21.0,True,False,Genuine,Report #8980
Diary,Furniture,1000.0,21.0,True,False,Genuine,
Diary,Furniture,1000.0,21.0,True,False,Fake,Report #8981
Diary,Furniture,1000.0,21.0,True,False,Fake,
Diary,Furniture,1000.0,21.0,True,False,,Report #8982
Diary,Furniture,1000.0,21.0,True,False,,
Diary,Furniture,1000.0,21.0,False,True,Genuine,Report #8983
Diary,Furniture,1000.0,21.0,False,True,Genuine,
Diary,Furniture,1000.0,21.0,False,True,Fake,Report #8984
Diary,Furniture,1000.0,21.0,False,True,Fake,
Diary,Furniture,1000.0,21.0,False,True,,Report #8985
Diary,Furniture,1000.0,21.0,False,True,,
Diary,Furniture,1000.0,21.0,False,False,Genuine,Report #8986
Diary,Furniture,1000.0,21.0,False,False,Genuine,
Diary,Furniture,1000.0,21.0,False,False,Fake,Report #8987
Diary,Furniture,1000.0,21.0,False,False,Fake,
Diary,Furniture,1000.0,21.0,False,False,,Report #8988
Diary,Furniture,1000.0,21.0,False,False,,
Diary,Furniture,1000.0,99.0,True,True,Genuine,Report #8989
Diary,Furniture,1000.0,99.0,True,True,Genuine,
Diary,Furniture,1000.0,99.0,True,True,Fake,Report #8990
Diary,Furniture,1000.0,99.0,True,True,Fake,
Diary,Furniture,1000.0,99.0,True,True,,Report #8991
Diary,Furniture,1000.0,99.0,True,True,,
Diary,Furniture,1000.0,99.0,True,False,Genuine,Report #8992
Diary,Furniture,1000.0,99.0,True,False,Genuine,
Diary,Furniture,1000.0,99.0,True,False,Fake,Report #8993
Diary,Furniture,1000.0,99.0,True,False,Fake,
Diary,Furniture,1000.0,99.0,True,False,,Report #8994
Diary,Furniture,1000.0,99.0,True,False,,
Diary,Furniture,1000.0,99.0,False,True,Genuine,Report #8995
Diary,Furniture,1000.0,99.0,False,True,Genuine,
Diary,Furniture,1000.0,99.0,False,True,Fake,Report #8996
Diary,Furniture,1000.0,99.0,False,True,Fake,
Diary,Furniture,1000.0,99.0,False,True,,Report #8997
Diary,Furniture,1000.0,99.0,False,True,,
Diary,Furniture,1000.0,99.0,False,False,Genuine,Report #8998
Diary,Furniture,1000.0,99.0,False,False,Genuine,
Diary,Furniture,1000.0,99.0,False,False,Fake,Report #8999
Diary,Furniture,1000.0,99.0,False,False,Fake,
Diary,Furniture,1000.0,99.0,False,False,,Report #9000
Diary,Furniture,1000.0,99.0,False,False,,
Diary,Furniture,1000.0,100.0,True,True,Genuine,Report #9001
Diary,Furniture,1000.0,100.0,True,True,Genuine,
Diary,Furniture,1000.0,100.0,True,True,Fake,Report #9002
Diary,Furniture,1000.0,100.0,True,True,Fake,
Diary,Furniture,1000.0,100.0,True,True,,Report #9003
Diary,Furniture,1000.0,100.0,True,True,,
Diary,Furniture,1000.0,100.0,True,False,Genuine,Report #9004
Diary,Furniture,1000.0,100.0,True,False,Genuine,
Diary,Furniture,1000.0,100.0,True,False,Fake,Report #9005
Diary,Furniture,1000.0,100.0,True,False,Fake,
Diary,Furniture,1000.0,100.0,True,False,,Report #9006
Diary,Furniture,1000.0,100.0,True,False,,
Diary,Furniture,1000.0,100.0,False,True,Genuine,Report #9007
Diary,Furniture,1000.0,100.0,False,True,Genuine,
Diary,Furniture,1000.0,100.0,False,True,Fake,Report #9008
Diary,Furniture,1000.0,100.0,False,True,Fake,
Diary,Furniture,1000.0,100.0,False,True,,Report #9009
Diary,Furniture,1000.0,100.0,False,True,,
Diary,Furniture,1000.0,100.0,False,False,Genuine,Report #9010
Diary,Furniture,1000.0,100.0,False,False,Genuine,
Diary,Furniture,1000.0,100.0,False,False,Fake,Report #9011
Diary,Furniture,1000.0,100.0,False,False,Fake,
Diary,Furniture,1000.0,100.0,False,False,,Report #9012
Diary,Furniture,1000.0,100.0,False,False,,
Diary,Furniture,1000.0,101.0,True,True,Genuine,Report #9013
Diary,Furniture,1000.0,101.0,True,True,Genuine,
Diary,Furniture,1000.0,101.0,True,True,Fake,Report #9014
Diary,Furniture,1000.0,101.0,True,True,Fake,
Diary,Furniture,1000.0,101.0,True,True,,Report #9015
Diary,Furniture,1000.0,101.0,True,True,,
Diary,Furniture,1000.0,101.0,True,False,Genuine,Report #9016
Diary,Furniture,1000.0,101.0,True,False,Genuine,
Diary,Furniture,1000.0,101.0,True,False,Fake,Report #9017
Diary,Furniture,1000.0,101.0,True,False,Fake,
Diary,Furniture,1000.0,101.0,True,False,,Report #9018
Diary,Furniture,1000.0,101.0,True,False,,
Diary,Furniture,1000.0,101.0,False,True,Genuine,Report #9019
Diary,Furniture,1000.0,101.0,False,True,Genuine,
Diary,Furniture,1000.0,101.0,False,True,Fake,Report #9020
Diary,Furniture,1000.0,101.0,False,True,Fake,
Diary,Furniture,1000.0,101.0,False,True,,Report #9021
Diary,Furniture,1000.0,101.0,False,True,,
Diary,Furniture,1000.0,101.0,False,False,Genuine,Report #9022
Diary,Furniture,1000.0,101.0,False,False,Genuine,
Diary,Furniture,1000.0,101.0,False,False,Fake,Report #9023
Diary,Furniture,1000.0,101.0,False,False,Fake,
Diary,Furniture,1000.0,101.0,False,False,,Report #9024
Diary,Furniture,1000.0,101.0,False,False,,
Diary,Furniture,1000000.0,-1.0,True,True,Genuine,Report #9025
Diary,Furniture,1000000.0,-1.0,True,True,Genuine,
Diary,Furniture,1000000.0,-1.0,True,True,Fake,Report #9026
Diary,Furniture,1000000.0,-1.0,True,True,Fake,
Diary,Furniture,1000000.0,-1.0,True,True,,Report #9027
Diary,Furniture,1000000.0,-1.0,True,True,,
Diary,Furniture,1000000.0,-1.0,True,False,Genuine,Report #9028
Diary,Furniture,1000000.0,-1.0,True,False,Genuine,
Diary,Furniture,1000000.0,-1.0,True,False,Fake,Report #9029
Diary,Furniture,1000000.0,-1.0,True,False,Fake,
Diary,Furniture,1000000.0,-1.0,True,False,,Report #9030
Diary,Furniture,1000000.0,-1.0,True,False,,
Diary,Furniture,1000000.0,-1.0,False,True,Genuine,Report #9031
Diary,Furniture,1000000.0,-1.0,False,True,Genuine,
Diary,Furniture,1000000.0,-1.0,False,True,Fake,Report #9032
Diary,Furniture,1000000.0,-1.0,False,True,Fake,
Diary,Furniture,1000000.0,-1.0,False,True,,Report #9033
Diary,Furniture,1000000.0,-1.0,False,True,,
Diary,Furniture,1000000.0,-1.0,False,False,Genuine,Report #9034
Diary,Furniture,1000000.0,-1.0,False,False,Genuine,
Diary,Furniture,1000000.0,-1.0,False,False,Fake,Report #9035
Diary,Furniture,1000000.0,-1.0,False,False,Fake,
Diary,Furniture,1000000.0,-1.0,False,False,,Report #9036
Diary,Furniture,1000000.0,-1.0,False,False,,
Diary,Furniture,1000000.0,0.0,True,True,Genuine,Report #9037
Diary,Furniture,1000000.0,0.0,True,True,Genuine,
Diary,Furniture,1000000.0,0.0,True,True,Fake,Report #9038
Diary,Furniture,1000000.0,0.0,True,True,Fake,
Diary,Furniture,1000000.0,0.0,True,True,,Report #9039
Diary,Furniture,1000000.0,0.0,True,True,,
Diary,Furniture,1000000.0,0.0,True,False,Genuine,Report #9040
Diary,Furniture,1000000.0,0.0,True,False,Genuine,
Diary,Furniture,1000000.0,0.0,True,False,Fake,Report #9041
Diary,Furniture,1000000.0,0.0,True,False,Fake,
Diary,Furniture,1000000.0,0.0,True,False,,Report #9042
Diary,Furniture,1000000.0,0.0,True,False,,
Diary,Furniture,1000000.0,0.0,False,True,Genuine,Report #9043
Diary,Furniture,1000000.0,0.0,False,True,Genuine,
Diary,Furniture,1000000.0,0.0,False,True,Fake,Report #9044
Diary,Furniture,1000000.0,0.0,False,True,Fake,
Diary,Furniture,1000000.0,0.0,False,True,,Report #9045
Diary,Furniture,1000000.0,0.0,False,True,,
Diary,Furniture,1000000.0,0.0,False,False,Genuine,Report #9046
Diary,Furniture,1000000.0,0.0,False,False,Genuine,
Diary,Furniture,1000000.0,0.0,False,False,Fake,Report #9047
Diary,Furniture,1000000.0,0.0,False,False,Fake,
Diary,Furniture,1000000.0,0.0,False,False,,Report #9048
Diary,Furniture,1000000.0,0.0,False,False,,
Diary,Furniture,1000000.0,19.0,True,True,Genuine,Report #9049
Diary,Furniture,1000000.0,19.0,True,True,Genuine,
Diary,Furniture,1000000.0,19.0,True,True,Fake,Report #9050
Diary,Furniture,1000000.0,19.0,True,True,Fake,
Diary,Furniture,1000000.0,19.0,True,True,,Report #9051
Diary,Furniture,1000000.0,19.0,True,True,,
Diary,Furniture,1000000.0,19.0,True,False,Genuine,Report #9052
Diary,Furniture,1000000.0,19.0,True,False,Genuine,
Diary,Furniture,1000000.0,19.0,True,False,Fake,Report #9053
Diary,Furniture,1000000.0,19.0,True,False,Fake,
Diary,Furniture,1000000.0,19.0,True,False,,Report #9054
Diary,Furniture,1000000.0,19.0,True,False,,
Diary,Furniture,1000000.0,19.0,False,True,Genuine,Report #9055
Diary,Furniture,1000000.0,19.0,False,True,Genuine,
Diary,Furniture,1000000.0,19.0,False,True,Fake,Report #9056
Diary,Furniture,1000000.0,19.0,False,True,Fake,
Diary,Furniture,1000000.0,19.0,False,True,,Report #9057
Diary,Furniture,1000000.0,19.0,False,True,,
Diary,Furniture,1000000.0,19.0,False,False,Genuine,Report #9058
Diary,Furniture,1000000.0,19.0,False,False,Genuine,
Diary,Furniture,1000000.0,19.0,False,False,Fake,Report #9059
Diary,Furniture,1000000.0,19.0,False,False,Fake,
Diary,Furniture,1000000.0,19.0,False,False,,Report #9060
Diary,Furniture,1000000.0,19.0,False,False,,
Diary,Furniture,1000000.0,20.0,True,True,Genuine,Report #9061
Diary,Furniture,1000000.0,20.0,True,True,Genuine,
Diary,Furniture,1000000.0,20.0,True,True,Fake,Report #9062
Diary,Furniture,1000000.0,20.0,True,True,Fake,
Diary,Furniture,1000000.0,20.0,True,True,,Report #9063
Diary,Furniture,1000000.0,20.0,True,True,,
Diary,Furniture,1000000.0,20.0,True,False,Genuine,Report #9064
Diary,Furniture,1000000.0,20.0,True,False,Genuine,
Diary,Furniture,1000000.0,20.0,True,False,Fake,Report #9065
Diary,Furniture,1000000.0,20.0,True,False,Fake,
Diary,Furniture,1000000.0,20.0,True,False,,Report #9066
Diary,Furniture,1000000.0,20.0,True,False,,
Diary,Furniture,1000000.0,20.0,False,True,Genuine,Report #9067
Diary,Furniture,1000000.0,20.0,False,True,Genuine,
Diary,Furniture,1000000.0,20.0,False,True,Fake,Report #9068
Diary,Furniture,1000000.0,20.0,False,True,Fake,
Diary,Furniture,1000000.0,20.0,False,True,,Report #9069
Diary,Furniture,1000000.0,20.0,False,True,,
Diary,Furniture,1000000.0,20.0,False,False,Genuine,Report #9070
Diary,Furniture,1000000.0,20.0,False,False,Genuine,
Diary,Furniture,1000000.0,20.0,False,False,Fake,Report #9071
Diary,Furniture,1000000.0,20.0,False,False,Fake,
Diary,Furniture,1000000.0,20.0,False,False,,Report #9072
Diary,Furniture,1000000.0,20.0,False,False,,
Diary,Furniture,1000000.0,21.0,True,True,Genuine,Report #9073
Diary,Furniture,1000000.0,21.0,True,True,Genuine,
Diary,Furniture,1000000.0,21.0,True,True,Fake,Report #9074
Diary,Furniture,1000000.0,21.0,True,True,Fake,
Diary,Furniture,1000000.0,21.0,True,True,,Report #9075
Diary,Furniture,1000000.0,21.0,True,True,,
Diary,Furniture,1000000.0,21.0,True,False,Genuine,Report #9076
Diary,Furniture,1000000.0,21.0,True,False,Genuine,
Diary,Furniture,1000000.0,21.0,True,False,Fake,Report #9077
Diary,Furniture,1000000.0,21.0,True,False,Fake,
Diary,Furniture,1000000.0,21.0,True,False,,Report #9078
Diary,Furniture,1000000.0,21.0,True,False,,
Diary,Furniture,1000000.0,21.0,False,True,Genuine,Report #9079
Diary,Furniture,1000000.0,21.0,False,True,Genuine,
Diary,Furniture,1000000.0,21.0,False,True,Fake,Report #9080
Diary,Furniture,1000000.0,21.0,False,True,Fake,
Diary,Furniture,1000000.0,21.0,False,True,,Report #9081
Diary,Furniture,1000000.0,21.0,False,True,,
Diary,Furniture,1000000.0,21.0,False,False,Genuine,Report #9082
Diary,Furniture,1000000.0,21.0,False,False,Genuine,
Diary,Furniture,1000000.0,21.0,False,False,Fake,Report #9083
Diary,Furniture,1000000.0,21.0,False,False,Fake,
Diary,Furniture,1000000.0,21.0,False,False,,Report #9084
Diary,Furniture,1000000.0,21.0,False,False,,
Diary,Furniture,1000000.0,99.0,True,True,Genuine,Report #9085
Diary,Furniture,1000000.0,99.0,True,True,Genuine,
Diary,Furniture,1000000.0,99.0,True,True,Fake,Report #9086
Diary,Furniture,1000000.0,99.0,True,True,Fake,
Diary,Furniture,1000000.0,99.0,True,True,,Report #9087
Diary,Furniture,1000000.0,99.0,True,True,,
Diary,Furniture,1000000.0,99.0,True,False,Genuine,Report #9088
Diary,Furniture,1000000.0,99.0,True,False,Genuine,
Diary,Furniture,1000000.0,99.0,True,False,Fake,Report #9089
Diary,Furniture,1000000.0,99.0,True,False,Fake,
Diary,Furniture,1000000.0,99.0,True,False,,Report #9090
Diary,Furniture,1000000.0,99.0,True,False,,
Diary,Furniture,1000000.0,99.0,False,True,Genuine,Report #9091
Diary,Furniture,1000000.0,99.0,False,True,Genuine,
Diary,Furniture,1000000.0,99.0,False,True,Fake,Report #9092
Diary,Furniture,1000000.0,99.0,False,True,Fake,
Diary,Furniture,1000000.0,99.0,False,True,,Report #9093
Diary,Furniture,1000000.0,99.0,False,True,,
Diary,Furniture,1000000.0,99.0,False,False,Genuine,Report #9094
Diary,Furniture,1000000.0,99.0,False,False,Genuine,
Diary,Furniture,1000000.0,99.0,False,False,Fake,Report #9095
Diary,Furniture,1000000.0,99.0,False,False,Fake,
Diary,Furniture,1000000.0,99.0,False,False,,Report #9096
Diary,Furniture,1000000.0,99.0,False,False,,
Diary,Furniture,1000000.0,100.0,True,True,Genuine,Report #9097
Diary,Furniture,1000000.0,100.0,True,True,Genuine,
Diary,Furniture,1000000.0,100.0,True,True,Fake,Report #9098
Diary,Furniture,1000000.0,100.0,True,True,Fake,
Diary,Furniture,1000000.0,100.0,True,True,,Report #9099
Diary,Furniture,1000000.0,100.0,True,True,,
Diary,Furniture,1000000.0,100.0,True,False,Genuine,Report #9100
Diary,Furniture,1000000.0,100.0,True,False,Genuine,
Diary,Furniture,1000000.0,100.0,True,False,Fake,Report #9101
Diary,Furniture,1000000.0,100.0,True,False,Fake,
Diary,Furniture,1000000.0,100.0,True,False,,Report #9102
Diary,Furniture,1000000.0,100.0,True,False,,
Diary,Furniture,1000000.0,100.0,False,True,Genuine,Report #9103
Diary,Furniture,1000000.0,100.0,False,True,Genuine,
Diary,Furniture,1000000.0,100.0,False,True,Fake,Report #9104
Diary,Furniture,1000000.0,100.0,False,True,Fake,
Diary,Furniture,1000000.0,100.0,False,True,,Report #9105
Diary,Furniture,1000000.0,100.0,False,True,,
Diary,Furniture,1000000.0,100.0,False,False,Genuine,Report #9106
Diary,Furniture,1000000.0,100.0,False,False,Genuine,
Diary,Furniture,1000000.0,100.0,False,False,Fake,Report #9107
Diary,Furniture,1000000.0,100.0,False,False,Fake,
Diary,Furniture,1000000.0,100.0,False,False,,Report #9108
Diary,Furniture,1000000.0,100.0,False,False,,
Diary,Furniture,1000000.0,101.0,True,True,Genuine,Report #9109
Diary,Furniture,1000000.0,101.0,True,True,Genuine,
Diary,Furniture,1000000.0,101.0,True,True,Fake,Report #9110
Diary,Furniture,1000000.0,101.0,True,True,Fake,
Diary,Furniture,1000000.0,101.0,True,True,,Report #9111
Diary,Furniture,1000000.0,101.0,True,True,,
Diary,Furniture,1000000.0,101.0,True,False,Genuine,Report #9112
Diary,Furniture,1000000.0,101.0,True,False,Genuine,
Diary,Furniture,1000000.0,101.0,True,False,Fake,Report #9113
Diary,Furniture,1000000.0,101.0,True,False,Fake,
Diary,Furniture,1000000.0,101.0,True,False,,Report #9114
Diary,Furniture,1000000.0,101.0,True,False,,
Diary,Furniture,1000000.0,101.0,False,True,Genuine,Report #9115
Diary,Furniture,1000000.0,101.0,False,True,Genuine,
Diary,Furniture,1000000.0,101.0,False,True,Fake,Report #9116
Diary,Furniture,1000000.0,101.0,False,True,Fake,
Diary,Furniture,1000000.0,101.0,False,True,,Report #9117
Diary,Furniture,1000000.0,101.0,False,True,,
Diary,Furniture,1000000.0,101.0,False,False,Genuine,Report #9118
Diary,Furniture,1000000.0,101.0,False,False,Genuine,
Diary,Furniture,1000000.0,101.0,False,False,Fake,Report #9119
Diary,Furniture,1000000.0,101.0,False,False,Fake,
Diary,Furniture,1000000.0,101.0,False,False,,Report #9120
Diary,Furniture,1000000.0,101.0,False,False,,
Diary,HistoryWritings,0.0,-1.0,True,True,Genuine,Report #9121
Diary,HistoryWritings,0.0,-1.0,True,True,Genuine,
Diary,HistoryWritings,0.0,-1.0,True,True,Fake,Report #9122
Diary,HistoryWritings,0.0,-1.0,True,True,Fake,
Diary,HistoryWritings,0.0,-1.0,True,True,,Report #9123
Diary,HistoryWritings,0.0,-1.0,True,True,,
Diary,HistoryWritings,0.0,-1.0,True,False,Genuine,Report #9124
Diary,HistoryWritings,0.0,-1.0,True,False,Genuine,
Diary,HistoryWritings,0.0,-1.0,True,False,Fake,Report #9125
Diary,HistoryWritings,0.0,-1.0,True,False,Fake,
Diary,HistoryWritings,0.0,-1.0,True,False,,Report #9126
Diary,HistoryWritings,0.0,-1.0,True,False,,
Diary,HistoryWritings,0.0,-1.0,False,True,Genuine,Report #9127
Diary,HistoryWritings,0.0,-1.0,False,True,Genuine,
Diary,HistoryWritings,0.0,-1.0,False,True,Fake,Report #9128
Diary,HistoryWritings,0.0,-1.0,False,True,Fake,
Diary,HistoryWritings,0.0,-1.0,False,True,,Report #9129
Diary,HistoryWritings,0.0,-1.0,False,True,,
Diary,HistoryWritings,0.0,-1.0,False,False,Genuine,Report #9130
Diary,HistoryWritings,0.0,-1.0,False,False,Genuine,
Diary,HistoryWritings,0.0,-1.0,False,False,Fake,Report #9131
Diary,HistoryWritings,0.0,-1.0,False,False,Fake,
Diary,HistoryWritings,0.0,-1.0,False,False,,Report #9132
Diary,HistoryWritings,0.0,-1.0,False,False,,
Diary,HistoryWritings,0.0,0.0,True,True,Genuine,Report #9133
Diary,HistoryWritings,0.0,0.0,True,True,Genuine,
Diary,HistoryWritings,0.0,0.0,True,True,Fake,Report #9134
Diary,HistoryWritings,0.0,0.0,True,True,Fake,
Diary,HistoryWritings,0.0,0.0,True,True,,Report #9135
Diary,HistoryWritings,0.0,0.0,True,True,,
Diary,HistoryWritings,0.0,0.0,True,False,Genuine,Report #9136
Diary,HistoryWritings,0.0,0.0,True,False,Genuine,
Diary,HistoryWritings,0.0,0.0,True,False,Fake,Report #9137
Diary,HistoryWritings,0.0,0.0,True,False,Fake,
Diary,HistoryWritings,0.0,0.0,True,False,,Report #9138
Diary,HistoryWritings,0.0,0.0,True,False,,
Diary,HistoryWritings,0.0,0.0,False,True,Genuine,Report #9139
Diary,HistoryWritings,0.0,0.0,False,True,Genuine,
Diary,HistoryWritings,0.0,0.0,False,True,Fake,Report #9140
Diary,HistoryWritings,0.0,0.0,False,True,Fake,
Diary,HistoryWritings,0.0,0.0,False,True,,Report #9141
Diary,HistoryWritings,0.0,0.0,False,True,,
Diary,HistoryWritings,0.0,0.0,False,False,Genuine,Report #9142
Diary,HistoryWritings,0.0,0.0,False,False,Genuine,
Diary,HistoryWritings,0.0,0.0,False,False,Fake,Report #9143
Diary,HistoryWritings,0.0,0.0,False,False,Fake,
Diary,HistoryWritings,0.0,0.0,False,False,,Report #9144
Diary,HistoryWritings,0.0,0.0,False,False,,
Diary,HistoryWritings,0.0,19.0,True,True,Genuine,Report #9145
Diary,HistoryWritings,0.0,19.0,True,True,Genuine,
Diary,HistoryWritings,0.0,19.0,True,True,Fake,Report #9146
Diary,HistoryWritings,0.0,19.0,True,True,Fake,
Diary,HistoryWritings,0.0,19.0,True,True,,Report #9147
Diary,HistoryWritings,0.0,19.0,True,True,,
Diary,HistoryWritings,0.0,19.0,True,False,Genuine,Report #9148
Diary,HistoryWritings,0.0,19.0,True,False,Genuine,
Diary,HistoryWritings,0.0,19.0,True,False,Fake,Report #9149
Diary,HistoryWritings,0.0,19.0,True,False,Fake,
Diary,HistoryWritings,0.0,19.0,True,False,,Report #9150
Diary,HistoryWritings,0.0,19.0,True,False,,
Diary,HistoryWritings,0.0,19.0,False,True,Genuine,Report #9151
Diary,HistoryWritings,0.0,19.0,False,True,Genuine,
Diary,HistoryWritings,0.0,19.0,False,True,Fake,Report #9152
Diary,HistoryWritings,0.0,19.0,False,True,Fake,
Diary,HistoryWritings,0.0,19.0,False,True,,Report #9153
Diary,HistoryWritings,0.0,19.0,False,True,,
Diary,HistoryWritings,0.0,19.0,False,False,Genuine,Report #9154
Diary,HistoryWritings,0.0,19.0,False,False,Genuine,
Diary,HistoryWritings,0.0,19.0,False,False,Fake,Report #9155
Diary,HistoryWritings,0.0,19.0,False,False,Fake,
Diary,HistoryWritings,0.0,19.0,False,False,,Report #9156
Diary,HistoryWritings,0.0,19.0,False,False,,
Diary,HistoryWritings,0.0,20.0,True,True,Genuine,Report #9157
Diary,HistoryWritings,0.0,20.0,True,True,Genuine,
Diary,HistoryWritings,0.0,20.0,True,True,Fake,Report #9158
Diary,HistoryWritings,0.0,20.0,True,True,Fake,
Diary,HistoryWritings,0.0,20.0,True,True,,Report #9159
Diary,HistoryWritings,0.0,20.0,True,True,,
Diary,HistoryWritings,0.0,20.0,True,False,Genuine,Report #9160
Diary,HistoryWritings,0.0,20.0,True,False,Genuine,
Diary,HistoryWritings,0.0,20.0,True,False,Fake,Report #9161
Diary,HistoryWritings,0.0,20.0,True,False,Fake,
Diary,HistoryWritings,0.0,20.0,True,False,,Report #9162
Diary,HistoryWritings,0.0,20.0,True,False,,
Diary,HistoryWritings,0.0,20.0,False,True,Genuine,Report #9163
Diary,HistoryWritings,0.0,20.0,False,True,Genuine,
Diary,HistoryWritings,0.0,20.0,False,True,Fake,Report #9164
Diary,HistoryWritings,0.0,20.0,False,True,Fake,
Diary,HistoryWritings,0.0,20.0,False,True,,Report #9165
Diary,HistoryWritings,0.0,20.0,False,True,,
Diary,HistoryWritings,0.0,20.0,False,False,Genuine,Report #9166
Diary,HistoryWritings,0.0,20.0,False,False,Genuine,
Diary,HistoryWritings,0.0,20.0,False,False,Fake,Report #9167
Diary,HistoryWritings,0.0,20.0,False,False,Fake,
Diary,HistoryWritings,0.0,20.0,False,False,,Report #9168
Diary,HistoryWritings,0.0,20.0,False,False,,
Diary,HistoryWritings,0.0,21.0,True,True,Genuine,Report #9169
Diary,HistoryWritings,0.0,21.0,True,True,Genuine,
Diary,HistoryWritings,0.0,21.0,True,True,Fake,Report #9170
Diary,HistoryWritings,0.0,21.0,True,True,Fake,
Diary,HistoryWritings,0.0,21.0,True,True,,Report #9171
Diary,HistoryWritings,0.0,21.0,True,True,,
Diary,HistoryWritings,0.0,21.0,True,False,Genuine,Report #9172
Diary,HistoryWritings,0.0,21.0,True,False,Genuine,
Diary,HistoryWritings,0.0,21.0,True,False,Fake,Report #9173
Diary,HistoryWritings,0.0,21.0,True,False,Fake,
Diary,HistoryWritings,0.0,21.0,True,False,,Report #9174
Diary,HistoryWritings,0.0,21.0,True,False,,
Diary,HistoryWritings,0.0,21.0,False,True,Genuine,Report #9175
Diary,HistoryWritings,0.0,21.0,False,True,Genuine,
Diary,HistoryWritings,0.0,21.0,False,True,Fake,Report #9176
Diary,HistoryWritings,0.0,21.0,False,True,Fake,
Diary,HistoryWritings,0.0,21.0,False,True,,Report #9177
Diary,HistoryWritings,0.0,21.0,False,True,,
Diary,HistoryWritings,0.0,21.0,False,False,Genuine,Report #9178
Diary,HistoryWritings,0.0,21.0,False,False,Genuine,
Diary,HistoryWritings,0.0,21.0,False,False,Fake,Report #9179
Diary,HistoryWritings,0.0,21.0,False,False,Fake,
Diary,HistoryWritings,0.0,21.0,False,False,,Report #9180
Diary,HistoryWritings,0.0,21.0,False,False,,
Diary,HistoryWritings,0.0,99.0,True,True,Genuine,Report #9181
Diary,HistoryWritings,0.0,99.0,True,True,Genuine,
Diary,HistoryWritings,0.0,99.0,True,True,Fake,Report #9182
Diary,HistoryWritings,0.0,99.0,True,True,Fake,
Diary,HistoryWritings,0.0,99.0,True,True,,Report #9183
Diary,HistoryWritings,0.0,99.0,True,True,,
Diary,HistoryWritings,0.0,99.0,True,False,Genuine,Report #9184
Diary,HistoryWritings,0.0,99.0,True,False,Genuine,
Diary,HistoryWritings,0.0,99.0,True,False,Fake,Report #9185
Diary,HistoryWritings,0.0,99.0,True,False,Fake,
Diary,HistoryWritings,0.0,99.0,True,False,,Report #9186
Diary,HistoryWritings,0.0,99.0,True,False,,
Diary,HistoryWritings,0.0,99.0,False,True,Genuine,Report #9187
Diary,HistoryWritings,0.0,99.0,False,True,Genuine,
Diary,HistoryWritings,0.0,99.0,False,True,Fake,Report #9188
Diary,HistoryWritings,0.0,99.0,False,True,Fake,
Diary,HistoryWritings,0.0,99.0,False,True,,Report #9189
Diary,HistoryWritings,0.0,99.0,False,True,,
Diary,HistoryWritings,0.0,99.0,False,False,Genuine,Report #9190
Diary,HistoryWritings,0.0,99.0,False,False,Genuine,
Diary,HistoryWritings,0.0,99.0,False,False,Fake,Report #9191
Diary,HistoryWritings,0.0,99.0,False,False,Fake,
Diary,HistoryWritings,0.0,99.0,False,False,,Report #9192
Diary,HistoryWritings,0.0,99.0,False,False,,
Diary,HistoryWritings,0.0,100.0,True,True,Genuine,Report #9193
Diary,HistoryWritings,0.0,100.0,True,True,Genuine,
Diary,HistoryWritings,0.0,100.0,True,True,Fake,Report #9194
Diary,HistoryWritings,0.0,100.0,True,True,Fake,
Diary,HistoryWritings,0.0,100.0,True,True,,Report #9195
Diary,HistoryWritings,0.0,100.0,True,True,,
Diary,HistoryWritings,0.0,100.0,True,False,Genuine,Report #9196
Diary,HistoryWritings,0.0,100.0,True,False,Genuine,
Diary,HistoryWritings,0.0,100.0,True,False,Fake,Report #9197
Diary,HistoryWritings,0.0,100.0,True,False,Fake,
Diary,HistoryWritings,0.0,100.0,True,False,,Report #9198
Diary,HistoryWritings,0.0,100.0,True,False,,
Diary,HistoryWritings,0.0,100.0,False,True,Genuine,Report #9199
Diary,HistoryWritings,0.0,100.0,False,True,Genuine,
Diary,HistoryWritings,0.0,100.0,False,True,Fake,Report #9200
Diary,HistoryWritings,0.0,100.0,False,True,Fake,
Diary,HistoryWritings,0.0,100.0,False,True,,Report #9201
Diary,HistoryWritings,0.0,100.0,False,True,,
Diary,HistoryWritings,0.0,100.0,False,False,Genuine,Report #9202
Diary,HistoryWritings,0.0,100.0,False,False,Genuine,
Diary,HistoryWritings,0.0,100.0,False,False,Fake,Report #9203
Diary,HistoryWritings,0.0,100.0,False,False,Fake,
Diary,HistoryWritings,0.0,100.0,False,False,,Report #9204
Diary,HistoryWritings,0.0,100.0,False,False,,
Diary,HistoryWritings,0.0,101.0,True,True,Genuine,Report #9205
Diary,HistoryWritings,0.0,101.0,True,True,Genuine,
Diary,HistoryWritings,0.0,101.0,True,True,Fake,Report #9206
Diary,HistoryWritings,0.0,101.0,True,True,Fake,
Diary,HistoryWritings,0.0,101.0,True,True,,Report #9207
Diary,HistoryWritings,0.0,101.0,True,True,,
Diary,HistoryWritings,0.0,101.0,True,False,Genuine,Report #9208
Diary,HistoryWritings,0.0,101.0,True,False,Genuine,
Diary,HistoryWritings,0.0,101.0,True,False,Fake,Report #9209
Diary,HistoryWritings,0.0,101.0,True,False,Fake,
Diary,HistoryWritings,0.0,101.0,True,False,,Report #9210
Diary,HistoryWritings,0.0,101.0,True,False,,
Diary,HistoryWritings,0.0,101.0,False,True,Genuine,Report #9211
Diary,HistoryWritings,0.0,101.0,False,True,Genuine,
Diary,HistoryWritings,0.0,101.0,False,True,Fake,Report #9212
Diary,HistoryWritings,0.0,101.0,False,True,Fake,
Diary,HistoryWritings,0.0,101.0,False,True,,Report #9213
Diary,HistoryWritings,0.0,101.0,False,True,,
Diary,HistoryWritings,0.0,101.0,False,False,Genuine,Report #9214
Diary,HistoryWritings,0.0,101.0,False,False,Genuine,
Diary,HistoryWritings,0.0,101.0,False,False,Fake,Report #9215
Diary,HistoryWritings,0.0,101.0,False,False,Fake,
Diary,HistoryWritings,0.0,101.0,False,False,,Report #9216
Diary,HistoryWritings,0.0,101.0,False,False,,
Diary,HistoryWritings,1.0,-1.0,True,True,Genuine,Report #9217
Diary,HistoryWritings,1.0,-1.0,True,True,Genuine,
Diary,HistoryWritings,1.0,-1.0,True,True,Fake,Report #9218
Diary,HistoryWritings,1.0,-1.0,True,True,Fake,
Diary,HistoryWritings,1.0,-1.0,True,True,,Report #9219
Diary,HistoryWritings,1.0,-1.0,True,True,,
Diary,HistoryWritings,1.0,-1.0,True,False,Genuine,Report #9220
Diary,HistoryWritings,1.0,-1.0,True,False,Genuine,
Diary,HistoryWritings,1.0,-1.0,True,False,Fake,Report #9221
Diary,HistoryWritings,1.0,-1.0,True,False,Fake,
Diary,HistoryWritings,1.0,-1.0,True,False,,Report #9222
Diary,HistoryWritings,1.0,-1.0,True,False,,
Diary,HistoryWritings,1.0,-1.0,False,True,Genuine,Report #9223
Diary,HistoryWritings,1.0,-1.0,False,True,Genuine,
Diary,HistoryWritings,1.0,-1.0,False,True,Fake,Report #9224
Diary,HistoryWritings,1.0,-1.0,False,True,Fake,
Diary,HistoryWritings,1.0,-1.0,False,True,,Report #9225
Diary,HistoryWritings,1.0,-1.0,False,True,,
Diary,HistoryWritings,1.0,-1.0,False,False,Genuine,Report #9226
Diary,HistoryWritings,1.0,-1.0,False,False,Genuine,
Diary,HistoryWritings,1.0,-1.0,False,False,Fake,Report #9227
Diary,HistoryWritings,1.0,-1.0,False,False,Fake,
Diary,HistoryWritings,1.0,-1.0,False,False,,Report #9228
Diary,HistoryWritings,1.0,-1.0,False,False,,
Diary,HistoryWritings,1.0,0.0,True,True,Genuine,Report #9229
Diary,HistoryWritings,1.0,0.0,True,True,Genuine,
Diary,HistoryWritings,1.0,0.0,True,True,Fake,Report #9230
Diary,HistoryWritings,1.0,0.0,True,True,Fake,
Diary,HistoryWritings,1.0,0.0,True,True,,Report #9231
Diary,HistoryWritings,1.0,0.0,True,True,,
Diary,HistoryWritings,1.0,0.0,True,False,Genuine,Report #9232
Diary,HistoryWritings,1.0,0.0,True,False,Genuine,
Diary,HistoryWritings,1.0,0.0,True,False,Fake,Report #9233
Diary,HistoryWritings,1.0,0.0,True,False,Fake,
Diary,HistoryWritings,1.0,0.0,True,False,,Report #9234
Diary,HistoryWritings,1.0,0.0,True,False,,
Diary,HistoryWritings,1.0,0.0,False,True,Genuine,Report #9235
Diary,HistoryWritings,1.0,0.0,False,True,Genuine,
Diary,HistoryWritings,1.0,0.0,False,True,Fake,Report #9236
Diary,HistoryWritings,1.0,0.0,False,True,Fake,
Diary,HistoryWritings,1.0,0.0,False,True,,Report #9237
Diary,HistoryWritings,1.0,0.0,False,True,,
Diary,HistoryWritings,1.0,0.0,False,False,Genuine,Report #9238
Diary,HistoryWritings,1.0,0.0,False,False,Genuine,
Diary,HistoryWritings,1.0,0.0,False,False,Fake,Report #9239
Diary,HistoryWritings,1.0,0.0,False,False,Fake,
Diary,HistoryWritings,1.0,0.0,False,False,,Report #9240
Diary,HistoryWritings,1.0,0.0,False,False,,
Diary,HistoryWritings,1.0,19.0,True,True,Genuine,Report #9241
Diary,HistoryWritings,1.0,19.0,True,True,Genuine,
Diary,HistoryWritings,1.0,19.0,True,True,Fake,Report #9242
Diary,HistoryWritings,1.0,19.0,True,True,Fake,
Diary,HistoryWritings,1.0,19.0,True,True,,Report #9243
Diary,HistoryWritings,1.0,19.0,True,True,,
Diary,HistoryWritings,1.0,19.0,True,False,Genuine,Report #9244
Diary,HistoryWritings,1.0,19.0,True,False,Genuine,
Diary,HistoryWritings,1.0,19.0,True,False,Fake,Report #9245
Diary,HistoryWritings,1.0,19.0,True,False,Fake,
Diary,HistoryWritings,1.0,19.0,True,False,,Report #9246
Diary,HistoryWritings,1.0,19.0,True,False,,
Diary,HistoryWritings,1.0,19.0,False,True,Genuine,Report #9247
Diary,HistoryWritings,1.0,19.0,False,True,Genuine,
Diary,HistoryWritings,1.0,19.0,False,True,Fake,Report #9248
Diary,HistoryWritings,1.0,19.0,False,True,Fake,
Diary,HistoryWritings,1.0,19.0,False,True,,Report #9249
Diary,HistoryWritings,1.0,19.0,False,True,,
Diary,HistoryWritings,1.0,19.0,False,False,Genuine,Report #9250
Diary,HistoryWritings,1.0,19.0,False,False,Genuine,
Diary,HistoryWritings,1.0,19.0,False,False,Fake,Report #9251
Diary,HistoryWritings,1.0,19.0,False,False,Fake,
Diary,HistoryWritings,1.0,19.0,False,False,,Report #9252
Diary,HistoryWritings,1.0,19.0,False,False,,
Diary,HistoryWritings,1.0,20.0,True,True,Genuine,Report #9253
Diary,HistoryWritings,1.0,20.0,True,True,Genuine,
Diary,HistoryWritings,1.0,20.0,True,True,Fake,Report #9254
Diary,HistoryWritings,1.0,20.0,True,True,Fake,
Diary,HistoryWritings,1.0,20.0,True,True,,Report #9255
Diary,HistoryWritings,1.0,20.0,True,True,,
Diary,HistoryWritings,1.0,20.0,True,False,Genuine,Report #9256
Diary,HistoryWritings,1.0,20.0,True,False,Genuine,
Diary,HistoryWritings,1.0,20.0,True,False,Fake,Report #9257
Diary,HistoryWritings,1.0,20.0,True,False,Fake,
Diary,HistoryWritings,1.0,20.0,True,False,,Report #9258
Diary,HistoryWritings,1.0,20.0,True,False,,
Diary,HistoryWritings,1.0,20.0,False,True,Genuine,Report #9259
Diary,HistoryWritings,1.0,20.0,False,True,Genuine,
Diary,HistoryWritings,1.0,20.0,False,True,Fake,Report #9260
Diary,HistoryWritings,1.0,20.0,False,True,Fake,
Diary,HistoryWritings,1.0,20.0,False,True,,Report #9261
Diary,HistoryWritings,1.0,20.0,False,True,,
Diary,HistoryWritings,1.0,20.0,False,False,Genuine,Report #9262
Diary,HistoryWritings,1.0,20.0,False,False,Genuine,
Diary,HistoryWritings,1.0,20.0,False,False,Fake,Report #9263
Diary,HistoryWritings,1.0,20.0,False,False,Fake,
Diary,HistoryWritings,1.0,20.0,False,False,,Report #9264
Diary,HistoryWritings,1.0,20.0,False,False,,
Diary,HistoryWritings,1.0,21.0,True,True,Genuine,Report #9265
Diary,HistoryWritings,1.0,21.0,True,True,Genuine,
Diary,HistoryWritings,1.0,21.0,True,True,Fake,Report #9266
Diary,HistoryWritings,1.0,21.0,True,True,Fake,
Diary,HistoryWritings,1.0,21.0,True,True,,Report #9267
Diary,HistoryWritings,1.0,21.0,True,True,,
Diary,HistoryWritings,1.0,21.0,True,False,Genuine,Report #9268
Diary,HistoryWritings,1.0,21.0,True,False,Genuine,
Diary,HistoryWritings,1.0,21.0,True,False,Fake,Report #9269
Diary,HistoryWritings,1.0,21.0,True,False,Fake,
Diary,HistoryWritings,1.0,21.0,True,False,,Report #9270
Diary,HistoryWritings,1.0,21.0,True,False,,
Diary,HistoryWritings,1.0,21.0,False,True,Genuine,Report #9271
Diary,HistoryWritings,1.0,21.0,False,True,Genuine,
Diary,HistoryWritings,1.0,21.0,False,True,Fake,Report #9272
Diary,HistoryWritings,1.0,21.0,False,True,Fake,
Diary,HistoryWritings,1.0,21.0,False,True,,Report #9273
Diary,HistoryWritings,1.0,21.0,False,True,,
Diary,HistoryWritings,1.0,21.0,False,False,Genuine,Report #9274
Diary,HistoryWritings,1.0,21.0,False,False,Genuine,
Diary,HistoryWritings,1.0,21.0,False,False,Fake,Report #9275
Diary,HistoryWritings,1.0,21.0,False,False,Fake,
Diary,HistoryWritings,1.0,21.0,False,False,,Report #9276
Diary,HistoryWritings,1.0,21.0,False,False,,
Diary,HistoryWritings,1.0,99.0,True,True,Genuine,Report #9277
Diary,HistoryWritings,1.0,99.0,True,True,Genuine,
Diary,HistoryWritings,1.0,99.0,True,True,Fake,Report #9278
Diary,HistoryWritings,1.0,99.0,True,True,Fake,
Diary,HistoryWritings,1.0,99.0,True,True,,Report #9279
Diary,HistoryWritings,1.0,99.0,True,True,,
Diary,HistoryWritings,1.0,99.0,True,False,Genuine,Report #9280
Diary,HistoryWritings,1.0,99.0,True,False,Genuine,
Diary,HistoryWritings,1.0,99.0,True,False,Fake,Report #9281
Diary,HistoryWritings,1.0,99.0,True,False,Fake,
Diary,HistoryWritings,1.0,99.0,True,False,,Report #9282
Diary,HistoryWritings,1.0,99.0,True,False,,
Diary,HistoryWritings,1.0,99.0,False,True,Genuine,Report #9283
Diary,HistoryWritings,1.0,99.0,False,True,Genuine,
Diary,HistoryWritings,1.0,99.0,False,True,Fake,Report #9284
Diary,HistoryWritings,1.0,99.0,False,True,Fake,
Diary,HistoryWritings,1.0,99.0,False,True,,Report #9285
Diary,HistoryWritings,1.0,99.0,False,True,,
Diary,HistoryWritings,1.0,99.0,False,False,Genuine,Report #9286
Diary,HistoryWritings,1.0,99.0,False,False,Genuine,
Diary,HistoryWritings,1.0,99.0,False,False,Fake,Report #9287
Diary,HistoryWritings,1.0,99.0,False,False,Fake,
Diary,HistoryWritings,1.0,99.0,False,False,,Report #9288
Diary,HistoryWritings,1.0,99.0,False,False,,
Diary,HistoryWritings,1.0,100.0,True,True,Genuine,Report #9289
Diary,HistoryWritings,1.0,100.0,True,True,Genuine,
Diary,HistoryWritings,1.0,100.0,True,True,Fake,Report #9290
Diary,HistoryWritings,1.0,100.0,True,True,Fake,
Diary,HistoryWritings,1.0,100.0,True,True,,Report #9291
Diary,HistoryWritings,1.0,100.0,True,True,,
Diary,HistoryWritings,1.0,100.0,True,False,Genuine,Report #9292
Diary,HistoryWritings,1.0,100.0,True,False,Genuine,
Diary,HistoryWritings,1.0,100.0,True,False,Fake,Report #9293
Diary,HistoryWritings,1.0,100.0,True,False,Fake,
Diary,HistoryWritings,1.0,100.0,True,False,,Report #9294
Diary,HistoryWritings,1.0,100.0,True,False,,
Diary,HistoryWritings,1.0,100.0,False,True,Genuine,Report #9295
Diary,HistoryWritings,1.0,100.0,False,True,Genuine,
Diary,HistoryWritings,1.0,100.0,False,True,Fake,Report #9296
Diary,HistoryWritings,1.0,100.0,False,True,Fake,
Diary,HistoryWritings,1.0,100.0,False,True,,Report #9297
Diary,HistoryWritings,1.0,100.0,False,True,,
Diary,HistoryWritings,1.0,100.0,False,False,Genuine,Report #9298
Diary,HistoryWritings,1.0,100.0,False,False,Genuine,
Diary,HistoryWritings,1.0,100.0,False,False,Fake,Report #9299
Diary,HistoryWritings,1.0,100.0,False,False,Fake,
Diary,HistoryWritings,1.0,100.0,False,False,,Report #9300
Diary,HistoryWritings,1.0,100.0,False,False,,
Diary,HistoryWritings,1.0,101.0,True,True,Genuine,Report #9301
Diary,HistoryWritings,1.0,101.0,True,True,Genuine,
Diary,HistoryWritings,1.0,101.0,True,True,Fake,Report #9302
Diary,HistoryWritings,1.0,101.0,True,True,Fake,
Diary,HistoryWritings,1.0,101.0,True,True,,Report #9303
Diary,HistoryWritings,1.0,101.0,True,True,,
Diary,HistoryWritings,1.0,101.0,True,False,Genuine,Report #9304
Diary,HistoryWritings,1.0,101.0,True,False,Genuine,
Diary,HistoryWritings,1.0,101.0,True,False,Fake,Report #9305
Diary,HistoryWritings,1.0,101.0,True,False,Fake,
Diary,HistoryWritings,1.0,101.0,True,False,,Report #9306
Diary,HistoryWritings,1.0,101.0,True,False,,
Diary,HistoryWritings,1.0,101.0,False,True,Genuine,Report #9307
Diary,HistoryWritings,1.0,101.0,False,True,Genuine,
Diary,HistoryWritings,1.0,101.0,False,True,Fake,Report #9308
Diary,HistoryWritings,1.0,101.0,False,True,Fake,
Diary,HistoryWritings,1.0,101.0,False,True,,Report #9309
Diary,HistoryWritings,1.0,101.0,False,True,,
Diary,HistoryWritings,1.0,101.0,False,False,Genuine,Report #9310
Diary,HistoryWritings,1.0,101.0,False,False,Genuine,
Diary,HistoryWritings,1.0,101.0,False,False,Fake,Report #9311
Diary,HistoryWritings,1.0,101.0,False,False,Fake,
Diary,HistoryWritings,1.0,101.0,False,False,,Report #9312
Diary,HistoryWritings,1.0,101.0,False,False,,
Diary,HistoryWritings,100.0,-1.0,True,True,Genuine,Report #9313
Diary,HistoryWritings,100.0,-1.0,True,True,Genuine,
Diary,HistoryWritings,100.0,-1.0,True,True,Fake,Report #9314
Diary,HistoryWritings,100.0,-1.0,True,True,Fake,
Diary,HistoryWritings,100.0,-1.0,True,True,,Report #9315
Diary,HistoryWritings,100.0,-1.0,True,True,,
Diary,HistoryWritings,100.0,-1.0,True,False,Genuine,Report #9316
Diary,HistoryWritings,100.0,-1.0,True,False,Genuine,
Diary,HistoryWritings,100.0,-1.0,True,False,Fake,Report #9317
Diary,HistoryWritings,100.0,-1.0,True,False,Fake,
Diary,HistoryWritings,100.0,-1.0,True,False,,Report #9318
Diary,HistoryWritings,100.0,-1.0,True,False,,
Diary,HistoryWritings,100.0,-1.0,False,True,Genuine,Report #9319
Diary,HistoryWritings,100.0,-1.0,False,True,Genuine,
Diary,HistoryWritings,100.0,-1.0,False,True,Fake,Report #9320
Diary,HistoryWritings,100.0,-1.0,False,True,Fake,
Diary,HistoryWritings,100.0,-1.0,False,True,,Report #9321
Diary,HistoryWritings,100.0,-1.0,False,True,,
Diary,HistoryWritings,100.0,-1.0,False,False,Genuine,Report #9322
Diary,HistoryWritings,100.0,-1.0,False,False,Genuine,
Diary,HistoryWritings,100.0,-1.0,False,False,Fake,Report #9323
Diary,HistoryWritings,100.0,-1.0,False,False,Fake,
Diary,HistoryWritings,100.0,-1.0,False,False,,Report #9324
Diary,HistoryWritings,100.0,-1.0,False,False,,
Diary,HistoryWritings,100.0,0.0,True,True,Genuine,Report #9325
Diary,HistoryWritings,100.0,0.0,True,True,Genuine,
Diary,HistoryWritings,100.0,0.0,True,True,Fake,Report #9326
Diary,HistoryWritings,100.0,0.0,True,True,Fake,
Diary,HistoryWritings,100.0,0.0,True,True,,Report #9327
Diary,HistoryWritings,100.0,0.0,True,True,,
Diary,HistoryWritings,100.0,0.0,True,False,Genuine,Report #9328
Diary,HistoryWritings,100.0,0.0,True,False,Genuine,
Diary,HistoryWritings,100.0,0.0,True,False,Fake,Report #9329
Diary,HistoryWritings,100.0,0.0,True,False,Fake,
Diary,HistoryWritings,100.0,0.0,True,False,,Report #9330
Diary,HistoryWritings,100.0,0.0,True,False,,
Diary,HistoryWritings,100.0,0.0,False,True,Genuine,Report #9331
Diary,HistoryWritings,100.0,0.0,False,True,Genuine,
Diary,HistoryWritings,100.0,0.0,False,True,Fake,Report #9332
Diary,HistoryWritings,100.0,0.0,False,True,Fake,
Diary,HistoryWritings,100.0,0.0,False,True,,Report #9333
Diary,HistoryWritings,100.0,0.0,False,True,,
Diary,HistoryWritings,100.0,0.0,False,False,Genuine,Report #9334
Diary,HistoryWritings,100.0,0.0,False,False,Genuine,
Diary,HistoryWritings,100.0,0.0,False,False,Fake,Report #9335
Diary,HistoryWritings,100.0,0.0,False,False,Fake,
Diary,HistoryWritings,100.0,0.0,False,False,,Report #9336
Diary,HistoryWritings,100.0,0.0,False,False,,
Diary,HistoryWritings,100.0,19.0,True,True,Genuine,Report #9337
Diary,HistoryWritings,100.0,19.0,True,True,Genuine,
Diary,HistoryWritings,100.0,19.0,True,True,Fake,Report #9338
Diary,HistoryWritings,100.0,19.0,True,True,Fake,
Diary,HistoryWritings,100.0,19.0,True,True,,Report #9339
Diary,HistoryWritings,100.0,19.0,True,True,,
Diary,HistoryWritings,100.0,19.0,True,False,Genuine,Report #9340
Diary,HistoryWritings,100.0,19.0,True,False,Genuine,
Diary,HistoryWritings,100.0,19.0,True,False,Fake,Report #9341
Diary,HistoryWritings,100.0,19.0,True,False,Fake,
Diary,HistoryWritings,100.0,19.0,True,False,,Report #9342
Diary,HistoryWritings,100.0,19.0,True,False,,
Diary,HistoryWritings,100.0,19.0,False,True,Genuine,Report #9343
Diary,HistoryWritings,100.0,19.0,False,True,Genuine,
Diary,HistoryWritings,100.0,19.0,False,True,Fake,Report #9344
Diary,HistoryWritings,100.0,19.0,False,True,Fake,
Diary,HistoryWritings,100.0,19.0,False,True,,Report #9345
Diary,HistoryWritings,100.0,19.0,False,True,,
Diary,HistoryWritings,100.0,19.0,False,False,Genuine,Report #9346
Diary,HistoryWritings,100.0,19.0,False,False,Genuine,
Diary,HistoryWritings,100.0,19.0,False,False,Fake,Report #9347
Diary,HistoryWritings,100.0,19.0,False,False,Fake,
Diary,HistoryWritings,100.0,19.0,False,False,,Report #9348
Diary,HistoryWritings,100.0,19.0,False,False,,
Diary,HistoryWritings,100.0,20.0,True,True,Genuine,Report #9349
Diary,HistoryWritings,100.0,20.0,True,True,Genuine,
Diary,HistoryWritings,100.0,20.0,True,True,Fake,Report #9350
Diary,HistoryWritings,100.0,20.0,True,True,Fake,
Diary,HistoryWritings,100.0,20.0,True,True,,Report #9351
Diary,HistoryWritings,100.0,20.0,True,True,,
Diary,HistoryWritings,100.0,20.0,True,False,Genuine,Report #9352
Diary,HistoryWritings,100.0,20.0,True,False,Genuine,
Diary,HistoryWritings,100.0,20.0,True,False,Fake,Report #9353
Diary,HistoryWritings,100.0,20.0,True,False,Fake,
Diary,HistoryWritings,100.0,20.0,True,False,,Report #9354
Diary,HistoryWritings,100.0,20.0,True,False,,
Diary,HistoryWritings,100.0,20.0,False,True,Genuine,Report #9355
Diary,HistoryWritings,100.0,20.0,False,True,Genuine,
Diary,HistoryWritings,100.0,20.0,False,True,Fake,Report #9356
Diary,HistoryWritings,100.0,20.0,False,True,Fake,
Diary,HistoryWritings,100.0,20.0,False,True,,Report #9357
Diary,HistoryWritings,100.0,20.0,False,True,,
Diary,HistoryWritings,100.0,20.0,False,False,Genuine,Report #9358
Diary,HistoryWritings,100.0,20.0,False,False,Genuine,
Diary,HistoryWritings,100.0,20.0,False,False,Fake,Report #9359
Diary,HistoryWritings,100.0,20.0,False,False,Fake,
Diary,HistoryWritings,100.0,20.0,False,False,,Report #9360
Diary,HistoryWritings,100.0,20.0,False,False,,
Diary,HistoryWritings,100.0,21.0,True,True,Genuine,Report #9361
Diary,HistoryWritings,100.0,21.0,True,True,Genuine,
Diary,HistoryWritings,100.0,21.0,True,True,Fake,Report #9362
Diary,HistoryWritings,100.0,21.0,True,True,Fake,
Diary,HistoryWritings,100.0,21.0,True,True,,Report #9363
Diary,HistoryWritings,100.0,21.0,True,True,,
Diary,HistoryWritings,100.0,21.0,True,False,Genuine,Report #9364
Diary,HistoryWritings,100.0,21.0,True,False,Genuine,
Diary,HistoryWritings,100.0,21.0,True,False,Fake,Report #9365
Diary,HistoryWritings,100.0,21.0,True,False,Fake,
Diary,HistoryWritings,100.0,21.0,True,False,,Report #9366
Diary,HistoryWritings,100.0,21.0,True,False,,
Diary,HistoryWritings,100.0,21.0,False,True,Genuine,Report #9367
Diary,HistoryWritings,100.0,21.0,False,True,Genuine,
Diary,HistoryWritings,100.0,21.0,False,True,Fake,Report #9368
Diary,HistoryWritings,100.0,21.0,False,True,Fake,
Diary,HistoryWritings,100.0,21.0,False,True,,Report #9369
Diary,HistoryWritings,100.0,21.0,False,True,,
Diary,HistoryWritings,100.0,21.0,False,False,Genuine,Report #9370
Diary,HistoryWritings,100.0,21.0,False,False,Genuine,
Diary,HistoryWritings,100.0,21.0,False,False,Fake,Report #9371
Diary,HistoryWritings,100.0,21.0,False,False,Fake,
Diary,HistoryWritings,100.0,21.0,False,False,,Report #9372
Diary,HistoryWritings,100.0,21.0,False,False,,
Diary,HistoryWritings,100.0,99.0,True,True,Genuine,Report #9373
Diary,HistoryWritings,100.0,99.0,True,True,Genuine,
Diary,HistoryWritings,100.0,99.0,True,True,Fake,Report #9374
Diary,HistoryWritings,100.0,99.0,True,True,Fake,
Diary,HistoryWritings,100.0,99.0,True,True,,Report #9375
Diary,HistoryWritings,100.0,99.0,True,True,,
Diary,HistoryWritings,100.0,99.0,True,False,Genuine,Report #9376
Diary,HistoryWritings,100.0,99.0,True,False,Genuine,
Diary,HistoryWritings,100.0,99.0,True,False,Fake,Report #9377
Diary,HistoryWritings,100.0,99.0,True,False,Fake,
Diary,HistoryWritings,100.0,99.0,True,False,,Report #9378
Diary,HistoryWritings,100.0,99.0,True,False,,
Diary,HistoryWritings,100.0,99.0,False,True,Genuine,Report #9379
Diary,HistoryWritings,100.0,99.0,False,True,Genuine,
Diary,HistoryWritings,100.0,99.0,False,True,Fake,Report #9380
Diary,HistoryWritings,100.0,99.0,False,True,Fake,
Diary,HistoryWritings,100.0,99.0,False,True,,Report #9381
Diary,HistoryWritings,100.0,99.0,False,True,,
Diary,HistoryWritings,100.0,99.0,False,False,Genuine,Report #9382
Diary,HistoryWritings,100.0,99.0,False,False,Genuine,
Diary,HistoryWritings,100.0,99.0,False,False,Fake,Report #9383
Diary,HistoryWritings,100.0,99.0,False,False,Fake,
Diary,HistoryWritings,100.0,99.0,False,False,,Report #9384
Diary,HistoryWritings,100.0,99.0,False,False,,
Diary,HistoryWritings,100.0,100.0,True,True,Genuine,Report #9385
Diary,HistoryWritings,100.0,100.0,True,True,Genuine,
Diary,HistoryWritings,100.0,100.0,True,True,Fake,Report #9386
Diary,HistoryWritings,100.0,100.0,True,True,Fake,
Diary,HistoryWritings,100.0,100.0,True,True,,Report #9387
Diary,HistoryWritings,100.0,100.0,True,True,,
Diary,HistoryWritings,100.0,100.0,True,False,Genuine,Report #9388
Diary,HistoryWritings,100.0,100.0,True,False,Genuine,
Diary,HistoryWritings,100.0,100.0,True,False,Fake,Report #9389
Diary,HistoryWritings,100.0,100.0,True,False,Fake,
Diary,HistoryWritings,100.0,100.0,True,False,,Report #9390
Diary,HistoryWritings,100.0,100.0,True,False,,
Diary,HistoryWritings,100.0,100.0,False,True,Genuine,Report #9391
Diary,HistoryWritings,100.0,100.0,False,True,Genuine,
Diary,HistoryWritings,100.0,100.0,False,True,Fake,Report #9392
Diary,HistoryWritings,100.0,100.0,False,True,Fake,
Diary,HistoryWritings,100.0,100.0,False,True,,Report #9393
Diary,HistoryWritings,100.0,100.0,False,True,,
Diary,HistoryWritings,100.0,100.0,False,False,Genuine,Report #9394
Diary,HistoryWritings,100.0,100.0,False,False,Genuine,
Diary,HistoryWritings,100.0,100.0,False,False,Fake,Report #9395
Diary,HistoryWritings,100.0,100.0,False,False,Fake,
Diary,HistoryWritings,100.0,100.0,False,False,,Report #9396
Diary,HistoryWritings,100.0,100.0,False,False,,
Diary,HistoryWritings,100.0,101.0,True,True,Genuine,Report #9397
Diary,HistoryWritings,100.0,101.0,True,True,Genuine,
Diary,HistoryWritings,100.0,101.0,True,True,Fake,Report #9398
Diary,HistoryWritings,100.0,101.0,True,True,Fake,
Diary,HistoryWritings,100.0,101.0,True,True,,Report #9399
Diary,HistoryWritings,100.0,101.0,True,True,,
Diary,HistoryWritings,100.0,101.0,True,False,Genuine,Report #9400
Diary,HistoryWritings,100.0,101.0,True,False,Genuine,
Diary,HistoryWritings,100.0,101.0,True,False,Fake,Report #9401
Diary,HistoryWritings,100.0,101.0,True,False,Fake,
Diary,HistoryWritings,100.0,101.0,True,False,,Report #9402
Diary,HistoryWritings,100.0,101.0,True,False,,
Diary,HistoryWritings,100.0,101.0,False,True,Genuine,Report #9403
Diary,HistoryWritings,100.0,101.0,False,True,Genuine,
Diary,HistoryWritings,100.0,101.0,False,True,Fake,Report #9404
Diary,HistoryWritings,100.0,101.0,False,True,Fake,
Diary,HistoryWritings,100.0,101.0,False,True,,Report #9405
Diary,HistoryWritings,100.0,101.0,False,True,,
Diary,HistoryWritings,100.0,101.0,False,False,Genuine,Report #9406
Diary,HistoryWritings,100.0,101.0,False,False,Genuine,
Diary,HistoryWritings,100.0,101.0,False,False,Fake,Report #9407
Diary,HistoryWritings,100.0,101.0,False,False,Fake,
Diary,HistoryWritings,100.0,101.0,False,False,,Report #9408
Diary,HistoryWritings,100.0,101.0,False,False,,
Diary,HistoryWritings,1000.0,-1.0,True,True,Genuine,Report #9409
Diary,HistoryWritings,1000.0,-1.0,True,True,Genuine,
Diary,HistoryWritings,1000.0,-1.0,True,True,Fake,Report #9410
Diary,HistoryWritings,1000.0,-1.0,True,True,Fake,
Diary,HistoryWritings,1000.0,-1.0,True,True,,Report #9411
Diary,HistoryWritings,1000.0,-1.0,True,True,,
Diary,HistoryWritings,1000.0,-1.0,True,False,Genuine,Report #9412
Diary,HistoryWritings,1000.0,-1.0,True,False,Genuine,
Diary,HistoryWritings,1000.0,-1.0,True,False,Fake,Report #9413
Diary,HistoryWritings,1000.0,-1.0,True,False,Fake,
Diary,HistoryWritings,1000.0,-1.0,True,False,,Report #9414
Diary,HistoryWritings,1000.0,-1.0,True,False,,
Diary,HistoryWritings,1000.0,-1.0,False,True,Genuine,Report #9415
Diary,HistoryWritings,1000.0,-1.0,False,True,Genuine,
Diary,HistoryWritings,1000.0,-1.0,False,True,Fake,Report #9416
Diary,HistoryWritings,1000.0,-1.0,False,True,Fake,
Diary,HistoryWritings,1000.0,-1.0,False,True,,Report #9417
Diary,HistoryWritings,1000.0,-1.0,False,True,,
Diary,HistoryWritings,1000.0,-1.0,False,False,Genuine,Report #9418
Diary,HistoryWritings,1000.0,-1.0,False,False,Genuine,
Diary,HistoryWritings,1000.0,-1.0,False,False,Fake,Report #9419
Diary,HistoryWritings,1000.0,-1.0,False,False,Fake,
Diary,HistoryWritings,1000.0,-1.0,False,False,,Report #9420
Diary,HistoryWritings,1000.0,-1.0,False,False,,
Diary,HistoryWritings,1000.0,0.0,True,True,Genuine,Report #9421
Diary,HistoryWritings,1000.0,0.0,True,True,Genuine,
Diary,HistoryWritings,1000.0,0.0,True,True,Fake,Report #9422
Diary,HistoryWritings,1000.0,0.0,True,True,Fake,
Diary,HistoryWritings,1000.0,0.0,True,True,,Report #9423
Diary,HistoryWritings,1000.0,0.0,True,True,,
Diary,HistoryWritings,1000.0,0.0,True,False,Genuine,Report #9424
Diary,HistoryWritings,1000.0,0.0,True,False,Genuine,
Diary,HistoryWritings,1000.0,0.0,True,False,Fake,Report #9425
Diary,HistoryWritings,1000.0,0.0,True,False,Fake,
Diary,HistoryWritings,1000.0,0.0,True,False,,Report #9426
Diary,HistoryWritings,1000.0,0.0,True,False,,
Diary,HistoryWritings,1000.0,0.0,False,True,Genuine,Report #9427
Diary,HistoryWritings,1000.0,0.0,False,True,Genuine,
Diary,HistoryWritings,1000.0,0.0,False,True,Fake,Report #9428
Diary,HistoryWritings,1000.0,0.0,False,True,Fake,
Diary,HistoryWritings,1000.0,0.0,False,True,,Report #9429
Diary,HistoryWritings,1000.0,0.0,False,True,,
Diary,HistoryWritings,1000.0,0.0,False,False,Genuine,Report #9430
Diary,HistoryWritings,1000.0,0.0,False,False,Genuine,
Diary,HistoryWritings,1000.0,0.0,False,False,Fake,Report #9431
Diary,HistoryWritings,1000.0,0.0,False,False,Fake,
Diary,HistoryWritings,1000.0,0.0,False,False,,Report #9432
Diary,HistoryWritings,1000.0,0.0,False,False,,
Diary,HistoryWritings,1000.0,19.0,True,True,Genuine,Report #9433
Diary,HistoryWritings,1000.0,19.0,True,True,Genuine,
Diary,HistoryWritings,1000.0,19.0,True,True,Fake,Report #9434
Diary,HistoryWritings,1000.0,19.0,True,True,Fake,
Diary,HistoryWritings,1000.0,19.0,True,True,,Report #9435
Diary,HistoryWritings,1000.0,19.0,True,True,,
Diary,HistoryWritings,1000.0,19.0,True,False,Genuine,Report #9436
Diary,HistoryWritings,1000.0,19.0,True,False,Genuine,
Diary,HistoryWritings,1000.0,19.0,True,False,Fake,Report #9437
Diary,HistoryWritings,1000.0,19.0,True,False,Fake,
Diary,HistoryWritings,1000.0,19.0,True,False,,Report #9438
Diary,HistoryWritings,1000.0,19.0,True,False,,
Diary,HistoryWritings,1000.0,19.0,False,True,Genuine,Report #9439
Diary,HistoryWritings,1000.0,19.0,False,True,Genuine,
Diary,HistoryWritings,1000.0,19.0,False,True,Fake,Report #9440
Diary,HistoryWritings,1000.0,19.0,False,True,Fake,
Diary,HistoryWritings,1000.0,19.0,False,True,,Report #9441
Diary,HistoryWritings,1000.0,19.0,False,True,,
Diary,HistoryWritings,1000.0,19.0,False,False,Genuine,Report #9442
Diary,HistoryWritings,1000.0,19.0,False,False,Genuine,
Diary,HistoryWritings,1000.0,19.0,False,False,Fake,Report #9443
Diary,HistoryWritings,1000.0,19.0,False,False,Fake,
Diary,HistoryWritings,1000.0,19.0,False,False,,Report #9444
Diary,HistoryWritings,1000.0,19.0,False,False,,
Diary,HistoryWritings,1000.0,20.0,True,True,Genuine,Report #9445
Diary,HistoryWritings,1000.0,20.0,True,True,Genuine,
Diary,HistoryWritings,1000.0,20.0,True,True,Fake,Report #9446
Diary,HistoryWritings,1000.0,20.0,True,True,Fake,
Diary,HistoryWritings,1000.0,20.0,True,True,,Report #9447
Diary,HistoryWritings,1000.0,20.0,True,True,,
Diary,HistoryWritings,1000.0,20.0,True,False,Genuine,Report #9448
Diary,HistoryWritings,1000.0,20.0,True,False,Genuine,
Diary,HistoryWritings,1000.0,20.0,True,False,Fake,Report #9449
Diary,HistoryWritings,1000.0,20.0,True,False,Fake,
Diary,HistoryWritings,1000.0,20.0,True,False,,Report #9450
Diary,HistoryWritings,1000.0,20.0,True,False,,
Diary,HistoryWritings,1000.0,20.0,False,True,Genuine,Report #9451
Diary,HistoryWritings,1000.0,20.0,False,True,Genuine,
Diary,HistoryWritings,1000.0,20.0,False,True,Fake,Report #9452
Diary,HistoryWritings,1000.0,20.0,False,True,Fake,
Diary,HistoryWritings,1000.0,20.0,False,True,,Report #9453
Diary,HistoryWritings,1000.0,20.0,False,True,,
Diary,HistoryWritings,1000.0,20.0,False,False,Genuine,Report #9454
Diary,HistoryWritings,1000.0,20.0,False,False,Genuine,
Diary,HistoryWritings,1000.0,20.0,False,False,Fake,Report #9455
Diary,HistoryWritings,1000.0,20.0,False,False,Fake,
Diary,HistoryWritings,1000.0,20.0,False,False,,Report #9456
Diary,HistoryWritings,1000.0,20.0,False,False,,
Diary,HistoryWritings,1000.0,21.0,True,True,Genuine,Report #9457
Diary,HistoryWritings,1000.0,21.0,True,True,Genuine,
Diary,HistoryWritings,1000.0,21.0,True,True,Fake,Report #9458
Diary,HistoryWritings,1000.0,21.0,True,True,Fake,
Diary,HistoryWritings,1000.0,21.0,True,True,,Report #9459
Diary,HistoryWritings,1000.0,21.0,True,True,,
Diary,HistoryWritings,1000.0,21.0,True,False,Genuine,Report #9460
Diary,HistoryWritings,1000.0,21.0,True,False,Genuine,
Diary,HistoryWritings,1000.0,21.0,True,False,Fake,Report #9461
Diary,HistoryWritings,1000.0,21.0,True,False,Fake,
Diary,HistoryWritings,1000.0,21.0,True,False,,Report #9462
Diary,HistoryWritings,1000.0,21.0,True,False,,
Diary,HistoryWritings,1000.0,21.0,False,True,Genuine,Report #9463
Diary,HistoryWritings,1000.0,21.0,False,True,Genuine,
Diary,HistoryWritings,1000.0,21.0,False,True,Fake,Report #9464
Diary,HistoryWritings,1000.0,21.0,False,True,Fake,
Diary,HistoryWritings,1000.0,21.0,False,True,,Report #9465
Diary,HistoryWritings,1000.0,21.0,False,True,,
Diary,HistoryWritings,1000.0,21.0,False,False,Genuine,Report #9466
Diary,HistoryWritings,1000.0,21.0,False,False,Genuine,
Diary,HistoryWritings,1000.0,21.0,False,False,Fake,Report #9467
Diary,HistoryWritings,1000.0,21.0,False,False,Fake,
Diary,HistoryWritings,1000.0,21.0,False,False,,Report #9468
Diary,HistoryWritings,1000.0,21.0,False,False,,
Diary,HistoryWritings,1000.0,99.0,True,True,Genuine,Report #9469
Diary,HistoryWritings,1000.0,99.0,True,True,Genuine,
Diary,HistoryWritings,1000.0,99.0,True,True,Fake,Report #9470
Diary,HistoryWritings,1000.0,99.0,True,True,Fake,
Diary,HistoryWritings,1000.0,99.0,True,True,,Report #9471
Diary,HistoryWritings,1000.0,99.0,True,True,,
Diary,HistoryWritings,1000.0,99.0,True,False,Genuine,Report #9472
Diary,HistoryWritings,1000.0,99.0,True,False,Genuine,
Diary,HistoryWritings,1000.0,99.0,True,False,Fake,Report #9473
Diary,HistoryWritings,1000.0,99.0,True,False,Fake,
Diary,HistoryWritings,1000.0,99.0,True,False,,Report #9474
Diary,HistoryWritings,1000.0,99.0,True,False,,
Diary,HistoryWritings,1000.0,99.0,False,True,Genuine,Report #9475
Diary,HistoryWritings,1000.0,99.0,False,True,Genuine,
Diary,HistoryWritings,1000.0,99.0,False,True,Fake,Report #9476
Diary,HistoryWritings,1000.0,99.0,False,True,Fake,
Diary,HistoryWritings,1000.0,99.0,False,True,,Report #9477
Diary,HistoryWritings,1000.0,99.0,False,True,,
Diary,HistoryWritings,1000.0,99.0,False,False,Genuine,Report #9478
Diary,HistoryWritings,1000.0,99.0,False,False,Genuine,
Diary,HistoryWritings,1000.0,99.0,False,False,Fake,Report #9479
Diary,HistoryWritings,1000.0,99.0,False,False,Fake,
Diary,HistoryWritings,1000.0,99.0,False,False,,Report #9480
Diary,HistoryWritings,1000.0,99.0,False,False,,
Diary,HistoryWritings,1000.0,100.0,True,True,Genuine,Report #9481
Diary,HistoryWritings,1000.0,100.0,True,True,Genuine,
Diary,HistoryWritings,1000.0,100.0,True,True,Fake,Report #9482
Diary,HistoryWritings,1000.0,100.0,True,True,Fake,
Diary,HistoryWritings,1000.0,100.0,True,True,,Report #9483
Diary,HistoryWritings,1000.0,100.0,True,True,,
Diary,HistoryWritings,1000.0,100.0,True,False,Genuine,Report #9484
Diary,HistoryWritings,1000.0,100.0,True,False,Genuine,
Diary,HistoryWritings,1000.0,100.0,True,False,Fake,Report #9485
Diary,HistoryWritings,1000.0,100.0,True,False,Fake,
Diary,HistoryWritings,1000.0,100.0,True,False,,Report #9486
Diary,HistoryWritings,1000.0,100.0,True,False,,
Diary,HistoryWritings,1000.0,100.0,False,True,Genuine,Report #9487
Diary,HistoryWritings,1000.0,100.0,False,True,Genuine,
Diary,HistoryWritings,1000.0,100.0,False,True,Fake,Report #9488
Diary,HistoryWritings,1000.0,100.0,False,True,Fake,
Diary,HistoryWritings,1000.0,100.0,False,True,,Report #9489
Diary,HistoryWritings,1000.0,100.0,False,True,,
Diary,HistoryWritings,1000.0,100.0,False,False,Genuine,Report #9490
Diary,HistoryWritings,1000.0,100.0,False,False,Genuine,
Diary,HistoryWritings,1000.0,100.0,False,False,Fake,Report #9491
Diary,HistoryWritings,1000.0,100.0,False,False,Fake,
Diary,HistoryWritings,1000.0,100.0,False,False,,Report #9492
Diary,HistoryWritings,1000.0,100.0,False,False,,
Diary,HistoryWritings,1000.0,101.0,True,True,Genuine,Report #9493
Diary,HistoryWritings,1000.0,101.0,True,True,Genuine,
Diary,HistoryWritings,1000.0,101.0,True,True,Fake,Report #9494
Diary,HistoryWritings,1000.0,101.0,True,True,Fake,
Diary,HistoryWritings,1000.0,101.0,True,True,,Report #9495
Diary,HistoryWritings,1000.0,101.0,True,True,,
Diary,HistoryWritings,1000.0,101.0,True,False,Genuine,Report #9496
Diary,HistoryWritings,1000.0,101.0,True,False,Genuine,
Diary,HistoryWritings,1000.0,101.0,True,False,Fake,Report #9497
Diary,HistoryWritings,1000.0,101.0,True,False,Fake,
Diary,HistoryWritings,1000.0,101.0,True,False,,Report #9498
Diary,HistoryWritings,1000.0,101.0,True,False,,
Diary,HistoryWritings,1000.0,101.0,False,True,Genuine,Report #9499
Diary,HistoryWritings,1000.0,101.0,False,True,Genuine,
Diary,HistoryWritings,1000.0,101.0,False,True,Fake,Report #9500
Diary,HistoryWritings,1000.0,101.0,False,True,Fake,
Diary,HistoryWritings,1000.0,101.0,False,True,,Report #9501
Diary,HistoryWritings,1000.0,101.0,False,True,,
Diary,HistoryWritings,1000.0,101.0,False,False,Genuine,Report #9502
Diary,HistoryWritings,1000.0,101.0,False,False,Genuine,
Diary,HistoryWritings,1000.0,101.0,False,False,Fake,Report #9503
Diary,HistoryWritings,1000.0,101.0,False,False,Fake,
Diary,HistoryWritings,1000.0,101.0,False,False,,Report #9504
Diary,HistoryWritings,1000.0,101.0,False,False,,
Diary,HistoryWritings,1000000.0,-1.0,True,True,Genuine,Report #9505
Diary,HistoryWritings,1000000.0,-1.0,True,True,Genuine,
Diary,HistoryWritings,1000000.0,-1.0,True,True,Fake,Report #9506
Diary,HistoryWritings,1000000.0,-1.0,True,True,Fake,
Diary,HistoryWritings,1000000.0,-1.0,True,True,,Report #9507
Diary,HistoryWritings,1000000.0,-1.0,True,True,,
Diary,HistoryWritings,1000000.0,-1.0,True,False,Genuine,Report #9508
Diary,HistoryWritings,1000000.0,-1.0,True,False,Genuine,
Diary,HistoryWritings,1000000.0,-1.0,True,False,Fake,Report #9509
Diary,HistoryWritings,1000000.0,-1.0,True,False,Fake,
Diary,HistoryWritings,1000000.0,-1.0,True,False,,Report #9510
Diary,HistoryWritings,1000000.0,-1.0,True,False,,
Diary,HistoryWritings,1000000.0,-1.0,False,True,Genuine,Report #9511
Diary,HistoryWritings,1000000.0,-1.0,False,True,Genuine,
Diary,HistoryWritings,1000000.0,-1.0,False,True,Fake,Report #9512
Diary,HistoryWritings,1000000.0,-1.0,False,True,Fake,
Diary,HistoryWritings,1000000.0,-1.0,False,True,,Report #9513
Diary,HistoryWritings,1000000.0,-1.0,False,True,,
Diary,HistoryWritings,1000000.0,-1.0,False,False,Genuine,Report #9514
Diary,HistoryWritings,1000000.0,-1.0,False,False,Genuine,
Diary,HistoryWritings,1000000.0,-1.0,False,False,Fake,Report #9515
Diary,HistoryWritings,1000000.0,-1.0,False,False,Fake,
Diary,HistoryWritings,1000000.0,-1.0,False,False,,Report #9516
Diary,HistoryWritings,1000000.0,-1.0,False,False,,
Diary,HistoryWritings,1000000.0,0.0,True,True,Genuine,Report #9517
Diary,HistoryWritings,1000000.0,0.0,True,True,Genuine,
Diary,HistoryWritings,1000000.0,0.0,True,True,Fake,Report #9518
Diary,HistoryWritings,1000000.0,0.0,True,True,Fake,
Diary,HistoryWritings,1000000.0,0.0,True,True,,Report #9519
Diary,HistoryWritings,1000000.0,0.0,True,True,,
Diary,HistoryWritings,1000000.0,0.0,True,False,Genuine,Report #9520
Diary,HistoryWritings,1000000.0,0.0,True,False,Genuine,
Diary,HistoryWritings,1000000.0,0.0,True,False,Fake,Report #9521
Diary,HistoryWritings,1000000.0,0.0,True,False,Fake,
Diary,HistoryWritings,1000000.0,0.0,True,False,,Report #9522
Diary,HistoryWritings,1000000.0,0.0,True,False,,
Diary,HistoryWritings,1000000.0,0.0,False,True,Genuine,Report #9523
Diary,HistoryWritings,1000000.0,0.0,False,True,Genuine,
Diary,HistoryWritings,1000000.0,0.0,False,True,Fake,Report #9524
Diary,HistoryWritings,1000000.0,0.0,False,True,Fake,
Diary,HistoryWritings,1000000.0,0.0,False,True,,Report #9525
Diary,HistoryWritings,1000000.0,0.0,False,True,,
Diary,HistoryWritings,1000000.0,0.0,False,False,Genuine,Report #9526
Diary,HistoryWritings,1000000.0,0.0,False,False,Genuine,
Diary,HistoryWritings,1000000.0,0.0,False,False,Fake,Report #9527
Diary,HistoryWritings,1000000.0,0.0,False,False,Fake,
Diary,HistoryWritings,1000000.0,0.0,False,False,,Report #9528
Diary,HistoryWritings,1000000.0,0.0,False,False,,
Diary,HistoryWritings,1000000.0,19.0,True,True,Genuine,Report #9529
Diary,HistoryWritings,1000000.0,19.0,True,True,Genuine,
Diary,HistoryWritings,1000000.0,19.0,True,True,Fake,Report #9530
Diary,HistoryWritings,1000000.0,19.0,True,True,Fake,
Diary,HistoryWritings,1000000.0,19.0,True,True,,Report #9531
Diary,HistoryWritings,1000000.0,19.0,True,True,,
Diary,HistoryWritings,1000000.0,19.0,True,False,Genuine,Report #9532
Diary,HistoryWritings,1000000.0,19.0,True,False,Genuine,
Diary,HistoryWritings,1000000.0,19.0,True,False,Fake,Report #9533
Diary,HistoryWritings,1000000.0,19.0,True,False,Fake,
Diary,HistoryWritings,1000000.0,19.0,True,False,,Report #9534
Diary,HistoryWritings,1000000.0,19.0,True,False,,
Diary,HistoryWritings,1000000.0,19.0,False,True,Genuine,Report #9535
Diary,HistoryWritings,1000000.0,19.0,False,True,Genuine,
Diary,HistoryWritings,1000000.0,19.0,False,True,Fake,Report #9536
Diary,HistoryWritings,1000000.0,19.0,False,True,Fake,
Diary,HistoryWritings,1000000.0,19.0,False,True,,Report #9537
Diary,HistoryWritings,1000000.0,19.0,False,True,,
Diary,HistoryWritings,1000000.0,19.0,False,False,Genuine,Report #9538
Diary,HistoryWritings,1000000.0,19.0,False,False,Genuine,
Diary,HistoryWritings,1000000.0,19.0,False,False,Fake,Report #9539
Diary,HistoryWritings,1000000.0,19.0,False,False,Fake,
Diary,HistoryWritings,1000000.0,19.0,False,False,,Report #9540
Diary,HistoryWritings,1000000.0,19.0,False,False,,
Diary,HistoryWritings,1000000.0,20.0,True,True,Genuine,Report #9541
Diary,HistoryWritings,1000000.0,20.0,True,True,Genuine,
Diary,HistoryWritings,1000000.0,20.0,True,True,Fake,Report #9542
Diary,HistoryWritings,1000000.0,20.0,True,True,Fake,
Diary,HistoryWritings,1000000.0,20.0,True,True,,Report #9543
Diary,HistoryWritings,1000000.0,20.0,True,True,,
Diary,HistoryWritings,1000000.0,20.0,True,False,Genuine,Report #9544
Diary,HistoryWritings,1000000.0,20.0,True,False,Genuine,
Diary,HistoryWritings,1000000.0,20.0,True,False,Fake,Report #9545
Diary,HistoryWritings,1000000.0,20.0,True,False,Fake,
Diary,HistoryWritings,1000000.0,20.0,True,False,,Report #9546
Diary,HistoryWritings,1000000.0,20.0,True,False,,
Diary,HistoryWritings,1000000.0,20.0,False,True,Genuine,Report #9547
Diary,HistoryWritings,1000000.0,20.0,False,True,Genuine,
Diary,HistoryWritings,1000000.0,20.0,False,True,Fake,Report #9548
Diary,HistoryWritings,1000000.0,20.0,False,True,Fake,
Diary,HistoryWritings,1000000.0,20.0,False,True,,Report #9549
Diary,HistoryWritings,1000000.0,20.0,False,True,,
Diary,HistoryWritings,1000000.0,20.0,False,False,Genuine,Report #9550
Diary,HistoryWritings,1000000.0,20.0,False,False,Genuine,
Diary,HistoryWritings,1000000.0,20.0,False,False,Fake,Report #9551
Diary,HistoryWritings,1000000.0,20.0,False,False,Fake,
Diary,HistoryWritings,1000000.0,20.0,False,False,,Report #9552
Diary,HistoryWritings,1000000.0,20.0,False,False,,
Diary,HistoryWritings,1000000.0,21.0,True,True,Genuine,Report #9553
Diary,HistoryWritings,1000000.0,21.0,True,True,Genuine,
Diary,HistoryWritings,1000000.0,21.0,True,True,Fake,Report #9554
Diary,HistoryWritings,1000000.0,21.0,True,True,Fake,
Diary,HistoryWritings,1000000.0,21.0,True,True,,Report #9555
Diary,HistoryWritings,1000000.0,21.0,True,True,,
Diary,HistoryWritings,1000000.0,21.0,True,False,Genuine,Report #9556
Diary,HistoryWritings,1000000.0,21.0,True,False,Genuine,
Diary,HistoryWritings,1000000.0,21.0,True,False,Fake,Report #9557
Diary,HistoryWritings,1000000.0,21.0,True,False,Fake,
Diary,HistoryWritings,1000000.0,21.0,True,False,,Report #9558
Diary,HistoryWritings,1000000.0,21.0,True,False,,
Diary,HistoryWritings,1000000.0,21.0,False,True,Genuine,Report #9559
Diary,HistoryWritings,1000000.0,21.0,False,True,Genuine,
Diary,HistoryWritings,1000000.0,21.0,False,True,Fake,Report #9560
Diary,HistoryWritings,1000000.0,21.0,False,True,Fake,
Diary,HistoryWritings,1000000.0,21.0,False,True,,Report #9561
Diary,HistoryWritings,1000000.0,21.0,False,True,,
Diary,HistoryWritings,1000000.0,21.0,False,False,Genuine,Report #9562
Diary,HistoryWritings,1000000.0,21.0,False,False,Genuine,
Diary,HistoryWritings,1000000.0,21.0,False,False,Fake,Report #9563
Diary,HistoryWritings,1000000.0,21.0,False,False,Fake,
Diary,HistoryWritings,1000000.0,21.0,False,False,,Report #9564
Diary,HistoryWritings,1000000.0,21.0,False,False,,
Diary,HistoryWritings,1000000.0,99.0,True,True,Genuine,Report #9565
Diary,HistoryWritings,1000000.0,99.0,True,True,Genuine,
Diary,HistoryWritings,1000000.0,99.0,True,True,Fake,Report #9566
Diary,HistoryWritings,1000000.0,99.0,True,True,Fake,
Diary,HistoryWritings,1000000.0,99.0,True,True,,Report #9567
Diary,HistoryWritings,1000000.0,99.0,True,True,,
Diary,HistoryWritings,1000000.0,99.0,True,False,Genuine,Report #9568
Diary,HistoryWritings,1000000.0,99.0,True,False,Genuine,
Diary,HistoryWritings,1000000.0,99.0,True,False,Fake,Report #9569
Diary,HistoryWritings,1000000.0,99.0,True,False,Fake,
Diary,HistoryWritings,1000000.0,99.0,True,False,,Report #9570
Diary,HistoryWritings,1000000.0,99.0,True,False,,
Diary,HistoryWritings,1000000.0,99.0,False,True,Genuine,Report #9571
Diary,HistoryWritings,1000000.0,99.0,False,True,Genuine,
Diary,HistoryWritings,1000000.0,99.0,False,True,Fake,Report #9572
Diary,HistoryWritings,1000000.0,99.0,False,True,Fake,
Diary,HistoryWritings,1000000.0,99.0,False,True,,Report #9573
Diary,HistoryWritings,1000000.0,99.0,False,True,,
Diary,HistoryWritings,1000000.0,99.0,False,False,Genuine,Report #9574
Diary,HistoryWritings,1000000.0,99.0,False,False,Genuine,
Diary,HistoryWritings,1000000.0,99.0,False,False,Fake,Report #9575
Diary,HistoryWritings,1000000.0,99.0,False,False,Fake,
Diary,HistoryWritings,1000000.0,99.0,False,False,,Report #9576
Diary,HistoryWritings,1000000.0,99.0,False,False,,
Diary,HistoryWritings,1000000.0,100.0,True,True,Genuine,Report #9577
Diary,HistoryWritings,1000000.0,100.0,True,True,Genuine,
Diary,HistoryWritings,1000000.0,100.0,True,True,Fake,Report #9578
Diary,HistoryWritings,1000000.0,100.0,True,True,Fake,
Diary,HistoryWritings,1000000.0,100.0,True,True,,Report #9579
Diary,HistoryWritings,1000000.0,100.0,True,True,,
Diary,HistoryWritings,1000000.0,100.0,True,False,Genuine,Report #9580
Diary,HistoryWritings,1000000.0,100.0,True,False,Genuine,
Diary,HistoryWritings,1000000.0,100.0,True,False,Fake,Report #9581
Diary,HistoryWritings,1000000.0,100.0,True,False,Fake,
Diary,HistoryWritings,1000000.0,100.0,True,False,,Report #9582
Diary,HistoryWritings,1000000.0,100.0,True,False,,
Diary,HistoryWritings,1000000.0,100.0,False,True,Genuine,Report #9583
Diary,HistoryWritings,1000000.0,100.0,False,True,Genuine,
Diary,HistoryWritings,1000000.0,100.0,False,True,Fake,Report #9584
Diary,HistoryWritings,1000000.0,100.0,False,True,Fake,
Diary,HistoryWritings,1000000.0,100.0,False,True,,Report #9585
Diary,HistoryWritings,1000000.0,100.0,False,True,,
Diary,HistoryWritings,1000000.0,100.0,False,False,Genuine,Report #9586
Diary,HistoryWritings,1000000.0,100.0,False,False,Genuine,
Diary,HistoryWritings,1000000.0,100.0,False,False,Fake,Report #9587
Diary,HistoryWritings,1000000.0,100.0,False,False,Fake,
Diary,HistoryWritings,1000000.0,100.0,False,False,,Report #9588
Diary,HistoryWritings,1000000.0,100.0,False,False,,
Diary,HistoryWritings,1000000.0,101.0,True,True,Genuine,Report #9589
Diary,HistoryWritings,1000000.0,101.0,True,True,Genuine,
Diary,HistoryWritings,1000000.0,101.0,True,True,Fake,Report #9590
Diary,HistoryWritings,1000000.0,101.0,True,True,Fake,
Diary,HistoryWritings,1000000.0,101.0,True,True,,Report #9591
Diary,HistoryWritings,1000000.0,101.0,True,True,,
Diary,HistoryWritings,1000000.0,101.0,True,False,Genuine,Report #9592
Diary,HistoryWritings,1000000.0,101.0,True,False,Genuine,
Diary,HistoryWritings,1000000.0,101.0,True,False,Fake,Report #9593
Diary,HistoryWritings,1000000.0,101.0,True,False,Fake,
Diary,HistoryWritings,1000000.0,101.0,True,False,,Report #9594
Diary,HistoryWritings,1000000.0,101.0,True,False,,
Diary,HistoryWritings,1000000.0,101.0,False,True,Genuine,Report #9595
Diary,HistoryWritings,1000000.0,101.0,False,True,Genuine,
Diary,HistoryWritings,1000000.0,101.0,False,True,Fake,Report #9596
Diary,HistoryWritings,1000000.0,101.0,False,True,Fake,
Diary,HistoryWritings,1000000.0,101.0,False,True,,Report #9597
Diary,HistoryWritings,1000000.0,101.0,False,True,,
Diary,HistoryWritings,1000000.0,101.0,False,False,Genuine,Report #9598
Diary,HistoryWritings,1000000.0,101.0,False,False,Genuine,
Diary,HistoryWritings,1000000.0,101.0,False,False,Fake,Report #9599
Diary,HistoryWritings,1000000.0,101.0,False,False,Fake,
Diary,HistoryWritings,1000000.0,101.0,False,False,,Report #9600
Diary,HistoryWritings,1000000.0,101.0,False,False,,
,Paintings,0.0,-1.0,True,True,Genuine,Report #9601
,Paintings,0.0,-1.0,True,True,Genuine,
,Paintings,0.0,-1.0,True,True,Fake,Report #9602
,Paintings,0.0,-1.0,True,True,Fake,
,Paintings,0.0,-1.0,True,True,,Report #9603
,Paintings,0.0,-1.0,True,True,,
,Paintings,0.0,-1.0,True,False,Genuine,Report #9604
,Paintings,0.0,-1.0,True,False,Genuine,
,Paintings,0.0,-1.0,True,False,Fake,Report #9605
,Paintings,0.0,-1.0,True,False,Fake,
,Paintings,0.0,-1.0,True,False,,Report #9606
,Paintings,0.0,-1.0,True,False,,
,Paintings,0.0,-1.0,False,True,Genuine,Report #9607
,Paintings,0.0,-1.0,False,True,Genuine,
,Paintings,0.0,-1.0,False,True,Fake,Report #9608
,Paintings,0.0,-1.0,False,True,Fake,
,Paintings,0.0,-1.0,False,True,,Report #9609
,Paintings,0.0,-1.0,False,True,,
,Paintings,0.0,-1.0,False,False,Genuine,Report #9610
,Paintings,0.0,-1.0,False,False,Genuine,
,Paintings,0.0,-1.0,False,False,Fake,Report #9611
,Paintings,0.0,-1.0,False,False,Fake,
,Paintings,0.0,-1.0,False,False,,Report #9612
,Paintings,0.0,-1.0,False,False,,
,Paintings,0.0,0.0,True,True,Genuine,Report #9613
,Paintings,0.0,0.0,True,True,Genuine,
,Paintings,0.0,0.0,True,True,Fake,Report #9614
,Paintings,0.0,0.0,True,True,Fake,
,Paintings,0.0,0.0,True,True,,Report #9615
,Paintings,0.0,0.0,True,True,,
,Paintings,0.0,0.0,True,False,Genuine,Report #9616
,Paintings,0.0,0.0,True,False,Genuine,
,Paintings,0.0,0.0,True,False,Fake,Report #9617
,Paintings,0.0,0.0,True,False,Fake,
,Paintings,0.0,0.0,True,False,,Report #9618
,Paintings,0.0,0.0,True,False,,
,Paintings,0.0,0.0,False,True,Genuine,Report #9619
,Paintings,0.0,0.0,False,True,Genuine,
,Paintings,0.0,0.0,False,True,Fake,Report #9620
,Paintings,0.0,0.0,False,True,Fake,
,Paintings,0.0,0.0,False,True,,Report #9621
,Paintings,0.0,0.0,False,True,,
,Paintings,0.0,0.0,False,False,Genuine,Report #9622
,Paintings,0.0,0.0,False,False,Genuine,
,Paintings,0.0,0.0,False,False,Fake,Report #9623
,Paintings,0.0,0.0,False,False,Fake,
,Paintings,0.0,0.0,False,False,,Report #9624
,Paintings,0.0,0.0,False,False,,
,Paintings,0.0,19.0,True,True,Genuine,Report #9625
,Paintings,0.0,19.0,True,True,Genuine,
,Paintings,0.0,19.0,True,True,Fake,Report #9626
,Paintings,0.0,19.0,True,True,Fake,
,Paintings,0.0,19.0,True,True,,Report #9627
,Paintings,0.0,19.0,True,True,,
,Paintings,0.0,19.0,True,False,Genuine,Report #9628
,Paintings,0.0,19.0,True,False,Genuine,
,Paintings,0.0,19.0,True,False,Fake,Report #9629
,Paintings,0.0,19.0,True,False,Fake,
,Paintings,0.0,19.0,True,False,,Report #9630
,Paintings,0.0,19.0,True,False,,
,Paintings,0.0,19.0,False,True,Genuine,Report #9631
,Paintings,0.0,19.0,False,True,Genuine,
,Paintings,0.0,19.0,False,True,Fake,Report #9632
,Paintings,0.0,19.0,False,True,Fake,
,Paintings,0.0,19.0,False,True,,Report #9633
,Paintings,0.0,19.0,False,True,,
,Paintings,0.0,19.0,False,False,Genuine,Report #9634
,Paintings,0.0,19.0,False,False,Genuine,
,Paintings,0.0,19.0,False,False,Fake,Report #9635
,Paintings,0.0,19.0,False,False,Fake,
,Paintings,0.0,19.0,False,False,,Report #9636
,Paintings,0.0,19.0,False,False,,
,Paintings,0.0,20.0,True,True,Genuine,Report #9637
,Paintings,0.0,20.0,True,True,Genuine,
,Paintings,0.0,20.0,True,True,Fake,Report #9638
,Paintings,0.0,20.0,True,True,Fake,
,Paintings,0.0,20.0,True,True,,Report #9639
,Paintings,0.0,20.0,True,True,,
,Paintings,0.0,20.0,True,False,Genuine,Report #9640
,Paintings,0.0,20.0,True,False,Genuine,
,Paintings,0.0,20.0,True,False,Fake,Report #9641
,Paintings,0.0,20.0,True,False,Fake,
,Paintings,0.0,20.0,True,False,,Report #9642
,Paintings,0.0,20.0,True,False,,
,Paintings,0.0,20.0,False,True,Genuine,Report #9643
,Paintings,0.0,20.0,False,True,Genuine,
,Paintings,0.0,20.0,False,True,Fake,Report #9644
,Paintings,0.0,20.0,False,True,Fake,
,Paintings,0.0,20.0,False,True,,Report #9645
,Paintings,0.0,20.0,False,True,,
,Paintings,0.0,20.0,False,False,Genuine,Report #9646
,Paintings,0.0,20.0,False,False,Genuine,
,Paintings,0.0,20.0,False,False,Fake,Report #9647
,Paintings,0.0,20.0,False,False,Fake,
,Paintings,0.0,20.0,False,False,,Report #9648
,Paintings,0.0,20.0,False,False,,
,Paintings,0.0,21.0,True,True,Genuine,Report #9649
,Paintings,0.0,21.0,True,True,Genuine,
,Paintings,0.0,21.0,True,True,Fake,Report #9650
,Paintings,0.0,21.0,True,True,Fake,
,Paintings,0.0,21.0,True,True,,Report #9651
,Paintings,0.0,21.0,True,True,,
,Paintings,0.0,21.0,True,False,Genuine,Report #9652
,Paintings,0.0,21.0,True,False,Genuine,
,Paintings,0.0,21.0,True,False,Fake,Report #9653
,Paintings,0.0,21.0,True,False,Fake,
,Paintings,0.0,21.0,True,False,,Report #9654
,Paintings,0.0,21.0,True,False,,
,Paintings,0.0,21.0,False,True,Genuine,Report #9655
,Paintings,0.0,21.0,False,True,Genuine,
,Paintings,0.0,21.0,False,True,Fake,Report #9656
,Paintings,0.0,21.0,False,True,Fake,
,Paintings,0.0,21.0,False,True,,Report #9657
,Paintings,0.0,21.0,False,True,,
,Paintings,0.0,21.0,False,False,Genuine,Report #9658
,Paintings,0.0,21.0,False,False,Genuine,
,Paintings,0.0,21.0,False,False,Fake,Report #9659
,Paintings,0.0,21.0,False,False,Fake,
,Paintings,0.0,21.0,False,False,,Report #9660
,Paintings,0.0,21.0,False,False,,
,Paintings,0.0,99.0,True,True,Genuine,Report #9661
,Paintings,0.0,99.0,True,True,Genuine,
,Paintings,0.0,99.0,True,True,Fake,Report #9662
,Paintings,0.0,99.0,True,True,Fake,
,Paintings,0.0,99.0,True,True,,Report #9663
,Paintings,0.0,99.0,True,True,,
,Paintings,0.0,99.0,True,False,Genuine,Report #9664
,Paintings,0.0,99.0,True,False,Genuine,
,Paintings,0.0,99.0,True,False,Fake,Report #9665
,Paintings,0.0,99.0,True,False,Fake,
,Paintings,0.0,99.0,True,False,,Report #9666
,Paintings,0.0,99.0,True,False,,
,Paintings,0.0,99.0,False,True,Genuine,Report #9667
,Paintings,0.0,99.0,False,True,Genuine,
,Paintings,0.0,99.0,False,True,Fake,Report #9668
,Paintings,0.0,99.0,False,True,Fake,
,Paintings,0.0,99.0,False,True,,Report #9669
,Paintings,0.0,99.0,False,True,,
,Paintings,0.0,99.0,False,False,Genuine,Report #9670
,Paintings,0.0,99.0,False,False,Genuine,
,Paintings,0.0,99.0,False,False,Fake,Report #9671
,Paintings,0.0,99.0,False,False,Fake,
,Paintings,0.0,99.0,False,False,,Report #9672
,Paintings,0.0,99.0,False,False,,
,Paintings,0.0,100.0,True,True,Genuine,Report #9673
,Paintings,0.0,100.0,True,True,Genuine,
,Paintings,0.0,100.0,True,True,Fake,Report #9674
,Paintings,0.0,100.0,True,True,Fake,
,Paintings,0.0,100.0,True,True,,Report #9675
,Paintings,0.0,100.0,True,True,,
,Paintings,0.0,100.0,True,False,Genuine,Report #9676
,Paintings,0.0,100.0,True,False,Genuine,
,Paintings,0.0,100.0,True,False,Fake,Report #9677
,Paintings,0.0,100.0,True,False,Fake,
,Paintings,0.0,100.0,True,False,,Report #9678
,Paintings,0.0,100.0,True,False,,
,Paintings,0.0,100.0,False,True,Genuine,Report #9679
,Paintings,0.0,100.0,False,True,Genuine,
,Paintings,0.0,100.0,False,True,Fake,Report #9680
,Paintings,0.0,100.0,False,True,Fake,
,Paintings,0.0,100.0,False,True,,Report #9681
,Paintings,0.0,100.0,False,True,,
,Paintings,0.0,100.0,False,False,Genuine,Report #9682
,Paintings,0.0,100.0,False,False,Genuine,
,Paintings,0.0,100.0,False,False,Fake,Report #9683
,Paintings,0.0,100.0,False,False,Fake,
,Paintings,0.0,100.0,False,False,,Report #9684
,Paintings,0.0,100.0,False,False,,
,Paintings,0.0,101.0,True,True,Genuine,Report #9685
,Paintings,0.0,101.0,True,True,Genuine,
,Paintings,0.0,101.0,True,True,Fake,Report #9686
,Paintings,0.0,101.0,True,True,Fake,
,Paintings,0.0,101.0,True,True,,Report #9687
,Paintings,0.0,101.0,True,True,,
,Paintings,0.0,101.0,True,False,Genuine,Report #9688
,Paintings,0.0,101.0,True,False,Genuine,
,Paintings,0.0,101.0,True,False,Fake,Report #9689
,Paintings,0.0,101.0,True,False,Fake,
,Paintings,0.0,101.0,True,False,,Report #9690
,Paintings,0.0,101.0,True,False,,
,Paintings,0.0,101.0,False,True,Genuine,Report #9691
,Paintings,0.0,101.0,False,True,Genuine,
,Paintings,0.0,101.0,False,True,Fake,Report #9692
,Paintings,0.0,101.0,False,True,Fake,
,Paintings,0.0,101.0,False,True,,Report #9693
,Paintings,0.0,101.0,False,True,,
,Paintings,0.0,101.0,False,False,Genuine,Report #9694
,Paintings,0.0,101.0,False,False,Genuine,
,Paintings,0.0,101.0,False,False,Fake,Report #9695
,Paintings,0.0,101.0,False,False,Fake,
,Paintings,0.0,101.0,False,False,,Report #9696
,Paintings,0.0,101.0,False,False,,
,Paintings,1.0,-1.0,True,True,Genuine,Report #9697
,Paintings,1.0,-1.0,True,True,Genuine,
,Paintings,1.0,-1.0,True,True,Fake,Report #9698
,Paintings,1.0,-1.0,True,True,Fake,
,Paintings,1.0,-1.0,True,True,,Report #9699
,Paintings,1.0,-1.0,True,True,,
,Paintings,1.0,-1.0,True,False,Genuine,Report #9700
,Paintings,1.0,-1.0,True,False,Genuine,
,Paintings,1.0,-1.0,True,False,Fake,Report #9701
,Paintings,1.0,-1.0,True,False,Fake,
,Paintings,1.0,-1.0,True,False,,Report #9702
,Paintings,1.0,-1.0,True,False,,
,Paintings,1.0,-1.0,False,True,Genuine,Report #9703
,Paintings,1.0,-1.0,False,True,Genuine,
,Paintings,1.0,-1.0,False,True,Fake,Report #9704
,Paintings,1.0,-1.0,False,True,Fake,
,Paintings,1.0,-1.0,False,True,,Report #9705
,Paintings,1.0,-1.0,False,True,,
,Paintings,1.0,-1.0,False,False,Genuine,Report #9706
,Paintings,1.0,-1.0,False,False,Genuine,
,Paintings,1.0,-1.0,False,False,Fake,Report #9707
,Paintings,1.0,-1.0,False,False,Fake,
,Paintings,1.0,-1.0,False,False,,Report #9708
,Paintings,1.0,-1.0,False,False,,
,Paintings,1.0,0.0,True,True,Genuine,Report #9709
,Paintings,1.0,0.0,True,True,Genuine,
,Paintings,1.0,0.0,True,True,Fake,Report #9710
,Paintings,1.0,0.0,True,True,Fake,
,Paintings,1.0,0.0,True,True,,Report #9711
,Paintings,1.0,0.0,True,True,,
,Paintings,1.0,0.0,True,False,Genuine,Report #9712
,Paintings,1.0,0.0,True,False,Genuine,
,Paintings,1.0,0.0,True,False,Fake,Report #9713
,Paintings,1.0,0.0,True,False,Fake,
,Paintings,1.0,0.0,True,False,,Report #9714
,Paintings,1.0,0.0,True,False,,
,Paintings,1.0,0.0,False,True,Genuine,Report #9715
,Paintings,1.0,0.0,False,True,Genuine,
,Paintings,1.0,0.0,False,True,Fake,Report #9716
,Paintings,1.0,0.0,False,True,Fake,
,Paintings,1.0,0.0,False,True,,Report #9717
,Paintings,1.0,0.0,False,True,,
,Paintings,1.0,0.0,False,False,Genuine,Report #9718
,Paintings,1.0,0.0,False,False,Genuine,
,Paintings,1.0,0.0,False,False,Fake,Report #9719
,Paintings,1.0,0.0,False,False,Fake,
,Paintings,1.0,0.0,False,False,,Report #9720
,Paintings,1.0,0.0,False,False,,
,Paintings,1.0,19.0,True,True,Genuine,Report #9721
,Paintings,1.0,19.0,True,True,Genuine,
,Paintings,1.0,19.0,True,True,Fake,Report #9722
,Paintings,1.0,19.0,True,True,Fake,
,Paintings,1.0,19.0,True,True,,Report #9723
,Paintings,1.0,19.0,True,True,,
,Paintings,1.0,19.0,True,False,Genuine,Report #9724
,Paintings,1.0,19.0,True,False,Genuine,
,Paintings,1.0,19.0,True,False,Fake,Report #9725
,Paintings,1.0,19.0,True,False,Fake,
,Paintings,1.0,19.0,True,False,,Report #9726
,Paintings,1.0,19.0,True,False,,
,Paintings,1.0,19.0,False,True,Genuine,Report #9727
,Paintings,1.0,19.0,False,True,Genuine,
,Paintings,1.0,19.0,False,True,Fake,Report #9728
,Paintings,1.0,19.0,False,True,Fake,
,Paintings,1.0,19.0,False,True,,Report #9729
,Paintings,1.0,19.0,False,True,,
,Paintings,1.0,19.0,False,False,Genuine,Report #9730
,Paintings,1.0,19.0,False,False,Genuine,
,Paintings,1.0,19.0,False,False,Fake,Report #9731
,Paintings,1.0,19.0,False,False,Fake,
,Paintings,1.0,19.0,False,False,,Report #9732
,Paintings,1.0,19.0,False,False,,
,Paintings,1.0,20.0,True,True,Genuine,Report #9733
,Paintings,1.0,20.0,True,True,Genuine,
,Paintings,1.0,20.0,True,True,Fake,Report #9734
,Paintings,1.0,20.0,True,True,Fake,
,Paintings,1.0,20.0,True,True,,Report #9735
,Paintings,1.0,20.0,True,True,,
,Paintings,1.0,20.0,True,False,Genuine,Report #9736
,Paintings,1.0,20.0,True,False,Genuine,
,Paintings,1.0,20.0,True,False,Fake,Report #9737
,Paintings,1.0,20.0,True,False,Fake,
,Paintings,1.0,20.0,True,False,,Report #9738
,Paintings,1.0,20.0,True,False,,
,Paintings,1.0,20.0,False,True,Genuine,Report #9739
,Paintings,1.0,20.0,False,True,Genuine,
,Paintings,1.0,20.0,False,True,Fake,Report #9740
,Paintings,1.0,20.0,False,True,Fake,
,Paintings,1.0,20.0,False,True,,Report #9741
,Paintings,1.0,20.0,False,True,,
,Paintings,1.0,20.0,False,False,Genuine,Report #9742
,Paintings,1.0,20.0,False,False,Genuine,
,Paintings,1.0,20.0,False,False,Fake,Report #9743
,Paintings,1.0,20.0,False,False,Fake,
,Paintings,1.0,20.0,False,False,,Report #9744
,Paintings,1.0,20.0,False,False,,
,Paintings,1.0,21.0,True,True,Genuine,Report #9745
,Paintings,1.0,21.0,True,True,Genuine,
,Paintings,1.0,21.0,True,True,Fake,Report #9746
,Paintings,1.0,21.0,True,True,Fake,
,Paintings,1.0,21.0,True,True,,Report #9747
,Paintings,1.0,21.0,True,True,,
,Paintings,1.0,21.0,True,False,Genuine,Report #9748
,Paintings,1.0,21.0,True,False,Genuine,
,Paintings,1.0,21.0,True,False,Fake,Report #9749
,Paintings,1.0,21.0,True,False,Fake,
,Paintings,1.0,21.0,True,False,,Report #9750
,Paintings,1.0,21.0,True,False,,
,Paintings,1.0,21.0,False,True,Genuine,Report #9751
,Paintings,1.0,21.0,False,True,Genuine,
,Paintings,1.0,21.0,False,True,Fake,Report #9752
,Paintings,1.0,21.0,False,True,Fake,
,Paintings,1.0,21.0,False,True,,Report #9753
,Paintings,1.0,21.0,False,True,,
,Paintings,1.0,21.0,False,False,Genuine,Report #9754
,Paintings,1.0,21.0,False,False,Genuine,
,Paintings,1.0,21.0,False,False,Fake,Report #9755
,Paintings,1.0,21.0,False,False,Fake,
,Paintings,1.0,21.0,False,False,,Report #9756
,Paintings,1.0,21.0,False,False,,
,Paintings,1.0,99.0,True,True,Genuine,Report #9757
,Paintings,1.0,99.0,True,True,Genuine,
,Paintings,1.0,99.0,True,True,Fake,Report #9758
,Paintings,1.0,99.0,True,True,Fake,
,Paintings,1.0,99.0,True,True,,Report #9759
,Paintings,1.0,99.0,True,True,,
,Paintings,1.0,99.0,True,False,Genuine,Report #9760
,Paintings,1.0,99.0,True,False,Genuine,
,Paintings,1.0,99.0,True,False,Fake,Report #9761
,Paintings,1.0,99.0,True,False,Fake,
,Paintings,1.0,99.0,True,False,,Report #9762
,Paintings,1.0,99.0,True,False,,
,Paintings,1.0,99.0,False,True,Genuine,Report #9763
,Paintings,1.0,99.0,False,True,Genuine,
,Paintings,1.0,99.0,False,True,Fake,Report #9764
,Paintings,1.0,99.0,False,True,Fake,
,Paintings,1.0,99.0,False,True,,Report #9765
,Paintings,1.0,99.0,False,True,,
,Paintings,1.0,99.0,False,False,Genuine,Report #9766
,Paintings,1.0,99.0,False,False,Genuine,
,Paintings,1.0,99.0,False,False,Fake,Report #9767
,Paintings,1.0,99.0,False,False,Fake,
,Paintings,1.0,99.0,False,False,,Report #9768
,Paintings,1.0,99.0,False,False,,
,Paintings,1.0,100.0,True,True,Genuine,Report #9769
,Paintings,1.0,100.0,True,True,Genuine,
,Paintings,1.0,100.0,True,True,Fake,Report #9770
,Paintings,1.0,100.0,True,True,Fake,
,Paintings,1.0,100.0,True,True,,Report #9771
,Paintings,1.0,100.0,True,True,,
,Paintings,1.0,100.0,True,False,Genuine,Report #9772
,Paintings,1.0,100.0,True,False,Genuine,
,Paintings,1.0,100.0,True,False,Fake,Report #9773
,Paintings,1.0,100.0,True,False,Fake,
,Paintings,1.0,100.0,True,False,,Report #9774
,Paintings,1.0,100.0,True,False,,
,Paintings,1.0,100.0,False,True,Genuine,Report #9775
,Paintings,1.0,100.0,False,True,Genuine,
,Paintings,1.0,100.0,False,True,Fake,Report #9776
,Paintings,1.0,100.0,False,True,Fake,
,Paintings,1.0,100.0,False,True,,Report #9777
,Paintings,1.0,100.0,False,True,,
,Paintings,1.0,100.0,False,False,Genuine,Report #9778
,Paintings,1.0,100.0,False,False,Genuine,
,Paintings,1.0,100.0,False,False,Fake,Report #9779
,Paintings,1.0,100.0,False,False,Fake,
,Paintings,1.0,100.0,False,False,,Report #9780
,Paintings,1.0,100.0,False,False,,
,Paintings,1.0,101.0,True,True,Genuine,Report #9781
,Paintings,1.0,101.0,True,True,Genuine,
,Paintings,1.0,101.0,True,True,Fake,Report #9782
,Paintings,1.0,101.0,True,True,Fake,
,Paintings,1.0,101.0,True,True,,Report #9783
,Paintings,1.0,101.0,True,True,,
,Paintings,1.0,101.0,True,False,Genuine,Report #9784
,Paintings,1.0,101.0,True,False,Genuine,
,Paintings,1.0,101.0,True,False,Fake,Report #9785
,Paintings,1.0,101.0,True,False,Fake,
,Paintings,1.0,101.0,True,False,,Report #9786
,Paintings,1.0,101.0,True,False,,
,Paintings,1.0,101.0,False,True,Genuine,Report #9787
,Paintings,1.0,101.0,False,True,Genuine,
,Paintings,1.0,101.0,False,True,Fake,Report #9788
,Paintings,1.0,101.0,False,True,Fake,
,Paintings,1.0,101.0,False,True,,Report #9789
,Paintings,1.0,101.0,False,True,,
,Paintings,1.0,101.0,False,False,Genuine,Report #9790
,Paintings,1.0,101.0,False,False,Genuine,
,Paintings,1.0,101.0,False,False,Fake,Report #9791
,Paintings,1.0,101.0,False,False,Fake,
,Paintings,1.0,101.0,False,False,,Report #9792
,Paintings,1.0,101.0,False,False,,
,Paintings,100.0,-1.0,True,True,Genuine,Report #9793
,Paintings,100.0,-1.0,True,True,Genuine,
,Paintings,100.0,-1.0,True,True,Fake,Report #9794
,Paintings,100.0,-1.0,True,True,Fake,
,Paintings,100.0,-1.0,True,True,,Report #9795
,Paintings,100.0,-1.0,True,True,,
,Paintings,100.0,-1.0,True,False,Genuine,Report #9796
,Paintings,100.0,-1.0,True,False,Genuine,
,Paintings,100.0,-1.0,True,False,Fake,Report #9797
,Paintings,100.0,-1.0,True,False,Fake,
,Paintings,100.0,-1.0,True,False,,Report #9798
,Paintings,100.0,-1.0,True,False,,
,Paintings,100.0,-1.0,False,True,Genuine,Report #9799
,Paintings,100.0,-1.0,False,True,Genuine,
,Paintings,100.0,-1.0,False,True,Fake,Report #9800
,Paintings,100.0,-1.0,False,True,Fake,
,Paintings,100.0,-1.0,False,True,,Report #9801
,Paintings,100.0,-1.0,False,True,,
,Paintings,100.0,-1.0,False,False,Genuine,Report #9802
,Paintings,100.0,-1.0,False,False,Genuine,
,Paintings,100.0,-1.0,False,False,Fake,Report #9803
,Paintings,100.0,-1.0,False,False,Fake,
,Paintings,100.0,-1.0,False,False,,Report #9804
,Paintings,100.0,-1.0,False,False,,
,Paintings,100.0,0.0,True,True,Genuine,Report #9805
,Paintings,100.0,0.0,True,True,Genuine,
,Paintings,100.0,0.0,True,True,Fake,Report #9806
,Paintings,100.0,0.0,True,True,Fake,
,Paintings,100.0,0.0,True,True,,Report #9807
,Paintings,100.0,0.0,True,True,,
,Paintings,100.0,0.0,True,False,Genuine,Report #9808
,Paintings,100.0,0.0,True,False,Genuine,
,Paintings,100.0,0.0,True,False,Fake,Report #9809
,Paintings,100.0,0.0,True,False,Fake,
,Paintings,100.0,0.0,True,False,,Report #9810
,Paintings,100.0,0.0,True,False,,
,Paintings,100.0,0.0,False,True,Genuine,Report #9811
,Paintings,100.0,0.0,False,True,Genuine,
,Paintings,100.0,0.0,False,True,Fake,Report #9812
,Paintings,100.0,0.0,False,True,Fake,
,Paintings,100.0,0.0,False,True,,Report #9813
,Paintings,100.0,0.0,False,True,,
,Paintings,100.0,0.0,False,False,Genuine,Report #9814
,Paintings,100.0,0.0,False,False,Genuine,
,Paintings,100.0,0.0,False,False,Fake,Report #9815
,Paintings,100.0,0.0,False,False,Fake,
,Paintings,100.0,0.0,False,False,,Report #9816
,Paintings,100.0,0.0,False,False,,
,Paintings,100.0,19.0,True,True,Genuine,Report #9817
,Paintings,100.0,19.0,True,True,Genuine,
,Paintings,100.0,19.0,True,True,Fake,Report #9818
,Paintings,100.0,19.0,True,True,Fake,
,Paintings,100.0,19.0,True,True,,Report #9819
,Paintings,100.0,19.0,True,True,,
,Paintings,100.0,19.0,True,False,Genuine,Report #9820
,Paintings,100.0,19.0,True,False,Genuine,
,Paintings,100.0,19.0,True,False,Fake,Report #9821
,Paintings,100.0,19.0,True,False,Fake,
,Paintings,100.0,19.0,True,False,,Report #9822
,Paintings,100.0,19.0,True,False,,
,Paintings,100.0,19.0,False,True,Genuine,Report #9823
,Paintings,100.0,19.0,False,True,Genuine,
,Paintings,100.0,19.0,False,True,Fake,Report #9824
,Paintings,100.0,19.0,False,True,Fake,
,Paintings,100.0,19.0,False,True,,Report #9825
,Paintings,100.0,19.0,False,True,,
,Paintings,100.0,19.0,False,False,Genuine,Report #9826
,Paintings,100.0,19.0,False,False,Genuine,
,Paintings,100.0,19.0,False,False,Fake,Report #9827
,Paintings,100.0,19.0,False,False,Fake,
,Paintings,100.0,19.0,False,False,,Report #9828
,Paintings,100.0,19.0,False,False,,
,Paintings,100.0,20.0,True,True,Genuine,Report #9829
,Paintings,100.0,20.0,True,True,Genuine,
,Paintings,100.0,20.0,True,True,Fake,Report #9830
,Paintings,100.0,20.0,True,True,Fake,
,Paintings,100.0,20.0,True,True,,Report #9831
,Paintings,100.0,20.0,True,True,,
,Paintings,100.0,20.0,True,False,Genuine,Report #9832
,Paintings,100.0,20.0,True,False,Genuine,
,Paintings,100.0,20.0,True,False,Fake,Report #9833
,Paintings,100.0,20.0,True,False,Fake,
,Paintings,100.0,20.0,True,False,,Report #9834
,Paintings,100.0,20.0,True,False,,
,Paintings,100.0,20.0,False,True,Genuine,Report #9835
,Paintings,100.0,20.0,False,True,Genuine,
,Paintings,100.0,20.0,False,True,Fake,Report #9836
,Paintings,100.0,20.0,False,True,Fake,
,Paintings,100.0,20.0,False,True,,Report #9837
,Paintings,100.0,20.0,False,True,,
,Paintings,100.0,20.0,False,False,Genuine,Report #9838
,Paintings,100.0,20.0,False,False,Genuine,
,Paintings,100.0,20.0,False,False,Fake,Report #9839
,Paintings,100.0,20.0,False,False,Fake,
,Paintings,100.0,20.0,False,False,,Report #9840
,Paintings,100.0,20.0,False,False,,
,Paintings,100.0,21.0,True,True,Genuine,Report #9841
,Paintings,100.0,21.0,True,True,Genuine,
,Paintings,100.0,21.0,True,True,Fake,Report #9842
,Paintings,100.0,21.0,True,True,Fake,
,Paintings,100.0,21.0,True,True,,Report #9843
,Paintings,100.0,21.0,True,True,,
,Paintings,100.0,21.0,True,False,Genuine,Report #9844
,Paintings,100.0,21.0,True,False,Genuine,
,Paintings,100.0,21.0,True,False,Fake,Report #9845
,Paintings,100.0,21.0,True,False,Fake,
,Paintings,100.0,21.0,True,False,,Report #9846
,Paintings,100.0,21.0,True,False,,
,Paintings,100.0,21.0,False,True,Genuine,Report #9847
,Paintings,100.0,21.0,False,True,Genuine,
,Paintings,100.0,21.0,False,True,Fake,Report #9848
,Paintings,100.0,21.0,False,True,Fake,
,Paintings,100.0,21.0,False,True,,Report #9849
,Paintings,100.0,21.0,False,True,,
,Paintings,100.0,21.0,False,False,Genuine,Report #9850
,Paintings,100.0,21.0,False,False,Genuine,
,Paintings,100.0,21.0,False,False,Fake,Report #9851
,Paintings,100.0,21.0,False,False,Fake,
,Paintings,100.0,21.0,False,False,,Report #9852
,Paintings,100.0,21.0,False,False,,
,Paintings,100.0,99.0,True,True,Genuine,Report #9853
,Paintings,100.0,99.0,True,True,Genuine,
,Paintings,100.0,99.0,True,True,Fake,Report #9854
,Paintings,100.0,99.0,True,True,Fake,
,Paintings,100.0,99.0,True,True,,Report #9855
,Paintings,100.0,99.0,True,True,,
,Paintings,100.0,99.0,True,False,Genuine,Report #9856
,Paintings,100.0,99.0,True,False,Genuine,
,Paintings,100.0,99.0,True,False,Fake,Report #9857
,Paintings,100.0,99.0,True,False,Fake,
,Paintings,100.0,99.0,True,False,,Report #9858
,Paintings,100.0,99.0,True,False,,
,Paintings,100.0,99.0,False,True,Genuine,Report #9859
,Paintings,100.0,99.0,False,True,Genuine,
,Paintings,100.0,99.0,False,True,Fake,Report #9860
,Paintings,100.0,99.0,False,True,Fake,
,Paintings,100.0,99.0,False,True,,Report #9861
,Paintings,100.0,99.0,False,True,,
,Paintings,100.0,99.0,False,False,Genuine,Report #9862
,Paintings,100.0,99.0,False,False,Genuine,
,Paintings,100.0,99.0,False,False,Fake,Report #9863
,Paintings,100.0,99.0,False,False,Fake,
,Paintings,100.0,99.0,False,False,,Report #9864
,Paintings,100.0,99.0,False,False,,
,Paintings,100.0,100.0,True,True,Genuine,Report #9865
,Paintings,100.0,100.0,True,True,Genuine,
,Paintings,100.0,100.0,True,True,Fake,Report #9866
,Paintings,100.0,100.0,True,True,Fake,
,Paintings,100.0,100.0,True,True,,Report #9867
,Paintings,100.0,100.0,True,True,,
,Paintings,100.0,100.0,True,False,Genuine,Report #9868
,Paintings,100.0,100.0,True,False,Genuine,
,Paintings,100.0,100.0,True,False,Fake,Report #9869
,Paintings,100.0,100.0,True,False,Fake,
,Paintings,100.0,100.0,True,False,,Report #9870
,Paintings,100.0,100.0,True,False,,
,Paintings,100.0,100.0,False,True,Genuine,Report #9871
,Paintings,100.0,100.0,False,True,Genuine,
,Paintings,100.0,100.0,False,True,Fake,Report #9872
,Paintings,100.0,100.0,False,True,Fake,
,Paintings,100.0,100.0,False,True,,Report #9873
,Paintings,100.0,100.0,False,True,,
,Paintings,100.0,100.0,False,False,Genuine,Report #9874
,Paintings,100.0,100.0,False,False,Genuine,
,Paintings,100.0,100.0,False,False,Fake,Report #9875
,Paintings,100.0,100.0,False,False,Fake,
,Paintings,100.0,100.0,False,False,,Report #9876
,Paintings,100.0,100.0,False,False,,
,Paintings,100.0,101.0,True,True,Genuine,Report #9877
,Paintings,100.0,101.0,True,True,Genuine,
,Paintings,100.0,101.0,True,True,Fake,Report #9878
,Paintings,100.0,101.0,True,True,Fake,
,Paintings,100.0,101.0,True,True,,Report #9879
,Paintings,100.0,101.0,True,True,,
,Paintings,100.0,101.0,True,False,Genuine,Report #9880
,Paintings,100.0,101.0,True,False,Genuine,
,Paintings,100.0,101.0,True,False,Fake,Report #9881
,Paintings,100.0,101.0,True,False,Fake,
,Paintings,100.0,101.0,True,False,,Report #9882
,Paintings,100.0,101.0,True,False,,
,Paintings,100.0,101.0,False,True,Genuine,Report #9883
,Paintings,100.0,101.0,False,True,Genuine,
,Paintings,100.0,101.0,False,True,Fake,Report #9884
,Paintings,100.0,101.0,False,True,Fake,
,Paintings,100.0,101.0,False,True,,Report #9885
,Paintings,100.0,101.0,False,True,,
,Paintings,100.0,101.0,False,False,Genuine,Report #9886
,Paintings,100.0,101.0,False,False,Genuine,
,Paintings,100.0,101.0,False,False,Fake,Report #9887
,Paintings,100.0,101.0,False,False,Fake,
,Paintings,100.0,101.0,False,False,,Report #9888
,Paintings,100.0,101.0,False,False,,
,Paintings,1000.0,-1.0,True,True,Genuine,Report #9889
,Paintings,1000.0,-1.0,True,True,Genuine,
,Paintings,1000.0,-1.0,True,True,Fake,Report #9890
,Paintings,1000.0,-1.0,True,True,Fake,
,Paintings,1000.0,-1.0,True,True,,Report #9891
,Paintings,1000.0,-1.0,True,True,,
,Paintings,1000.0,-1.0,True,False,Genuine,Report #9892
,Paintings,1000.0,-1.0,True,False,Genuine,
,Paintings,1000.0,-1.0,True,False,Fake,Report #9893
,Paintings,1000.0,-1.0,True,False,Fake,
,Paintings,1000.0,-1.0,True,False,,Report #9894
,Paintings,1000.0,-1.0,True,False,,
,Paintings,1000.0,-1.0,False,True,Genuine,Report #9895
,Paintings,1000.0,-1.0,False,True,Genuine,
,Paintings,1000.0,-1.0,False,True,Fake,Report #9896
,Paintings,1000.0,-1.0,False,True,Fake,
,Paintings,1000.0,-1.0,False,True,,Report #9897
,Paintings,1000.0,-1.0,False,True,,
,Paintings,1000.0,-1.0,False,False,Genuine,Report #9898
,Paintings,1000.0,-1.0,False,False,Genuine,
,Paintings,1000.0,-1.0,False,False,Fake,Report #9899
,Paintings,1000.0,-1.0,False,False,Fake,
,Paintings,1000.0,-1.0,False,False,,Report #9900
,Paintings,1000.0,-1.0,False,False,,
,Paintings,1000.0,0.0,True,True,Genuine,Report #9901
,Paintings,1000.0,0.0,True,True,Genuine,
,Paintings,1000.0,0.0,True,True,Fake,Report #9902
,Paintings,1000.0,0.0,True,True,Fake,
,Paintings,1000.0,0.0,True,True,,Report #9903
,Paintings,1000.0,0.0,True,True,,
,Paintings,1000.0,0.0,True,False,Genuine,Report #9904
,Paintings,1000.0,0.0,True,False,Genuine,
,Paintings,1000.0,0.0,True,False,Fake,Report #9905
,Paintings,1000.0,0.0,True,False,Fake,
,Paintings,1000.0,0.0,True,False,,Report #9906
,Paintings,1000.0,0.0,True,False,,
,Paintings,1000.0,0.0,False,True,Genuine,Report #9907
,Paintings,1000.0,0.0,False,True,Genuine,
,Paintings,1000.0,0.0,False,True,Fake,Report #9908
,Paintings,1000.0,0.0,False,True,Fake,
,Paintings,1000.0,0.0,False,True,,Report #9909
,Paintings,1000.0,0.0,False,True,,
,Paintings,1000.0,0.0,False,False,Genuine,Report #9910
,Paintings,1000.0,0.0,False,False,Genuine,
,Paintings,1000.0,0.0,False,False,Fake,Report #9911
,Paintings,1000.0,0.0,False,False,Fake,
,Paintings,1000.0,0.0,False,False,,Report #9912
,Paintings,1000.0,0.0,False,False,,
,Paintings,1000.0,19.0,True,True,Genuine,Report #9913
,Paintings,1000.0,19.0,True,True,Genuine,
,Paintings,1000.0,19.0,True,True,Fake,Report #9914
,Paintings,1000.0,19.0,True,True,Fake,
,Paintings,1000.0,19.0,True,True,,Report #9915
,Paintings,1000.0,19.0,True,True,,
,Paintings,1000.0,19.0,True,False,Genuine,Report #9916
,Paintings,1000.0,19.0,True,False,Genuine,
,Paintings,1000.0,19.0,True,False,Fake,Report #9917
,Paintings,1000.0,19.0,True,False,Fake,
,Paintings,1000.0,19.0,True,False,,Report #9918
,Paintings,1000.0,19.0,True,False,,
,Paintings,1000.0,19.0,False,True,Genuine,Report #9919
,Paintings,1000.0,19.0,False,True,Genuine,
,Paintings,1000.0,19.0,False,True,Fake,Report #9920
,Paintings,1000.0,19.0,False,True,Fake,
,Paintings,1000.0,19.0,False,True,,Report #9921
,Paintings,1000.0,19.0,False,True,,
,Paintings,1000.0,19.0,False,False,Genuine,Report #9922
,Paintings,1000.0,19.0,False,False,Genuine,
,Paintings,1000.0,19.0,False,False,Fake,Report #9923
,Paintings,1000.0,19.0,False,False,Fake,
,Paintings,1000.0,19.0,False,False,,Report #9924
,Paintings,1000.0,19.0,False,False,,
,Paintings,1000.0,20.0,True,True,Genuine,Report #9925
,Paintings,1000.0,20.0,True,True,Genuine,
,Paintings,1000.0,20.0,True,True,Fake,Report #9926
,Paintings,1000.0,20.0,True,True,Fake,
,Paintings,1000.0,20.0,True,True,,Report #9927
,Paintings,1000.0,20.0,True,True,,
,Paintings,1000.0,20.0,True,False,Genuine,Report #9928
,Paintings,1000.0,20.0,True,False,Genuine,
,Paintings,1000.0,20.0,True,False,Fake,Report #9929
,Paintings,1000.0,20.0,True,False,Fake,
,Paintings,1000.0,20.0,True,False,,Report #9930
,Paintings,1000.0,20.0,True,False,,
,Paintings,1000.0,20.0,False,True,Genuine,Report #9931
,Paintings,1000.0,20.0,False,True,Genuine,
,Paintings,1000.0,20.0,False,True,Fake,Report #9932
,Paintings,1000.0,20.0,False,True,Fake,
,Paintings,1000.0,20.0,False,True,,Report #9933
,Paintings,1000.0,20.0,False,True,,
,Paintings,1000.0,20.0,False,False,Genuine,Report #9934
,Paintings,1000.0,20.0,False,False,Genuine,
,Paintings,1000.0,20.0,False,False,Fake,Report #9935
,Paintings,1000.0,20.0,False,False,Fake,
,Paintings,1000.0,20.0,False,False,,Report #9936
,Paintings,1000.0,20.0,False,False,,
,Paintings,1000.0,21.0,True,True,Genuine,Report #9937
,Paintings,1000.0,21.0,True,True,Genuine,
,Paintings,1000.0,21.0,True,True,Fake,Report #9938
,Paintings,1000.0,21.0,True,True,Fake,
,Paintings,1000.0,21.0,True,True,,Report #9939
,Paintings,1000.0,21.0,True,True,,
,Paintings,1000.0,21.0,True,False,Genuine,Report #9940
,Paintings,1000.0,21.0,True,False,Genuine,
,Paintings,1000.0,21.0,True,False,Fake,Report #9941
,Paintings,1000.0,21.0,True,False,Fake,
,Paintings,1000.0,21.0,True,False,,Report #9942
,Paintings,1000.0,21.0,True,False,,
,Paintings,1000.0,21.0,False,True,Genuine,Report #9943
,Paintings,1000.0,21.0,False,True,Genuine,
,Paintings,1000.0,21.0,False,True,Fake,Report #9944
,Paintings,1000.0,21.0,False,True,Fake,
,Paintings,1000.0,21.0,False,True,,Report #9945
,Paintings,1000.0,21.0,False,True,,
,Paintings,1000.0,21.0,False,False,Genuine,Report #9946
,Paintings,1000.0,21.0,False,False,Genuine,
,Paintings,1000.0,21.0,False,False,Fake,Report #9947
,Paintings,1000.0,21.0,False,False,Fake,
,Paintings,1000.0,21.0,False,False,,Report #9948
,Paintings,1000.0,21.0,False,False,,
,Paintings,1000.0,99.0,True,True,Genuine,Report #9949
,Paintings,1000.0,99.0,True,True,Genuine,
,Paintings,1000.0,99.0,True,True,Fake,Report #9950
,Paintings,1000.0,99.0,True,True,Fake,
,Paintings,1000.0,99.0,True,True,,Report #9951
,Paintings,1000.0,99.0,True,True,,
,Paintings,1000.0,99.0,True,False,Genuine,Report #9952
,Paintings,1000.0,99.0,True,False,Genuine,
,Paintings,1000.0,99.0,True,False,Fake,Report #9953
,Paintings,1000.0,99.0,True,False,Fake,
,Paintings,1000.0,99.0,True,False,,Report #9954
,Paintings,1000.0,99.0,True,False,,
,Paintings,1000.0,99.0,False,True,Genuine,Report #9955
,Paintings,1000.0,99.0,False,True,Genuine,
,Paintings,1000.0,99.0,False,True,Fake,Report #9956
,Paintings,1000.0,99.0,False,True,Fake,
,Paintings,1000.0,99.0,False,True,,Report #9957
,Paintings,1000.0,99.0,False,True,,
,Paintings,1000.0,99.0,False,False,Genuine,Report #9958
,Paintings,1000.0,99.0,False,False,Genuine,
,Paintings,1000.0,99.0,False,False,Fake,Report #9959
,Paintings,1000.0,99.0,False,False,Fake,
,Paintings,1000.0,99.0,False,False,,Report #9960
,Paintings,1000.0,99.0,False,False,,
,Paintings,1000.0,100.0,True,True,Genuine,Report #9961
,Paintings,1000.0,100.0,True,True,Genuine,
,Paintings,1000.0,100.0,True,True,Fake,Report #9962
,Paintings,1000.0,100.0,True,True,Fake,
,Paintings,1000.0,100.0,True,True,,Report #9963
,Paintings,1000.0,100.0,True,True,,
,Paintings,1000.0,100.0,True,False,Genuine,Report #9964
,Paintings,1000.0,100.0,True,False,Genuine,
,Paintings,1000.0,100.0,True,False,Fake,Report #9965
,Paintings,1000.0,100.0,True,False,Fake,
,Paintings,1000.0,100.0,True,False,,Report #9966
,Paintings,1000.0,100.0,True,False,,
,Paintings,1000.0,100.0,False,True,Genuine,Report #9967
,Paintings,1000.0,100.0,False,True,Genuine,
,Paintings,1000.0,100.0,False,True,Fake,Report #9968
,Paintings,1000.0,100.0,False,True,Fake,
,Paintings,1000.0,100.0,False,True,,Report #9969
,Paintings,1000.0,100.0,False,True,,
,Paintings,1000.0,100.0,False,False,Genuine,Report #9970
,Paintings,1000.0,100.0,False,False,Genuine,
,Paintings,1000.0,100.0,False,False,Fake,Report #9971
,Paintings,1000.0,100.0,False,False,Fake,
,Paintings,1000.0,100.0,False,False,,Report #9972
,Paintings,1000.0,100.0,False,False,,
,Paintings,1000.0,101.0,True,True,Genuine,Report #9973
,Paintings,1000.0,101.0,True,True,Genuine,
,Paintings,1000.0,101.0,True,True,Fake,Report #9974
,Paintings,1000.0,101.0,True,True,Fake,
,Paintings,1000.0,101.0,True,True,,Report #9975
,Paintings,1000.0,101.0,True,True,,
,Paintings,1000.0,101.0,True,False,Genuine,Report #9976
,Paintings,1000.0,101.0,True,False,Genuine,
,Paintings,1000.0,101.0,True,False,Fake,Report #9977
,Paintings,1000.0,101.0,True,False,Fake,
,Paintings,1000.0,101.0,True,False,,Report #9978
,Paintings,1000.0,101.0,True,False,,
,Paintings,1000.0,101.0,False,True,Genuine,Report #9979
,Paintings,1000.0,101.0,False,True,Genuine,
,Paintings,1000.0,101.0,False,True,Fake,Report #9980
,Paintings,1000.0,101.0,False,True,Fake,
,Paintings,1000.0,101.0,False,True,,Report #9981
,Paintings,1000.0,101.0,False,True,,
,Paintings,1000.0,101.0,False,False,Genuine,Report #9982
,Paintings,1000.0,101.0,False,False,Genuine,
,Paintings,1000.0,101.0,False,False,Fake,Report #9983
,Paintings,1000.0,101.0,False,False,Fake,
,Paintings,1000.0,101.0,False,False,,Report #9984
,Paintings,1000.0,101.0,False,False,,
,Paintings,1000000.0,-1.0,True,True,Genuine,Report #9985
,Paintings,1000000.0,-1.0,True,True,Genuine,
,Paintings,1000000.0,-1.0,True,True,Fake,Report #9986
,Paintings,1000000.0,-1.0,True,True,Fake,
,Paintings,1000000.0,-1.0,True,True,,Report #9987
,Paintings,1000000.0,-1.0,True,True,,
,Paintings,1000000.0,-1.0,True,False,Genuine,Report #9988
,Paintings,1000000.0,-1.0,True,False,Genuine,
,Paintings,1000000.0,-1.0,True,False,Fake,Report #9989
,Paintings,1000000.0,-1.0,True,False,Fake,
,Paintings,1000000.0,-1.0,True,False,,Report #9990
,Paintings,1000000.0,-1.0,True,False,,
,Paintings,1000000.0,-1.0,False,True,Genuine,Report #9991
,Paintings,1000000.0,-1.0,False,True,Genuine,
,Paintings,1000000.0,-1.0,False,True,Fake,Report #9992
,Paintings,1000000.0,-1.0,False,True,Fake,
,Paintings,1000000.0,-1.0,False,True,,Report #9993
,Paintings,1000000.0,-1.0,False,True,,
,Paintings,1000000.0,-1.0,False,False,Genuine,Report #9994
,Paintings,1000000.0,-1.0,False,False,Genuine,
,Paintings,1000000.0,-1.0,False,False,Fake,Report #9995
,Paintings,1000000.0,-1.0,False,False,Fake,
,Paintings,1000000.0,-1.0,False,False,,Report #9996
,Paintings,1000000.0,-1.0,False,False,,
,Paintings,1000000.0,0.0,True,True,Genuine,Report #9997
,Paintings,1000000.0,0.0,True,True,Genuine,
,Paintings,1000000.0,0.0,True,True,Fake,Report #9998
,Paintings,1000000.0,0.0,True,True,Fake,
,Paintings,1000000.0,0.0,True,True,,Report #9999
,Paintings,1000000.0,0.0,True,True,,
,Paintings,1000000.0,0.0,True,False,Genuine,Report #10000
,Paintings,1000000.0,0.0,True,False,Genuine,
,Paintings,1000000.0,0.0,True,False,Fake,Report #10001
,Paintings,1000000.0,0.0,True,False,Fake,
,Paintings,1000000.0,0.0,True,False,,Report #10002
,Paintings,1000000.0,0.0,True,False,,
,Paintings,1000000.0,0.0,False,True,Genuine,Report #10003
,Paintings,1000000.0,0.0,False,True,Genuine,
,Paintings,1000000.0,0.0,False,True,Fake,Report #10004
,Paintings,1000000.0,0.0,False,True,Fake,
,Paintings,1000000.0,0.0,False,True,,Report #10005
,Paintings,1000000.0,0.0,False,True,,
,Paintings,1000000.0,0.0,False,False,Genuine,Report #10006
,Paintings,1000000.0,0.0,False,False,Genuine,
,Paintings,1000000.0,0.0,False,False,Fake,Report #10007
,Paintings,1000000.0,0.0,False,False,Fake,
,Paintings,1000000.0,0.0,False,False,,Report #10008
,Paintings,1000000.0,0.0,False,False,,
,Paintings,1000000.0,19.0,True,True,Genuine,Report #10009
,Paintings,1000000.0,19.0,True,True,Genuine,
,Paintings,1000000.0,19.0,True,True,Fake,Report #10010
,Paintings,1000000.0,19.0,True,True,Fake,
,Paintings,1000000.0,19.0,True,True,,Report #10011
,Paintings,1000000.0,19.0,True,True,,
,Paintings,1000000.0,19.0,True,False,Genuine,Report #10012
,Paintings,1000000.0,19.0,True,False,Genuine,
,Paintings,1000000.0,19.0,True,False,Fake,Report #10013
,Paintings,1000000.0,19.0,True,False,Fake,
,Paintings,1000000.0,19.0,True,False,,Report #10014
,Paintings,1000000.0,19.0,True,False,,
,Paintings,1000000.0,19.0,False,True,Genuine,Report #10015
,Paintings,1000000.0,19.0,False,True,Genuine,
,Paintings,1000000.0,19.0,False,True,Fake,Report #10016
,Paintings,1000000.0,19.0,False,True,Fake,
,Paintings,1000000.0,19.0,False,True,,Report #10017
,Paintings,1000000.0,19.0,False,True,,
,Paintings,1000000.0,19.0,False,False,Genuine,Report #10018
,Paintings,1000000.0,19.0,False,False,Genuine,
,Paintings,1000000.0,19.0,False,False,Fake,Report #10019
,Paintings,1000000.0,19.0,False,False,Fake,
,Paintings,1000000.0,19.0,False,False,,Report #10020
,Paintings,1000000.0,19.0,False,False,,
,Paintings,1000000.0,20.0,True,True,Genuine,Report #10021
,Paintings,1000000.0,20.0,True,True,Genuine,
,Paintings,1000000.0,20.0,True,True,Fake,Report #10022
,Paintings,1000000.0,20.0,True,True,Fake,
,Paintings,1000000.0,20.0,True,True,,Report #10023
,Paintings,1000000.0,20.0,True,True,,
,Paintings,1000000.0,20.0,True,False,Genuine,Report #10024
,Paintings,1000000.0,20.0,True,False,Genuine,
,Paintings,1000000.0,20.0,True,False,Fake,Report #10025
,Paintings,1000000.0,20.0,True,False,Fake,
,Paintings,1000000.0,20.0,True,False,,Report #10026
,Paintings,1000000.0,20.0,True,False,,
,Paintings,1000000.0,20.0,False,True,Genuine,Report #10027
,Paintings,1000000.0,20.0,False,True,Genuine,
,Paintings,1000000.0,20.0,False,True,Fake,Report #10028
,Paintings,1000000.0,20.0,False,True,Fake,
,Paintings,1000000.0,20.0,False,True,,Report #10029
,Paintings,1000000.0,20.0,False,True,,
,Paintings,1000000.0,20.0,False,False,Genuine,Report #10030
,Paintings,1000000.0,20.0,False,False,Genuine,
,Paintings,1000000.0,20.0,False,False,Fake,Report #10031
,Paintings,1000000.0,20.0,False,False,Fake,
,Paintings,1000000.0,20.0,False,False,,Report #10032
,Paintings,1000000.0,20.0,False,False,,
,Paintings,1000000.0,21.0,True,True,Genuine,Report #10033
,Paintings,1000000.0,21.0,True,True,Genuine,
,Paintings,1000000.0,21.0,True,True,Fake,Report #10034
,Paintings,1000000.0,21.0,True,True,Fake,
,Paintings,1000000.0,21.0,True,True,,Report #10035
,Paintings,1000000.0,21.0,True,True,,
,Paintings,1000000.0,21.0,True,False,Genuine,Report #10036
,Paintings,1000000.0,21.0,True,False,Genuine,
,Paintings,1000000.0,21.0,True,False,Fake,Report #10037
,Paintings,1000000.0,21.0,True,False,Fake,
,Paintings,1000000.0,21.0,True,False,,Report #10038
,Paintings,1000000.0,21.0,True,False,,
,Paintings,1000000.0,21.0,False,True,Genuine,Report #10039
,Paintings,1000000.0,21.0,False,True,Genuine,
,Paintings,1000000.0,21.0,False,True,Fake,Report #10040
,Paintings,1000000.0,21.0,False,True,Fake,
,Paintings,1000000.0,21.0,False,True,,Report #10041
,Paintings,1000000.0,21.0,False,True,,
,Paintings,1000000.0,21.0,False,False,Genuine,Report #10042
,Paintings,1000000.0,21.0,False,False,Genuine,
,Paintings,1000000.0,21.0,False,False,Fake,Report #10043
,Paintings,1000000.0,21.0,False,False,Fake,
,Paintings,1000000.0,21.0,False,False,,Report #10044
,Paintings,1000000.0,21.0,False,False,,
,Paintings,1000000.0,99.0,True,True,Genuine,Report #10045
,Paintings,1000000.0,99.0,True,True,Genuine,
,Paintings,1000000.0,99.0,True,True,Fake,Report #10046
,Paintings,1000000.0,99.0,True,True,Fake,
,Paintings,1000000.0,99.0,True,True,,Report #10047
,Paintings,1000000.0,99.0,True,True,,
,Paintings,1000000.0,99.0,True,False,Genuine,Report #10048
,Paintings,1000000.0,99.0,True,False,Genuine,
,Paintings,1000000.0,99.0,True,False,Fake,Report #10049
,Paintings,1000000.0,99.0,True,False,Fake,
,Paintings,1000000.0,99.0,True,False,,Report #10050
,Paintings,1000000.0,99.0,True,False,,
,Paintings,1000000.0,99.0,False,True,Genuine,Report #10051
,Paintings,1000000.0,99.0,False,True,Genuine,
,Paintings,1000000.0,99.0,False,True,Fake,Report #10052
,Paintings,1000000.0,99.0,False,True,Fake,
,Paintings,1000000.0,99.0,False,True,,Report #10053
,Paintings,1000000.0,99.0,False,True,,
,Paintings,1000000.0,99.0,False,False,Genuine,Report #10054
,Paintings,1000000.0,99.0,False,False,Genuine,
,Paintings,1000000.0,99.0,False,False,Fake,Report #10055
,Paintings,1000000.0,99.0,False,False,Fake,
,Paintings,1000000.0,99.0,False,False,,Report #10056
,Paintings,1000000.0,99.0,False,False,,
,Paintings,1000000.0,100.0,True,True,Genuine,Report #10057
,Paintings,1000000.0,100.0,True,True,Genuine,
,Paintings,1000000.0,100.0,True,True,Fake,Report #10058
,Paintings,1000000.0,100.0,True,True,Fake,
,Paintings,1000000.0,100.0,True,True,,Report #10059
,Paintings,1000000.0,100.0,True,True,,
,Paintings,1000000.0,100.0,True,False,Genuine,Report #10060
,Paintings,1000000.0,100.0,True,False,Genuine,
,Paintings,1000000.0,100.0,True,False,Fake,Report #10061
,Paintings,1000000.0,100.0,True,False,Fake,
,Paintings,1000000.0,100.0,True,False,,Report #10062
,Paintings,1000000.0,100.0,True,False,,
,Paintings,1000000.0,100.0,False,True,Genuine,Report #10063
,Paintings,1000000.0,100.0,False,True,Genuine,
,Paintings,1000000.0,100.0,False,True,Fake,Report #10064
,Paintings,1000000.0,100.0,False,True,Fake,
,Paintings,1000000.0,100.0,False,True,,Report #10065
,Paintings,1000000.0,100.0,False,True,,
,Paintings,1000000.0,100.0,False,False,Genuine,Report #10066
,Paintings,1000000.0,100.0,False,False,Genuine,
,Paintings,1000000.0,100.0,False,False,Fake,Report #10067
,Paintings,1000000.0,100.0,False,False,Fake,
,Paintings,1000000.0,100.0,False,False,,Report #10068
,Paintings,1000000.0,100.0,False,False,,
,Paintings,1000000.0,101.0,True,True,Genuine,Report #10069
,Paintings,1000000.0,101.0,True,True,Genuine,
,Paintings,1000000.0,101.0,True,True,Fake,Report #10070
,Paintings,1000000.0,101.0,True,True,Fake,
,Paintings,1000000.0,101.0,True,True,,Report #10071
,Paintings,1000000.0,101.0,True,True,,
,Paintings,1000000.0,101.0,True,False,Genuine,Report #10072
,Paintings,1000000.0,101.0,True,False,Genuine,
,Paintings,1000000.0,101.0,True,False,Fake,Report #10073
,Paintings,1000000.0,101.0,True,False,Fake,
,Paintings,1000000.0,101.0,True,False,,Report #10074
,Paintings,1000000.0,101.0,True,False,,
,Paintings,1000000.0,101.0,False,True,Genuine,Report #10075
,Paintings,1000000.0,101.0,False,True,Genuine,
,Paintings,1000000.0,101.0,False,True,Fake,Report #10076
,Paintings,1000000.0,101.0,False,True,Fake,
,Paintings,1000000.0,101.0,False,True,,Report #10077
,Paintings,1000000.0,101.0,False,True,,
,Paintings,1000000.0,101.0,False,False,Genuine,Report #10078
,Paintings,1000000.0,101.0,False,False,Genuine,
,Paintings,1000000.0,101.0,False,False,Fake,Report #10079
,Paintings,1000000.0,101.0,False,False,Fake,
,Paintings,1000000.0,101.0,False,False,,Report #10080
,Paintings,1000000.0,101.0,False,False,,
,Knife,0.0,-1.0,True,True,Genuine,Report #10081
,Knife,0.0,-1.0,True,True,Genuine,
,Knife,0.0,-1.0,True,True,Fake,Report #10082
,Knife,0.0,-1.0,True,True,Fake,
,Knife,0.0,-1.0,True,True,,Report #10083
,Knife,0.0,-1.0,True,True,,
,Knife,0.0,-1.0,True,False,Genuine,Report #10084
,Knife,0.0,-1.0,True,False,Genuine,
,Knife,0.0,-1.0,True,False,Fake,Report #10085
,Knife,0.0,-1.0,True,False,Fake,
,Knife,0.0,-1.0,True,False,,Report #10086
,Knife,0.0,-1.0,True,False,,
,Knife,0.0,-1.0,False,True,Genuine,Report #10087
,Knife,0.0,-1.0,False,True,Genuine,
,Knife,0.0,-1.0,False,True,Fake,Report #10088
,Knife,0.0,-1.0,False,True,Fake,
,Knife,0.0,-1.0,False,True,,Report #10089
,Knife,0.0,-1.0,False,True,,
,Knife,0.0,-1.0,False,False,Genuine,Report #10090
,Knife,0.0,-1.0,False,False,Genuine,
,Knife,0.0,-1.0,False,False,Fake,Report #10091
,Knife,0.0,-1.0,False,False,Fake,
,Knife,0.0,-1.0,False,False,,Report #10092
,Knife,0.0,-1.0,False,False,,
,Knife,0.0,0.0,True,True,Genuine,Report #10093
,Knife,0.0,0.0,True,True,Genuine,
,Knife,0.0,0.0,True,True,Fake,Report #10094
,Knife,0.0,0.0,True,True,Fake,
,Knife,0.0,0.0,True,True,,Report #10095
,Knife,0.0,0.0,True,True,,
,Knife,0.0,0.0,True,False,Genuine,Report #10096
,Knife,0.0,0.0,True,False,Genuine,
,Knife,0.0,0.0,True,False,Fake,Report #10097
,Knife,0.0,0.0,True,False,Fake,
,Knife,0.0,0.0,True,False,,Report #10098
,Knife,0.0,0.0,True,False,,
,Knife,0.0,0.0,False,True,Genuine,Report #10099
,Knife,0.0,0.0,False,True,Genuine,
,Knife,0.0,0.0,False,True,Fake,Report #10100
,Knife,0.0,0.0,False,True,Fake,
,Knife,0.0,0.0,False,True,,Report #10101
,Knife,0.0,0.0,False,True,,
,Knife,0.0,0.0,False,False,Genuine,Report #10102
,Knife,0.0,0.0,False,False,Genuine,
,Knife,0.0,0.0,False,False,Fake,Report #10103
,Knife,0.0,0.0,False,False,Fake,
,Knife,0.0,0.0,False,False,,Report #10104
,Knife,0.0,0.0,False,False,,
,Knife,0.0,19.0,True,True,Genuine,Report #10105
,Knife,0.0,19.0,True,True,Genuine,
,Knife,0.0,19.0,True,True,Fake,Report #10106
,Knife,0.0,19.0,True,True,Fake,
,Knife,0.0,19.0,True,True,,Report #10107
,Knife,0.0,19.0,True,True,,
,Knife,0.0,19.0,True,False,Genuine,Report #10108
,Knife,0.0,19.0,True,False,Genuine,
,Knife,0.0,19.0,True,False,Fake,Report #10109
,Knife,0.0,19.0,True,False,Fake,
,Knife,0.0,19.0,True,False,,Report #10110
,Knife,0.0,19.0,True,False,,
,Knife,0.0,19.0,False,True,Genuine,Report #10111
,Knife,0.0,19.0,False,True,Genuine,
,Knife,0.0,19.0,False,True,Fake,Report #10112
,Knife,0.0,19.0,False,True,Fake,
,Knife,0.0,19.0,False,True,,Report #10113
,Knife,0.0,19.0,False,True,,
,Knife,0.0,19.0,False,False,Genuine,Report #10114
,Knife,0.0,19.0,False,False,Genuine,
,Knife,0.0,19.0,False,False,Fake,Report #10115
,Knife,0.0,19.0,False,False,Fake,
,Knife,0.0,19.0,False,False,,Report #10116
,Knife,0.0,19.0,False,False,,
,Knife,0.0,20.0,True,True,Genuine,Report #10117
,Knife,0.0,20.0,True,True,Genuine,
,Knife,0.0,20.0,True,True,Fake,Report #10118
,Knife,0.0,20.0,True,True,Fake,
,Knife,0.0,20.0,True,True,,Report #10119
,Knife,0.0,20.0,True,True,,
,Knife,0.0,20.0,True,False,Genuine,Report #10120
,Knife,0.0,20.0,True,False,Genuine,
,Knife,0.0,20.0,True,False,Fake,Report #10121
,Knife,0.0,20.0,True,False,Fake,
,Knife,0.0,20.0,True,False,,Report #10122
,Knife,0.0,20.0,True,False,,
,Knife,0.0,20.0,False,True,Genuine,Report #10123
,Knife,0.0,20.0,False,True,Genuine,
,Knife,0.0,20.0,False,True,Fake,Report #10124
,Knife,0.0,20.0,False,True,Fake,
,Knife,0.0,20.0,False,True,,Report #10125
,Knife,0.0,20.0,False,True,,
,Knife,0.0,20.0,False,False,Genuine,Report #10126
,Knife,0.0,20.0,False,False,Genuine,
,Knife,0.0,20.0,False,False,Fake,Report #10127
,Knife,0.0,20.0,False,False,Fake,
,Knife,0.0,20.0,False,False,,Report #10128
,Knife,0.0,20.0,False,False,,
,Knife,0.0,21.0,True,True,Genuine,Report #10129
,Knife,0.0,21.0,True,True,Genuine,
,Knife,0.0,21.0,True,True,Fake,Report #10130
,Knife,0.0,21.0,True,True,Fake,
,Knife,0.0,21.0,True,True,,Report #10131
,Knife,0.0,21.0,True,True,,
,Knife,0.0,21.0,True,False,Genuine,Report #10132
,Knife,0.0,21.0,True,False,Genuine,
,Knife,0.0,21.0,True,False,Fake,Report #10133
,Knife,0.0,21.0,True,False,Fake,
,Knife,0.0,21.0,True,False,,Report #10134
,Knife,0.0,21.0,True,False,,
,Knife,0.0,21.0,False,True,Genuine,Report #10135
,Knife,0.0,21.0,False,True,Genuine,
,Knife,0.0,21.0,False,True,Fake,Report #10136
,Knife,0.0,21.0,False,True,Fake,
,Knife,0.0,21.0,False,True,,Report #10137
,Knife,0.0,21.0,False,True,,
,Knife,0.0,21.0,False,False,Genuine,Report #10138
,Knife,0.0,21.0,False,False,Genuine,
,Knife,0.0,21.0,False,False,Fake,Report #10139
,Knife,0.0,21.0,False,False,Fake,
,Knife,0.0,21.0,False,False,,Report #10140
,Knife,0.0,21.0,False,False,,
,Knife,0.0,99.0,True,True,Genuine,Report #10141
,Knife,0.0,99.0,True,True,Genuine,
,Knife,0.0,99.0,True,True,Fake,Report #10142
,Knife,0.0,99.0,True,True,Fake,
,Knife,0.0,99.0,True,True,,Report #10143
,Knife,0.0,99.0,True,True,,
,Knife,0.0,99.0,True,False,Genuine,Report #10144
,Knife,0.0,99.0,True,False,Genuine,
,Knife,0.0,99.0,True,False,Fake,Report #10145
,Knife,0.0,99.0,True,False,Fake,
,Knife,0.0,99.0,True,False,,Report #10146
,Knife,0.0,99.0,True,False,,
,Knife,0.0,99.0,False,True,Genuine,Report #10147
,Knife,0.0,99.0,False,True,Genuine,
,Knife,0.0,99.0,False,True,Fake,Report #10148
,Knife,0.0,99.0,False,True,Fake,
,Knife,0.0,99.0,False,True,,Report #10149
,Knife,0.0,99.0,False,True,,
,Knife,0.0,99.0,False,False,Genuine,Report #10150
,Knife,0.0,99.0,False,False,Genuine,
,Knife,0.0,99.0,False,False,Fake,Report #10151
,Knife,0.0,99.0,False,False,Fake,
,Knife,0.0,99.0,False,False,,Report #10152
,Knife,0.0,99.0,False,False,,
,Knife,0.0,100.0,True,True,Genuine,Report #10153
,Knife,0.0,100.0,True,True,Genuine,
,Knife,0.0,100.0,True,True,Fake,Report #10154
,Knife,0.0,100.0,True,True,Fake,
,Knife,0.0,100.0,True,True,,Report #10155
,Knife,0.0,100.0,True,True,,
,Knife,0.0,100.0,True,False,Genuine,Report #10156
,Knife,0.0,100.0,True,False,Genuine,
,Knife,0.0,100.0,True,False,Fake,Report #10157
,Knife,0.0,100.0,True,False,Fake,
,Knife,0.0,100.0,True,False,,Report #10158
,Knife,0.0,100.0,True,False,,
,Knife,0.0,100.0,False,True,Genuine,Report #10159
,Knife,0.0,100.0,False,True,Genuine,
,Knife,0.0,100.0,False,True,Fake,Report #10160
,Knife,0.0,100.0,False,True,Fake,
,Knife,0.0,100.0,False,True,,Report #10161
,Knife,0.0,100.0,False,True,,
,Knife,0.0,100.0,False,False,Genuine,Report #10162
,Knife,0.0,100.0,False,False,Genuine,
,Knife,0.0,100.0,False,False,Fake,Report #10163
,Knife,0.0,100.0,False,False,Fake,
,Knife,0.0,100.0,False,False,,Report #10164
,Knife,0.0,100.0,False,False,,
,Knife,0.0,101.0,True,True,Genuine,Report #10165
,Knife,0.0,101.0,True,True,Genuine,
,Knife,0.0,101.0,True,True,Fake,Report #10166
,Knife,0.0,101.0,True,True,Fake,
,Knife,0.0,101.0,True,True,,Report #10167
,Knife,0.0,101.0,True,True,,
,Knife,0.0,101.0,True,False,Genuine,Report #10168
,Knife,0.0,101.0,True,False,Genuine,
,Knife,0.0,101.0,True,False,Fake,Report #10169
,Knife,0.0,101.0,True,False,Fake,
,Knife,0.0,101.0,True,False,,Report #10170
,Knife,0.0,101.0,True,False,,
,Knife,0.0,101.0,False,True,Genuine,Report #10171
,Knife,0.0,101.0,False,True,Genuine,
,Knife,0.0,101.0,False,True,Fake,Report #10172
,Knife,0.0,101.0,False,True,Fake,
,Knife,0.0,101.0,False,True,,Report #10173
,Knife,0.0,101.0,False,True,,
,Knife,0.0,101.0,False,False,Genuine,Report #10174
,Knife,0.0,101.0,False,False,Genuine,
,Knife,0.0,101.0,False,False,Fake,Report #10175
,Knife,0.0,101.0,False,False,Fake,
,Knife,0.0,101.0,False,False,,Report #10176
,Knife,0.0,101.0,False,False,,
,Knife,1.0,-1.0,True,True,Genuine,Report #10177
,Knife,1.0,-1.0,True,True,Genuine,
,Knife,1.0,-1.0,True,True,Fake,Report #10178
,Knife,1.0,-1.0,True,True,Fake,
,Knife,1.0,-1.0,True,True,,Report #10179
,Knife,1.0,-1.0,True,True,,
,Knife,1.0,-1.0,True,False,Genuine,Report #10180
,Knife,1.0,-1.0,True,False,Genuine,
,Knife,1.0,-1.0,True,False,Fake,Report #10181
,Knife,1.0,-1.0,True,False,Fake,
,Knife,1.0,-1.0,True,False,,Report #10182
,Knife,1.0,-1.0,True,False,,
,Knife,1.0,-1.0,False,True,Genuine,Report #10183
,Knife,1.0,-1.0,False,True,Genuine,
,Knife,1.0,-1.0,False,True,Fake,Report #10184
,Knife,1.0,-1.0,False,True,Fake,
,Knife,1.0,-1.0,False,True,,Report #10185
,Knife,1.0,-1.0,False,True,,
,Knife,1.0,-1.0,False,False,Genuine,Report #10186
,Knife,1.0,-1.0,False,False,Genuine,
,Knife,1.0,-1.0,False,False,Fake,Report #10187
,Knife,1.0,-1.0,False,False,Fake,
,Knife,1.0,-1.0,False,False,,Report #10188
,Knife,1.0,-1.0,False,False,,
,Knife,1.0,0.0,True,True,Genuine,Report #10189
,Knife,1.0,0.0,True,True,Genuine,
,Knife,1.0,0.0,True,True,Fake,Report #10190
,Knife,1.0,0.0,True,True,Fake,
,Knife,1.0,0.0,True,True,,Report #10191
,Knife,1.0,0.0,True,True,,
,Knife,1.0,0.0,True,False,Genuine,Report #10192
,Knife,1.0,0.0,True,False,Genuine,
,Knife,1.0,0.0,True,False,Fake,Report #10193
,Knife,1.0,0.0,True,False,Fake,
,Knife,1.0,0.0,True,False,,Report #10194
,Knife,1.0,0.0,True,False,,
,Knife,1.0,0.0,False,True,Genuine,Report #10195
,Knife,1.0,0.0,False,True,Genuine,
,Knife,1.0,0.0,False,True,Fake,Report #10196
,Knife,1.0,0.0,False,True,Fake,
,Knife,1.0,0.0,False,True,,Report #10197
,Knife,1.0,0.0,False,True,,
,Knife,1.0,0.0,False,False,Genuine,Report #10198
,Knife,1.0,0.0,False,False,Genuine,
,Knife,1.0,0.0,False,False,Fake,Report #10199
,Knife,1.0,0.0,False,False,Fake,
,Knife,1.0,0.0,False,False,,Report #10200
,Knife,1.0,0.0,False,False,,
,Knife,1.0,19.0,True,True,Genuine,Report #10201
,Knife,1.0,19.0,True,True,Genuine,
,Knife,1.0,19.0,True,True,Fake,Report #10202
,Knife,1.0,19.0,True,True,Fake,
,Knife,1.0,19.0,True,True,,Report #10203
,Knife,1.0,19.0,True,True,,
,Knife,1.0,19.0,True,False,Genuine,Report #10204
,Knife,1.0,19.0,True,False,Genuine,
,Knife,1.0,19.0,True,False,Fake,Report #10205
,Knife,1.0,19.0,True,False,Fake,
,Knife,1.0,19.0,True,False,,Report #10206
,Knife,1.0,19.0,True,False,,
,Knife,1.0,19.0,False,True,Genuine,Report #10207
,Knife,1.0,19.0,False,True,Genuine,
,Knife,1.0,19.0,False,True,Fake,Report #10208
,Knife,1.0,19.0,False,True,Fake,
,Knife,1.0,19.0,False,True,,Report #10209
,Knife,1.0,19.0,False,True,,
,Knife,1.0,19.0,False,False,Genuine,Report #10210
,Knife,1.0,19.0,False,False,Genuine,
,Knife,1.0,19.0,False,False,Fake,Report #10211
,Knife,1.0,19.0,False,False,Fake,
,Knife,1.0,19.0,False,False,,Report #10212
,Knife,1.0,19.0,False,False,,
,Knife,1.0,20.0,True,True,Genuine,Report #10213
,Knife,1.0,20.0,True,True,Genuine,
,Knife,1.0,20.0,True,True,Fake,Report #10214
,Knife,1.0,20.0,True,True,Fake,
,Knife,1.0,20.0,True,True,,Report #10215
,Knife,1.0,20.0,True,True,,
,Knife,1.0,20.0,True,False,Genuine,Report #10216
,Knife,1.0,20.0,True,False,Genuine,
,Knife,1.0,20.0,True,False,Fake,Report #10217
,Knife,1.0,20.0,True,False,Fake,
,Knife,1.0,20.0,True,False,,Report #10218
,Knife,1.0,20.0,True,False,,
,Knife,1.0,20.0,False,True,Genuine,Report #10219
,Knife,1.0,20.0,False,True,Genuine,
,Knife,1.0,20.0,False,True,Fake,Report #10220
,Knife,1.0,20.0,False,True,Fake,
,Knife,1.0,20.0,False,True,,Report #10221
,Knife,1.0,20.0,False,True,,
,Knife,1.0,20.0,False,False,Genuine,Report #10222
,Knife,1.0,20.0,False,False,Genuine,
,Knife,1.0,20.0,False,False,Fake,Report #10223
,Knife,1.0,20.0,False,False,Fake,
,Knife,1.0,20.0,False,False,,Report #10224
,Knife,1.0,20.0,False,False,,
,Knife,1.0,21.0,True,True,Genuine,Report #10225
,Knife,1.0,21.0,True,True,Genuine,
,Knife,1.0,21.0,True,True,Fake,Report #10226
,Knife,1.0,21.0,True,True,Fake,
,Knife,1.0,21.0,True,True,,Report #10227
,Knife,1.0,21.0,True,True,,
,Knife,1.0,21.0,True,False,Genuine,Report #10228
,Knife,1.0,21.0,True,False,Genuine,
,Knife,1.0,21.0,True,False,Fake,Report #10229
,Knife,1.0,21.0,True,False,Fake,
,Knife,1.0,21.0,True,False,,Report #10230
,Knife,1.0,21.0,True,False,,
,Knife,1.0,21.0,False,True,Genuine,Report #10231
,Knife,1.0,21.0,False,True,Genuine,
,Knife,1.0,21.0,False,True,Fake,Report #10232
,Knife,1.0,21.0,False,True,Fake,
,Knife,1.0,21.0,False,True,,Report #10233
,Knife,1.0,21.0,False,True,,
,Knife,1.0,21.0,False,False,Genuine,Report #10234
,Knife,1.0,21.0,False,False,Genuine,
,Knife,1.0,21.0,False,False,Fake,Report #10235
,Knife,1.0,21.0,False,False,Fake,
,Knife,1.0,21.0,False,False,,Report #10236
,Knife,1.0,21.0,False,False,,
,Knife,1.0,99.0,True,True,Genuine,Report #10237
,Knife,1.0,99.0,True,True,Genuine,
,Knife,1.0,99.0,True,True,Fake,Report #10238
,Knife,1.0,99.0,True,True,Fake,
,Knife,1.0,99.0,True,True,,Report #10239
,Knife,1.0,99.0,True,True,,
,Knife,1.0,99.0,True,False,Genuine,Report #10240
,Knife,1.0,99.0,True,False,Genuine,
,Knife,1.0,99.0,True,False,Fake,Report #10241
,Knife,1.0,99.0,True,False,Fake,
,Knife,1.0,99.0,True,False,,Report #10242
,Knife,1.0,99.0,True,False,,
,Knife,1.0,99.0,False,True,Genuine,Report #10243
,Knife,1.0,99.0,False,True,Genuine,
,Knife,1.0,99.0,False,True,Fake,Report #10244
,Knife,1.0,99.0,False,True,Fake,
,Knife,1.0,99.0,False,True,,Report #10245
,Knife,1.0,99.0,False,True,,
,Knife,1.0,99.0,False,False,Genuine,Report #10246
,Knife,1.0,99.0,False,False,Genuine,
,Knife,1.0,99.0,False,False,Fake,Report #10247
,Knife,1.0,99.0,False,False,Fake,
,Knife,1.0,99.0,False,False,,Report #10248
,Knife,1.0,99.0,False,False,,
,Knife,1.0,100.0,True,True,Genuine,Report #10249
,Knife,1.0,100.0,True,True,Genuine,
,Knife,1.0,100.0,True,True,Fake,Report #10250
,Knife,1.0,100.0,True,True,Fake,
,Knife,1.0,100.0,True,True,,Report #10251
,Knife,1.0,100.0,True,True,,
,Knife,1.0,100.0,True,False,Genuine,Report #10252
,Knife,1.0,100.0,True,False,Genuine,
,Knife,1.0,100.0,True,False,Fake,Report #10253
,Knife,1.0,100.0,True,False,Fake,
,Knife,1.0,100.0,True,False,,Report #10254
,Knife,1.0,100.0,True,False,,
,Knife,1.0,100.0,False,True,Genuine,Report #10255
,Knife,1.0,100.0,False,True,Genuine,
,Knife,1.0,100.0,False,True,Fake,Report #10256
,Knife,1.0,100.0,False,True,Fake,
,Knife,1.0,100.0,False,True,,Report #10257
,Knife,1.0,100.0,False,True,,
,Knife,1.0,100.0,False,False,Genuine,Report #10258
,Knife,1.0,100.0,False,False,Genuine,
,Knife,1.0,100.0,False,False,Fake,Report #10259
,Knife,1.0,100.0,False,False,Fake,
,Knife,1.0,100.0,False,False,,Report #10260
,Knife,1.0,100.0,False,False,,
,Knife,1.0,101.0,True,True,Genuine,Report #10261
,Knife,1.0,101.0,True,True,Genuine,
,Knife,1.0,101.0,True,True,Fake,Report #10262
,Knife,1.0,101.0,True,True,Fake,
,Knife,1.0,101.0,True,True,,Report #10263
,Knife,1.0,101.0,True,True,,
,Knife,1.0,101.0,True,False,Genuine,Report #10264
,Knife,1.0,101.0,True,False,Genuine,
,Knife,1.0,101.0,True,False,Fake,Report #10265
,Knife,1.0,101.0,True,False,Fake,
,Knife,1.0,101.0,True,False,,Report #10266
,Knife,1.0,101.0,True,False,,
,Knife,1.0,101.0,False,True,Genuine,Report #10267
,Knife,1.0,101.0,False,True,Genuine,
,Knife,1.0,101.0,False,True,Fake,Report #10268
,Knife,1.0,101.0,False,True,Fake,
,Knife,1.0,101.0,False,True,,Report #10269
,Knife,1.0,101.0,False,True,,
,Knife,1.0,101.0,False,False,Genuine,Report #10270
,Knife,1.0,101.0,False,False,Genuine,
,Knife,1.0,101.0,False,False,Fake,Report #10271
,Knife,1.0,101.0,False,False,Fake,
,Knife,1.0,101.0,False,False,,Report #10272
,Knife,1.0,101.0,False,False,,
,Knife,100.0,-1.0,True,True,Genuine,Report #10273
,Knife,100.0,-1.0,True,True,Genuine,
,Knife,100.0,-1.0,True,True,Fake,Report #10274
,Knife,100.0,-1.0,True,True,Fake,
,Knife,100.0,-1.0,True,True,,Report #10275
,Knife,100.0,-1.0,True,True,,
,Knife,100.0,-1.0,True,False,Genuine,Report #10276
,Knife,100.0,-1.0,True,False,Genuine,
,Knife,100.0,-1.0,True,False,Fake,Report #10277
,Knife,100.0,-1.0,True,False,Fake,
,Knife,100.0,-1.0,True,False,,Report #10278
,Knife,100.0,-1.0,True,False,,
,Knife,100.0,-1.0,False,True,Genuine,Report #10279
,Knife,100.0,-1.0,False,True,Genuine,
,Knife,100.0,-1.0,False,True,Fake,Report #10280
,Knife,100.0,-1.0,False,True,Fake,
,Knife,100.0,-1.0,False,True,,Report #10281
,Knife,100.0,-1.0,False,True,,
,Knife,100.0,-1.0,False,False,Genuine,Report #10282
,Knife,100.0,-1.0,False,False,Genuine,
,Knife,100.0,-1.0,False,False,Fake,Report #10283
,Knife,100.0,-1.0,False,False,Fake,
,Knife,100.0,-1.0,False,False,,Report #10284
,Knife,100.0,-1.0,False,False,,
,Knife,100.0,0.0,True,True,Genuine,Report #10285
,Knife,100.0,0.0,True,True,Genuine,
,Knife,100.0,0.0,True,True,Fake,Report #10286
,Knife,100.0,0.0,True,True,Fake,
,Knife,100.0,0.0,True,True,,Report #10287
,Knife,100.0,0.0,True,True,,
,Knife,100.0,0.0,True,False,Genuine,Report #10288
,Knife,100.0,0.0,True,False,Genuine,
,Knife,100.0,0.0,True,False,Fake,Report #10289
,Knife,100.0,0.0,True,False,Fake,
,Knife,100.0,0.0,True,False,,Report #10290
,Knife,100.0,0.0,True,False,,
,Knife,100.0,0.0,False,True,Genuine,Report #10291
,Knife,100.0,0.0,False,True,Genuine,
,Knife,100.0,0.0,False,True,Fake,Report #10292
,Knife,100.0,0.0,False,True,Fake,
,Knife,100.0,0.0,False,True,,Report #10293
,Knife,100.0,0.0,False,True,,
,Knife,100.0,0.0,False,False,Genuine,Report #10294
,Knife,100.0,0.0,False,False,Genuine,
,Knife,100.0,0.0,False,False,Fake,Report #10295
,Knife,100.0,0.0,False,False,Fake,
,Knife,100.0,0.0,False,False,,Report #10296
,Knife,100.0,0.0,False,False,,
,Knife,100.0,19.0,True,True,Genuine,Report #10297
,Knife,100.0,19.0,True,True,Genuine,
,Knife,100.0,19.0,True,True,Fake,Report #10298
,Knife,100.0,19.0,True,True,Fake,
,Knife,100.0,19.0,True,True,,Report #10299
,Knife,100.0,19.0,True,True,,
,Knife,100.0,19.0,True,False,Genuine,Report #10300
,Knife,100.0,19.0,True,False,Genuine,
,Knife,100.0,19.0,True,False,Fake,Report #10301
,Knife,100.0,19.0,True,False,Fake,
,Knife,100.0,19.0,True,False,,Report #10302
,Knife,100.0,19.0,True,False,,
,Knife,100.0,19.0,False,True,Genuine,Report #10303
,Knife,100.0,19.0,False,True,Genuine,
,Knife,100.0,19.0,False,True,Fake,Report #10304
,Knife,100.0,19.0,False,True,Fake,
,Knife,100.0,19.0,False,True,,Report #10305
,Knife,100.0,19.0,False,True,,
,Knife,100.0,19.0,False,False,Genuine,Report #10306
,Knife,100.0,19.0,False,False,Genuine,
,Knife,100.0,19.0,False,False,Fake,Report #10307
,Knife,100.0,19.0,False,False,Fake,
,Knife,100.0,19.0,False,False,,Report #10308
,Knife,100.0,19.0,False,False,,
,Knife,100.0,20.0,True,True,Genuine,Report #10309
,Knife,100.0,20.0,True,True,Genuine,
,Knife,100.0,20.0,True,True,Fake,Report #10310
,Knife,100.0,20.0,True,True,Fake,
,Knife,100.0,20.0,True,True,,Report #10311
,Knife,100.0,20.0,True,True,,
,Knife,100.0,20.0,True,False,Genuine,Report #10312
,Knife,100.0,20.0,True,False,Genuine,
,Knife,100.0,20.0,True,False,Fake,Report #10313
,Knife,100.0,20.0,True,False,Fake,
,Knife,100.0,20.0,True,False,,Report #10314
,Knife,100.0,20.0,True,False,,
,Knife,100.0,20.0,False,True,Genuine,Report #10315
,Knife,100.0,20.0,False,True,Genuine,
,Knife,100.0,20.0,False,True,Fake,Report #10316
,Knife,100.0,20.0,False,True,Fake,
,Knife,100.0,20.0,False,True,,Report #10317
,Knife,100.0,20.0,False,True,,
,Knife,100.0,20.0,False,False,Genuine,Report #10318
,Knife,100.0,20.0,False,False,Genuine,
,Knife,100.0,20.0,False,False,Fake,Report #10319
,Knife,100.0,20.0,False,False,Fake,
,Knife,100.0,20.0,False,False,,Report #10320
,Knife,100.0,20.0,False,False,,
,Knife,100.0,21.0,True,True,Genuine,Report #10321
,Knife,100.0,21.0,True,True,Genuine,
,Knife,100.0,21.0,True,True,Fake,Report #10322
,Knife,100.0,21.0,True,True,Fake,
,Knife,100.0,21.0,True,True,,Report #10323
,Knife,100.0,21.0,True,True,,
,Knife,100.0,21.0,True,False,Genuine,Report #10324
,Knife,100.0,21.0,True,False,Genuine,
,Knife,100.0,21.0,True,False,Fake,Report #10325
,Knife,100.0,21.0,True,False,Fake,
,Knife,100.0,21.0,True,False,,Report #10326
,Knife,100.0,21.0,True,False,,
,Knife,100.0,21.0,False,True,Genuine,Report #10327
,Knife,100.0,21.0,False,True,Genuine,
,Knife,100.0,21.0,False,True,Fake,Report #10328
,Knife,100.0,21.0,False,True,Fake,
,Knife,100.0,21.0,False,True,,Report #10329
,Knife,100.0,21.0,False,True,,
,Knife,100.0,21.0,False,False,Genuine,Report #10330
,Knife,100.0,21.0,False,False,Genuine,
,Knife,100.0,21.0,False,False,Fake,Report #10331
,Knife,100.0,21.0,False,False,Fake,
,Knife,100.0,21.0,False,False,,Report #10332
,Knife,100.0,21.0,False,False,,
,Knife,100.0,99.0,True,True,Genuine,Report #10333
,Knife,100.0,99.0,True,True,Genuine,
,Knife,100.0,99.0,True,True,Fake,Report #10334
,Knife,100.0,99.0,True,True,Fake,
,Knife,100.0,99.0,True,True,,Report #10335
,Knife,100.0,99.0,True,True,,
,Knife,100.0,99.0,True,False,Genuine,Report #10336
,Knife,100.0,99.0,True,False,Genuine,
,Knife,100.0,99.0,True,False,Fake,Report #10337
,Knife,100.0,99.0,True,False,Fake,
,Knife,100.0,99.0,True,False,,Report #10338
,Knife,100.0,99.0,True,False,,
,Knife,100.0,99.0,False,True,Genuine,Report #10339
,Knife,100.0,99.0,False,True,Genuine,
,Knife,100.0,99.0,False,True,Fake,Report #10340
,Knife,100.0,99.0,False,True,Fake,
,Knife,100.0,99.0,False,True,,Report #10341
,Knife,100.0,99.0,False,True,,
,Knife,100.0,99.0,False,False,Genuine,Report #10342
,Knife,100.0,99.0,False,False,Genuine,
,Knife,100.0,99.0,False,False,Fake,Report #10343
,Knife,100.0,99.0,False,False,Fake,
,Knife,100.0,99.0,False,False,,Report #10344
,Knife,100.0,99.0,False,False,,
,Knife,100.0,100.0,True,True,Genuine,Report #10345
,Knife,100.0,100.0,True,True,Genuine,
,Knife,100.0,100.0,True,True,Fake,Report #10346
,Knife,100.0,100.0,True,True,Fake,
,Knife,100.0,100.0,True,True,,Report #10347
,Knife,100.0,100.0,True,True,,
,Knife,100.0,100.0,True,False,Genuine,Report #10348
,Knife,100.0,100.0,True,False,Genuine,
,Knife,100.0,100.0,True,False,Fake,Report #10349
,Knife,100.0,100.0,True,False,Fake,
,Knife,100.0,100.0,True,False,,Report #10350
,Knife,100.0,100.0,True,False,,
,Knife,100.0,100.0,False,True,Genuine,Report #10351
,Knife,100.0,100.0,False,True,Genuine,
,Knife,100.0,100.0,False,True,Fake,Report #10352
,Knife,100.0,100.0,False,True,Fake,
,Knife,100.0,100.0,False,True,,Report #10353
,Knife,100.0,100.0,False,True,,
,Knife,100.0,100.0,False,False,Genuine,Report #10354
,Knife,100.0,100.0,False,False,Genuine,
,Knife,100.0,100.0,False,False,Fake,Report #10355
,Knife,100.0,100.0,False,False,Fake,
,Knife,100.0,100.0,False,False,,Report #10356
,Knife,100.0,100.0,False,False,,
,Knife,100.0,101.0,True,True,Genuine,Report #10357
,Knife,100.0,101.0,True,True,Genuine,
,Knife,100.0,101.0,True,True,Fake,Report #10358
,Knife,100.0,101.0,True,True,Fake,
,Knife,100.0,101.0,True,True,,Report #10359
,Knife,100.0,101.0,True,True,,
,Knife,100.0,101.0,True,False,Genuine,Report #10360
,Knife,100.0,101.0,True,False,Genuine,
,Knife,100.0,101.0,True,False,Fake,Report #10361
,Knife,100.0,101.0,True,False,Fake,
,Knife,100.0,101.0,True,False,,Report #10362
,Knife,100.0,101.0,True,False,,
,Knife,100.0,101.0,False,True,Genuine,Report #10363
,Knife,100.0,101.0,False,True,Genuine,
,Knife,100.0,101.0,False,True,Fake,Report #10364
,Knife,100.0,101.0,False,True,Fake,
,Knife,100.0,101.0,False,True,,Report #10365
,Knife,100.0,101.0,False,True,,
,Knife,100.0,101.0,False,False,Genuine,Report #10366
,Knife,100.0,101.0,False,False,Genuine,
,Knife,100.0,101.0,False,False,Fake,Report #10367
,Knife,100.0,101.0,False,False,Fake,
,Knife,100.0,101.0,False,False,,Report #10368
,Knife,100.0,101.0,False,False,,
,Knife,1000.0,-1.0,True,True,Genuine,Report #10369
,Knife,1000.0,-1.0,True,True,Genuine,
,Knife,1000.0,-1.0,True,True,Fake,Report #10370
,Knife,1000.0,-1.0,True,True,Fake,
,Knife,1000.0,-1.0,True,True,,Report #10371
,Knife,1000.0,-1.0,True,True,,
,Knife,1000.0,-1.0,True,False,Genuine,Report #10372
,Knife,1000.0,-1.0,True,False,Genuine,
,Knife,1000.0,-1.0,True,False,Fake,Report #10373
,Knife,1000.0,-1.0,True,False,Fake,
,Knife,1000.0,-1.0,True,False,,Report #10374
,Knife,1000.0,-1.0,True,False,,
,Knife,1000.0,-1.0,False,True,Genuine,Report #10375
,Knife,1000.0,-1.0,False,True,Genuine,
,Knife,1000.0,-1.0,False,True,Fake,Report #10376
,Knife,1000.0,-1.0,False,True,Fake,
,Knife,1000.0,-1.0,False,True,,Report #10377
,Knife,1000.0,-1.0,False,True,,
,Knife,1000.0,-1.0,False,False,Genuine,Report #10378
,Knife,1000.0,-1.0,False,False,Genuine,
,Knife,1000.0,-1.0,False,False,Fake,Report #10379
,Knife,1000.0,-1.0,False,False,Fake,
,Knife,1000.0,-1.0,False,False,,Report #10380
,Knife,1000.0,-1.0,False,False,,
,Knife,1000.0,0.0,True,True,Genuine,Report #10381
,Knife,1000.0,0.0,True,True,Genuine,
,Knife,1000.0,0.0,True,True,Fake,Report #10382
,Knife,1000.0,0.0,True,True,Fake,
,Knife,1000.0,0.0,True,True,,Report #10383
,Knife,1000.0,0.0,True,True,,
,Knife,1000.0,0.0,True,False,Genuine,Report #10384
,Knife,1000.0,0.0,True,False,Genuine,
,Knife,1000.0,0.0,True,False,Fake,Report #10385
,Knife,1000.0,0.0,True,False,Fake,
,Knife,1000.0,0.0,True,False,,Report #10386
,Knife,1000.0,0.0,True,False,,
,Knife,1000.0,0.0,False,True,Genuine,Report #10387
,Knife,1000.0,0.0,False,True,Genuine,
,Knife,1000.0,0.0,False,True,Fake,Report #10388
,Knife,1000.0,0.0,False,True,Fake,
,Knife,1000.0,0.0,False,True,,Report #10389
,Knife,1000.0,0.0,False,True,,
,Knife,1000.0,0.0,False,False,Genuine,Report #10390
,Knife,1000.0,0.0,False,False,Genuine,
,Knife,1000.0,0.0,False,False,Fake,Report #10391
,Knife,1000.0,0.0,False,False,Fake,
,Knife,1000.0,0.0,False,False,,Report #10392
,Knife,1000.0,0.0,False,False,,
,Knife,1000.0,19.0,True,True,Genuine,Report #10393
,Knife,1000.0,19.0,True,True,Genuine,
,Knife,1000.0,19.0,True,True,Fake,Report #10394
,Knife,1000.0,19.0,True,True,Fake,
,Knife,1000.0,19.0,True,True,,Report #10395
,Knife,1000.0,19.0,True,True,,
,Knife,1000.0,19.0,True,False,Genuine,Report #10396
,Knife,1000.0,19.0,True,False,Genuine,
,Knife,1000.0,19.0,True,False,Fake,Report #10397
,Knife,1000.0,19.0,True,False,Fake,
,Knife,1000.0,19.0,True,False,,Report #10398
,Knife,1000.0,19.0,True,False,,
,Knife,1000.0,19.0,False,True,Genuine,Report #10399
,Knife,1000.0,19.0,False,True,Genuine,
,Knife,1000.0,19.0,False,True,Fake,Report #10400
,Knife,1000.0,19.0,False,True,Fake,
,Knife,1000.0,19.0,False,True,,Report #10401
,Knife,1000.0,19.0,False,True,,
,Knife,1000.0,19.0,False,False,Genuine,Report #10402
,Knife,1000.0,19.0,False,False,Genuine,
,Knife,1000.0,19.0,False,False,Fake,Report #10403
,Knife,1000.0,19.0,False,False,Fake,
,Knife,1000.0,19.0,False,False,,Report #10404
,Knife,1000.0,19.0,False,False,,
,Knife,1000.0,20.0,True,True,Genuine,Report #10405
,Knife,1000.0,20.0,True,True,Genuine,
,Knife,1000.0,20.0,True,True,Fake,Report #10406
,Knife,1000.0,20.0,True,True,Fake,
,Knife,1000.0,20.0,True,True,,Report #10407
,Knife,1000.0,20.0,True,True,,
,Knife,1000.0,20.0,True,False,Genuine,Report #10408
,Knife,1000.0,20.0,True,False,Genuine,
,Knife,1000.0,20.0,True,False,Fake,Report #10409
,Knife,1000.0,20.0,True,False,Fake,
,Knife,1000.0,20.0,True,False,,Report #10410
,Knife,1000.0,20.0,True,False,,
,Knife,1000.0,20.0,False,True,Genuine,Report #10411
,Knife,1000.0,20.0,False,True,Genuine,
,Knife,1000.0,20.0,False,True,Fake,Report #10412
,Knife,1000.0,20.0,False,True,Fake,
,Knife,1000.0,20.0,False,True,,Report #10413
,Knife,1000.0,20.0,False,True,,
,Knife,1000.0,20.0,False,False,Genuine,Report #10414
,Knife,1000.0,20.0,False,False,Genuine,
,Knife,1000.0,20.0,False,False,Fake,Report #10415
,Knife,1000.0,20.0,False,False,Fake,
,Knife,1000.0,20.0,False,False,,Report #10416
,Knife,1000.0,20.0,False,False,,
,Knife,1000.0,21.0,True,True,Genuine,Report #10417
,Knife,1000.0,21.0,True,True,Genuine,
,Knife,1000.0,21.0,True,True,Fake,Report #10418
,Knife,1000.0,21.0,True,True,Fake,
,Knife,1000.0,21.0,True,True,,Report #10419
,Knife,1000.0,21.0,True,True,,
,Knife,1000.0,21.0,True,False,Genuine,Report #10420
,Knife,1000.0,21.0,True,False,Genuine,
,Knife,1000.0,21.0,True,False,Fake,Report #10421
,Knife,1000.0,21.0,True,False,Fake,
,Knife,1000.0,21.0,True,False,,Report #10422
,Knife,1000.0,21.0,True,False,,
,Knife,1000.0,21.0,False,True,Genuine,Report #10423
,Knife,1000.0,21.0,False,True,Genuine,
,Knife,1000.0,21.0,False,True,Fake,Report #10424
,Knife,1000.0,21.0,False,True,Fake,
,Knife,1000.0,21.0,False,True,,Report #10425
,Knife,1000.0,21.0,False,True,,
,Knife,1000.0,21.0,False,False,Genuine,Report #10426
,Knife,1000.0,21.0,False,False,Genuine,
,Knife,1000.0,21.0,False,False,Fake,Report #10427
,Knife,1000.0,21.0,False,False,Fake,
,Knife,1000.0,21.0,False,False,,Report #10428
,Knife,1000.0,21.0,False,False,,
,Knife,1000.0,99.0,True,True,Genuine,Report #10429
,Knife,1000.0,99.0,True,True,Genuine,
,Knife,1000.0,99.0,True,True,Fake,Report #10430
,Knife,1000.0,99.0,True,True,Fake,
,Knife,1000.0,99.0,True,True,,Report #10431
,Knife,1000.0,99.0,True,True,,
,Knife,1000.0,99.0,True,False,Genuine,Report #10432
,Knife,1000.0,99.0,True,False,Genuine,
,Knife,1000.0,99.0,True,False,Fake,Report #10433
,Knife,1000.0,99.0,True,False,Fake,
,Knife,1000.0,99.0,True,False,,Report #10434
,Knife,1000.0,99.0,True,False,,
,Knife,1000.0,99.0,False,True,Genuine,Report #10435
,Knife,1000.0,99.0,False,True,Genuine,
,Knife,1000.0,99.0,False,True,Fake,Report #10436
,Knife,1000.0,99.0,False,True,Fake,
,Knife,1000.0,99.0,False,True,,Report #10437
,Knife,1000.0,99.0,False,True,,
,Knife,1000.0,99.0,False,False,Genuine,Report #10438
,Knife,1000.0,99.0,False,False,Genuine,
,Knife,1000.0,99.0,False,False,Fake,Report #10439
,Knife,1000.0,99.0,False,False,Fake,
,Knife,1000.0,99.0,False,False,,Report #10440
,Knife,1000.0,99.0,False,False,,
,Knife,1000.0,100.0,True,True,Genuine,Report #10441
,Knife,1000.0,100.0,True,True,Genuine,
,Knife,1000.0,100.0,True,True,Fake,Report #10442
,Knife,1000.0,100.0,True,True,Fake,
,Knife,1000.0,100.0,True,True,,Report #10443
,Knife,1000.0,100.0,True,True,,
,Knife,1000.0,100.0,True,False,Genuine,Report #10444
,Knife,1000.0,100.0,True,False,Genuine,
,Knife,1000.0,100.0,True,False,Fake,Report #10445
,Knife,1000.0,100.0,True,False,Fake,
,Knife,1000.0,100.0,True,False,,Report #10446
,Knife,1000.0,100.0,True,False,,
,Knife,1000.0,100.0,False,True,Genuine,Report #10447
,Knife,1000.0,100.0,False,True,Genuine,
,Knife,1000.0,100.0,False,True,Fake,Report #10448
,Knife,1000.0,100.0,False,True,Fake,
,Knife,1000.0,100.0,False,True,,Report #10449
,Knife,1000.0,100.0,False,True,,
,Knife,1000.0,100.0,False,False,Genuine,Report #10450
,Knife,1000.0,100.0,False,False,Genuine,
,Knife,1000.0,100.0,False,False,Fake,Report #10451
,Knife,1000.0,100.0,False,False,Fake,
,Knife,1000.0,100.0,False,False,,Report #10452
,Knife,1000.0,100.0,False,False,,
,Knife,1000.0,101.0,True,True,Genuine,Report #10453
,Knife,1000.0,101.0,True,True,Genuine,
,Knife,1000.0,101.0,True,True,Fake,Report #10454
,Knife,1000.0,101.0,True,True,Fake,
,Knife,1000.0,101.0,True,True,,Report #10455
,Knife,1000.0,101.0,True,True,,
,Knife,1000.0,101.0,True,False,Genuine,Report #10456
,Knife,1000.0,101.0,True,False,Genuine,
,Knife,1000.0,101.0,True,False,Fake,Report #10457
,Knife,1000.0,101.0,True,False,Fake,
,Knife,1000.0,101.0,True,False,,Report #10458
,Knife,1000.0,101.0,True,False,,
,Knife,1000.0,101.0,False,True,Genuine,Report #10459
,Knife,1000.0,101.0,False,True,Genuine,
,Knife,1000.0,101.0,False,True,Fake,Report #10460
,Knife,1000.0,101.0,False,True,Fake,
,Knife,1000.0,101.0,False,True,,Report #10461
,Knife,1000.0,101.0,False,True,,
,Knife,1000.0,101.0,False,False,Genuine,Report #10462
,Knife,1000.0,101.0,False,False,Genuine,
,Knife,1000.0,101.0,False,False,Fake,Report #10463
,Knife,1000.0,101.0,False,False,Fake,
,Knife,1000.0,101.0,False,False,,Report #10464
,Knife,1000.0,101.0,False,False,,
,Knife,1000000.0,-1.0,True,True,Genuine,Report #10465
,Knife,1000000.0,-1.0,True,True,Genuine,
,Knife,1000000.0,-1.0,True,True,Fake,Report #10466
,Knife,1000000.0,-1.0,True,True,Fake,
,Knife,1000000.0,-1.0,True,True,,Report #10467
,Knife,1000000.0,-1.0,True,True,,
,Knife,1000000.0,-1.0,True,False,Genuine,Report #10468
,Knife,1000000.0,-1.0,True,False,Genuine,
,Knife,1000000.0,-1.0,True,False,Fake,Report #10469
,Knife,1000000.0,-1.0,True,False,Fake,
,Knife,1000000.0,-1.0,True,False,,Report #10470
,Knife,1000000.0,-1.0,True,False,,
,Knife,1000000.0,-1.0,False,True,Genuine,Report #10471
,Knife,1000000.0,-1.0,False,True,Genuine,
,Knife,1000000.0,-1.0,False,True,Fake,Report #10472
,Knife,1000000.0,-1.0,False,True,Fake,
,Knife,1000000.0,-1.0,False,True,,Report #10473
,Knife,1000000.0,-1.0,False,True,,
,Knife,1000000.0,-1.0,False,False,Genuine,Report #10474
,Knife,1000000.0,-1.0,False,False,Genuine,
,Knife,1000000.0,-1.0,False,False,Fake,Report #10475
,Knife,1000000.0,-1.0,False,False,Fake,
,Knife,1000000.0,-1.0,False,False,,Report #10476
,Knife,1000000.0,-1.0,False,False,,
,Knife,1000000.0,0.0,True,True,Genuine,Report #10477
,Knife,1000000.0,0.0,True,True,Genuine,
,Knife,1000000.0,0.0,True,True,Fake,Report #10478
,Knife,1000000.0,0.0,True,True,Fake,
,Knife,1000000.0,0.0,True,True,,Report #10479
,Knife,1000000.0,0.0,True,True,,
,Knife,1000000.0,0.0,True,False,Genuine,Report #10480
,Knife,1000000.0,0.0,True,False,Genuine,
,Knife,1000000.0,0.0,True,False,Fake,Report #10481
,Knife,1000000.0,0.0,True,False,Fake,
,Knife,1000000.0,0.0,True,False,,Report #10482
,Knife,1000000.0,0.0,True,False,,
,Knife,1000000.0,0.0,False,True,Genuine,Report #10483
,Knife,1000000.0,0.0,False,True,Genuine,
,Knife,1000000.0,0.0,False,True,Fake,Report #10484
,Knife,1000000.0,0.0,False,True,Fake,
,Knife,1000000.0,0.0,False,True,,Report #10485
,Knife,1000000.0,0.0,False,True,,
,Knife,1000000.0,0.0,False,False,Genuine,Report #10486
,Knife,1000000.0,0.0,False,False,Genuine,
,Knife,1000000.0,0.0,False,False,Fake,Report #10487
,Knife,1000000.0,0.0,False,False,Fake,
,Knife,1000000.0,0.0,False,False,,Report #10488
,Knife,1000000.0,0.0,False,False,,
,Knife,1000000.0,19.0,True,True,Genuine,Report #10489
,Knife,1000000.0,19.0,True,True,Genuine,
,Knife,1000000.0,19.0,True,True,Fake,Report #10490
,Knife,1000000.0,19.0,True,True,Fake,
,Knife,1000000.0,19.0,True,True,,Report #10491
,Knife,1000000.0,19.0,True,True,,
,Knife,1000000.0,19.0,True,False,Genuine,Report #10492
,Knife,1000000.0,19.0,True,False,Genuine,
,Knife,1000000.0,19.0,True,False,Fake,Report #10493
,Knife,1000000.0,19.0,True,False,Fake,
,Knife,1000000.0,19.0,True,False,,Report #10494
,Knife,1000000.0,19.0,True,False,,
,Knife,1000000.0,19.0,False,True,Genuine,Report #10495
,Knife,1000000.0,19.0,False,True,Genuine,
,Knife,1000000.0,19.0,False,True,Fake,Report #10496
,Knife,1000000.0,19.0,False,True,Fake,
,Knife,1000000.0,19.0,False,True,,Report #10497
,Knife,1000000.0,19.0,False,True,,
,Knife,1000000.0,19.0,False,False,Genuine,Report #10498
,Knife,1000000.0,19.0,False,False,Genuine,
,Knife,1000000.0,19.0,False,False,Fake,Report #10499
,Knife,1000000.0,19.0,False,False,Fake,
,Knife,1000000.0,19.0,False,False,,Report #10500
,Knife,1000000.0,19.0,False,False,,
,Knife,1000000.0,20.0,True,True,Genuine,Report #10501
,Knife,1000000.0,20.0,True,True,Genuine,
,Knife,1000000.0,20.0,True,True,Fake,Report #10502
,Knife,1000000.0,20.0,True,True,Fake,
,Knife,1000000.0,20.0,True,True,,Report #10503
,Knife,1000000.0,20.0,True,True,,
,Knife,1000000.0,20.0,True,False,Genuine,Report #10504
,Knife,1000000.0,20.0,True,False,Genuine,
,Knife,1000000.0,20.0,True,False,Fake,Report #10505
,Knife,1000000.0,20.0,True,False,Fake,
,Knife,1000000.0,20.0,True,False,,Report #10506
,Knife,1000000.0,20.0,True,False,,
,Knife,1000000.0,20.0,False,True,Genuine,Report #10507
,Knife,1000000.0,20.0,False,True,Genuine,
,Knife,1000000.0,20.0,False,True,Fake,Report #10508
,Knife,1000000.0,20.0,False,True,Fake,
,Knife,1000000.0,20.0,False,True,,Report #10509
,Knife,1000000.0,20.0,False,True,,
,Knife,1000000.0,20.0,False,False,Genuine,Report #10510
,Knife,1000000.0,20.0,False,False,Genuine,
,Knife,1000000.0,20.0,False,False,Fake,Report #10511
,Knife,1000000.0,20.0,False,False,Fake,
,Knife,1000000.0,20.0,False,False,,Report #10512
,Knife,1000000.0,20.0,False,False,,
,Knife,1000000.0,21.0,True,True,Genuine,Report #10513
,Knife,1000000.0,21.0,True,True,Genuine,
,Knife,1000000.0,21.0,True,True,Fake,Report #10514
,Knife,1000000.0,21.0,True,True,Fake,
,Knife,1000000.0,21.0,True,True,,Report #10515
,Knife,1000000.0,21.0,True,True,,
,Knife,1000000.0,21.0,True,False,Genuine,Report #10516
,Knife,1000000.0,21.0,True,False,Genuine,
,Knife,1000000.0,21.0,True,False,Fake,Report #10517
,Knife,1000000.0,21.0,True,False,Fake,
,Knife,1000000.0,21.0,True,False,,Report #10518
,Knife,1000000.0,21.0,True,False,,
,Knife,1000000.0,21.0,False,True,Genuine,Report #10519
,Knife,1000000.0,21.0,False,True,Genuine,
,Knife,1000000.0,21.0,False,True,Fake,Report #10520
,Knife,1000000.0,21.0,False,True,Fake,
,Knife,1000000.0,21.0,False,True,,Report #10521
,Knife,1000000.0,21.0,False,True,,
,Knife,1000000.0,21.0,False,False,Genuine,Report #10522
,Knife,1000000.0,21.0,False,False,Genuine,
,Knife,1000000.0,21.0,False,False,Fake,Report #10523
,Knife,1000000.0,21.0,False,False,Fake,
,Knife,1000000.0,21.0,False,False,,Report #10524
,Knife,1000000.0,21.0,False,False,,
,Knife,1000000.0,99.0,True,True,Genuine,Report #10525
,Knife,1000000.0,99.0,True,True,Genuine,
,Knife,1000000.0,99.0,True,True,Fake,Report #10526
,Knife,1000000.0,99.0,True,True,Fake,
,Knife,1000000.0,99.0,True,True,,Report #10527
,Knife,1000000.0,99.0,True,True,,
,Knife,1000000.0,99.0,True,False,Genuine,Report #10528
,Knife,1000000.0,99.0,True,False,Genuine,
,Knife,1000000.0,99.0,True,False,Fake,Report #10529
,Knife,1000000.0,99.0,True,False,Fake,
,Knife,1000000.0,99.0,True,False,,Report #10530
,Knife,1000000.0,99.0,True,False,,
,Knife,1000000.0,99.0,False,True,Genuine,Report #10531
,Knife,1000000.0,99.0,False,True,Genuine,
,Knife,1000000.0,99.0,False,True,Fake,Report #10532
,Knife,1000000.0,99.0,False,True,Fake,
,Knife,1000000.0,99.0,False,True,,Report #10533
,Knife,1000000.0,99.0,False,True,,
,Knife,1000000.0,99.0,False,False,Genuine,Report #10534
,Knife,1000000.0,99.0,False,False,Genuine,
,Knife,1000000.0,99.0,False,False,Fake,Report #10535
,Knife,1000000.0,99.0,False,False,Fake,
,Knife,1000000.0,99.0,False,False,,Report #10536
,Knife,1000000.0,99.0,False,False,,
,Knife,1000000.0,100.0,True,True,Genuine,Report #10537
,Knife,1000000.0,100.0,True,True,Genuine,
,Knife,1000000.0,100.0,True,True,Fake,Report #10538
,Knife,1000000.0,100.0,True,True,Fake,
,Knife,1000000.0,100.0,True,True,,Report #10539
,Knife,1000000.0,100.0,True,True,,
,Knife,1000000.0,100.0,True,False,Genuine,Report #10540
,Knife,1000000.0,100.0,True,False,Genuine,
,Knife,1000000.0,100.0,True,False,Fake,Report #10541
,Knife,1000000.0,100.0,True,False,Fake,
,Knife,1000000.0,100.0,True,False,,Report #10542
,Knife,1000000.0,100.0,True,False,,
,Knife,1000000.0,100.0,False,True,Genuine,Report #10543
,Knife,1000000.0,100.0,False,True,Genuine,
,Knife,1000000.0,100.0,False,True,Fake,Report #10544
,Knife,1000000.0,100.0,False,True,Fake,
,Knife,1000000.0,100.0,False,True,,Report #10545
,Knife,1000000.0,100.0,False,True,,
,Knife,1000000.0,100.0,False,False,Genuine,Report #10546
,Knife,1000000.0,100.0,False,False,Genuine,
,Knife,1000000.0,100.0,False,False,Fake,Report #10547
,Knife,1000000.0,100.0,False,False,Fake,
,Knife,1000000.0,100.0,False,False,,Report #10548
,Knife,1000000.0,100.0,False,False,,
,Knife,1000000.0,101.0,True,True,Genuine,Report #10549
,Knife,1000000.0,101.0,True,True,Genuine,
,Knife,1000000.0,101.0,True,True,Fake,Report #10550
,Knife,1000000.0,101.0,True,True,Fake,
,Knife,1000000.0,101.0,True,True,,Report #10551
,Knife,1000000.0,101.0,True,True,,
,Knife,1000000.0,101.0,True,False,Genuine,Report #10552
,Knife,1000000.0,101.0,True,False,Genuine,
,Knife,1000000.0,101.0,True,False,Fake,Report #10553
,Knife,1000000.0,101.0,True,False,Fake,
,Knife,1000000.0,101.0,True,False,,Report #10554
,Knife,1000000.0,101.0,True,False,,
,Knife,1000000.0,101.0,False,True,Genuine,Report #10555
,Knife,1000000.0,101.0,False,True,Genuine,
,Knife,1000000.0,101.0,False,True,Fake,Report #10556
,Knife,1000000.0,101.0,False,True,Fake,
,Knife,1000000.0,101.0,False,True,,Report #10557
,Knife,1000000.0,101.0,False,True,,
,Knife,1000000.0,101.0,False,False,Genuine,Report #10558
,Knife,1000000.0,101.0,False,False,Genuine,
,Knife,1000000.0,101.0,False,False,Fake,Report #10559
,Knife,1000000.0,101.0,False,False,Fake,
,Knife,1000000.0,101.0,False,False,,Report #10560
,Knife,1000000.0,101.0,False,False,,
,Plates,0.0,-1.0,True,True,Genuine,Report #10561
,Plates,0.0,-1.0,True,True,Genuine,
,Plates,0.0,-1.0,True,True,Fake,Report #10562
,Plates,0.0,-1.0,True,True,Fake,
,Plates,0.0,-1.0,True,True,,Report #10563
,Plates,0.0,-1.0,True,True,,
,Plates,0.0,-1.0,True,False,Genuine,Report #10564
,Plates,0.0,-1.0,True,False,Genuine,
,Plates,0.0,-1.0,True,False,Fake,Report #10565
,Plates,0.0,-1.0,True,False,Fake,
,Plates,0.0,-1.0,True,False,,Report #10566
,Plates,0.0,-1.0,True,False,,
,Plates,0.0,-1.0,False,True,Genuine,Report #10567
,Plates,0.0,-1.0,False,True,Genuine,
,Plates,0.0,-1.0,False,True,Fake,Report #10568
,Plates,0.0,-1.0,False,True,Fake,
,Plates,0.0,-1.0,False,True,,Report #10569
,Plates,0.0,-1.0,False,True,,
,Plates,0.0,-1.0,False,False,Genuine,Report #10570
,Plates,0.0,-1.0,False,False,Genuine,
,Plates,0.0,-1.0,False,False,Fake,Report #10571
,Plates,0.0,-1.0,False,False,Fake,
,Plates,0.0,-1.0,False,False,,Report #10572
,Plates,0.0,-1.0,False,False,,
,Plates,0.0,0.0,True,True,Genuine,Report #10573
,Plates,0.0,0.0,True,True,Genuine,
,Plates,0.0,0.0,True,True,Fake,Report #10574
,Plates,0.0,0.0,True,True,Fake,
,Plates,0.0,0.0,True,True,,Report #10575
,Plates,0.0,0.0,True,True,,
,Plates,0.0,0.0,True,False,Genuine,Report #10576
,Plates,0.0,0.0,True,False,Genuine,
,Plates,0.0,0.0,True,False,Fake,Report #10577
,Plates,0.0,0.0,True,False,Fake,
,Plates,0.0,0.0,True,False,,Report #10578
,Plates,0.0,0.0,True,False,,
,Plates,0.0,0.0,False,True,Genuine,Report #10579
,Plates,0.0,0.0,False,True,Genuine,
,Plates,0.0,0.0,False,True,Fake,Report #10580
,Plates,0.0,0.0,False,True,Fake,
,Plates,0.0,0.0,False,True,,Report #10581
,Plates,0.0,0.0,False,True,,
,Plates,0.0,0.0,False,False,Genuine,Report #10582
,Plates,0.0,0.0,False,False,Genuine,
,Plates,0.0,0.0,False,False,Fake,Report #10583
,Plates,0.0,0.0,False,False,Fake,
,Plates,0.0,0.0,False,False,,Report #10584
,Plates,0.0,0.0,False,False,,
,Plates,0.0,19.0,True,True,Genuine,Report #10585
,Plates,0.0,19.0,True,True,Genuine,
,Plates,0.0,19.0,True,True,Fake,Report #10586
,Plates,0.0,19.0,True,True,Fake,
,Plates,0.0,19.0,True,True,,Report #10587
,Plates,0.0,19.0,True,True,,
,Plates,0.0,19.0,True,False,Genuine,Report #10588
,Plates,0.0,19.0,True,False,Genuine,
,Plates,0.0,19.0,True,False,Fake,Report #10589
,Plates,0.0,19.0,True,False,Fake,
,Plates,0.0,19.0,True,False,,Report #10590
,Plates,0.0,19.0,True,False,,
,Plates,0.0,19.0,False,True,Genuine,Report #10591
,Plates,0.0,19.0,False,True,Genuine,
,Plates,0.0,19.0,False,True,Fake,Report #10592
,Plates,0.0,19.0,False,True,Fake,
,Plates,0.0,19.0,False,True,,Report #10593
,Plates,0.0,19.0,False,True,,
,Plates,0.0,19.0,False,False,Genuine,Report #10594
,Plates,0.0,19.0,False,False,Genuine,
,Plates,0.0,19.0,False,False,Fake,Report #10595
,Plates,0.0,19.0,False,False,Fake,
,Plates,0.0,19.0,False,False,,Report #10596
,Plates,0.0,19.0,False,False,,
,Plates,0.0,20.0,True,True,Genuine,Report #10597
,Plates,0.0,20.0,True,True,Genuine,
,Plates,0.0,20.0,True,True,Fake,Report #10598
,Plates,0.0,20.0,True,True,Fake,
,Plates,0.0,20.0,True,True,,Report #10599
,Plates,0.0,20.0,True,True,,
,Plates,0.0,20.0,True,False,Genuine,Report #10600
,Plates,0.0,20.0,True,False,Genuine,
,Plates,0.0,20.0,True,False,Fake,Report #10601
,Plates,0.0,20.0,True,False,Fake,
,Plates,0.0,20.0,True,False,,Report #10602
,Plates,0.0,20.0,True,False,,
,Plates,0.0,20.0,False,True,Genuine,Report #10603
,Plates,0.0,20.0,False,True,Genuine,
,Plates,0.0,20.0,False,True,Fake,Report #10604
,Plates,0.0,20.0,False,True,Fake,
,Plates,0.0,20.0,False,True,,Report #10605
,Plates,0.0,20.0,False,True,,
,Plates,0.0,20.0,False,False,Genuine,Report #10606
,Plates,0.0,20.0,False,False,Genuine,
,Plates,0.0,20.0,False,False,Fake,Report #10607
,Plates,0.0,20.0,False,False,Fake,
,Plates,0.0,20.0,False,False,,Report #10608
,Plates,0.0,20.0,False,False,,
,Plates,0.0,21.0,True,True,Genuine,Report #10609
,Plates,0.0,21.0,True,True,Genuine,
,Plates,0.0,21.0,True,True,Fake,Report #10610
,Plates,0.0,21.0,True,True,Fake,
,Plates,0.0,21.0,True,True,,Report #10611
,Plates,0.0,21.0,True,True,,
,Plates,0.0,21.0,True,False,Genuine,Report #10612
,Plates,0.0,21.0,True,False,Genuine,
,Plates,0.0,21.0,True,False,Fake,Report #10613
,Plates,0.0,21.0,True,False,Fake,
,Plates,0.0,21.0,True,False,,Report #10614
,Plates,0.0,21.0,True,False,,
,Plates,0.0,21.0,False,True,Genuine,Report #10615
,Plates,0.0,21.0,False,True,Genuine,
,Plates,0.0,21.0,False,True,Fake,Report #10616
,Plates,0.0,21.0,False,True,Fake,
,Plates,0.0,21.0,False,True,,Report #10617
,Plates,0.0,21.0,False,True,,
,Plates,0.0,21.0,False,False,Genuine,Report #10618
,Plates,0.0,21.0,False,False,Genuine,
,Plates,0.0,21.0,False,False,Fake,Report #10619
,Plates,0.0,21.0,False,False,Fake,
,Plates,0.0,21.0,False,False,,Report #10620
,Plates,0.0,21.0,False,False,,
,Plates,0.0,99.0,True,True,Genuine,Report #10621
,Plates,0.0,99.0,True,True,Genuine,
,Plates,0.0,99.0,True,True,Fake,Report #10622
,Plates,0.0,99.0,True,True,Fake,
,Plates,0.0,99.0,True,True,,Report #10623
,Plates,0.0,99.0,True,True,,
,Plates,0.0,99.0,True,False,Genuine,Report #10624
,Plates,0.0,99.0,True,False,Genuine,
,Plates,0.0,99.0,True,False,Fake,Report #10625
,Plates,0.0,99.0,True,False,Fake,
,Plates,0.0,99.0,True,False,,Report #10626
,Plates,0.0,99.0,True,False,,
,Plates,0.0,99.0,False,True,Genuine,Report #10627
,Plates,0.0,99.0,False,True,Genuine,
,Plates,0.0,99.0,False,True,Fake,Report #10628
,Plates,0.0,99.0,False,True,Fake,
,Plates,0.0,99.0,False,True,,Report #10629
,Plates,0.0,99.0,False,True,,
,Plates,0.0,99.0,False,False,Genuine,Report #10630
,Plates,0.0,99.0,False,False,Genuine,
,Plates,0.0,99.0,False,False,Fake,Report #10631
,Plates,0.0,99.0,False,False,Fake,
,Plates,0.0,99.0,False,False,,Report #10632
,Plates,0.0,99.0,False,False,,
,Plates,0.0,100.0,True,True,Genuine,Report #10633
,Plates,0.0,100.0,True,True,Genuine,
,Plates,0.0,100.0,True,True,Fake,Report #10634
,Plates,0.0,100.0,True,True,Fake,
,Plates,0.0,100.0,True,True,,Report #10635
,Plates,0.0,100.0,True,True,,
,Plates,0.0,100.0,True,False,Genuine,Report #10636
,Plates,0.0,100.0,True,False,Genuine,
,Plates,0.0,100.0,True,False,Fake,Report #10637
,Plates,0.0,100.0,True,False,Fake,
,Plates,0.0,100.0,True,False,,Report #10638
,Plates,0.0,100.0,True,False,,
,Plates,0.0,100.0,False,True,Genuine,Report #10639
,Plates,0.0,100.0,False,True,Genuine,
,Plates,0.0,100.0,False,True,Fake,Report #10640
,Plates,0.0,100.0,False,True,Fake,
,Plates,0.0,100.0,False,True,,Report #10641
,Plates,0.0,100.0,False,True,,
,Plates,0.0,100.0,False,False,Genuine,Report #10642
,Plates,0.0,100.0,False,False,Genuine,
,Plates,0.0,100.0,False,False,Fake,Report #10643
,Plates,0.0,100.0,False,False,Fake,
,Plates,0.0,100.0,False,False,,Report #10644
,Plates,0.0,100.0,False,False,,
,Plates,0.0,101.0,True,True,Genuine,Report #10645
,Plates,0.0,101.0,True,True,Genuine,
,Plates,0.0,101.0,True,True,Fake,Report #10646
,Plates,0.0,101.0,True,True,Fake,
,Plates,0.0,101.0,True,True,,Report #10647
,Plates,0.0,101.0,True,True,,
,Plates,0.0,101.0,True,False,Genuine,Report #10648
,Plates,0.0,101.0,True,False,Genuine,
,Plates,0.0,101.0,True,False,Fake,Report #10649
,Plates,0.0,101.0,True,False,Fake,
,Plates,0.0,101.0,True,False,,Report #10650
,Plates,0.0,101.0,True,False,,
,Plates,0.0,101.0,False,True,Genuine,Report #10651
,Plates,0.0,101.0,False,True,Genuine,
,Plates,0.0,101.0,False,True,Fake,Report #10652
,Plates,0.0,101.0,False,True,Fake,
,Plates,0.0,101.0,False,True,,Report #10653
,Plates,0.0,101.0,False,True,,
,Plates,0.0,101.0,False,False,Genuine,Report #10654
,Plates,0.0,101.0,False,False,Genuine,
,Plates,0.0,101.0,False,False,Fake,Report #10655
,Plates,0.0,101.0,False,False,Fake,
,Plates,0.0,101.0,False,False,,Report #10656
,Plates,0.0,101.0,False,False,,
,Plates,1.0,-1.0,True,True,Genuine,Report #10657
,Plates,1.0,-1.0,True,True,Genuine,
,Plates,1.0,-1.0,True,True,Fake,Report #10658
,Plates,1.0,-1.0,True,True,Fake,
,Plates,1.0,-1.0,True,True,,Report #10659
,Plates,1.0,-1.0,True,True,,
,Plates,1.0,-1.0,True,False,Genuine,Report #10660
,Plates,1.0,-1.0,True,False,Genuine,
,Plates,1.0,-1.0,True,False,Fake,Report #10661
,Plates,1.0,-1.0,True,False,Fake,
,Plates,1.0,-1.0,True,False,,Report #10662
,Plates,1.0,-1.0,True,False,,
,Plates,1.0,-1.0,False,True,Genuine,Report #10663
,Plates,1.0,-1.0,False,True,Genuine,
,Plates,1.0,-1.0,False,True,Fake,Report #10664
,Plates,1.0,-1.0,False,True,Fake,
,Plates,1.0,-1.0,False,True,,Report #10665
,Plates,1.0,-1.0,False,True,,
,Plates,1.0,-1.0,False,False,Genuine,Report #10666
,Plates,1.0,-1.0,False,False,Genuine,
,Plates,1.0,-1.0,False,False,Fake,Report #10667
,Plates,1.0,-1.0,False,False,Fake,
,Plates,1.0,-1.0,False,False,,Report #10668
,Plates,1.0,-1.0,False,False,,
,Plates,1.0,0.0,True,True,Genuine,Report #10669
,Plates,1.0,0.0,True,True,Genuine,
,Plates,1.0,0.0,True,True,Fake,Report #10670
,Plates,1.0,0.0,True,True,Fake,
,Plates,1.0,0.0,True,True,,Report #10671
,Plates,1.0,0.0,True,True,,
,Plates,1.0,0.0,True,False,Genuine,Report #10672
,Plates,1.0,0.0,True,False,Genuine,
,Plates,1.0,0.0,True,False,Fake,Report #10673
,Plates,1.0,0.0,True,False,Fake,
,Plates,1.0,0.0,True,False,,Report #10674
,Plates,1.0,0.0,True,False,,
,Plates,1.0,0.0,False,True,Genuine,Report #10675
,Plates,1.0,0.0,False,True,Genuine,
,Plates,1.0,0.0,False,True,Fake,Report #10676
,Plates,1.0,0.0,False,True,Fake,
,Plates,1.0,0.0,False,True,,Report #10677
,Plates,1.0,0.0,False,True,,
,Plates,1.0,0.0,False,False,Genuine,Report #10678
,Plates,1.0,0.0,False,False,Genuine,
,Plates,1.0,0.0,False,False,Fake,Report #10679
,Plates,1.0,0.0,False,False,Fake,
,Plates,1.0,0.0,False,False,,Report #10680
,Plates,1.0,0.0,False,False,,
,Plates,1.0,19.0,True,True,Genuine,Report #10681
,Plates,1.0,19.0,True,True,Genuine,
,Plates,1.0,19.0,True,True,Fake,Report #10682
,Plates,1.0,19.0,True,True,Fake,
,Plates,1.0,19.0,True,True,,Report #10683
,Plates,1.0,19.0,True,True,,
,Plates,1.0,19.0,True,False,Genuine,Report #10684
,Plates,1.0,19.0,True,False,Genuine,
,Plates,1.0,19.0,True,False,Fake,Report #10685
,Plates,1.0,19.0,True,False,Fake,
,Plates,1.0,19.0,True,False,,Report #10686
,Plates,1.0,19.0,True,False,,
,Plates,1.0,19.0,False,True,Genuine,Report #10687
,Plates,1.0,19.0,False,True,Genuine,
,Plates,1.0,19.0,False,True,Fake,Report #10688
,Plates,1.0,19.0,False,True,Fake,
,Plates,1.0,19.0,False,True,,Report #10689
,Plates,1.0,19.0,False,True,,
,Plates,1.0,19.0,False,False,Genuine,Report #10690
,Plates,1.0,19.0,False,False,Genuine,
,Plates,1.0,19.0,False,False,Fake,Report #10691
,Plates,1.0,19.0,False,False,Fake,
,Plates,1.0,19.0,False,False,,Report #10692
,Plates,1.0,19.0,False,False,,
,Plates,1.0,20.0,True,True,Genuine,Report #10693
,Plates,1.0,20.0,True,True,Genuine,
,Plates,1.0,20.0,True,True,Fake,Report #10694
,Plates,1.0,20.0,True,True,Fake,
,Plates,1.0,20.0,True,True,,Report #10695
,Plates,1.0,20.0,True,True,,
,Plates,1.0,20.0,True,False,Genuine,Report #10696
,Plates,1.0,20.0,True,False,Genuine,
,Plates,1.0,20.0,True,False,Fake,Report #10697
,Plates,1.0,20.0,True,False,Fake,
,Plates,1.0,20.0,True,False,,Report #10698
,Plates,1.0,20.0,True,False,,
,Plates,1.0,20.0,False,True,Genuine,Report #10699
,Plates,1.0,20.0,False,True,Genuine,
,Plates,1.0,20.0,False,True,Fake,Report #10700
,Plates,1.0,20.0,False,True,Fake,
,Plates,1.0,20.0,False,True,,Report #10701
,Plates,1.0,20.0,False,True,,
,Plates,1.0,20.0,False,False,Genuine,Report #10702
,Plates,1.0,20.0,False,False,Genuine,
,Plates,1.0,20.0,False,False,Fake,Report #10703
,Plates,1.0,20.0,False,False,Fake,
,Plates,1.0,20.0,False,False,,Report #10704
,Plates,1.0,20.0,False,False,,
,Plates,1.0,21.0,True,True,Genuine,Report #10705
,Plates,1.0,21.0,True,True,Genuine,
,Plates,1.0,21.0,True,True,Fake,Report #10706
,Plates,1.0,21.0,True,True,Fake,
,Plates,1.0,21.0,True,True,,Report #10707
,Plates,1.0,21.0,True,True,,
,Plates,1.0,21.0,True,False,Genuine,Report #10708
,Plates,1.0,21.0,True,False,Genuine,
,Plates,1.0,21.0,True,False,Fake,Report #10709
,Plates,1.0,21.0,True,False,Fake,
,Plates,1.0,21.0,True,False,,Report #10710
,Plates,1.0,21.0,True,False,,
,Plates,1.0,21.0,False,True,Genuine,Report #10711
,Plates,1.0,21.0,False,True,Genuine,
,Plates,1.0,21.0,False,True,Fake,Report #10712
,Plates,1.0,21.0,False,True,Fake,
,Plates,1.0,21.0,False,True,,Report #10713
,Plates,1.0,21.0,False,True,,
,Plates,1.0,21.0,False,False,Genuine,Report #10714
,Plates,1.0,21.0,False,False,Genuine,
,Plates,1.0,21.0,False,False,Fake,Report #10715
,Plates,1.0,21.0,False,False,Fake,
,Plates,1.0,21.0,False,False,,Report #10716
,Plates,1.0,21.0,False,False,,
,Plates,1.0,99.0,True,True,Genuine,Report #10717
,Plates,1.0,99.0,True,True,Genuine,
,Plates,1.0,99.0,True,True,Fake,Report #10718
,Plates,1.0,99.0,True,True,Fake,
,Plates,1.0,99.0,True,True,,Report #10719
,Plates,1.0,99.0,True,True,,
,Plates,1.0,99.0,True,False,Genuine,Report #10720
,Plates,1.0,99.0,True,False,Genuine,
,Plates,1.0,99.0,True,False,Fake,Report #10721
,Plates,1.0,99.0,True,False,Fake,
,Plates,1.0,99.0,True,False,,Report #10722
,Plates,1.0,99.0,True,False,,
,Plates,1.0,99.0,False,True,Genuine,Report #10723
,Plates,1.0,99.0,False,True,Genuine,
,Plates,1.0,99.0,False,True,Fake,Report #10724
,Plates,1.0,99.0,False,True,Fake,
,Plates,1.0,99.0,False,True,,Report #10725
,Plates,1.0,99.0,False,True,,
,Plates,1.0,99.0,False,False,Genuine,Report #10726
,Plates,1.0,99.0,False,False,Genuine,
,Plates,1.0,99.0,False,False,Fake,Report #10727
,Plates,1.0,99.0,False,False,Fake,
,Plates,1.0,99.0,False,False,,Report #10728
,Plates,1.0,99.0,False,False,,
,Plates,1.0,100.0,True,True,Genuine,Report #10729
,Plates,1.0,100.0,True,True,Genuine,
,Plates,1.0,100.0,True,True,Fake,Report #10730
,Plates,1.0,100.0,True,True,Fake,
,Plates,1.0,100.0,True,True,,Report #10731
,Plates,1.0,100.0,True,True,,
,Plates,1.0,100.0,True,False,Genuine,Report #10732
,Plates,1.0,100.0,True,False,Genuine,
,Plates,1.0,100.0,True,False,Fake,Report #10733
,Plates,1.0,100.0,True,False,Fake,
,Plates,1.0,100.0,True,False,,Report #10734
,Plates,1.0,100.0,True,False,,
,Plates,1.0,100.0,False,True,Genuine,Report #10735
,Plates,1.0,100.0,False,True,Genuine,
,Plates,1.0,100.0,False,True,Fake,Report #10736
,Plates,1.0,100.0,False,True,Fake,
,Plates,1.0,100.0,False,True,,Report #10737
,Plates,1.0,100.0,False,True,,
,Plates,1.0,100.0,False,False,Genuine,Report #10738
,Plates,1.0,100.0,False,False,Genuine,
,Plates,1.0,100.0,False,False,Fake,Report #10739
,Plates,1.0,100.0,False,False,Fake,
,Plates,1.0,100.0,False,False,,Report #10740
,Plates,1.0,100.0,False,False,,
,Plates,1.0,101.0,True,True,Genuine,Report #10741
,Plates,1.0,101.0,True,True,Genuine,
,Plates,1.0,101.0,True,True,Fake,Report #10742
,Plates,1.0,101.0,True,True,Fake,
,Plates,1.0,101.0,True,True,,Report #10743
,Plates,1.0,101.0,True,True,,
,Plates,1.0,101.0,True,False,Genuine,Report #10744
,Plates,1.0,101.0,True,False,Genuine,
,Plates,1.0,101.0,True,False,Fake,Report #10745
,Plates,1.0,101.0,True,False,Fake,
,Plates,1.0,101.0,True,False,,Report #10746
,Plates,1.0,101.0,True,False,,
,Plates,1.0,101.0,False,True,Genuine,Report #10747
,Plates,1.0,101.0,False,True,Genuine,
,Plates,1.0,101.0,False,True,Fake,Report #10748
,Plates,1.0,101.0,False,True,Fake,
,Plates,1.0,101.0,False,True,,Report #10749
,Plates,1.0,101.0,False,True,,
,Plates,1.0,101.0,False,False,Genuine,Report #10750
,Plates,1.0,101.0,False,False,Genuine,
,Plates,1.0,101.0,False,False,Fake,Report #10751
,Plates,1.0,101.0,False,False,Fake,
,Plates,1.0,101.0,False,False,,Report #10752
,Plates,1.0,101.0,False,False,,
,Plates,100.0,-1.0,True,True,Genuine,Report #10753
,Plates,100.0,-1.0,True,True,Genuine,
,Plates,100.0,-1.0,True,True,Fake,Report #10754
,Plates,100.0,-1.0,True,True,Fake,
,Plates,100.0,-1.0,True,True,,Report #10755
,Plates,100.0,-1.0,True,True,,
,Plates,100.0,-1.0,True,False,Genuine,Report #10756
,Plates,100.0,-1.0,True,False,Genuine,
,Plates,100.0,-1.0,True,False,Fake,Report #10757
,Plates,100.0,-1.0,True,False,Fake,
,Plates,100.0,-1.0,True,False,,Report #10758
,Plates,100.0,-1.0,True,False,,
,Plates,100.0,-1.0,False,True,Genuine,Report #10759
,Plates,100.0,-1.0,False,True,Genuine,
,Plates,100.0,-1.0,False,True,Fake,Report #10760
,Plates,100.0,-1.0,False,True,Fake,
,Plates,100.0,-1.0,False,True,,Report #10761
,Plates,100.0,-1.0,False,True,,
,Plates,100.0,-1.0,False,False,Genuine,Report #10762
,Plates,100.0,-1.0,False,False,Genuine,
,Plates,100.0,-1.0,False,False,Fake,Report #10763
,Plates,100.0,-1.0,False,False,Fake,
,Plates,100.0,-1.0,False,False,,Report #10764
,Plates,100.0,-1.0,False,False,,
,Plates,100.0,0.0,True,True,Genuine,Report #10765
,Plates,100.0,0.0,True,True,Genuine,
,Plates,100.0,0.0,True,True,Fake,Report #10766
,Plates,100.0,0.0,True,True,Fake,
,Plates,100.0,0.0,True,True,,Report #10767
,Plates,100.0,0.0,True,True,,
,Plates,100.0,0.0,True,False,Genuine,Report #10768
,Plates,100.0,0.0,True,False,Genuine,
,Plates,100.0,0.0,True,False,Fake,Report #10769
,Plates,100.0,0.0,True,False,Fake,
,Plates,100.0,0.0,True,False,,Report #10770
,Plates,100.0,0.0,True,False,,
,Plates,100.0,0.0,False,True,Genuine,Report #10771
,Plates,100.0,0.0,False,True,Genuine,
,Plates,100.0,0.0,False,True,Fake,Report #10772
,Plates,100.0,0.0,False,True,Fake,
,Plates,100.0,0.0,False,True,,Report #10773
,Plates,100.0,0.0,False,True,,
,Plates,100.0,0.0,False,False,Genuine,Report #10774
,Plates,100.0,0.0,False,False,Genuine,
,Plates,100.0,0.0,False,False,Fake,Report #10775
,Plates,100.0,0.0,False,False,Fake,
,Plates,100.0,0.0,False,False,,Report #10776
,Plates,100.0,0.0,False,False,,
,Plates,100.0,19.0,True,True,Genuine,Report #10777
,Plates,100.0,19.0,True,True,Genuine,
,Plates,100.0,19.0,True,True,Fake,Report #10778
,Plates,100.0,19.0,True,True,Fake,
,Plates,100.0,19.0,True,True,,Report #10779
,Plates,100.0,19.0,True,True,,
,Plates,100.0,19.0,True,False,Genuine,Report #10780
,Plates,100.0,19.0,True,False,Genuine,
,Plates,100.0,19.0,True,False,Fake,Report #10781
,Plates,100.0,19.0,True,False,Fake,
,Plates,100.0,19.0,True,False,,Report #10782
,Plates,100.0,19.0,True,False,,
,Plates,100.0,19.0,False,True,Genuine,Report #10783
,Plates,100.0,19.0,False,True,Genuine,
,Plates,100.0,19.0,False,True,Fake,Report #10784
,Plates,100.0,19.0,False,True,Fake,
,Plates,100.0,19.0,False,True,,Report #10785
,Plates,100.0,19.0,False,True,,
,Plates,100.0,19.0,False,False,Genuine,Report #10786
,Plates,100.0,19.0,False,False,Genuine,
,Plates,100.0,19.0,False,False,Fake,Report #10787
,Plates,100.0,19.0,False,False,Fake,
,Plates,100.0,19.0,False,False,,Report #10788
,Plates,100.0,19.0,False,False,,
,Plates,100.0,20.0,True,True,Genuine,Report #10789
,Plates,100.0,20.0,True,True,Genuine,
,Plates,100.0,20.0,True,True,Fake,Report #10790
,Plates,100.0,20.0,True,True,Fake,
,Plates,100.0,20.0,True,True,,Report #10791
,Plates,100.0,20.0,True,True,,
,Plates,100.0,20.0,True,False,Genuine,Report #10792
,Plates,100.0,20.0,True,False,Genuine,
,Plates,100.0,20.0,True,False,Fake,Report #10793
,Plates,100.0,20.0,True,False,Fake,
,Plates,100.0,20.0,True,False,,Report #10794
,Plates,100.0,20.0,True,False,,
,Plates,100.0,20.0,False,True,Genuine,Report #10795
,Plates,100.0,20.0,False,True,Genuine,
,Plates,100.0,20.0,False,True,Fake,Report #10796
,Plates,100.0,20.0,False,True,Fake,
,Plates,100.0,20.0,False,True,,Report #10797
,Plates,100.0,20.0,False,True,,
,Plates,100.0,20.0,False,False,Genuine,Report #10798
,Plates,100.0,20.0,False,False,Genuine,
,Plates,100.0,20.0,False,False,Fake,Report #10799
,Plates,100.0,20.0,False,False,Fake,
,Plates,100.0,20.0,False,False,,Report #10800
,Plates,100.0,20.0,False,False,,
,Plates,100.0,21.0,True,True,Genuine,Report #10801
,Plates,100.0,21.0,True,True,Genuine,
,Plates,100.0,21.0,True,True,Fake,Report #10802
,Plates,100.0,21.0,True,True,Fake,
,Plates,100.0,21.0,True,True,,Report #10803
,Plates,100.0,21.0,True,True,,
,Plates,100.0,21.0,True,False,Genuine,Report #10804
,Plates,100.0,21.0,True,False,Genuine,
,Plates,100.0,21.0,True,False,Fake,Report #10805
,Plates,100.0,21.0,True,False,Fake,
,Plates,100.0,21.0,True,False,,Report #10806
,Plates,100.0,21.0,True,False,,
,Plates,100.0,21.0,False,True,Genuine,Report #10807
,Plates,100.0,21.0,False,True,Genuine,
,Plates,100.0,21.0,False,True,Fake,Report #10808
,Plates,100.0,21.0,False,True,Fake,
,Plates,100.0,21.0,False,True,,Report #10809
,Plates,100.0,21.0,False,True,,
,Plates,100.0,21.0,False,False,Genuine,Report #10810
,Plates,100.0,21.0,False,False,Genuine,
,Plates,100.0,21.0,False,False,Fake,Report #10811
,Plates,100.0,21.0,False,False,Fake,
,Plates,100.0,21.0,False,False,,Report #10812
,Plates,100.0,21.0,False,False,,
,Plates,100.0,99.0,True,True,Genuine,Report #10813
,Plates,100.0,99.0,True,True,Genuine,
,Plates,100.0,99.0,True,True,Fake,Report #10814
,Plates,100.0,99.0,True,True,Fake,
,Plates,100.0,99.0,True,True,,Report #10815
,Plates,100.0,99.0,True,True,,
,Plates,100.0,99.0,True,False,Genuine,Report #10816
,Plates,100.0,99.0,True,False,Genuine,
,Plates,100.0,99.0,True,False,Fake,Report #10817
,Plates,100.0,99.0,True,False,Fake,
,Plates,100.0,99.0,True,False,,Report #10818
,Plates,100.0,99.0,True,False,,
,Plates,100.0,99.0,False,True,Genuine,Report #10819
,Plates,100.0,99.0,False,True,Genuine,
,Plates,100.0,99.0,False,True,Fake,Report #10820
,Plates,100.0,99.0,False,True,Fake,
,Plates,100.0,99.0,False,True,,Report #10821
,Plates,100.0,99.0,False,True,,
,Plates,100.0,99.0,False,False,Genuine,Report #10822
,Plates,100.0,99.0,False,False,Genuine,
,Plates,100.0,99.0,False,False,Fake,Report #10823
,Plates,100.0,99.0,False,False,Fake,
,Plates,100.0,99.0,False,False,,Report #10824
,Plates,100.0,99.0,False,False,,
,Plates,100.0,100.0,True,True,Genuine,Report #10825
,Plates,100.0,100.0,True,True,Genuine,
,Plates,100.0,100.0,True,True,Fake,Report #10826
,Plates,100.0,100.0,True,True,Fake,
,Plates,100.0,100.0,True,True,,Report #10827
,Plates,100.0,100.0,True,True,,
,Plates,100.0,100.0,True,False,Genuine,Report #10828
,Plates,100.0,100.0,True,False,Genuine,
,Plates,100.0,100.0,True,False,Fake,Report #10829
,Plates,100.0,100.0,True,False,Fake,
,Plates,100.0,100.0,True,False,,Report #10830
,Plates,100.0,100.0,True,False,,
,Plates,100.0,100.0,False,True,Genuine,Report #10831
,Plates,100.0,100.0,False,True,Genuine,
,Plates,100.0,100.0,False,True,Fake,Report #10832
,Plates,100.0,100.0,False,True,Fake,
,Plates,100.0,100.0,False,True,,Report #10833
,Plates,100.0,100.0,False,True,,
,Plates,100.0,100.0,False,False,Genuine,Report #10834
,Plates,100.0,100.0,False,False,Genuine,
,Plates,100.0,100.0,False,False,Fake,Report #10835
,Plates,100.0,100.0,False,False,Fake,
,Plates,100.0,100.0,False,False,,Report #10836
,Plates,100.0,100.0,False,False,,
,Plates,100.0,101.0,True,True,Genuine,Report #10837
,Plates,100.0,101.0,True,True,Genuine,
,Plates,100.0,101.0,True,True,Fake,Report #10838
,Plates,100.0,101.0,True,True,Fake,
,Plates,100.0,101.0,True,True,,Report #10839
,Plates,100.0,101.0,True,True,,
,Plates,100.0,101.0,True,False,Genuine,Report #10840
,Plates,100.0,101.0,True,False,Genuine,
,Plates,100.0,101.0,True,False,Fake,Report #10841
,Plates,100.0,101.0,True,False,Fake,
,Plates,100.0,101.0,True,False,,Report #10842
,Plates,100.0,101.0,True,False,,
,Plates,100.0,101.0,False,True,Genuine,Report #10843
,Plates,100.0,101.0,False,True,Genuine,
,Plates,100.0,101.0,False,True,Fake,Report #10844
,Plates,100.0,101.0,False,True,Fake,
,Plates,100.0,101.0,False,True,,Report #10845
,Plates,100.0,101.0,False,True,,
,Plates,100.0,101.0,False,False,Genuine,Report #10846
,Plates,100.0,101.0,False,False,Genuine,
,Plates,100.0,101.0,False,False,Fake,Report #10847
,Plates,100.0,101.0,False,False,Fake,
,Plates,100.0,101.0,False,False,,Report #10848
,Plates,100.0,101.0,False,False,,
,Plates,1000.0,-1.0,True,True,Genuine,Report #10849
,Plates,1000.0,-1.0,True,True,Genuine,
,Plates,1000.0,-1.0,True,True,Fake,Report #10850
,Plates,1000.0,-1.0,True,True,Fake,
,Plates,1000.0,-1.0,True,True,,Report #10851
,Plates,1000.0,-1.0,True,True,,
,Plates,1000.0,-1.0,True,False,Genuine,Report #10852
,Plates,1000.0,-1.0,True,False,Genuine,
,Plates,1000.0,-1.0,True,False,Fake,Report #10853
,Plates,1000.0,-1.0,True,False,Fake,
,Plates,1000.0,-1.0,True,False,,Report #10854
,Plates,1000.0,-1.0,True,False,,
,Plates,1000.0,-1.0,False,True,Genuine,Report #10855
,Plates,1000.0,-1.0,False,True,Genuine,
,Plates,1000.0,-1.0,False,True,Fake,Report #10856
,Plates,1000.0,-1.0,False,True,Fake,
,Plates,1000.0,-1.0,False,True,,Report #10857
,Plates,1000.0,-1.0,False,True,,
,Plates,1000.0,-1.0,False,False,Genuine,Report #10858
,Plates,1000.0,-1.0,False,False,Genuine,
,Plates,1000.0,-1.0,False,False,Fake,Report #10859
,Plates,1000.0,-1.0,False,False,Fake,
,Plates,1000.0,-1.0,False,False,,Report #10860
,Plates,1000.0,-1.0,False,False,,
,Plates,1000.0,0.0,True,True,Genuine,Report #10861
,Plates,1000.0,0.0,True,True,Genuine,
,Plates,1000.0,0.0,True,True,Fake,Report #10862
,Plates,1000.0,0.0,True,True,Fake,
,Plates,1000.0,0.0,True,True,,Report #10863
,Plates,1000.0,0.0,True,True,,
,Plates,1000.0,0.0,True,False,Genuine,Report #10864
,Plates,1000.0,0.0,True,False,Genuine,
,Plates,1000.0,0.0,True,False,Fake,Report #10865
,Plates,1000.0,0.0,True,False,Fake,
,Plates,1000.0,0.0,True,False,,Report #10866
,Plates,1000.0,0.0,True,False,,
,Plates,1000.0,0.0,False,True,Genuine,Report #10867
,Plates,1000.0,0.0,False,True,Genuine,
,Plates,1000.0,0.0,False,True,Fake,Report #10868
,Plates,1000.0,0.0,False,True,Fake,
,Plates,1000.0,0.0,False,True,,Report #10869
,Plates,1000.0,0.0,False,True,,
,Plates,1000.0,0.0,False,False,Genuine,Report #10870
,Plates,1000.0,0.0,False,False,Genuine,
,Plates,1000.0,0.0,False,False,Fake,Report #10871
,Plates,1000.0,0.0,False,False,Fake,
,Plates,1000.0,0.0,False,False,,Report #10872
,Plates,1000.0,0.0,False,False,,
,Plates,1000.0,19.0,True,True,Genuine,Report #10873
,Plates,1000.0,19.0,True,True,Genuine,
,Plates,1000.0,19.0,True,True,Fake,Report #10874
,Plates,1000.0,19.0,True,True,Fake,
,Plates,1000.0,19.0,True,True,,Report #10875
,Plates,1000.0,19.0,True,True,,
,Plates,1000.0,19.0,True,False,Genuine,Report #10876
,Plates,1000.0,19.0,True,False,Genuine,
,Plates,1000.0,19.0,True,False,Fake,Report #10877
,Plates,1000.0,19.0,True,False,Fake,
,Plates,1000.0,19.0,True,False,,Report #10878
,Plates,1000.0,19.0,True,False,,
,Plates,1000.0,19.0,False,True,Genuine,Report #10879
,Plates,1000.0,19.0,False,True,Genuine,
,Plates,1000.0,19.0,False,True,Fake,Report #10880
,Plates,1000.0,19.0,False,True,Fake,
,Plates,1000.0,19.0,False,True,,Report #10881
,Plates,1000.0,19.0,False,True,,
,Plates,1000.0,19.0,False,False,Genuine,Report #10882
,Plates,1000.0,19.0,False,False,Genuine,
,Plates,1000.0,19.0,False,False,Fake,Report #10883
,Plates,1000.0,19.0,False,False,Fake,
,Plates,1000.0,19.0,False,False,,Report #10884
,Plates,1000.0,19.0,False,False,,
,Plates,1000.0,20.0,True,True,Genuine,Report #10885
,Plates,1000.0,20.0,True,True,Genuine,
,Plates,1000.0,20.0,True,True,Fake,Report #10886
,Plates,1000.0,20.0,True,True,Fake,
,Plates,1000.0,20.0,True,True,,Report #10887
,Plates,1000.0,20.0,True,True,,
,Plates,1000.0,20.0,True,False,Genuine,Report #10888
,Plates,1000.0,20.0,True,False,Genuine,
,Plates,1000.0,20.0,True,False,Fake,Report #10889
,Plates,1000.0,20.0,True,False,Fake,
,Plates,1000.0,20.0,True,False,,Report #10890
,Plates,1000.0,20.0,True,False,,
,Plates,1000.0,20.0,False,True,Genuine,Report #10891
,Plates,1000.0,20.0,False,True,Genuine,
,Plates,1000.0,20.0,False,True,Fake,Report #10892
,Plates,1000.0,20.0,False,True,Fake,
,Plates,1000.0,20.0,False,True,,Report #10893
,Plates,1000.0,20.0,False,True,,
,Plates,1000.0,20.0,False,False,Genuine,Report #10894
,Plates,1000.0,20.0,False,False,Genuine,
,Plates,1000.0,20.0,False,False,Fake,Report #10895
,Plates,1000.0,20.0,False,False,Fake,
,Plates,1000.0,20.0,False,False,,Report #10896
,Plates,1000.0,20.0,False,False,,
,Plates,1000.0,21.0,True,True,Genuine,Report #10897
,Plates,1000.0,21.0,True,True,Genuine,
,Plates,1000.0,21.0,True,True,Fake,Report #10898
,Plates,1000.0,21.0,True,True,Fake,
,Plates,1000.0,21.0,True,True,,Report #10899
,Plates,1000.0,21.0,True,True,,
,Plates,1000.0,21.0,True,False,Genuine,Report #10900
,Plates,1000.0,21.0,True,False,Genuine,
,Plates,1000.0,21.0,True,False,Fake,Report #10901
,Plates,1000.0,21.0,True,False,Fake,
,Plates,1000.0,21.0,True,False,,Report #10902
,Plates,1000.0,21.0,True,False,,
,Plates,1000.0,21.0,False,True,Genuine,Report #10903
,Plates,1000.0,21.0,False,True,Genuine,
,Plates,1000.0,21.0,False,True,Fake,Report #10904
,Plates,1000.0,21.0,False,True,Fake,
,Plates,1000.0,21.0,False,True,,Report #10905
,Plates,1000.0,21.0,False,True,,
,Plates,1000.0,21.0,False,False,Genuine,Report #10906
,Plates,1000.0,21.0,False,False,Genuine,
,Plates,1000.0,21.0,False,False,Fake,Report #10907
,Plates,1000.0,21.0,False,False,Fake,
,Plates,1000.0,21.0,False,False,,Report #10908
,Plates,1000.0,21.0,False,False,,
,Plates,1000.0,99.0,True,True,Genuine,Report #10909
,Plates,1000.0,99.0,True,True,Genuine,
,Plates,1000.0,99.0,True,True,Fake,Report #10910
,Plates,1000.0,99.0,True,True,Fake,
,Plates,1000.0,99.0,True,True,,Report #10911
,Plates,1000.0,99.0,True,True,,
,Plates,1000.0,99.0,True,False,Genuine,Report #10912
,Plates,1000.0,99.0,True,False,Genuine,
,Plates,1000.0,99.0,True,False,Fake,Report #10913
,Plates,1000.0,99.0,True,False,Fake,
,Plates,1000.0,99.0,True,False,,Report #10914
,Plates,1000.0,99.0,True,False,,
,Plates,1000.0,99.0,False,True,Genuine,Report #10915
,Plates,1000.0,99.0,False,True,Genuine,
,Plates,1000.0,99.0,False,True,Fake,Report #10916
,Plates,1000.0,99.0,False,True,Fake,
,Plates,1000.0,99.0,False,True,,Report #10917
,Plates,1000.0,99.0,False,True,,
,Plates,1000.0,99.0,False,False,Genuine,Report #10918
,Plates,1000.0,99.0,False,False,Genuine,
,Plates,1000.0,99.0,False,False,Fake,Report #10919
,Plates,1000.0,99.0,False,False,Fake,
,Plates,1000.0,99.0,False,False,,Report #10920
,Plates,1000.0,99.0,False,False,,
,Plates,1000.0,100.0,True,True,Genuine,Report #10921
,Plates,1000.0,100.0,True,True,Genuine,
,Plates,1000.0,100.0,True,True,Fake,Report #10922
,Plates,1000.0,100.0,True,True,Fake,
,Plates,1000.0,100.0,True,True,,Report #10923
,Plates,1000.0,100.0,True,True,,
,Plates,1000.0,100.0,True,False,Genuine,Report #10924
,Plates,1000.0,100.0,True,False,Genuine,
,Plates,1000.0,100.0,True,False,Fake,Report #10925
,Plates,1000.0,100.0,True,False,Fake,
,Plates,1000.0,100.0,True,False,,Report #10926
,Plates,1000.0,100.0,True,False,,
,Plates,1000.0,100.0,False,True,Genuine,Report #10927
,Plates,1000.0,100.0,False,True,Genuine,
,Plates,1000.0,100.0,False,True,Fake,Report #10928
,Plates,1000.0,100.0,False,True,Fake,
,Plates,1000.0,100.0,False,True,,Report #10929
,Plates,1000.0,100.0,False,True,,
,Plates,1000.0,100.0,False,False,Genuine,Report #10930
,Plates,1000.0,100.0,False,False,Genuine,
,Plates,1000.0,100.0,False,False,Fake,Report #10931
,Plates,1000.0,100.0,False,False,Fake,
,Plates,1000.0,100.0,False,False,,Report #10932
,Plates,1000.0,100.0,False,False,,
,Plates,1000.0,101.0,True,True,Genuine,Report #10933
,Plates,1000.0,101.0,True,True,Genuine,
,Plates,1000.0,101.0,True,True,Fake,Report #10934
,Plates,1000.0,101.0,True,True,Fake,
,Plates,1000.0,101.0,True,True,,Report #10935
,Plates,1000.0,101.0,True,True,,
,Plates,1000.0,101.0,True,False,Genuine,Report #10936
,Plates,1000.0,101.0,True,False,Genuine,
,Plates,1000.0,101.0,True,False,Fake,Report #10937
,Plates,1000.0,101.0,True,False,Fake,
,Plates,1000.0,101.0,True,False,,Report #10938
,Plates,1000.0,101.0,True,False,,
,Plates,1000.0,101.0,False,True,Genuine,Report #10939
,Plates,1000.0,101.0,False,True,Genuine,
,Plates,1000.0,101.0,False,True,Fake,Report #10940
,Plates,1000.0,101.0,False,True,Fake,
,Plates,1000.0,101.0,False,True,,Report #10941
,Plates,1000.0,101.0,False,True,,
,Plates,1000.0,101.0,False,False,Genuine,Report #10942
,Plates,1000.0,101.0,False,False,Genuine,
,Plates,1000.0,101.0,False,False,Fake,Report #10943
,Plates,1000.0,101.0,False,False,Fake,
,Plates,1000.0,101.0,False,False,,Report #10944
,Plates,1000.0,101.0,False,False,,
,Plates,1000000.0,-1.0,True,True,Genuine,Report #10945
,Plates,1000000.0,-1.0,True,True,Genuine,
,Plates,1000000.0,-1.0,True,True,Fake,Report #10946
,Plates,1000000.0,-1.0,True,True,Fake,
,Plates,1000000.0,-1.0,True,True,,Report #10947
,Plates,1000000.0,-1.0,True,True,,
,Plates,1000000.0,-1.0,True,False,Genuine,Report #10948
,Plates,1000000.0,-1.0,True,False,Genuine,
,Plates,1000000.0,-1.0,True,False,Fake,Report #10949
,Plates,1000000.0,-1.0,True,False,Fake,
,Plates,1000000.0,-1.0,True,False,,Report #10950
,Plates,1000000.0,-1.0,True,False,,
,Plates,1000000.0,-1.0,False,True,Genuine,Report #10951
,Plates,1000000.0,-1.0,False,True,Genuine,
,Plates,1000000.0,-1.0,False,True,Fake,Report #10952
,Plates,1000000.0,-1.0,False,True,Fake,
,Plates,1000000.0,-1.0,False,True,,Report #10953
,Plates,1000000.0,-1.0,False,True,,
,Plates,1000000.0,-1.0,False,False,Genuine,Report #10954
,Plates,1000000.0,-1.0,False,False,Genuine,
,Plates,1000000.0,-1.0,False,False,Fake,Report #10955
,Plates,1000000.0,-1.0,False,False,Fake,
,Plates,1000000.0,-1.0,False,False,,Report #10956
,Plates,1000000.0,-1.0,False,False,,
,Plates,1000000.0,0.0,True,True,Genuine,Report #10957
,Plates,1000000.0,0.0,True,True,Genuine,
,Plates,1000000.0,0.0,True,True,Fake,Report #10958
,Plates,1000000.0,0.0,True,True,Fake,
,Plates,1000000.0,0.0,True,True,,Report #10959
,Plates,1000000.0,0.0,True,True,,
,Plates,1000000.0,0.0,True,False,Genuine,Report #10960
,Plates,1000000.0,0.0,True,False,Genuine,
,Plates,1000000.0,0.0,True,False,Fake,Report #10961
,Plates,1000000.0,0.0,True,False,Fake,
,Plates,1000000.0,0.0,True,False,,Report #10962
,Plates,1000000.0,0.0,True,False,,
,Plates,1000000.0,0.0,False,True,Genuine,Report #10963
,Plates,1000000.0,0.0,False,True,Genuine,
,Plates,1000000.0,0.0,False,True,Fake,Report #10964
,Plates,1000000.0,0.0,False,True,Fake,
,Plates,1000000.0,0.0,False,True,,Report #10965
,Plates,1000000.0,0.0,False,True,,
,Plates,1000000.0,0.0,False,False,Genuine,Report #10966
,Plates,1000000.0,0.0,False,False,Genuine,
,Plates,1000000.0,0.0,False,False,Fake,Report #10967
,Plates,1000000.0,0.0,False,False,Fake,
,Plates,1000000.0,0.0,False,False,,Report #10968
,Plates,1000000.0,0.0,False,False,,
,Plates,1000000.0,19.0,True,True,Genuine,Report #10969
,Plates,1000000.0,19.0,True,True,Genuine,
,Plates,1000000.0,19.0,True,True,Fake,Report #10970
,Plates,1000000.0,19.0,True,True,Fake,
,Plates,1000000.0,19.0,True,True,,Report #10971
,Plates,1000000.0,19.0,True,True,,
,Plates,1000000.0,19.0,True,False,Genuine,Report #10972
,Plates,1000000.0,19.0,True,False,Genuine,
,Plates,1000000.0,19.0,True,False,Fake,Report #10973
,Plates,1000000.0,19.0,True,False,Fake,
,Plates,1000000.0,19.0,True,False,,Report #10974
,Plates,1000000.0,19.0,True,False,,
,Plates,1000000.0,19.0,False,True,Genuine,Report #10975
,Plates,1000000.0,19.0,False,True,Genuine,
,Plates,1000000.0,19.0,False,True,Fake,Report #10976
,Plates,1000000.0,19.0,False,True,Fake,
,Plates,1000000.0,19.0,False,True,,Report #10977
,Plates,1000000.0,19.0,False,True,,
,Plates,1000000.0,19.0,False,False,Genuine,Report #10978
,Plates,1000000.0,19.0,False,False,Genuine,
,Plates,1000000.0,19.0,False,False,Fake,Report #10979
,Plates,1000000.0,19.0,False,False,Fake,
,Plates,1000000.0,19.0,False,False,,Report #10980
,Plates,1000000.0,19.0,False,False,,
,Plates,1000000.0,20.0,True,True,Genuine,Report #10981
,Plates,1000000.0,20.0,True,True,Genuine,
,Plates,1000000.0,20.0,True,True,Fake,Report #10982
,Plates,1000000.0,20.0,True,True,Fake,
,Plates,1000000.0,20.0,True,True,,Report #10983
,Plates,1000000.0,20.0,True,True,,
,Plates,1000000.0,20.0,True,False,Genuine,Report #10984
,Plates,1000000.0,20.0,True,False,Genuine,
,Plates,1000000.0,20.0,True,False,Fake,Report #10985
,Plates,1000000.0,20.0,True,False,Fake,
,Plates,1000000.0,20.0,True,False,,Report #10986
,Plates,1000000.0,20.0,True,False,,
,Plates,1000000.0,20.0,False,True,Genuine,Report #10987
,Plates,1000000.0,20.0,False,True,Genuine,
,Plates,1000000.0,20.0,False,True,Fake,Report #10988
,Plates,1000000.0,20.0,False,True,Fake,
,Plates,1000000.0,20.0,False,True,,Report #10989
,Plates,1000000.0,20.0,False,True,,
,Plates,1000000.0,20.0,False,False,Genuine,Report #10990
,Plates,1000000.0,20.0,False,False,Genuine,
,Plates,1000000.0,20.0,False,False,Fake,Report #10991
,Plates,1000000.0,20.0,False,False,Fake,
,Plates,1000000.0,20.0,False,False,,Report #10992
,Plates,1000000.0,20.0,False,False,,
,Plates,1000000.0,21.0,True,True,Genuine,Report #10993
,Plates,1000000.0,21.0,True,True,Genuine,
,Plates,1000000.0,21.0,True,True,Fake,Report #10994
,Plates,1000000.0,21.0,True,True,Fake,
,Plates,1000000.0,21.0,True,True,,Report #10995
,Plates,1000000.0,21.0,True,True,,
,Plates,1000000.0,21.0,True,False,Genuine,Report #10996
,Plates,1000000.0,21.0,True,False,Genuine,
,Plates,1000000.0,21.0,True,False,Fake,Report #10997
,Plates,1000000.0,21.0,True,False,Fake,
,Plates,1000000.0,21.0,True,False,,Report #10998
,Plates,1000000.0,21.0,True,False,,
,Plates,1000000.0,21.0,False,True,Genuine,Report #10999
,Plates,1000000.0,21.0,False,True,Genuine,
,Plates,1000000.0,21.0,False,True,Fake,Report #11000
,Plates,1000000.0,21.0,False,True,Fake,
,Plates,1000000.0,21.0,False,True,,Report #11001
,Plates,1000000.0,21.0,False,True,,
,Plates,1000000.0,21.0,False,False,Genuine,Report #11002
,Plates,1000000.0,21.0,False,False,Genuine,
,Plates,1000000.0,21.0,False,False,Fake,Report #11003
,Plates,1000000.0,21.0,False,False,Fake,
,Plates,1000000.0,21.0,False,False,,Report #11004
,Plates,1000000.0,21.0,False,False,,
,Plates,1000000.0,99.0,True,True,Genuine,Report #11005
,Plates,1000000.0,99.0,True,True,Genuine,
,Plates,1000000.0,99.0,True,True,Fake,Report #11006
,Plates,1000000.0,99.0,True,True,Fake,
,Plates,1000000.0,99.0,True,True,,Report #11007
,Plates,1000000.0,99.0,True,True,,
,Plates,1000000.0,99.0,True,False,Genuine,Report #11008
,Plates,1000000.0,99.0,True,False,Genuine,
,Plates,1000000.0,99.0,True,False,Fake,Report #11009
,Plates,1000000.0,99.0,True,False,Fake,
,Plates,1000000.0,99.0,True,False,,Report #11010
,Plates,1000000.0,99.0,True,False,,
,Plates,1000000.0,99.0,False,True,Genuine,Report #11011
,Plates,1000000.0,99.0,False,True,Genuine,
,Plates,1000000.0,99.0,False,True,Fake,Report #11012
,Plates,1000000.0,99.0,False,True,Fake,
,Plates,1000000.0,99.0,False,True,,Report #11013
,Plates,1000000.0,99.0,False,True,,
,Plates,1000000.0,99.0,False,False,Genuine,Report #11014
,Plates,1000000.0,99.0,False,False,Genuine,
,Plates,1000000.0,99.0,False,False,Fake,Report #11015
,Plates,1000000.0,99.0,False,False,Fake,
,Plates,1000000.0,99.0,False,False,,Report #11016
,Plates,1000000.0,99.0,False,False,,
,Plates,1000000.0,100.0,True,True,Genuine,Report #11017
,Plates,1000000.0,100.0,True,True,Genuine,
,Plates,1000000.0,100.0,True,True,Fake,Report #11018
,Plates,1000000.0,100.0,True,True,Fake,
,Plates,1000000.0,100.0,True,True,,Report #11019
,Plates,1000000.0,100.0,True,True,,
,Plates,1000000.0,100.0,True,False,Genuine,Report #11020
,Plates,1000000.0,100.0,True,False,Genuine,
,Plates,1000000.0,100.0,True,False,Fake,Report #11021
,Plates,1000000.0,100.0,True,False,Fake,
,Plates,1000000.0,100.0,True,False,,Report #11022
,Plates,1000000.0,100.0,True,False,,
,Plates,1000000.0,100.0,False,True,Genuine,Report #11023
,Plates,1000000.0,100.0,False,True,Genuine,
,Plates,1000000.0,100.0,False,True,Fake,Report #11024
,Plates,1000000.0,100.0,False,True,Fake,
,Plates,1000000.0,100.0,False,True,,Report #11025
,Plates,1000000.0,100.0,False,True,,
,Plates,1000000.0,100.0,False,False,Genuine,Report #11026
,Plates,1000000.0,100.0,False,False,Genuine,
,Plates,1000000.0,100.0,False,False,Fake,Report #11027
,Plates,1000000.0,100.0,False,False,Fake,
,Plates,1000000.0,100.0,False,False,,Report #11028
,Plates,1000000.0,100.0,False,False,,
,Plates,1000000.0,101.0,True,True,Genuine,Report #11029
,Plates,1000000.0,101.0,True,True,Genuine,
,Plates,1000000.0,101.0,True,True,Fake,Report #11030
,Plates,1000000.0,101.0,True,True,Fake,
,Plates,1000000.0,101.0,True,True,,Report #11031
,Plates,1000000.0,101.0,True,True,,
,Plates,1000000.0,101.0,True,False,Genuine,Report #11032
,Plates,1000000.0,101.0,True,False,Genuine,
,Plates,1000000.0,101.0,True,False,Fake,Report #11033
,Plates,1000000.0,101.0,True,False,Fake,
,Plates,1000000.0,101.0,True,False,,Report #11034
,Plates,1000000.0,101.0,True,False,,
,Plates,1000000.0,101.0,False,True,Genuine,Report #11035
,Plates,1000000.0,101.0,False,True,Genuine,
,Plates,1000000.0,101.0,False,True,Fake,Report #11036
,Plates,1000000.0,101.0,False,True,Fake,
,Plates,1000000.0,101.0,False,True,,Report #11037
,Plates,1000000.0,101.0,False,True,,
,Plates,1000000.0,101.0,False,False,Genuine,Report #11038
,Plates,1000000.0,101.0,False,False,Genuine,
,Plates,1000000.0,101.0,False,False,Fake,Report #11039
,Plates,1000000.0,101.0,False,False,Fake,
,Plates,1000000.0,101.0,False,False,,Report #11040
,Plates,1000000.0,101.0,False,False,,
,Furniture,0.0,-1.0,True,True,Genuine,Report #11041
,Furniture,0.0,-1.0,True,True,Genuine,
,Furniture,0.0,-1.0,True,True,Fake,Report #11042
,Furniture,0.0,-1.0,True,True,Fake,
,Furniture,0.0,-1.0,True,True,,Report #11043
,Furniture,0.0,-1.0,True,True,,
,Furniture,0.0,-1.0,True,False,Genuine,Report #11044
,Furniture,0.0,-1.0,True,False,Genuine,
,Furniture,0.0,-1.0,True,False,Fake,Report #11045
,Furniture,0.0,-1.0,True,False,Fake,
,Furniture,0.0,-1.0,True,False,,Report #11046
,Furniture,0.0,-1.0,True,False,,
,Furniture,0.0,-1.0,False,True,Genuine,Report #11047
,Furniture,0.0,-1.0,False,True,Genuine,
,Furniture,0.0,-1.0,False,True,Fake,Report #11048
,Furniture,0.0,-1.0,False,True,Fake,
,Furniture,0.0,-1.0,False,True,,Report #11049
,Furniture,0.0,-1.0,False,True,,
,Furniture,0.0,-1.0,False,False,Genuine,Report #11050
,Furniture,0.0,-1.0,False,False,Genuine,
,Furniture,0.0,-1.0,False,False,Fake,Report #11051
,Furniture,0.0,-1.0,False,False,Fake,
,Furniture,0.0,-1.0,False,False,,Report #11052
,Furniture,0.0,-1.0,False,False,,
,Furniture,0.0,0.0,True,True,Genuine,Report #11053
,Furniture,0.0,0.0,True,True,Genuine,
,Furniture,0.0,0.0,True,True,Fake,Report #11054
,Furniture,0.0,0.0,True,True,Fake,
,Furniture,0.0,0.0,True,True,,Report #11055
,Furniture,0.0,0.0,True,True,,
,Furniture,0.0,0.0,True,False,Genuine,Report #11056
,Furniture,0.0,0.0,True,False,Genuine,
,Furniture,0.0,0.0,True,False,Fake,Report #11057
,Furniture,0.0,0.0,True,False,Fake,
,Furniture,0.0,0.0,True,False,,Report #11058
,Furniture,0.0,0.0,True,False,,
,Furniture,0.0,0.0,False,True,Genuine,Report #11059
,Furniture,0.0,0.0,False,True,Genuine,
,Furniture,0.0,0.0,False,True,Fake,Report #11060
,Furniture,0.0,0.0,False,True,Fake,
,Furniture,0.0,0.0,False,True,,Report #11061
,Furniture,0.0,0.0,False,True,,
,Furniture,0.0,0.0,False,False,Genuine,Report #11062
,Furniture,0.0,0.0,False,False,Genuine,
,Furniture,0.0,0.0,False,False,Fake,Report #11063
,Furniture,0.0,0.0,False,False,Fake,
,Furniture,0.0,0.0,False,False,,Report #11064
,Furniture,0.0,0.0,False,False,,
,Furniture,0.0,19.0,True,True,Genuine,Report #11065
,Furniture,0.0,19.0,True,True,Genuine,
,Furniture,0.0,19.0,True,True,Fake,Report #11066
,Furniture,0.0,19.0,True,True,Fake,
,Furniture,0.0,19.0,True,True,,Report #11067
,Furniture,0.0,19.0,True,True,,
,Furniture,0.0,19.0,True,False,Genuine,Report #11068
,Furniture,0.0,19.0,True,False,Genuine,
,Furniture,0.0,19.0,True,False,Fake,Report #11069
,Furniture,0.0,19.0,True,False,Fake,
,Furniture,0.0,19.0,True,False,,Report #11070
,Furniture,0.0,19.0,True,False,,
,Furniture,0.0,19.0,False,True,Genuine,Report #11071
,Furniture,0.0,19.0,False,True,Genuine,
,Furniture,0.0,19.0,False,True,Fake,Report #11072
,Furniture,0.0,19.0,False,True,Fake,
,Furniture,0.0,19.0,False,True,,Report #11073
,Furniture,0.0,19.0,False,True,,
,Furniture,0.0,19.0,False,False,Genuine,Report #11074
,Furniture,0.0,19.0,False,False,Genuine,
,Furniture,0.0,19.0,False,False,Fake,Report #11075
,Furniture,0.0,19.0,False,False,Fake,
,Furniture,0.0,19.0,False,False,,Report #11076
,Furniture,0.0,19.0,False,False,,
,Furniture,0.0,20.0,True,True,Genuine,Report #11077
,Furniture,0.0,20.0,True,True,Genuine,
,Furniture,0.0,20.0,True,True,Fake,Report #11078
,Furniture,0.0,20.0,True,True,Fake,
,Furniture,0.0,20.0,True,True,,Report #11079
,Furniture,0.0,20.0,True,True,,
,Furniture,0.0,20.0,True,False,Genuine,Report #11080
,Furniture,0.0,20.0,True,False,Genuine,
,Furniture,0.0,20.0,True,False,Fake,Report #11081
,Furniture,0.0,20.0,True,False,Fake,
,Furniture,0.0,20.0,True,False,,Report #11082
,Furniture,0.0,20.0,True,False,,
,Furniture,0.0,20.0,False,True,Genuine,Report #11083
,Furniture,0.0,20.0,False,True,Genuine,
,Furniture,0.0,20.0,False,True,Fake,Report #11084
,Furniture,0.0,20.0,False,True,Fake,
,Furniture,0.0,20.0,False,True,,Report #11085
,Furniture,0.0,20.0,False,True,,
,Furniture,0.0,20.0,False,False,Genuine,Report #11086
,Furniture,0.0,20.0,False,False,Genuine,
,Furniture,0.0,20.0,False,False,Fake,Report #11087
,Furniture,0.0,20.0,False,False,Fake,
,Furniture,0.0,20.0,False,False,,Report #11088
,Furniture,0.0,20.0,False,False,,
,Furniture,0.0,21.0,True,True,Genuine,Report #11089
,Furniture,0.0,21.0,True,True,Genuine,
,Furniture,0.0,21.0,True,True,Fake,Report #11090
,Furniture,0.0,21.0,True,True,Fake,
,Furniture,0.0,21.0,True,True,,Report #11091
,Furniture,0.0,21.0,True,True,,
,Furniture,0.0,21.0,True,False,Genuine,Report #11092
,Furniture,0.0,21.0,True,False,Genuine,
,Furniture,0.0,21.0,True,False,Fake,Report #11093
,Furniture,0.0,21.0,True,False,Fake,
,Furniture,0.0,21.0,True,False,,Report #11094
,Furniture,0.0,21.0,True,False,,
,Furniture,0.0,21.0,False,True,Genuine,Report #11095
,Furniture,0.0,21.0,False,True,Genuine,
,Furniture,0.0,21.0,False,True,Fake,Report #11096
,Furniture,0.0,21.0,False,True,Fake,
,Furniture,0.0,21.0,False,True,,Report #11097
,Furniture,0.0,21.0,False,True,,
,Furniture,0.0,21.0,False,False,Genuine,Report #11098
,Furniture,0.0,21.0,False,False,Genuine,
,Furniture,0.0,21.0,False,False,Fake,Report #11099
,Furniture,0.0,21.0,False,False,Fake,
,Furniture,0.0,21.0,False,False,,Report #11100
,Furniture,0.0,21.0,False,False,,
,Furniture,0.0,99.0,True,True,Genuine,Report #11101
,Furniture,0.0,99.0,True,True,Genuine,
,Furniture,0.0,99.0,True,True,Fake,Report #11102
,Furniture,0.0,99.0,True,True,Fake,
,Furniture,0.0,99.0,True,True,,Report #11103
,Furniture,0.0,99.0,True,True,,
,Furniture,0.0,99.0,True,False,Genuine,Report #11104
,Furniture,0.0,99.0,True,False,Genuine,
,Furniture,0.0,99.0,True,False,Fake,Report #11105
,Furniture,0.0,99.0,True,False,Fake,
,Furniture,0.0,99.0,True,False,,Report #11106
,Furniture,0.0,99.0,True,False,,
,Furniture,0.0,99.0,False,True,Genuine,Report #11107
,Furniture,0.0,99.0,False,True,Genuine,
,Furniture,0.0,99.0,False,True,Fake,Report #11108
,Furniture,0.0,99.0,False,True,Fake,
,Furniture,0.0,99.0,False,True,,Report #11109
,Furniture,0.0,99.0,False,True,,
,Furniture,0.0,99.0,False,False,Genuine,Report #11110
,Furniture,0.0,99.0,False,False,Genuine,
,Furniture,0.0,99.0,False,False,Fake,Report #11111
,Furniture,0.0,99.0,False,False,Fake,
,Furniture,0.0,99.0,False,False,,Report #11112
,Furniture,0.0,99.0,False,False,,
,Furniture,0.0,100.0,True,True,Genuine,Report #11113
,Furniture,0.0,100.0,True,True,Genuine,
,Furniture,0.0,100.0,True,True,Fake,Report #11114
,Furniture,0.0,100.0,True,True,Fake,
,Furniture,0.0,100.0,True,True,,Report #11115
,Furniture,0.0,100.0,True,True,,
,Furniture,0.0,100.0,True,False,Genuine,Report #11116
,Furniture,0.0,100.0,True,False,Genuine,
,Furniture,0.0,100.0,True,False,Fake,Report #11117
,Furniture,0.0,100.0,True,False,Fake,
,Furniture,0.0,100.0,True,False,,Report #11118
,Furniture,0.0,100.0,True,False,,
,Furniture,0.0,100.0,False,True,Genuine,Report #11119
,Furniture,0.0,100.0,False,True,Genuine,
,Furniture,0.0,100.0,False,True,Fake,Report #11120
,Furniture,0.0,100.0,False,True,Fake,
,Furniture,0.0,100.0,False,True,,Report #11121
,Furniture,0.0,100.0,False,True,,
,Furniture,0.0,100.0,False,False,Genuine,Report #11122
,Furniture,0.0,100.0,False,False,Genuine,
,Furniture,0.0,100.0,False,False,Fake,Report #11123
,Furniture,0.0,100.0,False,False,Fake,
,Furniture,0.0,100.0,False,False,,Report #11124
,Furniture,0.0,100.0,False,False,,
,Furniture,0.0,101.0,True,True,Genuine,Report #11125
,Furniture,0.0,101.0,True,True,Genuine,
,Furniture,0.0,101.0,True,True,Fake,Report #11126
,Furniture,0.0,101.0,True,True,Fake,
,Furniture,0.0,101.0,True,True,,Report #11127
,Furniture,0.0,101.0,True,True,,
,Furniture,0.0,101.0,True,False,Genuine,Report #11128
,Furniture,0.0,101.0,True,False,Genuine,
,Furniture,0.0,101.0,True,False,Fake,Report #11129
,Furniture,0.0,101.0,True,False,Fake,
,Furniture,0.0,101.0,True,False,,Report #11130
,Furniture,0.0,101.0,True,False,,
,Furniture,0.0,101.0,False,True,Genuine,Report #11131
,Furniture,0.0,101.0,False,True,Genuine,
,Furniture,0.0,101.0,False,True,Fake,Report #11132
,Furniture,0.0,101.0,False,True,Fake,
,Furniture,0.0,101.0,False,True,,Report #11133
,Furniture,0.0,101.0,False,True,,
,Furniture,0.0,101.0,False,False,Genuine,Report #11134
,Furniture,0.0,101.0,False,False,Genuine,
,Furniture,0.0,101.0,False,False,Fake,Report #11135
,Furniture,0.0,101.0,False,False,Fake,
,Furniture,0.0,101.0,False,False,,Report #11136
,Furniture,0.0,101.0,False,False,,
,Furniture,1.0,-1.0,True,True,Genuine,Report #11137
,Furniture,1.0,-1.0,True,True,Genuine,
,Furniture,1.0,-1.0,True,True,Fake,Report #11138
,Furniture,1.0,-1.0,True,True,Fake,
,Furniture,1.0,-1.0,True,True,,Report #11139
,Furniture,1.0,-1.0,True,True,,
,Furniture,1.0,-1.0,True,False,Genuine,Report #11140
,Furniture,1.0,-1.0,True,False,Genuine,
,Furniture,1.0,-1.0,True,False,Fake,Report #11141
,Furniture,1.0,-1.0,True,False,Fake,
,Furniture,1.0,-1.0,True,False,,Report #11142
,Furniture,1.0,-1.0,True,False,,
,Furniture,1.0,-1.0,False,True,Genuine,Report #11143
,Furniture,1.0,-1.0,False,True,Genuine,
,Furniture,1.0,-1.0,False,True,Fake,Report #11144
,Furniture,1.0,-1.0,False,True,Fake,
,Furniture,1.0,-1.0,False,True,,Report #11145
,Furniture,1.0,-1.0,False,True,,
,Furniture,1.0,-1.0,False,False,Genuine,Report #11146
,Furniture,1.0,-1.0,False,False,Genuine,
,Furniture,1.0,-1.0,False,False,Fake,Report #11147
,Furniture,1.0,-1.0,False,False,Fake,
,Furniture,1.0,-1.0,False,False,,Report #11148
,Furniture,1.0,-1.0,False,False,,
,Furniture,1.0,0.0,True,True,Genuine,Report #11149
,Furniture,1.0,0.0,True,True,Genuine,
,Furniture,1.0,0.0,True,True,Fake,Report #11150
,Furniture,1.0,0.0,True,True,Fake,
,Furniture,1.0,0.0,True,True,,Report #11151
,Furniture,1.0,0.0,True,True,,
,Furniture,1.0,0.0,True,False,Genuine,Report #11152
,Furniture,1.0,0.0,True,False,Genuine,
,Furniture,1.0,0.0,True,False,Fake,Report #11153
,Furniture,1.0,0.0,True,False,Fake,
,Furniture,1.0,0.0,True,False,,Report #11154
,Furniture,1.0,0.0,True,False,,
,Furniture,1.0,0.0,False,True,Genuine,Report #11155
,Furniture,1.0,0.0,False,True,Genuine,
,Furniture,1.0,0.0,False,True,Fake,Report #11156
,Furniture,1.0,0.0,False,True,Fake,
,Furniture,1.0,0.0,False,True,,Report #11157
,Furniture,1.0,0.0,False,True,,
,Furniture,1.0,0.0,False,False,Genuine,Report #11158
,Furniture,1.0,0.0,False,False,Genuine,
,Furniture,1.0,0.0,False,False,Fake,Report #11159
,Furniture,1.0,0.0,False,False,Fake,
,Furniture,1.0,0.0,False,False,,Report #11160
,Furniture,1.0,0.0,False,False,,
,Furniture,1.0,19.0,True,True,Genuine,Report #11161
,Furniture,1.0,19.0,True,True,Genuine,
,Furniture,1.0,19.0,True,True,Fake,Report #11162
,Furniture,1.0,19.0,True,True,Fake,
,Furniture,1.0,19.0,True,True,,Report #11163
,Furniture,1.0,19.0,True,True,,
,Furniture,1.0,19.0,True,False,Genuine,Report #11164
,Furniture,1.0,19.0,True,False,Genuine,
,Furniture,1.0,19.0,True,False,Fake,Report #11165
,Furniture,1.0,19.0,True,False,Fake,
,Furniture,1.0,19.0,True,False,,Report #11166
,Furniture,1.0,19.0,True,False,,
,Furniture,1.0,19.0,False,True,Genuine,Report #11167
,Furniture,1.0,19.0,False,True,Genuine,
,Furniture,1.0,19.0,False,True,Fake,Report #11168
,Furniture,1.0,19.0,False,True,Fake,
,Furniture,1.0,19.0,False,True,,Report #11169
,Furniture,1.0,19.0,False,True,,
,Furniture,1.0,19.0,False,False,Genuine,Report #11170
,Furniture,1.0,19.0,False,False,Genuine,
,Furniture,1.0,19.0,False,False,Fake,Report #11171
,Furniture,1.0,19.0,False,False,Fake,
,Furniture,1.0,19.0,False,False,,Report #11172
,Furniture,1.0,19.0,False,False,,
,Furniture,1.0,20.0,True,True,Genuine,Report #11173
,Furniture,1.0,20.0,True,True,Genuine,
,Furniture,1.0,20.0,True,True,Fake,Report #11174
,Furniture,1.0,20.0,True,True,Fake,
,Furniture,1.0,20.0,True,True,,Report #11175
,Furniture,1.0,20.0,True,True,,
,Furniture,1.0,20.0,True,False,Genuine,Report #11176
,Furniture,1.0,20.0,True,False,Genuine,
,Furniture,1.0,20.0,True,False,Fake,Report #11177
,Furniture,1.0,20.0,True,False,Fake,
,Furniture,1.0,20.0,True,False,,Report #11178
,Furniture,1.0,20.0,True,False,,
,Furniture,1.0,20.0,False,True,Genuine,Report #11179
,Furniture,1.0,20.0,False,True,Genuine,
,Furniture,1.0,20.0,False,True,Fake,Report #11180
,Furniture,1.0,20.0,False,True,Fake,
,Furniture,1.0,20.0,False,True,,Report #11181
,Furniture,1.0,20.0,False,True,,
,Furniture,1.0,20.0,False,False,Genuine,Report #11182
,Furniture,1.0,20.0,False,False,Genuine,
,Furniture,1.0,20.0,False,False,Fake,Report #11183
,Furniture,1.0,20.0,False,False,Fake,
,Furniture,1.0,20.0,False,False,,Report #11184
,Furniture,1.0,20.0,False,False,,
,Furniture,1.0,21.0,True,True,Genuine,Report #11185
,Furniture,1.0,21.0,True,True,Genuine,
,Furniture,1.0,21.0,True,True,Fake,Report #11186
,Furniture,1.0,21.0,True,True,Fake,
,Furniture,1.0,21.0,True,True,,Report #11187
,Furniture,1.0,21.0,True,True,,
,Furniture,1.0,21.0,True,False,Genuine,Report #11188
,Furniture,1.0,21.0,True,False,Genuine,
,Furniture,1.0,21.0,True,False,Fake,Report #11189
,Furniture,1.0,21.0,True,False,Fake,
,Furniture,1.0,21.0,True,False,,Report #11190
,Furniture,1.0,21.0,True,False,,
,Furniture,1.0,21.0,False,True,Genuine,Report #11191
,Furniture,1.0,21.0,False,True,Genuine,
,Furniture,1.0,21.0,False,True,Fake,Report #11192
,Furniture,1.0,21.0,False,True,Fake,
,Furniture,1.0,21.0,False,True,,Report #11193
,Furniture,1.0,21.0,False,True,,
,Furniture,1.0,21.0,False,False,Genuine,Report #11194
,Furniture,1.0,21.0,False,False,Genuine,
,Furniture,1.0,21.0,False,False,Fake,Report #11195
,Furniture,1.0,21.0,False,False,Fake,
,Furniture,1.0,21.0,False,False,,Report #11196
,Furniture,1.0,21.0,False,False,,
,Furniture,1.0,99.0,True,True,Genuine,Report #11197
,Furniture,1.0,99.0,True,True,Genuine,
,Furniture,1.0,99.0,True,True,Fake,Report #11198
,Furniture,1.0,99.0,True,True,Fake,
,Furniture,1.0,99.0,True,True,,Report #11199
,Furniture,1.0,99.0,True,True,,
,Furniture,1.0,99.0,True,False,Genuine,Report #11200
,Furniture,1.0,99.0,True,False,Genuine,
,Furniture,1.0,99.0,True,False,Fake,Report #11201
,Furniture,1.0,99.0,True,False,Fake,
,Furniture,1.0,99.0,True,False,,Report #11202
,Furniture,1.0,99.0,True,False,,
,Furniture,1.0,99.0,False,True,Genuine,Report #11203
,Furniture,1.0,99.0,False,True,Genuine,
,Furniture,1.0,99.0,False,True,Fake,Report #11204
,Furniture,1.0,99.0,False,True,Fake,
,Furniture,1.0,99.0,False,True,,Report #11205
,Furniture,1.0,99.0,False,True,,
,Furniture,1.0,99.0,False,False,Genuine,Report #11206
,Furniture,1.0,99.0,False,False,Genuine,
,Furniture,1.0,99.0,False,False,Fake,Report #11207
,Furniture,1.0,99.0,False,False,Fake,
,Furniture,1.0,99.0,False,False,,Report #11208
,Furniture,1.0,99.0,False,False,,
,Furniture,1.0,100.0,True,True,Genuine,Report #11209
,Furniture,1.0,100.0,True,True,Genuine,
,Furniture,1.0,100.0,True,True,Fake,Report #11210
,Furniture,1.0,100.0,True,True,Fake,
,Furniture,1.0,100.0,True,True,,Report #11211
,Furniture,1.0,100.0,True,True,,
,Furniture,1.0,100.0,True,False,Genuine,Report #11212
,Furniture,1.0,100.0,True,False,Genuine,
,Furniture,1.0,100.0,True,False,Fake,Report #11213
,Furniture,1.0,100.0,True,False,Fake,
,Furniture,1.0,100.0,True,False,,Report #11214
,Furniture,1.0,100.0,True,False,,
,Furniture,1.0,100.0,False,True,Genuine,Report #11215
,Furniture,1.0,100.0,False,True,Genuine,
,Furniture,1.0,100.0,False,True,Fake,Report #11216
,Furniture,1.0,100.0,False,True,Fake,
,Furniture,1.0,100.0,False,True,,Report #11217
,Furniture,1.0,100.0,False,True,,
,Furniture,1.0,100.0,False,False,Genuine,Report #11218
,Furniture,1.0,100.0,False,False,Genuine,
,Furniture,1.0,100.0,False,False,Fake,Report #11219
,Furniture,1.0,100.0,False,False,Fake,
,Furniture,1.0,100.0,False,False,,Report #11220
,Furniture,1.0,100.0,False,False,,
,Furniture,1.0,101.0,True,True,Genuine,Report #11221
,Furniture,1.0,101.0,True,True,Genuine,
,Furniture,1.0,101.0,True,True,Fake,Report #11222
,Furniture,1.0,101.0,True,True,Fake,
,Furniture,1.0,101.0,True,True,,Report #11223
,Furniture,1.0,101.0,True,True,,
,Furniture,1.0,101.0,True,False,Genuine,Report #11224
,Furniture,1.0,101.0,True,False,Genuine,
,Furniture,1.0,101.0,True,False,Fake,Report #11225
,Furniture,1.0,101.0,True,False,Fake,
,Furniture,1.0,101.0,True,False,,Report #11226
,Furniture,1.0,101.0,True,False,,
,Furniture,1.0,101.0,False,True,Genuine,Report #11227
,Furniture,1.0,101.0,False,True,Genuine,
,Furniture,1.0,101.0,False,True,Fake,Report #11228
,Furniture,1.0,101.0,False,True,Fake,
,Furniture,1.0,101.0,False,True,,Report #11229
,Furniture,1.0,101.0,False,True,,
,Furniture,1.0,101.0,False,False,Genuine,Report #11230
,Furniture,1.0,101.0,False,False,Genuine,
,Furniture,1.0,101.0,False,False,Fake,Report #11231
,Furniture,1.0,101.0,False,False,Fake,
,Furniture,1.0,101.0,False,False,,Report #11232
,Furniture,1.0,101.0,False,False,,
,Furniture,100.0,-1.0,True,True,Genuine,Report #11233
,Furniture,100.0,-1.0,True,True,Genuine,
,Furniture,100.0,-1.0,True,True,Fake,Report #11234
,Furniture,100.0,-1.0,True,True,Fake,
,Furniture,100.0,-1.0,True,True,,Report #11235
,Furniture,100.0,-1.0,True,True,,
,Furniture,100.0,-1.0,True,False,Genuine,Report #11236
,Furniture,100.0,-1.0,True,False,Genuine,
,Furniture,100.0,-1.0,True,False,Fake,Report #11237
,Furniture,100.0,-1.0,True,False,Fake,
,Furniture,100.0,-1.0,True,False,,Report #11238
,Furniture,100.0,-1.0,True,False,,
,Furniture,100.0,-1.0,False,True,Genuine,Report #11239
,Furniture,100.0,-1.0,False,True,Genuine,
,Furniture,100.0,-1.0,False,True,Fake,Report #11240
,Furniture,100.0,-1.0,False,True,Fake,
,Furniture,100.0,-1.0,False,True,,Report #11241
,Furniture,100.0,-1.0,False,True,,
,Furniture,100.0,-1.0,False,False,Genuine,Report #11242
,Furniture,100.0,-1.0,False,False,Genuine,
,Furniture,100.0,-1.0,False,False,Fake,Report #11243
,Furniture,100.0,-1.0,False,False,Fake,
,Furniture,100.0,-1.0,False,False,,Report #11244
,Furniture,100.0,-1.0,False,False,,
,Furniture,100.0,0.0,True,True,Genuine,Report #11245
,Furniture,100.0,0.0,True,True,Genuine,
,Furniture,100.0,0.0,True,True,Fake,Report #11246
,Furniture,100.0,0.0,True,True,Fake,
,Furniture,100.0,0.0,True,True,,Report #11247
,Furniture,100.0,0.0,True,True,,
,Furniture,100.0,0.0,True,False,Genuine,Report #11248
,Furniture,100.0,0.0,True,False,Genuine,
,Furniture,100.0,0.0,True,False,Fake,Report #11249
,Furniture,100.0,0.0,True,False,Fake,
,Furniture,100.0,0.0,True,False,,Report #11250
,Furniture,100.0,0.0,True,False,,
,Furniture,100.0,0.0,False,True,Genuine,Report #11251
,Furniture,100.0,0.0,False,True,Genuine,
,Furniture,100.0,0.0,False,True,Fake,Report #11252
,Furniture,100.0,0.0,False,True,Fake,
,Furniture,100.0,0.0,False,True,,Report #11253
,Furniture,100.0,0.0,False,True,,
,Furniture,100.0,0.0,False,False,Genuine,Report #11254
,Furniture,100.0,0.0,False,False,Genuine,
,Furniture,100.0,0.0,False,False,Fake,Report #11255
,Furniture,100.0,0.0,False,False,Fake,
,Furniture,100.0,0.0,False,False,,Report #11256
,Furniture,100.0,0.0,False,False,,
,Furniture,100.0,19.0,True,True,Genuine,Report #11257
,Furniture,100.0,19.0,True,True,Genuine,
,Furniture,100.0,19.0,True,True,Fake,Report #11258
,Furniture,100.0,19.0,True,True,Fake,
,Furniture,100.0,19.0,True,True,,Report #11259
,Furniture,100.0,19.0,True,True,,
,Furniture,100.0,19.0,True,False,Genuine,Report #11260
,Furniture,100.0,19.0,True,False,Genuine,
,Furniture,100.0,19.0,True,False,Fake,Report #11261
,Furniture,100.0,19.0,True,False,Fake,
,Furniture,100.0,19.0,True,False,,Report #11262
,Furniture,100.0,19.0,True,False,,
,Furniture,100.0,19.0,False,True,Genuine,Report #11263
,Furniture,100.0,19.0,False,True,Genuine,
,Furniture,100.0,19.0,False,True,Fake,Report #11264
,Furniture,100.0,19.0,False,True,Fake,
,Furniture,100.0,19.0,False,True,,Report #11265
,Furniture,100.0,19.0,False,True,,
,Furniture,100.0,19.0,False,False,Genuine,Report #11266
,Furniture,100.0,19.0,False,False,Genuine,
,Furniture,100.0,19.0,False,False,Fake,Report #11267
,Furniture,100.0,19.0,False,False,Fake,
,Furniture,100.0,19.0,False,False,,Report #11268
,Furniture,100.0,19.0,False,False,,
,Furniture,100.0,20.0,True,True,Genuine,Report #11269
,Furniture,100.0,20.0,True,True,Genuine,
,Furniture,100.0,20.0,True,True,Fake,Report #11270
,Furniture,100.0,20.0,True,True,Fake,
,Furniture,100.0,20.0,True,True,,Report #11271
,Furniture,100.0,20.0,True,True,,
,Furniture,100.0,20.0,True,False,Genuine,Report #11272
,Furniture,100.0,20.0,True,False,Genuine,
,Furniture,100.0,20.0,True,False,Fake,Report #11273
,Furniture,100.0,20.0,True,False,Fake,
,Furniture,100.0,20.0,True,False,,Report #11274
,Furniture,100.0,20.0,True,False,,
,Furniture,100.0,20.0,False,True,Genuine,Report #11275
,Furniture,100.0,20.0,False,True,Genuine,
,Furniture,100.0,20.0,False,True,Fake,Report #11276
,Furniture,100.0,20.0,False,True,Fake,
,Furniture,100.0,20.0,False,True,,Report #11277
,Furniture,100.0,20.0,False,True,,
,Furniture,100.0,20.0,False,False,Genuine,Report #11278
,Furniture,100.0,20.0,False,False,Genuine,
,Furniture,100.0,20.0,False,False,Fake,Report #11279
,Furniture,100.0,20.0,False,False,Fake,
,Furniture,100.0,20.0,False,False,,Report #11280
,Furniture,100.0,20.0,False,False,,
,Furniture,100.0,21.0,True,True,Genuine,Report #11281
,Furniture,100.0,21.0,True,True,Genuine,
,Furniture,100.0,21.0,True,True,Fake,Report #11282
,Furniture,100.0,21.0,True,True,Fake,
,Furniture,100.0,21.0,True,True,,Report #11283
,Furniture,100.0,21.0,True,True,,
,Furniture,100.0,21.0,True,False,Genuine,Report #11284
,Furniture,100.0,21.0,True,False,Genuine,
,Furniture,100.0,21.0,True,False,Fake,Report #11285
,Furniture,100.0,21.0,True,False,Fake,
,Furniture,100.0,21.0,True,False,,Report #11286
,Furniture,100.0,21.0,True,False,,
,Furniture,100.0,21.0,False,True,Genuine,Report #11287
,Furniture,100.0,21.0,False,True,Genuine,
,Furniture,100.0,21.0,False,True,Fake,Report #11288
,Furniture,100.0,21.0,False,True,Fake,
,Furniture,100.0,21.0,False,True,,Report #11289
,Furniture,100.0,21.0,False,True,,
,Furniture,100.0,21.0,False,False,Genuine,Report #11290
,Furniture,100.0,21.0,False,False,Genuine,
,Furniture,100.0,21.0,False,False,Fake,Report #11291
,Furniture,100.0,21.0,False,False,Fake,
,Furniture,100.0,21.0,False,False,,Report #11292
,Furniture,100.0,21.0,False,False,,
,Furniture,100.0,99.0,True,True,Genuine,Report #11293
,Furniture,100.0,99.0,True,True,Genuine,
,Furniture,100.0,99.0,True,True,Fake,Report #11294
,Furniture,100.0,99.0,True,True,Fake,
,Furniture,100.0,99.0,True,True,,Report #11295
,Furniture,100.0,99.0,True,True,,
,Furniture,100.0,99.0,True,False,Genuine,Report #11296
,Furniture,100.0,99.0,True,False,Genuine,
,Furniture,100.0,99.0,True,False,Fake,Report #11297
,Furniture,100.0,99.0,True,False,Fake,
,Furniture,100.0,99.0,True,False,,Report #11298
,Furniture,100.0,99.0,True,False,,
,Furniture,100.0,99.0,False,True,Genuine,Report #11299
,Furniture,100.0,99.0,False,True,Genuine,
,Furniture,100.0,99.0,False,True,Fake,Report #11300
,Furniture,100.0,99.0,False,True,Fake,
,Furniture,100.0,99.0,False,True,,Report #11301
,Furniture,100.0,99.0,False,True,,
,Furniture,100.0,99.0,False,False,Genuine,Report #11302
,Furniture,100.0,99.0,False,False,Genuine,
,Furniture,100.0,99.0,False,False,Fake,Report #11303
,Furniture,100.0,99.0,False,False,Fake,
,Furniture,100.0,99.0,False,False,,Report #11304
,Furniture,100.0,99.0,False,False,,
,Furniture,100.0,100.0,True,True,Genuine,Report #11305
,Furniture,100.0,100.0,True,True,Genuine,
,Furniture,100.0,100.0,True,True,Fake,Report #11306
,Furniture,100.0,100.0,True,True,Fake,
,Furniture,100.0,100.0,True,True,,Report #11307
,Furniture,100.0,100.0,True,True,,
,Furniture,100.0,100.0,True,False,Genuine,Report #11308
,Furniture,100.0,100.0,True,False,Genuine,
,Furniture,100.0,100.0,True,False,Fake,Report #11309
,Furniture,100.0,100.0,True,False,Fake,
,Furniture,100.0,100.0,True,False,,Report #11310
,Furniture,100.0,100.0,True,False,,
,Furniture,100.0,100.0,False,True,Genuine,Report #11311
,Furniture,100.0,100.0,False,True,Genuine,
,Furniture,100.0,100.0,False,True,Fake,Report #11312
,Furniture,100.0,100.0,False,True,Fake,
,Furniture,100.0,100.0,False,True,,Report #11313
,Furniture,100.0,100.0,False,True,,
,Furniture,100.0,100.0,False,False,Genuine,Report #11314
,Furniture,100.0,100.0,False,False,Genuine,
,Furniture,100.0,100.0,False,False,Fake,Report #11315
,Furniture,100.0,100.0,False,False,Fake,
,Furniture,100.0,100.0,False,False,,Report #11316
,Furniture,100.0,100.0,False,False,,
,Furniture,100.0,101.0,True,True,Genuine,Report #11317
,Furniture,100.0,101.0,True,True,Genuine,
,Furniture,100.0,101.0,True,True,Fake,Report #11318
,Furniture,100.0,101.0,True,True,Fake,
,Furniture,100.0,101.0,True,True,,Report #11319
,Furniture,100.0,101.0,True,True,,
,Furniture,100.0,101.0,True,False,Genuine,Report #11320
,Furniture,100.0,101.0,True,False,Genuine,
,Furniture,100.0,101.0,True,False,Fake,Report #11321
,Furniture,100.0,101.0,True,False,Fake,
,Furniture,100.0,101.0,True,False,,Report #11322
,Furniture,100.0,101.0,True,False,,
,Furniture,100.0,101.0,False,True,Genuine,Report #11323
,Furniture,100.0,101.0,False,True,Genuine,
,Furniture,100.0,101.0,False,True,Fake,Report #11324
,Furniture,100.0,101.0,False,True,Fake,
,Furniture,100.0,101.0,False,True,,Report #11325
,Furniture,100.0,101.0,False,True,,
,Furniture,100.0,101.0,False,False,Genuine,Report #11326
,Furniture,100.0,101.0,False,False,Genuine,
,Furniture,100.0,101.0,False,False,Fake,Report #11327
,Furniture,100.0,101.0,False,False,Fake,
,Furniture,100.0,101.0,False,False,,Report #11328
,Furniture,100.0,101.0,False,False,,
,Furniture,1000.0,-1.0,True,True,Genuine,Report #11329
,Furniture,1000.0,-1.0,True,True,Genuine,
,Furniture,1000.0,-1.0,True,True,Fake,Report #11330
,Furniture,1000.0,-1.0,True,True,Fake,
,Furniture,1000.0,-1.0,True,True,,Report #11331
,Furniture,1000.0,-1.0,True,True,,
,Furniture,1000.0,-1.0,True,False,Genuine,Report #11332
,Furniture,1000.0,-1.0,True,False,Genuine,
,Furniture,1000.0,-1.0,True,False,Fake,Report #11333
,Furniture,1000.0,-1.0,True,False,Fake,
,Furniture,1000.0,-1.0,True,False,,Report #11334
,Furniture,1000.0,-1.0,True,False,,
,Furniture,1000.0,-1.0,False,True,Genuine,Report #11335
,Furniture,1000.0,-1.0,False,True,Genuine,
,Furniture,1000.0,-1.0,False,True,Fake,Report #11336
,Furniture,1000.0,-1.0,False,True,Fake,
,Furniture,1000.0,-1.0,False,True,,Report #11337
,Furniture,1000.0,-1.0,False,True,,
,Furniture,1000.0,-1.0,False,False,Genuine,Report #11338
,Furniture,1000.0,-1.0,False,False,Genuine,
,Furniture,1000.0,-1.0,False,False,Fake,Report #11339
,Furniture,1000.0,-1.0,False,False,Fake,
,Furniture,1000.0,-1.0,False,False,,Report #11340
,Furniture,1000.0,-1.0,False,False,,
,Furniture,1000.0,0.0,True,True,Genuine,Report #11341
,Furniture,1000.0,0.0,True,True,Genuine,
,Furniture,1000.0,0.0,True,True,Fake,Report #11342
,Furniture,1000.0,0.0,True,True,Fake,
,Furniture,1000.0,0.0,True,True,,Report #11343
,Furniture,1000.0,0.0,True,True,,
,Furniture,1000.0,0.0,True,False,Genuine,Report #11344
,Furniture,1000.0,0.0,True,False,Genuine,
,Furniture,1000.0,0.0,True,False,Fake,Report #11345
,Furniture,1000.0,0.0,True,False,Fake,
,Furniture,1000.0,0.0,True,False,,Report #11346
,Furniture,1000.0,0.0,True,False,,
,Furniture,1000.0,0.0,False,True,Genuine,Report #11347
,Furniture,1000.0,0.0,False,True,Genuine,
,Furniture,1000.0,0.0,False,True,Fake,Report #11348
,Furniture,1000.0,0.0,False,True,Fake,
,Furniture,1000.0,0.0,False,True,,Report #11349
,Furniture,1000.0,0.0,False,True,,
,Furniture,1000.0,0.0,False,False,Genuine,Report #11350
,Furniture,1000.0,0.0,False,False,Genuine,
,Furniture,1000.0,0.0,False,False,Fake,Report #11351
,Furniture,1000.0,0.0,False,False,Fake,
,Furniture,1000.0,0.0,False,False,,Report #11352
,Furniture,1000.0,0.0,False,False,,
,Furniture,1000.0,19.0,True,True,Genuine,Report #11353
,Furniture,1000.0,19.0,True,True,Genuine,
,Furniture,1000.0,19.0,True,True,Fake,Report #11354
,Furniture,1000.0,19.0,True,True,Fake,
,Furniture,1000.0,19.0,True,True,,Report #11355
,Furniture,1000.0,19.0,True,True,,
,Furniture,1000.0,19.0,True,False,Genuine,Report #11356
,Furniture,1000.0,19.0,True,False,Genuine,
,Furniture,1000.0,19.0,True,False,Fake,Report #11357
,Furniture,1000.0,19.0,True,False,Fake,
,Furniture,1000.0,19.0,True,False,,Report #11358
,Furniture,1000.0,19.0,True,False,,
,Furniture,1000.0,19.0,False,True,Genuine,Report #11359
,Furniture,1000.0,19.0,False,True,Genuine,
,Furniture,1000.0,19.0,False,True,Fake,Report #11360
,Furniture,1000.0,19.0,False,True,Fake,
,Furniture,1000.0,19.0,False,True,,Report #11361
,Furniture,1000.0,19.0,False,True,,
,Furniture,1000.0,19.0,False,False,Genuine,Report #11362
,Furniture,1000.0,19.0,False,False,Genuine,
,Furniture,1000.0,19.0,False,False,Fake,Report #11363
,Furniture,1000.0,19.0,False,False,Fake,
,Furniture,1000.0,19.0,False,False,,Report #11364
,Furniture,1000.0,19.0,False,False,,
,Furniture,1000.0,20.0,True,True,Genuine,Report #11365
,Furniture,1000.0,20.0,True,True,Genuine,
,Furniture,1000.0,20.0,True,True,Fake,Report #11366
,Furniture,1000.0,20.0,True,True,Fake,
,Furniture,1000.0,20.0,True,True,,Report #11367
,Furniture,1000.0,20.0,True,True,,
,Furniture,1000.0,20.0,True,False,Genuine,Report #11368
,Furniture,1000.0,20.0,True,False,Genuine,
,Furniture,1000.0,20.0,True,False,Fake,Report #11369
,Furniture,1000.0,20.0,True,False,Fake,
,Furniture,1000.0,20.0,True,False,,Report #11370
,Furniture,1000.0,20.0,True,False,,
,Furniture,1000.0,20.0,False,True,Genuine,Report #11371
,Furniture,1000.0,20.0,False,True,Genuine,
,Furniture,1000.0,20.0,False,True,Fake,Report #11372
,Furniture,1000.0,20.0,False,True,Fake,
,Furniture,1000.0,20.0,False,True,,Report #11373
,Furniture,1000.0,20.0,False,True,,
,Furniture,1000.0,20.0,False,False,Genuine,Report #11374
,Furniture,1000.0,20.0,False,False,Genuine,
,Furniture,1000.0,20.0,False,False,Fake,Report #11375
,Furniture,1000.0,20.0,False,False,Fake,
,Furniture,1000.0,20.0,False,False,,Report #11376
,Furniture,1000.0,20.0,False,False,,
,Furniture,1000.0,21.0,True,True,Genuine,Report #11377
,Furniture,1000.0,21.0,True,True,Genuine,
,Furniture,1000.0,21.0,True,True,Fake,Report #11378
,Furniture,1000.0,21.0,True,True,Fake,
,Furniture,1000.0,21.0,True,True,,Report #11379
,Furniture,1000.0,21.0,True,True,,
,Furniture,1000.0,21.0,True,False,Genuine,Report #11380
,Furniture,1000.0,21.0,True,False,Genuine,
,Furniture,1000.0,21.0,True,False,Fake,Report #11381
,Furniture,1000.0,21.0,True,False,Fake,
,Furniture,1000.0,21.0,True,False,,Report #11382
,Furniture,1000.0,21.0,True,False,,
,Furniture,1000.0,21.0,False,True,Genuine,Report #11383
,Furniture,1000.0,21.0,False,True,Genuine,
,Furniture,1000.0,21.0,False,True,Fake,Report #11384
,Furniture,1000.0,21.0,False,True,Fake,
,Furniture,1000.0,21.0,False,True,,Report #11385
,Furniture,1000.0,21.0,False,True,,
,Furniture,1000.0,21.0,False,False,Genuine,Report #11386
,Furniture,1000.0,21.0,False,False,Genuine,
,Furniture,1000.0,21.0,False,False,Fake,Report #11387
,Furniture,1000.0,21.0,False,False,Fake,
,Furniture,1000.0,21.0,False,False,,Report #11388
,Furniture,1000.0,21.0,False,False,,
,Furniture,1000.0,99.0,True,True,Genuine,Report #11389
,Furniture,1000.0,99.0,True,True,Genuine,
,Furniture,1000.0,99.0,True,True,Fake,Report #11390
,Furniture,1000.0,99.0,True,True,Fake,
,Furniture,1000.0,99.0,True,True,,Report #11391
,Furniture,1000.0,99.0,True,True,,
,Furniture,1000.0,99.0,True,False,Genuine,Report #11392
,Furniture,1000.0,99.0,True,False,Genuine,
,Furniture,1000.0,99.0,True,False,Fake,Report #11393
,Furniture,1000.0,99.0,True,False,Fake,
,Furniture,1000.0,99.0,True,False,,Report #11394
,Furniture,1000.0,99.0,True,False,,
,Furniture,1000.0,99.0,False,True,Genuine,Report #11395
,Furniture,1000.0,99.0,False,True,Genuine,
,Furniture,1000.0,99.0,False,True,Fake,Report #11396
,Furniture,1000.0,99.0,False,True,Fake,
,Furniture,1000.0,99.0,False,True,,Report #11397
,Furniture,1000.0,99.0,False,True,,
,Furniture,1000.0,99.0,False,False,Genuine,Report #11398
,Furniture,1000.0,99.0,False,False,Genuine,
,Furniture,1000.0,99.0,False,False,Fake,Report #11399
,Furniture,1000.0,99.0,False,False,Fake,
,Furniture,1000.0,99.0,False,False,,Report #11400
,Furniture,1000.0,99.0,False,False,,
,Furniture,1000.0,100.0,True,True,Genuine,Report #11401
,Furniture,1000.0,100.0,True,True,Genuine,
,Furniture,1000.0,100.0,True,True,Fake,Report #11402
,Furniture,1000.0,100.0,True,True,Fake,
,Furniture,1000.0,100.0,True,True,,Report #11403
,Furniture,1000.0,100.0,True,True,,
,Furniture,1000.0,100.0,True,False,Genuine,Report #11404
,Furniture,1000.0,100.0,True,False,Genuine,
,Furniture,1000.0,100.0,True,False,Fake,Report #11405
,Furniture,1000.0,100.0,True,False,Fake,
,Furniture,1000.0,100.0,True,False,,Report #11406
,Furniture,1000.0,100.0,True,False,,
,Furniture,1000.0,100.0,False,True,Genuine,Report #11407
,Furniture,1000.0,100.0,False,True,Genuine,
,Furniture,1000.0,100.0,False,True,Fake,Report #11408
,Furniture,1000.0,100.0,False,True,Fake,
,Furniture,1000.0,100.0,False,True,,Report #11409
,Furniture,1000.0,100.0,False,True,,
,Furniture,1000.0,100.0,False,False,Genuine,Report #11410
,Furniture,1000.0,100.0,False,False,Genuine,
,Furniture,1000.0,100.0,False,False,Fake,Report #11411
,Furniture,1000.0,100.0,False,False,Fake,
,Furniture,1000.0,100.0,False,False,,Report #11412
,Furniture,1000.0,100.0,False,False,,
,Furniture,1000.0,101.0,True,True,Genuine,Report #11413
,Furniture,1000.0,101.0,True,True,Genuine,
,Furniture,1000.0,101.0,True,True,Fake,Report #11414
,Furniture,1000.0,101.0,True,True,Fake,
,Furniture,1000.0,101.0,True,True,,Report #11415
,Furniture,1000.0,101.0,True,True,,
,Furniture,1000.0,101.0,True,False,Genuine,Report #11416
,Furniture,1000.0,101.0,True,False,Genuine,
,Furniture,1000.0,101.0,True,False,Fake,Report #11417
,Furniture,1000.0,101.0,True,False,Fake,
,Furniture,1000.0,101.0,True,False,,Report #11418
,Furniture,1000.0,101.0,True,False,,
,Furniture,1000.0,101.0,False,True,Genuine,Report #11419
,Furniture,1000.0,101.0,False,True,Genuine,
,Furniture,1000.0,101.0,False,True,Fake,Report #11420
,Furniture,1000.0,101.0,False,True,Fake,
,Furniture,1000.0,101.0,False,True,,Report #11421
,Furniture,1000.0,101.0,False,True,,
,Furniture,1000.0,101.0,False,False,Genuine,Report #11422
,Furniture,1000.0,101.0,False,False,Genuine,
,Furniture,1000.0,101.0,False,False,Fake,Report #11423
,Furniture,1000.0,101.0,False,False,Fake,
,Furniture,1000.0,101.0,False,False,,Report #11424
,Furniture,1000.0,101.0,False,False,,
,Furniture,1000000.0,-1.0,True,True,Genuine,Report #11425
,Furniture,1000000.0,-1.0,True,True,Genuine,
,Furniture,1000000.0,-1.0,True,True,Fake,Report #11426
,Furniture,1000000.0,-1.0,True,True,Fake,
,Furniture,1000000.0,-1.0,True,True,,Report #11427
,Furniture,1000000.0,-1.0,True,True,,
,Furniture,1000000.0,-1.0,True,False,Genuine,Report #11428
,Furniture,1000000.0,-1.0,True,False,Genuine,
,Furniture,1000000.0,-1.0,True,False,Fake,Report #11429
,Furniture,1000000.0,-1.0,True,False,Fake,
,Furniture,1000000.0,-1.0,True,False,,Report #11430
,Furniture,1000000.0,-1.0,True,False,,
,Furniture,1000000.0,-1.0,False,True,Genuine,Report #11431
,Furniture,1000000.0,-1.0,False,True,Genuine,
,Furniture,1000000.0,-1.0,False,True,Fake,Report #11432
,Furniture,1000000.0,-1.0,False,True,Fake,
,Furniture,1000000.0,-1.0,False,True,,Report #11433
,Furniture,1000000.0,-1.0,False,True,,
,Furniture,1000000.0,-1.0,False,False,Genuine,Report #11434
,Furniture,1000000.0,-1.0,False,False,Genuine,
,Furniture,1000000.0,-1.0,False,False,Fake,Report #11435
,Furniture,1000000.0,-1.0,False,False,Fake,
,Furniture,1000000.0,-1.0,False,False,,Report #11436
,Furniture,1000000.0,-1.0,False,False,,
,Furniture,1000000.0,0.0,True,True,Genuine,Report #11437
,Furniture,1000000.0,0.0,True,True,Genuine,
,Furniture,1000000.0,0.0,True,True,Fake,Report #11438
,Furniture,1000000.0,0.0,True,True,Fake,
,Furniture,1000000.0,0.0,True,True,,Report #11439
,Furniture,1000000.0,0.0,True,True,,
,Furniture,1000000.0,0.0,True,False,Genuine,Report #11440
,Furniture,1000000.0,0.0,True,False,Genuine,
,Furniture,1000000.0,0.0,True,False,Fake,Report #11441
,Furniture,1000000.0,0.0,True,False,Fake,
,Furniture,1000000.0,0.0,True,False,,Report #11442
,Furniture,1000000.0,0.0,True,False,,
,Furniture,1000000.0,0.0,False,True,Genuine,Report #11443
,Furniture,1000000.0,0.0,False,True,Genuine,
,Furniture,1000000.0,0.0,False,True,Fake,Report #11444
,Furniture,1000000.0,0.0,False,True,Fake,
,Furniture,1000000.0,0.0,False,True,,Report #11445
,Furniture,1000000.0,0.0,False,True,,
,Furniture,1000000.0,0.0,False,False,Genuine,Report #11446
,Furniture,1000000.0,0.0,False,False,Genuine,
,Furniture,1000000.0,0.0,False,False,Fake,Report #11447
,Furniture,1000000.0,0.0,False,False,Fake,
,Furniture,1000000.0,0.0,False,False,,Report #11448
,Furniture,1000000.0,0.0,False,False,,
,Furniture,1000000.0,19.0,True,True,Genuine,Report #11449
,Furniture,1000000.0,19.0,True,True,Genuine,
,Furniture,1000000.0,19.0,True,True,Fake,Report #11450
,Furniture,1000000.0,19.0,True,True,Fake,
,Furniture,1000000.0,19.0,True,True,,Report #11451
,Furniture,1000000.0,19.0,True,True,,
,Furniture,1000000.0,19.0,True,False,Genuine,Report #11452
,Furniture,1000000.0,19.0,True,False,Genuine,
,Furniture,1000000.0,19.0,True,False,Fake,Report #11453
,Furniture,1000000.0,19.0,True,False,Fake,
,Furniture,1000000.0,19.0,True,False,,Report #11454
,Furniture,1000000.0,19.0,True,False,,
,Furniture,1000000.0,19.0,False,True,Genuine,Report #11455
,Furniture,1000000.0,19.0,False,True,Genuine,
,Furniture,1000000.0,19.0,False,True,Fake,Report #11456
,Furniture,1000000.0,19.0,False,True,Fake,
,Furniture,1000000.0,19.0,False,True,,Report #11457
,Furniture,1000000.0,19.0,False,True,,
,Furniture,1000000.0,19.0,False,False,Genuine,Report #11458
,Furniture,1000000.0,19.0,False,False,Genuine,
,Furniture,1000000.0,19.0,False,False,Fake,Report #11459
,Furniture,1000000.0,19.0,False,False,Fake,
,Furniture,1000000.0,19.0,False,False,,Report #11460
,Furniture,1000000.0,19.0,False,False,,
,Furniture,1000000.0,20.0,True,True,Genuine,Report #11461
,Furniture,1000000.0,20.0,True,True,Genuine,
,Furniture,1000000.0,20.0,True,True,Fake,Report #11462
,Furniture,1000000.0,20.0,True,True,Fake,
,Furniture,1000000.0,20.0,True,True,,Report #11463
,Furniture,1000000.0,20.0,True,True,,
,Furniture,1000000.0,20.0,True,False,Genuine,Report #11464
,Furniture,1000000.0,20.0,True,False,Genuine,
,Furniture,1000000.0,20.0,True,False,Fake,Report #11465
,Furniture,1000000.0,20.0,True,False,Fake,
,Furniture,1000000.0,20.0,True,False,,Report #11466
,Furniture,1000000.0,20.0,True,False,,
,Furniture,1000000.0,20.0,False,True,Genuine,Report #11467
,Furniture,1000000.0,20.0,False,True,Genuine,
,Furniture,1000000.0,20.0,False,True,Fake,Report #11468
,Furniture,1000000.0,20.0,False,True,Fake,
,Furniture,1000000.0,20.0,False,True,,Report #11469
,Furniture,1000000.0,20.0,False,True,,
,Furniture,1000000.0,20.0,False,False,Genuine,Report #11470
,Furniture,1000000.0,20.0,False,False,Genuine,
,Furniture,1000000.0,20.0,False,False,Fake,Report #11471
,Furniture,1000000.0,20.0,False,False,Fake,
,Furniture,1000000.0,20.0,False,False,,Report #11472
,Furniture,1000000.0,20.0,False,False,,
,Furniture,1000000.0,21.0,True,True,Genuine,Report #11473
,Furniture,1000000.0,21.0,True,True,Genuine,
,Furniture,1000000.0,21.0,True,True,Fake,Report #11474
,Furniture,1000000.0,21.0,True,True,Fake,
,Furniture,1000000.0,21.0,True,True,,Report #11475
,Furniture,1000000.0,21.0,True,True,,
,Furniture,1000000.0,21.0,True,False,Genuine,Report #11476
,Furniture,1000000.0,21.0,True,False,Genuine,
,Furniture,1000000.0,21.0,True,False,Fake,Report #11477
,Furniture,1000000.0,21.0,True,False,Fake,
,Furniture,1000000.0,21.0,True,False,,Report #11478
,Furniture,1000000.0,21.0,True,False,,
,Furniture,1000000.0,21.0,False,True,Genuine,Report #11479
,Furniture,1000000.0,21.0,False,True,Genuine,
,Furniture,1000000.0,21.0,False,True,Fake,Report #11480
,Furniture,1000000.0,21.0,False,True,Fake,
,Furniture,1000000.0,21.0,False,True,,Report #11481
,Furniture,1000000.0,21.0,False,True,,
,Furniture,1000000.0,21.0,False,False,Genuine,Report #11482
,Furniture,1000000.0,21.0,False,False,Genuine,
,Furniture,1000000.0,21.0,False,False,Fake,Report #11483
,Furniture,1000000.0,21.0,False,False,Fake,
,Furniture,1000000.0,21.0,False,False,,Report #11484
,Furniture,1000000.0,21.0,False,False,,
,Furniture,1000000.0,99.0,True,True,Genuine,Report #11485
,Furniture,1000000.0,99.0,True,True,Genuine,
,Furniture,1000000.0,99.0,True,True,Fake,Report #11486
,Furniture,1000000.0,99.0,True,True,Fake,
,Furniture,1000000.0,99.0,True,True,,Report #11487
,Furniture,1000000.0,99.0,True,True,,
,Furniture,1000000.0,99.0,True,False,Genuine,Report #11488
,Furniture,1000000.0,99.0,True,False,Genuine,
,Furniture,1000000.0,99.0,True,False,Fake,Report #11489
,Furniture,1000000.0,99.0,True,False,Fake,
,Furniture,1000000.0,99.0,True,False,,Report #11490
,Furniture,1000000.0,99.0,True,False,,
,Furniture,1000000.0,99.0,False,True,Genuine,Report #11491
,Furniture,1000000.0,99.0,False,True,Genuine,
,Furniture,1000000.0,99.0,False,True,Fake,Report #11492
,Furniture,1000000.0,99.0,False,True,Fake,
,Furniture,1000000.0,99.0,False,True,,Report #11493
,Furniture,1000000.0,99.0,False,True,,
,Furniture,1000000.0,99.0,False,False,Genuine,Report #11494
,Furniture,1000000.0,99.0,False,False,Genuine,
,Furniture,1000000.0,99.0,False,False,Fake,Report #11495
,Furniture,1000000.0,99.0,False,False,Fake,
,Furniture,1000000.0,99.0,False,False,,Report #11496
,Furniture,1000000.0,99.0,False,False,,
,Furniture,1000000.0,100.0,True,True,Genuine,Report #11497
,Furniture,1000000.0,100.0,True,True,Genuine,
,Furniture,1000000.0,100.0,True,True,Fake,Report #11498
,Furniture,1000000.0,100.0,True,True,Fake,
,Furniture,1000000.0,100.0,True,True,,Report #11499
,Furniture,1000000.0,100.0,True,True,,
,Furniture,1000000.0,100.0,True,False,Genuine,Report #11500
,Furniture,1000000.0,100.0,True,False,Genuine,
,Furniture,1000000.0,100.0,True,False,Fake,Report #11501
,Furniture,1000000.0,100.0,True,False,Fake,
,Furniture,1000000.0,100.0,True,False,,Report #11502
,Furniture,1000000.0,100.0,True,False,,
,Furniture,1000000.0,100.0,False,True,Genuine,Report #11503
,Furniture,1000000.0,100.0,False,True,Genuine,
,Furniture,1000000.0,100.0,False,True,Fake,Report #11504
,Furniture,1000000.0,100.0,False,True,Fake,
,Furniture,1000000.0,100.0,False,True,,Report #11505
,Furniture,1000000.0,100.0,False,True,,
,Furniture,1000000.0,100.0,False,False,Genuine,Report #11506
,Furniture,1000000.0,100.0,False,False,Genuine,
,Furniture,1000000.0,100.0,False,False,Fake,Report #11507
,Furniture,1000000.0,100.0,False,False,Fake,
,Furniture,1000000.0,100.0,False,False,,Report #11508
,Furniture,1000000.0,100.0,False,False,,
,Furniture,1000000.0,101.0,True,True,Genuine,Report #11509
,Furniture,1000000.0,101.0,True,True,Genuine,
,Furniture,1000000.0,101.0,True,True,Fake,Report #11510
,Furniture,1000000.0,101.0,True,True,Fake,
,Furniture,1000000.0,101.0,True,True,,Report #11511
,Furniture,1000000.0,101.0,True,True,,
,Furniture,1000000.0,101.0,True,False,Genuine,Report #11512
,Furniture,1000000.0,101.0,True,False,Genuine,
,Furniture,1000000.0,101.0,True,False,Fake,Report #11513
,Furniture,1000000.0,101.0,True,False,Fake,
,Furniture,1000000.0,101.0,True,False,,Report #11514
,Furniture,1000000.0,101.0,True,False,,
,Furniture,1000000.0,101.0,False,True,Genuine,Report #11515
,Furniture,1000000.0,101.0,False,True,Genuine,
,Furniture,1000000.0,101.0,False,True,Fake,Report #11516
,Furniture,1000000.0,101.0,False,True,Fake,
,Furniture,1000000.0,101.0,False,True,,Report #11517
,Furniture,1000000.0,101.0,False,True,,
,Furniture,1000000.0,101.0,False,False,Genuine,Report #11518
,Furniture,1000000.0,101.0,False,False,Genuine,
,Furniture,1000000.0,101.0,False,False,Fake,Report #11519
,Furniture,1000000.0,101.0,False,False,Fake,
,Furniture,1000000.0,101.0,False,False,,Report #11520
,Furniture,1000000.0,101.0,False,False,,
,HistoryWritings,0.0,-1.0,True,True,Genuine,Report #11521
,HistoryWritings,0.0,-1.0,True,True,Genuine,
,HistoryWritings,0.0,-1.0,True,True,Fake,Report #11522
,HistoryWritings,0.0,-1.0,True,True,Fake,
,HistoryWritings,0.0,-1.0,True,True,,Report #11523
,HistoryWritings,0.0,-1.0,True,True,,
,HistoryWritings,0.0,-1.0,True,False,Genuine,Report #11524
,HistoryWritings,0.0,-1.0,True,False,Genuine,
,HistoryWritings,0.0,-1.0,True,False,Fake,Report #11525
,HistoryWritings,0.0,-1.0,True,False,Fake,
,HistoryWritings,0.0,-1.0,True,False,,Report #11526
,HistoryWritings,0.0,-1.0,True,False,,
,HistoryWritings,0.0,-1.0,False,True,Genuine,Report #11527
,HistoryWritings,0.0,-1.0,False,True,Genuine,
,HistoryWritings,0.0,-1.0,False,True,Fake,Report #11528
,HistoryWritings,0.0,-1.0,False,True,Fake,
,HistoryWritings,0.0,-1.0,False,True,,Report #11529
,HistoryWritings,0.0,-1.0,False,True,,
,HistoryWritings,0.0,-1.0,False,False,Genuine,Report #11530
,HistoryWritings,0.0,-1.0,False,False,Genuine,
,HistoryWritings,0.0,-1.0,False,False,Fake,Report #11531
,HistoryWritings,0.0,-1.0,False,False,Fake,
,HistoryWritings,0.0,-1.0,False,False,,Report #11532
,HistoryWritings,0.0,-1.0,False,False,,
,HistoryWritings,0.0,0.0,True,True,Genuine,Report #11533
,HistoryWritings,0.0,0.0,True,True,Genuine,
,HistoryWritings,0.0,0.0,True,True,Fake,Report #11534
,HistoryWritings,0.0,0.0,True,True,Fake,
,HistoryWritings,0.0,0.0,True,True,,Report #11535
,HistoryWritings,0.0,0.0,True,True,,
,HistoryWritings,0.0,0.0,True,False,Genuine,Report #11536
,HistoryWritings,0.0,0.0,True,False,Genuine,
,HistoryWritings,0.0,0.0,True,False,Fake,Report #11537
,HistoryWritings,0.0,0.0,True,False,Fake,
,HistoryWritings,0.0,0.0,True,False,,Report #11538
,HistoryWritings,0.0,0.0,True,False,,
,HistoryWritings,0.0,0.0,False,True,Genuine,Report #11539
,HistoryWritings,0.0,0.0,False,True,Genuine,
,HistoryWritings,0.0,0.0,False,True,Fake,Report #11540
,HistoryWritings,0.0,0.0,False,True,Fake,
,HistoryWritings,0.0,0.0,False,True,,Report #11541
,HistoryWritings,0.0,0.0,False,True,,
,HistoryWritings,0.0,0.0,False,False,Genuine,Report #11542
,HistoryWritings,0.0,0.0,False,False,Genuine,
,HistoryWritings,0.0,0.0,False,False,Fake,Report #11543
,HistoryWritings,0.0,0.0,False,False,Fake,
,HistoryWritings,0.0,0.0,False,False,,Report #11544
,HistoryWritings,0.0,0.0,False,False,,
,HistoryWritings,0.0,19.0,True,True,Genuine,Report #11545
,HistoryWritings,0.0,19.0,True,True,Genuine,
,HistoryWritings,0.0,19.0,True,True,Fake,Report #11546
,HistoryWritings,0.0,19.0,True,True,Fake,
,HistoryWritings,0.0,19.0,True,True,,Report #11547
,HistoryWritings,0.0,19.0,True,True,,
,HistoryWritings,0.0,19.0,True,False,Genuine,Report #11548
,HistoryWritings,0.0,19.0,True,False,Genuine,
,HistoryWritings,0.0,19.0,True,False,Fake,Report #11549
,HistoryWritings,0.0,19.0,True,False,Fake,
,HistoryWritings,0.0,19.0,True,False,,Report #11550
,HistoryWritings,0.0,19.0,True,False,,
,HistoryWritings,0.0,19.0,False,True,Genuine,Report #11551
,HistoryWritings,0.0,19.0,False,True,Genuine,
,HistoryWritings,0.0,19.0,False,True,Fake,Report #11552
,HistoryWritings,0.0,19.0,False,True,Fake,
,HistoryWritings,0.0,19.0,False,True,,Report #11553
,HistoryWritings,0.0,19.0,False,True,,
,HistoryWritings,0.0,19.0,False,False,Genuine,Report #11554
,HistoryWritings,0.0,19.0,False,False,Genuine,
,HistoryWritings,0.0,19.0,False,False,Fake,Report #11555
,HistoryWritings,0.0,19.0,False,False,Fake,
,HistoryWritings,0.0,19.0,False,False,,Report #11556
,HistoryWritings,0.0,19.0,False,False,,
,HistoryWritings,0.0,20.0,True,True,Genuine,Report #11557
,HistoryWritings,0.0,20.0,True,True,Genuine,
,HistoryWritings,0.0,20.0,True,True,Fake,Report #11558
,HistoryWritings,0.0,20.0,True,True,Fake,
,HistoryWritings,0.0,20.0,True,True,,Report #11559
,HistoryWritings,0.0,20.0,True,True,,
,HistoryWritings,0.0,20.0,True,False,Genuine,Report #11560
,HistoryWritings,0.0,20.0,True,False,Genuine,
,HistoryWritings,0.0,20.0,True,False,Fake,Report #11561
,HistoryWritings,0.0,20.0,True,False,Fake,
,HistoryWritings,0.0,20.0,True,False,,Report #11562
,HistoryWritings,0.0,20.0,True,False,,
,HistoryWritings,0.0,20.0,False,True,Genuine,Report #11563
,HistoryWritings,0.0,20.0,False,True,Genuine,
,HistoryWritings,0.0,20.0,False,True,Fake,Report #11564
,HistoryWritings,0.0,20.0,False,True,Fake,
,HistoryWritings,0.0,20.0,False,True,,Report #11565
,HistoryWritings,0.0,20.0,False,True,,
,HistoryWritings,0.0,20.0,False,False,Genuine,Report #11566
,HistoryWritings,0.0,20.0,False,False,Genuine,
,HistoryWritings,0.0,20.0,False,False,Fake,Report #11567
,HistoryWritings,0.0,20.0,False,False,Fake,
,HistoryWritings,0.0,20.0,False,False,,Report #11568
,HistoryWritings,0.0,20.0,False,False,,
,HistoryWritings,0.0,21.0,True,True,Genuine,Report #11569
,HistoryWritings,0.0,21.0,True,True,Genuine,
,HistoryWritings,0.0,21.0,True,True,Fake,Report #11570
,HistoryWritings,0.0,21.0,True,True,Fake,
,HistoryWritings,0.0,21.0,True,True,,Report #11571
,HistoryWritings,0.0,21.0,True,True,,
,HistoryWritings,0.0,21.0,True,False,Genuine,Report #11572
,HistoryWritings,0.0,21.0,True,False,Genuine,
,HistoryWritings,0.0,21.0,True,False,Fake,Report #11573
,HistoryWritings,0.0,21.0,True,False,Fake,
,HistoryWritings,0.0,21.0,True,False,,Report #11574
,HistoryWritings,0.0,21.0,True,False,,
,HistoryWritings,0.0,21.0,False,True,Genuine,Report #11575
,HistoryWritings,0.0,21.0,False,True,Genuine,
,HistoryWritings,0.0,21.0,False,True,Fake,Report #11576
,HistoryWritings,0.0,21.0,False,True,Fake,
,HistoryWritings,0.0,21.0,False,True,,Report #11577
,HistoryWritings,0.0,21.0,False,True,,
,HistoryWritings,0.0,21.0,False,False,Genuine,Report #11578
,HistoryWritings,0.0,21.0,False,False,Genuine,
,HistoryWritings,0.0,21.0,False,False,Fake,Report #11579
,HistoryWritings,0.0,21.0,False,False,Fake,
,HistoryWritings,0.0,21.0,False,False,,Report #11580
,HistoryWritings,0.0,21.0,False,False,,
,HistoryWritings,0.0,99.0,True,True,Genuine,Report #11581
,HistoryWritings,0.0,99.0,True,True,Genuine,
,HistoryWritings,0.0,99.0,True,True,Fake,Report #11582
,HistoryWritings,0.0,99.0,True,True,Fake,
,HistoryWritings,0.0,99.0,True,True,,Report #11583
,HistoryWritings,0.0,99.0,True,True,,
,HistoryWritings,0.0,99.0,True,False,Genuine,Report #11584
,HistoryWritings,0.0,99.0,True,False,Genuine,
,HistoryWritings,0.0,99.0,True,False,Fake,Report #11585
,HistoryWritings,0.0,99.0,True,False,Fake,
,HistoryWritings,0.0,99.0,True,False,,Report #11586
,HistoryWritings,0.0,99.0,True,False,,
,HistoryWritings,0.0,99.0,False,True,Genuine,Report #11587
,HistoryWritings,0.0,99.0,False,True,Genuine,
,HistoryWritings,0.0,99.0,False,True,Fake,Report #11588
,HistoryWritings,0.0,99.0,False,True,Fake,
,HistoryWritings,0.0,99.0,False,True,,Report #11589
,HistoryWritings,0.0,99.0,False,True,,
,HistoryWritings,0.0,99.0,False,False,Genuine,Report #11590
,HistoryWritings,0.0,99.0,False,False,Genuine,
,HistoryWritings,0.0,99.0,False,False,Fake,Report #11591
,HistoryWritings,0.0,99.0,False,False,Fake,
,HistoryWritings,0.0,99.0,False,False,,Report #11592
,HistoryWritings,0.0,99.0,False,False,,
,HistoryWritings,0.0,100.0,True,True,Genuine,Report #11593
,HistoryWritings,0.0,100.0,True,True,Genuine,
,HistoryWritings,0.0,100.0,True,True,Fake,Report #11594
,HistoryWritings,0.0,100.0,True,True,Fake,
,HistoryWritings,0.0,100.0,True,True,,Report #11595
,HistoryWritings,0.0,100.0,True,True,,
,HistoryWritings,0.0,100.0,True,False,Genuine,Report #11596
,HistoryWritings,0.0,100.0,True,False,Genuine,
,HistoryWritings,0.0,100.0,True,False,Fake,Report #11597
,HistoryWritings,0.0,100.0,True,False,Fake,
,HistoryWritings,0.0,100.0,True,False,,Report #11598
,HistoryWritings,0.0,100.0,True,False,,
,HistoryWritings,0.0,100.0,False,True,Genuine,Report #11599
,HistoryWritings,0.0,100.0,False,True,Genuine,
,HistoryWritings,0.0,100.0,False,True,Fake,Report #11600
,HistoryWritings,0.0,100.0,False,True,Fake,
,HistoryWritings,0.0,100.0,False,True,,Report #11601
,HistoryWritings,0.0,100.0,False,True,,
,HistoryWritings,0.0,100.0,False,False,Genuine,Report #11602
,HistoryWritings,0.0,100.0,False,False,Genuine,
,HistoryWritings,0.0,100.0,False,False,Fake,Report #11603
,HistoryWritings,0.0,100.0,False,False,Fake,
,HistoryWritings,0.0,100.0,False,False,,Report #11604
,HistoryWritings,0.0,100.0,False,False,,
,HistoryWritings,0.0,101.0,True,True,Genuine,Report #11605
,HistoryWritings,0.0,101.0,True,True,Genuine,
,HistoryWritings,0.0,101.0,True,True,Fake,Report #11606
,HistoryWritings,0.0,101.0,True,True,Fake,
,HistoryWritings,0.0,101.0,True,True,,Report #11607
,HistoryWritings,0.0,101.0,True,True,,
,HistoryWritings,0.0,101.0,True,False,Genuine,Report #11608
,HistoryWritings,0.0,101.0,True,False,Genuine,
,HistoryWritings,0.0,101.0,True,False,Fake,Report #11609
,HistoryWritings,0.0,101.0,True,False,Fake,
,HistoryWritings,0.0,101.0,True,False,,Report #11610
,HistoryWritings,0.0,101.0,True,False,,
,HistoryWritings,0.0,101.0,False,True,Genuine,Report #11611
,HistoryWritings,0.0,101.0,False,True,Genuine,
,HistoryWritings,0.0,101.0,False,True,Fake,Report #11612
,HistoryWritings,0.0,101.0,False,True,Fake,
,HistoryWritings,0.0,101.0,False,True,,Report #11613
,HistoryWritings,0.0,101.0,False,True,,
,HistoryWritings,0.0,101.0,False,False,Genuine,Report #11614
,HistoryWritings,0.0,101.0,False,False,Genuine,
,HistoryWritings,0.0,101.0,False,False,Fake,Report #11615
,HistoryWritings,0.0,101.0,False,False,Fake,
,HistoryWritings,0.0,101.0,False,False,,Report #11616
,HistoryWritings,0.0,101.0,False,False,,
,HistoryWritings,1.0,-1.0,True,True,Genuine,Report #11617
,HistoryWritings,1.0,-1.0,True,True,Genuine,
,HistoryWritings,1.0,-1.0,True,True,Fake,Report #11618
,HistoryWritings,1.0,-1.0,True,True,Fake,
,HistoryWritings,1.0,-1.0,True,True,,Report #11619
,HistoryWritings,1.0,-1.0,True,True,,
,HistoryWritings,1.0,-1.0,True,False,Genuine,Report #11620
,HistoryWritings,1.0,-1.0,True,False,Genuine,
,HistoryWritings,1.0,-1.0,True,False,Fake,Report #11621
,HistoryWritings,1.0,-1.0,True,False,Fake,
,HistoryWritings,1.0,-1.0,True,False,,Report #11622
,HistoryWritings,1.0,-1.0,True,False,,
,HistoryWritings,1.0,-1.0,False,True,Genuine,Report #11623
,HistoryWritings,1.0,-1.0,False,True,Genuine,
,HistoryWritings,1.0,-1.0,False,True,Fake,Report #11624
,HistoryWritings,1.0,-1.0,False,True,Fake,
,HistoryWritings,1.0,-1.0,False,True,,Report #11625
,HistoryWritings,1.0,-1.0,False,True,,
,HistoryWritings,1.0,-1.0,False,False,Genuine,Report #11626
,HistoryWritings,1.0,-1.0,False,False,Genuine,
,HistoryWritings,1.0,-1.0,False,False,Fake,Report #11627
,HistoryWritings,1.0,-1.0,False,False,Fake,
,HistoryWritings,1.0,-1.0,False,False,,Report #11628
,HistoryWritings,1.0,-1.0,False,False,,
,HistoryWritings,1.0,0.0,True,True,Genuine,Report #11629
,HistoryWritings,1.0,0.0,True,True,Genuine,
,HistoryWritings,1.0,0.0,True,True,Fake,Report #11630
,HistoryWritings,1.0,0.0,True,True,Fake,
,HistoryWritings,1.0,0.0,True,True,,Report #11631
,HistoryWritings,1.0,0.0,True,True,,
,HistoryWritings,1.0,0.0,True,False,Genuine,Report #11632
,HistoryWritings,1.0,0.0,True,False,Genuine,
,HistoryWritings,1.0,0.0,True,False,Fake,Report #11633
,HistoryWritings,1.0,0.0,True,False,Fake,
,HistoryWritings,1.0,0.0,True,False,,Report #11634
,HistoryWritings,1.0,0.0,True,False,,
,HistoryWritings,1.0,0.0,False,True,Genuine,Report #11635
,HistoryWritings,1.0,0.0,False,True,Genuine,
,HistoryWritings,1.0,0.0,False,True,Fake,Report #11636
,HistoryWritings,1.0,0.0,False,True,Fake,
,HistoryWritings,1.0,0.0,False,True,,Report #11637
,HistoryWritings,1.0,0.0,False,True,,
,HistoryWritings,1.0,0.0,False,False,Genuine,Report #11638
,HistoryWritings,1.0,0.0,False,False,Genuine,
,HistoryWritings,1.0,0.0,False,False,Fake,Report #11639
,HistoryWritings,1.0,0.0,False,False,Fake,
,HistoryWritings,1.0,0.0,False,False,,Report #11640
,HistoryWritings,1.0,0.0,False,False,,
,HistoryWritings,1.0,19.0,True,True,Genuine,Report #11641
,HistoryWritings,1.0,19.0,True,True,Genuine,
,HistoryWritings,1.0,19.0,True,True,Fake,Report #11642
,HistoryWritings,1.0,19.0,True,True,Fake,
,HistoryWritings,1.0,19.0,True,True,,Report #11643
,HistoryWritings,1.0,19.0,True,True,,
,HistoryWritings,1.0,19.0,True,False,Genuine,Report #11644
,HistoryWritings,1.0,19.0,True,False,Genuine,
,HistoryWritings,1.0,19.0,True,False,Fake,Report #11645
,HistoryWritings,1.0,19.0,True,False,Fake,
,HistoryWritings,1.0,19.0,True,False,,Report #11646
,HistoryWritings,1.0,19.0,True,False,,
,HistoryWritings,1.0,19.0,False,True,Genuine,Report #11647
,HistoryWritings,1.0,19.0,False,True,Genuine,
,HistoryWritings,1.0,19.0,False,True,Fake,Report #11648
,HistoryWritings,1.0,19.0,False,True,Fake,
,HistoryWritings,1.0,19.0,False,True,,Report #11649
,HistoryWritings,1.0,19.0,False,True,,
,HistoryWritings,1.0,19.0,False,False,Genuine,Report #11650
,HistoryWritings,1.0,19.0,False,False,Genuine,
,HistoryWritings,1.0,19.0,False,False,Fake,Report #11651
,HistoryWritings,1.0,19.0,False,False,Fake,
,HistoryWritings,1.0,19.0,False,False,,Report #11652
,HistoryWritings,1.0,19.0,False,False,,
,HistoryWritings,1.0,20.0,True,True,Genuine,Report #11653
,HistoryWritings,1.0,20.0,True,True,Genuine,
,HistoryWritings,1.0,20.0,True,True,Fake,Report #11654
,HistoryWritings,1.0,20.0,True,True,Fake,
,HistoryWritings,1.0,20.0,True,True,,Report #11655
,HistoryWritings,1.0,20.0,True,True,,
,HistoryWritings,1.0,20.0,True,False,Genuine,Report #11656
,HistoryWritings,1.0,20.0,True,False,Genuine,
,HistoryWritings,1.0,20.0,True,False,Fake,Report #11657
,HistoryWritings,1.0,20.0,True,False,Fake,
,HistoryWritings,1.0,20.0,True,False,,Report #11658
,HistoryWritings,1.0,20.0,True,False,,
,HistoryWritings,1.0,20.0,False,True,Genuine,Report #11659
,HistoryWritings,1.0,20.0,False,True,Genuine,
,HistoryWritings,1.0,20.0,False,True,Fake,Report #11660
,HistoryWritings,1.0,20.0,False,True,Fake,
,HistoryWritings,1.0,20.0,False,True,,Report #11661
,HistoryWritings,1.0,20.0,False,True,,
,HistoryWritings,1.0,20.0,False,False,Genuine,Report #11662
,HistoryWritings,1.0,20.0,False,False,Genuine,
,HistoryWritings,1.0,20.0,False,False,Fake,Report #11663
,HistoryWritings,1.0,20.0,False,False,Fake,
,HistoryWritings,1.0,20.0,False,False,,Report #11664
,HistoryWritings,1.0,20.0,False,False,,
,HistoryWritings,1.0,21.0,True,True,Genuine,Report #11665
,HistoryWritings,1.0,21.0,True,True,Genuine,
,HistoryWritings,1.0,21.0,True,True,Fake,Report #11666
,HistoryWritings,1.0,21.0,True,True,Fake,
,HistoryWritings,1.0,21.0,True,True,,Report #11667
,HistoryWritings,1.0,21.0,True,True,,
,HistoryWritings,1.0,21.0,True,False,Genuine,Report #11668
,HistoryWritings,1.0,21.0,True,False,Genuine,
,HistoryWritings,1.0,21.0,True,False,Fake,Report #11669
,HistoryWritings,1.0,21.0,True,False,Fake,
,HistoryWritings,1.0,21.0,True,False,,Report #11670
,HistoryWritings,1.0,21.0,True,False,,
,HistoryWritings,1.0,21.0,False,True,Genuine,Report #11671
,HistoryWritings,1.0,21.0,False,True,Genuine,
,HistoryWritings,1.0,21.0,False,True,Fake,Report #11672
,HistoryWritings,1.0,21.0,False,True,Fake,
,HistoryWritings,1.0,21.0,False,True,,Report #11673
,HistoryWritings,1.0,21.0,False,True,,
,HistoryWritings,1.0,21.0,False,False,Genuine,Report #11674
,HistoryWritings,1.0,21.0,False,False,Genuine,
,HistoryWritings,1.0,21.0,False,False,Fake,Report #11675
,HistoryWritings,1.0,21.0,False,False,Fake,
,HistoryWritings,1.0,21.0,False,False,,Report #11676
,HistoryWritings,1.0,21.0,False,False,,
,HistoryWritings,1.0,99.0,True,True,Genuine,Report #11677
,HistoryWritings,1.0,99.0,True,True,Genuine,
,HistoryWritings,1.0,99.0,True,True,Fake,Report #11678
,HistoryWritings,1.0,99.0,True,True,Fake,
,HistoryWritings,1.0,99.0,True,True,,Report #11679
,HistoryWritings,1.0,99.0,True,True,,
,HistoryWritings,1.0,99.0,True,False,Genuine,Report #11680
,HistoryWritings,1.0,99.0,True,False,Genuine,
,HistoryWritings,1.0,99.0,True,False,Fake,Report #11681
,HistoryWritings,1.0,99.0,True,False,Fake,
,HistoryWritings,1.0,99.0,True,False,,Report #11682
,HistoryWritings,1.0,99.0,True,False,,
,HistoryWritings,1.0,99.0,False,True,Genuine,Report #11683
,HistoryWritings,1.0,99.0,False,True,Genuine,
,HistoryWritings,1.0,99.0,False,True,Fake,Report #11684
,HistoryWritings,1.0,99.0,False,True,Fake,
,HistoryWritings,1.0,99.0,False,True,,Report #11685
,HistoryWritings,1.0,99.0,False,True,,
,HistoryWritings,1.0,99.0,False,False,Genuine,Report #11686
,HistoryWritings,1.0,99.0,False,False,Genuine,
,HistoryWritings,1.0,99.0,False,False,Fake,Report #11687
,HistoryWritings,1.0,99.0,False,False,Fake,
,HistoryWritings,1.0,99.0,False,False,,Report #11688
,HistoryWritings,1.0,99.0,False,False,,
,HistoryWritings,1.0,100.0,True,True,Genuine,Report #11689
,HistoryWritings,1.0,100.0,True,True,Genuine,
,HistoryWritings,1.0,100.0,True,True,Fake,Report #11690
,HistoryWritings,1.0,100.0,True,True,Fake,
,HistoryWritings,1.0,100.0,True,True,,Report #11691
,HistoryWritings,1.0,100.0,True,True,,
,HistoryWritings,1.0,100.0,True,False,Genuine,Report #11692
,HistoryWritings,1.0,100.0,True,False,Genuine,
,HistoryWritings,1.0,100.0,True,False,Fake,Report #11693
,HistoryWritings,1.0,100.0,True,False,Fake,
,HistoryWritings,1.0,100.0,True,False,,Report #11694
,HistoryWritings,1.0,100.0,True,False,,
,HistoryWritings,1.0,100.0,False,True,Genuine,Report #11695
,HistoryWritings,1.0,100.0,False,True,Genuine,
,HistoryWritings,1.0,100.0,False,True,Fake,Report #11696
,HistoryWritings,1.0,100.0,False,True,Fake,
,HistoryWritings,1.0,100.0,False,True,,Report #11697
,HistoryWritings,1.0,100.0,False,True,,
,HistoryWritings,1.0,100.0,False,False,Genuine,Report #11698
,HistoryWritings,1.0,100.0,False,False,Genuine,
,HistoryWritings,1.0,100.0,False,False,Fake,Report #11699
,HistoryWritings,1.0,100.0,False,False,Fake,
,HistoryWritings,1.0,100.0,False,False,,Report #11700
,HistoryWritings,1.0,100.0,False,False,,
,HistoryWritings,1.0,101.0,True,True,Genuine,Report #11701
,HistoryWritings,1.0,101.0,True,True,Genuine,
,HistoryWritings,1.0,101.0,True,True,Fake,Report #11702
,HistoryWritings,1.0,101.0,True,True,Fake,
,HistoryWritings,1.0,101.0,True,True,,Report #11703
,HistoryWritings,1.0,101.0,True,True,,
,HistoryWritings,1.0,101.0,True,False,Genuine,Report #11704
,HistoryWritings,1.0,101.0,True,False,Genuine,
,HistoryWritings,1.0,101.0,True,False,Fake,Report #11705
,HistoryWritings,1.0,101.0,True,False,Fake,
,HistoryWritings,1.0,101.0,True,False,,Report #11706
,HistoryWritings,1.0,101.0,True,False,,
,HistoryWritings,1.0,101.0,False,True,Genuine,Report #11707
,HistoryWritings,1.0,101.0,False,True,Genuine,
,HistoryWritings,1.0,101.0,False,True,Fake,Report #11708
,HistoryWritings,1.0,101.0,False,True,Fake,
,HistoryWritings,1.0,101.0,False,True,,Report #11709
,HistoryWritings,1.0,101.0,False,True,,
,HistoryWritings,1.0,101.0,False,False,Genuine,Report #11710
,HistoryWritings,1.0,101.0,False,False,Genuine,
,HistoryWritings,1.0,101.0,False,False,Fake,Report #11711
,HistoryWritings,1.0,101.0,False,False,Fake,
,HistoryWritings,1.0,101.0,False,False,,Report #11712
,HistoryWritings,1.0,101.0,False,False,,
,HistoryWritings,100.0,-1.0,True,True,Genuine,Report #11713
,HistoryWritings,100.0,-1.0,True,True,Genuine,
,HistoryWritings,100.0,-1.0,True,True,Fake,Report #11714
,HistoryWritings,100.0,-1.0,True,True,Fake,
,HistoryWritings,100.0,-1.0,True,True,,Report #11715
,HistoryWritings,100.0,-1.0,True,True,,
,HistoryWritings,100.0,-1.0,True,False,Genuine,Report #11716
,HistoryWritings,100.0,-1.0,True,False,Genuine,
,HistoryWritings,100.0,-1.0,True,False,Fake,Report #11717
,HistoryWritings,100.0,-1.0,True,False,Fake,
,HistoryWritings,100.0,-1.0,True,False,,Report #11718
,HistoryWritings,100.0,-1.0,True,False,,
,HistoryWritings,100.0,-1.0,False,True,Genuine,Report #11719
,HistoryWritings,100.0,-1.0,False,True,Genuine,
,HistoryWritings,100.0,-1.0,False,True,Fake,Report #11720
,HistoryWritings,100.0,-1.0,False,True,Fake,
,HistoryWritings,100.0,-1.0,False,True,,Report #11721
,HistoryWritings,100.0,-1.0,False,True,,
,HistoryWritings,100.0,-1.0,False,False,Genuine,Report #11722
,HistoryWritings,100.0,-1.0,False,False,Genuine,
,HistoryWritings,100.0,-1.0,False,False,Fake,Report #11723
,HistoryWritings,100.0,-1.0,False,False,Fake,
,HistoryWritings,100.0,-1.0,False,False,,Report #11724
,HistoryWritings,100.0,-1.0,False,False,,
,HistoryWritings,100.0,0.0,True,True,Genuine,Report #11725
,HistoryWritings,100.0,0.0,True,True,Genuine,
,HistoryWritings,100.0,0.0,True,True,Fake,Report #11726
,HistoryWritings,100.0,0.0,True,True,Fake,
,HistoryWritings,100.0,0.0,True,True,,Report #11727
,HistoryWritings,100.0,0.0,True,True,,
,HistoryWritings,100.0,0.0,True,False,Genuine,Report #11728
,HistoryWritings,100.0,0.0,True,False,Genuine,
,HistoryWritings,100.0,0.0,True,False,Fake,Report #11729
,HistoryWritings,100.0,0.0,True,False,Fake,
,HistoryWritings,100.0,0.0,True,False,,Report #11730
,HistoryWritings,100.0,0.0,True,False,,
,HistoryWritings,100.0,0.0,False,True,Genuine,Report #11731
,HistoryWritings,100.0,0.0,False,True,Genuine,
,HistoryWritings,100.0,0.0,False,True,Fake,Report #11732
,HistoryWritings,100.0,0.0,False,True,Fake,
,HistoryWritings,100.0,0.0,False,True,,Report #11733
,HistoryWritings,100.0,0.0,False,True,,
,HistoryWritings,100.0,0.0,False,False,Genuine,Report #11734
,HistoryWritings,100.0,0.0,False,False,Genuine,
,HistoryWritings,100.0,0.0,False,False,Fake,Report #11735
,HistoryWritings,100.0,0.0,False,False,Fake,
,HistoryWritings,100.0,0.0,False,False,,Report #11736
,HistoryWritings,100.0,0.0,False,False,,
,HistoryWritings,100.0,19.0,True,True,Genuine,Report #11737
,HistoryWritings,100.0,19.0,True,True,Genuine,
,HistoryWritings,100.0,19.0,True,True,Fake,Report #11738
,HistoryWritings,100.0,19.0,True,True,Fake,
,HistoryWritings,100.0,19.0,True,True,,Report #11739
,HistoryWritings,100.0,19.0,True,True,,
,HistoryWritings,100.0,19.0,True,False,Genuine,Report #11740
,HistoryWritings,100.0,19.0,True,False,Genuine,
,HistoryWritings,100.0,19.0,True,False,Fake,Report #11741
,HistoryWritings,100.0,19.0,True,False,Fake,
,HistoryWritings,100.0,19.0,True,False,,Report #11742
,HistoryWritings,100.0,19.0,True,False,,
,HistoryWritings,100.0,19.0,False,True,Genuine,Report #11743
,HistoryWritings,100.0,19.0,False,True,Genuine,
,HistoryWritings,100.0,19.0,False,True,Fake,Report #11744
,HistoryWritings,100.0,19.0,False,True,Fake,
,HistoryWritings,100.0,19.0,False,True,,Report #11745
,HistoryWritings,100.0,19.0,False,True,,
,HistoryWritings,100.0,19.0,False,False,Genuine,Report #11746
,HistoryWritings,100.0,19.0,False,False,Genuine,
,HistoryWritings,100.0,19.0,False,False,Fake,Report #11747
,HistoryWritings,100.0,19.0,False,False,Fake,
,HistoryWritings,100.0,19.0,False,False,,Report #11748
,HistoryWritings,100.0,19.0,False,False,,
,HistoryWritings,100.0,20.0,True,True,Genuine,Report #11749
,HistoryWritings,100.0,20.0,True,True,Genuine,
,HistoryWritings,100.0,20.0,True,True,Fake,Report #11750
,HistoryWritings,100.0,20.0,True,True,Fake,
,HistoryWritings,100.0,20.0,True,True,,Report #11751
,HistoryWritings,100.0,20.0,True,True,,
,HistoryWritings,100.0,20.0,True,False,Genuine,Report #11752
,HistoryWritings,100.0,20.0,True,False,Genuine,
,HistoryWritings,100.0,20.0,True,False,Fake,Report #11753
,HistoryWritings,100.0,20.0,True,False,Fake,
,HistoryWritings,100.0,20.0,True,False,,Report #11754
,HistoryWritings,100.0,20.0,True,False,,
,HistoryWritings,100.0,20.0,False,True,Genuine,Report #11755
,HistoryWritings,100.0,20.0,False,True,Genuine,
,HistoryWritings,100.0,20.0,False,True,Fake,Report #11756
,HistoryWritings,100.0,20.0,False,True,Fake,
,HistoryWritings,100.0,20.0,False,True,,Report #11757
,HistoryWritings,100.0,20.0,False,True,,
,HistoryWritings,100.0,20.0,False,False,Genuine,Report #11758
,HistoryWritings,100.0,20.0,False,False,Genuine,
,HistoryWritings,100.0,20.0,False,False,Fake,Report #11759
,HistoryWritings,100.0,20.0,False,False,Fake,
,HistoryWritings,100.0,20.0,False,False,,Report #11760
,HistoryWritings,100.0,20.0,False,False,,
,HistoryWritings,100.0,21.0,True,True,Genuine,Report #11761
,HistoryWritings,100.0,21.0,True,True,Genuine,
,HistoryWritings,100.0,21.0,True,True,Fake,Report #11762
,HistoryWritings,100.0,21.0,True,True,Fake,
,HistoryWritings,100.0,21.0,True,True,,Report #11763
,HistoryWritings,100.0,21.0,True,True,,
,HistoryWritings,100.0,21.0,True,False,Genuine,Report #11764
,HistoryWritings,100.0,21.0,True,False,Genuine,
,HistoryWritings,100.0,21.0,True,False,Fake,Report #11765
,HistoryWritings,100.0,21.0,True,False,Fake,
,HistoryWritings,100.0,21.0,True,False,,Report #11766
,HistoryWritings,100.0,21.0,True,False,,
,HistoryWritings,100.0,21.0,False,True,Genuine,Report #11767
,HistoryWritings,100.0,21.0,False,True,Genuine,
,HistoryWritings,100.0,21.0,False,True,Fake,Report #11768
,HistoryWritings,100.0,21.0,False,True,Fake,
,HistoryWritings,100.0,21.0,False,True,,Report #11769
,HistoryWritings,100.0,21.0,False,True,,
,HistoryWritings,100.0,21.0,False,False,Genuine,Report #11770
,HistoryWritings,100.0,21.0,False,False,Genuine,
,HistoryWritings,100.0,21.0,False,False,Fake,Report #11771
,HistoryWritings,100.0,21.0,False,False,Fake,
,HistoryWritings,100.0,21.0,False,False,,Report #11772
,HistoryWritings,100.0,21.0,False,False,,
,HistoryWritings,100.0,99.0,True,True,Genuine,Report #11773
,HistoryWritings,100.0,99.0,True,True,Genuine,
,HistoryWritings,100.0,99.0,True,True,Fake,Report #11774
,HistoryWritings,100.0,99.0,True,True,Fake,
,HistoryWritings,100.0,99.0,True,True,,Report #11775
,HistoryWritings,100.0,99.0,True,True,,
,HistoryWritings,100.0,99.0,True,False,Genuine,Report #11776
,HistoryWritings,100.0,99.0,True,False,Genuine,
,HistoryWritings,100.0,99.0,True,False,Fake,Report #11777
,HistoryWritings,100.0,99.0,True,False,Fake,
,HistoryWritings,100.0,99.0,True,False,,Report #11778
,HistoryWritings,100.0,99.0,True,False,,
,HistoryWritings,100.0,99.0,False,True,Genuine,Report #11779
,HistoryWritings,100.0,99.0,False,True,Genuine,
,HistoryWritings,100.0,99.0,False,True,Fake,Report #11780
,HistoryWritings,100.0,99.0,False,True,Fake,
,HistoryWritings,100.0,99.0,False,True,,Report #11781
,HistoryWritings,100.0,99.0,False,True,,
,HistoryWritings,100.0,99.0,False,False,Genuine,Report #11782
,HistoryWritings,100.0,99.0,False,False,Genuine,
,HistoryWritings,100.0,99.0,False,False,Fake,Report #11783
,HistoryWritings,100.0,99.0,False,False,Fake,
,HistoryWritings,100.0,99.0,False,False,,Report #11784
,HistoryWritings,100.0,99.0,False,False,,
,HistoryWritings,100.0,100.0,True,True,Genuine,Report #11785
,HistoryWritings,100.0,100.0,True,True,Genuine,
,HistoryWritings,100.0,100.0,True,True,Fake,Report #11786
,HistoryWritings,100.0,100.0,True,True,Fake,
,HistoryWritings,100.0,100.0,True,True,,Report #11787
,HistoryWritings,100.0,100.0,True,True,,
,HistoryWritings,100.0,100.0,True,False,Genuine,Report #11788
,HistoryWritings,100.0,100.0,True,False,Genuine,
,HistoryWritings,100.0,100.0,True,False,Fake,Report #11789
,HistoryWritings,100.0,100.0,True,False,Fake,
,HistoryWritings,100.0,100.0,True,False,,Report #11790
,HistoryWritings,100.0,100.0,True,False,,
,HistoryWritings,100.0,100.0,False,True,Genuine,Report #11791
,HistoryWritings,100.0,100.0,False,True,Genuine,
,HistoryWritings,100.0,100.0,False,True,Fake,Report #11792
,HistoryWritings,100.0,100.0,False,True,Fake,
,HistoryWritings,100.0,100.0,False,True,,Report #11793
,HistoryWritings,100.0,100.0,False,True,,
,HistoryWritings,100.0,100.0,False,False,Genuine,Report #11794
,HistoryWritings,100.0,100.0,False,False,Genuine,
,HistoryWritings,100.0,100.0,False,False,Fake,Report #11795
,HistoryWritings,100.0,100.0,False,False,Fake,
,HistoryWritings,100.0,100.0,False,False,,Report #11796
,HistoryWritings,100.0,100.0,False,False,,
,HistoryWritings,100.0,101.0,True,True,Genuine,Report #11797
,HistoryWritings,100.0,101.0,True,True,Genuine,
,HistoryWritings,100.0,101.0,True,True,Fake,Report #11798
,HistoryWritings,100.0,101.0,True,True,Fake,
,HistoryWritings,100.0,101.0,True,True,,Report #11799
,HistoryWritings,100.0,101.0,True,True,,
,HistoryWritings,100.0,101.0,True,False,Genuine,Report #11800
,HistoryWritings,100.0,101.0,True,False,Genuine,
,HistoryWritings,100.0,101.0,True,False,Fake,Report #11801
,HistoryWritings,100.0,101.0,True,False,Fake,
,HistoryWritings,100.0,101.0,True,False,,Report #11802
,HistoryWritings,100.0,101.0,True,False,,
,HistoryWritings,100.0,101.0,False,True,Genuine,Report #11803
,HistoryWritings,100.0,101.0,False,True,Genuine,
,HistoryWritings,100.0,101.0,False,True,Fake,Report #11804
,HistoryWritings,100.0,101.0,False,True,Fake,
,HistoryWritings,100.0,101.0,False,True,,Report #11805
,HistoryWritings,100.0,101.0,False,True,,
,HistoryWritings,100.0,101.0,False,False,Genuine,Report #11806
,HistoryWritings,100.0,101.0,False,False,Genuine,
,HistoryWritings,100.0,101.0,False,False,Fake,Report #11807
,HistoryWritings,100.0,101.0,False,False,Fake,
,HistoryWritings,100.0,101.0,False,False,,Report #11808
,HistoryWritings,100.0,101.0,False,False,,
,HistoryWritings,1000.0,-1.0,True,True,Genuine,Report #11809
,HistoryWritings,1000.0,-1.0,True,True,Genuine,
,HistoryWritings,1000.0,-1.0,True,True,Fake,Report #11810
,HistoryWritings,1000.0,-1.0,True,True,Fake,
,HistoryWritings,1000.0,-1.0,True,True,,Report #11811
,HistoryWritings,1000.0,-1.0,True,True,,
,HistoryWritings,1000.0,-1.0,True,False,Genuine,Report #11812
,HistoryWritings,1000.0,-1.0,True,False,Genuine,
,HistoryWritings,1000.0,-1.0,True,False,Fake,Report #11813
,HistoryWritings,1000.0,-1.0,True,False,Fake,
,HistoryWritings,1000.0,-1.0,True,False,,Report #11814
,HistoryWritings,1000.0,-1.0,True,False,,
,HistoryWritings,1000.0,-1.0,False,True,Genuine,Report #11815
,HistoryWritings,1000.0,-1.0,False,True,Genuine,
,HistoryWritings,1000.0,-1.0,False,True,Fake,Report #11816
,HistoryWritings,1000.0,-1.0,False,True,Fake,
,HistoryWritings,1000.0,-1.0,False,True,,Report #11817
,HistoryWritings,1000.0,-1.0,False,True,,
,HistoryWritings,1000.0,-1.0,False,False,Genuine,Report #11818
,HistoryWritings,1000.0,-1.0,False,False,Genuine,
,HistoryWritings,1000.0,-1.0,False,False,Fake,Report #11819
,HistoryWritings,1000.0,-1.0,False,False,Fake,
,HistoryWritings,1000.0,-1.0,False,False,,Report #11820
,HistoryWritings,1000.0,-1.0,False,False,,
,HistoryWritings,1000.0,0.0,True,True,Genuine,Report #11821
,HistoryWritings,1000.0,0.0,True,True,Genuine,
,HistoryWritings,1000.0,0.0,True,True,Fake,Report #11822
,HistoryWritings,1000.0,0.0,True,True,Fake,
,HistoryWritings,1000.0,0.0,True,True,,Report #11823
,HistoryWritings,1000.0,0.0,True,True,,
,HistoryWritings,1000.0,0.0,True,False,Genuine,Report #11824
,HistoryWritings,1000.0,0.0,True,False,Genuine,
,HistoryWritings,1000.0,0.0,True,False,Fake,Report #11825
,HistoryWritings,1000.0,0.0,True,False,Fake,
,HistoryWritings,1000.0,0.0,True,False,,Report #11826
,HistoryWritings,1000.0,0.0,True,False,,
,HistoryWritings,1000.0,0.0,False,True,Genuine,Report #11827
,HistoryWritings,1000.0,0.0,False,True,Genuine,
,HistoryWritings,1000.0,0.0,False,True,Fake,Report #11828
,HistoryWritings,1000.0,0.0,False,True,Fake,
,HistoryWritings,1000.0,0.0,False,True,,Report #11829
,HistoryWritings,1000.0,0.0,False,True,,
,HistoryWritings,1000.0,0.0,False,False,Genuine,Report #11830
,HistoryWritings,1000.0,0.0,False,False,Genuine,
,HistoryWritings,1000.0,0.0,False,False,Fake,Report #11831
,HistoryWritings,1000.0,0.0,False,False,Fake,
,HistoryWritings,1000.0,0.0,False,False,,Report #11832
,HistoryWritings,1000.0,0.0,False,False,,
,HistoryWritings,1000.0,19.0,True,True,Genuine,Report #11833
,HistoryWritings,1000.0,19.0,True,True,Genuine,
,HistoryWritings,1000.0,19.0,True,True,Fake,Report #11834
,HistoryWritings,1000.0,19.0,True,True,Fake,
,HistoryWritings,1000.0,19.0,True,True,,Report #11835
,HistoryWritings,1000.0,19.0,True,True,,
,HistoryWritings,1000.0,19.0,True,False,Genuine,Report #11836
,HistoryWritings,1000.0,19.0,True,False,Genuine,
,HistoryWritings,1000.0,19.0,True,False,Fake,Report #11837
,HistoryWritings,1000.0,19.0,True,False,Fake,
,HistoryWritings,1000.0,19.0,True,False,,Report #11838
,HistoryWritings,1000.0,19.0,True,False,,
,HistoryWritings,1000.0,19.0,False,True,Genuine,Report #11839
,HistoryWritings,1000.0,19.0,False,True,Genuine,
,HistoryWritings,1000.0,19.0,False,True,Fake,Report #11840
,HistoryWritings,1000.0,19.0,False,True,Fake,
,HistoryWritings,1000.0,19.0,False,True,,Report #11841
,HistoryWritings,1000.0,19.0,False,True,,
,HistoryWritings,1000.0,19.0,False,False,Genuine,Report #11842
,HistoryWritings,1000.0,19.0,False,False,Genuine,
,HistoryWritings,1000.0,19.0,False,False,Fake,Report #11843
,HistoryWritings,1000.0,19.0,False,False,Fake,
,HistoryWritings,1000.0,19.0,False,False,,Report #11844
,HistoryWritings,1000.0,19.0,False,False,,
,HistoryWritings,1000.0,20.0,True,True,Genuine,Report #11845
,HistoryWritings,1000.0,20.0,True,True,Genuine,
,HistoryWritings,1000.0,20.0,True,True,Fake,Report #11846
,HistoryWritings,1000.0,20.0,True,True,Fake,
,HistoryWritings,1000.0,20.0,True,True,,Report #11847
,HistoryWritings,1000.0,20.0,True,True,,
,HistoryWritings,1000.0,20.0,True,False,Genuine,Report #11848
,HistoryWritings,1000.0,20.0,True,False,Genuine,
,HistoryWritings,1000.0,20.0,True,False,Fake,Report #11849
,HistoryWritings,1000.0,20.0,True,False,Fake,
,HistoryWritings,1000.0,20.0,True,False,,Report #11850
,HistoryWritings,1000.0,20.0,True,False,,
,HistoryWritings,1000.0,20.0,False,True,Genuine,Report #11851
,HistoryWritings,1000.0,20.0,False,True,Genuine,
,HistoryWritings,1000.0,20.0,False,True,Fake,Report #11852
,HistoryWritings,1000.0,20.0,False,True,Fake,
,HistoryWritings,1000.0,20.0,False,True,,Report #11853
,HistoryWritings,1000.0,20.0,False,True,,
,HistoryWritings,1000.0,20.0,False,False,Genuine,Report #11854
,HistoryWritings,1000.0,20.0,False,False,Genuine,
,HistoryWritings,1000.0,20.0,False,False,Fake,Report #11855
,HistoryWritings,1000.0,20.0,False,False,Fake,
,HistoryWritings,1000.0,20.0,False,False,,Report #11856
,HistoryWritings,1000.0,20.0,False,False,,
,HistoryWritings,1000.0,21.0,True,True,Genuine,Report #11857
,HistoryWritings,1000.0,21.0,True,True,Genuine,
,HistoryWritings,1000.0,21.0,True,True,Fake,Report #11858
,HistoryWritings,1000.0,21.0,True,True,Fake,
,HistoryWritings,1000.0,21.0,True,True,,Report #11859
,HistoryWritings,1000.0,21.0,True,True,,
,HistoryWritings,1000.0,21.0,True,False,Genuine,Report #11860
,HistoryWritings,1000.0,21.0,True,False,Genuine,
,HistoryWritings,1000.0,21.0,True,False,Fake,Report #11861
,HistoryWritings,1000.0,21.0,True,False,Fake,
,HistoryWritings,1000.0,21.0,True,False,,Report #11862
,HistoryWritings,1000.0,21.0,True,False,,
,HistoryWritings,1000.0,21.0,False,True,Genuine,Report #11863
,HistoryWritings,1000.0,21.0,False,True,Genuine,
,HistoryWritings,1000.0,21.0,False,True,Fake,Report #11864
,HistoryWritings,1000.0,21.0,False,True,Fake,
,HistoryWritings,1000.0,21.0,False,True,,Report #11865
,HistoryWritings,1000.0,21.0,False,True,,
,HistoryWritings,1000.0,21.0,False,False,Genuine,Report #11866
,HistoryWritings,1000.0,21.0,False,False,Genuine,
,HistoryWritings,1000.0,21.0,False,False,Fake,Report #11867
,HistoryWritings,1000.0,21.0,False,False,Fake,
,HistoryWritings,1000.0,21.0,False,False,,Report #11868
,HistoryWritings,1000.0,21.0,False,False,,
,HistoryWritings,1000.0,99.0,True,True,Genuine,Report #11869
,HistoryWritings,1000.0,99.0,True,True,Genuine,
,HistoryWritings,1000.0,99.0,True,True,Fake,Report #11870
,HistoryWritings,1000.0,99.0,True,True,Fake,
,HistoryWritings,1000.0,99.0,True,True,,Report #11871
,HistoryWritings,1000.0,99.0,True,True,,
,HistoryWritings,1000.0,99.0,True,False,Genuine,Report #11872
,HistoryWritings,1000.0,99.0,True,False,Genuine,
,HistoryWritings,1000.0,99.0,True,False,Fake,Report #11873
,HistoryWritings,1000.0,99.0,True,False,Fake,
,HistoryWritings,1000.0,99.0,True,False,,Report #11874
,HistoryWritings,1000.0,99.0,True,False,,
,HistoryWritings,1000.0,99.0,False,True,Genuine,Report #11875
,HistoryWritings,1000.0,99.0,False,True,Genuine,
,HistoryWritings,1000.0,99.0,False,True,Fake,Report #11876
,HistoryWritings,1000.0,99.0,False,True,Fake,
,HistoryWritings,1000.0,99.0,False,True,,Report #11877
,HistoryWritings,1000.0,99.0,False,True,,
,HistoryWritings,1000.0,99.0,False,False,Genuine,Report #11878
,HistoryWritings,1000.0,99.0,False,False,Genuine,
,HistoryWritings,1000.0,99.0,False,False,Fake,Report #11879
,HistoryWritings,1000.0,99.0,False,False,Fake,
,HistoryWritings,1000.0,99.0,False,False,,Report #11880
,HistoryWritings,1000.0,99.0,False,False,,
,HistoryWritings,1000.0,100.0,True,True,Genuine,Report #11881
,HistoryWritings,1000.0,100.0,True,True,Genuine,
,HistoryWritings,1000.0,100.0,True,True,Fake,Report #11882
,HistoryWritings,1000.0,100.0,True,True,Fake,
,HistoryWritings,1000.0,100.0,True,True,,Report #11883
,HistoryWritings,1000.0,100.0,True,True,,
,HistoryWritings,1000.0,100.0,True,False,Genuine,Report #11884
,HistoryWritings,1000.0,100.0,True,False,Genuine,
,HistoryWritings,1000.0,100.0,True,False,Fake,Report #11885
,HistoryWritings,1000.0,100.0,True,False,Fake,
,HistoryWritings,1000.0,100.0,True,False,,Report #11886
,HistoryWritings,1000.0,100.0,True,False,,
,HistoryWritings,1000.0,100.0,False,True,Genuine,Report #11887
,HistoryWritings,1000.0,100.0,False,True,Genuine,
,HistoryWritings,1000.0,100.0,False,True,Fake,Report #11888
,HistoryWritings,1000.0,100.0,False,True,Fake,
,HistoryWritings,1000.0,100.0,False,True,,Report #11889
,HistoryWritings,1000.0,100.0,False,True,,
,HistoryWritings,1000.0,100.0,False,False,Genuine,Report #11890
,HistoryWritings,1000.0,100.0,False,False,Genuine,
,HistoryWritings,1000.0,100.0,False,False,Fake,Report #11891
,HistoryWritings,1000.0,100.0,False,False,Fake,
,HistoryWritings,1000.0,100.0,False,False,,Report #11892
,HistoryWritings,1000.0,100.0,False,False,,
,HistoryWritings,1000.0,101.0,True,True,Genuine,Report #11893
,HistoryWritings,1000.0,101.0,True,True,Genuine,
,HistoryWritings,1000.0,101.0,True,True,Fake,Report #11894
,HistoryWritings,1000.0,101.0,True,True,Fake,
,HistoryWritings,1000.0,101.0,True,True,,Report #11895
,HistoryWritings,1000.0,101.0,True,True,,
,HistoryWritings,1000.0,101.0,True,False,Genuine,Report #11896
,HistoryWritings,1000.0,101.0,True,False,Genuine,
,HistoryWritings,1000.0,101.0,True,False,Fake,Report #11897
,HistoryWritings,1000.0,101.0,True,False,Fake,
,HistoryWritings,1000.0,101.0,True,False,,Report #11898
,HistoryWritings,1000.0,101.0,True,False,,
,HistoryWritings,1000.0,101.0,False,True,Genuine,Report #11899
,HistoryWritings,1000.0,101.0,False,True,Genuine,
,HistoryWritings,1000.0,101.0,False,True,Fake,Report #11900
,HistoryWritings,1000.0,101.0,False,True,Fake,
,HistoryWritings,1000.0,101.0,False,True,,Report #11901
,HistoryWritings,1000.0,101.0,False,True,,
,HistoryWritings,1000.0,101.0,False,False,Genuine,Report #11902
,HistoryWritings,1000.0,101.0,False,False,Genuine,
,HistoryWritings,1000.0,101.0,False,False,Fake,Report #11903
,HistoryWritings,1000.0,101.0,False,False,Fake,
,HistoryWritings,1000.0,101.0,False,False,,Report #11904
,HistoryWritings,1000.0,101.0,False,False,,
,HistoryWritings,1000000.0,-1.0,True,True,Genuine,Report #11905
,HistoryWritings,1000000.0,-1.0,True,True,Genuine,
,HistoryWritings,1000000.0,-1.0,True,True,Fake,Report #11906
,HistoryWritings,1000000.0,-1.0,True,True,Fake,
,HistoryWritings,1000000.0,-1.0,True,True,,Report #11907
,HistoryWritings,1000000.0,-1.0,True,True,,
,HistoryWritings,1000000.0,-1.0,True,False,Genuine,Report #11908
,HistoryWritings,1000000.0,-1.0,True,False,Genuine,
,HistoryWritings,1000000.0,-1.0,True,False,Fake,Report #11909
,HistoryWritings,1000000.0,-1.0,True,False,Fake,
,HistoryWritings,1000000.0,-1.0,True,False,,Report #11910
,HistoryWritings,1000000.0,-1.0,True,False,,
,HistoryWritings,1000000.0,-1.0,False,True,Genuine,Report #11911
,HistoryWritings,1000000.0,-1.0,False,True,Genuine,
,HistoryWritings,1000000.0,-1.0,False,True,Fake,Report #11912
,HistoryWritings,1000000.0,-1.0,False,True,Fake,
,HistoryWritings,1000000.0,-1.0,False,True,,Report #11913
,HistoryWritings,1000000.0,-1.0,False,True,,
,HistoryWritings,1000000.0,-1.0,False,False,Genuine,Report #11914
,HistoryWritings,1000000.0,-1.0,False,False,Genuine,
,HistoryWritings,1000000.0,-1.0,False,False,Fake,Report #11915
,HistoryWritings,1000000.0,-1.0,False,False,Fake,
,HistoryWritings,1000000.0,-1.0,False,False,,Report #11916
,HistoryWritings,1000000.0,-1.0,False,False,,
,HistoryWritings,1000000.0,0.0,True,True,Genuine,Report #11917
,HistoryWritings,1000000.0,0.0,True,True,Genuine,
,HistoryWritings,1000000.0,0.0,True,True,Fake,Report #11918
,HistoryWritings,1000000.0,0.0,True,True,Fake,
,HistoryWritings,1000000.0,0.0,True,True,,Report #11919
,HistoryWritings,1000000.0,0.0,True,True,,
,HistoryWritings,1000000.0,0.0,True,False,Genuine,Report #11920
,HistoryWritings,1000000.0,0.0,True,False,Genuine,
,HistoryWritings,1000000.0,0.0,True,False,Fake,Report #11921
,HistoryWritings,1000000.0,0.0,True,False,Fake,
,HistoryWritings,1000000.0,0.0,True,False,,Report #11922
,HistoryWritings,1000000.0,0.0,True,False,,
,HistoryWritings,1000000.0,0.0,False,True,Genuine,Report #11923
,HistoryWritings,1000000.0,0.0,False,True,Genuine,
,HistoryWritings,1000000.0,0.0,False,True,Fake,Report #11924
,HistoryWritings,1000000.0,0.0,False,True,Fake,
,HistoryWritings,1000000.0,0.0,False,True,,Report #11925
,HistoryWritings,1000000.0,0.0,False,True,,
,HistoryWritings,1000000.0,0.0,False,False,Genuine,Report #11926
,HistoryWritings,1000000.0,0.0,False,False,Genuine,
,HistoryWritings,1000000.0,0.0,False,False,Fake,Report #11927
,HistoryWritings,1000000.0,0.0,False,False,Fake,
,HistoryWritings,1000000.0,0.0,False,False,,Report #11928
,HistoryWritings,1000000.0,0.0,False,False,,
,HistoryWritings,1000000.0,19.0,True,True,Genuine,Report #11929
,HistoryWritings,1000000.0,19.0,True,True,Genuine,
,HistoryWritings,1000000.0,19.0,True,True,Fake,Report #11930
,HistoryWritings,1000000.0,19.0,True,True,Fake,
,HistoryWritings,1000000.0,19.0,True,True,,Report #11931
,HistoryWritings,1000000.0,19.0,True,True,,
,HistoryWritings,1000000.0,19.0,True,False,Genuine,Report #11932
,HistoryWritings,1000000.0,19.0,True,False,Genuine,
,HistoryWritings,1000000.0,19.0,True,False,Fake,Report #11933
,HistoryWritings,1000000.0,19.0,True,False,Fake,
,HistoryWritings,1000000.0,19.0,True,False,,Report #11934
,HistoryWritings,1000000.0,19.0,True,False,,
,HistoryWritings,1000000.0,19.0,False,True,Genuine,Report #11935
,HistoryWritings,1000000.0,19.0,False,True,Genuine,
,HistoryWritings,1000000.0,19.0,False,True,Fake,Report #11936
,HistoryWritings,1000000.0,19.0,False,True,Fake,
,HistoryWritings,1000000.0,19.0,False,True,,Report #11937
,HistoryWritings,1000000.0,19.0,False,True,,
,HistoryWritings,1000000.0,19.0,False,False,Genuine,Report #11938
,HistoryWritings,1000000.0,19.0,False,False,Genuine,
,HistoryWritings,1000000.0,19.0,False,False,Fake,Report #11939
,HistoryWritings,1000000.0,19.0,False,False,Fake,
,HistoryWritings,1000000.0,19.0,False,False,,Report #11940
,HistoryWritings,1000000.0,19.0,False,False,,
,HistoryWritings,1000000.0,20.0,True,True,Genuine,Report #11941
,HistoryWritings,1000000.0,20.0,True,True,Genuine,
,HistoryWritings,1000000.0,20.0,True,True,Fake,Report #11942
,HistoryWritings,1000000.0,20.0,True,True,Fake,
,HistoryWritings,1000000.0,20.0,True,True,,Report #11943
,HistoryWritings,1000000.0,20.0,True,True,,
,HistoryWritings,1000000.0,20.0,True,False,Genuine,Report #11944
,HistoryWritings,1000000.0,20.0,True,False,Genuine,
,HistoryWritings,1000000.0,20.0,True,False,Fake,Report #11945
,HistoryWritings,1000000.0,20.0,True,False,Fake,
,HistoryWritings,1000000.0,20.0,True,False,,Report #11946
,HistoryWritings,1000000.0,20.0,True,False,,
,HistoryWritings,1000000.0,20.0,False,True,Genuine,Report #11947
,HistoryWritings,1000000.0,20.0,False,True,Genuine,
,HistoryWritings,1000000.0,20.0,False,True,Fake,Report #11948
,HistoryWritings,1000000.0,20.0,False,True,Fake,
,HistoryWritings,1000000.0,20.0,False,True,,Report #11949
,HistoryWritings,1000000.0,20.0,False,True,,
,HistoryWritings,1000000.0,20.0,False,False,Genuine,Report #11950
,HistoryWritings,1000000.0,20.0,False,False,Genuine,
,HistoryWritings,1000000.0,20.0,False,False,Fake,Report #11951
,HistoryWritings,1000000.0,20.0,False,False,Fake,
,HistoryWritings,1000000.0,20.0,False,False,,Report #11952
,HistoryWritings,1000000.0,20.0,False,False,,
,HistoryWritings,1000000.0,21.0,True,True,Genuine,Report #11953
,HistoryWritings,1000000.0,21.0,True,True,Genuine,
,HistoryWritings,1000000.0,21.0,True,True,Fake,Report #11954
,HistoryWritings,1000000.0,21.0,True,True,Fake,
,HistoryWritings,1000000.0,21.0,True,True,,Report #11955
,HistoryWritings,1000000.0,21.0,True,True,,
,HistoryWritings,1000000.0,21.0,True,False,Genuine,Report #11956
,HistoryWritings,1000000.0,21.0,True,False,Genuine,
,HistoryWritings,1000000.0,21.0,True,False,Fake,Report #11957
,HistoryWritings,1000000.0,21.0,True,False,Fake,
,HistoryWritings,1000000.0,21.0,True,False,,Report #11958
,HistoryWritings,1000000.0,21.0,True,False,,
,HistoryWritings,1000000.0,21.0,False,True,Genuine,Report #11959
,HistoryWritings,1000000.0,21.0,False,True,Genuine,
,HistoryWritings,1000000.0,21.0,False,True,Fake,Report #11960
,HistoryWritings,1000000.0,21.0,False,True,Fake,
,HistoryWritings,1000000.0,21.0,False,True,,Report #11961
,HistoryWritings,1000000.0,21.0,False,True,,
,HistoryWritings,1000000.0,21.0,False,False,Genuine,Report #11962
,HistoryWritings,1000000.0,21.0,False,False,Genuine,
,HistoryWritings,1000000.0,21.0,False,False,Fake,Report #11963
,HistoryWritings,1000000.0,21.0,False,False,Fake,
,HistoryWritings,1000000.0,21.0,False,False,,Report #11964
,HistoryWritings,1000000.0,21.0,False,False,,
,HistoryWritings,1000000.0,99.0,True,True,Genuine,Report #11965
,HistoryWritings,1000000.0,99.0,True,True,Genuine,
,HistoryWritings,1000000.0,99.0,True,True,Fake,Report #11966
,HistoryWritings,1000000.0,99.0,True,True,Fake,
,HistoryWritings,1000000.0,99.0,True,True,,Report #11967
,HistoryWritings,1000000.0,99.0,True,True,,
,HistoryWritings,1000000.0,99.0,True,False,Genuine,Report #11968
,HistoryWritings,1000000.0,99.0,True,False,Genuine,
,HistoryWritings,1000000.0,99.0,True,False,Fake,Report #11969
,HistoryWritings,1000000.0,99.0,True,False,Fake,
,HistoryWritings,1000000.0,99.0,True,False,,Report #11970
,HistoryWritings,1000000.0,99.0,True,False,,
,HistoryWritings,1000000.0,99.0,False,True,Genuine,Report #11971
,HistoryWritings,1000000.0,99.0,False,True,Genuine,
,HistoryWritings,1000000.0,99.0,False,True,Fake,Report #11972
,HistoryWritings,1000000.0,99.0,False,True,Fake,
,HistoryWritings,1000000.0,99.0,False,True,,Report #11973
,HistoryWritings,1000000.0,99.0,False,True,,
,HistoryWritings,1000000.0,99.0,False,False,Genuine,Report #11974
,HistoryWritings,1000000.0,99.0,False,False,Genuine,
,HistoryWritings,1000000.0,99.0,False,False,Fake,Report #11975
,HistoryWritings,1000000.0,99.0,False,False,Fake,
,HistoryWritings,1000000.0,99.0,False,False,,Report #11976
,HistoryWritings,1000000.0,99.0,False,False,,
,HistoryWritings,1000000.0,100.0,True,True,Genuine,Report #11977
,HistoryWritings,1000000.0,100.0,True,True,Genuine,
,HistoryWritings,1000000.0,100.0,True,True,Fake,Report #11978
,HistoryWritings,1000000.0,100.0,True,True,Fake,
,HistoryWritings,1000000.0,100.0,True,True,,Report #11979
,HistoryWritings,1000000.0,100.0,True,True,,
,HistoryWritings,1000000.0,100.0,True,False,Genuine,Report #11980
,HistoryWritings,1000000.0,100.0,True,False,Genuine,
,HistoryWritings,1000000.0,100.0,True,False,Fake,Report #11981
,HistoryWritings,1000000.0,100.0,True,False,Fake,
,HistoryWritings,1000000.0,100.0,True,False,,Report #11982
,HistoryWritings,1000000.0,100.0,True,False,,
,HistoryWritings,1000000.0,100.0,False,True,Genuine,Report #11983
,HistoryWritings,1000000.0,100.0,False,True,Genuine,
,HistoryWritings,1000000.0,100.0,False,True,Fake,Report #11984
,HistoryWritings,1000000.0,100.0,False,True,Fake,
,HistoryWritings,1000000.0,100.0,False,True,,Report #11985
,HistoryWritings,1000000.0,100.0,False,True,,
,HistoryWritings,1000000.0,100.0,False,False,Genuine,Report #11986
,HistoryWritings,1000000.0,100.0,False,False,Genuine,
,HistoryWritings,1000000.0,100.0,False,False,Fake,Report #11987
,HistoryWritings,1000000.0,100.0,False,False,Fake,
,HistoryWritings,1000000.0,100.0,False,False,,Report #11988
,HistoryWritings,1000000.0,100.0,False,False,,
,HistoryWritings,1000000.0,101.0,True,True,Genuine,Report #11989
,HistoryWritings,1000000.0,101.0,True,True,Genuine,
,HistoryWritings,1000000.0,101.0,True,True,Fake,Report #11990
,HistoryWritings,1000000.0,101.0,True,True,Fake,
,HistoryWritings,1000000.0,101.0,True,True,,Report #11991
,HistoryWritings,1000000.0,101.0,True,True,,
,HistoryWritings,1000000.0,101.0,True,False,Genuine,Report #11992
,HistoryWritings,1000000.0,101.0,True,False,Genuine,
,HistoryWritings,1000000.0,101.0,True,False,Fake,Report #11993
,HistoryWritings,1000000.0,101.0,True,False,Fake,
,HistoryWritings,1000000.0,101.0,True,False,,Report #11994
,HistoryWritings,1000000.0,101.0,True,False,,
,HistoryWritings,1000000.0,101.0,False,True,Genuine,Report #11995
,HistoryWritings,1000000.0,101.0,False,True,Genuine,
,HistoryWritings,1000000.0,101.0,False,True,Fake,Report #11996
,HistoryWritings,1000000.0,101.0,False,True,Fake,
,HistoryWritings,1000000.0,101.0,False,True,,Report #11997
,HistoryWritings,1000000.0,101.0,False,True,,
,HistoryWritings,1000000.0,101.0,False,False,Genuine,Report #11998
,HistoryWritings,1000000.0,101.0,False,False,Genuine,
,HistoryWritings,1000000.0,101.0,False,False,Fake,Report #11999
,HistoryWritings,1000000.0,101.0,False,False,Fake,
,HistoryWritings,1000000.0,101.0,False,False,,Report #12000
,HistoryWritings,1000000.0,101.0,False,False,,
"""


antiquesDataSource : Result Error (List Antique)
antiquesDataSource =
    Decode.decodeCsv Decode.FieldNamesFromFirstRow antiqueDecoder csvData
